from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

iddtxt = """!IDD_Version 8.0.0.008
! **************************************************************************
! This file is the Input Data Dictionary (IDD) for EnergyPlus.
! The IDD defines the syntax and data model for each type of input "Object."
! Lines in EnergyPlus input files (and IDD) are limited to 500 characters.
!
! Object Description
! ------------------
! To define an object (a record with data), develop a key word that is unique
! Each data item to the object can be A (Alphanumeric string) or N (numeric)
! Number each A and N.  This will show how the data items will be put into the
! arrays that are passed to the Input Processor "Get" (GetObjectItem) routines.
! All alpha fields are limited to 100 characters.  Numeric fields should be
! valid numerics (can include such as 1.0E+05) and are placed into double
! precision variables.
!
! NOTE: Even though a field may be optional, a comma representing that field
!   must be included (unless it is the last field in the object).  Since the
!   entire input is "field-oriented" and not "keyword-oriented", the EnergyPlus
!   Input Processor must have some representation (even if blank) for each
!   field.
!
! Object Documentation
! --------------------
! In addition, the following special comments appear one per line and
! most are followed by a value.  Comments may apply to a field or the object
! or a group of objects.
!
! Field-level comments:
!
!  \\field           Name of field
!                     (should be succinct and readable, blanks are encouraged)
!
!  \\note            Note describing the field and its valid values. If multiple lines,
!                   start each line with \\note. Limit line length to 100 characters.
!
!  \\required-field  To flag fields which may not be left blank
!                     (this comment has no "value")
!
!  \\begin-extensible  Marks the first field at which the object accepts an extensible
!                   field set.  A fixed number of fields from this marker define the
!                   extensible field set, see the object code \\extensible for
!                   more information.
!
!  \\units           Units (must be from EnergyPlus standard units list)
!                   EnergyPlus units are standard SI units
!
!  \\ip-units        IP-Units (for use by input processors with IP units)
!                   This is only used if the default conversion is not
!                   appropriate.
!
!  \\unitsBasedOnField  For fields that may have multiple possible units, indicates
!                   the field in the object that can be used to determine
!                   the units. The field reference is in the A2 form.
!
!  \\minimum         Minimum that includes the following value
!
!  \\minimum>        Minimum that must be > than the following value
!
!  \\maximum         Maximum that includes the following value
!
!  \\maximum<        Maximum that must be < than the following value
!
!  \\default         Default for the field (if N/A then omit entire line)
!
!  \\deprecated      This field is not really used and will be deleted from the object.
!                   The required information is gotten internally or
!                   not needed by the program.
!
!  \\autosizable     Flag to indicate that this field can be used with the Auto
!                   Sizing routines to produce calculated results for the
!                   field.  If a value follows this, then that will be used
!                   when the "Autosize" feature is flagged.  To trigger
!                   autosizing for a field, enter Autosize as the field's
!                   value.  Only applicable to numeric fields.
!
!  \\autocalculatable Flag to indicate that this field can be automatically
!                   calculated. To trigger auto calculation for a field, enter
!                   Autocalculate as the field's value.  Only applicable to
!                   numeric fields.
!
!  \\type            Type of data for the field -
!                     integer
!                     real
!                     alpha       (arbitrary string),
!                     choice      (alpha with specific list of choices, see
!                                 \\key)
!                     object-list (link to a list of objects defined elsewhere,
!                                  see \\object-list and \\reference)
!                     external-list (uses a special list from an external source,
!                                    see \\external-list)
!                     node        (name used in connecting HVAC components)
!
!  \\retaincase      Retains the alphabetic case for alpha type fields
!
!  \\key             Possible value for "\\type choice" (blanks are significant)
!                     use multiple \\key lines to indicate all valid choices
!
!  \\object-list     Name of a list of user-provided object names that are valid
!                     entries for this field (used with "\\reference")
!                     see Zone and BuildingSurface:Detailed objects below for
!                     examples.
!                  ** Note that a field may have multiple \\object-list commands.
!
!  \\external-list   The values for this field should be selected from a special
!                   list generated outside of the IDD file. The choices for the
!                   special lists are:
!                        autoRDDvariable
!                        autoRDDmeter
!                        autoRDDvariableMeter
!                   When one of these are selected the options for the field
!                   are taken from the RDD or MDD file or both.
!
!  \\reference       Name of a list of names to which this object belongs
!                     used with "\\type object-list" and with "\\object-list"
!                     see Zone and BuildingSurface:Detailed objects below for
!                     examples:
!
!                        Zone,
!                          A1 , \\field Name
!                               \\type alpha
!                               \\reference ZoneNames
!
!                        BuildingSurface:Detailed,
!                          A4 , \\field Zone Name
!                               \\note Zone the surface is a part of
!                               \\type object-list
!                               \\object-list ZoneNames
!
!             For each zone, the field "Name" may be referenced
!             by other objects, such as BuildingSurface:Detailed, so it is
!             commented with "\\reference ZoneNames"
!             Fields that reference a zone name, such as BuildingSurface:Detailed's
!             "Zone Name", are commented as
!             "\\type object-list" and "\\object-list ZoneNames"
!             ** Note that a field may have multiple \\reference commands.
!             ** This is useful if the object belongs to a small specific
!             object-list as well as a larger more general object-list.
!
! Object-level comments:
!
!  \\memo            Memo describing the object. If multiple lines, start each line
!                   with \\memo.
!                   Limit line length to 100 characters.
!
!  \\unique-object   To flag objects which should appear only once in an idf
!                     (this comment has no "value")
!
!  \\required-object To flag objects which are required in every idf
!                     (this comment has no "value")
!
!  \\min-fields      Minimum number of fields that should be included in the
!                   object.  If appropriate, the Input Processor will fill
!                   any missing fields with defaults (for numeric fields).
!                   It will also supply that number of fields to the "get"
!                   routines using blanks for alpha fields (note -- blanks
!                   may not be allowable for some alpha fields).
!
!  \\obsolete        This object has been replaced though is kept (and is read)
!                   in the current version.  Please refer to documentation as
!                   to the dispersal of the object.  If this object is
!                   encountered in an IDF, the InputProcessor will post an
!                   appropriate message to the error file.
!                   usage:  \\obsolete New=>[New object name]
!
!  \\extensible:<#>  This object is dynamically extensible -- meaning, if you
!           change the IDD appropriately (if the object has a simple list
!           structure -- just add items to the list arguments (i.e. BRANCH
!           LIST). These will be automatically redimensioned and used during
!           the simulation. <#> should be entered by the developer to signify
!           how many of the last fields are needed to be extended (and EnergyPlus
!           will attempt to auto-extend the object).  The first field of the first
!           instance of the extensible field set is marked with \\begin-extensible.
!
!  \\begin-extensible See previous item, marks beginning of extensible fields in
!                   an object.
!
!  \\format          The object should have a special format when saved in
!                   the IDF Editor with the special format option enabled.
!                   The options include SingleLine, Vertices, CompactSchedule,
!                   FluidProperties, ViewFactors, and Spectral.
!                   The SingleLine option puts all the fields for the object
!                   on a single line. The Vertices option is used in objects
!                   that use X, Y and Z fields to format those three fields
!                   on a single line.
!                   The CompactSchedule formats that specific object.
!                   The FluidProperty option formats long lists of fluid
!                   properties to ten values per line.
!                   The ViewFactor option formats three fields related to
!                   view factors per line.
!                   The Spectral option formats the four fields related to
!                   window glass spectral data per line.
!
!   \\reference-class-name Adds the name of the class to the reference list
!                   similar to \\reference.
!
! Group-level comments:
!
!  \\group          Name for a group of related objects
!
!
! Notes on comments
! -----------------
!
! 1.  If a particular comment is not applicable (such as units, or default)
! then simply omit the comment rather than indicating N/A.
!
! 2.  Memos and notes should be brief (recommend 5 lines or less per block).
! More extensive explanations are expected to be in the user documentation
!
! Default IP conversions (no \\ip-units necessary)
!      $/(m3/s)               =>   $/(ft3/min)         0.000472000059660808
!      $/(W/K)                =>   $/(Btu/h-F)         0.52667614683731
!      $/kW                   =>   $/(kBtuh/h)         0.293083235638921
!      $/m2                   =>   $/ft2               0.0928939733269818
!      $/m3                   =>   $/ft3               0.0283127014102352
!      (kg/s)/W               =>   (lbm/sec)/(Btu/hr)  0.646078115385742
!      1/K                    =>   1/F                 0.555555555555556
!      1/m                    =>   1/ft                0.3048
!      A/K                    =>   A/F                 0.555555555555556
!      C                      =>   F                   1.8 (plus 32)
!      cm                     =>   in                  0.3937
!      cm2                    =>   inch2               0.15500031000062
!      deltaC                 =>   deltaF              1.8
!      deltaJ/kg              =>   deltaBtu/lb         0.0004299
!      g/GJ                   =>   lb/MWh              0.00793664091373665
!      g/kg                   =>   grains/lb           7
!      g/MJ                   =>   lb/MWh              7.93664091373665
!      g/mol                  =>   lb/mol              0.0022046
!      g/m-s                  =>   lb/ft-s             0.000671968949659
!      g/m-s-K                =>   lb/ft-s-F           0.000373574867724868
!      GJ                     =>   ton-hrs             78.9889415481832
!      J                      =>   Wh                  0.000277777777777778
!      J/K                    =>   Btu/F               526.565
!      J/kg                   =>   Btu/lb              0.00042986
!      J/kg-K                 =>   Btu/lb-F            0.000239005736137667
!      J/kg-K2                =>   Btu/lb-F2           0.000132889924714692
!      J/kg-K3                =>   Btu/lb-F3           7.38277359526066E-05
!      J/m2-K                 =>   Btu/ft2-F           4.89224766847393E-05
!      J/m3                   =>   Btu/ft3             2.68096514745308E-05
!      J/m3-K                 =>   Btu/ft3-F           1.49237004739337E-05
!      K                      =>   R                   1.8
!      K/m                    =>   F/ft                0.54861322767449
!      kg                     =>   lb                  2.2046
!      kg/J                   =>   lb/Btu              2325.83774250441
!      kg/kg-K                =>   lb/lb-F             0.555555555555556
!      kg/m                   =>   lb/ft               0.67196893069637
!      kg/m2                  =>   lb/ft2              0.204794053596664
!      kg/m3                  =>   lb/ft3              0.062428
!      kg/m-s                 =>   lb/ft-s             0.67196893069637
!      kg/m-s-K               =>   lb/ft-s-F           0.373316072609094
!      kg/m-s-K2              =>   lb/ft-s-F2          0.207397818116164
!      kg/Pa-s-m2             =>   lb/psi-s-ft2        1412.00523459398
!      kg/s                   =>   lb/s                2.20462247603796
!      kg/s2                  =>   lb/s2               2.2046
!      kg/s-m                 =>   lb/s-ft             0.67196893069637
!      kJ/kg                  =>   Btu/lb              0.429925
!      kPa                    =>   psi                 0.145038
!      L/day                  =>   pint/day            2.11337629827348
!      L/GJ                   =>   gal/kWh             0.000951022349025202
!      L/kWh                  =>   pint/kWh            2.11337629827348
!      L/MJ                   =>   gal/kWh             0.951022349025202
!      lux                    =>   foot-candles        0.092902267
!      m                      =>   ft                  3.28083989501312
!      m/hr                   =>   ft/hr               3.28083989501312
!      m/s                    =>   ft/min              196.850393700787
!      m/s                    =>   miles/hr            2.2369362920544
!      m/yr                   =>   inch/yr             39.3700787401575
!      m2                     =>   ft2                 10.7639104167097
!      m2/m                   =>   ft2/ft              3.28083989501312
!      m2/person              =>   ft2/person          10.764961
!      m2/s                   =>   ft2/s               10.7639104167097
!      m2-K/W                 =>   ft2-F-hr/Btu        5.678263
!      m3                     =>   ft3                 35.3146667214886
!      m3                     =>   gal                 264.172037284185
!      m3/GJ                  =>   ft3/MWh             127.13292
!      m3/hr                  =>   ft3/hr              35.3146667214886
!      m3/hr-m2               =>   ft3/hr-ft2          3.28083989501312
!      m3/hr-person           =>   ft3/hr-person       35.3146667214886
!      m3/kg                  =>   ft3/lb              16.018
!      m3/m2                  =>   ft3/ft2             3.28083989501312
!      m3/MJ                  =>   ft3/kWh             127.13292
!      m3/person              =>   ft3/person          35.3146667214886
!      m3/s                   =>   ft3/min             2118.88000328931
!      m3/s-m                 =>   ft3/min-ft          645.89
!      m3/s-m2                =>   ft3/min-ft2         196.85
!      m3/s-person            =>   ft3/min-person      2118.6438
!      m3/s-W                 =>   (ft3/min)/(Btu/h)   621.099127332943
!      N-m                    =>   lbf-in              8.85074900525547
!      N-s/m2                 =>   lbf-s/ft2           0.0208857913669065
!      Pa                     =>   psi                 0.000145037743897283
!      percent/K              =>   percent/F           0.555555555555556
!      person/m2              =>   person/ft2          0.0928939733269818
!      s/m                    =>   s/ft                0.3048
!      V/K                    =>   V/F                 0.555555555555556
!      W                      =>   Btu/h               3.4121412858518
!      W/(m3/s)               =>   W/(ft3/min)         0.0004719475
!      W/K                    =>   Btu/h-F             1.89563404769544
!      W/m                    =>   Btu/h-ft            1.04072
!      W/m2                   =>   Btu/h-ft2           0.316957210776545
!      W/m2                   =>   W/ft2               0.09290304
!      W/m2-K                 =>   Btu/h-ft2-F         0.176110194261872
!      W/m2-K2                =>   Btu/h-ft2-F2        0.097826
!      W/m-K                  =>   Btu-in/h-ft2-F      6.93481276005548
!      W/m-K2                 =>   Btu/h-F2-ft         0.321418310071648
!      W/m-K3                 =>   Btu/h-F3-ft         0.178565727817582
!      W/person               =>   Btu/h-person        3.4121412858518
!
! Other conversions supported (needs the \\ip-units code)
!
!      kPa                    =>   inHg                0.29523
!      m                      =>   in                  39.3700787401575
!      m3/s                   =>   gal/min             15850.3222370511
!      Pa                     =>   ftH2O               0.00033455
!      Pa                     =>   inH2O               0.00401463
!      Pa                     =>   inHg                0.00029613
!      Pa                     =>   Pa                  1
!      W                      =>   W                   1
!      W/m2                   =>   W/m2                1
!      W/m-K                  =>   Btu/h-ft-F          0.577796066000163
!      W/person               =>   W/person            1
!
! Units fields that are not translated
!      $
!      1/hr
!      A
!      Ah
!      A/V
!      Availability
!      Control
!      cycles/hr
!      days
!      deg
!      dimensionless
!      eV
!      hr
!      J/J
!      kg/kg
!      kg-H2O/kg-air
!      kmol
!      kmol/s
!      m3/m3
!      minutes
!      Mode
!      ms
!      ohms
!      percent
!      ppm
!      rev/min
!      s
!      V
!      VA
!      W/m2 or deg C
!      W/m2, W or deg C
!      W/s
!      W/W
!      years
! **************************************************************************

Lead Input;

Simulation Data;

\\group Simulation Parameters

Version,
      \\memo Specifies the EnergyPlus version of the IDF file.
      \\unique-object
      \\format singleLine
  A1 ; \\Field Version Identifier
      \\required-field
      \\default 7.0

SimulationControl,
      \\unique-object
      \\memo Note that the following 3 fields are related to the Sizing:Zone, Sizing:System,
      \\memo and Sizing:Plant objects.  Having these fields set to Yes but no corresponding
      \\memo Sizing object will not cause the sizing to be done. However, having any of these
      \\memo fields set to No, the corresponding Sizing object is ignored.
      \\memo Note also, if you want to do system sizing, you must also do zone sizing in the same
      \\memo run or an error will result.
  A1, \\field Do Zone Sizing Calculation
      \\note If Yes, Zone sizing is accomplished from corresponding Sizing:Zone objects
      \\note and autosize fields.
      \\type choice
      \\key Yes
      \\key No
      \\default No
  A2, \\field Do System Sizing Calculation
      \\note If Yes, System sizing is accomplished from corresponding Sizing:System objects
      \\note and autosize fields.
      \\note If Yes, Zone sizing (previous field) must also be Yes.
      \\type choice
      \\key Yes
      \\key No
      \\default No
  A3, \\field Do Plant Sizing Calculation
      \\note If Yes, Plant sizing is accomplished from corresponding Sizing:Plant objects
      \\note and autosize fields.
      \\type choice
      \\key Yes
      \\key No
      \\default No
  A4, \\field Run Simulation for Sizing Periods
      \\note If Yes, SizingPeriod:* objects are executed and results from those may be displayed..
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A5; \\field Run Simulation for Weather File Run Periods
      \\note If Yes, RunPeriod:* objects are executed and results from those may be displayed..
      \\type choice
      \\key Yes
      \\key No
      \\default Yes

Building,
       \\memo Describes parameters that are used during the simulation
       \\memo of the building. There are necessary correlations between the entries for
       \\memo this object and some entries in the Site:WeatherStation and
       \\memo Site:HeightVariation objects, specifically the Terrain field.
       \\unique-object
       \\required-object
       \\min-fields 8
  A1 , \\field Name
       \\required-field
       \\retaincase
       \\default NONE
  N1 , \\field North Axis
       \\note degrees from true North
       \\units deg
       \\type real
       \\default 0.0
  A2 , \\field Terrain
       \\note  Country=FlatOpenCountry | Suburbs=CountryTownsSuburbs | City=CityCenter | Ocean=body of water (5km) | Urban=Urban-Industrial-Forest
       \\type choice
       \\key Country
       \\key Suburbs
       \\key City
       \\key Ocean
       \\key Urban
       \\default Suburbs
  N2 , \\field Loads Convergence Tolerance Value
       \\note Loads Convergence Tolerance Value is a fraction of load
       \\type real
       \\minimum> 0.0
       \\maximum .5
       \\default .04
  N3 , \\field Temperature Convergence Tolerance Value
       \\units deltaC
       \\type real
       \\minimum> 0.0
       \\maximum .5
       \\default .4
  A3 , \\field Solar Distribution
       \\note  MinimalShadowing | FullExterior | FullInteriorAndExterior | FullExteriorWithReflections | FullInteriorAndExteriorWithReflections
       \\type choice
       \\key MinimalShadowing
       \\key FullExterior
       \\key FullInteriorAndExterior
       \\key FullExteriorWithReflections
       \\key FullInteriorAndExteriorWithReflections
       \\default FullExterior
  N4 , \\field Maximum Number of Warmup Days
       \\note EnergyPlus will only use as many warmup days as needed to reach convergence tolerance.
       \\note This field's value should NOT be set less than 25.
       \\type integer
       \\minimum> 0
       \\default 25
  N5 ; \\field Minimum Number of Warmup Days
       \\note The minimum number of warmup days that produce enough temperature and flux history
       \\note to start EnergyPlus simulation for all reference buildings was suggested to be 6.
       \\note When this field is greater than the maximum warmup days defined previous field
       \\note the maximum number of warmup days will be reset to the minimum value entered here.
       \\note Warmup days will be set to be the value you entered when it is less than the default 6.
       \\type integer
       \\minimum> 0
       \\default 6

ShadowCalculation,
       \\unique-object
       \\memo This object is used to control details of the solar, shading, and daylighting models
  A1 , \\field Calculation Method
       \\note choose calculation method. note that TimestepFrequency is only needed for certain cases
       \\note and can increase execution time significantly.
       \\type choice
       \\key AverageOverDaysInFrequency
       \\key TimestepFrequency
       \\default AverageOverDaysInFrequency
  N1 , \\field Calculation Frequency
       \\required-field
       \\type integer
       \\minimum 1
       \\default 20
       \\note enter number of days
       \\note this field is only used if the previous field is set to AverageOverDaysInFrequency
       \\note 0=Use Default Periodic Calculation|<else> calculate every <value> day
       \\note only really applicable to RunPeriods
       \\note warning issued if >31
  N2 , \\field Maximum Figures in Shadow Overlap Calculations
       \\note Number of allowable figures in shadow overlap calculations
       \\type integer
       \\minimum 200
       \\default 15000
  A2 , \\field Polygon Clipping Algorithm
       \\note Advanced Feature.  Internal default is SutherlandHodgman
       \\note Refer to InputOutput Reference and Engineering Reference for more information
       \\type choice
       \\key ConvexWeilerAtherton
       \\key SutherlandHodgman
  A3 ; \\field Sky Diffuse Modeling Algorithm
       \\note Advanced Feature.  Internal default is SimpleSkyDiffuseModeling
       \\note If you have shading elements that change transmittance over the
       \\note year, you may wish to choose the detailed method.
       \\note Refer to InputOutput Reference and Engineering Reference for more information
       \\type choice
       \\key SimpleSkyDiffuseModeling
       \\key DetailedSkyDiffuseModeling


SurfaceConvectionAlgorithm:Inside,
       \\memo Default indoor surface heat transfer convection algorithm to be used for all zones
       \\unique-object
       \\format singleLine
  A1 ; \\field Algorithm
       \\required-field
       \\type choice
       \\key Simple
       \\key TARP
       \\key CeilingDiffuser
       \\key AdaptiveConvectionAlgorithm
       \\default TARP
       \\note Simple = constant value natural convection (ASHRAE)
       \\note TARP = variable natural convection based on temperature difference (ASHRAE, Walton)
       \\note CeilingDiffuser = ACH-based forced and mixed convection correlations
       \\note  for ceiling diffuser configuration with simple natural convection limit
       \\note AdaptiveConvectionAlgorithm = dynamic selection of convection models based on conditions

SurfaceConvectionAlgorithm:Outside,
       \\memo Default outside surface heat transfer convection algorithm to be used for all zones
       \\unique-object
       \\format singleLine
  A1 ; \\field Algorithm
       \\required-field
       \\type choice
       \\key SimpleCombined
       \\key TARP
       \\key MoWiTT
       \\key DOE-2
       \\key AdaptiveConvectionAlgorithm
       \\default DOE-2
       \\note SimpleCombined = Combined radiation and convection coefficient using simple ASHRAE model
       \\note TARP = correlation from models developed by ASHRAE, Walton, and Sparrow et. al.
       \\note MoWiTT = correlation from measurements by Klems and Yazdanian for smooth surfaces
       \\note DOE-2 = correlation from measurements by Klems and Yazdanian for rough surfaces
       \\note AdaptiveConvectionAlgorithm = dynamic selection of correlations based on conditions


HeatBalanceAlgorithm,
       \\memo Determines which Heat Balance Algorithm will be used ie.
       \\memo CTF (Conduction Transfer Functions),
       \\memo EMPD (Effective Moisture Penetration Depth with Conduction Transfer Functions).
       \\memo Advanced/Research Usage: CondFD (Conduction Finite Difference)
       \\memo Advanced/Research Usage: ConductionFiniteDifferenceSimplified
       \\memo Advanced/Research Usage: HAMT (Combined Heat And Moisture Finite Element)
       \\unique-object
       \\format singleLine
  A1 , \\field Algorithm
       \\required-field
       \\type choice
       \\key ConductionTransferFunction
       \\key MoisturePenetrationDepthConductionTransferFunction
       \\key ConductionFiniteDifference
       \\key CombinedHeatAndMoistureFiniteElement
       \\default ConductionTransferFunction
  N1 , \\field Surface Temperature Upper Limit
       \\type real
       \\minimum 200
       \\default 200
       \\units C
  N2 , \\field Minimum Surface Convection Heat Transfer Coefficient Value
       \\units W/m2-K
       \\default 0.1
       \\minimum> 0.0
  N3 ; \\field Maximum Surface Convection Heat Transfer Coefficient Value
       \\units W/m2-K
       \\default 1000
       \\minimum 1.0

HeatBalanceSettings:ConductionFiniteDifference,
       \\memo Determines settings for the Conduction Finite Difference
       \\memo algorithm for surface heat transfer modeling.
       \\unique-object
  A1 , \\field Difference Scheme
       \\type choice
       \\key CrankNicholsonSecondOrder
       \\key FullyImplicitFirstOrder
       \\default FullyImplicitFirstOrder
  N1 , \\field Space Discretization Constant
       \\note increase or decrease number of nodes
       \\type real
       \\default 3
  N2 , \\field Relaxation Factor
       \\type real
       \\default 1.0
       \\minimum 0.01
       \\maximum 1.0
  N3 ; \\field Inside Face Surface Temperature Convergence Criteria
       \\type real
       \\default 0.002
       \\minimum 1.0E-7
       \\maximum 0.01

ZoneAirHeatBalanceAlgorithm,
       \\memo Determines which algorithm will be used to solve the zone air heat balance.
       \\unique-object
       \\format singleLine
  A1 ; \\field Algorithm
       \\type choice
       \\key ThirdOrderBackwardDifference
       \\key AnalyticalSolution
       \\key EulerMethod
       \\default ThirdOrderBackwardDifference

ZoneAirContaminantBalance,
       \\memo Determines which contaminant concentration will be simulates.
       \\unique-object
       \\format singleLine
  A1 , \\field Carbon Dioxide Concentration
       \\note If Yes, CO2 simulation will be performed.
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A2 , \\field Outdoor Carbon Dioxide Schedule Name
       \\note Schedule values should be in parts per million (ppm)
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Generic Contaminant Concentration
       \\note If Yes, generic contaminant simulation will be performed.
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A4 ; \\field Outdoor Generic Contaminant Schedule Name
       \\note Schedule values should be generic contaminant concentration in parts per
       \\note million (ppm)
       \\type object-list
       \\object-list ScheduleNames

ZoneCapacitanceMultiplier:ResearchSpecial,
       \\unique-object
       \\format singleLine
       \\memo Multiplier altering the relative capacitance of the air compared to an empty zone
       \\min-fields 4
  N1 , \\field Temperature Capacity Multiplier
       \\type real
       \\default 1.0
       \\minimum> 0.0
       \\note Used to alter the capacitance of zone air with respect to heat or temperature
  N2 , \\field Humidity Capacity Multiplier
       \\type real
       \\default 1.0
       \\minimum> 0.0
       \\note Used to alter the capacitance of zone air with respect to moisture or humidity ratio
  N3 , \\field Carbon Dioxide Capacity Multiplier
       \\type real
       \\default 1.0
       \\minimum> 0.0
       \\note Used to alter the capacitance of zone air with respect to zone air carbob dioxide concentration
  N4 ; \\field Generic Contaminant Capacity Multiplier
       \\type real
       \\default 1.0
       \\minimum> 0.0
       \\note Used to alter the capacitance of zone air with respect to zone air generic contaminant concentration

Timestep,
       \\memo Specifies the "basic" timestep for the simulation. The
       \\memo value entered here is also known as the Zone Timestep.  This is used in
       \\memo the Zone Heat Balance Model calculation as the driving timestep for heat
       \\memo transfer and load calculations.
       \\unique-object
       \\format singleLine
  N1 ; \\field Number of Timesteps per Hour
       \\required-field
       \\note Number in hour: normal validity 4 to 60: 6 suggested
       \\note Must be evenly divisible into 60
       \\note Allowable values include 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, and 60
       \\note Normal 6 is mimimum as lower values may cause inaccuracies
       \\note A minimum value of 20 is suggested for both ConductionFiniteDifference
       \\note and CombinedHeatAndMoistureFiniteElement surface heat balance alogorithms
       \\note A minimum of 12 is suggested for simulations involving a Vegetated Roof (Material:RoofVegetation).
       \\default 6
       \\type integer
       \\minimum 1
       \\maximum 60

ConvergenceLimits,
       \\memo Specifies limits on HVAC system simulation timesteps and iterations.
       \\memo This item is an advanced feature that should be used only with caution.
       \\unique-object
  N1 , \\field Minimum System Timestep
       \\units minutes
       \\type integer
       \\note 0 sets the minimum to the zone timestep (ref: Timestep)
       \\note 1 is normal (ratchet down to 1 minute)
       \\note setting greater than zone timestep (in minutes) will effectively set to zone timestep
       \\minimum 0
       \\maximum 60
  N2 , \\field Maximum HVAC Iterations
       \\type integer
       \\default 20
       \\minimum 1
  N3 , \\field Minimum Plant Iterations
       \\note Controls the minimum number of plant system solver iterations within a single HVAC iteration
       \\note Larger values will increase runtime but might improve solution accuracy for complicated plant systems
       \\note Complex plants include: several interconnected loops, heat recovery, thermal load following generators, etc.
       \\type integer
       \\default 2
       \\minimum 1
  N4 ; \\field Maximum Plant Iterations
       \\note Controls the maximum number of plant system solver iterations within a single HVAC iteration
       \\note Smaller values might decrease runtime but could decrease solution accuracy for complicated plant systems
       \\type integer
       \\default 8
       \\minimum 2

ProgramControl,
       \\memo used to support various efforts in time reduction for simulation including threading
  N1 ; \\field Number of Threads Allowed
       \\type integer
       \\minimum 0
       \\note This is currently used only in the Interior Radiant Exchange module -- view factors on # surfaces
       \\note if value is 0, then maximum number allowed will be used.


\\group Compliance Objects

Compliance:Building,
       \\memo Building level inputs related to compliance to building standards, building codes, and beyond energy code programs.
       \\unique-object
       \\min-fields 1
  N1;  \\field Building Rotation for Appendix G
       \\note Additional degrees of rotation to be used with the requirement in ASHRAE Standard 90.1 Appendix G
       \\note that states that the baseline building should be rotated in four directions.
       \\units deg
       \\type real
       \\default 0.0

\\group Location and Climate

Site:Location,
       \\memo Specifies the building's location. Only one location is allowed.
       \\memo Weather data file location, if it exists, will override this object.
       \\unique-object
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\type  alpha
  N1 , \\field Latitude
       \\units deg
       \\minimum -90.0
       \\maximum +90.0
       \\default 0.0
       \\note + is North, - is South, degree minutes represented in decimal (i.e. 30 minutes is .5)
       \\type real
  N2 , \\field Longitude
       \\units deg
       \\minimum -180.0
       \\maximum +180.0
       \\default 0.0
       \\note - is West, + is East, degree minutes represented in decimal (i.e. 30 minutes is .5)
       \\type real
  N3 , \\field Time Zone
       \\note basic these limits on the WorldTimeZone Map (2003)
       \\units hr
       \\minimum -12.0
       \\maximum +14.0
       \\default 0.0
       \\note  Time relative to GMT. Decimal hours.
       \\type real
  N4 ; \\field Elevation
       \\units m
       \\minimum -300.0
       \\maximum< 8900.0
       \\default 0.0
       \\type real

SizingPeriod:DesignDay,
      \\memo The design day object creates the parameters for the program to create
      \\memo the 24 hour weather profile that can be used for sizing as well as
      \\memo running to test the other simulation parameters. Parameters in this
      \\memo include a date (month and day), a day type (which uses the appropriate
      \\memo schedules for either sizing or simple tests), min/max temperatures,
      \\memo wind speeds, and solar radiation values.
  A1, \\field Name
       \\type alpha
       \\required-field
       \\reference DesignDays
       \\reference RunPeriodsAndDesignDays
  N1,  \\field Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N2,  \\field Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
       \\note must be valid for Month field
  A2,  \\field Day Type
       \\required-field
       \\note Day Type selects the schedules appropriate for this design day
       \\type choice
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  N3,  \\field Maximum Dry-Bulb Temperature
       \\required-field
       \\units C
       \\minimum -70
       \\maximum 70
       \\type real
  N4,  \\field Daily Dry-Bulb Temperature Range
       \\note Must still produce appropriate maximum dry bulb (within range)
       \\note This field is not needed if Dry-Bulb Temperature Range Modifier Type
       \\note is "delta".
       \\units deltaC
       \\minimum 0
       \\default 0
       \\type real
  A3,  \\field Dry-Bulb Temperature Range Modifier Type
       \\note Type of modifier to the dry-bulb temperature calculated for the timestep
       \\type choice
       \\key MultiplierSchedule
       \\key DifferenceSchedule
       \\key DefaultMultipliers
       \\default DefaultMultipliers
  A4,  \\field Dry-Bulb Temperature Range Modifier Day Schedule Name
       \\type object-list
       \\object-list DayScheduleNames
       \\note Only used when previous field is "MultiplierSchedule" or "DifferenceSchedule".
       \\note For type "MultiplierSchedule"  the hour/time interval values should specify
       \\note   the fraction (0-1) of the dry-bulb temperature range to be subtracted
       \\note   from the maximum dry-bulb temperature for each timestep in the day
       \\note For type "DifferenceSchedule" the values should specify a number to be subtracted
       \\note   from the maximum dry-bulb temperature for each timestep in the day.
       \\note Note that numbers in the difference schedule cannot be negative as that
       \\note   would result in a higher maximum than the maximum previously specified.
  A5,  \\field Humidity Condition Type
       \\note values/schedules indicated here and in subsequent fields create the humidity
       \\note values in the 24 hour design day conditions profile.
       \\type choice
       \\key WetBulb
       \\key DewPoint
       \\key HumidityRatio
       \\key Enthalpy
       \\key RelativeHumiditySchedule
       \\key WetBulbProfileMultiplierSchedule
       \\key WetBulbProfileDifferenceSchedule
       \\key WetBulbProfileDefaultMultipliers
       \\default WetBulb
  N5,  \\field Wetbulb or DewPoint at Maximum Dry-Bulb
       \\note Wetbulb or dewpoint temperature coincident with the maximum temperature.
       \\note Required only if field Humidity Condition Type is "Wetbulb", "Dewpoint",
       \\note "WetBulbProfileMultiplierSchedule", "WetBulbProfileDifferenceSchedule",
       \\note or "WetBulbProfileDefaultMultipliers"
       \\type real
       \\units C
  A6,  \\field Humidity Condition Day Schedule Name
       \\type object-list
       \\object-list DayScheduleNames
       \\note Only used when Humidity Condition Type is "RelativeHumiditySchedule",
       \\note  "WetBulbProfileMultiplierSchedule", or "WetBulbProfileDifferenceSchedule"
       \\note For type "RelativeHumiditySchedule", the hour/time interval values should specify
       \\note   relative humidity (percent) from 0.0 to 100.0.
       \\note For type "WetBulbProfileMultiplierSchedule" the hour/time interval values should specify
       \\note   the fraction (0-1) of the wet-bulb temperature range to be subtracted from the
       \\note   maximum wet-bulb temperature for each timestep in the day (units = Fraction)
       \\note For type "WetBulbProfileDifferenceSchedule" the values should specify a number to be subtracted
       \\note   from the maximum wet-bulb temperature for each timestep in the day. (units = deltaC)
  N6,  \\field Humidity Ratio at Maximum Dry-Bulb
       \\note Humidity ratio coincident with the maximum temperature (constant humidity ratio throughout day).
       \\note Required only if field Humidity Condition Type is "HumidityRatio".
       \\type real
       \\units kgWater/kgDryAir
  N7,  \\field Enthalpy at Maximum Dry-Bulb  !will require units transition.
       \\note Enthalpy coincident with the maximum temperature.
       \\note Required only if field Humidity Condition Type is "Enthalpy".
       \\type real
       \\units J/kg
  N8,  \\field Daily Wet-Bulb Temperature Range
       \\units deltaC
       \\note Required only if Humidity Condition Type = "WetbulbProfileMultiplierSchedule" or
       \\note "WetBulbProfileDefaultMultipliers"
  N9,  \\field Barometric Pressure
       \\note This field's value is also checked against the calculated "standard barometric pressure"
       \\note for the location.  If out of range (>10%) or blank, then is replaced by standard value.
       \\units Pa
       \\minimum 31000
       \\maximum 120000
       \\type real
       \\ip-units inHg
  N10, \\field Wind Speed
       \\required-field
       \\units m/s
       \\minimum 0
       \\maximum 40
       \\ip-units miles/hr
       \\type real
  N11, \\field Wind Direction
       \\required-field
       \\units deg
       \\minimum 0
       \\maximum 360
       \\note North=0.0 East=90.0
       \\note 0 and 360 are the same direction.
       \\type real
  A7,  \\field Rain Indicator
       \\note Yes is raining (all day), No is not raining
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A8,  \\field Snow Indicator
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Yes is Snow on Ground, No is no Snow on Ground
  A9,  \\field Daylight Saving Time Indicator
       \\note Yes -- use schedules modified for Daylight Saving Time Schedules.
       \\note No - do not use schedules modified for Daylight Saving Time Schedules
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A10, \\field Solar Model Indicator
       \\type choice
       \\key ASHRAEClearSky
       \\key ZhangHuang
       \\key Schedule
       \\key ASHRAETau
       \\default ASHRAEClearSky
  A11, \\field Beam Solar Day Schedule Name
       \\note if Solar Model Indicator = Schedule, then beam schedule name (for day)
       \\type object-list
       \\object-list DayScheduleNames
  A12, \\field Diffuse Solar Day Schedule Name
       \\note if Solar Model Indicator = Schedule, then diffuse schedule name (for day)
       \\type object-list
       \\object-list DayScheduleNames
  N12, \\field ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub)
       \\units dimensionless
       \\note Required if Solar Model Indicator = ASHRAETau
       \\minimum 0
       \\maximum 1
       \\default 0
  N13, \\field ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud)
       \\units dimensionless
       \\note Required if Solar Model Indicator = ASHRAETau
       \\minimum 0
       \\maximum 3
       \\default 0
  N14; \\field Sky Clearness
       \\note Used if Sky Model Indicator = ASHRAEClearSky or ZhangHuang
       \\minimum 0.0
       \\maximum 1.2
       \\default 0.0
       \\note 0.0 is totally unclear, 1.0 is totally clear
       \\type real

SizingPeriod:WeatherFileDays,
       \\memo Use a weather file period for design sizing calculations.
  A1 , \\field Name
       \\reference DesignDays
       \\reference RunPeriodsAndDesignDays
       \\required-field
       \\note user supplied name for reporting
  N1 , \\field Begin Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N2 , \\field Begin Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  N3 , \\field End Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N4 , \\field End Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  A2 , \\field Day of Week for Start Day
       \\note  =[|Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|SummerDesignDay|WinterDesignDay|
       \\note    |CustomDay1|CustomDay2];
       \\note if you use SummerDesignDay or WinterDesignDay or the CustomDays then this will apply
       \\note to the whole period; other days (i.e., Monday) will signify a start day and
       \\note normal sequence ofsubsequent days
       \\default Monday
       \\type choice
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A3,  \\field Use Weather File Daylight Saving Period
       \\note If yes or blank, use daylight saving period as specified on Weatherfile.
       \\note If no, do not use the daylight saving period as specified on the Weatherfile.
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A4;  \\field Use Weather File Rain and Snow Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes

SizingPeriod:WeatherFileConditionType,
       \\memo Use a weather file period for design sizing calculations.
       \\memo EPW weather files are created with typical and extreme periods
       \\memo created heuristically from the weather file data.  For more
       \\memo details on these periods, see AuxiliaryPrograms document.
  A1 , \\field Name
       \\required-field
       \\reference DesignDays
       \\reference RunPeriodsAndDesignDays
       \\note user supplied name for reporting
  A2 , \\field Period Selection
       \\required-field
       \\retaincase
       \\note Following is a list of all possible types of Extreme and Typical periods that
       \\note might be identified in the Weather File. Not all possible types are available
       \\note for all weather files.
       \\type choice
       \\key SummerExtreme
       \\key SummerTypical
       \\key WinterExtreme
       \\key WinterTypical
       \\key AutumnTypical
       \\key SpringTypical
       \\key WetSeason
       \\key DrySeason
       \\key NoDrySeason
       \\key NoWetSeason
       \\key TropicalHot
       \\key TropicalCold
       \\key NoDrySeasonMax
       \\key NoDrySeasonMin
       \\key NoWetSeasonMax
       \\key NoWetSeasonMin
  A3 , \\field Day of Week for Start Day
       \\note  =[|Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|SummerDesignDay|WinterDesignDay|
       \\note    |CustomDay1|CustomDay2];
       \\note if you use SummerDesignDay or WinterDesignDay or the CustomDays then this will apply
       \\note to the whole period; other days (i.e., Monday) will signify a start day and
       \\note normal sequence ofsubsequent days
       \\default Monday
       \\type choice
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A4,  \\field Use Weather File Daylight Saving Period
       \\note If yes or blank, use daylight saving period as specified on Weatherfile.
       \\note If no, do not use the daylight saving period as specified on the Weatherfile.
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A5;  \\field Use Weather File Rain and Snow Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes

RunPeriod,
       \\memo Specified a range of dates and other parameters for a weather file simulation.
       \\memo Multiple run periods may be input, but they may not overlap.
       \\min-fields 11
  A1 , \\field Name
       \\reference RunPeriods
       \\reference RunPeriodsAndDesignDays
       \\note descriptive name (used in reporting mainly)
       \\note if blank, weather file title is used.  if not blank, must be unique
  N1 , \\field Begin Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N2 , \\field Begin Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  N3 , \\field End Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N4 , \\field End Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  A2 , \\field Day of Week for Start Day
       \\note  =<blank - use WeatherFile>|Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday];
       \\default UseWeatherFile
       \\type choice
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key UseWeatherFile
  A3,  \\field Use Weather File Holidays and Special Days
       \\note If yes or blank, use holidays as specified on Weatherfile.
       \\note If no, do not use the holidays specified on the Weatherfile.
       \\note Note: You can still specify holidays/special days using the RunPeriodControl:SpecialDays object(s).
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A4,  \\field Use Weather File Daylight Saving Period
       \\note If yes or blank, use daylight saving period as specified on Weatherfile.
       \\note If no, do not use the daylight saving period as specified on the Weatherfile.
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A5,  \\field Apply Weekend Holiday Rule
       \\note if yes and single day holiday falls on weekend, "holiday" occurs on following Monday
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A6,  \\field Use Weather File Rain Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
  A7,  \\field Use Weather File Snow Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
  N5,  \\field Number of Times Runperiod to be Repeated
       \\type integer
       \\minimum 1
       \\default 1
  A8,  \\field Increment Day of Week on repeat
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
  N6;  \\field Start Year
       \\note this is the start year for the start date.  If the leap year is "Yes" in the weather file header
       \\note (that is HOLIDAYS/SPECIAL DAYS header first field), then any year which is a leap year will assume
       \\note there will be a Feb 29. A repeat of this runperiod will automatically increment the year.

RunPeriod:CustomRange,
       \\memo run simulation for a custom created weather file
       \\min-fields 13
  A1 , \\field Name
       \\reference RunPeriods
       \\reference RunPeriodsAndDesignDays
       \\note descriptive name (used in reporting mainly)
       \\note if blank, weather file title is used.  if not blank, must be unique
  N1 , \\field Begin Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N2 , \\field Begin Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  N3,  \\field Begin Year
       \\required-field
       \\note must be start year of this date on weather file
  N4 , \\field End Month
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N5 , \\field End Day of Month
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  N6,  \\field End Year
       \\required-field
       \\note must be end year of this date on weather file
  A2 , \\field Day of Week for Start Day
       \\note  =<blank - use WeatherFile>|Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday];
       \\default UseWeatherFile
       \\type choice
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key UseWeatherFile
  A3,  \\field Use Weather File Holidays and Special Days
       \\note If yes or blank, use holidays as specified on Weatherfile.
       \\note If no, do not use the holidays specified on the Weatherfile.
       \\note Note: You can still specify holidays/special days using the RunPeriodControl:SpecialDays object(s).
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A4,  \\field Use Weather File Daylight Saving Period
       \\note If yes or blank, use daylight saving period as specified on Weatherfile.
       \\note If no, do not use the daylight saving period as specified on the Weatherfile.
       \\type choice
       \\default Yes
       \\key Yes
       \\key No
  A5,  \\field Apply Weekend Holiday Rule
       \\note if yes and single day holiday falls on weekend, "holiday" occurs on following Monday
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A6,  \\field Use Weather File Rain Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
  A7;  \\field Use Weather File Snow Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes

RunPeriodControl:SpecialDays,
     \\min-fields 4
     \\memo This object sets up holidays/special days to be used during weather file
     \\memo run periods.  (These are not used with SizingPeriod:* objects.)
     \\memo Depending on the value in the run period, days on the weather file may also
     \\memo be used.  However, the weather file specification will take precedence over
     \\memo any specification shown here.  (No error message on duplicate days or overlapping
     \\memo days).
 A1, \\field Name
     \\required-field
 A2, \\field Start Date
     \\required-field
     \\note  Dates can be several formats:
     \\note  <number>/<number>  (month/day)
     \\note  <number> <Month>
     \\note  <Month> <number>
     \\note <Nth> <Weekday> in <Month)
     \\note Last <WeekDay> in <Month>
     \\note <Month> can be January, February, March, April, May, June, July, August, September, October, November, December
     \\note Months can be the first 3 letters of the month
     \\note <Weekday> can be Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday
     \\note <Nth> can be 1 or 1st, 2 or 2nd, etc. up to 5(?)
 N1, \\field Duration
     \\units days
     \\minimum 1
     \\maximum 366
     \\default 1
 A3; \\field Special Day Type
       \\required-field
       \\note Special Day Type selects the schedules appropriate for each day so labeled
       \\type choice
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
       \\default Holiday

RunPeriodControl:DaylightSavingTime,
     \\unique-object
     \\min-fields 2
     \\memo This object sets up the daylight saving time period for any RunPeriod.
     \\memo Ignores any daylight saving time period on the weather file and uses this definition.
     \\memo These are not used with SizingPeriod:DesignDay objects.
     \\memo Use with SizingPeriod:WeatherFileDays object can be controlled in that object.
 A1, \\field Start Date
     \\required-field
 A2; \\field End Date
     \\required-field
     \\note  Dates can be several formats:
     \\note  <number>/<number>  (month/day)
     \\note  <number> <Month>
     \\note  <Month> <number>
     \\note <Nth> <Weekday> in <Month)
     \\note Last <WeekDay> in <Month>
     \\note <Month> can be January, February, March, April, May, June, July, August, September, October, November, December
     \\note Months can be the first 3 letters of the month
     \\note <Weekday> can be Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday
     \\note <Nth> can be 1 or 1st, 2 or 2nd, etc. up to 5(?)

WeatherProperty:SkyTemperature,
       \\memo This object is used to override internal sky temperature calculations.
  A1,  \\field Name
       \\note blank in this field will apply to all run periods (that is, all objects=
       \\note SizingPeriod:WeatherFileDays, SizingPeriod:WeatherFileConditionType or RunPeriod
       \\note otherwise, this name must match one of the environment object names.
       \\type object-list
       \\object-list RunPeriodsAndDesignDays
  A2,  \\field Calculation Type
       \\required-field
       \\type choice
       \\key ScheduleValue
       \\key DifferenceScheduleDryBulbValue
       \\key DifferenceScheduleDewPointValue
  A3;  \\field Schedule Name
       \\required-field
       \\note if name matches a SizingPeriod:DesignDay, put in a day schedule of this name
       \\note if name is for a SizingPeriod:WeatherFileDays, SizingPeriod:WeatherFileConditionType or
       \\note RunPeriod, put in a full year schedule that covers the appropriate days.
       \\type object-list
       \\object-list ScheduleAndDayScheduleNames

Site:WeatherStation,
       \\unique-object
       \\memo This object should only be used for non-standard weather data.  Standard weather data
       \\memo such as TMY2, IWEC, and ASHRAE design day data are all measured at the
       \\memo default conditions and do not require this object.
  N1 , \\field Wind Sensor Height Above Ground
       \\type real
       \\units m
       \\default 10.0
       \\minimum> 0.0
  N2 , \\field Wind Speed Profile Exponent
       \\type real
       \\default 0.14
       \\minimum 0.0
  N3 , \\field Wind Speed Profile Boundary Layer Thickness
       \\type real
       \\units m
       \\default 270.0
       \\minimum 0.0
  N4 ; \\field Air Temperature Sensor Height Above Ground
       \\type real
       \\units m
       \\default 1.5
       \\minimum 0.0

Site:HeightVariation,
       \\unique-object
       \\memo This object is used if the user requires advanced control over height-dependent
       \\memo variations in wind speed and temperature.  When this object is not present, the default model
       \\memo for temperature dependence on height is used, and the wind speed is modeled according
       \\memo to the Terrain field of the BUILDING object.
  N1 , \\field Wind Speed Profile Exponent
       \\note Set to zero for no wind speed dependence on height.
       \\type real
       \\default 0.22
       \\minimum 0.0
  N2 , \\field Wind Speed Profile Boundary Layer Thickness
       \\type real
       \\units m
       \\default 370.0
       \\minimum> 0.0
  N3 ; \\field Air Temperature Gradient Coefficient
       \\note Set to zero for no air temperature dependence on height.
       \\type real
       \\units K/m
       \\default 0.0065
       \\minimum 0.0

Site:GroundTemperature:BuildingSurface,
       \\memo These temperatures are specifically for those surfaces that have the outside environment
       \\memo of "Ground".  Documentation about what values these should be is located in the
       \\memo Auxiliary programs document (Ground Heat Transfer) as well as the InputOutput Reference.
       \\memo CAUTION - Do not use the "undisturbed" ground temperatures from the weather data.
       \\memo These values are too extreme for the soil under a conditioned building.
       \\memo For best results, use the Slab or Basement program to calculate custom monthly
       \\memo average ground temperatures (see Auxiliary Programs).  For typical commercial
       \\memo buildings in the USA, a reasonable default value is 2C less than the average indoor space temperature.
       \\unique-object
       \\min-fields 12
       \\format singleLine
  N1 , \\field January Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N2 , \\field February Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N3 , \\field March Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N4 , \\field April Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N5 , \\field May Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N6 , \\field June Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N7 , \\field July Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N8 , \\field August Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N9 , \\field September Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N10, \\field October Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N11, \\field November Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18
  N12; \\field December Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 18

Site:GroundTemperature:FCfactorMethod,
       \\memo These temperatures are specifically for underground walls and ground floors
       \\memo defined with the C-factor and F-factor methods, and should be close to the
       \\memo monthly average outdoor air temperature delayed by 3 months for the location.
       \\unique-object
       \\min-fields 12
       \\format singleLine
  N1 , \\field January Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N2 , \\field February Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N3 , \\field March Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N4 , \\field April Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N5 , \\field May Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N6 , \\field June Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N7 , \\field July Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N8 , \\field August Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N9 , \\field September Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N10, \\field October Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N11, \\field November Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N12; \\field December Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13

Site:GroundTemperature:Shallow,
       \\memo These temperatures are specifically for the Surface Ground Heat Exchanger and
       \\memo should probably be close to the average outdoor air temperature for the location.
       \\memo They are not used in other models.
       \\unique-object
       \\min-fields 12
       \\format singleLine
  N1 , \\field January Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N2 , \\field February Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N3 , \\field March Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N4 , \\field April Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N5 , \\field May Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N6 , \\field June Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N7 , \\field July Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N8 , \\field August Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N9 , \\field September Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N10, \\field October Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N11, \\field November Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13
  N12; \\field December Surface Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 13

Site:GroundTemperature:Deep,
       \\memo These temperatures are specifically for the ground heat exchangers that would use
       \\memo "deep" (3-4 m depth) ground temperatures for their heat source.
       \\memo They are not used in other models.
       \\unique-object
       \\min-fields 12
       \\format singleLine
  N1 , \\field January Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N2 , \\field February Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N3 , \\field March Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N4 , \\field April Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N5 , \\field May Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N6 , \\field June Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N7 , \\field July Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N8 , \\field August Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N9 , \\field September Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N10, \\field October Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N11, \\field November Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16
  N12; \\field December Deep Ground Temperature
       \\required-field
       \\units C
       \\type real
       \\default 16

Site:GroundReflectance,
        \\memo Specifies the ground reflectance values used to calculate ground reflected solar.
        \\memo The ground reflectance can be further modified when snow is on the ground
        \\memo by Site:GroundReflectance:SnowModifier.
  \\unique-object
  \\min-fields 12
  \\format singleLine
  N1 , \\field January Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N2 , \\field February Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N3 , \\field March Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N4 , \\field April Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N5 , \\field May Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N6 , \\field June Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N7 , \\field July Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N8 , \\field August Ground Reflectance
        \\default 0.2
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\units dimensionless
   N9 , \\field September Ground Reflectance
         \\default 0.2
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\units dimensionless
   N10 , \\field October Ground Reflectance
         \\default 0.2
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\units dimensionless
   N11 , \\field November Ground Reflectance
         \\default 0.2
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\units dimensionless
   N12 ; \\field December Ground Reflectance
         \\default 0.2
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\units dimensionless

Site:GroundReflectance:SnowModifier,
      \\memo Specifies ground reflectance multipliers when snow resident on the ground.
      \\memo These multipliers are applied to the "normal" ground reflectances specified
      \\memo in Site:GroundReflectance.
  N1, \\field Ground Reflected Solar Modifier
  \\minimum 0.0
  \\default 1.0
    \\note Value for modifying the "normal" ground reflectance when Snow is on ground
    \\note  when calculating the "Ground Reflected Solar Radiation Value"
    \\note  a value of 1.0 here uses the "normal" ground reflectance
    \\note  Ground Reflected Solar = (BeamSolar*CosSunZenith + DiffuseSolar)*GroundReflectance
    \\note    This would be further modified by the Snow Ground Reflectance Modifier when Snow was on the ground
    \\note    When Snow on ground, effective GroundReflectance is normal GroundReflectance*"Ground Reflectance Snow Modifier"
    \\note  Ground Reflectance achieved in this manner will be restricted to [0.0,1.0]
  N2; \\field Daylighting Ground Reflected Solar Modifier
  \\minimum 0.0
  \\default 1.0
    \\note Value for modifying the "normal" daylighting ground reflectance when Snow is on ground
    \\note  when calculating the "Ground Reflected Solar Radiation Value"
    \\note  a value of 1.0 here uses the "normal" ground reflectance
    \\note  Ground Reflected Solar = (BeamSolar*CosSunZenith + DiffuseSolar)*GroundReflectance
    \\note    This would be further modified by the Snow Ground Reflectance Modifier when Snow was on the ground
    \\note    When Snow on ground, effective GroundReflectance is normal GroundReflectance*"Daylighting Ground Reflectance Snow Modifier"
    \\note  Ground Reflectance achieved in this manner will be restricted to [0.0,1.0]

Site:WaterMainsTemperature,
       \\memo Used to calculate water mains temperatures delivered by underground water main pipes.
       \\memo Water mains temperatures are a function of outdoor climate conditions
       \\memo and vary with time of year.
  A1 , \\field Calculation Method
       \\required-field
       \\type choice
       \\key Schedule
       \\key Correlation
  A2 , \\field Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Annual Average Outdoor Air Temperature
       \\type real
       \\units C
  N2 ; \\field Maximum Difference In Monthly Average Outdoor Air Temperatures
       \\type real
       \\units deltaC
       \\minimum 0

Site:Precipitation,
      \\memo Used to describe the amount of water precipitation at the building site.
      \\memo Precipitation includes both rain and the equivalent water content of snow.
  A1, \\field Precipitation Model Type
      \\type choice
      \\key ScheduleAndDesignLevel
  N1, \\field Design Level for Total Annual Precipitation
      \\note meters of water per year used for design level
      \\units m/yr
  A2, \\field Precipitation Rates Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Schedule values in meters of water per hour
      \\note values should be non-negative
  N2; \\field Average Total Annual Precipitation
      \\note meters of water per year from average weather statistics
      \\minimum 0
      \\units m/yr

RoofIrrigation,
      \\memo Used to describe the amount of irrigation on the ecoroof surface over the course
      \\memo of the simulation runperiod.
  A1, \\field Irrigation Model Type
      \\type choice
      \\key Schedule
      \\key SmartSchedule
      \\note SmartSchedule will not allow irrigation when soil is already moist.
      \\note Current threshold set at 30% of saturation.
  A2, \\field Irrigation Rate Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Schedule values in meters of water per hour
      \\note values should be non-negative
  N1; \\field Irrigation Maximum Saturation Threshold
      \\note Used with SmartSchedule to set the saturation level at which no
      \\note irrigation is allowed.
      \\units percent
      \\minimum 0.0
      \\maximum 100.0
      \\default 40.0

\\group Schedules

ScheduleTypeLimits,
       \\memo ScheduleTypeLimits specifies the data types and limits for the values contained in schedules
  A1,  \\field Name
       \\required-field
       \\reference ScheduleTypeLimitsNames
       \\note used to validate schedule types in various schedule objects
  N1,  \\field Lower Limit Value
       \\note lower limit (real or integer) for the Schedule Type.  e.g. if fraction, this is 0.0
       \\unitsBasedOnField A3
  N2,  \\field Upper Limit Value
       \\note upper limit (real or integer) for the Schedule Type.  e.g. if fraction, this is 1.0
       \\unitsBasedOnField A3
  A2,  \\field Numeric Type
       \\note Numeric type is either Continuous (all numbers within the min and
       \\note max are valid or Discrete (only integer numbers between min and
       \\note max are valid.  (Could also allow REAL and INTEGER to mean the
       \\note same things)
       \\type choice
       \\key Continuous
       \\key Discrete
  A3;  \\field Unit Type
       \\note Temperature (C or F)
       \\note DeltaTemperature (C or F)
       \\note PrecipitationRate (m/hr or ft/hr)
       \\note Angle (degrees)
       \\note Convection Coefficient (W/m2-K or Btu/sqft-hr-F)
       \\note Activity Level (W/person)
       \\note Velocity (m/s or ft/min)
       \\note Capacity (W or Btu/h)
       \\note Power (W)
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key DeltaTemperature
       \\key PrecipitationRate
       \\key Angle
       \\key ConvectionCoefficient
       \\key ActivityLevel
       \\key Velocity
       \\key Capacity
       \\key Power
       \\key Availability
       \\key Percent
       \\key Control
       \\key Mode
       \\default Dimensionless

Schedule:Day:Hourly,
       \\min-fields 26
       \\memo A Schedule:Day:Hourly contains 24 values for each hour of the day.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DayScheduleNames
       \\reference ScheduleAndDayScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  N1 , \\field Hour 1
       \\type real
       \\default 0
  N2 , \\field Hour 2
       \\type real
       \\default 0
  N3 , \\field Hour 3
       \\type real
       \\default 0
  N4 , \\field Hour 4
       \\type real
       \\default 0
  N5 , \\field Hour 5
       \\type real
       \\default 0
  N6 , \\field Hour 6
       \\type real
       \\default 0
  N7 , \\field Hour 7
       \\type real
       \\default 0
  N8 , \\field Hour 8
       \\type real
       \\default 0
  N9 , \\field Hour 9
       \\type real
       \\default 0
  N10, \\field Hour 10
       \\type real
       \\default 0
  N11, \\field Hour 11
       \\type real
       \\default 0
  N12, \\field Hour 12
       \\type real
       \\default 0
  N13, \\field Hour 13
       \\type real
       \\default 0
  N14, \\field Hour 14
       \\type real
       \\default 0
  N15, \\field Hour 15
       \\type real
       \\default 0
  N16, \\field Hour 16
       \\type real
       \\default 0
  N17, \\field Hour 17
       \\type real
       \\default 0
  N18, \\field Hour 18
       \\type real
       \\default 0
  N19, \\field Hour 19
       \\type real
       \\default 0
  N20, \\field Hour 20
       \\type real
       \\default 0
  N21, \\field Hour 21
       \\type real
       \\default 0
  N22, \\field Hour 22
       \\type real
       \\default 0
  N23, \\field Hour 23
       \\type real
       \\default 0
  N24; \\field Hour 24
       \\type real
       \\default 0

Schedule:Day:Interval,
       \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
       \\memo A Schedule:Day:Interval contains a full day of values with specified end times for each value
       \\memo Currently, is set up to allow for 10 minute intervals for an entire day.
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DayScheduleNames
       \\reference ScheduleAndDayScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field Interpolate to Timestep
       \\note when the interval does not match the user specified timestep a Yes choice will average between the intervals request (to
       \\note timestep resolution.  a No choice will use the interval value at the simulation timestep without regard to if it matches
       \\note the boundary or not.
       \\type choice
       \\key Yes
       \\key No
       \\default No
 A4  , \\field Time 1
       \\begin-extensible
       \\note "until" includes the time entered.
       \\units hh:mm
 N1  , \\field Value Until Time 1
 A5  , \\field Time 2
       \\note "until" includes the time entered.
       \\units hh:mm
 N2  , \\field Value Until Time 2
 A6  , \\field Time 3
       \\note "until" includes the time entered.
       \\units hh:mm
 N3  , \\field Value Until Time 3
 A7  , \\field Time 4
       \\note "until" includes the time entered.
       \\units hh:mm
 N4  , \\field Value Until Time 4
 A8  , \\field Time 5
       \\note "until" includes the time entered.
       \\units hh:mm
 N5  , \\field Value Until Time 5
 A9  , \\field Time 6
       \\note "until" includes the time entered.
       \\units hh:mm
 N6  , \\field Value Until Time 6
 A10 , \\field Time 7
       \\note "until" includes the time entered.
       \\units hh:mm
 N7  , \\field Value Until Time 7
 A11 , \\field Time 8
       \\note "until" includes the time entered.
       \\units hh:mm
 N8  , \\field Value Until Time 8
 A12 , \\field Time 9
       \\note "until" includes the time entered.
       \\units hh:mm
 N9  , \\field Value Until Time 9
 A13 , \\field Time 10
       \\note "until" includes the time entered.
       \\units hh:mm
 N10 , \\field Value Until Time 10
 A14 , \\field Time 11
       \\note "until" includes the time entered.
       \\units hh:mm
 N11 , \\field Value Until Time 11
 A15 , \\field Time 12
       \\note "until" includes the time entered.
       \\units hh:mm
 N12 , \\field Value Until Time 12
 A16 , \\field Time 13
       \\note "until" includes the time entered.
       \\units hh:mm
 N13 , \\field Value Until Time 13
 A17 , \\field Time 14
       \\note "until" includes the time entered.
       \\units hh:mm
 N14 , \\field Value Until Time 14
 A18 , \\field Time 15
       \\note "until" includes the time entered.
       \\units hh:mm
 N15 , \\field Value Until Time 15
 A19 , \\field Time 16
       \\note "until" includes the time entered.
       \\units hh:mm
 N16 , \\field Value Until Time 16
 A20 , \\field Time 17
       \\note "until" includes the time entered.
       \\units hh:mm
 N17 , \\field Value Until Time 17
 A21 , \\field Time 18
       \\note "until" includes the time entered.
       \\units hh:mm
 N18 , \\field Value Until Time 18
 A22 , \\field Time 19
       \\note "until" includes the time entered.
       \\units hh:mm
 N19 , \\field Value Until Time 19
 A23 , \\field Time 20
       \\note "until" includes the time entered.
       \\units hh:mm
 N20 , \\field Value Until Time 20
 A24 , \\field Time 21
       \\note "until" includes the time entered.
       \\units hh:mm
 N21 , \\field Value Until Time 21
 A25 , \\field Time 22
       \\note "until" includes the time entered.
       \\units hh:mm
 N22 , \\field Value Until Time 22
 A26 , \\field Time 23
       \\note "until" includes the time entered.
       \\units hh:mm
 N23 , \\field Value Until Time 23
 A27 , \\field Time 24
       \\note "until" includes the time entered.
       \\units hh:mm
 N24 , \\field Value Until Time 24
 A28 , \\field Time 25
       \\note "until" includes the time entered.
       \\units hh:mm
 N25 , \\field Value Until Time 25
 A29 , \\field Time 26
       \\note "until" includes the time entered.
       \\units hh:mm
 N26 , \\field Value Until Time 26
 A30 , \\field Time 27
       \\note "until" includes the time entered.
       \\units hh:mm
 N27 , \\field Value Until Time 27
 A31 , \\field Time 28
       \\note "until" includes the time entered.
       \\units hh:mm
 N28 , \\field Value Until Time 28
 A32 , \\field Time 29
       \\note "until" includes the time entered.
       \\units hh:mm
 N29 , \\field Value Until Time 29
 A33 , \\field Time 30
       \\note "until" includes the time entered.
       \\units hh:mm
 N30 , \\field Value Until Time 30
 A34 , \\field Time 31
       \\note "until" includes the time entered.
       \\units hh:mm
 N31 , \\field Value Until Time 31
 A35 , \\field Time 32
       \\note "until" includes the time entered.
       \\units hh:mm
 N32 , \\field Value Until Time 32
 A36 , \\field Time 33
       \\note "until" includes the time entered.
       \\units hh:mm
 N33 , \\field Value Until Time 33
 A37 , \\field Time 34
       \\note "until" includes the time entered.
       \\units hh:mm
 N34 , \\field Value Until Time 34
 A38 , \\field Time 35
       \\note "until" includes the time entered.
       \\units hh:mm
 N35 , \\field Value Until Time 35
 A39 , \\field Time 36
       \\note "until" includes the time entered.
       \\units hh:mm
 N36 , \\field Value Until Time 36
 A40 , \\field Time 37
       \\note "until" includes the time entered.
       \\units hh:mm
 N37 , \\field Value Until Time 37
 A41 , \\field Time 38
       \\note "until" includes the time entered.
       \\units hh:mm
 N38 , \\field Value Until Time 38
 A42 , \\field Time 39
       \\note "until" includes the time entered.
       \\units hh:mm
 N39 , \\field Value Until Time 39
 A43 , \\field Time 40
       \\note "until" includes the time entered.
       \\units hh:mm
 N40 , \\field Value Until Time 40
 A44 , \\field Time 41
       \\note "until" includes the time entered.
       \\units hh:mm
 N41 , \\field Value Until Time 41
 A45 , \\field Time 42
       \\note "until" includes the time entered.
       \\units hh:mm
 N42 , \\field Value Until Time 42
 A46 , \\field Time 43
       \\note "until" includes the time entered.
       \\units hh:mm
 N43 , \\field Value Until Time 43
 A47 , \\field Time 44
       \\note "until" includes the time entered.
       \\units hh:mm
 N44 , \\field Value Until Time 44
 A48 , \\field Time 45
       \\note "until" includes the time entered.
       \\units hh:mm
 N45 , \\field Value Until Time 45
 A49 , \\field Time 46
       \\note "until" includes the time entered.
       \\units hh:mm
 N46 , \\field Value Until Time 46
 A50 , \\field Time 47
       \\note "until" includes the time entered.
       \\units hh:mm
 N47 , \\field Value Until Time 47
 A51 , \\field Time 48
       \\note "until" includes the time entered.
       \\units hh:mm
 N48 , \\field Value Until Time 48
 A52 , \\field Time 49
       \\note "until" includes the time entered.
       \\units hh:mm
 N49 , \\field Value Until Time 49
 A53 , \\field Time 50
       \\note "until" includes the time entered.
       \\units hh:mm
 N50 , \\field Value Until Time 50
 A54 , \\field Time 51
       \\note "until" includes the time entered.
       \\units hh:mm
 N51 , \\field Value Until Time 51
 A55 , \\field Time 52
       \\note "until" includes the time entered.
       \\units hh:mm
 N52 , \\field Value Until Time 52
 A56 , \\field Time 53
       \\note "until" includes the time entered.
       \\units hh:mm
 N53 , \\field Value Until Time 53
 A57 , \\field Time 54
       \\note "until" includes the time entered.
       \\units hh:mm
 N54 , \\field Value Until Time 54
 A58 , \\field Time 55
       \\note "until" includes the time entered.
       \\units hh:mm
 N55 , \\field Value Until Time 55
 A59 , \\field Time 56
       \\note "until" includes the time entered.
       \\units hh:mm
 N56 , \\field Value Until Time 56
 A60 , \\field Time 57
       \\note "until" includes the time entered.
       \\units hh:mm
 N57 , \\field Value Until Time 57
 A61 , \\field Time 58
       \\note "until" includes the time entered.
       \\units hh:mm
 N58 , \\field Value Until Time 58
 A62 , \\field Time 59
       \\note "until" includes the time entered.
       \\units hh:mm
 N59 , \\field Value Until Time 59
 A63 , \\field Time 60
       \\note "until" includes the time entered.
       \\units hh:mm
 N60 , \\field Value Until Time 60
 A64 , \\field Time 61
       \\note "until" includes the time entered.
       \\units hh:mm
 N61 , \\field Value Until Time 61
 A65 , \\field Time 62
       \\note "until" includes the time entered.
       \\units hh:mm
 N62 , \\field Value Until Time 62
 A66 , \\field Time 63
       \\note "until" includes the time entered.
       \\units hh:mm
 N63 , \\field Value Until Time 63
 A67 , \\field Time 64
       \\note "until" includes the time entered.
       \\units hh:mm
 N64 , \\field Value Until Time 64
 A68 , \\field Time 65
       \\note "until" includes the time entered.
       \\units hh:mm
 N65 , \\field Value Until Time 65
 A69 , \\field Time 66
       \\note "until" includes the time entered.
       \\units hh:mm
 N66 , \\field Value Until Time 66
 A70 , \\field Time 67
       \\note "until" includes the time entered.
       \\units hh:mm
 N67 , \\field Value Until Time 67
 A71 , \\field Time 68
       \\note "until" includes the time entered.
       \\units hh:mm
 N68 , \\field Value Until Time 68
 A72 , \\field Time 69
       \\note "until" includes the time entered.
       \\units hh:mm
 N69 , \\field Value Until Time 69
 A73 , \\field Time 70
       \\note "until" includes the time entered.
       \\units hh:mm
 N70 , \\field Value Until Time 70
 A74 , \\field Time 71
       \\note "until" includes the time entered.
       \\units hh:mm
 N71 , \\field Value Until Time 71
 A75 , \\field Time 72
       \\note "until" includes the time entered.
       \\units hh:mm
 N72 , \\field Value Until Time 72
 A76 , \\field Time 73
       \\note "until" includes the time entered.
       \\units hh:mm
 N73 , \\field Value Until Time 73
 A77 , \\field Time 74
       \\note "until" includes the time entered.
       \\units hh:mm
 N74 , \\field Value Until Time 74
 A78 , \\field Time 75
       \\note "until" includes the time entered.
       \\units hh:mm
 N75 , \\field Value Until Time 75
 A79 , \\field Time 76
       \\note "until" includes the time entered.
       \\units hh:mm
 N76 , \\field Value Until Time 76
 A80 , \\field Time 77
       \\note "until" includes the time entered.
       \\units hh:mm
 N77 , \\field Value Until Time 77
 A81 , \\field Time 78
       \\note "until" includes the time entered.
       \\units hh:mm
 N78 , \\field Value Until Time 78
 A82 , \\field Time 79
       \\note "until" includes the time entered.
       \\units hh:mm
 N79 , \\field Value Until Time 79
 A83 , \\field Time 80
       \\note "until" includes the time entered.
       \\units hh:mm
 N80 , \\field Value Until Time 80
 A84 , \\field Time 81
       \\note "until" includes the time entered.
       \\units hh:mm
 N81 , \\field Value Until Time 81
 A85 , \\field Time 82
       \\note "until" includes the time entered.
       \\units hh:mm
 N82 , \\field Value Until Time 82
 A86 , \\field Time 83
       \\note "until" includes the time entered.
       \\units hh:mm
 N83 , \\field Value Until Time 83
 A87 , \\field Time 84
       \\note "until" includes the time entered.
       \\units hh:mm
 N84 , \\field Value Until Time 84
 A88 , \\field Time 85
       \\note "until" includes the time entered.
       \\units hh:mm
 N85 , \\field Value Until Time 85
 A89 , \\field Time 86
       \\note "until" includes the time entered.
       \\units hh:mm
 N86 , \\field Value Until Time 86
 A90 , \\field Time 87
       \\note "until" includes the time entered.
       \\units hh:mm
 N87 , \\field Value Until Time 87
 A91 , \\field Time 88
       \\note "until" includes the time entered.
       \\units hh:mm
 N88 , \\field Value Until Time 88
 A92 , \\field Time 89
       \\note "until" includes the time entered.
       \\units hh:mm
 N89 , \\field Value Until Time 89
 A93 , \\field Time 90
       \\note "until" includes the time entered.
       \\units hh:mm
 N90 , \\field Value Until Time 90
 A94 , \\field Time 91
       \\note "until" includes the time entered.
       \\units hh:mm
 N91 , \\field Value Until Time 91
 A95 , \\field Time 92
       \\note "until" includes the time entered.
       \\units hh:mm
 N92 , \\field Value Until Time 92
 A96 , \\field Time 93
       \\note "until" includes the time entered.
       \\units hh:mm
 N93 , \\field Value Until Time 93
 A97 , \\field Time 94
       \\note "until" includes the time entered.
       \\units hh:mm
 N94 , \\field Value Until Time 94
 A98 , \\field Time 95
       \\note "until" includes the time entered.
       \\units hh:mm
 N95 , \\field Value Until Time 95
 A99 , \\field Time 96
       \\note "until" includes the time entered.
       \\units hh:mm
 N96 , \\field Value Until Time 96
 A100, \\field Time 97
       \\note "until" includes the time entered.
       \\units hh:mm
 N97 , \\field Value Until Time 97
 A101, \\field Time 98
       \\note "until" includes the time entered.
       \\units hh:mm
 N98 , \\field Value Until Time 98
 A102, \\field Time 99
       \\note "until" includes the time entered.
       \\units hh:mm
 N99 , \\field Value Until Time 99
 A103, \\field Time 100
       \\note "until" includes the time entered.
       \\units hh:mm
 N100, \\field Value Until Time 100
 A104, \\field Time 101
       \\note "until" includes the time entered.
       \\units hh:mm
 N101, \\field Value Until Time 101
 A105, \\field Time 102
       \\note "until" includes the time entered.
       \\units hh:mm
 N102, \\field Value Until Time 102
 A106, \\field Time 103
       \\note "until" includes the time entered.
       \\units hh:mm
 N103, \\field Value Until Time 103
 A107, \\field Time 104
       \\note "until" includes the time entered.
       \\units hh:mm
 N104, \\field Value Until Time 104
 A108, \\field Time 105
       \\note "until" includes the time entered.
       \\units hh:mm
 N105, \\field Value Until Time 105
 A109, \\field Time 106
       \\note "until" includes the time entered.
       \\units hh:mm
 N106, \\field Value Until Time 106
 A110, \\field Time 107
       \\note "until" includes the time entered.
       \\units hh:mm
 N107, \\field Value Until Time 107
 A111, \\field Time 108
       \\note "until" includes the time entered.
       \\units hh:mm
 N108, \\field Value Until Time 108
 A112, \\field Time 109
       \\note "until" includes the time entered.
       \\units hh:mm
 N109, \\field Value Until Time 109
 A113, \\field Time 110
       \\note "until" includes the time entered.
       \\units hh:mm
 N110, \\field Value Until Time 110
 A114, \\field Time 111
       \\note "until" includes the time entered.
       \\units hh:mm
 N111, \\field Value Until Time 111
 A115, \\field Time 112
       \\note "until" includes the time entered.
       \\units hh:mm
 N112, \\field Value Until Time 112
 A116, \\field Time 113
       \\note "until" includes the time entered.
       \\units hh:mm
 N113, \\field Value Until Time 113
 A117, \\field Time 114
       \\note "until" includes the time entered.
       \\units hh:mm
 N114, \\field Value Until Time 114
 A118, \\field Time 115
       \\note "until" includes the time entered.
       \\units hh:mm
 N115, \\field Value Until Time 115
 A119, \\field Time 116
       \\note "until" includes the time entered.
       \\units hh:mm
 N116, \\field Value Until Time 116
 A120, \\field Time 117
       \\note "until" includes the time entered.
       \\units hh:mm
 N117, \\field Value Until Time 117
 A121, \\field Time 118
       \\note "until" includes the time entered.
       \\units hh:mm
 N118, \\field Value Until Time 118
 A122, \\field Time 119
       \\note "until" includes the time entered.
       \\units hh:mm
 N119, \\field Value Until Time 119
 A123, \\field Time 120
       \\note "until" includes the time entered.
       \\units hh:mm
 N120, \\field Value Until Time 120
 A124, \\field Time 121
       \\note "until" includes the time entered.
       \\units hh:mm
 N121, \\field Value Until Time 121
 A125, \\field Time 122
       \\note "until" includes the time entered.
       \\units hh:mm
 N122, \\field Value Until Time 122
 A126, \\field Time 123
       \\note "until" includes the time entered.
       \\units hh:mm
 N123, \\field Value Until Time 123
 A127, \\field Time 124
       \\note "until" includes the time entered.
       \\units hh:mm
 N124, \\field Value Until Time 124
 A128, \\field Time 125
       \\note "until" includes the time entered.
       \\units hh:mm
 N125, \\field Value Until Time 125
 A129, \\field Time 126
       \\note "until" includes the time entered.
       \\units hh:mm
 N126, \\field Value Until Time 126
 A130, \\field Time 127
       \\note "until" includes the time entered.
       \\units hh:mm
 N127, \\field Value Until Time 127
 A131, \\field Time 128
       \\note "until" includes the time entered.
       \\units hh:mm
 N128, \\field Value Until Time 128
 A132, \\field Time 129
       \\note "until" includes the time entered.
       \\units hh:mm
 N129, \\field Value Until Time 129
 A133, \\field Time 130
       \\note "until" includes the time entered.
       \\units hh:mm
 N130, \\field Value Until Time 130
 A134, \\field Time 131
       \\note "until" includes the time entered.
       \\units hh:mm
 N131, \\field Value Until Time 131
 A135, \\field Time 132
       \\note "until" includes the time entered.
       \\units hh:mm
 N132, \\field Value Until Time 132
 A136, \\field Time 133
       \\note "until" includes the time entered.
       \\units hh:mm
 N133, \\field Value Until Time 133
 A137, \\field Time 134
       \\note "until" includes the time entered.
       \\units hh:mm
 N134, \\field Value Until Time 134
 A138, \\field Time 135
       \\note "until" includes the time entered.
       \\units hh:mm
 N135, \\field Value Until Time 135
 A139, \\field Time 136
       \\note "until" includes the time entered.
       \\units hh:mm
 N136, \\field Value Until Time 136
 A140, \\field Time 137
       \\note "until" includes the time entered.
       \\units hh:mm
 N137, \\field Value Until Time 137
 A141, \\field Time 138
       \\note "until" includes the time entered.
       \\units hh:mm
 N138, \\field Value Until Time 138
 A142, \\field Time 139
       \\note "until" includes the time entered.
       \\units hh:mm
 N139, \\field Value Until Time 139
 A143, \\field Time 140
       \\note "until" includes the time entered.
       \\units hh:mm
 N140, \\field Value Until Time 140
 A144, \\field Time 141
       \\note "until" includes the time entered.
       \\units hh:mm
 N141, \\field Value Until Time 141
 A145, \\field Time 142
       \\note "until" includes the time entered.
       \\units hh:mm
 N142, \\field Value Until Time 142
 A146, \\field Time 143
       \\note "until" includes the time entered.
       \\units hh:mm
 N143, \\field Value Until Time 143
 A147, \\field Time 144
       \\note "until" includes the time entered.
       \\units hh:mm
 N144; \\field Value Until Time 144


Schedule:Day:List,
      \\memo Schedule:Day:List will allow the user to list 24 hours worth of values, which can be sub-hourly in nature.
      \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DayScheduleNames
       \\reference ScheduleAndDayScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field Interpolate to Timestep
       \\note when the interval does not match the user specified timestep a "Yes" choice will average between the intervals request (to
       \\note timestep resolution.  a "No" choice will use the interval value at the simulation timestep without regard to if it matches
       \\note the boundary or not.
       \\type choice
       \\key Yes
       \\key No
       \\default No
  N1 , \\field Minutes per Item
       \\note Must be evenly divisible into 60
       \\type integer
       \\minimum 1
       \\maximum 60
  N2,  \\field Value 1
       \\default 0.0
        N3,N4,         N5,N6,N7,N8,         N9,N10,N11,N12,      N13,N14,N15,N16,     N17,N18,N19,N20, \\note fields as indicated
  N21,N22,N23,N24,     N25,N26,N27,N28,     N29,N30,N31,N32,     N33,N34,N35,N36,     N37,N38,N39,N40, \\note fields as indicated
  N41,N42,N43,N44,     N45,N46,N47,N48,     N49,N50,N51,N52,     N53,N54,N55,N56,     N57,N58,N59,N60, \\note fields as indicated
  N61,N62,N63,N64,     N65,N66,N67,N68,     N69,N70,N71,N72,     N73,N74,N75,N76,     N77,N78,N79,N80, \\note fields as indicated
  N81,N82,N83,N84,     N85,N86,N87,N88,     N89,N90,N91,N92,     N93,N94,N95,N96,     N97,N98,N99,N100, \\note fields as indicated

  N101,N102,N103,N104, N105,N106,N107,N108, N109,N110,N111,N112, N113,N114,N115,N116, N117,N118,N119,N120, \\note fields as indicated
  N121,N122,N123,N124, N125,N126,N127,N128, N129,N130,N131,N132, N133,N134,N135,N136, N137,N138,N139,N140, \\note fields as indicated
  N141,N142,N143,N144, N145,N146,N147,N148, N149,N150,N151,N152, N153,N154,N155,N156, N157,N158,N159,N160, \\note fields as indicated
  N161,N162,N163,N164, N165,N166,N167,N168, N169,N170,N171,N172, N173,N174,N175,N176, N177,N178,N179,N180, \\note fields as indicated
  N181,N182,N183,N184, N185,N186,N187,N188, N189,N190,N191,N192, N193,N194,N195,N196, N197,N198,N199,N200, \\note fields as indicated

  N201,N202,N203,N204, N205,N206,N207,N208, N209,N210,N211,N212, N213,N214,N215,N216, N217,N218,N219,N220, \\note fields as indicated
  N221,N222,N223,N224, N225,N226,N227,N228, N229,N230,N231,N232, N233,N234,N235,N236, N237,N238,N239,N240, \\note fields as indicated
  N241,N242,N243,N244, N245,N246,N247,N248, N249,N250,N251,N252, N253,N254,N255,N256, N257,N258,N259,N260, \\note fields as indicated
  N261,N262,N263,N264, N265,N266,N267,N268, N269,N270,N271,N272, N273,N274,N275,N276, N277,N278,N279,N280, \\note fields as indicated
  N281,N282,N283,N284, N285,N286,N287,N288, N289,N290,N291,N292, N293,N294,N295,N296, N297,N298,N299,N300, \\note fields as indicated

  N301,N302,N303,N304, N305,N306,N307,N308, N309,N310,N311,N312, N313,N314,N315,N316, N317,N318,N319,N320, \\note fields as indicated
  N321,N322,N323,N324, N325,N326,N327,N328, N329,N330,N331,N332, N333,N334,N335,N336, N337,N338,N339,N340, \\note fields as indicated
  N341,N342,N343,N344, N345,N346,N347,N348, N349,N350,N351,N352, N353,N354,N355,N356, N357,N358,N359,N360, \\note fields as indicated
  N361,N362,N363,N364, N365,N366,N367,N368, N369,N370,N371,N372, N373,N374,N375,N376, N377,N378,N379,N380, \\note fields as indicated
  N381,N382,N383,N384, N385,N386,N387,N388, N389,N390,N391,N392, N393,N394,N395,N396, N397,N398,N399,N400, \\note fields as indicated

  N401,N402,N403,N404, N405,N406,N407,N408, N409,N410,N411,N412, N413,N414,N415,N416, N417,N418,N419,N420, \\note fields as indicated
  N421,N422,N423,N424, N425,N426,N427,N428, N429,N430,N431,N432, N433,N434,N435,N436, N437,N438,N439,N440, \\note fields as indicated
  N441,N442,N443,N444, N445,N446,N447,N448, N449,N450,N451,N452, N453,N454,N455,N456, N457,N458,N459,N460, \\note fields as indicated
  N461,N462,N463,N464, N465,N466,N467,N468, N469,N470,N471,N472, N473,N474,N475,N476, N477,N478,N479,N480, \\note fields as indicated
  N481,N482,N483,N484, N485,N486,N487,N488, N489,N490,N491,N492, N493,N494,N495,N496, N497,N498,N499,N500, \\note fields as indicated

  N501,N502,N503,N504, N505,N506,N507,N508, N509,N510,N511,N512, N513,N514,N515,N516, N517,N518,N519,N520, \\note fields as indicated
  N521,N522,N523,N524, N525,N526,N527,N528, N529,N530,N531,N532, N533,N534,N535,N536, N537,N538,N539,N540, \\note fields as indicated
  N541,N542,N543,N544, N545,N546,N547,N548, N549,N550,N551,N552, N553,N554,N555,N556, N557,N558,N559,N560, \\note fields as indicated
  N561,N562,N563,N564, N565,N566,N567,N568, N569,N570,N571,N572, N573,N574,N575,N576, N577,N578,N579,N580, \\note fields as indicated
  N581,N582,N583,N584, N585,N586,N587,N588, N589,N590,N591,N592, N593,N594,N595,N596, N597,N598,N599,N600, \\note fields as indicated

  N601,N602,N603,N604, N605,N606,N607,N608, N609,N610,N611,N612, N613,N614,N615,N616, N617,N618,N619,N620, \\note fields as indicated
  N621,N622,N623,N624, N625,N626,N627,N628, N629,N630,N631,N632, N633,N634,N635,N636, N637,N638,N639,N640, \\note fields as indicated
  N641,N642,N643,N644, N645,N646,N647,N648, N649,N650,N651,N652, N653,N654,N655,N656, N657,N658,N659,N660, \\note fields as indicated
  N661,N662,N663,N664, N665,N666,N667,N668, N669,N670,N671,N672, N673,N674,N675,N676, N677,N678,N679,N680, \\note fields as indicated
  N681,N682,N683,N684, N685,N686,N687,N688, N689,N690,N691,N692, N693,N694,N695,N696, N697,N698,N699,N700, \\note fields as indicated

  N701,N702,N703,N704, N705,N706,N707,N708, N709,N710,N711,N712, N713,N714,N715,N716, N717,N718,N719,N720, \\note fields as indicated
  N721,N722,N723,N724, N725,N726,N727,N728, N729,N730,N731,N732, N733,N734,N735,N736, N737,N738,N739,N740, \\note fields as indicated
  N741,N742,N743,N744, N745,N746,N747,N748, N749,N750,N751,N752, N753,N754,N755,N756, N757,N758,N759,N760, \\note fields as indicated
  N761,N762,N763,N764, N765,N766,N767,N768, N769,N770,N771,N772, N773,N774,N775,N776, N777,N778,N779,N780, \\note fields as indicated
  N781,N782,N783,N784, N785,N786,N787,N788, N789,N790,N791,N792, N793,N794,N795,N796, N797,N798,N799,N800, \\note fields as indicated

  N801,N802,N803,N804, N805,N806,N807,N808, N809,N810,N811,N812, N813,N814,N815,N816, N817,N818,N819,N820, \\note fields as indicated
  N821,N822,N823,N824, N825,N826,N827,N828, N829,N830,N831,N832, N833,N834,N835,N836, N837,N838,N839,N840, \\note fields as indicated
  N841,N842,N843,N844, N845,N846,N847,N848, N849,N850,N851,N852, N853,N854,N855,N856, N857,N858,N859,N860, \\note fields as indicated
  N861,N862,N863,N864, N865,N866,N867,N868, N869,N870,N871,N872, N873,N874,N875,N876, N877,N878,N879,N880, \\note fields as indicated
  N881,N882,N883,N884, N885,N886,N887,N888, N889,N890,N891,N892, N893,N894,N895,N896, N897,N898,N899,N900, \\note fields as indicated

  N901,N902,N903,N904, N905,N906,N907,N908, N909,N910,N911,N912, N913,N914,N915,N916, N917,N918,N919,N920, \\note fields as indicated
  N921,N922,N923,N924, N925,N926,N927,N928, N929,N930,N931,N932, N933,N934,N935,N936, N937,N938,N939,N940, \\note fields as indicated
  N941,N942,N943,N944, N945,N946,N947,N948, N949,N950,N951,N952, N953,N954,N955,N956, N957,N958,N959,N960, \\note fields as indicated
  N961,N962,N963,N964, N965,N966,N967,N968, N969,N970,N971,N972, N973,N974,N975,N976, N977,N978,N979,N980, \\note fields as indicated
  N981,N982,N983,N984, N985,N986,N987,N988, N989,N990,N991,N992, N993,N994,N995,N996, N997,N998,N999,N1000, \\note fields as indicated

  N1001,N1002,N1003,N1004, N1005,N1006,N1007,N1008, N1009,N1010,N1011,N1012, N1013,N1014,N1015,N1016, N1017,N1018,N1019,N1020, \\note fields as indicated
  N1021,N1022,N1023,N1024, N1025,N1026,N1027,N1028, N1029,N1030,N1031,N1032, N1033,N1034,N1035,N1036, N1037,N1038,N1039,N1040, \\note fields as indicated
  N1041,N1042,N1043,N1044, N1045,N1046,N1047,N1048, N1049,N1050,N1051,N1052, N1053,N1054,N1055,N1056, N1057,N1058,N1059,N1060, \\note fields as indicated
  N1061,N1062,N1063,N1064, N1065,N1066,N1067,N1068, N1069,N1070,N1071,N1072, N1073,N1074,N1075,N1076, N1077,N1078,N1079,N1080, \\note fields as indicated
  N1081,N1082,N1083,N1084, N1085,N1086,N1087,N1088, N1089,N1090,N1091,N1092, N1093,N1094,N1095,N1096, N1097,N1098,N1099,N1100, \\note fields as indicated

  N1101,N1102,N1103,N1104, N1105,N1106,N1107,N1108, N1109,N1110,N1111,N1112, N1113,N1114,N1115,N1116, N1117,N1118,N1119,N1120, \\note fields as indicated
  N1121,N1122,N1123,N1124, N1125,N1126,N1127,N1128, N1129,N1130,N1131,N1132, N1133,N1134,N1135,N1136, N1137,N1138,N1139,N1140, \\note fields as indicated
  N1141,N1142,N1143,N1144, N1145,N1146,N1147,N1148, N1149,N1150,N1151,N1152, N1153,N1154,N1155,N1156, N1157,N1158,N1159,N1160, \\note fields as indicated
  N1161,N1162,N1163,N1164, N1165,N1166,N1167,N1168, N1169,N1170,N1171,N1172, N1173,N1174,N1175,N1176, N1177,N1178,N1179,N1180, \\note fields as indicated
  N1181,N1182,N1183,N1184, N1185,N1186,N1187,N1188, N1189,N1190,N1191,N1192, N1193,N1194,N1195,N1196, N1197,N1198,N1199,N1200, \\note fields as indicated

  N1201,N1202,N1203,N1204, N1205,N1206,N1207,N1208, N1209,N1210,N1211,N1212, N1213,N1214,N1215,N1216, N1217,N1218,N1219,N1220,\\note fields as indicated
  N1221,N1222,N1223,N1224, N1225,N1226,N1227,N1228, N1229,N1230,N1231,N1232, N1233,N1234,N1235,N1236, N1237,N1238,N1239,N1240,\\note fields as indicated
  N1241,N1242,N1243,N1244, N1245,N1246,N1247,N1248, N1249,N1250,N1251,N1252, N1253,N1254,N1255,N1256, N1257,N1258,N1259,N1260,\\note fields as indicated
  N1261,N1262,N1263,N1264, N1265,N1266,N1267,N1268, N1269,N1270,N1271,N1272, N1273,N1274,N1275,N1276, N1277,N1278,N1279,N1280,\\note fields as indicated
  N1281,N1282,N1283,N1284, N1285,N1286,N1287,N1288, N1289,N1290,N1291,N1292, N1293,N1294,N1295,N1296, N1297,N1298,N1299,N1300,\\note fields as indicated

  N1301,N1302,N1303,N1304, N1305,N1306,N1307,N1308, N1309,N1310,N1311,N1312, N1313,N1314,N1315,N1316, N1317,N1318,N1319,N1320,\\note fields as indicated
  N1321,N1322,N1323,N1324, N1325,N1326,N1327,N1328, N1329,N1330,N1331,N1332, N1333,N1334,N1335,N1336, N1337,N1338,N1339,N1340,\\note fields as indicated
  N1341,N1342,N1343,N1344, N1345,N1346,N1347,N1348, N1349,N1350,N1351,N1352, N1353,N1354,N1355,N1356, N1357,N1358,N1359,N1360,\\note fields as indicated
  N1361,N1362,N1363,N1364, N1365,N1366,N1367,N1368, N1369,N1370,N1371,N1372, N1373,N1374,N1375,N1376, N1377,N1378,N1379,N1380,\\note fields as indicated
  N1381,N1382,N1383,N1384, N1385,N1386,N1387,N1388, N1389,N1390,N1391,N1392, N1393,N1394,N1395,N1396, N1397,N1398,N1399,N1400,\\note fields as indicated

  N1401,N1402,N1403,N1404, N1405,N1406,N1407,N1408, N1409,N1410,N1411,N1412, N1413,N1414,N1415,N1416, N1417,N1418,N1419,N1420,\\note fields as indicated
  N1421,N1422,N1423,N1424, N1425,N1426,N1427,N1428, N1429,N1430,N1431,N1432, N1433,N1434,N1435,N1436, N1437,N1438,N1439,N1440,\\note fields as indicated
  N1441;\\note fields as indicated

Schedule:Week:Daily,
  \\min-fields 13
       \\memo  A Schedule:Week:Daily contains 12 Schedule:Day:Hourly objects, one for each day type.
  A1 , \\field Name
       \\required-field
       \\reference WeekScheduleNames
       \\type alpha
  A2 , \\field Sunday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A3 , \\field Monday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A4 , \\field Tuesday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A5 , \\field Wednesday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A6 , \\field Thursday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A7 , \\field Friday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A8 , \\field Saturday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A9 , \\field Holiday Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A10, \\field SummerDesignDay Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A11, \\field WinterDesignDay Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A12, \\field CustomDay1 Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A13; \\field CustomDay2 Schedule:Day Name
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames

Schedule:Week:Compact,
  \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
  \\memo Compact definition for Schedule:Day:List
  \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\reference WeekScheduleNames
       \\type alpha
  A2 , \\field DayType List 1
       \\begin-extensible
       \\note "For" is an optional prefix/start of the For fields.  Choices can be combined on single line
       \\note if separated by spaces. i.e. "Holiday Weekends"
       \\note Should have a space after For, if it is included. i.e. "For Alldays"
       \\required-field
       \\type choice
       \\key AllDays
       \\key Weekdays
       \\key Weekends
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A3 , \\field Schedule:Day Name 1
       \\required-field
       \\type object-list
       \\object-list DayScheduleNames
  A4 , \\field DayType List 2
       \\type choice
       \\key AllOtherDays
       \\key Weekdays
       \\key Weekends
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A5 , \\field Schedule:Day Name 2
       \\type object-list
       \\object-list DayScheduleNames
  A6 , \\field DayType List 3
       \\type choice
       \\key AllOtherDays
       \\key Weekdays
       \\key Weekends
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A7 , \\field Schedule:Day Name 3
       \\type object-list
       \\object-list DayScheduleNames
  A8 , \\field DayType List 4
       \\type choice
       \\key AllOtherDays
       \\key Weekdays
       \\key Weekends
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A9 , \\field Schedule:Day Name 4
       \\type object-list
       \\object-list DayScheduleNames
  A10, \\field DayType List 5
       \\type choice
       \\key AllOtherDays
       \\key Weekdays
       \\key Weekends
       \\key Sunday
       \\key Monday
       \\key Tuesday
       \\key Wednesday
       \\key Thursday
       \\key Friday
       \\key Saturday
       \\key Holiday
       \\key SummerDesignDay
       \\key WinterDesignDay
       \\key CustomDay1
       \\key CustomDay2
  A11; \\field Schedule:Day Name 5
       \\type object-list
       \\object-list DayScheduleNames

Schedule:Year,
 \\min-fields 7
       \\memo A Schedule:Year contains from 1 to 52 week schedules
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field Schedule:Week Name 1
       \\required-field
       \\type object-list
       \\object-list WeekScheduleNames
  N1 , \\field Start Month 1
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 12
  N2 , \\field Start Day 1
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 31
  N3 , \\field End Month 1
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 12
  N4 , \\field End Day 1
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 31
  A4 , \\field Schedule:Week Name 2
       \\type object-list
       \\object-list WeekScheduleNames
  N5 , \\field Start Month 2
       \\type integer
       \\minimum 1
       \\maximum 12
  N6 , \\field Start Day 2
       \\type integer
       \\minimum 1
       \\maximum 31
  N7 , \\field End Month 2
       \\type integer
       \\minimum 1
       \\maximum 12
  N8 , \\field End Day 2
       \\type integer
       \\minimum 1
       \\maximum 31
  A5 , \\field Schedule:Week Name 3
       \\type object-list
       \\object-list WeekScheduleNames
  N9 , \\field Start Month 3
       \\type integer
       \\minimum 1
       \\maximum 12
  N10, \\field Start Day 3
       \\type integer
       \\minimum 1
       \\maximum 31
  N11, \\field End Month 3
       \\type integer
       \\minimum 1
       \\maximum 12
  N12, \\field End Day 3
       \\type integer
       \\minimum 1
       \\maximum 31
  A6 , \\field Schedule:Week Name 4
       \\type object-list
       \\object-list WeekScheduleNames
  N13, \\field Start Month 4
       \\type integer
       \\minimum 1
       \\maximum 12
  N14, \\field Start Day 4
       \\type integer
       \\minimum 1
       \\maximum 31
  N15, \\field End Month 4
       \\type integer
       \\minimum 1
       \\maximum 12
  N16, \\field End Day 4
       \\type integer
       \\minimum 1
       \\maximum 31
  A7 , \\field Schedule:Week Name 5
       \\type object-list
       \\object-list WeekScheduleNames
  N17, \\field Start Month 5
       \\type integer
       \\minimum 1
       \\maximum 12
  N18, \\field Start Day 5
       \\type integer
       \\minimum 1
       \\maximum 31
  N19, \\field End Month 5
       \\type integer
       \\minimum 1
       \\maximum 12
  N20, \\field End Day 5
       \\type integer
       \\minimum 1
       \\maximum 31
  A8 , \\field Schedule:Week Name 6
       \\type object-list
       \\object-list WeekScheduleNames
  N21, \\field Start Month 6
       \\type integer
       \\minimum 1
       \\maximum 12
  N22, \\field Start Day 6
       \\type integer
       \\minimum 1
       \\maximum 31
  N23, \\field End Month 6
       \\type integer
       \\minimum 1
       \\maximum 12
  N24, \\field End Day 6
       \\type integer
       \\minimum 1
       \\maximum 31
  A9 , \\field Schedule:Week Name 7
       \\type object-list
       \\object-list WeekScheduleNames
  N25, \\field Start Month 7
       \\type integer
       \\minimum 1
       \\maximum 12
  N26, \\field Start Day 7
       \\type integer
       \\minimum 1
       \\maximum 31
  N27, \\field End Month 7
       \\type integer
       \\minimum 1
       \\maximum 12
  N28, \\field End Day 7
       \\type integer
       \\minimum 1
       \\maximum 31
  A10, \\field Schedule:Week Name 8
       \\type object-list
       \\object-list WeekScheduleNames
  N29, \\field Start Month 8
       \\type integer
       \\minimum 1
       \\maximum 12
  N30, \\field Start Day 8
       \\type integer
       \\minimum 1
       \\maximum 31
  N31, \\field End Month 8
       \\type integer
       \\minimum 1
       \\maximum 12
  N32, \\field End Day 8
       \\type integer
       \\minimum 1
       \\maximum 31
  A11, \\field Schedule:Week Name 9
       \\type object-list
       \\object-list WeekScheduleNames
  N33, \\field Start Month 9
       \\type integer
       \\minimum 1
       \\maximum 12
  N34, \\field Start Day 9
       \\type integer
       \\minimum 1
       \\maximum 31
  N35, \\field End Month 9
       \\type integer
       \\minimum 1
       \\maximum 12
  N36, \\field End Day 9
       \\type integer
       \\minimum 1
       \\maximum 31
  A12, \\field Schedule:Week Name 10
       \\type object-list
       \\object-list WeekScheduleNames
  N37, \\field Start Month 10
       \\type integer
       \\minimum 1
       \\maximum 12
  N38, \\field Start Day 10
       \\type integer
       \\minimum 1
       \\maximum 31
  N39, \\field End Month 10
       \\type integer
       \\minimum 1
       \\maximum 12
  N40, \\field End Day 10
       \\type integer
       \\minimum 1
       \\maximum 31
  A13, \\field Schedule:Week Name 11
       \\type object-list
       \\object-list WeekScheduleNames
  N41, \\field Start Month 11
       \\type integer
       \\minimum 1
       \\maximum 12
  N42, \\field Start Day 11
       \\type integer
       \\minimum 1
       \\maximum 31
  N43, \\field End Month 11
       \\type integer
       \\minimum 1
       \\maximum 12
  N44, \\field End Day 11
       \\type integer
       \\minimum 1
       \\maximum 31
  A14, \\field Schedule:Week Name 12
       \\type object-list
       \\object-list WeekScheduleNames
  N45, \\field Start Month 12
       \\type integer
       \\minimum 1
       \\maximum 12
  N46, \\field Start Day 12
       \\type integer
       \\minimum 1
       \\maximum 31
  N47, \\field End Month 12
       \\type integer
       \\minimum 1
       \\maximum 12
  N48, \\field End Day 12
       \\type integer
       \\minimum 1
       \\maximum 31
  A15, \\field Schedule:Week Name 13
       \\type object-list
       \\object-list WeekScheduleNames
  N49, \\field Start Month 13
       \\type integer
       \\minimum 1
       \\maximum 12
  N50, \\field Start Day 13
       \\type integer
       \\minimum 1
       \\maximum 31
  N51, \\field End Month 13
       \\type integer
       \\minimum 1
       \\maximum 12
  N52, \\field End Day 13
       \\type integer
       \\minimum 1
       \\maximum 31
  A16, \\field Schedule:Week Name 14
       \\type object-list
       \\object-list WeekScheduleNames
  N53, \\field Start Month 14
       \\type integer
       \\minimum 1
       \\maximum 12
  N54, \\field Start Day 14
       \\type integer
       \\minimum 1
       \\maximum 31
  N55, \\field End Month 14
       \\type integer
       \\minimum 1
       \\maximum 12
  N56, \\field End Day 14
       \\type integer
       \\minimum 1
       \\maximum 31
  A17, \\field Schedule:Week Name 15
       \\type object-list
       \\object-list WeekScheduleNames
  N57, \\field Start Month 15
       \\type integer
       \\minimum 1
       \\maximum 12
  N58, \\field Start Day 15
       \\type integer
       \\minimum 1
       \\maximum 31
  N59, \\field End Month 15
       \\type integer
       \\minimum 1
       \\maximum 12
  N60, \\field End Day 15
       \\type integer
       \\minimum 1
       \\maximum 31
  A18, \\field Schedule:Week Name 16
       \\type object-list
       \\object-list WeekScheduleNames
  N61, \\field Start Month 16
       \\type integer
       \\minimum 1
       \\maximum 12
  N62, \\field Start Day 16
       \\type integer
       \\minimum 1
       \\maximum 31
  N63, \\field End Month 16
       \\type integer
       \\minimum 1
       \\maximum 12
  N64, \\field End Day 16
       \\type integer
       \\minimum 1
       \\maximum 31
  A19, \\field Schedule:Week Name 17
       \\type object-list
       \\object-list WeekScheduleNames
  N65, \\field Start Month 17
       \\type integer
       \\minimum 1
       \\maximum 12
  N66, \\field Start Day 17
       \\type integer
       \\minimum 1
       \\maximum 31
  N67, \\field End Month 17
       \\type integer
       \\minimum 1
       \\maximum 12
  N68, \\field End Day 17
       \\type integer
       \\minimum 1
       \\maximum 31
  A20, \\field Schedule:Week Name 18
       \\type object-list
       \\object-list WeekScheduleNames
  N69, \\field Start Month 18
       \\type integer
       \\minimum 1
       \\maximum 12
  N70, \\field Start Day 18
       \\type integer
       \\minimum 1
       \\maximum 31
  N71, \\field End Month 18
       \\type integer
       \\minimum 1
       \\maximum 12
  N72, \\field End Day 18
       \\type integer
       \\minimum 1
       \\maximum 31
  A21, \\field Schedule:Week Name 19
       \\type object-list
       \\object-list WeekScheduleNames
  N73, \\field Start Month 19
       \\type integer
       \\minimum 1
       \\maximum 12
  N74, \\field Start Day 19
       \\type integer
       \\minimum 1
       \\maximum 31
  N75, \\field End Month 19
       \\type integer
       \\minimum 1
       \\maximum 12
  N76, \\field End Day 19
       \\type integer
       \\minimum 1
       \\maximum 31
  A22, \\field Schedule:Week Name 20
       \\type object-list
       \\object-list WeekScheduleNames
  N77, \\field Start Month 20
       \\type integer
       \\minimum 1
       \\maximum 12
  N78, \\field Start Day 20
       \\type integer
       \\minimum 1
       \\maximum 31
  N79, \\field End Month 20
       \\type integer
       \\minimum 1
       \\maximum 12
  N80, \\field End Day 20
       \\type integer
       \\minimum 1
       \\maximum 31
  A23, \\field Schedule:Week Name 21
       \\type object-list
       \\object-list WeekScheduleNames
  N81, \\field Start Month 21
       \\type integer
       \\minimum 1
       \\maximum 12
  N82, \\field Start Day 21
       \\type integer
       \\minimum 1
       \\maximum 31
  N83, \\field End Month 21
       \\type integer
       \\minimum 1
       \\maximum 12
  N84, \\field End Day 21
       \\type integer
       \\minimum 1
       \\maximum 31
  A24, \\field Schedule:Week Name 22
       \\type object-list
       \\object-list WeekScheduleNames
  N85, \\field Start Month 22
       \\type integer
       \\minimum 1
       \\maximum 12
  N86, \\field Start Day 22
       \\type integer
       \\minimum 1
       \\maximum 31
  N87, \\field End Month 22
       \\type integer
       \\minimum 1
       \\maximum 12
  N88, \\field End Day 22
       \\type integer
       \\minimum 1
       \\maximum 31
  A25, \\field Schedule:Week Name 23
       \\type object-list
       \\object-list WeekScheduleNames
  N89, \\field Start Month 23
       \\type integer
       \\minimum 1
       \\maximum 12
  N90, \\field Start Day 23
       \\type integer
       \\minimum 1
       \\maximum 31
  N91, \\field End Month 23
       \\type integer
       \\minimum 1
       \\maximum 12
  N92, \\field End Day 23
       \\type integer
       \\minimum 1
       \\maximum 31
  A26, \\field Schedule:Week Name 24
       \\type object-list
       \\object-list WeekScheduleNames
  N93, \\field Start Month 24
       \\type integer
       \\minimum 1
       \\maximum 12
  N94, \\field Start Day 24
       \\type integer
       \\minimum 1
       \\maximum 31
  N95, \\field End Month 24
       \\type integer
       \\minimum 1
       \\maximum 12
  N96, \\field End Day 24
       \\type integer
       \\minimum 1
       \\maximum 31
  A27, \\field Schedule:Week Name 25
       \\type object-list
       \\object-list WeekScheduleNames
  N97, \\field Start Month 25
       \\type integer
       \\minimum 1
       \\maximum 12
  N98, \\field Start Day 25
       \\type integer
       \\minimum 1
       \\maximum 31
  N99, \\field End Month 25
       \\type integer
       \\minimum 1
       \\maximum 12
 N100, \\field End Day 25
       \\type integer
       \\minimum 1
       \\maximum 31
  A28, \\field Schedule:Week Name 26
       \\type object-list
       \\object-list WeekScheduleNames
 N101, \\field Start Month 26
       \\type integer
       \\minimum 1
       \\maximum 12
 N102, \\field Start Day 26
       \\type integer
       \\minimum 1
       \\maximum 31
 N103, \\field End Month 26
       \\type integer
       \\minimum 1
       \\maximum 12
 N104, \\field End Day 26
       \\type integer
       \\minimum 1
       \\maximum 31
 \\note Schedule:Week for Weeks 27-53 are condensed
 A29,N105,N106,N107,N108, \\note For Week  27
 A30,N109,N110,N111,N112, \\note For Week  28
 A31,N113,N114,N115,N116, \\note For Week  29
 A32,N117,N118,N119,N120, \\note For Week  30
 A33,N121,N122,N123,N124, \\note For Week  31
 A34,N125,N126,N127,N128, \\note For Week  32
 A35,N129,N130,N131,N132, \\note For Week  33
 A36,N133,N134,N135,N136, \\note For Week  34
 A37,N137,N138,N139,N140, \\note For Week  35
 A38,N141,N142,N143,N144, \\note For Week  36
 A39,N145,N146,N147,N148, \\note For Week  37
 A40,N149,N150,N151,N152, \\note For Week  38
 A41,N153,N154,N155,N156, \\note For Week  39
 A42,N157,N158,N159,N160, \\note For Week  40
 A43,N161,N162,N163,N164, \\note For Week  41
 A44,N165,N166,N167,N168, \\note For Week  42
 A45,N169,N170,N171,N172, \\note For Week  43
 A46,N173,N174,N175,N176, \\note For Week  44
 A47,N177,N178,N179,N180, \\note For Week  45
 A48,N181,N182,N183,N184, \\note For Week  46
 A49,N185,N186,N187,N188, \\note For Week  47
 A50,N189,N190,N191,N192, \\note For Week  48
 A51,N193,N194,N195,N196, \\note For Week  49
 A52,N197,N198,N199,N200, \\note For Week  50
 A53,N201,N202,N203,N204, \\note For Week  51
 A54,N205,N206,N207,N208, \\note For Week  52
 A55,N209,N210,N211,N212; \\note For Week  53

Schedule:Compact,
   \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
   \\min-fields 5
   \\memo Irregular object.  Does not follow the usual definition for fields.  Fields A3... are:
   \\memo Through: Date
   \\memo For: Applicable days (ref: Schedule:Week:Compact)
   \\memo Interpolate: Yes/No (ref: Schedule:Day:Interval) -- optional, if not used will be "No"
   \\memo Until: <Time> (ref: Schedule:Day:Interval)
   \\memo <numeric value>
   \\memo words "Through","For","Interpolate","Until" must be included.
   \\format compactSchedule
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field Field 1
       \\begin-extensible
  A4 , \\field Field 2
  A5 , \\field Field 3
  A6 , \\field Field 4
  A7 , \\field Field 5
  A8 , \\field Field 6
  A9 , \\field Field 7
  A10, \\field Field 8
  A11, \\field Field 9
  A12, \\field Field 10
  A13, \\field Field 11
  A14, \\field Field 12
  A15, \\field Field 13
  A16, \\field Field 14
  A17, \\field Field 15
  A18, \\field Field 16
  A19, \\field Field 17
  A20, \\field Field 18
  A21, \\field Field 19
  A22, \\field Field 20
  A23, \\field Field 21
  A24, \\field Field 22
  A25, \\field Field 23
  A26, \\field Field 24
  A27, \\field Field 25
  A28, \\field Field 26
  A29, \\field Field 27
  A30, \\field Field 28
  A31, \\field Field 29
  A32, \\field Field 30
  A33, \\field Field 31
  A34, \\field Field 32
  A35, \\field Field 33
  A36, \\field Field 34
  A37, \\field Field 35
  A38, \\field Field 36
  A39, \\field Field 37
  A40, \\field Field 38
  A41, \\field Field 39
  A42, \\field Field 40
  A43, \\field Field 41
  A44, \\field Field 42
  A45, \\field Field 43
  A46, \\field Field 44
  A47, \\field Field 45
  A48, \\field Field 46
  A49, \\field Field 47
  A50, \\field Field 48
  A51, \\field Field 49
  A52, \\field Field 50
  A53, \\field Field 51
  A54, \\field Field 52
  A55, \\field Field 53
  A56, \\field Field 54
  A57, \\field Field 55
  A58, \\field Field 56
  A59, \\field Field 57
  A60, \\field Field 58
  A61, \\field Field 59
  A62, \\field Field 60
  A63, \\field Field 61
  A64, \\field Field 62
  A65, \\field Field 63
  A66, \\field Field 64
  A67, \\field Field 65
  A68, \\field Field 66
  A69, \\field Field 67
  A70, \\field Field 68
  A71, \\field Field 69
  A72, \\field Field 70
  A73, \\field Field 71
  A74, \\field Field 72
  A75, \\field Field 73
  A76, \\field Field 74
  A77, \\field Field 75
  A78, \\field Field 76
  A79, \\field Field 77
  A80, \\field Field 78
  A81, \\field Field 79
  A82, \\field Field 80
  A83, \\field Field 81
  A84, \\field Field 82
  A85, \\field Field 83
  A86, \\field Field 84
  A87, \\field Field 85
  A88, \\field Field 86
  A89, \\field Field 87
  A90, \\field Field 88
  A91, \\field Field 89
  A92, \\field Field 90
  A93, \\field Field 91
  A94, \\field Field 92
  A95, \\field Field 93
  A96, \\field Field 94
  A97, \\field Field 95
  A98, \\field Field 96
  A99, \\field Field 97
 A100, \\field Field 98
 A101, \\field Field 99
 A102, \\field Field 100
 A103, \\field Field 101
 A104, \\field Field 102
 A105, \\field Field 103
 A106, \\field Field 104
 A107, \\field Field 105
 A108, \\field Field 106
 A109, \\field Field 107
 A110, \\field Field 108
 A111, \\field Field 109
 A112, \\field Field 110
 A113, \\field Field 111
 A114, \\field Field 112
 A115, \\field Field 113
 A116, \\field Field 114
 A117, \\field Field 115
 A118, \\field Field 116
 A119, \\field Field 117
 A120, \\field Field 118
 A121, \\field Field 119
 A122, \\field Field 120
 A123, \\field Field 121
 A124, \\field Field 122
 A125, \\field Field 123
 A126, \\field Field 124
 A127, \\field Field 125
 A128, \\field Field 126
 A129, \\field Field 127
 A130, \\field Field 128
 A131, \\field Field 129
 A132, \\field Field 130
 A133, \\field Field 131
 A134, \\field Field 132
 A135, \\field Field 133
 A136, \\field Field 134
 A137, \\field Field 135
 A138, \\field Field 136
 A139, \\field Field 137
 A140, \\field Field 138
 A141, \\field Field 139
 A142, \\field Field 140
 A143, \\field Field 141
 A144, \\field Field 142
 A145, \\field Field 143
 A146, \\field Field 144
 A147, \\field Field 145
 A148, \\field Field 146
 A149, \\field Field 147
 A150, \\field Field 148
 A151, \\field Field 149
 A152, \\field Field 150
 A153,A154,A155,A156,A157,A158,A159,A160,A161,A162,A163,A164,A165,A166,A167, \\note fields as indicated
 A168,A169,A170,A171,A172,A173,A174,A175,A176,A177,A178,A179,A180,A181,A182, \\note fields as indicated
 A183,A184,A185,A186,A187,A188,A189,A190,A191,A192,A193,A194,A195,A196,A197, \\note fields as indicated
 A198,A199,A200,A201,A202,A203,A204,A205,A206,A207,A208,A209,A210,A211,A212, \\note fields as indicated
 A213,A214,A215,A216,A217,A218,A219,A220,A221,A222,A223,A224,A225,A226,A227, \\note fields as indicated
 A228,A229,A230,A231,A232,A233,A234,A235,A236,A237,A238,A239,A240,A241,A242, \\note fields as indicated
 A243,A244,A245,A246,A247,A248,A249,A250,A251,A252,A253,A254,A255,A256,A257, \\note fields as indicated
 A258,A259,A260,A261,A262,A263,A264,A265,A266,A267,A268,A269,A270,A271,A272, \\note fields as indicated
 A273,A274,A275,A276,A277,A278,A279,A280,A281,A282,A283,A284,A285,A286,A287, \\note fields as indicated
 A288,A289,A290,A291,A292,A293,A294,A295,A296,A297,A298,A299,A300,A301,A302, \\note fields as indicated
 A303,A304,A305,A306,A307,A308,A309,A310,A311,A312,A313,A314,A315,A316,A317, \\note fields as indicated
 A318,A319,A320,A321,A322,A323,A324,A325,A326,A327,A328,A329,A330,A331,A332, \\note fields as indicated
 A333,A334,A335,A336,A337,A338,A339,A340,A341,A342,A343,A344,A345,A346,A347, \\note fields as indicated
 A348,A349,A350,A351,A352,A353,A354,A355,A356,A357,A358,A359,A360,A361,A362, \\note fields as indicated
 A363,A364,A365,A366,A367,A368,A369,A370,A371,A372,A373,A374,A375,A376,A377, \\note fields as indicated
 A378,A379,A380,A381,A382,A383,A384,A385,A386,A387,A388,A389,A390,A391,A392, \\note fields as indicated
 A393,A394,A395,A396,A397,A398,A399,A400,A401,A402,A403,A404,A405,A406,A407, \\note fields as indicated
 A408,A409,A410,A411,A412,A413,A414,A415,A416,A417,A418,A419,A420,A421,A422, \\note fields as indicated
 A423,A424,A425,A426,A427,A428,A429,A430,A431,A432,A433,A434,A435,A436,A437, \\note fields as indicated
 A438,A439,A440,A441,A442,A443,A444,A445,A446,A447,A448,A449,A450,A451,A452, \\note fields as indicated
 A453,A454,A455,A456,A457,A458,A459,A460,A461,A462,A463,A464,A465,A466,A467, \\note fields as indicated
 A468,A469,A470,A471,A472,A473,A474,A475,A476,A477,A478,A479,A480,A481,A482, \\note fields as indicated
 A483,A484,A485,A486,A487,A488,A489,A490,A491,A492,A493,A494,A495,A496,A497, \\note fields as indicated
 A498,A499,A500,A501,A502,A503,A504,A505,A506,A507,A508,A509,A510,A511,A512, \\note fields as indicated
 A513,A514,A515,A516,A517,A518,A519,A520,A521,A522,A523,A524,A525,A526,A527, \\note fields as indicated
 A528,A529,A530,A531,A532,A533,A534,A535,A536,A537,A538,A539,A540,A541,A542, \\note fields as indicated
 A543,A544,A545,A546,A547,A548,A549,A550,A551,A552,A553,A554,A555,A556,A557, \\note fields as indicated
 A558,A559,A560,A561,A562,A563,A564,A565,A566,A567,A568,A569,A570,A571,A572, \\note fields as indicated
 A573,A574,A575,A576,A577,A578,A579,A580,A581,A582,A583,A584,A585,A586,A587, \\note fields as indicated
 A588,A589,A590,A591,A592,A593,A594,A595,A596,A597,A598,A599,A600,A601,A602, \\note fields as indicated
 A603,A604,A605,A606,A607,A608,A609,A610,A611,A612,A613,A614,A615,A616,A617, \\note fields as indicated
 A618,A619,A620,A621,A622,A623,A624,A625,A626,A627,A628,A629,A630,A631,A632, \\note fields as indicated
 A633,A634,A635,A636,A637,A638,A639,A640,A641,A642,A643,A644,A645,A646,A647, \\note fields as indicated
 A648,A649,A650,A651,A652,A653,A654,A655,A656,A657,A658,A659,A660,A661,A662, \\note fields as indicated
 A663,A664,A665,A666,A667,A668,A669,A670,A671,A672,A673,A674,A675,A676,A677, \\note fields as indicated
 A678,A679,A680,A681,A682,A683,A684,A685,A686,A687,A688,A689,A690,A691,A692, \\note fields as indicated
 A693,A694,A695,A696,A697,A698,A699,A700,A701,A702,A703,A704,A705,A706,A707, \\note fields as indicated
 A708,A709,A710,A711,A712,A713,A714,A715,A716,A717,A718,A719,A720,A721,A722, \\note fields as indicated
 A723,A724,A725,A726,A727,A728,A729,A730,A731,A732,A733,A734,A735,A736,A737, \\note fields as indicated
 A738,A739,A740,A741,A742,A743,A744,A745,A746,A747,A748,A749,A750,A751,A752, \\note fields as indicated
 A753,A754,A755,A756,A757,A758,A759,A760,A761,A762,A763,A764,A765,A766,A767, \\note fields as indicated
 A768,A769,A770,A771,A772,A773,A774,A775,A776,A777,A778,A779,A780,A781,A782, \\note fields as indicated
 A783,A784,A785,A786,A787,A788,A789,A790,A791,A792,A793,A794,A795,A796,A797, \\note fields as indicated
 A798,A799,A800,A801,A802,A803,A804,A805,A806,A807,A808,A809,A810,A811,A812, \\note fields as indicated
 A813,A814,A815,A816,A817,A818,A819,A820,A821,A822,A823,A824,A825,A826,A827, \\note fields as indicated
 A828,A829,A830,A831,A832,A833,A834,A835,A836,A837,A838,A839,A840,A841,A842, \\note fields as indicated
 A843,A844,A845,A846,A847,A848,A849,A850,A851,A852,A853,A854,A855,A856,A857, \\note fields as indicated
 A858,A859,A860,A861,A862,A863,A864,A865,A866,A867,A868,A869,A870,A871,A872, \\note fields as indicated
 A873,A874,A875,A876,A877,A878,A879,A880,A881,A882,A883,A884,A885,A886,A887, \\note fields as indicated
 A888,A889,A890,A891,A892,A893,A894,A895,A896,A897,A898,A899,A900,A901,A902, \\note fields as indicated
 A903,A904,A905,A906,A907,A908,A909,A910,A911,A912,A913,A914,A915,A916,A917, \\note fields as indicated
 A918,A919,A920,A921,A922,A923,A924,A925,A926,A927,A928,A929,A930,A931,A932, \\note fields as indicated
 A933,A934,A935,A936,A937,A938,A939,A940,A941,A942,A943,A944,A945,A946,A947, \\note fields as indicated
 A948,A949,A950,A951,A952,A953,A954,A955,A956,A957,A958,A959,A960,A961,A962, \\note fields as indicated
 A963,A964,A965,A966,A967,A968,A969,A970,A971,A972,A973,A974,A975,A976,A977, \\note fields as indicated
 A978,A979,A980,A981,A982,A983,A984,A985,A986,A987,A988,A989,A990,A991,A992, \\note fields as indicated
 A993,A994,A995,A996,A997,A998,A999,A1000,A1001,A1002,A1003,A1004,A1005,A1006,A1007, \\note fields as indicated
 A1008,A1009,A1010,A1011,A1012,A1013,A1014,A1015,A1016,A1017,A1018,A1019,A1020,A1021,A1022, \\note fields as indicated
 A1023,A1024,A1025,A1026,A1027,A1028,A1029,A1030,A1031,A1032,A1033,A1034,A1035,A1036,A1037, \\note fields as indicated
 A1038,A1039,A1040,A1041,A1042,A1043,A1044,A1045,A1046,A1047,A1048,A1049,A1050,A1051,A1052, \\note fields as indicated
 A1053,A1054,A1055,A1056,A1057,A1058,A1059,A1060,A1061,A1062,A1063,A1064,A1065,A1066,A1067, \\note fields as indicated
 A1068,A1069,A1070,A1071,A1072,A1073,A1074,A1075,A1076,A1077,A1078,A1079,A1080,A1081,A1082, \\note fields as indicated
 A1083,A1084,A1085,A1086,A1087,A1088,A1089,A1090,A1091,A1092,A1093,A1094,A1095,A1096,A1097, \\note fields as indicated
 A1098,A1099,A1100,A1101,A1102,A1103,A1104,A1105,A1106,A1107,A1108,A1109,A1110,A1111,A1112, \\note fields as indicated
 A1113,A1114,A1115,A1116,A1117,A1118,A1119,A1120,A1121,A1122,A1123,A1124,A1125,A1126,A1127, \\note fields as indicated
 A1128,A1129,A1130,A1131,A1132,A1133,A1134,A1135,A1136,A1137,A1138,A1139,A1140,A1141,A1142, \\note fields as indicated
 A1143,A1144,A1145,A1146,A1147,A1148,A1149,A1150,A1151,A1152,A1153,A1154,A1155,A1156,A1157, \\note fields as indicated
 A1158,A1159,A1160,A1161,A1162,A1163,A1164,A1165,A1166,A1167,A1168,A1169,A1170,A1171,A1172, \\note fields as indicated
 A1173,A1174,A1175,A1176,A1177,A1178,A1179,A1180,A1181,A1182,A1183,A1184,A1185,A1186,A1187, \\note fields as indicated
 A1188,A1189,A1190,A1191,A1192,A1193,A1194,A1195,A1196,A1197,A1198,A1199,A1200,A1201,A1202, \\note fields as indicated
 A1203,A1204,A1205,A1206,A1207,A1208,A1209,A1210,A1211,A1212,A1213,A1214,A1215,A1216,A1217, \\note fields as indicated
 A1218,A1219,A1220,A1221,A1222,A1223,A1224,A1225,A1226,A1227,A1228,A1229,A1230,A1231,A1232, \\note fields as indicated
 A1233,A1234,A1235,A1236,A1237,A1238,A1239,A1240,A1241,A1242,A1243,A1244,A1245,A1246,A1247, \\note fields as indicated
 A1248,A1249,A1250,A1251,A1252,A1253,A1254,A1255,A1256,A1257,A1258,A1259,A1260,A1261,A1262, \\note fields as indicated
 A1263,A1264,A1265,A1266,A1267,A1268,A1269,A1270,A1271,A1272,A1273,A1274,A1275,A1276,A1277, \\note fields as indicated
 A1278,A1279,A1280,A1281,A1282,A1283,A1284,A1285,A1286,A1287,A1288,A1289,A1290,A1291,A1292, \\note fields as indicated
 A1293,A1294,A1295,A1296,A1297,A1298,A1299,A1300,A1301,A1302,A1303,A1304,A1305,A1306,A1307, \\note fields as indicated
 A1308,A1309,A1310,A1311,A1312,A1313,A1314,A1315,A1316,A1317,A1318,A1319,A1320,A1321,A1322, \\note fields as indicated
 A1323,A1324,A1325,A1326,A1327,A1328,A1329,A1330,A1331,A1332,A1333,A1334,A1335,A1336,A1337, \\note fields as indicated
 A1338,A1339,A1340,A1341,A1342,A1343,A1344,A1345,A1346,A1347,A1348,A1349,A1350,A1351,A1352, \\note fields as indicated
 A1353,A1354,A1355,A1356,A1357,A1358,A1359,A1360,A1361,A1362,A1363,A1364,A1365,A1366,A1367, \\note fields as indicated
 A1368,A1369,A1370,A1371,A1372,A1373,A1374,A1375,A1376,A1377,A1378,A1379,A1380,A1381,A1382, \\note fields as indicated
 A1383,A1384,A1385,A1386,A1387,A1388,A1389,A1390,A1391,A1392,A1393,A1394,A1395,A1396,A1397, \\note fields as indicated
 A1398,A1399,A1400,A1401,A1402,A1403,A1404,A1405,A1406,A1407,A1408,A1409,A1410,A1411,A1412, \\note fields as indicated
 A1413,A1414,A1415,A1416,A1417,A1418,A1419,A1420,A1421,A1422,A1423,A1424,A1425,A1426,A1427, \\note fields as indicated
 A1428,A1429,A1430,A1431,A1432,A1433,A1434,A1435,A1436,A1437,A1438,A1439,A1440,A1441,A1442, \\note fields as indicated
 A1443,A1444,A1445,A1446,A1447,A1448,A1449,A1450,A1451,A1452,A1453,A1454,A1455,A1456,A1457, \\note fields as indicated
 A1458,A1459,A1460,A1461,A1462,A1463,A1464,A1465,A1466,A1467,A1468,A1469,A1470,A1471,A1472, \\note fields as indicated
 A1473,A1474,A1475,A1476,A1477,A1478,A1479,A1480,A1481,A1482,A1483,A1484,A1485,A1486,A1487, \\note fields as indicated
 A1488,A1489,A1490,A1491,A1492,A1493,A1494,A1495,A1496,A1497,A1498,A1499,A1500,A1501,A1502, \\note fields as indicated
 A1503,A1504,A1505,A1506,A1507,A1508,A1509,A1510,A1511,A1512,A1513,A1514,A1515,A1516,A1517, \\note fields as indicated
 A1518,A1519,A1520,A1521,A1522,A1523,A1524,A1525,A1526,A1527,A1528,A1529,A1530,A1531,A1532, \\note fields as indicated
 A1533,A1534,A1535,A1536,A1537,A1538,A1539,A1540,A1541,A1542,A1543,A1544,A1545,A1546,A1547, \\note fields as indicated
 A1548,A1549,A1550,A1551,A1552,A1553,A1554,A1555,A1556,A1557,A1558,A1559,A1560,A1561,A1562, \\note fields as indicated
 A1563,A1564,A1565,A1566,A1567,A1568,A1569,A1570,A1571,A1572,A1573,A1574,A1575,A1576,A1577, \\note fields as indicated
 A1578,A1579,A1580,A1581,A1582,A1583,A1584,A1585,A1586,A1587,A1588,A1589,A1590,A1591,A1592, \\note fields as indicated
 A1593,A1594,A1595,A1596,A1597,A1598,A1599,A1600,A1601,A1602,A1603,A1604,A1605,A1606,A1607, \\note fields as indicated
 A1608,A1609,A1610,A1611,A1612,A1613,A1614,A1615,A1616,A1617,A1618,A1619,A1620,A1621,A1622, \\note fields as indicated
 A1623,A1624,A1625,A1626,A1627,A1628,A1629,A1630,A1631,A1632,A1633,A1634,A1635,A1636,A1637, \\note fields as indicated
 A1638,A1639,A1640,A1641,A1642,A1643,A1644,A1645,A1646,A1647,A1648,A1649,A1650,A1651,A1652, \\note fields as indicated
 A1653,A1654,A1655,A1656,A1657,A1658,A1659,A1660,A1661,A1662,A1663,A1664,A1665,A1666,A1667, \\note fields as indicated
 A1668,A1669,A1670,A1671,A1672,A1673,A1674,A1675,A1676,A1677,A1678,A1679,A1680,A1681,A1682, \\note fields as indicated
 A1683,A1684,A1685,A1686,A1687,A1688,A1689,A1690,A1691,A1692,A1693,A1694,A1695,A1696,A1697, \\note fields as indicated
 A1698,A1699,A1700,A1701,A1702,A1703,A1704,A1705,A1706,A1707,A1708,A1709,A1710,A1711,A1712, \\note fields as indicated
 A1713,A1714,A1715,A1716,A1717,A1718,A1719,A1720,A1721,A1722,A1723,A1724,A1725,A1726,A1727, \\note fields as indicated
 A1728,A1729,A1730,A1731,A1732,A1733,A1734,A1735,A1736,A1737,A1738,A1739,A1740,A1741,A1742, \\note fields as indicated
 A1743,A1744,A1745,A1746,A1747,A1748,A1749,A1750,A1751,A1752,A1753,A1754,A1755,A1756,A1757, \\note fields as indicated
 A1758,A1759,A1760,A1761,A1762,A1763,A1764,A1765,A1766,A1767,A1768,A1769,A1770,A1771,A1772, \\note fields as indicated
 A1773,A1774,A1775,A1776,A1777,A1778,A1779,A1780,A1781,A1782,A1783,A1784,A1785,A1786,A1787, \\note fields as indicated
 A1788,A1789,A1790,A1791,A1792,A1793,A1794,A1795,A1796,A1797,A1798,A1799,A1800,A1801,A1802, \\note fields as indicated
 A1803,A1804,A1805,A1806,A1807,A1808,A1809,A1810,A1811,A1812,A1813,A1814,A1815,A1816,A1817, \\note fields as indicated
 A1818,A1819,A1820,A1821,A1822,A1823,A1824,A1825,A1826,A1827,A1828,A1829,A1830,A1831,A1832, \\note fields as indicated
 A1833,A1834,A1835,A1836,A1837,A1838,A1839,A1840,A1841,A1842,A1843,A1844,A1845,A1846,A1847, \\note fields as indicated
 A1848,A1849,A1850,A1851,A1852,A1853,A1854,A1855,A1856,A1857,A1858,A1859,A1860,A1861,A1862, \\note fields as indicated
 A1863,A1864,A1865,A1866,A1867,A1868,A1869,A1870,A1871,A1872,A1873,A1874,A1875,A1876,A1877, \\note fields as indicated
 A1878,A1879,A1880,A1881,A1882,A1883,A1884,A1885,A1886,A1887,A1888,A1889,A1890,A1891,A1892, \\note fields as indicated
 A1893,A1894,A1895,A1896,A1897,A1898,A1899,A1900,A1901,A1902,A1903,A1904,A1905,A1906,A1907, \\note fields as indicated
 A1908,A1909,A1910,A1911,A1912,A1913,A1914,A1915,A1916,A1917,A1918,A1919,A1920,A1921,A1922, \\note fields as indicated
 A1923,A1924,A1925,A1926,A1927,A1928,A1929,A1930,A1931,A1932,A1933,A1934,A1935,A1936,A1937, \\note fields as indicated
 A1938,A1939,A1940,A1941,A1942,A1943,A1944,A1945,A1946,A1947,A1948,A1949,A1950,A1951,A1952, \\note fields as indicated
 A1953,A1954,A1955,A1956,A1957,A1958,A1959,A1960,A1961,A1962,A1963,A1964,A1965,A1966,A1967, \\note fields as indicated
 A1968,A1969,A1970,A1971,A1972,A1973,A1974,A1975,A1976,A1977,A1978,A1979,A1980,A1981,A1982, \\note fields as indicated
 A1983,A1984,A1985,A1986,A1987,A1988,A1989,A1990,A1991,A1992,A1993,A1994,A1995,A1996,A1997, \\note fields as indicated
 A1998,A1999,A2000,A2001,A2002,A2003,A2004,A2005,A2006,A2007,A2008,A2009,A2010,A2011,A2012, \\note fields as indicated
 A2013,A2014,A2015,A2016,A2017,A2018,A2019,A2020,A2021,A2022,A2023,A2024,A2025,A2026,A2027, \\note fields as indicated
 A2028,A2029,A2030,A2031,A2032,A2033,A2034,A2035,A2036,A2037,A2038,A2039,A2040,A2041,A2042, \\note fields as indicated
 A2043,A2044,A2045,A2046,A2047,A2048,A2049,A2050,A2051,A2052,A2053,A2054,A2055,A2056,A2057, \\note fields as indicated
 A2058,A2059,A2060,A2061,A2062,A2063,A2064,A2065,A2066,A2067,A2068,A2069,A2070,A2071,A2072, \\note fields as indicated
 A2073,A2074,A2075,A2076,A2077,A2078,A2079,A2080,A2081,A2082,A2083,A2084,A2085,A2086,A2087, \\note fields as indicated
 A2088,A2089,A2090,A2091,A2092,A2093,A2094,A2095,A2096,A2097,A2098,A2099,A2100,A2101,A2102, \\note fields as indicated
 A2103,A2104,A2105,A2106,A2107,A2108,A2109,A2110,A2111,A2112,A2113,A2114,A2115,A2116,A2117, \\note fields as indicated
 A2118,A2119,A2120,A2121,A2122,A2123,A2124,A2125,A2126,A2127,A2128,A2129,A2130,A2131,A2132, \\note fields as indicated
 A2133,A2134,A2135,A2136,A2137,A2138,A2139,A2140,A2141,A2142,A2143,A2144,A2145,A2146,A2147, \\note fields as indicated
 A2148,A2149,A2150,A2151,A2152,A2153,A2154,A2155,A2156,A2157,A2158,A2159,A2160,A2161,A2162, \\note fields as indicated
 A2163,A2164,A2165,A2166,A2167,A2168,A2169,A2170,A2171,A2172,A2173,A2174,A2175,A2176,A2177, \\note fields as indicated
 A2178,A2179,A2180,A2181,A2182,A2183,A2184,A2185,A2186,A2187,A2188,A2189,A2190,A2191,A2192, \\note fields as indicated
 A2193,A2194,A2195,A2196,A2197,A2198,A2199,A2200,A2201,A2202,A2203,A2204,A2205,A2206,A2207, \\note fields as indicated
 A2208,A2209,A2210,A2211,A2212,A2213,A2214,A2215,A2216,A2217,A2218,A2219,A2220,A2221,A2222, \\note fields as indicated
 A2223,A2224,A2225,A2226,A2227,A2228,A2229,A2230,A2231,A2232,A2233,A2234,A2235,A2236,A2237, \\note fields as indicated
 A2238,A2239,A2240,A2241,A2242,A2243,A2244,A2245,A2246,A2247,A2248,A2249,A2250,A2251,A2252, \\note fields as indicated
 A2253,A2254,A2255,A2256,A2257,A2258,A2259,A2260,A2261,A2262,A2263,A2264,A2265,A2266,A2267, \\note fields as indicated
 A2268,A2269,A2270,A2271,A2272,A2273,A2274,A2275,A2276,A2277,A2278,A2279,A2280,A2281,A2282, \\note fields as indicated
 A2283,A2284,A2285,A2286,A2287,A2288,A2289,A2290,A2291,A2292,A2293,A2294,A2295,A2296,A2297, \\note fields as indicated
 A2298,A2299,A2300,A2301,A2302,A2303,A2304,A2305,A2306,A2307,A2308,A2309,A2310,A2311,A2312, \\note fields as indicated
 A2313,A2314,A2315,A2316,A2317,A2318,A2319,A2320,A2321,A2322,A2323,A2324,A2325,A2326,A2327, \\note fields as indicated
 A2328,A2329,A2330,A2331,A2332,A2333,A2334,A2335,A2336,A2337,A2338,A2339,A2340,A2341,A2342, \\note fields as indicated
 A2343,A2344,A2345,A2346,A2347,A2348,A2349,A2350,A2351,A2352,A2353,A2354,A2355,A2356,A2357, \\note fields as indicated
 A2358,A2359,A2360,A2361,A2362,A2363,A2364,A2365,A2366,A2367,A2368,A2369,A2370,A2371,A2372, \\note fields as indicated
 A2373,A2374,A2375,A2376,A2377,A2378,A2379,A2380,A2381,A2382,A2383,A2384,A2385,A2386,A2387, \\note fields as indicated
 A2388,A2389,A2390,A2391,A2392,A2393,A2394,A2395,A2396,A2397,A2398,A2399,A2400,A2401,A2402, \\note fields as indicated
 A2403,A2404,A2405,A2406,A2407,A2408,A2409,A2410,A2411,A2412,A2413,A2414,A2415,A2416,A2417, \\note fields as indicated
 A2418,A2419,A2420,A2421,A2422,A2423,A2424,A2425,A2426,A2427,A2428,A2429,A2430,A2431,A2432, \\note fields as indicated
 A2433,A2434,A2435,A2436,A2437,A2438,A2439,A2440,A2441,A2442,A2443,A2444,A2445,A2446,A2447, \\note fields as indicated
 A2448,A2449,A2450,A2451,A2452,A2453,A2454,A2455,A2456,A2457,A2458,A2459,A2460,A2461,A2462, \\note fields as indicated
 A2463,A2464,A2465,A2466,A2467,A2468,A2469,A2470,A2471,A2472,A2473,A2474,A2475,A2476,A2477, \\note fields as indicated
 A2478,A2479,A2480,A2481,A2482,A2483,A2484,A2485,A2486,A2487,A2488,A2489,A2490,A2491,A2492, \\note fields as indicated
 A2493,A2494,A2495,A2496,A2497,A2498,A2499,A2500,A2501,A2502,A2503,A2504,A2505,A2506,A2507, \\note fields as indicated
 A2508,A2509,A2510,A2511,A2512,A2513,A2514,A2515,A2516,A2517,A2518,A2519,A2520,A2521,A2522, \\note fields as indicated
 A2523,A2524,A2525,A2526,A2527,A2528,A2529,A2530,A2531,A2532,A2533,A2534,A2535,A2536,A2537, \\note fields as indicated
 A2538,A2539,A2540,A2541,A2542,A2543,A2544,A2545,A2546,A2547,A2548,A2549,A2550,A2551,A2552, \\note fields as indicated
 A2553,A2554,A2555,A2556,A2557,A2558,A2559,A2560,A2561,A2562,A2563,A2564,A2565,A2566,A2567, \\note fields as indicated
 A2568,A2569,A2570,A2571,A2572,A2573,A2574,A2575,A2576,A2577,A2578,A2579,A2580,A2581,A2582, \\note fields as indicated
 A2583,A2584,A2585,A2586,A2587,A2588,A2589,A2590,A2591,A2592,A2593,A2594,A2595,A2596,A2597, \\note fields as indicated
 A2598,A2599,A2600,A2601,A2602,A2603,A2604,A2605,A2606,A2607,A2608,A2609,A2610,A2611,A2612, \\note fields as indicated
 A2613,A2614,A2615,A2616,A2617,A2618,A2619,A2620,A2621,A2622,A2623,A2624,A2625,A2626,A2627, \\note fields as indicated
 A2628,A2629,A2630,A2631,A2632,A2633,A2634,A2635,A2636,A2637,A2638,A2639,A2640,A2641,A2642, \\note fields as indicated
 A2643,A2644,A2645,A2646,A2647,A2648,A2649,A2650,A2651,A2652,A2653,A2654,A2655,A2656,A2657, \\note fields as indicated
 A2658,A2659,A2660,A2661,A2662,A2663,A2664,A2665,A2666,A2667,A2668,A2669,A2670,A2671,A2672, \\note fields as indicated
 A2673,A2674,A2675,A2676,A2677,A2678,A2679,A2680,A2681,A2682,A2683,A2684,A2685,A2686,A2687, \\note fields as indicated
 A2688,A2689,A2690,A2691,A2692,A2693,A2694,A2695,A2696,A2697,A2698,A2699,A2700,A2701,A2702, \\note fields as indicated
 A2703,A2704,A2705,A2706,A2707,A2708,A2709,A2710,A2711,A2712,A2713,A2714,A2715,A2716,A2717, \\note fields as indicated
 A2718,A2719,A2720,A2721,A2722,A2723,A2724,A2725,A2726,A2727,A2728,A2729,A2730,A2731,A2732, \\note fields as indicated
 A2733,A2734,A2735,A2736,A2737,A2738,A2739,A2740,A2741,A2742,A2743,A2744,A2745,A2746,A2747, \\note fields as indicated
 A2748,A2749,A2750,A2751,A2752,A2753,A2754,A2755,A2756,A2757,A2758,A2759,A2760,A2761,A2762, \\note fields as indicated
 A2763,A2764,A2765,A2766,A2767,A2768,A2769,A2770,A2771,A2772,A2773,A2774,A2775,A2776,A2777, \\note fields as indicated
 A2778,A2779,A2780,A2781,A2782,A2783,A2784,A2785,A2786,A2787,A2788,A2789,A2790,A2791,A2792, \\note fields as indicated
 A2793,A2794,A2795,A2796,A2797,A2798,A2799,A2800,A2801,A2802,A2803,A2804,A2805,A2806,A2807, \\note fields as indicated
 A2808,A2809,A2810,A2811,A2812,A2813,A2814,A2815,A2816,A2817,A2818,A2819,A2820,A2821,A2822, \\note fields as indicated
 A2823,A2824,A2825,A2826,A2827,A2828,A2829,A2830,A2831,A2832,A2833,A2834,A2835,A2836,A2837, \\note fields as indicated
 A2838,A2839,A2840,A2841,A2842,A2843,A2844,A2845,A2846,A2847,A2848,A2849,A2850,A2851,A2852, \\note fields as indicated
 A2853,A2854,A2855,A2856,A2857,A2858,A2859,A2860,A2861,A2862,A2863,A2864,A2865,A2866,A2867, \\note fields as indicated
 A2868,A2869,A2870,A2871,A2872,A2873,A2874,A2875,A2876,A2877,A2878,A2879,A2880,A2881,A2882, \\note fields as indicated
 A2883,A2884,A2885,A2886,A2887,A2888,A2889,A2890,A2891,A2892,A2893,A2894,A2895,A2896,A2897, \\note fields as indicated
 A2898,A2899,A2900,A2901,A2902,A2903,A2904,A2905,A2906,A2907,A2908,A2909,A2910,A2911,A2912, \\note fields as indicated
 A2913,A2914,A2915,A2916,A2917,A2918,A2919,A2920,A2921,A2922,A2923,A2924,A2925,A2926,A2927, \\note fields as indicated
 A2928,A2929,A2930,A2931,A2932,A2933,A2934,A2935,A2936,A2937,A2938,A2939,A2940,A2941,A2942, \\note fields as indicated
 A2943,A2944,A2945,A2946,A2947,A2948,A2949,A2950,A2951,A2952,A2953,A2954,A2955,A2956,A2957, \\note fields as indicated
 A2958,A2959,A2960,A2961,A2962,A2963,A2964,A2965,A2966,A2967,A2968,A2969,A2970,A2971,A2972, \\note fields as indicated
 A2973,A2974,A2975,A2976,A2977,A2978,A2979,A2980,A2981,A2982,A2983,A2984,A2985,A2986,A2987, \\note fields as indicated
 A2988,A2989,A2990,A2991,A2992,A2993,A2994,A2995,A2996,A2997,A2998,A2999,A3000,A3001,A3002, \\note fields as indicated
 A3003,A3004,A3005,A3006,A3007,A3008,A3009,A3010,A3011,A3012,A3013,A3014,A3015,A3016,A3017, \\note fields as indicated
 A3018,A3019,A3020,A3021,A3022,A3023,A3024,A3025,A3026,A3027,A3028,A3029,A3030,A3031,A3032, \\note fields as indicated
 A3033,A3034,A3035,A3036,A3037,A3038,A3039,A3040,A3041,A3042,A3043,A3044,A3045,A3046,A3047, \\note fields as indicated
 A3048,A3049,A3050,A3051,A3052,A3053,A3054,A3055,A3056,A3057,A3058,A3059,A3060,A3061,A3062, \\note fields as indicated
 A3063,A3064,A3065,A3066,A3067,A3068,A3069,A3070,A3071,A3072,A3073,A3074,A3075,A3076,A3077, \\note fields as indicated
 A3078,A3079,A3080,A3081,A3082,A3083,A3084,A3085,A3086,A3087,A3088,A3089,A3090,A3091,A3092, \\note fields as indicated
 A3093,A3094,A3095,A3096,A3097,A3098,A3099,A3100,A3101,A3102,A3103,A3104,A3105,A3106,A3107, \\note fields as indicated
 A3108,A3109,A3110,A3111,A3112,A3113,A3114,A3115,A3116,A3117,A3118,A3119,A3120,A3121,A3122, \\note fields as indicated
 A3123,A3124,A3125,A3126,A3127,A3128,A3129,A3130,A3131,A3132,A3133,A3134,A3135,A3136,A3137, \\note fields as indicated
 A3138,A3139,A3140,A3141,A3142,A3143,A3144,A3145,A3146,A3147,A3148,A3149,A3150,A3151,A3152, \\note fields as indicated
 A3153,A3154,A3155,A3156,A3157,A3158,A3159,A3160,A3161,A3162,A3163,A3164,A3165,A3166,A3167, \\note fields as indicated
 A3168,A3169,A3170,A3171,A3172,A3173,A3174,A3175,A3176,A3177,A3178,A3179,A3180,A3181,A3182, \\note fields as indicated
 A3183,A3184,A3185,A3186,A3187,A3188,A3189,A3190,A3191,A3192,A3193,A3194,A3195,A3196,A3197, \\note fields as indicated
 A3198,A3199,A3200,A3201,A3202,A3203,A3204,A3205,A3206,A3207,A3208,A3209,A3210,A3211,A3212, \\note fields as indicated
 A3213,A3214,A3215,A3216,A3217,A3218,A3219,A3220,A3221,A3222,A3223,A3224,A3225,A3226,A3227, \\note fields as indicated
 A3228,A3229,A3230,A3231,A3232,A3233,A3234,A3235,A3236,A3237,A3238,A3239,A3240,A3241,A3242, \\note fields as indicated
 A3243,A3244,A3245,A3246,A3247,A3248,A3249,A3250,A3251,A3252,A3253,A3254,A3255,A3256,A3257, \\note fields as indicated
 A3258,A3259,A3260,A3261,A3262,A3263,A3264,A3265,A3266,A3267,A3268,A3269,A3270,A3271,A3272, \\note fields as indicated
 A3273,A3274,A3275,A3276,A3277,A3278,A3279,A3280,A3281,A3282,A3283,A3284,A3285,A3286,A3287, \\note fields as indicated
 A3288,A3289,A3290,A3291,A3292,A3293,A3294,A3295,A3296,A3297,A3298,A3299,A3300,A3301,A3302, \\note fields as indicated
 A3303,A3304,A3305,A3306,A3307,A3308,A3309,A3310,A3311,A3312,A3313,A3314,A3315,A3316,A3317, \\note fields as indicated
 A3318,A3319,A3320,A3321,A3322,A3323,A3324,A3325,A3326,A3327,A3328,A3329,A3330,A3331,A3332, \\note fields as indicated
 A3333,A3334,A3335,A3336,A3337,A3338,A3339,A3340,A3341,A3342,A3343,A3344,A3345,A3346,A3347, \\note fields as indicated
 A3348,A3349,A3350,A3351,A3352,A3353,A3354,A3355,A3356,A3357,A3358,A3359,A3360,A3361,A3362, \\note fields as indicated
 A3363,A3364,A3365,A3366,A3367,A3368,A3369,A3370,A3371,A3372,A3373,A3374,A3375,A3376,A3377, \\note fields as indicated
 A3378,A3379,A3380,A3381,A3382,A3383,A3384,A3385,A3386,A3387,A3388,A3389,A3390,A3391,A3392, \\note fields as indicated
 A3393,A3394,A3395,A3396,A3397,A3398,A3399,A3400,A3401,A3402,A3403,A3404,A3405,A3406,A3407, \\note fields as indicated
 A3408,A3409,A3410,A3411,A3412,A3413,A3414,A3415,A3416,A3417,A3418,A3419,A3420,A3421,A3422, \\note fields as indicated
 A3423,A3424,A3425,A3426,A3427,A3428,A3429,A3430,A3431,A3432,A3433,A3434,A3435,A3436,A3437, \\note fields as indicated
 A3438,A3439,A3440,A3441,A3442,A3443,A3444,A3445,A3446,A3447,A3448,A3449,A3450,A3451,A3452, \\note fields as indicated
 A3453,A3454,A3455,A3456,A3457,A3458,A3459,A3460,A3461,A3462,A3463,A3464,A3465,A3466,A3467, \\note fields as indicated
 A3468,A3469,A3470,A3471,A3472,A3473,A3474,A3475,A3476,A3477,A3478,A3479,A3480,A3481,A3482, \\note fields as indicated
 A3483,A3484,A3485,A3486,A3487,A3488,A3489,A3490,A3491,A3492,A3493,A3494,A3495,A3496,A3497, \\note fields as indicated
 A3498,A3499,A3500,A3501,A3502,A3503,A3504,A3505,A3506,A3507,A3508,A3509,A3510,A3511,A3512, \\note fields as indicated
 A3513,A3514,A3515,A3516,A3517,A3518,A3519,A3520,A3521,A3522,A3523,A3524,A3525,A3526,A3527, \\note fields as indicated
 A3528,A3529,A3530,A3531,A3532,A3533,A3534,A3535,A3536,A3537,A3538,A3539,A3540,A3541,A3542, \\note fields as indicated
 A3543,A3544,A3545,A3546,A3547,A3548,A3549,A3550,A3551,A3552,A3553,A3554,A3555,A3556,A3557, \\note fields as indicated
 A3558,A3559,A3560,A3561,A3562,A3563,A3564,A3565,A3566,A3567,A3568,A3569,A3570,A3571,A3572, \\note fields as indicated
 A3573,A3574,A3575,A3576,A3577,A3578,A3579,A3580,A3581,A3582,A3583,A3584,A3585,A3586,A3587, \\note fields as indicated
 A3588,A3589,A3590,A3591,A3592,A3593,A3594,A3595,A3596,A3597,A3598,A3599,A3600,A3601,A3602, \\note fields as indicated
 A3603,A3604,A3605,A3606,A3607,A3608,A3609,A3610,A3611,A3612,A3613,A3614,A3615,A3616,A3617, \\note fields as indicated
 A3618,A3619,A3620,A3621,A3622,A3623,A3624,A3625,A3626,A3627,A3628,A3629,A3630,A3631,A3632, \\note fields as indicated
 A3633,A3634,A3635,A3636,A3637,A3638,A3639,A3640,A3641,A3642,A3643,A3644,A3645,A3646,A3647, \\note fields as indicated
 A3648,A3649,A3650,A3651,A3652,A3653,A3654,A3655,A3656,A3657,A3658,A3659,A3660,A3661,A3662, \\note fields as indicated
 A3663,A3664,A3665,A3666,A3667,A3668,A3669,A3670,A3671,A3672,A3673,A3674,A3675,A3676,A3677, \\note fields as indicated
 A3678,A3679,A3680,A3681,A3682,A3683,A3684,A3685,A3686,A3687,A3688,A3689,A3690,A3691,A3692, \\note fields as indicated
 A3693,A3694,A3695,A3696,A3697,A3698,A3699,A3700,A3701,A3702,A3703,A3704,A3705,A3706,A3707, \\note fields as indicated
 A3708,A3709,A3710,A3711,A3712,A3713,A3714,A3715,A3716,A3717,A3718,A3719,A3720,A3721,A3722, \\note fields as indicated
 A3723,A3724,A3725,A3726,A3727,A3728,A3729,A3730,A3731,A3732,A3733,A3734,A3735,A3736,A3737, \\note fields as indicated
 A3738,A3739,A3740,A3741,A3742,A3743,A3744,A3745,A3746,A3747,A3748,A3749,A3750,A3751,A3752, \\note fields as indicated
 A3753,A3754,A3755,A3756,A3757,A3758,A3759,A3760,A3761,A3762,A3763,A3764,A3765,A3766,A3767, \\note fields as indicated
 A3768,A3769,A3770,A3771,A3772,A3773,A3774,A3775,A3776,A3777,A3778,A3779,A3780,A3781,A3782, \\note fields as indicated
 A3783,A3784,A3785,A3786,A3787,A3788,A3789,A3790,A3791,A3792,A3793,A3794,A3795,A3796,A3797, \\note fields as indicated
 A3798,A3799,A3800,A3801,A3802,A3803,A3804,A3805,A3806,A3807,A3808,A3809,A3810,A3811,A3812, \\note fields as indicated
 A3813,A3814,A3815,A3816,A3817,A3818,A3819,A3820,A3821,A3822,A3823,A3824,A3825,A3826,A3827, \\note fields as indicated
 A3828,A3829,A3830,A3831,A3832,A3833,A3834,A3835,A3836,A3837,A3838,A3839,A3840,A3841,A3842, \\note fields as indicated
 A3843,A3844,A3845,A3846,A3847,A3848,A3849,A3850,A3851,A3852,A3853,A3854,A3855,A3856,A3857, \\note fields as indicated
 A3858,A3859,A3860,A3861,A3862,A3863,A3864,A3865,A3866,A3867,A3868,A3869,A3870,A3871,A3872, \\note fields as indicated
 A3873,A3874,A3875,A3876,A3877,A3878,A3879,A3880,A3881,A3882,A3883,A3884,A3885,A3886,A3887, \\note fields as indicated
 A3888,A3889,A3890,A3891,A3892,A3893,A3894,A3895,A3896,A3897,A3898,A3899,A3900,A3901,A3902, \\note fields as indicated
 A3903,A3904,A3905,A3906,A3907,A3908,A3909,A3910,A3911,A3912,A3913,A3914,A3915,A3916,A3917, \\note fields as indicated
 A3918,A3919,A3920,A3921,A3922,A3923,A3924,A3925,A3926,A3927,A3928,A3929,A3930,A3931,A3932, \\note fields as indicated
 A3933,A3934,A3935,A3936,A3937,A3938,A3939,A3940,A3941,A3942,A3943,A3944,A3945,A3946,A3947, \\note fields as indicated
 A3948,A3949,A3950,A3951,A3952,A3953,A3954,A3955,A3956,A3957,A3958,A3959,A3960,A3961,A3962, \\note fields as indicated
 A3963,A3964,A3965,A3966,A3967,A3968,A3969,A3970,A3971,A3972,A3973,A3974,A3975,A3976,A3977, \\note fields as indicated
 A3978,A3979,A3980,A3981,A3982,A3983,A3984,A3985,A3986,A3987,A3988,A3989,A3990,A3991,A3992, \\note fields as indicated
 A3993,A3994,A3995,A3996,A3997,A3998,A3999,A4000,A4001,A4002,A4003,A4004,A4005,A4006,A4007, \\note fields as indicated
 A4008,A4009,A4010,A4011,A4012,A4013,A4014,A4015,A4016,A4017,A4018,A4019,A4020,A4021,A4022, \\note fields as indicated
 A4023,A4024,A4025,A4026,A4027,A4028,A4029,A4030,A4031,A4032,A4033,A4034,A4035,A4036,A4037, \\note fields as indicated
 A4038,A4039,A4040,A4041,A4042,A4043,A4044,A4045,A4046,A4047,A4048,A4049,A4050,A4051,A4052, \\note fields as indicated
 A4053,A4054,A4055,A4056,A4057,A4058,A4059,A4060,A4061,A4062,A4063,A4064,A4065,A4066,A4067, \\note fields as indicated
 A4068,A4069,A4070,A4071,A4072,A4073,A4074,A4075,A4076,A4077,A4078,A4079,A4080,A4081,A4082, \\note fields as indicated
 A4083,A4084,A4085,A4086,A4087,A4088,A4089,A4090,A4091,A4092,A4093,A4094,A4095,A4096,A4097, \\note fields as indicated
 A4098,A4099,A4100,A4101,A4102,A4103,A4104,A4105,A4106,A4107,A4108,A4109,A4110,A4111,A4112, \\note fields as indicated
 A4113,A4114,A4115,A4116,A4117,A4118,A4119,A4120,A4121,A4122,A4123,A4124,A4125,A4126,A4127, \\note fields as indicated
 A4128,A4129,A4130,A4131,A4132,A4133,A4134,A4135,A4136,A4137,A4138,A4139,A4140,A4141,A4142, \\note fields as indicated
 A4143,A4144,A4145,A4146,A4147,A4148,A4149,A4150,A4151,A4152,A4153,A4154,A4155,A4156,A4157, \\note fields as indicated
 A4158,A4159,A4160,A4161,A4162,A4163,A4164,A4165,A4166,A4167,A4168,A4169,A4170,A4171,A4172, \\note fields as indicated
 A4173,A4174,A4175,A4176,A4177,A4178,A4179,A4180,A4181,A4182,A4183,A4184,A4185,A4186,A4187, \\note fields as indicated
 A4188,A4189,A4190,A4191,A4192,A4193,A4194,A4195,A4196,A4197,A4198,A4199,A4200,A4201,A4202, \\note fields as indicated
 A4203,A4204,A4205,A4206,A4207,A4208,A4209,A4210,A4211,A4212,A4213,A4214,A4215,A4216,A4217, \\note fields as indicated
 A4218,A4219,A4220,A4221,A4222,A4223,A4224,A4225,A4226,A4227,A4228,A4229,A4230,A4231,A4232, \\note fields as indicated
 A4233,A4234,A4235,A4236,A4237,A4238,A4239,A4240,A4241,A4242,A4243,A4244,A4245,A4246,A4247, \\note fields as indicated
 A4248,A4249,A4250,A4251,A4252,A4253,A4254,A4255,A4256,A4257,A4258,A4259,A4260,A4261,A4262, \\note fields as indicated
 A4263,A4264,A4265,A4266,A4267,A4268,A4269,A4270,A4271,A4272,A4273,A4274,A4275,A4276,A4277, \\note fields as indicated
 A4278,A4279,A4280,A4281,A4282,A4283,A4284,A4285,A4286,A4287,A4288,A4289,A4290,A4291,A4292, \\note fields as indicated
 A4293,A4294,A4295,A4296,A4297,A4298,A4299,A4300,A4301,A4302,A4303,A4304,A4305,A4306,A4307, \\note fields as indicated
 A4308,A4309,A4310,A4311,A4312,A4313,A4314,A4315,A4316,A4317,A4318,A4319,A4320,A4321,A4322, \\note fields as indicated
 A4323,A4324,A4325,A4326,A4327,A4328,A4329,A4330,A4331,A4332,A4333,A4334,A4335,A4336,A4337, \\note fields as indicated
 A4338,A4339,A4340,A4341,A4342,A4343,A4344,A4345,A4346,A4347,A4348,A4349,A4350,A4351,A4352, \\note fields as indicated
 A4353,A4354,A4355,A4356,A4357,A4358,A4359,A4360,A4361,A4362,A4363,A4364,A4365,A4366,A4367, \\note fields as indicated
 A4368,A4369,A4370,A4371,A4372,A4373,A4374,A4375,A4376,A4377,A4378,A4379,A4380,A4381,A4382, \\note fields as indicated
 A4383,A4384,A4385,A4386,A4387,A4388,A4389,A4390,A4391,A4392,A4393,A4394,A4395,A4396,A4397, \\note fields as indicated
 A4398,A4399,A4400,A4401,A4402,A4403,A4404,A4405,A4406,A4407,A4408,A4409,A4410,A4411,A4412, \\note fields as indicated
 A4413,A4414,A4415,A4416,A4417,A4418,A4419,A4420,A4421,A4422,A4423,A4424,A4425,A4426,A4427, \\note fields as indicated
 A4428,A4429,A4430,A4431,A4432,A4433,A4434,A4435,A4436,A4437,A4438,A4439,A4440,A4441,A4442, \\note fields as indicated
 A4443,A4444,A4445,A4446,A4447,A4448,A4449,A4450,A4451,A4452,A4453,A4454,A4455,A4456,A4457, \\note fields as indicated
 A4458,A4459,A4460,A4461,A4462,A4463,A4464,A4465,A4466,A4467,A4468,A4469,A4470,A4471,A4472, \\note fields as indicated
 A4473,A4474,A4475,A4476,A4477,A4478,A4479,A4480,A4481,A4482,A4483,A4484,A4485,A4486,A4487, \\note fields as indicated
 A4488,A4489,A4490,A4491,A4492,A4493,A4494,A4495,A4496,A4497,A4498,A4499,A4500; \\note fields as indicated

Schedule:Constant,
   \\memo Constant hourly value for entire year.
   \\format singleLine
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  N1 ; \\field Hourly Value
       \\type real
       \\default 0

Schedule:File,
 \\min-fields 5
       \\memo A Schedule:File points to a text computer file that has 8760-8784 hours of data.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field File Name
       \\required-field
       \\retaincase
  N1 , \\field Column Number
       \\required-field
       \\type integer
       \\minimum 1
  N2 , \\field Rows to Skip at Top
       \\required-field
       \\type integer
       \\minimum 0
  N3 , \\field Number of Hours of Data
       \\note 8760 hours does not account for leap years, 8784 does.
       \\note should be either 8760 or 8784
       \\default 8760
       \\minimum 8760
       \\maximum 8784
  A4 , \\field Column Separator
       \\type choice
       \\key Comma
       \\key Tab
       \\key Fixed
       \\key Semicolon
       \\default Comma
  A5 , \\field Interpolate to Timestep
       \\note when the interval does not match the user specified timestep a "Yes" choice will average between the intervals request (to
       \\note timestep resolution.  a "No" choice will use the interval value at the simulation timestep without regard to if it matches
       \\note the boundary or not.
       \\type choice
       \\key Yes
       \\key No
       \\default No
  N4 ; \\field Minutes per Item
       \\note Must be evenly divisible into 60
       \\type integer
       \\minimum 1
       \\maximum 60

\\group Surface Construction Elements

Material,
    \\memo Regular materials described with full set of thermal properties
    \\min-fields 6
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
  A2 , \\field Roughness
       \\required-field
       \\type choice
       \\key VeryRough
       \\key Rough
       \\key MediumRough
       \\key MediumSmooth
       \\key Smooth
       \\key VerySmooth
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum> 0
       \\maximum 3.0
       \\ip-units in
  N2 , \\field Conductivity
       \\required-field
       \\units W/m-K
       \\type real
       \\minimum> 0
  N3 , \\field Density
       \\required-field
       \\units kg/m3
       \\type real
       \\minimum> 0
  N4 , \\field Specific Heat
       \\required-field
       \\units J/kg-K
       \\type real
       \\minimum 100
  N5 , \\field Thermal Absorptance
       \\type real
       \\minimum> 0
       \\default .9
       \\maximum 0.99999
  N6 , \\field Solar Absorptance
       \\type real
       \\default .7
       \\minimum 0
       \\maximum 1
  N7 ; \\field Visible Absorptance
       \\type real
       \\minimum 0
       \\default .7
       \\maximum 1

Material:NoMass,
    \\memo Regular materials properties described whose principal description is R (Thermal Resistance)
     \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
  A2 , \\field Roughness
       \\required-field
       \\type choice
       \\key VeryRough
       \\key Rough
       \\key MediumRough
       \\key MediumSmooth
       \\key Smooth
       \\key VerySmooth
  N1 , \\field Thermal Resistance
       \\required-field
       \\units m2-K/W
       \\type real
       \\minimum .001
  N2 , \\field Thermal Absorptance
       \\type real
       \\minimum> 0
       \\default .9
       \\maximum 0.99999
  N3 , \\field Solar Absorptance
       \\type real
       \\minimum 0
       \\default .7
       \\maximum 1
  N4 ; \\field Visible Absorptance
       \\type real
       \\minimum 0
       \\default .7
       \\maximum 1

Material:InfraredTransparent,
       \\memo Special infrared transparent material.  Similar to a Material:Nomass with low thermal resistance.
       \\memo  High absorptance in both wavelengths.
       \\memo  Area will be doubled internally to make internal radiant exchange accurate.
       \\memo  Should be only material in single layer surface construction.
       \\memo  All thermal properties are set internally. User needs only to supply name.
       \\memo Cannot be used with ConductionFiniteDifference solution algorithms
       \\min-fields 1
  A1 ; \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName

Material:AirGap,
       \\min-fields 2
       \\memo Air Space in Opaque Construction
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
  N1 ; \\field Thermal Resistance
       \\required-field
       \\units m2-K/W
       \\type real
       \\minimum> 0

Material:RoofVegetation,
  \\memo EcoRoof model, plant layer plus soil layer
  \\memo Implemented by Portland State University
  \\memo (Sailor et al., January, 2007)
  \\memo only one material must be referenced per simulation though the same EcoRoof material could be
  \\memo used in multiple constructions. New moisture redistribution scheme (2010) requires higher
  \\memo number of timesteps per hour (minimum 12 recommended).
  \\min-fields 18
  A1 ,\\field Name
      \\required-field
      \\type alpha
      \\reference MaterialName
  N1 ,\\field Height of Plants
      \\type real
      \\note The ecoroof module is designed for short plants and shrubs.
      \\minimum> 0.005
      \\maximum  1.0
      \\units m
      \\default .2
  N2 ,\\field Leaf Area Index
      \\required-field
      \\units dimensionless
      \\note Entire surface is assumed covered, so decrease LAI accordingly.
      \\minimum> 0.001
      \\maximum 5.0
      \\default 1.0
      \\type real
  N3 ,\\field Leaf Reflectivity
      \\required-field
      \\type real
      \\units dimensionless
      \\note Leaf reflectivity (albedo) is typically 0.18-0.25
      \\maximum 0.5
      \\minimum 0.05
      \\default 0.22
  N4 ,\\field Leaf Emissivity
      \\required-field
      \\type real
      \\minimum 0.8
      \\maximum 1.0
      \\default 0.95
  N5 ,\\field Minimum Stomatal Resistance
      \\units s/m
      \\default 180.0
      \\note This depends upon plant type
      \\minimum 50.0
      \\maximum 300.
      \\type real
  A2 ,\\field Soil Layer Name
      \\type alpha
      \\default Green Roof Soil
  A3 ,\\field Roughness
      \\required-field
      \\type choice
      \\key VeryRough
      \\key MediumRough
      \\key Rough
      \\key Smooth
      \\key MediumSmooth
      \\key VerySmooth
      \\default MediumRough
  N6 ,\\field Thickness
      \\note thickness of the soil layer of the EcoRoof
      \\required-field
      \\units m
      \\type real
      \\note Soil depths of 0.15m (6in) and 0.30m (12in) are common.
      \\default 0.1
      \\minimum> 0.05
      \\maximum 0.7
      \\ip-units in
  N7 ,\\field Conductivity of Dry Soil
      \\required-field
      \\units W/m-K
      \\type real
      \\note Thermal conductivity of dry soil.
      \\note Typical ecoroof soils range from 0.3 to 0.5
      \\default 0.35
      \\minimum  0.2
      \\maximum 1.5
  N8 ,\\field Density of Dry Soil
      \\required-field
      \\units kg/m3
      \\note Density of dry soil (the code modifies this as the soil becomes moist)
      \\note Typical ecoroof soils range from 400 to 1000 (dry to wet)
      \\type real
      \\minimum 300
      \\maximum 2000
      \\default 1100
  N9 ,\\field Specific Heat of Dry Soil
      \\required-field
      \\units J/kg-K
      \\type real
      \\note Specific heat of dry soil
      \\minimum> 500
      \\maximum 2000
      \\default 1200
  N10,\\field Thermal Absorptance
      \\type real
      \\note Soil emissivity is typically in range of 0.90 to 0.98
      \\minimum> 0.8
      \\default .9
      \\maximum 1.0
  N11,\\field Solar Absorptance
      \\type real
      \\note Solar absorptance of dry soil (1-albedo) is typically 0.60 to 0.85
      \\note corresponding to a dry albedo of 0.15 to 0.40
      \\default .70
      \\minimum 0.40
      \\maximum 0.9
  N12,\\field Visible Absorptance
      \\type real
      \\minimum> 0.5
      \\default .75
      \\maximum 1.0
  N13,\\field Saturation Volumetric Moisture Content of the Soil Layer
      \\type real
      \\note Maximum moisture content is typically less than 0.5
      \\minimum> 0.1
      \\maximum 0.5
      \\default 0.3
  N14,\\field Residual Volumetric Moisture Content of the Soil Layer
      \\type real
      \\minimum  0.01
      \\maximum  0.1
      \\default 0.01
  N15,\\field Initial Volumetric Moisture Content of the Soil Layer
      \\type real
      \\minimum> 0.05
      \\maximum  0.5
      \\default 0.1
  A4; \\field Moisture Diffusion Calculation Method
      \\note Advanced calculation requires increased number of timesteps (recommended >20).
      \\type choice
      \\key Simple
      \\key Advanced
      \\default Advanced

WindowMaterial:SimpleGlazingSystem,
       \\min-fields 3
       \\memo Alternate method of describing windows
       \\memo This window material object is used to define an entire glazing system
       \\memo using simple performance parameters.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference GlazingMaterialName
  N1 , \\field U-Factor
       \\required-field
       \\note Enter U-Factor including film coefficients
       \\note Note that the effective upper limit for U-factor is 5.8 W/m2-K
       \\units W/m2-K
       \\minimum> 0
       \\maximum  7
  N2 , \\field Solar Heat Gain Coefficient
       \\required-field
       \\note SHGC at Normal Incidence
       \\minimum> 0
       \\maximum< 1
  N3 ; \\field Visible Transmittance
       \\note VT at Normal Incidence
       \\note optional
       \\minimum> 0
       \\maximum< 1

WindowMaterial:Glazing,
    \\min-fields 14
    \\memo Glass material properties for Windows or Glass Doors
    \\memo Transmittance/Reflectance input method.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference GlazingMaterialName
  A2 , \\field Optical Data Type
       \\required-field
       \\type choice
       \\key SpectralAverage
       \\key Spectral
       \\key BSDF
  A3 , \\field Window Glass Spectral Data Set Name
       \\note Used only when Optical Data Type = Spectral
       \\object-list SpectralDataSets
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
       \\ip-units in
  N2 , \\field Solar Transmittance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N3 , \\field Front Side Solar Reflectance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\note Front Side is side closest to outdoor air
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N4 , \\field Back Side Solar Reflectance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\note Back Side is side closest to zone air
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N5 , \\field Visible Transmittance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N6 , \\field Front Side Visible Reflectance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N7 , \\field Back Side Visible Reflectance at Normal Incidence
       \\note Used only when Optical Data Type = SpectralAverage
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N8 , \\field Infrared Transmittance at Normal Incidence
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N9 , \\field Front Side Infrared Hemispherical Emissivity
       \\type real
       \\minimum> 0.0
       \\maximum< 1.0
       \\default 0.84
  N10, \\field Back Side Infrared Hemispherical Emissivity
       \\type real
       \\minimum> 0.0
       \\maximum< 1.0
       \\default 0.84
  N11, \\field Conductivity
       \\units W/m-K
       \\type real
       \\minimum> 0.0
       \\default 0.9
  N12, \\field Dirt Correction Factor for Solar and Visible Transmittance
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\default 1.0
  A4,  \\field Solar Diffusing
       \\type choice
       \\key No
       \\key Yes
       \\default No
  N13, \\field Young's modulus
       \\note coefficient used for deflection calculations. Used only with complex
       \\note fenestration when deflection model is set to TemperatureAndPressureInput
       \\units Pa
       \\type real
       \\minimum> 0.0
       \\default 7.2e10
  N14; \\field Poisson's ratio
       \\note coefficient used for deflection calculations. Used only with complex
       \\note fenestration when deflection model is set to TemperatureAndPressureInput
       \\type real
       \\minimum > 0.0
       \\maximum < 1.0
       \\default 0.22

WindowMaterial:GlazingGroup:Thermochromic,
    \\min-fields 3
    \\extensible:2 - repeat last two fields remembering to remove ; from "inner" fields.
    \\memo thermochromic glass at different temperatures
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference GlazingMaterialName
  N1 , \\field Optical Data Temperature 1
       \\begin-extensible
       \\required-field
       \\units C
       \\ip-units F
       \\type real
  A2 , \\field Window Material Glazing Name 1
       \\required-field
       \\type object-list
       \\object-list GlazingMaterialName
  N2 , \\field Optical Data Temperature 2
       \\units C
       \\ip-units F
       \\type real
  A3 , \\field Window Material Glazing Name 2
       \\type object-list
       \\object-list GlazingMaterialName
  N3 , \\field Optical Data Temperature 3
       \\units C
       \\ip-units F
       \\type real
  A4 , \\field Window Material Glazing Name 3
       \\type object-list
       \\object-list GlazingMaterialName
  N4 , \\field Optical Data Temperature 4
       \\units C
       \\ip-units F
       \\type real
  A5 , \\field Window Material Glazing Name 4
       \\type object-list
       \\object-list GlazingMaterialName
  N5 , \\field Optical Data Temperature 5
       \\units C
       \\ip-units F
       \\type real
  A6 , \\field Window Material Glazing Name 5
       \\type object-list
       \\object-list GlazingMaterialName
  N6 , \\field Optical Data Temperature 6
       \\units C
       \\ip-units F
       \\type real
  A7 , \\field Window Material Glazing Name 6
       \\type object-list
       \\object-list GlazingMaterialName
  N7 , \\field Optical Data Temperature 7
       \\units C
       \\ip-units F
       \\type real
  A8 , \\field Window Material Glazing Name 7
       \\type object-list
       \\object-list GlazingMaterialName
  N8 , \\field Optical Data Temperature 8
       \\units C
       \\ip-units F
       \\type real
  A9 , \\field Window Material Glazing Name 8
       \\type object-list
       \\object-list GlazingMaterialName
  N9 , \\field Optical Data Temperature 9
       \\units C
       \\ip-units F
       \\type real
  A10, \\field Window Material Glazing Name 9
       \\type object-list
       \\object-list GlazingMaterialName
  N10, \\field Optical Data Temperature 10
       \\units C
       \\ip-units F
       \\type real
  A11, \\field Window Material Glazing Name 10
       \\type object-list
       \\object-list GlazingMaterialName
  N11, \\field Optical Data Temperature 11
       \\units C
       \\ip-units F
       \\type real
  A12, \\field Window Material Glazing Name 11
       \\type object-list
       \\object-list GlazingMaterialName
  N12, \\field Optical Data Temperature 12
       \\units C
       \\ip-units F
       \\type real
  A13, \\field Window Material Glazing Name 12
       \\type object-list
       \\object-list GlazingMaterialName
  N13, \\field Optical Data Temperature 13
       \\units C
       \\ip-units F
       \\type real
  A14, \\field Window Material Glazing Name 13
       \\type object-list
       \\object-list GlazingMaterialName
  N14, \\field Optical Data Temperature 14
       \\units C
       \\ip-units F
       \\type real
  A15, \\field Window Material Glazing Name 14
       \\type object-list
       \\object-list GlazingMaterialName
  N15, \\field Optical Data Temperature 15
       \\units C
       \\ip-units F
       \\type real
  A16, \\field Window Material Glazing Name 15
       \\type object-list
       \\object-list GlazingMaterialName
  N16, \\field Optical Data Temperature 16
       \\units C
       \\ip-units F
       \\type real
  A17, \\field Window Material Glazing Name 16
       \\type object-list
       \\object-list GlazingMaterialName
  N17, \\field Optical Data Temperature 17
       \\units C
       \\ip-units F
       \\type real
  A18, \\field Window Material Glazing Name 17
       \\type object-list
       \\object-list GlazingMaterialName
  N18, \\field Optical Data Temperature 18
       \\units C
       \\ip-units F
       \\type real
  A19, \\field Window Material Glazing Name 18
       \\type object-list
       \\object-list GlazingMaterialName
  N19, \\field Optical Data Temperature 19
       \\units C
       \\ip-units F
       \\type real
  A20, \\field Window Material Glazing Name 19
       \\type object-list
       \\object-list GlazingMaterialName
  N20, \\field Optical Data Temperature 20
       \\units C
       \\ip-units F
       \\type real
  A21, \\field Window Material Glazing Name 20
       \\type object-list
       \\object-list GlazingMaterialName
  N21, \\field Optical Data Temperature 21
       \\units C
       \\ip-units F
       \\type real
  A22, \\field Window Material Glazing Name 21
       \\type object-list
       \\object-list GlazingMaterialName
  N22, \\field Optical Data Temperature 22
       \\units C
       \\ip-units F
       \\type real
  A23, \\field Window Material Glazing Name 22
       \\type object-list
       \\object-list GlazingMaterialName
  N23, \\field Optical Data Temperature 23
       \\units C
       \\ip-units F
       \\type real
  A24, \\field Window Material Glazing Name 23
       \\type object-list
       \\object-list GlazingMaterialName
  N24, \\field Optical Data Temperature 24
       \\units C
       \\ip-units F
       \\type real
  A25, \\field Window Material Glazing Name 24
       \\type object-list
       \\object-list GlazingMaterialName
  N25, \\field Optical Data Temperature 25
       \\units C
       \\ip-units F
       \\type real
  A26, \\field Window Material Glazing Name 25
       \\type object-list
       \\object-list GlazingMaterialName
  N26, \\field Optical Data Temperature 26
       \\units C
       \\ip-units F
       \\type real
  A27, \\field Window Material Glazing Name 26
       \\type object-list
       \\object-list GlazingMaterialName
  N27, \\field Optical Data Temperature 27
       \\units C
       \\ip-units F
       \\type real
  A28, \\field Window Material Glazing Name 27
       \\type object-list
       \\object-list GlazingMaterialName
  N28, \\field Optical Data Temperature 28
       \\units C
       \\ip-units F
       \\type real
  A29, \\field Window Material Glazing Name 28
       \\type object-list
       \\object-list GlazingMaterialName
  N29, \\field Optical Data Temperature 29
       \\units C
       \\ip-units F
       \\type real
  A30, \\field Window Material Glazing Name 29
       \\type object-list
       \\object-list GlazingMaterialName
  N30, \\field Optical Data Temperature 30
       \\units C
       \\ip-units F
       \\type real
  A31, \\field Window Material Glazing Name 30
       \\type object-list
       \\object-list GlazingMaterialName
  N31, \\field Optical Data Temperature 31
       \\units C
       \\ip-units F
       \\type real
  A32, \\field Window Material Glazing Name 31
       \\type object-list
       \\object-list GlazingMaterialName
  N32, \\field Optical Data Temperature 32
       \\units C
       \\ip-units F
       \\type real
  A33, \\field Window Material Glazing Name 32
       \\type object-list
       \\object-list GlazingMaterialName
  N33, \\field Optical Data Temperature 33
       \\units C
       \\ip-units F
       \\type real
  A34, \\field Window Material Glazing Name 33
       \\type object-list
       \\object-list GlazingMaterialName
  N34, \\field Optical Data Temperature 34
       \\units C
       \\ip-units F
       \\type real
  A35, \\field Window Material Glazing Name 34
       \\type object-list
       \\object-list GlazingMaterialName
  N35, \\field Optical Data Temperature 35
       \\units C
       \\ip-units F
       \\type real
  A36, \\field Window Material Glazing Name 35
       \\type object-list
       \\object-list GlazingMaterialName
  N36, \\field Optical Data Temperature 36
       \\units C
       \\ip-units F
       \\type real
  A37, \\field Window Material Glazing Name 36
       \\type object-list
       \\object-list GlazingMaterialName
  N37, \\field Optical Data Temperature 37
       \\units C
       \\ip-units F
       \\type real
  A38, \\field Window Material Glazing Name 37
       \\type object-list
       \\object-list GlazingMaterialName
  N38, \\field Optical Data Temperature 38
       \\units C
       \\ip-units F
       \\type real
  A39, \\field Window Material Glazing Name 38
       \\type object-list
       \\object-list GlazingMaterialName
  N39, \\field Optical Data Temperature 39
       \\units C
       \\ip-units F
       \\type real
  A40, \\field Window Material Glazing Name 39
       \\type object-list
       \\object-list GlazingMaterialName
  N40, \\field Optical Data Temperature 40
       \\units C
       \\ip-units F
       \\type real
  A41, \\field Window Material Glazing Name 40
       \\type object-list
       \\object-list GlazingMaterialName
  N41, \\field Optical Data Temperature 41
       \\units C
       \\ip-units F
       \\type real
  A42, \\field Window Material Glazing Name 41
       \\type object-list
       \\object-list GlazingMaterialName
  N42, \\field Optical Data Temperature 42
       \\units C
       \\ip-units F
       \\type real
  A43, \\field Window Material Glazing Name 42
       \\type object-list
       \\object-list GlazingMaterialName
  N43, \\field Optical Data Temperature 43
       \\units C
       \\ip-units F
       \\type real
  A44, \\field Window Material Glazing Name 43
       \\type object-list
       \\object-list GlazingMaterialName
  N44, \\field Optical Data Temperature 44
       \\units C
       \\ip-units F
       \\type real
  A45, \\field Window Material Glazing Name 44
       \\type object-list
       \\object-list GlazingMaterialName
  N45, \\field Optical Data Temperature 45
       \\units C
       \\ip-units F
       \\type real
  A46; \\field Window Material Glazing Name 45
       \\type object-list
       \\object-list GlazingMaterialName

WindowMaterial:Glazing:RefractionExtinctionMethod,
    \\memo Glass material properties for Windows or Glass Doors
    \\memo Index of Refraction/Extinction Coefficient input method
    \\memo Not to be used for coated glass
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference GlazingMaterialName
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
       \\ip-units in
  N2 , \\field Solar Index of Refraction
       \\required-field
       \\type real
       \\minimum> 1.0
  N3 , \\field Solar Extinction Coefficient
       \\required-field
       \\units 1/m
       \\type real
       \\minimum> 0.0
  N4 , \\field Visible Index of Refraction
       \\required-field
       \\type real
       \\minimum> 1.0
  N5 , \\field Visible Extinction Coefficient
       \\required-field
       \\units 1/m
       \\type real
       \\minimum> 0.0
  N6 , \\field Infrared Transmittance at Normal Incidence
       \\type real
       \\maximum< 1.0
       \\minimum 0.0
       \\default 0.0
  N7 , \\field Infrared Hemispherical Emissivity
       \\note Emissivity of front and back side assumed equal
       \\type real
       \\maximum< 1.0
       \\minimum> 0.0
       \\default 0.84
  N8 , \\field Conductivity
       \\units W/m-K
       \\type real
       \\minimum> 0.0
       \\default 0.9
  N9 , \\field Dirt Correction Factor for Solar and Visible Transmittance
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\default 1.0
  A2 ; \\field Solar Diffusing
       \\type choice
       \\key No
       \\key Yes
       \\default No

WindowMaterial:Gas,
    \\memo Gas material properties that are used in Windows or Glass Doors
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference WindowGasAndGasMixtures
  A2 , \\field Gas Type
       \\required-field
       \\type choice
       \\key Air
       \\key Argon
       \\key Krypton
       \\key Xenon
       \\key Custom
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
       \\ip-units in
  N2 , \\field Conductivity Coefficient A
       \\units W/m-K
       \\type real
       \\note Used only if Gas Type = Custom
  N3 , \\field Conductivity Coefficient B
       \\units W/m-K2
       \\type real
       \\note Used only if Gas Type = Custom
  N4 , \\field Conductivity Coefficient C
       \\units W/m-K3
       \\type real
       \\note Used only if Gas Type = Custom
  N5 , \\field Viscosity Coefficient A
       \\units kg/m-s
       \\type real
       \\note Used only if Gas Type = Custom
       \\minimum> 0.0
  N6 , \\field Viscosity Coefficient B
       \\units kg/m-s-K
       \\type real
       \\note Used only if Gas Type = Custom
  N7 , \\field Viscosity Coefficient C
       \\units kg/m-s-K2
       \\type real
       \\note Used only if Gas Type = Custom
  N8 , \\field Specific Heat Coefficient A
       \\units J/kg-K
       \\type real
       \\note Used only if Gas Type = Custom
       \\minimum> 0.0
  N9 , \\field Specific Heat Coefficient B
       \\units J/kg-K2
       \\type real
       \\note Used only if Gas Type = Custom
  N10 ,\\field Specific Heat Coefficient C
       \\units J/kg-K3
       \\type real
       \\note Used only if Gas Type = Custom
  N11 ,\\field Molecular Weight
       \\type real
       \\note Used only if Gas Type = Custom
       \\units g/mol
       \\minimum 20.0
       \\maximum 200.0
  N12 ;\\field Specific Heat Ratio
       \\type real
       \\note Used only if Gas Type = Custom
       \\minimum >1.0

WindowGap:SupportPillar,
    \\memo used to define pillar geometry for support pillars
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WindowGapSupportPillars
  N1 , \\field Spacing
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 0.04
 N2 ; \\field Radius
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 0.0004

WindowGap:DeflectionState,
       \\memo Used to enter data describing deflection state of the gap. It is referenced from
       \\memo WindowMaterial:Gap object only and it is used only when deflection model is set to
       \\memo MeasuredDeflection, otherwise it is ignored.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WindowGapDeflectionStates
  N1 , \\field Deflected Thickness
       \\note if left blank will be considered that gap has no deflection
       \\units m
       \\type real
       \\minimum 0.0
  N2 , \\field Initial Temperature
       \\units C
       \\type real
       \\minimum 0.0
       \\default 25
  N3 ; \\field Initial Pressure
       \\units Pa
       \\type real
       \\minimum 0.0
       \\default 101325

WindowMaterial:GasMixture,
    \\memo Gas mixtures that are used in Windows or Glass Doors
       \\min-fields 7
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference WindowGasAndGasMixtures
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
  N2 , \\field Number of Gases in Mixture
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 4
  A2 , \\field Gas 1 Type
       \\required-field
       \\type choice
       \\key Air
       \\key Argon
       \\key Krypton
       \\key Xenon
  N3 , \\field Gas 1 Fraction
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  A3 , \\field Gas 2 Type
       \\required-field
       \\type choice
       \\key Air
       \\key Argon
       \\key Krypton
       \\key Xenon
  N4 , \\field Gas 2 Fraction
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  A4 , \\field Gas 3 Type
       \\type choice
       \\key Air
       \\key Argon
       \\key Krypton
       \\key Xenon
  N5 , \\field Gas 3 Fraction
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  A5 , \\field Gas 4 Type
       \\type choice
       \\key Air
       \\key Argon
       \\key Krypton
       \\key Xenon
  N6 ; \\field Gas 4 Fraction
       \\type real
       \\minimum> 0.0
       \\maximum 1.0

WindowMaterial:Gap,
       \\memo Used to define the gap between two layers in a complex fenestration system, where the
       \\memo Construction:ComplexFenestrationState object is used. It is referenced as a layer in the
       \\memo Construction:ComplexFenestrationState object. It cannot be referenced as a layer from the
       \\memo Construction object.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
  N1 , \\field Thickness
       \\required-field
       \\units m
       \\type real
       \\minimum >0
  A2 , \\field Gas (or Gas Mixture)
       \\note This field should reference only WindowMaterial:Gas
       \\note or WindowMaterial:GasMixture objects
       \\required-field
       \\type object-list
       \\object-list WindowGasAndGasMixtures
  N2 , \\field Pressure
       \\required-field
       \\units Pa
       \\type real
       \\default 101325
  A3 , \\field Deflection State
       \\type object-list
       \\object-list WindowGapDeflectionStates
  A4 ; \\field Support Pillar
       \\type object-list
       \\object-list WindowGapSupportPillars

WindowMaterial:Shade,
       \\memo Specifies the properties of window shade materials. Reflectance and emissivity
       \\memo properties are assumed to be the same on both sides of the shade. Shades are considered
       \\memo to be perfect diffusers (all transmitted and reflected radiation is
       \\memo hemispherically-diffuse) independent of angle of incidence.
  \\min-fields 15
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference WindowShadesScreensAndBlinds
  N1 , \\field Solar Transmittance
       \\required-field
       \\note Assumed independent of incidence angle
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N2 , \\field Solar Reflectance
       \\required-field
       \\note Assumed same for both sides
       \\note Assumed independent of incidence angle
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N3 , \\field Visible Transmittance
       \\required-field
       \\note Assumed independent of incidence angle
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N4 , \\field Visible Reflectance
       \\required-field
       \\note Assumed same for both sides
       \\note Assumed independent of incidence angle
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N5 , \\field Infrared Hemispherical Emissivity
       \\required-field
       \\type real
       \\units dimensionless
       \\maximum< 1
       \\minimum> 0
  N6 , \\field Infrared Transmittance
       \\required-field
       \\type real
       \\units dimensionless
       \\maximum< 1
       \\minimum 0
  N7 , \\field Thickness
       \\required-field
       \\type real
       \\units m
       \\minimum> 0
       \\ip-units in
  N8 , \\field Conductivity
       \\required-field
       \\units W/m-K
       \\type real
       \\minimum> 0
  N9 , \\field Shade to Glass Distance
       \\type real
       \\units m
       \\minimum 0.001
       \\maximum 1.0
       \\default 0.050
       \\ip-units in
  N10, \\field Top Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N11, \\field Bottom Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N12, \\field Left-Side Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N13, \\field Right-Side Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N14; \\field Airflow Permeability
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 0.8
       \\default 0.0

WindowMaterial:ComplexShade,
    \\memo Complex window shading layer thermal properties
     \\min-fields 12
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WindowComplexShades
       \\reference WindowShadesScreensAndBlinds
  A2 , \\field Layer Type
       \\type choice
       \\key Venetian
       \\key Woven
       \\key Perforated
  N1 , \\field Thickness
       \\type real
       \\units m
       \\minimum> 0
       \\default 0.002
  N2 , \\field Conductivity
       \\units W/m-K
       \\type real
       \\minimum> 0
       \\default 1
  N3 , \\field IR Transmittance
       \\type real
       \\maximum 1
       \\minimum 0
       \\default 0
  N4 , \\field Front Emissivity
       \\type real
       \\maximum 1
       \\minimum 0
       \\default 0.84
  N5 , \\field Back Emissivity
       \\type real
       \\maximum 1
       \\minimum 0
       \\default 0.84
  N6 , \\field TopOpeningMultiplier
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0
  N7 , \\field BottomOpeningMultiplier
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0
  N8 , \\field LeftSideOpeningMultiplier
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0
  N9 , \\field RightSideOpeningMultiplier
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0
 N10 , \\field FrontOpeningMultiplier
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0.05
 N11 , \\field Slat Width
       \\units m
       \\type real
       \\minimum> 0
       \\default 0.016
 N12 , \\field Slat Spacing
       \\note Distance between adjacent slat faces
       \\units m
       \\type real
       \\minimum> 0
       \\default 0.012
  N13 , \\field Slat Thickness
       \\note Distance between top and bottom surfaces of slat
       \\note Slat is assumed to be rectangular in cross section and flat
       \\type real
       \\units m
       \\minimum> 0
       \\default 0.0006
  N14 , \\field Slat Angle
       \\type real
       \\units deg
       \\default 90
       \\minimum -90
       \\maximum 90
  N15 , \\field Slat Conductivity
       \\type real
       \\units W/m-K
       \\minimum> 0
       \\default 160.0
  N16 ; \\field Slat Curve
       \\note this value represents curvature radius of the slat.
       \\note if the slat is flat use zero.
       \\note if this value is not zero, then it must be > SlatWidth/2.
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0

WindowMaterial:Blind,
    \\memo Window blind thermal properties
     \\min-fields 29
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference WindowShadesScreensAndBlinds
  A2 , \\field Slat Orientation
       \\type choice
       \\key Horizontal
       \\key Vertical
       \\default Horizontal
  N1 , \\field Slat Width
       \\required-field
       \\units m
       \\type real
       \\minimum> 0
       \\maximum 1
       \\ip-units in
  N2 , \\field Slat Separation
       \\required-field
       \\note Distance between adjacent slat faces
       \\units m
       \\type real
       \\minimum> 0
       \\maximum 1
       \\ip-units in
  N3 , \\field Slat Thickness
       \\note Distance between top and bottom surfaces of slat
       \\note Slat is assumed to be rectangular in cross section and flat
       \\type real
       \\units m
       \\minimum> 0
       \\maximum 0.1
       \\default 0.00025
       \\ip-units in
  N4 , \\field Slat Angle
       \\note If WindowProperty:ShadingControl for the window that incorporates this blind
       \\note has Type of Slat Angle Control for Blinds = FixedSlatAngle,
       \\note then this is the fixed value of the slat angle;
       \\note If WindowProperty:ShadingControl for the window that incorporates this blind
       \\note has Type of Slat Angle Control for Blinds = BlockBeamSolar,
       \\note then this is the slat angle when slat angle control
       \\note is not in effect (e.g., when there is no beam solar on the blind);
       \\note Not used if WindowProperty:ShadingControl for the window that incorporates this blind
       \\note has Type of Slat Angle Control for Blinds = ScheduledSlatAngle.
       \\type real
       \\units deg
       \\default 45
       \\minimum 0
       \\maximum 180
  N5 , \\field Slat Conductivity
       \\type real
       \\units W/m-K
       \\minimum> 0
       \\default 221.0
       \\note default is for aluminum
  N6 , \\field Slat Beam Solar Transmittance
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0
  N7 , \\field Front Side Slat Beam Solar Reflectance
       \\required-field
       \\type real
       \\maximum< 1
       \\minimum 0
  N8 , \\field Back Side Slat Beam Solar Reflectance
       \\required-field
       \\type real
       \\maximum< 1
       \\minimum 0
  N9 , \\field Slat Diffuse Solar Transmittance
       \\note Must equal "Slat beam solar transmittance"
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0
  N10, \\field Front Side Slat Diffuse Solar Reflectance
       \\note Must equal "Front Side Slat Beam Solar Reflectance"
       \\required-field
       \\type real
       \\maximum< 1
       \\minimum 0
  N11, \\field Back Side Slat Diffuse Solar Reflectance
       \\note Must equal "Back Side Slat Beam Solar Reflectance"
       \\required-field
       \\type real
       \\maximum< 1
       \\minimum 0
  N12, \\field Slat Beam Visible Transmittance
       \\note Required for detailed daylighting calculation
       \\type real
       \\maximum< 1
       \\minimum 0
       \\required-field
  N13, \\field Front Side Slat Beam Visible Reflectance
       \\note Required for detailed daylighting calculation
       \\type real
       \\maximum< 1
       \\minimum 0
  N14, \\field Back Side Slat Beam Visible Reflectance
       \\note Required for detailed daylighting calculation
       \\type real
       \\maximum< 1
       \\minimum 0
  N15, \\field Slat Diffuse Visible Transmittance
       \\note Used only for detailed daylighting calculation
       \\note Must equal "Slat Beam Visible Transmittance"
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0
  N16, \\field Front Side Slat Diffuse Visible Reflectance
       \\note Required for detailed daylighting calculation
       \\note Must equal "Front Side Slat Beam Visible Reflectance"
       \\type real
       \\maximum< 1
       \\minimum 0
  N17, \\field Back Side Slat Diffuse Visible Reflectance
       \\note Required for detailed daylighting calculation
       \\note Must equal "Back Side Slat Beam Visible Reflectance"
       \\type real
       \\maximum< 1
       \\minimum 0
  N18, \\field Slat Infrared Hemispherical Transmittance
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0
  N19, \\field Front Side Slat Infrared Hemispherical Emissivity
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0.9
  N20, \\field Back Side Slat Infrared Hemispherical Emissivity
       \\type real
       \\maximum< 1
       \\minimum 0
       \\default 0.9
  N21, \\field Blind to Glass Distance
       \\type real
       \\units m
       \\minimum 0.01
       \\maximum 1.0
       \\default 0.050
       \\ip-units in
  N22, \\field Blind Top Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N23, \\field Blind Bottom Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N24, \\field Blind Left Side Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N25, \\field Blind Right Side Opening Multiplier
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.5
  N26, \\field Minimum Slat Angle
       \\note Used only if WindowProperty:ShadingControl for the window that incorporates
       \\note this blind varies the slat angle (i.e., WindowProperty:ShadingControl with
       \\note Type of Slat Angle Control for Blinds = ScheduledSlatAngle
       \\note or BlockBeamSolar)
       \\type real
       \\units deg
       \\default 0
       \\minimum 0
       \\maximum 180
  N27; \\field Maximum Slat Angle
       \\note Used only if WindowProperty:ShadingControl for the window that incorporates
       \\note this blind varies the slat angle (i.e., WindowProperty:ShadingControl with
       \\note Type of Slat Angle Control for Blinds = ScheduledSlatAngle
       \\note or BlockBeamSolar)
       \\type real
       \\units deg
       \\default 180
       \\minimum 0
       \\maximum 180

WindowMaterial:Screen,
       \\memo Window screen physical properties. Can only be located on the exterior side of a window construction.
       \\min-fields 9
  A1 , \\field Name
       \\note Enter a unique name for this window screen material.
       \\required-field
       \\type alpha
       \\reference MaterialName
       \\reference WindowShadesScreensAndBlinds
  A2 , \\field Reflected Beam Transmittance Accounting Method
       \\note Select the method used to account for the beam solar reflected off the material surface.
       \\type choice
       \\key DoNotModel
       \\key ModelAsDirectBeam
       \\key ModelAsDiffuse
       \\default ModelAsDiffuse
  N1 , \\field Diffuse Solar Reflectance
       \\note Diffuse reflectance of the screen material over the entire solar radiation spectrum.
       \\note Assumed to be the same for both sides of the screen.
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N2 , \\field Diffuse Visible Reflectance
       \\note Diffuse visible reflectance of the screen material averaged over the solar spectrum
       \\note and weighted by the response of the human eye.
       \\note Assumed to be the same for both sides of the screen.
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum 0
       \\maximum< 1
  N3 , \\field Thermal Hemispherical Emissivity
       \\note Long-wave emissivity of the screen material.
       \\note Assumed to be the same for both sides of the screen.
       \\type real
       \\units dimensionless
       \\maximum< 1
       \\minimum> 0
       \\default 0.9
  N4 , \\field Conductivity
       \\note Thermal conductivity of the screen material.
       \\type real
       \\units W/m-K
       \\minimum> 0
       \\default 221.0
       \\note Default is for aluminum.
  N5 , \\field Screen Material Spacing
       \\note Spacing assumed to be the same in both directions.
       \\required-field
       \\type real
       \\units m
       \\minimum> 0
       \\ip-units in
  N6 , \\field Screen Material Diameter
       \\note Diameter assumed to be the same in both directions.
       \\required-field
       \\type real
       \\units m
       \\minimum> 0
       \\ip-units in
  N7 , \\field Screen to Glass Distance
       \\note Distance from the window screen to the adjacent glass surface.
       \\type real
       \\units m
       \\minimum 0.001
       \\maximum 1.0
       \\default 0.025
       \\ip-units in
  N8 , \\field Top Opening Multiplier
       \\note Effective area for air flow at the top of the screen divided by the perpendicular
       \\note area between the glass and the top of the screen.
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N9 , \\field Bottom Opening Multiplier
       \\note Effective area for air flow at the bottom of the screen divided by the perpendicular
       \\note area between the glass and the bottom of the screen.
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N10, \\field Left Side Opening Multiplier
       \\note Effective area for air flow at the left side of the screen divided by the perpendicular
       \\note area between the glass and the left side of the screen.
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N11, \\field Right Side Opening Multiplier
       \\note Effective area for air flow at the right side of the screen divided by the perpendicular
       \\note area between the glass and the right side of the screen.
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N12; \\field Angle of Resolution for Screen Transmittance Output Map
       \\note Select the resolution of azimuth and altitude angles for the screen transmittance map.
       \\note A value of 0 means no transmittance map will be generated.
       \\note Valid values for this field are 0, 1, 2, 3 and 5.
       \\type choice
       \\units deg
       \\key 0
       \\key 1
       \\key 2
       \\key 3
       \\key 5
       \\default 0

MaterialProperty:MoisturePenetrationDepth:Settings,
    \\memo Additional properties for moisture using EMPD procedure
    \\memo HeatBalanceAlgorithm choice=MoisturePenetrationDepthConductionTransferFunction only
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Material Name that the moisture properties will be added to.
       \\note Additional material properties required to perform the EMPD model.
       \\note Effective Mean Penetration Depth (EMPD)
  N1 , \\field Moisture Penetration Depth
       \\required-field
       \\note This is the penetration depth
       \\units m
       \\minimum 0.0
       \\type real
  N2 , \\field Moisture Equation Coefficient a
       \\required-field
       \\units dimensionless
       \\type real
  N3 , \\field Moisture Equation Coefficient b
       \\required-field
       \\units dimensionless
       \\type real
  N4 , \\field Moisture Equation Coefficient c
       \\required-field
       \\units dimensionless
       \\type real
  N5 ; \\field Moisture Equation Coefficient d
       \\required-field
       \\units dimensionless
       \\type real

MaterialProperty:PhaseChange,
    \\memo Additional properties for temperature dependent thermal conductivity
    \\memo and enthalpy for Phase Change Materials (PCM)
    \\memo HeatBalanceAlgorithm = CondFD(ConductionFiniteDifference) solution algorithm only.
    \\memo Constructions with this should use the detailed CondFD process.
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Regular Material Name to which the additional properties will be added.
       \\note this the material name for the basic material properties.
  N1 , \\field Temperature Coefficient for Thermal Conductivity
       \\note The base temperature is 20C.
       \\note This is the thermal conductivity change per degree excursion from 20C.
       \\note This variable conductivity function is overridden by the VariableThermalConductivity object, if present.
       \\units W/m-K2
       \\type real
       \\default 0.0
  N2 , \\field Temperature 1
       \\required-field
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N3,  \\field Enthalpy 1
       \\required-field
       \\note for Temperature-enthalpy function corresponding to temperature 1
       \\units J/kg
       \\default 0.0
  N4 , \\field Temperature 2
       \\required-field
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N5,  \\field Enthalpy 2
       \\required-field
       \\note for Temperature-enthalpy function corresponding to temperature 2
       \\units J/kg
       \\type real
       \\default 0.0
  N6 , \\field Temperature 3
       \\required-field
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N7,  \\field Enthalpy 3
       \\required-field
       \\note for Temperature-enthalpy function corresponding to temperature 3
       \\units J/kg
       \\type real
       \\default 0.0
  N8 , \\field Temperature 4
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N9,  \\field Enthalpy 4
       \\note for Temperature-enthalpy function corresponding to temperature 4
       \\units J/kg
       \\type real
       \\default 0.0
  N10, \\field Temperature 5
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N11, \\field Enthalpy 5
       \\note for Temperature-enthalpy function corresponding to temperature 5
       \\units J/kg
       \\type real
       \\default 0.0
  N12, \\field Temperature 6
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N13, \\field Enthalpy 6
       \\note for Temperature-enthalpy function corresponding to temperature 6
       \\units J/kg
       \\type real
       \\default 0.0
  N14, \\field Temperature 7
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N15, \\field Enthalpy 7
       \\note for Temperature-enthalpy function corresponding to temperature 7
       \\units J/kg
       \\type real
       \\default 0.0
  N16, \\field Temperature 8
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N17, \\field Enthalpy 8
       \\note for Temperature-enthalpy function corresponding to temperature 8
       \\units J/kg
       \\type real
       \\default 0.0
  N18 , \\field Temperature 9
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N19,  \\field Enthalpy 9
       \\note for Temperature-enthalpy function corresponding to temperature 1
       \\units J/kg
       \\default 0.0
  N20 , \\field Temperature 10
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N21,  \\field Enthalpy 10
       \\note for Temperature-enthalpy function corresponding to temperature 2
       \\units J/kg
       \\type real
       \\default 0.0
  N22 , \\field Temperature 11
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N23,  \\field Enthalpy 11
       \\note for Temperature-enthalpy function corresponding to temperature 3
       \\units J/kg
       \\type real
       \\default 0.0
  N24 , \\field Temperature 12
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N25,  \\field Enthalpy 12
       \\note for Temperature-enthalpy function corresponding to temperature 14
       \\units J/kg
       \\type real
       \\default 0.0
  N26, \\field Temperature 13
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N27, \\field Enthalpy 13
       \\note for Temperature-enthalpy function corresponding to temperature 15
       \\units J/kg
       \\type real
       \\default 0.0
  N28, \\field Temperature 14
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N29, \\field Enthalpy 14
       \\note for Temperature-enthalpy function corresponding to temperature 16
       \\units J/kg
       \\type real
       \\default 0.0
  N30, \\field Temperature 15
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N31, \\field Enthalpy 15
       \\note for Temperature-enthalpy function corresponding to temperature 17
       \\units J/kg
       \\type real
       \\default 0.0
  N32, \\field Temperature 16
       \\note for Temperature-enthalpy function
       \\units C
       \\type real
       \\default 0.0
  N33; \\field Enthalpy 16
       \\note for Temperature-enthalpy function corresponding to temperature 16
       \\units J/kg
       \\type real
       \\default 0.0

MaterialProperty:VariableThermalConductivity,
    \\memo Additional properties for temperature dependent thermal conductivity
    \\memo using piecewise linear temperature-conductivity function.
    \\memo HeatBalanceAlgorithm = CondFD(ConductionFiniteDifference) solution algorithm only.
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Regular Material Name to which the additional properties will be added.
       \\note this the material name for the basic material properties.
  N1 , \\field Temperature 1
       \\required-field
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N2,  \\field Thermal Conductivity 1
       \\required-field
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 1
       \\units W/m-K
       \\default 0.0
  N3 , \\field Temperature 2
       \\required-field
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N4,  \\field Thermal Conductivity 2
       \\required-field
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 2
       \\units W/m-K
       \\type real
       \\default 0.0
  N5 , \\field Temperature 3
       \\required-field
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N6,  \\field Thermal Conductivity 3
       \\required-field
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 3
       \\units W/m-K
       \\type real
       \\default 0.0
  N7 , \\field Temperature 4
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N8,  \\field Thermal Conductivity 4
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 4
       \\units W/m-K
       \\type real
       \\default 0.0
  N9, \\field Temperature 5
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N10, \\field Thermal Conductivity 5
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 5
       \\units W/m-K
       \\type real
       \\default 0.0
  N11, \\field Temperature 6
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N12, \\field Thermal Conductivity 6
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 6
       \\units W/m-K
       \\type real
       \\default 0.0
  N13, \\field Temperature 7
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N14, \\field Thermal Conductivity 7
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 7
       \\units W/m-K
       \\type real
       \\default 0.0
  N15, \\field Temperature 8
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N16, \\field Thermal Conductivity 8
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 8
       \\units W/m-K
       \\type real
       \\default 0.0
  N17 , \\field Temperature 9
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N18,  \\field Thermal Conductivity 9
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 9
       \\units W/m-K
       \\default 0.0
  N19 , \\field Temperature 10
       \\note for Temperature-Thermal Conductivity function
       \\units C
       \\type real
       \\default 0.0
  N20;  \\field Thermal Conductivity 10
       \\note for Temperature-Thermal Conductivity function corresponding to temperature 10
       \\units W/m-K
       \\type real
       \\default 0.0

MaterialProperty:HeatAndMoistureTransfer:Settings,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Additional material properties for surfaces.
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1, \\field Material Name
      \\required-field
      \\type object-list
      \\object-list MaterialName
      \\note Material Name that the moisture properties will be added to.
      \\note This augments material properties needed for combined heat and moisture transfer for surfaces.
  N1, \\field Porosity
      \\required-field
      \\units m3/m3
      \\minimum 0.0
      \\maximum 1.0
      \\type real
  N2; \\field Initial Water Content Ratio
      \\note units are the water/material density ratio at the begining of each run period.
      \\required-field
      \\units kg/kg
      \\minimum 0.0
      \\default 0.2
      \\type real

MaterialProperty:HeatAndMoistureTransfer:SorptionIsotherm,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Relationship between moisture content and relative humidity fraction.
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Material Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note The Material Name that the moisture sorption isotherm will be added to.
  N1,  \\field Number of Isotherm Coordinates
       \\required-field
       \\note Number of data Coordinates
       \\minimum 1
       \\maximum 25
       \\type integer
  N2, \\field Relative Humidity Fraction 1
      \\required-field
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N3, \\field Moisture Content 1
      \\required-field
      \\minimum 0.0
      \\units kg/m3
  N4, \\field Relative Humidity Fraction 2
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N5, \\field Moisture Content 2
      \\minimum 0.0
      \\units kg/m3
  N6, \\field Relative Humidity Fraction 3
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N7, \\field Moisture Content 3
      \\minimum 0.0
      \\units kg/m3
  N8, \\field Relative Humidity Fraction 4
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N9, \\field Moisture Content 4
      \\units kg/m3
      \\minimum 0.0
  N10, \\field Relative Humidity Fraction 5
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N11, \\field Moisture Content 5
      \\minimum 0.0
      \\units kg/m3
  N12, \\field Relative Humidity Fraction 6
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N13, \\field Moisture Content 6
      \\minimum 0.0
      \\units kg/m3
  N14, \\field Relative Humidity Fraction 7
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N15, \\field Moisture Content 7
      \\minimum 0.0
      \\units kg/m3
  N16, \\field Relative Humidity Fraction 8
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N17, \\field Moisture Content 8
      \\minimum 0.0
      \\units kg/m3
  N18,\\field Relative Humidity Fraction 9
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N19, \\field Moisture Content 9
      \\minimum 0.0
      \\units kg/m3
  N20,\\field Relative Humidity Fraction 10
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N21, \\field Moisture Content 10
      \\minimum 0.0
      \\units kg/m3
  N22,\\field Relative Humidity Fraction 11
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N23, \\field Moisture Content 11
      \\minimum 0.0
      \\units kg/m3
  N24,\\field Relative Humidity Fraction 12
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N25, \\field Moisture Content 12
      \\minimum 0.0
      \\units kg/m3
  N26,\\field Relative Humidity Fraction 13
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N27, \\field Moisture Content 13
      \\minimum 0.0
      \\units kg/m3
  N28,\\field Relative Humidity Fraction 14
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N29,\\field Moisture Content 14
      \\minimum 0.0
      \\units kg/m3
  N30,\\field Relative Humidity Fraction 15
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N31,\\field Moisture Content 15
      \\minimum 0.0
      \\units kg/m3
  N32,\\field Relative Humidity Fraction 16
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N33,\\field Moisture Content 16
      \\minimum 0.0
      \\units kg/m3
  N34,\\field Relative Humidity Fraction 17
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N35,\\field Moisture Content 17
      \\minimum 0.0
      \\units kg/m3
  N36,\\field Relative Humidity Fraction 18
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N37,\\field Moisture Content 18
      \\minimum 0.0
      \\units kg/m3
  N38,\\field Relative Humidity Fraction 19
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N39,\\field Moisture Content 19
      \\minimum 0.0
      \\units kg/m3
  N40,\\field Relative Humidity Fraction 20
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N41,\\field Moisture Content 20
      \\minimum 0.0
      \\units kg/m3
  N42,\\field Relative Humidity Fraction 21
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N43,\\field Moisture Content 21
      \\minimum 0.0
      \\units kg/m3
  N44,\\field Relative Humidity Fraction 22
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N45,\\field Moisture Content 22
      \\minimum 0.0
      \\units kg/m3
  N46,\\field Relative Humidity Fraction 23
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N47,\\field Moisture Content 23
      \\minimum 0.0
      \\units kg/m3
  N48,\\field Relative Humidity Fraction 24
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N49,\\field Moisture Content 24
      \\minimum 0.0
      \\units kg/m3
  N50,\\field Relative Humidity Fraction 25
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N51;\\field Moisture Content 25
      \\minimum 0.0
      \\units kg/m3

MaterialProperty:HeatAndMoistureTransfer:Suction,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Relationship between liquid suction transport coefficient and moisture content
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Material Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Material Name that the moisture properties will be added to.
  N1,  \\field Number of Suction points
       \\required-field
       \\note Number of Suction Liquid Transport Coefficient coordinates
       \\minimum 1
       \\maximum 25
       \\type integer
  N2,\\field Moisture Content 1
     \\required-field
     \\minimum 0.0
     \\units kg/m3
  N3,\\field Liquid Transport Coefficient 1
     \\required-field
     \\minimum 0.0
     \\units m2/s
  N4,\\field Moisture Content 2
      \\minimum 0.0
     \\units kg/m3
  N5,\\field Liquid Transport Coefficient 2
      \\minimum 0.0
     \\units m2/s
  N6,\\field Moisture Content 3
      \\minimum 0.0
     \\units kg/m3
  N7,\\field Liquid Transport Coefficient 3
      \\minimum 0.0
     \\units m2/s
  N8,\\field Moisture Content 4
      \\minimum 0.0
     \\units kg/m3
  N9,\\field Liquid Transport Coefficient 4
      \\minimum 0.0
     \\units m2/s
  N10,\\field Moisture Content 5
      \\minimum 0.0
     \\units kg/m3
  N11,\\field Liquid Transport Coefficient 5
      \\minimum 0.0
     \\units m2/s
  N12,\\field Moisture Content 6
      \\minimum 0.0
     \\units kg/m3
  N13,\\field Liquid Transport Coefficient 6
      \\minimum 0.0
     \\units m2/s
  N14,\\field Moisture Content 7
      \\minimum 0.0
     \\units kg/m3
  N15,\\field Liquid Transport Coefficient 7
      \\minimum 0.0
     \\units m2/s
  N16,\\field Moisture Content 8
      \\minimum 0.0
     \\units kg/m3
  N17,\\field Liquid Transport Coefficient 8
      \\minimum 0.0
     \\units m2/s
  N18,\\field Moisture Content 9
      \\minimum 0.0
     \\units kg/m3
  N19,\\field Liquid Transport Coefficient 9
      \\minimum 0.0
     \\units m2/s
  N20,\\field Moisture Content 10
      \\minimum 0.0
     \\units kg/m3
  N21,\\field Liquid Transport Coefficient 10
      \\minimum 0.0
     \\units m2/s
  N22,\\field Moisture Content 11
      \\minimum 0.0
     \\units kg/m3
  N23,\\field Liquid Transport Coefficient 11
      \\minimum 0.0
     \\units m2/s
  N24,\\field Moisture Content 12
      \\minimum 0.0
     \\units kg/m3
  N25,\\field Liquid Transport Coefficient 12
      \\minimum 0.0
     \\units m2/s
  N26,\\field Moisture Content 13
      \\minimum 0.0
     \\units kg/m3
  N27,\\field Liquid Transport Coefficient 13
      \\minimum 0.0
     \\units m2/s
  N28,\\field Moisture Content 14
      \\minimum 0.0
     \\units kg/m3
  N29,\\field Liquid Transport Coefficient 14
      \\minimum 0.0
     \\units m2/s
  N30,\\field Moisture Content 15
      \\minimum 0.0
     \\units kg/m3
  N31,\\field Liquid Transport Coefficient 15
      \\minimum 0.0
     \\units m2/s
  N32,\\field Moisture Content 16
      \\minimum 0.0
     \\units kg/m3
  N33,\\field Liquid Transport Coefficient 16
      \\minimum 0.0
     \\units m2/s
  N34,\\field Moisture Content 17
      \\minimum 0.0
     \\units kg/m3
  N35,\\field Liquid Transport Coefficient 17
      \\minimum 0.0
     \\units m2/s
  N36,\\field Moisture Content 18
      \\minimum 0.0
     \\units kg/m3
  N37,\\field Liquid Transport Coefficient 18
      \\minimum 0.0
     \\units m2/s
  N38,\\field Moisture Content 19
      \\minimum 0.0
     \\units kg/m3
  N39,\\field Liquid Transport Coefficient 19
      \\minimum 0.0
     \\units m2/s
  N40,\\field Moisture Content 20
      \\minimum 0.0
     \\units kg/m3
  N41,\\field Liquid Transport Coefficient 20
      \\minimum 0.0
     \\units m2/s
  N42,\\field Moisture Content 21
      \\minimum 0.0
     \\units kg/m3
  N43,\\field Liquid Transport Coefficient 21
      \\minimum 0.0
     \\units m2/s
  N44,\\field Moisture Content 22
      \\minimum 0.0
     \\units kg/m3
  N45,\\field Liquid Transport Coefficient 22
      \\minimum 0.0
     \\units m2/s
  N46,\\field Moisture Content 23
      \\minimum 0.0
     \\units kg/m3
  N47,\\field Liquid Transport Coefficient 23
      \\minimum 0.0
     \\units m2/s
  N48,\\field Moisture Content 24
      \\minimum 0.0
     \\units kg/m3
  N49,\\field Liquid Transport Coefficient 24
      \\minimum 0.0
     \\units m2/s
  N50,\\field Moisture Content 25
      \\minimum 0.0
     \\units kg/m3
  N51;\\field Liquid Transport Coefficient 25
      \\minimum 0.0
     \\units m2/s

MaterialProperty:HeatAndMoistureTransfer:Redistribution,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Relationship between liquid transport coefficient and moisture content
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Material Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Moisture Material Name that the moisture properties will be added to.
  N1,  \\field Number of Redistribution points
       \\required-field
       \\note number of data points
       \\minimum 1
       \\maximum 25
       \\type integer
  N2,\\field Moisture Content 1
     \\minimum 0.0
     \\required-field
     \\units kg/m3
  N3,\\field Liquid Transport Coefficient 1
     \\minimum 0.0
     \\required-field
     \\units m2/s
  N4,\\field Moisture Content 2
      \\minimum 0.0
     \\units kg/m3
  N5,\\field Liquid Transport Coefficient 2
      \\minimum 0.0
     \\units m2/s
  N6,\\field Moisture Content 3
      \\minimum 0.0
     \\units kg/m3
  N7,\\field Liquid Transport Coefficient 3
      \\minimum 0.0
     \\units m2/s
  N8,\\field Moisture Content 4
      \\minimum 0.0
     \\units kg/m3
  N9,\\field Liquid Transport Coefficient 4
      \\minimum 0.0
     \\units m2/s
  N10,\\field Moisture Content 5
      \\minimum 0.0
     \\units kg/m3
  N11,\\field Liquid Transport Coefficient 5
      \\minimum 0.0
     \\units m2/s
  N12,\\field Moisture Content 6
      \\minimum 0.0
     \\units kg/m3
  N13,\\field Liquid Transport Coefficient 6
      \\minimum 0.0
     \\units m2/s
  N14,\\field Moisture Content 7
      \\minimum 0.0
     \\units kg/m3
  N15,\\field Liquid Transport Coefficient 7
      \\minimum 0.0
     \\units m2/s
  N16,\\field Moisture Content 8
      \\minimum 0.0
     \\units kg/m3
  N17,\\field Liquid Transport Coefficient 8
      \\minimum 0.0
     \\units m2/s
  N18,\\field Moisture Content 9
      \\minimum 0.0
     \\units kg/m3
  N19,\\field Liquid Transport Coefficient 9
      \\minimum 0.0
     \\units m2/s
  N20,\\field Moisture Content 10
      \\minimum 0.0
     \\units kg/m3
  N21,\\field Liquid Transport Coefficient 10
      \\minimum 0.0
     \\units m2/s
  N22,\\field Moisture Content 11
      \\minimum 0.0
     \\units kg/m3
  N23,\\field Liquid Transport Coefficient 11
      \\minimum 0.0
     \\units m2/s
  N24,\\field Moisture Content 12
      \\minimum 0.0
     \\units kg/m3
  N25,\\field Liquid Transport Coefficient 12
      \\minimum 0.0
     \\units m2/s
  N26,\\field Moisture Content 13
      \\minimum 0.0
     \\units kg/m3
  N27,\\field Liquid Transport Coefficient 13
      \\minimum 0.0
     \\units m2/s
  N28,\\field Moisture Content 14
      \\minimum 0.0
     \\units kg/m3
  N29,\\field Liquid Transport Coefficient 14
      \\minimum 0.0
     \\units m2/s
  N30,\\field Moisture Content 15
      \\minimum 0.0
     \\units kg/m3
  N31,\\field Liquid Transport Coefficient 15
      \\minimum 0.0
     \\units m2/s
  N32,\\field Moisture Content 16
      \\minimum 0.0
     \\units kg/m3
  N33,\\field Liquid Transport Coefficient 16
      \\minimum 0.0
     \\units m2/s
  N34,\\field Moisture Content 17
      \\minimum 0.0
     \\units kg/m3
  N35,\\field Liquid Transport Coefficient 17
      \\minimum 0.0
     \\units m2/s
  N36,\\field Moisture Content 18
      \\minimum 0.0
     \\units kg/m3
  N37,\\field Liquid Transport Coefficient 18
      \\minimum 0.0
     \\units m2/s
  N38,\\field Moisture Content 19
      \\minimum 0.0
     \\units kg/m3
  N39,\\field Liquid Transport Coefficient 19
      \\minimum 0.0
     \\units m2/s
  N40,\\field Moisture Content 20
      \\minimum 0.0
     \\units kg/m3
  N41,\\field Liquid Transport Coefficient 20
      \\minimum 0.0
     \\units m2/s
  N42,\\field Moisture Content 21
      \\minimum 0.0
     \\units kg/m3
  N43,\\field Liquid Transport Coefficient 21
      \\minimum 0.0
     \\units m2/s
  N44,\\field Moisture Content 22
      \\minimum 0.0
     \\units kg/m3
  N45,\\field Liquid Transport Coefficient 22
      \\minimum 0.0
     \\units m2/s
  N46,\\field Moisture Content 23
      \\minimum 0.0
     \\units kg/m3
  N47,\\field Liquid Transport Coefficient 23
      \\minimum 0.0
     \\units m2/s
  N48,\\field Moisture Content 24
      \\minimum 0.0
     \\units kg/m3
  N49,\\field Liquid Transport Coefficient 24
      \\minimum 0.0
     \\units m2/s
  N50,\\field Moisture Content 25
      \\minimum 0.0
     \\units kg/m3
  N51;\\field Liquid Transport Coefficient 25
      \\minimum 0.0
     \\units m2/s

MaterialProperty:HeatAndMoistureTransfer:Diffusion,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Relationship between water vapor diffusion and relative humidity fraction
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Material Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Moisture Material Name that the moisture properties will be added to.
  N1,  \\field Number of Data Pairs
       \\required-field
       \\note Water Vapor Diffusion Resistance Factor
       \\minimum 1
       \\maximum 25
       \\type integer
  N2, \\field Relative Humidity Fraction 1
      \\required-field
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N3,\\field Water Vapor Diffusion Resistance Factor 1
      \\required-field
      \\minimum 0.0
      \\units dimensionless
  N4, \\field Relative Humidity Fraction 2
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N5,\\field Water Vapor Diffusion Resistance Factor 2
      \\minimum 0.0
      \\units dimensionless
  N6, \\field Relative Humidity Fraction 3
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N7,\\field Water Vapor Diffusion Resistance Factor 3
      \\minimum 0.0
      \\units dimensionless
  N8, \\field Relative Humidity Fraction 4
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N9,\\field Water Vapor Diffusion Resistance Factor 4
      \\minimum 0.0
      \\units dimensionless
  N10, \\field Relative Humidity Fraction 5
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N11,\\field Water Vapor Diffusion Resistance Factor 5
      \\minimum 0.0
      \\units dimensionless
  N12,\\field Relative Humidity Fraction 6
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N13,\\field Water Vapor Diffusion Resistance Factor 6
      \\minimum 0.0
      \\units dimensionless
  N14,\\field Relative Humidity Fraction 7
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N15,\\field Water Vapor Diffusion Resistance Factor 7
      \\minimum 0.0
      \\units dimensionless
  N16,\\field Relative Humidity Fraction 8
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N17,\\field Water Vapor Diffusion Resistance Factor 8
      \\minimum 0.0
      \\units dimensionless
  N18,\\field Relative Humidity Fraction 9
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N19,\\field Water Vapor Diffusion Resistance Factor 9
      \\minimum 0.0
      \\units dimensionless
  N20,\\field Relative Humidity Fraction 10
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N21,\\field Water Vapor Diffusion Resistance Factor 10
      \\minimum 0.0
      \\units dimensionless
  N22,\\field Relative Humidity Fraction 11
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N23,\\field Water Vapor Diffusion Resistance Factor 11
      \\minimum 0.0
      \\units dimensionless
 N24,\\field Relative Humidity Fraction 12
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N25,\\field Water Vapor Diffusion Resistance Factor 12
      \\minimum 0.0
      \\units dimensionless
  N26,\\field Relative Humidity Fraction 13
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N27,\\field Water Vapor Diffusion Resistance Factor 13
      \\minimum 0.0
      \\units dimensionless
  N28,\\field Relative Humidity Fraction 14
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N29,\\field Water Vapor Diffusion Resistance Factor 14
      \\minimum 0.0
      \\units dimensionless
  N30,\\field Relative Humidity Fraction 15
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N31,\\field Water Vapor Diffusion Resistance Factor 15
      \\minimum 0.0
      \\units dimensionless
  N32,\\field Relative Humidity Fraction 16
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N33,\\field Water Vapor Diffusion Resistance Factor 16
      \\minimum 0.0
      \\units dimensionless
  N34,\\field Relative Humidity Fraction 17
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N35,\\field Water Vapor Diffusion Resistance Factor 17
      \\minimum 0.0
      \\units dimensionless
  N36,\\field Relative Humidity Fraction 18
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N37,\\field Water Vapor Diffusion Resistance Factor 18
      \\minimum 0.0
      \\units dimensionless
  N38,\\field Relative Humidity Fraction 19
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N39,\\field Water Vapor Diffusion Resistance Factor 19
      \\minimum 0.0
      \\units dimensionless
  N40,\\field Relative Humidity Fraction 20
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N41,\\field Water Vapor Diffusion Resistance Factor 20
      \\minimum 0.0
      \\units dimensionless
  N42,\\field Relative Humidity Fraction 21
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N43,\\field Water Vapor Diffusion Resistance Factor 21
      \\minimum 0.0
      \\units dimensionless
  N44,\\field Relative Humidity Fraction 22
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N45,\\field Water Vapor Diffusion Resistance Factor 22
      \\minimum 0.0
      \\units dimensionless
  N46,\\field Relative Humidity Fraction 23
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N47,\\field Water Vapor Diffusion Resistance Factor 23
      \\minimum 0.0
      \\units dimensionless
  N48,\\field Relative Humidity Fraction 24
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N49,\\field Water Vapor Diffusion Resistance Factor 24
      \\minimum 0.0
      \\units dimensionless
  N50,\\field Relative Humidity Fraction 25
      \\note The relative humidity is entered as a fraction.
      \\minimum 0.0
      \\maximum 1.0
      \\units dimensionless
  N51;\\field Water Vapor Diffusion Resistance Factor 25
      \\minimum 0.0
      \\units dimensionless

MaterialProperty:HeatAndMoistureTransfer:ThermalConductivity,
    \\memo HeatBalanceAlgorithm = CombinedHeatAndMoistureFiniteElement solution algorithm only.
    \\memo Relationship between thermal conductivity and moisture content
    \\memo Has no effect with other HeatBalanceAlgorithm solution algorithms
  A1 , \\field Material Name
       \\required-field
       \\type object-list
       \\object-list MaterialName
       \\note Moisture Material Name that the Thermal Conductivity will be added to.
  N1,  \\field Number of Thermal Coordinates
       \\required-field
       \\note number of data coordinates
       \\minimum 1
       \\maximum 25
       \\type integer
  N2, \\field Moisture Content 1
      \\required-field
      \\minimum 0.0
      \\units kg/m3
  N3, \\field Thermal Conductivity 1
      \\required-field
      \\minimum> 0.0
      \\units W/m-K
  N4, \\field Moisture Content 2
      \\minimum 0.0
      \\units kg/m3
  N5, \\field Thermal Conductivity 2
      \\minimum> 0.0
      \\units W/m-K
  N6, \\field Moisture Content 3
      \\minimum 0.0
      \\units kg/m3
  N7, \\field Thermal Conductivity 3
      \\minimum> 0.0
      \\units W/m-K
  N8, \\field Moisture Content 4
      \\minimum 0.0
      \\units kg/m3
  N9, \\field Thermal Conductivity 4
      \\minimum> 0.0
      \\units W/m-K
  N10, \\field Moisture Content 5
      \\minimum 0.0
      \\units kg/m3
  N11,\\field Thermal Conductivity 5
      \\minimum> 0.0
      \\units W/m-K
  N12, \\field Moisture Content 6
      \\minimum 0.0
      \\units kg/m3
  N13,\\field Thermal Conductivity 6
      \\minimum> 0.0
      \\units W/m-K
  N14,\\field Moisture Content 7
      \\minimum 0.0
      \\units kg/m3
  N15,\\field Thermal Conductivity 7
      \\minimum> 0.0
      \\units W/m-K
  N16,\\field Moisture Content 8
      \\minimum 0.0
      \\units kg/m3
  N17,\\field Thermal Conductivity 8
      \\minimum> 0.0
      \\units W/m-K
  N18,\\field Moisture Content 9
      \\minimum 0.0
      \\units kg/m3
  N19,\\field Thermal Conductivity 9
      \\minimum> 0.0
      \\units W/m-K
  N20,\\field Moisture Content 10
      \\minimum 0.0
      \\units kg/m3
  N21,\\field Thermal Conductivity 10
      \\minimum> 0.0
      \\units W/m-K
  N22,\\field Moisture Content 11
      \\minimum 0.0
      \\units kg/m3
  N23,\\field Thermal Conductivity 11
      \\minimum> 0.0
      \\units W/m-K
  N24,\\field Moisture Content 12
      \\minimum 0.0
      \\units kg/m3
  N25,\\field Thermal Conductivity 12
      \\minimum> 0.0
      \\units W/m-K
  N26,\\field Moisture Content 13
      \\minimum 0.0
      \\units kg/m3
  N27,\\field Thermal Conductivity 13
      \\minimum> 0.0
      \\units W/m-K
  N28,\\field Moisture Content 14
      \\minimum 0.0
      \\units kg/m3
  N29,\\field Thermal Conductivity 14
      \\minimum> 0.0
      \\units W/m-K
  N30,\\field Moisture Content 15
      \\minimum 0.0
      \\units kg/m3
  N31,\\field Thermal Conductivity 15
      \\minimum> 0.0
      \\units W/m-K
  N32,\\field Moisture Content 16
      \\minimum 0.0
      \\units kg/m3
  N33,\\field Thermal Conductivity 16
      \\minimum> 0.0
      \\units W/m-K
  N34,\\field Moisture Content 17
      \\minimum 0.0
      \\units kg/m3
  N35,\\field Thermal Conductivity 17
      \\minimum> 0.0
      \\units W/m-K
  N36,\\field Moisture Content 18
      \\minimum 0.0
      \\units kg/m3
  N37,\\field Thermal Conductivity 18
      \\minimum> 0.0
      \\units W/m-K
  N38,\\field Moisture Content 19
      \\minimum 0.0
      \\units kg/m3
  N39,\\field Thermal Conductivity 19
      \\minimum> 0.0
      \\units W/m-K
  N40,\\field Moisture Content 20
      \\minimum 0.0
      \\units kg/m3
  N41,\\field Thermal Conductivity 20
      \\minimum> 0.0
      \\units W/m-K
  N42,\\field Moisture Content 21
      \\minimum 0.0
      \\units kg/m3
  N43,\\field Thermal Conductivity 21
      \\minimum> 0.0
      \\units W/m-K
  N44,\\field Moisture Content 22
      \\minimum 0.0
      \\units kg/m3
  N45,\\field Thermal Conductivity 22
      \\minimum> 0.0
      \\units W/m-K
  N46,\\field Moisture Content 23
      \\minimum 0.0
      \\units kg/m3
  N47,\\field Thermal Conductivity 23
      \\minimum> 0.0
      \\units W/m-K
  N48,\\field Moisture Content 24
      \\minimum 0.0
      \\units kg/m3
  N49,\\field Thermal Conductivity 24
      \\minimum> 0.0
      \\units W/m-K
  N50,\\field Moisture Content 25
      \\minimum 0.0
      \\units kg/m3
  N51;\\field Thermal Conductivity 25
      \\minimum> 0.0
      \\units W/m-K

MaterialProperty:GlazingSpectralData,
       \\memo Name is followed by up to 800 sets of normal-incidence measured values of
       \\memo [wavelength, transmittance, front reflectance, back reflectance] for wavelengths
       \\memo covering the solar spectrum (from about 0.25 to 2.5 microns)
       \\format Spectral
  A1,  \\field Name
       \\required-field
       \\reference SpectralDataSets
  N1,N2,N3,N4,         N5,N6,N7,N8,         N9,N10,N11,N12,      N13,N14,N15,N16,     N17,N18,N19,N20, \\note fields as indicated
  N21,N22,N23,N24,     N25,N26,N27,N28,     N29,N30,N31,N32,     N33,N34,N35,N36,     N37,N38,N39,N40, \\note fields as indicated
  N41,N42,N43,N44,     N45,N46,N47,N48,     N49,N50,N51,N52,     N53,N54,N55,N56,     N57,N58,N59,N60, \\note fields as indicated
  N61,N62,N63,N64,     N65,N66,N67,N68,     N69,N70,N71,N72,     N73,N74,N75,N76,     N77,N78,N79,N80, \\note fields as indicated
  N81,N82,N83,N84,     N85,N86,N87,N88,     N89,N90,N91,N92,     N93,N94,N95,N96,     N97,N98,N99,N100, \\note fields as indicated

  N101,N102,N103,N104, N105,N106,N107,N108, N109,N110,N111,N112, N113,N114,N115,N116, N117,N118,N119,N120, \\note fields as indicated
  N121,N122,N123,N124, N125,N126,N127,N128, N129,N130,N131,N132, N133,N134,N135,N136, N137,N138,N139,N140, \\note fields as indicated
  N141,N142,N143,N144, N145,N146,N147,N148, N149,N150,N151,N152, N153,N154,N155,N156, N157,N158,N159,N160, \\note fields as indicated
  N161,N162,N163,N164, N165,N166,N167,N168, N169,N170,N171,N172, N173,N174,N175,N176, N177,N178,N179,N180, \\note fields as indicated
  N181,N182,N183,N184, N185,N186,N187,N188, N189,N190,N191,N192, N193,N194,N195,N196, N197,N198,N199,N200, \\note fields as indicated

  N201,N202,N203,N204, N205,N206,N207,N208, N209,N210,N211,N212, N213,N214,N215,N216, N217,N218,N219,N220, \\note fields as indicated
  N221,N222,N223,N224, N225,N226,N227,N228, N229,N230,N231,N232, N233,N234,N235,N236, N237,N238,N239,N240, \\note fields as indicated
  N241,N242,N243,N244, N245,N246,N247,N248, N249,N250,N251,N252, N253,N254,N255,N256, N257,N258,N259,N260, \\note fields as indicated
  N261,N262,N263,N264, N265,N266,N267,N268, N269,N270,N271,N272, N273,N274,N275,N276, N277,N278,N279,N280, \\note fields as indicated
  N281,N282,N283,N284, N285,N286,N287,N288, N289,N290,N291,N292, N293,N294,N295,N296, N297,N298,N299,N300, \\note fields as indicated

  N301,N302,N303,N304, N305,N306,N307,N308, N309,N310,N311,N312, N313,N314,N315,N316, N317,N318,N319,N320, \\note fields as indicated
  N321,N322,N323,N324, N325,N326,N327,N328, N329,N330,N331,N332, N333,N334,N335,N336, N337,N338,N339,N340, \\note fields as indicated
  N341,N342,N343,N344, N345,N346,N347,N348, N349,N350,N351,N352, N353,N354,N355,N356, N357,N358,N359,N360, \\note fields as indicated
  N361,N362,N363,N364, N365,N366,N367,N368, N369,N370,N371,N372, N373,N374,N375,N376, N377,N378,N379,N380, \\note fields as indicated
  N381,N382,N383,N384, N385,N386,N387,N388, N389,N390,N391,N392, N393,N394,N395,N396, N397,N398,N399,N400, \\note fields as indicated

  N401,N402,N403,N404, N405,N406,N407,N408, N409,N410,N411,N412, N413,N414,N415,N416, N417,N418,N419,N420, \\note fields as indicated
  N421,N422,N423,N424, N425,N426,N427,N428, N429,N430,N431,N432, N433,N434,N435,N436, N437,N438,N439,N440, \\note fields as indicated
  N441,N442,N443,N444, N445,N446,N447,N448, N449,N450,N451,N452, N453,N454,N455,N456, N457,N458,N459,N460, \\note fields as indicated
  N461,N462,N463,N464, N465,N466,N467,N468, N469,N470,N471,N472, N473,N474,N475,N476, N477,N478,N479,N480, \\note fields as indicated
  N481,N482,N483,N484, N485,N486,N487,N488, N489,N490,N491,N492, N493,N494,N495,N496, N497,N498,N499,N500, \\note fields as indicated

  N501,N502,N503,N504, N505,N506,N507,N508, N509,N510,N511,N512, N513,N514,N515,N516, N517,N518,N519,N520, \\note fields as indicated
  N521,N522,N523,N524, N525,N526,N527,N528, N529,N530,N531,N532, N533,N534,N535,N536, N537,N538,N539,N540, \\note fields as indicated
  N541,N542,N543,N544, N545,N546,N547,N548, N549,N550,N551,N552, N553,N554,N555,N556, N557,N558,N559,N560, \\note fields as indicated
  N561,N562,N563,N564, N565,N566,N567,N568, N569,N570,N571,N572, N573,N574,N575,N576, N577,N578,N579,N580, \\note fields as indicated
  N581,N582,N583,N584, N585,N586,N587,N588, N589,N590,N591,N592, N593,N594,N595,N596, N597,N598,N599,N600, \\note fields as indicated

  N601,N602,N603,N604, N605,N606,N607,N608, N609,N610,N611,N612, N613,N614,N615,N616, N617,N618,N619,N620, \\note fields as indicated
  N621,N622,N623,N624, N625,N626,N627,N628, N629,N630,N631,N632, N633,N634,N635,N636, N637,N638,N639,N640, \\note fields as indicated
  N641,N642,N643,N644, N645,N646,N647,N648, N649,N650,N651,N652, N653,N654,N655,N656, N657,N658,N659,N660, \\note fields as indicated
  N661,N662,N663,N664, N665,N666,N667,N668, N669,N670,N671,N672, N673,N674,N675,N676, N677,N678,N679,N680, \\note fields as indicated
  N681,N682,N683,N684, N685,N686,N687,N688, N689,N690,N691,N692, N693,N694,N695,N696, N697,N698,N699,N700, \\note fields as indicated

  N701,N702,N703,N704, N705,N706,N707,N708, N709,N710,N711,N712, N713,N714,N715,N716, N717,N718,N719,N720, \\note fields as indicated
  N721,N722,N723,N724, N725,N726,N727,N728, N729,N730,N731,N732, N733,N734,N735,N736, N737,N738,N739,N740, \\note fields as indicated
  N741,N742,N743,N744, N745,N746,N747,N748, N749,N750,N751,N752, N753,N754,N755,N756, N757,N758,N759,N760, \\note fields as indicated
  N761,N762,N763,N764, N765,N766,N767,N768, N769,N770,N771,N772, N773,N774,N775,N776, N777,N778,N779,N780, \\note fields as indicated
  N781,N782,N783,N784, N785,N786,N787,N788, N789,N790,N791,N792, N793,N794,N795,N796, N797,N798,N799,N800, \\note fields as indicated

  N801,N802,N803,N804, N805,N806,N807,N808, N809,N810,N811,N812, N813,N814,N815,N816, N817,N818,N819,N820, \\note fields as indicated
  N821,N822,N823,N824, N825,N826,N827,N828, N829,N830,N831,N832, N833,N834,N835,N836, N837,N838,N839,N840, \\note fields as indicated
  N841,N842,N843,N844, N845,N846,N847,N848, N849,N850,N851,N852, N853,N854,N855,N856, N857,N858,N859,N860, \\note fields as indicated
  N861,N862,N863,N864, N865,N866,N867,N868, N869,N870,N871,N872, N873,N874,N875,N876, N877,N878,N879,N880, \\note fields as indicated
  N881,N882,N883,N884, N885,N886,N887,N888, N889,N890,N891,N892, N893,N894,N895,N896, N897,N898,N899,N900, \\note fields as indicated

  N901,N902,N903,N904, N905,N906,N907,N908, N909,N910,N911,N912, N913,N914,N915,N916, N917,N918,N919,N920, \\note fields as indicated
  N921,N922,N923,N924, N925,N926,N927,N928, N929,N930,N931,N932, N933,N934,N935,N936, N937,N938,N939,N940, \\note fields as indicated
  N941,N942,N943,N944, N945,N946,N947,N948, N949,N950,N951,N952, N953,N954,N955,N956, N957,N958,N959,N960, \\note fields as indicated
  N961,N962,N963,N964, N965,N966,N967,N968, N969,N970,N971,N972, N973,N974,N975,N976, N977,N978,N979,N980, \\note fields as indicated
  N981,N982,N983,N984, N985,N986,N987,N988, N989,N990,N991,N992, N993,N994,N995,N996, N997,N998,N999,N1000, \\note fields as indicated

  N1001,N1002,N1003,N1004, N1005,N1006,N1007,N1008, N1009,N1010,N1011,N1012, N1013,N1014,N1015,N1016, N1017,N1018,N1019,N1020, \\note fields as indicated
  N1021,N1022,N1023,N1024, N1025,N1026,N1027,N1028, N1029,N1030,N1031,N1032, N1033,N1034,N1035,N1036, N1037,N1038,N1039,N1040, \\note fields as indicated
  N1041,N1042,N1043,N1044, N1045,N1046,N1047,N1048, N1049,N1050,N1051,N1052, N1053,N1054,N1055,N1056, N1057,N1058,N1059,N1060, \\note fields as indicated
  N1061,N1062,N1063,N1064, N1065,N1066,N1067,N1068, N1069,N1070,N1071,N1072, N1073,N1074,N1075,N1076, N1077,N1078,N1079,N1080, \\note fields as indicated
  N1081,N1082,N1083,N1084, N1085,N1086,N1087,N1088, N1089,N1090,N1091,N1092, N1093,N1094,N1095,N1096, N1097,N1098,N1099,N1100, \\note fields as indicated

  N1101,N1102,N1103,N1104, N1105,N1106,N1107,N1108, N1109,N1110,N1111,N1112, N1113,N1114,N1115,N1116, N1117,N1118,N1119,N1120, \\note fields as indicated
  N1121,N1122,N1123,N1124, N1125,N1126,N1127,N1128, N1129,N1130,N1131,N1132, N1133,N1134,N1135,N1136, N1137,N1138,N1139,N1140, \\note fields as indicated
  N1141,N1142,N1143,N1144, N1145,N1146,N1147,N1148, N1149,N1150,N1151,N1152, N1153,N1154,N1155,N1156, N1157,N1158,N1159,N1160, \\note fields as indicated
  N1161,N1162,N1163,N1164, N1165,N1166,N1167,N1168, N1169,N1170,N1171,N1172, N1173,N1174,N1175,N1176, N1177,N1178,N1179,N1180, \\note fields as indicated
  N1181,N1182,N1183,N1184, N1185,N1186,N1187,N1188, N1189,N1190,N1191,N1192, N1193,N1194,N1195,N1196, N1197,N1198,N1199,N1200, \\note fields as indicated

  N1201,N1202,N1203,N1204, N1205,N1206,N1207,N1208, N1209,N1210,N1211,N1212, N1213,N1214,N1215,N1216, N1217,N1218,N1219,N1220, \\note fields as indicated
  N1221,N1222,N1223,N1224, N1225,N1226,N1227,N1228, N1229,N1230,N1231,N1232, N1233,N1234,N1235,N1236, N1237,N1238,N1239,N1240, \\note fields as indicated
  N1241,N1242,N1243,N1244, N1245,N1246,N1247,N1248, N1249,N1250,N1251,N1252, N1253,N1254,N1255,N1256, N1257,N1258,N1259,N1260, \\note fields as indicated
  N1261,N1262,N1263,N1264, N1265,N1266,N1267,N1268, N1269,N1270,N1271,N1272, N1273,N1274,N1275,N1276, N1277,N1278,N1279,N1280, \\note fields as indicated
  N1281,N1282,N1283,N1284, N1285,N1286,N1287,N1288, N1289,N1290,N1291,N1292, N1293,N1294,N1295,N1296, N1297,N1298,N1299,N1300, \\note fields as indicated

  N1301,N1302,N1303,N1304, N1305,N1306,N1307,N1308, N1309,N1310,N1311,N1312, N1313,N1314,N1315,N1316, N1317,N1318,N1319,N1320, \\note fields as indicated
  N1321,N1322,N1323,N1324, N1325,N1326,N1327,N1328, N1329,N1330,N1331,N1332, N1333,N1334,N1335,N1336, N1337,N1338,N1339,N1340, \\note fields as indicated
  N1341,N1342,N1343,N1344, N1345,N1346,N1347,N1348, N1349,N1350,N1351,N1352, N1353,N1354,N1355,N1356, N1357,N1358,N1359,N1360, \\note fields as indicated
  N1361,N1362,N1363,N1364, N1365,N1366,N1367,N1368, N1369,N1370,N1371,N1372, N1373,N1374,N1375,N1376, N1377,N1378,N1379,N1380, \\note fields as indicated
  N1381,N1382,N1383,N1384, N1385,N1386,N1387,N1388, N1389,N1390,N1391,N1392, N1393,N1394,N1395,N1396, N1397,N1398,N1399,N1400, \\note fields as indicated

  N1401,N1402,N1403,N1404, N1405,N1406,N1407,N1408, N1409,N1410,N1411,N1412, N1413,N1414,N1415,N1416, N1417,N1418,N1419,N1420, \\note fields as indicated
  N1421,N1422,N1423,N1424, N1425,N1426,N1427,N1428, N1429,N1430,N1431,N1432, N1433,N1434,N1435,N1436, N1437,N1438,N1439,N1440, \\note fields as indicated
  N1441,N1442,N1443,N1444, N1445,N1446,N1447,N1448, N1449,N1450,N1451,N1452, N1453,N1454,N1455,N1456, N1457,N1458,N1459,N1460, \\note fields as indicated
  N1461,N1462,N1463,N1464, N1465,N1466,N1467,N1468, N1469,N1470,N1471,N1472, N1473,N1474,N1475,N1476, N1477,N1478,N1479,N1480, \\note fields as indicated
  N1481,N1482,N1483,N1484, N1485,N1486,N1487,N1488, N1489,N1490,N1491,N1492, N1493,N1494,N1495,N1496, N1497,N1498,N1499,N1500, \\note fields as indicated

  N1501,N1502,N1503,N1504, N1505,N1506,N1507,N1508, N1509,N1510,N1511,N1512, N1513,N1514,N1515,N1516, N1517,N1518,N1519,N1520, \\note fields as indicated
  N1521,N1522,N1523,N1524, N1525,N1526,N1527,N1528, N1529,N1530,N1531,N1532, N1533,N1534,N1535,N1536, N1537,N1538,N1539,N1540, \\note fields as indicated
  N1541,N1542,N1543,N1544, N1545,N1546,N1547,N1548, N1549,N1550,N1551,N1552, N1553,N1554,N1555,N1556, N1557,N1558,N1559,N1560, \\note fields as indicated
  N1561,N1562,N1563,N1564, N1565,N1566,N1567,N1568, N1569,N1570,N1571,N1572, N1573,N1574,N1575,N1576, N1577,N1578,N1579,N1580, \\note fields as indicated
  N1581,N1582,N1583,N1584, N1585,N1586,N1587,N1588, N1589,N1590,N1591,N1592, N1593,N1594,N1595,N1596, N1597,N1598,N1599,N1600, \\note fields as indicated

  N1601,N1602,N1603,N1604, N1605,N1606,N1607,N1608, N1609,N1610,N1611,N1612, N1613,N1614,N1615,N1616, N1617,N1618,N1619,N1620, \\note fields as indicated
  N1621,N1622,N1623,N1624, N1625,N1626,N1627,N1628, N1629,N1630,N1631,N1632, N1633,N1634,N1635,N1636, N1637,N1638,N1639,N1640, \\note fields as indicated
  N1641,N1642,N1643,N1644, N1645,N1646,N1647,N1648, N1649,N1650,N1651,N1652, N1653,N1654,N1655,N1656, N1657,N1658,N1659,N1660, \\note fields as indicated
  N1661,N1662,N1663,N1664, N1665,N1666,N1667,N1668, N1669,N1670,N1671,N1672, N1673,N1674,N1675,N1676, N1677,N1678,N1679,N1680, \\note fields as indicated
  N1681,N1682,N1683,N1684, N1685,N1686,N1687,N1688, N1689,N1690,N1691,N1692, N1693,N1694,N1695,N1696, N1697,N1698,N1699,N1700, \\note fields as indicated

  N1701,N1702,N1703,N1704, N1705,N1706,N1707,N1708, N1709,N1710,N1711,N1712, N1713,N1714,N1715,N1716, N1717,N1718,N1719,N1720, \\note fields as indicated
  N1721,N1722,N1723,N1724, N1725,N1726,N1727,N1728, N1729,N1730,N1731,N1732, N1733,N1734,N1735,N1736, N1737,N1738,N1739,N1740, \\note fields as indicated
  N1741,N1742,N1743,N1744, N1745,N1746,N1747,N1748, N1749,N1750,N1751,N1752, N1753,N1754,N1755,N1756, N1757,N1758,N1759,N1760, \\note fields as indicated
  N1761,N1762,N1763,N1764, N1765,N1766,N1767,N1768, N1769,N1770,N1771,N1772, N1773,N1774,N1775,N1776, N1777,N1778,N1779,N1780, \\note fields as indicated
  N1781,N1782,N1783,N1784, N1785,N1786,N1787,N1788, N1789,N1790,N1791,N1792, N1793,N1794,N1795,N1796, N1797,N1798,N1799,N1800, \\note fields as indicated

  N1801,N1802,N1803,N1804, N1805,N1806,N1807,N1808, N1809,N1810,N1811,N1812, N1813,N1814,N1815,N1816, N1817,N1818,N1819,N1820,  \\note fields as indicated
  N1821,N1822,N1823,N1824, N1825,N1826,N1827,N1828, N1829,N1830,N1831,N1832, N1833,N1834,N1835,N1836, N1837,N1838,N1839,N1840,  \\note fields as indicated
  N1841,N1842,N1843,N1844, N1845,N1846,N1847,N1848, N1849,N1850,N1851,N1852, N1853,N1854,N1855,N1856, N1857,N1858,N1859,N1860,  \\note fields as indicated
  N1861,N1862,N1863,N1864, N1865,N1866,N1867,N1868, N1869,N1870,N1871,N1872, N1873,N1874,N1875,N1876, N1877,N1878,N1879,N1880,  \\note fields as indicated
  N1881,N1882,N1883,N1884, N1885,N1886,N1887,N1888, N1889,N1890,N1891,N1892, N1893,N1894,N1895,N1896, N1897,N1898,N1899,N1900,  \\note fields as indicated

  N1901,N1902,N1903,N1904, N1905,N1906,N1907,N1908, N1909,N1910,N1911,N1912, N1913,N1914,N1915,N1916, N1917,N1918,N1919,N1920,  \\note fields as indicated
  N1921,N1922,N1923,N1924, N1925,N1926,N1927,N1928, N1929,N1930,N1931,N1932, N1933,N1934,N1935,N1936, N1937,N1938,N1939,N1940,  \\note fields as indicated
  N1941,N1942,N1943,N1944, N1945,N1946,N1947,N1948, N1949,N1950,N1951,N1952, N1953,N1954,N1955,N1956, N1957,N1958,N1959,N1960,  \\note fields as indicated
  N1961,N1962,N1963,N1964, N1965,N1966,N1967,N1968, N1969,N1970,N1971,N1972, N1973,N1974,N1975,N1976, N1977,N1978,N1979,N1980,  \\note fields as indicated
  N1981,N1982,N1983,N1984, N1985,N1986,N1987,N1988, N1989,N1990,N1991,N1992, N1993,N1994,N1995,N1996, N1997,N1998,N1999,N2000,  \\note fields as indicated

  N2001,N2002,N2003,N2004, N2005,N2006,N2007,N2008, N2009,N2010,N2011,N2012, N2013,N2014,N2015,N2016, N2017,N2018,N2019,N2020,  \\note fields as indicated
  N2021,N2022,N2023,N2024, N2025,N2026,N2027,N2028, N2029,N2030,N2031,N2032, N2033,N2034,N2035,N2036, N2037,N2038,N2039,N2040,  \\note fields as indicated
  N2041,N2042,N2043,N2044, N2045,N2046,N2047,N2048, N2049,N2050,N2051,N2052, N2053,N2054,N2055,N2056, N2057,N2058,N2059,N2060,  \\note fields as indicated
  N2061,N2062,N2063,N2064, N2065,N2066,N2067,N2068, N2069,N2070,N2071,N2072, N2073,N2074,N2075,N2076, N2077,N2078,N2079,N2080,  \\note fields as indicated
  N2081,N2082,N2083,N2084, N2085,N2086,N2087,N2088, N2089,N2090,N2091,N2092, N2093,N2094,N2095,N2096, N2097,N2098,N2099,N2100,  \\note fields as indicated

  N2101,N2102,N2103,N2104, N2105,N2106,N2107,N2108, N2109,N2110,N2111,N2112, N2113,N2114,N2115,N2116, N2117,N2118,N2119,N2120,  \\note fields as indicated
  N2121,N2122,N2123,N2124, N2125,N2126,N2127,N2128, N2129,N2130,N2131,N2132, N2133,N2134,N2135,N2136, N2137,N2138,N2139,N2140,  \\note fields as indicated
  N2141,N2142,N2143,N2144, N2145,N2146,N2147,N2148, N2149,N2150,N2151,N2152, N2153,N2154,N2155,N2156, N2157,N2158,N2159,N2160,  \\note fields as indicated
  N2161,N2162,N2163,N2164, N2165,N2166,N2167,N2168, N2169,N2170,N2171,N2172, N2173,N2174,N2175,N2176, N2177,N2178,N2179,N2180,  \\note fields as indicated
  N2181,N2182,N2183,N2184, N2185,N2186,N2187,N2188, N2189,N2190,N2191,N2192, N2193,N2194,N2195,N2196, N2197,N2198,N2199,N2200,  \\note fields as indicated

  N2201,N2202,N2203,N2204, N2205,N2206,N2207,N2208, N2209,N2210,N2211,N2212, N2213,N2214,N2215,N2216, N2217,N2218,N2219,N2220,  \\note fields as indicated
  N2221,N2222,N2223,N2224, N2225,N2226,N2227,N2228, N2229,N2230,N2231,N2232, N2233,N2234,N2235,N2236, N2237,N2238,N2239,N2240,  \\note fields as indicated
  N2241,N2242,N2243,N2244, N2245,N2246,N2247,N2248, N2249,N2250,N2251,N2252, N2253,N2254,N2255,N2256, N2257,N2258,N2259,N2260,  \\note fields as indicated
  N2261,N2262,N2263,N2264, N2265,N2266,N2267,N2268, N2269,N2270,N2271,N2272, N2273,N2274,N2275,N2276, N2277,N2278,N2279,N2280,  \\note fields as indicated
  N2281,N2282,N2283,N2284, N2285,N2286,N2287,N2288, N2289,N2290,N2291,N2292, N2293,N2294,N2295,N2296, N2297,N2298,N2299,N2300,  \\note fields as indicated

  N2301,N2302,N2303,N2304, N2305,N2306,N2307,N2308, N2309,N2310,N2311,N2312, N2313,N2314,N2315,N2316, N2317,N2318,N2319,N2320,  \\note fields as indicated
  N2321,N2322,N2323,N2324, N2325,N2326,N2327,N2328, N2329,N2330,N2331,N2332, N2333,N2334,N2335,N2336, N2337,N2338,N2339,N2340,  \\note fields as indicated
  N2341,N2342,N2343,N2344, N2345,N2346,N2347,N2348, N2349,N2350,N2351,N2352, N2353,N2354,N2355,N2356, N2357,N2358,N2359,N2360,  \\note fields as indicated
  N2361,N2362,N2363,N2364, N2365,N2366,N2367,N2368, N2369,N2370,N2371,N2372, N2373,N2374,N2375,N2376, N2377,N2378,N2379,N2380,  \\note fields as indicated
  N2381,N2382,N2383,N2384, N2385,N2386,N2387,N2388, N2389,N2390,N2391,N2392, N2393,N2394,N2395,N2396, N2397,N2398,N2399,N2400,  \\note fields as indicated

  N2401,N2402,N2403,N2404, N2405,N2406,N2407,N2408, N2409,N2410,N2411,N2412, N2413,N2414,N2415,N2416, N2417,N2418,N2419,N2420,  \\note fields as indicated
  N2421,N2422,N2423,N2424, N2425,N2426,N2427,N2428, N2429,N2430,N2431,N2432, N2433,N2434,N2435,N2436, N2437,N2438,N2439,N2440,  \\note fields as indicated
  N2441,N2442,N2443,N2444, N2445,N2446,N2447,N2448, N2449,N2450,N2451,N2452, N2453,N2454,N2455,N2456, N2457,N2458,N2459,N2460,  \\note fields as indicated
  N2461,N2462,N2463,N2464, N2465,N2466,N2467,N2468, N2469,N2470,N2471,N2472, N2473,N2474,N2475,N2476, N2477,N2478,N2479,N2480,  \\note fields as indicated
  N2481,N2482,N2483,N2484, N2485,N2486,N2487,N2488, N2489,N2490,N2491,N2492, N2493,N2494,N2495,N2496, N2497,N2498,N2499,N2500,  \\note fields as indicated

  N2501,N2502,N2503,N2504, N2505,N2506,N2507,N2508, N2509,N2510,N2511,N2512, N2513,N2514,N2515,N2516, N2517,N2518,N2519,N2520,  \\note fields as indicated
  N2521,N2522,N2523,N2524, N2525,N2526,N2527,N2528, N2529,N2530,N2531,N2532, N2533,N2534,N2535,N2536, N2537,N2538,N2539,N2540,  \\note fields as indicated
  N2541,N2542,N2543,N2544, N2545,N2546,N2547,N2548, N2549,N2550,N2551,N2552, N2553,N2554,N2555,N2556, N2557,N2558,N2559,N2560,  \\note fields as indicated
  N2561,N2562,N2563,N2564, N2565,N2566,N2567,N2568, N2569,N2570,N2571,N2572, N2573,N2574,N2575,N2576, N2577,N2578,N2579,N2580,  \\note fields as indicated
  N2581,N2582,N2583,N2584, N2585,N2586,N2587,N2588, N2589,N2590,N2591,N2592, N2593,N2594,N2595,N2596, N2597,N2598,N2599,N2600,  \\note fields as indicated

  N2601,N2602,N2603,N2604, N2605,N2606,N2607,N2608, N2609,N2610,N2611,N2612, N2613,N2614,N2615,N2616, N2617,N2618,N2619,N2620,  \\note fields as indicated
  N2621,N2622,N2623,N2624, N2625,N2626,N2627,N2628, N2629,N2630,N2631,N2632, N2633,N2634,N2635,N2636, N2637,N2638,N2639,N2640,  \\note fields as indicated
  N2641,N2642,N2643,N2644, N2645,N2646,N2647,N2648, N2649,N2650,N2651,N2652, N2653,N2654,N2655,N2656, N2657,N2658,N2659,N2660,  \\note fields as indicated
  N2661,N2662,N2663,N2664, N2665,N2666,N2667,N2668, N2669,N2670,N2671,N2672, N2673,N2674,N2675,N2676, N2677,N2678,N2679,N2680,  \\note fields as indicated
  N2681,N2682,N2683,N2684, N2685,N2686,N2687,N2688, N2689,N2690,N2691,N2692, N2693,N2694,N2695,N2696, N2697,N2698,N2699,N2700,  \\note fields as indicated

  N2701,N2702,N2703,N2704, N2705,N2706,N2707,N2708, N2709,N2710,N2711,N2712, N2713,N2714,N2715,N2716, N2717,N2718,N2719,N2720,  \\note fields as indicated
  N2721,N2722,N2723,N2724, N2725,N2726,N2727,N2728, N2729,N2730,N2731,N2732, N2733,N2734,N2735,N2736, N2737,N2738,N2739,N2740,  \\note fields as indicated
  N2741,N2742,N2743,N2744, N2745,N2746,N2747,N2748, N2749,N2750,N2751,N2752, N2753,N2754,N2755,N2756, N2757,N2758,N2759,N2760,  \\note fields as indicated
  N2761,N2762,N2763,N2764, N2765,N2766,N2767,N2768, N2769,N2770,N2771,N2772, N2773,N2774,N2775,N2776, N2777,N2778,N2779,N2780,  \\note fields as indicated
  N2781,N2782,N2783,N2784, N2785,N2786,N2787,N2788, N2789,N2790,N2791,N2792, N2793,N2794,N2795,N2796, N2797,N2798,N2799,N2800,  \\note fields as indicated

  N2801,N2802,N2803,N2804, N2805,N2806,N2807,N2808, N2809,N2810,N2811,N2812, N2813,N2814,N2815,N2816, N2817,N2818,N2819,N2820,  \\note fields as indicated
  N2821,N2822,N2823,N2824, N2825,N2826,N2827,N2828, N2829,N2830,N2831,N2832, N2833,N2834,N2835,N2836, N2837,N2838,N2839,N2840,  \\note fields as indicated
  N2841,N2842,N2843,N2844, N2845,N2846,N2847,N2848, N2849,N2850,N2851,N2852, N2853,N2854,N2855,N2856, N2857,N2858,N2859,N2860,  \\note fields as indicated
  N2861,N2862,N2863,N2864, N2865,N2866,N2867,N2868, N2869,N2870,N2871,N2872, N2873,N2874,N2875,N2876, N2877,N2878,N2879,N2880,  \\note fields as indicated
  N2881,N2882,N2883,N2884, N2885,N2886,N2887,N2888, N2889,N2890,N2891,N2892, N2893,N2894,N2895,N2896, N2897,N2898,N2899,N2900,  \\note fields as indicated

  N2901,N2902,N2903,N2904, N2905,N2906,N2907,N2908, N2909,N2910,N2911,N2912, N2913,N2914,N2915,N2916, N2917,N2918,N2919,N2920,  \\note fields as indicated
  N2921,N2922,N2923,N2924, N2925,N2926,N2927,N2928, N2929,N2930,N2931,N2932, N2933,N2934,N2935,N2936, N2937,N2938,N2939,N2940,  \\note fields as indicated
  N2941,N2942,N2943,N2944, N2945,N2946,N2947,N2948, N2949,N2950,N2951,N2952, N2953,N2954,N2955,N2956, N2957,N2958,N2959,N2960,  \\note fields as indicated
  N2961,N2962,N2963,N2964, N2965,N2966,N2967,N2968, N2969,N2970,N2971,N2972, N2973,N2974,N2975,N2976, N2977,N2978,N2979,N2980,  \\note fields as indicated
  N2981,N2982,N2983,N2984, N2985,N2986,N2987,N2988, N2989,N2990,N2991,N2992, N2993,N2994,N2995,N2996, N2997,N2998,N2999,N3000,  \\note fields as indicated

  N3001,N3002,N3003,N3004, N3005,N3006,N3007,N3008, N3009,N3010,N3011,N3012, N3013,N3014,N3015,N3016, N3017,N3018,N3019,N3020,  \\note fields as indicated
  N3021,N3022,N3023,N3024, N3025,N3026,N3027,N3028, N3029,N3030,N3031,N3032, N3033,N3034,N3035,N3036, N3037,N3038,N3039,N3040,  \\note fields as indicated
  N3041,N3042,N3043,N3044, N3045,N3046,N3047,N3048, N3049,N3050,N3051,N3052, N3053,N3054,N3055,N3056, N3057,N3058,N3059,N3060,  \\note fields as indicated
  N3061,N3062,N3063,N3064, N3065,N3066,N3067,N3068, N3069,N3070,N3071,N3072, N3073,N3074,N3075,N3076, N3077,N3078,N3079,N3080,  \\note fields as indicated
  N3081,N3082,N3083,N3084, N3085,N3086,N3087,N3088, N3089,N3090,N3091,N3092, N3093,N3094,N3095,N3096, N3097,N3098,N3099,N3100,  \\note fields as indicated

  N3101,N3102,N3103,N3104, N3105,N3106,N3107,N3108, N3109,N3110,N3111,N3112, N3113,N3114,N3115,N3116, N3117,N3118,N3119,N3120,  \\note fields as indicated
  N3121,N3122,N3123,N3124, N3125,N3126,N3127,N3128, N3129,N3130,N3131,N3132, N3133,N3134,N3135,N3136, N3137,N3138,N3139,N3140,  \\note fields as indicated
  N3141,N3142,N3143,N3144, N3145,N3146,N3147,N3148, N3149,N3150,N3151,N3152, N3153,N3154,N3155,N3156, N3157,N3158,N3159,N3160,  \\note fields as indicated
  N3161,N3162,N3163,N3164, N3165,N3166,N3167,N3168, N3169,N3170,N3171,N3172, N3173,N3174,N3175,N3176, N3177,N3178,N3179,N3180,  \\note fields as indicated
  N3181,N3182,N3183,N3184, N3185,N3186,N3187,N3188, N3189,N3190,N3191,N3192, N3193,N3194,N3195,N3196, N3197,N3198,N3199,N3200;  \\note fields as indicated

Construction,
       \\memo Start with outside layer and work your way to the inside layer
       \\memo Up to 10 layers total, 8 for windows
       \\memo Enter the material name for each layer
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
  A2 , \\field Outside Layer
       \\required-field
       \\type object-list
       \\object-list MaterialName
  A3 , \\field Layer 2
       \\type object-list
       \\object-list MaterialName
  A4 , \\field Layer 3
       \\type object-list
       \\object-list MaterialName
  A5 , \\field Layer 4
       \\type object-list
       \\object-list MaterialName
  A6 , \\field Layer 5
       \\type object-list
       \\object-list MaterialName
  A7 , \\field Layer 6
       \\type object-list
       \\object-list MaterialName
  A8 , \\field Layer 7
       \\type object-list
       \\object-list MaterialName
  A9 , \\field Layer 8
       \\type object-list
       \\object-list MaterialName
  A10, \\field Layer 9
       \\type object-list
       \\object-list MaterialName
  A11; \\field Layer 10
       \\type object-list
       \\object-list MaterialName

Construction:CfactorUndergroundWall,
       \\memo Alternate method of describing underground wall constructions
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
  N1 , \\field C-Factor
       \\required-field
       \\units W/m2-K
       \\type real
       \\minimum> 0.0
       \\note Enter C-Factor without film coefficients or soil
  N2 ; \\field Height
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
       \\note Enter height of the underground wall

Construction:FfactorGroundFloor,
       \\memo Alternate method of describing slab-on-grade or underground floor constructions
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
  N1 , \\field F-Factor
       \\required-field
       \\units W/m-K
       \\ip-units Btu/h-ft-F
       \\type real
       \\minimum> 0.0
  N2 , \\field Area
       \\required-field
       \\units m2
       \\type real
       \\minimum> 0.0
       \\note Enter area of the floor
  N3 ; \\field PerimeterExposed
       \\required-field
       \\units m
       \\type real
       \\minimum 0.0
       \\note Enter exposed perimeter of the floor

Construction:InternalSource,
       \\memo Start with outside layer and work your way to the inside Layer
       \\memo Up to 10 layers total, 8 for windows
       \\memo Enter the material name for each layer
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
  N1 , \\field Source Present After Layer Number
       \\required-field
       \\type integer
       \\minimum 1
       \\note refers to the list of materials which follows
  N2 , \\field Temperature Calculation Requested After Layer Number
       \\required-field
       \\type integer
       \\note refers to the list of materials which follows
  N3 , \\field Dimensions for the CTF Calculation
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 2
       \\note 1 = 1-dimensional calculation, 2 = 2-dimensional calculation
  N4 , \\field Tube Spacing
       \\required-field
       \\type real
       \\units m
       \\note uniform spacing between tubes or resistance wires in direction
       \\note perpendicular to main intended direction of heat transfer
  A2 , \\field Outside Layer
       \\required-field
       \\type object-list
       \\object-list MaterialName
  A3 , \\field Layer 2
       \\type object-list
       \\object-list MaterialName
  A4 , \\field Layer 3
       \\type object-list
       \\object-list MaterialName
  A5 , \\field Layer 4
       \\type object-list
       \\object-list MaterialName
  A6 , \\field Layer 5
       \\type object-list
       \\object-list MaterialName
  A7 , \\field Layer 6
       \\type object-list
       \\object-list MaterialName
  A8 , \\field Layer 7
       \\type object-list
       \\object-list MaterialName
  A9 , \\field Layer 8
       \\type object-list
       \\object-list MaterialName
  A10, \\field Layer 9
       \\type object-list
       \\object-list MaterialName
  A11; \\field Layer 10
       \\type object-list
       \\object-list MaterialName

WindowThermalModel:Params,
       \\memo object is used to select which thermal model should be used in tarcog simulations
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WindowThermalModelParameters
  A2 , \\field standard
       \\type choice
       \\key ISO15099
       \\key EN673Declared
       \\key EN673Design
       \\default ISO15099
  A3 , \\field Thermal Model
       \\type choice
       \\key ISO15099
       \\key ScaledCavityWidth
       \\key ConvectiveScalarModel_NoSDThickness
       \\key ConvectiveScalarModel_withSDThickness
       \\default ISO15099
  N1 , \\field SDScalar
       \\type real
       \\minimum  0.0
       \\maximum 1.0
       \\default 1
  A4 , \\field Deflection Model
       \\type choice
       \\key NoDeflection
       \\key TemperatureAndPressureInput
       \\key MeasuredDeflection
       \\default NoDeflection
  N2 , \\field Vacuum Pressure Limit
       \\type real
       \\units Pa
       \\minimum >0
       \\default 13.238
  N3 , \\field Initial temperature
       \\note This is temperature in time of window fabrication
       \\type real
       \\units C
       \\minimum >0
       \\default 25
  N4 ; \\field Initial pressure
       \\note This is pressure in time of window fabrication
       \\type real
       \\units Pa
       \\minimum >0
       \\default 101325


Construction:ComplexFenestrationState,
       \\memo Describes one state for a complex glazing system
       \\memo These input objects are typically generated by using WINDOW software and export to IDF syntax
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
       \\reference ComplexFenestrationStates
  A2 , \\field Basis Type
       \\required-field
       \\type choice
       \\key LBNLWINDOW
       \\key UserDefined
       \\default LBNLWINDOW
  A3 , \\field Basis Symmetry Type
       \\type choice
       \\key Axisymmetric
       \\key None
       \\default None
  A4 , \\field Window Thermal Model
       \\required-field
       \\type object-list
       \\object-list WindowThermalModelParameters
  A5 , \\field Basis Matrix Name
       \\type object-list
       \\object-list DataMatrices
       \\required-field
  A6 , \\field Solar Optical Complex Front Transmittance Matrix Name
       \\type object-list
       \\object-list DataMatrices
       \\required-field
  A7 , \\field Solar Optical Complex Back Reflectance Matrix Name
       \\type object-list
       \\object-list DataMatrices
       \\required-field
  A8 , \\field Visible Optical Complex Front Transmittance Matrix Name
       \\type object-list
       \\object-list DataMatrices
       \\required-field
  A9 , \\field Visible Optical Complex Back Transmittance Matrix Name
       \\type object-list
       \\object-list DataMatrices
       \\required-field
  A10, \\field Outside Layer Material Name
       \\note optional field  currently unused
       \\note for future development
       \\type object-list
       \\object-list MaterialName
  A11,  \\field Outside Layer Directional Front Absoptance Matrix Name
       \\required-field
       \\type object-list
       \\object-list DataMatrices
  A12, \\field Outside Layer Directional Back Absoptance Matrix Name
       \\required-field
       \\type object-list
       \\object-list DataMatrices
  A13, \\field Layer 2 Material Name
       \\type object-list
       \\object-list MaterialName
  A14, \\field Layer 2 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A15, \\field Layer 2 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A16, \\field Layer 3 Material Name
       \\type object-list
       \\object-list MaterialName
  A17, \\field Layer 3 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A18, \\field Layer 3 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A19, \\field Layer 4 Material Name
       \\type object-list
       \\object-list MaterialName
  A20, \\field Layer 4 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A21, \\field Layer 4 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A22, \\field Layer 5 Material Name
       \\type object-list
       \\object-list MaterialName
  A23, \\field Layer 5 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A24, \\field Layer 5 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A25, \\field Layer 6 Material Name
       \\type object-list
       \\object-list MaterialName
  A26, \\field Layer 6 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A27, \\field Layer 6 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A28, \\field Layer 7 Material Name
       \\type object-list
       \\object-list MaterialName
  A29, \\field Layer 7 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A30, \\field Layer 7 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A31, \\field Layer 8 Material Name
       \\type object-list
       \\object-list MaterialName
  A32, \\field Layer 8 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A33, \\field Layer 8 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A34, \\field Layer 9 Material Name
       \\type object-list
       \\object-list MaterialName
  A35, \\field Layer 9 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A36, \\field Layer 9 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A37, \\field Layer 10 Material Name
       \\type object-list
       \\object-list MaterialName
  A38, \\field Layer 10 Directional Front Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices
  A39; \\field Layer 10 Directional Back Absoptance Matrix Name
       \\type object-list
       \\object-list DataMatrices

Construction:WindowDataFile,
       \\memo Initiates search of the Window data file for a window called Name.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ConstructionNames
  A2 ; \\field File Name
       \\retaincase
       \\note default file name is "Window5DataFile.dat"
       \\note limit on this field is 100 characters.

\\group Thermal Zones and Surfaces

GlobalGeometryRules,
      \\memo Specifes the geometric rules used to describe the input of surface vertices and
      \\memo daylighting reference points.
      \\required-object
      \\unique-object
  A1, \\field Starting Vertex Position
      \\required-field
      \\note Specified as entry for a 4 sided surface/rectangle
      \\note Surfaces are specified as viewed from outside the surface
      \\note Shading surfaces as viewed from behind.  (towards what they are shading)
      \\type choice
      \\key UpperLeftCorner
      \\key LowerLeftCorner
      \\key UpperRightCorner
      \\key LowerRightCorner
  A2, \\field Vertex Entry Direction
      \\required-field
      \\type choice
      \\key Counterclockwise
      \\key Clockwise
  A3, \\field Coordinate System
      \\required-field
      \\note relative -- coordinates are entered relative to zone origin
      \\note world -- all coordinates entered are "absolute" for this facility
      \\note absolute -- same as world
      \\type choice
      \\key Relative
      \\key World
      \\key Absolute
  A4, \\field Daylighting Reference Point Coordinate System
      \\type choice
      \\key Relative
      \\default Relative
      \\note Relative -- coordinates are entered relative to zone origin
      \\key World
      \\note World -- all coordinates entered are "absolute" for this facility
      \\key Absolute
      \\note absolute -- same as world
  A5; \\field Rectangular Surface Coordinate System
      \\type choice
      \\key Relative
      \\default Relative
      \\note Relative -- Starting corner is entered relative to zone origin
      \\key World
      \\note World -- Starting corner is entered in "absolute"
      \\key Absolute
      \\note absolute -- same as world

GeometryTransform,
      \\memo Provides a simple method of altering the footprint geometry of a model. The intent
      \\memo is to provide a single parameter that can be used to reshape the building description
      \\memo contained in the rest of the input file.
     \\unique-object
  A1, \\field Plane of Transform
      \\required-field
      \\type choice
      \\key XY
      \\default XY
      \\note only current allowed value is "XY"
  N1, \\field Current Aspect Ratio
      \\required-field
      \\minimum> 0
      \\note Aspect ratio of building as described in idf
  N2; \\field New Aspect Ratio
      \\required-field
      \\minimum> 0
      \\note Aspect ratio to transform to during run

Zone,
       \\memo Defines a thermal zone of the building.
  \\format vertices
  A1 , \\Field Name
       \\required-field
       \\type alpha
       \\reference ZoneNames
       \\reference OutFaceEnvNames
       \\reference ZoneAndZoneListNames
       \\reference AirflowNetworkNodeAndZoneNames
  N1 , \\field Direction of Relative North
       \\units deg
       \\type real
       \\default 0
  N2 , \\field X Origin
       \\units m
       \\type real
       \\default 0
  N3 , \\field Y Origin
       \\units m
       \\type real
       \\default 0
  N4 , \\field Z Origin
       \\units m
       \\type real
       \\default 0
  N5 , \\field Type
       \\type integer
       \\maximum 1
       \\minimum 1
       \\default 1
  N6 , \\field Multiplier
       \\type integer
       \\minimum 1
       \\default 1
  N7 , \\field Ceiling Height
       \\note If this field is 0.0, negative or autocalculate, then the average height
       \\note of the zone is automatically calculated and used in subsequent calculations.
       \\note If this field is positive, then the number entered here will be used.
       \\note Note that the Zone Ceiling Height is the distance from the Floor to
       \\note the Ceiling in the Zone, not an absolute height from the ground.
       \\units m
       \\type real
       \\autocalculatable
       \\default autocalculate
  N8 , \\field Volume
       \\note If this field is 0.0, negative or autocalculate, then the volume of the zone
       \\note is automatically calculated and used in subsequent calculations.
       \\note If this field is positive, then the number entered here will be used.
       \\units m3
       \\type real
       \\autocalculatable
       \\default autocalculate
  N9 , \\field Floor Area
       \\note If this field is 0.0, negative or autocalculate, then the floor area of the zone
       \\note is automatically calculated and used in subsequent calculations.
       \\note If this field is positive, then the number entered here will be used.
       \\units m2
       \\type real
       \\autocalculatable
       \\default autocalculate
  A2 , \\field Zone Inside Convection Algorithm
       \\type choice
       \\key Simple
       \\key TARP
       \\key CeilingDiffuser
       \\key AdaptiveConvectionAlgorithm
       \\key TrombeWall
       \\note Will default to same value as SurfaceConvectionAlgorithm:Inside object
       \\note setting this field overrides the default SurfaceConvectionAlgorithm:Inside for this zone
       \\note Simple = constant natural convection (ASHRAE)
       \\note TARP = variable natural convection based on temperature difference (ASHRAE)
       \\note CeilingDiffuser = ACH based forced and mixed convection correlations
       \\note  for ceiling diffuser configuration with simple natural convection limit
       \\note AdaptiveConvectionAlgorithm = dynamic selection of convection models based on conditions
       \\note TrombeWall = variable natural convection in an enclosed rectangular cavity
  A3,  \\field Zone Outside Convection Algorithm
       \\note Will default to same value as SurfaceConvectionAlgorithm:Outside object
       \\note setting this field overrides the default SurfaceConvectionAlgorithm:Outside for this zone
       \\type choice
       \\key SimpleCombined
       \\key TARP
       \\key DOE-2
       \\key MoWiTT
       \\key AdaptiveConvectionAlgorithm
       \\note SimpleCombined = Combined radiation and convection coefficient using simple ASHRAE model
       \\note TARP = correlation from models developed by ASHRAE, Walton, and Sparrow et. al.
       \\note MoWiTT = correlation from measurements by Klems and Yazdanian for smooth surfaces
       \\note DOE-2 = correlation from measurements by Klems and Yazdanian for rough surfaces
       \\note AdaptiveConvectionAlgorithm = dynamic selection of correlations based on conditions
  A4;  \\field Part of Total Floor Area
       \\type choice
       \\key Yes
       \\key No
       \\default Yes

ZoneList,
       \\memo Defines a list of thermal zones which can be referenced as a group. The ZoneList name
       \\memo may be used elsewhere in the input to apply a parameter to all zones in the list.
       \\memo ZoneLists can be used effectively with the following objects: People, Lights,
       \\memo ElectricEquipment, GasEquipment, HotWaterEquipment, ZoneInfiltration:DesignFlowRate,
       \\memo ZoneVentilation:DesignFlowRate, Sizing:Zone, ZoneControl:Thermostat, and others.
       \\min-fields 2
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
  A1 , \\field Name
       \\note Name of the Zone List
       \\required-field
       \\type alpha
       \\reference ZoneListNames
       \\reference ZoneAndZoneListNames
  A2 , \\field Zone 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Zone 2 Name
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Zone 3 Name
       \\type object-list
       \\object-list ZoneNames
  A5 , \\field Zone 4 Name
       \\type object-list
       \\object-list ZoneNames
  A6 , \\field Zone 5 Name
       \\type object-list
       \\object-list ZoneNames
  A7 , \\field Zone 6 Name
       \\type object-list
       \\object-list ZoneNames
  A8 , \\field Zone 7 Name
       \\type object-list
       \\object-list ZoneNames
  A9 , \\field Zone 8 Name
       \\type object-list
       \\object-list ZoneNames
  A10, \\field Zone 9 Name
       \\type object-list
       \\object-list ZoneNames
  A11, \\field Zone 10 Name
       \\type object-list
       \\object-list ZoneNames
  A12, \\field Zone 11 Name
       \\type object-list
       \\object-list ZoneNames
  A13, \\field Zone 12 Name
       \\type object-list
       \\object-list ZoneNames
  A14, \\field Zone 13 Name
       \\type object-list
       \\object-list ZoneNames
  A15, \\field Zone 14 Name
       \\type object-list
       \\object-list ZoneNames
  A16, \\field Zone 15 Name
       \\type object-list
       \\object-list ZoneNames
  A17, \\field Zone 16 Name
       \\type object-list
       \\object-list ZoneNames
  A18, \\field Zone 17 Name
       \\type object-list
       \\object-list ZoneNames
  A19, \\field Zone 18 Name
       \\type object-list
       \\object-list ZoneNames
  A20, \\field Zone 19 Name
       \\type object-list
       \\object-list ZoneNames
  A21, \\field Zone 20 Name
       \\type object-list
       \\object-list ZoneNames
  A22, \\field Zone 21 Name
       \\type object-list
       \\object-list ZoneNames
  A23, \\field Zone 22 Name
       \\type object-list
       \\object-list ZoneNames
  A24, \\field Zone 23 Name
       \\type object-list
       \\object-list ZoneNames
  A25, \\field Zone 24 Name
       \\type object-list
       \\object-list ZoneNames
  A26, \\field Zone 25 Name
       \\type object-list
       \\object-list ZoneNames
  A27, \\field Zone 26 Name
       \\type object-list
       \\object-list ZoneNames
  A28, \\field Zone 27 Name
       \\type object-list
       \\object-list ZoneNames
  A29, \\field Zone 28 Name
       \\type object-list
       \\object-list ZoneNames
  A30, \\field Zone 29 Name
       \\type object-list
       \\object-list ZoneNames
  A31, \\field Zone 30 Name
       \\type object-list
       \\object-list ZoneNames
  A32, \\field Zone 31 Name
       \\type object-list
       \\object-list ZoneNames
  A33, \\field Zone 32 Name
       \\type object-list
       \\object-list ZoneNames
  A34, \\field Zone 33 Name
       \\type object-list
       \\object-list ZoneNames
  A35, \\field Zone 34 Name
       \\type object-list
       \\object-list ZoneNames
  A36, \\field Zone 35 Name
       \\type object-list
       \\object-list ZoneNames
  A37, \\field Zone 36 Name
       \\type object-list
       \\object-list ZoneNames
  A38, \\field Zone 37 Name
       \\type object-list
       \\object-list ZoneNames
  A39, \\field Zone 38 Name
       \\type object-list
       \\object-list ZoneNames
  A40, \\field Zone 39 Name
       \\type object-list
       \\object-list ZoneNames
  A41, \\field Zone 40 Name
       \\type object-list
       \\object-list ZoneNames
  A42, \\field Zone 41 Name
       \\type object-list
       \\object-list ZoneNames
  A43, \\field Zone 42 Name
       \\type object-list
       \\object-list ZoneNames
  A44, \\field Zone 43 Name
       \\type object-list
       \\object-list ZoneNames
  A45, \\field Zone 44 Name
       \\type object-list
       \\object-list ZoneNames
  A46, \\field Zone 45 Name
       \\type object-list
       \\object-list ZoneNames
  A47, \\field Zone 46 Name
       \\type object-list
       \\object-list ZoneNames
  A48, \\field Zone 47 Name
       \\type object-list
       \\object-list ZoneNames
  A49, \\field Zone 48 Name
       \\type object-list
       \\object-list ZoneNames
  A50, \\field Zone 49 Name
       \\type object-list
       \\object-list ZoneNames
  A51, \\field Zone 50 Name
       \\type object-list
       \\object-list ZoneNames
  A52, \\field Zone 51 Name
       \\type object-list
       \\object-list ZoneNames
  A53, \\field Zone 52 Name
       \\type object-list
       \\object-list ZoneNames
  A54, \\field Zone 53 Name
       \\type object-list
       \\object-list ZoneNames
  A55, \\field Zone 54 Name
       \\type object-list
       \\object-list ZoneNames
  A56, \\field Zone 55 Name
       \\type object-list
       \\object-list ZoneNames
  A57, \\field Zone 56 Name
       \\type object-list
       \\object-list ZoneNames
  A58, \\field Zone 57 Name
       \\type object-list
       \\object-list ZoneNames
  A59, \\field Zone 58 Name
       \\type object-list
       \\object-list ZoneNames
  A60, \\field Zone 59 Name
       \\type object-list
       \\object-list ZoneNames
  A61, \\field Zone 60 Name
       \\type object-list
       \\object-list ZoneNames
  A62, \\field Zone 61 Name
       \\type object-list
       \\object-list ZoneNames
  A63, \\field Zone 62 Name
       \\type object-list
       \\object-list ZoneNames
  A64, \\field Zone 63 Name
       \\type object-list
       \\object-list ZoneNames
  A65, \\field Zone 64 Name
       \\type object-list
       \\object-list ZoneNames
  A66, \\field Zone 65 Name
       \\type object-list
       \\object-list ZoneNames
  A67, \\field Zone 66 Name
       \\type object-list
       \\object-list ZoneNames
  A68, \\field Zone 67 Name
       \\type object-list
       \\object-list ZoneNames
  A69, \\field Zone 68 Name
       \\type object-list
       \\object-list ZoneNames
  A70, \\field Zone 69 Name
       \\type object-list
       \\object-list ZoneNames
  A71, \\field Zone 70 Name
       \\type object-list
       \\object-list ZoneNames
  A72, \\field Zone 71 Name
       \\type object-list
       \\object-list ZoneNames
  A73, \\field Zone 72 Name
       \\type object-list
       \\object-list ZoneNames
  A74, \\field Zone 73 Name
       \\type object-list
       \\object-list ZoneNames
  A75, \\field Zone 74 Name
       \\type object-list
       \\object-list ZoneNames
  A76, \\field Zone 75 Name
       \\type object-list
       \\object-list ZoneNames
  A77, \\field Zone 76 Name
       \\type object-list
       \\object-list ZoneNames
  A78, \\field Zone 77 Name
       \\type object-list
       \\object-list ZoneNames
  A79, \\field Zone 78 Name
       \\type object-list
       \\object-list ZoneNames
  A80, \\field Zone 79 Name
       \\type object-list
       \\object-list ZoneNames
  A81, \\field Zone 80 Name
       \\type object-list
       \\object-list ZoneNames
  A82, \\field Zone 81 Name
       \\type object-list
       \\object-list ZoneNames
  A83, \\field Zone 82 Name
       \\type object-list
       \\object-list ZoneNames
  A84, \\field Zone 83 Name
       \\type object-list
       \\object-list ZoneNames
  A85, \\field Zone 84 Name
       \\type object-list
       \\object-list ZoneNames
  A86, \\field Zone 85 Name
       \\type object-list
       \\object-list ZoneNames
  A87, \\field Zone 86 Name
       \\type object-list
       \\object-list ZoneNames
  A88, \\field Zone 87 Name
       \\type object-list
       \\object-list ZoneNames
  A89, \\field Zone 88 Name
       \\type object-list
       \\object-list ZoneNames
  A90, \\field Zone 89 Name
       \\type object-list
       \\object-list ZoneNames
  A91, \\field Zone 90 Name
       \\type object-list
       \\object-list ZoneNames
  A92, \\field Zone 91 Name
       \\type object-list
       \\object-list ZoneNames
  A93, \\field Zone 92 Name
       \\type object-list
       \\object-list ZoneNames
  A94, \\field Zone 93 Name
       \\type object-list
       \\object-list ZoneNames
  A95, \\field Zone 94 Name
       \\type object-list
       \\object-list ZoneNames
  A96, \\field Zone 95 Name
       \\type object-list
       \\object-list ZoneNames
  A97, \\field Zone 96 Name
       \\type object-list
       \\object-list ZoneNames
  A98, \\field Zone 97 Name
       \\type object-list
       \\object-list ZoneNames
  A99, \\field Zone 98 Name
       \\type object-list
       \\object-list ZoneNames
  A100, \\field Zone 99 Name
       \\type object-list
       \\object-list ZoneNames
  A101, \\field Zone 100 Name
       \\type object-list
       \\object-list ZoneNames
  A102,A103,A104,A105,A106,A107,A108,A109,A110,A111, \\note fields as indicated
  A112,A113,A114,A115,A116,A117,A118,A119,A120,A121, \\note fields as indicated
  A122,A123,A124,A125,A126,A127,A128,A129,A130,A131, \\note fields as indicated
  A132,A133,A134,A135,A136,A137,A138,A139,A140,A141, \\note fields as indicated
  A142,A143,A144,A145,A146,A147,A148,A149,A150,A151, \\note fields as indicated
  A152,A153,A154,A155,A156,A157,A158,A159,A160,A161, \\note fields as indicated
  A162,A163,A164,A165,A166,A167,A168,A169,A170,A171, \\note fields as indicated
  A172,A173,A174,A175,A176,A177,A178,A179,A180,A181, \\note fields as indicated
  A182,A183,A184,A185,A186,A187,A188,A189,A190,A191, \\note fields as indicated
  A192,A193,A194,A195,A196,A197,A198,A199,A200,A201, \\note fields as indicated
  A202,A203,A204,A205,A206,A207,A208,A209,A210,A211, \\note fields as indicated
  A212,A213,A214,A215,A216,A217,A218,A219,A220,A221, \\note fields as indicated
  A222,A223,A224,A225,A226,A227,A228,A229,A230,A231, \\note fields as indicated
  A232,A233,A234,A235,A236,A237,A238,A239,A240,A241, \\note fields as indicated
  A242,A243,A244,A245,A246,A247,A248,A249,A250,A251, \\note fields as indicated
  A252,A253,A254,A255,A256,A257,A258,A259,A260,A261, \\note fields as indicated
  A262,A263,A264,A265,A266,A267,A268,A269,A270,A271, \\note fields as indicated
  A272,A273,A274,A275,A276,A277,A278,A279,A280,A281, \\note fields as indicated
  A282,A283,A284,A285,A286,A287,A288,A289,A290,A291, \\note fields as indicated
  A292,A293,A294,A295,A296,A297,A298,A299,A300,A301, \\note fields as indicated
  A302,A303,A304,A305,A306,A307,A308,A309,A310,A311, \\note fields as indicated
  A312,A313,A314,A315,A316,A317,A318,A319,A320,A321, \\note fields as indicated
  A322,A323,A324,A325,A326,A327,A328,A329,A330,A331, \\note fields as indicated
  A332,A333,A334,A335,A336,A337,A338,A339,A340,A341, \\note fields as indicated
  A342,A343,A344,A345,A346,A347,A348,A349,A350,A351, \\note fields as indicated
  A352,A353,A354,A355,A356,A357,A358,A359,A360,A361, \\note fields as indicated
  A362,A363,A364,A365,A366,A367,A368,A369,A370,A371, \\note fields as indicated
  A372,A373,A374,A375,A376,A377,A378,A379,A380,A381, \\note fields as indicated
  A382,A383,A384,A385,A386,A387,A388,A389,A390,A391, \\note fields as indicated
  A392,A393,A394,A395,A396,A397,A398,A399,A400,A401, \\note fields as indicated
  A402,A403,A404,A405,A406,A407,A408,A409,A410,A411, \\note fields as indicated
  A412,A413,A414,A415,A416,A417,A418,A419,A420,A421, \\note fields as indicated
  A422,A423,A424,A425,A426,A427,A428,A429,A430,A431, \\note fields as indicated
  A432,A433,A434,A435,A436,A437,A438,A439,A440,A441, \\note fields as indicated
  A442,A443,A444,A445,A446,A447,A448,A449,A450,A451, \\note fields as indicated
  A452,A453,A454,A455,A456,A457,A458,A459,A460,A461, \\note fields as indicated
  A462,A463,A464,A465,A466,A467,A468,A469,A470,A471, \\note fields as indicated
  A472,A473,A474,A475,A476,A477,A478,A479,A480,A481, \\note fields as indicated
  A482,A483,A484,A485,A486,A487,A488,A489,A490,A491, \\note fields as indicated
  A492,A493,A494,A495,A496,A497,A498,A499,A500,A501; \\note fields as indicated

ZoneGroup,
       \\memo Adds a multiplier to a ZoneList. This can be used to reduce the amount of input
       \\memo necessary for simulating repetitive structures, such as the identical floors of a
       \\memo multi-story building.
       \\min-fields 2
  A1 , \\field Name
       \\note Name of the Zone Group
       \\required-field
       \\type alpha
  A2,  \\field Zone List Name
       \\required-field
       \\type object-list
       \\object-list ZoneListNames
  N1;  \\field Zone List Multiplier
       \\type integer
       \\default 1
       \\minimum 1

BuildingSurface:Detailed,
  \\memo Allows for detailed entry of building heat transfer surfaces.  Does not include subsurfaces such as windows or doors.
  \\extensible:3 -- duplicate last set of x,y,z coordinates (last 3 fields), remembering to remove ; from "inner" fields.
  \\format vertices
  \\min-fields 19
  A1 , \\Field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Surface Type
       \\required-field
       \\type choice
       \\key Floor
       \\key Wall
       \\key Ceiling
       \\key Roof
  A3 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A4 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  A5 , \\field Outside Boundary Condition
       \\required-field
       \\type choice
       \\key Adiabatic
       \\key Surface
       \\key Zone
       \\key Outdoors
       \\key Ground
       \\key GroundFCfactorMethod
       \\key OtherSideCoefficients
       \\key OtherSideConditionsModel
       \\key GroundSlabPreprocessorAverage
       \\key GroundSlabPreprocessorCore
       \\key GroundSlabPreprocessorPerimeter
       \\key GroundBasementPreprocessorAverageWall
       \\key GroundBasementPreprocessorAverageFloor
       \\key GroundBasementPreprocessorUpperWall
       \\key GroundBasementPreprocessorLowerWall
  A6,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Non-blank only if the field Outside Boundary Condition is Surface,
       \\note Zone, OtherSideCoefficients or OtherSideConditionsModel
       \\note If Surface, specify name of corresponding surface in adjacent zone or
       \\note specify current surface name for internal partition separating like zones
       \\note If Zone, specify the name of the corresponding zone and
       \\note the program will generate the corresponding interzone surface
       \\note If OtherSideCoefficients, specify name of SurfaceProperty:OtherSideCoefficients
       \\note If OtherSideConditionsModel, specify name of SurfaceProperty:OtherSideConditionsModel
  A7 , \\field Sun Exposure
       \\required-field
       \\type choice
       \\key SunExposed
       \\key NoSun
       \\default SunExposed
  A8,  \\field Wind Exposure
       \\required-field
       \\type choice
       \\key WindExposed
       \\key NoWind
       \\default WindExposed
  N1,  \\field View Factor to Ground
       \\type real
       \\note From the exterior of the surface
       \\note Unused if one uses the "reflections" options in Solar Distribution in Building input
       \\note unless a DaylightingDevice:Shelf or DaylightingDevice:Tubular object has been specified.
       \\note autocalculate will automatically calculate this value from the tilt of the surface
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default autocalculate
  N2 , \\field Number of Vertices
       \\note shown with 120 vertex coordinates -- extensible object
       \\note  "extensible" -- duplicate last set of x,y,z coordinates (last 3 fields),
       \\note remembering to remove ; from "inner" fields.
       \\note for clarity in any error messages, renumber the fields as well.
       \\note (and changing z terminator to a comma "," for all but last one which needs a semi-colon ";")
       \\autocalculatable
       \\minimum 3
       \\default autocalculate
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  If world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N3,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N5 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N7,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N8,  \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N12, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N13, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N14, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Y-coordinate
       \\type real
       \\units m
  N17, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Y-coordinate
       \\type real
       \\units m
  N20, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Y-coordinate
       \\type real
       \\units m
  N23, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Y-coordinate
       \\type real
       \\units m
  N26, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Y-coordinate
       \\type real
       \\units m
  N29, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Y-coordinate
       \\type real
       \\units m
  N32, \\field Vertex 10 Z-coordinate
       \\units m
       \\type real
  N33, \\field Vertex 11 X-coordinate
       \\units m
       \\type real
  N34, \\field Vertex 11 Y-coordinate
       \\type real
       \\units m
  N35, \\field Vertex 11 Z-coordinate
       \\units m
       \\type real
  N36, \\field Vertex 12 X-coordinate
       \\units m
       \\type real
  N37, \\field Vertex 12 Y-coordinate
       \\type real
       \\units m
  N38, \\field Vertex 12 Z-coordinate
       \\units m
       \\type real
  N39, \\field Vertex 13 X-coordinate
       \\units m
       \\type real
  N40, \\field Vertex 13 Y-coordinate
       \\type real
       \\units m
  N41, \\field Vertex 13 Z-coordinate
       \\units m
       \\type real
  N42, \\field Vertex 14 X-coordinate
       \\units m
       \\type real
  N43, \\field Vertex 14 Y-coordinate
       \\type real
       \\units m
  N44, \\field Vertex 14 Z-coordinate
       \\units m
       \\type real
  N45, \\field Vertex 15 X-coordinate
       \\units m
       \\type real
  N46, \\field Vertex 15 Y-coordinate
       \\type real
       \\units m
  N47, \\field Vertex 15 Z-coordinate
       \\units m
       \\type real
  N48, \\field Vertex 16 X-coordinate
       \\units m
       \\type real
  N49, \\field Vertex 16 Y-coordinate
       \\type real
       \\units m
  N50, \\field Vertex 16 Z-coordinate
       \\units m
       \\type real
  N51, \\field Vertex 17 X-coordinate
       \\units m
       \\type real
  N52, \\field Vertex 17 Y-coordinate
       \\type real
       \\units m
  N53, \\field Vertex 17 Z-coordinate
       \\units m
       \\type real
  N54, \\field Vertex 18 X-coordinate
       \\units m
       \\type real
  N55, \\field Vertex 18 Y-coordinate
       \\type real
       \\units m
  N56, \\field Vertex 18 Z-coordinate
       \\units m
       \\type real
  N57, \\field Vertex 19 X-coordinate
       \\units m
       \\type real
  N58, \\field Vertex 19 Y-coordinate
       \\type real
       \\units m
  N59, \\field Vertex 19 Z-coordinate
       \\units m
       \\type real
  N60, \\field Vertex 20 X-coordinate
       \\units m
       \\type real
  N61, \\field Vertex 20 Y-coordinate
       \\type real
       \\units m
  N62, \\field Vertex 20 Z-coordinate
       \\units m
       \\type real
  N63, \\field Vertex 21 X-coordinate
       \\units m
       \\type real
  N64, \\field Vertex 21 Y-coordinate
       \\type real
       \\units m
  N65, \\field Vertex 21 Z-coordinate
       \\units m
       \\type real
  N66, \\field Vertex 22 X-coordinate
       \\units m
       \\type real
  N67, \\field Vertex 22 Y-coordinate
       \\type real
       \\units m
  N68, \\field Vertex 22 Z-coordinate
       \\units m
       \\type real
  N69, \\field Vertex 23 X-coordinate
       \\units m
       \\type real
  N70, \\field Vertex 23 Y-coordinate
       \\type real
       \\units m
  N71, \\field Vertex 23 Z-coordinate
       \\units m
       \\type real
  N72, \\field Vertex 24 X-coordinate
       \\units m
       \\type real
  N73, \\field Vertex 24 Y-coordinate
       \\type real
       \\units m
  N74, \\field Vertex 24 Z-coordinate
       \\units m
       \\type real
  N75, \\field Vertex 25 X-coordinate
       \\units m
       \\type real
  N76, \\field Vertex 25 Y-coordinate
       \\type real
       \\units m
  N77, \\field Vertex 25 Z-coordinate
       \\units m
       \\type real
  N78, \\field Vertex 26 X-coordinate
       \\units m
       \\type real
  N79, \\field Vertex 26 Y-coordinate
       \\type real
       \\units m
  N80, \\field Vertex 26 Z-coordinate
       \\units m
       \\type real
  N81, \\field Vertex 27 X-coordinate
       \\units m
       \\type real
  N82, \\field Vertex 27 Y-coordinate
       \\type real
       \\units m
  N83, \\field Vertex 27 Z-coordinate
       \\units m
       \\type real
  N84, \\field Vertex 28 X-coordinate
       \\units m
       \\type real
  N85, \\field Vertex 28 Y-coordinate
       \\type real
       \\units m
  N86, \\field Vertex 28 Z-coordinate
       \\units m
       \\type real
  N87, \\field Vertex 29 X-coordinate
       \\units m
       \\type real
  N88, \\field Vertex 29 Y-coordinate
       \\type real
       \\units m
  N89, \\field Vertex 29 Z-coordinate
       \\units m
       \\type real
  N90, \\field Vertex 30 X-coordinate
       \\units m
       \\type real
  N91, \\field Vertex 30 Y-coordinate
       \\type real
       \\units m
  N92, \\field Vertex 30 Z-coordinate
       \\units m
       \\type real
  N93, \\field Vertex 31 X-coordinate
       \\units m
       \\type real
  N94, \\field Vertex 31 Y-coordinate
       \\units m
       \\type real
  N95, \\field Vertex 31 Z-coordinate
       \\units m
       \\type real
  N96, \\field Vertex 32 X-coordinate
       \\units m
       \\type real
  N97, \\field Vertex 32 Y-coordinate
       \\units m
       \\type real
  N98, \\field Vertex 32 Z-coordinate
       \\units m
       \\type real
  N99, \\field Vertex 33 X-coordinate
       \\units m
       \\type real
  N100, \\field Vertex 33 Y-coordinate
       \\units m
       \\type real
  N101, \\field Vertex 33 Z-coordinate
       \\units m
       \\type real
  N102, \\field Vertex 34 X-coordinate
       \\units m
       \\type real
  N103, \\field Vertex 34 Y-coordinate
       \\units m
       \\type real
  N104, \\field Vertex 34 Z-coordinate
       \\units m
       \\type real
  N105, \\field Vertex 35 X-coordinate
       \\units m
       \\type real
  N106, \\field Vertex 35 Y-coordinate
       \\units m
       \\type real
  N107, \\field Vertex 35 Z-coordinate
       \\units m
       \\type real
  N108, \\field Vertex 36 X-coordinate
       \\units m
       \\type real
  N109, \\field Vertex 36 Y-coordinate
       \\units m
       \\type real
  N110, \\field Vertex 36 Z-coordinate
       \\units m
       \\type real
  N111, \\field Vertex 37 X-coordinate
       \\units m
       \\type real
  N112, \\field Vertex 37 Y-coordinate
       \\units m
       \\type real
  N113, \\field Vertex 37 Z-coordinate
       \\units m
       \\type real
  N114, \\field Vertex 38 X-coordinate
       \\units m
       \\type real
  N115, \\field Vertex 38 Y-coordinate
       \\units m
       \\type real
  N116, \\field Vertex 38 Z-coordinate
       \\units m
       \\type real
  N117, \\field Vertex 39 X-coordinate
       \\units m
       \\type real
  N118, \\field Vertex 39 Y-coordinate
       \\units m
       \\type real
  N119, \\field Vertex 39 Z-coordinate
       \\units m
       \\type real
  N120, \\field Vertex 40 X-coordinate
       \\units m
       \\type real
  N121, \\field Vertex 40 Y-coordinate
       \\units m
       \\type real
  N122, \\field Vertex 40 Z-coordinate
       \\units m
       \\type real
  N123, \\field Vertex 41 X-coordinate
       \\units m
       \\type real
  N124, \\field Vertex 41 Y-coordinate
       \\units m
       \\type real
  N125, \\field Vertex 41 Z-coordinate
       \\units m
       \\type real
  N126, \\field Vertex 42 X-coordinate
       \\units m
       \\type real
  N127, \\field Vertex 42 Y-coordinate
       \\units m
       \\type real
  N128, \\field Vertex 42 Z-coordinate
       \\units m
       \\type real
  N129, \\field Vertex 43 X-coordinate
       \\units m
       \\type real
  N130, \\field Vertex 43 Y-coordinate
       \\units m
       \\type real
  N131, \\field Vertex 43 Z-coordinate
       \\units m
       \\type real
  N132, \\field Vertex 44 X-coordinate
       \\units m
       \\type real
  N133, \\field Vertex 44 Y-coordinate
       \\units m
       \\type real
  N134, \\field Vertex 44 Z-coordinate
       \\units m
       \\type real
  N135, \\field Vertex 45 X-coordinate
       \\units m
       \\type real
  N136, \\field Vertex 45 Y-coordinate
       \\units m
       \\type real
  N137, \\field Vertex 45 Z-coordinate
       \\units m
       \\type real
  N138, \\field Vertex 46 X-coordinate
       \\units m
       \\type real
  N139, \\field Vertex 46 Y-coordinate
       \\units m
       \\type real
  N140, \\field Vertex 46 Z-coordinate
       \\units m
       \\type real
  N141, \\field Vertex 47 X-coordinate
       \\units m
       \\type real
  N142, \\field Vertex 47 Y-coordinate
       \\units m
       \\type real
  N143, \\field Vertex 47 Z-coordinate
       \\units m
       \\type real
  N144, \\field Vertex 48 X-coordinate
       \\units m
       \\type real
  N145, \\field Vertex 48 Y-coordinate
       \\units m
       \\type real
  N146, \\field Vertex 48 Z-coordinate
       \\units m
       \\type real
  N147, \\field Vertex 49 X-coordinate
       \\units m
       \\type real
  N148, \\field Vertex 49 Y-coordinate
       \\units m
       \\type real
  N149, \\field Vertex 49 Z-coordinate
       \\units m
       \\type real
  N150, \\field Vertex 50 X-coordinate
       \\units m
       \\type real
  N151, \\field Vertex 50 Y-coordinate
       \\units m
       \\type real
  N152, \\field Vertex 50 Z-coordinate
       \\units m
       \\type real
  N153, \\field Vertex 51 X-coordinate
       \\units m
       \\type real
  N154, \\field Vertex 51 Y-coordinate
       \\units m
       \\type real
  N155, \\field Vertex 51 Z-coordinate
       \\units m
       \\type real
  N156, \\field Vertex 52 X-coordinate
       \\units m
       \\type real
  N157, \\field Vertex 52 Y-coordinate
       \\units m
       \\type real
  N158, \\field Vertex 52 Z-coordinate
       \\units m
       \\type real
  N159, \\field Vertex 53 X-coordinate
       \\units m
       \\type real
  N160, \\field Vertex 53 Y-coordinate
       \\units m
       \\type real
  N161, \\field Vertex 53 Z-coordinate
       \\units m
       \\type real
  N162, \\field Vertex 54 X-coordinate
       \\units m
       \\type real
  N163, \\field Vertex 54 Y-coordinate
       \\units m
       \\type real
  N164, \\field Vertex 54 Z-coordinate
       \\units m
       \\type real
  N165, \\field Vertex 55 X-coordinate
       \\units m
       \\type real
  N166, \\field Vertex 55 Y-coordinate
       \\units m
       \\type real
  N167, \\field Vertex 55 Z-coordinate
       \\units m
       \\type real
  N168, \\field Vertex 56 X-coordinate
       \\units m
       \\type real
  N169, \\field Vertex 56 Y-coordinate
       \\units m
       \\type real
  N170, \\field Vertex 56 Z-coordinate
       \\units m
       \\type real
  N171, \\field Vertex 57 X-coordinate
       \\units m
       \\type real
  N172, \\field Vertex 57 Y-coordinate
       \\units m
       \\type real
  N173, \\field Vertex 57 Z-coordinate
       \\units m
       \\type real
  N174, \\field Vertex 58 X-coordinate
       \\units m
       \\type real
  N175, \\field Vertex 58 Y-coordinate
       \\units m
       \\type real
  N176, \\field Vertex 58 Z-coordinate
       \\units m
       \\type real
  N177, \\field Vertex 59 X-coordinate
       \\units m
       \\type real
  N178, \\field Vertex 59 Y-coordinate
       \\units m
       \\type real
  N179, \\field Vertex 59 Z-coordinate
       \\units m
       \\type real
  N180, \\field Vertex 60 X-coordinate
       \\units m
       \\type real
  N181, \\field Vertex 60 Y-coordinate
       \\units m
       \\type real
  N182, \\field Vertex 60 Z-coordinate
       \\units m
       \\type real
  N183, \\field Vertex 61 X-coordinate
       \\units m
       \\type real
  N184, \\field Vertex 61 Y-coordinate
       \\units m
       \\type real
  N185, \\field Vertex 61 Z-coordinate
       \\units m
       \\type real
  N186, \\field Vertex 62 X-coordinate
       \\units m
       \\type real
  N187, \\field Vertex 62 Y-coordinate
       \\units m
       \\type real
  N188, \\field Vertex 62 Z-coordinate
       \\units m
       \\type real
  N189, \\field Vertex 63 X-coordinate
       \\units m
       \\type real
  N190, \\field Vertex 63 Y-coordinate
       \\units m
       \\type real
  N191, \\field Vertex 63 Z-coordinate
       \\units m
       \\type real
  N192, \\field Vertex 64 X-coordinate
       \\units m
       \\type real
  N193, \\field Vertex 64 Y-coordinate
       \\units m
       \\type real
  N194, \\field Vertex 64 Z-coordinate
       \\units m
       \\type real
  N195, \\field Vertex 65 X-coordinate
       \\units m
       \\type real
  N196, \\field Vertex 65 Y-coordinate
       \\units m
       \\type real
  N197, \\field Vertex 65 Z-coordinate
       \\units m
       \\type real
  N198, \\field Vertex 66 X-coordinate
       \\units m
       \\type real
  N199, \\field Vertex 66 Y-coordinate
       \\units m
       \\type real
  N200, \\field Vertex 66 Z-coordinate
       \\units m
       \\type real
  N201, \\field Vertex 67 X-coordinate
       \\units m
       \\type real
  N202, \\field Vertex 67 Y-coordinate
       \\units m
       \\type real
  N203, \\field Vertex 67 Z-coordinate
       \\units m
       \\type real
  N204, \\field Vertex 68 X-coordinate
       \\units m
       \\type real
  N205, \\field Vertex 68 Y-coordinate
       \\units m
       \\type real
  N206, \\field Vertex 68 Z-coordinate
       \\units m
       \\type real
  N207, \\field Vertex 69 X-coordinate
       \\units m
       \\type real
  N208, \\field Vertex 69 Y-coordinate
       \\units m
       \\type real
  N209, \\field Vertex 69 Z-coordinate
       \\units m
       \\type real
  N210, \\field Vertex 70 X-coordinate
       \\units m
       \\type real
  N211, \\field Vertex 70 Y-coordinate
       \\units m
       \\type real
  N212, \\field Vertex 70 Z-coordinate
       \\units m
       \\type real
  N213, \\field Vertex 71 X-coordinate
       \\units m
       \\type real
  N214, \\field Vertex 71 Y-coordinate
       \\units m
       \\type real
  N215, \\field Vertex 71 Z-coordinate
       \\units m
       \\type real
  N216, \\field Vertex 72 X-coordinate
       \\units m
       \\type real
  N217, \\field Vertex 72 Y-coordinate
       \\units m
       \\type real
  N218, \\field Vertex 72 Z-coordinate
       \\units m
       \\type real
  N219, \\field Vertex 73 X-coordinate
       \\units m
       \\type real
  N220, \\field Vertex 73 Y-coordinate
       \\units m
       \\type real
  N221, \\field Vertex 73 Z-coordinate
       \\units m
       \\type real
  N222, \\field Vertex 74 X-coordinate
       \\units m
       \\type real
  N223, \\field Vertex 74 Y-coordinate
       \\units m
       \\type real
  N224, \\field Vertex 74 Z-coordinate
       \\units m
       \\type real
  N225, \\field Vertex 75 X-coordinate
       \\units m
       \\type real
  N226, \\field Vertex 75 Y-coordinate
       \\units m
       \\type real
  N227, \\field Vertex 75 Z-coordinate
       \\units m
       \\type real
  N228, \\field Vertex 76 X-coordinate
       \\units m
       \\type real
  N229, \\field Vertex 76 Y-coordinate
       \\units m
       \\type real
  N230, \\field Vertex 76 Z-coordinate
       \\units m
       \\type real
  N231, \\field Vertex 77 X-coordinate
       \\units m
       \\type real
  N232, \\field Vertex 77 Y-coordinate
       \\units m
       \\type real
  N233, \\field Vertex 77 Z-coordinate
       \\units m
       \\type real
  N234, \\field Vertex 78 X-coordinate
       \\units m
       \\type real
  N235, \\field Vertex 78 Y-coordinate
       \\units m
       \\type real
  N236, \\field Vertex 78 Z-coordinate
       \\units m
       \\type real
  N237, \\field Vertex 79 X-coordinate
       \\units m
       \\type real
  N238, \\field Vertex 79 Y-coordinate
       \\units m
       \\type real
  N239, \\field Vertex 79 Z-coordinate
       \\units m
       \\type real
  N240, \\field Vertex 80 X-coordinate
       \\units m
       \\type real
  N241, \\field Vertex 80 Y-coordinate
       \\units m
       \\type real
  N242, \\field Vertex 80 Z-coordinate
       \\units m
       \\type real
  N243, \\field Vertex 81 X-coordinate
       \\units m
       \\type real
  N244, \\field Vertex 81 Y-coordinate
       \\units m
       \\type real
  N245, \\field Vertex 81 Z-coordinate
       \\units m
       \\type real
  N246, \\field Vertex 82 X-coordinate
       \\units m
       \\type real
  N247, \\field Vertex 82 Y-coordinate
       \\units m
       \\type real
  N248, \\field Vertex 82 Z-coordinate
       \\units m
       \\type real
  N249, \\field Vertex 83 X-coordinate
       \\units m
       \\type real
  N250, \\field Vertex 83 Y-coordinate
       \\units m
       \\type real
  N251, \\field Vertex 83 Z-coordinate
       \\units m
       \\type real
  N252, \\field Vertex 84 X-coordinate
       \\units m
       \\type real
  N253, \\field Vertex 84 Y-coordinate
       \\units m
       \\type real
  N254, \\field Vertex 84 Z-coordinate
       \\units m
       \\type real
  N255, \\field Vertex 85 X-coordinate
       \\units m
       \\type real
  N256, \\field Vertex 85 Y-coordinate
       \\units m
       \\type real
  N257, \\field Vertex 85 Z-coordinate
       \\units m
       \\type real
  N258, \\field Vertex 86 X-coordinate
       \\units m
       \\type real
  N259, \\field Vertex 86 Y-coordinate
       \\units m
       \\type real
  N260, \\field Vertex 86 Z-coordinate
       \\units m
       \\type real
  N261, \\field Vertex 87 X-coordinate
       \\units m
       \\type real
  N262, \\field Vertex 87 Y-coordinate
       \\units m
       \\type real
  N263, \\field Vertex 87 Z-coordinate
       \\units m
       \\type real
  N264, \\field Vertex 88 X-coordinate
       \\units m
       \\type real
  N265, \\field Vertex 88 Y-coordinate
       \\units m
       \\type real
  N266, \\field Vertex 88 Z-coordinate
       \\units m
       \\type real
  N267, \\field Vertex 89 X-coordinate
       \\units m
       \\type real
  N268, \\field Vertex 89 Y-coordinate
       \\units m
       \\type real
  N269, \\field Vertex 89 Z-coordinate
       \\units m
       \\type real
  N270, \\field Vertex 90 X-coordinate
       \\units m
       \\type real
  N271, \\field Vertex 90 Y-coordinate
       \\units m
       \\type real
  N272, \\field Vertex 90 Z-coordinate
       \\units m
       \\type real
  N273, \\field Vertex 91 X-coordinate
       \\units m
       \\type real
  N274, \\field Vertex 91 Y-coordinate
       \\units m
       \\type real
  N275, \\field Vertex 91 Z-coordinate
       \\units m
       \\type real
  N276, \\field Vertex 92 X-coordinate
       \\units m
       \\type real
  N277, \\field Vertex 92 Y-coordinate
       \\units m
       \\type real
  N278, \\field Vertex 92 Z-coordinate
       \\units m
       \\type real
  N279, \\field Vertex 93 X-coordinate
       \\units m
       \\type real
  N280, \\field Vertex 93 Y-coordinate
       \\units m
       \\type real
  N281, \\field Vertex 93 Z-coordinate
       \\units m
       \\type real
  N282, \\field Vertex 94 X-coordinate
       \\units m
       \\type real
  N283, \\field Vertex 94 Y-coordinate
       \\units m
       \\type real
  N284, \\field Vertex 94 Z-coordinate
       \\units m
       \\type real
  N285, \\field Vertex 95 X-coordinate
       \\units m
       \\type real
  N286, \\field Vertex 95 Y-coordinate
       \\units m
       \\type real
  N287, \\field Vertex 95 Z-coordinate
       \\units m
       \\type real
  N288, \\field Vertex 96 X-coordinate
       \\units m
       \\type real
  N289, \\field Vertex 96 Y-coordinate
       \\units m
       \\type real
  N290, \\field Vertex 96 Z-coordinate
       \\units m
       \\type real
  N291, \\field Vertex 97 X-coordinate
       \\units m
       \\type real
  N292, \\field Vertex 97 Y-coordinate
       \\units m
       \\type real
  N293, \\field Vertex 97 Z-coordinate
       \\units m
       \\type real
  N294, \\field Vertex 98 X-coordinate
       \\units m
       \\type real
  N295, \\field Vertex 98 Y-coordinate
       \\units m
       \\type real
  N296, \\field Vertex 98 Z-coordinate
       \\units m
       \\type real
  N297, \\field Vertex 99 X-coordinate
       \\units m
       \\type real
  N298, \\field Vertex 99 Y-coordinate
       \\units m
       \\type real
  N299, \\field Vertex 99 Z-coordinate
       \\units m
       \\type real
  N300, \\field Vertex 100 X-coordinate
       \\units m
       \\type real
  N301, \\field Vertex 100 Y-coordinate
       \\units m
       \\type real
  N302, \\field Vertex 100 Z-coordinate
       \\units m
       \\type real
  N303, \\field Vertex 101 X-coordinate
       \\units m
       \\type real
  N304, \\field Vertex 101 Y-coordinate
       \\units m
       \\type real
  N305, \\field Vertex 101 Z-coordinate
       \\units m
       \\type real
  N306, \\field Vertex 102 X-coordinate
       \\units m
       \\type real
  N307, \\field Vertex 102 Y-coordinate
       \\units m
       \\type real
  N308, \\field Vertex 102 Z-coordinate
       \\units m
       \\type real
  N309, \\field Vertex 103 X-coordinate
       \\units m
       \\type real
  N310, \\field Vertex 103 Y-coordinate
       \\units m
       \\type real
  N311, \\field Vertex 103 Z-coordinate
       \\units m
       \\type real
  N312, \\field Vertex 104 X-coordinate
       \\units m
       \\type real
  N313, \\field Vertex 104 Y-coordinate
       \\units m
       \\type real
  N314, \\field Vertex 104 Z-coordinate
       \\units m
       \\type real
  N315, \\field Vertex 105 X-coordinate
       \\units m
       \\type real
  N316, \\field Vertex 105 Y-coordinate
       \\units m
       \\type real
  N317, \\field Vertex 105 Z-coordinate
       \\units m
       \\type real
  N318, \\field Vertex 106 X-coordinate
       \\units m
       \\type real
  N319, \\field Vertex 106 Y-coordinate
       \\units m
       \\type real
  N320, \\field Vertex 106 Z-coordinate
       \\units m
       \\type real
  N321, \\field Vertex 107 X-coordinate
       \\units m
       \\type real
  N322, \\field Vertex 107 Y-coordinate
       \\units m
       \\type real
  N323, \\field Vertex 107 Z-coordinate
       \\units m
       \\type real
  N324, \\field Vertex 108 X-coordinate
       \\units m
       \\type real
  N325, \\field Vertex 108 Y-coordinate
       \\units m
       \\type real
  N326, \\field Vertex 108 Z-coordinate
       \\units m
       \\type real
  N327, \\field Vertex 109 X-coordinate
       \\units m
       \\type real
  N328, \\field Vertex 109 Y-coordinate
       \\units m
       \\type real
  N329, \\field Vertex 109 Z-coordinate
       \\units m
       \\type real
  N330, \\field Vertex 110 X-coordinate
       \\units m
       \\type real
  N331, \\field Vertex 110 Y-coordinate
       \\units m
       \\type real
  N332, \\field Vertex 110 Z-coordinate
       \\units m
       \\type real
  N333, \\field Vertex 111 X-coordinate
       \\units m
       \\type real
  N334, \\field Vertex 111 Y-coordinate
       \\units m
       \\type real
  N335, \\field Vertex 111 Z-coordinate
       \\units m
       \\type real
  N336, \\field Vertex 112 X-coordinate
       \\units m
       \\type real
  N337, \\field Vertex 112 Y-coordinate
       \\units m
       \\type real
  N338, \\field Vertex 112 Z-coordinate
       \\units m
       \\type real
  N339, \\field Vertex 113 X-coordinate
       \\units m
       \\type real
  N340, \\field Vertex 113 Y-coordinate
       \\units m
       \\type real
  N341, \\field Vertex 113 Z-coordinate
       \\units m
       \\type real
  N342, \\field Vertex 114 X-coordinate
       \\units m
       \\type real
  N343, \\field Vertex 114 Y-coordinate
       \\units m
       \\type real
  N344, \\field Vertex 114 Z-coordinate
       \\units m
       \\type real
  N345, \\field Vertex 115 X-coordinate
       \\units m
       \\type real
  N346, \\field Vertex 115 Y-coordinate
       \\units m
       \\type real
  N347, \\field Vertex 115 Z-coordinate
       \\units m
       \\type real
  N348, \\field Vertex 116 X-coordinate
       \\units m
       \\type real
  N349, \\field Vertex 116 Y-coordinate
       \\units m
       \\type real
  N350, \\field Vertex 116 Z-coordinate
       \\units m
       \\type real
  N351, \\field Vertex 117 X-coordinate
       \\units m
       \\type real
  N352, \\field Vertex 117 Y-coordinate
       \\units m
       \\type real
  N353, \\field Vertex 117 Z-coordinate
       \\units m
       \\type real
  N354, \\field Vertex 118 X-coordinate
       \\units m
       \\type real
  N355, \\field Vertex 118 Y-coordinate
       \\units m
       \\type real
  N356, \\field Vertex 118 Z-coordinate
       \\units m
       \\type real
  N357, \\field Vertex 119 X-coordinate
       \\units m
       \\type real
  N358, \\field Vertex 119 Y-coordinate
       \\units m
       \\type real
  N359, \\field Vertex 119 Z-coordinate
       \\units m
       \\type real
  N360, \\field Vertex 120 X-coordinate
       \\units m
       \\type real
  N361, \\field Vertex 120 Y-coordinate
       \\units m
       \\type real
  N362; \\field Vertex 120 Z-coordinate
       \\units m
       \\type real

Wall:Detailed,
  \\memo Allows for detailed entry of wall heat transfer surfaces.
  \\extensible:3 -- duplicate last set of x,y,z coordinates (last 3 fields), remembering to remove ; from "inner" fields.
  \\format vertices
  \\min-fields 18
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition
       \\required-field
       \\type choice
       \\key Adiabatic
       \\key Surface
       \\key Zone
       \\key Outdoors
       \\key Ground
       \\key GroundFCfactorMethod
       \\key OtherSideCoefficients
       \\key OtherSideConditionsModel
       \\key GroundSlabPreprocessorAverage
       \\key GroundSlabPreprocessorCore
       \\key GroundSlabPreprocessorPerimeter
       \\key GroundBasementPreprocessorAverageWall
       \\key GroundBasementPreprocessorAverageFloor
       \\key GroundBasementPreprocessorUpperWall
       \\key GroundBasementPreprocessorLowerWall
  A5,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Non-blank only if the field Outside Boundary Condition is Surface,
       \\note Zone, OtherSideCoefficients or OtherSideConditionsModel
       \\note If Surface, specify name of corresponding surface in adjacent zone or
       \\note specify current surface name for internal partition separating like zones
       \\note If Zone, specify the name of the corresponding zone and
       \\note the program will generate the corresponding interzone surface
       \\note If OtherSideCoefficients, specify name of SurfaceProperty:OtherSideCoefficients
       \\note If OtherSideConditionsModel, specify name of SurfaceProperty:OtherSideConditionsModel
  A6 , \\field Sun Exposure
       \\required-field
       \\type choice
       \\key SunExposed
       \\key NoSun
       \\default SunExposed
  A7,  \\field Wind Exposure
       \\required-field
       \\type choice
       \\key WindExposed
       \\key NoWind
       \\default WindExposed
  N1,  \\field View Factor to Ground
       \\type real
       \\note From the exterior of the surface
       \\note Unused if one uses the "reflections" options in Solar Distribution in Building input
       \\note unless a DaylightingDevice:Shelf or DaylightingDevice:Tubular object has been specified.
       \\note autocalculate will automatically calculate this value from the tilt of the surface
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default autocalculate
  N2 , \\field Number of Vertices
       \\note shown with 10 vertex coordinates -- extensible object
       \\note  "extensible" -- duplicate last set of x,y,z coordinates, renumbering please
       \\note (and changing z terminator to a comma "," for all but last one which needs a semi-colon ";")
       \\autocalculatable
       \\minimum 3
       \\default autocalculate
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  If world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N3,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N5 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N7,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N8,  \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N12, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N13, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N14, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Y-coordinate
       \\type real
       \\units m
  N17, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Y-coordinate
       \\type real
       \\units m
  N20, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Y-coordinate
       \\type real
       \\units m
  N23, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Y-coordinate
       \\type real
       \\units m
  N26, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Y-coordinate
       \\type real
       \\units m
  N29, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Y-coordinate
       \\type real
       \\units m
  N32; \\field Vertex 10 Z-coordinate
       \\units m
       \\type real

RoofCeiling:Detailed,
  \\memo Allows for detailed entry of roof/ceiling heat transfer surfaces.
  \\extensible:3 -- duplicate last set of x,y,z coordinates (last 3 fields), remembering to remove ; from "inner" fields.
  \\format vertices
  \\min-fields 18
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition
       \\required-field
       \\type choice
       \\key Adiabatic
       \\key Surface
       \\key Zone
       \\key Outdoors
       \\key Ground
       \\key OtherSideCoefficients
       \\key OtherSideConditionsModel
       \\key GroundSlabPreprocessorAverage
       \\key GroundSlabPreprocessorCore
       \\key GroundSlabPreprocessorPerimeter
       \\key GroundBasementPreprocessorAverageWall
       \\key GroundBasementPreprocessorAverageFloor
       \\key GroundBasementPreprocessorUpperWall
       \\key GroundBasementPreprocessorLowerWall
 A5,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Non-blank only if the field Outside Boundary Condition is Surface,
       \\note Zone, OtherSideCoefficients or OtherSideConditionsModel
       \\note If Surface, specify name of corresponding surface in adjacent zone or
       \\note specify current surface name for internal partition separating like zones
       \\note If Zone, specify the name of the corresponding zone and
       \\note the program will generate the corresponding interzone surface
       \\note If OtherSideCoefficients, specify name of SurfaceProperty:OtherSideCoefficients
       \\note If OtherSideConditionsModel, specify name of SurfaceProperty:OtherSideConditionsModel
  A6 , \\field Sun Exposure
       \\required-field
       \\type choice
       \\key SunExposed
       \\key NoSun
       \\default SunExposed
  A7,  \\field Wind Exposure
       \\required-field
       \\type choice
       \\key WindExposed
       \\key NoWind
       \\default WindExposed
  N1,  \\field View Factor to Ground
       \\type real
       \\note From the exterior of the surface
       \\note Unused if one uses the "reflections" options in Solar Distribution in Building input
       \\note unless a DaylightingDevice:Shelf or DaylightingDevice:Tubular object has been specified.
       \\note autocalculate will automatically calculate this value from the tilt of the surface
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default autocalculate
  N2 , \\field Number of Vertices
       \\note shown with 10 vertex coordinates -- extensible object
       \\note  "extensible" -- duplicate last set of x,y,z coordinates, renumbering please
       \\note (and changing z terminator to a comma "," for all but last one which needs a semi-colon ";")
       \\autocalculatable
       \\minimum 3
       \\default autocalculate
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  If world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N3,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N5 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N7,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N8,  \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N12, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N13, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N14, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Y-coordinate
       \\type real
       \\units m
  N17, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Y-coordinate
       \\type real
       \\units m
  N20, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Y-coordinate
       \\type real
       \\units m
  N23, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Y-coordinate
       \\type real
       \\units m
  N26, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Y-coordinate
       \\type real
       \\units m
  N29, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Y-coordinate
       \\type real
       \\units m
  N32; \\field Vertex 10 Z-coordinate
       \\units m
       \\type real

Floor:Detailed,
  \\memo Allows for detailed entry of floor heat transfer surfaces.
  \\extensible:3 -- duplicate last set of x,y,z coordinates (last 3 fields), remembering to remove ; from "inner" fields.
  \\format vertices
  \\min-fields 18
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition
       \\required-field
       \\type choice
       \\key Adiabatic
       \\key Surface
       \\key Zone
       \\key Outdoors
       \\key Ground
       \\key GroundFCfactorMethod
       \\key OtherSideCoefficients
       \\key OtherSideConditionsModel
       \\key GroundSlabPreprocessorAverage
       \\key GroundSlabPreprocessorCore
       \\key GroundSlabPreprocessorPerimeter
       \\key GroundBasementPreprocessorAverageWall
       \\key GroundBasementPreprocessorAverageFloor
       \\key GroundBasementPreprocessorUpperWall
       \\key GroundBasementPreprocessorLowerWall
 A5,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Non-blank only if the field Outside Boundary Condition is Surface,
       \\note Zone, OtherSideCoefficients or OtherSideConditionsModel
       \\note If Surface, specify name of corresponding surface in adjacent zone or
       \\note specify current surface name for internal partition separating like zones
       \\note If Zone, specify the name of the corresponding zone and
       \\note the program will generate the corresponding interzone surface
       \\note If OtherSideCoefficients, specify name of SurfaceProperty:OtherSideCoefficients
       \\note If OtherSideConditionsModel, specify name of SurfaceProperty:OtherSideConditionsModel
  A6 , \\field Sun Exposure
       \\required-field
       \\type choice
       \\key SunExposed
       \\key NoSun
       \\default SunExposed
  A7,  \\field Wind Exposure
       \\required-field
       \\type choice
       \\key WindExposed
       \\key NoWind
       \\default WindExposed
  N1,  \\field View Factor to Ground
       \\type real
       \\note From the exterior of the surface
       \\note Unused if one uses the "reflections" options in Solar Distribution in Building input
       \\note unless a DaylightingDevice:Shelf or DaylightingDevice:Tubular object has been specified.
       \\note autocalculate will automatically calculate this value from the tilt of the surface
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default autocalculate
  N2 , \\field Number of Vertices
       \\note shown with 10 vertex coordinates -- extensible object
       \\note  "extensible" -- duplicate last set of x,y,z coordinates, renumbering please
       \\note (and changing z terminator to a comma "," for all but last one which needs a semi-colon ";")
       \\autocalculatable
       \\minimum 3
       \\default autocalculate
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  If world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N3,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N5 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N7,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N8,  \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N12, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N13, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N14, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Y-coordinate
       \\type real
       \\units m
  N17, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Y-coordinate
       \\type real
       \\units m
  N20, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Y-coordinate
       \\type real
       \\units m
  N23, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Y-coordinate
       \\type real
       \\units m
  N26, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Y-coordinate
       \\type real
       \\units m
  N29, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Y-coordinate
       \\type real
       \\units m
  N32; \\field Vertex 10 Z-coordinate
       \\units m
       \\type real

Wall:Exterior,
       \\memo Allows for simplified entry of exterior walls.
       \\memo View Factor to Ground is automatically calculated.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of wall (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Walls are usually tilted 90 degrees
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting (x,y,z) coordinate is the Lower Left Corner of the Wall
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Wall:Adiabatic,
       \\memo Allows for simplified entry of interior walls.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of wall (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Walls are usually tilted 90 degrees
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting (x,y,z) coordinate is the Lower Left Corner of the Wall
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Wall:Underground,
       \\memo Allows for simplified entry of underground walls.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file.
       \\note If the construction is type "Construction:CfactorUndergroundWall",
       \\note then the GroundFCfactorMethod will be used.
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of wall (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Walls are usually tilted 90 degrees
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting (x,y,z) coordinate is the Lower Left Corner of the Wall
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Wall:Interzone,
       \\memo Allows for simplified entry of interzone walls (walls between zones).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone for the inside of the surface
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition Object
       \\required-field
       \\note Specify a surface name in an adjacent zone for known interior walls.
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior wall in the adjacent zone.
       \\type object-list
       \\object-list OutFaceEnvNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of wall (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Walls are usually tilted 90 degrees
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting (x,y,z) coordinate is the Lower Left Corner of the Wall
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Roof,
       \\memo Allows for simplified entry of roofs (exterior).
       \\memo View Factor to Ground is automatically calculated.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of Roof
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Flat Roofs are tilted 0 degrees
       \\minimum 0
       \\maximum 180
       \\default 0
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note If not Flat, Starting coordinate is the Lower Left Corner of the Roof
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

Ceiling:Adiabatic,
       \\memo Allows for simplified entry of interior ceilings.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of Ceiling
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Ceilings are usually tilted 0 degrees
       \\minimum 0
       \\maximum 180
       \\default 0
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note If not Flat, Starting coordinate is the Lower Left Corner of the Ceiling
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

Ceiling:Interzone,
       \\memo Allows for simplified entry of ceilings using adjacent zone
       \\memo (interzone) heat transfer - adjacent surface should be a floor
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone for the inside of the surface
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition Object
       \\required-field
       \\note Specify a surface name in an adjacent zone for known interior floors
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior floor in the adjacent zone.
       \\type object-list
       \\object-list OutFaceEnvNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of wall (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Ceilings are usually tilted 0 degrees
       \\minimum 0
       \\maximum 180
       \\default 0
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note If not Flat, should be Lower Left Corner (from outside)
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

Floor:GroundContact,
       \\memo Allows for simplified entry of exterior floors with ground contact.
       \\memo View Factors to Ground is automatically calculated.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\note If the construction is type "Construction:FfactorGroundFloor",
       \\note then the GroundFCfactorMethod will be used.
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\units deg
       \\minimum 0
       \\maximum 360
  N2,  \\field Tilt Angle
       \\note Floors are usually tilted 180 degrees
       \\units deg
       \\minimum 0
       \\maximum 180
       \\default 180
  N3,  \\field Starting X Coordinate
       \\note if not flat, should be lower left corner (from outside)
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

Floor:Adiabatic,
       \\memo Allows for simplified entry of exterior floors
       \\memo ignoring ground contact or interior floors.
       \\memo View Factor to Ground is automatically calculated.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\type object-list
       \\object-list ZoneNames
  N1,  \\field Azimuth Angle
       \\units deg
       \\minimum 0
       \\maximum 360
  N2,  \\field Tilt Angle
       \\note Floors are usually tilted 180 degrees
       \\units deg
       \\minimum 0
       \\maximum 180
       \\default 180
  N3,  \\field Starting X Coordinate
       \\note if not flat, should be lower left corner (from outside)
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

Floor:Interzone,
       \\memo Allows for simplified entry of floors using adjacent zone
       \\memo (interzone) heat transfer - adjacent surface should be a ceiling.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SurfaceNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference HeatTranBaseSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone for the inside of the surface
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Outside Boundary Condition Object
       \\required-field
       \\note Specify a surface name in an adjacent zone for known interior ceilings.
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior ceiling in the adjacent zone.
       \\type object-list
       \\object-list OutFaceEnvNames
  N1,  \\field Azimuth Angle
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\note Floors are usually tilted 180 degrees
       \\minimum 0
       \\maximum 180
       \\default 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note If not Flat, should be Lower Left Corner (from outside)
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\note Along X Axis
       \\units m
  N7;  \\field Width
       \\note Along Y Axis
       \\units m

FenestrationSurface:Detailed,
       \\memo Allows for detailed entry of subsurfaces
       \\memo (windows, doors, glass doors, tubular daylighting devices).
       \\min-fields 19
       \\format vertices
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Surface Type
       \\required-field
       \\type choice
       \\key Window
       \\key Door
       \\key GlassDoor
       \\key TubularDaylightDome
       \\key TubularDaylightDiffuser
  A3 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A4 , \\field Building Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A5,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Non-blank only if base surface field Outside Boundary Condition is
       \\note Surface or OtherSideCoefficients
       \\note If Base Surface's Surface, specify name of corresponding subsurface in adjacent zone or
       \\note specify current subsurface name for internal partition separating like zones
       \\note If OtherSideCoefficients, specify name of SurfaceProperty:OtherSideCoefficients
       \\note  or leave blank to inherit Base Surface's OtherSide Coefficients
  N1, \\field View Factor to Ground
       \\type real
       \\note From the exterior of the surface
       \\note Unused if one uses the "reflections" options in Solar Distribution in Building input
       \\note unless a DaylightingDevice:Shelf or DaylightingDevice:Tubular object has been specified.
       \\note autocalculate will automatically calculate this value from the tilt of the surface
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default autocalculate
  A6, \\field Shading Control Name
       \\note enter the name of a WindowProperty:ShadingControl object
       \\type object-list
       \\object-list WindowShadeControlNames
       \\note used for windows and glass doors only
       \\note If not specified, window or glass door has no shading (blind, roller shade, etc.)
  A7, \\field Frame and Divider Name
       \\note Enter the name of a WindowProperty:FrameAndDivider object
       \\type object-list
       \\object-list WindowFrameAndDividerNames
       \\note Used only for exterior windows (rectangular) and glass doors.
       \\note Unused for triangular windows.
       \\note If not specified (blank), window or glass door has no frame or divider
       \\note and no beam solar reflection from reveal surfaces.
  N2 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N3 , \\field Number of Vertices
       \\minimum 3
       \\maximum 4
       \\autocalculatable
       \\default autocalculate
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  If world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N4,  \\field Vertex 1 X-coordinate
       \\required-field
       \\units m
       \\type real
  N5 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N6 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N7,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N8,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N10,  \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N12, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N13, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
       \\note Not used for triangles
  N14, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
       \\note Not used for triangles
  N15; \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
       \\note Not used for triangles

Window,
       \\memo Allows for simplified entry of Windows.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Window is on (i.e., Base Surface)
       \\note Window assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4, \\field Shading Control Name
       \\note enter the name of a WindowProperty:ShadingControl object
       \\type object-list
       \\object-list WindowShadeControlNames
       \\note used for windows and glass doors only
       \\note If not specified, window or glass door has no shading (blind, roller shade, etc.)
  A5, \\field Frame and Divider Name
       \\note Enter the name of a WindowProperty:FrameAndDivider object
       \\type object-list
       \\object-list WindowFrameAndDividerNames
       \\note Used only for exterior windows (rectangular) and glass doors.
       \\note Unused for triangular windows.
       \\note If not specified (blank), window or glass door has no frame or divider
       \\note and no beam solar reflection from reveal surfaces.
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Window starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Window starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

Door,
       \\memo Allows for simplified entry of opaque Doors.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Door is on (i.e., Base Surface)
       \\note Door assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Door starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Door starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

GlazedDoor,
       \\memo Allows for simplified entry of glass Doors.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Door is on (i.e., Base Surface)
       \\note Door assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4, \\field Shading Control Name
       \\note enter the name of a WindowProperty:ShadingControl object
       \\type object-list
       \\object-list WindowShadeControlNames
       \\note used for windows and glass doors only
       \\note If not specified, window or glass door has no shading (blind, roller shade, etc.)
  A5, \\field Frame and Divider Name
       \\note Enter the name of a WindowProperty:FrameAndDivider object
       \\type object-list
       \\object-list WindowFrameAndDividerNames
       \\note Used only for exterior windows (rectangular) and glass doors.
       \\note Unused for triangular windows.
       \\note If not specified (blank), window or glass door has no frame or divider
       \\note and no beam solar reflection from reveal surfaces.
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Door starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Door starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

Window:Interzone,
       \\memo Allows for simplified entry of interzone windows (adjacent to
       \\memo other zones).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Window is on (i.e., Base Surface)
       \\note Window assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Specify a surface name in an adjacent zone for known interior windows.
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior window in the adjacent zone.
       \\note a blank field will set up a Window in an adjacent zone
       \\note (same zone as adjacent to base surface)
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Window starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Window starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

Door:Interzone,
       \\memo Allows for simplified entry of interzone (opaque interior) doors (adjacent to
       \\memo other zones).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Door is on (i.e., Base Surface)
       \\note Door assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Specify a surface name in an adjacent zone for known interior doors.
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior door in the adjacent zone.
       \\note a blank field will set up a Window in an adjacent zone
       \\note (same zone as adjacent to base surface)
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Door starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Door starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

GlazedDoor:Interzone,
       \\memo Allows for simplified entry of interzone (glass interior) doors (adjacent to
       \\memo other zones).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SubSurfNames
       \\reference SurfAndSubSurfNames
       \\reference AllHeatTranSurfNames
       \\reference OutFaceEnvNames
       \\reference AllHeatTranAngFacNames
       \\reference RadGroupAndSurfNames
       \\reference SurfGroupAndHTSurfNames
       \\reference AllShadingAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Building Surface Name
       \\note Name of Surface (Wall, usually) the Door is on (i.e., Base Surface)
       \\note Door assumes the azimuth and tilt angles of the surface it is on.
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4,  \\field Outside Boundary Condition Object
       \\type object-list
       \\object-list OutFaceEnvNames
       \\note Specify a surface name in an adjacent zone for known interior doors.
       \\note Specify a zone name of an adjacent zone to automatically generate
       \\note the interior door in the adjacent zone.
       \\note a blank field will set up a Window in an adjacent zone
       \\note (same zone as adjacent to base surface)
  N1 , \\field Multiplier
       \\note Used only for Surface Type = WINDOW, GLASSDOOR or DOOR
       \\note Non-integer values will be truncated to integer
       \\default 1.0
       \\minimum 1.0
  N2,  \\field Starting X Coordinate
       \\note Door starting coordinate is specified relative to the Base Surface origin.
       \\units m
  N3,  \\field Starting Z Coordinate
       \\note How far up the wall the Door starts. (in 2-d, this would be a Y Coordinate)
       \\units m
  N4,  \\field Length
       \\units m
  N5;  \\field Height
       \\units m

WindowProperty:ShadingControl,
       \\memo Specifies the type, location, and controls for window shades, window blinds, and
       \\memo switchable glazing. Referenced by the surface objects for exterior windows and glass
       \\memo doors (ref: FenestrationSurface:Detailed, Window, and GlazedDoor).
    \\min-fields 11
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WindowShadeControlNames
       \\note Referenced by surfaces that are exterior windows
       \\note Not used by interzone windows
  A2 , \\field Shading Type
       \\required-field
       \\type choice
       \\key InteriorShade
       \\key ExteriorShade
       \\key ExteriorScreen
       \\key InteriorBlind
       \\key ExteriorBlind
       \\key BetweenGlassShade
       \\key BetweenGlassBlind
       \\key SwitchableGlazing
  A3 , \\field Construction with Shading Name
       \\type object-list
       \\object-list ConstructionNames
       \\note Required if Shading Type = SwitchableGlazing
       \\note Required if Shading Type = interior or exterior shade or blind, or exterior screen, and
       \\note "Shading Device Material Name" is not specified.
       \\note If both "Construction with Shading Name" and "Shading Device Material Name" are entered,
       \\note the former takes precedence.
  A4 , \\field Shading Control Type
       \\required-field
       \\type choice
       \\key AlwaysOn
       \\key AlwaysOff
       \\key OnIfScheduleAllows
       \\note OnIfScheduleAllows requires that Schedule Name be specified and
       \\note Shading Control Is Scheduled = Yes.
       \\note AlwaysOn, AlwaysOff and OnIfScheduleAllows are the only valid control types for ExteriorScreen.
       \\note The following six control types are used primarily to reduce
       \\note zone cooling load due to window solar gain
       \\key OnIfHighSolarOnWindow
       \\key OnIfHighHorizontalSolar
       \\key OnIfHighOutdoorAirTemperature
       \\key OnIfHighZoneAirTemperature
       \\key OnIfHighZoneCooling
       \\key OnIfHighGlare
       \\note Following entry should be used only if Shading Type = SwitchableGlazing
       \\note and window is in a daylit zone
       \\key MeetDaylightIlluminanceSetpoint
       \\note The following three control types are used to reduce zone Heating load. They can be
       \\note used with any Shading Type but are most appropriate for opaque interior or exterior
       \\note shades with high insulating value ("opaque movable insulation")
       \\key OnNightIfLowOutdoorTempAndOffDay
       \\key OnNightIfLowInsideTempAndOffDay
       \\key OnNightIfHeatingAndOffDay
       \\note The following two control types are used to reduce zone heating and cooling load.
       \\note They can be used with any Shading Type but are most appropriate for translucent interior
       \\note or exterior shades with high insulating value ("translucent movable insulation")
       \\key OnNightIfLowOutdoorTempAndOnDayIfCooling
       \\key OnNightIfHeatingAndOnDayIfCooling
       \\note The following two control types are used to reduce zone Cooling load.
       \\note They can be used with any Shading Type but are most appropriate for interior
       \\note or exterior blinds,interior or exterior shades with low insulating value, or
       \\note switchable glazing
       \\key OffNightAndOnDayIfCoolingAndHighSolarOnWindow
       \\key OnNightAndOnDayIfCoolingAndHighSolarOnWindow
       \\note The following four control types require that both Setpoint and Setpoint2 be specified
       \\note Setpoint will correspond to outdoor air temp or zone air temp (deg C)
       \\note Setpoint2 will correspond to solar on window or horizontal solar (W/m2)
       \\key OnIfHighOutdoorAirTempAndHighSolarOnWindow
       \\key OnIfHighOutdoorAirTempAndHighHorizontalSolar
       \\key OnIfHighZoneAirTempAndHighSolarOnWindow
       \\key OnIfHighZoneAirTempAndHighHorizontalSolar
  A5 , \\field Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Required if Shading Control Is Scheduled = Yes.
       \\note If schedule value = 1, shading control is active, i.e., shading can take place only
       \\note if the control test passes. If schedule value = 0, shading is off whether or not
       \\note the control test passes. Schedule Name is required if Shading Control Is Scheduled = Yes.
       \\note If Schedule Name is not specified, shading control is assumed to be active at all times.
  N1 , \\field Setpoint
       \\units W/m2, W or deg C
       \\note W/m2 for solar-based controls, W for cooling- or heating-based controls,
       \\note deg C for temperature-based controls.
       \\note Unused for Shading Control Type = AlwaysOn, AlwaysOff, OnIfScheduleAllows,
       \\note OnIfHighGlare, Glare, and DaylightIlluminance
       \\type real
  A6 , \\field Shading Control Is Scheduled
       \\type choice
       \\key No
       \\key Yes
       \\default No
       \\note If Yes, Schedule Name is required; if No, Schedule Name is not used.
       \\note Shading Control Is Scheduled = Yes is required if Shading Control Type = OnIfScheduleAllows.
  A7 , \\field Glare Control Is Active
       \\type choice
       \\key No
       \\key Yes
       \\default No
       \\note If Yes and window is in a daylit zone, shading is on if zone's discomfort glare index exceeds
       \\note the maximum discomfort glare index specified in the Daylighting object referenced by the zone.
       \\note The glare test is OR'ed with the test specified by Shading Control Type.
       \\note Glare Control Is Active = Yes is required if Shading Control Type = OnIfHighGlare.
  A8,  \\field Shading Device Material Name
       \\type object-list
       \\object-list WindowShadesScreensAndBlinds
       \\note Enter then name of a WindowMaterial:Shade, WindowMaterial:Screen or WindowMaterial:Blind object.
       \\note Required if "Construction with Shading Name" is not specified.
       \\note Not used if Shading Control Type = SwitchableGlazing, BetweenGlassShade, or BetweenGlassBlind.
       \\note If both "Construction with Shading Name" and "Shading Device Material Name" are entered,
       \\note the former takes precedence.
  A9 , \\field Type of Slat Angle Control for Blinds
       \\type choice
       \\key FixedSlatAngle
       \\key ScheduledSlatAngle
       \\key BlockBeamSolar
       \\default FixedSlatAngle
       \\note Used only if Shading Type = InteriorBlind, ExteriorBlind or BetweenGlassBlind.
       \\note If choice is ScheduledSlatAngle then Slat Angle Schedule Name is required.
  A10, \\field Slat Angle Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Used only if Shading Type = InteriorBlind, ExteriorBlind or BetweenGlassBlind.
       \\note Required if Type of Slat Angle Control for Blinds = ScheduledSlatAngle
       \\note Schedule values should be degrees (0 minimum, 180 maximum)
  N2 ; \\field Setpoint 2
       \\units W/m2 or deg C
       \\note W/m2 for solar-based controls, deg C for temperature-based controls.
       \\note Used only as the second setpoint for the following two-setpoint control types:
       \\note OnIfHighOutdoorAirTempAndHighSolarOnWindow, OnIfHighOutdoorAirTempAndHighHorizontalSolar,
       \\note OnIfHighZoneAirTempAndHighSolarOnWindow, and OnIfHighZoneAirTempAndHighHorizontalSolar
       \\type real
       \\ip-units unknown

WindowProperty:FrameAndDivider,
         \\memo Specifies the dimensions of a window frame, dividers, and inside reveal surfaces.
         \\memo Referenced by the surface objects for exterior windows and glass doors
         \\memo (ref: FenestrationSurface:Detailed, Window, and GlazedDoor).
      \\min-fields 20
    A1 , \\field Name
         \\required-field
         \\type alpha
         \\reference WindowFrameAndDividerNames
         \\note Referenced by surfaces that are exterior windows
         \\note Not used by interzone windows
    N1 , \\field Frame Width
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0
         \\note Width of frame in plane of window
         \\note Frame width assumed the same on all sides of window
    N2 , \\field Frame Outside Projection
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 0.5
         \\default 0.0
         \\note Amount that frame projects outward from the outside face of the glazing
    N3 , \\field Frame Inside Projection
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 0.5
         \\default 0.0
         \\note Amount that frame projects inward from the inside face of the glazing
    N4 , \\field Frame Conductance
         \\units W/m2-K
         \\type real
         \\minimum 0.0
         \\note Effective conductance of frame
         \\note Excludes air films
         \\note Obtained from WINDOW 5 or other 2-D calculation
    N5 , \\field Ratio of Frame-Edge Glass Conductance to Center-Of-Glass Conductance
         \\type real
         \\minimum 1.0
         \\maximum 4.0
         \\default 1.0
         \\note Excludes air films; applies only to multipane windows
         \\note Obtained from WINDOW 5 or other 2-D calculation
    N6 , \\field Frame Solar Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.7
         \\note Assumed same on outside and inside of frame
    N7 , \\field Frame Visible Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.7
         \\note Assumed same on outside and inside of frame
    N8 , \\field Frame Thermal Hemispherical Emissivity
         \\type real
         \\minimum> 0.0
         \\default 0.9
         \\note Assumed same on outside and inside of frame
    A2 , \\field Divider Type
         \\required-field
         \\type choice
         \\key DividedLite
         \\key Suspended
         \\default DividedLite
    N9 , \\field Divider Width
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 0.5
         \\default 0.0
         \\note Width of dividers in plane of window
         \\note Width assumed the same for all dividers
    N10, \\field Number of Horizontal Dividers
         \\type real
         \\minimum 0
         \\default 0
         \\note "Horizontal" means parallel to local window X-axis
    N11, \\field Number of Vertical Dividers
         \\type real
         \\minimum 0
         \\default 0
         \\note "Vertical" means parallel to local window Y-axis
    N12, \\field Divider Outside Projection
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 0.5
         \\default 0.0
         \\note Amount that divider projects outward from the outside face of the glazing
         \\note Outside projection assumed the same for all divider elements
    N13, \\field Divider Inside Projection
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 0.5
         \\default 0.0
         \\note Amount that divider projects inward from the inside face of the glazing
         \\note Inside projection assumed the same for all divider elements
    N14, \\field Divider Conductance
         \\units W/m2-K
         \\type real
         \\minimum 0.0
         \\default 0.0
         \\note Effective conductance of divider
         \\note Excludes air films
         \\note Obtained from WINDOW 5 or other 2-D calculation
    N15, \\field Ratio of Divider-Edge Glass Conductance to Center-Of-Glass Conductance
         \\type real
         \\minimum 1.0
         \\maximum 4.0
         \\default 1.0
         \\note Excludes air films
         \\note Obtained from WINDOW 5 or other 2-D calculation
    N16, \\field Divider Solar Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0
         \\note Assumed same on outside and inside of divider
    N17, \\field Divider Visible Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0
         \\note Assumed same on outside and inside of divider
    N18, \\field Divider Thermal Hemispherical Emissivity
         \\type real
         \\minimum> 0.0
         \\maximum< 1.0
         \\default 0.9
         \\note Assumed same on outside and inside of divider
    N19, \\field Outside Reveal Solar Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0
    N20, \\field Inside Sill Depth
         \\units m
         \\type real
         \\minimum 0.0
         \\maximum 2.0
         \\default 0.0
    N21, \\field Inside Sill Solar Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0
    N22, \\field Inside Reveal Depth
         \\note Distance from plane of inside surface of glazing
         \\note to plane of inside surface of wall.
         \\note Outside reveal depth is determined from the geometry
         \\note of the window and the wall it is on; it is non-zero if the plane of
         \\note the outside surface of the glazing is set back from the plane of the
         \\note outside surface of the wall.
         \\units m
         \\minimum 0.0
         \\maximum 2.0
         \\default 0.0
    N23; \\field Inside Reveal Solar Absorptance
         \\type real
         \\minimum 0.0
         \\maximum 1.0
         \\default 0.0

WindowProperty:AirflowControl,
    \\memo Used to control forced airflow through a gap between glass layers
    \\min-fields 7
  A1 , \\field Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
       \\note Name must be that of an exterior window with two or three glass layers.
  A2 , \\field Airflow Source
       \\type choice
       \\key IndoorAir
       \\key OutdoorAir
       \\default IndoorAir
  A3 , \\field Airflow Destination
       \\type choice
       \\key IndoorAir
       \\key OutdoorAir
       \\key ReturnAir
       \\default OutdoorAir
  N1 , \\field Maximum Flow Rate
       \\units m3/s-m
       \\note Above is m3/s per m of glazing width
       \\ip-units ft3/min-ft
       \\default 0.0
       \\minimum 0.0
  A4 , \\field Airflow Control Type
       \\type choice
       \\key AlwaysOnAtMaximumFlow
       \\key AlwaysOff
       \\key ScheduledOnly
       \\note ScheduledOnly requires that Airflow Has Multiplier Schedule Name = Yes
       \\note  and that Airflow Multiplier Schedule Name is specified.
       \\default AlwaysOnAtMaximumFlow
  A5 , \\field Airflow Is Scheduled
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note If Yes, then Airflow Multiplier Schedule Name must be specified
  A6 ; \\field Airflow Multiplier Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Required if Airflow Is Scheduled = Yes.
       \\note Schedule values are 0.0 or 1.0 and multiply Maximum Air Flow.

WindowProperty:StormWindow,
       \\min-fields 7
       \\memo This is a movable exterior glass layer that is usually applied in the winter
       \\memo and removed in the summer.
  A1,  \\field Window Name
       \\note Must be the name of a FenestrationSurface:Detailed object with Surface Type = WINDOW.
       \\note The WindowProperty:StormWindow object can only be used with exterior windows.
       \\required-field
       \\type alpha
       \\object-list SubSurfNames
  A2,  \\field Storm Glass Layer Name
       \\note Must be a WindowMaterial:Glazing or WindowMaterial:Glazing:RefractionExtinctionMethod
       \\note Gap between storm glass layer and adjacent glass layer is assumed to be filled
       \\note with Air
       \\required-field
       \\type object-list
       \\object-list GlazingMaterialName
  N1,  \\field Distance Between Storm Glass Layer and Adjacent Glass
       \\type real
       \\units m
       \\minimum> 0.0
       \\maximum 0.5
       \\default 0.050
  N2,  \\field Month that Storm Glass Layer is Put On
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N3,  \\field Day of Month that Storm Glass Layer is Put On
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer
  N4,  \\field Month that Storm Glass Layer is Taken Off
       \\required-field
       \\minimum 1
       \\maximum 12
       \\type integer
  N5;  \\field Day of Month that Storm Glass Layer is Taken Off
       \\required-field
       \\minimum 1
       \\maximum 31
       \\type integer

InternalMass,
       \\memo Used to describe internal zone surface area that does not need to be part of geometric
       \\memo representation. This should be the total surface area exposed to the zone air.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllHeatTranSurfNames
       \\reference AllHeatTranAngFacNames
       \\reference SurfGroupAndHTSurfNames
  A2 , \\field Construction Name
       \\required-field
       \\note To be matched with a construction in this input file
       \\type object-list
       \\object-list ConstructionNames
  A3 , \\field Zone Name
       \\required-field
       \\note Zone the surface is a part of
       \\note used to be Interior Environment
       \\type object-list
       \\object-list ZoneNames
  N1 ; \\field Surface Area
       \\required-field
       \\units m2
       \\minimum> 0

Shading:Site,
       \\memo used for shading elements such as trees
       \\memo these items are fixed in space and would not move with relative geometry
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of shading device (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting coordinate is the Lower Left Corner of the Shade
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Shading:Building,
       \\memo used for shading elements such as trees, other buildings, parts of this building not being modeled
       \\memo these items are relative to the current building and would move with relative geometry
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  N1,  \\field Azimuth Angle
       \\note Facing direction of outside of shading device (S=180,N=0,E=90,W=270)
       \\minimum 0
       \\maximum 360
       \\units deg
  N2,  \\field Tilt Angle
       \\default 90
       \\minimum 0
       \\maximum 180
       \\units deg
  N3,  \\field Starting X Coordinate
       \\note Starting coordinate is the Lower Left Corner of the Shade
       \\units m
  N4,  \\field Starting Y Coordinate
       \\units m
  N5,  \\field Starting Z Coordinate
       \\units m
  N6,  \\field Length
       \\units m
  N7;  \\field Height
       \\units m

Shading:Site:Detailed,
  \\extensible:3 -- duplicate last set of x,y,z coordinates, renumbering please (and changing z terminator to , for all but last)
       \\memo used for shading elements such as trees
       \\memo these items are fixed in space and would not move with relative geometry
       \\format vertices
       \\min-fields 12
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2,  \\field Transmittance Schedule Name
       \\note Transmittance schedule for the shading device, defaults to zero (always opaque)
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Number of Vertices
       \\required-field
       \\note shown with 6 vertex coordinates -- extensible object
       \\autocalculatable
       \\default autocalculate
       \\minimum 3
       \\note Rules for vertices are given in GlobalGeometryRules coordinates --
       \\note For this object all surface coordinates are in world coordinates.
  N2,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N3 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N5,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N7, \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N8, \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N12, \\field Vertex 4 Y-coordinate
       \\units m
       \\type real
  N13, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N14, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 Y-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N17, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 Y-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N20, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 Y-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N23, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 Y-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N26, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 Y-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N29, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 Y-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Z-coordinate
       \\units m
       \\type real
  N32, \\field Vertex 11 X-coordinate
       \\units m
       \\type real
  N33, \\field Vertex 11 Y-coordinate
       \\units m
       \\type real
  N34, \\field Vertex 11 Z-coordinate
       \\units m
       \\type real
  N35, \\field Vertex 12 X-coordinate
       \\units m
       \\type real
  N36, \\field Vertex 12 Y-coordinate
       \\units m
       \\type real
  N37, \\field Vertex 12 Z-coordinate
       \\units m
       \\type real
  N38, \\field Vertex 13 X-coordinate
       \\units m
       \\type real
  N39, \\field Vertex 13 Y-coordinate
       \\units m
       \\type real
  N40, \\field Vertex 13 Z-coordinate
       \\units m
       \\type real
  N41, \\field Vertex 14 X-coordinate
       \\units m
       \\type real
  N42, \\field Vertex 14 Y-coordinate
       \\units m
       \\type real
  N43, \\field Vertex 14 Z-coordinate
       \\units m
       \\type real
  N44, \\field Vertex 15 X-coordinate
       \\units m
       \\type real
  N45, \\field Vertex 15 Y-coordinate
       \\units m
       \\type real
  N46, \\field Vertex 15 Z-coordinate
       \\units m
       \\type real
  N47, \\field Vertex 16 X-coordinate
       \\units m
       \\type real
  N48, \\field Vertex 16 Y-coordinate
       \\units m
       \\type real
  N49, \\field Vertex 16 Z-coordinate
       \\units m
       \\type real
  N50, \\field Vertex 17 X-coordinate
       \\units m
       \\type real
  N51, \\field Vertex 17 Y-coordinate
       \\units m
       \\type real
  N52, \\field Vertex 17 Z-coordinate
       \\units m
       \\type real
  N53, \\field Vertex 18 X-coordinate
       \\units m
       \\type real
  N54, \\field Vertex 18 Y-coordinate
       \\units m
       \\type real
  N55, \\field Vertex 18 Z-coordinate
       \\units m
       \\type real
  N56, \\field Vertex 19 X-coordinate
       \\units m
       \\type real
  N57, \\field Vertex 19 Y-coordinate
       \\units m
       \\type real
  N58, \\field Vertex 19 Z-coordinate
       \\units m
       \\type real
  N59, \\field Vertex 20 X-coordinate
       \\units m
       \\type real
  N60, \\field Vertex 20 Y-coordinate
       \\units m
       \\type real
  N61, \\field Vertex 20 Z-coordinate
       \\units m
       \\type real
  N62, \\field Vertex 21 X-coordinate
       \\units m
       \\type real
  N63, \\field Vertex 21 Y-coordinate
       \\units m
       \\type real
  N64, \\field Vertex 21 Z-coordinate
       \\units m
       \\type real
  N65, \\field Vertex 22 X-coordinate
       \\units m
       \\type real
  N66, \\field Vertex 22 Y-coordinate
       \\units m
       \\type real
  N67, \\field Vertex 22 Z-coordinate
       \\units m
       \\type real
  N68, \\field Vertex 23 X-coordinate
       \\units m
       \\type real
  N69, \\field Vertex 23 Y-coordinate
       \\units m
       \\type real
  N70, \\field Vertex 23 Z-coordinate
       \\units m
       \\type real
  N71, \\field Vertex 24 X-coordinate
       \\units m
       \\type real
  N72, \\field Vertex 24 Y-coordinate
       \\units m
       \\type real
  N73, \\field Vertex 24 Z-coordinate
       \\units m
       \\type real
  N74, \\field Vertex 25 X-coordinate
       \\units m
       \\type real
  N75, \\field Vertex 25 Y-coordinate
       \\units m
       \\type real
  N76, \\field Vertex 25 Z-coordinate
       \\units m
       \\type real
  N77, \\field Vertex 26 X-coordinate
       \\units m
       \\type real
  N78, \\field Vertex 26 Y-coordinate
       \\units m
       \\type real
  N79, \\field Vertex 26 Z-coordinate
       \\units m
       \\type real
  N80, \\field Vertex 27 X-coordinate
       \\units m
       \\type real
  N81, \\field Vertex 27 Y-coordinate
       \\units m
       \\type real
  N82, \\field Vertex 27 Z-coordinate
       \\units m
       \\type real
  N83, \\field Vertex 28 X-coordinate
       \\units m
       \\type real
  N84, \\field Vertex 28 Y-coordinate
       \\units m
       \\type real
  N85, \\field Vertex 28 Z-coordinate
       \\units m
       \\type real
  N86, \\field Vertex 29 X-coordinate
       \\units m
       \\type real
  N87, \\field Vertex 29 Y-coordinate
       \\units m
       \\type real
  N88, \\field Vertex 29 Z-coordinate
       \\units m
       \\type real
  N89, \\field Vertex 30 X-coordinate
       \\units m
       \\type real
  N90, \\field Vertex 30 Y-coordinate
       \\units m
       \\type real
  N91, \\field Vertex 30 Z-coordinate
       \\units m
       \\type real
  N92, \\field Vertex 31 X-coordinate
       \\units m
       \\type real
  N93, \\field Vertex 31 Y-coordinate
       \\units m
       \\type real
  N94, \\field Vertex 31 Z-coordinate
       \\units m
       \\type real
  N95, \\field Vertex 32 X-coordinate
       \\units m
       \\type real
  N96, \\field Vertex 32 Y-coordinate
       \\units m
       \\type real
  N97, \\field Vertex 32 Z-coordinate
       \\units m
       \\type real
  N98, \\field Vertex 33 X-coordinate
       \\units m
       \\type real
  N99, \\field Vertex 33 Y-coordinate
       \\units m
       \\type real
  N100, \\field Vertex 33 Z-coordinate
       \\units m
       \\type real
  N101, \\field Vertex 34 X-coordinate
       \\units m
       \\type real
  N102, \\field Vertex 34 Y-coordinate
       \\units m
       \\type real
  N103, \\field Vertex 34 Z-coordinate
       \\units m
       \\type real
  N104, \\field Vertex 35 X-coordinate
       \\units m
       \\type real
  N105, \\field Vertex 35 Y-coordinate
       \\units m
       \\type real
  N106, \\field Vertex 35 Z-coordinate
       \\units m
       \\type real
  N107, \\field Vertex 36 X-coordinate
       \\units m
       \\type real
  N108, \\field Vertex 36 Y-coordinate
       \\units m
       \\type real
  N109, \\field Vertex 36 Z-coordinate
       \\units m
       \\type real
  N110, \\field Vertex 37 X-coordinate
       \\units m
       \\type real
  N111, \\field Vertex 37 Y-coordinate
       \\units m
       \\type real
  N112, \\field Vertex 37 Z-coordinate
       \\units m
       \\type real
  N113, \\field Vertex 38 X-coordinate
       \\units m
       \\type real
  N114, \\field Vertex 38 Y-coordinate
       \\units m
       \\type real
  N115, \\field Vertex 38 Z-coordinate
       \\units m
       \\type real
  N116, \\field Vertex 39 X-coordinate
       \\units m
       \\type real
  N117, \\field Vertex 39 Y-coordinate
       \\units m
       \\type real
  N118, \\field Vertex 39 Z-coordinate
       \\units m
       \\type real
  N119, \\field Vertex 40 X-coordinate
       \\units m
       \\type real
  N120, \\field Vertex 40 Y-coordinate
       \\units m
       \\type real
  N121, \\field Vertex 40 Z-coordinate
       \\units m
       \\type real
  N122, \\field Vertex 41 X-coordinate
       \\units m
       \\type real
  N123, \\field Vertex 41 Y-coordinate
       \\units m
       \\type real
  N124, \\field Vertex 41 Z-coordinate
       \\units m
       \\type real
  N125, \\field Vertex 42 X-coordinate
       \\units m
       \\type real
  N126, \\field Vertex 42 Y-coordinate
       \\units m
       \\type real
  N127, \\field Vertex 42 Z-coordinate
       \\units m
       \\type real
  N128, \\field Vertex 43 X-coordinate
       \\units m
       \\type real
  N129, \\field Vertex 43 Y-coordinate
       \\units m
       \\type real
  N130, \\field Vertex 43 Z-coordinate
       \\units m
       \\type real
  N131, \\field Vertex 44 X-coordinate
       \\units m
       \\type real
  N132, \\field Vertex 44 Y-coordinate
       \\units m
       \\type real
  N133, \\field Vertex 44 Z-coordinate
       \\units m
       \\type real
  N134, \\field Vertex 45 X-coordinate
       \\units m
       \\type real
  N135, \\field Vertex 45 Y-coordinate
       \\units m
       \\type real
  N136, \\field Vertex 45 Z-coordinate
       \\units m
       \\type real
  N137, \\field Vertex 46 X-coordinate
       \\units m
       \\type real
  N138, \\field Vertex 46 Y-coordinate
       \\units m
       \\type real
  N139, \\field Vertex 46 Z-coordinate
       \\units m
       \\type real
  N140, \\field Vertex 47 X-coordinate
       \\units m
       \\type real
  N141, \\field Vertex 47 Y-coordinate
       \\units m
       \\type real
  N142, \\field Vertex 47 Z-coordinate
       \\units m
       \\type real
  N143, \\field Vertex 48 X-coordinate
       \\units m
       \\type real
  N144, \\field Vertex 48 Y-coordinate
       \\units m
       \\type real
  N145, \\field Vertex 48 Z-coordinate
       \\units m
       \\type real
  N146, \\field Vertex 49 X-coordinate
       \\units m
       \\type real
  N147, \\field Vertex 49 Y-coordinate
       \\units m
       \\type real
  N148, \\field Vertex 49 Z-coordinate
       \\units m
       \\type real
  N149, \\field Vertex 50 X-coordinate
       \\units m
       \\type real
  N150, \\field Vertex 50 Y-coordinate
       \\units m
       \\type real
  N151, \\field Vertex 50 Z-coordinate
       \\units m
       \\type real
  N152, \\field Vertex 51 X-coordinate
       \\units m
       \\type real
  N153, \\field Vertex 51 Y-coordinate
       \\units m
       \\type real
  N154, \\field Vertex 51 Z-coordinate
       \\units m
       \\type real
  N155, \\field Vertex 52 X-coordinate
       \\units m
       \\type real
  N156, \\field Vertex 52 Y-coordinate
       \\units m
       \\type real
  N157, \\field Vertex 52 Z-coordinate
       \\units m
       \\type real
  N158, \\field Vertex 53 X-coordinate
       \\units m
       \\type real
  N159, \\field Vertex 53 Y-coordinate
       \\units m
       \\type real
  N160, \\field Vertex 53 Z-coordinate
       \\units m
       \\type real
  N161, \\field Vertex 54 X-coordinate
       \\units m
       \\type real
  N162, \\field Vertex 54 Y-coordinate
       \\units m
       \\type real
  N163, \\field Vertex 54 Z-coordinate
       \\units m
       \\type real
  N164, \\field Vertex 55 X-coordinate
       \\units m
       \\type real
  N165, \\field Vertex 55 Y-coordinate
       \\units m
       \\type real
  N166, \\field Vertex 55 Z-coordinate
       \\units m
       \\type real
  N167, \\field Vertex 56 X-coordinate
       \\units m
       \\type real
  N168, \\field Vertex 56 Y-coordinate
       \\units m
       \\type real
  N169, \\field Vertex 56 Z-coordinate
       \\units m
       \\type real
  N170, \\field Vertex 57 X-coordinate
       \\units m
       \\type real
  N171, \\field Vertex 57 Y-coordinate
       \\units m
       \\type real
  N172, \\field Vertex 57 Z-coordinate
       \\units m
       \\type real
  N173, \\field Vertex 58 X-coordinate
       \\units m
       \\type real
  N174, \\field Vertex 58 Y-coordinate
       \\units m
       \\type real
  N175, \\field Vertex 58 Z-coordinate
       \\units m
       \\type real
  N176, \\field Vertex 59 X-coordinate
       \\units m
       \\type real
  N177, \\field Vertex 59 Y-coordinate
       \\units m
       \\type real
  N178, \\field Vertex 59 Z-coordinate
       \\units m
       \\type real
  N179, \\field Vertex 60 X-coordinate
       \\units m
       \\type real
  N180, \\field Vertex 60 Y-coordinate
       \\units m
       \\type real
  N181, \\field Vertex 60 Z-coordinate
       \\units m
       \\type real
  N182, \\field Vertex 61 X-coordinate
       \\units m
       \\type real
  N183, \\field Vertex 61 Y-coordinate
       \\units m
       \\type real
  N184, \\field Vertex 61 Z-coordinate
       \\units m
       \\type real
  N185, \\field Vertex 62 X-coordinate
       \\units m
       \\type real
  N186, \\field Vertex 62 Y-coordinate
       \\units m
       \\type real
  N187, \\field Vertex 62 Z-coordinate
       \\units m
       \\type real
  N188, \\field Vertex 63 X-coordinate
       \\units m
       \\type real
  N189, \\field Vertex 63 Y-coordinate
       \\units m
       \\type real
  N190, \\field Vertex 63 Z-coordinate
       \\units m
       \\type real
  N191, \\field Vertex 64 X-coordinate
       \\units m
       \\type real
  N192, \\field Vertex 64 Y-coordinate
       \\units m
       \\type real
  N193, \\field Vertex 64 Z-coordinate
       \\units m
       \\type real
  N194, \\field Vertex 65 X-coordinate
       \\units m
       \\type real
  N195, \\field Vertex 65 Y-coordinate
       \\units m
       \\type real
  N196, \\field Vertex 65 Z-coordinate
       \\units m
       \\type real
  N197, \\field Vertex 66 X-coordinate
       \\units m
       \\type real
  N198, \\field Vertex 66 Y-coordinate
       \\units m
       \\type real
  N199, \\field Vertex 66 Z-coordinate
       \\units m
       \\type real
  N200, \\field Vertex 67 X-coordinate
       \\units m
       \\type real
  N201, \\field Vertex 67 Y-coordinate
       \\units m
       \\type real
  N202, \\field Vertex 67 Z-coordinate
       \\units m
       \\type real
  N203, \\field Vertex 68 X-coordinate
       \\units m
       \\type real
  N204, \\field Vertex 68 Y-coordinate
       \\units m
       \\type real
  N205, \\field Vertex 68 Z-coordinate
       \\units m
       \\type real
  N206, \\field Vertex 69 X-coordinate
       \\units m
       \\type real
  N207, \\field Vertex 69 Y-coordinate
       \\units m
       \\type real
  N208, \\field Vertex 69 Z-coordinate
       \\units m
       \\type real
  N209, \\field Vertex 70 X-coordinate
       \\units m
       \\type real
  N210, \\field Vertex 70 Y-coordinate
       \\units m
       \\type real
  N211, \\field Vertex 70 Z-coordinate
       \\units m
       \\type real
  N212, \\field Vertex 71 X-coordinate
       \\units m
       \\type real
  N213, \\field Vertex 71 Y-coordinate
       \\units m
       \\type real
  N214, \\field Vertex 71 Z-coordinate
       \\units m
       \\type real
  N215, \\field Vertex 72 X-coordinate
       \\units m
       \\type real
  N216, \\field Vertex 72 Y-coordinate
       \\units m
       \\type real
  N217, \\field Vertex 72 Z-coordinate
       \\units m
       \\type real
  N218, \\field Vertex 73 X-coordinate
       \\units m
       \\type real
  N219, \\field Vertex 73 Y-coordinate
       \\units m
       \\type real
  N220, \\field Vertex 73 Z-coordinate
       \\units m
       \\type real
  N221, \\field Vertex 74 X-coordinate
       \\units m
       \\type real
  N222, \\field Vertex 74 Y-coordinate
       \\units m
       \\type real
  N223, \\field Vertex 74 Z-coordinate
       \\units m
       \\type real
  N224, \\field Vertex 75 X-coordinate
       \\units m
       \\type real
  N225, \\field Vertex 75 Y-coordinate
       \\units m
       \\type real
  N226, \\field Vertex 75 Z-coordinate
       \\units m
       \\type real
  N227, \\field Vertex 76 X-coordinate
       \\units m
       \\type real
  N228, \\field Vertex 76 Y-coordinate
       \\units m
       \\type real
  N229, \\field Vertex 76 Z-coordinate
       \\units m
       \\type real
  N230, \\field Vertex 77 X-coordinate
       \\units m
       \\type real
  N231, \\field Vertex 77 Y-coordinate
       \\units m
       \\type real
  N232, \\field Vertex 77 Z-coordinate
       \\units m
       \\type real
  N233, \\field Vertex 78 X-coordinate
       \\units m
       \\type real
  N234, \\field Vertex 78 Y-coordinate
       \\units m
       \\type real
  N235, \\field Vertex 78 Z-coordinate
       \\units m
       \\type real
  N236, \\field Vertex 79 X-coordinate
       \\units m
       \\type real
  N237, \\field Vertex 79 Y-coordinate
       \\units m
       \\type real
  N238, \\field Vertex 79 Z-coordinate
       \\units m
       \\type real
  N239, \\field Vertex 80 X-coordinate
       \\units m
       \\type real
  N240, \\field Vertex 80 Y-coordinate
       \\units m
       \\type real
  N241, \\field Vertex 80 Z-coordinate
       \\units m
       \\type real
  N242, \\field Vertex 81 X-coordinate
       \\units m
       \\type real
  N243, \\field Vertex 81 Y-coordinate
       \\units m
       \\type real
  N244, \\field Vertex 81 Z-coordinate
       \\units m
       \\type real
  N245, \\field Vertex 82 X-coordinate
       \\units m
       \\type real
  N246, \\field Vertex 82 Y-coordinate
       \\units m
       \\type real
  N247, \\field Vertex 82 Z-coordinate
       \\units m
       \\type real
  N248, \\field Vertex 83 X-coordinate
       \\units m
       \\type real
  N249, \\field Vertex 83 Y-coordinate
       \\units m
       \\type real
  N250, \\field Vertex 83 Z-coordinate
       \\units m
       \\type real
  N251, \\field Vertex 84 X-coordinate
       \\units m
       \\type real
  N252, \\field Vertex 84 Y-coordinate
       \\units m
       \\type real
  N253, \\field Vertex 84 Z-coordinate
       \\units m
       \\type real
  N254, \\field Vertex 85 X-coordinate
       \\units m
       \\type real
  N255, \\field Vertex 85 Y-coordinate
       \\units m
       \\type real
  N256, \\field Vertex 85 Z-coordinate
       \\units m
       \\type real
  N257, \\field Vertex 86 X-coordinate
       \\units m
       \\type real
  N258, \\field Vertex 86 Y-coordinate
       \\units m
       \\type real
  N259, \\field Vertex 86 Z-coordinate
       \\units m
       \\type real
  N260, \\field Vertex 87 X-coordinate
       \\units m
       \\type real
  N261, \\field Vertex 87 Y-coordinate
       \\units m
       \\type real
  N262, \\field Vertex 87 Z-coordinate
       \\units m
       \\type real
  N263, \\field Vertex 88 X-coordinate
       \\units m
       \\type real
  N264, \\field Vertex 88 Y-coordinate
       \\units m
       \\type real
  N265, \\field Vertex 88 Z-coordinate
       \\units m
       \\type real
  N266, \\field Vertex 89 X-coordinate
       \\units m
       \\type real
  N267, \\field Vertex 89 Y-coordinate
       \\units m
       \\type real
  N268, \\field Vertex 89 Z-coordinate
       \\units m
       \\type real
  N269, \\field Vertex 90 X-coordinate
       \\units m
       \\type real
  N270, \\field Vertex 90 Y-coordinate
       \\units m
       \\type real
  N271, \\field Vertex 90 Z-coordinate
       \\units m
       \\type real
  N272, \\field Vertex 91 X-coordinate
       \\units m
       \\type real
  N273, \\field Vertex 91 Y-coordinate
       \\units m
       \\type real
  N274, \\field Vertex 91 Z-coordinate
       \\units m
       \\type real
  N275, \\field Vertex 92 X-coordinate
       \\units m
       \\type real
  N276, \\field Vertex 92 Y-coordinate
       \\units m
       \\type real
  N277, \\field Vertex 92 Z-coordinate
       \\units m
       \\type real
  N278, \\field Vertex 93 X-coordinate
       \\units m
       \\type real
  N279, \\field Vertex 93 Y-coordinate
       \\units m
       \\type real
  N280, \\field Vertex 93 Z-coordinate
       \\units m
       \\type real
  N281, \\field Vertex 94 X-coordinate
       \\units m
       \\type real
  N282, \\field Vertex 94 Y-coordinate
       \\units m
       \\type real
  N283, \\field Vertex 94 Z-coordinate
       \\units m
       \\type real
  N284, \\field Vertex 95 X-coordinate
       \\units m
       \\type real
  N285, \\field Vertex 95 Y-coordinate
       \\units m
       \\type real
  N286, \\field Vertex 95 Z-coordinate
       \\units m
       \\type real
  N287, \\field Vertex 96 X-coordinate
       \\units m
       \\type real
  N288, \\field Vertex 96 Y-coordinate
       \\units m
       \\type real
  N289, \\field Vertex 96 Z-coordinate
       \\units m
       \\type real
  N290, \\field Vertex 97 X-coordinate
       \\units m
       \\type real
  N291, \\field Vertex 97 Y-coordinate
       \\units m
       \\type real
  N292, \\field Vertex 97 Z-coordinate
       \\units m
       \\type real
  N293, \\field Vertex 98 X-coordinate
       \\units m
       \\type real
  N294, \\field Vertex 98 Y-coordinate
       \\units m
       \\type real
  N295, \\field Vertex 98 Z-coordinate
       \\units m
       \\type real
  N296, \\field Vertex 99 X-coordinate
       \\units m
       \\type real
  N297, \\field Vertex 99 Y-coordinate
       \\units m
       \\type real
  N298, \\field Vertex 99 Z-coordinate
       \\units m
       \\type real
  N299, \\field Vertex 100 X-coordinate
       \\units m
       \\type real
  N300, \\field Vertex 100 Y-coordinate
       \\units m
       \\type real
  N301, \\field Vertex 100 Z-coordinate
       \\units m
       \\type real
  N302, \\field Vertex 101 X-coordinate
       \\units m
       \\type real
  N303, \\field Vertex 101 Y-coordinate
       \\units m
       \\type real
  N304, \\field Vertex 101 Z-coordinate
       \\units m
       \\type real
  N305, \\field Vertex 102 X-coordinate
       \\units m
       \\type real
  N306, \\field Vertex 102 Y-coordinate
       \\units m
       \\type real
  N307, \\field Vertex 102 Z-coordinate
       \\units m
       \\type real
  N308, \\field Vertex 103 X-coordinate
       \\units m
       \\type real
  N309, \\field Vertex 103 Y-coordinate
       \\units m
       \\type real
  N310, \\field Vertex 103 Z-coordinate
       \\units m
       \\type real
  N311, \\field Vertex 104 X-coordinate
       \\units m
       \\type real
  N312, \\field Vertex 104 Y-coordinate
       \\units m
       \\type real
  N313, \\field Vertex 104 Z-coordinate
       \\units m
       \\type real
  N314, \\field Vertex 105 X-coordinate
       \\units m
       \\type real
  N315, \\field Vertex 105 Y-coordinate
       \\units m
       \\type real
  N316, \\field Vertex 105 Z-coordinate
       \\units m
       \\type real
  N317, \\field Vertex 106 X-coordinate
       \\units m
       \\type real
  N318, \\field Vertex 106 Y-coordinate
       \\units m
       \\type real
  N319, \\field Vertex 106 Z-coordinate
       \\units m
       \\type real
  N320, \\field Vertex 107 X-coordinate
       \\units m
       \\type real
  N321, \\field Vertex 107 Y-coordinate
       \\units m
       \\type real
  N322, \\field Vertex 107 Z-coordinate
       \\units m
       \\type real
  N323, \\field Vertex 108 X-coordinate
       \\units m
       \\type real
  N324, \\field Vertex 108 Y-coordinate
       \\units m
       \\type real
  N325, \\field Vertex 108 Z-coordinate
       \\units m
       \\type real
  N326, \\field Vertex 109 X-coordinate
       \\units m
       \\type real
  N327, \\field Vertex 109 Y-coordinate
       \\units m
       \\type real
  N328, \\field Vertex 109 Z-coordinate
       \\units m
       \\type real
  N329, \\field Vertex 110 X-coordinate
       \\units m
       \\type real
  N330, \\field Vertex 110 Y-coordinate
       \\units m
       \\type real
  N331, \\field Vertex 110 Z-coordinate
       \\units m
       \\type real
  N332, \\field Vertex 111 X-coordinate
       \\units m
       \\type real
  N333, \\field Vertex 111 Y-coordinate
       \\units m
       \\type real
  N334, \\field Vertex 111 Z-coordinate
       \\units m
       \\type real
  N335, \\field Vertex 112 X-coordinate
       \\units m
       \\type real
  N336, \\field Vertex 112 Y-coordinate
       \\units m
       \\type real
  N337, \\field Vertex 112 Z-coordinate
       \\units m
       \\type real
  N338, \\field Vertex 113 X-coordinate
       \\units m
       \\type real
  N339, \\field Vertex 113 Y-coordinate
       \\units m
       \\type real
  N340, \\field Vertex 113 Z-coordinate
       \\units m
       \\type real
  N341, \\field Vertex 114 X-coordinate
       \\units m
       \\type real
  N342, \\field Vertex 114 Y-coordinate
       \\units m
       \\type real
  N343, \\field Vertex 114 Z-coordinate
       \\units m
       \\type real
  N344, \\field Vertex 115 X-coordinate
       \\units m
       \\type real
  N345, \\field Vertex 115 Y-coordinate
       \\units m
       \\type real
  N346, \\field Vertex 115 Z-coordinate
       \\units m
       \\type real
  N347, \\field Vertex 116 X-coordinate
       \\units m
       \\type real
  N348, \\field Vertex 116 Y-coordinate
       \\units m
       \\type real
  N349, \\field Vertex 116 Z-coordinate
       \\units m
       \\type real
  N350, \\field Vertex 117 X-coordinate
       \\units m
       \\type real
  N351, \\field Vertex 117 Y-coordinate
       \\units m
       \\type real
  N352, \\field Vertex 117 Z-coordinate
       \\units m
       \\type real
  N353, \\field Vertex 118 X-coordinate
       \\units m
       \\type real
  N354, \\field Vertex 118 Y-coordinate
       \\units m
       \\type real
  N355, \\field Vertex 118 Z-coordinate
       \\units m
       \\type real
  N356, \\field Vertex 119 X-coordinate
       \\units m
       \\type real
  N357, \\field Vertex 119 Y-coordinate
       \\units m
       \\type real
  N358, \\field Vertex 119 Z-coordinate
       \\units m
       \\type real
  N359, \\field Vertex 120 X-coordinate
       \\units m
       \\type real
  N360, \\field Vertex 120 Y-coordinate
       \\units m
       \\type real
  N361; \\field Vertex 120 Z-coordinate
       \\units m
       \\type real

Shading:Building:Detailed,
  \\extensible:3 -- duplicate last set of x,y,z coordinates, renumbering please (and changing z terminator to , for all but last)
       \\memo used for shading elements such as trees, other buildings, parts of this building not being modeled
       \\memo these items are relative to the current building and would move with relative geometry
       \\format vertices
       \\min-fields 12
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2,  \\field Transmittance Schedule Name
       \\note Transmittance schedule for the shading device, defaults to zero (always opaque)
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Number of Vertices
       \\required-field
       \\note shown with 6 vertex coordinates -- extensible object
       \\autocalculatable
       \\default autocalculate
       \\minimum 3
       \\note Rules for vertices are given in GlobalGeometryRules coordinates --
       \\note For this object all surface coordinates are relative to the building origin (0,0,0)
       \\note and will rotate with the BUILDING north axis.
  N2,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N3 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N5,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N7, \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N8, \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N12, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N13, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N14, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 Y-coordinate
       \\units m
       \\type real
  N16, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N17, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 Y-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N20, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 Y-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N23, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 Y-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N26, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 Y-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N29, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 Y-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Z-coordinate
       \\units m
       \\type real
  N32, \\field Vertex 11 X-coordinate
       \\units m
       \\type real
  N33, \\field Vertex 11 Y-coordinate
       \\units m
       \\type real
  N34, \\field Vertex 11 Z-coordinate
       \\units m
       \\type real
  N35, \\field Vertex 12 X-coordinate
       \\units m
       \\type real
  N36, \\field Vertex 12 Y-coordinate
       \\units m
       \\type real
  N37, \\field Vertex 12 Z-coordinate
       \\units m
       \\type real
  N38, \\field Vertex 13 X-coordinate
       \\units m
       \\type real
  N39, \\field Vertex 13 Y-coordinate
       \\units m
       \\type real
  N40, \\field Vertex 13 Z-coordinate
       \\units m
       \\type real
  N41, \\field Vertex 14 X-coordinate
       \\units m
       \\type real
  N42, \\field Vertex 14 Y-coordinate
       \\units m
       \\type real
  N43, \\field Vertex 14 Z-coordinate
       \\units m
       \\type real
  N44, \\field Vertex 15 X-coordinate
       \\units m
       \\type real
  N45, \\field Vertex 15 Y-coordinate
       \\units m
       \\type real
  N46, \\field Vertex 15 Z-coordinate
       \\units m
       \\type real
  N47, \\field Vertex 16 X-coordinate
       \\units m
       \\type real
  N48, \\field Vertex 16 Y-coordinate
       \\units m
       \\type real
  N49, \\field Vertex 16 Z-coordinate
       \\units m
       \\type real
  N50, \\field Vertex 17 X-coordinate
       \\units m
       \\type real
  N51, \\field Vertex 17 Y-coordinate
       \\units m
       \\type real
  N52, \\field Vertex 17 Z-coordinate
       \\units m
       \\type real
  N53, \\field Vertex 18 X-coordinate
       \\units m
       \\type real
  N54, \\field Vertex 18 Y-coordinate
       \\units m
       \\type real
  N55, \\field Vertex 18 Z-coordinate
       \\units m
       \\type real
  N56, \\field Vertex 19 X-coordinate
       \\units m
       \\type real
  N57, \\field Vertex 19 Y-coordinate
       \\units m
       \\type real
  N58, \\field Vertex 19 Z-coordinate
       \\units m
       \\type real
  N59, \\field Vertex 20 X-coordinate
       \\units m
       \\type real
  N60, \\field Vertex 20 Y-coordinate
       \\units m
       \\type real
  N61, \\field Vertex 20 Z-coordinate
       \\units m
       \\type real
  N62, \\field Vertex 21 X-coordinate
       \\units m
       \\type real
  N63, \\field Vertex 21 Y-coordinate
       \\units m
       \\type real
  N64, \\field Vertex 21 Z-coordinate
       \\units m
       \\type real
  N65, \\field Vertex 22 X-coordinate
       \\units m
       \\type real
  N66, \\field Vertex 22 Y-coordinate
       \\units m
       \\type real
  N67, \\field Vertex 22 Z-coordinate
       \\units m
       \\type real
  N68, \\field Vertex 23 X-coordinate
       \\units m
       \\type real
  N69, \\field Vertex 23 Y-coordinate
       \\units m
       \\type real
  N70, \\field Vertex 23 Z-coordinate
       \\units m
       \\type real
  N71, \\field Vertex 24 X-coordinate
       \\units m
       \\type real
  N72, \\field Vertex 24 Y-coordinate
       \\units m
       \\type real
  N73, \\field Vertex 24 Z-coordinate
       \\units m
       \\type real
  N74, \\field Vertex 25 X-coordinate
       \\units m
       \\type real
  N75, \\field Vertex 25 Y-coordinate
       \\units m
       \\type real
  N76, \\field Vertex 25 Z-coordinate
       \\units m
       \\type real
  N77, \\field Vertex 26 X-coordinate
       \\units m
       \\type real
  N78, \\field Vertex 26 Y-coordinate
       \\units m
       \\type real
  N79, \\field Vertex 26 Z-coordinate
       \\units m
       \\type real
  N80, \\field Vertex 27 X-coordinate
       \\units m
       \\type real
  N81, \\field Vertex 27 Y-coordinate
       \\units m
       \\type real
  N82, \\field Vertex 27 Z-coordinate
       \\units m
       \\type real
  N83, \\field Vertex 28 X-coordinate
       \\units m
       \\type real
  N84, \\field Vertex 28 Y-coordinate
       \\units m
       \\type real
  N85, \\field Vertex 28 Z-coordinate
       \\units m
       \\type real
  N86, \\field Vertex 29 X-coordinate
       \\units m
       \\type real
  N87, \\field Vertex 29 Y-coordinate
       \\units m
       \\type real
  N88, \\field Vertex 29 Z-coordinate
       \\units m
       \\type real
  N89, \\field Vertex 30 X-coordinate
       \\units m
       \\type real
  N90, \\field Vertex 30 Y-coordinate
       \\units m
       \\type real
  N91, \\field Vertex 30 Z-coordinate
       \\units m
       \\type real
  N92, \\field Vertex 31 X-coordinate
       \\units m
       \\type real
  N93, \\field Vertex 31 Y-coordinate
       \\units m
       \\type real
  N94, \\field Vertex 31 Z-coordinate
       \\units m
       \\type real
  N95, \\field Vertex 32 X-coordinate
       \\units m
       \\type real
  N96, \\field Vertex 32 Y-coordinate
       \\units m
       \\type real
  N97, \\field Vertex 32 Z-coordinate
       \\units m
       \\type real
  N98, \\field Vertex 33 X-coordinate
       \\units m
       \\type real
  N99, \\field Vertex 33 Y-coordinate
       \\units m
       \\type real
  N100, \\field Vertex 33 Z-coordinate
       \\units m
       \\type real
  N101, \\field Vertex 34 X-coordinate
       \\units m
       \\type real
  N102, \\field Vertex 34 Y-coordinate
       \\units m
       \\type real
  N103, \\field Vertex 34 Z-coordinate
       \\units m
       \\type real
  N104, \\field Vertex 35 X-coordinate
       \\units m
       \\type real
  N105, \\field Vertex 35 Y-coordinate
       \\units m
       \\type real
  N106, \\field Vertex 35 Z-coordinate
       \\units m
       \\type real
  N107, \\field Vertex 36 X-coordinate
       \\units m
       \\type real
  N108, \\field Vertex 36 Y-coordinate
       \\units m
       \\type real
  N109, \\field Vertex 36 Z-coordinate
       \\units m
       \\type real
  N110, \\field Vertex 37 X-coordinate
       \\units m
       \\type real
  N111, \\field Vertex 37 Y-coordinate
       \\units m
       \\type real
  N112, \\field Vertex 37 Z-coordinate
       \\units m
       \\type real
  N113, \\field Vertex 38 X-coordinate
       \\units m
       \\type real
  N114, \\field Vertex 38 Y-coordinate
       \\units m
       \\type real
  N115, \\field Vertex 38 Z-coordinate
       \\units m
       \\type real
  N116, \\field Vertex 39 X-coordinate
       \\units m
       \\type real
  N117, \\field Vertex 39 Y-coordinate
       \\units m
       \\type real
  N118, \\field Vertex 39 Z-coordinate
       \\units m
       \\type real
  N119, \\field Vertex 40 X-coordinate
       \\units m
       \\type real
  N120, \\field Vertex 40 Y-coordinate
       \\units m
       \\type real
  N121, \\field Vertex 40 Z-coordinate
       \\units m
       \\type real
  N122, \\field Vertex 41 X-coordinate
       \\units m
       \\type real
  N123, \\field Vertex 41 Y-coordinate
       \\units m
       \\type real
  N124, \\field Vertex 41 Z-coordinate
       \\units m
       \\type real
  N125, \\field Vertex 42 X-coordinate
       \\units m
       \\type real
  N126, \\field Vertex 42 Y-coordinate
       \\units m
       \\type real
  N127, \\field Vertex 42 Z-coordinate
       \\units m
       \\type real
  N128, \\field Vertex 43 X-coordinate
       \\units m
       \\type real
  N129, \\field Vertex 43 Y-coordinate
       \\units m
       \\type real
  N130, \\field Vertex 43 Z-coordinate
       \\units m
       \\type real
  N131, \\field Vertex 44 X-coordinate
       \\units m
       \\type real
  N132, \\field Vertex 44 Y-coordinate
       \\units m
       \\type real
  N133, \\field Vertex 44 Z-coordinate
       \\units m
       \\type real
  N134, \\field Vertex 45 X-coordinate
       \\units m
       \\type real
  N135, \\field Vertex 45 Y-coordinate
       \\units m
       \\type real
  N136, \\field Vertex 45 Z-coordinate
       \\units m
       \\type real
  N137, \\field Vertex 46 X-coordinate
       \\units m
       \\type real
  N138, \\field Vertex 46 Y-coordinate
       \\units m
       \\type real
  N139, \\field Vertex 46 Z-coordinate
       \\units m
       \\type real
  N140, \\field Vertex 47 X-coordinate
       \\units m
       \\type real
  N141, \\field Vertex 47 Y-coordinate
       \\units m
       \\type real
  N142, \\field Vertex 47 Z-coordinate
       \\units m
       \\type real
  N143, \\field Vertex 48 X-coordinate
       \\units m
       \\type real
  N144, \\field Vertex 48 Y-coordinate
       \\units m
       \\type real
  N145, \\field Vertex 48 Z-coordinate
       \\units m
       \\type real
  N146, \\field Vertex 49 X-coordinate
       \\units m
       \\type real
  N147, \\field Vertex 49 Y-coordinate
       \\units m
       \\type real
  N148, \\field Vertex 49 Z-coordinate
       \\units m
       \\type real
  N149, \\field Vertex 50 X-coordinate
       \\units m
       \\type real
  N150, \\field Vertex 50 Y-coordinate
       \\units m
       \\type real
  N151, \\field Vertex 50 Z-coordinate
       \\units m
       \\type real
  N152, \\field Vertex 51 X-coordinate
       \\units m
       \\type real
  N153, \\field Vertex 51 Y-coordinate
       \\units m
       \\type real
  N154, \\field Vertex 51 Z-coordinate
       \\units m
       \\type real
  N155, \\field Vertex 52 X-coordinate
       \\units m
       \\type real
  N156, \\field Vertex 52 Y-coordinate
       \\units m
       \\type real
  N157, \\field Vertex 52 Z-coordinate
       \\units m
       \\type real
  N158, \\field Vertex 53 X-coordinate
       \\units m
       \\type real
  N159, \\field Vertex 53 Y-coordinate
       \\units m
       \\type real
  N160, \\field Vertex 53 Z-coordinate
       \\units m
       \\type real
  N161, \\field Vertex 54 X-coordinate
       \\units m
       \\type real
  N162, \\field Vertex 54 Y-coordinate
       \\units m
       \\type real
  N163, \\field Vertex 54 Z-coordinate
       \\units m
       \\type real
  N164, \\field Vertex 55 X-coordinate
       \\units m
       \\type real
  N165, \\field Vertex 55 Y-coordinate
       \\units m
       \\type real
  N166, \\field Vertex 55 Z-coordinate
       \\units m
       \\type real
  N167, \\field Vertex 56 X-coordinate
       \\units m
       \\type real
  N168, \\field Vertex 56 Y-coordinate
       \\units m
       \\type real
  N169, \\field Vertex 56 Z-coordinate
       \\units m
       \\type real
  N170, \\field Vertex 57 X-coordinate
       \\units m
       \\type real
  N171, \\field Vertex 57 Y-coordinate
       \\units m
       \\type real
  N172, \\field Vertex 57 Z-coordinate
       \\units m
       \\type real
  N173, \\field Vertex 58 X-coordinate
       \\units m
       \\type real
  N174, \\field Vertex 58 Y-coordinate
       \\units m
       \\type real
  N175, \\field Vertex 58 Z-coordinate
       \\units m
       \\type real
  N176, \\field Vertex 59 X-coordinate
       \\units m
       \\type real
  N177, \\field Vertex 59 Y-coordinate
       \\units m
       \\type real
  N178, \\field Vertex 59 Z-coordinate
       \\units m
       \\type real
  N179, \\field Vertex 60 X-coordinate
       \\units m
       \\type real
  N180, \\field Vertex 60 Y-coordinate
       \\units m
       \\type real
  N181, \\field Vertex 60 Z-coordinate
       \\units m
       \\type real
  N182, \\field Vertex 61 X-coordinate
       \\units m
       \\type real
  N183, \\field Vertex 61 Y-coordinate
       \\units m
       \\type real
  N184, \\field Vertex 61 Z-coordinate
       \\units m
       \\type real
  N185, \\field Vertex 62 X-coordinate
       \\units m
       \\type real
  N186, \\field Vertex 62 Y-coordinate
       \\units m
       \\type real
  N187, \\field Vertex 62 Z-coordinate
       \\units m
       \\type real
  N188, \\field Vertex 63 X-coordinate
       \\units m
       \\type real
  N189, \\field Vertex 63 Y-coordinate
       \\units m
       \\type real
  N190, \\field Vertex 63 Z-coordinate
       \\units m
       \\type real
  N191, \\field Vertex 64 X-coordinate
       \\units m
       \\type real
  N192, \\field Vertex 64 Y-coordinate
       \\units m
       \\type real
  N193, \\field Vertex 64 Z-coordinate
       \\units m
       \\type real
  N194, \\field Vertex 65 X-coordinate
       \\units m
       \\type real
  N195, \\field Vertex 65 Y-coordinate
       \\units m
       \\type real
  N196, \\field Vertex 65 Z-coordinate
       \\units m
       \\type real
  N197, \\field Vertex 66 X-coordinate
       \\units m
       \\type real
  N198, \\field Vertex 66 Y-coordinate
       \\units m
       \\type real
  N199, \\field Vertex 66 Z-coordinate
       \\units m
       \\type real
  N200, \\field Vertex 67 X-coordinate
       \\units m
       \\type real
  N201, \\field Vertex 67 Y-coordinate
       \\units m
       \\type real
  N202, \\field Vertex 67 Z-coordinate
       \\units m
       \\type real
  N203, \\field Vertex 68 X-coordinate
       \\units m
       \\type real
  N204, \\field Vertex 68 Y-coordinate
       \\units m
       \\type real
  N205, \\field Vertex 68 Z-coordinate
       \\units m
       \\type real
  N206, \\field Vertex 69 X-coordinate
       \\units m
       \\type real
  N207, \\field Vertex 69 Y-coordinate
       \\units m
       \\type real
  N208, \\field Vertex 69 Z-coordinate
       \\units m
       \\type real
  N209, \\field Vertex 70 X-coordinate
       \\units m
       \\type real
  N210, \\field Vertex 70 Y-coordinate
       \\units m
       \\type real
  N211, \\field Vertex 70 Z-coordinate
       \\units m
       \\type real
  N212, \\field Vertex 71 X-coordinate
       \\units m
       \\type real
  N213, \\field Vertex 71 Y-coordinate
       \\units m
       \\type real
  N214, \\field Vertex 71 Z-coordinate
       \\units m
       \\type real
  N215, \\field Vertex 72 X-coordinate
       \\units m
       \\type real
  N216, \\field Vertex 72 Y-coordinate
       \\units m
       \\type real
  N217, \\field Vertex 72 Z-coordinate
       \\units m
       \\type real
  N218, \\field Vertex 73 X-coordinate
       \\units m
       \\type real
  N219, \\field Vertex 73 Y-coordinate
       \\units m
       \\type real
  N220, \\field Vertex 73 Z-coordinate
       \\units m
       \\type real
  N221, \\field Vertex 74 X-coordinate
       \\units m
       \\type real
  N222, \\field Vertex 74 Y-coordinate
       \\units m
       \\type real
  N223, \\field Vertex 74 Z-coordinate
       \\units m
       \\type real
  N224, \\field Vertex 75 X-coordinate
       \\units m
       \\type real
  N225, \\field Vertex 75 Y-coordinate
       \\units m
       \\type real
  N226, \\field Vertex 75 Z-coordinate
       \\units m
       \\type real
  N227, \\field Vertex 76 X-coordinate
       \\units m
       \\type real
  N228, \\field Vertex 76 Y-coordinate
       \\units m
       \\type real
  N229, \\field Vertex 76 Z-coordinate
       \\units m
       \\type real
  N230, \\field Vertex 77 X-coordinate
       \\units m
       \\type real
  N231, \\field Vertex 77 Y-coordinate
       \\units m
       \\type real
  N232, \\field Vertex 77 Z-coordinate
       \\units m
       \\type real
  N233, \\field Vertex 78 X-coordinate
       \\units m
       \\type real
  N234, \\field Vertex 78 Y-coordinate
       \\units m
       \\type real
  N235, \\field Vertex 78 Z-coordinate
       \\units m
       \\type real
  N236, \\field Vertex 79 X-coordinate
       \\units m
       \\type real
  N237, \\field Vertex 79 Y-coordinate
       \\units m
       \\type real
  N238, \\field Vertex 79 Z-coordinate
       \\units m
       \\type real
  N239, \\field Vertex 80 X-coordinate
       \\units m
       \\type real
  N240, \\field Vertex 80 Y-coordinate
       \\units m
       \\type real
  N241, \\field Vertex 80 Z-coordinate
       \\units m
       \\type real
  N242, \\field Vertex 81 X-coordinate
       \\units m
       \\type real
  N243, \\field Vertex 81 Y-coordinate
       \\units m
       \\type real
  N244, \\field Vertex 81 Z-coordinate
       \\units m
       \\type real
  N245, \\field Vertex 82 X-coordinate
       \\units m
       \\type real
  N246, \\field Vertex 82 Y-coordinate
       \\units m
       \\type real
  N247, \\field Vertex 82 Z-coordinate
       \\units m
       \\type real
  N248, \\field Vertex 83 X-coordinate
       \\units m
       \\type real
  N249, \\field Vertex 83 Y-coordinate
       \\units m
       \\type real
  N250, \\field Vertex 83 Z-coordinate
       \\units m
       \\type real
  N251, \\field Vertex 84 X-coordinate
       \\units m
       \\type real
  N252, \\field Vertex 84 Y-coordinate
       \\units m
       \\type real
  N253, \\field Vertex 84 Z-coordinate
       \\units m
       \\type real
  N254, \\field Vertex 85 X-coordinate
       \\units m
       \\type real
  N255, \\field Vertex 85 Y-coordinate
       \\units m
       \\type real
  N256, \\field Vertex 85 Z-coordinate
       \\units m
       \\type real
  N257, \\field Vertex 86 X-coordinate
       \\units m
       \\type real
  N258, \\field Vertex 86 Y-coordinate
       \\units m
       \\type real
  N259, \\field Vertex 86 Z-coordinate
       \\units m
       \\type real
  N260, \\field Vertex 87 X-coordinate
       \\units m
       \\type real
  N261, \\field Vertex 87 Y-coordinate
       \\units m
       \\type real
  N262, \\field Vertex 87 Z-coordinate
       \\units m
       \\type real
  N263, \\field Vertex 88 X-coordinate
       \\units m
       \\type real
  N264, \\field Vertex 88 Y-coordinate
       \\units m
       \\type real
  N265, \\field Vertex 88 Z-coordinate
       \\units m
       \\type real
  N266, \\field Vertex 89 X-coordinate
       \\units m
       \\type real
  N267, \\field Vertex 89 Y-coordinate
       \\units m
       \\type real
  N268, \\field Vertex 89 Z-coordinate
       \\units m
       \\type real
  N269, \\field Vertex 90 X-coordinate
       \\units m
       \\type real
  N270, \\field Vertex 90 Y-coordinate
       \\units m
       \\type real
  N271, \\field Vertex 90 Z-coordinate
       \\units m
       \\type real
  N272, \\field Vertex 91 X-coordinate
       \\units m
       \\type real
  N273, \\field Vertex 91 Y-coordinate
       \\units m
       \\type real
  N274, \\field Vertex 91 Z-coordinate
       \\units m
       \\type real
  N275, \\field Vertex 92 X-coordinate
       \\units m
       \\type real
  N276, \\field Vertex 92 Y-coordinate
       \\units m
       \\type real
  N277, \\field Vertex 92 Z-coordinate
       \\units m
       \\type real
  N278, \\field Vertex 93 X-coordinate
       \\units m
       \\type real
  N279, \\field Vertex 93 Y-coordinate
       \\units m
       \\type real
  N280, \\field Vertex 93 Z-coordinate
       \\units m
       \\type real
  N281, \\field Vertex 94 X-coordinate
       \\units m
       \\type real
  N282, \\field Vertex 94 Y-coordinate
       \\units m
       \\type real
  N283, \\field Vertex 94 Z-coordinate
       \\units m
       \\type real
  N284, \\field Vertex 95 X-coordinate
       \\units m
       \\type real
  N285, \\field Vertex 95 Y-coordinate
       \\units m
       \\type real
  N286, \\field Vertex 95 Z-coordinate
       \\units m
       \\type real
  N287, \\field Vertex 96 X-coordinate
       \\units m
       \\type real
  N288, \\field Vertex 96 Y-coordinate
       \\units m
       \\type real
  N289, \\field Vertex 96 Z-coordinate
       \\units m
       \\type real
  N290, \\field Vertex 97 X-coordinate
       \\units m
       \\type real
  N291, \\field Vertex 97 Y-coordinate
       \\units m
       \\type real
  N292, \\field Vertex 97 Z-coordinate
       \\units m
       \\type real
  N293, \\field Vertex 98 X-coordinate
       \\units m
       \\type real
  N294, \\field Vertex 98 Y-coordinate
       \\units m
       \\type real
  N295, \\field Vertex 98 Z-coordinate
       \\units m
       \\type real
  N296, \\field Vertex 99 X-coordinate
       \\units m
       \\type real
  N297, \\field Vertex 99 Y-coordinate
       \\units m
       \\type real
  N298, \\field Vertex 99 Z-coordinate
       \\units m
       \\type real
  N299, \\field Vertex 100 X-coordinate
       \\units m
       \\type real
  N300, \\field Vertex 100 Y-coordinate
       \\units m
       \\type real
  N301, \\field Vertex 100 Z-coordinate
       \\units m
       \\type real
  N302, \\field Vertex 101 X-coordinate
       \\units m
       \\type real
  N303, \\field Vertex 101 Y-coordinate
       \\units m
       \\type real
  N304, \\field Vertex 101 Z-coordinate
       \\units m
       \\type real
  N305, \\field Vertex 102 X-coordinate
       \\units m
       \\type real
  N306, \\field Vertex 102 Y-coordinate
       \\units m
       \\type real
  N307, \\field Vertex 102 Z-coordinate
       \\units m
       \\type real
  N308, \\field Vertex 103 X-coordinate
       \\units m
       \\type real
  N309, \\field Vertex 103 Y-coordinate
       \\units m
       \\type real
  N310, \\field Vertex 103 Z-coordinate
       \\units m
       \\type real
  N311, \\field Vertex 104 X-coordinate
       \\units m
       \\type real
  N312, \\field Vertex 104 Y-coordinate
       \\units m
       \\type real
  N313, \\field Vertex 104 Z-coordinate
       \\units m
       \\type real
  N314, \\field Vertex 105 X-coordinate
       \\units m
       \\type real
  N315, \\field Vertex 105 Y-coordinate
       \\units m
       \\type real
  N316, \\field Vertex 105 Z-coordinate
       \\units m
       \\type real
  N317, \\field Vertex 106 X-coordinate
       \\units m
       \\type real
  N318, \\field Vertex 106 Y-coordinate
       \\units m
       \\type real
  N319, \\field Vertex 106 Z-coordinate
       \\units m
       \\type real
  N320, \\field Vertex 107 X-coordinate
       \\units m
       \\type real
  N321, \\field Vertex 107 Y-coordinate
       \\units m
       \\type real
  N322, \\field Vertex 107 Z-coordinate
       \\units m
       \\type real
  N323, \\field Vertex 108 X-coordinate
       \\units m
       \\type real
  N324, \\field Vertex 108 Y-coordinate
       \\units m
       \\type real
  N325, \\field Vertex 108 Z-coordinate
       \\units m
       \\type real
  N326, \\field Vertex 109 X-coordinate
       \\units m
       \\type real
  N327, \\field Vertex 109 Y-coordinate
       \\units m
       \\type real
  N328, \\field Vertex 109 Z-coordinate
       \\units m
       \\type real
  N329, \\field Vertex 110 X-coordinate
       \\units m
       \\type real
  N330, \\field Vertex 110 Y-coordinate
       \\units m
       \\type real
  N331, \\field Vertex 110 Z-coordinate
       \\units m
       \\type real
  N332, \\field Vertex 111 X-coordinate
       \\units m
       \\type real
  N333, \\field Vertex 111 Y-coordinate
       \\units m
       \\type real
  N334, \\field Vertex 111 Z-coordinate
       \\units m
       \\type real
  N335, \\field Vertex 112 X-coordinate
       \\units m
       \\type real
  N336, \\field Vertex 112 Y-coordinate
       \\units m
       \\type real
  N337, \\field Vertex 112 Z-coordinate
       \\units m
       \\type real
  N338, \\field Vertex 113 X-coordinate
       \\units m
       \\type real
  N339, \\field Vertex 113 Y-coordinate
       \\units m
       \\type real
  N340, \\field Vertex 113 Z-coordinate
       \\units m
       \\type real
  N341, \\field Vertex 114 X-coordinate
       \\units m
       \\type real
  N342, \\field Vertex 114 Y-coordinate
       \\units m
       \\type real
  N343, \\field Vertex 114 Z-coordinate
       \\units m
       \\type real
  N344, \\field Vertex 115 X-coordinate
       \\units m
       \\type real
  N345, \\field Vertex 115 Y-coordinate
       \\units m
       \\type real
  N346, \\field Vertex 115 Z-coordinate
       \\units m
       \\type real
  N347, \\field Vertex 116 X-coordinate
       \\units m
       \\type real
  N348, \\field Vertex 116 Y-coordinate
       \\units m
       \\type real
  N349, \\field Vertex 116 Z-coordinate
       \\units m
       \\type real
  N350, \\field Vertex 117 X-coordinate
       \\units m
       \\type real
  N351, \\field Vertex 117 Y-coordinate
       \\units m
       \\type real
  N352, \\field Vertex 117 Z-coordinate
       \\units m
       \\type real
  N353, \\field Vertex 118 X-coordinate
       \\units m
       \\type real
  N354, \\field Vertex 118 Y-coordinate
       \\units m
       \\type real
  N355, \\field Vertex 118 Z-coordinate
       \\units m
       \\type real
  N356, \\field Vertex 119 X-coordinate
       \\units m
       \\type real
  N357, \\field Vertex 119 Y-coordinate
       \\units m
       \\type real
  N358, \\field Vertex 119 Z-coordinate
       \\units m
       \\type real
  N359, \\field Vertex 120 X-coordinate
       \\units m
       \\type real
  N360, \\field Vertex 120 Y-coordinate
       \\units m
       \\type real
  N361; \\field Vertex 120 Z-coordinate
       \\units m
       \\type real

Shading:Overhang,
      \\memo Overhangs are usually flat shading surfaces that reference a window or door.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AttachedShadingSurfNames
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2 , \\field Window or Door Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  N1,  \\field Height above Window or Door
       \\units m
  N2,  \\field Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N3,  \\field Left extension from Window/Door Width
       \\units m
  N4,  \\field Right extension from Window/Door Width
       \\note N3 + N4 + Window/Door Width is Overhang Length
       \\units m
  N5;  \\field Depth
       \\units m
       \\minimum 0

Shading:Overhang:Projection,
      \\memo Overhangs are typically flat shading surfaces that reference a window or door.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AttachedShadingSurfNames
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2 , \\field Window or Door Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  N1,  \\field Height above Window or Door
       \\units m
  N2,  \\field Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N3,  \\field Left extension from Window/Door Width
       \\units m
  N4,  \\field Right extension from Window/Door Width
       \\units m
       \\note N3 + N4 + Window/Door Width is Overhang Length
  N5;  \\field Depth as Fraction of Window/Door Height
       \\units dimensionless
       \\minimum 0

Shading:Fin,
      \\memo Fins are usually shading surfaces that are perpendicular to a window or door.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AttachedShadingSurfNames
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2 , \\field Window or Door Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  N1,  \\field Left Extension from Window/Door
       \\units m
  N2,  \\field Left Distance Above Top of Window
       \\units m
  N3,  \\field Left Distance Below Bottom of Window
       \\units m
       \\note N2 + N3 + height of Window/Door is height of Fin
  N4,  \\field Left Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N5,  \\field Left Depth
       \\units m
       \\minimum 0
  N6,  \\field Right Extension from Window/Door
       \\units m
  N7,  \\field Right Distance Above Top of Window
       \\units m
  N8,  \\field Right Distance Below Bottom of Window
       \\note N7 + N8 + height of Window/Door is height of Fin
       \\units m
  N9,  \\field Right Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N10; \\field Right Depth
       \\units m
       \\minimum 0

Shading:Fin:Projection,
      \\memo Fins are usually shading surfaces that are perpendicular to a window or door.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AttachedShadingSurfNames
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2 , \\field Window or Door Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  N1,  \\field Left Extension from Window/Door
       \\units m
  N2,  \\field Left Distance Above Top of Window
       \\units m
  N3,  \\field Left Distance Below Bottom of Window
       \\units m
       \\note N2 + N3 + height of Window/Door is height of Fin
  N4,  \\field Left Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N5,  \\field Left Depth as Fraction of Window/Door Width
       \\units dimensionless
       \\minimum 0
  N6,  \\field Right Extension from Window/Door
       \\units m
  N7,  \\field Right Distance Above Top of Window
       \\units m
  N8,  \\field Right Distance Below Bottom of Window
       \\note N7 + N8 + height of Window/Door is height of Fin
       \\units m
  N9,  \\field Right Tilt Angle from Window/Door
       \\units deg
       \\default 90
       \\minimum 0
       \\maximum 180
  N10; \\field Right Depth as Fraction of Window/Door Width
       \\units dimensionless
       \\minimum 0

Shading:Zone:Detailed,
  \\extensible:3 -- duplicate last set of x,y,z coordinates, renumbering please (and changing z terminator to , for all but last)
       \\memo used For fins, overhangs, elements that shade the building, are attached to the building
       \\memo but are not part of the heat transfer calculations
       \\format vertices
       \\min-fields 13
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AttachedShadingSurfNames
       \\reference AllShadingAndHTSurfNames
       \\reference AllShadingSurfNames
  A2 , \\field Base Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A3,  \\field Transmittance Schedule Name
       \\note Transmittance schedule for the shading device, defaults to zero (always opaque)
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Number of Vertices
       \\required-field
       \\note shown with 6 vertex coordinates -- extensible object
       \\autocalculatable
       \\default autocalculate
       \\minimum 3
       \\note vertices are given in GlobalGeometryRules coordinates -- if relative, all surface coordinates
       \\note are "relative" to the Zone Origin.  if world, then building and zone origins are used
       \\note for some internal calculations, but all coordinates are given in an "absolute" system.
  N2,  \\field Vertex 1 X-coordinate
       \\begin-extensible
       \\required-field
       \\units m
       \\type real
  N3 , \\field Vertex 1 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N4 , \\field Vertex 1 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N5,  \\field Vertex 2 X-coordinate
       \\required-field
       \\units m
       \\type real
  N6,  \\field Vertex 2 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N7, \\field Vertex 2 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N8, \\field Vertex 3 X-coordinate
       \\required-field
       \\units m
       \\type real
  N9,  \\field Vertex 3 Y-coordinate
       \\required-field
       \\units m
       \\type real
  N10, \\field Vertex 3 Z-coordinate
       \\required-field
       \\units m
       \\type real
  N11, \\field Vertex 4 X-coordinate
       \\units m
       \\type real
  N12, \\field Vertex 4 Y-coordinate
       \\type real
       \\units m
  N13, \\field Vertex 4 Z-coordinate
       \\units m
       \\type real
  N14, \\field Vertex 5 X-coordinate
       \\units m
       \\type real
  N15, \\field Vertex 5 Y-coordinate
       \\type real
       \\units m
  N16, \\field Vertex 5 Z-coordinate
       \\units m
       \\type real
  N17, \\field Vertex 6 X-coordinate
       \\units m
       \\type real
  N18, \\field Vertex 6 Y-coordinate
       \\units m
       \\type real
  N19, \\field Vertex 6 Z-coordinate
       \\units m
       \\type real
  N20, \\field Vertex 7 X-coordinate
       \\units m
       \\type real
  N21, \\field Vertex 7 Y-coordinate
       \\units m
       \\type real
  N22, \\field Vertex 7 Z-coordinate
       \\units m
       \\type real
  N23, \\field Vertex 8 X-coordinate
       \\units m
       \\type real
  N24, \\field Vertex 8 Y-coordinate
       \\units m
       \\type real
  N25, \\field Vertex 8 Z-coordinate
       \\units m
       \\type real
  N26, \\field Vertex 9 X-coordinate
       \\units m
       \\type real
  N27, \\field Vertex 9 Y-coordinate
       \\units m
       \\type real
  N28, \\field Vertex 9 Z-coordinate
       \\units m
       \\type real
  N29, \\field Vertex 10 X-coordinate
       \\units m
       \\type real
  N30, \\field Vertex 10 Y-coordinate
       \\units m
       \\type real
  N31, \\field Vertex 10 Z-coordinate
       \\units m
       \\type real
  N32, \\field Vertex 11 X-coordinate
       \\units m
       \\type real
  N33, \\field Vertex 11 Y-coordinate
       \\units m
       \\type real
  N34, \\field Vertex 11 Z-coordinate
       \\units m
       \\type real
  N35, \\field Vertex 12 X-coordinate
       \\units m
       \\type real
  N36, \\field Vertex 12 Y-coordinate
       \\units m
       \\type real
  N37, \\field Vertex 12 Z-coordinate
       \\units m
       \\type real
  N38, \\field Vertex 13 X-coordinate
       \\units m
       \\type real
  N39, \\field Vertex 13 Y-coordinate
       \\units m
       \\type real
  N40, \\field Vertex 13 Z-coordinate
       \\units m
       \\type real
  N41, \\field Vertex 14 X-coordinate
       \\units m
       \\type real
  N42, \\field Vertex 14 Y-coordinate
       \\units m
       \\type real
  N43, \\field Vertex 14 Z-coordinate
       \\units m
       \\type real
  N44, \\field Vertex 15 X-coordinate
       \\units m
       \\type real
  N45, \\field Vertex 15 Y-coordinate
       \\units m
       \\type real
  N46, \\field Vertex 15 Z-coordinate
       \\units m
       \\type real
  N47, \\field Vertex 16 X-coordinate
       \\units m
       \\type real
  N48, \\field Vertex 16 Y-coordinate
       \\units m
       \\type real
  N49, \\field Vertex 16 Z-coordinate
       \\units m
       \\type real
  N50, \\field Vertex 17 X-coordinate
       \\units m
       \\type real
  N51, \\field Vertex 17 Y-coordinate
       \\units m
       \\type real
  N52, \\field Vertex 17 Z-coordinate
       \\units m
       \\type real
  N53, \\field Vertex 18 X-coordinate
       \\units m
       \\type real
  N54, \\field Vertex 18 Y-coordinate
       \\units m
       \\type real
  N55, \\field Vertex 18 Z-coordinate
       \\units m
       \\type real
  N56, \\field Vertex 19 X-coordinate
       \\units m
       \\type real
  N57, \\field Vertex 19 Y-coordinate
       \\units m
       \\type real
  N58, \\field Vertex 19 Z-coordinate
       \\units m
       \\type real
  N59, \\field Vertex 20 X-coordinate
       \\units m
       \\type real
  N60, \\field Vertex 20 Y-coordinate
       \\units m
       \\type real
  N61, \\field Vertex 20 Z-coordinate
       \\units m
       \\type real
  N62, \\field Vertex 21 X-coordinate
       \\units m
       \\type real
  N63, \\field Vertex 21 Y-coordinate
       \\units m
       \\type real
  N64, \\field Vertex 21 Z-coordinate
       \\units m
       \\type real
  N65, \\field Vertex 22 X-coordinate
       \\units m
       \\type real
  N66, \\field Vertex 22 Y-coordinate
       \\units m
       \\type real
  N67, \\field Vertex 22 Z-coordinate
       \\units m
       \\type real
  N68, \\field Vertex 23 X-coordinate
       \\units m
       \\type real
  N69, \\field Vertex 23 Y-coordinate
       \\units m
       \\type real
  N70, \\field Vertex 23 Z-coordinate
       \\units m
       \\type real
  N71, \\field Vertex 24 X-coordinate
       \\units m
       \\type real
  N72, \\field Vertex 24 Y-coordinate
       \\units m
       \\type real
  N73, \\field Vertex 24 Z-coordinate
       \\units m
       \\type real
  N74, \\field Vertex 25 X-coordinate
       \\units m
       \\type real
  N75, \\field Vertex 25 Y-coordinate
       \\units m
       \\type real
  N76, \\field Vertex 25 Z-coordinate
       \\units m
       \\type real
  N77, \\field Vertex 26 X-coordinate
       \\units m
       \\type real
  N78, \\field Vertex 26 Y-coordinate
       \\units m
       \\type real
  N79, \\field Vertex 26 Z-coordinate
       \\units m
       \\type real
  N80, \\field Vertex 27 X-coordinate
       \\units m
       \\type real
  N81, \\field Vertex 27 Y-coordinate
       \\units m
       \\type real
  N82, \\field Vertex 27 Z-coordinate
       \\units m
       \\type real
  N83, \\field Vertex 28 X-coordinate
       \\units m
       \\type real
  N84, \\field Vertex 28 Y-coordinate
       \\units m
       \\type real
  N85, \\field Vertex 28 Z-coordinate
       \\units m
       \\type real
  N86, \\field Vertex 29 X-coordinate
       \\units m
       \\type real
  N87, \\field Vertex 29 Y-coordinate
       \\units m
       \\type real
  N88, \\field Vertex 29 Z-coordinate
       \\units m
       \\type real
  N89, \\field Vertex 30 X-coordinate
       \\units m
       \\type real
  N90, \\field Vertex 30 Y-coordinate
       \\units m
       \\type real
  N91, \\field Vertex 30 Z-coordinate
       \\units m
       \\type real
  N92, \\field Vertex 31 X-coordinate
       \\units m
       \\type real
  N93, \\field Vertex 31 Y-coordinate
       \\units m
       \\type real
  N94, \\field Vertex 31 Z-coordinate
       \\units m
       \\type real
  N95, \\field Vertex 32 X-coordinate
       \\units m
       \\type real
  N96, \\field Vertex 32 Y-coordinate
       \\units m
       \\type real
  N97, \\field Vertex 32 Z-coordinate
       \\units m
       \\type real
  N98, \\field Vertex 33 X-coordinate
       \\units m
       \\type real
  N99, \\field Vertex 33 Y-coordinate
       \\units m
       \\type real
  N100, \\field Vertex 33 Z-coordinate
       \\units m
       \\type real
  N101, \\field Vertex 34 X-coordinate
       \\units m
       \\type real
  N102, \\field Vertex 34 Y-coordinate
       \\units m
       \\type real
  N103, \\field Vertex 34 Z-coordinate
       \\units m
       \\type real
  N104, \\field Vertex 35 X-coordinate
       \\units m
       \\type real
  N105, \\field Vertex 35 Y-coordinate
       \\units m
       \\type real
  N106, \\field Vertex 35 Z-coordinate
       \\units m
       \\type real
  N107, \\field Vertex 36 X-coordinate
       \\units m
       \\type real
  N108, \\field Vertex 36 Y-coordinate
       \\units m
       \\type real
  N109, \\field Vertex 36 Z-coordinate
       \\units m
       \\type real
  N110, \\field Vertex 37 X-coordinate
       \\units m
       \\type real
  N111, \\field Vertex 37 Y-coordinate
       \\units m
       \\type real
  N112, \\field Vertex 37 Z-coordinate
       \\units m
       \\type real
  N113, \\field Vertex 38 X-coordinate
       \\units m
       \\type real
  N114, \\field Vertex 38 Y-coordinate
       \\units m
       \\type real
  N115, \\field Vertex 38 Z-coordinate
       \\units m
       \\type real
  N116, \\field Vertex 39 X-coordinate
       \\units m
       \\type real
  N117, \\field Vertex 39 Y-coordinate
       \\units m
       \\type real
  N118, \\field Vertex 39 Z-coordinate
       \\units m
       \\type real
  N119, \\field Vertex 40 X-coordinate
       \\units m
       \\type real
  N120, \\field Vertex 40 Y-coordinate
       \\units m
       \\type real
  N121, \\field Vertex 40 Z-coordinate
       \\units m
       \\type real
  N122, \\field Vertex 41 X-coordinate
       \\units m
       \\type real
  N123, \\field Vertex 41 Y-coordinate
       \\units m
       \\type real
  N124, \\field Vertex 41 Z-coordinate
       \\units m
       \\type real
  N125, \\field Vertex 42 X-coordinate
       \\units m
       \\type real
  N126, \\field Vertex 42 Y-coordinate
       \\units m
       \\type real
  N127, \\field Vertex 42 Z-coordinate
       \\units m
       \\type real
  N128, \\field Vertex 43 X-coordinate
       \\units m
       \\type real
  N129, \\field Vertex 43 Y-coordinate
       \\units m
       \\type real
  N130, \\field Vertex 43 Z-coordinate
       \\units m
       \\type real
  N131, \\field Vertex 44 X-coordinate
       \\units m
       \\type real
  N132, \\field Vertex 44 Y-coordinate
       \\units m
       \\type real
  N133, \\field Vertex 44 Z-coordinate
       \\units m
       \\type real
  N134, \\field Vertex 45 X-coordinate
       \\units m
       \\type real
  N135, \\field Vertex 45 Y-coordinate
       \\units m
       \\type real
  N136, \\field Vertex 45 Z-coordinate
       \\units m
       \\type real
  N137, \\field Vertex 46 X-coordinate
       \\units m
       \\type real
  N138, \\field Vertex 46 Y-coordinate
       \\units m
       \\type real
  N139, \\field Vertex 46 Z-coordinate
       \\units m
       \\type real
  N140, \\field Vertex 47 X-coordinate
       \\units m
       \\type real
  N141, \\field Vertex 47 Y-coordinate
       \\units m
       \\type real
  N142, \\field Vertex 47 Z-coordinate
       \\units m
       \\type real
  N143, \\field Vertex 48 X-coordinate
       \\units m
       \\type real
  N144, \\field Vertex 48 Y-coordinate
       \\units m
       \\type real
  N145, \\field Vertex 48 Z-coordinate
       \\units m
       \\type real
  N146, \\field Vertex 49 X-coordinate
       \\units m
       \\type real
  N147, \\field Vertex 49 Y-coordinate
       \\units m
       \\type real
  N148, \\field Vertex 49 Z-coordinate
       \\units m
       \\type real
  N149, \\field Vertex 50 X-coordinate
       \\units m
       \\type real
  N150, \\field Vertex 50 Y-coordinate
       \\units m
       \\type real
  N151, \\field Vertex 50 Z-coordinate
       \\units m
       \\type real
  N152, \\field Vertex 51 X-coordinate
       \\units m
       \\type real
  N153, \\field Vertex 51 Y-coordinate
       \\units m
       \\type real
  N154, \\field Vertex 51 Z-coordinate
       \\units m
       \\type real
  N155, \\field Vertex 52 X-coordinate
       \\units m
       \\type real
  N156, \\field Vertex 52 Y-coordinate
       \\units m
       \\type real
  N157, \\field Vertex 52 Z-coordinate
       \\units m
       \\type real
  N158, \\field Vertex 53 X-coordinate
       \\units m
       \\type real
  N159, \\field Vertex 53 Y-coordinate
       \\units m
       \\type real
  N160, \\field Vertex 53 Z-coordinate
       \\units m
       \\type real
  N161, \\field Vertex 54 X-coordinate
       \\units m
       \\type real
  N162, \\field Vertex 54 Y-coordinate
       \\units m
       \\type real
  N163, \\field Vertex 54 Z-coordinate
       \\units m
       \\type real
  N164, \\field Vertex 55 X-coordinate
       \\units m
       \\type real
  N165, \\field Vertex 55 Y-coordinate
       \\units m
       \\type real
  N166, \\field Vertex 55 Z-coordinate
       \\units m
       \\type real
  N167, \\field Vertex 56 X-coordinate
       \\units m
       \\type real
  N168, \\field Vertex 56 Y-coordinate
       \\units m
       \\type real
  N169, \\field Vertex 56 Z-coordinate
       \\units m
       \\type real
  N170, \\field Vertex 57 X-coordinate
       \\units m
       \\type real
  N171, \\field Vertex 57 Y-coordinate
       \\units m
       \\type real
  N172, \\field Vertex 57 Z-coordinate
       \\units m
       \\type real
  N173, \\field Vertex 58 X-coordinate
       \\units m
       \\type real
  N174, \\field Vertex 58 Y-coordinate
       \\units m
       \\type real
  N175, \\field Vertex 58 Z-coordinate
       \\units m
       \\type real
  N176, \\field Vertex 59 X-coordinate
       \\units m
       \\type real
  N177, \\field Vertex 59 Y-coordinate
       \\units m
       \\type real
  N178, \\field Vertex 59 Z-coordinate
       \\units m
       \\type real
  N179, \\field Vertex 60 X-coordinate
       \\units m
       \\type real
  N180, \\field Vertex 60 Y-coordinate
       \\units m
       \\type real
  N181, \\field Vertex 60 Z-coordinate
       \\units m
       \\type real
  N182, \\field Vertex 61 X-coordinate
       \\units m
       \\type real
  N183, \\field Vertex 61 Y-coordinate
       \\units m
       \\type real
  N184, \\field Vertex 61 Z-coordinate
       \\units m
       \\type real
  N185, \\field Vertex 62 X-coordinate
       \\units m
       \\type real
  N186, \\field Vertex 62 Y-coordinate
       \\units m
       \\type real
  N187, \\field Vertex 62 Z-coordinate
       \\units m
       \\type real
  N188, \\field Vertex 63 X-coordinate
       \\units m
       \\type real
  N189, \\field Vertex 63 Y-coordinate
       \\units m
       \\type real
  N190, \\field Vertex 63 Z-coordinate
       \\units m
       \\type real
  N191, \\field Vertex 64 X-coordinate
       \\units m
       \\type real
  N192, \\field Vertex 64 Y-coordinate
       \\units m
       \\type real
  N193, \\field Vertex 64 Z-coordinate
       \\units m
       \\type real
  N194, \\field Vertex 65 X-coordinate
       \\units m
       \\type real
  N195, \\field Vertex 65 Y-coordinate
       \\units m
       \\type real
  N196, \\field Vertex 65 Z-coordinate
       \\units m
       \\type real
  N197, \\field Vertex 66 X-coordinate
       \\units m
       \\type real
  N198, \\field Vertex 66 Y-coordinate
       \\units m
       \\type real
  N199, \\field Vertex 66 Z-coordinate
       \\units m
       \\type real
  N200, \\field Vertex 67 X-coordinate
       \\units m
       \\type real
  N201, \\field Vertex 67 Y-coordinate
       \\units m
       \\type real
  N202, \\field Vertex 67 Z-coordinate
       \\units m
       \\type real
  N203, \\field Vertex 68 X-coordinate
       \\units m
       \\type real
  N204, \\field Vertex 68 Y-coordinate
       \\units m
       \\type real
  N205, \\field Vertex 68 Z-coordinate
       \\units m
       \\type real
  N206, \\field Vertex 69 X-coordinate
       \\units m
       \\type real
  N207, \\field Vertex 69 Y-coordinate
       \\units m
       \\type real
  N208, \\field Vertex 69 Z-coordinate
       \\units m
       \\type real
  N209, \\field Vertex 70 X-coordinate
       \\units m
       \\type real
  N210, \\field Vertex 70 Y-coordinate
       \\units m
       \\type real
  N211, \\field Vertex 70 Z-coordinate
       \\units m
       \\type real
  N212, \\field Vertex 71 X-coordinate
       \\units m
       \\type real
  N213, \\field Vertex 71 Y-coordinate
       \\units m
       \\type real
  N214, \\field Vertex 71 Z-coordinate
       \\units m
       \\type real
  N215, \\field Vertex 72 X-coordinate
       \\units m
       \\type real
  N216, \\field Vertex 72 Y-coordinate
       \\units m
       \\type real
  N217, \\field Vertex 72 Z-coordinate
       \\units m
       \\type real
  N218, \\field Vertex 73 X-coordinate
       \\units m
       \\type real
  N219, \\field Vertex 73 Y-coordinate
       \\units m
       \\type real
  N220, \\field Vertex 73 Z-coordinate
       \\units m
       \\type real
  N221, \\field Vertex 74 X-coordinate
       \\units m
       \\type real
  N222, \\field Vertex 74 Y-coordinate
       \\units m
       \\type real
  N223, \\field Vertex 74 Z-coordinate
       \\units m
       \\type real
  N224, \\field Vertex 75 X-coordinate
       \\units m
       \\type real
  N225, \\field Vertex 75 Y-coordinate
       \\units m
       \\type real
  N226, \\field Vertex 75 Z-coordinate
       \\units m
       \\type real
  N227, \\field Vertex 76 X-coordinate
       \\units m
       \\type real
  N228, \\field Vertex 76 Y-coordinate
       \\units m
       \\type real
  N229, \\field Vertex 76 Z-coordinate
       \\units m
       \\type real
  N230, \\field Vertex 77 X-coordinate
       \\units m
       \\type real
  N231, \\field Vertex 77 Y-coordinate
       \\units m
       \\type real
  N232, \\field Vertex 77 Z-coordinate
       \\units m
       \\type real
  N233, \\field Vertex 78 X-coordinate
       \\units m
       \\type real
  N234, \\field Vertex 78 Y-coordinate
       \\units m
       \\type real
  N235, \\field Vertex 78 Z-coordinate
       \\units m
       \\type real
  N236, \\field Vertex 79 X-coordinate
       \\units m
       \\type real
  N237, \\field Vertex 79 Y-coordinate
       \\units m
       \\type real
  N238, \\field Vertex 79 Z-coordinate
       \\units m
       \\type real
  N239, \\field Vertex 80 X-coordinate
       \\units m
       \\type real
  N240, \\field Vertex 80 Y-coordinate
       \\units m
       \\type real
  N241, \\field Vertex 80 Z-coordinate
       \\units m
       \\type real
  N242, \\field Vertex 81 X-coordinate
       \\units m
       \\type real
  N243, \\field Vertex 81 Y-coordinate
       \\units m
       \\type real
  N244, \\field Vertex 81 Z-coordinate
       \\units m
       \\type real
  N245, \\field Vertex 82 X-coordinate
       \\units m
       \\type real
  N246, \\field Vertex 82 Y-coordinate
       \\units m
       \\type real
  N247, \\field Vertex 82 Z-coordinate
       \\units m
       \\type real
  N248, \\field Vertex 83 X-coordinate
       \\units m
       \\type real
  N249, \\field Vertex 83 Y-coordinate
       \\units m
       \\type real
  N250, \\field Vertex 83 Z-coordinate
       \\units m
       \\type real
  N251, \\field Vertex 84 X-coordinate
       \\units m
       \\type real
  N252, \\field Vertex 84 Y-coordinate
       \\units m
       \\type real
  N253, \\field Vertex 84 Z-coordinate
       \\units m
       \\type real
  N254, \\field Vertex 85 X-coordinate
       \\units m
       \\type real
  N255, \\field Vertex 85 Y-coordinate
       \\units m
       \\type real
  N256, \\field Vertex 85 Z-coordinate
       \\units m
       \\type real
  N257, \\field Vertex 86 X-coordinate
       \\units m
       \\type real
  N258, \\field Vertex 86 Y-coordinate
       \\units m
       \\type real
  N259, \\field Vertex 86 Z-coordinate
       \\units m
       \\type real
  N260, \\field Vertex 87 X-coordinate
       \\units m
       \\type real
  N261, \\field Vertex 87 Y-coordinate
       \\units m
       \\type real
  N262, \\field Vertex 87 Z-coordinate
       \\units m
       \\type real
  N263, \\field Vertex 88 X-coordinate
       \\units m
       \\type real
  N264, \\field Vertex 88 Y-coordinate
       \\units m
       \\type real
  N265, \\field Vertex 88 Z-coordinate
       \\units m
       \\type real
  N266, \\field Vertex 89 X-coordinate
       \\units m
       \\type real
  N267, \\field Vertex 89 Y-coordinate
       \\units m
       \\type real
  N268, \\field Vertex 89 Z-coordinate
       \\units m
       \\type real
  N269, \\field Vertex 90 X-coordinate
       \\units m
       \\type real
  N270, \\field Vertex 90 Y-coordinate
       \\units m
       \\type real
  N271, \\field Vertex 90 Z-coordinate
       \\units m
       \\type real
  N272, \\field Vertex 91 X-coordinate
       \\units m
       \\type real
  N273, \\field Vertex 91 Y-coordinate
       \\units m
       \\type real
  N274, \\field Vertex 91 Z-coordinate
       \\units m
       \\type real
  N275, \\field Vertex 92 X-coordinate
       \\units m
       \\type real
  N276, \\field Vertex 92 Y-coordinate
       \\units m
       \\type real
  N277, \\field Vertex 92 Z-coordinate
       \\units m
       \\type real
  N278, \\field Vertex 93 X-coordinate
       \\units m
       \\type real
  N279, \\field Vertex 93 Y-coordinate
       \\units m
       \\type real
  N280, \\field Vertex 93 Z-coordinate
       \\units m
       \\type real
  N281, \\field Vertex 94 X-coordinate
       \\units m
       \\type real
  N282, \\field Vertex 94 Y-coordinate
       \\units m
       \\type real
  N283, \\field Vertex 94 Z-coordinate
       \\units m
       \\type real
  N284, \\field Vertex 95 X-coordinate
       \\units m
       \\type real
  N285, \\field Vertex 95 Y-coordinate
       \\units m
       \\type real
  N286, \\field Vertex 95 Z-coordinate
       \\units m
       \\type real
  N287, \\field Vertex 96 X-coordinate
       \\units m
       \\type real
  N288, \\field Vertex 96 Y-coordinate
       \\units m
       \\type real
  N289, \\field Vertex 96 Z-coordinate
       \\units m
       \\type real
  N290, \\field Vertex 97 X-coordinate
       \\units m
       \\type real
  N291, \\field Vertex 97 Y-coordinate
       \\units m
       \\type real
  N292, \\field Vertex 97 Z-coordinate
       \\units m
       \\type real
  N293, \\field Vertex 98 X-coordinate
       \\units m
       \\type real
  N294, \\field Vertex 98 Y-coordinate
       \\units m
       \\type real
  N295, \\field Vertex 98 Z-coordinate
       \\units m
       \\type real
  N296, \\field Vertex 99 X-coordinate
       \\units m
       \\type real
  N297, \\field Vertex 99 Y-coordinate
       \\units m
       \\type real
  N298, \\field Vertex 99 Z-coordinate
       \\units m
       \\type real
  N299, \\field Vertex 100 X-coordinate
       \\units m
       \\type real
  N300, \\field Vertex 100 Y-coordinate
       \\units m
       \\type real
  N301, \\field Vertex 100 Z-coordinate
       \\units m
       \\type real
  N302, \\field Vertex 101 X-coordinate
       \\units m
       \\type real
  N303, \\field Vertex 101 Y-coordinate
       \\units m
       \\type real
  N304, \\field Vertex 101 Z-coordinate
       \\units m
       \\type real
  N305, \\field Vertex 102 X-coordinate
       \\units m
       \\type real
  N306, \\field Vertex 102 Y-coordinate
       \\units m
       \\type real
  N307, \\field Vertex 102 Z-coordinate
       \\units m
       \\type real
  N308, \\field Vertex 103 X-coordinate
       \\units m
       \\type real
  N309, \\field Vertex 103 Y-coordinate
       \\units m
       \\type real
  N310, \\field Vertex 103 Z-coordinate
       \\units m
       \\type real
  N311, \\field Vertex 104 X-coordinate
       \\units m
       \\type real
  N312, \\field Vertex 104 Y-coordinate
       \\units m
       \\type real
  N313, \\field Vertex 104 Z-coordinate
       \\units m
       \\type real
  N314, \\field Vertex 105 X-coordinate
       \\units m
       \\type real
  N315, \\field Vertex 105 Y-coordinate
       \\units m
       \\type real
  N316, \\field Vertex 105 Z-coordinate
       \\units m
       \\type real
  N317, \\field Vertex 106 X-coordinate
       \\units m
       \\type real
  N318, \\field Vertex 106 Y-coordinate
       \\units m
       \\type real
  N319, \\field Vertex 106 Z-coordinate
       \\units m
       \\type real
  N320, \\field Vertex 107 X-coordinate
       \\units m
       \\type real
  N321, \\field Vertex 107 Y-coordinate
       \\units m
       \\type real
  N322, \\field Vertex 107 Z-coordinate
       \\units m
       \\type real
  N323, \\field Vertex 108 X-coordinate
       \\units m
       \\type real
  N324, \\field Vertex 108 Y-coordinate
       \\units m
       \\type real
  N325, \\field Vertex 108 Z-coordinate
       \\units m
       \\type real
  N326, \\field Vertex 109 X-coordinate
       \\units m
       \\type real
  N327, \\field Vertex 109 Y-coordinate
       \\units m
       \\type real
  N328, \\field Vertex 109 Z-coordinate
       \\units m
       \\type real
  N329, \\field Vertex 110 X-coordinate
       \\units m
       \\type real
  N330, \\field Vertex 110 Y-coordinate
       \\units m
       \\type real
  N331, \\field Vertex 110 Z-coordinate
       \\units m
       \\type real
  N332, \\field Vertex 111 X-coordinate
       \\units m
       \\type real
  N333, \\field Vertex 111 Y-coordinate
       \\units m
       \\type real
  N334, \\field Vertex 111 Z-coordinate
       \\units m
       \\type real
  N335, \\field Vertex 112 X-coordinate
       \\units m
       \\type real
  N336, \\field Vertex 112 Y-coordinate
       \\units m
       \\type real
  N337, \\field Vertex 112 Z-coordinate
       \\units m
       \\type real
  N338, \\field Vertex 113 X-coordinate
       \\units m
       \\type real
  N339, \\field Vertex 113 Y-coordinate
       \\units m
       \\type real
  N340, \\field Vertex 113 Z-coordinate
       \\units m
       \\type real
  N341, \\field Vertex 114 X-coordinate
       \\units m
       \\type real
  N342, \\field Vertex 114 Y-coordinate
       \\units m
       \\type real
  N343, \\field Vertex 114 Z-coordinate
       \\units m
       \\type real
  N344, \\field Vertex 115 X-coordinate
       \\units m
       \\type real
  N345, \\field Vertex 115 Y-coordinate
       \\units m
       \\type real
  N346, \\field Vertex 115 Z-coordinate
       \\units m
       \\type real
  N347, \\field Vertex 116 X-coordinate
       \\units m
       \\type real
  N348, \\field Vertex 116 Y-coordinate
       \\units m
       \\type real
  N349, \\field Vertex 116 Z-coordinate
       \\units m
       \\type real
  N350, \\field Vertex 117 X-coordinate
       \\units m
       \\type real
  N351, \\field Vertex 117 Y-coordinate
       \\units m
       \\type real
  N352, \\field Vertex 117 Z-coordinate
       \\units m
       \\type real
  N353, \\field Vertex 118 X-coordinate
       \\units m
       \\type real
  N354, \\field Vertex 118 Y-coordinate
       \\units m
       \\type real
  N355, \\field Vertex 118 Z-coordinate
       \\units m
       \\type real
  N356, \\field Vertex 119 X-coordinate
       \\units m
       \\type real
  N357, \\field Vertex 119 Y-coordinate
       \\units m
       \\type real
  N358, \\field Vertex 119 Z-coordinate
       \\units m
       \\type real
  N359, \\field Vertex 120 X-coordinate
       \\units m
       \\type real
  N360, \\field Vertex 120 Y-coordinate
       \\units m
       \\type real
  N361; \\field Vertex 120 Z-coordinate
       \\units m
       \\type real

ShadingProperty:Reflectance,
       \\min-fields 3
       \\memo If this object is not defined for a shading surface the default values
       \\memo listed in following fields will be used in the solar reflection calculation.
  A1,  \\field Shading Surface Name
       \\required-field
       \\type object-list
       \\object-list AllShadingSurfNames
  N1,  \\field Diffuse Solar Reflectance of Unglazed Part of Shading Surface
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.2
  N2,  \\field Diffuse Visible Reflectance of Unglazed Part of Shading Surface
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.2
  N3,  \\field Fraction of Shading Surface That Is Glazed
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  A2;  \\field Glazing Construction Name
       \\note Required if Fraction of Shading Surface That Is Glazed > 0.0

\\group Advanced Construction, Surface, Zone Concepts

SurfaceProperty:HeatTransferAlgorithm,
       \\memo Determines which Heat Balance Algorithm will be used for a specific surface
       \\memo Allows selectively overriding the global setting in HeatBalanceAlgorithm
       \\memo CTF (Conduction Transfer Functions),
       \\memo EMPD (Effective Moisture Penetration Depth with Conduction Transfer Functions).
       \\memo Advanced/Research Usage: CondFD (Conduction Finite Difference)
       \\memo Advanced/Research Usage: HAMT (Combined Heat And Moisture Finite Element)
       \\min-fields 2
  A1 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A2 ; \\field Algorithm
       \\required-field
       \\type choice
       \\key ConductionTransferFunction
       \\key MoisturePenetrationDepthConductionTransferFunction
       \\key ConductionFiniteDifference
       \\key CombinedHeatAndMoistureFiniteElement
       \\default ConductionTransferFunction

SurfaceProperty:HeatTransferAlgorithm:MultipleSurface,
       \\memo Determines which Heat Balance Algorithm will be used for a group of surface types
       \\memo Allows selectively overriding the global setting in HeatBalanceAlgorithm
       \\memo CTF (Conduction Transfer Functions),
       \\memo EMPD (Effective Moisture Penetration Depth with Conduction Transfer Functions).
       \\memo Advanced/Research Usage: CondFD (Conduction Finite Difference)
       \\memo Advanced/Research Usage: HAMT (Combined Heat And Moisture Finite Element)
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Surface Type
       \\required-field
       \\type choice
       \\key AllExteriorSurfaces
       \\key AllExteriorWalls
       \\key AllExteriorRoofs
       \\key AllExteriorFloors
       \\key AllGroundContactSurfaces
       \\key AllInteriorSurfaces
       \\key AllInteriorWalls
       \\key AllInteriorCeilings
       \\key AllInteriorFloors
  A3 ; \\field Algorithm
       \\required-field
       \\type choice
       \\key ConductionTransferFunction
       \\key MoisturePenetrationDepthConductionTransferFunction
       \\key ConductionFiniteDifference
       \\key CombinedHeatAndMoistureFiniteElement
       \\default ConductionTransferFunction

SurfaceProperty:HeatTransferAlgorithm:SurfaceList,
       \\min-fields 3
       \\extensible:1
       \\memo Determines which Heat Balance Algorithm will be used for a list of surfaces
       \\memo Allows selectively overriding the global setting in HeatBalanceAlgorithm
       \\memo CTF (Conduction Transfer Functions),
       \\memo EMPD (Effective Moisture Penetration Depth with Conduction Transfer Functions).
       \\memo Advanced/Research Usage: CondFD (Conduction Finite Difference)
       \\memo Advanced/Research Usage: HAMT (Combined Heat And Moisture Finite Element)
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Algorithm
       \\required-field
       \\type choice
       \\key ConductionTransferFunction
       \\key MoisturePenetrationDepthConductionTransferFunction
       \\key ConductionFiniteDifference
       \\key CombinedHeatAndMoistureFiniteElement
       \\default ConductionTransferFunction
  A3 , \\field Surface Name 1
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  A4 , \\field Surface Name 2
       \\type object-list
       \\object-list SurfaceNames
  A5 , \\field Surface Name 3
       \\type object-list
       \\object-list SurfaceNames
  A6 , \\field Surface Name 4
       \\type object-list
       \\object-list SurfaceNames
  A7 , \\field Surface Name 5
       \\type object-list
       \\object-list SurfaceNames
  A8 ; \\field Surface Name 6
       \\type object-list
       \\object-list SurfaceNames

SurfaceProperty:HeatTransferAlgorithm:Construction,
       \\memo Determines which Heat Balance Algorithm will be used for surfaces that have a specific type of construction
       \\memo Allows selectively overriding the global setting in HeatBalanceAlgorithm
       \\memo CTF (Conduction Transfer Functions),
       \\memo EMPD (Effective Moisture Penetration Depth with Conduction Transfer Functions).
       \\memo Advanced/Research Usage: CondFD (Conduction Finite Difference)
       \\memo Advanced/Research Usage: HAMT (Combined Heat And Moisture Finite Element)
       \\min-fields 3
  A1 , \\field Name
       \\type alpha
  A2 , \\field Algorithm
       \\required-field
       \\type choice
       \\key ConductionTransferFunction
       \\key MoisturePenetrationDepthConductionTransferFunction
       \\key ConductionFiniteDifference
       \\key CombinedHeatAndMoistureFiniteElement
       \\default ConductionTransferFunction
  A3 ; \\field Construction Name
       \\required-field
       \\type object-list
       \\object-list ConstructionNames

SurfaceControl:MovableInsulation,
      \\memo Exterior or Interior Insulation on opaque surfaces
  A1, \\field Insulation Type
      \\required-field
      \\type choice
      \\key Outside
      \\key Inside
  A2, \\field Surface Name
      \\required-field
      \\type object-list
      \\object-list SurfaceNames
  A3, \\field Material Name
      \\required-field
      \\object-list MaterialName
  A4; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

SurfaceProperty:OtherSideCoefficients,
      \\memo This object sets the other side conditions for a surface in a variety of ways.
      \\min-fields 8
  A1, \\field Name
      \\required-field
      \\reference OSCNames
      \\reference OutFaceEnvNames
  N1, \\field Combined Convective/Radiative Film Coefficient
      \\units W/m2-K
      \\required-field
      \\type real
      \\note if>0, this field becomes the exterior convective/radiative film coefficient
      \\note and the other fields are used to calculate the outdoor air temperature
      \\note then exterior surface temperature based on outdoor air and specified coefficient
      \\note if<=0, then remaining fields calculate the outside surface temperature
      \\note The following fields are used in the equation:
      \\note OtherSideTemp=N2*N3 + N4*OutdoorDry-bulb + N5*GroundTemp + N6*WindSpeed*OutdoorDry-bulb + N7*TempZone + N9*TempPrev
  N2, \\field Constant Temperature
      \\units C
      \\type real
      \\default 0
      \\note This parameter will be overwritten by the values from the Constant Temperature Schedule Name (below) if one is present
  N3, \\field Constant Temperature Coefficient
      \\note This coefficient is used even with a Schedule.  It should normally be 1.0 in that case.
      \\note This field is ignored if Sinusoidal Variation of Constant Temperature Coefficient = Yes.
      \\default 1
  N4, \\field External Dry-Bulb Temperature Coefficient
      \\type real
      \\default 0
  N5, \\field Ground Temperature Coefficient
      \\type real
      \\default 0
  N6, \\field Wind Speed Coefficient
      \\type real
      \\default 0
  N7, \\field Zone Air Temperature Coefficient
      \\type real
      \\default 0
  A2, \\field Constant Temperature Schedule Name
      \\note Name of schedule for values of constant temperature.
      \\note Schedule values replace any value specified in the field Constant Temperature.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Sinusoidal Variation of Constant Temperature Coefficient
      \\note Optionally used to vary Constant Temperature Coefficient with unitary sine wave
      \\type choice
      \\key Yes
      \\key No
      \\default No
  N8, \\field Period of Sinusoidal Variation
      \\note Use with sinusoidal variation to define the time period
      \\type real
      \\units hr
      \\default 24
      \\minimum> 0
  N9, \\field Previous Other Side Temperature Coefficient
      \\note This coefficient multiplies the other side temperature result from the
      \\note previous zone timestep
      \\type real
      \\default 0
 N10, \\field Minimum Other Side Temperature Limit
      \\note This field specifies a lower limit for the other side temperature result.
      \\note Blank indicates no limit
      \\type real
      \\units C
 N11; \\field Maximum Other Side Temperature Limit
      \\note This field specifies an upper limit for the other side temperature result.
      \\note Blank indicates no limit
      \\type real
      \\units C


SurfaceProperty:OtherSideConditionsModel,
      \\memo This object sets up modifying the other side conditions for a surface from other model results.
  A1, \\field Name
      \\required-field
      \\reference OSCMNames
      \\reference OutFaceEnvNames
  A2; \\field Type of Modeling
      \\type choice
      \\key GapConvectionRadiation
      \\key UndergroundPipingSystemSurface
      \\note GapConvectionRadiation provides boundary conditions for convection
      \\note  and linearized thermal radiation across a gap or cavity
      \\note  on the other side of the surface that are modeled sperately.
      \\note UndergroundPipingSystemSurface provides boundary conditions for
      \\note  surfaces in contact with PipingSystem:Underground domains
      \\default GapConvectionRadiation


SurfaceConvectionAlgorithm:Inside:AdaptiveModelSelections,
       \\unique-object
       \\memo Options to change the individual convection model equations for dynamic selection when using AdaptiveConvectiongAlgorithm
       \\memo This object is only needed to make changes to the default model selections for any or all of the surface categories.
       \\memo This object is for the inside face, the side of the surface facing a thermal zone.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Simple Bouyancy Vertical Wall Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
       \\note This is for vertical walls
       \\type choice
       \\default FohannoPolidoriVerticalWall
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq3WallAwayFromHeat
       \\key KhalifaEq6NonHeatedWalls
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A3 , \\field Simple Bouyancy Vertical Wall User Curve Name
       \\note The SurfaceConvectionAlgorithm:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A4 , \\field Simple Bouyancy Stable Horizontal Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
       \\note This is for horizontal surfaces with heat flow directed for stable thermal stratification
       \\type choice
       \\default AlamdariHammondStableHorizontal
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A5 , \\field Simple Bouyancy Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A6 , \\field Simple Bouyancy Unstable Horizontal Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
        \\note This is for passive horizontal surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default AlamdariHammondUnstableHorizontal
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A7 , \\field Simple Bouyancy Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A8 , \\field Simple Bouyancy Stable Tilted Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
       \\note This is for tilted surfaces with heat flow for stable thermal stratification
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A9 , \\field Simple Bouyancy Stable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A10 , \\field Simple Bouyancy Unstable Tilted Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
       \\note This is for tilted surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A11, \\field Simple Bouyancy Unstable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A12, \\field Simple Bouyancy Windows Equation Source
       \\note Applies to zone with no HVAC or when HVAC is off
       \\note This is for all window surfaces
       \\type choice
       \\default ISO15099Windows
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key KaradagChilledCeiling
       \\key ISO15099Windows
       \\key UserCurve
  A13, \\field Simple Bouyancy Windows Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A14, \\field Floor Heat Ceiling Cool Vertical Wall Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for vertical walls
       \\type choice
       \\default KhalifaEq3WallAwayFromHeat
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq3WallAwayFromHeat
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A15, \\field Floor Heat Ceiling Cool Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A16, \\field Floor Heat Ceiling Cool Stable Horizontal Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for passive horizontal surfaces with heat flow for stable thermal stratification
       \\type choice
       \\default AlamdariHammondStableHorizontal
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A17, \\field Floor Heat Ceiling Cool Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A18, \\field Floor Heat Ceiling Cool Unstable Horizontal Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for passive horizontal surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default KhalifaEq4CeilingAwayFromHeat
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key KhalifaEq4CeilingAwayFromHeat
       \\key UserCurve
  A19, \\field Floor Heat Ceiling Cool Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A20, \\field Floor Heat Ceiling Cool Heated Floor Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for a floor with active heating elements
       \\type choice
       \\default AwbiHattonHeatedFloor
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key AwbiHattonHeatedFloor
       \\key UserCurve
  A21, \\field Floor Heat Ceiling Cool Heated Floor Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A22, \\field Floor Heat Ceiling Cool Chilled Ceiling Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for a ceiling with active cooling elements
       \\type choice
       \\default KaradagChilledCeiling
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key KaradagChilledCeiling
       \\key UserCurve
  A23, \\field Floor Heat Ceiling Cool Chilled Ceiling Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A24, \\field Floor Heat Ceiling Cool Stable Tilted Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for tilted surfaces with heat flow for stable thermal stratification
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key ISO15099Windows
       \\key UserCurve
  A25, \\field Floor Heat Ceiling Cool Stable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A26, \\field Floor Heat Ceiling Cool Unstable Tilted Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for tilted surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key ISO15099Windows
       \\key UserCurve
  A27, \\field Floor Heat Ceiling Cool Unstable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A28, \\field Floor Heat Ceiling Cool Window Equation Source
       \\note Applies to zone with in-floor heating and/or in-ceiling cooling
       \\note This is for all window surfaces
       \\type choice
       \\default ISO15099Windows
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A29, \\field Floor Heat Ceiling Cool Window Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A30, \\field Wall Panel Heating Vertical Wall Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for vertical walls that are not actively heated
       \\type choice
       \\default KhalifaEq6NonHeatedWalls
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq6NonHeatedWalls
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A31, \\field Wall Panel Heating Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A32, \\field Wall Panel Heating Heated Wall Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for vertical walls that are being actively heated
       \\type choice
       \\default AwbiHattonHeatedWall
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq5WallNearHeat
       \\key AwbiHattonHeatedWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A33, \\field Wall Panel Heating Heated Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A34, \\field Wall Panel Heating Stable Horizontal Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for horizontal surfaces with heat flow directed for stable thermal stratification
       \\type choice
       \\default AlamdariHammondStableHorizontal
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A35, \\field Wall Panel Heating Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A36, \\field Wall Panel Heating Unstable Horizontal Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for horizontal surfaces with heat flow directed for unstable thermal stratification
       \\type choice
       \\default KhalifaEq7Ceiling
       \\key ASHRAEVerticalWall
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key KhalifaEq7Ceiling
       \\key KaradagChilledCeiling
       \\key UserCurve
  A37, \\field Wall Panel Heating Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A38, \\field Wall Panel Heating Stable Tilted Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for tilted surfaces with heat flow for stable thermal stratification
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key ISO15099Windows
       \\key UserCurve
  A39, \\field Wall Panel Heating Stable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A40, \\field Wall Panel Heating Unstable Tilted Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for tilted surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key ISO15099Windows
       \\key UserCurve
  A41, \\field Wall Panel Heating Unstable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A42, \\field Wall Panel Heating Window Equation Source
       \\note Applies to zone with in-wall panel heating
       \\note This is for all window surfaces
       \\type choice
       \\default ISO15099Windows
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A43, \\field Wall Panel Heating Window Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A44, \\field Convective Zone Heater Vertical Wall Equation Source
       \\note Applies to zone with convective heater
       \\note This is for vertical walls not directly affected by heater
       \\type choice
       \\default FohannoPolidoriVerticalWall
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq3WallAwayFromHeat
       \\key KhalifaEq6NonHeatedWalls
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A45, \\field Convective Zone Heater Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A46, \\field Convective Zone Heater Vertical Walls Near Heater Equation Source
       \\note Applies to zone with convective heater
       \\note This is for vertical walls that are directly affected by heater
       \\note Walls are considered "near" when listed in field set for Fraction of Radiant Energy to Surface
       \\type choice
       \\default KhalifaEq5WallNearHeat
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq5WallNearHeat
       \\key AwbiHattonHeatedWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A47, \\field Convective Zone Heater Vertical Walls Near Heater Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A48, \\field Convective Zone Heater Stable Horizontal Equation Source
       \\note Applies to zone with convective heater
       \\note This is for horizontal surfaces with heat flow directed for stable thermal stratification
       \\type choice
       \\default AlamdariHammondStableHorizontal
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A49, \\field Convective Zone Heater Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A50, \\field Convective Zone Heater Unstable Horizontal Equation Source
       \\note Applies to zone with convective heater
       \\note This is for horizontal surfaces with heat flow directed for unstable thermal stratification
       \\type choice
       \\default KhalifaEq7Ceiling
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key KhalifaEq4CeilingAwayFromHeat
       \\key KhalifaEq7Ceiling
       \\key UserCurve
  A51, \\field Convective Zone Heater Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A52, \\field Convective Zone Heater Stable Tilted Equation Source
       \\note Applies to zone with convective heater
       \\note This is for tilted surfaces with heat flow for stable thermal stratification
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A53, \\field Convective Zone Heater Stable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A54, \\field Convective Zone Heater Unstable Tilted Equation Source
       \\note Applies to zone with convective heater
       \\note This is for tilted surfaces with heat flow for unstable thermal stratification
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A55, \\field Convective Zone Heater Unstable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A56, \\field Convective Zone Heater Windows Equation Source
       \\note Applies to zone with convective heater
       \\note This is for all window surfaces
       \\type choice
       \\default ISO15099Windows
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key KhalifaEq3WallAwayFromHeat
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
  A57, \\field Convective Zone Heater Windows Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A58, \\field Central Air Diffuser Wall Equation Source
       \\note Applies to zone with mechanical forced central air with diffusers
       \\note This is for all wall surfaces
       \\type choice
       \\default GoldsteinNovoselacCeilingDiffuserWalls
       \\key ASHRAEVerticalWall
       \\key FisherPedersenCeilingDiffuserWalls
       \\key AlamdariHammondVerticalWall
       \\key BeausoleilMorrisonMixedAssistedWall
       \\key BeausoleilMorrisonMixedOpposingWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key GoldsteinNovoselacCeilingDiffuserWalls
       \\key UserCurve
  A59, \\field Central Air Diffuser Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A60, \\field Central Air Diffuser Ceiling Equation Source
       \\note Applies to zone with mechanical forced central air with diffusers
       \\note This is for all ceiling surfaces
       \\type choice
       \\default FisherPedersenCeilingDiffuserCeiling
       \\key FisherPedersenCeilingDiffuserCeiling
       \\key BeausoleilMorrisonMixedStableCeiling
       \\key BeausoleilMorrisonMixedUnstableCeiling
       \\key UserCurve
  A61, \\field Central Air Diffuser Ceiling Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A62, \\field Central Air Diffuser Floor Equation Source
       \\note Applies to zone with mechanical forced central air with diffusers
       \\note This is for all floor surfaces
       \\type choice
       \\default GoldsteinNovoselacCeilingDiffuserFloor
       \\key FisherPedersenCeilingDiffuserFloor
       \\key BeausoleilMorrisonMixedStableFloor
       \\key BeausoleilMorrisonMixedUnstableFloor
       \\key GoldsteinNovoselacCeilingDiffuserFloor
       \\key UserCurve
  A63, \\field Central Air Diffuser Floor Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A64, \\field Central Air Diffuser Window Equation Source
       \\note Applies to zone with mechanical forced central air with diffusers
       \\note This is for all window surfaces
       \\type choice
       \\default GoldsteinNovoselacCeilingDiffuserWindow
       \\key ASHRAEVerticalWall
       \\key FisherPedersenCeilingDiffuserWalls
       \\key BeausoleilMorrisonMixedAssistedWall
       \\key BeausoleilMorrisonMixedOpposingWall
       \\key FohannoPolidoriVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key ISO15099Windows
       \\key GoldsteinNovoselacCeilingDiffuserWindow
       \\key UserCurve
  A65, \\field Central Air Diffuser Window Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A66, \\field Mechanical Zone Fan Circulation Vertical Wall Equation Source
       \\note reference choice fields
       \\type choice
       \\default KhalifaEq3WallAwayFromHeat
       \\key KhalifaEq3WallAwayFromHeat
       \\key ASHRAEVerticalWall
       \\key FisherPedersenCeilingDiffuserWalls
       \\key AlamdariHammondVerticalWall
       \\key BeausoleilMorrisonMixedAssistedWall
       \\key BeausoleilMorrisonMixedOpposingWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key GoldsteinNovoselacCeilingDiffuserWalls
       \\key UserCurve
  A67, \\field Mechanical Zone Fan Circulation Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A68, \\field Mechanical Zone Fan Circulation Stable Horizontal Equation Source
       \\note reference choice fields
       \\type choice
       \\default AlamdariHammondStableHorizontal
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A69, \\field Mechanical Zone Fan Circulation Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A70, \\field Mechanical Zone Fan Circulation Unstable Horizontal Equation Source
       \\note reference choice fields
       \\type choice
       \\default KhalifaEq4CeilingAwayFromHeat
       \\key KhalifaEq4CeilingAwayFromHeat
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A71, \\field Mechanical Zone Fan Circulation Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A72, \\field Mechanical Zone Fan Circulation Stable Tilted Equation Source
       \\note reference choice fields
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key UserCurve
  A73, \\field Mechanical Zone Fan Circulation Stable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A74, \\field Mechanical Zone Fan Circulation Unstable Tilted Equation Source
       \\note reference choice fields
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A75, \\field Mechanical Zone Fan Circulation Unstable Tilted Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A76, \\field Mechanical Zone Fan Circulation Window Equation Source
       \\note reference choice fields
       \\type choice
       \\default ISO15099Windows
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key GoldsteinNovoselacCeilingDiffuserWindow
       \\key UserCurve
  A77, \\field Mechanical Zone Fan Circulation Window Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A78, \\field Mixed Regime Bouyancy Assisting Flow on Walls Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedAssistedWall
       \\key BeausoleilMorrisonMixedAssistedWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key ASHRAEVerticalWall
       \\key FisherPedersenCeilingDiffuserWalls
       \\key GoldsteinNovoselacCeilingDiffuserWalls
       \\key UserCurve
  A79, \\field Mixed Regime Bouyancy Assisting Flow on Walls Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A80, \\field Mixed Regime Bouyancy Oppossing Flow on Walls Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedOpposingWall
       \\key BeausoleilMorrisonMixedOpposingWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key ASHRAEVerticalWall
       \\key FisherPedersenCeilingDiffuserWalls
       \\key GoldsteinNovoselacCeilingDiffuserWalls
       \\key UserCurve
  A81, \\field Mixed Regime Bouyancy Oppossing Flow on Walls Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A82, \\field Mixed Regime Stable Floor Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedStableFloor
       \\key BeausoleilMorrisonMixedStableFloor
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A83, \\field Mixed Regime Stable Floor Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A84, \\field Mixed Regime Unstable Floor Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedUnstableFloor
       \\key BeausoleilMorrisonMixedUnstableFloor
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A85, \\field Mixed Regime Unstable Floor Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A86, \\field Mixed Regime Stable Ceiling Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedStableCeiling
       \\key BeausoleilMorrisonMixedStableCeiling
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
  A87, \\field Mixed Regime Stable Ceiling Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A88, \\field Mixed Regime Unstable Ceiling Equation Source
       \\note reference choice fields
       \\type choice
       \\default BeausoleilMorrisonMixedUnstableCeiling
       \\key BeausoleilMorrisonMixedUnstableCeiling
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
  A89, \\field Mixed Regime Unstable Ceiling Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels
  A90, \\field Mixed Regime Window Equation Source
       \\note reference choice fields
       \\type choice
       \\default GoldsteinNovoselacCeilingDiffuserWindow
       \\key GoldsteinNovoselacCeilingDiffuserWindow
       \\key ISO15099Windows
       \\key UserCurve
  A91; \\field Mixed Regime Window Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Inside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionInsideModels

SurfaceConvectionAlgorithm:Outside:AdaptiveModelSelections,
       \\unique-object
       \\memo Options to change the individual convection model equations for dynamic selection when using AdaptiveConvectiongAlgorithm
       \\memo This object is only needed to make changes to the default model selections for any or all of the surface categories.
       \\memo This object is for the outside face, the side of the surface facing away from the thermal zone.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Wind Convection Windward Vertical Wall Equation Source
       \\type choice
       \\default TARPWindward
       \\key SimpleCombined
       \\key TARPWindward
       \\key MoWiTTWindward
       \\key DOE2Windward
       \\key NusseltJurges
       \\key McAdams
       \\key Mitchell
       \\key BlockenWindward
       \\key EmmelVertical
       \\key UserCurve
  A3 , \\field Wind Convection Windward Equation Vertical Wall User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels
  A4 , \\field Wind Convection Leeward Vertical Wall Equation Source
       \\type choice
       \\default TARPLeeward
       \\key SimpleCombined
       \\key TARPLeeward
       \\key MoWiTTLeeward
       \\key DOE2Leeward
       \\key EmmelVertical
       \\key NusseltJurges
       \\key McAdams
       \\key Mitchell
       \\key UserCurve
  A5 , \\field Wind Convection Leeward Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels
  A6 , \\field Wind Convection Horizontal Roof Equation Source
       \\type choice
       \\default ClearRoof
       \\key SimpleCombined
       \\key TARPWindward
       \\key MoWiTTWindward
       \\key DOE2Windward
       \\key NusseltJurges
       \\key McAdams
       \\key Mitchell
       \\key BlockenWindward
       \\key EmmelRoof
       \\key ClearRoof
       \\key UserCurve
  A7 , \\field Wind Convection Horizontal Roof User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels
  A8 , \\field Natural Convection Vertical Wall Equation Source
       \\note This is for vertical walls
       \\type choice
       \\default ASHRAEVerticalWall
       \\key ASHRAEVerticalWall
       \\key AlamdariHammondVerticalWall
       \\key FohannoPolidoriVerticalWall
       \\key ISO15099Windows
       \\key UserCurve
       \\key None
  A9 , \\field Natural Convection Vertical Wall Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels
  A10, \\field Natural Convection Stable Horizontal Equation Source
       \\note This is for horizontal surfaces with heat flow directed for stable thermal stratification
       \\type choice
       \\default WaltonStableHorizontalOrTilt
       \\key WaltonStableHorizontalOrTilt
       \\key AlamdariHammondStableHorizontal
       \\key UserCurve
       \\key None
  A11, \\field Natural Convection Stable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels
  A12, \\field Natural Convection Unstable Horizontal Equation Source
       \\type choice
       \\default WaltonUnstableHorizontalOrTilt
       \\key WaltonUnstableHorizontalOrTilt
       \\key AlamdariHammondUnstableHorizontal
       \\key UserCurve
       \\key None
  A13; \\field Natural Convection Unstable Horizontal Equation User Curve Name
       \\note The SurfaceConvectionAlgorithm:Outside:UserCurve named in this field is used when the previous field is set to UserCurve
       \\type object-list
       \\object-list UserConvectionOutsideModels


SurfaceConvectionAlgorithm:Inside:UserCurve,
       \\memo Used to describe a custom model equation for surface convection heat transfer coefficient
       \\memo If more than one curve is referenced they are all used and added together.
  A1 , \\field Name
       \\reference UserConvectionInsideModels
       \\reference UserConvectionModels
  A2 , \\field Reference Temperature for Convection Heat Transfer
       \\note Controls which temperature is differenced from surface temperature when using the Hc value
       \\type choice
       \\key MeanAirTemperature
       \\key AdjacentAirTemperature
       \\key SupplyAirTemperature
  A3 , \\field Hc Function of Temperature Difference Curve Name
       \\note  Curve's "x" is absolute value of delta-T (Surface temperature minus reference temperature, (C))
       \\type alpha
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A4 , \\field Hc Function of Temperature Difference Divided by Height Curve Name
       \\note Curve's "x" is absolute value of delta-T/Height (Surface temp minus Air temp)/(vertical length scale), (C/m)
       \\note when used for an inside face the vertical length scale is the zone's interior height
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A5 , \\field Hc Function of Air Change Rate Curve Name
       \\note Curve's "x" is mechanical ACH (Air Changes per hour from mechanical air system), (1/hr)
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A6 ; \\field Hc Function of Air System Volume Flow Rate Divided by Zone Perimeter Length Curve Name
       \\note Curve's "x" is mechanical system air flow rate (m3/s) divided by zone's length along
       \\note exterior walls (m).
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves

SurfaceConvectionAlgorithm:Outside:UserCurve,
       \\memo Used to describe a custom model equation for surface convection heat transfer coefficient
       \\memo If more than one curve is referenced they are all used and added together.
  A1 , \\field Name
       \\reference UserConvectionOutsideModels
       \\reference UserConvectionModels
  A2 , \\field Wind Speed Type for Curve
       \\type choice
       \\key WeatherFile
       \\key HeightAdjust
       \\key ParallelComponent
       \\key ParallelComponentHeightAdjust
       \\default HeightAdjust
  A3 , \\field Hf Function of Wind Speed Curve Name
       \\note Curve's "x" is wind speed of the type determined in the previous field (m/s)
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A4 , \\field Hn Function of Temperature Difference Curve Name
       \\note  Curve's "x" is absolute value of delta-T (Surface temperature minus air temperature, (C))
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A5 ; \\field Hn Function of Temperature Difference Divided by Height Curve Name
       \\note Curve's "x" is absolute value of delta-T/Height (Surface temp minus Air temp)/(vertical length scale), (C/m)
       \\note when used for an outside face the vertical length scale is the exterior facade's overall height
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves

SurfaceProperty:ConvectionCoefficients,
      \\memo Allow user settable interior and/or exterior convection coefficients.
      \\memo Note that some other factors may limit the lower bounds for these values, such as
      \\memo for windows, the interior convection coefficient must be >.28,
      \\memo for trombe wall algorithm selection (zone), the interior convection coefficient must be >.1
      \\memo for TARP interior convection, the lower limit is also .1
      \\memo Minimum and maximum limits are set in HeatBalanceAlgorithm object.
      \\memo Defaults in HeatBalanceAlgorithm object are [.1,1000].
  A1, \\field Surface Name
      \\required-field
      \\type object-list
      \\object-list SurfaceNames
  A2, \\field Convection Coefficient 1 Location
      \\required-field
      \\type choice
      \\key Outside
      \\key Inside
  A3, \\field Convection Coefficient 1 Type
      \\required-field
      \\type choice
      \\key Value
      \\key Schedule
      \\key UserCurve
      \\key Simple
      \\key SimpleCombined
      \\key TARP
      \\key DOE-2
      \\key MoWitt
      \\key AdaptiveConvectionAlgorithm
      \\key ASHRAEVerticalWall
      \\key WaltonUnstableHorizontalOrTilt
      \\key WaltonStableHorizontalOrTilt
      \\key FisherPedersenCeilingDiffuserWalls
      \\key FisherPedersenCeilingDiffuserCeiling
      \\key FisherPedersenCeilingDiffuserFloor
      \\key AlamdariHammondStableHorizontal
      \\key AlamdariHammondUnstableHorizontal
      \\key AlamdariHammondVerticalWall
      \\key KhalifaEq3WallAwayFromHeat
      \\key KhalifaEq4CeilingAwayFromHeat
      \\key KhalifaEq5WallNearHeat
      \\key KhalifaEq6NonHeatedWalls
      \\key KhalifaEq7Ceiling
      \\key AwbiHattonHeatedFloor
      \\key AwbiHattonHeatedWall
      \\key BeausoleilMorrisonMixedAssistedWall
      \\key BeausoleilMorrisonMixedOpposingWall
      \\key BeausoleilMorrisonMixedStableFloor
      \\key BeausoleilMorrisonMixedUnstableFloor
      \\key BeausoleilMorrisonMixedStableCeiling
      \\key BeausoleilMorrisonMixedUnstableCeiling
      \\key FohannoPolidoriVerticalWall
      \\key KaradagChilledCeiling
      \\key ISO15099Windows
      \\key GoldsteinNovoselacCeilingDiffuserWindow
      \\key GoldsteinNovoselacCeilingDiffuserWalls
      \\key GoldsteinNovoselacCeilingDiffuserFloor
      \\key NusseltJurges
      \\key McAdams
      \\key Mitchell
      \\key EmmelVertical
      \\key EmmelRoof
      \\key ClearRoof
  N1, \\field Convection Coefficient 1
      \\note used if Convection Type=Value, min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\units W/m2-K
  A4, \\field Convection Coefficient 1 Schedule Name
      \\note used if Convection Type=Schedule,  min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Convection Coefficient 1 User Curve Name
      \\note used if Convection Type = UserCurve
      \\type object-list
      \\object-list UserConvectionModels
  A6, \\field Convection Coefficient 2 Location
      \\type choice
      \\key Outside
      \\key Inside
  A7, \\field Convection Coefficient 2 Type
      \\type choice
      \\key Value
      \\key Schedule
      \\key UserCurve
      \\key Simple
      \\key SimpleCombined
      \\key TARP
      \\key DOE-2
      \\key MoWitt
      \\key AdaptiveConvectionAlgorithm
      \\key ASHRAEVerticalWall
      \\key WaltonUnstableHorizontalOrTilt
      \\key WaltonStableHorizontalOrTilt
      \\key FisherPedersenCeilingDiffuserWalls
      \\key FisherPedersenCeilingDiffuserCeiling
      \\key FisherPedersenCeilingDiffuserFloor
      \\key AlamdariHammondStableHorizontal
      \\key AlamdariHammondUnstableHorizontal
      \\key AlamdariHammondVerticalWall
      \\key KhalifaEq3WallAwayFromHeat
      \\key KhalifaEq4CeilingAwayFromHeat
      \\key KhalifaEq5WallNearHeat
      \\key KhalifaEq6NonHeatedWalls
      \\key KhalifaEq7Ceiling
      \\key AwbiHattonHeatedFloor
      \\key AwbiHattonHeatedWall
      \\key BeausoleilMorrisonMixedAssistedWall
      \\key BeausoleilMorrisonMixedOpposingWall
      \\key BeausoleilMorrisonMixedStableFloor
      \\key BeausoleilMorrisonMixedUnstableFloor
      \\key BeausoleilMorrisonMixedStableCeiling
      \\key BeausoleilMorrisonMixedUnstableCeiling
      \\key FohannoPolidoriVerticalWall
      \\key KaradagChilledCeiling
      \\key ISO15099Windows
      \\key GoldsteinNovoselacCeilingDiffuserWindow
      \\key GoldsteinNovoselacCeilingDiffuserWalls
      \\key GoldsteinNovoselacCeilingDiffuserFloor
      \\key NusseltJurges
      \\key McAdams
      \\key Mitchell
      \\key EmmelVertical
      \\key EmmelRoof
      \\key ClearRoof
  N2, \\field Convection Coefficient 2
      \\note used if Convection Type=Value, min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\default .1
      \\units W/m2-K
  A8, \\field Convection Coefficient 2 Schedule Name
      \\note used if Convection Type=Schedule,  min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\type object-list
      \\object-list ScheduleNames
  A9; \\field Convection Coefficient 2 User Curve Name
      \\note used if Convection Type = UserCurve
      \\type object-list
      \\object-list UserConvectionModels

SurfaceProperty:ConvectionCoefficients:MultipleSurface,
      \\memo Allow user settable interior and/or exterior convection coefficients.
      \\memo Note that some other factors may limit the lower bounds for these values, such as
      \\memo for windows, the interior convection coefficient must be >.28,
      \\memo for trombe wall algorithm selection (zone), the interior convection coefficient must be >.1
      \\memo for TARP interior convection, the lower limit is also .1
      \\memo Minimum and maximum limits are set in HeatBalanceAlgorithm object.
      \\memo Defaults in HeatBalanceAlgorithm object are [.1,1000].
  A1, \\field Surface Type
      \\required-field
      \\type choice
      \\key AllExteriorSurfaces
      \\key AllExteriorWindows
      \\key AllExteriorWalls
      \\key AllExteriorRoofs
      \\key AllExteriorFloors
      \\key AllInteriorSurfaces
      \\key AllInteriorWalls
      \\key AllInteriorWindows
      \\key AllInteriorCeilings
      \\key AllInteriorFloors
  A2, \\field Convection Coefficient 1 Location
      \\required-field
      \\type choice
      \\key Outside
      \\key Inside
  A3, \\field Convection Coefficient 1 Type
      \\required-field
      \\type choice
      \\key Value
      \\key Schedule
      \\key Simple
      \\key SimpleCombined
      \\key TARP
      \\key DOE-2
      \\key MoWitt
      \\key AdaptiveConvectionAlgorithm
      \\key ASHRAEVerticalWall
      \\key WaltonUnstableHorizontalOrTilt
      \\key WaltonStableHorizontalOrTilt
      \\key FisherPedersenCeilingDiffuserWalls
      \\key FisherPedersenCeilingDiffuserCeiling
      \\key FisherPedersenCeilingDiffuserFloor
      \\key AlamdariHammondStableHorizontal
      \\key AlamdariHammondUnstableHorizontal
      \\key AlamdariHammondVerticalWall
      \\key KhalifaEq3WallAwayFromHeat
      \\key KhalifaEq4CeilingAwayFromHeat
      \\key KhalifaEq5WallNearHeat
      \\key KhalifaEq6NonHeatedWalls
      \\key KhalifaEq7Ceiling
      \\key AwbiHattonHeatedFloor
      \\key AwbiHattonHeatedWall
      \\key BeausoleilMorrisonMixedAssistedWall
      \\key BeausoleilMorrisonMixedOpposingWall
      \\key BeausoleilMorrisonMixedStableFloor
      \\key BeausoleilMorrisonMixedUnstableFloor
      \\key BeausoleilMorrisonMixedStableCeiling
      \\key BeausoleilMorrisonMixedUnstableCeiling
      \\key FohannoPolidoriVerticalWall
      \\key KaradagChilledCeiling
      \\key ISO15099Windows
      \\key GoldsteinNovoselacCeilingDiffuserWindow
      \\key GoldsteinNovoselacCeilingDiffuserWalls
      \\key GoldsteinNovoselacCeilingDiffuserFloor
      \\key NusseltJurges
      \\key McAdams
      \\key Mitchell
      \\key BlockenWindard
      \\key EmmelVertical
      \\key EmmelRoof
      \\key ClearRoof
      \\key UserCurve
  N1, \\field Convection Coefficient 1
      \\note used if Convection Type=Value, min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\units W/m2-K
  A4, \\field Convection Coefficient 1 Schedule Name
      \\note used if Convection Type=Schedule,  min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Convection Coefficient 1 User Curve Name
      \\note used if Convection Type = UserCurve
      \\type object-list
      \\object-list UserConvectionModels
  A6, \\field Convection Coefficient 2 Location
      \\type choice
      \\key Outside
      \\key Inside
  A7, \\field Convection Coefficient 2 Type
      \\type choice
      \\key Value
      \\key Schedule
      \\key Simple
      \\key SimpleCombined
      \\key TARP
      \\key DOE-2
      \\key MoWitt
      \\key AdaptiveConvectionAlgorithm
      \\key ASHRAEVerticalWall
      \\key WaltonUnstableHorizontalOrTilt
      \\key WaltonStableHorizontalOrTilt
      \\key FisherPedersenCeilingDiffuserWalls
      \\key FisherPedersenCeilingDiffuserCeiling
      \\key FisherPedersenCeilingDiffuserFloor
      \\key AlamdariHammondStableHorizontal
      \\key AlamdariHammondUnstableHorizontal
      \\key AlamdariHammondVerticalWall
      \\key KhalifaEq3WallAwayFromHeat
      \\key KhalifaEq4CeilingAwayFromHeat
      \\key KhalifaEq5WallNearHeat
      \\key KhalifaEq6NonHeatedWalls
      \\key KhalifaEq7Ceiling
      \\key AwbiHattonHeatedFloor
      \\key AwbiHattonHeatedWall
      \\key BeausoleilMorrisonMixedAssistedWall
      \\key BeausoleilMorrisonMixedOpposingWall
      \\key BeausoleilMorrisonMixedStableFloor
      \\key BeausoleilMorrisonMixedUnstableFloor
      \\key BeausoleilMorrisonMixedStableCeiling
      \\key BeausoleilMorrisonMixedUnstableCeiling
      \\key FohannoPolidoriVerticalWall
      \\key KaradagChilledCeiling
      \\key ISO15099Windows
      \\key GoldsteinNovoselacCeilingDiffuserWindow
      \\key GoldsteinNovoselacCeilingDiffuserWalls
      \\key GoldsteinNovoselacCeilingDiffuserFloor
      \\key NusseltJurges
      \\key McAdams
      \\key Mitchell
      \\key BlockenWindard
      \\key EmmelVertical
      \\key EmmelRoof
      \\key ClearRoof
      \\key UserCurve
  N2, \\field Convection Coefficient 2
      \\note used if Convection Type=Value, min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\default .1
      \\units W/m2-K
  A8, \\field Convection Coefficient 2 Schedule Name
      \\note used if Convection Type=Schedule,  min and max limits are set in HeatBalanceAlgorithm object.
      \\note Default limits are Minimum >= 0.1 and Maximum <= 1000
      \\type object-list
      \\object-list ScheduleNames
  A9; \\field Convection Coefficient 2 User Curve Name
      \\note used if Convection Type = UserCurve
      \\type object-list
      \\object-list UserConvectionModels

SurfaceProperties:VaporCoefficients,
      \\memo The interior and external vapor transfer coefficients.
      \\memo Normally these value are calculated using the heat convection coefficient values.
      \\memo Use this object to used fixed constant values.
      \\memo Units are kg/Pa.s.m2
      \\memo This will only work with the CombinedHeatAndMoistureFiniteElement algorithm for surfaces.
      \\memo Other algorithms will ignore these coefficients
  A1, \\field Surface Name
      \\required-field
      \\type object-list
      \\object-list SurfaceNames
  A2, \\field Constant External Vapor Transfer Coefficient
      \\required-field
      \\type choice
      \\key Yes
      \\key No
      \\default No
  N1, \\field External Vapor Coefficient Value
      \\units kg/Pa-s-m2
      \\default 0
      \\minimum 0.0
  A3, \\field Constant Internal vapor Transfer Coefficient
      \\required-field
      \\type choice
      \\key Yes
      \\key No
      \\default No
  N2; \\field Internal Vapor Coefficient Value
      \\units kg/Pa-s-m2
      \\default 0
      \\minimum 0.0

SurfaceProperty:ExteriorNaturalVentedCavity,
       \\memo Used to describe the decoupled layer, or baffle, and the characteristics of the cavity
       \\memo and openings for naturally ventilated exterior surfaces. This object is also used in
       \\memo conjunction with the OtherSideConditionsModel.
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
  A1,  \\field Name
       \\required-field
       \\type alpha
       \\reference VentedCavityNames
  A2,  \\field Boundary Conditions Model Name
       \\note Enter the name of a SurfaceProperty:OtherSideConditionsModel object
       \\required-field
       \\type object-list
       \\object-list OSCMNames
  N1,  \\field Area Fraction of Openings
       \\type real
       \\minimum> 0
       \\maximum 1.0
       \\units dimensionless
  N2,  \\field Thermal Emissivity of Exterior Baffle Material
       \\type real
       \\minimum 0
       \\maximum 1
       \\units dimensionless
  N3,  \\field Solar Absorbtivity of Exterior Baffle
       \\type real
       \\minimum 0
       \\maximum 1
       \\units dimensionless
  N4, \\field Height Scale for Buoyancy-Driven Ventilation
       \\type real
       \\units m
       \\minimum> 0.0
  N5,  \\field Effective Thickness of Cavity Behind Exterior Baffle
       \\type real
       \\minimum> 0.
       \\units m
       \\note if corrugated, use average depth
  N6,  \\field Ratio of Actual Surface Area to Projected Surface Area
       \\type real
       \\note this parameter is used to help account for corrugations in the collector
       \\default 1.0
       \\minimum 0.8
       \\maximum 2.0
       \\units dimensionless
  A3,  \\field Roughness of Exterior Surface
       \\required-field
       \\type choice
       \\key VeryRough
       \\key Rough
       \\key MediumRough
       \\key MediumSmooth
       \\key Smooth
       \\key VerySmooth
  N7,  \\field Effectiveness for Perforations with Respect to Wind
       \\type real
       \\units dimensionless
       \\default 0.25
       \\minimum> 0
       \\maximum 1.5
  N8,  \\field Discharge Coefficient for Openings with Respect to Buoyancy Driven Flow
       \\type real
       \\units dimensionless
       \\default 0.65
       \\minimum> 0.0
       \\maximum 1.5
  A4,  \\field Surface 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A5,  \\field Surface 2 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A6,  \\field Surface 3 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A7,  \\field Surface 4 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A8,  \\field Surface 5 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A9,  \\field Surface 6 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A10,  \\field Surface 7 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A11,  \\field Surface 8 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A12,  \\field Surface 9 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A13;  \\field Surface 10 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames

ZoneProperty:UserViewFactors:bySurfaceName,
       \\memo View factors for Surface to Surface in a zone.
       \\memo (Number of Surfaces)**2 must be entered.
       \\extensible:3 - copy last three fields, remembering to remove ;
       \\format ViewFactor
  A1,  \\field Zone Name
       \\type object-list
       \\object-list ZoneNames
  A2,  \\field From Surface 1
       \\begin-extensible
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A3,  \\field To Surface 1
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N1,  \\field View Factor 1
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A4,  \\field From Surface 2
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A5,  \\field To Surface 2
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N2,  \\field View Factor 2
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A6,  \\field From Surface 3
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A7,  \\field To Surface 3
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N3,  \\field View Factor 3
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A8,  \\field From Surface 4
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A9,  \\field To Surface 4
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N4,  \\field View Factor 4
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A10, \\field From Surface 5
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A11, \\field To Surface 5
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N5,  \\field View Factor 5
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A12, \\field From Surface 6
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A13, \\field To Surface 6
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N6,  \\field View Factor 6
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A14, \\field From Surface 7
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A15, \\field To Surface 7
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N7,  \\field View Factor 7
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A16, \\field From Surface 8
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A17, \\field To Surface 8
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N8,  \\field View Factor 8
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A18, \\field From Surface 9
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A19, \\field To Surface 9
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N9,  \\field View Factor 9
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A20, \\field From Surface 10
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A21, \\field To Surface 10
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N10, \\field View Factor 10
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A22, \\field From Surface 11
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A23, \\field To Surface 11
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N11, \\field View Factor 11
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A24, \\field From Surface 12
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A25, \\field To Surface 12
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N12, \\field View Factor 12
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A26, \\field From Surface 13
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A27, \\field To Surface 13
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N13, \\field View Factor 13
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A28, \\field From Surface 14
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A29, \\field To Surface 14
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N14, \\field View Factor 14
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A30, \\field From Surface 15
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A31, \\field To Surface 15
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N15, \\field View Factor 15
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A32, \\field From Surface 16
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A33, \\field To Surface 16
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N16, \\field View Factor 16
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A34, \\field From Surface 17
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A35, \\field To Surface 17
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N17, \\field View Factor 17
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A36, \\field From Surface 18
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A37, \\field To Surface 18
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N18, \\field View Factor 18
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A38, \\field From Surface 19
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A39, \\field To Surface 19
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N19, \\field View Factor 19
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A40, \\field From Surface 20
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A41, \\field To Surface 20
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N20, \\field View Factor 20
       \\note This value is the view factor value From Surface => To Surface
       \\type real
       \\maximum 1.0
  A42,A43,N21, \\note fields as indicated
  A44,A45,N22, \\note fields as indicated
  A46,A47,N23, \\note fields as indicated
  A48,A49,N24, \\note fields as indicated
  A50,A51,N25, \\note fields as indicated
  A52,A53,N26, \\note fields as indicated
  A54,A55,N27, \\note fields as indicated
  A56,A57,N28, \\note fields as indicated
  A58,A59,N29, \\note fields as indicated
  A60,A61,N30, \\note fields as indicated
  A62,A63,N31, \\note fields as indicated
  A64,A65,N32, \\note fields as indicated
  A66,A67,N33, \\note fields as indicated
  A68,A69,N34, \\note fields as indicated
  A70,A71,N35, \\note fields as indicated
  A72,A73,N36, \\note fields as indicated
  A74,A75,N37, \\note fields as indicated
  A76,A77,N38, \\note fields as indicated
  A78,A79,N39, \\note fields as indicated
  A80,A81,N40, \\note fields as indicated
  A82,A83,N41, \\note fields as indicated
  A84,A85,N42, \\note fields as indicated
  A86,A87,N43, \\note fields as indicated
  A88,A89,N44, \\note fields as indicated
  A90,A91,N45, \\note fields as indicated
  A92,A93,N46, \\note fields as indicated
  A94,A95,N47, \\note fields as indicated
  A96,A97,N48, \\note fields as indicated
  A98,A99,N49, \\note fields as indicated
  A100,A101,N50, \\note fields as indicated
  A102,A103,N51, \\note fields as indicated
  A104,A105,N52, \\note fields as indicated
  A106,A107,N53, \\note fields as indicated
  A108,A109,N54, \\note fields as indicated
  A110,A111,N55, \\note fields as indicated
  A112,A113,N56, \\note fields as indicated
  A114,A115,N57, \\note fields as indicated
  A116,A117,N58, \\note fields as indicated
  A118,A119,N59, \\note fields as indicated
  A120,A121,N60, \\note fields as indicated
  A122,A123,N61, \\note fields as indicated
  A124,A125,N62, \\note fields as indicated
  A126,A127,N63, \\note fields as indicated
  A128,A129,N64, \\note fields as indicated
  A130,A131,N65, \\note fields as indicated
  A132,A133,N66, \\note fields as indicated
  A134,A135,N67, \\note fields as indicated
  A136,A137,N68, \\note fields as indicated
  A138,A139,N69, \\note fields as indicated
  A140,A141,N70, \\note fields as indicated
  A142,A143,N71, \\note fields as indicated
  A144,A145,N72, \\note fields as indicated
  A146,A147,N73, \\note fields as indicated
  A148,A149,N74, \\note fields as indicated
  A150,A151,N75, \\note fields as indicated
  A152,A153,N76, \\note fields as indicated
  A154,A155,N77, \\note fields as indicated
  A156,A157,N78, \\note fields as indicated
  A158,A159,N79, \\note fields as indicated
  A160,A161,N80, \\note fields as indicated
  A162,A163,N81, \\note fields as indicated
  A164,A165,N82, \\note fields as indicated
  A166,A167,N83, \\note fields as indicated
  A168,A169,N84, \\note fields as indicated
  A170,A171,N85, \\note fields as indicated
  A172,A173,N86, \\note fields as indicated
  A174,A175,N87, \\note fields as indicated
  A176,A177,N88, \\note fields as indicated
  A178,A179,N89, \\note fields as indicated
  A180,A181,N90, \\note fields as indicated
  A182,A183,N91, \\note fields as indicated
  A184,A185,N92, \\note fields as indicated
  A186,A187,N93, \\note fields as indicated
  A188,A189,N94, \\note fields as indicated
  A190,A191,N95, \\note fields as indicated
  A192,A193,N96, \\note fields as indicated
  A194,A195,N97, \\note fields as indicated
  A196,A197,N98, \\note fields as indicated
  A198,A199,N99, \\note fields as indicated
  A200,A201,N100, \\note fields as indicated
  A202,A203,N101, \\note fields as indicated
  A204,A205,N102, \\note fields as indicated
  A206,A207,N103, \\note fields as indicated
  A208,A209,N104, \\note fields as indicated
  A210,A211,N105, \\note fields as indicated
  A212,A213,N106, \\note fields as indicated
  A214,A215,N107, \\note fields as indicated
  A216,A217,N108, \\note fields as indicated
  A218,A219,N109, \\note fields as indicated
  A220,A221,N110, \\note fields as indicated
  A222,A223,N111, \\note fields as indicated
  A224,A225,N112, \\note fields as indicated
  A226,A227,N113, \\note fields as indicated
  A228,A229,N114, \\note fields as indicated
  A230,A231,N115, \\note fields as indicated
  A232,A233,N116, \\note fields as indicated
  A234,A235,N117, \\note fields as indicated
  A236,A237,N118, \\note fields as indicated
  A238,A239,N119, \\note fields as indicated
  A240,A241,N120, \\note fields as indicated
  A242,A243,N121; \\note fields as indicated

\\group Detailed Ground Heat Transfer

GroundHeatTransfer:Control,
  \\memo Object determines if the Slab and Basement preprocessors
  \\memo are going to be executed.
  A1 , \\field Name
    \\note This field is included for consistency.11
  A2 , \\field Run Basement Preprocessor
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A3 ; \\field Run Slab Preprocessor
       \\type choice
       \\key Yes
       \\key No
       \\default No

GroundHeatTransfer:Slab:Materials,
    \\memo Object gives an overall description of the slab ground heat transfer model.
        N1, \\field NMAT: Number of materials
            \\note This field specifies the number of different materials that will be used in the model.
            \\note Typically only a ground material and a slab material are used. (2 materials)
            \\minimum> 0
          \\required-field
        N2, \\field ALBEDO: Surface Albedo: No Snow
            \\note Two fields specify the albedo value of the surface: first for no snow coverage days;
            \\note second for days with snow coverage. The albedo is the solar reflectivity of the surface,
            \\note and can vary from 0.05 for blacktop to 0.95 for fresh snow.
            \\note Typical values for North America reported by Bahnfleth range from 0.16 to 0.4.
            \\minimum 0.0
            \\maximum 1.0
          \\default 0.16
        N3, \\field ALBEDO: Surface Albedo: Snow
            \\minimum 0.0
            \\maximum 1.0
          \\default 0.40
        N4, \\field EPSLW: Surface Emissivity: No Snow
          \\note EPSLW (No Snow and Snow) specifies the long wavelength (thermal) emissivity of the ground surface.
          \\note primarily important for nighttime radiation to sky.
          \\note typical value .95
            \\minimum> 0.0
          \\default 0.94
        N5, \\field EPSLW: Surface Emissivity: Snow
            \\minimum> 0.0
          \\default 0.86
        N6, \\field Z0: Surface Roughness: No Snow
          \\note fields Z0 (No Snow and Snow) describe the height at which an experimentally velocity profile goes to zero.
          \\note typical value= .75 cm
         \\minimum> 0.0
          \\default .75
          \\units cm
        N7, \\field Z0: Surface Roughness: Snow
            \\note typical value= .05 cm
         \\minimum> 0.0
          \\default 0.25
          \\units cm
        N8, \\field HIN: Indoor HConv: Downward Flow
            \\note These fields specify the combined convective and radiative heat transfer coefficient between
            \\note the slab top inside surface and the room air for the cases where heat is flowing downward,
            \\note and upward. The program toggles between the two if the direction of the heat flux changes.
            \\note Typical values can be found in the ASHRAE Handbook of Fundamentals, but should be
            \\note about 6 W/(m2-K) for downward heat flow and 9 W/(m2-K) for upward heat flow.
            \\note typical value= 4-10
            \\units W/m2-K
            \\minimum> 0.0
          \\default 6.13
        N9; \\field HIN: Indoor HConv: Upward
            \\minimum> 0.0
            \\note typical value= 4-10
            \\units W/m2-K
          \\default 9.26

GroundHeatTransfer:Slab:MatlProps,
    \\memo This object contains the material properties for the materials
    \\memo used in the model. The fields are mostly self explanatory.
     N1,  \\field RHO: Slab Material density
          \\note Density of Slab Material
          \\note typical value= 2300.0
          \\units kg/m3
          \\minimum> 0.0
          \\default 2300
     N2,  \\field RHO: Soil Density
          \\note Density of Soil Material
          \\note typical value= 1200.0
          \\minimum> 0.0
          \\default 1200
          \\units kg/m3
     N3,  \\field CP: Slab CP
          \\note Specific Heat of Slab Material
          \\note typical value=650.0
          \\units J/kg-K
          \\minimum> 0.0
          \\default 650
     N4,  \\field CP: Soil CP
          \\note Specific Heat of Soil Material
          \\note typical value= 1200.0
          \\minimum> 0.0
          \\default 1200
          \\units J/kg-K
     N5,  \\field TCON: Slab k
          \\note Conductivity of Slab Material
          \\note typical value= .9
          \\minimum> 0.0
          \\units W/m-K
          \\default 0.9
     N6;  \\field TCON: Soil k
          \\note Conductivity of Soil Material
          \\note typical value= 1.0
          \\minimum> 0.0
          \\units W/m-K
          \\default 1.0

GroundHeatTransfer:Slab:BoundConds,
      \\memo Supplies some of the boundary conditions used in the ground heat transfer calculations.
        A1, \\field EVTR: Is surface evapotranspiration modeled
            \\note This field specifies whether or not to use the evapotransporation model.
            \\note The inclusion of evapotransporation in the calculation has the greatest
            \\note effect in warm dry climates, primarily on the ground surface temperature.
            \\note This field can be used to turn the evapotransporation off and on to check
            \\note sensitivity to it.
            \\type choice
            \\key TRUE
            \\key FALSE
          \\required-field
        A2, \\field FIXBC: is the lower boundary at a fixed temperature
            \\note This field permits using a fixed temperature at the lower surface of the model
            \\note instead of a zero heat flux condition. This change normally has a very small
            \\note effect on the results.
            \\note FALSE selects the zero flux lower boundary condition
            \\type choice
            \\key TRUE
            \\key FALSE
          \\required-field
        N1, \\field TDEEPin
            \\note User input lower boundary temperature if FIXBC is TRUE
            \\note Blank for FIXBC FALSE or to use the calculated 1-D deep ground temperature.
            \\units C
        A3, \\field USRHflag: Is the ground surface h specified by the user?
            \\note This field flags the use of a user specified heat transfer coefficient
            \\note on the ground surface. This condition is used primarily for testing.
            \\note For normal runs (USPHflag is FALSE) and the program calculates the heat
            \\note transfer coefficient using the weather conditions.
            \\type choice
            \\key TRUE
            \\key FALSE
          \\required-field
        N2; \\field USERH: User specified ground surface heat transfer coefficient
            \\note Used only if USRHflag is TRUE and the heat transfer coefficient value is
            \\note specified in this field.
            \\units W/m2-K

GroundHeatTransfer:Slab:BldgProps,
          \\memo Object provides information about the building and its operating conditions
          \\memo Monthly Average Temperature SetPoint fields
          \\memo specify the average indoor building set point temperatures for each
          \\memo month of the year. These fields are useful for simulating a building
          \\memo that is not temperature controlled for some of the year.
          \\memo In such a case, the average indoor set point temperatures
          \\memo can be obtained by first running the model in EnergyPlus with an
          \\memo insulated floor boundary condition, and then using the resulting
          \\memo monthly average zone temperatures in these fields.
      N1, \\field IYRS: Number of years to iterate
          \\note This field specifies the number of years to iterate.
          \\note Either the ground heat transfer calculations come to an
          \\note an annual steady periodic condition by converging to a tolerance
          \\note (see ConvTol field) or it runs for this number of years.
          \\note A ten year maximum is usually sufficient.
          \\default 10
          \\minimum 1
      N2, \\field Shape: Slab shape
          \\note Use only the value 0 here. Only a rectangular shape is implemented.
          \\minimum 0
          \\maximum 0
      N3, \\field HBLDG: Building height
          \\note This field supplies the building height. This is used to calculate
          \\note the building shadowing on the ground.
          \\minimum 0
          \\note typical value= 0-20
          \\units m
      N4, \\field TIN1: January Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N5, \\field TIN2: February Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N6, \\field TIN3: March Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N7, \\field TIN4: April Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N8, \\field TIN5: May Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N9, \\field TIN6: June Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N10,\\field TIN7: July Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N11,\\field TIN8: August Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N12,\\field TIN9: September Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N13,\\field TIN10: October Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N14,\\field TIN11: November Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N15,\\field TIN12: December Indoor Average Temperature Setpoint
          \\note see memo on object for more information
          \\default 22
          \\units C
      N16,\\field TINAmp: Daily Indoor sine wave variation amplitude
          \\note This field permits imposing a daily sinusoidal variation
          \\note in the indoor setpoint temperature to simulate the effect
          \\note of a setback profile.
          \\note The value specified is the amplitude of the sine wave.
          \\default 0
          \\units deltaC
      N17;\\field ConvTol: Convergence Tolerance
          \\note This field specifies the convergence tolerance used to
          \\note control the iteration. When the temperature change of all nodes
          \\note is less than the convergence value, iteration ceases.
          \\default 0.1

GroundHeatTransfer:Slab:Insulation,
    \\memo This object supplies the information about insulation used around the slab.
    \\memo There are two possible configurations: under the slab or vertical insulation
    \\memo around the slab.
        N1, \\field RINS: R value of under slab insulation
            \\note This field provides the thermal resistance value
            \\note of the under slab insulation. It should be zero
            \\note if the vertical insulation configuration is selected.
            \\note typical value= 0-2.0
            \\units m2-K/W
          \\default 0.0
        N2, \\field DINS: Width of strip of under slab insulation
            \\note This specifies the width of the perimeter strip of insulation
            \\note under the slab. It should be zero if for the vertical insulation
            \\note configuration is selected.
            \\note typical value= 0-2.0
            \\units m
          \\default 0.0
        N3, \\field RVINS: R value of vertical insulation
            \\note This field specifies the thermal resistance of the vertical
            \\note insulation. It should be zero if the under slab insulation
            \\note configuration is selected.
            \\note typical value= 0-3.0
            \\units m2-K/W
          \\default 0.0
        N4, \\field ZVINS: Depth of vertical insulation
            \\note This field specifies the depth of the vertical insulation
            \\note into the ground in meters. It starts at the slab upper surface
            \\note and extends into the ground.
            \\note It should be zero if the under slab insulation
            \\note configuration is selected.
            \\note only use values= .2 .4 .6 .8 1.0 1.5 2.0 2.5 3.0
            \\units m
          \\default 0
        N5; \\field IVINS: Flag: Is there vertical insulation
            \\note Specifies if the vertical insulation configuration is being used.
            \\note values: 1=yes vertical insulation 0=no underslab insulation
          \\required-field
          \\type choice
          \\key 0
          \\key 1
          \\default 0

GroundHeatTransfer:Slab:EquivalentSlab,
    \\memo Using an equivalent slab allows non-rectangular shapes to be modeled accurately.
    \\memo Object uses the area - perimeter (area/perimeter) ratio to determine the
    \\memo size of an equivalent rectangular slab.
    \\memo EnergyPlus users normally use this option.
     N1,  \\field APRatio: The area to perimeter ratio for this slab
          \\note Equivalent square slab is simulated,  side is 4*APRatio.
          \\units m
          \\minimum 1.5
          \\maximum 22
          \\required-field
     N2,  \\field SLABDEPTH: Thickness of slab on grade
          \\note This field specifies the thickness of the slab. The slab top surface is level
          \\note with the ground surface, so this is the depth into the ground.
          \\note The slab depth has a significant effect on the temperature calculation,
          \\note and it is also important for the auto-grid process.
          \\note The finite difference grids are set in such a way that they use the slab thickness
          \\note to determine the vertical grid spacing. Autogridding will fail if the slab thickness
          \\note is specified larger than 0.25 meters.
          \\default 0.1
          \\units m
     N3,  \\field CLEARANCE: Distance from edge of slab to domain edge
          \\note This field specifies the distance from the slab to the edge of
          \\note the area that will be modeled with the grid system. It is the basic size
          \\note dimension that is used to set the horizontal extent of the domain.
          \\note 15 meters is a reasonable value.
          \\default 15.0
          \\units m
     N4;  \\field ZCLEARANCE: Distance from bottom of slab to domain bottom
          \\note This field specifies the vertical distance from the slab to the
          \\note bottom edge of the area that will be modeled with the grid system.
          \\note 15 meters is a reasonable value.
          \\default 15.0
          \\units m

GroundHeatTransfer:Slab:AutoGrid,
            \\memo AutoGrid only necessary when EquivalentSlab option not chosen.
            \\memo  Not normally needed by EnergyPlus users.
            \\memo  This object permits user selection of rectangular slab dimensions.
            \\memo  NO SLAB DIMENSIONS LESS THAN 6 m.
        N1, \\field SLABX: X dimension of the building slab
            \\minimum 6
            \\note typical values= 6 to 60.0
            \\units m
          \\required-field
        N2, \\field SLABY: Y dimension of the building slab
            \\minimum 6
            \\note typical values= 6 to 60.0
            \\units m
          \\required-field
        N3, \\field SLABDEPTH: Thickness of slab on grade
            \\default 0.1
            \\units m
        N4, \\field CLEARANCE: Distance from edge of slab to domain edge
            \\default 15.0
            \\units m
        N5; \\field ZCLEARANCE: Distance from bottom of slab to domain bottom
            \\default 15.0
            \\units m

GroundHeatTransfer:Slab:ManualGrid,
            \\memo Manual Grid only necessary when using manual gridding (not recommended)
            \\memo Used only in special cases when previous two objects are not used.
            \\memo User must input complete gridding information.
        N1, \\field NX: Number of cells in the X direction
          \\minimum 1
          \\required-field
        N2, \\field NY: Number of cells in the Y direction
          \\minimum 1
          \\required-field
        N3, \\field NZ: Number of cells in the Z direction
          \\minimum 1
          \\required-field
        N4, \\field IBOX: X direction cell indicator of slab edge
            \\note typical values= 1-10
          \\required-field
        N5; \\field JBOX: Y direction cell indicator of slab edge
            \\note typical values= 1-10
          \\required-field

GroundHeatTransfer:Slab:XFACE,
         \\memo This is only needed when using manual gridding (not recommended)
         \\memo XFACE: X Direction cell face coordinates: m
        N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,                  \\note fields as indicated
        N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25, N26, N27, N28, N29,    \\note fields as indicated
        N30, N31, N32, N33, N34, N35, N36, N37, N38, N39, N40;                        \\note fields as indicated

GroundHeatTransfer:Slab:YFACE,
         \\memo This is only needed when using manual gridding (not recommended)
         \\memo YFACE: Y Direction cell face coordinates: m,
        N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,                  \\note fields as indicated
        N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25, N26, N27, N28, N29,    \\note fields as indicated
        N30, N31, N32, N33, N34, N35, N36, N37, N38, N39, N40;                        \\note fields as indicated

GroundHeatTransfer:Slab:ZFACE,
         \\memo This is only needed when usuing manual gridding (not recommended)
         \\memo ZFACE: Z Direction cell face coordinates: m
        N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,                  \\note fields as indicated
        N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25;                        \\note fields as indicated

GroundHeatTransfer:Basement:SimParameters,
          \\memo Specifies certain parameters that control the Basement preprocessor ground heat
          \\memo transfer simulation.
     N1,  \\field F: Multiplier for the ADI solution
         \\note 0<F<1.0,
          \\note typically 0.1 (0.3 for high k soil - saturated sand is about 2.6 w/m-K)
          \\minimum> 0.0
          \\maximum 1.0
      N2; \\field IYRS: Maximum number of yearly iterations:
         \\note typically 15-30]
          \\minimum 0
          \\default 15

GroundHeatTransfer:Basement:MatlProps,
         \\memo Specifies the material properties for the Basement preprocessor ground heat
         \\memo transfer simulation. Only the Foundation Wall, Floor Slab, Soil,
         \\memo and Gravel properties are currently used.
     N1, \\field NMAT: Number of materials in this domain
         \\maximum 6
          \\required-field
     N2, \\field Density for Foundation Wall
         \\minimum> 0.0
         \\default 2243
         \\units kg/m3
     N3, \\field density for Floor Slab
         \\minimum> 0.0
         \\default 2243
         \\units kg/m3
     N4, \\field density for Ceiling
         \\minimum> 0.0
         \\default 311
         \\units kg/m3
     N5, \\field density for Soil
         \\minimum> 0.0
         \\default 1500
         \\units kg/m3
     N6, \\field density for Gravel
         \\minimum> 0.0
         \\default 2000
         \\units kg/m3
     N7, \\field density for Wood
         \\minimum> 0.0
         \\default 449
         \\units kg/m3
     N8,  \\field Specific heat for foundation wall
          \\minimum> 0.0
          \\default 880
          \\units J/kg-K
     N9,  \\field Specific heat for floor slab
          \\minimum> 0.0
          \\default 880
          \\units J/kg-K
     N10, \\field Specific heat for ceiling
          \\minimum> 0.0
          \\default 1530
          \\units J/kg-K
     N11, \\field Specific heat for soil
          \\minimum> 0.0
          \\default 840
          \\units J/kg-K
     N12, \\field Specific heat for gravel
          \\minimum> 0.0
          \\default 720
          \\units J/kg-K
     N13, \\field Specific heat for wood
          \\minimum> 0.0
          \\default 1530
          \\units J/kg-K
     N14, \\field Thermal conductivity for foundation wall
          \\minimum> 0.0
          \\default 1.4
          \\units W/m-K
     N15, \\field Thermal conductivity for floor slab
          \\minimum> 0.0
          \\default 1.4
          \\units W/m-K
     N16, \\field Thermal conductivity for ceiling
          \\minimum> 0.0
          \\default 0.09
          \\units W/m-K
     N17, \\field thermal conductivity for soil
          \\minimum> 0.0
          \\default 1.1
          \\units W/m-K
     N18, \\field thermal conductivity for gravel
          \\minimum> 0.0
          \\default 1.9
          \\units W/m-K
     N19; \\field thermal conductivity for wood
          \\minimum> 0.0
          \\default 0.12
          \\units W/m-K

GroundHeatTransfer:Basement:Insulation,
         \\memo Describes the insulation used on an exterior basement wall for the Basement
         \\memo preprocessor ground heat transfer simulation.
     N1, \\field REXT: R Value of any exterior insulation
         \\minimum> 0.0
         \\units m2-K/W
     A1; \\field INSFULL: Flag: Is the wall fully insulated?
          \\required-field
         \\type choice
          \\key TRUE
          \\key FALSE
         \\note  True for full insulation
         \\note  False for insulation half way down side wall from grade line

GroundHeatTransfer:Basement:SurfaceProps,
         \\memo Specifies the soil surface properties for the Basement preprocessor ground
         \\memo heat transfer simulation.
     N1, \\field ALBEDO: Surface albedo for No snow conditions
         \\minimum 0.0
          \\maximum 1.0
         \\default 0.16
     N2, \\field ALBEDO: Surface albedo for snow conditions
         \\minimum 0.0
          \\maximum 1.0
         \\default 0.40
     N3, \\field EPSLN: Surface emissivity No Snow
          \\minimum 0.0
          \\maximum 1.0
         \\default 0.94
     N4, \\field EPSLN: Surface emissivity with Snow
          \\minimum 0.0
          \\maximum 1.0
         \\default 0.86
     N5, \\field VEGHT: Surface roughness No snow conditions
          \\minimum 0.0
         \\default 6.0
         \\units cm
     N6, \\field VEGHT: Surface roughness Snow conditions
          \\minimum 0.0
         \\default 0.25
         \\units cm
     A1; \\field PET: Flag, Potential evapotranspiration on?
          \\note  Typically, PET is False
          \\required-field
         \\type choice
          \\key TRUE
          \\key FALSE
          \\default FALSE

GroundHeatTransfer:Basement:BldgData,
         \\memo Specifies the surface and gravel thicknesses used for the Basement
         \\memo preprocessor ground heat transfer simulation.
     N1, \\field DWALL: Wall thickness
         \\units m
         \\minimum 0.2
          \\default 0.2
     N2, \\field DSLAB: Floor slab thickness
         \\units m
         \\minimum> 0.0
         \\maximum 0.25
          \\default 0.1
     N3, \\field DGRAVXY: Width of gravel pit beside basement wall
         \\minimum> 0.0
         \\units  m
          \\default 0.3
     N4, \\field DGRAVZN: Gravel depth extending above the floor slab
         \\minimum> 0.0
         \\units m
          \\default 0.2
     N5; \\field DGRAVZP: Gravel depth below the floor slab
         \\minimum> 0.0
         \\units m
         \\default 0.1

GroundHeatTransfer:Basement:Interior,
       \\memo Provides the information needed to simulate the inside boundary conditions for
       \\memo the Basement preprocessor ground heat transfer simulation.
   A1, \\field COND: Flag: Is the basement conditioned?
          \\required-field
       \\type choice
          \\key TRUE
          \\key FALSE
          \\default TRUE
       \\note for EnergyPlus this should be TRUE
   N1, \\field HIN: Downward convection only heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 0.92
   N2, \\field HIN: Upward convection only heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 4.04
   N3, \\field HIN: Horizontal convection only heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 3.08
   N4, \\field HIN: Downward combined (convection and radiation) heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 6.13
   N5, \\field HIN: Upward combined (convection and radiation) heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 9.26
   N6; \\field HIN: Horizontal combined (convection and radiation) heat transfer coefficient
       \\units W/m2-K
          \\minimum> 0.0
          \\default 8.29

GroundHeatTransfer:Basement:ComBldg,
   \\memo ComBldg contains the monthly average temperatures (C) and possibility of daily variation amplitude
   N1,  \\field January average temperature
        \\units C
          \\default 22
   N2,  \\field February average temperature
        \\units C
          \\default 22
   N3,  \\field March average temperature
        \\units C
          \\default 22
   N4,  \\field April average temperature
        \\units C
          \\default 22
   N5,  \\field May average temperature
        \\units C
          \\default 22
   N6,  \\field June average temperature
        \\units C
          \\default 22
   N7,  \\field July average temperature
        \\units C
          \\default 22
   N8,  \\field August average temperature
        \\units C
          \\default 22
   N9,  \\field September average temperature
        \\units C
          \\default 22
   N10, \\field October average temperature
        \\units C
          \\default 22
   N11, \\field November average temperature
        \\units C
          \\default 22
   N12, \\field December average temperature
        \\units C
          \\default 22
   N13; \\field Daily variation sine wave amplitude
          \\units deltaC
          \\default 0
        \\note (Normally zero, just for checking)

GroundHeatTransfer:Basement:EquivSlab,  !  Supplies the EquivSizing Flag
    \\memo Using an equivalent slab allows non-rectangular shapes to be
    \\memo modeled accurately.
    \\memo The simulation default should be EquivSizing=True
   N1, \\field APRatio: The area to perimeter ratio for this slab
       \\minimum> 0.0
       \\units m
          \\required-field
   A1; \\field EquivSizing: Flag
          \\required-field
       \\type choice
          \\key TRUE
          \\key FALSE
       \\note Will the dimensions of an equivalent slab be calculated (TRUE)
       \\note or will the dimensions be input directly? (FALSE)]
       \\note Only advanced special simulations should use FALSE.

GroundHeatTransfer:Basement:EquivAutoGrid,
    \\memo EquivAutoGrid necessary when EquivSizing=TRUE, TRUE is is the normal case.
   N1, \\field CLEARANCE: Distance from outside of wall to edge of 3-D ground domain
          \\minimum 0.0
       \\units m
       \\default 15
   N2, \\field SlabDepth: Thickness of the floor slab
          \\minimum 0.0
       \\units m
       \\default  0.1
   N3; \\field BaseDepth: Depth of the basement wall below grade
          \\minimum 0.0
       \\units  m
          \\default 2

GroundHeatTransfer:Basement:AutoGrid,
   \\memo AutoGrid only necessary when EquivSizing is false
   \\memo If the modelled building is not a rectangle or square, Equivalent
   \\memo sizing MUST be used to get accurate results
   N1, \\field CLEARANCE: Distance from outside of wall to edge,
          \\minimum 0
       \\units m
       \\default 15
   N2, \\field SLABX: X dimension of the building slab
          \\required-field
       \\units m
       \\minimum 0
       \\maximum 60
   N3, \\field SLABY: Y dimension of the building slab
          \\required-field
       \\units m
       \\minimum 0
       \\maximum 60
   N4, \\field ConcAGHeight: Height of the foundation wall above grade
       \\units m
          \\minimum 0
          \\default 0
   N5, \\field SlabDepth: Thickness of the floor slab
       \\units m
       \\default 0.1
   N6; \\field BaseDepth: Depth of the basement wall below grade
       \\units m
          \\minimum 0
          \\default 2

GroundHeatTransfer:Basement:ManualGrid,
    \\memo Manual Grid only necessary using manual gridding (not recommended)
       N1, \\field NX: Number of cells in the X direction: 20]
          \\minimum 1
          \\required-field
       N2, \\field NY: Number of cells in the Y direction: 20]
          \\minimum 1
          \\required-field
       N3, \\field NZAG: Number of cells in the Z direction. above grade: 4 Always]
          \\minimum 1
          \\required-field
       N4, \\field NZBG: Number of cells in Z direction. below grade: 10-35]
          \\minimum 1
          \\required-field
       N5, \\field IBASE: X direction cell indicator of slab edge: 5-20]
          \\required-field
       N6, \\field JBASE: Y direction cell indicator of slab edge: 5-20]
          \\required-field
       N7; \\field KBASE: Z direction cell indicator of the top of the floor slab: 5-20]
          \\required-field

GroundHeatTransfer:Basement:XFACE,
    \\memo This is only needed when using manual gridding (not recommended)
    \\memo XFACE: X Direction cell face coordinates: m
       N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,  \\note fields as indicated
       N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25, N26,   \\note fields as indicated
       N27, N28, N29, N30, N31, N32, N33, N34, N35, N36, N37, N38,   \\note fields as indicated
       N39, N40, N41, N42, N43, N44;                                 \\note fields as indicated

GroundHeatTransfer:Basement:YFACE,
    \\memo This is only needed when using manual gridding (not recommended)
    \\memo YFACE: Y Direction cell face coordinates: m
       N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,  \\note fields as indicated
       N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25, N26,   \\note fields as indicated
       N27, N28, N29, N30, N31, N32, N33, N34, N35, N36, N37, N38,   \\note fields as indicated
       N39, N40, N41, N42, N43, N44;                                 \\note fields as indicated

GroundHeatTransfer:Basement:ZFACE,
    \\memo This is only needed when using manual gridding (not recommended)
    \\memo ZFACE: Z Direction cell face coordinates: m
       N1, N2, N3, N4, N5, N6, N7, N8, N9, N10, N11, N12, N13, N14,  \\note fields as indicated
       N15, N16, N17, N18, N19, N20, N21, N22, N23, N24, N25, N26,   \\note fields as indicated
       N27, N28, N29, N30, N31, N32, N33, N34, N35, N36, N37, N38,   \\note fields as indicated
       N39, N40;                                                     \\note fields as indicated

\\group Room Air Models

RoomAirModelType,
      \\memo Selects the type of room air model to be used in a given zone. If no RoomAirModelType
      \\memo object is specified then the default Mixing model (all zone air at the same
      \\memo temperature) will be used.
  A1, \\field Name
      \\required-field
      \\type alpha
  A2, \\field Zone Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
  A3, \\field Room-Air Modeling Type
      \\required-field
      \\type choice
      \\key Mixing
      \\note Complete mixing air model
      \\key UserDefined
      \\note UserDefined Room Air Temperature Patterns
      \\note needs RoomAir:TemperaturePattern:UserDefined object referencing this Zone
      \\key OneNodeDisplacementVentilation
      \\note Mundt roomair model for displacement ventilation with single floor air node
      \\note needs RoomAirSettings:OneNodeDisplacementVentilation object referencing this Zone
      \\key ThreeNodeDisplacementVentilation
      \\note (UCSD three-node displacement ventilation model)
      \\note needs RoomAirSettings:ThreeNodeDisplacementVentilation object referencing this Zone
      \\key CrossVentilation
      \\note (UCSD two-zone cross ventilation model)
      \\note needs RoomAirSettings:CrossVentilation object referencing this Zone
      \\key UnderFloorAirDistributionInterior
      \\note 2-Node UFAD model for interior zones
      \\note needs RoomAirSettings:UnderFloorAirDistributionInterior object referencing this Zone
      \\key UnderFloorAirDistributionExterior
      \\note (2-Node UFAD model for exterior zones)
      \\note needs RoomAirSettings:UnderFloorAirDistributionExterior object referencing this Zone
      \\default Mixing
  A4; \\field Air Temperature Coupling Strategy
      \\type choice
      \\key Direct
      \\key Indirect
      \\default Direct

RoomAir:TemperaturePattern:UserDefined,
       \\memo Used to explicitly define temperature patterns that are to be applied to the mean air
       \\memo temperature within a thermal zone.  Used with RoomAirModelType = UserDefined.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Availability Schedule Name
       \\note Availability schedule name for this model. Schedule value > 0 means the model is
       \\note active. Schedule value = 0 means the model is inactive and the zone will be modeled
       \\note as fully mixed (Mixing). If this field is blank, the model is always active.
       \\type object-list
       \\object-list ScheduleNames
  A4 ; \\field Pattern Control Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note The schedule should contain integer values that
       \\note correspond to unique Control Integer fields in
       \\note one of the RoomAir:TemperaturePattern:* objects.

RoomAir:TemperaturePattern:ConstantGradient,
      \\memo Used to model room air with a fixed temperature gradient in the vertical direction.
      \\memo Used in combination with RoomAir:TemperaturePattern:UserDefined.
  A1, \\field Name
      \\required-field
      \\type alpha
  N1, \\field Control Integer for Pattern Control Schedule Name
      \\note reference this entry in Schedule Name
      \\type integer
      \\required-field
  N2, \\field Thermostat Offset
      \\note = (Temp at thermostat- Mean Air Temp)
      \\units deltaC
      \\type real
  N3, \\field Return Air Offset
      \\note = (Tleaving - Mean Air Temp )
      \\units deltaC
      \\type real
  N4, \\field Exhaust Air Offset
      \\note = (Texhaust - Mean Air Temp) deg C
      \\units deltaC
      \\type real
  N5; \\field Temperature Gradient
      \\note Slope of temperature change in vertical direction
      \\units K/m
      \\type real

RoomAir:TemperaturePattern:TwoGradient,
      \\memo Used to model room air with two temperature gradients in the vertical direction.
      \\memo Used in combination with RoomAir:TemperaturePattern:UserDefined.
  A1, \\field Name
      \\required-field
      \\type alpha
  N1, \\field Control Integer for Pattern Control Schedule Name
      \\note reference this entry in Schedule Name
      \\type integer
      \\required-field
  N2, \\field Thermostat Height
      \\note = Distance from floor of zone
      \\units m
      \\type real
  N3, \\field Return Air Height
      \\note = Distance from floor of zone
      \\units m
      \\type real
  N4, \\field Exhaust Air Height
      \\note = Distance from floor of zone
      \\units m
      \\type real
  N5, \\field Temperature Gradient Lower Bound
      \\note Slope of temperature change in vertical direction
      \\units K/m
      \\type real
  N6, \\field Temperature Gradient Upper  Bound
      \\note Slope of temperature change in vertical direction
      \\units K/m
      \\type real
  A2, \\field Gradient Interpolation Mode
      \\type choice
      \\key OutdoorDryBulbTemperature
      \\key ZoneDryBulbTemperature
      \\key ZoneAndOutdoorTemperatureDifference
      \\key SensibleCoolingLoad
      \\key SensibleHeatingLoad
  N7, \\field Upper Temperature Bound
      \\units C
      \\type real
  N8, \\field Lower Temperature Bound
      \\units C
      \\type real
  N9, \\field Upper Heat Rate Bound
      \\units W
      \\type real
  N10; \\field Lower Heat Rate Bound
      \\units W
      \\type real

RoomAir:TemperaturePattern:NondimensionalHeight,
      \\memo Defines a distribution pattern for air temperatures relative to the current mean air
      \\memo temperature as a function of height. The height, referred to as Zeta, is non-dimensional
      \\memo by normalizing with the zone ceiling height.
      \\memo Used in combination with RoomAir:TemperaturePattern:UserDefined.
      \\extensible:2 - repeat last two fields remembering to remove ; from "inner" fields.
  A1, \\field Name
      \\required-field
      \\type alpha
  N1, \\field Control Integer for Pattern Control Schedule Name
      \\note this value should appear in as a schedule value
      \\type integer
      \\required-field
  N2, \\field Thermostat Offset
      \\note = (Temp at thermostat- Mean Air Temp)
      \\units deltaC
      \\type real
  N3, \\field Return Air Offset
      \\note = (Temp leaving - Mean Air Temp ) deg C
      \\units deltaC
      \\type real
  N4, \\field Exhaust Air Offset
      \\note = (Temp exhaust - Mean Air Temp) deg C
      \\units deltaC
      \\type real
      \\note the remaining fields have pairs that describe the relative
      \\note temperature pattern in the vertical direction of a zone
      \\note Zeta is the nondimensional height (in z-direction). on [0..1]
      \\note DeltaTai =  (Tai - MAT) in units of deg. C
      \\note relative deg C on [-10.0 .. 20.0 ]
  N5, \\field Pair 1 Zeta Nondimensional Height
      \\begin-extensible
      \\type real
      \\required-field
  N6, \\field Pair 1 Delta Adjacent Air Temperature
      \\type real
      \\required-field
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N7, \\field Pair 2 Zeta Nondimensional Height
      \\type real
      \\required-field
  N8, \\field Pair 2 Delta Adjacent Air Temperature
      \\type real
      \\required-field
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N9, \\field Pair 3 Zeta Nondimensional Height
      \\type real
  N10,\\field Pair 3 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N11,\\field Pair 4 Zeta Nondimensional Height
      \\type real
  N12,\\field Pair 4 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N13,\\field Pair 5 Zeta Nondimensional Height
      \\type real
  N14,\\field Pair 5 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N15,\\field Pair 6 Zeta Nondimensional Height
      \\type real
  N16,\\field Pair 6 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N17,\\field Pair 7 Zeta Nondimensional Height
      \\type real
  N18,\\field Pair 7 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N19,\\field Pair 8 Zeta Nondimensional Height
      \\type real
  N20,\\field Pair 8 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N21,\\field Pair 9 Zeta Nondimensional Height
      \\type real
  N22,\\field Pair 9 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N23,\\field Pair 10 Zeta Nondimensional Height
      \\type real
  N24,\\field Pair 10 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N25,\\field Pair 11 Zeta Nondimensional Height
      \\type real
  N26,\\field Pair 11 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N27,\\field Pair 12 Zeta Nondimensional Height
      \\type real
  N28,\\field Pair 12 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N29,\\field Pair 13 Zeta Nondimensional Height
      \\type real
  N30,\\field Pair 13 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N31,\\field Pair 14 Zeta Nondimensional Height
      \\type real
  N32,\\field Pair 14 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N33,\\field Pair 15 Zeta Nondimensional Height
      \\type real
  N34,\\field Pair 15 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N35,\\field Pair 16 Zeta Nondimensional Height
      \\type real
  N36,\\field Pair 16 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N37,\\field Pair 17 Zeta Nondimensional Height
      \\type real
  N38,\\field Pair 17 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N39,\\field Pair 18 Zeta Nondimensional Height
      \\type real
  N40,\\field Pair 18 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0
  N41,\\field Pair 19 Zeta Nondimensional Height
      \\type real
  N42;\\field Pair 19 Delta Adjacent Air Temperature
      \\type real
      \\units deltaC
      \\minimum -10.0
      \\maximum 20.0

RoomAir:TemperaturePattern:SurfaceMapping,
      \\memo Defines a distribution pattern for the air temperatures adjacent to individual surfaces.
      \\memo This allows controlling the adjacent air temperature on a surface-by-surface basis
      \\memo rather than by height. This allows modeling different adjacent air temperatures on
      \\memo the opposite sides of the zone. Used in combination with
      \\memo RoomAir:TemperaturePattern:UserDefined.
      \\extensible:2 - repeat last two fields remembering to remove ; from "inner" fields.
  A1, \\field Name
      \\required-field
      \\type alpha
  N1, \\field Control Integer for Pattern Control Schedule Name
      \\note reference this entry in schedule
      \\type integer
      \\required-field
  N2, \\field Thermostat Offset
      \\note = (Temp at thermostat- Mean Air Temp)
      \\units deltaC
  N3, \\field Return Air Offset
      \\note = (Tleaving - Mean Air Temp ) deg C
      \\units deltaC
  N4, \\field Exhaust Air Offset
      \\note = (Texhaust - Mean Air Temp) deg C
      \\units deltaC
  A2, \\field Surface Name Pair 1
      \\begin-extensible
      \\required-field
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N5, \\field Delta Adjacent Air Temperature Pair 1
      \\required-field
      \\type real
      \\units deltaC
  A3, \\field Surface Name Pair 2
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N6, \\field Delta Adjacent Air Temperature Pair 2
      \\type real
      \\units deltaC
  A4, \\field Surface Name Pair 3
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N7, \\field Delta Adjacent Air Temperature Pair 3
      \\type real
      \\units deltaC
  A5, \\field Surface Name Pair 4
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N8, \\field Delta Adjacent Air Temperature Pair 4
      \\type real
      \\units deltaC
  A6, \\field Surface Name Pair 5
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N9, \\field Delta Adjacent Air Temperature Pair 5
      \\type real
      \\units deltaC
  A7, \\field Surface Name Pair 6
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N10,\\field Delta Adjacent Air Temperature Pair 6
      \\type real
      \\units deltaC
  A8, \\field Surface Name Pair 7
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N11,\\field Delta Adjacent Air Temperature Pair 7
      \\type real
      \\units deltaC
  A9, \\field Surface Name Pair 8
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N12,\\field Delta Adjacent Air Temperature Pair 8
      \\type real
      \\units deltaC
  A10,\\field Surface Name Pair 9
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N13,\\field Delta Adjacent Air Temperature Pair 9
      \\type real
      \\units deltaC
  A11,\\field Surface Name Pair 10
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N14,\\field Delta Adjacent Air Temperature Pair 10
      \\type real
      \\units deltaC
  A12,\\field Surface Name Pair 11
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N15,\\field Delta Adjacent Air Temperature Pair 11
      \\type real
      \\units deltaC
  A13,\\field Surface Name Pair 12
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N16,\\field Delta Adjacent Air Temperature Pair 12
      \\type real
      \\units deltaC
  A14,\\field Surface Name Pair 13
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N17,\\field Delta Adjacent Air Temperature Pair 13
      \\type real
      \\units deltaC
  A15,\\field Surface Name Pair 14
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N18,\\field Delta Adjacent Air Temperature Pair 14
      \\type real
      \\units deltaC
  A16,\\field Surface Name Pair 15
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N19,\\field Delta Adjacent Air Temperature Pair 15
      \\type real
      \\units deltaC
  A17,\\field Surface Name Pair 16
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N20,\\field Delta Adjacent Air Temperature Pair 16
      \\type real
      \\units deltaC
  A18,\\field Surface Name Pair 17
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N21,\\field Delta Adjacent Air Temperature Pair 17
      \\type real
      \\units deltaC
  A19,\\field Surface Name Pair 18
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N22,\\field Delta Adjacent Air Temperature Pair 18
      \\type real
      \\units deltaC
  A20,\\field Surface Name Pair 19
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N23,\\field Delta Adjacent Air Temperature Pair 19
      \\type real
      \\units deltaC
  A21,\\field Surface Name Pair 20
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N24,\\field Delta Adjacent Air Temperature Pair 20
      \\type real
      \\units deltaC
  A22,\\field Surface Name Pair 21
      \\type object-list
      \\object-list AllHeatTranSurfNames
  N25;\\field Delta Adjacent Air Temperature Pair 21
      \\type real
      \\units deltaC

RoomAir:Node,
      \\memo Define an air node for some types of nodal room air models
  A1, \\field Name
      \\type Alpha
  A2, \\field Node Type
      \\required-field
      \\type choice
      \\key Inlet
      \\key Floor
      \\key Control
      \\key Ceiling
      \\key MundtRoom
      \\key Return
  A3, \\field Zone Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
  N1, \\field Height of Nodal Control Volume Center
      \\required-field
      \\type real
      \\units m
  A4 , \\field Surface 1 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A5 , \\field Surface 2 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A6 , \\field Surface 3 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A7 , \\field Surface 4 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A8 , \\field Surface 5 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A9 , \\field Surface 6 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A10, \\field Surface 7 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A11, \\field Surface 8 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A12, \\field Surface 9 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A13, \\field Surface 10 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A14, \\field Surface 11 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A15, \\field Surface 12 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A16, \\field Surface 13 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A17, \\field Surface 14 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A18, \\field Surface 15 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A19, \\field Surface 16 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A20, \\field Surface 17 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A21, \\field Surface 18 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A22, \\field Surface 19 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A23, \\field Surface 20 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  A24; \\field Surface 21 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames

RoomAirSettings:OneNodeDisplacementVentilation,
       \\memo The Mundt model for displacement ventilation
  A1 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Fraction of Convective Internal Loads Added to Floor Air
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N2 ; \\field Fraction of Infiltration Internal Loads Added to Floor Air
       \\type real
       \\minimum 0.0
       \\maximum 1.0

RoomAirSettings:ThreeNodeDisplacementVentilation,
       \\memo The UCSD model for Displacement Ventilation
  A1 , \\field Zone Name
       \\note Name of Zone being described. Any existing zone name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A2 , \\field Gain Distribution Schedule Name
       \\note Distribution of the convective heat gains between the occupied and mixed zones.
       \\note 0<= Accepted Value <= 1.
       \\note In the DV model 1 means all convective gains in the lower layer.
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Number of Plumes per Occupant
       \\note Used only in the UCSD displacement ventilation model.
       \\note Effective number of separate plumes per occupant in the occupied zone.
       \\note Plumes that merge together in the occupied zone count as one.
       \\required-field
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N2 , \\field Thermostat Height
       \\note Height of thermostat/temperature control sensor above floor
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.1
  N3 , \\field Comfort Height
       \\note Height at which air temperature is calculated for comfort purposes
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.1
  N4 ; \\field Temperature Difference Threshold for Reporting
       \\note Minimum temperature difference between predicted upper and lower layer
       \\note temperatures above which DV auxilliary outputs are calculated.
       \\note These outputs are 'DV Transition Height', 'DV Fraction Min Recommended Flow Rate'
       \\note 'DV Average Temp Gradient' and 'DV Maximum Temp Gradient'.  They
       \\note are set to negative values when the temperature difference is less than the
       \\note threshold and the output 'DV Zone Is Mixed' is set to 1
       \\units deltaC
       \\required-field
       \\type real
       \\minimum 0.0
       \\default 0.4

RoomAirSettings:CrossVentilation,
       \\memo This UCSD Cross Ventilation Room Air Model provides a simple model for heat transfer
       \\memo and vertical temperature profile prediction in cross ventilated rooms. The model
       \\memo distinguishes two regions in the room, the main jet region and the recirculations,
       \\memo and predicts characteristic airflow velocities and average air temperatures.
       \\memo Used with RoomAirModelType = CrossVentilation.
  A1 , \\field Zone Name
       \\note Name of Zone being described. Any existing zone name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A2 , \\field Gain Distribution Schedule Name
       \\note Distribution of the convective heat gains between the jet and recirculation zones.
       \\note 0<= Accepted Value <= 1.
       \\note In the CV model 1 means all convective gains in the jet region.
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A3 ; \\field Airflow Region Used for Thermal Comfort Evaluation
       \\note Required field whenever thermal comfort is predicted
       \\note defines Air temperature and Airflow velocity that will be used in the Fanger model
       \\note conditions must refer to one of the two regions: jet or recirculation
       \\type choice
       \\key Jet
       \\key Recirculation

RoomAirSettings:UnderFloorAirDistributionInterior,
       \\memo This Room Air Model is applicable to interior spaces that are served by an underfloor
       \\memo air distribution system. The dominant sources of heat gain should be from people,
       \\memo equipment, and other localized sources located in the occupied part of the room.
       \\memo The model should be used with caution in zones which have large heat gains or losses
       \\memo through exterior walls or windows or which have considerable direct solar gain.
       \\memo Used with RoomAirModelType = UnderFloorAirDistributionInterior.
       \\min-fields 15
  A1 , \\field Zone Name
       \\note Name of Zone with underfloor air distribution
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Number of Diffusers
       \\note Total number of diffusers in this zone
       \\type real
       \\minimum> 0.0
       \\autocalculatable
       \\default autocalculate
  N2 , \\field Power per Plume
       \\type real
       \\minimum 0.0
       \\units W
       \\autocalculatable
       \\default autocalculate
  N3 , \\field Design Effective Area of Diffuser
       \\type real
       \\units m2
       \\minimum> 0.0
       \\autocalculatable
       \\default Autocalculate
  N4 , \\field Diffuser Slot Angle from Vertical
       \\type real
       \\units deg
       \\minimum 0.0
       \\maximum 90.
       \\autocalculatable
       \\default Autocalculate
  N5 , \\field Thermostat Height
       \\note Height of thermostat/temperature control sensor above floor
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.2
  N6 , \\field Comfort Height
       \\note Height at which air temperature is calculated for comfort purposes
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.1
  N7 , \\field Temperature Difference Threshold for Reporting
       \\note Minimum temperature difference between predicted upper and lower layer
       \\note temperatures above which UFAD auxilliary outputs are calculated.
       \\note These outputs are 'UF Transition Height'and 'UF Average Temp Gradient'.  They
       \\note are set to zero values when the temperature difference is less than the
       \\note threshold and the output 'UF Zone Is Mixed' is set to 1
       \\units deltaC
       \\required-field
       \\type real
       \\minimum 0.0
       \\default 0.4
  A2 , \\field Floor Diffuser Type
       \\required-field
       \\type choice
       \\key Custom
       \\key Swirl
       \\key VariableArea
       \\key HorizontalSwirl
       \\key LinearBarGrille
       \\default Swirl
  N8 , \\field Transition Height
       \\note user-specified height above floor of boundary between occupied and upper subzones
       \\type real
       \\units m
       \\minimum> 0.0
       \\autocalculatable
       \\default 1.7
  N9,  \\field Coefficient A
       \\note Coefficient A in Formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N10, \\field Coefficient B
       \\note Coefficient B in Formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N11, \\field Coefficient C
       \\note Coefficient C in Formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N12, \\field Coefficient D
       \\note Coefficient D in Formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N13; \\field Coefficient E
       \\note Coefficient E in Formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate

RoomAirSettings:UnderFloorAirDistributionExterior,
       \\memo Applicable to exterior spaces that are served by an underfloor air distribution system.
       \\memo The dominant sources of heat gain should be from people, equipment, and other
       \\memo localized sources located in the occupied part of the room, as well as convective gain
       \\memo  coming from a warm window. Used with RoomAirModelType = CrossVentilation.
  \\min-fields 15
  A1 , \\field Zone Name
       \\note Name of Zone being described. Any existing zone name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Number of Diffusers per Zone
       \\type real
       \\minimum> 0.0
       \\autocalculatable
       \\default Autocalculate
  N2 , \\field Power per Plume
       \\type real
       \\minimum 0.0
       \\units W
       \\autocalculatable
       \\default autocalculate
  N3 , \\field Design Effective Area of Diffuser
       \\type real
       \\units m2
       \\minimum> 0.0
       \\autocalculatable
       \\default Autocalculate
  N4 , \\field Diffuser Slot Angle from Vertical
       \\type real
       \\units deg
       \\minimum 0.0
       \\maximum 90.
       \\autocalculatable
       \\default autocalculate
  N5 , \\field Thermostat Height
       \\note Height of thermostat/temperature control sensor above floor
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.2
  N6 , \\field Comfort Height
       \\note Height at which Air temperature is calculated for comfort purposes
       \\units m
       \\type real
       \\minimum> 0.0
       \\default 1.1
  N7 , \\field Temperature Difference Threshold for Reporting
       \\note Minimum temperature difference between upper and lower layer
       \\note temperatures above which UFAD auxilliary outputs are calculated.
       \\note These outputs are 'UF Transition Height'and 'UF Average Temp Gradient'.  They
       \\note are set to zero values when the temperature difference is less than the
       \\note threshold and the output 'UF Zone Is Mixed' is set to 1
       \\units deltaC
       \\required-field
       \\type real
       \\minimum 0.0
       \\default 0.4
  A2 , \\field Floor Diffuser Type
       \\required-field
       \\type choice
       \\key Custom
       \\key Swirl
       \\key VariableArea
       \\key HorizontalSwirl
       \\key LinearBarGrille
       \\default Swirl
  N8 , \\field Transition Height
       \\note User-specified height above floor of boundary between occupied and upper subzones
       \\type real
       \\units m
       \\minimum> 0.0
       \\autocalculatable
       \\default 1.7
  N9,  \\field Coefficient A in formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N10, \\field Coefficient B in formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N11, \\field Coefficient C in formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N12, \\field Coefficient D in formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate
  N13; \\field Coefficient E in formula Kc = A*Gamma**B + C + D*Gamma + E*Gamma**2
       \\note Kc is the fraction of the total zone load attributable to the lower subzone
       \\type real
       \\autocalculatable
       \\default Autocalculate

\\group Internal Gains

People,
   \\memo Sets internal gains and contaminant rates for occupants in the zone.
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference PeopleNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Number of People Schedule Name
       \\required-field
       \\type object-list
       \\note units in schedule should be fraction applied to number of people (0.0 - 1.0)
       \\object-list ScheduleNames
  A4 , \\field Number of People Calculation Method
       \\note The entered calculation method is used to create the maximum number of people
       \\note for this set of attributes (i.e. sensible fraction, schedule, etc)
       \\note Choices: People -- simply enter number of occupants.
       \\note People per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Number of people
       \\note Zone Floor Area per Person -- enter the number to apply.  Floor Area / Value = Number of people
       \\required-field
       \\type choice
       \\key People
       \\key People/Area
       \\key Area/Person
       \\default People
  N1 , \\field Number of People
       \\type real
       \\minimum 0
  N2 , \\field People per Zone Floor Area
       \\type real
       \\minimum 0
       \\units person/m2
  N3 , \\field Zone Floor Area per Person
       \\type real
       \\minimum 0
       \\units m2/person
  N4 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N5,  \\field Sensible Heat Fraction
       \\note if input, overrides program calculated sensible/latent split
       \\autocalculatable
       \\default autocalculate
       \\minimum 0.0
       \\maximum 1.0
  A5 , \\field Activity Level Schedule Name
       \\required-field
       \\note Note that W has to be converted to mets in TC routine
       \\type object-list
       \\note units in schedule are W/person
       \\object-list ScheduleNames
  N6 , \\field Carbon Dioxide Generation Rate
       \\note CO2 generation rate per unit of activity level.
       \\type real
       \\units m3/s-W
       \\default 3.82E-8
       \\minimum 0.0
       \\maximum 3.82E-7
       \\note The default value is obtained from ASHRAE Std 62.1 at 0.0084 cfm/met/person over
       \\note the general adult population.
  A6 , \\field Enable ASHRAE 55 Comfort Warnings
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A7 , \\field Mean Radiant Temperature Calculation Type
       \\note optional (only required for thermal comfort runs)
       \\type choice
       \\key ZoneAveraged
       \\key SurfaceWeighted
       \\key AngleFactor
       \\default ZoneAveraged
  A8 , \\field Surface Name/Angle Factor List Name
       \\type object-list
       \\object-list AllHeatTranAngFacNames
       \\note optional (only required for thermal comfort runs)
  A9 , \\field Work Efficiency Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note units in schedule are 0.0 to 1.0
       \\note optional (only required for thermal comfort runs)
  A10, \\field Clothing Insulation Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note use "Clo" from ASHRAE or Thermal Comfort guides
       \\note optional (only required for thermal comfort runs)
  A11, \\field Air Velocity Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note units in the schedule are m/s
       \\note optional (only required for thermal comfort runs)
  A12, \\field Thermal Comfort Model 1 Type
       \\type choice
       \\key Fanger
       \\key Pierce
       \\key KSU
       \\key AdaptiveASH55
       \\key AdaptiveCEN15251
       \\note optional (only needed for people thermal comfort results reporting)
  A13, \\field Thermal Comfort Model 2 Type
       \\type choice
       \\key Fanger
       \\key Pierce
       \\key KSU
       \\key AdaptiveASH55
       \\key AdaptiveCEN15251
       \\note optional (second type of thermal comfort model and results reporting)
  A14, \\field Thermal Comfort Model 3 Type
       \\type choice
       \\key Fanger
       \\key Pierce
       \\key KSU
       \\key AdaptiveASH55
       \\key AdaptiveCEN15251
       \\note optional (third thermal comfort model and report type)
  A15, \\field Thermal Comfort Model 4 Type
       \\type choice
       \\key Fanger
       \\key Pierce
       \\key KSU
       \\key AdaptiveASH55
       \\key AdaptiveCEN15251
       \\note optional (fourth thermal comfort model and report type)
  A16; \\field Thermal Comfort Model 5 Type
       \\type choice
       \\key Fanger
       \\key Pierce
       \\key KSU
       \\key AdaptiveASH55
       \\key AdaptiveCEN15251
       \\note optional (fifth thermal comfort model and report type)

ComfortViewFactorAngles,
       \\memo Used to specify radiant view factors for thermal comfort calculations.
  A1 , \\field Name
       \\type alpha
       \\reference AllHeatTranAngFacNames
  A2 , \\field Zone Name
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Surface 1 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N1 , \\field Angle Factor 1
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A4 , \\field Surface 2 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N2 , \\field Angle Factor 2
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A5 , \\field Surface 3 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N3 , \\field Angle Factor 3
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A6 , \\field Surface 4 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N4 , \\field Angle Factor 4
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A7 , \\field Surface 5 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N5 , \\field Angle Factor 5
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A8 , \\field Surface 6 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N6 , \\field Angle Factor 6
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A9 , \\field Surface 7 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N7 , \\field Angle Factor 7
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A10, \\field Surface 8 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N8 , \\field Angle Factor 8
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A11, \\field Surface 9 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N9 , \\field Angle Factor 9
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A12, \\field Surface 10 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N10, \\field Angle Factor 10
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A13, \\field Surface 11 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N11, \\field Angle Factor 11
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A14, \\field Surface 12 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N12, \\field Angle Factor 12
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A15, \\field Surface 13 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N13, \\field Angle Factor 13
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A16, \\field Surface 14 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N14, \\field Angle Factor 14
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A17, \\field Surface 15 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N15, \\field Angle Factor 15
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A18, \\field Surface 16 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N16, \\field Angle Factor 16
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A19, \\field Surface 17 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N17, \\field Angle Factor 17
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A20, \\field Surface 18 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N18, \\field Angle Factor 18
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A21, \\field Surface 19 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N19, \\field Angle Factor 19
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A22, \\field Surface 20 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N20; \\field Angle Factor 20
       \\type real
       \\minimum 0.0
       \\maximum 1.0

Lights,
   \\memo Sets internal gains for lights in the zone.
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
       \\min-fields 10
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference LightsNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in schedule should be fraction applied to design level of lights, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of lights
       \\note for this set of attributes
       \\note Choices: LightingLevel => Lighting Level -- simply enter watts of lights
       \\note Watts/Area => Watts per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Lights
       \\note Watts/Person => Watts per Person -- enter the number to apply.  Value * Occupants = Lights
       \\required-field
       \\type choice
       \\key LightingLevel
       \\key Watts/Area
       \\key Watts/Person
       \\default LightingLevel
  N1 , \\field Lighting Level
       \\units W
       \\type real
       \\minimum 0
       \\ip-units W
  N2 , \\field Watts per Zone Floor Area
       \\type real
       \\minimum 0
       \\units W/m2
       \\ip-units W/ft2
  N3 , \\field Watts per Person
       \\type real
       \\minimum 0
       \\units W/person
       \\ip-units W/person
  N4 , \\field Return Air Fraction
       \\required-field
       \\note Used only for sizing calculation if return-air-fraction
       \\note coefficients are specified.
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 , \\field Fraction Visible
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N7 , \\field Fraction Replaceable
       \\required-field
       \\note For Daylighting:Controls and Daylighting:DElight:Controls,
       \\note must be 0 or 1:  0 = no dimming control, 1 = full dimming control
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A5 , \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General
  A6 , \\field Return Air Fraction Calculated from Plenum Temperature
       \\type choice
       \\key Yes
       \\key No
       \\default No
  N8 , \\field Return Air Fraction Function of Plenum Temperature Coefficient 1
       \\note Used only if Return Air Fraction Is Calculated from Plenum Temperature = Yes
       \\note Equation is Return Air Fraction = Coefficient#1 - Coefficient#2 X PlenumTemp(degC)
       \\type real
       \\minimum 0.0
       \\default 0.0
  N9 ; \\field Return Air Fraction Function of Plenum Temperature Coefficient 2
       \\note Used only if Return Air Fraction Is Calculated from Plenum Temperature = Yes
       \\note Equation is Return Air Fraction = Coefficient#1 - Coefficient#2 X PlenumTemp(degC)
       \\type real
       \\units 1/K
       \\minimum 0.0
       \\default 0.0

ElectricEquipment,
   \\memo Sets internal gains for electric equipment in the zone.
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ElectricEquipmentNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in schedule should be fraction applied to design level of electric equipment, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of electric equipment
       \\note for this set of attributes
       \\note Choices: EquipmentLevel => Equipment Level -- simply enter watts of equipment
       \\note Watts/Area => Watts per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Equipment Level
       \\note Watts/Person => Watts per Person -- enter the number to apply.  Value * Occupants = Equipment Level
       \\required-field
       \\type choice
       \\key EquipmentLevel
       \\key Watts/Area
       \\key Watts/Person
       \\default EquipmentLevel
  N1 , \\field Design Level
       \\units W
       \\type real
       \\minimum 0
       \\ip-units W
  N2 , \\field Watts per Zone Floor Area
       \\type real
       \\minimum 0
       \\units W/m2
       \\ip-units W/ft2
  N3 , \\field Watts per Person
       \\type real
       \\minimum 0
       \\units W/person
       \\ip-units W/person
  N4 , \\field Fraction Latent
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 , \\field Fraction Lost
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  A5 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

GasEquipment,
   \\memo Sets internal gains and contaminant rates for gas equipment in the zone.
   \\memo If you use a ZoneList in the Zone name field then this definition applies to all those zones.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GasEquipmentNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to design level of gas equipment, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of gas equipment
       \\note for this set of attributes
       \\note Choices: EquipmentLevel => Design Level -- simply enter power input of equipment
       \\note Watts/Area or Power/Area => Power per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Equipment Level
       \\note Watts/Person or Power/Person => Power per Person -- enter the number to apply.  Value * Occupants = Equipment Level
       \\required-field
       \\type choice
       \\key EquipmentLevel
       \\key Watts/Area
       \\key Watts/Person
       \\key Power/Area
       \\key Power/Person
       \\default EquipmentLevel
  N1 , \\field Design Level
       \\units W
       \\type real
       \\minimum 0
       \\ip-units Btu/h
  N2 , \\field Power per Zone Floor Area
       \\type real
       \\minimum 0
       \\units W/m2
       \\ip-units Btu/h-ft2
  N3 , \\field Power per Person
       \\type real
       \\minimum 0
       \\units W/Person
       \\ip-units Btu/h-person
  N4 , \\field Fraction Latent
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 , \\field Fraction Lost
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N7 , \\field Carbon Dioxide Generation Rate
       \\note CO2 generation rate per unit of power input
       \\type real
       \\units m3/s-W
       \\ip-units (ft3/min)/(Btu/h)
       \\default 0.0
       \\minimum 0.0
       \\maximum 4.0E-7
       \\note The default value assumes the equipment is fully vented.
       \\note For unvented equipment, a suggested value is 3.45E-8 m3/s-W. This value is
       \\note converted from a natural gas CO2 emission rate of 117 lbs CO2 per million Btu.
       \\note The maximum value assumes to be 10 times of the recommended value.
  A5 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

HotWaterEquipment,
   \\memo Sets internal gains for hot water equipment in the zone.
   \\memo If you use a ZoneList in the Zone name field then this definition applies to all those zones.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HotWaterEquipmentNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to design level of hot water equipment, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of hot water equipment
       \\note for this set of attributes
       \\note Choices: EquipmentLevel => Design Level -- simply enter power input of equipment
       \\note Watts/Area or Power/Area => Power per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Equipment Level
       \\note Watts/Person or Power/Person => Power per Person -- enter the number to apply.  Value * Occupants = Equipment Level
       \\required-field
       \\type choice
       \\key EquipmentLevel
       \\key Watts/Area
       \\key Watts/Person
       \\key Power/Area
       \\key Power/Person
       \\default EquipmentLevel
  N1 , \\field Design Level
       \\units W
       \\type real
       \\minimum 0
       \\ip-units Btu/h
  N2 , \\field Power per Zone Floor Area
       \\type real
       \\minimum 0
       \\units W/m2
       \\ip-units Btu/h-ft2
  N3 , \\field Power per Person
       \\type real
       \\minimum 0
       \\units W/Person
       \\ip-units Btu/h-person
  N4 , \\field Fraction Latent
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 , \\field Fraction Lost
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  A5 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

SteamEquipment,
   \\memo Sets internal gains for steam equipment in the zone.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SteamEquipmentNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to design level of steam equipment, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of steam equipment
       \\note for this set of attributes
       \\note Choices: EquipmentLevel => Design Level -- simply enter power input of equipment
       \\note Watts/Area or Power/Area => Power per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Equipment Level
       \\note Watts/Person or Power/Person => Power per Person -- enter the number to apply.  Value * Occupants = Equipment Level
       \\required-field
       \\type choice
       \\key EquipmentLevel
       \\key Watts/Area
       \\key Watts/Person
       \\key Power/Area
       \\key Power/Person
       \\default EquipmentLevel
  N1 , \\field Design Level
       \\units W
       \\type real
       \\minimum 0
       \\ip-units Btu/h
  N2 , \\field Power per Zone Floor Area
       \\type real
       \\minimum 0
       \\units W/m2
       \\ip-units Btu/h-ft2
  N3 , \\field Power per Person
       \\type real
       \\minimum 0
       \\units W/Person
       \\ip-units Btu/h-person
  N4 , \\field Fraction Latent
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 , \\field Fraction Lost
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  A5 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

OtherEquipment,
   \\memo Sets internal gains or losses for "other" equipment in the zone.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference OtherEquipmentNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to design level of other equipment, generally (0.0 - 1.0)
  A4 , \\field Design Level Calculation Method
       \\note The entered calculation method is used to create the maximum amount of other equipment.
       \\note to set a loss, use a negative value in the following fields.
       \\note for this set of attributes
       \\note Choices: EquipmentLevel => Design Level -- simply enter power input of equipment
       \\note Watts/Area or Power/Area => Power per Zone Floor Area -- enter the number to apply.  Value * Floor Area = Equipment Level
       \\note Watts/Person or Power/Person => Power per Person -- enter the number to apply.  Value * Occupants = Equipment Level
       \\required-field
       \\type choice
       \\key EquipmentLevel
       \\key Watts/Area
       \\key Watts/Person
       \\key Power/Area
       \\key Power/Person
       \\default EquipmentLevel
  N1 , \\field Design Level
       \\units W
       \\type real
       \\ip-units Btu/h
  N2 , \\field Power per Zone Floor Area
       \\type real
       \\units W/m2
       \\ip-units Btu/h-ft2
  N3 , \\field Power per Person
       \\type real
       \\units W/Person
       \\ip-units Btu/h-person
  N4 , \\field Fraction Latent
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  N6 ; \\field Fraction Lost
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0

ZoneBaseboard:OutdoorTemperatureControlled,
       \\memo Specifies outside temperature-controlled electric baseboard heating.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference BaseboardHeatNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to capacity of the baseboard heat equipment, generally (0.0 - 1.0)
  N1 , \\field Capacity at Low Temperature
       \\required-field
       \\type real
       \\units W
       \\minimum> 0
  N2 , \\field Low Temperature
       \\required-field
       \\units C
       \\type real
  N3 , \\field Capacity at High Temperature
       \\required-field
       \\type real
       \\units W
       \\minimum 0
  N4 , \\field High Temperature
       \\required-field
       \\units C
       \\type real
  N5 , \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0
  A4 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

ZoneContaminantSourceAndSink:CarbonDioxide,
   \\memo Represents internal CO2 gains and sinks in the zone.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CO2GenerationNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Design Generation Rate
       \\units m3/s
       \\type real
       \\Note Positive values represent sources and negative values represent sinks.
  A3 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the Design Generation Rate

ZoneContaminantSourceAndSink:Generic:Constant,
   \\memo Sets internal generic contaminant gains and sinks in a zone with constant values.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantSourceAndSinkNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Design Generation Rate
       \\units m3/s
       \\type real
       \\minimum 0.0
       \\Note The values represent source.
  A3 , \\field Generation Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the Design Generation Rate
  N2 , \\field Design Removal Coefficient
       \\units m3/s
       \\type real
       \\minimum 0.0
       \\Note The value represent sink.
  A4 ; \\field Removal Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Design removal Coefficient

SurfaceContaminantSourceAndSink:Generic:PressureDriven,
   \\memo Simulate generic contaminant source driven by the pressure difference across a surface.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfAndSubSurfNames
  N1 , \\field Design Generation Rate Coefficient
       \\units m3/s
       \\type real
       \\minimum 0.0
  A3 , \\field Generation Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Design Generation Rate Coefficient
  N2 ; \\field Generation Exponent
       \\units dimensionless
       \\type real
       \\minimum> 0.0
       \\maximum 1.0

ZoneContaminantSourceAndSink:Generic:CutoffModel,
   \\memo Simulate generic contaminant source driven by the cutoff concentration model.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Design Generation Rate Coefficient
       \\units m3/s
       \\type real
       \\minimum 0.0
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Design Generation Rate Coefficient
  N2 ; \\field Cutoff Generic Contaminant at which Emission Ceases
       \\units ppm
       \\type real
       \\minimum> 0.0
       \\note When the zone concentration level is greater than the cutoff level, emission stops,
       \\note and the source level is zero.

ZoneContaminantSourceAndSink:Generic:DecaySource,
   \\memo Simulate generic contaminant source driven by the cutoff concentration model.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Initial Emission Rate
       \\units m3/s
       \\type real
       \\minimum 0.0
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Initial Emission Rate. When the value is equal to 1.0, the time will be reset to
       \\note zero.
  N2 ; \\field Delay Time Constant
       \\units s
       \\type real
       \\minimum> 0.0

SurfaceContaminantSourceAndSink:Generic:BoundaryLayerDiffusion,
   \\memo Simulate generic contaminant source driven by the boundary layer diffusion controlled model.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  N1 , \\field Mass Transfer Coefficient
       \\units m/s
       \\type real
       \\minimum 0.0
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Initial Emission Rate. When the value is equal to 1.0, the time will be reset to
       \\note zero.
  N2 ; \\field Henry adsorption constant or partition coefficient
       \\units dimensionless
       \\type real
       \\minimum> 0.0

SurfaceContaminantSourceAndSink:Generic:DepositionVelocitySink,
   \\memo Simulate generic contaminant source driven by the boundary layer diffusion controlled model.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list SurfaceNames
  N1 , \\field Deposition Velocity
       \\units m/s
       \\type real
       \\minimum 0.0
  A3 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Initial Emission Rate. When the value is equal to 1.0, the time will be reset to
       \\note zero.

ZoneContaminantSourceAndSink:Generic:DepositionRateSink,
   \\memo Simulate generic contaminant source driven by the boundary layer diffusion controlled model.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference GenericContaminantGenerationNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Deposition Rate
       \\units m/s
       \\type real
       \\minimum 0.0
  A3 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Value in this schedule should be a fraction (generally 0.0 - 1.0) applied to the
       \\note Initial Emission Rate. When the value is equal to 1.0, the time will be reset to
       \\note zero.

\\group Daylighting

Daylighting:Controls,
       \\min-fields 19
       \\memo Dimming of overhead electric lighting is determined from
       \\memo interior daylight illuminance calculated at one or two reference points.
       \\memo reference points are given in coordinates specified in the GlobalGeometryRules object
       \\memo Daylighting Reference Point CoordinateSystem field
       \\memo Glare from daylighting is also calculated.
  A1 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Total Daylighting Reference Points
       \\type integer
       \\minimum 1
       \\maximum 2
       \\default 1
  N2 , \\field X-Coordinate of First Reference Point
       \\units m
       \\required-field
       \\type real
  N3 , \\field Y-Coordinate of First Reference Point
       \\units m
       \\required-field
       \\type real
  N4 , \\field Z-Coordinate of First Reference Point
       \\units m
       \\default 0.8
       \\type real
  N5 , \\field X-Coordinate of Second Reference Point
       \\units m
       \\type real
       \\note Required if Total Daylighting Reference Points = 2
  N6 , \\field Y-Coordinate of Second Reference Point
       \\units m
       \\type real
       \\note Required if Total Daylighting Reference Points = 2
  N7 , \\field Z-Coordinate of Second Reference Point
       \\units m
       \\type real
       \\minimum 0.0
       \\default 0.8
  N8 , \\field Fraction of Zone Controlled by First Reference Point
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N9 , \\field Fraction of Zone Controlled by Second Reference Point
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N10, \\field Illuminance Setpoint at First Reference Point
       \\units lux
       \\type real
       \\minimum 0.0
       \\default 500
  N11, \\field Illuminance Setpoint at Second Reference Point
       \\units lux
       \\type real
       \\minimum 0.0
       \\default 500
  N12, \\field Lighting Control Type
       \\type integer
       \\minimum 1
       \\maximum 3
       \\note 1=continuous,2=stepped,3=continuous/off
       \\default 1
  N13, \\field Glare Calculation Azimuth Angle of View Direction Clockwise from Zone y-Axis
       \\required-field
       \\units deg
       \\minimum 0
       \\maximum 360
  N14, \\field Maximum Allowable Discomfort Glare Index
       \\type real
       \\minimum 1
       \\note The default is for general office work
       \\default 22
  N15, \\field Minimum Input Power Fraction for Continuous Dimming Control
       \\type real
       \\minimum 0.0
       \\maximum 0.6
       \\default 0.3
  N16, \\field Minimum Light Output Fraction for Continuous Dimming Control
       \\type real
       \\minimum 0.0
       \\maximum 0.6
       \\default 0.2
  N17, \\field Number of Stepped Control Steps
       \\note for Lighting Control Type=2, this field cannot be zero.
       \\type integer
       \\default 1
  N18, \\field Probability Lighting will be Reset When Needed in Manual Stepped Control
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A2 ; \\field Availability Schedule Name
       \\type object-list
       \\object-list ScheduleNames

Daylighting:DELight:Controls,
       \\min-fields 8
       \\memo Dimming of overhead electric lighting is determined from
       \\memo DElight calculated interior daylight illuminance at one or more reference points.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DElightZoneNames
  A2 , \\field Zone Name
       \\note Name of Thermal Zone hosting the given DElight Zone
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Lighting Control Type
       \\type integer
       \\minimum 1
       \\maximum 3
       \\note 1=continuous,2=stepped,3=continuous/off
       \\default 1
  N2 , \\field Minimum Input Power Fraction for Continuous Dimming Control
       \\type real
       \\minimum 0.0
       \\maximum 0.6
       \\default 0.3
  N3 , \\field Minimum Light Output Fraction for Continuous Dimming Control
       \\type real
       \\minimum 0.0
       \\maximum 0.6
       \\default 0.2
  N4 , \\field Number of Stepped Control Steps
       \\type integer
       \\note for Lighting Control Type=2, this field cannot be zero.
       \\default 1
  N5 , \\field Probability Lighting will be Reset When Needed in Manual Stepped Control
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N6;  \\field Gridding Resolution
       \\units m2
       \\type real
       \\note Maximum surface area for nodes in gridding all surfaces in the DElight zone.
       \\note All reflective and transmitting surfaces will be subdivided
       \\note into approximately square nodes that do not exceed this maximum.
       \\note Higher resolution subdivisions require greater calculation times,
       \\note but generally produce more accurate results.
       \\minimum> 0.0

Daylighting:DELight:ReferencePoint,
       \\min-fields 7
       \\memo DElight reference point for illuminance calculation and electric lighting dimming.
       \\memo reference points are given in coordinates specified in the GlobalGeometryRules object
       \\memo Daylighting Reference Point CoordinateSystem field
       \\memo There is a maximum number of 100 reference points per DElight daylighting zone.
  A1 , \\field Name
       \\type alpha
  A2 , \\field DElight Name
       \\required-field
       \\type object-list
       \\object-list DElightZoneNames
  N1 , \\field X-coordinate of Reference Point
       \\required-field
       \\units m
       \\type real
  N2,  \\field Y-coordinate of Reference Point
       \\required-field
       \\units m
       \\type real
  N3,  \\field Z-coordinate of Reference Point
       \\required-field
       \\units m
       \\type real
  N4 , \\field Fraction of Zone Controlled by Reference Point
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N5;  \\field Illuminance Setpoint at Reference Point
       \\required-field
       \\units lux
       \\type real
       \\minimum 0.0
       \\default 500

Daylighting:DELight:ComplexFenestration,
       \\min-fields 5
       \\memo Used for DElight Complex Fenestration of all types
  A1,  \\field Name
       \\required-field
       \\note Only used for user reference
       \\type alpha
  A2,  \\field Complex Fenestration Type
       \\required-field
       \\note Used to select the appropriate Complex Fenestration BTDF data
       \\type alpha
  A3,  \\field Building Surface Name
       \\required-field
       \\note This is a reference to a valid surface object (such as BuildingSurface:Detailed) hosting
       \\note this complex fenestration, analogous to the base surface Name
       \\note field for subsurfaces such as Windows.
       \\type object-list
       \\object-list SurfaceNames
  A4,  \\field Window Name
       \\required-field
       \\note This is a reference to a valid FenestrationSurface:Detailed window object
       \\note used to account for the geometry, and the solar and thermal gains/losses,
       \\note of the Complex Fenestration
       \\type object-list
       \\object-list SubSurfNames
  N1;  \\field Fenestration Rotation
       \\units deg
       \\type real
       \\note In-plane counter-clockwise rotation angle of the Complex Fenestration
       \\note optical reference direction and the base edge of the Complex Fenestration.
       \\note The Rotation will typically be zero when the host and CFS surfaces
       \\note are rectangular and height and width edges are aligned.
       \\default 0.0

DaylightingDevice:Tubular,
       \\memo Defines a tubular daylighting device (TDD) consisting of three components:
       \\memo a dome, a pipe, and a diffuser. The dome and diffuser are defined separately using the
       \\memo FenestrationSurface:Detailed object.
       \\extensible:2 Duplicate the last two fields Transition Zone Name and Transition Zone Length
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Dome Name
       \\note This must refer to a subsurface object of type TubularDaylightDome
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  A3 , \\field Diffuser Name
       \\note This must refer to a subsurface object of type TubularDaylightDiffuser
       \\note Delivery zone is specified in the diffuser object
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  A4 , \\field Construction Name
       \\required-field
       \\type object-list
       \\object-list ConstructionNames
  N1 , \\field Diameter
       \\required-field
       \\units m
       \\type real
       \\minimum> 0
  N2 , \\field Total Length
       \\note The exterior exposed length is the difference between total and sum of zone lengths
       \\required-field
       \\units m
       \\type real
       \\minimum> 0
  N3 , \\field Effective Thermal Resistance
       \\note R value between TubularDaylightDome and TubularDaylightDiffuser
       \\required-field
       \\units m2-K/W
       \\type real
       \\minimum> 0
       \\default 0.28
  A5 , \\field Transition Zone 1 Name
       \\begin-extensible
       \\type object-list
       \\object-list ZoneNames
  N4 , \\field Transition Zone 1 Length
       \\units m
       \\type real
       \\minimum 0.0
  A6 , \\field Transition Zone 2 Name
       \\type object-list
       \\object-list ZoneNames
  N5 , \\field Transition Zone 2 Length
       \\units m
       \\type real
       \\minimum 0.0
  A7 , \\field Transition Zone 3 Name
       \\type object-list
       \\object-list ZoneNames
  N6 , \\field Transition Zone 3 Length
       \\units m
       \\type real
       \\minimum 0.0
  A8 , \\field Transition Zone 4 Name
       \\type object-list
       \\object-list ZoneNames
  N7; \\field Transition Zone 4 Length
       \\units m
       \\type real
       \\minimum 0.0

DaylightingDevice:Shelf,
       \\memo Defines a daylighting which can have an inside shelf, an outside shelf, or both.
       \\memo The inside shelf is defined as a building surface and the outside shelf is defined
       \\memo as a shading surface.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Window Name
       \\required-field
       \\type object-list
       \\object-list SubSurfNames
  A3 , \\field Inside Shelf Name
       \\note This must refer to a BuildingSurface:Detailed or equivalent object
       \\note This surface must be its own Surface for other side boundary conditions.
       \\type object-list
       \\object-list SurfaceNames
  A4 , \\field Outside Shelf Name
       \\note This must refer to a Shading:Zone:Detailed object
       \\type object-list
       \\object-list AttachedShadingSurfNames
  A5 , \\field Outside Shelf Construction Name
       \\note Required if outside shelf is specified
       \\type object-list
       \\object-list ConstructionNames
  N1 ; \\field View Factor to Outside Shelf
       \\type real
       \\minimum 0.0
       \\maximum 1.0

DaylightingDevice:LightWell,
       \\min-fields 5
       \\memo Applies only to exterior windows in daylighting-controlled zones or
       \\memo in zones that share an interior window with a daylighting-controlled  zone.
       \\memo Generally used with skylights.
  A1,  \\field Exterior Window Name
       \\required-field
       \\type alpha
       \\object-list SubSurfNames
  N1,  \\field Height of Well
       \\note Distance from Bottom of Window to Bottom of Well
       \\required-field
       \\type real
       \\units m
       \\minimum 0.0
  N2,  \\field Perimeter of Bottom of Well
       \\required-field
       \\units m
       \\type real
       \\minimum> 0.0
  N3,  \\field Area of Bottom of Well
       \\required-field
       \\type real
       \\units m2
       \\minimum> 0.0
  N4;  \\field Visible Reflectance of Well Walls
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0

Output:DaylightFactors,
   \\memo Reports hourly daylight factors for each exterior window for four sky types
   \\memo (clear, turbid clear, intermediate, and overcast).
  \\format singleLine
   A1;  \\field Reporting Days
   \\required-field
   \\type choice
   \\key SizingDays
   \\key AllShadowCalculationDays

Output:IlluminanceMap,
   \\min-fields 9
   \\memo reference points are given in coordinates specified in the GlobalGeometryRules object
   \\memo Daylighting Reference Point CoordinateSystem field
  A1 , \\field Name
       \\required-field
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Z height
       \\units m
       \\type real
       \\default 0.0
  N2 , \\field X Minimum Coordinate
       \\units m
       \\type real
       \\default 0.0
  N3 , \\field X Maximum Coordinate
       \\units m
       \\type real
       \\default 1.0
  N4 , \\field Number of X Grid Points
       \\note Maximum number of total grid points must be <= 2500 (X*Y)
       \\type integer
       \\minimum> 0
       \\default 2
  N5 , \\field Y Minimum Coordinate
       \\units m
       \\type real
       \\default 0.0
  N6 , \\field Y Maximum Coordinate
       \\units m
       \\type real
       \\default 1.0
  N7 ; \\field Number of Y Grid Points
       \\note Maximum number of total grid points must be <= 2500 (X*Y)
       \\type integer
       \\minimum> 0
       \\default 2

OutputControl:IlluminanceMap:Style,
       \\memo default style for the Daylighting Illuminance Map is comma -- this works well for
       \\memo importing into spreadsheet programs such as Excel(tm) but not so well for word
       \\memo processing progams -- there tab may be a better choice.  fixed puts spaces between
       \\memo the "columns"
       \\unique-object
   A1; \\field Column Separator
       \\type choice
       \\key Comma
       \\key Tab
       \\key Fixed
       \\default Comma

\\group Zone Airflow

ZoneInfiltration:DesignFlowRate,
       \\memo  Infiltration is specified as a design level which is modified by a Schedule fraction, temperature difference and wind speed:
       \\memo  Infiltration=Idesign * FSchedule * (A + B*|(Tzone-Todb)| + C*WindSpd + D * WindSpd**2)
       \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
       \\memo to all the zones in the ZoneList.
       \\min-fields 12
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Design Flow Rate Calculation Method
       \\note The entered calculation method is used to create the maximum amount of infiltration
       \\note for this set of attributes
       \\note Choices: Flow/Zone => Design Flow Rate -- simply enter Design Flow Rate
       \\note Flow/Area => Flow per Zone Floor Area - Value * Floor Area (zone) = Design Flow Rate
       \\note Flow/ExteriorArea => Flow per Exterior Surface Area - Value * Exterior Surface Area (zone) = Design Flow Rate
       \\note Flow/ExteriorWallArea => Flow per Exterior Surface Area - Value * Exterior Wall Surface Area (zone) = Design Flow Rate
       \\note AirChanges/Hour => Air Changes per Hour - Value * Floor Volume (zone) adjusted for m3/s = Design Volume Flow Rate
       \\note  "Idesign" in Equation is the result.
       \\required-field
       \\type choice
       \\key Flow/Zone
       \\key Flow/Area
       \\key Flow/ExteriorArea
       \\key Flow/ExteriorWallArea
       \\key AirChanges/Hour
       \\default Flow/Zone
  N1 , \\field Design Flow Rate
       \\units m3/s
       \\type real
       \\minimum 0
       \\ip-units ft3/min
  N2 , \\field Flow per Zone Floor Area
       \\type real
       \\minimum 0
       \\units m3/s-m2
  N3 , \\field Flow per Exterior Surface Area
       \\note use key Flow/ExteriorArea for all exterior surface area
       \\note use key Flow/ExteriorWallArea to include only exterior wall area
       \\units m3/s-m2
       \\type real
       \\minimum 0
  N4 , \\field Air Changes per Hour
       \\units 1/hr
       \\type real
       \\minimum 0
  N5 , \\field Constant Term Coefficient
       \\required-field
       \\note  "A" in Equation
       \\type real
       \\default 1
  N6 , \\field Temperature Term Coefficient
       \\required-field
       \\note  "B" in Equation
       \\type real
       \\default 0
  N7 , \\field Velocity Term Coefficient
       \\required-field
       \\note  "C" in Equation
       \\type real
       \\default 0
  N8 ; \\field Velocity Squared Term Coefficient
       \\required-field
       \\note  "D" in Equation
       \\type real
       \\default 0

ZoneInfiltration:EffectiveLeakageArea,
       \\min-fields 6
       \\memo  Infiltration is specified as effective leakage area at 4 Pa, schedule fraction, stack and wind coefficients, and
       \\memo  is a function of temperature difference and wind speed:
       \\memo  Infiltration=FSchedule * (AL /1000) SQRT(Cs*|(Tzone-Todb)| +  Cw*WindSpd**2 )
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Effective Air Leakage Area
       \\units cm2
       \\note "AL" in Equation
       \\note units are cm2 (square centimeters)
       \\minimum> 0
       \\type real
       \\required-field
  N2 , \\field Stack Coefficient
       \\note "Cs" in Equation
       \\minimum> 0
       \\type real
       \\required-field
  N3 ; \\field Wind Coefficient
       \\note "Cw" in Equation
       \\minimum> 0
       \\type real
       \\required-field


ZoneInfiltration:FlowCoefficient,
       \\min-fields 8
       \\memo  Infiltration is specified as flow coefficient, schedule fraction, stack and wind coefficients, and
       \\memo  is a function of temperature difference and wind speed:
       \\memo  Infiltration=FSchedule * SQRT( (c * Cs*|(Tzone-Todb)|**n)**2 + (c* Cw*(s * WindSpd)**2n)**2 )
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Flow Coefficient
       \\note "c" in Equation
       \\minimum> 0
       \\type real
       \\required-field
  N2 , \\field Stack Coefficient
       \\note "Cs" in Equation
       \\minimum> 0
       \\type real
       \\required-field
  N3 , \\field Pressure Exponent
       \\note "n" in Equation
       \\minimum> 0
       \\type real
       \\required-field
       \\default 0.67
  N4 , \\field Wind Coefficient
       \\note "Cw" in Equation
       \\minimum> 0
       \\type real
       \\required-field
  N5 ; \\field Shelter Factor
       \\note "s" in Equation
       \\minimum> 0
       \\type real
       \\required-field


ZoneVentilation:DesignFlowRate,
   \\memo  Ventilation is specified as a design level which is modified by a schedule fraction, temperature difference and wind speed:
   \\memo  Ventilation=Vdesign * Fschedule * (A + B*|(Tzone-Todb)| + C*WindSpd + D * WindSpd**2)
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
   \\min-fields 15
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference VentilationNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Design Flow Rate Calculation Method
       \\note The entered calculation method is used to create the maximum amount of ventilation
       \\note for this set of attributes
       \\note Choices: Flow/Zone => Design Flow Rate -- simply enter Design Flow Rate
       \\note Flow/Area => Flow Rate per Zone Floor Area - Value * Floor Area (zone) = Design Flow Rate
       \\note Flow/Person => Flow Rate per Person - Value * #people = Design Flow Rate
       \\note AirChanges/Hour => Air Changes per Hour - Value * Floor Volume (zone) adjusted for m3/s = Design Volume Flow Rate
       \\note  "Vdesign" in Equation is the result.
       \\required-field
       \\type choice
       \\key Flow/Zone
       \\key Flow/Area
       \\key Flow/Person
       \\key AirChanges/Hour
       \\default Flow/Zone
  N1 , \\field Design Flow Rate
       \\units m3/s
       \\type real
       \\minimum 0
  N2 , \\field Flow Rate per Zone Floor Area
       \\type real
       \\minimum 0
       \\units m3/s-m2
  N3 , \\field Flow Rate per Person
       \\type real
       \\minimum 0
       \\units m3/s-person
  N4 , \\field Air Changes per Hour
       \\units 1/hr
       \\type real
       \\minimum 0
  A5 , \\field Ventilation Type
       \\required-field
       \\type choice
       \\key Natural
       \\key Intake
       \\key Exhaust
       \\key Balanced
       \\default Natural
  N5 , \\field Fan Pressure Rise
       \\required-field
       \\note pressure rise across the fan
       \\type real
       \\units Pa
       \\minimum 0
       \\default 0
  N6 , \\field Fan Total Efficiency
       \\required-field
       \\type real
       \\minimum> 0
       \\default 1
  N7 , \\field Constant Term Coefficient
       \\required-field
       \\note  "A" in Equation
       \\type real
       \\default 1
  N8 , \\field Temperature Term Coefficient
       \\required-field
       \\note  "B" in Equation
       \\type real
       \\default 0
  N9 , \\field Velocity Term Coefficient
       \\required-field
       \\note  "C" in Equation
       \\type real
       \\default 0
  N10, \\field Velocity Squared Term Coefficient
       \\required-field
       \\note  "D" in Equation
       \\type real
       \\default 0
  N11, \\field Minimum Indoor Temperature
       \\note this is the indoor temperature below which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
  A6 , \\field Minimum Indoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time below which
       \\note ventilation is shutoff.
  N12, \\field Maximum Indoor Temperature
       \\note this is the indoor temperature above which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
  A7 , \\field Maximum Indoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time above which
       \\note ventilation is shutoff.
  N13, \\field Delta Temperature
       \\note This is the temperature differential between indoor and outdoor below which ventilation is shutoff.
       \\note If ((IndoorTemp - OutdoorTemp) < DeltaTemperature) then ventilation is not allowed.
       \\note For example, if delta temperature is 2C, ventilation is assumed to be available if the outside air temperature
       \\note is at least 2C cooler than the zone air temperature. The values for this field can include negative numbers.
       \\note This allows ventilation to occur even if the outdoor temperature is above the indoor temperature.
       \\units deltaC
       \\type real
       \\minimum -100
       \\default -100
  A8 , \\field Delta Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the temperature differential between indoor and outdoor
       \\note versus time below which ventilation is shutoff.
  N14, \\field Minimum Outdoor Temperature
       \\note this is the outdoor temperature below which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
  A9 , \\field Minimum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time below which
       \\note ventilation is shutoff.
  N15, \\field Maximum Outdoor Temperature
       \\note this is the outdoor temperature above which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
  A10, \\field Maximum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time above which
       \\note ventilation is shutoff.
  N16; \\field Maximum Wind Speed
       \\note this is the outdoor wind speed above which ventilation is shutoff
       \\units m/s
       \\type real
       \\minimum 0
       \\maximum 40
       \\default 40

ZoneVentilation:WindandStackOpenArea,
  \\min-fields 8
       \\memo This object is specified as natural ventilation driven by wind and stack effect only:
       \\memo Ventilation Wind = Cw * Opening Area * Schedule * WindSpd
       \\memo Ventilation Stack = Cd * Opening Area * Schedule * SQRT(2*g*DH*(|(Tzone-Todb)|/Tzone))
       \\memo Total Ventilation = SQRT((Ventilation Wind)^2 + (Ventilation Stack)^2)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference VentilationNames
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Opening Area
       \\note This is the opening area used to calculate stack effect and wind driven ventilation.
       \\units m2
       \\type real
       \\minimum 0
       \\default 0
  A3 , \\field Opening Area Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the fraction values applied to the opening area given in the previous
       \\note input field (0.0 - 1.0).
  N2 , \\field Opening Effectiveness
       \\note This field is used to calculate wind driven ventilation.
       \\note  "Cw" in the wind-driven equation and the maximum value is 1.0.
       \\note When the input is Autocalculate, the program calculates Cw based on an angle between
       \\note wind direction and effective angle
       \\note Cw = 0.55 at angle = 0, and Cw = 0.3 at angle=180
       \\note Linear interpolation is used to calculate Cw based on the above two values.
       \\units dimensionless
       \\autocalculatable
       \\type real
       \\minimum 0
       \\maximum 1
       \\default Autocalculate
  N3 , \\field Effective Angle
       \\note This field is defined as normal angle of the opening area and is used when input
       \\note field Opening Effectiveness = Autocalculate.
       \\type real
       \\units deg
       \\minimum 0.0
       \\maximum< 360.0
       \\default 0
  N4 , \\field Height Difference
       \\note This is the height difference between the midpoint of an opening and
       \\note the neutral pressure level.
       \\units m
       \\type real
       \\minimum 0
       \\default 0
       \\note  "DH" in the stack equation.
  N5 , \\field Discharge Coefficient for Opening
       \\note This is the discharge coefficient used to calculate stack effect.
       \\note  "Cd" in the stack equation and the maximum value is 1.0.
       \\note When the input is Autocalculate, the following equation is used to calculate the
       \\note coefficient:
       \\note Cd = 0.4 + 0.0045*|(Tzone-Todb)|
       \\autocalculatable
       \\minimum 0.0
       \\maximum 1.0
       \\default Autocalculate
  N6 , \\field Minimum Indoor Temperature
       \\note This is the indoor temperature below which ventilation is shutoff.
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
  A4 , \\field Minimum Indoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time below which
       \\note ventilation is shutoff.
  N7 , \\field Maximum Indoor Temperature
       \\note This is the indoor temperature above which ventilation is shutoff.
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
  A5 , \\field Maximum Indoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time above which
       \\note ventilation is shutoff.
  N8 , \\field Delta Temperature
       \\note This is the temperature differential between indoor and outdoor below
       \\note which ventilation is shutoff.
       \\units deltaC
       \\type real
       \\minimum -100
       \\default -100
  A6 , \\field Delta Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the temperature differential between indoor and outdoor
       \\note versus time below which ventilation is shutoff.
  N9 , \\field Minimum Outdoor Temperature
       \\note This is the outdoor temperature below which ventilation is shutoff.
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
  A7 , \\field Minimum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time below which
       \\note ventilation is shutoff.
  N10, \\field Maximum Outdoor Temperature
       \\note This is the outdoor temperature above which ventilation is shutoff.
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
  A8 , \\field Maximum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time above which
       \\note ventilation is shutoff.
  N11; \\field Maximum Wind Speed
       \\note This is the outdoor wind speed above which ventilation is shutoff.
       \\units m/s
       \\type real
       \\minimum 0
       \\maximum 40
       \\default 40

ZoneAirBalance:OutdoorAir,
       \\memo Provide a combined zone outdoor air flow by including interactions between
       \\memo mechanical ventilation, infiltration and duct leakage.
       \\memo This object will combine outdoor flows from all ZoneInfiltration and
       \\memo ZoneVentilation objects in the same zone. Balanced flows will be summed, while
       \\memo unbalanced flows will be added in quadrature.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Air Balance Method
       \\type choice
       \\key Quadrature
       \\key None
       \\default Quadrature
       \\note None: Only perform simple calculations without using a combined zone outdoor air.
       \\note Quadrature: A combined outdoor air is used in the quadrature sum.
  N1 , \\field Induced Outdoor Air Due to Unbalanced Duct Leakage
       \\units m3/s
       \\type real
       \\minimum 0
       \\default 0
  A4 ; \\field Induced Outdoor Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the fraction values applied to the Induced Outdoor Air given in the
       \\note previous input field (0.0 - 1.0).

ZoneMixing,
       \\memo ZoneMixing is a simple air exchange from one zone to another. Note that this statement
       \\memo only affects the energy balance of the "receiving" zone and will not produce
       \\memo any effect on the "source" zone. Mixing statements can be complementary and include
       \\memo multiple zones, but the balancing of flows between zones is left to the user's
       \\memo discretion.
 \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Design Flow Rate Calculation Method
       \\note The entered calculation method is used to create the maximum amount of ventilation
       \\note for this set of attributes
       \\note Choices: Flow/Zone => Design Flow Rate -- simply enter Design Flow Rate
       \\note Flow/Area => Flow Rate per Zone Floor Area - Value * Floor Area (zone) = Design Flow Rate
       \\note Flow/Person => Flow Rate per Person - Value * #people = Design Flow Rate
       \\note AirChanges/Hour => Air Changes per Hour - Value * Floor Volume (zone) adjusted for m3/s = Design Volume Flow Rate
       \\note  "Vdesign" in Equation is the result.
       \\required-field
       \\type choice
       \\key Flow/Zone
       \\key Flow/Area
       \\key Flow/Person
       \\key AirChanges/Hour
       \\default Flow/Zone
  N1 , \\field Design Flow Rate
       \\units m3/s
       \\type real
       \\minimum 0
  N2 , \\field Flow Rate per Zone Floor Area
       \\type real
       \\minimum 0
       \\units m3/s-m2
  N3 , \\field Flow Rate per Person
       \\type real
       \\minimum 0
       \\units m3/s-person
  N4 , \\field Air Changes per Hour
       \\units 1/hr
       \\type real
       \\minimum 0
  A5 , \\field Source Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N5 , \\field Delta Temperature
       \\units deltaC
       \\type real
       \\default 0
       \\note This field contains the constant temperature differential between source and
       \\note receiving zones below which mixing is shutoff.
  A6 , \\field Delta Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the temperature differential between source and receiving
       \\note zones versus time below which mixing is shutoff.
  A7 , \\field Minimum Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the zone dry-bulb temperature versus time below which
       \\note mixing is shutoff.
  A8 , \\field Maximum Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the zone dry-bulb temperature versus time above which
       \\note mixing is shutoff.
  A9 , \\field Minimum Source Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the source zone dry-bulb temperature versus time below
       \\note which mixing is shutoff.
  A10, \\field Maximum Source Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the source zone dry-bulb temperature versus time above
       \\note which mixing is shutoff.
  A11, \\field Minimum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time below which
       \\note mixing is shutoff.
  A12; \\field Maximum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time above which
       \\note mixing is shutoff.

ZoneCrossMixing,
       \\memo ZoneCrossMixing exchanges an equal amount of air between two zones. Note that this
       \\memo statement affects the energy balance of both zones.
  \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Design Flow Rate Calculation Method
       \\note The entered calculation method is used to create the maximum amount of ventilation
       \\note for this set of attributes
       \\note Choices: Flow/Zone => Design Flow Rate -- simply enter Design Flow Rate
       \\note Flow/Area => Flow Rate per Zone Floor Area - Value * Floor Area (zone) = Design Flow Rate
       \\note Flow/Person => Flow Rate per Person - Value * #people = Design Flow Rate
       \\note AirChanges/Hour => Air Changes per Hour - Value * Floor Volume (zone) adjusted for m3/s = Design Volume Flow Rate
       \\note  "Vdesign" in Equation is the result.
       \\required-field
       \\type choice
       \\key Flow/Zone
       \\key Flow/Person
       \\key Flow/Area
       \\key AirChanges/Hour
       \\default Flow/Zone
  N1 , \\field Design Flow Rate
       \\units m3/s
       \\type real
       \\minimum 0
  N2 , \\field Flow Rate per Zone Floor Area
       \\type real
       \\minimum 0
       \\units m3/s-m2
  N3 , \\field Flow Rate per Person
       \\type real
       \\minimum 0
       \\units m3/s-person
  N4 , \\field Air Changes per Hour
       \\units 1/hr
       \\type real
       \\minimum 0
  A5 , \\field Source Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  N5 , \\field Delta Temperature
       \\units deltaC
       \\minimum 0.0
       \\type real
       \\default 0
       \\note This field contains the constant temperature differential between source and
       \\note receiving zones below which cross mixing is shutoff. This value must be greater
       \\note than or equal to zero.
  A6 , \\field Delta Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the temperature differential between source and receiving
       \\note zones versus time below which cross mixing is shutoff.
  A7 , \\field Minimum Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time below which
       \\note cross mixing is shutoff.
  A8 , \\field Maximum Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the indoor temperature versus time above which
       \\note cross mixing is shutoff.
  A9 , \\field Minimum Source Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the source zone dry-bulb temperature versus time below
       \\note which cross mixing is shutoff.
  A10, \\field Maximum Source Zone Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the source zone dry-bulb temperature versus time above
       \\note which cross mixing is shutoff.
  A11, \\field Minimum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time below which
       \\note cross mixing is shutoff.
  A12; \\field Maximum Outdoor Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This schedule contains the outdoor temperature versus time above which
       \\note cross mixing is shutoff.

ZoneRefrigerationDoorMixing,
  \\min-fields 4
       \\memo Refrigeration Door Mixing is used for an opening between two zones that are at the
       \\memo   same elevation but have different air temperatures.  In this case, the mixing air flow
       \\memo   between the two zones is determined by the density difference between the two zones.
       \\memo   This would typically be used between two zones in a refrigerated warehouse that are
       \\memo   controlled at different temperatures.  It could also be used to model a door to a walk-in
       \\memo   refrigerated space if that space were modeled as a zone instead of using the object Refrigeration:WalkIn.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Zone 1 Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Zone 2 Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A4 , \\field Schedule Name
       \\note This schedule defines the fraction of the time the refrigeration door is open
       \\note  For example, if the warehouse is closed at night and there are no door openings
       \\note   between two zones, the value for that time period would be 0.
       \\note   If doors were propped open, the value  over that time period would be 1.0
       \\note   If the doors were open about 20% of the time, the value over that period would be 0.2
       \\note   Schedule values must lie between 0 and 1.0
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Door Height
       \\units m
       \\type real
       \\minimum 0
       \\maximum 50.
       \\default 3.0
  N2 , \\field Door Area
       \\type real
       \\minimum 0
       \\maximum 400.
       \\default 9.
       \\units m2
  A5;  \\field Door Protection Type
       \\note Door protection can reduce the air flow through a refrigeration door
       \\note The default value is "None"
       \\note Choices: "None", "AirCurtain", and "StripCurtain"
       \\note A strip curtain reduces the air flow more than an air curtain
       \\type choice
       \\key None
       \\key AirCurtain
       \\key StripCurtain
       \\default None

ZoneEarthtube,
  \\min-fields 22
      \\memo  Earth Tube is specified as a design level which is modified by a Schedule fraction, temperature difference and wind speed:
      \\memo  Earthtube=Edesign * Fschedule * (A + B*|(Tzone-Todb)| + C*WindSpd + D * WindSpd**2)
  A1, \\field Zone Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Schedule Name
      \\required-field
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Design Flow Rate
      \\required-field
      \\units m3/s
      \\note  "Edesign" in Equation
      \\type real
      \\minimum 0
  N2, \\field Minimum Zone Temperature when Cooling
      \\required-field
      \\note this is the indoor temperature below which the earth tube is shut off
      \\units C
      \\type real
      \\minimum -100
      \\maximum 100
  N3, \\field Maximum Zone Temperature when Heating
      \\required-field
      \\note this is the indoor temperature above which the earth tube is shut off
      \\units C
      \\type real
      \\minimum -100
      \\maximum 100
  N4, \\field Delta Temperature
      \\required-field
      \\note This is the temperature difference between indoor and outdoor below which the earth tube is shut off
      \\units deltaC
      \\type real
      \\minimum 0
  A3, \\field Earthtube Type
      \\required-field
      \\type choice
      \\key Natural
      \\key Intake
      \\key Exhaust
      \\default Natural
  N5, \\field Fan Pressure Rise
      \\required-field
      \\note pressure rise across the fan
      \\type real
      \\units Pa
      \\minimum 0
      \\default 0
  N6, \\field Fan Total Efficiency
      \\required-field
      \\type real
      \\minimum> 0
      \\default 1
  N7, \\field Pipe Radius
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\default 1
  N8, \\field Pipe Thickness
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\default 0.2
  N9, \\field Pipe Length
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\default 15
 N10, \\field Pipe Thermal Conductivity
      \\required-field
      \\units W/m-K
      \\type real
      \\minimum> 0
      \\default 200
 N11, \\field Pipe Depth Under Ground Surface
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\default 3
  A4, \\field Soil Condition
      \\required-field
      \\type choice
      \\key HeavyAndSaturated
      \\key HeavyAndDamp
      \\key HeavyAndDry
      \\key LightAndDry
      \\default HeavyAndDamp
 N12, \\field Average Soil Surface Temperature
      \\required-field
      \\units C
      \\type real
      \\default 0
 N13, \\field Amplitude of Soil Surface Temperature
      \\required-field
      \\units C
      \\type real
      \\minimum 0
      \\default 0
 N14, \\field Phase Constant of Soil Surface Temperature
      \\required-field
      \\units days
      \\type real
      \\minimum 0
      \\default 0
 N15, \\field Constant Term Flow Coefficient
      \\required-field
      \\note  "A" in Equation
      \\type real
      \\default 1
 N16, \\field Temperature Term Flow Coefficient
      \\required-field
      \\note  "B" in Equation
      \\type real
      \\default 0
 N17, \\field Velocity Term Flow Coefficient
      \\required-field
      \\note  "C" in Equation
      \\type real
      \\default 0
 N18; \\field Velocity Squared Term Flow Coefficient
      \\required-field
      \\note  "D" in Equation
      \\type real
      \\default 0

ZoneCoolTower:Shower,
     \\memo A cooltower (sometimes referred to as a wind tower or a shower cooling tower)
     \\memo models passive downdraught evaporative cooling (PDEC) that is designed to capture the
     \\memo wind at the top of a tower and cool the outdoor air using water evaporation before
     \\memo delivering it to a space.
 A1, \\field Name
     \\required-field
 A2, \\field Availability Schedule Name
     \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
     \\note If this field is blank, the system is always available.
     \\type object-list
     \\object-list ScheduleNames
 A3, \\field Zone Name
     \\required-field
     \\type object-list
     \\object-list ZoneNames
 A4, \\field Water Supply Storage Tank Name
     \\note In case of stand alone tank or underground water, leave this input blank
     \\type object-list
     \\object-list WaterStorageTankNames
 A5, \\field Flow Control Type
     \\note Water flow schedule should be selected when the water flow rate is known.
     \\note Wind-driven flow should be selected when the water flow rate is unknown.
     \\type choice
     \\key WaterFlowSchedule
     \\key WindDrivenFlow
     \\default WindDrivenFlow
 A6, \\field Pump Flow Rate Schedule Name
     \\required-field
     \\type object-list
     \\object-list ScheduleNames
 N1, \\field Maximum Water Flow Rate
     \\required-field
     \\units m3/s
     \\type real
 N2, \\field Effective Tower Height
     \\note This field is from either the spray or the wet pad to the top of the outlet.
     \\required-field
     \\units m
     \\type real
 N3, \\field Airflow Outlet Area
     \\note User have to specify effective area when outlet area is relatively bigger than the cross sectional area
     \\note of cooltower. If the number of outlet is more than one, assume the air passes through only one.
     \\required-field
     \\units m2
     \\type real
 N4, \\field Maximum Air Flow Rate
     \\required-field
     \\units m3/s
     \\type real
     \\minimum 0
 N5, \\field Minimum Indoor Temperature
     \\required-field
     \\note This field is to specify the indoor temperature below which cooltower is shutoff.
     \\units C
     \\type real
     \\minimum -100
     \\maximum 100
 N6, \\field Fraction of Water Loss
     \\minimum 0
     \\maximum 1
 N7, \\field Fraction of Flow Schedule
     \\minimum 0
     \\maximum 1
 N8; \\field Rated Power Consumption
     \\required-field
     \\units W
     \\type real

ZoneThermalChimney,
       \\memo A thermal chimney is a vertical shaft utilizing solar radiation to enhance natural
       \\memo ventilation. It consists of an absorber wall, air gap and glass cover with high solar
       \\memo transmissivity.
   \\min-fields 10
   A1, \\field Name
        \\required-field
   A2,  \\field Zone Name
        \\note Name of zone that is the thermal chimney
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A3, \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   N1, \\field Width of the Absorber Wall
        \\required-field
        \\units m
        \\type real
        \\minimum 0
   N2, \\field Cross Sectional Area of Air Channel Outlet
        \\required-field
        \\units m2
        \\type real
        \\minimum 0
   N3, \\field Discharge Coefficient
        \\required-field
        \\type real
        \\minimum 0
        \\maximum 1
        \\default 0.8
   A4, \\field Zone 1 Name
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   N4, \\field Distance from Top of Thermal Chimney to Inlet 1
        \\required-field
        \\units m
        \\type real
        \\minimum 0
   N5, \\field Relative Ratios of Air Flow Rates Passing through Zone 1
        \\required-field
        \\type real
        \\minimum 0
        \\maximum 1
        \\default 1.0
   N6, \\field Cross Sectional Areas of Air Channel Inlet 1
        \\required-field
        \\units m2
        \\type real
        \\minimum 0
   A5, \\field Zone 2 Name
        \\type object-list
        \\object-list ZoneNames
   N7, \\field Distance from Top of Thermal Chimney to Inlet 2
        \\units m
        \\type real
        \\minimum 0
   N8, \\field Relative Ratios of Air Flow Rates Passing through Zone 2
        \\type real
        \\minimum 0
        \\maximum 1
   N9, \\field Cross Sectional Areas of Air Channel Inlet 2
        \\units m2
        \\type real
        \\minimum 0
   A6, \\field Zone 3 Name
        \\type object-list
        \\object-list ZoneNames
   N10, \\field Distance from Top of Thermal Chimney to Inlet 3
        \\units m
        \\type real
        \\minimum 0
   N11, \\field Relative Ratios of Air Flow Rates Passing through Zone 3
        \\type real
        \\minimum 0
        \\maximum 1
   N12, \\field Cross Sectional Areas of Air Channel Inlet 3
        \\units m2
        \\type real
        \\minimum 0
   A7, \\field Zone 4 Name
        \\type object-list
        \\object-list ZoneNames
   N13, \\field Distance from Top of Thermal Chimney to Inlet 4
        \\units m
        \\type real
        \\minimum 0
   N14, \\field Relative Ratios of Air Flow Rates Passing through Zone 4
        \\type real
        \\minimum 0
        \\maximum 1
   N15, \\field Cross Sectional Areas of Air Channel Inlet 4
        \\units m2
        \\type real
        \\minimum 0
   A8, \\field Zone 5 Name
        \\type object-list
        \\object-list ZoneNames
   N16, \\field Distance from Top of Thermal Chimney to Inlet 5
        \\units m
        \\type real
        \\minimum 0
   N17, \\field Relative Ratios of Air Flow Rates Passing through Zone 5
        \\type real
        \\minimum 0
        \\maximum 1
   N18, \\field Cross Sectional Areas of Air Channel Inlet 5
        \\units m2
        \\type real
        \\minimum 0
   A9, \\field Zone 6 Name
        \\type object-list
        \\object-list ZoneNames
   N19, \\field Distance from Top of Thermal Chimney to Inlet 6
        \\units m
        \\type real
        \\minimum 0
   N20, \\field Relative Ratios of Air Flow Rates Passing through Zone 6
        \\type real
        \\minimum 0
        \\maximum 1
   N21, \\field Cross Sectional Areas of Air Channel Inlet 6
        \\units m2
        \\type real
        \\minimum 0
   A10, \\field Zone 7 Name
        \\type object-list
        \\object-list ZoneNames
   N22, \\field Distance from Top of Thermal Chimney to Inlet 7
        \\units m
        \\type real
        \\minimum 0
   N23, \\field Relative Ratios of Air Flow Rates Passing through Zone 7
        \\type real
        \\minimum 0
        \\maximum 1
   N24, \\field Cross Sectional Areas of Air Channel Inlet 7
        \\units m2
        \\type real
        \\minimum 0
   A11, \\field Zone 8 Name
        \\type object-list
        \\object-list ZoneNames
   N25, \\field Distance from Top of Thermal Chimney to Inlet 8
        \\units m
        \\type real
        \\minimum 0
   N26, \\field Relative Ratios of Air Flow Rates Passing through Zone 8
        \\type real
        \\minimum 0
        \\maximum 1
   N27, \\field Cross Sectional Areas of Air Channel Inlet 8
        \\units m2
        \\type real
        \\minimum 0
   A12, \\field Zone 9 Name
        \\type object-list
        \\object-list ZoneNames
   N28, \\field Distance from Top of Thermal Chimney to Inlet 9
        \\units m
        \\type real
        \\minimum 0
   N29, \\field Relative Ratios of Air Flow Rates Passing through Zone 9
        \\type real
        \\minimum 0
        \\maximum 1
   N30, \\field Cross Sectional Areas of Air Channel Inlet 9
        \\units m2
        \\type real
        \\minimum 0
   A13, \\field Zone 10 Name
        \\type object-list
        \\object-list ZoneNames
   N31, \\field Distance from Top of Thermal Chimney to Inlet 10
        \\units m
        \\type real
        \\minimum 0
   N32, \\field Relative Ratios of Air Flow Rates Passing through Zone 10
        \\type real
        \\minimum 0
        \\maximum 1
   N33, \\field Cross Sectional Areas of Air Channel Inlet 10
        \\units m2
        \\type real
        \\minimum 0
   A14, \\field Zone 11 Name
        \\type object-list
        \\object-list ZoneNames
   N34, \\field Distance from Top of Thermal Chimney to Inlet 11
        \\units m
        \\type real
        \\minimum 0
   N35, \\field Relative Ratios of Air Flow Rates Passing through Zone 11
        \\type real
        \\minimum 0
        \\maximum 1
   N36, \\field Cross Sectional Areas of Air Channel Inlet 11
        \\units m2
        \\type real
        \\minimum 0
   A15, \\field Zone 12 Name
        \\type object-list
        \\object-list ZoneNames
   N37, \\field Distance from Top of Thermal Chimney to Inlet 12
        \\units m
        \\type real
        \\minimum 0
   N38, \\field Relative Ratios of Air Flow Rates Passing through Zone 12
        \\type real
        \\minimum 0
        \\maximum 1
   N39, \\field Cross Sectional Areas of Air Channel Inlet 12
        \\units m2
        \\type real
        \\minimum 0
   A16, \\field Zone 13 Name
        \\type object-list
        \\object-list ZoneNames
   N40, \\field Distance from Top of Thermal Chimney to Inlet 13
        \\units m
        \\type real
        \\minimum 0
   N41, \\field Relative Ratios of Air Flow Rates Passing through Zone 13
        \\type real
        \\minimum 0
        \\maximum 1
   N42, \\field Cross Sectional Areas of Air Channel Inlet 13
        \\units m2
        \\type real
        \\minimum 0
   A17, \\field Zone 14 Name
        \\type object-list
        \\object-list ZoneNames
   N43, \\field Distance from Top of Thermal Chimney to Inlet 14
        \\units m
        \\type real
        \\minimum 0
   N44, \\field Relative Ratios of Air Flow Rates Passing through Zone 14
        \\type real
        \\minimum 0
        \\maximum 1
   N45, \\field Cross Sectional Areas of Air Channel Inlet 14
        \\units m2
        \\type real
        \\minimum 0
   A18, \\field Zone 15 Name
        \\type object-list
        \\object-list ZoneNames
   N46, \\field Distance from Top of Thermal Chimney to Inlet 15
        \\units m
        \\type real
        \\minimum 0
   N47, \\field Relative Ratios of Air Flow Rates Passing through Zone 15
        \\type real
        \\minimum 0
        \\maximum 1
   N48, \\field Cross Sectional Areas of Air Channel Inlet 15
        \\units m2
        \\type real
        \\minimum 0
   A19, \\field Zone 16 Name
        \\type object-list
        \\object-list ZoneNames
   N49, \\field Distance from Top of Thermal Chimney to Inlet 16
        \\units m
        \\type real
        \\minimum 0
   N50, \\field Relative Ratios of Air Flow Rates Passing through Zone 16
        \\type real
        \\minimum 0
        \\maximum 1
   N51, \\field Cross Sectional Areas of Air Channel Inlet 16
        \\units m2
        \\type real
        \\minimum 0
   A20, \\field Zone 17 Name
        \\type object-list
        \\object-list ZoneNames
   N52, \\field Distance from Top of Thermal Chimney to Inlet 17
        \\units m
        \\type real
        \\minimum 0
   N53, \\field Relative Ratios of Air Flow Rates Passing through Zone 17
        \\type real
        \\minimum 0
        \\maximum 1
   N54, \\field Cross Sectional Areas of Air Channel Inlet 17
        \\units m2
        \\type real
        \\minimum 0
   A21, \\field Zone 18 Name
        \\type object-list
        \\object-list ZoneNames
   N55, \\field Distance from Top of Thermal Chimney to Inlet 18
        \\units m
        \\type real
        \\minimum 0
   N56, \\field Relative Ratios of Air Flow Rates Passing through Zone 18
        \\type real
        \\minimum 0
        \\maximum 1
   N57, \\field Cross Sectional Areas of Air Channel Inlet 18
        \\units m2
        \\type real
        \\minimum 0
   A22, \\field Zone 19 Name
        \\type object-list
        \\object-list ZoneNames
   N58, \\field Distance from Top of Thermal Chimney to Inlet 19
        \\units m
        \\type real
        \\minimum 0
   N59, \\field Relative Ratios of Air Flow Rates Passing through Zone 19
        \\type real
        \\minimum 0
        \\maximum 1
   N60, \\field Cross Sectional Areas of Air Channel Inlet 19
        \\units m2
        \\type real
        \\minimum 0
   A23, \\field Zone 20 Name
        \\type object-list
        \\object-list ZoneNames
   N61, \\field Distance from Top of Thermal Chimney to Inlet 20
        \\units m
        \\type real
        \\minimum 0
   N62, \\field Relative Ratios of Air Flow Rates Passing through Zone 20
        \\type real
        \\minimum 0
        \\maximum 1
   N63; \\field Cross Sectional Areas of Air Channel Inlet 20
        \\units m2
        \\type real
        \\minimum 0

\\group Natural Ventilation and Duct Leakage

! Basic parameters
AirflowNetwork:SimulationControl,
      \\min-fields 13
      \\unique-object
      \\memo This object defines the global parameters used in an Airflow Network simulation.
 A1 , \\field Name
      \\required-field
      \\note Enter a unique name for this object.
 A2 , \\field AirflowNetwork Control
      \\type choice
      \\key MultizoneWithDistribution
      \\key MultizoneWithoutDistribution
      \\key MultizoneWithDistributionOnlyDuringFanOperation
      \\key NoMultizoneOrDistribution
      \\default NoMultizoneOrDistribution
      \\note NoMultizoneOrDistribution: Only perform Simple calculations (objects ZoneInfiltration:*,
      \\note ZoneVentilation:*, ZoneMixing, ZoneCrossMixing, ZoneRefrigerationDoorMixing,
      \\note ZoneAirBalance:OutdoorAir, ZoneEarthtube, ZoneThermalChimney, and ZoneCoolTower:Shower);
      \\note MultizoneWithoutDistribution: Use AirflowNetwork objects to simulate multizone
      \\note Airflows driven by wind during simulation time,
      \\note and objects of ZoneInfiltration:*, ZoneVentilation:*, ZoneMixing, ZoneCrossMixing
      \\note ZoneRefrigerationDoorMixing, ZoneAirBalance:OutdoorAir, ZoneEarthtube,
      \\note ZoneThermalChimney, and ZoneCoolTower:Shower are ignored;
      \\note MultizoneWithDistributionOnlyDuringFanOperation: Perform distribution system
      \\note calculations during system fan on time
      \\note and Simple calculations during system Fan off time;
      \\note MultizoneWithDistribution: Perform distribution system calculations during system
      \\note fan on time and multizone Airflow driven by wind during system fan off time.
 A3 , \\field Wind Pressure Coefficient Type
      \\type choice
      \\key Input
      \\key SurfaceAverageCalculation
      \\default SurfaceAverageCalculation
      \\note Input: User must enter AirflowNetwork:MultiZone:WindPressureCoefficientArray,
      \\note AirflowNetwork:MultiZone:ExternalNode, and
      \\note AirflowNetwork:MultiZone:WindPressureCoefficientValues objects.
      \\note SurfaceAverageCalculation: used only for rectangular buildings.
      \\note If SurfaceAverageCalculation is selected,
      \\note AirflowNetwork:MultiZone:WindPressureCoefficientArray, AirflowNetwork:MultiZone:ExternalNode,
      \\note and AirflowNetwork:MultiZone:WindPressureCoefficientValues objects are not used.
 A4 , \\field AirflowNetwork Wind Pressure Coefficient Array Name
      \\type object-list
      \\object-list WPCSetNames
      \\note Used only if Wind Pressure Coefficient Type = Input, otherwise this field may be left blank.
 A5 , \\field Height Selection for Local Wind Pressure Calculation
      \\type choice
      \\key ExternalNode
      \\key OpeningHeight
      \\default OpeningHeight
      \\note If ExternalNode is selected, the height given in the
      \\note AirflowNetwork:MultiZone:ExternalNode object will be used.
      \\note If OpeningHeight is selected, the surface opening height (centroid) will be used to
      \\note calculate local wind pressure
      \\note This field is ignored when the choice of the Wind Pressure Coefficient Type field is
      \\note SurfaceAverageCalculation.
 A6 , \\field Building Type
      \\note Used only if Wind Pressure Coefficient Type = SurfaceAverageCalculation,
      \\note otherwise this field may be left blank.
      \\type choice
      \\key LowRise
      \\key HighRise
      \\default LowRise
 N1 , \\field Maximum Number of Iterations
      \\type integer
      \\units dimensionless
      \\default 500
      \\minimum> 10
      \\maximum 30000
      \\note Determines the maximum number of iterations used to converge on a solution. If this limit
      \\note is exceeded, the program terminates.
 A7 , \\field Initialization Type
      \\type choice
      \\key LinearInitializationMethod
      \\key ZeroNodePressures
      \\default ZeroNodePressures
 N2 , \\field Relative Airflow Convergence Tolerance
      \\type real
      \\units dimensionless
      \\default 1.E-4
      \\minimum> 0
      \\note This tolerance is defined as the absolute value of the sum of the mass Flow Rates
      \\note divided by the sum of the absolute value of the mass Flow Rates. The mass Flow Rates
      \\note described here refer to the mass Flow Rates at all Nodes in the AirflowNetwork model.
      \\note The solution converges when both this tolerance and the tolerance in the next field
      \\note (Absolute Airflow Convergence Tolerance) are satisfied.
 N3 , \\field Absolute Airflow Convergence Tolerance
      \\type real
      \\units kg/s
      \\default 1.E-6
      \\minimum> 0
      \\note This tolerance is defined as the absolute value of the sum of the mass flow rates. The mass
      \\note flow rates described here refer to the mass flow rates at all nodes in the AirflowNetwork
      \\note model. The solution converges when both this tolerance and the tolerance in the previous
      \\note field (Relative Airflow Convergence Tolerance) are satisfied.
 N4 , \\field Convergence Acceleration Limit
      \\type real
      \\units dimensionless
      \\note Used only for AirflowNetwork:SimulationControl
      \\minimum -1
      \\maximum 1
      \\default -0.5
 N5 , \\field Azimuth Angle of Long Axis of Building
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 180.0
      \\default 0.0
      \\note Degrees clockwise from true North.
      \\note Used only if Wind Pressure Coefficient Type = SurfaceAverageCalculation.
 N6 ; \\field Ratio of Building Width Along Short Axis to Width Along Long Axis
      \\type real
      \\minimum> 0.0
      \\maximum 1.0
      \\default 1.0
      \\note Used only if Wind Pressure Coefficient Type = SurfaceAverageCalculation.

AirflowNetwork:MultiZone:Zone,
      \\min-fields 8
      \\memo This object is used to simultaneously control a thermal zone's window and door openings,
      \\memo both exterior and interior.
  A1, \\field Zone Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
      \\note Enter the zone name where ventilation control is required.
  A2, \\field Ventilation Control Mode
      \\type choice
      \\key Temperature
      \\key Enthalpy
      \\key Constant
      \\key ASHRAE55Adaptive
      \\key CEN15251Adaptive
      \\key NoVent
      \\default NoVent
      \\note When Ventilation Control Mode = Temperature or Enthalpy, the following
      \\note fields are used to modulate the Ventilation Open Factor for all
      \\note window and door openings in the zone according to the zone's
      \\note indoor-outdoor temperature or enthalpy difference.
      \\note Constant: controlled by field Venting Schedule Name.
      \\note NoVent: control will not open window or door during simulation (Ventilation Open Factor = 0).
  A3, \\field Ventilation Control Zone Temperature Setpoint Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Used only if Ventilation Control Mode = Temperature or Enthalpy.
  N1, \\field Minimum Venting Open Factor
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
      \\note Used only if Ventilation Control Mode = Temperature or Enthalpy.
  N2, \\field Indoor and Outdoor Temperature Difference Lower Limit For Maximum Venting Open Factor
      \\type real
      \\units deltaC
      \\minimum 0.0
      \\maximum< 100.0
      \\default 0.0
      \\note Applicable only if Ventilation Control Mode = Temperature.
      \\note This value must be less than the corresponding upper value (next field).
  N3, \\field Indoor and Outdoor Temperature Difference Upper Limit for Minimun Venting Open Factor
      \\type real
      \\units deltaC
      \\minimum> 0.0
      \\default 100.0
      \\note Applicable only if Ventilation Control Mode = Temperature.
      \\note This value must be greater than the corresponding lower value (previous field).
  N4, \\field Indoor and Outdoor Enthalpy Difference Lower Limit For Maximum Venting Open Factor
      \\type real
      \\units deltaJ/kg
      \\minimum 0.0
      \\maximum< 300000.0
      \\default 0.0
      \\note Applicable only if Ventilation Control Mode = Enthalpy.
      \\note This value must be less than the corresponding upper value (next field).
  N5, \\field Indoor and Outdoor Enthalpy Difference Upper Limit for Minimun Venting Open Factor
      \\type real
      \\units deltaJ/kg
      \\minimum> 0.0
      \\default 300000.0
      \\note Applicable only if Ventilation Control Mode = Enthalpy.
      \\note This value must be greater than the corresponding lower value (previous field).
  A4; \\field Venting Availability Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Non-zero Schedule value means venting is allowed if other venting control conditions are
      \\note satisfied. A zero (or negative) Schedule value means venting is not allowed under any
      \\note The Schedule values should be greater than or equal to 0 and less than or equal to 1.
      \\note circumstances. If this Schedule is not specified then venting is allowed if
      \\note other venting control conditions are satisfied.
      \\note Not used if Ventilation Control Mode = NoVent.

AirflowNetwork:MultiZone:Surface,
      \\min-fields 4
      \\memo This object specifies the properties of a surface linkage through which air flows.
      \\memo Airflow Report: Node 1 as an inside face zone;
      \\memo Node 2 as an outside face zone or external node.
  A1, \\field Surface Name
      \\required-field
      \\type object-list
      \\object-list SurfAndSubSurfNames
      \\note Enter the name of a heat transfer surface.
  A2, \\field Leakage Component Name
      \\required-field
      \\type object-list
      \\object-list SurfaceAirflowLeakageNames
      \\note Enter the name of an Airflow Network leakage component. A leakage component is
      \\note one of the following AirflowNetwork:Multizone objects:
      \\note AirflowNetwork:MultiZone:Component:DetailedOpening,
      \\note AirflowNetwork:MultiZone:Component:SimpleOpening,
      \\note AirflowNetwork:MultiZone:Surface:Crack,
      \\note AirflowNetwork:MultiZone:Surface:EffectiveLeakageArea,
      \\note AirflowNetwork:MultiZone:Component:HorizontalOpening, or
      \\note AirflowNetwork:MultiZone:Component:ZoneExhaustFan.
      \\note When the zone exhaust fan name is entered, any surface control fields below A3 are
      \\note ignored when the zone exhaust fan turns on.
  A3, \\field External Node Name
      \\type object-list
      \\object-list ExternalNodeNames
      \\note Used if Wind Pressure Coefficient Type = Input in the AirflowNetwork:SimulationControl object,
      \\note otherwise this field may be left blank.
  N1, \\field Window/Door Opening Factor, or Crack Factor
      \\required-field
      \\type real
      \\units dimensionless
      \\minimum> 0.0
      \\maximum 1.0
      \\default 1.0
      \\note This field specifies a multiplier for a crack, window, or door.
  A4, \\field Ventilation Control Mode
      \\type choice
      \\key Temperature
      \\key Enthalpy
      \\key Constant
      \\key ASHRAE55Adaptive
      \\key CEN15251Adaptive
      \\key NoVent
      \\key ZoneLevel
      \\key AdjacentTemperature
      \\key AdjacentEnthalpy
      \\default ZoneLevel
      \\note When Ventilation Control Mode = Temperature or Enthalpy, the following
      \\note fields are used to modulate the Ventilation Open Factor for a
      \\note window or door opening according to the parent zone's
      \\note indoor-outdoor temperature or enthalpy difference.
      \\note When Ventilation Control Mode = AdjacentTemperature or AdjacentEnthalpy, the following
      \\note fields are used to modulate the Ventilation Open Factor for an interior
      \\note window or door opening according to temperature or enthalpy difference
      \\note between the parent zone and the adjacent zone.
      \\note Constant: controlled by field Venting Schedule Name.
      \\note NoVent: control will not open window or door during simulation (Ventilation Open Factor = 0).
      \\note ZoneLevel: control will be controlled by AirflowNetwork:MultiZone:Zone
      \\note Mode.
  A5, \\field Ventilation Control Zone Temperature Setpoint Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Used only if Ventilation Control Mode = Temperature or Enthalpy.
  N2, \\field Minimum Venting Open Factor
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
      \\note Used only if Ventilation Control Mode = Temperature or Enthalpy.
  N3, \\field Indoor and Outdoor Temperature Difference Lower Limit For Maximum Venting Open Factor
      \\note Applicable only if Ventilation Control Mode = Temperature
      \\type real
      \\units deltaC
      \\minimum 0.0
      \\maximum< 100
      \\default 0.0
  N4, \\field Indoor and Outdoor Temperature Difference Upper Limit for Minimun Venting Open Factor
      \\type real
      \\units deltaC
      \\minimum> 0.0
      \\default 100.0
      \\note Applicable only if Ventilation Control Mode = Temperature.
      \\note This value must be greater than the corresponding lower value (previous field).
  N5, \\field Indoor and Outdoor Enthalpy Difference Lower Limit For Maximum Venting Open Factor
      \\type real
      \\units deltaJ/kg
      \\minimum 0.0
      \\maximum< 300000.0
      \\default 0.0
      \\note Applicable only if Ventilation Control Mode = Enthalpy.
      \\note This value must be less than the corresponding upper value (next field).
  N6, \\field Indoor and Outdoor Enthalpy Difference Upper Limit for Minimun Venting Open Factor
      \\type real
      \\units deltaJ/kg
      \\minimum> 0.0
      \\default 300000.0
      \\note Applicable only if Ventilation Control Mode = Enthalpy.
      \\note This value must be greater than the corresponding lower value (previous field).
  A6; \\field Venting Availability Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Non-zero schedule value means venting is allowed if other venting control conditions are
      \\note satisfied. A zero (or negative) schedule value means venting is not allowed under any
      \\note circumstances. The schedule values should be greater than or equal to 0 and less than or
      \\note equal to 1. If this schedule is not specified then venting is allowed if
      \\note other venting control conditions are satisfied.
      \\note Not used if Ventilation Control Mode = NoVent or ZoneLevel.

AirflowNetwork:MultiZone:ReferenceCrackConditions,
      \\min-fields 4
      \\memo This object specifies the conditions under which the air mass flow coefficient was measured.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference ReferenceCrackConditions
      \\note Enter a unique name for this object.
 N1 , \\field Reference Temperature
      \\type real
      \\units C
      \\default 20
      \\note Enter the reference temperature under which the surface crack data were obtained.
 N2 , \\field Reference Barometric Pressure
      \\type real
      \\units Pa
      \\default 101325
      \\minimum 31000
      \\maximum 120000
      \\ip-units inHg
      \\note Enter the reference barometric pressure under which the surface crack data were obtained.
 N3 ; \\field Reference Humidity Ratio
      \\type real
      \\units kgWater/kgDryAir
      \\default 0
      \\note Enter the reference humidity ratio under which the surface crack data were obtained.

AirflowNetwork:MultiZone:Surface:Crack,
      \\min-fields 3
      \\memo This object specifies the properties of airflow through a crack.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference SurfaceAirflowLeakageNames
      \\note Enter a unique name for this object.
 N1 , \\field Air Mass Flow Coefficient at Reference Conditions
      \\type real
      \\required-field
      \\units kg/s
      \\minimum> 0
      \\note Enter the air mass flow coefficient at the conditions defined
      \\note in the Reference Crack Conditions object.
      \\note Defined at 1 Pa pressure difference across this crack.
 N2 , \\field Air Mass Flow Exponent
      \\type real
      \\units dimensionless
      \\minimum 0.5
      \\maximum 1.0
      \\default 0.65
      \\note Enter the air mass flow exponent for the surface crack.
 A2 ; \\field Reference Crack Conditions
      \\type object-list
      \\object-list ReferenceCrackConditions
      \\note Select a AirflowNetwork:MultiZone:ReferenceCrackConditions name associated with
      \\note the air mass flow coefficient entered above.

AirflowNetwork:MultiZone:Surface:EffectiveLeakageArea,
      \\min-fields 5
      \\memo This object is used to define surface air leakage.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference SurfaceAirflowLeakageNames
      \\note Enter a unique name for this object.
 N1 , \\field Effective Leakage Area
      \\required-field
      \\type real
      \\units m2
      \\minimum> 0
      \\note Enter the effective leakage area.
 N2 , \\field Discharge Coefficient
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\default 1.0
      \\note Enter the coefficient used in the air mass flow equation.
 N3 , \\field Reference Pressure Difference
      \\type real
      \\units Pa
      \\minimum> 0
      \\default 4.0
      \\note Enter the pressure difference used to define the air mass flow coefficient and exponent.
 N4 ; \\field Air Mass Flow Exponent
      \\units dimensionless
      \\type real
      \\default .65
      \\minimum 0.5
      \\maximum 1.0
      \\note Enter the exponent used in the air mass flow equation.

AirflowNetwork:MultiZone:Component:DetailedOpening,
      \\min-fields 16
      \\memo This object specifies the properties of airflow through windows and doors (window, door and
      \\memo glass door heat transfer subsurfaces) when they are closed or open.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference SurfaceAirflowLeakageNames
      \\note Enter a unique name for this object.
 N1 , \\field Air Mass Flow Coefficient When Opening is Closed
      \\required-field
      \\type real
      \\units kg/s-m
      \\minimum> 0
      \\note Defined at 1 Pa per meter of crack length. Enter the coefficient used in the following
      \\note equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening (window or door) is closed.
 N2 , \\field Air Mass Flow Exponent When Opening is Closed
      \\units dimensionless
      \\type real
      \\minimum 0.5
      \\maximum 1.0
      \\default 0.65
      \\note Enter the exponent used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening (window or door) is closed.
 A2 , \\field Type of Rectanguler Large Vertical Opening (LVO)
      \\type choice
      \\key NonPivoted
      \\key HorizontallyPivoted
      \\note Select the type of vertical opening: Non-pivoted opening or Horizontally pivoted opening.
      \\default NonPivoted
 N3 , \\field Extra Crack Length or Height of Pivoting Axis
      \\type real
      \\units m
      \\minimum 0
      \\default 0
      \\note Extra crack length is used for LVO Non-pivoted type with multiple openable parts.
      \\note Height of pivoting axis is used for LVO Horizontally pivoted type.
      \\note Specifies window or door characteristics that depend on the LVO type.
      \\note For Non-pivoted Type (rectangular windows and doors), this field is the extra crack length
      \\note in meters due to multiple openable parts, if present.  Extra here means in addition
      \\note to the length of the cracks on the top, bottom and sides of the window/door.
      \\note For Horizontally pivoted Type, this field gives the height of the
      \\note pivoting axis measured from the bottom of the glazed part of the window (m).
 N4 , \\field Number of Sets of Opening Factor Data
      \\required-field
      \\type integer
      \\minimum 2
      \\maximum 4
      \\note Enter the number of the following sets of data for opening factor,
      \\note discharge coefficient, width factor, height factor, and start height factor.
 N5 , \\field Opening Factor 1
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 0
      \\default 0
      \\note This value must be specified as 0.
 N6 , \\field Discharge Coefficient for Opening Factor 1
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\maximum 1
      \\default 0.001
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through a window or door at that Opening Factor.
 N7 , \\field Width Factor for Opening Factor 1
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Width Factor is the opening width divided by the window or door width.
 N8 , \\field Height Factor for Opening Factor 1
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Height Factor is the opening height divided by the window or door height.
 N9 , \\field Start Height Factor for Opening Factor 1
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Start Height Factor is the Start Height divided by the window or door height.
      \\note Start Height is the distance between the bottom of the window or door and the
      \\note bottom of the window or door opening. The sum of the Height Factor and the Start Height
      \\note Factor must be less than 1.0 in order to have the opening within the window or door
      \\note dimensions.
 N10, \\field Opening Factor 2
      \\required-field
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\maximum 1
      \\note If Number of Sets of Opening Factor Data = 2, this value must be 1.0.
      \\note If Number of Sets of Opening Factor Data = 3, this value must be less than 1.0.
      \\note If Number of Sets of Opening Factor Data = 4, this value must be less than the
      \\note value entered for Opening factor 3 and greater than the value entered
      \\note for Opening factor 1.
 N11, \\field Discharge Coefficient for Opening Factor 2
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\maximum 1
      \\default 1
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through a window or door at that Opening Factor.
 N12, \\field Width Factor for Opening Factor 2
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\maximum 1
      \\default 1
      \\note The Width Factor is the opening width divided by the window or door width.
 N13, \\field Height Factor for Opening Factor 2
      \\type real
      \\units dimensionless
      \\minimum> 0
      \\maximum 1
      \\default 1
      \\note The Height Factor is the opening height divided by the window or door height.
 N14, \\field Start Height Factor for Opening Factor 2
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum< 1
      \\default 0
      \\note The Start Height Factor is the Start Height divided by the window or door height.
      \\note Start Height is the distance between the bottom of the window or door and the
      \\note bottom of the window or door opening. The sum of the Height Factor and the Start Height
      \\note Factor must be less than 1.0 in order to have the opening within the window or door
      \\note dimensions.
 N15, \\field Opening Factor 3
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\note If Number of Sets of Opening Factor Data = 3, this value must be 1.0.
      \\note If Number of Sets of Opening Factor Data = 4, this value must be less than 1.0,
      \\note and greater than value entered for Opening factor 2.
 N16, \\field Discharge Coefficient for Opening Factor 3
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through a window or door at that Opening Factor.
 N17, \\field Width Factor for Opening Factor 3
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Width Factor is the opening width divided by the window or door width.
 N18, \\field Height Factor for Opening Factor 3
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Height Factor is the opening height divided by the window or door height.
 N19, \\field Start Height Factor for Opening Factor 3
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Start Height Factor is the Start Height divided by the window or door height.
      \\note Start Height is the distance between the bottom of the window or door and the
      \\note bottom of the window or door opening. The sum of the Height Factor and the Start Height
      \\note Factor must be less than 1.0 in order to have the opening within the window or door
      \\note dimensions.
 N20, \\field Opening Factor 4
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\note If Number of Sets of Opening Factor Data = 4, this value must be 1.0
 N21, \\field Discharge Coefficient for Opening Factor 4
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through a window or door at that Opening Factor.
 N22, \\field Width Factor for Opening Factor 4
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Width Factor is the opening width divided by the window or door width.
 N23, \\field Height Factor for Opening Factor 4
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Height Factor is the opening height divided by the window or door height.
 N24; \\field Start Height Factor for Opening Factor 4
      \\type real
      \\units dimensionless
      \\minimum 0
      \\maximum 1
      \\default 0
      \\note The Start Height Factor is the Start Height divided by the window or door height.
      \\note Start Height is the distance between the bottom of the window or door and the
      \\note bottom of the window or door opening. The sum of the Height Factor and the Start Height
      \\note Factor must be less than 1.0 in order to have the opening within the window or door
      \\note dimensions.

AirflowNetwork:MultiZone:Component:SimpleOpening,
      \\min-fields 5
      \\memo This object specifies the properties of air flow through windows and doors (window, door and
      \\memo glass door heat transfer subsurfaces) when they are closed or open.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference SurfaceAirflowLeakageNames
      \\note Enter a unique name for this object.
 N1 , \\field Air Mass Flow Coefficient When Opening is Closed
      \\required-field
      \\type real
      \\minimum> 0
      \\units kg/s-m
      \\note Defined at 1 Pa pressure difference. Enter the coefficient used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening (window or door) is closed.
 N2 , \\field Air Mass Flow Exponent When Opening is Closed
      \\units dimensionless
      \\type real
      \\default .65
      \\minimum 0.5
      \\maximum 1.0
      \\note Enter the exponent used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening (window or door) is closed.
 N3 , \\field Minimum Density Difference for Two-Way Flow
      \\required-field
      \\units kg/m3
      \\type real
      \\minimum> 0
      \\note Enter the minimum density difference above which two-way flow may occur due to stack effect.
 N4 ; \\field Discharge Coefficient
      \\required-field
      \\units dimensionless
      \\type real
      \\minimum> 0
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through a window or door at that Opening Factor.

AirflowNetwork:MultiZone:Component:HorizontalOpening,
      \\min-fields 5
      \\memo This object specifies the properties of air flow through a horizontal opening
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference SurfaceAirflowLeakageNames
      \\note Enter a unique name for this object.
 N1 , \\field Air Mass Flow Coefficient When Opening is Closed
      \\required-field
      \\type real
      \\minimum> 0
      \\units kg/s-m
      \\note Defined at 1 Pa pressure difference. Enter the coefficient used in the following equation:
      \\note Mass flow rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening is closed.
 N2 , \\field Air Mass Flow Exponent When Opening is Closed
      \\units dimensionless
      \\type real
      \\default .65
      \\minimum 0.5
      \\maximum 1.0
      \\note Enter the exponent used in the following equation:
      \\note Mass flow rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when opening is closed.
 N3 , \\field Sloping Plane Angle
      \\units deg
      \\type real
      \\minimum> 0
      \\maximum 90
      \\default 90
      \\note Sloping plane angle = 90 is equivalent to fully open.
 N4 ; \\field Discharge Coefficient
      \\required-field
      \\units dimensionless
      \\type real
      \\minimum> 0
      \\note The Discharge Coefficient indicates the fractional effectiveness
      \\note for air flow through the opening at that Opening Factor.

AirflowNetwork:MultiZone:Component:ZoneExhaustFan,
      \\min-fields 3
      \\memo This object specifies the additional properties for a zone exhaust fan
      \\memo to perform multizone airflow calculations.
 A1 , \\field Name
      \\required-field
      \\type object-list
      \\object-list FansZoneExhaust
      \\note Enter the name of a Fan:ZoneExhaust object.
 N1 , \\field Air Mass Flow Coefficient When the Zone Exhaust Fan is Off at Reference Conditions
      \\required-field
      \\type real
      \\units kg/s
      \\minimum> 0
      \\note Enter the air mass flow coefficient at the conditions defined
      \\note in the Reference Crack Conditions object.
      \\note Defined at 1 Pa pressure difference. Enter the coefficient used in the following
      \\note equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when the fan is off.
 N2 , \\field Air Mass Flow Exponent When the Zone Exhaust Fan is Off
      \\units dimensionless
      \\type real
      \\minimum 0.5
      \\maximum 1.0
      \\default 0.65
      \\note Enter the exponent used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent.
      \\note Used only when the fan is off.
 A2 ; \\field Reference Crack Conditions
      \\type object-list
      \\object-list ReferenceCrackConditions
      \\note Select a AirflowNetwork:MultiZone:ReferenceCrackConditions name associated with
      \\note the air mass flow coefficient entered above.

AirflowNetwork:MultiZone:ExternalNode,
      \\min-fields 3
      \\memo This object defines outdoor environmental conditions outside of the building.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference ExternalNodeNames
      \\note Enter a unique name for this object.
      \\note This node name will be referenced by a particular building facade.
 N1 , \\field External Node Height
      \\type real
      \\units m
      \\default 0.0
      \\note Designates the reference height used to calculate relative pressure.
 A2 ; \\field Wind Pressure Coefficient Values Object Name
      \\required-field
      \\type object-list
      \\object-list WPCValueNames
      \\note Enter the name of the AirflowNetwork:MultiZone:WindPressureCoefficientValues object.

AirflowNetwork:MultiZone:WindPressureCoefficientArray,
      \\min-fields 3
      \\memo Used only if Wind Pressure Coefficient (WPC) Type = Input in the AirflowNetwork:SimulationControl
      \\memo object. Number of WPC Values in the corresponding AirflowNetwork:MultiZone:WindPressureCoefficientValues
      \\memo object must be the same as the number of wind directions specified for
      \\memo this AirflowNetwork:MultiZone:WindPressureCoefficientArray object.
 A1 , \\field Name
      \\required-field
      \\reference WPCSetNames
      \\type alpha
      \\note Enter a unique name for the object.
 N1 , \\field Wind Direction 1
      \\required-field
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 1st WPC Array value.
 N2 , \\field Wind Direction 2
      \\required-field
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 2nd WPC Array value.
 N3 , \\field Wind Direction 3
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 3rd WPC Array value.
 N4 , \\field Wind Direction 4
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 4th WPC Array value.
 N5 , \\field Wind Direction 5
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 5th WPC Array value.
 N6 , \\field Wind Direction 6
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 6th WPC Array value.
 N7 , \\field Wind Direction 7
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 7th WPC Array value.
 N8 , \\field Wind Direction 8
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 8th WPC Array value.
 N9 , \\field Wind Direction 9
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 9th WPC Array value.
 N10, \\field Wind Direction 10
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 10th WPC Array value.
 N11, \\field Wind Direction 11
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 11th WPC Array value.
 N12, \\field Wind Direction 12
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 12th WPC Array value.
 N13, \\field Wind Direction 13
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 13th WPC Array value.
 N14, \\field Wind Direction 14
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 14th WPC Array value.
 N15, \\field Wind Direction 15
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 15th WPC Array value.
 N16, \\field Wind Direction 16
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 16th WPC Array value.
 N17, \\field Wind Direction 17
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 17th WPC Array value.
 N18, \\field Wind Direction 18
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 18th WPC Array value.
 N19, \\field Wind Direction 19
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 19th WPC Array value.
 N20, \\field Wind Direction 20
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 20th WPC Array value.
 N21, \\field Wind Direction 21
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 21st WPC Array value.
 N22, \\field Wind Direction 22
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 22nd WPC Array value.
 N23, \\field Wind Direction 23
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 23rd WPC Array value.
 N24, \\field Wind Direction 24
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 24th WPC Array value.
 N25, \\field Wind Direction 25
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 25th WPC Array value.
 N26, \\field Wind Direction 26
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 26th WPC Array value.
 N27, \\field Wind Direction 27
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 27th WPC Array value.
 N28, \\field Wind Direction 28
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 28th WPC Array value.
 N29, \\field Wind Direction 29
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 29th WPC Array value.
 N30, \\field Wind Direction 30
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 30th WPC Array value.
 N31, \\field Wind Direction 31
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 31st WPC Array value.
 N32, \\field Wind Direction 32
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 32nd WPC Array value.
 N33, \\field Wind Direction 33
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 33rd WPC Array value.
 N34, \\field Wind Direction 34
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 34th WPC Array value.
 N35, \\field Wind Direction 35
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 35th WPC Array value.
 N36; \\field Wind Direction 36
      \\type real
      \\units deg
      \\minimum 0.0
      \\maximum 360.0
      \\note Enter the wind direction corresponding to the 36th WPC Array value.

AirflowNetwork:MultiZone:WindPressureCoefficientValues,
      \\min-fields 4
      \\memo Used only if Wind Pressure Coefficient (WPC) Type = INPUT in the AirflowNetwork:SimulationControl
      \\memo object. The number of WPC numeric inputs must correspond to the number of wind direction
      \\memo inputs in the AirflowNetwork:Multizone:WindPressureCoefficientArray object.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference WPCValueNames
      \\note Enter a unique name for this object.
 A2 , \\field AirflowNetwork:MultiZone:WindPressureCoefficientArray Name
      \\required-field
      \\type object-list
      \\object-list WPCSetNames
      \\note Enter the name of the AirflowNetwork:Multizone:WindPressureCoefficientArray object.
 N1 , \\field Wind Pressure Coefficient Value 1
      \\required-field
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 1st wind direction.
 N2 , \\field Wind Pressure Coefficient Value 2
      \\required-field
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 2nd wind direction.
 N3 , \\field Wind Pressure Coefficient Value 3
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 3rd wind direction.
 N4 , \\field Wind Pressure Coefficient Value 4
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 4th wind direction.
 N5 , \\field Wind Pressure Coefficient Value 5
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 5th wind direction.
 N6 , \\field Wind Pressure Coefficient Value 6
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 6th wind direction.
 N7 , \\field Wind Pressure Coefficient Value 7
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 7th wind direction.
 N8 , \\field Wind Pressure Coefficient Value 8
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 8th wind direction.
 N9 , \\field Wind Pressure Coefficient Value 9
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 9th wind direction.
 N10, \\field Wind Pressure Coefficient Value 10
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 10th wind direction.
 N11, \\field Wind Pressure Coefficient Value 11
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 11th wind direction.
 N12, \\field Wind Pressure Coefficient Value 12
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 12th wind direction.
 N13, \\field Wind Pressure Coefficient Value 13
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 13th wind direction.
 N14, \\field Wind Pressure Coefficient Value 14
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 14th wind direction.
 N15, \\field Wind Pressure Coefficient Value 15
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 15th wind direction.
 N16, \\field Wind Pressure Coefficient Value 16
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 16th wind direction.
 N17, \\field Wind Pressure Coefficient Value 17
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 17th wind direction.
 N18, \\field Wind Pressure Coefficient Value 18
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 18th wind direction.
 N19, \\field Wind Pressure Coefficient Value 19
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 19th wind direction.
 N20, \\field Wind Pressure Coefficient Value 20
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 20th wind direction.
 N21, \\field Wind Pressure Coefficient Value 21
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 21st wind direction.
 N22, \\field Wind Pressure Coefficient Value 22
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 22nd wind direction.
 N23, \\field Wind Pressure Coefficient Value 23
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 23rd wind direction.
 N24, \\field Wind Pressure Coefficient Value 24
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 24th wind direction.
 N25, \\field Wind Pressure Coefficient Value 25
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 25th wind direction.
 N26, \\field Wind Pressure Coefficient Value 26
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 26th wind direction.
 N27, \\field Wind Pressure Coefficient Value 27
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 27th wind direction.
 N28, \\field Wind Pressure Coefficient Value 28
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 28th wind direction.
 N29, \\field Wind Pressure Coefficient Value 29
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 29th wind direction.
 N30, \\field Wind Pressure Coefficient Value 30
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 30th wind direction.
 N31, \\field Wind Pressure Coefficient Value 31
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 31st wind direction.
 N32, \\field Wind Pressure Coefficient Value 32
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 32nd wind direction.
 N33, \\field Wind Pressure Coefficient Value 33
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 33rd wind direction.
 N34, \\field Wind Pressure Coefficient Value 34
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 34th wind direction.
 N35, \\field Wind Pressure Coefficient Value 35
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 35th wind direction.
 N36; \\field Wind Pressure Coefficient Value 36
      \\type real
      \\units dimensionless
      \\note Enter the WPC Value corresponding to the 36th wind direction.

AirflowNetwork:Distribution:Node,
      \\min-fields 4
      \\memo This object represents an air distribution node in the AirflowNetwork model.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference AirflowNetworkNodeAndZoneNames
      \\note Enter a unique name for this object.
 A2 , \\field Component Name or Node Name
      \\type alpha
      \\note Designates node names defined in another object. The node name may occur in air branches.
      \\note Enter a node name to represent a node already defined in an air loop.
      \\note Leave this field blank if the Node or Object Type field below is entered as
      \\note AirLoopHVAC:ZoneMixer, AirLoopHVAC:ZoneSplitter, AirLoopHVAC:OutdoorAirSystem, or Other.
 A3 , \\field Component Object Type or Node Type
      \\type choice
      \\key AirLoopHVAC:ZoneMixer
      \\key AirLoopHVAC:ZoneSplitter
      \\key AirLoopHVAC:OutdoorAirSystem
      \\key OAMixerOutdoorAirStreamNode
      \\key OutdoorAir:NodeList
      \\key OutdoorAir:Node
      \\key Other
      \\default Other
      \\note Designates Node type for the Node or Component Name defined in the field above.
      \\note AirLoopHVAC:ZoneMixer -- Represents a AirLoopHVAC:ZoneMixer object.
      \\note AirLoopHVAC:ZoneSplitter -- Represents a AirLoopHVAC:ZoneSplitter object.
      \\note AirLoopHVAC:OutdoorAirSystem -- Represents an AirLoopHVAC:OutdoorAirSystem object.
      \\note OAMixerOutdoorAirStreamNode -- Represents an external node used in the OutdoorAir:Mixer
      \\note OutdoorAir:NodeList -- Represents an external node when a heat exchanger is used before
      \\note the OutdoorAir:Mixer
      \\note OutdoorAir:Node -- Represents an external node when a heat exchanger is used before
      \\note the OutdoorAir:Mixer
      \\note Other -- none of the above, the Node name already defined in the previous field is part
      \\note of an air loop.
 N1 ; \\field Node Height
      \\type real
      \\units m
      \\default 0.0
      \\note Enter the reference height used to calculate the relative pressure.

AirflowNetwork:Distribution:Component:Leak,
      \\min-fields 3
      \\memo This object defines the characteristics of a supply or return air leak.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference AirflowNetworkComponentNames
      \\note Enter a unique name for this object.
 N1 , \\field Air Mass Flow Coefficient
      \\required-field
      \\type real
      \\units kg/s
      \\minimum> 0
      \\note Defined at 1 Pa pressure difference across this component.
      \\note Enter the coefficient used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent
 N2 ; \\field Air Mass Flow Exponent
      \\type real
      \\units dimensionless
      \\minimum 0.5
      \\maximum 1.0
      \\default 0.65
      \\note Enter the exponent used in the following equation:
      \\note Mass Flow Rate = Air Mass Flow Coefficient * (dP)^Air Mass Flow Exponent

AirflowNetwork:Distribution:Component:LeakageRatio,
      \\min-fields 5
      \\memo This object is used to define supply and return air leaks with respect to the fan's maximum
      \\memo air flow rate.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference AirflowNetworkComponentNames
      \\note Enter a unique name for this object.
 N1 , \\field Effective Leakage Ratio
      \\type real
      \\units dimensionless
      \\minimum> 0.0
      \\maximum 1.0
      \\note Defined as a ratio of leak flow rate to the maximum flow rate.
 N2 , \\field Maximum Flow Rate
      \\required-field
      \\units m3/s
      \\type real
      \\minimum> 0.0
      \\note Enter the maximum air flow rate in this air loop.
 N3 , \\field Reference Pressure Difference
      \\required-field
      \\units Pa
      \\type real
      \\minimum> 0.0
      \\note Enter the pressure corresponding to the Effective leakage ratio entered above.
 N4 ; \\field Air Mass Flow Exponent
      \\units dimensionless
      \\type real
      \\default 0.65
      \\minimum 0.5
      \\maximum 1.0
      \\note Enter the exponent used in the air mass flow equation.

AirflowNetwork:Distribution:Component:Duct,
      \\min-fields 8
      \\memo This object defines the relationship between pressure and air flow through the duct.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference AirflowNetworkComponentNames
      \\note Enter a unique name for this object.
 N1 , \\field Duct Length
      \\required-field
      \\type real
      \\units m
      \\minimum> 0.0
      \\note Enter the length of the duct.
 N2 , \\field Hydraulic Diameter
      \\required-field
      \\type real
      \\units m
      \\minimum> 0.0
      \\note Enter the hydraulic diameter of the duct.
      \\note Hydraulic diameter is defined as 4 multiplied by cross section area divided by perimeter
 N3 , \\field Cross Section Area
      \\required-field
      \\type real
      \\units m2
      \\minimum> 0.0
      \\note Enter the cross section area of the duct.
 N4 , \\field Surface Roughness
      \\type real
      \\units m
      \\default 0.0009
      \\minimum> 0.0
      \\note Enter the inside surface roughness of the duct.
 N5 , \\field Coefficient for Local Dynamic Loss Due to Fitting
      \\type real
      \\units dimensionless
      \\default 0.0
      \\minimum 0.0
      \\note Enter the coefficient used to calculate dynamic losses of fittings (e.g. elbows).
 N6 , \\field Overall Heat Transmittance Coefficient (U-Factor) from Air to Air
      \\note including film coefficients at both surfaces
      \\type real
      \\units W/m2-K
      \\minimum> 0.0
      \\default 0.772
      \\note Enter the overall U-value for this duct.
      \\note Default value of 0.772 is equivalent to 1.06 m2-K/W (R6) duct insulation with
      \\note film coefficients for outside and inside equal to 5 and 25 W/m2-K, respectively.
 N7 ; \\field Overall Moisture Transmittance Coefficient from Air to Air
      \\type real
      \\units kg/m2
      \\minimum> 0.0
      \\default 0.001
      \\note Enter the overall moisture transmittance coefficient
      \\note including moisture film coefficients at both surfaces.

AirflowNetwork:Distribution:Component:Fan,
      \\min-fields 2
      \\memo This object defines the name of the constant volume supply Air Fan used in an Air loop.
 A1 , \\field Fan Name
      \\required-field
      \\type object-list
      \\object-list FansCVandOnOff
      \\reference AirflowNetworkComponentNames
      \\note Enter the name of the constant volume fan in the primary air loop.
 A2 ; \\field Supply Fan Object Type
      \\type choice
      \\key Fan:OnOff
      \\key Fan:ConstantVolume
      \\key Fan:VariableVolume
      \\default Fan:ConstantVolume

AirflowNetwork:Distribution:Component:Coil,
      \\min-fields 4
      \\memo This object defines the name of a coil used in an air loop.
 A1 , \\field Coil Name
      \\required-field
      \\type object-list
      \\reference AirflowNetworkComponentNames
      \\object-list AFNCoilNames
      \\note Enter the name of a cooling or heating coil in the primary Air loop.
 A2 , \\field Coil Object Type
      \\required-field
      \\type choice
      \\key Coil:Cooling:DX:SingleSpeed
      \\key Coil:Heating:Gas
      \\key Coil:Heating:Electric
      \\key Coil:Heating:DX:SingleSpeed
      \\key Coil:Cooling:Water
      \\key Coil:Heating:Water
      \\key Coil:Cooling:Water:DetailedGeometry
      \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
      \\key Coil:Cooling:DX:MultiSpeed
      \\key Coil:Heating:DX:MultiSpeed
      \\key Coil:Heating:Desuperheater
      \\note Select the type of coil corresponding to the name entered in the field above.
 N1 , \\field Air Path Length
      \\required-field
      \\type real
      \\units m
      \\minimum> 0
      \\note Enter the air path length (depth) for the coil.
 N2 ; \\field Air Path Hydraulic Diameter
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\note Enter the hydraulic diameter of this coil. The hydraulic diameter is
      \\note defined as 4 multiplied by the cross section area divided by perimeter.

AirflowNetwork:Distribution:Component:HeatExchanger,
      \\min-fields 4
      \\memo This object defines the name of an air-to-air heat exchanger used in an air loop.
 A1 , \\field HeatExchanger Name
      \\required-field
      \\type object-list
      \\object-list AFNHeatExchangerNames
      \\reference AirflowNetworkComponentNames
      \\note Enter the name of an air-to-air heat exchanger in the primary Air loop.
 A2 , \\field HeatExchanger Object Type
      \\required-field
      \\type choice
      \\key HeatExchanger:AirToAir:FlatPlate
      \\key HeatExchanger:AirToAir:SensibleAndLatent
      \\key HeatExchanger:Desiccant:BalancedFlow
      \\note Select the type of heat exchanger corresponding to the name entered in the field above.
 N1 , \\field Air Path Length
      \\required-field
      \\type real
      \\units m
      \\minimum> 0
      \\note Enter the air path length (depth) for the heat exchanger.
 N2 ; \\field Air Path Hydraulic Diameter
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\note Enter the hydraulic diameter of this heat exchanger. The hydraulic diameter is
      \\note defined as 4 multiplied by the cross section area divided by perimeter.

AirflowNetwork:Distribution:Component:TerminalUnit,
      \\min-fields 4
      \\memo This object defines the name of a terminal unit in an air loop.
 A1 , \\field Terminal Unit Name
      \\required-field
      \\type object-list
      \\reference AirflowNetworkComponentNames
      \\object-list AFNTerminalUnitNames
      \\note Enter the name of a terminal unit in the AirLoopHVAC.
 A2 , \\field Terminal Unit Object Type
      \\required-field
      \\type choice
      \\key AirTerminal:SingleDuct:ConstantVolume:Reheat
      \\key AirTerminal:SingleDuct:VAV:Reheat
      \\note Select the type of terminal unit corresponding to the name entered in the field above.
 N1 , \\field Air Path Length
      \\required-field
      \\type real
      \\units m
      \\minimum> 0
      \\note Enter the air path length (depth) for the terminal unit.
 N2 ; \\field Air Path Hydraulic Diameter
      \\required-field
      \\units m
      \\type real
      \\minimum> 0
      \\note Enter the hydraulic diameter of this terminal unit. The hydraulic diameter is
      \\note defined as 4 multiplied by the cross section area divided by perimeter.

AirflowNetwork:Distribution:Component:ConstantPressureDrop,
      \\min-fields 2
      \\memo This object defines the characteristics of a constant pressure drop component (e.g. filter).
      \\memo Each node connected to this object can not be a node of mixer, splitter, a node of air primary
      \\memo loop, or zone equipment loop. It is recommended to connect to a duct component at both ends.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\reference AirflowNetworkComponentNames
      \\note Enter a unique name for this object.
 N1 ; \\field Pressure Difference Across the Component
      \\required-field
      \\units Pa
      \\type real
      \\minimum> 0.0
      \\note Enter the pressure drop across this component.

AirflowNetwork:Distribution:Linkage,
      \\min-fields 4
      \\memo This object defines the connection between two nodes and a component.
 A1 , \\field Name
      \\required-field
      \\type alpha
      \\note Enter a unique name for this object.
      \\reference AirflowNetwork LinkageNames
 A2 , \\field Node 1 Name
      \\required-field
      \\type alpha
      \\object-list AirflowNetworkNodeAndZoneNames
      \\note Enter the name of zone or AirflowNetwork Node.
 A3 , \\field Node 2 Name
      \\required-field
      \\type alpha
      \\object-list AirflowNetworkNodeAndZoneNames
      \\note Enter the name of zone or AirflowNetwork Node.
 A4 , \\field Component Name
      \\required-field
      \\type object-list
      \\object-list AirflowNetworkComponentNames
      \\note Enter the name of an AirflowNetwork component. A component is one of the
      \\note following AirflowNetwork:Distribution:Component objects: Leak, LeakageRatio,
      \\note Duct, ConstantVolumeFan, Coil, TerminalUnit, ConstantPressureDrop, or HeatExchanger.
 A5 ; \\field Thermal Zone Name
      \\type object-list
      \\object-list ZoneNames
      \\note Only used if component = AirflowNetwork:Distribution:Component:Duct
      \\note The zone name is where AirflowNetwork:Distribution:Component:Duct is exposed. Leave this field blank if the duct
      \\note conduction loss is ignored.

\\group Exterior Equipment
       ! This equipment is used merely for reporting.
Exterior:Lights,
       \\memo only used for Meter type reporting, does not affect building loads
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExteriorLightsNames
  A2 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in schedule should be fraction applied to capacity of the exterior lights equipment, generally (0.0 - 1.0)
  N1 , \\field Design Level
       \\required-field
       \\units W
       \\type real
       \\minimum 0
       \\ip-units W
  A3,  \\field Control Option
       \\note Astronomical Clock option overrides schedule to turn lights off when sun is up
       \\type choice
       \\key ScheduleNameOnly
       \\key AstronomicalClock
  A4 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

Exterior:FuelEquipment,
       \\memo only used for Meter type reporting, does not affect building loads
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExteriorFuelEquipmentNames
  A2 , \\field Fuel Use Type
       \\required-field
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Diesel
       \\key Gasoline
       \\key Coal
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
       \\key DistrictCooling
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in schedule should be fraction applied to capacity of the exterior fuel equipment, generally (0.0 - 1.0)
  N1 , \\field Design Level
       \\required-field
       \\units W
       \\type real
       \\minimum 0
       \\ip-units W
  A4 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

Exterior:WaterEquipment,
       \\memo only used for Meter type reporting, does not affect building loads
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExteriorWaterEquipmentNames
  A2 , \\field Fuel Use Type
       \\type choice
       \\key Water
       \\default Water
  A3 , \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note units in Schedule should be fraction applied to capacity of the exterior water equipment, generally (0.0 - 1.0)
  N1 , \\field Design Level
       \\required-field
       \\units m3/s
       \\type real
       \\minimum 0
  A4 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

!****************************************************************************************
!*                   BUILDING/SYSTEM LOOP SPECIFICATIONS                                *
!*                                                                                      *
!*        The following syntax specifies the Loops, and equipment associated            *
!*        with HVAC and Building Systems.  Specifications include Plant Loops,          *
!*        Condenser Loops, Air Loops and Zone Equipment.  The following conventions     *
!*        are followed:                                                                 *
!*          1.  The word 'KEY' followed by a description indicates that the alpha       *
!*              string must be selected from the list indicated by the key description. *
!*          2.  The syntax is organized in the following major blocks:                  *
!*               --PLANT LOOP SPECIFICATION                                             *
!*               --CONDENSER LOOP SPECIFICATION
!*               --AIR LOOP SPECIFICATION
!*               --ZONE EQUIPMENT SPECIFICATION
!*               --FLOW CONTROL
!*               --PLANT LOOP CONNECTORS
!*               --PLANT EQUIPMENT
!*               --CONDENSING EQUIPMENT
!*               --PUMPS
!*               --COILS
!*               --AIR LOOP COMPONENTS
!*               --AIR DISTRIBUTION UNITS
!*               --LOCAL CONVECTIVE EQUIPMENT
!*               --HIGH TEMPERATURE RADIANT CONVECTIVE UNITS
!*               --LOW TEMPERATURE RADIANT CONVECTIVE UNITS
!*               --AIR LOOP CONNECTORS
!*               --System Control and Controllers
!************************************************************************************

\\Group HVAC Templates

HVACTemplate:Thermostat,
      \\min-fields 5
      \\memo Zone thermostat control.  Referenced schedules must be
      \\memo defined elsewhere in the idf.  Thermostat control type is
      \\memo dual setpoint with deadband.  It is not necessary to create
      \\memo a thermostat object for every zone, only for each unique
      \\memo set of setpoint schedules.  For example, an office building
      \\memo may have two thermostat objects, one for "Office" and one
      \\memo for "Storage".
  A1, \\field Name
      \\required-field
      \\note This name is referenced by HVACTemplate:Zone:* objects
      \\reference CompactHVACThermostats
  A2, \\field Heating Setpoint Schedule Name
      \\note  Leave blank if constant setpoint specified below, must enter schedule or constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Constant Heating Setpoint
      \\note  Ignored if schedule specified above, must enter schedule or constant setpoint
      \\units C
  A3, \\field Cooling Setpoint Schedule Name
      \\note  Leave blank if constant setpoint specified below, must enter schedule or constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N2; \\field Constant Cooling Setpoint
      \\note  Ignored if schedule specified above, must enter schedule or constant setpoint
      \\units C

HVACTemplate:Zone:IdealLoadsAirSystem,
      \\min-fields 2
      \\memo Zone with ideal air system that meets heating or cooling loads
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2; \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats

HVACTemplate:Zone:FanCoil,
      \\min-fields 25
      \\memo 4 pipe fan coil unit with optional outdoor air.
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Supply Air Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N3, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A3, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N4, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N5, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N6, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A4, \\field System Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  N7, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N8, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 75
  N9, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
 N10, \\field Supply Fan Motor in Air Stream Fraction
      \\minimum 0.0
      \\maximum 1.0
      \\default 1.0
  A5, \\field Cooling Coil Type
      \\type choice
      \\key  ChilledWater
      \\key  ChilledWaterDetailedFlatModel
      \\default  ChilledWater
  A6, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N11, \\field Cooling Coil Design Setpoint
      \\note Used for sizing when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
      \\default 14.0
      \\units C
  A7, \\field Heating Coil Type
      \\type choice
      \\key  HotWater
      \\default HotWater
  A8, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N12, \\field Heating Coil Design Setpoint
      \\note Used for sizing when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
      \\default 50.0
      \\units C
  A9, \\field Dedicated Outdoor Air System Name
      \\note Enter the name of an HVACTemplate:System:DedicatedOutdoorAir object if this
      \\note zone is served by a separate dedicated outdoor air system (DOAS).
      \\note Leave field blank if no DOAS serves this zone.
      \\type object-list
      \\object-list HVACTemplateDOASSystems
 A10, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Cooling Coil Design Setpoint (above)
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N13, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A11, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Heating Coil Design Setpoint (above)
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N14, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A12, \\field Design Specification Outdoor Air Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A13; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:PTAC,
      \\min-fields 34
      \\memo Packaged Terminal Air Conditioner
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Cooling Supply Air Flow Rate
      \\note Supply air flow rate during cooling operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Heating Supply Air Flow Rate
      \\note Supply air flow rate during heating operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N3, \\field No Load Supply Air Flow Rate
      \\note Supply air flow rate when no cooling or heating is needed
      \\note Only used when heat pump fan operating mode is continuous. This air flow rate
      \\note is used when no heating or cooling is required and the DX coil compressor is off.
      \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
      \\note (either cooling or heating) is used.
      \\note A value entered in this field will *not* be multiplied by the sizing factor or
      \\note by zone multipliers.  It is best to autosize or leave blank when using zone multipliers.
      \\type real
      \\minimum 0
      \\autosizable
      \\units m3/s
  N4, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N5, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A3, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N6, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N7, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N8, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A4, \\field System Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Supply Fan Operating Mode Schedule Name
      \\note Refers to a schedule to specify unitary supply fan operating mode.
      \\note Schedule Name values of 0 indicate cycling fan (auto)
      \\note Schedule values of 1 indicate continuous fan (on)
      \\note Keywords "Cycling" or "Continuous" may also be used here.
      \\type object-list
      \\object-list ScheduleNames
      \\default Cycling
  A6, \\field Supply Fan Placement
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default DrawThrough
  N9, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
 N10, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 75
 N11, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  A7, \\field Cooling Coil Type
      \\type choice
      \\key  SingleSpeedDX
      \\default SingleSpeedDX
  A8, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N12, \\field Cooling Coil Rated Capacity
      \\note Gross capacity excluding effect of supply air fan heat
      \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
      \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units W
 N13, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
      \\note Sensible and total capacities do not include effect of supply fan heat
      \\type real
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N14, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\type real
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A9, \\field Heating Coil Type
      \\type choice
      \\key  Electric
      \\key  HotWater
      \\key  Gas
      \\default Electric
 A10, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N15, \\field Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N16, \\field Gas Heating Coil Efficiency
      \\note Applies only if Heating Coil Type is Gas
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N17, \\field Gas Heating Coil Parasitic Electric Load
      \\note Applies only if Heating Coil Type is Gas
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 A11, \\field Dedicated Outdoor Air System Name
      \\note Enter the name of an HVACTemplate:System:DedicatedOutdoorAir object if this
      \\note zone is served by a separate dedicated outdoor air system (DOAS).
      \\note Leave field blank if no DOAS serves this zone.
      \\type object-list
      \\object-list HVACTemplateDOASSystems
 A12, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N18, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 14.0
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N19, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A13, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N20, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N21, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A14, \\field Design Specification Outdoor Air Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A15; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:PTHP,
      \\min-fields 44
      \\memo Packaged Terminal Heat Pump
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Cooling Supply Air Flow Rate
      \\note Supply air flow rate during cooling operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Heating Supply Air Flow Rate
      \\note Supply air flow rate during heating operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N3, \\field No Load Supply Air Flow Rate
      \\note Supply air flow rate when no cooling or heating is needed
      \\note Only used when heat pump fan operating mode is continuous. This air flow rate
      \\note is used when no heating or cooling is required and the DX coil compressor is off.
      \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
      \\note (either cooling or heating) is used.
      \\note A value entered in this field will *not* be multiplied by the sizing factor or
      \\note by zone multipliers.  It is best to autosize or leave blank when using zone multipliers.
      \\type real
      \\minimum 0
      \\autosizable
      \\units m3/s
  N4, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N5, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A3, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N6, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N7, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N8, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A4, \\field System Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Supply Fan Operating Mode Schedule Name
      \\note Refers to a schedule to specify unitary supply fan operating mode.
      \\note Schedule values of 0 indicate cycling fan (auto)
      \\note Schedule values of 1 indicate continuous fan (on)
      \\note Keywords "Cycling" or "Continuous" may also be used here.
      \\type object-list
      \\object-list ScheduleNames
      \\default Cycling
  A6, \\field Supply Fan Placement
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default DrawThrough
  N9, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
 N10, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 75
 N11, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  A7, \\field Cooling Coil Type
      \\type choice
      \\key  SingleSpeedDX
      \\default SingleSpeedDX
  A8, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N12, \\field Cooling Coil Rated Capacity
      \\note Gross capacity excluding effect of supply air fan heat
      \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
      \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units W
 N13, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
      \\note Sensible and total capacities do not include effect of supply fan heat
      \\type real
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N14, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\type real
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A9, \\field Heat Pump Heating Coil Type
      \\type choice
      \\key  SingleSpeedDXHeatPump
      \\default SingleSpeedDXHeatPump
 A10, \\field Heat Pump Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N15, \\field Heat Pump Heating Coil Rated Capacity
      \\note Capacity excluding supply air fan heat
      \\note Rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
      \\note Rating point heating coil entering air dry-bulb 21.11 C, coil entering wet-bulb 15.55 C
      \\type real
      \\units W
      \\minimum> 0.0
      \\autosizable
      \\default autosize
 N16, \\field Heat Pump Heating Coil Rated COP
      \\note Heat Pump Heating Coil Rated Capacity divided by power input to the compressor and outdoor fan,
      \\note Does not include supply air fan heat or supply air fan electrical energy
      \\note Rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
      \\note Rating point heating coil entering air dry-bulb 21.11 C, coil entering wet-bulb 15.55 C
      \\units W/W
      \\minimum> 0.0
      \\default 2.75
 N17, \\field Heat Pump Heating Minimum Outdoor Dry-Bulb Temperature
      \\type real
      \\minimum -20.0
      \\default -8.0
      \\units C
 N18, \\field Heat Pump Defrost Maximum Outdoor Dry-Bulb Temperature
      \\type real
      \\minimum 0.0
      \\maximum 7.22
      \\default 5.0
      \\units C
 A11, \\field Heat Pump Defrost Strategy
      \\type choice
      \\key ReverseCycle
      \\key Resistive
      \\default ReverseCycle
 A12, \\field Heat Pump Defrost Control
      \\type choice
      \\key Timed
      \\key OnDemand
      \\default Timed
 N19, \\field Heat Pump Defrost Time Period Fraction
      \\type real
      \\minimum 0.0
      \\default 0.058333
      \\note Fraction of time in defrost mode
      \\note only applicable if Timed defrost control is specified
 A13, \\field Supplemental Heating Coil Type
      \\type choice
      \\key  Electric
      \\key  Gas
      \\key  HotWater
      \\default Electric
 A14, \\field Supplemental Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N20, \\field Supplemental Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N21, \\field Supplemental Heating Coil Maximum Outdoor Dry-Bulb Temperature
      \\type real
      \\maximum 21.0
      \\default 21.0
      \\units C
      \\note Supplemental heater will not operate when outdoor temperature exceeds this value.
 N22, \\field Supplemental Gas Heating Coil Efficiency
      \\note Applies only if Supplemental Heating Coil Type is Gas
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N23, \\field Supplemental Gas Heating Coil Parasitic Electric Load
      \\note Applies only if Supplemental Heating Coil Type is Gas
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 A15, \\field Dedicated Outdoor Air System Name
      \\note Enter the name of an HVACTemplate:System:DedicatedOutdoorAir object if this
      \\note zone is served by a separate dedicated outdoor air system (DOAS).
      \\note Leave field blank if no DOAS serves this zone.
      \\type object-list
      \\object-list HVACTemplateDOASSystems
 A16, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N24, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 14.0
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N25, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A17, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N26, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N27, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A18, \\field Design Specification Outdoor Air Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A19; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:Unitary,
      \\min-fields 21
      \\memo Zone terminal unit, constant volume, no controls.
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template Unitary System Name
      \\required-field
      \\note Enter the name of a HVACTemplate:System:Unitary object serving this zone
      \\type object-list
      \\object-list CompactHVACSystemUnitary
  A3, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Supply Air Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N3, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A4, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N4, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N5, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N6, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A5, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
  A6, \\field Return Plenum Name
      \\note Plenum zone name.  Return plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
  A7, \\field Baseboard Heating Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  None
      \\default None
  A8, \\field Baseboard Heating Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  N7, \\field Baseboard Heating Capacity
      \\autosizable
      \\default autosize
      \\units W
  A9, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\note SystemSupplyAirTemperature = use the value from HVACTemplate:System:Unitary or HVACTemplate:System:UnitaryHeatPump:AirToAir
      \\note Cooling Design Supply Air Temperature
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\key SystemSupplyAirTemperature
      \\default SystemSupplyAirTemperature
  N8, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 12.8
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
  N9, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A10, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\note SystemSupplyAirTemperature = use the value from HVACTemplate:System:Unitary or HVACTemplate:System:UnitaryHeatPump:AirToAir
      \\note Heating Design Supply Air Temperature
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\key SystemSupplyAirTemperature
      \\default SystemSupplyAirTemperature
 N10, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N11, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A11, \\field Design Specification Outdoor Air Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A12; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:VAV,
      \\min-fields 32
      \\memo Zone terminal unit, variable volume, reheat optional.
      \\memo Referenced schedules must be defined elsewhere in the idf.
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template VAV System Name
      \\required-field
      \\note Name of a HVACTemplate:System:VAV object serving this zone
      \\type object-list
      \\object-list CompactHVACSystemVAV
  A3, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Supply Air Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N3, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A4 ,\\field Zone Minimum Air Flow Input Method
      \\type choice
      \\key Constant
      \\key FixedFlowRate
      \\key Scheduled
      \\default Constant
      \\note Constant = Constant Minimum Air Flow Fraction (a fraction of Maximum Air Flow Rate)
      \\note FixedFlowRate = Fixed Minimum Air Flow Rate (a fixed minimum air volume flow rate)
      \\note Scheduled = Scheduled Minimum Air Flow Fraction (a fraction of Maximum Air Flow
  N4 ,\\field Constant Minimum Air Flow Fraction
      \\note This field is used if the field Zone Minimum Air Flow Input Method is Constant
      \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
      \\note is optional. If a value is entered, then it is used for sizing normal-action reheat coils.
      \\note If both this field and the following field are entered, the larger result is used.
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.2
  N5 ,\\field Fixed Minimum Air Flow Rate
      \\type real
      \\units m3/s
      \\note This field is used if the field Zone Minimum Air Flow Input Method is FixedFlowRate.
      \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
      \\note is optional. If a value is entered, then it is used for sizing normal-action reheat coils.
      \\note If both this field and the previous field are entered, the larger result is used.
  A5 ,\\field Minimum Air Flow Fraction Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note This field is used if the field Zone Minimum Air Flow Input Method is Scheduled
      \\note Schedule values are fractions, 0.0 to 1.0.
      \\note If the field Constant Minimum Air Flow Fraction is blank, then the average of the
      \\note minimum and maximum schedule values is used for sizing normal-action reheat coils.
  A6, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N6, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N7, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N8, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A7, \\field Reheat Coil Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  None
      \\default None
  A8, \\field Reheat Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A9, \\field Damper Heating Action
      \\type choice
      \\key Normal
      \\key Reverse
      \\default Reverse
  N9, \\field Maximum Flow per Zone Floor Area During Reheat
      \\type real
      \\units m3/s-m2
      \\autocalculatable
      \\note Used only when Reheat Coil Object Type = Coil:Heating:Water and Damper Heating Action = Reverse
      \\note When autocalculating, the maximum flow per zone is set to 0.002032 m3/s-m2 (0.4 cfm/sqft)
      \\note This optional field limits the maximum flow allowed in reheat mode.
      \\note If this field and the following field are left blank, the maximum flow will not be limited.
      \\note At no time will the maximum flow rate calculated here exceed the value of
      \\note Maximum Air Flow Rate.
 N10, \\field Maximum Flow Fraction During Reheat
      \\type real
      \\autocalculatable
      \\note Used only when Reheat Coil Object Type = Coil:Heating:Water and Damper Heating Action = Reverse
      \\note When autocalculating, the maximum flow fraction is set to the ratio of
      \\note 0.002032 m3/s-m2 (0.4 cfm/sqft) multiplied by the zone floor area and the
      \\note Maximum Air Flow Rate.
      \\note This optional field limits the maximum flow allowed in reheat mode.
      \\note If this field and the previous field are left blank, the maximum flow will not be limited.
      \\note At no time will the maximum flow rate calculated here exceed the value of
      \\note Maximum Air Flow Rate.
 N11, \\field Maximum Reheat Air Temperature
      \\note Specifies the maximum allowable supply air temperature leaving the reheat coil.
      \\note If left blank, there is no limit and no default. If unknown, 35C (95F) is recommended.
      \\type real
      \\units C
      \\minimum> 0.0
 A10, \\field Design Specification Outdoor Air Object Name for Control
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
      \\note When the name of a DesignSpecification:OutdoorAir object is entered, the terminal
      \\note unit will increase flow as needed to meet this outdoor air requirement.
      \\note If Outdoor Air Flow per Person is non-zero, then the outdoor air requirement will
      \\note be computed based on the current number of occupants in the zone.
      \\note At no time will the supply air flow rate exceed the value for Maximum Air Flow Rate.
      \\note If this field is blank, then the terminal unit will not be controlled for outdoor air flow.
      \\note Note that this field is used only for specifying the design outdoor air flow rate used
      \\note for control. The field Design Specification Outdoor Air Object Name for Sizing
      \\note (see below) is used to specify the design outdoor air flow rate.
      \\type object-list
 A11, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A12, \\field Return Plenum Name
      \\note Plenum zone name.  Return plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A13, \\field Baseboard Heating Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  None
      \\default None
 A14, \\field Baseboard Heating Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N12, \\field Baseboard Heating Capacity
      \\autosizable
      \\default autosize
      \\units W
 A15, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\note SystemSupplyAirTemperature = use the value from HVACTemplate:System:VAV Cooling Coil Design Setpoint
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\key SystemSupplyAirTemperature
      \\default SystemSupplyAirTemperature
 N13, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 12.8
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N14, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A16, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N15, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N16, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A17, \\field Design Specification Outdoor Air Object Name for Sizing
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\note Note that this field is used only for specifying the design outdoor air flow rate used
      \\note for sizing. The field Design Specification Outdoor Air Object Name for Control
      \\note (see above) is used to actively control the VAV terminal air flow rate.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A18; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:VAV:FanPowered,
      \\min-fields 31
      \\memo Zone terminal unit, fan powered variable volume, reheat optional.
      \\memo Referenced schedules must be defined elsewhere in the idf.
  A1, \\field Zone Name
      \\required-field
      \\note Zone Name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template VAV System Name
      \\required-field
      \\note Enter then name of a HVACTemplate:System:VAV object serving this zone.
      \\type object-list
      \\object-list CompactHVACSystemVAV
  A3, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Primary Supply Air Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N3, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N4, \\field Primary Supply Air Minimum Flow Fraction
      \\autosizable
      \\default autosize
      \\minimum 0.0
      \\maximum 1.0
  N5, \\field Secondary Supply Air Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\autosizable
      \\default autosize
      \\units m3/s
  A4, \\field Flow Type
      \\type choice
      \\key  Parallel
      \\key  Series
      \\default Parallel
  N6, \\field Parallel Fan On Flow Fraction
      \\note The fraction of the primary air flow at which fan turns on
      \\note Applicable only to Parallel Flow Type
      \\autosizable
      \\default autosize
  A5, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N7, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N8, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/area, Sum, or Maximum
  N9, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A6, \\field Reheat Coil Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\default Electric
  A7, \\field Reheat Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N10, \\field Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
 N11, \\field Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 1000
 N12, \\field Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  A8, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
  A9, \\field Return Plenum Name
      \\note Plenum zone name.  Return plenum runs through only this zone.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A10, \\field Baseboard Heating Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  None
      \\default None
 A11, \\field Baseboard Heating Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N13, \\field Baseboard Heating Capacity
      \\autosizable
      \\default autosize
      \\units W
 A12, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\note SystemSupplyAirTemperature = use the value from HVACTemplate:System:VAV Cooling Coil Design Setpoint
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\key SystemSupplyAirTemperature
      \\default SystemSupplyAirTemperature
 N14, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 12.8
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N15, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A13, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N16, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N17, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A14, \\field Zone PIU Fan Schedule Name
      \\note This is the operating schedule for the zone PIU fan.
      \\note For a parallel PIU, the fan operates only when the primary air flow is below the
      \\note Parallel Fan On Flow Fraction and the Zone PIU Fan Schedule is on, or it is
      \\note activated by an availability manager.
      \\note For a series PIU, the zone fan operates whenever the Zone PIU Fan Schedule is on, or it
      \\note is activated by an availability manager.
      \\note If this field is left blank, the System Availability Schedule for the
      \\note HVACTemplate:System serving this zone will be used.
      \\type object-list
      \\object-list ScheduleNames
 A15, \\field Design Specification Outdoor Air Object Name
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A16; \\field Design Specification Zone Air Distribution Object Name
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:Zone:WaterToAirHeatPump,
      \\min-fields 39
      \\memo Wate to Air Heat Pump to be used with HVACTemplate:Plant:MixedWaterLoop
  A1, \\field Zone Name
      \\required-field
      \\note Zone name must match a building zone name
      \\type object-list
      \\object-list ZoneNames
  A2, \\field Template Thermostat Name
      \\note Enter the name of a HVACTemplate:Thermostat object.
      \\note If blank, then it is assumed that standard thermostat objects
      \\note have been defined for this zone.
      \\type object-list
      \\object-list CompactHVACThermostats
  N1, \\field Cooling Supply Air Flow Rate
      \\note Supply air flow rate during cooling operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Heating Supply Air Flow Rate
      \\note Supply air flow rate during heating operation
      \\note This field may be set to "autosize".  If a value is entered, it will be
      \\note multiplied by the Supply Air Sizing Factor and by zone multipliers.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N3, \\field No Load Supply Air Flow Rate
      \\note Supply air flow rate when no cooling or heating is needed
      \\note Only used when heat pump fan operating mode is continuous. This air flow rate
      \\note is used when no heating or cooling is required.
      \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
      \\note (either cooling or heating) is used.
      \\note A value entered in this field will *not* be multiplied by the sizing factor or
      \\note by zone multipliers.  It is best to autosize or leave blank when using zone multipliers.
      \\type real
      \\minimum 0
      \\autosizable
      \\units m3/s
  N4, \\field Zone Heating Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  N5, \\field Zone Cooling Sizing Factor
      \\note If blank, value from Sizing:Parameters will be used.
  A3, \\field Outdoor Air Method
      \\note Flow/Person, Flow/Zone, Flow/Area, Sum, and Maximum use the values in the next three
      \\note fields: Outdoor Air Flow Rate per Person, Outdoor Air Flow Rate per Zone Floor Area,
      \\note and Outdoor Air Flow Rate per Zone.
      \\note DetailedSpecification ignores these three Outdoor Air Flow Rate fields and instead
      \\note references design specification objects named in the fields
      \\note Design Specification Outdoor Air Object Name and Design Specification Zone Air
      \\note Distribution Object Name.
      \\type choice
      \\key Flow/Person
      \\key Flow/Zone
      \\key Flow/Area
      \\key Sum
      \\key Maximum
      \\key DetailedSpecification
      \\default Flow/Person
  N6, \\field Outdoor Air Flow Rate per Person
      \\units m3/s
      \\note Default 0.00944 is 20 cfm per person
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Person, Sum, or Maximum
      \\default 0.00944
  N7, \\field Outdoor Air Flow Rate per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Area, Sum, or Maximum
  N8, \\field Outdoor Air Flow Rate per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\note This input is used if the field Outdoor Air Method is
      \\note Flow/Zone, Sum, or Maximum
  A4, \\field System Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Supply Fan Operating Mode Schedule Name
      \\note Refers to a schedule to specify unitary supply fan operating mode.
      \\note Schedule values of 0 indicate cycling fan (auto)
      \\note Schedule values of 1 indicate continuous fan (on)
      \\note Keywords "Cycling" or "Continuous" may also be used here.
      \\type object-list
      \\object-list ScheduleNames
      \\default Cycling
  A6, \\field Supply Fan Placement
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default DrawThrough
  N9, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
 N10, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 75
 N11, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  A7, \\field Cooling Coil Type
      \\type choice
      \\key  Coil:Cooling:WaterToAirHeatPump:EquationFit
      \\default Coil:Cooling:WaterToAirHeatPump:EquationFit
 N12, \\field Cooling Coil Rated Capacity
      \\note Gross capacity excluding effect of supply air fan heat
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units W
 N13, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
      \\note Sensible and total capacities do not include effect of supply fan heat
      \\type real
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N14, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\type real
      \\units W/W
      \\minimum> 0.0
      \\default 3.5
  A8, \\field Heat Pump Heating Coil Type
      \\type choice
      \\key  Coil:Heating:WaterToAirHeatPump:EquationFit
      \\default Coil:Heating:WaterToAirHeatPump:EquationFit
 N15, \\field Heat Pump Heating Coil Rated Capacity
      \\note Capacity excluding supply air fan heat
      \\type real
      \\units W
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units W
 N16, \\field Heat Pump Heating Coil Rated COP
      \\note Heat Pump Heating Coil Rated Capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply air fan heat or supply air fan electrical energy
      \\units W/W
      \\minimum> 0.0
      \\default 4.2
  A9, \\field Supplemental Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N17, \\field Supplemental Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N18, \\field Maximum Cycling Rate
      \\type real
      \\units cycles/hr
      \\minimum 0.0
      \\maximum 5.0
      \\default 2.5
      \\note The maximum on-off cycling rate for the compressor
      \\note Suggested value is 2.5 for a typical heat pump
 N19, \\field Heat Pump Time Constant
      \\type real
      \\units s
      \\minimum 0.0
      \\maximum 500.0
      \\default 60.0
      \\note Time constant for the cooling coil's capacity to reach steady state after startup
      \\note Suggested value is 60 for a typical heat pump
 N20, \\field Fraction of On-Cycle Power Use
      \\minimum 0.0
      \\maximum 0.05
      \\default 0.01
      \\note The fraction of on-cycle power use to adjust the part load fraction based on
      \\note the off-cycle power consumption due to crankcase heaters, controls, fans, and etc.
      \\note Suggested value is 0.01 for a typical heat pump
 N21, \\field Heat Pump Fan Delay Time
      \\units s
      \\minimum 0.0
      \\default 60
      \\note Programmed time delay for heat pump fan to shut off after compressor cycle off.
      \\note Only required when fan operating mode is cycling
      \\note Enter 0 when fan operating mode is continuous
 A10, \\field Dedicated Outdoor Air System Name
      \\note Enter the name of an HVACTemplate:System:DedicatedOutdoorAir object if this
      \\note zone is served by a separate dedicated outdoor air system (DOAS).
      \\note Leave field blank if no DOAS serves this zone.
      \\type object-list
      \\object-list HVACTemplateDOASSystems
 A11, \\field Supplemental Heating Coil Type
      \\type choice
      \\key  Electric
      \\key  HotWater
      \\default Electric
 A12, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Cooling Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Cooling Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N22, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\default 14.0
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N23, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 11.11
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
 A13, \\field Zone Heating Design Supply Air Temperature Input Method
      \\note SupplyAirTemperature = use the value from Zone Heating Design Supply Air Temperature
      \\note TemperatureDifference = use the value from Zone Heating Design Supply Air Temperature Difference
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
 N24, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\default 50.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
 N25, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\default 30.0
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
 A14, \\field Heat Pump Coil Water Flow Mode
      \\type choice
      \\key Constant
      \\key Cycling
      \\key ConstantOnDemand
      \\default Cycling
      \\note used only when the heat pump coils are of the type WaterToAirHeatPump:EquationFit
      \\note Constant results in 100% water flow regardless of compressor PLR
      \\note Cycling results in water flow that matches compressor PLR
      \\note ConstantOnDemand results in 100% water flow whenever the coil is on, but is 0% whenever the coil has no load
 A15, \\field Design Specification Outdoor Air Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
 A16; \\field Design Specification Zone Air Distribution Object Name
      \\note This field is used only when Outdoor Air Method=DetailedSpecification.
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

HVACTemplate:System:Unitary,
      \\min-fields 47
      \\memo Unitary furnace with air conditioner
      \\memo Referenced schedules must be defined elsewhere in the idf.
  A1, \\field Name
      \\required-field
      \\reference CompactHVACSystemUnitary
      \\reference HVACTemplateSystems
  A2, \\field System Availability Schedule Name
      \\note If blank, always on; Unitary System always on.  Schedule is used in availability manager
      \\note and fan scheduling.
      \\note Also see "Night Cycle Control" field.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Control Zone or Thermostat Location Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
  N1, \\field Supply Fan Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum> 0.0
  A4, \\field Supply Fan Operating Mode Schedule Name
      \\note Refers to a schedule to specify unitary supply fan operating mode.
      \\note Schedule values of 0 indicate cycling fan (auto)
      \\note Schedule values of 1 indicate continuous fan (on)
      \\note Keywords "Cycling" or "Continuous" may also be used here.
      \\type object-list
      \\object-list ScheduleNames
      \\default Cycling
  N2, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N3, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 600
  N4, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  N5, \\field Supply Fan Motor in Air Stream Fraction
      \\minimum 0.0
      \\maximum 1.0
      \\default 1.0
  A5, \\field Cooling Coil Type
      \\type choice
      \\key  SingleSpeedDX
      \\key  None
      \\default SingleSpeedDX
  A6, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  N6, \\field Cooling Design Supply Air Temperature
      \\note Used for sizing.
      \\default 12.8
      \\units C
  N7, \\field Cooling Coil Capacity
      \\note Gross cooling capacity (net capacity plus fan heat)
      \\autosizable
      \\default autosize
      \\units W
  N8, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Gross SHR
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
  N9, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply air fan heat or supply air fan electrical energy
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A7, \\field Heating Coil Type
      \\required-field
      \\type choice
      \\key  Electric
      \\key  Gas
      \\key  HotWater
  A8, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N10, \\field Heating Design Supply Air Temperature
      \\note Used for sizing.
      \\default 50.0
      \\units C
 N11, \\field Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N12, \\field Gas Heating Coil Efficiency
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N13, \\field Gas Heating Coil Parasitic Electric Load
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 N14, \\field Maximum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 N15, \\field Minimum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
  A9, \\field Minimum Outdoor Air Schedule Name
      \\note Schedule values multiply the minimum outdoor air flow rate
      \\note If blank, always one
      \\type object-list
      \\object-list ScheduleNames
 A10, \\field Economizer Type
      \\type choice
      \\key FixedDryBulb
      \\key FixedEnthalpy
      \\key DifferentialDryBulb
      \\key DifferentialEnthalpy
      \\key FixedDewPointAndDryBulb
      \\key ElectronicEnthalpy
      \\key DifferentialDryBulbAndEnthalpy
      \\key NoEconomizer
      \\default NoEconomizer
 A11, \\field Economizer Lockout
      \\type choice
      \\key NoLockout
      \\key LockoutWithHeating
      \\key LockoutWithCompressor
      \\default NoLockout
 N16, \\field Economizer Upper Temperature Limit
      \\note Outdoor temperature above which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units C
 N17, \\field Economizer Lower Temperature Limit
      \\note Outdoor temperature below which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units C
 N18, \\field Economizer Upper Enthalpy Limit
      \\note Outdoor enthalpy above which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units J/kg
 N19, \\field Economizer Maximum Limit Dewpoint Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dewpoint temperature limit for FixedDewPointAndDryBulb
      \\note economizer control type. No input or blank input means this limit is not operative.
      \\note Limit is applied regardless of economizer control type.
 A12, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A13, \\field Return Plenum Name
      \\note Plenum zone name.  Return plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A14, \\field Supply Fan Placement
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default BlowThrough
 A15, \\field Night Cycle Control
      \\type choice
      \\key StayOff
      \\key CycleOnAny
      \\key CycleOnControlZone
      \\default StayOff
 A16, \\field Night Cycle Control Zone Name
      \\note Applicable only if Night Cycle Control is Cycle On Control Zone.
      \\type object-list
      \\object-list ZoneNames
 A17, \\field Heat Recovery Type
      \\type choice
      \\key None
      \\key Sensible
      \\key Enthalpy
      \\default None
 N20, \\field Sensible Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.70
 N21, \\field Latent Heat Recovery Effectiveness
      \\note Applicable only if Heat Recovery Type is Enthalpy.
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.65
 A18, \\field Dehumidification Control Type
      \\note None = meet sensible cooling load only
      \\note CoolReheatHeatingCoil = cool beyond the dry-bulb setpoint
      \\note as required to meet the humidity setpoint, reheat with main heating coil.
      \\note CoolReheatDesuperheater = cool beyond the dry-bulb setpoint
      \\note as required to meet the humidity setpoint, reheat with desuperheater coil.
      \\type choice
      \\key None
      \\key CoolReheatHeatingCoil
      \\key CoolReheatDesuperheater
      \\default None
 A19, \\field Dehumidification Control Zone Name
      \\note This field is not currently used - the thermostat control zone is also the dehumidification control zone
      \\type object-list
      \\object-list ZoneNames
 N22, \\field Dehumidification Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 60.0
      \\units percent
 A20, \\field Humidifier Type
      \\type choice
      \\key None
      \\key ElectricSteam
      \\default None
 A21, \\field Humidifier Availability Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
 N23, \\field Humidifier Rated Capacity
      \\note Moisture output rate at full rated power input.
      \\note The humidifier does not currently autosize, so the default is very large
      \\note to allow for adequate capacity.
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.000001
      \\ip-units gal/min
 N24, \\field Humidifier Rated Electric Power
      \\note Electric power input at rated capacity moisture output.
      \\note Power consumption is proportional to moisture output with no part-load penalty.
      \\type real
      \\units W
      \\minimum 0.0
      \\autosizable
      \\default autosize
      \\ip-units W
 A22, \\field Humidifier Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N25; \\field Humidifier Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 30.0
      \\units percent

HVACTemplate:System:UnitaryHeatPump:AirToAir,
      \\min-fields 56
      \\memo Unitary furnace with electric air-to-air heat pump
      \\memo Referenced schedules must be defined elsewhere in the idf.
  A1, \\field Name
      \\required-field
      \\reference CompactHVACSystemUnitary
      \\reference HVACTemplateSystems
  A2, \\field System Availability Schedule Name
      \\note If blank, always on; Unitary System always on.  Schedule is used in availability manager
      \\note and fan scheduling.
      \\note Also see "Night Cycle Control" field.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Control Zone or Thermostat Location Name
      \\required-field
      \\type object-list
      \\object-list ZoneNames
  N1, \\field Cooling Supply Air Flow Rate
      \\note Supply air flow rate during cooling operation
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N2, \\field Heating Supply Air Flow Rate
      \\note Supply air flow rate during heating operation
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\type real
      \\minimum> 0.0
      \\autosizable
      \\default autosize
      \\units m3/s
  N3, \\field No Load Supply Air Flow Rate
      \\note Supply air flow rate when no cooling or heating is needed
      \\note Only used when heat pump fan operating mode is Continuous. This air flow rate
      \\note is used when no heating or cooling is required and the DX coil compressor is off.
      \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
      \\note (either cooling or heating) is used.
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\type real
      \\minimum 0
      \\default autosize
      \\autosizable
      \\units m3/s
  A4, \\field Supply Fan Operating Mode Schedule Name
      \\note Refers to a schedule to specify unitary supply fan operating mode.
      \\note Schedule values of 0 indicate cycling fan (auto)
      \\note Schedule values of 1 indicate continuous fan (on)
      \\note Keywords "Cycling" or "Continuous" may also be used here.
      \\type object-list
      \\object-list ScheduleNames
      \\default Cycling
  A5, \\field Supply Fan Placement
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default BlowThrough
  N4, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N5, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 600
  N6, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  N7, \\field Supply Fan Motor in Air Stream Fraction
      \\minimum 0.0
      \\maximum 1.0
      \\default 1.0
  A6, \\field Cooling Coil Type
      \\type choice
      \\key  SingleSpeedDX
      \\default SingleSpeedDX
  A7, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  N8, \\field Cooling Design Supply Air Temperature
      \\note Used for sizing.
      \\default 12.8
      \\units C
  N9, \\field Cooling Coil Capacity
      \\note Gross capacity excluding effect of supply air fan heat
      \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
      \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
      \\autosizable
      \\default autosize
      \\units W
 N10, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
      \\note Sensible and total capacities do not include effect of supply fan heat
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N11, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A8, \\field Heat Pump Heating Coil Type
      \\type choice
      \\key  SingleSpeedDXHeatPump
      \\default SingleSpeedDXHeatPump
  A9, \\field Heat Pump Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N12, \\field Heating Design Supply Air Temperature
      \\note Used for sizing.
      \\default 50.0
      \\units C
 N13, \\field Heat Pump Heating Coil Rated Capacity
      \\note Capacity excluding supply air fan heat
      \\note Rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
      \\note Rating point heating coil entering air dry-bulb 21.11 C, coil entering wet-bulb 15.55 C
      \\type real
      \\units W
      \\minimum> 0.0
      \\autosizable
      \\default autosize
 N14, \\field Heat Pump Heating Coil Rated COP
      \\note Heat Pump Heating Coil Rated Capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply air fan heat or supply air fan electrical energy.
      \\note Rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
      \\note Rating point heating coil entering air dry-bulb 21.11 C, coil entering wet-bulb 15.55 C
      \\units W/W
      \\minimum> 0.0
      \\default 2.75
 N15, \\field Heat Pump Heating Minimum Outdoor Dry-Bulb Temperature
      \\type real
      \\minimum -20.0
      \\default -8.0
      \\units C
 N16, \\field Heat Pump Defrost Maximum Outdoor Dry-Bulb Temperature
      \\type real
      \\minimum 0.0
      \\maximum 7.22
      \\default 5.0
      \\units C
 A10, \\field Heat Pump Defrost Strategy
      \\type choice
      \\key ReverseCycle
      \\key Resistive
      \\default ReverseCycle
 A11, \\field Heat Pump Defrost Control
      \\type choice
      \\key Timed
      \\key OnDemand
      \\default Timed
 N17, \\field Heat Pump Defrost Time Period Fraction
      \\type real
      \\minimum 0.0
      \\default 0.058333
      \\note Fraction of time in defrost mode
      \\note only applicable if Timed defrost control is specified
 A12, \\field Supplemental Heating Coil Type
      \\type choice
      \\key  Electric
      \\key  Gas
      \\key  HotWater
      \\default Electric
 A13, \\field Supplemental Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 N18, \\field Supplemental Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N19, \\field Supplemental Heating Coil Maximum Outdoor Dry-Bulb Temperature
      \\type real
      \\maximum 21.0
      \\default 21.0
      \\units C
      \\note Supplemental heater will not operate when outdoor temperature exceeds this value.
 N20, \\field Supplemental Gas Heating Coil Efficiency
      \\note Applies only if Supplemental Heating Coil Type is Gas
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N21, \\field Supplemental Gas Heating Coil Parasitic Electric Load
      \\note Applies only if Supplemental Heating Coil Type is Gas
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 N22, \\field Maximum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 N23, \\field Minimum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 A14, \\field Minimum Outdoor Air Schedule Name
      \\note Schedule values multiply the minimum outdoor air flow rate
      \\note If blank, multiplier is always one
      \\type object-list
      \\object-list ScheduleNames
 A15, \\field Economizer Type
      \\type choice
      \\key FixedDryBulb
      \\key FixedEnthalpy
      \\key DifferentialDryBulb
      \\key DifferentialEnthalpy
      \\key FixedDewPointAndDryBulb
      \\key ElectronicEnthalpy
      \\key DifferentialDryBulbAndEnthalpy
      \\key NoEconomizer
      \\default NoEconomizer
 A16, \\field Economizer Lockout
      \\type choice
      \\key NoLockout
      \\key LockoutWithHeating
      \\key LockoutWithCompressor
      \\default NoLockout
 N24, \\field Economizer Maximum Limit Dry-Bulb Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dry-bulb temperature limit for FixedDryBulb
      \\note economizer control type. No input or blank input means this limit is
      \\note not operative. Limit is applied regardless of economizer control type.
 N25, \\field Economizer Maximum Limit Enthalpy
      \\type real
      \\units J/kg
      \\note Enter the maximum outdoor enthalpy limit for FixedEnthalpy economizer control type.
      \\note No input or blank input means this limit is not operative
      \\note Limit is applied regardless of economizer control type.
 N26, \\field Economizer Maximum Limit Dewpoint Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dewpoint temperature limit for FixedDewPointAndDryBulb
      \\note economizer control type. No input or blank input means this limit is not operative.
      \\note Limit is applied regardless of economizer control type.
 N27, \\field Economizer Minimum Limit Dry-Bulb Temperature
      \\type real
      \\units C
      \\note Enter the minimum outdoor dry-bulb temperature limit for economizer control.
      \\note No input or blank input means this limit is not operative
      \\note Limit is applied regardless of economizer control type.
 A17, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A18, \\field Return Plenum Name
      \\note Plenum zone name.  Return plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A19, \\field Night Cycle Control
      \\type choice
      \\key StayOff
      \\key CycleOnAny
      \\key CycleOnControlZone
      \\default StayOff
 A20, \\field Night Cycle Control Zone Name
      \\note Applicable only if Night Cycle Control is Cycle On Control Zone.
      \\type object-list
      \\object-list ZoneNames
 A21, \\field Heat Recovery Type
      \\type choice
      \\key None
      \\key Sensible
      \\key Enthalpy
      \\default None
 N28, \\field Sensible Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.70
 N29, \\field Latent Heat Recovery Effectiveness
      \\note Applicable only if Heat Recovery Type is Enthalpy.
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.65
 A22, \\field Humidifier Type
      \\type choice
      \\key None
      \\key ElectricSteam
      \\default None
 A23, \\field Humidifier Availability Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
 N30, \\field Humidifier Rated Capacity
      \\note Moisture output rate at full rated power input.
      \\note The humidifier does not currently autosize, so the default is very large
      \\note to allow for adequate capacity.
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.000001
      \\ip-units gal/min
 N31, \\field Humidifier Rated Electric Power
      \\note Electric power input at rated capacity moisture output.
      \\note Power consumption is proportional to moisture output with no part-load penalty.
      \\type real
      \\units W
      \\minimum 0.0
      \\autosizable
      \\default autosize
      \\ip-units W
 A24, \\field Humidifier Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N32; \\field Humidifier Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 30.0
      \\units percent

HVACTemplate:System:VAV,
      \\min-fields 55
      \\memo Variable Air Volume (VAV) air loop with optional heating coil
      \\memo and optional preheat.
      \\memo Referenced Schedules must be defined elsewhere in the idf.
  A1, \\field Name
      \\required-field
      \\reference CompactHVACSystemVAV
      \\reference HVACTemplateSystems
  A2, \\field System Availability Schedule Name
      \\note If blank, always on; VAV System always on.  Schedule is used in availability manager
      \\note and fan scheduling.
      \\note Also see "Night Cycle Control" field.
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Supply Fan Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum> 0.0
  N2, \\field Supply Fan Minimum Flow Rate
      \\note This field is only used to set a minimum part load on the VAV fan power curve.
      \\note Autosize or zero is recommended.
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
  N3, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N4, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 1000
  N5, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  N6, \\field Supply Fan Motor in Air Stream Fraction
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A3, \\field Cooling Coil Type
      \\type choice
      \\key  ChilledWater
      \\key  ChilledWaterDetailedFlatModel
      \\default  ChilledWater
  A4, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A5, \\field Cooling Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N7, \\field Cooling Coil Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Cooling Coil Setpoint Schedule Name is specified.
      \\default 12.8
      \\units C
  A6, \\field Heating Coil Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  Gas
      \\key  None
      \\default None
  A7, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A8, \\field Heating Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N8, \\field Heating Coil Design Setpoint
      \\note   Used for sizing and as constant setpoint if no Heating Coil Setpoint Schedule Name is specified.
      \\default 10.0
      \\units C
  N9, \\field Gas Heating Coil Efficiency
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N10, \\field Gas Heating Coil Parasitic Electric Load
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
  A9, \\field Preheat Coil Type
      \\type choice
      \\key  Electric
      \\key  Gas
      \\key  None
      \\default None
 A10, \\field Preheat Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 A11, \\field Preheat Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
 N11, \\field Preheat Coil Design Setpoint
      \\note   Used for sizing and as constant setpoint if no Preheat Coil Setpoint Schedule Name specified.
      \\units C
      \\default 7.2
 N12, \\field Gas Preheat Coil Efficiency
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N13, \\field Gas Preheat Coil Parasitic Electric Load
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 N14, \\field Maximum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 N15, \\field Minimum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 A12, \\field Minimum Outdoor Air Control Type
      \\type choice
      \\key FixedMinimum
      \\key ProportionalMinimum
      \\default ProportionalMinimum
 A13, \\field Minimum Outdoor Air Schedule Name
      \\note Schedule values multiply the Minimum Outdoor Air Flow Rate
      \\note If blank, multiplier is always one
      \\type object-list
      \\object-list ScheduleNames
 A14, \\field Economizer Type
      \\type choice
      \\key FixedDryBulb
      \\key FixedEnthalpy
      \\key DifferentialDryBulb
      \\key DifferentialEnthalpy
      \\key FixedDewPointAndDryBulb
      \\key ElectronicEnthalpy
      \\key DifferentialDryBulbAndEnthalpy
      \\key NoEconomizer
      \\default NoEconomizer
 A15, \\field Economizer Lockout
      \\type choice
      \\key NoLockout
      \\default NoLockout
 N16, \\field Economizer Upper Temperature Limit
      \\note Outdoor temperature above which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units C
 N17, \\field Economizer Lower Temperature Limit
      \\note Outdoor temperature below which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units C
 N18, \\field Economizer Upper Enthalpy Limit
      \\note Outdoor enthalpy above which economizer is disabled and
      \\note heat recovery is enabled (if available).
      \\note Blank means no limit.
      \\units J/kg
 N19, \\field Economizer Maximum Limit Dewpoint Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dewpoint temperature limit for FixedDewPointAndDryBulb
      \\note economizer control type. No input or blank input means this limit is not operative.
      \\note Limit is applied regardless of economizer control type.
 A16, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A17, \\field Return Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A18, \\field Supply Fan Placement
      \\type choice
      \\key DrawThrough
      \\key BlowThrough
      \\default DrawThrough
 A19, \\field Supply Fan Part-Load Power Coefficients
      \\note This field selects a predefined set of fan power coefficients.
      \\note The ASHRAE 90.1-2004 Appendix G coefficients are from TABLE G3.1.3.15, Method 2.
      \\note The other sets of coefficients are from the EnergyPlus Input
      \\note Output Reference, Fan Coefficient Values table.
      \\type choice
      \\key InletVaneDampers
      \\key OutletDampers
      \\key VariableSpeedMotor
      \\key ASHRAE90.1-2004AppendixG
      \\key VariableSpeedMotorPressureReset
      \\default InletVaneDampers
 A20, \\field Night Cycle Control
      \\type choice
      \\key StayOff
      \\key CycleOnAny
      \\key CycleOnControlZone
      \\key CycleOnAnyZoneFansOnly
      \\default StayOff
 A21, \\field Night Cycle Control Zone Name
      \\note Applicable only if Night Cycle Control is Cycle On Control Zone.
      \\type object-list
      \\object-list ZoneNames
 A22, \\field Heat Recovery Type
      \\type choice
      \\key None
      \\key Sensible
      \\key Enthalpy
      \\default None
 N20, \\field Sensible Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.70
 N21, \\field Latent Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.65
 A23, \\field Cooling Coil Setpoint Reset Type
      \\note Overrides Cooling Coil Setpoint Schedule Name
      \\note Warmest = as warm as possible yet meet all zone cooling loads
      \\type choice
      \\key None
      \\key Warmest
      \\key OutdoorAirTemperatureReset
      \\default None
 A24, \\field Heating Coil Setpoint Reset Type
      \\note Overrides Heating Coil Setpoint Schedule Name
      \\type choice
      \\key None
      \\key OutdoorAirTemperatureReset
      \\default None
 A25, \\field Dehumidification Control Type
      \\note None = meet sensible load only
      \\note CoolReheat = cool beyond the dry-bulb setpoint
      \\note as required to meet the humidity setpoint.
      \\type choice
      \\key None
      \\key CoolReheat
      \\default None
 A26, \\field Dehumidification Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N22, \\field Dehumidification Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 60.0
      \\units percent
 A27, \\field Humidifier Type
      \\type choice
      \\key None
      \\key ElectricSteam
      \\default None
 A28, \\field Humidifier Availability Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
 N23, \\field Humidifier Rated Capacity
      \\note Moisture output rate at full rated power input.
      \\note The humidifier does not currently autosize, so the default is very large
      \\note to allow for adequate capacity.
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.000001
      \\ip-units gal/min
 N24, \\field Humidifier Rated Electric Power
      \\note Electric power input at rated capacity moisture output.
      \\note Power consumption is proportional to moisture output with no part-load penalty.
      \\type real
      \\units W
      \\minimum 0.0
      \\autosizable
      \\default autosize
      \\ip-units W
 A29, \\field Humidifier Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N25, \\field Humidifier Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 30.0
      \\units percent
 A30; \\field Sizing Option
      \\note Select whether autosized system supply flow rate is the sum of Coincident or NonCoincident
      \\note zone air flow rates.
      \\type choice
      \\key Coincident
      \\key NonCoincident
      \\default NonCoincident

HVACTemplate:System:PackagedVAV,
      \\min-fields 53
      \\memo Packaged Variable Air Volume (PVAV) air loop with optional heating coil
      \\memo and optional preheat.
      \\memo Referenced Schedules must be defined elsewhere in the idf.
  A1, \\field Name
      \\required-field
      \\reference CompactHVACSystemVAV
      \\reference HVACTemplateSystems
  A2, \\field System Availability Schedule Name
      \\note If blank, always on; PVAV System always on.  Schedule is used in availability manager
      \\note and fan scheduling.
      \\note Also see "Night Cycle Control" field.
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Supply Fan Maximum Flow Rate
      \\note This field may be set to "autosize".  If a value is entered, it will *not* be
      \\note multiplied by any sizing factor or by zone multipliers.  If using zone multipliers
      \\note a value entered here must be large enough to serve the multiplied zones.
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum> 0.0
  N2, \\field Supply Fan Minimum Flow Rate
      \\note This field is only used to set a minimum part load on the VAV fan power curve.
      \\note Autosize or zero is recommended.
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
  A3, \\field Supply Fan Placement
      \\type choice
      \\key DrawThrough
      \\key BlowThrough
      \\default DrawThrough
  N3, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N4, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 1000
  N5, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  N6, \\field Supply Fan Motor in Air Stream Fraction
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A4, \\field Cooling Coil Type
      \\type choice
      \\key  TwoSpeedDX
      \\key  TwoSpeedHumidControlDX
      \\default TwoSpeedDX
  A5, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A6, \\field Cooling Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N7, \\field Cooling Coil Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Cooling Coil Setpoint Schedule Name is specified.
      \\default 12.8
      \\units C
  N8, \\field Cooling Coil Capacity
      \\note Gross cooling capacity (net capacity plus fan heat)
      \\autosizable
      \\default autosize
      \\units W
  N9, \\field Cooling Coil Rated Sensible Heat Ratio
      \\note Gross SHR
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N10, \\field Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A7, \\field Heating Coil Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  Gas
      \\key  None
      \\default None
  A8, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A9, \\field Heating Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
 N11, \\field Heating Coil Design Setpoint
      \\note   Used for sizing and as constant setpoint if no Heating Coil Setpoint Schedule Name is specified.
      \\default 10.0
      \\units C
 N12, \\field Heating Coil Capacity
      \\autosizable
      \\default autosize
      \\units W
 N13, \\field Gas Heating Coil Efficiency
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N14, \\field Gas Heating Coil Parasitic Electric Load
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 N15, \\field Maximum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 N16, \\field Minimum Outdoor Air Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum 0.0
 A10, \\field Minimum Outdoor Air Control Type
      \\type choice
      \\key FixedMinimum
      \\key ProportionalMinimum
      \\default ProportionalMinimum
 A11, \\field Minimum Outdoor Air Schedule Name
      \\note Schedule values multiply the Minimum Outdoor Air Flow Rate
      \\note If blank, multiplier is always one
      \\type object-list
      \\object-list ScheduleNames
 A12, \\field Economizer Type
      \\type choice
      \\key FixedDryBulb
      \\key FixedEnthalpy
      \\key DifferentialDryBulb
      \\key DifferentialEnthalpy
      \\key FixedDewPointAndDryBulb
      \\key ElectronicEnthalpy
      \\key DifferentialDryBulbAndEnthalpy
      \\key NoEconomizer
      \\default NoEconomizer
 A13, \\field Economizer Lockout
      \\type choice
      \\key NoLockout
      \\key LockoutWithHeating
      \\key LockoutWithCompressor
      \\default NoLockout
 N17, \\field Economizer Maximum Limit Dry-Bulb Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dry-bulb temperature limit for FixedDryBulb
      \\note economizer control type. No input or blank input means this limit is
      \\note not operative. Limit is applied regardless of economizer control type.
 N18, \\field Economizer Maximum Limit Enthalpy
      \\type real
      \\units J/kg
      \\note Enter the maximum outdoor enthalpy limit for FixedEnthalpy economizer control type.
      \\note No input or blank input means this limit is not operative
      \\note Limit is applied regardless of economizer control type.
 N19, \\field Economizer Maximum Limit Dewpoint Temperature
      \\type real
      \\units C
      \\note Enter the maximum outdoor dewpoint temperature limit for FixedDewPointAndDryBulb
      \\note economizer control type. No input or blank input means this limit is not operative.
      \\note Limit is applied regardless of economizer control type.
 N20, \\field Economizer Minimum Limit Dry-Bulb Temperature
      \\type real
      \\units C
      \\note Enter the minimum outdoor dry-bulb temperature limit for economizer control.
      \\note No input or blank input means this limit is not operative
      \\note Limit is applied regardless of economizer control type.
 A14, \\field Supply Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A15, \\field Return Plenum Name
      \\note Plenum zone name.  Supply plenum serves all zones on this system.
      \\note Blank if none.
      \\type object-list
      \\object-list ZoneNames
 A16, \\field Supply Fan Part-Load Power Coefficients
      \\note This field selects a predefined set of fan power coefficients.
      \\note The ASHRAE 90.1-2004 Appendix G coefficients are from TABLE G3.1.3.15, Method 2.
      \\note The other sets of coefficients are from the EnergyPlus Input
      \\note Output Reference, Fan Coefficient Values table.
      \\type choice
      \\key InletVaneDampers
      \\key OutletDampers
      \\key VariableSpeedMotor
      \\key ASHRAE90.1-2004AppendixG
      \\key VariableSpeedMotorPressureReset
      \\default InletVaneDampers
 A17, \\field Night Cycle Control
      \\type choice
      \\key StayOff
      \\key CycleOnAny
      \\key CycleOnControlZone
      \\key CycleOnAnyZoneFansOnly
      \\default StayOff
 A18, \\field Night Cycle Control Zone Name
      \\note Applicable only if Night Cycle Control is Cycle On Control Zone.
      \\type object-list
      \\object-list ZoneNames
 A19, \\field Heat Recovery Type
      \\type choice
      \\key None
      \\key Sensible
      \\key Enthalpy
      \\default None
 N21, \\field Sensible Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.70
 N22, \\field Latent Heat Recovery Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.65
 A20, \\field Cooling Coil Setpoint Reset Type
      \\note Overrides Cooling Coil Setpoint Schedule Name
      \\note Warmest = as warm as possible yet meet all zone cooling loads
      \\type choice
      \\key None
      \\key Warmest
      \\key OutdoorAirTemperatureReset
      \\default None
 A21, \\field Heating Coil Setpoint Reset Type
      \\note Overrides Heating Coil Setpoint Schedule Name
      \\type choice
      \\key None
      \\key OutdoorAirTemperatureReset
      \\default None
 A22, \\field Dehumidification Control Type
      \\note None = meet sensible load only
      \\note CoolReheat = cool beyond the dry-bulb setpoint
      \\note as required to meet the humidity setpoint.
      \\type choice
      \\key None
      \\key CoolReheat
      \\default None
 A23, \\field Dehumidification Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N23, \\field Dehumidification Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 60.0
      \\units percent
 A24, \\field Humidifier Type
      \\type choice
      \\key None
      \\key ElectricSteam
      \\default None
 A25, \\field Humidifier Availability Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
 N24, \\field Humidifier Rated Capacity
      \\note Moisture output rate at full rated power input.
      \\note The humidifier does not currently autosize, so the default is very large
      \\note to allow for adequate capacity.
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.000001
      \\ip-units gal/min
 N25, \\field Humidifier Rated Electric Power
      \\note Electric power input at rated capacity moisture output.
      \\note Power consumption is proportional to moisture output with no part-load penalty.
      \\type real
      \\units W
      \\minimum 0.0
      \\autosizable
      \\default autosize
      \\ip-units W
 A26, \\field Humidifier Control Zone Name
      \\note Zone name where humidistat is located
      \\type object-list
      \\object-list ZoneNames
 N26, \\field Humidifier Setpoint
      \\note Zone relative humidity setpoint in percent (0 to 100)
      \\type real
      \\minimum 0.0
      \\maximum 100.0
      \\default 30.0
      \\units percent
 A27; \\field Sizing Option
      \\note Select whether autosized system supply flow rate is the sum of Coincident or NonCoincident
      \\note zone air flow rates.
      \\type choice
      \\key Coincident
      \\key NonCoincident
      \\default NonCoincident

HVACTemplate:System:DedicatedOutdoorAir,
      \\min-fields 46
      \\memo This object creates a dedicated outdoor air system that must be used with
      \\memo HVACTemplate:Zone:* objects for FanCoil PTAC PTHP and WaterToAirHeatPump.
      \\memo Does not support HVACTemplate:Zone:VAV or other central multizone systems
      \\memo Referenced Schedules must be defined elsewhere in the idf.
  A1, \\field Name
      \\reference HVACTemplateDOASSystems
      \\reference HVACTemplateSystems
  A2, \\field System Availability Schedule Name
      \\note If blank, always on; DOAS System always on.  Schedule is used in availability manager
      \\note and fan scheduling.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Air Outlet Type
      \\type choice
      \\key  DirectIntoZone
      \\default  DirectIntoZone
  N1, \\field Supply Fan Flow Rate
      \\autosizable
      \\default autosize
      \\units m3/s
      \\minimum> 0.0
  N2, \\field Supply Fan Total Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.7
  N3, \\field Supply Fan Delta Pressure
      \\units Pa
      \\ip-units inH2O
      \\minimum 0
      \\default 1000
  N4, \\field Supply Fan Motor Efficiency
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.9
  N5, \\field Supply Fan Motor in Air Stream Fraction
      \\minimum 0.0
      \\maximum 1.0
      \\default 1.0
  A4, \\field Supply Fan Placement
      \\type choice
      \\key DrawThrough
      \\key BlowThrough
      \\default DrawThrough
  A5, \\field Cooling Coil Type
      \\type choice
      \\key  ChilledWater
      \\key  ChilledWaterDetailedFlatModel
      \\key  TwoSpeedDX
      \\key  TwoStageDX
      \\key  TwoStageHumidityControlDX
      \\key  HeatExchangerAssistedChilledWater
      \\key  HeatExchangerAssistedDX
      \\key  None
      \\default  ChilledWater
  A6, \\field Cooling Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
  A7, \\field Cooling Coil Setpoint Control Type
      \\type choice
      \\key  FixedSetpoint
      \\key  Scheduled
      \\key  OutdoorAirTemperatureReset
      \\key  None
      \\default FixedSetpoint
  N6, \\field Cooling Coil Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Cooling Coil Setpoint Schedule Name is specified.
      \\default 12.8
      \\units C
  A8, \\field Cooling Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N7, \\field Cooling Coil Setpoint at Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 15.6
  N8, \\field Cooling Coil Reset Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\note Defaults are 15.6C (60F) at 15.6C (60F) to 12.8C (55F) at 23.3C (74F)
      \\units C
      \\default 15.6
  N9, \\field Cooling Coil Setpoint at Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 12.8
 N10, \\field Cooling Coil Reset Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 23.3
 N11, \\field DX Cooling Coil Capacity
      \\note Gross cooling capacity (net capacity plus fan heat)
      \\autosizable
      \\default autosize
      \\units W
 N12, \\field DX Cooling Coil Rated Sensible Heat Ratio
      \\note Gross SHR
      \\autosizable
      \\minimum 0.5
      \\maximum 1.0
      \\default autosize
 N13, \\field DX Cooling Coil Rated COP
      \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
      \\note does not include supply fan heat or supply fan electrical energy input
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  A9, \\field Heating Coil Type
      \\type choice
      \\key  HotWater
      \\key  Electric
      \\key  Gas
      \\key  None
      \\default HotWater
 A10, \\field Heating Coil Availability Schedule Name
      \\note If blank, always on
      \\type object-list
      \\object-list ScheduleNames
 A11, \\field Heating Coil Setpoint Control Type
      \\note When selecting OutdoorAirTemperatureReset, the Heating Coil Design Setpoint may need to be changed
      \\type choice
      \\key  FixedSetpoint
      \\key  Scheduled
      \\key  OutdoorAirTemperatureReset
      \\key  None
      \\default FixedSetpoint
 N14, \\field Heating Coil Design Setpoint
      \\note   Used for sizing and as constant setpoint if no Heating Coil Setpoint Schedule Name is specified.
      \\default 12.2
      \\units C
 A12, \\field Heating Coil Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
 N15, \\field Heating Coil Setpoint at Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\note Defaults 15.0C (59F) at 7.8C (46F) to 12.2C (54F) at 12.2C (54F)
      \\units C
      \\default 15.0
 N16, \\field Heating Coil Reset Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 7.8
 N17, \\field Heating Coil Setpoint at Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 12.2
 N18, \\field Heating Coil Reset Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 12.2
 N19, \\field Gas Heating Coil Efficiency
      \\maximum 1.0
      \\minimum 0.0
      \\default 0.8
 N20, \\field Gas Heating Coil Parasitic Electric Load
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
 A13, \\field Heat Recovery Type
      \\type choice
      \\key None
      \\key Sensible
      \\key Enthalpy
      \\default None
 N21, \\field Heat Recovery Sensible Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.70
 N22, \\field Heat Recovery Latent Effectiveness
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.65
 A14, \\field Heat Recovery Heat Exchanger Type
      \\type choice
      \\key Plate
      \\key Rotary
      \\default Plate
 A15, \\field Heat Recovery Frost Control Type
      \\type choice
      \\key None
      \\key ExhaustAirRecirculation
      \\key ExhaustOnly
      \\key MinimumExhaustTemperature
      \\default None
 A16, \\field Dehumidification Control Type
      \\note None = meet sensible load only
      \\note CoolReheatHeatingCoil = cool beyond the dry-bulb setpoint, reheat with heating coil
      \\note Valid for all cooling coil types.  If no heating coil specified, cold supply temps may occur.
      \\note CoolReheatDesuperheater = cool beyond the dry-bulb setpoint
      \\note as required to meet the humidity setpoint, reheat with desuperheater coil.
      \\note Valid only for Cooling Coil Type = TwoSpeedDX, TwoStageDX, TwoStageHumidityControlDX, or HeatExchangerAssistedDX.
      \\note Multimode = activate enhanced dehumidification mode as needed and meet sensible load.
      \\note Valid only for Cooling Coil Type = TwoStageHumidityControlDX
      \\type choice
      \\key None
      \\key CoolReheatHeatingCoil
      \\key CoolReheatDesuperheater
      \\key Multimode
      \\default None
 N23, \\field Dehumidification Setpoint
      \\note The supply air humidity ratio for dehumidification control.
      \\note Default of 0.00924 kgWater/kgDryAir is equivalent to 12.8C (55F) dewpoint.
      \\note Ignored if Dehumidification Setpoint Schedule specified below
      \\type real
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.00924
      \\units kgWater/kgDryAir
 A17, \\field Humidifier Type
      \\type choice
      \\key None
      \\key ElectricSteam
      \\default None
 A18, \\field Humidifier Availability Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
 N24, \\field Humidifier Rated Capacity
      \\note Moisture output rate at full rated power input.
      \\note The humidifier does not currently autosize, so the default is very large
      \\note to allow for adequate capacity.
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.000001
      \\ip-units gal/min
 N25, \\field Humidifier Rated Electric Power
      \\note Electric power input at rated capacity moisture output.
      \\note Power consumption is proportional to moisture output with no part-load penalty.
      \\type real
      \\units W
      \\minimum 0.0
      \\autosizable
      \\default autosize
      \\ip-units W
 N26, \\field Humidifier Constant Setpoint
      \\note The supply air humidity ratio for humidification control.
      \\note Ignored if Humidifier Setpoint Schedule specified below
      \\type real
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.003
      \\units kgWater/kgDryAir
 A19, \\field Dehumidification Setpoint Schedule Name
      \\note Leave blank to use constant setpoint specified in Dehumidification Setpoint above.
      \\note Schedule values must be in units of humidity ratio (kgWater/kgDryAir or lbWater/lbDryAir)
      \\type object-list
      \\object-list ScheduleNames
 A20; \\field Humidifier Setpoint Schedule Name
      \\note Leave blank to use constant setpoint specified in Humidifer Constant Setpoint above.
      \\note Schedule values must be in units of humidity ratio (kgWater/kgDryAir or lbWater/lbDryAir)
      \\type object-list
      \\object-list ScheduleNames

HVACTemplate:Plant:ChilledWaterLoop,
      \\min-fields 21
      \\memo Plant and condenser loops to serve all HVACTemplate
      \\memo chilled water coils, chillers, and towers.
      \\unique-object
  A1, \\field Name
      \\required-field
  A2, \\field Pump Schedule Name
      \\note If blank, always available
      \\note Applies to both chilled water and condenser loop pumps
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Pump Control Type
      \\note Applies to both chilled water and condenser loop pumps
      \\type choice
      \\key Intermittent
      \\key Continuous
      \\default Intermittent
  A4, \\field Chiller Plant Operation Scheme Type
      \\note Default operation type makes all equipment available
      \\note at all times operating in order of Priority specified
      \\note in HVACTemplate:Plant:Chiller objects.
      \\type choice
      \\key Default
      \\key UserDefined
      \\default Default
  A5, \\field Chiller Plant Equipment Operation Schemes Name
      \\type object-list
      \\object-list PlantOperationSchemes
      \\note Name of a PlantEquipmentOperationSchemes object
      \\note Ignored if Chiller Plant Operation Scheme Type = Default
  A6, \\field Chilled Water Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Chilled Water Design Setpoint
      \\note   Used for sizing and as constant setpoint if no Chilled Water Setpoint Schedule Name is specified.
      \\default 7.22
      \\units C
  A7, \\field Chilled Water Pump Configuration
      \\note VariablePrimaryNoSecondary - variable flow to chillers and coils
      \\note ConstantPrimaryNoSecondary - constant flow to chillers and coils, excess bypassed
      \\note ConstantPrimaryVariableSecondary - constant flow to chillers, variable flow to coils
      \\note VariablePrimaryConstantSecondary - currently unsupported - variable flow to chillers, constant flow to coils
      \\type choice
      \\key ConstantPrimaryNoSecondary
      \\key VariablePrimaryNoSecondary
      \\key ConstantPrimaryVariableSecondary
      \\default ConstantPrimaryNoSecondary
  N2, \\field Primary Chilled Water Pump Rated Head
      \\units Pa
      \\minimum 0.0
      \\default 179352
      \\note default head is 60 feet H2O
      \\ip-units ftH2O
  N3, \\field Secondary Chilled Water Pump Rated Head
      \\units Pa
      \\minimum 0.0
      \\default 179352
      \\note default head is 60 feet H2O
      \\ip-units ftH2O
  A8, \\field Condenser Plant Operation Scheme Type
      \\note Default operation type makes all equipment available
      \\note at all times operating in order of Priority specified
      \\note in HVACTemplate:Plant:Tower objects.
      \\type choice
      \\key Default
      \\key UserDefined
      \\default Default
      \\note  May be left blank if not serving any water cooled chillers
  A9, \\field Condenser Equipment Operation Schemes Name
      \\type object-list
      \\object-list CondenserOperationSchemes
      \\note Name of a CondenserEquipmentOperationSchemes object
      \\note Ignored if Condenser Plant Operation Scheme Type = Default
      \\note  May be left blank if not serving any water cooled chillers
 A10, \\field Condenser Water Temperature Control Type
      \\note  May be left blank if not serving any water cooled chillers
      \\type choice
      \\key OutdoorWetBulbTemperature
      \\key SpecifiedSetpoint
 A11, \\field Condenser Water Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\note  May be left blank if not serving any water cooled chillers
      \\type object-list
      \\object-list ScheduleNames
  N4, \\field Condenser Water Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Condenser Water Setpoint Schedule Name is specified.
      \\note  May be left blank if not serving any water cooled chillers
      \\default 29.4
      \\units C
  N5, \\field Condenser Water Pump Rated Head
      \\note  May be left blank if not serving any water cooled chillers
      \\units Pa
      \\minimum 0.0
      \\default 179352
      \\note default head is 60 feet H2O
      \\ip-units ftH2O
 A12, \\field Chilled Water Setpoint Reset Type
      \\note Overrides Chilled Water Setpoint Schedule Name
      \\type choice
      \\key None
      \\key OutdoorAirTemperatureReset
      \\default None
  N6, \\field Chilled Water Setpoint at Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 12.2
  N7, \\field Chilled Water Reset Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 15.6
  N8, \\field Chilled Water Setpoint at Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 6.7
  N9, \\field Chilled Water Reset Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 26.7
 A13, \\field Chilled Water Primary Pump Type
      \\note Describes the type of pump configuration used for the primary portion of the chilled water loop.
      \\type choice
      \\key SinglePump
      \\key PumpPerChiller
      \\key TwoHeaderedPumps
      \\key ThreeHeaderedPumps
      \\key FourHeaderedPumps
      \\key FiveHeaderedPumps
      \\default SinglePump
 A14, \\field Chilled Water Secondary Pump Type
      \\note Describes the type of pump configuration used for the secondary portion of the chilled water loop.
      \\type choice
      \\key SinglePump
      \\key TwoHeaderedPumps
      \\key ThreeHeaderedPumps
      \\key FourHeaderedPumps
      \\key FiveHeaderedPumps
      \\default SinglePump
 A15, \\field Condenser Water Pump Type
      \\note Describes the type of pump configuration used for the condenser water loop.
      \\type choice
      \\key SinglePump
      \\key PumpPerTower
      \\key TwoHeaderedPumps
      \\key ThreeHeaderedPumps
      \\key FourHeaderedPumps
      \\key FiveHeaderedPumps
      \\default SinglePump
  A16,\\field Chilled Water Supply Side Bypass Pipe
      \\note Determines if a supply side bypass pipe is present in the chilled water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A17,\\field Chilled Water Demand Side Bypass Pipe
      \\note Determines if a demand side bypass pipe is present in the chilled water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A18,\\field Condenser Water Supply Side Bypass Pipe
      \\note Determines if a supply side bypass pipe is present in the condenser water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A19,\\field Condenser Water Demand Side Bypass Pipe
      \\note Determines if a demand side bypass pipe is present in the condenser water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A20,\\field Fluid Type
      \\type choice
      \\key Water
      \\key EthyleneGlycol30
      \\key EthyleneGlycol40
      \\key EthyleneGlycol50
      \\key EthyleneGlycol60
      \\key PropyleneGlycol30
      \\key PropyleneGlycol40
      \\key PropyleneGlycol50
      \\key PropyleneGlycol60
      \\default Water
  N10,\\field Loop Design Delta Temperature
      \\note The temperature difference used in sizing the loop flow rate.
      \\units deltaC
      \\default 6.67
  N11,\\field Minimum Outdoor Dry Bulb Temperature
      \\note The minimum outdoor dry bulb temperature that the chilled water loops operate.
      \\note Leave blank for no limit.
      \\units C
  A21,\\field Chilled Water Load Distribution Scheme
      \\type choice
      \\key Optimal
      \\key Sequential
      \\key Uniform
      \\default Sequential
  A22;\\field Condenser Water Load Distribution Scheme
      \\type choice
      \\key Optimal
      \\key Sequential
      \\key Uniform
      \\default Sequential

HVACTemplate:Plant:Chiller,
      \\min-fields 7
      \\memo This object adds a chiller to an HVACTemplate:Plant:ChilledWaterLoop.
  A1, \\field Name
      \\required-field
  A2, \\field Chiller Type
      \\required-field
      \\type choice
      \\key DistrictChilledWater
      \\key ElectricCentrifugalChiller
      \\key ElectricScrewChiller
      \\key ElectricReciprocatingChiller
  N1, \\field Capacity
      \\autosizable
      \\default autosize
      \\units W
      \\minimum> 0.0
  N2, \\field Nominal COP
      \\required-field
      \\note Not applicable if Chiller Type is DistrictChilledWater
      \\note Electric Reciprocating Chiller
      \\units W/W
      \\minimum> 0.0
  A3, \\field Condenser Type
      \\note Not applicable if Chiller Type is DistrictChilledWater
      \\type choice
      \\key AirCooled
      \\key WaterCooled
      \\key EvaporativelyCooled
      \\default WaterCooled
  A4, \\field Priority
      \\note If Chiller Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:ChilledWaterLoop, then equipment
      \\note operates in Priority order, 1, 2, 3, etc.
  N3, \\field Sizing Factor
      \\note Multiplies the autosized capacity and flow rates
      \\type real
      \\minimum> 0.0
      \\default 1.0
  N4, \\field Minimum Part Load Ratio
      \\type real
      \\note Part load ratio below which the chiller starts cycling on/off to meet the load.
      \\note Must be less than or equal to Maximum Part Load Ratio.
      \\minimum 0.0
      \\default 0.0
  N5, \\field Maximum Part Load Ratio
      \\type real
      \\note Maximum allowable part load ratio. Must be greater than or equal to Minimum Part Load Ratio.
      \\minimum> 0.0
      \\default 1.0
  N6, \\field Optimum Part Load Ratio
      \\type real
      \\note Optimum part load ratio where the chiller is most efficient.
      \\note Must be greater than or equal to the Minimum Part Load Ratio
      \\note and less than or equal to the Maximum Part Load Ratio.
      \\minimum> 0.0
      \\default 1.0
  N7, \\field Minimum Unloading Ratio
      \\type real
      \\note Part load ratio where the chiller can no longer unload and false loading begins.
      \\note Minimum unloading ratio must be greater than or equal to the Minimum Part Load Ratio
      \\note and less than or equal to the Maximum Part Load Ratio.
      \\minimum 0.0
      \\default 0.25
  N8; \\field Leaving Chilled Water Lower Temperature Limit
      \\type real
      \\default 5.0
      \\units C

HVACTemplate:Plant:Chiller:ObjectReference,
      \\min-fields 4
      \\memo This object references a detailed chiller object and adds it to
      \\memo an HVACTemplate:Plant:ChilledWaterLoop. The user must create a complete
      \\memo detailed chiller object with all required curve or performance objects.
  A1, \\field Name
      \\note The name of this object.
      \\required-field
  A2, \\field Chiller Object Type
      \\required-field
      \\type choice
      \\key Chiller:Electric:EIR
      \\key Chiller:Electric:ReformulatedEIR
      \\default Chiller:Electric:EIR
  A3, \\field Chiller Name
      \\note The name of the detailed chiller object.
      \\required-field
      \\type object-list
      \\object-list Chillers
  N1; \\field Priority
      \\note If Chiller Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:ChilledWaterLoop, then equipment
      \\note operates in Priority order, 1, 2, 3, etc.

HVACTemplate:Plant:Tower,
      \\min-fields 9
      \\memo This object adds a cooling tower to an HVACTemplate:Plant:ChilledWaterLoop or MixedWaterLoop.
  A1, \\field Name
      \\required-field
  A2, \\field Tower Type
      \\required-field
      \\type choice
      \\key SingleSpeed
      \\key TwoSpeed
  N1, \\field High Speed Nominal Capacity
      \\note Applicable for tower type SingleSpeed and TwoSpeed
      \\note Nominal tower capacity with entering water at 35C (95F), leaving water at
      \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
      \\note dry-bulb temperature, with the tower fan operating at high speed. Design water
      \\note flow rate assumed to be 5.382E-8 m3/s per watt(3 gpm/ton). Nominal tower capacity
      \\note times (1.25) gives the actual tower heat rejection at these operating conditions.
      \\autosizable
      \\default autosize
      \\units W
      \\minimum> 0.0
  N2, \\field High Speed Fan Power
      \\note Applicable for tower type SingleSpeed and TwoSpeed
      \\autosizable
      \\default autosize
      \\units W
      \\ip-units W
      \\minimum> 0.0
  N3, \\field Low Speed Nominal Capacity
      \\note Applicable only for Tower Type TwoSpeed
      \\note Nominal tower capacity with entering water at 35C (95F), leaving water at
      \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
      \\note dry-bulb temperature, with the tower fan operating at low speed. Design water flow
      \\note rate assumed to be 5.382E-8 m3/s per watt of tower high-speed nominal capacity
      \\note (3 gpm/ton). Nominal tower capacity times (1.25) gives the actual tower heat
      \\note rejection at these operating conditions.
      \\autosizable
      \\default autosize
      \\units W
      \\minimum> 0.0
  N4, \\field Low Speed Fan Power
      \\note Applicable only for Tower Type TwoSpeed
      \\autosizable
      \\default autosize
      \\units W
      \\ip-units W
      \\minimum> 0.0
  N5, \\field Free Convection Capacity
      \\note Applicable for Tower Type SingleSpeed and TwoSpeed
      \\note Tower capacity in free convection regime with entering water at 35C (95F),
      \\note leaving water at 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature
      \\note and 35C (95F) dry-bulb temperature. Design water flow rate assumed to be
      \\note 5.382E-8 m3/s per watt of tower high-speed nominal capacity (3 gpm/ton). Tower
      \\note free convection capacity times (1.25) gives the actual tower heat rejection at
      \\note these operating conditions.
      \\autosizable
      \\default autosize
      \\units W
      \\minimum 0.0
  A3, \\field Priority
      \\note Applicable for all Tower Types
      \\note If Condenser Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:ChilledWaterLoop, then equipment
      \\note operates in Priority order, 1, 2, 3, etc.
  N6; \\field Sizing Factor
      \\note Multiplies the autosized capacity and flow rates
      \\type real
      \\minimum> 0.0
      \\default 1.0

HVACTemplate:Plant:Tower:ObjectReference,
      \\min-fields 4
      \\memo This object references a detailed cooling tower object and adds it to
      \\memo an HVACTemplate:Plant:ChilledWaterLoop or MixedWaterLoop. The user must create a complete
      \\memo detailed cooling tower object with all required curve or performance objects.
  A1, \\field Name
      \\note The name of this object.
      \\required-field
  A2, \\field Cooling Tower Object Type
      \\required-field
      \\type choice
      \\key CoolingTower:SingleSpeed
      \\key CoolingTower:TwoSpeed
      \\key CoolingTower:VariableSpeed
      \\default CoolingTower:SingleSpeed
  A3, \\field Cooling Tower Name
      \\note The name of the detailed cooling tower object.
      \\required-field
      \\type object-list
      \\object-list CoolingTowers
  N1; \\field Priority
      \\note If Condenser Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:ChilledWaterLoop or MixedWaterLoop, then equipment
      \\note operates in Priority order, 1, 2, 3, etc.

HVACTemplate:Plant:HotWaterLoop,
      \\min-fields 14
      \\unique-object
      \\memo Plant loop to serve all HVACTemplate
      \\memo hot water coils and boilers.
  A1, \\field Name
      \\required-field
  A2, \\field Pump Schedule Name
      \\note If blank, always available
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Pump Control Type
      \\type choice
      \\key Intermittent
      \\key Continuous
      \\default Intermittent
  A4, \\field Hot Water Plant Operation Scheme Type
      \\note Default operation type makes all equipment available
      \\note at all times operating in order of Priority specified
      \\note in HVACTemplate:Plant:Boiler objects.
      \\type choice
      \\key Default
      \\key UserDefined
      \\default Default
  A5, \\field Hot Water Plant Equipment Operation Schemes Name
      \\type object-list
      \\object-list PlantOperationSchemes
      \\note Name of a PlantEquipmentOperationSchemes object
      \\note Ignored if Plant Operation Scheme Type = Default
  A6, \\field Hot Water Setpoint Schedule Name
      \\note Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Hot Water Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Setpoint Schedule Name is specified.
      \\default 82.0
      \\units C
  A7, \\field Hot Water Pump Configuration
      \\note VariableFlow - variable flow to boilers and coils, excess bypassed
      \\note ConstantFlow - constant flow to boilers and coils, excess bypassed
      \\type choice
      \\key VariableFlow
      \\key ConstantFlow
      \\default ConstantFlow
  N2, \\field Hot Water Pump Rated Head
      \\units Pa
      \\minimum 0.0
      \\default 179352
      \\note Default head is 60 feet H2O
      \\ip-units ftH2O
  A8, \\field Hot Water Setpoint Reset Type
      \\note Overrides Hot Water Setpoint Schedule Name
      \\type choice
      \\key None
      \\key OutdoorAirTemperatureReset
      \\default None
  N3, \\field Hot Water Setpoint at Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 82.2
  N4, \\field Hot Water Reset Outdoor Dry-Bulb Low
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default -6.7
  N5, \\field Hot Water Setpoint at Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 65.6
  N6, \\field Hot Water Reset Outdoor Dry-Bulb High
      \\note Applicable only for OutdoorAirTemperatureReset control.
      \\units C
      \\default 10.0
  A9, \\field Hot Water Pump Type
      \\note Describes the type of pump configuration used for the hot water loop.
      \\type choice
      \\key SinglePump
      \\key PumpPerBoiler
      \\key TwoHeaderedPumps
      \\key ThreeHeaderedPumps
      \\key FourHeaderedPumps
      \\key FiveHeaderedPumps
      \\default SinglePump
  A10,\\field Supply Side Bypass Pipe
      \\note Determines if a supply side bypass pipe is present in the hot water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A11,\\field Demand Side Bypass Pipe
      \\note Determines if a demand side bypass pipe is present in the hot water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A12,\\field Fluid Type
      \\type choice
      \\key Water
      \\key EthyleneGlycol30
      \\key EthyleneGlycol40
      \\key EthyleneGlycol50
      \\key EthyleneGlycol60
      \\key PropyleneGlycol30
      \\key PropyleneGlycol40
      \\key PropyleneGlycol50
      \\key PropyleneGlycol60
      \\default Water
  N7, \\field Loop Design Delta Temperature
      \\note The temperature difference used in sizing the loop flow rate.
      \\units deltaC
      \\default 11.0
  N8, \\field Maximum Outdoor Dry Bulb Temperature
      \\note The maximum outdoor dry bulb temperature that the hot water loops operate.
      \\note Leave blank for no limit.
      \\units C
  A13;\\field Load Distribution Scheme
      \\type choice
      \\key Optimal
      \\key Sequential
      \\key Uniform
      \\default Sequential


HVACTemplate:Plant:Boiler,
      \\min-fields 7
      \\memo This object adds a boiler to an HVACTemplate:Plant:HotWaterLoop or MixedWaterLoop.
  A1, \\field Name
      \\required-field
  A2, \\field Boiler Type
      \\required-field
      \\type choice
      \\key DistrictHotWater
      \\key HotWaterBoiler
  N1, \\field Capacity
      \\autosizable
      \\default autosize
      \\units W
      \\minimum> 0.0
  N2, \\field Efficiency
      \\note Not applicable  if Boiler Type is DistrictHotWater
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.8
  A3, \\field Fuel Type
      \\note Not applicable  if Boiler Type is DistrictHotWater
      \\type choice
      \\key Electricity
      \\key NaturalGas
      \\key PropaneGas
      \\key FuelOil#1
      \\key FuelOil#2
      \\key Coal
      \\key Diesel
      \\key Gasoline
      \\key OtherFuel1
      \\key OtherFuel2
  A4, \\field Priority
      \\note If Hot Water Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:HotWaterLoop, then equipment
      \\note operates in priority order, 1, 2, 3, etc.
  N3, \\field Sizing Factor
      \\note Multiplies the autosized capacity and flow rates
      \\type real
      \\minimum> 0.0
      \\default 1.0
  N4, \\field Minimum Part Load Ratio
      \\type real
      \\minimum 0.0
      \\default 0.0
  N5, \\field Maximum Part Load Ratio
      \\type real
      \\minimum 0.0
      \\default 1.1
  N6, \\field Optimum Part Load Ratio
      \\type real
      \\minimum 0.0
      \\default 1.0
  N7; \\field Water Outlet Upper Temperature Limit
      \\type real
      \\units C
      \\default 100.0

HVACTemplate:Plant:Boiler:ObjectReference,
      \\min-fields 4
      \\memo This object references a detailed boiler object and adds it to
      \\memo an HVACTemplate:Plant:HotWaterLoop or MixedWaterLoop. The user must create a complete
      \\memo detailed boiler object with all required curve or performance objects.
  A1, \\field Name
      \\note The name of this object.
      \\required-field
  A2, \\field Boiler Object Type
      \\required-field
      \\type choice
      \\key Boiler:HotWater
      \\default Boiler:HotWater
  A3, \\field Boiler Name
      \\note The name of the detailed boiler object.
      \\required-field
      \\type object-list
      \\object-list Boilers
  N1; \\field Priority
      \\note If Hot Water Plant Operation Scheme Type=Default
      \\note in HVACTemplate:Plant:HotWaterLoop or MixedWaterLoop, then equipment
      \\note operates in Priority order, 1, 2, 3, etc.


HVACTemplate:Plant:MixedWaterLoop,
      \\min-fields 11
      \\memo Central plant loop portion of a water source heat pump system.
      \\unique-object
  A1, \\field Name
      \\required-field
  A2, \\field Pump Schedule Name
      \\note If blank, always available
      \\note Applies to both chilled water and condenser loop pumps
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Pump Control Type
      \\note Applies to both chilled water and condenser loop pumps
      \\type choice
      \\key Intermittent
      \\key Continuous
      \\default Intermittent
  A4, \\field Operation Scheme Type
      \\note Default operation type makes all equipment available
      \\note at all times operating in order of Priority specified
      \\note in HVACTemplate:Plant:Boiler and HVACTemplate:Plant:Tower objects.
      \\type choice
      \\key Default
      \\key UserDefined
      \\default Default
  A5, \\field Equipment Operation Schemes Name
      \\type object-list
      \\object-list PlantOperationSchemes
      \\note Name of a PlantEquipmentOperationSchemes object
      \\note Ignored if Plant Operation Scheme Type = Default
  A6, \\field High Temperature Setpoint Schedule Name
      \\note Leave blank if constant setpoint
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field High Temperature Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Setpoint Schedule Name is specified.
      \\default 33.0
      \\units C
  A7, \\field Low Temperature Setpoint Schedule Name
      \\note  Leave blank if constant setpoint
      \\note  May be left blank if not serving any water cooled chillers
      \\type object-list
      \\object-list ScheduleNames
  N2, \\field Low Temperature Design Setpoint
      \\note  Used for sizing and as constant setpoint if no Condenser Water Setpoint Schedule Name is specified.
      \\note  May be left blank if not serving any water cooled chillers
      \\default 20.0
      \\units C
  A8, \\field Water Pump Configuration
      \\note VariableFlow - variable flow to boilers and coils, excess bypassed
      \\note ConstantFlow - constant flow to boilers and coils, excess bypassed
      \\type choice
      \\key VariableFlow
      \\key ConstantFlow
      \\default ConstantFlow
  N3, \\field Water Pump Rated Head
      \\note  May be left blank if not serving any water cooled chillers
      \\units Pa
      \\minimum 0.0
      \\default 179352
      \\note default head is 60 feet H2O
      \\ip-units ftH2O
  A9, \\field Water Pump Type
      \\note Describes the type of pump configuration used for the mixed water loop.
      \\type choice
      \\key SinglePump
      \\key PumpPerTowerOrBoiler
      \\key TwoHeaderedPumps
      \\key ThreeHeaderedPumps
      \\key FourHeaderedPumps
      \\key FiveHeaderedPumps
      \\default SinglePump
  A10,\\field Supply Side Bypass Pipe
      \\note Determines if a supply side bypass pipe is present in the hot water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A11,\\field Demand Side Bypass Pipe
      \\note Determines if a demand side bypass pipe is present in the hot water loop.
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
  A12,\\field Fluid Type
      \\type choice
      \\key Water
      \\key EthyleneGlycol30
      \\key EthyleneGlycol40
      \\key EthyleneGlycol50
      \\key EthyleneGlycol60
      \\key PropyleneGlycol30
      \\key PropyleneGlycol40
      \\key PropyleneGlycol50
      \\key PropyleneGlycol60
      \\default Water
  N4, \\field Loop Design Delta Temperature
      \\note The temperature difference used in sizing the loop flow rate.
      \\units deltaC
      \\default 5.6
  A13;\\field Load Distribution Scheme
      \\type choice
      \\key Optimal
      \\key Sequential
      \\key Uniform
      \\default Sequential

\\group HVAC Design Objects

DesignSpecification:OutdoorAir,
  \\min-fields 1
  \\memo This object is used to describe general outdoor air requirements which
  \\memo are referenced by other objects.
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference DesignSpecificationOutdoorAirNames
  A2, \\field Outdoor Air Method
      \\type choice
      \\key Flow/Person
      \\key Flow/Area
      \\key Flow/Zone
      \\key AirChanges/Hour
      \\key Sum
      \\key Maximum
      \\default Flow/Person
      \\note Flow/Person => Outdoor Air Flow per Person * Occupancy = Design Flow Rate,
      \\note Flow/Area => Outdoor Air Flow per Zone Floor Area * Zone Floor Area = Design Flow Rate,
      \\note Flow/Zone => Outdoor Air Flow per Zone = Design Flow Rate,
      \\note AirChanges/Hour => Outdoor Air Flow Air Changes per Hour * Zone Volume adjusted for m3/s = Design Flow Rate
  N1, \\field Outdoor Air Flow per Person
      \\type real
      \\units m3/s-person
      \\default 0.00944
      \\minimum 0
      \\note 0.00944 m3/s is equivalent to 20 cfm per person
      \\note This input should be used if the field Outdoor Air Method is Flow/Person.
      \\note This input is used if the field Outdoor Air Method is Flow/Person, Sum, or Maximum
  N2, \\field Outdoor Air Flow per Zone Floor Area
      \\units m3/s-m2
      \\default 0.0
      \\minimum 0
      \\note This input should be used if the field Outdoor Air Method is Flow/Area.
      \\note This input is used if the field Outdoor Air Method is Flow/Area, Sum, or Maximum
  N3, \\field Outdoor Air Flow per Zone
      \\type real
      \\units m3/s
      \\default 0.0
      \\minimum 0
      \\note This input should be used if the field Outdoor Air Method is Flow/Zone.
      \\note This input is used if the field Outdoor Air Method is Flow/Zone, Sum, or Maximum
  N4, \\field Outdoor Air Flow Air Changes per Hour
      \\units 1/hr
      \\type real
      \\default 0.0
      \\minimum 0
      \\note This input should be used if the field Outdoor Air Method is AirChanges/Hour.
      \\note This input is used if the field Outdoor Air Method is AirChanges/Hour, Sum, or Maximum
  A3; \\field Outdoor Air Flow Rate Fraction Schedule Name
      \\type object-list
      \\object-list ScheduleNames
      \\note Schedule values are multiplied by the Outdoor Air Flow rate calculated using
      \\note the previous four inputs. Schedule values are limited to 0 to 1.

DesignSpecification:ZoneAirDistribution,
  \\min-fields 1
  \\memo This object is used to describe zone air distribution in terms of air distribution
  \\memo effectiveness and secondary recirculation fraction. It is referenced by Sizing:Zone
  \\memo and Controller:MechanicalVentilation objects
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference DesignSpecificationZoneAirDistributionNames
   N1,  \\field Zone Air Distribution Effectiveness in Cooling Mode
        \\type real
        \\default 1.0
        \\minimum> 0.0
        \\units dimensionless
   N2,  \\field Zone Air Distribution Effectiveness in Heating Mode
        \\type real
        \\default 1.0
        \\minimum> 0.0
        \\units dimensionless
   A2,  \\field Zone Air Distribution Effectiveness Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note optionally used to replace Zone Air Distribution Effectiveness in Cooling and
        \\note Heating Mode
   N3;  \\field Zone Secondary Recirculation Fraction
        \\type real
        \\default 0.0
        \\minimum 0.0
        \\units dimensionless

Sizing:Parameters,
  \\unique-object
      \\memo Specifies global heating and cooling sizing factors/ratios.
      \\memo These ratios are applied at the zone level to all of the zone heating and cooling loads
      \\memo and air flow rates. Then these new loads and air flow rates are used to calculate the
      \\memo system level flow rates and capacities and are used in all component sizing calculations.
      \\memo Specifies the width (in load timesteps) of a moving average window
      \\memo which is used to smooth the peak load across more than one timestep.
  \\min-fields 1
  N1, \\field Heating Sizing Factor
      \\type real
      \\minimum> 0.0
      \\default 1.0
  N2, \\field Cooling Sizing Factor
      \\type real
      \\minimum> 0.0
      \\default 1.0
  N3; \\field Timesteps in Averaging Window
      \\note blank => set the timesteps in averaging window to
      \\note Number of Timesteps per Hour resulting in a 1 hour averaging window
      \\note default is number of timesteps for 1 hour averaging window
      \\type integer
      \\minimum 1

Sizing:Zone,
      \\memo Specifies the data needed to perform a zone design air flow calculation.
      \\memo The calculation is done for every sizing period included in the input. The maximum
      \\memo cooling and heating load and cooling, heating, and ventilation air flows are then saved
      \\memo for system level and zone component design calculations.
  \\min-fields 18
  A1, \\field Zone or ZoneList Name
      \\required-field
      \\type object-list
      \\object-list ZoneAndZoneListNames
  A2, \\field Zone Cooling Design Supply Air Temperature Input Method
      \\required-field
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
  N1, \\field Zone Cooling Design Supply Air Temperature
      \\type real
      \\units C
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
  N2, \\field Zone Cooling Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\note Zone Cooling Design Supply Air Temperature is only used when Zone Cooling Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be subtracted from the zone temperature
      \\note at peak load to calculate the Zone Cooling Design Supply Air Temperature.
  A3, \\field Zone Heating Design Supply Air Temperature Input Method
      \\required-field
      \\type choice
      \\key SupplyAirTemperature
      \\key TemperatureDifference
      \\default SupplyAirTemperature
  N3, \\field Zone Heating Design Supply Air Temperature
      \\type real
      \\units C
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = SupplyAirTemperature
  N4, \\field Zone Heating Design Supply Air Temperature Difference
      \\type real
      \\units deltaC
      \\note Zone Heating Design Supply Air Temperature is only used when Zone Heating Design
      \\note Supply Air Temperature Input Method = TemperatureDifference
      \\note The absolute value of this field will be added to the zone temperature
      \\note at peak load to calculate the Zone Heating Design Supply Air Temperature.
  N5, \\field Zone Cooling Design Supply Air Humidity Ratio
      \\required-field
      \\minimum 0.0
      \\type real
      \\units kgWater/kgDryAir
  N6, \\field Zone Heating Design Supply Air Humidity Ratio
      \\required-field
      \\minimum 0.0
      \\type real
      \\units kgWater/kgDryAir
  A4, \\field Design Specification Outdoor Air Object Name
      \\type object-list
      \\object-list DesignSpecificationOutdoorAirNames
  N7, \\field Zone Heating Sizing Factor
      \\note if blank or zero, global heating sizing factor from Sizing:Parameters is used.
      \\minimum 0
  N8, \\field Zone Cooling Sizing Factor
      \\note if blank or zero, global cooling sizing factor from Sizing:Parameters is used.
      \\minimum 0
  A5, \\field Cooling Design Air Flow Method
      \\type choice
      \\key Flow/Zone
      \\key DesignDay
      \\key DesignDayWithLimit
      \\default DesignDay
  N9, \\field Cooling Design Air Flow Rate
      \\type real
      \\units m3/s
      \\minimum 0
      \\default 0
      \\note This input is used if Cooling Design Air Flow Method is Flow/Zone
      \\note This value will be multiplied by the global or zone sizing factor and
      \\note by zone multipliers.
  N10,\\field Cooling Minimum Air Flow per Zone Floor Area
      \\type real
      \\units m3/s-m2
      \\minimum 0
      \\default .000762
      \\note default is .15 cfm/ft2
      \\note This input is used if Cooling Design Air Flow Method is DesignDayWithLimit
  N11,\\field Cooling Minimum Air Flow
      \\type real
      \\units m3/s
      \\minimum 0
      \\default 0
      \\note This input is used if Cooling Design Air Flow Method is DesignDayWithLimit
  N12,\\field Cooling Minimum Air Flow Fraction
      \\note fraction of the Cooling design Air Flow Rate
      \\type real
      \\minimum 0
      \\default 0
      \\note This input is currently used in sizing the Fan minimum Flow Rate.
      \\note It does not currently affect other component autosizing.
  A6, \\field Heating Design Air Flow Method
      \\type choice
      \\key Flow/Zone
      \\key DesignDay
      \\key DesignDayWithLimit
      \\default DesignDay
  N13,\\field Heating Design Air Flow Rate
      \\type real
      \\units m3/s
      \\minimum 0
      \\default 0
      \\note This input is used if Heating Design Air Flow Method is Flow/Zone.
      \\note This value will be multiplied by the global or zone sizing factor and
      \\note by zone multipliers.
  N14,\\field Heating Maximum Air Flow per Zone Floor Area
      \\type real
      \\units m3/s-m2
      \\minimum 0
      \\default .002032
      \\note default is .40 cfm/ft2
      \\note This field is used to size the heating design flow rate when Heating Design Air Flow Method = Flow/Zone.
      \\note This input is used for autosizing components when Heating Design Air Flow Method = DesignDayWithLimit.
  N15,\\field Heating Maximum Air Flow
      \\type real
      \\units m3/s
      \\minimum 0
      \\default .1415762
      \\note default is 300 cfm
      \\note This input is used for autosizing components when Heating Design Air Flow Method = DesignDayWithLimit.
  N16,\\field Heating Maximum Air Flow Fraction
      \\note fraction of the Heating Design Air Flow Rate
      \\type real
      \\minimum 0
      \\default 0.3
      \\note This input is used for autosizing components when Heating Design Air Flow Method = DesignDayWithLimit.
  A7; \\field Design Specification Zone Air Distribution Object Name
      \\type object-list
      \\object-list DesignSpecificationZoneAirDistributionNames

Sizing:System,
      \\memo Specifies the input needed to perform sizing calculations for a central forced air
      \\memo system design air flow, heating capacity, and cooling capacity.
  \\min-fields 20
  A1, \\field AirLoop Name
      \\required-field
      \\type object-list
      \\object-list AirPrimaryLoops
  A2, \\field Type of Load to Size On
      \\note Specifies the basis for sizing the system supply air flow rate
      \\note Sensible and VentilationRequirement are the only available options
      \\note Sensible uses the zone design air flow rates
      \\note VentilationRequirement uses the system ventilation requirement
      \\required-field
      \\type choice
      \\key Sensible
      \\key VentilationRequirement
      !\\key Latent
      !\\key Total
      \\default Sensible
  N1, \\field Design Outdoor Air Flow Rate
      \\type real
      \\default autosize
      \\minimum 0.0
      \\units m3/s
      \\autosizable
  N2, \\field Minimum System Air Flow Ratio
      \\required-field
      \\type real
      \\minimum 0.0
      \\maximum 1.0
  N3, \\field Preheat Design Temperature
      \\required-field
      \\type real
      \\units C
  N4, \\field Preheat Design Humidity Ratio
      \\required-field
      \\type real
      \\units kgWater/kgDryAir
  N5, \\field Precool Design Temperature
      \\required-field
      \\type real
      \\units C
  N6, \\field Precool Design Humidity Ratio
      \\required-field
      \\type real
      \\units kgWater/kgDryAir
  N7, \\field Central Cooling Design Supply Air Temperature
      \\required-field
      \\type real
      \\units C
  N8, \\field Central Heating Design Supply Air Temperature
      \\required-field
      \\type real
      \\units C
  A3, \\field Sizing Option
      \\type choice
      \\key Coincident
      \\key NonCoincident
      \\default NonCoincident
  A4, \\field 100% Outdoor Air in Cooling
      \\type choice
      \\key Yes
      \\key No
      \\default No
  A5, \\field 100% Outdoor Air in Heating
      \\type choice
      \\key Yes
      \\key No
      \\default No
  N9, \\field Central Cooling Design Supply Air Humidity Ratio
      \\required-field
      \\type real
      \\default 0.008
      \\units kgWater/kgDryAir
  N10, \\field Central Heating Design Supply Air Humidity Ratio
      \\required-field
      \\type real
      \\default 0.008
      \\units kgWater/kgDryAir
  A6, \\field Cooling Design Air Flow Method
      \\type choice
      \\key Flow/System
      \\key DesignDay
      \\default DesignDay
  N11, \\field Cooling Design Air Flow Rate
      \\note This input is used if Cooling Design Air Flow Method is Flow/System
      \\note This value will *not* be multiplied by any sizing factor or by zone multipliers.
      \\note If using zone multipliers, this value must be large enough to serve the multiplied zones.
      \\type real
      \\units m3/s
      \\minimum 0
      \\default 0
  A7, \\field Heating Design Air Flow Method
      \\type choice
      \\key Flow/System
      \\key DesignDay
      \\default DesignDay
  N12,\\field Heating Design Air Flow Rate
      \\note This input is used if Heating Design Air Flow Method is Flow/System
      \\note This value will *not* be multiplied by any sizing factor or by zone multipliers.
      \\note If using zone multipliers, this value must be large enough to serve the multiplied zones.
      \\type real
      \\units m3/s
      \\minimum 0
      \\default 0
  A8, \\field System Outdoor Air Method
      \\type choice
      \\key ZoneSum
      \\key VentilationRateProcedure
      \\default ZoneSum
  N13;\\field Zone Maximum Outdoor Air Fraction
      \\type real
      \\default 1.0
      \\minimum> 0.0
      \\units dimensionless

Sizing:Plant,
      \\memo Specifies the input needed to autosize plant loop flow rates and equipment capacities.
      \\memo This information is initially used by components that use water for heating or cooling
      \\memo such as hot or chilled water coils to calculate their maximum water flow rates. These
      \\memo flow rates are then summed for use in calculating the Plant Loop flow rates.
  \\min-fields 4
  A1, \\field Plant or Condenser Loop Name
      \\note Enter the name of a PlantLoop or a CondenserLoop object
      \\required-field
      \\type object-list
      \\object-list PlantLoops
  A2, \\field Loop Type
      \\required-field
      \\type choice
      \\key Heating
      \\key Cooling
      \\key Condenser
      \\key Steam
  N1, \\field Design Loop Exit Temperature
      \\required-field
      \\type real
      \\units C
  N2; \\field Loop Design Temperature Difference
      \\required-field
      \\type real
      \\units deltaC
      \\minimum> 0.0

OutputControl:Sizing:Style,
       \\memo default style for the Sizing output files is comma -- this works well for
       \\memo importing into spreadsheet programs such as Excel(tm) but not so well for word
       \\memo processing progams -- there tab may be a better choice.  fixed puts spaces between
       \\memo the "columns"
       \\unique-object
   A1; \\field Column Separator
       \\required-field
       \\type choice
       \\key Comma
       \\key Tab
       \\key Fixed

\\Group Zone HVAC Controls and Thermostats

ZoneControl:Humidistat,
       \\memo Specifies zone relative humidity setpoint schedules for humidifying and dehumidifying.
  \\min-fields 3
  A1 , \\field Name
       \\required-field
  A2 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Humidifying Relative Humidity Setpoint Schedule Name
       \\required-field
       \\note hourly schedule values should be in Relative Humidity (percent)
       \\type object-list
       \\object-list ScheduleNames
  A4 ; \\field Dehumidifying Relative Humidity Setpoint Schedule Name
       \\note hourly schedule values should be in Relative Humidity (percent)
       \\type object-list
       \\object-list ScheduleNames

ZoneControl:Thermostat,
   \\memo Define the Thermostat settings for a zone or list of zones.
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
  A1 , \\field Name
       \\required-field
       \\reference ZoneControlThermostaticNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Control Type Schedule Name
       \\note This schedule contains appropriate control types for thermostat.
       \\note Control types are integers: 0 - Uncontrolled (floating, no thermostat), 1 = ThermostatSetpoint:SingleHeating,
       \\note 2 = ThermostatSetpoint:SingleCooling, 3 = ThermostatSetpoint:SingleHeatingOrCooling, 4 = ThermostatSetpoint:DualSetpoint
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Control 1 Object Type
       \\required-field
       \\type choice
       \\key ThermostatSetpoint:SingleHeating
       \\key ThermostatSetpoint:SingleCooling
       \\key ThermostatSetpoint:SingleHeatingOrCooling
       \\key ThermostatSetpoint:DualSetpoint
  A5 , \\field Control 1 Name
       \\note Control names are names of individual control objects (e.g. ThermostatSetpoint:SingleHeating)
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\required-field
       \\type object-list
       \\object-list ControlTypeNames
  A6 , \\field Control 2 Object Type
       \\type choice
       \\key ThermostatSetpoint:SingleHeating
       \\key ThermostatSetpoint:SingleCooling
       \\key ThermostatSetpoint:SingleHeatingOrCooling
       \\key ThermostatSetpoint:DualSetpoint
  A7 , \\field Control 2 Name
       \\note Control names are names of individual control objects (e.g. ThermostatSetpoint:SingleHeating)
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ControlTypeNames
  A8 , \\field Control 3 Object Type
       \\type choice
       \\key ThermostatSetpoint:SingleHeating
       \\key ThermostatSetpoint:SingleCooling
       \\key ThermostatSetpoint:SingleHeatingOrCooling
       \\key ThermostatSetpoint:DualSetpoint
  A9 , \\field Control 3 Name
       \\note Control names are names of individual control objects (e.g. ThermostatSetpoint:SingleHeating)
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ControlTypeNames
  A10, \\field Control 4 Object Type
       \\type choice
       \\key ThermostatSetpoint:SingleHeating
       \\key ThermostatSetpoint:SingleCooling
       \\key ThermostatSetpoint:SingleHeatingOrCooling
       \\key ThermostatSetpoint:DualSetpoint
  A11; \\field Control 4 Name
       \\note Control names are names of individual control objects (e.g. ThermostatSetpoint:SingleHeating)
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ControlTypeNames

ZoneControl:Thermostat:OperativeTemperature,
       \\memo This object can be used with the ZoneList option on a thermostat or with one
       \\memo of the zones on that list (but you won't be able to use the object list to
       \\memo pick only one of those zones.  Thermostat names are <Zone Name> <global Thermostat name> internally.
  A1,  \\field Thermostat Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note This object modifies a ZoneControl:Thermostat object to add a
       \\note radiative fraction.
       \\required-field
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A2,  \\field Radiative Fraction Input Mode
       \\required-field
       \\type choice
       \\key Constant
       \\key Scheduled
  N1,  \\field Fixed Radiative Fraction
       \\minimum 0.0
       \\maximum< 0.9
  A3;  \\field Radiative Fraction Schedule Name
       \\note Schedule values of 0.0 indicate no operative temperature control
       \\type object-list
       \\object-list ScheduleNames

ZoneControl:Thermostat:ThermalComfort,
   \\memo If you use a ZoneList in the Zone or ZoneList name field then this definition applies
   \\memo to all the zones in the ZoneList.
       \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\reference ZoneControlThermalComfortNames
  A2 , \\field Zone or ZoneList Name
       \\required-field
       \\type object-list
       \\object-list ZoneAndZoneListNames
  A3 , \\field Averaging Method
       \\note The method used to calculate thermal comfort dry-bulb temperature setpoint
       \\note for multiple people objects in a zone
       \\type choice
       \\key SpecificObject
       \\key ObjectAverage
       \\key PeopleAverage
       \\default PeopleAverage
  A4 , \\field Specific People Name
       \\note Used only when Averaging Method = SpecificObject in the previous field.
       \\type object-list
       \\object-list PeopleNames
  N1 , \\field Minimum Dry-Bulb Temperature Setpoint
       \\required-field
       \\type real
       \\units C
       \\minimum 0
       \\maximum 50
       \\default 0
  N2 , \\field Maximum Dry-Bulb Temperature Setpoint
       \\required-field
       \\type real
       \\units C
       \\minimum 0
       \\maximum 50
       \\default 50
  A5 , \\field Thermal Comfort Control Type Schedule Name
       \\note The Thermal Comfort Control Type Schedule contains values that are appropriate control types.
       \\note Thermal Comfort Control types are integers: 0 - Uncontrolled (floating),
       \\note 1 = ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating
       \\note 2 = ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling
       \\note 3 = ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling
       \\note 4 = ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A6 , \\field Thermal Comfort Control 1 Object Type
       \\required-field
       \\type choice
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint
  A7 , \\field Thermal Comfort Control 1 Name
       \\note Control type names are names for individual control type objects.
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\required-field
       \\type object-list
       \\object-list ThermalComfortControlTypeNames
  A8 , \\field Thermal Comfort Control 2 Object Type
       \\type choice
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint
  A9 , \\field Thermal Comfort Control 2 Name
       \\note Control Type names are names for individual control type objects.
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ThermalComfortControlTypeNames
  A10, \\field Thermal Comfort Control 3 Object Type
       \\type choice
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint
  A11, \\field Thermal Comfort Control 3 Name
       \\note Control type names are names for individual control type objects.
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ThermalComfortControlTypeNames
  A12, \\field Thermal Comfort Control 4 Object Type
       \\type choice
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling
       \\key ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint
  A13; \\field Thermal Comfort Control 4 Name
       \\note Control type names are names for individual control type objects.
       \\note Schedule values in these objects list actual setpoint temperatures for the control types
       \\type object-list
       \\object-list ThermalComfortControlTypeNames

ZoneControl:Thermostat:TemperatureAndHumidity,
       \\min-fields 2
       \\memo This object modifies a ZoneControl:Thermostat object to effect temperature control based on
       \\memo zone air humidity conditions.
  A1,  \\field Thermostat Name
       \\required-field
       \\type object-list
       \\object-list ZoneControlThermostaticNames
       \\note Enter the name of a ZoneControl:Thermostat object whose operation is to be modified to
       \\note effect temperature control based on zone air humidity conditions. If the ZoneControl:
       \\note Thermostat object references a ZoneList, simply enter the name of the ZoneControl:Thermostat
       \\note object and this temperature and humidity thermostat control will be applied to all zones
       \\note in the ZoneList. If the ZoneControl:Thermostat object references a ZoneList but it is
       \\note desired that only a single zone within the ZoneList be controlled based on temperature and
       \\note humidity control, then the name to be put here is <Zone Name> <Thermostat Name> where the
       \\note Thermostat Name is the the name of the ZoneControl:Thermostat object.
  A2 , \\field Dehumidifying Relative Humidity Setpoint Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values should be in Relative Humidity (percent)
  A3,  \\field Dehumidification Control Type
       \\type choice
       \\key Overcool
       \\key None
       \\default Overcool
  A4,  \\field Overcool Range Input Method
       \\type choice
       \\key Constant
       \\key Scheduled
       \\default Constant
  N1,  \\field Overcool Constant Range
       \\type real
       \\units deltaC
       \\minimum 0.0
       \\maximum 3.0
       \\default 1.7
       \\note Maximum Overcool temperature range for cooling setpoint reduction.
       \\note Used with Dehumidification Control Type = Overcool.
       \\note A value of 0.0 indicates no zone temperature overcooling will be provided to
       \\note gain additional dehumidification.
  A5,  \\field Overcool Range Schedule Name
       \\note Schedule values of 0.0 indicates no zone temperature overcooling will be
       \\note provided to gain additional dehumidification.
       \\note Schedule values should be >= 0 and <= 3 (deltaC).
       \\type object-list
       \\object-list ScheduleNames
  N2;  \\field Overcool Control Ratio
       \\type real
       \\units percent/K
       \\minimum 0.0
       \\default 3.6
       \\note The value of this input field is used to adjust the cooling setpoint temperature
       \\note (established by the associated ZoneControl:Thermostat object) downward based on the
       \\note difference between the zone air relative humidity level and the dehumidifying
       \\note relative humidity setpoint.

ThermostatSetpoint:SingleHeating,
       \\memo Used for a heating only thermostat. The setpoint can be scheduled and varied throughout
       \\memo the simulation but only heating is allowed with this control type.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControlTypeNames
  A2 ; \\field Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:SingleCooling,
       \\memo Used for a cooling only thermostat. The setpoint can be scheduled and varied throughout
       \\memo the simulation but only cooling is allowed.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControlTypeNames
  A2 ; \\field Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:SingleHeatingOrCooling,
       \\memo Used for a heating and cooling thermostat with a single setpoint. The setpoint can be
       \\memo scheduled and varied throughout the simulation for both heating and cooling.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControlTypeNames
  A2 ; \\field Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:DualSetpoint,
       \\memo Used for a heating and cooling thermostat with dual setpoints. The setpoints can be
       \\memo scheduled and varied throughout the simulation for both heating and cooling.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControlTypeNames
  A2 , \\field Heating Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A3 ; \\field Cooling Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:ThermalComfort:Fanger:SingleHeating,
       \\memo Used for heating only thermal comfort control. The PMV setpoint can be scheduled and
       \\memo varied throughout the simulation but only heating is allowed with this control type.
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ThermalComfortControlTypeNames
  A2 ; \\field Fanger Thermal Comfort Schedule Name
       \\note Schedule values should be Predicted Mean Vote (PMV)
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:ThermalComfort:Fanger:SingleCooling,
       \\memo Used for cooling only thermal comfort control. The PMV setpoint can be scheduled and
       \\memo varied throughout the simulation but only cooling is allowed with this control type.
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ThermalComfortControlTypeNames
  A2 ; \\field Fanger Thermal Comfort Schedule Name
       \\note Schedule values should be Predicted Mean Vote (PMV)
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:ThermalComfort:Fanger:SingleHeatingOrCooling,
       \\memo Used for heating and cooling thermal comfort control with a single setpoint. The PMV
       \\memo setpoint can be scheduled and varied throughout the simulation for both heating and
       \\memo cooling.
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ThermalComfortControlTypeNames
  A2 ; \\field Fanger Thermal Comfort Schedule Name
       \\note Schedule values should be Predicted Mean Vote (PMV)
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint,
       \\memo Used for heating and cooling thermal comfort control with dual setpoints. The PMV
       \\memo setpoints can be scheduled and varied throughout the simulation for both heating and
       \\memo cooling.
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ThermalComfortControlTypeNames
  A2 , \\field Fanger Thermal Comfort Heating Schedule Name
       \\note Schedule values should be Predicted Mean Vote (PMV)
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A3 ; \\field Fanger Thermal Comfort Cooling Schedule Name
       \\note Schedule values should be Predicted Mean Vote (PMV)
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

ZoneControl:ContaminantController,
       \\memo Used to control a zone to a specified indoor level of CO2 or generic contaminants, or
       \\memo to specify minimum CO2 concentration schedule name for a zone.
       \\min-fields 4
  A1 , \\field Name
       \\required-field
  A2 , \\field Controlled Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A3 , \\field Carbon Dioxide Control Availability Schedule Name
       \\note Availability schedule name for CO2 controller. Schedule value > 0 means the CO2
       \\note controller is enabled. If this field is blank, then CO2  controller is always enabled.
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Carbon Dioxide Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values should be carbon dioxide concentration in parts per million (ppm)
  A5 , \\field Minimum Carbon Dioxide Concentration Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values should be carbon dioxide concentration in parts per
       \\note million (ppm)
       \\note This field is used when the field System Outdoor Air Method =
       \\note ProportionalControl in Controller:MechanicalVentilation
  A6 , \\field Generic Contaminant Control Availability Schedule Name
       \\note Availability schedule name for generic contaminant controller. Schedule value > 0 means
       \\note the generic contaminant controller is enabled. If this field is blank, then generic
       \\note contaminant controller is always enabled.
       \\type object-list
       \\object-list ScheduleNames
  A7 ; \\field Generic Contaminant Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values should be generic contaminant concentration in parts per
       \\note million (ppm)
       \\note This field is used when the field System Outdoor Air Method =
       \\note IndoorAirQualityProcedureGenericContaminant in Controller:MechanicalVentilation

\\group Zone HVAC Forced Air Units

ZoneHVAC:IdealLoadsAirSystem,
       \\memo Ideal system used to calculate loads without modeling a full HVAC system. All that is
       \\memo required for the ideal system are zone controls, zone equipment configurations, and
       \\memo the ideal loads system component. This component can be thought of as an ideal unit
       \\memo that mixes zone air with the specified amount of outdoor air and then adds or removes
       \\memo heat and moisture at 100% efficiency in order to meet the specified controls. Energy
       \\memo use is reported as DistrictHeating and DistrictCooling.
       \\min-fields 26
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Zone Supply Air Node Name
       \\note Must match a zone air inlet node name.
       \\required-field
       \\type node
  A4 , \\field Zone Exhaust Air Node Name
       \\note Should match a zone air exhaust node name.
       \\note This field is optional, but is required if this
       \\note this object is used with other forced air equipment.
       \\type node
  N1 , \\field Maximum Heating Supply Air Temperature
       \\units C
       \\minimum> 0
       \\maximum< 100
       \\default 50
  N2 , \\field Minimum Cooling Supply Air Temperature
       \\units C
       \\minimum> -100
       \\maximum< 50
       \\default 13
  N3 , \\field Maximum Heating Supply Air Humidity Ratio
       \\units kgWater/kgDryAir
       \\minimum> 0
       \\default 0.0156
  N4 , \\field Minimum Cooling Supply Air Humidity Ratio
       \\units kgWater/kgDryAir
       \\minimum> 0
       \\default 0.0077
  A5 , \\field Heating Limit
       \\type choice
       \\key NoLimit
       \\key LimitFlowRate
       \\key LimitCapacity
       \\key LimitFlowRateAndCapacity
       \\default NoLimit
  N5 , \\field Maximum Heating Air Flow Rate
       \\note This field is ignored if Heating Limit = NoLimit
       \\note If this field is blank, there is no limit.
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N6 , \\field Maximum Sensible Heating Capacity
       \\note This field is ignored if Heating Limit = NoLimit
       \\note If this field is blank, there is no limit.
       \\units W
       \\minimum 0.0
       \\autosizable
  A6 , \\field Cooling Limit
       \\type choice
       \\key NoLimit
       \\key LimitFlowRate
       \\key LimitCapacity
       \\key LimitFlowRateAndCapacity
       \\default NoLimit
  N7 , \\field Maximum Cooling Air Flow Rate
       \\note This field is ignored if Cooling Limit = NoLimit
       \\note This field is required if Outdoor Air Control Type
       \\note is TemperatureEconomizer or EnthalpyEconomizer
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N8 , \\field Maximum Total Cooling Capacity
       \\note This field is ignored if Cooling Limit = NoLimit
       \\units W
       \\minimum 0.0
       \\autosizable
  A7 , \\field Heating Availability Schedule Name
       \\note If blank, heating is always available.
       \\type object-list
       \\object-list ScheduleNames
  A8 , \\field Cooling Availability Schedule Name
       \\note If blank, cooling is always available.
       \\type object-list
       \\object-list ScheduleNames
  A9 , \\field Dehumidification Control Type
       \\note ConstantSensibleHeatRatio means that the ideal loads system
       \\note will be controlled to meet the sensible cooling load, and the
       \\note latent cooling rate will be computed using a constant
       \\note sensible heat ratio (SHR)
       \\note Humidistat means that there is a ZoneControl:Humidistat for this
       \\note zone and the ideal loads system will attempt to satisfy the humidistat.
       \\note None means that there is no dehumidification.
       \\note ConstantSupplyHumidityRatio means that during cooling the supply air
       \\note will always be at the Minimum Cooling Supply Humidity Ratio.
       \\type choice
       \\key ConstantSensibleHeatRatio
       \\key Humidistat
       \\key None
       \\key ConstantSupplyHumidityRatio
       \\default ConstantSensibleHeatRatio
  N9 , \\field Cooling Sensible Heat Ratio
       \\note This field is applicable only when Dehumidification Control Type is ConstantSensibleHeatRatio
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\default 0.7
  A10, \\field Humidification Control Type
       \\note None means that there is no humidification.
       \\note Humidistat means that there is a ZoneControl:Humidistat for this
       \\note zone and the ideal loads system will attempt to satisfy the humidistat.
       \\note ConstantSupplyHumidityRatio means that during heating the supply air
       \\note will always be at the Maximum Heating Supply Humidity Ratio.
       \\type choice
       \\key None
       \\key Humidistat
       \\key ConstantSupplyHumidityRatio
       \\default None
  A11, \\field Design Specification Outdoor Air Object Name
       \\type object-list
       \\object-list DesignSpecificationOutdoorAirNames
       \\note When the name of a DesignSpecification:OutdoorAir object is entered, the minimum
       \\note outdoor air flow rate will be computed using these specifications. The outdoor air
       \\note flow rate will also be affected by the next two fields.
       \\note If this field is blank, there will be no outdoor air and the remaining fields will
       \\note be ignored.
  A12, \\field Outdoor Air Inlet Node Name
       \\type node
       \\note This field is required if the system provides outdoor air
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
  A13, \\field Demand Controlled Ventilation Type
       \\note This field controls how the minimum outdoor air flow rate is calculated.
       \\note None means that design occupancy will be uased to compute the minimum outdoor air flow rate
       \\note OccupancySchedule means that current occupancy level will be used.
       \\note CO2Setpoint means that the design occupancy will be used to compute the minimum outdoor air flow
       \\note reate and the outdoor air flow rate may be increased if necessary to maintain the indoor air carbon
       \\note dioxide setpoint defined in a ZoneControl:ContaminantController object.
       \\type choice
       \\key None
       \\key OccupancySchedule
       \\key CO2Setpoint
       \\default None
  A14, \\field Outdoor Air Economizer Type
       \\note DifferentialDryBulb and DifferentialEnthalpy will increase the outdoor air flow rate
       \\note when there is a cooling load and the outdoor air temperature or enthalpy
       \\note is below the zone exhaust air temperature or enthalpy.
       \\type choice
       \\key NoEconomizer
       \\key DifferentialDryBulb
       \\key DifferentialEnthalpy
       \\default NoEconomizer
  A15, \\field Heat Recovery Type
       \\type choice
       \\key None
       \\key Sensible
       \\key Enthalpy
       \\default None
  N10, \\field Sensible Heat Recovery Effectiveness
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.70
  N11; \\field Latent Heat Recovery Effectiveness
       \\note Applicable only if Heat Recovery Type is Enthalpy.
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.65

ZoneHVAC:FourPipeFanCoil,
        \\memo Four pipe fan coil system. Forced-convection hydronic heating-cooling unit with
        \\memo supply fan, hot water heating coil, chilled water cooling coil, and fixed-position
        \\memo outdoor air mixer.
        \\min-fields 24
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Capacity Control Method
        \\required-field
        \\type choice
        \\key ConstantFanVariableFlow
        \\key CyclingFan
        \\key VariableFanVariableFlow
        \\key VariableFanConstantFlow
   N1 , \\field Maximum Supply Air Flow Rate
        \\required-field
        \\autosizable
        \\units m3/s
   N2 , \\field Low Speed Supply Air Flow Ratio
        \\type real
        \\minimum> 0.0
        \\default 0.33
   N3 , \\field Medium Speed Supply Air Flow Ratio
        \\note Medium Speed Supply Air Flow Ratio should be greater
        \\note than Low Speed Supply Air Flow Ratio
        \\type real
        \\minimum> 0.0
        \\default 0.66
   N4 , \\field Maximum Outdoor Air Flow Rate
        \\required-field
        \\autosizable
        \\units m3/s
   A4 , \\field Outdoor Air Schedule Name
        \\note Value of schedule multiplies maximum outdoor air flow rate
        \\type object-list
        \\object-list ScheduleNames
   A5 , \\field Air Inlet Node Name
        \\required-field
        \\type node
   A6 , \\field Air Outlet Node Name
        \\required-field
        \\type node
   A7,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one type OutdoorAir:Mixer object is available.
   A8 , \\field Outdoor Air Mixer Name
        \\required-field
        \\type object-list
        \\object-list OutdoorAirMixers
   A9 , \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\key Fan:VariableVolume
        \\note Fan type must be according to capacity control method (see I/O)
        \\note Fan:ConstantVolume only works when continuous fan operation is used the entire
        \\note simulation (all supply air fan operating mode schedule values are greater than 0).
        \\note If any fan operating mode schedule values are 0 a Fan:OnOff object must be used.
        \\note For ConstantFanVariableFlow a Fan:OnOff or Fan:ConstantVolume is valid.
        \\note For CyclingFan, a Fan:OnOff is valid.
        \\note For VariableFanVariableFlow or VariableFanConstantFlow a Fan:VariableVolume is valid.
        \\note The fan's inlet node should be the same as the outdoor air mixer's mixed air node.
   A10, \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOffandVAV
   A11, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:Water
        \\key Coil:Cooling:Water:DetailedGeometry
        \\key CoilSystem:Cooling:Water:HeatExchangerAssisted
   A12, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsWater
   N5 , \\field Maximum Cold Water Flow Rate
        \\required-field
        \\autosizable
        \\units m3/s
        \\ip-units gal/min
   N6 , \\field Minimum Cold Water Flow Rate
        \\default 0.0
        \\units m3/s
        \\ip-units gal/min
   N7 , \\field Cooling Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A13, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Water
   A14, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsWater
   N8 , \\field Maximum Hot Water Flow Rate
        \\required-field
        \\autosizable
        \\units m3/s
        \\ip-units gal/min
   N9 , \\field Minimum Hot Water Flow Rate
        \\default 0.0
        \\units m3/s
        \\ip-units gal/min
   N10, \\field Heating Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A15; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:WindowAirConditioner,
        \\memo Window air conditioner. Forced-convection cooling-only unit with supply fan, direct
        \\memo expansion (DX) cooling coil, and fixed-position outdoor air mixer.
        \\min-fields 15
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   N1 , \\field Maximum Supply Air Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
   N2 , \\field Maximum Outdoor Air Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
   A3 , \\field Air Inlet Node Name
        \\required-field
        \\type node
   A4 , \\field Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one OutdoorAir:Mixer object type is available.
   A6 , \\field Outdoor Air Mixer Name
        \\required-field
        \\type object-list
        \\object-list OutdoorAirMixers
   A7 , \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works when continuous fan operation is used the entire
        \\note simulation (all supply air fan operating mode schedule values are greater than 0).
        \\note If any fan operating mode schedule values are 0 a Fan:OnOff object must be used.
   A8 , \\field Supply Air Fan Name
        \\required-field
        \\note Fan type Fan:ConstantVolume is used with continuous fan
        \\note and fan type Fan:OnOff is used with cycling Fan.
        \\type object-list
        \\object-list FansCVandOnOff
   A9,  \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
   A10, \\field DX Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
   A11, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that controls fan operation. Schedule Name values of 0 denote
        \\note cycling fan operation (fan cycles with cooling coil). Schedule values greater
        \\note than 0 denote constant fan operation (fan runs continually regardless of coil operation).
        \\note The fan operating mode defaults to cycling fan operation if this field is left blank.
   A12, \\field Fan Placement
        \\required-field
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
   N3 , \\field Cooling Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A13; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:PackagedTerminalAirConditioner,
        \\memo Packaged terminal air conditioner (PTAC).  Forced-convection heating-cooling unit
        \\memo with supply fan, direct expansion (DX) cooling coil, heating coil (gas, electric, hot
        \\memo water, or steam) and fixed-position outdoor air mixer.
        \\min-fields 18
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note Unique name for this packaged terminal air conditioner object.
   A2,  \\field Availability Schedule Name
        \\required-field
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values of 0 denote the unit is off.
   A3 , \\field Air Inlet Node Name
        \\required-field
        \\type node
        \\note Air inlet node for the PTAC must be a zone air exhaust Node.
   A4 , \\field Air Outlet Node Name
        \\required-field
        \\type node
        \\note Air outlet node for the PTAC must be a zone air inlet node.
   A5,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one OutdoorAir:Mixer object type is available.
   A6 , \\field Outdoor Air Mixer Name
        \\required-field
        \\type object-list
        \\object-list OutdoorAirMixers
        \\note Needs to match the name of the PTAC outdoor air mixer object.
   N1 , \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N2 , \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N3 , \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to fan size.
        \\note Only used when supply air fan operating mode schedule values specify continuous fan
        \\note (schedule values greater than 0 specify continuous fan operation).
        \\note This air flow rate is used when no heating or cooling is required and the cooling or
        \\note heating coil is off. If this field is left blank or zero, the supply air flow rate
        \\note from the previous on cycle (either cooling or heating) is used.
   N4 , \\field Outdoor Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during cooling operation.
   N5 , \\field Outdoor Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during heating operation.
   N6 , \\field Outdoor Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Only used when supply air fan operating mode schedule values specify continuous fan
        \\note (schedule values greater than 0 specify continuous fan operation).
        \\note This air flow rate is used when no heating or cooling is required and the cooling or
        \\note heating coil is off. If this field is left blank or zero, the outdoor air flow rate
        \\note from the previous on cycle (either cooling or heating) is used.
   A7 , \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works when continuous fan operation is used the entire
        \\note simulation (all supply air fan operating mode schedule values are greater than 0).
        \\note If any fan operating mode schedule values are 0 a Fan:OnOff object must be used.
   A8 , \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
        \\note Needs to match in the fan object.
   A9 , \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note Select the type of heating coil.
   A10 , \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note Needs to match in the heating coil object.
   A11, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key Coil:Cooling:DX:VariableSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\note Select the type of Cooling Coil.
        \\note Only works with Coil:Cooling:DX:SingleSpeed or
        \\note CoilSystem:Cooling:DX:HeatExchangerAssisted or
        \\note Coil:Cooling:DX:VariableSpeed.
   A12, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
        \\object-list CoolingCoilsDXVariableSpeed
        \\note Needs to match a DX cooling coil object.
   A13, \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default DrawThrough
        \\note Select fan placement as either blow through or draw through.
   A14, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that controls fan operation. Schedule Name values of 0 denote
        \\note cycling fan operation (fan cycles with cooling or heating coil). Schedule Name values greater
        \\note than 0 denote constant fan operation (fan runs continually regardless of coil operation).
   A15; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:PackagedTerminalHeatPump,
        \\memo Packaged terminal heat pump (PTHP). Forced-convection heating-cooling unit with
        \\memo supply fan, direct expansion (DX) cooling coil, DX heating coil (air-to-air heat
        \\memo pump), supplemental heating coil (gas, electric, hot water, or steam), and
        \\memo fixed-position outdoor air mixer.
        \\min-fields 27
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note Unique name for this packaged terminal heat pump object.
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values of 0 denote the unit is off.
   A3 , \\field Air Inlet Node Name
        \\required-field
        \\type node
        \\note Air inlet node for the PTHP must be a zone air exhaust node.
   A4 , \\field Air Outlet Node Name
        \\required-field
        \\type node
        \\note Air outlet node for the PTHP must be a zone air inlet node.
   A5,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one OutdoorAir:Mixer object type is available.
   A6 , \\field Outdoor Air Mixer Name
        \\required-field
        \\type object-list
        \\object-list OutdoorAirMixers
        \\note Needs to match name of outdoor air mixer object.
   N1 , \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N2 , \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N3 , \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to fan size.
        \\note Only used when heat pump fan operating mode is continuous. This air flow rate
        \\note is used when no heating or cooling is required and the DX coil compressor is off.
        \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
        \\note (either cooling or heating) is used.
   N4 , \\field Outdoor Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during cooling operation.
   N5 , \\field Outdoor Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during heating operation.
   N6 , \\field Outdoor Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Only used when heat pump Fan operating mode is continuous. This air flow rate
        \\note is used when no heating or cooling is required and the DX coil compressor is off.
        \\note If this field is left blank or zero, the outdoor air flow rate from the previous on cycle
        \\note (either cooling or heating) is used.
   A7 , \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with fan operating mode is continuous.
   A8 , \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
        \\note Needs to match a fan object.
   A9 , \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:DX:SingleSpeed
        \\key Coil:Heating:DX:VariableSpeed
        \\note Only works with Coil:Heating:DX:SingleSpeed or
        \\note Coil:Heating:DX:VariableSpeed.
   A10 , \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsDXSingleSpeed
        \\object-list HeatingCoilsDXVariableSpeed
        \\note Needs to match in the DX Heating Coil object.
   N7 , \\field Heating Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
        \\units dimensionless
        \\note Defines Heating convergence tolerence as a fraction of Heating load to be met.
   N8 , \\field Minimum Outdoor Dry-Bulb Temperature for Compressor Operation
        \\type real
        \\minimum -20.0
        \\default -8.0
        \\units C
        \\note Needs to match the corresponding minimum outdoor temperature defined
        \\note in the DX Heating Coil object.
   A11, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key Coil:Cooling:DX:VariableSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\note Only works with Coil:Cooling:DX:SingleSpeed or
        \\note CoilSystem:Cooling:DX:HeatExchangerAssisted or
        \\note Coil:Cooling:DX:VariableSpeed.
   A12, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
        \\object-list CoolingCoilsDXVariableSpeed
        \\note Needs to match in the DX Cooling Coil object.
   N9 , \\field Cooling Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
        \\units dimensionless
        \\note Defines Cooling convergence tolerence as a fraction of the Cooling load to be met.
   A13, \\field Supplemental Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coil.
   A14, \\field Supplemental Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note Needs to match in the supplemental heating coil object.
   N10, \\field Maximum Supply Air Temperature from Supplemental Heater
        \\required-field
        \\type real
        \\units C
        \\autosizable
        \\note Supply air temperature from the supplemental heater will not exceed this value.
   N11, \\field Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation
        \\type real
        \\maximum 21.0
        \\default 21.0
        \\units C
        \\note Supplemental heater will not operate when outdoor temperature exceeds this value.
   A15, \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default DrawThrough
        \\note Select fan placement as either blow through or draw through.
   A16, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that controls fan operation. Schedule values of 0 denote
        \\note cycling fan operation (fan cycles with cooling or heating coil). Schedule Name values greater
        \\note than 0 denote constant fan operation (fan runs continually regardless of coil operation).
        \\note The fan operating mode defaults to cycling fan operation if this field is left blank.
   A17; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:WaterToAirHeatPump,
        \\memo Water-to-air heat pump. Forced-convection heating-cooling unit with supply fan,
        \\memo water-to-air cooling and heating coils, supplemental heating coil (gas, electric, hot
        \\memo water, or steam), and fixed-position outdoor air mixer.
        \\min-fields 25
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one OutdoorAir:Mixer object type is available.
   A6,  \\field Outdoor Air Mixer Name
        \\type object-list
        \\object-list OutdoorAirMixers
        \\note This optional field specifies the name of the outdoor air mixer object.
        \\note When used, this name needs to match name of outdoor air mixer object.
   N1 , \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N2 , \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to fan size.
   N3 , \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to fan size.
        \\note Only used when heat pump fan operating mode is continuous. This air flow rate
        \\note is used when no heating or cooling is required and the DX coil compressor is off.
        \\note If this field is left blank or zero, the supply air flow rate from the previous on cycle
        \\note (either cooling or heating) is used.
   N4 , \\field Outdoor Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during cooling operation.
   N5 , \\field Outdoor Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to supply air flow rate during heating operation.
   N6 , \\field Outdoor Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Only used when heat pump Fan operating mode is continuous. This air flow rate
        \\note is used when no heating or cooling is required and the DX coil compressor is off.
        \\note If this field is left blank or zero, the outdoor air flow rate from the previous on cycle
        \\note (either cooling or heating) is used.
   A7,  \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\note  Only works with On/Off Fan
   A8,  \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansOnOff
        \\note Needs to match Fan:OnOff object
   A9,  \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:WaterToAirHeatPump:EquationFit
        \\key Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit
   A10, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsWaterToAirHP
        \\object-list HeatingCoilsWaterToAirVSHP
        \\note  Needs to match in the water-to-air heatpump heating coil object
   A11, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:WaterToAirHeatPump:EquationFit
        \\key Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit
   A12, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsWaterToAirHP
        \\object-list CoolingCoilsWaterToAirVSHP
        \\note Needs to match in the water-to-air heatpump cooling coil object
   N7,  \\field Maximum Cycling Rate
        \\type real
        \\units cycles/hr
        \\minimum 0.0
        \\maximum 5.0
        \\default 2.5
        \\note The maximum on-off cycling rate for the compressor
        \\note Suggested value is 2.5 for a typical heat pump
   N8,  \\field Heat Pump Time Constant
        \\type real
        \\units s
        \\minimum 0.0
        \\maximum 500.0
        \\default 60.0
        \\note Time constant for the cooling coil's capacity to reach steady state after startup
        \\note Suggested value is 60 for a typical heat pump
   N9,  \\field Fraction of On-Cycle Power Use
        \\minimum 0.0
        \\maximum 0.05
        \\default 0.01
        \\note The fraction of on-cycle power use to adjust the part load fraction based on
        \\note the off-cycle power consumption due to crankcase heaters, controls, fans, and etc.
        \\note Suggested value is 0.01 for a typical heat pump
   N10, \\field Heat Pump Fan Delay Time
        \\units s
        \\minimum 0.0
        \\default 60
        \\note Programmed time delay for heat pump fan to shut off after compressor cycle off.
        \\note Only required when fan operating mode is cycling
        \\note Enter 0 when fan operating mode is continuous
   A13, \\field Supplemental Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A14, \\field Supplemental Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note  Needs to match in the supplemental heating coil object
   N11, \\field Maximum Supply Air Temperature from Supplemental Heater
        \\required-field
        \\type real
        \\units C
        \\autosizable
   N12, \\field Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation
        \\type real
        \\maximum 21.0
        \\default 21.0
        \\units C
   A15, \\field Outdoor Dry-Bulb Temperature Sensor Node Name
        \\type node
   A16, \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A17, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that controls fan operation. Schedule values of 0 denote
        \\note cycling fan operation (fan cycles with cooling or heating coil). Schedule values greater
        \\note than 0 denote constant fan operation (fan runs continually regardless of coil operation).
        \\note The fan operating mode defaults to cycling fan operation if this field is left blank.
   A18, \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists
   A19; \\field Heat Pump Coil Water Flow Mode
        \\type choice
        \\key Constant
        \\key Cycling
        \\key ConstantOnDemand
        \\default Cycling
        \\note used only when the heat pump coils are of the type WaterToAirHeatPump:EquationFit
        \\note Constant results in 100% water flow regardless of compressor PLR
        \\note Cycling results in water flow that matches compressor PLR
        \\note ConstantOnDemand results in 100% water flow whenever the coil is on, but is 0% whenever the coil has no load

ZoneHVAC:Dehumidifier:DX,
        \\memo This object calculates the performance of zone (room) air dehumidifiers.
        \\memo Meant to model conventional direct expansion (DX) cooling-based room air
        \\memo dehumidifiers (reject 100% of condenser heat to the zone air), but this
        \\memo object might be able to be used to model other room air dehumidifier types.
        \\min-fields 13
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note Unique name for this direct expansion (DX) zone dehumidifier object.
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values of 0 denote the unit is off.
        \\note Schedule values >0.0 (usually 1.0) indicate that the dehumidifier is available to operate.
   A3 , \\field Air Inlet Node Name
        \\required-field
        \\type node
        \\note Air inlet node for the dehumidifier must be a zone air exhaust node.
   A4 , \\field Air Outlet Node Name
        \\required-field
        \\type node
        \\note Air outlet node for the dehumidifier must be a zone air inlet node.
   N1,  \\field Rated Water Removal
        \\required-field
        \\type real
        \\units L/day
        \\minimum> 0.0
        \\note Rating point: air entering dehumidifier at 26.7 C (80 F) dry-bulb and 60% relative humidity.
   N2,  \\field Rated Energy Factor
        \\required-field
        \\type real
        \\units L/kWh
        \\minimum> 0.0
        \\note Rating point: air entering dehumidifier at 26.7 C (80 F) dry-bulb and 60% relative humidity.
   N3,  \\field Rated Air Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
   A5,  \\field Water Removal Curve Name
        \\required-field
        \\type object-list
        \\object-list BiQuadraticCurves
        \\note Name of a curve that describes the water removal rate (normalized to rated conditions)
        \\note as a function of the dry-bulb temperature and relative humidity of the air
        \\note entering the dehumidifier.
        \\note Curve output = (actual water removal/rated water removal) = a + b*T + c*T**2 + d*RH +
        \\note                                                             e*RH**2 + f*T*RH
        \\note T = inlet air dry-bulb temperature (C)
        \\note RH = inlet air RH (%)
   A6,  \\field Energy Factor Curve Name
        \\required-field
        \\type object-list
        \\object-list BiQuadraticCurves
        \\note Name of a curve that describes the energy factor (normalized to rated conditions)
        \\note as a function of the dry-bulb temperature and relative humidity of the air
        \\note entering the dehumidifier.
        \\note Curve output = (actual energy factor/rated energy factor) = a + b*T + c*T**2 + d*RH +
        \\note                                                             e*RH**2 + f*T*RH
        \\note T = inlet air dry-bulb temperature (C)
        \\note RH = inlet air RH (%)
   A7,  \\field Part Load Fraction Correlation Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note Name of a curve that describes the part load fraction (PLF) of the system as
        \\note a function of the part load ratio. Used to calculate dehumidifier run time fraction
        \\note and electric power.
        \\note quadratic curve = a + b*PLR + c*PLR**2
        \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
        \\note PLR = part load ratio (dehumidification load/steady state water removal capacity)
   N4,  \\field Minimum Dry-Bulb Temperature for Dehumidifier Operation
        \\type real
        \\units C
        \\default 10.0
        \\note Dehumidifier shut off if inlet air (zone) temperature is below this value.
        \\note This value must be less than the Maximum Dry-Bulb Temperature for Dehumidifier Operation.
   N5,  \\field Maximum Dry-Bulb Temperature for Dehumidifier Operation
        \\type real
        \\units C
        \\default 35.0
        \\note Dehumidifier shut off if inlet air (zone) temperature is above this value.
        \\note This value must be greater than the Minimum Dry-Bulb Temperature for Dehumidifier Operation.
   N6,  \\field Off-Cycle Parasitic Electric Load
        \\type real
        \\units W
        \\minimum 0.0
        \\default 0.0
        \\note Parasitic electric power consumed when the dehumidifier is available to operate, but
        \\note does not operate (i.e., no high humidity load to be met).
        \\note Off cycle parasitic power is 0 when the availability schedule is 0.
        \\note This electric load is considered as a heat gain to the zone air.
   A8;  \\field Condensate Collection Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Name of storage tank used to collect water removed by the dehumidifier.

ZoneHVAC:EnergyRecoveryVentilator,
        \\memo This compound component models a stand-alone energy recovery ventilator (ERV)
        \\memo that conditions outdoor ventilation air and supplies that air directly to a zone.
        \\memo The ERV unit is modeled as a collection of components: air-to-air heat exchanger,
        \\memo supply air fan, exhaust air fan and an optional controller to avoid overheating
        \\memo of the supply air (economizer or free cooling operation).
        \\min-fields 7
  A1,  \\field Name
        \\required-field
        \\type alpha
  A2 ,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
  A3 ,  \\field Heat Exchanger Name
        \\required-field
        \\note Heat exchanger type must be HeatExchanger:AirToAir:SensibleAndLatent
        \\type object-list
        \\object-list HXAirToAirSensibleAndLatentNames
  N1 ,  \\field Supply Air Flow Rate
        \\required-field
        \\autosizable
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\note This flow rate must match the supply fan's air flow rate.
  N2 ,  \\field Exhaust Air Flow Rate
        \\required-field
        \\autosizable
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\note This flow rate must match the supply fan air flow rate.
  A4 ,  \\field Supply Air Fan Name
        \\required-field
        \\note Fan type must be Fan:OnOff
        \\type object-list
        \\object-list FansOnOff
  A5 ,  \\field Exhaust Air Fan Name
        \\required-field
        \\note Fan type must be Fan:OnOff
        \\type object-list
        \\object-list FansOnOff
  A6 ,  \\field Controller Name
        \\note Enter the name of a ZoneHVAC:EnergyRecoveryVentilator:Controller object.
        \\type object-list
        \\object-list ControllerStandAloneEnergyRecoveryVentilator
  N3 ,  \\field Ventilation Rate per Unit Floor Area
        \\type real
        \\units m3/s-m2
        \\minimum 0.0
        \\note 0.000508 m3/s-m2 corresponds to 0.1 ft3/min-ft2
        \\note Used only when supply and exhaust air flow rates are autosized.
  N4 ,  \\field Ventilation Rate per Occupant
        \\type real
        \\units m3/s-person
        \\minimum 0.0
        \\note 0.00236 m3/s-person corresponds to 5 ft3/min-person
        \\note Used only when supply and exhaust air flow rates are autosized.
  A7;   \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:EnergyRecoveryVentilator:Controller,
       \\memo This controller is used exclusively by the ZoneHVAC:EnergyRecoveryVentilator object
       \\memo to allow economizer (free cooling) operation when possible.
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControllerStandAloneEnergyRecoveryVentilator
  N1 , \\field Temperature High Limit
       \\type real
       \\units C
       \\note Enter the maximum outdoor dry-bulb temperature limit for economizer operation.
       \\note No input or blank input means this limit is not operative
  N2 , \\field Temperature Low Limit
       \\type real
       \\units C
       \\note Enter the minimum outdoor dry-bulb temperature limit for economizer operation.
       \\note No input or blank input means this limit is not operative
  N3 , \\field Enthalpy High Limit
       \\type real
       \\units J/kg
       \\note Enter the maximum outdoor enthalpy limit for economizer operation.
       \\note No input or blank input means this limit is not operative
  N4 , \\field Dewpoint Temperature Limit
       \\type real
       \\units C
       \\note Enter the maximum outdoor dewpoint temperature limit for economizer operation.
       \\note No input or blank input means this limit is not operative
  A2 , \\field Electronic Enthalpy Limit Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Enter the name of a quadratic or cubic curve which defines the maximum outdoor
       \\note humidity ratio (function of outdoor dry-bulb temperature) for economizer operation.
       \\note No input or blank input means this limit is not operative
  A3 , \\field Exhaust Air Temperature Limit
       \\type choice
       \\key ExhaustAirTemperatureLimit
       \\key NoExhaustAirTemperatureLimit
       \\default NoExhaustAirTemperatureLimit
  A4 , \\field Exhaust Air Enthalpy Limit
       \\type choice
       \\key ExhaustAirEnthalpyLimit
       \\key NoExhaustAirEnthalpyLimit
       \\default NoExhaustAirEnthalpyLimit
  A5 , \\field Time of Day Economizer Flow Control Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values greater than 0 indicate economizer operation is active. This
       \\note schedule may be used with or without the High Humidity Control option.
       \\note When used together, high humidity control has priority over economizer control.
  A6 , \\field High Humidity Control Flag
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Select Yes to modify air flow rates based on a zone humidistat.
       \\note Select No to disable this feature.
  A7 , \\field Humidistat Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\note Enter the name of the zone where the humidistat is located.
  N5 , \\field High Humidity Outdoor Air Flow Ratio
       \\type real
       \\minimum> 0.0
       \\default 1.0
       \\note Enter the ratio of supply (outdoor) air to the maximum supply air flow rate when modified
       \\note air flow rates are active based on high indoor humidity.
  A8 ; \\field Control High Indoor Humidity Based on Outdoor Humidity Ratio
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\note If NO is selected, the air flow rate is modified any time indoor relative
       \\note humidity is above humidistat setpoint. If YES is selected, outdoor air flow
       \\note rate is modified any time indoor relative humidity is above the humidistat
       \\note setpoint AND the outdoor humidity ratio is less than the indoor humidity ratio.

ZoneHVAC:UnitVentilator,
        \\memo Unit ventilator. Forced-convection ventilation unit with supply fan (constant-volume
        \\memo or variable-volume), optional chilled water cooling coil, optional heating coil
        \\memo (gas, electric, hot water, or steam) and controllable outdoor air mixer.
        \\min-fields 16
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   N1 , \\field Maximum Supply Air Flow Rate
        \\required-field
        \\units m3/s
        \\minimum> 0
        \\autosizable
   A3 , \\field Outdoor Air Control Type
        \\required-field
        \\type choice
        \\key FixedAmount
        \\key VariablePercent
        \\key FixedTemperature
   N2 , \\field Minimum Outdoor Air Flow Rate
        \\required-field
        \\units m3/s
        \\autosizable
        \\minimum 0.0
   A4 , \\field Minimum Outdoor Air Schedule Name
        \\required-field
        \\type object-list
        \\object-list ScheduleNames
        \\note schedule values multiply the minimum outdoor air flow rate
   N3 , \\field Maximum Outdoor Air Flow Rate
        \\required-field
        \\units m3/s
        \\minimum 0
        \\autosizable
   A5 , \\field Maximum Outdoor Air Fraction or Temperature Schedule Name
        \\note that this depends on the control type as to whether it is a fraction or temperature
        \\required-field
        \\type object-list
        \\object-list ScheduleNames
   A6 , \\field Air Inlet Node Name
        \\required-field
        \\type node
   A7 , \\field Air Outlet Node Name
        \\required-field
        \\type node
   A8,  \\field Outdoor Air Node Name
        \\required-field
        \\type node
   A9,  \\field Exhaust Air Node Name
        \\required-field
        \\type node
   A10, \\field Mixed Air Node Name
        \\required-field
        \\type node
        \\note inlet to coils
   A11, \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:ConstantVolume
        \\key Fan:VariableVolume
        \\note Allowable fan types are Fan:ConstantVolume and
        \\note Fan:VariableVolume
   A12, \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandVAV
   A13, \\field Coil Option
        \\required-field
        \\type choice
        \\key None
        \\key Heating
        \\key Cooling
        \\key HeatingAndCooling
   A14, \\field Heating Coil Object Type
        \\type choice
        \\key Coil:Heating:Water
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Steam
   A15, \\field Heating Coil Name
        \\type object-list
        \\object-list HeatingCoilName
   N4 , \\field Heating Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A16, \\field Cooling Coil Object Type
        \\type choice
        \\key Coil:Cooling:Water
        \\key Coil:Cooling:Water:DetailedGeometry
        \\key CoilSystem:Cooling:Water:HeatExchangerAssisted
   A17, \\field Cooling Coil Name
        \\type object-list
        \\object-list CoolingCoilsWater
   N5 , \\field Cooling Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A18; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:UnitHeater,
        \\memo Unit heater. Forced-convection heating-only unit with supply fan, heating coil
        \\memo (gas, electric, hot water, or steam) and fixed-position outdoor air mixer.
        \\min-fields 10
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\required-field
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Air Inlet Node Name
        \\type node
   A4 , \\field Air Outlet Node Name
        \\type node
   A5 , \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:ConstantVolume
        \\key Fan:VariableVolume
        \\note Allowable fan types are Fan:ConstantVolume and
        \\note Fan:VariableVolume
   A6 , \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandVAV
   N1 , \\field Maximum Supply Air Flow Rate
        \\required-field
        \\units m3/s
        \\minimum> 0
        \\autosizable
   A7 , \\field Fan Control Type
        \\required-field
        \\type choice
        \\key OnOff
        \\key Continuous
   A8 , \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Water
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Steam
   A9 , \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
   N2 , \\field Maximum Hot Water or Steam Flow Rate
        \\note Not used when heating coil is gas or electric
        \\units m3/s
        \\minimum> 0
        \\autosizable
        \\ip-units gal/min
   N3 , \\field Minimum Hot Water or Steam Flow Rate
        \\note Not used when heating coil is gas or electric
        \\units m3/s
        \\minimum 0
        \\default 0
        \\ip-units gal/min
   N4 , \\field Heating Convergence Tolerance
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A10; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists


ZoneHVAC:OutdoorAirUnit,
       \\memo The zone outdoor air unit models a single-zone dedicated outdoor air system (DOAS).
       \\memo Forced-convection 100% outdoor air unit with supply fan and optional equipment
       \\memo including exhaust fan, heating coil, cooling coil, and heat recovery.
       \\min-fields 18
   A1, \\field Name
       \\required-field
   A2, \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
   A3, \\field Zone Name
       \\required-field
       \\note (name of zone system is serving)
       \\type object-list
       \\object-list ZoneNames
   N1, \\field Outdoor Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum> 0
       \\autosizable
   A4, \\field Outdoor Air Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
   A5, \\field Supply Fan Name
       \\required-field
       \\note Allowable fan types are Fan:ConstantVolume and
       \\note Fan:VariableVolume
       \\type object-list
       \\object-list FansCVandVAV
  A6, \\field Supply Fan Placement
      \\required-field
      \\type choice
      \\key BlowThrough
      \\key DrawThrough
      \\default DrawThrough
   A7, \\field Exhaust Fan Name
       \\note Allowable fan types are Fan:ConstantVolume and
       \\note Fan:VariableVolume
       \\type object-list
       \\object-list FansCVandVAV
   N2, \\field Exhaust Air Flow Rate
       \\units m3/s
       \\autosizable
   A8, \\field Exhaust Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
   A9, \\field Unit Control Type
       \\required-field
       \\type choice
       \\key NeutralControl
       \\key TemperatureControl
       \\default NeutralControl
   A10, \\field High Air Control Temperature Schedule Name
       \\note Air and control temperatures for cooling. If outdoor air temperature
       \\note is above the high air control temperature, then the zone inlet air temperature
       \\note is set to the high air control temperature. If the outdoor air is between high and low
       \\note air control temperature, then there is no cooling/heating requirements.
       \\type object-list
       \\object-list ScheduleNames
   A11, \\field Low Air Control Temperature Schedule Name
       \\note Air and control temperatures for Heating. If outdoor air temperature
       \\note is below the low air control temperature, then the zone inlet air temperature
       \\note is set to the low air control temperature. If the outdoor air is between high and low
       \\note air control temperature, then there is no cooling/heating requirements.
       \\type object-list
       \\object-list ScheduleNames
  A12, \\field Outdoor Air Node Name
       \\required-field
       \\type node
  A13, \\field AirOutlet Node Name
       \\required-field
       \\type node
  A14, \\field AirInlet Node Name
       \\type node
       \\note air leaves zone
  A15, \\field Supply FanOutlet Node Name
       \\type node
       \\required-field
  A16, \\field Outdoor Air Unit List Name
       \\note Enter the name of an ZoneHVAC:OutdoorAirUnit:EquipmentList object.
       \\required-field
       \\type object-list
       \\object-list OutdoorAirUnitEquipmentLists
  A17; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:OutdoorAirUnit:EquipmentList,
       \\memo Equipment list for components in a ZoneHVAC:OutdoorAirUnit. Components are simulated
       \\memo sequentially in the order given in the equipment list.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference OutdoorAirUnitEquipmentLists
  A2 , \\field Component 1 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A3 , \\field Component 1 Name
  A4 , \\field Component 2 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A5 , \\field Component 2 Name
  A6 , \\field Component 3 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A7 , \\field Component 3 Name
  A8 , \\field Component 4 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A9 , \\field Component 4 Name
  A10 , \\field Component 5 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A11 , \\field Component 5 Name
  A12 , \\field Component 6 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A13 , \\field Component 6 Name
  A14 , \\field Component 7 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A15, \\field Component 7 Name
  A16 , \\field Component 8 Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
       \\key Coil:Heating:Water
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatexchangerAssisted
       \\key CoilSystem:Cooling:DX
       \\key CoilSystem:Heating:DX
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key Dehumidifier:Desiccant:NoFans
  A17; \\field Component 8 Name

ZoneHVAC:TerminalUnit:VariableRefrigerantFlow,
        \\memo Zone terminal unit with variable refrigerant flow (VRF) DX cooling and heating coils
        \\memo (air-to-air heat pump). The VRF terminal units are served by an
        \\memo AirConditioner:VariableRefrigerantFlow system.
        \\min-fields 19
  A1 ,  \\field Zone Terminal Unit Name
        \\required-field
        \\type alpha
        \\reference ZoneTerminalUnitNames
  A2 ,  \\field Terminal Unit Availability schedule
        \\type object-list
        \\object-list ScheduleNames
        \\note The unit is available the entire simulation if this field is left blank
        \\note Schedule values of 0 denote the unit is off.
  A3 ,  \\field Terminal Unit Air Inlet Node Name
        \\required-field
        \\type node
        \\note the inlet node to the terminal unit
  A4 ,  \\field Terminal Unit Air Outlet Node Name
        \\required-field
        \\type node
        \\note the outlet node of the terminal unit
  N1 ,  \\field Supply Air Flow Rate During Cooling Operation
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
  N2 ,  \\field Supply Air Flow Rate When No Cooling is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
  N3 ,  \\field Supply Air Flow Rate During Heating Operation
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
  N4 ,  \\field Supply Air Flow Rate When No Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
  N5 ,  \\field Outdoor Air Flow Rate During Cooling Operation
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
  N6 ,  \\field Outdoor Air Flow Rate During Heating Operation
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
  N7 ,  \\field Outdoor Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
  A5 ,  \\field Supply Air Fan Operating Mode Schedule Name
        \\required-field
        \\type object-list
        \\object-list ScheduleNames
  A6 ,  \\field Supply Air Fan placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
        \\note Select fan placement as either blow through or draw through.
  A7 ,  \\field Supply Air Fan Object Type
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\default Fan:ConstantVolume
  A8 ,  \\field Supply Air Fan Object Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
  A9 ,  \\field Outside Air Mixer Object Type
        \\type choice
        \\key OutdoorAir:Mixer
        \\note If this field is blank, and outside air mixer is not used.
  A10,  \\field Outside Air Mixer Object Name
        \\type object-list
        \\object-list OutdoorAirMixers
        \\note If this field is blank, and outside air mixer is not used.
  A11,  \\field Cooling Coil Object Type
        \\type choice
        \\key Coil:Cooling:DX:VariableRefrigerantFlow
        \\note Cooling Coil Type must be Coil:Cooling:DX:VariableRefrigerantFlow
        \\note This field may be left blank if heating-only mode is used
  A12,  \\field Cooling Coil Object Name
        \\type object-list
        \\object-list CoolingCoilsDXVarRefrigFlow
        \\note Cooling Coil Type must be Coil:Cooling:DX:VariableRefrigerantFlow
        \\note This field may be left blank if heating-only mode is used
  A13,  \\field Heating Coil Object Type
        \\type choice
        \\key Coil:Heating:DX:VariableRefrigerantFlow
        \\note Heating Coil Type must be Coil:Heating:DX:VariableRefrigerantFlow
        \\note This field may be left blank if cooling-only mode is used
  A14,  \\field Heating Coil Object Name
        \\type object-list
        \\object-list HeatingCoilsDXVarRefrigFlow
        \\note Heating Coil Type must be Coil:Heating:DX:VariableRefrigerantFlow
        \\note This field may be left blank if cooling-only mode is used
  N8 ,  \\field Zone Terminal Unit On Parasitic Electric Energy Use
        \\type real
        \\units W
        \\minimum 0
        \\default 0
  N9 ,  \\field Zone Terminal Unit Off Parasitic Electric Energy Use
        \\type real
        \\units W
        \\minimum 0
        \\default 0
  N10, \\field Rated Total Heating Capacity Sizing Ratio
       \\type real
       \\units W/W
       \\minimum 1.0
       \\default 1.0
       \\note If this terminal unit's heating coil is autosized, the heating capacity is sized
       \\note to be equal to the cooling capacity multiplied by this sizing ratio.
       \\note This input applies to the terminal unit heating coil and overrides the sizing
       \\note ratio entered in the AirConditioner:VariableRefrigerantFlow object.
  A15; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

\\group Zone HVAC Radiative/Convective Units

ZoneHVAC:Baseboard:RadiantConvective:Water,
       \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
       \\memo The number of surfaces can be expanded beyond 100, if necessary, by adding more
       \\memo groups to the end of the list
       \\min-fields 9
  A1,  \\field Name
       \\required-field
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Inlet Node Name
       \\required-field
       \\type node
  A4,  \\field Outlet Node Name
       \\required-field
       \\type node
  N1,  \\field Rated Average Water Temperature
       \\note Rated average water temperature is the average of the inlet and outlet water temperatures
       \\note at rated conditions.
       \\type real
       \\maximum 150.0
       \\minimum 20.0
       \\units C
       \\default 87.78
  N2,  \\field Rated Water Mass Flow Rate
       \\note Standard is I=B=R Rating document where all baseboards are rated at either 0.063 kg/s (1 gpm)
       \\note or 0.252 kg/s (4 gpm).  It is recommended that users find data for the baseboard heater that
       \\note corresponds to performance at 0.063 kg/s unless the flow rate is expected to be above 0.252 kg/s.
       \\note If the flow rate is expected to be above 0.252 kg/s, this field should be 0.252 kg/s.
       \\type real
       \\maximum 10.0
       \\minimum> 0.0
       \\units Kg/s
       \\default 0.063
  N3,  \\field Rated Capacity
       \\note Users must multiply the actual finned length published in the literature.
       \\note Rated Capacity is for an inlet air dry-bulb temperature of 18.0C, the Rated Water Mass Flow Rate
       \\note of 0.063kg/s or 0.252kg/s, and the Rated Average Water Temperature between 32.2C and 115.6C.
       \\autosizable
       \\type real
       \\units W
       \\default autosize
  N4,  \\field Maximum Water Flow Rate
       \\required-field
       \\autosizable
       \\type real
       \\units m3/s
  N5,  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  N6,  \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
  N7,  \\field Fraction of Radiant Energy Incident on People
       \\type real
       \\minimum 0
       \\maximum 1
  A5,  \\field Surface 1 Name
       \\begin-extensible
       \\note Radiant energy may be distributed to specific surfaces
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N8,  \\field Fraction of Radiant Energy to Surface 1
       \\type real
       \\minimum 0
       \\maximum 1
  A6,  \\field Surface 2 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N9, \\field Fraction of Radiant Energy to Surface 2
       \\type real
       \\minimum 0
       \\maximum 1
  A7,  \\field Surface 3 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N10, \\field Fraction of Radiant Energy to Surface 3
       \\type real
       \\minimum 0
       \\maximum 1
  A8,  \\field Surface 4 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N11, \\field Fraction of Radiant Energy to Surface 4
       \\type real
       \\minimum 0
       \\maximum 1
  A9,  \\field Surface 5 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N12, \\field Fraction of Radiant Energy to Surface 5
       \\type real
       \\minimum 0
       \\maximum 1
  A10, \\field Surface 6 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N13, \\field Fraction of Radiant Energy to Surface 6
       \\type real
       \\minimum 0
       \\maximum 1
  A11, \\field Surface 7 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N14, \\field Fraction of Radiant Energy to Surface 7
       \\type real
       \\minimum 0
       \\maximum 1
  A12, \\field Surface 8 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N15, \\field Fraction of Radiant Energy to Surface 8
       \\type real
       \\minimum 0
       \\maximum 1
  A13, \\field Surface 9 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N16, \\field Fraction of Radiant Energy to Surface 9
       \\type real
       \\minimum 0
       \\maximum 1
  A14, \\field Surface 10 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N17, \\field Fraction of Radiant Energy to Surface 10
       \\type real
       \\minimum 0
       \\maximum 1
  A15, \\field Surface 11 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N18, \\field Fraction of Radiant Energy to Surface 11
       \\type real
       \\minimum 0
       \\maximum 1
  A16, \\field Surface 12 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N19, \\field Fraction of Radiant Energy to Surface 12
       \\type real
       \\minimum 0
       \\maximum 1
  A17, \\field Surface 13 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N20, \\field Fraction of Radiant Energy to Surface 13
       \\type real
       \\minimum 0
       \\maximum 1
  A18, \\field Surface 14 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N21, \\field Fraction of Radiant Energy to Surface 14
       \\type real
       \\minimum 0
       \\maximum 1
  A19, \\field Surface 15 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N22, \\field Fraction of Radiant Energy to Surface 15
       \\type real
       \\minimum 0
       \\maximum 1
  A20, \\field Surface 16 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N23, \\field Fraction of Radiant Energy to Surface 16
       \\type real
       \\minimum 0
       \\maximum 1
  A21, \\field Surface 17 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N24, \\field Fraction of Radiant Energy to Surface 17
       \\type real
       \\minimum 0
       \\maximum 1
  A22, \\field Surface 18 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N25, \\field Fraction of Radiant Energy to Surface 18
       \\type real
       \\minimum 0
       \\maximum 1
  A23, \\field Surface 19 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N26, \\field Fraction of Radiant Energy to Surface 19
       \\type real
       \\minimum 0
       \\maximum 1
  A24, \\field Surface 20 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N27, \\field Fraction of Radiant Energy to Surface 20
       \\type real
       \\minimum 0
       \\maximum 1
  A25, \\field Surface 21 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N28, \\field Fraction of Radiant Energy to Surface 21
       \\type real
       \\minimum 0
       \\maximum 1
  A26, \\field Surface 22 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N29, \\field Fraction of Radiant Energy to Surface 22
       \\type real
       \\minimum 0
       \\maximum 1
  A27, \\field Surface 23 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N30, \\field Fraction of Radiant Energy to Surface 23
       \\type real
       \\minimum 0
       \\maximum 1
  A28, \\field Surface 24 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N31, \\field Fraction of Radiant Energy to Surface 24
       \\type real
       \\minimum 0
       \\maximum 1
  A29, \\field Surface 25 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N32, \\field Fraction of Radiant Energy to Surface 25
       \\type real
       \\minimum 0
       \\maximum 1
  A30, \\field Surface 26 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N33, \\field Fraction of Radiant Energy to Surface 26
       \\type real
       \\minimum 0
       \\maximum 1
  A31, \\field Surface 27 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N34, \\field Fraction of Radiant Energy to Surface 27
       \\type real
       \\minimum 0
       \\maximum 1
  A32, \\field Surface 28 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N35, \\field Fraction of Radiant Energy to Surface 28
       \\type real
       \\minimum 0
       \\maximum 1
  A33, \\field Surface 29 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N36, \\field Fraction of Radiant Energy to Surface 29
       \\type real
       \\minimum 0
       \\maximum 1
  A34, \\field Surface 30 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N37, \\field Fraction of Radiant Energy to Surface 30
       \\type real
       \\minimum 0
       \\maximum 1
  A35, \\field Surface 31 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N38, \\field Fraction of Radiant Energy to Surface 31
       \\type real
       \\minimum 0
       \\maximum 1
  A36, \\field Surface 32 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N39, \\field Fraction of Radiant Energy to Surface 32
       \\type real
       \\minimum 0
       \\maximum 1
  A37, \\field Surface 33 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N40, \\field Fraction of Radiant Energy to Surface 33
       \\type real
       \\minimum 0
       \\maximum 1
  A38, \\field Surface 34 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N41, \\field Fraction of Radiant Energy to Surface 34
       \\type real
       \\minimum 0
       \\maximum 1
  A39, \\field Surface 35 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N42, \\field Fraction of Radiant Energy to Surface 35
       \\type real
       \\minimum 0
       \\maximum 1
  A40, \\field Surface 36 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N43, \\field Fraction of Radiant Energy to Surface 36
       \\type real
       \\minimum 0
       \\maximum 1
  A41, \\field Surface 37 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N44, \\field Fraction of Radiant Energy to Surface 37
       \\type real
       \\minimum 0
       \\maximum 1
  A42, \\field Surface 38 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N45, \\field Fraction of Radiant Energy to Surface 38
       \\type real
       \\minimum 0
       \\maximum 1
  A43, \\field Surface 39 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N46, \\field Fraction of Radiant Energy to Surface 39
       \\type real
       \\minimum 0
       \\maximum 1
  A44, \\field Surface 40 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N47, \\field Fraction of Radiant Energy to Surface 40
       \\type real
       \\minimum 0
       \\maximum 1
  A45, \\field Surface 41 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N48, \\field Fraction of Radiant Energy to Surface 41
       \\type real
       \\minimum 0
       \\maximum 1
  A46, \\field Surface 42 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N49, \\field Fraction of Radiant Energy to Surface 42
       \\type real
       \\minimum 0
       \\maximum 1
  A47, \\field Surface 43 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N50, \\field Fraction of Radiant Energy to Surface 43
       \\type real
       \\minimum 0
       \\maximum 1
  A48, \\field Surface 44 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N51, \\field Fraction of Radiant Energy to Surface 44
       \\type real
       \\minimum 0
       \\maximum 1
  A49, \\field Surface 45 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N52, \\field Fraction of Radiant Energy to Surface 45
       \\type real
       \\minimum 0
       \\maximum 1
  A50, \\field Surface 46 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N53, \\field Fraction of Radiant Energy to Surface 46
       \\type real
       \\minimum 0
       \\maximum 1
  A51, \\field Surface 47 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N54, \\field Fraction of Radiant Energy to Surface 47
       \\type real
       \\minimum 0
       \\maximum 1
  A52, \\field Surface 48 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N55, \\field Fraction of Radiant Energy to Surface 48
       \\type real
       \\minimum 0
       \\maximum 1
  A53, \\field Surface 49 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N56, \\field Fraction of Radiant Energy to Surface 49
       \\type real
       \\minimum 0
       \\maximum 1
  A54, \\field Surface 50 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N57, \\field Fraction of Radiant Energy to Surface 50
       \\type real
       \\minimum 0
       \\maximum 1
  A55, \\field Surface 51 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N58, \\field Fraction of Radiant Energy to Surface 51
       \\type real
       \\minimum 0
       \\maximum 1
  A56, \\field Surface 52 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N59, \\field Fraction of Radiant Energy to Surface 52
       \\type real
       \\minimum 0
       \\maximum 1
  A57, \\field Surface 53 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N60, \\field Fraction of Radiant Energy to Surface 53
       \\type real
       \\minimum 0
       \\maximum 1
  A58, \\field Surface 54 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N61, \\field Fraction of Radiant Energy to Surface 54
       \\type real
       \\minimum 0
       \\maximum 1
  A59, \\field Surface 55 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N62, \\field Fraction of Radiant Energy to Surface 55
       \\type real
       \\minimum 0
       \\maximum 1
  A60, \\field Surface 56 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N63, \\field Fraction of Radiant Energy to Surface 56
       \\type real
       \\minimum 0
       \\maximum 1
  A61, \\field Surface 57 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N64, \\field Fraction of Radiant Energy to Surface 57
       \\type real
       \\minimum 0
       \\maximum 1
  A62, \\field Surface 58 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N65, \\field Fraction of Radiant Energy to Surface 58
       \\type real
       \\minimum 0
       \\maximum 1
  A63, \\field Surface 59 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N66, \\field Fraction of Radiant Energy to Surface 59
       \\type real
       \\minimum 0
       \\maximum 1
  A64, \\field Surface 60 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N67, \\field Fraction of Radiant Energy to Surface 60
       \\type real
       \\minimum 0
       \\maximum 1
  A65, \\field Surface 61 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N68, \\field Fraction of Radiant Energy to Surface 61
       \\type real
       \\minimum 0
       \\maximum 1
  A66, \\field Surface 62 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N69, \\field Fraction of Radiant Energy to Surface 62
       \\type real
       \\minimum 0
       \\maximum 1
  A67, \\field Surface 63 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N70, \\field Fraction of Radiant Energy to Surface 63
       \\type real
       \\minimum 0
       \\maximum 1
  A68, \\field Surface 64 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N71, \\field Fraction of Radiant Energy to Surface 64
       \\type real
       \\minimum 0
       \\maximum 1
  A69, \\field Surface 65 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N72, \\field Fraction of Radiant Energy to Surface 65
       \\type real
       \\minimum 0
       \\maximum 1
  A70, \\field Surface 66 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N73, \\field Fraction of Radiant Energy to Surface 66
       \\type real
       \\minimum 0
       \\maximum 1
  A71, \\field Surface 67 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N74, \\field Fraction of Radiant Energy to Surface 67
       \\type real
       \\minimum 0
       \\maximum 1
  A72, \\field Surface 68 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N75, \\field Fraction of Radiant Energy to Surface 68
       \\type real
       \\minimum 0
       \\maximum 1
  A73, \\field Surface 69 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N76, \\field Fraction of Radiant Energy to Surface 69
       \\type real
       \\minimum 0
       \\maximum 1
  A74, \\field Surface 70 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N77, \\field Fraction of Radiant Energy to Surface 70
       \\type real
       \\minimum 0
       \\maximum 1
  A75, \\field Surface 71 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N78, \\field Fraction of Radiant Energy to Surface 71
       \\type real
       \\minimum 0
       \\maximum 1
  A76, \\field Surface 72 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N79, \\field Fraction of Radiant Energy to Surface 72
       \\type real
       \\minimum 0
       \\maximum 1
  A77, \\field Surface 73 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N80, \\field Fraction of Radiant Energy to Surface 73
       \\type real
       \\minimum 0
       \\maximum 1
  A78, \\field Surface 74 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N81, \\field Fraction of Radiant Energy to Surface 74
       \\type real
       \\minimum 0
       \\maximum 1
  A79, \\field Surface 75 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N82, \\field Fraction of Radiant Energy to Surface 75
       \\type real
       \\minimum 0
       \\maximum 1
  A80, \\field Surface 76 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N83, \\field Fraction of Radiant Energy to Surface 76
       \\type real
       \\minimum 0
       \\maximum 1
  A81, \\field Surface 77 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N84, \\field Fraction of Radiant Energy to Surface 77
       \\type real
       \\minimum 0
       \\maximum 1
  A82, \\field Surface 78 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N85, \\field Fraction of Radiant Energy to Surface 78
       \\type real
       \\minimum 0
       \\maximum 1
  A83, \\field Surface 79 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N86, \\field Fraction of Radiant Energy to Surface 79
       \\type real
       \\minimum 0
       \\maximum 1
  A84, \\field Surface 80 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N87, \\field Fraction of Radiant Energy to Surface 80
       \\type real
       \\minimum 0
       \\maximum 1
  A85, \\field Surface 81 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N88, \\field Fraction of Radiant Energy to Surface 81
       \\type real
       \\minimum 0
       \\maximum 1
  A86, \\field Surface 82 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N89, \\field Fraction of Radiant Energy to Surface 82
       \\type real
       \\minimum 0
       \\maximum 1
  A87, \\field Surface 83 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N90, \\field Fraction of Radiant Energy to Surface 83
       \\type real
       \\minimum 0
       \\maximum 1
  A88, \\field Surface 84 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N91, \\field Fraction of Radiant Energy to Surface 84
       \\type real
       \\minimum 0
       \\maximum 1
  A89, \\field Surface 85 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N92, \\field Fraction of Radiant Energy to Surface 85
       \\type real
       \\minimum 0
       \\maximum 1
  A90, \\field Surface 86 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N93, \\field Fraction of Radiant Energy to Surface 86
       \\type real
       \\minimum 0
       \\maximum 1
  A91, \\field Surface 87 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N94, \\field Fraction of Radiant Energy to Surface 87
       \\type real
       \\minimum 0
       \\maximum 1
  A92, \\field Surface 88 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N95, \\field Fraction of Radiant Energy to Surface 88
       \\type real
       \\minimum 0
       \\maximum 1
  A93, \\field Surface 89 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N96, \\field Fraction of Radiant Energy to Surface 89
       \\type real
       \\minimum 0
       \\maximum 1
  A94, \\field Surface 90 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N97, \\field Fraction of Radiant Energy to Surface 90
       \\type real
       \\minimum 0
       \\maximum 1
  A95, \\field Surface 91 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N98, \\field Fraction of Radiant Energy to Surface 91
       \\type real
       \\minimum 0
       \\maximum 1
  A96, \\field Surface 92 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N99, \\field Fraction of Radiant Energy to Surface 92
       \\type real
       \\minimum 0
       \\maximum 1
  A97, \\field Surface 93 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N100, \\field Fraction of Radiant Energy to Surface 93
       \\type real
       \\minimum 0
       \\maximum 1
  A98, \\field Surface 94 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N101, \\field Fraction of Radiant Energy to Surface 94
       \\type real
       \\minimum 0
       \\maximum 1
  A99, \\field Surface 95 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N102, \\field Fraction of Radiant Energy to Surface 95
       \\type real
       \\minimum 0
       \\maximum 1
 A100, \\field Surface 96 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N103, \\field Fraction of Radiant Energy to Surface 96
       \\type real
       \\minimum 0
       \\maximum 1
 A101, \\field Surface 97 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N104, \\field Fraction of Radiant Energy to Surface 97
       \\type real
       \\minimum 0
       \\maximum 1
 A102, \\field Surface 98 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N105, \\field Fraction of Radiant Energy to Surface 98
       \\type real
       \\minimum 0
       \\maximum 1
 A103, \\field Surface 99 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N106, \\field Fraction of Radiant Energy to Surface 99
       \\type real
       \\minimum 0
       \\maximum 1
 A104, \\field Surface 100 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N107; \\field Fraction of Radiant Energy to Surface 100
       \\type real
       \\minimum 0
       \\maximum 1

ZoneHVAC:Baseboard:RadiantConvective:Steam,
       \\min-fields 7
       \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
       \\memo The number of surfaces can be expanded beyond 100, if necessary, by adding more
       \\memo groups to the end of the list
  A1,  \\field Name
       \\required-field
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Inlet Node Name
       \\required-field
       \\type node
  A4,  \\field Outlet Node Name
       \\required-field
       \\type node
  N1,  \\field Degree of SubCooling
       \\type real
       \\minimum 1.0
       \\default 5.0
       \\units C
  N2,  \\field Maximum Steam Flow Rate
       \\required-field
       \\type real
       \\autosizable
       \\minimum> 0.0
       \\units m3/s
  N3,  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  N4,  \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
  N5,  \\field Fraction of Radiant Energy Incident on People
       \\type real
       \\minimum 0
       \\maximum 1
  A5,  \\field Surface 1 Name
       \\begin-extensible
       \\note Radiant energy may be distributed to specific surfaces
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N6,  \\field Fraction of Radiant Energy to Surface 1
       \\type real
       \\minimum 0
       \\maximum 1
  A6,  \\field Surface 2 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N7,  \\field Fraction of Radiant Energy to Surface 2
       \\type real
       \\minimum 0
       \\maximum 1
  A7,  \\field Surface 3 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N8,  \\field Fraction of Radiant Energy to Surface 3
       \\type real
       \\minimum 0
       \\maximum 1
  A8,  \\field Surface 4 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N9,  \\field Fraction of Radiant Energy to Surface 4
       \\type real
       \\minimum 0
       \\maximum 1
  A9,  \\field Surface 5 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N10, \\field Fraction of Radiant Energy to Surface 5
       \\type real
       \\minimum 0
       \\maximum 1
  A10, \\field Surface 6 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N11, \\field Fraction of Radiant Energy to Surface 6
       \\type real
       \\minimum 0
       \\maximum 1
  A11, \\field Surface 7 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N12, \\field Fraction of Radiant Energy to Surface 7
       \\type real
       \\minimum 0
       \\maximum 1
  A12, \\field Surface 8 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N13, \\field Fraction of Radiant Energy to Surface 8
       \\type real
       \\minimum 0
       \\maximum 1
  A13, \\field Surface 9 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N14, \\field Fraction of Radiant Energy to Surface 9
       \\type real
       \\minimum 0
       \\maximum 1
  A14, \\field Surface 10 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N15, \\field Fraction of Radiant Energy to Surface 10
       \\type real
       \\minimum 0
       \\maximum 1
  A15, \\field Surface 11 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N16, \\field Fraction of Radiant Energy to Surface 11
       \\type real
       \\minimum 0
       \\maximum 1
  A16, \\field Surface 12 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N17, \\field Fraction of Radiant Energy to Surface 12
       \\type real
       \\minimum 0
       \\maximum 1
  A17, \\field Surface 13 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N18, \\field Fraction of Radiant Energy to Surface 13
       \\type real
       \\minimum 0
       \\maximum 1
  A18, \\field Surface 14 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N19, \\field Fraction of Radiant Energy to Surface 14
       \\type real
       \\minimum 0
       \\maximum 1
  A19, \\field Surface 15 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N20, \\field Fraction of Radiant Energy to Surface 15
       \\type real
       \\minimum 0
       \\maximum 1
  A20, \\field Surface 16 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N21, \\field Fraction of Radiant Energy to Surface 16
       \\type real
       \\minimum 0
       \\maximum 1
  A21, \\field Surface 17 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N22, \\field Fraction of Radiant Energy to Surface 17
       \\type real
       \\minimum 0
       \\maximum 1
  A22, \\field Surface 18 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N23, \\field Fraction of Radiant Energy to Surface 18
       \\type real
       \\minimum 0
       \\maximum 1
  A23, \\field Surface 19 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N24, \\field Fraction of Radiant Energy to Surface 19
       \\type real
       \\minimum 0
       \\maximum 1
  A24, \\field Surface 20 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N25, \\field Fraction of Radiant Energy to Surface 20
       \\type real
       \\minimum 0
       \\maximum 1
  A25, \\field Surface 21 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N26, \\field Fraction of Radiant Energy to Surface 21
       \\type real
       \\minimum 0
       \\maximum 1
  A26, \\field Surface 22 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N27, \\field Fraction of Radiant Energy to Surface 22
       \\type real
       \\minimum 0
       \\maximum 1
  A27, \\field Surface 23 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N28, \\field Fraction of Radiant Energy to Surface 23
       \\type real
       \\minimum 0
       \\maximum 1
  A28, \\field Surface 24 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N29, \\field Fraction of Radiant Energy to Surface 24
       \\type real
       \\minimum 0
       \\maximum 1
  A29, \\field Surface 25 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N30, \\field Fraction of Radiant Energy to Surface 25
       \\type real
       \\minimum 0
       \\maximum 1
  A30, \\field Surface 26 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N31, \\field Fraction of Radiant Energy to Surface 26
       \\type real
       \\minimum 0
       \\maximum 1
  A31, \\field Surface 27 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N32, \\field Fraction of Radiant Energy to Surface 27
       \\type real
       \\minimum 0
       \\maximum 1
  A32, \\field Surface 28 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N33, \\field Fraction of Radiant Energy to Surface 28
       \\type real
       \\minimum 0
       \\maximum 1
  A33, \\field Surface 29 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N34, \\field Fraction of Radiant Energy to Surface 29
       \\type real
       \\minimum 0
       \\maximum 1
  A34, \\field Surface 30 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N35, \\field Fraction of Radiant Energy to Surface 30
       \\type real
       \\minimum 0
       \\maximum 1
  A35, \\field Surface 31 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N36, \\field Fraction of Radiant Energy to Surface 31
       \\type real
       \\minimum 0
       \\maximum 1
  A36, \\field Surface 32 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N37, \\field Fraction of Radiant Energy to Surface 32
       \\type real
       \\minimum 0
       \\maximum 1
  A37, \\field Surface 33 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N38, \\field Fraction of Radiant Energy to Surface 33
       \\type real
       \\minimum 0
       \\maximum 1
  A38, \\field Surface 34 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N39, \\field Fraction of Radiant Energy to Surface 34
       \\type real
       \\minimum 0
       \\maximum 1
  A39, \\field Surface 35 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N40, \\field Fraction of Radiant Energy to Surface 35
       \\type real
       \\minimum 0
       \\maximum 1
  A40, \\field Surface 36 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N41, \\field Fraction of Radiant Energy to Surface 36
       \\type real
       \\minimum 0
       \\maximum 1
  A41, \\field Surface 37 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N42, \\field Fraction of Radiant Energy to Surface 37
       \\type real
       \\minimum 0
       \\maximum 1
  A42, \\field Surface 38 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N43, \\field Fraction of Radiant Energy to Surface 38
       \\type real
       \\minimum 0
       \\maximum 1
  A43, \\field Surface 39 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N44, \\field Fraction of Radiant Energy to Surface 39
       \\type real
       \\minimum 0
       \\maximum 1
  A44, \\field Surface 40 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N45, \\field Fraction of Radiant Energy to Surface 40
       \\type real
       \\minimum 0
       \\maximum 1
  A45, \\field Surface 41 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N46, \\field Fraction of Radiant Energy to Surface 41
       \\type real
       \\minimum 0
       \\maximum 1
  A46, \\field Surface 42 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N47, \\field Fraction of Radiant Energy to Surface 42
       \\type real
       \\minimum 0
       \\maximum 1
  A47, \\field Surface 43 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N48, \\field Fraction of Radiant Energy to Surface 43
       \\type real
       \\minimum 0
       \\maximum 1
  A48, \\field Surface 44 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N49, \\field Fraction of Radiant Energy to Surface 44
       \\type real
       \\minimum 0
       \\maximum 1
  A49, \\field Surface 45 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N50, \\field Fraction of Radiant Energy to Surface 45
       \\type real
       \\minimum 0
       \\maximum 1
  A50, \\field Surface 46 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N51, \\field Fraction of Radiant Energy to Surface 46
       \\type real
       \\minimum 0
       \\maximum 1
  A51, \\field Surface 47 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N52, \\field Fraction of Radiant Energy to Surface 47
       \\type real
       \\minimum 0
       \\maximum 1
  A52, \\field Surface 48 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N53, \\field Fraction of Radiant Energy to Surface 48
       \\type real
       \\minimum 0
       \\maximum 1
  A53, \\field Surface 49 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N54, \\field Fraction of Radiant Energy to Surface 49
       \\type real
       \\minimum 0
       \\maximum 1
  A54, \\field Surface 50 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N55, \\field Fraction of Radiant Energy to Surface 50
       \\type real
       \\minimum 0
       \\maximum 1
  A55, \\field Surface 51 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N56, \\field Fraction of Radiant Energy to Surface 51
       \\type real
       \\minimum 0
       \\maximum 1
  A56, \\field Surface 52 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N57, \\field Fraction of Radiant Energy to Surface 52
       \\type real
       \\minimum 0
       \\maximum 1
  A57, \\field Surface 53 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N58, \\field Fraction of Radiant Energy to Surface 53
       \\type real
       \\minimum 0
       \\maximum 1
  A58, \\field Surface 54 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N59, \\field Fraction of Radiant Energy to Surface 54
       \\type real
       \\minimum 0
       \\maximum 1
  A59, \\field Surface 55 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N60, \\field Fraction of Radiant Energy to Surface 55
       \\type real
       \\minimum 0
       \\maximum 1
  A60, \\field Surface 56 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N61, \\field Fraction of Radiant Energy to Surface 56
       \\type real
       \\minimum 0
       \\maximum 1
  A61, \\field Surface 57 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N62, \\field Fraction of Radiant Energy to Surface 57
       \\type real
       \\minimum 0
       \\maximum 1
  A62, \\field Surface 58 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N63, \\field Fraction of Radiant Energy to Surface 58
       \\type real
       \\minimum 0
       \\maximum 1
  A63, \\field Surface 59 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N64, \\field Fraction of Radiant Energy to Surface 59
       \\type real
       \\minimum 0
       \\maximum 1
  A64, \\field Surface 60 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N65, \\field Fraction of Radiant Energy to Surface 60
       \\type real
       \\minimum 0
       \\maximum 1
  A65, \\field Surface 61 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N66, \\field Fraction of Radiant Energy to Surface 61
       \\type real
       \\minimum 0
       \\maximum 1
  A66, \\field Surface 62 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N67, \\field Fraction of Radiant Energy to Surface 62
       \\type real
       \\minimum 0
       \\maximum 1
  A67, \\field Surface 63 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N68, \\field Fraction of Radiant Energy to Surface 63
       \\type real
       \\minimum 0
       \\maximum 1
  A68, \\field Surface 64 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N69, \\field Fraction of Radiant Energy to Surface 64
       \\type real
       \\minimum 0
       \\maximum 1
  A69, \\field Surface 65 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N70, \\field Fraction of Radiant Energy to Surface 65
       \\type real
       \\minimum 0
       \\maximum 1
  A70, \\field Surface 66 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N71, \\field Fraction of Radiant Energy to Surface 66
       \\type real
       \\minimum 0
       \\maximum 1
  A71, \\field Surface 67 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N72, \\field Fraction of Radiant Energy to Surface 67
       \\type real
       \\minimum 0
       \\maximum 1
  A72, \\field Surface 68 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N73, \\field Fraction of Radiant Energy to Surface 68
       \\type real
       \\minimum 0
       \\maximum 1
  A73, \\field Surface 69 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N74, \\field Fraction of Radiant Energy to Surface 69
       \\type real
       \\minimum 0
       \\maximum 1
  A74, \\field Surface 70 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N75, \\field Fraction of Radiant Energy to Surface 70
       \\type real
       \\minimum 0
       \\maximum 1
  A75, \\field Surface 71 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N76, \\field Fraction of Radiant Energy to Surface 71
       \\type real
       \\minimum 0
       \\maximum 1
  A76, \\field Surface 72 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N77, \\field Fraction of Radiant Energy to Surface 72
       \\type real
       \\minimum 0
       \\maximum 1
  A77, \\field Surface 73 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N78, \\field Fraction of Radiant Energy to Surface 73
       \\type real
       \\minimum 0
       \\maximum 1
  A78, \\field Surface 74 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N79, \\field Fraction of Radiant Energy to Surface 74
       \\type real
       \\minimum 0
       \\maximum 1
  A79, \\field Surface 75 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N80, \\field Fraction of Radiant Energy to Surface 75
       \\type real
       \\minimum 0
       \\maximum 1
  A80, \\field Surface 76 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N81, \\field Fraction of Radiant Energy to Surface 76
       \\type real
       \\minimum 0
       \\maximum 1
  A81, \\field Surface 77 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N82, \\field Fraction of Radiant Energy to Surface 77
       \\type real
       \\minimum 0
       \\maximum 1
  A82, \\field Surface 78 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N83, \\field Fraction of Radiant Energy to Surface 78
       \\type real
       \\minimum 0
       \\maximum 1
  A83, \\field Surface 79 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N84, \\field Fraction of Radiant Energy to Surface 79
       \\type real
       \\minimum 0
       \\maximum 1
  A84, \\field Surface 80 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N85, \\field Fraction of Radiant Energy to Surface 80
       \\type real
       \\minimum 0
       \\maximum 1
  A85, \\field Surface 81 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N86, \\field Fraction of Radiant Energy to Surface 81
       \\type real
       \\minimum 0
       \\maximum 1
  A86, \\field Surface 82 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N87, \\field Fraction of Radiant Energy to Surface 82
       \\type real
       \\minimum 0
       \\maximum 1
  A87, \\field Surface 83 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N88, \\field Fraction of Radiant Energy to Surface 83
       \\type real
       \\minimum 0
       \\maximum 1
  A88, \\field Surface 84 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N89, \\field Fraction of Radiant Energy to Surface 84
       \\type real
       \\minimum 0
       \\maximum 1
  A89, \\field Surface 85 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N90, \\field Fraction of Radiant Energy to Surface 85
       \\type real
       \\minimum 0
       \\maximum 1
  A90, \\field Surface 86 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N91, \\field Fraction of Radiant Energy to Surface 86
       \\type real
       \\minimum 0
       \\maximum 1
  A91, \\field Surface 87 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N92, \\field Fraction of Radiant Energy to Surface 87
       \\type real
       \\minimum 0
       \\maximum 1
  A92, \\field Surface 88 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N93, \\field Fraction of Radiant Energy to Surface 88
       \\type real
       \\minimum 0
       \\maximum 1
  A93, \\field Surface 89 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N94, \\field Fraction of Radiant Energy to Surface 89
       \\type real
       \\minimum 0
       \\maximum 1
  A94, \\field Surface 90 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N95, \\field Fraction of Radiant Energy to Surface 90
       \\type real
       \\minimum 0
       \\maximum 1
  A95, \\field Surface 91 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N96, \\field Fraction of Radiant Energy to Surface 91
       \\type real
       \\minimum 0
       \\maximum 1
  A96, \\field Surface 92 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N97, \\field Fraction of Radiant Energy to Surface 92
       \\type real
       \\minimum 0
       \\maximum 1
  A97, \\field Surface 93 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N98, \\field Fraction of Radiant Energy to Surface 93
       \\type real
       \\minimum 0
       \\maximum 1
  A98, \\field Surface 94 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N99, \\field Fraction of Radiant Energy to Surface 94
       \\type real
       \\minimum 0
       \\maximum 1
  A99, \\field Surface 95 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N100, \\field Fraction of Radiant Energy to Surface 95
       \\type real
       \\minimum 0
       \\maximum 1
 A100, \\field Surface 96 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N101, \\field Fraction of Radiant Energy to Surface 96
       \\type real
       \\minimum 0
       \\maximum 1
 A101, \\field Surface 97 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N102, \\field Fraction of Radiant Energy to Surface 97
       \\type real
       \\minimum 0
       \\maximum 1
 A102, \\field Surface 98 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N103, \\field Fraction of Radiant Energy to Surface 98
       \\type real
       \\minimum 0
       \\maximum 1
 A103, \\field Surface 99 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N104, \\field Fraction of Radiant Energy to Surface 99
       \\type real
       \\minimum 0
       \\maximum 1
 A104, \\field Surface 100 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N105; \\field Fraction of Radiant Energy to Surface 100
       \\type real
       \\minimum 0
       \\maximum 1

ZoneHVAC:Baseboard:RadiantConvective:Electric,
       \\min-fields 5
       \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
       \\memo The number of surfaces can be expanded beyond 100, if necessary, by adding more
       \\memo groups to the end of the list
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Nominal Capacity
       \\required-field
       \\units W
       \\autosizable
       \\ip-units W
  N2 , \\field Efficiency
       \\required-field
       \\maximum 1.0
       \\minimum> 0.0
       \\default 1.0
  N3,  \\field Fraction Radiant
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
  N4,  \\field Fraction of Radiant Energy Incident on People
       \\type real
       \\minimum 0
       \\maximum 1
  A3,  \\field Surface 1 Name
       \\begin-extensible
       \\note Radiant energy may be distributed to specific surfaces
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N5,  \\field Fraction of Radiant Energy to Surface 1
       \\type real
       \\minimum 0
       \\maximum 1
  A4,  \\field Surface 2 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N6,  \\field Fraction of Radiant Energy to Surface 2
       \\type real
       \\minimum 0
       \\maximum 1
  A5,  \\field Surface 3 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N7,  \\field Fraction of Radiant Energy to Surface 3
       \\type real
       \\minimum 0
       \\maximum 1
  A6,  \\field Surface 4 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N8,  \\field Fraction of Radiant Energy to Surface 4
       \\type real
       \\minimum 0
       \\maximum 1
  A7,  \\field Surface 5 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N9,  \\field Fraction of Radiant Energy to Surface 5
       \\type real
       \\minimum 0
       \\maximum 1
  A8,  \\field Surface 6 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N10, \\field Fraction of Radiant Energy to Surface 6
       \\type real
       \\minimum 0
       \\maximum 1
  A9,  \\field Surface 7 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N11, \\field Fraction of Radiant Energy to Surface 7
       \\type real
       \\minimum 0
       \\maximum 1
  A10, \\field Surface 8 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N12, \\field Fraction of Radiant Energy to Surface 8
       \\type real
       \\minimum 0
       \\maximum 1
  A11, \\field Surface 9 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N13, \\field Fraction of Radiant Energy to Surface 9
       \\type real
       \\minimum 0
       \\maximum 1
  A12, \\field Surface 10 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N14, \\field Fraction of Radiant Energy to Surface 10
       \\type real
       \\minimum 0
       \\maximum 1
  A13, \\field Surface 11 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N15, \\field Fraction of Radiant Energy to Surface 11
       \\type real
       \\minimum 0
       \\maximum 1
  A14, \\field Surface 12 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N16, \\field Fraction of Radiant Energy to Surface 12
       \\type real
       \\minimum 0
       \\maximum 1
  A15, \\field Surface 13 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N17, \\field Fraction of Radiant Energy to Surface 13
       \\type real
       \\minimum 0
       \\maximum 1
  A16, \\field Surface 14 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N18, \\field Fraction of Radiant Energy to Surface 14
       \\type real
       \\minimum 0
       \\maximum 1
  A17, \\field Surface 15 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N19, \\field Fraction of Radiant Energy to Surface 15
       \\type real
       \\minimum 0
       \\maximum 1
  A18, \\field Surface 16 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N20, \\field Fraction of Radiant Energy to Surface 16
       \\type real
       \\minimum 0
       \\maximum 1
  A19, \\field Surface 17 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N21, \\field Fraction of Radiant Energy to Surface 17
       \\type real
       \\minimum 0
       \\maximum 1
  A20, \\field Surface 18 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N22, \\field Fraction of Radiant Energy to Surface 18
       \\type real
       \\minimum 0
       \\maximum 1
  A21, \\field Surface 19 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N23, \\field Fraction of Radiant Energy to Surface 19
       \\type real
       \\minimum 0
       \\maximum 1
  A22, \\field Surface 20 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N24, \\field Fraction of Radiant Energy to Surface 20
       \\type real
       \\minimum 0
       \\maximum 1
  A23, \\field Surface 21 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N25, \\field Fraction of Radiant Energy to Surface 21
       \\type real
       \\minimum 0
       \\maximum 1
  A24, \\field Surface 22 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N26, \\field Fraction of Radiant Energy to Surface 22
       \\type real
       \\minimum 0
       \\maximum 1
  A25, \\field Surface 23 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N27, \\field Fraction of Radiant Energy to Surface 23
       \\type real
       \\minimum 0
       \\maximum 1
  A26, \\field Surface 24 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N28, \\field Fraction of Radiant Energy to Surface 24
       \\type real
       \\minimum 0
       \\maximum 1
  A27, \\field Surface 25 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N29, \\field Fraction of Radiant Energy to Surface 25
       \\type real
       \\minimum 0
       \\maximum 1
  A28, \\field Surface 26 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N30, \\field Fraction of Radiant Energy to Surface 26
       \\type real
       \\minimum 0
       \\maximum 1
  A29, \\field Surface 27 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N31, \\field Fraction of Radiant Energy to Surface 27
       \\type real
       \\minimum 0
       \\maximum 1
  A30, \\field Surface 28 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N32, \\field Fraction of Radiant Energy to Surface 28
       \\type real
       \\minimum 0
       \\maximum 1
  A31, \\field Surface 29 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N33, \\field Fraction of Radiant Energy to Surface 29
       \\type real
       \\minimum 0
       \\maximum 1
  A32, \\field Surface 30 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N34, \\field Fraction of Radiant Energy to Surface 30
       \\type real
       \\minimum 0
       \\maximum 1
  A33, \\field Surface 31 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N35, \\field Fraction of Radiant Energy to Surface 31
       \\type real
       \\minimum 0
       \\maximum 1
  A34, \\field Surface 32 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N36, \\field Fraction of Radiant Energy to Surface 32
       \\type real
       \\minimum 0
       \\maximum 1
  A35, \\field Surface 33 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N37, \\field Fraction of Radiant Energy to Surface 33
       \\type real
       \\minimum 0
       \\maximum 1
  A36, \\field Surface 34 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N38, \\field Fraction of Radiant Energy to Surface 34
       \\type real
       \\minimum 0
       \\maximum 1
  A37, \\field Surface 35 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N39, \\field Fraction of Radiant Energy to Surface 35
       \\type real
       \\minimum 0
       \\maximum 1
  A38, \\field Surface 36 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N40, \\field Fraction of Radiant Energy to Surface 36
       \\type real
       \\minimum 0
       \\maximum 1
  A39, \\field Surface 37 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N41, \\field Fraction of Radiant Energy to Surface 37
       \\type real
       \\minimum 0
       \\maximum 1
  A40, \\field Surface 38 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N42, \\field Fraction of Radiant Energy to Surface 38
       \\type real
       \\minimum 0
       \\maximum 1
  A41, \\field Surface 39 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N43, \\field Fraction of Radiant Energy to Surface 39
       \\type real
       \\minimum 0
       \\maximum 1
  A42, \\field Surface 40 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N44, \\field Fraction of Radiant Energy to Surface 40
       \\type real
       \\minimum 0
       \\maximum 1
  A43, \\field Surface 41 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N45, \\field Fraction of Radiant Energy to Surface 41
       \\type real
       \\minimum 0
       \\maximum 1
  A44, \\field Surface 42 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N46, \\field Fraction of Radiant Energy to Surface 42
       \\type real
       \\minimum 0
       \\maximum 1
  A45, \\field Surface 43 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N47, \\field Fraction of Radiant Energy to Surface 43
       \\type real
       \\minimum 0
       \\maximum 1
  A46, \\field Surface 44 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N48, \\field Fraction of Radiant Energy to Surface 44
       \\type real
       \\minimum 0
       \\maximum 1
  A47, \\field Surface 45 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N49, \\field Fraction of Radiant Energy to Surface 45
       \\type real
       \\minimum 0
       \\maximum 1
  A48, \\field Surface 46 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N50, \\field Fraction of Radiant Energy to Surface 46
       \\type real
       \\minimum 0
       \\maximum 1
  A49, \\field Surface 47 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N51, \\field Fraction of Radiant Energy to Surface 47
       \\type real
       \\minimum 0
       \\maximum 1
  A50, \\field Surface 48 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N52, \\field Fraction of Radiant Energy to Surface 48
       \\type real
       \\minimum 0
       \\maximum 1
  A51, \\field Surface 49 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N53, \\field Fraction of Radiant Energy to Surface 49
       \\type real
       \\minimum 0
       \\maximum 1
  A52, \\field Surface 50 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N54, \\field Fraction of Radiant Energy to Surface 50
       \\type real
       \\minimum 0
       \\maximum 1
  A53, \\field Surface 51 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N55, \\field Fraction of Radiant Energy to Surface 51
       \\type real
       \\minimum 0
       \\maximum 1
  A54, \\field Surface 52 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N56, \\field Fraction of Radiant Energy to Surface 52
       \\type real
       \\minimum 0
       \\maximum 1
  A55, \\field Surface 53 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N57, \\field Fraction of Radiant Energy to Surface 53
       \\type real
       \\minimum 0
       \\maximum 1
  A56, \\field Surface 54 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N58, \\field Fraction of Radiant Energy to Surface 54
       \\type real
       \\minimum 0
       \\maximum 1
  A57, \\field Surface 55 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N59, \\field Fraction of Radiant Energy to Surface 55
       \\type real
       \\minimum 0
       \\maximum 1
  A58, \\field Surface 56 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N60, \\field Fraction of Radiant Energy to Surface 56
       \\type real
       \\minimum 0
       \\maximum 1
  A59, \\field Surface 57 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N61, \\field Fraction of Radiant Energy to Surface 57
       \\type real
       \\minimum 0
       \\maximum 1
  A60, \\field Surface 58 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N62, \\field Fraction of Radiant Energy to Surface 58
       \\type real
       \\minimum 0
       \\maximum 1
  A61, \\field Surface 59 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N63, \\field Fraction of Radiant Energy to Surface 59
       \\type real
       \\minimum 0
       \\maximum 1
  A62, \\field Surface 60 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N64, \\field Fraction of Radiant Energy to Surface 60
       \\type real
       \\minimum 0
       \\maximum 1
  A63, \\field Surface 61 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N65, \\field Fraction of Radiant Energy to Surface 61
       \\type real
       \\minimum 0
       \\maximum 1
  A64, \\field Surface 62 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N66, \\field Fraction of Radiant Energy to Surface 62
       \\type real
       \\minimum 0
       \\maximum 1
  A65, \\field Surface 63 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N67, \\field Fraction of Radiant Energy to Surface 63
       \\type real
       \\minimum 0
       \\maximum 1
  A66, \\field Surface 64 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N68, \\field Fraction of Radiant Energy to Surface 64
       \\type real
       \\minimum 0
       \\maximum 1
  A67, \\field Surface 65 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N69, \\field Fraction of Radiant Energy to Surface 65
       \\type real
       \\minimum 0
       \\maximum 1
  A68, \\field Surface 66 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N70, \\field Fraction of Radiant Energy to Surface 66
       \\type real
       \\minimum 0
       \\maximum 1
  A69, \\field Surface 67 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N71, \\field Fraction of Radiant Energy to Surface 67
       \\type real
       \\minimum 0
       \\maximum 1
  A70, \\field Surface 68 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N72, \\field Fraction of Radiant Energy to Surface 68
       \\type real
       \\minimum 0
       \\maximum 1
  A71, \\field Surface 69 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N73, \\field Fraction of Radiant Energy to Surface 69
       \\type real
       \\minimum 0
       \\maximum 1
  A72, \\field Surface 70 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N74, \\field Fraction of Radiant Energy to Surface 70
       \\type real
       \\minimum 0
       \\maximum 1
  A73, \\field Surface 71 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N75, \\field Fraction of Radiant Energy to Surface 71
       \\type real
       \\minimum 0
       \\maximum 1
  A74, \\field Surface 72 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N76, \\field Fraction of Radiant Energy to Surface 72
       \\type real
       \\minimum 0
       \\maximum 1
  A75, \\field Surface 73 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N77, \\field Fraction of Radiant Energy to Surface 73
       \\type real
       \\minimum 0
       \\maximum 1
  A76, \\field Surface 74 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N78, \\field Fraction of Radiant Energy to Surface 74
       \\type real
       \\minimum 0
       \\maximum 1
  A77, \\field Surface 75 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N79, \\field Fraction of Radiant Energy to Surface 75
       \\type real
       \\minimum 0
       \\maximum 1
  A78, \\field Surface 76 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N80, \\field Fraction of Radiant Energy to Surface 76
       \\type real
       \\minimum 0
       \\maximum 1
  A79, \\field Surface 77 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N81, \\field Fraction of Radiant Energy to Surface 77
       \\type real
       \\minimum 0
       \\maximum 1
  A80, \\field Surface 78 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N82, \\field Fraction of Radiant Energy to Surface 78
       \\type real
       \\minimum 0
       \\maximum 1
  A81, \\field Surface 79 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N83, \\field Fraction of Radiant Energy to Surface 79
       \\type real
       \\minimum 0
       \\maximum 1
  A82, \\field Surface 80 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N84, \\field Fraction of Radiant Energy to Surface 80
       \\type real
       \\minimum 0
       \\maximum 1
  A83, \\field Surface 81 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N85, \\field Fraction of Radiant Energy to Surface 81
       \\type real
       \\minimum 0
       \\maximum 1
  A84, \\field Surface 82 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N86, \\field Fraction of Radiant Energy to Surface 82
       \\type real
       \\minimum 0
       \\maximum 1
  A85, \\field Surface 83 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N87, \\field Fraction of Radiant Energy to Surface 83
       \\type real
       \\minimum 0
       \\maximum 1
  A86, \\field Surface 84 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N88, \\field Fraction of Radiant Energy to Surface 84
       \\type real
       \\minimum 0
       \\maximum 1
  A87, \\field Surface 85 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N89, \\field Fraction of Radiant Energy to Surface 85
       \\type real
       \\minimum 0
       \\maximum 1
  A88, \\field Surface 86 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N90, \\field Fraction of Radiant Energy to Surface 86
       \\type real
       \\minimum 0
       \\maximum 1
  A89, \\field Surface 87 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N91, \\field Fraction of Radiant Energy to Surface 87
       \\type real
       \\minimum 0
       \\maximum 1
  A90, \\field Surface 88 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N92, \\field Fraction of Radiant Energy to Surface 88
       \\type real
       \\minimum 0
       \\maximum 1
  A91, \\field Surface 89 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N93, \\field Fraction of Radiant Energy to Surface 89
       \\type real
       \\minimum 0
       \\maximum 1
  A92, \\field Surface 90 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N94, \\field Fraction of Radiant Energy to Surface 90
       \\type real
       \\minimum 0
       \\maximum 1
  A93, \\field Surface 91 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N95, \\field Fraction of Radiant Energy to Surface 91
       \\type real
       \\minimum 0
       \\maximum 1
  A94, \\field Surface 92 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N96, \\field Fraction of Radiant Energy to Surface 92
       \\type real
       \\minimum 0
       \\maximum 1
  A95, \\field Surface 93 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N97, \\field Fraction of Radiant Energy to Surface 93
       \\type real
       \\minimum 0
       \\maximum 1
  A96, \\field Surface 94 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N98, \\field Fraction of Radiant Energy to Surface 94
       \\type real
       \\minimum 0
       \\maximum 1
  A97, \\field Surface 95 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
  N99, \\field Fraction of Radiant Energy to Surface 95
       \\type real
       \\minimum 0
       \\maximum 1
  A98, \\field Surface 96 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N100, \\field Fraction of Radiant Energy to Surface 96
       \\type real
       \\minimum 0
       \\maximum 1
  A99, \\field Surface 97 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N101, \\field Fraction of Radiant Energy to Surface 97
       \\type real
       \\minimum 0
       \\maximum 1
 A100, \\field Surface 98 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N102, \\field Fraction of Radiant Energy to Surface 98
       \\type real
       \\minimum 0
       \\maximum 1
 A101, \\field Surface 99 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N103, \\field Fraction of Radiant Energy to Surface 99
       \\type real
       \\minimum 0
       \\maximum 1
 A102, \\field Surface 100 Name
       \\type object-list
       \\object-list AllHeatTranSurfNames
 N104; \\field Fraction of Radiant Energy to Surface 100
       \\type real
       \\minimum 0
       \\maximum 1

ZoneHVAC:Baseboard:Convective:Water,
       \\memo Hot water baseboard heater, convection-only. Natural convection hydronic heating unit.
  A1 , \\field Name
        \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Inlet Node Name
        \\required-field
        \\type node
  A4 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field U-Factor Times Area Value
       \\required-field
       \\autosizable
       \\units W/K
  N2 , \\field Maximum Water Flow Rate
       \\required-field
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N3 ; \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001

ZoneHVAC:Baseboard:Convective:Electric,
       \\memo Electric baseboard heater, convection-only. Natural convection electric heating unit.
  A1 , \\field Name
        \\required-field
  A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
  N1 , \\field Nominal Capacity
        \\required-field
        \\units W
        \\autosizable
        \\ip-units W
  N2 ; \\field Efficiency
        \\required-field
        \\maximum 1.0
        \\minimum> 0.0
        \\default 1.0

ZoneHVAC:LowTemperatureRadiant:VariableFlow,
        \\memo Low temperature hydronic radiant heating and/or cooling system embedded in a building
        \\memo surface (wall, ceiling, or floor). Controlled by varying the hot or chilled water
        \\memo flow to the unit.
   \\min-fields 21
   A1 , \\field Name
       \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Name of zone system is serving
        \\type object-list
        \\object-list ZoneNames
   A4 , \\field Surface Name or Radiant Surface Group Name
        \\note Identifies surfaces that radiant system is embedded in.
        \\note For a system with multiple surfaces, enter the name of
        \\note a ZoneHVAC:LowTemperatureRadiant:SurfaceGroup object.
        \\type object-list
        \\object-list RadGroupAndSurfNames
   N1 , \\field Hydronic Tubing Inside Diameter
        \\units m
        \\minimum> 0
        \\default 0.013
        \\ip-units in
   N2 , \\field Hydronic Tubing Length
        \\note (total length of pipe embedded in surface)
        \\units m
        \\minimum> 0
        \\autosizable
   A5 , \\field Temperature Control Type
        \\note (Temperature on which unit is controlled)
        \\type choice
        \\key MeanAirTemperature
        \\key MeanRadiantTemperature
        \\key OperativeTemperature
        \\key OutdoorDryBulbTemperature
        \\key OutdoorWetBulbTemperature
        \\default MeanAirTemperature
   N3 , \\field Maximum Hot Water Flow
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\ip-units gal/min
   A6 , \\field Heating Water Inlet Node Name
        \\type node
   A7 , \\field Heating Water Outlet Node Name
        \\type node
   N4 , \\field Heating Control Throttling Range
        \\units deltaC
        \\minimum 0.5
        \\default 0.5
   A8 , \\field Heating Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   N5 , \\field Maximum Cold Water Flow
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\ip-units gal/min
   A9 , \\field Cooling Water Inlet Node Name
        \\type node
   A10, \\field Cooling Water Outlet Node Name
        \\type node
   N6 , \\field Cooling Control Throttling Range
        \\units deltaC
        \\minimum 0.5
        \\default 0.5
   A11, \\field Cooling Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A12, \\field Condensation Control Type
        \\type choice
        \\key Off
        \\key SimpleOff
        \\key VariableOff
        \\default SimpleOff
   N7 , \\field Condensation Control Dewpoint Offset
        \\units C
        \\default 1.0
   A13, \\field Number of Circuits
        \\type choice
        \\key OnePerSurface
        \\key CalculateFromCircuitLength
        \\default OnePerSurface
   N8 ; \\field Circuit Length
        \\units m
        \\default 106.7

ZoneHVAC:LowTemperatureRadiant:ConstantFlow,
        \\memo Low temperature hydronic radiant heating and/or cooling system embedded in a building
        \\memo surface (wall, ceiling, or floor). Controlled by varying the hot or chilled water
        \\memo temperature circulating through the unit.
        \\min-fields 29
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Name of zone system is serving
        \\type object-list
        \\object-list ZoneNames
   A4 , \\field Surface Name or Radiant Surface Group Name
        \\note Identifies surfaces that radiant system is embedded in.
        \\note For a system with multiple surfaces, enter the name of
        \\note a ZoneHVAC:LowTemperatureRadiant:SurfaceGroup object.
        \\type object-list
        \\object-list RadGroupAndSurfNames
   N1 , \\field Hydronic Tubing Inside Diameter
        \\units m
        \\minimum> 0
        \\default 0.013
   N2 , \\field Hydronic Tubing Length
        \\note Total length of pipe embedded in surface
        \\units m
        \\minimum> 0
   A5 , \\field Temperature Control Type
        \\note Temperature used to control system
        \\type choice
        \\key MeanAirTemperature
        \\key MeanRadiantTemperature
        \\key OperativeTemperature
        \\key OutdoorDryBulbTemperature
        \\key OutdoorWetBulbTemperature
        \\default MeanAirTemperature
   N3 , \\field Rated Flow Rate
        \\units m3/s
   A6 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the Rated Flow Rate of the pump on a time basis
        \\note the default is that the pump is ON and runs according to its other
        \\note operational requirements specified above.  The schedule is for special
        \\note pump operations. Values here are between 0 and 1 and are multipliers
        \\note on the previous field (Rated Flow Rate).
        \\type object-list
        \\object-list ScheduleNames
   N4 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
   N5 , \\field Rated Power Consumption
        \\units W
   N6 , \\field Motor Efficiency
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.9
   N7 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
   A7 , \\field Heating Water Inlet Node Name
        \\type node
   A8 , \\field Heating Water Outlet Node Name
        \\type node
   A9 , \\field Heating High Water Temperature Schedule Name
        \\note Water and control temperatures for heating work together to provide
        \\note a linear function that determines the water temperature sent to the
        \\note radiant system.  The current control temperature (see Temperature Control Type above) is
        \\note compared to the high and low control temperatures at the current time.
        \\note If the control temperature is above the high temperature, then the
        \\note inlet water temperature is set to the low water temperature.  If the
        \\note control temperature is below the low temperature, then the inlet
        \\note water temperature is set to the high water temperature.  If the control
        \\note temperature is between the high and low value, then the inlet water
        \\note temperature is linearly interpolated between the low and high water
        \\note temperature values.
        \\type object-list
        \\object-list ScheduleNames
   A10, \\field Heating Low Water Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A11, \\field Heating High Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A12, \\field Heating Low Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A13, \\field Cooling Water Inlet Node Name
        \\type node
   A14, \\field Cooling Water Outlet Node Name
        \\type node
   A15, \\field Cooling High Water Temperature Schedule Name
        \\note See note for Heating High Water Temperature Schedule above for
        \\note interpretation information (or see the Input/Output Reference).
        \\type object-list
        \\object-list ScheduleNames
   A16, \\field Cooling Low Water Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A17, \\field Cooling High Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A18, \\field Cooling Low Control Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A19, \\field Condensation Control Type
        \\type choice
        \\key Off
        \\key SimpleOff
        \\key VariableOff
        \\default SimpleOff
   N8 , \\field Condensation Control Dewpoint Offset
        \\units C
        \\default 1.0
   A20, \\field Number of Circuits
        \\type choice
        \\key OnePerSurface
        \\key CalculateFromCircuitLength
        \\default OnePerSurface
   N9 ; \\field Circuit Length
        \\units m
        \\default 106.7

ZoneHVAC:LowTemperatureRadiant:Electric,
        \\memo Electric resistance low temperature radiant system
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Name of zone system is serving
        \\type object-list
        \\object-list ZoneNames
   A4 , \\field Surface Name or Radiant Surface Group Name
        \\note Identifies surfaces that radiant system is embedded in.
        \\note For a system with multiple surfaces, enter the name of
        \\note a ZoneHVAC:LowTemperatureRadiant:SurfaceGroup object.
        \\type object-list
        \\object-list RadGroupAndSurfNames
   N1 , \\field Maximum Electrical Power to Panel
        \\units W
        \\minimum 0
        \\autosizable
        \\ip-units W
   A5 , \\field Temperature Control Type
        \\note Temperature used to control unit
        \\type choice
        \\key MeanAirTemperature
        \\key MeanRadiantTemperature
        \\key OperativeTemperature
        \\key OutdoorDryBulbTemperature
        \\key OutdoorWetBulbTemperature
        \\default MeanAirTemperature
   N2 , \\field Heating Throttling Range
        \\units deltaC
        \\minimum 0
        \\default 0
   A6 ; \\field Heating Setpoint Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames

ZoneHVAC:LowTemperatureRadiant:SurfaceGroup,
        \\memo This is used to allow the coordinate control of several radiant system surfaces.
        \\memo Note that the following flow fractions must sum up to 1.0
        \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
        \\memo The number of surfaces can be expanded beyond 100, if necessary, by adding more
        \\memo groups to the end of the list
   A1 , \\field Name
        \\required-field
        \\type alpha
        \\reference RadGroupAndSurfNames
   A2 , \\field Surface 1 Name
        \\begin-extensible
        \\required-field
        \\type object-list
        \\object-list SurfaceNames
   N1 , \\field Flow Fraction for Surface 1
        \\required-field
        \\minimum 0.0
   A3 , \\field Surface 2 Name
        \\type object-list
        \\object-list SurfaceNames
   N2 , \\field Flow Fraction for Surface 2
        \\minimum 0.0
   A4 , \\field Surface 3 Name
        \\type object-list
        \\object-list SurfaceNames
   N3 , \\field Flow Fraction for Surface 3
        \\minimum 0.0
   A5 , \\field Surface 4 Name
        \\type object-list
        \\object-list SurfaceNames
   N4 , \\field Flow Fraction for Surface 4
        \\minimum 0.0
   A6 , \\field Surface 5 Name
        \\type object-list
        \\object-list SurfaceNames
   N5 , \\field Flow Fraction for Surface 5
        \\minimum 0.0
   A7 , \\field Surface 6 Name
        \\type object-list
        \\object-list SurfaceNames
   N6 , \\field Flow Fraction for Surface 6
        \\minimum 0.0
   A8 , \\field Surface 7 Name
        \\type object-list
        \\object-list SurfaceNames
   N7 , \\field Flow Fraction for Surface 7
        \\minimum 0.0
   A9 , \\field Surface 8 Name
        \\type object-list
        \\object-list SurfaceNames
   N8 , \\field Flow Fraction for Surface 8
        \\minimum 0.0
  A10 , \\field Surface 9 Name
        \\type object-list
        \\object-list SurfaceNames
   N9 , \\field Flow Fraction for Surface 9
        \\minimum 0.0
  A11 , \\field Surface 10 Name
        \\type object-list
        \\object-list SurfaceNames
  N10 , \\field Flow Fraction for Surface 10
        \\minimum 0.0
  A12 , \\field Surface 11 Name
        \\type object-list
        \\object-list SurfaceNames
  N11 , \\field Flow Fraction for Surface 11
        \\minimum 0.0
  A13 , \\field Surface 12 Name
        \\type object-list
        \\object-list SurfaceNames
  N12 , \\field Flow Fraction for Surface 12
        \\minimum 0.0
  A14 , \\field Surface 13 Name
        \\type object-list
        \\object-list SurfaceNames
  N13 , \\field Flow Fraction for Surface 13
        \\minimum 0.0
  A15 , \\field Surface 14 Name
        \\type object-list
        \\object-list SurfaceNames
  N14 , \\field Flow Fraction for Surface 14
        \\minimum 0.0
  A16 , \\field Surface 15 Name
        \\type object-list
        \\object-list SurfaceNames
  N15 , \\field Flow Fraction for Surface 15
        \\minimum 0.0
  A17 , \\field Surface 16 Name
        \\type object-list
        \\object-list SurfaceNames
  N16 , \\field Flow Fraction for Surface 16
        \\minimum 0.0
  A18 , \\field Surface 17 Name
        \\type object-list
        \\object-list SurfaceNames
  N17 , \\field Flow Fraction for Surface 17
        \\minimum 0.0
  A19 , \\field Surface 18 Name
        \\type object-list
        \\object-list SurfaceNames
  N18 , \\field Flow Fraction for Surface 18
        \\minimum 0.0
  A20 , \\field Surface 19 Name
        \\type object-list
        \\object-list SurfaceNames
  N19 , \\field Flow Fraction for Surface 19
        \\minimum 0.0
  A21 , \\field Surface 20 Name
        \\type object-list
        \\object-list SurfaceNames
  N20 , \\field Flow Fraction for Surface 20
        \\minimum 0.0
  A22 , \\field Surface 21 Name
        \\type object-list
        \\object-list SurfaceNames
  N21 , \\field Flow Fraction for Surface 21
        \\minimum 0.0
  A23 , \\field Surface 22 Name
        \\type object-list
        \\object-list SurfaceNames
  N22 , \\field Flow Fraction for Surface 22
        \\minimum 0.0
  A24 , \\field Surface 23 Name
        \\type object-list
        \\object-list SurfaceNames
  N23 , \\field Flow Fraction for Surface 23
        \\minimum 0.0
  A25 , \\field Surface 24 Name
        \\type object-list
        \\object-list SurfaceNames
  N24 , \\field Flow Fraction for Surface 24
        \\minimum 0.0
  A26 , \\field Surface 25 Name
        \\type object-list
        \\object-list SurfaceNames
  N25 , \\field Flow Fraction for Surface 25
        \\minimum 0.0
  A27 , \\field Surface 26 Name
        \\type object-list
        \\object-list SurfaceNames
  N26 , \\field Flow Fraction for Surface 26
        \\minimum 0.0
  A28 , \\field Surface 27 Name
        \\type object-list
        \\object-list SurfaceNames
  N27 , \\field Flow Fraction for Surface 27
        \\minimum 0.0
  A29 , \\field Surface 28 Name
        \\type object-list
        \\object-list SurfaceNames
  N28 , \\field Flow Fraction for Surface 28
        \\minimum 0.0
  A30 , \\field Surface 29 Name
        \\type object-list
        \\object-list SurfaceNames
  N29 , \\field Flow Fraction for Surface 29
        \\minimum 0.0
  A31 , \\field Surface 30 Name
        \\type object-list
        \\object-list SurfaceNames
  N30 , \\field Flow Fraction for Surface 30
        \\minimum 0.0
  A32 , \\field Surface 31 Name
        \\type object-list
        \\object-list SurfaceNames
  N31 , \\field Flow Fraction for Surface 31
        \\minimum 0.0
  A33 , \\field Surface 32 Name
        \\type object-list
        \\object-list SurfaceNames
  N32 , \\field Flow Fraction for Surface 32
        \\minimum 0.0
  A34 , \\field Surface 33 Name
        \\type object-list
        \\object-list SurfaceNames
  N33 , \\field Flow Fraction for Surface 33
        \\minimum 0.0
  A35 , \\field Surface 34 Name
        \\type object-list
        \\object-list SurfaceNames
  N34 , \\field Flow Fraction for Surface 34
        \\minimum 0.0
  A36 , \\field Surface 35 Name
        \\type object-list
        \\object-list SurfaceNames
  N35 , \\field Flow Fraction for Surface 35
        \\minimum 0.0
  A37 , \\field Surface 36 Name
        \\type object-list
        \\object-list SurfaceNames
  N36 , \\field Flow Fraction for Surface 36
        \\minimum 0.0
  A38 , \\field Surface 37 Name
        \\type object-list
        \\object-list SurfaceNames
  N37 , \\field Flow Fraction for Surface 37
        \\minimum 0.0
  A39 , \\field Surface 38 Name
        \\type object-list
        \\object-list SurfaceNames
  N38 , \\field Flow Fraction for Surface 38
        \\minimum 0.0
  A40 , \\field Surface 39 Name
        \\type object-list
        \\object-list SurfaceNames
  N39 , \\field Flow Fraction for Surface 39
        \\minimum 0.0
  A41 , \\field Surface 40 Name
        \\type object-list
        \\object-list SurfaceNames
  N40 , \\field Flow Fraction for Surface 40
        \\minimum 0.0
  A42 , \\field Surface 41 Name
        \\type object-list
        \\object-list SurfaceNames
  N41 , \\field Flow Fraction for Surface 41
        \\minimum 0.0
  A43 , \\field Surface 42 Name
        \\type object-list
        \\object-list SurfaceNames
  N42 , \\field Flow Fraction for Surface 42
        \\minimum 0.0
  A44 , \\field Surface 43 Name
        \\type object-list
        \\object-list SurfaceNames
  N43 , \\field Flow Fraction for Surface 43
        \\minimum 0.0
  A45 , \\field Surface 44 Name
        \\type object-list
        \\object-list SurfaceNames
  N44 , \\field Flow Fraction for Surface 44
        \\minimum 0.0
  A46 , \\field Surface 45 Name
        \\type object-list
        \\object-list SurfaceNames
  N45 , \\field Flow Fraction for Surface 45
        \\minimum 0.0
  A47 , \\field Surface 46 Name
        \\type object-list
        \\object-list SurfaceNames
  N46 , \\field Flow Fraction for Surface 46
        \\minimum 0.0
  A48 , \\field Surface 47 Name
        \\type object-list
        \\object-list SurfaceNames
  N47 , \\field Flow Fraction for Surface 47
        \\minimum 0.0
  A49 , \\field Surface 48 Name
        \\type object-list
        \\object-list SurfaceNames
  N48 , \\field Flow Fraction for Surface 48
        \\minimum 0.0
  A50 , \\field Surface 49 Name
        \\type object-list
        \\object-list SurfaceNames
  N49 , \\field Flow Fraction for Surface 49
        \\minimum 0.0
  A51 , \\field Surface 50 Name
        \\type object-list
        \\object-list SurfaceNames
  N50 , \\field Flow Fraction for Surface 50
        \\minimum 0.0
  A52 , \\field Surface 51 Name
        \\type object-list
        \\object-list SurfaceNames
  N51 , \\field Flow Fraction for Surface 51
        \\minimum 0.0
  A53 , \\field Surface 52 Name
        \\type object-list
        \\object-list SurfaceNames
  N52 , \\field Flow Fraction for Surface 52
        \\minimum 0.0
  A54 , \\field Surface 53 Name
        \\type object-list
        \\object-list SurfaceNames
  N53 , \\field Flow Fraction for Surface 53
        \\minimum 0.0
  A55 , \\field Surface 54 Name
        \\type object-list
        \\object-list SurfaceNames
  N54 , \\field Flow Fraction for Surface 54
        \\minimum 0.0
  A56 , \\field Surface 55 Name
        \\type object-list
        \\object-list SurfaceNames
  N55 , \\field Flow Fraction for Surface 55
        \\minimum 0.0
  A57 , \\field Surface 56 Name
        \\type object-list
        \\object-list SurfaceNames
  N56 , \\field Flow Fraction for Surface 56
        \\minimum 0.0
  A58 , \\field Surface 57 Name
        \\type object-list
        \\object-list SurfaceNames
  N57 , \\field Flow Fraction for Surface 57
        \\minimum 0.0
  A59 , \\field Surface 58 Name
        \\type object-list
        \\object-list SurfaceNames
  N58 , \\field Flow Fraction for Surface 58
        \\minimum 0.0
  A60 , \\field Surface 59 Name
        \\type object-list
        \\object-list SurfaceNames
  N59 , \\field Flow Fraction for Surface 59
        \\minimum 0.0
  A61 , \\field Surface 60 Name
        \\type object-list
        \\object-list SurfaceNames
  N60 , \\field Flow Fraction for Surface 60
        \\minimum 0.0
  A62 , \\field Surface 61 Name
        \\type object-list
        \\object-list SurfaceNames
  N61 , \\field Flow Fraction for Surface 61
        \\minimum 0.0
  A63 , \\field Surface 62 Name
        \\type object-list
        \\object-list SurfaceNames
  N62 , \\field Flow Fraction for Surface 62
        \\minimum 0.0
  A64 , \\field Surface 63 Name
        \\type object-list
        \\object-list SurfaceNames
  N63 , \\field Flow Fraction for Surface 63
        \\minimum 0.0
  A65 , \\field Surface 64 Name
        \\type object-list
        \\object-list SurfaceNames
  N64 , \\field Flow Fraction for Surface 64
        \\minimum 0.0
  A66 , \\field Surface 65 Name
        \\type object-list
        \\object-list SurfaceNames
  N65 , \\field Flow Fraction for Surface 65
        \\minimum 0.0
  A67 , \\field Surface 66 Name
        \\type object-list
        \\object-list SurfaceNames
  N66 , \\field Flow Fraction for Surface 66
        \\minimum 0.0
  A68 , \\field Surface 67 Name
        \\type object-list
        \\object-list SurfaceNames
  N67 , \\field Flow Fraction for Surface 67
        \\minimum 0.0
  A69 , \\field Surface 68 Name
        \\type object-list
        \\object-list SurfaceNames
  N68 , \\field Flow Fraction for Surface 68
        \\minimum 0.0
  A70 , \\field Surface 69 Name
        \\type object-list
        \\object-list SurfaceNames
  N69 , \\field Flow Fraction for Surface 69
        \\minimum 0.0
  A71 , \\field Surface 70 Name
        \\type object-list
        \\object-list SurfaceNames
  N70 , \\field Flow Fraction for Surface 70
        \\minimum 0.0
  A72 , \\field Surface 71 Name
        \\type object-list
        \\object-list SurfaceNames
  N71 , \\field Flow Fraction for Surface 71
        \\minimum 0.0
  A73 , \\field Surface 72 Name
        \\type object-list
        \\object-list SurfaceNames
  N72 , \\field Flow Fraction for Surface 72
        \\minimum 0.0
  A74 , \\field Surface 73 Name
        \\type object-list
        \\object-list SurfaceNames
  N73 , \\field Flow Fraction for Surface 73
        \\minimum 0.0
  A75 , \\field Surface 74 Name
        \\type object-list
        \\object-list SurfaceNames
  N74 , \\field Flow Fraction for Surface 74
        \\minimum 0.0
  A76 , \\field Surface 75 Name
        \\type object-list
        \\object-list SurfaceNames
  N75 , \\field Flow Fraction for Surface 75
        \\minimum 0.0
  A77 , \\field Surface 76 Name
        \\type object-list
        \\object-list SurfaceNames
  N76 , \\field Flow Fraction for Surface 76
        \\minimum 0.0
  A78 , \\field Surface 77 Name
        \\type object-list
        \\object-list SurfaceNames
  N77 , \\field Flow Fraction for Surface 77
        \\minimum 0.0
  A79 , \\field Surface 78 Name
        \\type object-list
        \\object-list SurfaceNames
  N78 , \\field Flow Fraction for Surface 78
        \\minimum 0.0
  A80 , \\field Surface 79 Name
        \\type object-list
        \\object-list SurfaceNames
  N79 , \\field Flow Fraction for Surface 79
        \\minimum 0.0
  A81 , \\field Surface 80 Name
        \\type object-list
        \\object-list SurfaceNames
  N80 , \\field Flow Fraction for Surface 80
        \\minimum 0.0
  A82 , \\field Surface 81 Name
        \\type object-list
        \\object-list SurfaceNames
  N81 , \\field Flow Fraction for Surface 81
        \\minimum 0.0
  A83 , \\field Surface 82 Name
        \\type object-list
        \\object-list SurfaceNames
  N82 , \\field Flow Fraction for Surface 82
        \\minimum 0.0
  A84 , \\field Surface 83 Name
        \\type object-list
        \\object-list SurfaceNames
  N83 , \\field Flow Fraction for Surface 83
        \\minimum 0.0
  A85 , \\field Surface 84 Name
        \\type object-list
        \\object-list SurfaceNames
  N84 , \\field Flow Fraction for Surface 84
        \\minimum 0.0
  A86 , \\field Surface 85 Name
        \\type object-list
        \\object-list SurfaceNames
  N85 , \\field Flow Fraction for Surface 85
        \\minimum 0.0
  A87 , \\field Surface 86 Name
        \\type object-list
        \\object-list SurfaceNames
  N86 , \\field Flow Fraction for Surface 86
        \\minimum 0.0
  A88 , \\field Surface 87 Name
        \\type object-list
        \\object-list SurfaceNames
  N87 , \\field Flow Fraction for Surface 87
        \\minimum 0.0
  A89 , \\field Surface 88 Name
        \\type object-list
        \\object-list SurfaceNames
  N88 , \\field Flow Fraction for Surface 88
        \\minimum 0.0
  A90 , \\field Surface 89 Name
        \\type object-list
        \\object-list SurfaceNames
  N89 , \\field Flow Fraction for Surface 89
        \\minimum 0.0
  A91 , \\field Surface 90 Name
        \\type object-list
        \\object-list SurfaceNames
  N90 , \\field Flow Fraction for Surface 90
        \\minimum 0.0
  A92 , \\field Surface 91 Name
        \\type object-list
        \\object-list SurfaceNames
  N91 , \\field Flow Fraction for Surface 91
        \\minimum 0.0
  A93 , \\field Surface 92 Name
        \\type object-list
        \\object-list SurfaceNames
  N92 , \\field Flow Fraction for Surface 92
        \\minimum 0.0
  A94 , \\field Surface 93 Name
        \\type object-list
        \\object-list SurfaceNames
  N93 , \\field Flow Fraction for Surface 93
        \\minimum 0.0
  A95 , \\field Surface 94 Name
        \\type object-list
        \\object-list SurfaceNames
  N94 , \\field Flow Fraction for Surface 94
        \\minimum 0.0
  A96 , \\field Surface 95 Name
        \\type object-list
        \\object-list SurfaceNames
  N95 , \\field Flow Fraction for Surface 95
        \\minimum 0.0
  A97 , \\field Surface 96 Name
        \\type object-list
        \\object-list SurfaceNames
  N96 , \\field Flow Fraction for Surface 96
        \\minimum 0.0
  A98 , \\field Surface 97 Name
        \\type object-list
        \\object-list SurfaceNames
  N97 , \\field Flow Fraction for Surface 97
        \\minimum 0.0
  A99 , \\field Surface 98 Name
        \\type object-list
        \\object-list SurfaceNames
  N98 , \\field Flow Fraction for Surface 98
        \\minimum 0.0
 A100 , \\field Surface 99 Name
        \\type object-list
        \\object-list SurfaceNames
  N99 , \\field Flow Fraction for Surface 99
        \\minimum 0.0
 A101 , \\field Surface 100 Name
        \\type object-list
        \\object-list SurfaceNames
 N100 ; \\field Flow Fraction for Surface 100
        \\minimum 0.0

ZoneHVAC:HighTemperatureRadiant,
        \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
        \\memo The number of surfaces can be expanded beyond 100, if necessary, by adding more
        \\memo groups to the end of the list
   A1,  \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Name of zone system is serving
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   N1 , \\field Maximum Power Input
        \\required-field
        \\units W
        \\minimum 0
        \\autosizable
   A4 , \\field Fuel Type
        \\note Natural gas or electricity
        \\required-field
        \\type choice
        \\key NaturalGas
        \\key Electricity
   N2 , \\field Combustion Efficiency
        \\note Not used for non-gas radiant heaters
        \\required-field
        \\minimum 0
        \\maximum 1
        \\default 0.9
   N3 , \\field Fraction of Input Converted to Radiant Energy
        \\required-field
        \\note Radiant+latent+lost fractions must sum to 1 or less, remainder is considered convective heat
        \\minimum 0
        \\maximum 1
        \\default 0.7
   N4 , \\field Fraction of Input Converted to Latent Energy
        \\required-field
        \\minimum 0
        \\maximum 1
        \\default 0.0
   N5 , \\field Fraction of Input that Is Lost
        \\note Fraction of input vented to outdoor environment
        \\required-field
        \\minimum 0
        \\maximum 1
        \\default 0.0
   A5 , \\field Temperature Control Type
        \\note Temperature type used to control unit
        \\type choice
        \\key MeanAirTemperature
        \\key MeanRadiantTemperature
        \\key OperativeTemperature
        \\key MeanAirTemperatureSetpoint
        \\key MeanRadiantTemperatureSetpoint
        \\key OperativeTemperatureSetpoint
        \\default OperativeTemperature
   N6 , \\field Heating Throttling Range
        \\required-field
        \\units deltaC
        \\minimum 0
        \\default 2.0
   A6 , \\field Heating Setpoint Temperature Schedule Name
        \\note This setpoint is an "operative temperature" setpoint
        \\type object-list
        \\object-list ScheduleNames
   N7,  \\field Fraction of Radiant Energy Incident on People
        \\note This will affect thermal comfort but from an energy balance standpoint this value
        \\note gets added to the convective gains from the radiant heater
        \\minimum 0
        \\maximum 1
   A7,  \\field Surface 1 Name
        \\begin-extensible
        \\note Radiant energy may be distributed to specific surfaces
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N8,  \\field Fraction of Radiant Energy to Surface 1
        \\minimum 0
        \\maximum 1
   A8,  \\field Surface 2 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N9,  \\field Fraction of Radiant Energy to Surface 2
        \\minimum 0
        \\maximum 1
   A9,  \\field Surface 3 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N10, \\field Fraction of Radiant Energy to Surface 3
        \\minimum 0
        \\maximum 1
   A10, \\field Surface 4 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N11, \\field Fraction of Radiant Energy to Surface 4
        \\minimum 0
        \\maximum 1
   A11, \\field Surface 5 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N12, \\field Fraction of Radiant Energy to Surface 5
        \\minimum 0
        \\maximum 1
   A12, \\field Surface 6 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N13, \\field Fraction of Radiant Energy to Surface 6
        \\minimum 0
        \\maximum 1
   A13, \\field Surface 7 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N14, \\field Fraction of Radiant Energy to Surface 7
        \\minimum 0
        \\maximum 1
   A14, \\field Surface 8 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N15, \\field Fraction of Radiant Energy to Surface 8
        \\minimum 0
        \\maximum 1
   A15, \\field Surface 9 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N16, \\field Fraction of Radiant Energy to Surface 9
        \\minimum 0
        \\maximum 1
   A16, \\field Surface 10 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N17, \\field Fraction of Radiant Energy to Surface 10
        \\minimum 0
        \\maximum 1
   A17, \\field Surface 11 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N18, \\field Fraction of Radiant Energy to Surface 11
        \\minimum 0
        \\maximum 1
   A18, \\field Surface 12 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N19, \\field Fraction of Radiant Energy to Surface 12
        \\minimum 0
        \\maximum 1
   A19, \\field Surface 13 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N20, \\field Fraction of Radiant Energy to Surface 13
        \\minimum 0
        \\maximum 1
   A20, \\field Surface 14 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N21, \\field Fraction of Radiant Energy to Surface 14
        \\minimum 0
        \\maximum 1
   A21, \\field Surface 15 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N22, \\field Fraction of Radiant Energy to Surface 15
        \\minimum 0
        \\maximum 1
   A22, \\field Surface 16 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N23, \\field Fraction of Radiant Energy to Surface 16
        \\minimum 0
        \\maximum 1
   A23, \\field Surface 17 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N24, \\field Fraction of Radiant Energy to Surface 17
        \\minimum 0
        \\maximum 1
   A24, \\field Surface 18 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N25, \\field Fraction of Radiant Energy to Surface 18
        \\minimum 0
        \\maximum 1
   A25, \\field Surface 19 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N26, \\field Fraction of Radiant Energy to Surface 19
        \\minimum 0
        \\maximum 1
   A26, \\field Surface 20 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N27, \\field Fraction of Radiant Energy to Surface 20
        \\minimum 0
        \\maximum 1
   A27, \\field Surface 21 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N28, \\field Fraction of Radiant Energy to Surface 21
        \\minimum 0
        \\maximum 1
   A28, \\field Surface 22 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N29, \\field Fraction of Radiant Energy to Surface 22
        \\minimum 0
        \\maximum 1
   A29, \\field Surface 23 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N30, \\field Fraction of Radiant Energy to Surface 23
        \\minimum 0
        \\maximum 1
   A30, \\field Surface 24 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N31, \\field Fraction of Radiant Energy to Surface 24
        \\minimum 0
        \\maximum 1
   A31, \\field Surface 25 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N32, \\field Fraction of Radiant Energy to Surface 25
        \\minimum 0
        \\maximum 1
   A32, \\field Surface 26 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N33, \\field Fraction of Radiant Energy to Surface 26
        \\minimum 0
        \\maximum 1
   A33, \\field Surface 27 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N34, \\field Fraction of Radiant Energy to Surface 27
        \\minimum 0
        \\maximum 1
   A34, \\field Surface 28 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N35, \\field Fraction of Radiant Energy to Surface 28
        \\minimum 0
        \\maximum 1
   A35, \\field Surface 29 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N36, \\field Fraction of Radiant Energy to Surface 29
        \\minimum 0
        \\maximum 1
   A36, \\field Surface 30 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N37, \\field Fraction of Radiant Energy to Surface 30
        \\minimum 0
        \\maximum 1
   A37, \\field Surface 31 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N38, \\field Fraction of Radiant Energy to Surface 31
        \\minimum 0
        \\maximum 1
   A38, \\field Surface 32 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N39, \\field Fraction of Radiant Energy to Surface 32
        \\minimum 0
        \\maximum 1
   A39, \\field Surface 33 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N40, \\field Fraction of Radiant Energy to Surface 33
        \\minimum 0
        \\maximum 1
   A40, \\field Surface 34 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N41, \\field Fraction of Radiant Energy to Surface 34
        \\minimum 0
        \\maximum 1
   A41, \\field Surface 35 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N42, \\field Fraction of Radiant Energy to Surface 35
        \\minimum 0
        \\maximum 1
   A42, \\field Surface 36 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N43, \\field Fraction of Radiant Energy to Surface 36
        \\minimum 0
        \\maximum 1
   A43, \\field Surface 37 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N44, \\field Fraction of Radiant Energy to Surface 37
        \\minimum 0
        \\maximum 1
   A44, \\field Surface 38 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N45, \\field Fraction of Radiant Energy to Surface 38
        \\minimum 0
        \\maximum 1
   A45, \\field Surface 39 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N46, \\field Fraction of Radiant Energy to Surface 39
        \\minimum 0
        \\maximum 1
   A46, \\field Surface 40 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N47, \\field Fraction of Radiant Energy to Surface 40
        \\minimum 0
        \\maximum 1
   A47, \\field Surface 41 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N48, \\field Fraction of Radiant Energy to Surface 41
        \\minimum 0
        \\maximum 1
   A48, \\field Surface 42 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N49, \\field Fraction of Radiant Energy to Surface 42
        \\minimum 0
        \\maximum 1
   A49, \\field Surface 43 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N50, \\field Fraction of Radiant Energy to Surface 43
        \\minimum 0
        \\maximum 1
   A50, \\field Surface 44 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N51, \\field Fraction of Radiant Energy to Surface 44
        \\minimum 0
        \\maximum 1
   A51, \\field Surface 45 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N52, \\field Fraction of Radiant Energy to Surface 45
        \\minimum 0
        \\maximum 1
   A52, \\field Surface 46 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N53, \\field Fraction of Radiant Energy to Surface 46
        \\minimum 0
        \\maximum 1
   A53, \\field Surface 47 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N54, \\field Fraction of Radiant Energy to Surface 47
        \\minimum 0
        \\maximum 1
   A54, \\field Surface 48 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N55, \\field Fraction of Radiant Energy to Surface 48
        \\minimum 0
        \\maximum 1
   A55, \\field Surface 49 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N56, \\field Fraction of Radiant Energy to Surface 49
        \\minimum 0
        \\maximum 1
   A56, \\field Surface 50 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N57, \\field Fraction of Radiant Energy to Surface 50
        \\minimum 0
        \\maximum 1
   A57, \\field Surface 51 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N58, \\field Fraction of Radiant Energy to Surface 51
        \\minimum 0
        \\maximum 1
   A58, \\field Surface 52 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N59, \\field Fraction of Radiant Energy to Surface 52
        \\minimum 0
        \\maximum 1
   A59, \\field Surface 53 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N60, \\field Fraction of Radiant Energy to Surface 53
        \\minimum 0
        \\maximum 1
   A60, \\field Surface 54 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N61, \\field Fraction of Radiant Energy to Surface 54
        \\minimum 0
        \\maximum 1
   A61, \\field Surface 55 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N62, \\field Fraction of Radiant Energy to Surface 55
        \\minimum 0
        \\maximum 1
   A62, \\field Surface 56 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N63, \\field Fraction of Radiant Energy to Surface 56
        \\minimum 0
        \\maximum 1
   A63, \\field Surface 57 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N64, \\field Fraction of Radiant Energy to Surface 57
        \\minimum 0
        \\maximum 1
   A64, \\field Surface 58 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N65, \\field Fraction of Radiant Energy to Surface 58
        \\minimum 0
        \\maximum 1
   A65, \\field Surface 59 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N66, \\field Fraction of Radiant Energy to Surface 59
        \\minimum 0
        \\maximum 1
   A66, \\field Surface 60 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N67, \\field Fraction of Radiant Energy to Surface 60
        \\minimum 0
        \\maximum 1
   A67, \\field Surface 61 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N68, \\field Fraction of Radiant Energy to Surface 61
        \\minimum 0
        \\maximum 1
   A68, \\field Surface 62 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N69, \\field Fraction of Radiant Energy to Surface 62
        \\minimum 0
        \\maximum 1
   A69, \\field Surface 63 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N70, \\field Fraction of Radiant Energy to Surface 63
        \\minimum 0
        \\maximum 1
   A70, \\field Surface 64 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N71, \\field Fraction of Radiant Energy to Surface 64
        \\minimum 0
        \\maximum 1
   A71, \\field Surface 65 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N72, \\field Fraction of Radiant Energy to Surface 65
        \\minimum 0
        \\maximum 1
   A72, \\field Surface 66 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N73, \\field Fraction of Radiant Energy to Surface 66
        \\minimum 0
        \\maximum 1
   A73, \\field Surface 67 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N74, \\field Fraction of Radiant Energy to Surface 67
        \\minimum 0
        \\maximum 1
   A74, \\field Surface 68 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N75, \\field Fraction of Radiant Energy to Surface 68
        \\minimum 0
        \\maximum 1
   A75, \\field Surface 69 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N76, \\field Fraction of Radiant Energy to Surface 69
        \\minimum 0
        \\maximum 1
   A76, \\field Surface 70 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N77, \\field Fraction of Radiant Energy to Surface 70
        \\minimum 0
        \\maximum 1
   A77, \\field Surface 71 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N78, \\field Fraction of Radiant Energy to Surface 71
        \\minimum 0
        \\maximum 1
   A78, \\field Surface 72 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N79, \\field Fraction of Radiant Energy to Surface 72
        \\minimum 0
        \\maximum 1
   A79, \\field Surface 73 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N80, \\field Fraction of Radiant Energy to Surface 73
        \\minimum 0
        \\maximum 1
   A80, \\field Surface 74 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N81, \\field Fraction of Radiant Energy to Surface 74
        \\minimum 0
        \\maximum 1
   A81, \\field Surface 75 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N82, \\field Fraction of Radiant Energy to Surface 75
        \\minimum 0
        \\maximum 1
   A82, \\field Surface 76 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N83, \\field Fraction of Radiant Energy to Surface 76
        \\minimum 0
        \\maximum 1
   A83, \\field Surface 77 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N84, \\field Fraction of Radiant Energy to Surface 77
        \\minimum 0
        \\maximum 1
   A84, \\field Surface 78 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N85, \\field Fraction of Radiant Energy to Surface 78
        \\minimum 0
        \\maximum 1
   A85, \\field Surface 79 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N86, \\field Fraction of Radiant Energy to Surface 79
        \\minimum 0
        \\maximum 1
   A86, \\field Surface 80 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N87, \\field Fraction of Radiant Energy to Surface 80
        \\minimum 0
        \\maximum 1
   A87, \\field Surface 81 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N88, \\field Fraction of Radiant Energy to Surface 81
        \\minimum 0
        \\maximum 1
   A88, \\field Surface 82 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N89, \\field Fraction of Radiant Energy to Surface 82
        \\minimum 0
        \\maximum 1
   A89, \\field Surface 83 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N90, \\field Fraction of Radiant Energy to Surface 83
        \\minimum 0
        \\maximum 1
   A90, \\field Surface 84 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N91, \\field Fraction of Radiant Energy to Surface 84
        \\minimum 0
        \\maximum 1
   A91, \\field Surface 85 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N92, \\field Fraction of Radiant Energy to Surface 85
        \\minimum 0
        \\maximum 1
   A92, \\field Surface 86 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N93, \\field Fraction of Radiant Energy to Surface 86
        \\minimum 0
        \\maximum 1
   A93, \\field Surface 87 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N94, \\field Fraction of Radiant Energy to Surface 87
        \\minimum 0
        \\maximum 1
   A94, \\field Surface 88 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N95, \\field Fraction of Radiant Energy to Surface 88
        \\minimum 0
        \\maximum 1
   A95, \\field Surface 89 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N96, \\field Fraction of Radiant Energy to Surface 89
        \\minimum 0
        \\maximum 1
   A96, \\field Surface 90 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N97, \\field Fraction of Radiant Energy to Surface 90
        \\minimum 0
        \\maximum 1
   A97, \\field Surface 91 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N98, \\field Fraction of Radiant Energy to Surface 91
        \\minimum 0
        \\maximum 1
   A98, \\field Surface 92 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
   N99, \\field Fraction of Radiant Energy to Surface 92
        \\minimum 0
        \\maximum 1
   A99, \\field Surface 93 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N100, \\field Fraction of Radiant Energy to Surface 93
        \\minimum 0
        \\maximum 1
  A100, \\field Surface 94 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N101, \\field Fraction of Radiant Energy to Surface 94
        \\minimum 0
        \\maximum 1
  A101, \\field Surface 95 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N102, \\field Fraction of Radiant Energy to Surface 95
        \\minimum 0
        \\maximum 1
  A102, \\field Surface 96 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N103, \\field Fraction of Radiant Energy to Surface 96
        \\minimum 0
        \\maximum 1
  A103, \\field Surface 97 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N104, \\field Fraction of Radiant Energy to Surface 97
        \\minimum 0
        \\maximum 1
  A104, \\field Surface 98 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N105, \\field Fraction of Radiant Energy to Surface 98
        \\minimum 0
        \\maximum 1
  A105, \\field Surface 99 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N106, \\field Fraction of Radiant Energy to Surface 99
        \\minimum 0
        \\maximum 1
  A106, \\field Surface 100 Name
        \\type object-list
        \\object-list AllHeatTranSurfNames
  N107; \\field Fraction of Radiant Energy to Surface 100
        \\minimum 0
        \\maximum 1

ZoneHVAC:VentilatedSlab,
       \\memo Ventilated slab system where outdoor air flows through hollow cores in a building
       \\memo surface (wall, ceiling, or floor).
   A1, \\field Name
       \\required-field
   A2, \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
   A3, \\field Zone Name
       \\required-field
       \\note (name of zone system is serving)
       \\type object-list
       \\object-list ZoneNames
   A4 ,\\field Surface Name or Radiant Surface Group Name
       \\note (name of surface system is embedded in) or list of surfaces
       \\type object-list
       \\object-list RadGroupAndSurfNames
   N1, \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum> 0
       \\autosizable
   A5, \\field Outdoor Air Control Type
       \\required-field
       \\type choice
       \\key VariablePercent
       \\key FixedTemperature
       \\key FixedAmount
   N2, \\field Minimum Outdoor Air Flow Rate
       \\required-field
       \\units m3/s
       \\autosizable
       \\minimum 0.0
   A6, \\field Minimum Outdoor Air Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
   N3, \\field Maximum Outdoor Air Flow Rate
       \\required-field
       \\units m3/s
       \\autosizable
       \\minimum 0.0
       \\note schedule values multiply the minimum outdoor air flow rate
   A7, \\field Maximum Outdoor Air Fraction or Temperature Schedule Name
       \\note Note that this depends on the control type as to whether schedule values are a fraction or temperature
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
   A8, \\field System Configuration Type
       \\required-field
       \\type choice
       \\key SlabOnly
       \\key SlabAndZone
       \\key SeriesSlabs
       \\default SlabOnly
   N4, \\field Hollow Core Inside Diameter
       \\units m
       \\minimum 0
       \\default 0.05
       \\ip-units in
   N5, \\field Hollow Core Length
       \\note (length of core cavity embedded in surface)
       \\units m
       \\minimum 0
   N6, \\field Number of Cores
       \\note flow will be divided evenly among the cores
       \\minimum 0
   A9, \\field Temperature Control Type
       \\required-field
       \\note (temperature on which unit is controlled)
       \\type choice
       \\key MeanAirTemperature
       \\key MeanRadiantTemperature
       \\key OperativeTemperature
       \\key OutdoorDryBulbTemperature
       \\key OutdoorWetBulbTemperature
       \\key SurfaceTemperature
       \\key ZoneAirDewPointTemperature
       \\default OutdoorDryBulbTemperature
   A10, \\field Heating High Air Temperature Schedule Name
       \\required-field
       \\note Air and control temperatures for heating work together to provide
       \\note a linear function that determines the air temperature sent to the
       \\note radiant system. The current control temperature (see A14) is
       \\note compared to the high and low control temperatures at the current time.
       \\note If the control temperature is above the high temperature, then the
       \\note inlet air temperature is set to the low air temperature. If the
       \\note control temperature is below the low temperature, then the inlet
       \\note air temperature is set to the high air temperature. If the control
       \\note temperature is between the high and low value, then the inlet air
       \\note temperature is linearly interpolated between the low and high air
       \\note temperature values.
       \\type object-list
       \\object-list ScheduleNames
  A11, \\field Heating Low Air Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A12, \\field Heating High Control Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A13, \\field Heating Low Control Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A14, \\field Cooling High Air Temperature Schedule Name
       \\required-field
       \\note See note for heating high air temperature schedule above for
       \\note interpretation information (or see the Input/Output Reference).
       \\type object-list
       \\object-list ScheduleNames
  A15, \\field Cooling Low Air Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A16, \\field Cooling High Control Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A17, \\field Cooling Low Control Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A18, \\field Return Air Node Name
       \\note This is the zone return air inlet to the ventilated slab system outdoor air mixer.
       \\note This node is typically a zone exhaust node (do not connect to "Zone Return Air Node").
       \\required-field
       \\type node
  A19, \\field Slab In Node Name
       \\note This is the node entering the slab or series of slabs after the fan and coil(s).
       \\required-field
       \\type node
  A20, \\field Zone Supply Air Node Name
       \\note This is the node name exiting the slab.
       \\note This node is typically a zone inlet node.
       \\note Leave blank when the system configuration is SlabOnly or SeriesSlabs.
       \\type node
  A21, \\field Outdoor Air Node Name
       \\note This node is the outdoor air inlet to the ventilated slab oa mixer.
       \\note This node should also be specified in an OutdoorAir:Node or OutdoorAir:NodeList object.
       \\required-field
       \\type node
  A22, \\field Relief Air Node Name
       \\note This node is the relief air node from the ventilated slab outdoor air mixer.
       \\required-field
       \\type node
  A23, \\field Outdoor Air Mixer Outlet Node Name
       \\note This is the node name leaving the outdoor air mixer and entering the fan and coil(s).
       \\required-field
       \\type node
  A24, \\field Fan Outlet Node Name
       \\note This is the node name of the fan outlet.
       \\required-field
       \\type node
  A25, \\field Fan Name
       \\required-field
       \\note Allowable fan type is Fan:ConstantVolume
       \\type object-list
       \\object-list FansCV
  A26, \\field Coil Option Type
       \\required-field
       \\type choice
       \\key None
       \\key Heating
       \\key Cooling
       \\key HeatingAndCooling
  A27, \\field Heating Coil Object Type
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A28, \\field Heating Coil Name
       \\type object-list
       \\object-list HeatingCoilName
  A29, \\field Hot Water or Steam Inlet Node Name
       \\type node
  A30, \\field Cooling Coil Object Type
       \\type choice
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
       \\key CoilSystem:Cooling:Water:HeatExchangerAssisted
  A31, \\field Cooling Coil Name
       \\type object-list
       \\object-list CoolingCoilsWater
  A32, \\field Cold Water Inlet Node Name
       \\type node
  A33; \\field Availability Manager List Name
        \\note Enter the name of an AvailabilityManagerAssignmentList object.
        \\type object-list
        \\object-list SystemAvailabilityManagerLists

ZoneHVAC:VentilatedSlab:SlabGroup,
        \\memo This is used to allow the coordinate control of several ventilad slab system
        \\memo surfaces. Note that the flow fractions must sum up to 1.0.
        \\memo The number of surfaces can be expanded beyond 10, if necessary, by adding more
        \\memo groups to the end of the list
        \\extensible:7 - repeat last seven fields, remembering to remove ; from "inner" fields.
   A1 , \\field Name
         \\required-field
         \\type alpha
         \\reference RadGroupAndSurfNames
   A2  , \\field Zone 1 Name
         \\begin-extensible
         \\required-field
         \\type object-list
         \\object-list ZoneNames
   A3  , \\field Surface 1 Name
         \\required-field
         \\type object-list
         \\object-list SurfaceNames
   N1  , \\field Core Diameter for Surface 1
         \\required-field
         \\minimum 0.0
         \\type real
         \\units m
   N2  , \\field Core Length for Surface 1
         \\required-field
         \\minimum 0.0
         \\type real
         \\units m
   N3  , \\field Core Numbers for Surface 1
         \\required-field
         \\minimum 0.0
   A4  , \\field Slab Inlet Node Name for Surface 1
         \\required-field
         \\type node
   A5  , \\field Slab Outlet Node Name for Surface 1
         \\required-field
         \\type node
   A6  , \\field Zone 2 Name
         \\type object-list
         \\object-list ZoneNames
   A7  , \\field Surface 2 Name
         \\type object-list
         \\object-list SurfaceNames
   N4  , \\field Core Diameter for Surface 2
         \\type real
         \\units m
         \\minimum 0.0
   N5  , \\field Core Length for Surface 2
         \\type real
         \\units m
         \\minimum 0.0
   N6  , \\field Core Numbers for Surface 2
         \\minimum 0.0
   A8  , \\field Slab Inlet Node Name for Surface 2
         \\type node
   A9  , \\field Slab Outlet Node Name for Surface 2
         \\type node
   A10  , \\field Zone 3 Name
         \\type object-list
         \\object-list ZoneNames
   A11  , \\field Surface 3 Name
         \\type object-list
         \\object-list SurfaceNames
   N7  , \\field Core Diameter for Surface 3
         \\type real
         \\units m
         \\minimum 0.0
   N8  , \\field Core Length for Surface 3
         \\type real
         \\units m
         \\minimum 0.0
   N9  , \\field Core Numbers for Surface 3
         \\minimum 0.0
   A12  , \\field Slab Inlet Node Name for Surface 3
          \\type node
   A13  , \\field Slab Outlet Node Name for Surface 3
          \\type node
   A14  , \\field Zone 4 Name
         \\type object-list
         \\object-list ZoneNames
   A15  , \\field Surface 4 Name
         \\type object-list
         \\object-list SurfaceNames
   N10  , \\field Core Diameter for Surface 4
         \\type real
         \\units m
         \\minimum 0.0
   N11  , \\field Core Length for Surface 4
         \\type real
         \\units m
         \\minimum 0.0
   N12  , \\field Core Numbers for Surface 4
         \\minimum 0.0
   A16  , \\field Slab Inlet Node Name for Surface 4
          \\type node
   A17  , \\field Slab Outlet Node Name for Surface 4
          \\type node
   A18  , \\field Zone 5 Name
         \\type object-list
         \\object-list ZoneNames
   A19  , \\field Surface 5 Name
         \\type object-list
         \\object-list SurfaceNames
   N13  , \\field Core Diameter for Surface 5
         \\type real
         \\units m
         \\minimum 0.0
   N14  , \\field Core Length for Surface 5
         \\type real
         \\units m
         \\minimum 0.0
   N15  , \\field Core Numbers for Surface 5
         \\minimum 0.0
   A20  , \\field Slab Inlet Node Name for Surface 5
          \\type node
   A21  , \\field Slab Outlet Node Name for Surface 5
          \\type node
   A22  , \\field Zone 6 Name
         \\type object-list
         \\object-list ZoneNames
   A23  , \\field Surface 6 Name
         \\type object-list
         \\object-list SurfaceNames
   N16  , \\field Core Diameter for Surface 6
         \\type real
         \\units m
         \\minimum 0.0
   N17  , \\field Core Length for Surface 6
         \\type real
         \\units m
         \\minimum 0.0
   N18  , \\field Core Numbers for Surface 6
         \\minimum 0.0
   A24  , \\field Slab Inlet Node Name for Surface 6
          \\type node
   A25  , \\field Slab Outlet Node Name for Surface 6
          \\type node
   A26  , \\field Zone 7 Name
         \\type object-list
         \\object-list ZoneNames
   A27  , \\field Surface 7 Name
         \\type object-list
         \\object-list SurfaceNames
   N19  , \\field Core Diameter for Surface 7
         \\type real
         \\units m
         \\minimum 0.0
   N20  , \\field Core Length for Surface 7
         \\type real
         \\units m
         \\minimum 0.0
   N21  , \\field Core Numbers for Surface 7
         \\minimum 0.0
   A28  , \\field Slab Inlet Node Name for Surface 7
          \\type node
   A29  , \\field Slab Outlet Node Name for Surface 7
          \\type node
   A30  , \\field Zone 8 Name
         \\type object-list
         \\object-list ZoneNames
   A31  , \\field Surface 8 Name
         \\type object-list
         \\object-list SurfaceNames
   N22  , \\field Core Diameter for Surface 8
         \\type real
         \\units m
         \\minimum 0.0
   N23  , \\field Core Length for Surface 8
         \\type real
         \\units m
         \\minimum 0.0
   N24  , \\field Core Numbers for Surface 8
         \\minimum 0.0
   A32  , \\field Slab Inlet Node Name for Surface 8
          \\type node
   A33  , \\field Slab Outlet Node Name for Surface 8
          \\type node
   A34  , \\field Zone 9 Name
         \\type object-list
         \\object-list ZoneNames
   A35  , \\field Surface 9 Name
         \\type object-list
         \\object-list SurfaceNames
   N25  , \\field Core Diameter for Surface 9
         \\type real
         \\units m
         \\minimum 0.0
   N26  , \\field Core Length for Surface 9
         \\type real
         \\units m
         \\minimum 0.0
   N27  , \\field Core Numbers for Surface 9
         \\minimum 0.0
   A36  , \\field Slab Inlet Node Name for Surface 9
          \\type node
   A37  , \\field Slab Outlet Node Name for Surface 9
          \\type node
   A38  , \\field Zone 10 Name
         \\type object-list
         \\object-list ZoneNames
   A39  , \\field Surface 10 Name
         \\type object-list
         \\object-list SurfaceNames
   N28  , \\field Core Diameter for Surface 10
         \\type real
         \\units m
         \\minimum 0.0
   N29  , \\field Core Length for Surface 10
         \\type real
         \\units m
         \\minimum 0.0
   N30  , \\field Core Numbers for Surface 10
         \\minimum 0.0
   A40  , \\field Slab Inlet Node Name for Surface 10
          \\type node
   A41  ; \\field Slab Outlet Node Name for Surface 10
          \\type node

\\group Zone HVAC Air Loop Terminal Units

AirTerminal:SingleDuct:Uncontrolled,
       \\memo Central air system terminal unit, single duct, constant volume, no controls other than
       \\memo on/off schedule.
       \\min-fields 4
   A1, \\field Name
       \\required-field
   A2, \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
   A3, \\field Zone Supply Air Node Name
       \\required-field
       \\type node
   N1; \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable

AirTerminal:SingleDuct:ConstantVolume:Reheat,
       \\memo Central air system terminal unit, single duct, constant volume, with reheat coil (hot
       \\memo water, electric, gas, or steam).
  A1 , \\field Name
       \\required-field
       \\reference AFNTerminalUnitNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Air Outlet Node Name
       \\required-field
       \\type node
  A4,  \\field Air Inlet Node Name
       \\required-field
       \\type node
  N1,  \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  A5,  \\field Hot Water or Steam Inlet Node Name
       \\deprecated
       \\type node
  A6,  \\field Reheat Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A7,  \\field Reheat Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N2,  \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\ip-units gal/min
  N3,  \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  N4,  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  N5; \\field Maximum Reheat Air Temperature
      \\note Specifies the maximum allowable supply air temperature leaving the reheat coil.
      \\note If left blank, there is no limit and no default. If unknown, 35C (95F) is recommended.
      \\type real
      \\units C
      \\minimum> 0.0

AirTerminal:SingleDuct:VAV:NoReheat,
       \\memo Central air system terminal unit, single duct, variable volume, with no reheat coil.
  A1,  \\field Name
       \\required-field
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Air Outlet Node Name
       \\required-field
       \\type node
  A4,  \\field Air Inlet Node Name
       \\required-field
       \\type node
  N1,  \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  A5 , \\field Zone Minimum Air Flow Input Method
       \\required-field
       \\type choice
       \\key Constant
       \\key FixedFlowRate
       \\key Scheduled
       \\note Constant = Constant Minimum Air Flow Fraction (a fraction of Maximum Air Flow Rate)
       \\note FixedFlowRate = Fixed Minimum Air Flow Rate (a fixed minimum air volume flow rate)
       \\note Scheduled = Scheduled Minimum Air Flow Fraction (a fraction of Maximum Air Flow
  N2 , \\field Constant Minimum Air Flow Fraction
       \\note This field is used if the field Zone Minimum Air Flow Input Method is Constant
       \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
       \\note is optional; if a value is entered, then it is used for sizing normal-action reheat coils.
       \\note If both this field and the following field are entered, the larger result is used.
  N3 , \\field Fixed Minimum Air Flow Rate
       \\type real
       \\units m3/s
       \\note This field is used if the field Zone Minimum Air Flow Input Method is FixedFlowRate.
       \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
       \\note is optional; if a value is entered, then it is used for sizing normal-action reheat coils.
       \\note If both this field and the previous field are entered, the larger result is used.
  A6 , \\field Minimum Air Flow Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is used if the field Zone Minimum Air Flow Input Method is Scheduled
       \\note Schedule values are fractions, 0.0 to 1.0.
       \\note If the field Constant Minimum Air Flow Fraction is blank, then the average of the
       \\note minimum and maximum schedule values is used for sizing normal-action reheat coils.
  A7 ; \\field Design Specification Outdoor Air Object Name
       \\type object-list
       \\object-list DesignSpecificationOutdoorAirNames
       \\note When the name of a DesignSpecification:OutdoorAir object is entered, the terminal
       \\note unit will increase flow as needed to meet this outdoor air requirement.
       \\note If Outdoor Air Flow per Person is non-zero, then the outdoor air requirement will
       \\note be computed based on the current number of occupants in the zone.
       \\note At no time will the supply air flow rate exceed the value for Maximum Air Flow Rate.
       \\note If this field is blank, then the terminal unit will not be controlled for outdoor air flow.

AirTerminal:SingleDuct:VAV:Reheat,
       \\memo Central air system terminal unit, single duct, variable volume, with reheat coil (hot
       \\memo water, electric, gas, or steam).
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Damper Air Outlet Node Name
       \\note the outlet node of the damper and the inlet node of the reheat coil
       \\note this is an internal node to the terminal unit and connects the damper and reheat coil
       \\required-field
       \\type node
  A4 , \\field Air Inlet Node Name
       \\note the inlet node to the terminal unit and the damper
       \\required-field
       \\type node
  N1 , \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  A5 , \\field Zone Minimum Air Flow Input Method
       \\required-field
       \\type choice
       \\key Constant
       \\key FixedFlowRate
       \\key Scheduled
       \\note Constant = Constant Minimum Air Flow Fraction (a fraction of Maximum Air Flow Rate)
       \\note FixedFlowRate = Fixed Minimum Air Flow Rate (a fixed minimum air volume flow rate)
       \\note Scheduled = Scheduled Minimum Air Flow Fraction (a fraction of Maximum Air Flow
  N2 , \\field Constant Minimum Air Flow Fraction
       \\note This field is used if the field Zone Minimum Air Flow Input Method is Constant
       \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
       \\note is optional; if a value is entered, then it is used for sizing normal-action reheat coils.
       \\note If both this field and the following field are entered, the larger result is used.
  N3 , \\field Fixed Minimum Air Flow Rate
       \\type real
       \\units m3/s
       \\note This field is used if the field Zone Minimum Air Flow Input Method is FixedFlowRate.
       \\note If the field Zone Minimum Air Flow Input Method is Scheduled, then this field
       \\note is optional; if a value is entered, then it is used for sizing normal-action reheat coils.
       \\note If both this field and the previous field are entered, the larger result is used.
  A6 , \\field Minimum Air Flow Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is used if the field Zone Minimum Air Flow Input Method is Scheduled
       \\note Schedule values are fractions, 0.0 to 1.0.
       \\note If the field Constant Minimum Air Flow Fraction is blank, then the average of the
       \\note minimum and maximum schedule values is used for sizing normal-action reheat coils.
  A7 , \\field Reheat Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A8 , \\field Reheat Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N4 , \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\ip-units gal/min
  N5 , \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  A9 , \\field Air Outlet Node Name
       \\required-field
       \\type node
       \\note The outlet node of the terminal unit and the reheat coil.
       \\note This is also the zone inlet node.
  N6 , \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  A10, \\field Damper Heating Action
       \\type choice
       \\key Normal
       \\key Reverse
       \\default Normal
  N7 , \\field Maximum Flow per Zone Floor Area During Reheat
       \\type real
       \\units m3/s-m2
       \\autocalculatable
       \\note Used only when Reheat Coil Object Type = Coil:Heating:Water and Damper Heating Action = Reverse
       \\note When autocalculating, the maximum flow per zone is set to 0.002032 m3/s-m2 (0.4 cfm/sqft)
       \\note This optional field limits the maximum flow allowed in reheat mode.
       \\note If this field and the following field are left blank, the maximum flow will not be limited.
       \\note At no time will the maximum flow rate calculated here exceed the value of
       \\note Maximum Air Flow Rate.
  N8 , \\field Maximum Flow Fraction During Reheat
       \\type real
       \\autocalculatable
       \\note Used only when Reheat Coil Object Type = Coil:Heating:Water and Damper Heating Action = Reverse
       \\note When autocalculating, the maximum flow fraction is set to the ratio of
       \\note 0.002032 m3/s-m2 (0.4 cfm/sqft) multiplied by the zone floor area and the
       \\note Maximum Air Flow Rate.
       \\note This optional field limits the maximum flow allowed in reheat mode.
       \\note If this field and the previous field are left blank, the maximum flow will not be limited.
       \\note At no time will the maximum flow rate calculated here exceed the value of
       \\note Maximum Air Flow Rate.
  N9 , \\field Maximum Reheat Air Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\note Specifies the maximum allowable supply air temperature leaving the reheat coil.
       \\note If left blank, there is no limit and no default. If unknown, 35C (95F) is recommended.
  A11; \\field Design Specification Outdoor Air Object Name
       \\type object-list
       \\object-list DesignSpecificationOutdoorAirNames
       \\note When the name of a DesignSpecification:OutdoorAir object is entered, the terminal
       \\note unit will increase flow as needed to meet this outdoor air requirement.
       \\note If Outdoor Air Flow per Person is non-zero, then the outdoor air requirement will
       \\note be computed based on the current number of occupants in the zone.
       \\note At no time will the supply air flow rate exceed the value for Maximum Air Flow Rate.
       \\note If this field is blank, then the terminal unit will not be controlled for outdoor air flow.

AirTerminal:SingleDuct:VAV:Reheat:VariableSpeedFan,
       \\memo Central air system terminal unit, single duct, variable volume, with reheat coil (hot
       \\memo water, electric, gas, or steam) and variable-speed fan. These units are usually
       \\memo employed in underfloor air distribution (UFAD) systems where the air is supplied at
       \\memo low static pressure through an underfloor plenum. The fan is used to control the flow
       \\memo of conditioned air that enters the space.
  A1,  \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1,  \\field Maximum Cooling Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum 0.0
  N2,  \\field Maximum Heating Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum 0.0
  N3,  \\field Zone Minimum Air Flow Fraction
       \\required-field
       \\minimum 0.0
       \\maximum 1.0
       \\note fraction of cooling air flow rate
  A3,  \\field Air Inlet Node Name
       \\type node
       \\deprecated
  A4,  \\field Air Outlet Node Name
       \\type node
       \\deprecated
  A5,  \\field Heating Coil Air Inlet Node Name
       \\type node
       \\deprecated
  A6,  \\field Hot Water or Steam Inlet Node Name
       \\type node
       \\deprecated
  A7,  \\field Fan Object Type
       \\required-field
       \\type choice
       \\key Fan:VariableVolume
  A8,  \\field Fan Name
       \\required-field
       \\type object-list
       \\object-list FansVAV
  A9,  \\field Heating Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A10, \\field Heating Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N4,  \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when heating coil type is gas or electric
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N5,  \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when heating coil type is gas or electric
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  N6;  \\field Heating Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001

AirTerminal:SingleDuct:VAV:HeatAndCool:NoReheat,
       \\memo Central air system terminal unit, single duct, variable volume for both cooling and
       \\memo heating, with no reheat coil.
       \\min-fields 6
  A1,  \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Air Outlet Node Name
       \\required-field
       \\type node
       \\note The outlet node of the terminal unit.
       \\note This is also the zone inlet node.
  A4,  \\field Air Inlet Node Name
       \\required-field
       \\type node
  N1,  \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N2;  \\field Zone Minimum Air Flow Fraction
       \\required-field
       \\minimum 0.0
       \\maximum 1.0
       \\note fraction of maximum air flow

AirTerminal:SingleDuct:VAV:HeatAndCool:Reheat,
       \\memo Central air system terminal unit, single duct, variable volume for both cooling and
       \\memo heating, with reheat coil (hot water, electric, gas, or steam).
       \\min-fields 12
  A1,  \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Damper Air Outlet Node Name
       \\required-field
       \\type node
       \\note the outlet node of the damper and the inlet node of the reheat coil
       \\note this is an internal node to the terminal unit and connects the damper and reheat coil
  A4,  \\field Air Inlet Node Name
       \\required-field
       \\type node
       \\note the inlet node to the terminal unit and the damper
  N1,  \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N2,  \\field Zone Minimum Air Flow Fraction
       \\required-field
       \\minimum 0.0
       \\maximum 1.0
       \\note fraction of maximum air flow
  A5,  \\field Hot Water or Steam Inlet Node Name
       \\deprecated
       \\type node
  A6,  \\field Reheat Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A7,  \\field Reheat Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N3,  \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\ip-units gal/min
  N4,  \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  A8,  \\field Air Outlet Node Name
       \\required-field
       \\type node
       \\note The outlet node of the terminal unit and the reheat coil.
       \\note This is also the zone inlet node.
  N5,  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  N6; \\field Maximum Reheat Air Temperature
      \\note Specifies the maximum allowable supply air temperature leaving the reheat coil.
      \\note If left blank, there is no limit and no default. If unknown, 35C (95F) is recommended.
      \\type real
      \\units C
      \\minimum> 0.0

AirTerminal:SingleDuct:SeriesPIU:Reheat,
       \\memo Central air system terminal unit, single duct, variable volume, series powered
       \\memo induction unit (PIU), with reheat coil (hot water, electric, gas, or steam).
  A1,  \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1,  \\field Maximum Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N2,  \\field Maximum Primary Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N3,  \\field Minimum Primary Air Flow Fraction
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\autosizable
  A3,  \\field Supply Air Inlet Node Name
       \\type node
  A4,  \\field Secondary Air Inlet Node Name
       \\type node
  A5,  \\field Outlet Node Name
       \\type node
  A6,  \\field Reheat Coil Air Inlet Node Name
       \\type node
  A7,  \\field Zone Mixer Name
       \\type object-list
       \\object-list ZoneMixers
  A8,  \\field Fan Name
       \\note Fan type must be Fan:ConstantVolume
       \\type object-list
       \\object-list FansCV
  A9,  \\field Reheat Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A10, \\field Reheat Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N4,  \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N5,  \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  A11, \\field Hot Water or Steam Inlet Node Name
       \\type node
       \\deprecated
  N6;  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001

AirTerminal:SingleDuct:ParallelPIU:Reheat,
       \\memo Central air system terminal unit, single duct, variable volume, parallel powered
       \\memo induction unit (PIU), with reheat coil (hot water, electric, gas, or steam).
  A1,  \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1,  \\field Maximum Primary Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N2,  \\field Maximum Secondary Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N3,  \\field Minimum Primary Air Flow Fraction
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\autosizable
  N4,  \\field Fan On Flow Fraction
       \\required-field
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\autosizable
       \\note the fraction of the primary air flow at which fan turns on
  A3,  \\field Supply Air Inlet Node Name
       \\type node
  A4,  \\field Secondary Air Inlet Node Name
       \\type node
  A5,  \\field Outlet Node Name
       \\type node
  A6,  \\field Reheat Coil Air Inlet Node Name
       \\type node
       \\note mixer outlet node
  A7,  \\field Zone Mixer Name
       \\type object-list
       \\object-list ZoneMixers
  A8,  \\field Fan Name
       \\note Fan type must be Fan:ConstantVolume
       \\type object-list
       \\object-list FansCV
  A9,  \\field Reheat Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Steam
  A10, \\field Reheat Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N5,  \\field Maximum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N6,  \\field Minimum Hot Water or Steam Flow Rate
       \\note Not used when reheat coil type is gas or electric
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  A11, \\field Hot Water or Steam Inlet Node Name
       \\type node
       \\deprecated
  N7;  \\field Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001

AirTerminal:SingleDuct:ConstantVolume:FourPipeInduction,
       \\memo Central air system terminal unit, single duct, variable volume, induction unit with
       \\memo hot water reheat coil and chilled water recool coil.
  A1,  \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1,  \\field Maximum Total Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum 0.0
  N2,  \\field Induction Ratio
       \\required-field
       \\type real
       \\minimum 0.0
       \\default 2.5
       \\note ratio of induced air flow rate to primary air flow rate
  A3,  \\field Supply Air Inlet Node Name
       \\type node
       \\required-field
  A4,  \\field Induced Air Inlet Node Name
       \\type node
       \\required-field
       \\note should be a zone exhaust node, also the heating coil inlet node
  A5,  \\field Air Outlet Node Name
       \\type node
       \\required-field
       \\note should be a zone inlet node
  A6,  \\field Hot Water Inlet Node Name
       \\type node
       \\deprecated
  A7,  \\field Cold Water Inlet Node Name
       \\type node
       \\deprecated
  A8,  \\field Heating Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Heating:Water
  A9,  \\field Heating Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatingCoilName
  N3,  \\field Maximum Hot Water Flow Rate
       \\note Not used when heating coil type is gas or electric
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N4,  \\field Minimum Hot Water Flow Rate
       \\note Not used when heating coil type is gas or electric
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  N5,  \\field Heating Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  A10, \\field Cooling Coil Object Type
       \\type choice
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
  A11, \\field Cooling Coil Name
       \\type object-list
       \\object-list CoolingCoilName
  N6,  \\field Maximum Cold Water Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
  N7,  \\field Minimum Cold Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  N8,  \\field Cooling Convergence Tolerance
       \\type real
       \\minimum> 0.0
       \\default 0.001
  A12; \\field Zone Mixer Name
       \\type object-list
       \\object-list ZoneMixers

AirTerminal:SingleDuct:ConstantVolume:CooledBeam,
       \\memo Central air system terminal unit, single duct, constant volume, with cooled beam
       \\memo (active or passive).
       \\min-fields 23
  A1 , \\field Name
       \\type alpha
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Cooled Beam Type
       \\type choice
       \\required-field
       \\key Active
       \\key Passive
  A4 , \\field Supply Air Inlet Node Name
       \\type node
       \\required-field
  A5 , \\field Supply Air Outlet Node Name
       \\type node
       \\required-field
  A6 , \\field Chilled Water Inlet Node Name
       \\type node
       \\required-field
  A7 , \\field Chilled Water Outlet Node Name
       \\type node
       \\required-field
  N1 , \\field Supply Air Volumetric Flow Rate
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\default autosize
  N2,  \\field Maximum Total Chilled Water Volumetric Flow Rate
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\default autosize
  N3 , \\field Number of Beams
       \\note Number of individual beam units in the zone
       \\type integer
       \\minimum> 0
       \\autosizable
       \\default autosize
  N4 , \\field Beam Length
       \\note Length of an individual beam unit
       \\type real
       \\units m
       \\minimum> 0.0
       \\autosizable
       \\default autosize
  N5 , \\field Design Inlet Water Temperature
       \\type real
       \\units C
       \\minimum 0.0
       \\default 15.0
  N6 , \\field Design Outlet Water Temperature
       \\type real
       \\units C
       \\minimum 0.0
       \\default 17.0
  N7 , \\field Coil Surface Area per Coil Length
       \\type real
       \\units m2/m
       \\minimum 0.0
       \\default 5.422
  N8 , \\field Model Parameter a
       \\type real
       \\minimum 0.0
       \\default 15.3
  N9 , \\field Model Parameter n1
       \\type real
       \\minimum 0.0
       \\default 0.0
  N10, \\field Model Parameter n2
       \\type real
       \\minimum 0.0
       \\default 0.84
  N11, \\field Model Parameter n3
       \\type real
       \\minimum 0.0
       \\default 0.12
  N12, \\field Model Parameter a0
       \\note Free area of the coil in plan view per unit beam length
       \\type real
       \\units m2/m
       \\minimum 0.0
       \\default 0.171
  N13, \\field Model Parameter K1
       \\type real
       \\minimum 0.0
       \\default 0.0057
  N14, \\field Model Parameter n
       \\type real
       \\minimum 0.0
       \\default 0.4
  N15, \\field Coefficient of Induction Kin
       \\type real
       \\autocalculatable
       \\minimum 0.0
       \\maximum 4.0
       \\default Autocalculate
  N16; \\field Leaving Pipe Inside Diameter
       \\type real
       \\units m
       \\minimum> 0.0
       \\default 0.0145

AirTerminal:DualDuct:ConstantVolume,
       \\memo Central air system terminal unit, dual duct, constant volume.
       \\min-fields 6
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Outlet Node Name
       \\required-field
       \\type node
       \\note The outlet node of the terminal unit.
       \\note This is also the zone inlet node.
  A4 , \\field Hot Air Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Cold Air Inlet Node Name
       \\required-field
       \\type node
  N1 ; \\field Maximum Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable

AirTerminal:DualDuct:VAV,
       \\memo Central air system terminal unit, dual duct, variable volume.
       \\min-fields 7
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Outlet Node Name
       \\required-field
       \\type node
       \\note The outlet node of the terminal unit.
       \\note This is also the zone inlet node.
  A4 , \\field Hot Air Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Cold Air Inlet Node Name
       \\required-field
       \\type node
  N1 , \\field Maximum Damper Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N2 , \\field Zone Minimum Air Flow Fraction
       \\required-field
       \\note fraction of maximum air flow
       \\maximum 1.0
       \\minimum 0.0
       \\default 0.2
  A6 ; \\field Design Specification Outdoor Air Object Name
       \\type object-list
       \\object-list DesignSpecificationOutdoorAirNames
       \\note When the name of a DesignSpecification:OutdoorAir object is entered, the terminal
       \\note unit will increase flow as needed to meet this outdoor air requirement.
       \\note If Outdoor Air Flow per Person is non-zero, then the outdoor air requirement will
       \\note be computed based on the current number of occupants in the zone.
       \\note At no time will the supply air flow rate exceed the value for Maximum Air Flow Rate.
       \\note If this field is blank, then the terminal unit will not be controlled for outdoor air flow.

AirTerminal:DualDuct:VAV:OutdoorAir,
       \\memo Central air system terminal unit, dual duct, variable volume with special controls.
       \\memo One VAV duct is controlled to supply ventilation air and the other VAV duct is
       \\memo controlled to meet the zone cooling load.
       \\min-fields 7
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Outlet Node Name
       \\required-field
       \\note The outlet node of the terminal unit.
       \\note This is also the zone inlet node.
  A4 , \\field Outdoor Air Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Recirculated Air Inlet Node Name
       \\type node
  N1 , \\field Maximum Terminal Air Flow Rate
       \\required-field
       \\units m3/s
       \\minimum 0.0
       \\autosizable
       \\note If autosized this is the sum of flow needed for cooling and maximum required outdoor air
  A6 , \\field Design Specification Outdoor Air Object Name
       \\required-field
       \\type object-list
       \\object-list DesignSpecificationOutdoorAirNames
       \\note When the name of a DesignSpecification:OutdoorAir object is entered, the terminal
       \\note unit will increase flow as needed to meet this outdoor air requirement.
       \\note If Outdoor Air Flow per Person is non-zero, then the outdoor air requirement will
       \\note be computed based mode selected in the next field.
       \\note At no time will the supply air flow rate exceed the value for Maximum Air Flow Rate.
  A7 ; \\field Per Person Ventilation Rate Mode
       \\type choice
       \\key CurrentOccupancy
       \\key DesignOccupancy
       \\note CurrentOccupancy models demand controlled ventilation using the current number of people
       \\note DesignOccupancy uses the total Number of People in the zone and is constant


ZoneHVAC:AirDistributionUnit,
       \\memo Central air system air distribution unit, serves as a wrapper for a specific type of
       \\memo air terminal unit. This object is referenced in a ZoneHVAC:EquipmentList.
       \\min-fields 4
  A1 , \\field Name
       \\required-field
       \\reference AirDistributionUnits
  A2 , \\field Air Distribution Unit Outlet Node Name
       \\required-field
       \\type node
  A3 , \\field Air Terminal Object Type
       \\type choice
       \\key AirTerminal:DualDuct:ConstantVolume
       \\key AirTerminal:DualDuct:VAV
       \\key AirTerminal:SingleDuct:ConstantVolume:Reheat
       \\key AirTerminal:SingleDuct:VAV:Reheat
       \\key AirTerminal:SingleDuct:VAV:NoReheat
       \\key AirTerminal:SingleDuct:SeriesPIU:Reheat
       \\key AirTerminal:SingleDuct:ParallelPIU:Reheat
       \\key AirTerminal:SingleDuct:ConstantVolume:FourPipeInduction
       \\key AirTerminal:SingleDuct:VAV:Reheat:VariableSpeedFan
       \\key AirTerminal:SingleDuct:VAV:HeatAndCool:Reheat
       \\key AirTerminal:SingleDuct:VAV:HeatAndCool:NoReheat
       \\key AirTerminal:SingleDuct:ConstantVolume:CooledBeam
       \\key AirTerminal:DualDuct:VAV:OutdoorAir
       \\key AirTerminal:SingleDuct:UserDefined
       \\required-field
  A4 , \\field Air Terminal Name
       \\required-field
  N1 , \\field Nominal Upstream Leakage Fraction
       \\note fraction at system design Flow; leakage Flow constant, leakage fraction
       \\note varies with variable system Flow Rate.
       \\type real
       \\minimum 0
       \\maximum 0.3
       \\default 0
  N2 ; \\field Constant Downstream Leakage Fraction
       \\type real
       \\minimum 0
       \\maximum 0.3
       \\default 0

\\group Zone HVAC Equipment Connections

!*****************ZONE EQUIPMENT SPECIFICATION*********************
!Zone Heating and Cooling Equipment are attached to zones by the following statements:

ZoneHVAC:EquipmentList,
       \\extensible:4 Duplicate the four fields Zone Equipment Object Type through Heating Priority
       \\memo List equipment in simulation order.  Note that an ZoneHVAC:AirDistributionUnit or
       \\memo AirTerminal:SingleDuct:Uncontrolled object must be listed in this statement if there is a forced
       \\memo air system serving the zone from the air loop.
       \\memo Equipment is simulated in the order specified by Zone Equipment Cooling Sequence and
       \\memo Zone Equipment Heating or No-Load Sequence, depending on the thermostat request.
       \\memo For equipment of similar type, assign sequence 1 to the first system intended to
       \\memo serve that type of load.  For situations where one or more equipment types has limited capacity or
       \\memo limited control, order the sequence so that the most controllable piece of equipment runs last.
       \\memo For example, with a dedicated outdoor air system (DOAS), the air terminal for the DOAS
       \\memo should be assigned Heating Sequence = 1 and Cooling Sequence = 1.  Any other equipment should
       \\memo be assigned sequence 2 or higher so that it will see the net load after the DOAS air is added
       \\memo to the zone.
  A1 , \\field Name
       \\required-field
       \\reference ZoneEquipmentLists
  A2 , \\field Zone Equipment 1 Object Type
       \\begin-extensible
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
       \\required-field
  A3 , \\field Zone Equipment 1 Name
       \\required-field
  N1 , \\field Zone Equipment 1 Cooling Sequence
       \\required-field
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N2 , \\field Zone Equipment 1 Heating or No-Load Sequence
       \\required-field
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A4 , \\field Zone Equipment 2 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A5 , \\field Zone Equipment 2 Name
  N3 , \\field Zone Equipment 2 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N4 , \\field Zone Equipment 2 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A6 , \\field Zone Equipment 3 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A7 , \\field Zone Equipment 3 Name
  N5 , \\field Zone Equipment 3 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N6 , \\field Zone Equipment 3 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A8 , \\field Zone Equipment 4 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A9 , \\field Zone Equipment 4 Name
  N7 , \\field Zone Equipment 4 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N8 , \\field Zone Equipment 4 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A10, \\field Zone Equipment 5 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A11, \\field Zone Equipment 5 Name
  N9 , \\field Zone Equipment 5 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N10, \\field Zone Equipment 5 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A12, \\field Zone Equipment 6 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A13, \\field Zone Equipment 6 Name
  N11, \\field Zone Equipment 6 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N12, \\field Zone Equipment 6 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A14, \\field Zone Equipment 7 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A15, \\field Zone Equipment 7 Name
  N13, \\field Zone Equipment 7 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N14, \\field Zone Equipment 7 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A16, \\field Zone Equipment 8 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A17, \\field Zone Equipment 8 Name
  N15, \\field Zone Equipment 8 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N16, \\field Zone Equipment 8 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A18, \\field Zone Equipment 9 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A19, \\field Zone Equipment 9 Name
  N17, \\field Zone Equipment 9 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N18, \\field Zone Equipment 9 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A20, \\field Zone Equipment 10 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A21, \\field Zone Equipment 10 Name
  N19, \\field Zone Equipment 10 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N20, \\field Zone Equipment 10 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A22, \\field Zone Equipment 11 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A23, \\field Zone Equipment 11 Name
  N21, \\field Zone Equipment 11 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N22, \\field Zone Equipment 11 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A24, \\field Zone Equipment 12 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A25, \\field Zone Equipment 12 Name
  N23, \\field Zone Equipment 12 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N24, \\field Zone Equipment 12 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A26, \\field Zone Equipment 13 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A27, \\field Zone Equipment 13 Name
  N25, \\field Zone Equipment 13 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N26, \\field Zone Equipment 13 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A28, \\field Zone Equipment 14 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A29, \\field Zone Equipment 14 Name
  N27, \\field Zone Equipment 14 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N28, \\field Zone Equipment 14 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A30, \\field Zone Equipment 15 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A31, \\field Zone Equipment 15 Name
  N29, \\field Zone Equipment 15 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N30, \\field Zone Equipment 15 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A32, \\field Zone Equipment 16 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A33, \\field Zone Equipment 16 Name
  N31, \\field Zone Equipment 16 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N32, \\field Zone Equipment 16 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A34, \\field Zone Equipment 17 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A35, \\field Zone Equipment 17 Name
  N33, \\field Zone Equipment 17 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N34, \\field Zone Equipment 17 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load
  A36, \\field Zone Equipment 18 Object Type
       \\type choice
       \\key ZoneHVAC:TerminalUnit:VariableRefrigerantFlow
       \\key ZoneHVAC:AirDistributionUnit
       \\key AirTerminal:SingleDuct:Uncontrolled
       \\key ZoneHVAC:EnergyRecoveryVentilator
       \\key ZoneHVAC:FourPipeFanCoil
       \\key ZoneHVAC:OutdoorAirUnit
       \\key ZoneHVAC:PackagedTerminalAirConditioner
       \\key ZoneHVAC:PackagedTerminalHeatPump
       \\key ZoneHVAC:UnitHeater
       \\key ZoneHVAC:UnitVentilator
       \\key ZoneHVAC:VentilatedSlab
       \\key ZoneHVAC:WaterToAirHeatPump
       \\key ZoneHVAC:WindowAirConditioner
       \\key ZoneHVAC:Baseboard:RadiantConvective:Electric
       \\key ZoneHVAC:Baseboard:RadiantConvective:Water
       \\key ZoneHVAC:Baseboard:RadiantConvective:Steam
       \\key ZoneHVAC:Baseboard:Convective:Electric
       \\key ZoneHVAC:Baseboard:Convective:Water
       \\key ZoneHVAC:HighTemperatureRadiant
       \\key ZoneHVAC:LowTemperatureRadiant:VariableFlow
       \\key ZoneHVAC:LowTemperatureRadiant:ConstantFlow
       \\key ZoneHVAC:LowTemperatureRadiant:Electric
       \\key ZoneHVAC:Dehumidifier:DX
       \\key ZoneHVAC:IdealLoadsAirSystem
       \\key ZoneHVAC:RefrigerationChillerSet
       \\key Fan:ZoneExhaust
       \\key WaterHeater:HeatPump
  A37, \\field Zone Equipment 18 Name
  N35, \\field Zone Equipment 18 Cooling Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests cooling
  N36; \\field Zone Equipment 18 Heating or No-Load Sequence
       \\type integer
       \\minimum 1
       \\note Specifies the zone equipment simulation order
       \\note when the zone thermostat requests heating or no load

ZoneHVAC:EquipmentConnections,
       \\memo Specifies the HVAC equipment connections for a zone. Node names are specified for the
       \\memo zone air node, air inlet nodes, air exhaust nodes, and the air return node. A zone
       \\memo equipment list is referenced which lists all HVAC equipment connected to the zone.
  A1 , \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
  A2 , \\field Zone Conditioning Equipment List Name
       \\note Enter the name of a ZoneHVAC:EquipmentList object.
       \\required-field
       \\type object-list
       \\object-list ZoneEquipmentLists
  A3 , \\field Zone Air Inlet Node or NodeList Name
       \\type node
  A4 , \\field Zone Air Exhaust Node or NodeList Name
       \\type node
  A5 , \\field Zone Air Node Name
       \\required-field
       \\type node
  A6 ; \\field Zone Return Air Node Name
       \\required-field
       \\type node

\\Group Fans
!*****************AIR LOOP COMPONENTS*********************
Fan:ConstantVolume,
       \\memo Constant volume fan that is intended to operate continuously based on a time schedule.
       \\memo This fan will not cycle on and off based on cooling/heating load or other control
       \\memo signals.
  \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\reference Fans
       \\reference FansCV
       \\reference FansCVandOnOff
       \\reference FansCVandVAV
       \\reference FansCVandOnOffandVAV
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Fan Efficiency
       \\type real
       \\default 0.7
       \\minimum> 0.0
       \\maximum 1.0
  N2 , \\field Pressure Rise
       \\units Pa
       \\ip-units inH2O
       \\required-field
  N3 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N4 , \\field Motor Efficiency
       \\type real
       \\default 0.9
       \\minimum> 0.0
       \\maximum 1.0
  N5 , \\field Motor In Airstream Fraction
       \\note 0.0 means fan motor outside of air stream, 1.0 means motor inside of air stream
       \\type real
       \\default 1.0
       \\minimum 0.0
       \\maximum 1.0
  A3 , \\field Air Inlet Node Name
       \\type node
       \\required-field
  A4 , \\field Air Outlet Node Name
       \\type node
       \\required-field
  A5 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

Fan:VariableVolume,
       \\memo Variable air volume fan where the electric power input varies according to a
       \\memo performance curve as a function of flow fraction.
  \\min-fields 17
  A1 , \\field Name
       \\required-field
       \\reference Fans
       \\reference FansVAV
       \\reference FansCVandVAV
       \\reference FansOnOffandVAV
       \\reference FansCVandOnOffandVAV
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Fan Efficiency
       \\type real
       \\default 0.7
       \\minimum> 0.0
       \\maximum 1.0
  N2 , \\field Pressure Rise
       \\required-field
       \\units Pa
       \\ip-units inH2O
  N3 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  A3 , \\field Fan Power Minimum Flow Rate Input Method
       \\type choice
       \\key Fraction
       \\key FixedFlowRate
       \\default Fraction
  N4 , \\field Fan Power Minimum Flow Fraction
       \\type real
       \\maximum 1.0
       \\minimum 0.0
       \\default 0.25
  N5 , \\field Fan Power Minimum Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum 0.0
  N6 , \\field Motor Efficiency
       \\type real
       \\default 0.9
       \\minimum> 0.0
       \\maximum 1.0
  N7 , \\field Motor In Airstream Fraction
       \\note 0.0 means fan motor outside of air stream, 1.0 means motor inside of air stream
       \\type real
       \\default 1.0
       \\minimum 0.0
       \\maximum 1.0
  N8 , \\field Fan Power Coefficient 1
       \\note all Fan Power Coefficients should not be 0.0 or no fan power will be consumed.
       \\note Fan Power Coefficents are specified as function of full flow rate/power
       \\note Equation:
  N9 , \\field Fan Power Coefficient 2
  N10, \\field Fan Power Coefficient 3
  N11, \\field Fan Power Coefficient 4
  N12, \\field Fan Power Coefficient 5
  A4 , \\field Air Inlet Node Name
       \\type node
       \\required-field
  A5 , \\field Air Outlet Node Name
       \\type node
       \\required-field
  A6 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

Fan:OnOff,
       \\memo Constant volume fan that is intended to cycle on and off based on cooling/heating load
       \\memo or other control signals. This fan can also operate continuously like
       \\memo Fan:ConstantVolume.
  \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\reference Fans
       \\reference FansCVandOnOff
       \\reference FansOnOff
       \\reference FansOnOffandVAV
       \\reference FansCVandOnOffandVAV
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Fan Efficiency
       \\type real
       \\default 0.6
       \\minimum> 0.0
       \\maximum 1.0
  N2 , \\field Pressure Rise
       \\units Pa
       \\ip-units inH2O
       \\required-field
  N3 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N4 , \\field Motor Efficiency
       \\type real
       \\default 0.8
       \\minimum> 0.0
       \\maximum 1.0
  N5 , \\field Motor In Airstream Fraction
       \\note 0.0 means fan motor outside of air stream, 1.0 means motor inside of air stream
       \\type real
       \\default 1.0
       \\minimum 0.0
       \\maximum 1.0
  A3 , \\field Air Inlet Node Name
       \\type node
       \\required-field
  A4 , \\field Air Outlet Node Name
       \\type node
       \\required-field
  A5 , \\field Fan Power Ratio Function of Speed Ratio Curve Name
       \\type object-list
       \\object-list ExponentCurves
  A6 , \\field Fan Efficiency Ratio Function of Speed Ratio Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
  A7 ; \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General

Fan:ZoneExhaust,
       \\memo Models a fan that exhausts air from a zone.
  \\min-fields 7
  A1 , \\field Name
       \\required-field
       \\reference FansZoneExhaust
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Fan Efficiency
       \\type real
       \\default 0.6
       \\minimum> 0.0
       \\maximum 1.0
  N2 , \\field Pressure Rise
       \\units Pa
       \\ip-units inH2O
       \\required-field
  N3 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
  A3 , \\field Air Inlet Node Name
       \\type node
       \\required-field
  A4 , \\field Air Outlet Node Name
       \\type node
       \\required-field
  A5 , \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General
  A6 , \\field Flow Fraction Schedule Name
       \\note If field is used, then when fan runs the exhausted air flow rate is controlled to be the scheduled fraction times the Maximum Flow Rate
       \\type object-list
       \\object-list ScheduleNames
  A7 , \\field System Availability Manager Coupling Mode
       \\note Control if fan is to be interlocked with HVAC system Availability Managerrs or not.
       \\type choice
       \\key Coupled
       \\key Decoupled
       \\default Coupled
  A8 , \\field Minimum Zone Temperature Limit Schedule Name
       \\note If field is used, the exhaust fan will not run if the zone temperature is lower than this limit
       \\type object-list
       \\object-list ScheduleNames
  A9 ; \\field Balanced Exhaust Fraction Schedule Name
       \\note Used to control fan's impact on flow at the return air node. Enter the portion of the exhaust that is balanced by simple airflows.
       \\note
       \\type object-list
       \\object-list ScheduleNames

FanPerformance:NightVentilation,
       \\memo Specifies an alternate set of performance parameters for a fan. These alternate
       \\memo parameters are used when a system manager (such as AvailabilityManager:NightVentilation)
       \\memo sets a specified flow rate.  May be used with any type of fan except not with
       \\memo Fan:ComponentModel. If the fan model senses that a fixed flow rate has been set, it
       \\memo will use these alternate performance parameters. It is assumed that the fan will
       \\memo run at a fixed speed in the alternate mode.
  A1 , \\field Fan Name
       \\required-field
       \\type object-list
       \\object-list FansCVandVAV
  N1 , \\field Fan Efficiency
       \\type real
       \\required-field
       \\minimum> 0.0
       \\maximum 1.0
  N2 , \\field Pressure Rise
       \\units Pa
       \\ip-units inH2O
       \\required-field
  N3 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N4 , \\field Motor Efficiency
       \\type real
       \\required-field
       \\minimum> 0
       \\maximum 1.0
  N5 ; \\field Motor in Airstream Fraction
       \\type real
       \\default 1.0
       \\minimum 0.0
       \\maximum 1.0
       \\note 0.0 means fan motor outside of airstream
       \\note 1.0 means fan motor inside of airstream

Fan:ComponentModel,
       \\memo A detailed fan type for constant-air-volume (CAV) and variable-air-volume (VAV)
       \\memo systems. It includes inputs that describe the air-distribution system as well as the
       \\memo fan, drive belt (if used), motor, and variable-frequency-drive (if used).
  A1 , \\field Name
       \\required-field
       \\reference Fans
  A2 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A4 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Maximum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N2 , \\field Minimum Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N3 , \\field Fan Sizing Factor
       \\type real
       \\minimum 1.0
       \\default 1.0
       \\note Applied to specified or autosized max fan airflow
  N4 , \\field Fan Wheel Diameter
       \\required-field
       \\type real
       \\units m
       \\minimum> 0.0
       \\note Diameter of wheel outer circumference
  N5 , \\field Fan Outlet Area
       \\required-field
       \\type real
       \\units m2
       \\minimum> 0.0
       \\note Area at fan outlet plane for determining discharge velocity pressure
  N6 , \\field Maximum Fan Static Efficiency
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\note Maximum ratio between power delivered to air and fan shaft input power
       \\note Determined from fan performance data
  N7 , \\field Euler Number at Maximum Fan Static Efficiency
       \\required-field
       \\type real
       \\minimum> 0.0
       \\note Euler number (Eu) determined from fan performance data
  N8 , \\field Maximum Dimensionless Fan Airflow
       \\required-field
       \\type real
       \\minimum> 0.0
       \\note Corresponds to maximum ratio between fan airflow and
       \\note fan shaft rotational speed for specified fan wheel diameter
       \\note Determined from fan performance data
  N9 , \\field Motor Fan Pulley Ratio
       \\type real
       \\minimum> 0.0
       \\default 1.0
       \\autosizable
       \\note Ratio of motor pulley diameter to fan pulley diameter
  N10 , \\field Belt Maximum Torque
        \\required-field
        \\type real
        \\units N-m
        \\minimum> 0.0
        \\autosizable
        \\note Maximum torque transmitted by belt
  N11 , \\field Belt Sizing Factor
        \\type real
        \\minimum 1.0
        \\default 1.0
        \\note Applied to specified or autosized max torque transmitted by belt
  N12 , \\field Belt Fractional Torque Transition
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.167
        \\note Region 1 to 2 curve transition for belt normalized efficiency
  N13 , \\field Motor Maximum Speed
        \\required-field
        \\type real
        \\units rev/min
        \\minimum> 0.0
        \\note Maximum rotational speed of fan motor shaft
  N14 , \\field Maximum Motor Output Power
        \\required-field
        \\type real
        \\units W
        \\minimum> 0.0
        \\autosizable
        \\note Maximum power input to drive belt by motor
  N15 , \\field Motor Sizing Factor
        \\type real
        \\minimum 1.0
        \\default 1.0
        \\note Applied to specified or autosized motor output power
  N16 , \\field Motor In Airstream Fraction
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\default 1.0
        \\note 0.0 means motor outside air stream
        \\note 1.0 means motor inside air stream
  A5 , \\field VFD Efficiency Type
       \\type choice
       \\key Speed
       \\key Power
       \\note Efficiency depends on fraction of full-load motor speed
       \\note Efficiency depends on  fraction of full-load motor input power
       \\note If field blank, then assumes constant VFD efficiency (0.97)
  N17 , \\field Maximum VFD Output Power
        \\required-field
        \\type real
        \\units W
        \\minimum> 0.0
        \\autosizable
        \\note Maximum power input to motor by VFD
  N18 , \\field VFD Sizing Factor
        \\type real
        \\minimum 1.0
        \\default 1.0
        \\note Applied to specified or autosized VFD output power
  A6 , \\field Fan Pressure Rise Curve Name
       \\required-field
       \\type object-list
       \\object-list FanPressureRiseCurves
       \\note Pressure rise depends on volumetric flow, system resistances,
       \\note system leakage, and duct static pressure set point
  A7 , \\field Duct Static Pressure Reset Curve Name
       \\required-field
       \\type object-list
       \\object-list LinearCurves
       \\note Function of fan volumetric flow
       \\note Minimum and maximum fan airflows correspond respectively to
       \\note minimum and maximum duct static pressure set points
  A8 , \\field Normalized Fan Static Efficiency Curve Name-Non-Stall Region
       \\required-field
       \\type object-list
       \\object-list ExponentialSkewNormalCurves
       \\note xfan <= 0
       \\note Curve should have maximum of 1.0
  A9 , \\field Normalized Fan Static Efficiency Curve Name-Stall Region
       \\required-field
       \\type object-list
       \\object-list ExponentialSkewNormalCurves
       \\note xfan > 0
       \\note Curve should have maximum of 1.0
  A10 , \\field Normalized Dimensionless Airflow Curve Name-Non-Stall Region
        \\required-field
        \\type object-list
        \\object-list SigmoidCurves
        \\note xspd <= 0
        \\note Curve should have maximum of 1.0
  A11 , \\field Normalized Dimensionless Airflow Curve Name-Stall Region
        \\required-field
        \\type object-list
        \\object-list SigmoidCurves
        \\note xspd > 0
        \\note Curve should have maximum of 1.0
  A12 , \\field Maximum Belt Efficiency Curve Name
        \\type object-list
        \\object-list QuarticCurves
        \\note Determines maximum fan drive belt efficiency in log space
        \\note as function of xbelt,max
        \\note Curve should have minimum of -4.6 and maximum of 0.0
        \\note If field blank, assumes output of curve is always 1.0
  A13 , \\field Normalized Belt Efficiency Curve Name - Region 1
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Region 1 (0 <= xbelt < xbelt,trans)
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes output of curve is always 1.0 in Region 1
  A14 , \\field Normalized Belt Efficiency Curve Name - Region 2
        \\type object-list
        \\object-list ExponentialDecayCurves
        \\note Region 2 (xbelt,trans <= xbelt <= 1)
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes output of curve is always 1.0 in Region 2
  A15 , \\field Normalized Belt Efficiency Curve Name - Region 3
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Determines normalized drive belt efficiency Region 3 (xbelt > 1)
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes output of curve is always 1.0 in Region 3
  A16 , \\field Maximum Motor Efficiency Curve Name
        \\type object-list
        \\object-list RectangularHyperbola1Curves
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes output of curve is always 1.0
  A17 , \\field Normalized Motor Efficiency Curve Name
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes output of curve is always 1.0
  A18 , \\field VFD Efficiency Curve Name
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Determines VFD efficiency as function of motor load or speed fraction
        \\note Curve should have minimum > 0.0 and maximum of 1.0
        \\note If field blank, assumes constant VFD efficiency (0.97)
  A19 ; \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General

\\Group Coils

!*****************COILS*********************
Coil:Cooling:Water,
       \\memo Chilled water cooling coil, NTU-effectiveness model, with inputs for design entering
       \\memo and leaving conditionss.
   \\min-fields 15
  A1 , \\field Name
       \\required-field
       \\reference CoolingCoilName
       \\reference CoolingCoilsWater
       \\reference CoolingCoilsWaterNoHX
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Design Water Flow Rate
       \\units m3/s
       \\autosizable
       \\default autosize
       \\minimum 0
       \\ip-units gal/min
  N2 , \\field Design Air Flow Rate
       \\units m3/s
       \\autosizable
       \\default autosize
       \\minimum 0
       \\ip-units gal/min
  N3 , \\field Design Inlet Water Temperature
       \\units C
       \\autosizable
       \\default autosize
       \\minimum> 0
  N4 , \\field Design Inlet Air Temperature
       \\units C
       \\autosizable
       \\default autosize
       \\minimum> 0
  N5 , \\field Design Outlet Air Temperature
       \\units C
       \\autosizable
       \\default autosize
       \\minimum> 0
  N6 , \\field Design Inlet Air Humidity Ratio
       \\units kgWater/kgDryAir
       \\autosizable
       \\default autosize
       \\minimum 0
  N7 , \\field Design Outlet Air Humidity Ratio
       \\units kgWater/kgDryAir
       \\autosizable
       \\default autosize
       \\minimum 0
  A3 , \\field Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A6 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A7 , \\field Type of Analysis
       \\type choice
       \\key SimpleAnalysis
       \\key DetailedAnalysis
       \\default SimpleAnalysis
  A8 , \\field Heat Exchanger Configuration
       \\type choice
       \\key CrossFlow
       \\key CounterFlow
       \\default CounterFlow
  A9;  \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames

Coil:Cooling:Water:DetailedGeometry,
       \\memo Chilled water cooling coil, detailed flat fin coil model for continuous plate fins,
       \\memo with inputs for detailed coil geometry specificatons.
  A1 , \\field Name
       \\required-field
       \\reference CoolingCoilName
       \\reference CoolingCoilsWater
       \\reference CoolingCoilsWaterNoHX
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Maximum Water Flow Rate
       \\units m3/s
       \\autosizable
       \\default autosize
       \\ip-units gal/min
  N2 , \\field Tube Outside Surface Area
       \\units m2
       \\autosizable
       \\default autosize
       \\note Tube Primary Surface Area
  N3 , \\field Total Tube Inside Area
       \\units m2
       \\autosizable
       \\default autosize
       \\note Total tube inside surface area
       \\minimum> 0.0
  N4 , \\field Fin Surface Area
       \\units m2
       \\autosizable
       \\default autosize
  N5 , \\field Minimum Airflow Area
       \\units m2
       \\autosizable
       \\default autosize
       \\minimum> 0.0
  N6 , \\field Coil Depth
       \\autosizable
       \\default autosize
       \\units m
       \\ip-units in
       \\minimum> 0.0
  N7 , \\field Fin Diameter
       \\units m
       \\autosizable
       \\default autosize
       \\note Fin diameter or the coil height
       \\ip-units in
       \\minimum> 0.0
  N8 , \\field Fin Thickness
       \\type real
       \\units m
       \\ip-units in
       \\minimum> 0.0
       \\default .0015
  N9 , \\field Tube Inside Diameter
       \\units m
       \\note Inner diameter of tubes
       \\ip-units in
       \\default .01445
       \\minimum> 0.0
  N10, \\field Tube Outside Diameter
       \\units m
       \\note Outer diameter of tubes
       \\ip-units in
       \\default .0159
       \\minimum> 0.0
  N11, \\field Tube Thermal Conductivity
       \\type real
       \\units W/m-K
       \\minimum 1.0
       \\default 386.0
  N12, \\field Fin Thermal Conductivity
       \\type real
       \\units W/m-K
       \\minimum 1.0
       \\default 204.0
  N13, \\field Fin Spacing
       \\units m
       \\note Fin spacing or distance
       \\ip-units in
       \\default .0018
       \\minimum> 0.0
  N14, \\field Tube Depth Spacing
       \\units m
       \\ip-units in
       \\default .026
       \\minimum> 0.0
  N15, \\field Number of Tube Rows
       \\default 4
       \\minimum> 0.0
  N16, \\field Number of Tubes per Row
       \\autosizable
       \\default autosize
       \\minimum> 0.0
  A3 , \\field Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A6 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A7 ; \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames

Coil:Cooling:DX:SingleSpeed,
       \\memo Direct expansion (DX) cooling coil and condensing unit (includes electric compressor
       \\memo and condenser fan), single-speed. Optional inputs for moisture evaporation from wet
       \\memo coil when compressor cycles off with continuous fan operation.
  \\min-fields 14
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingCoilsDX
       \\reference CoolingCoilsDXSingleSpeed
       \\reference CoolingCoilsDXMultiModeOrSingleSpeed
       \\reference CoolingCoilsDXBypassFactorEmpirical
       \\reference DesuperHeatingCoilSources
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Rated Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note gross capacity excluding supply air fan heat
       \\note rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N2 , \\field Rated Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note sensible and total capacities do not include supply fan heat
  N3 , \\field Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N4 , \\field Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, Rated SHR and Rated COP
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total cooling capacity
  N5 , \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total cooling capacity.
       \\note This value is only used to calculate Seasonal Energy Efficiency Ratio (SEER), Energy
       \\note Efficiency Ratio (EER), Integrated Energy Efficiency Ratio (IEER), and the Standard Rating
       \\note (Net) Cooling Capacity which will be outputs in the EnergyPlus eio file.  This value is not
       \\note used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A6 , \\field Total Cooling Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = Fraction of the full load flow
  A7 , \\field Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A8 , \\field Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = Fraction of the full load flow
  A9 , \\field Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (cooling load/steady state capacity)
   N6, \\field Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
   N7, \\field Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
       \\note the compressor first turns off) and the coil's steady state latent capacity
       \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
   N8, \\field Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling Rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation model is disabled.
   N9, \\field Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation model is disabled.
  A10, \\field Condenser Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
  A11, \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N10, \\field Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N11, \\field Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N12, \\field Evaporative Condenser Pump Rated Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump
  N13, \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\units W
       \\ip-units W
  N14, \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0.0
       \\default 10.0
       \\units C
  A12, \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A13, \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  N15, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the DX coil is off.
  N16, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A14, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A15, \\field Sensible Heat Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*db + e*db**2 + f*wb*db
       \\note wb = entering wet-bulb temperature seen by the DX cooling coil (C)
       \\note db = entering dry-bulb temperature seen by the DX cooling coil (C)
       \\note entering temperature can be outside air or pretreated air.
  A16; \\field Sensible Heat Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow

Coil:Cooling:DX:TwoSpeed,
       \\memo Direct expansion (DX) cooling coil and condensing unit (includes electric compressor
       \\memo and condenser fan), two-speed (or variable-speed). Requires two sets of performance
       \\memo data and will interpolate between speeds. Modeled as a single coil (multi-speed
       \\memo compressor or multiple compressors with row split or intertwined coil).
  \\min-fields 19
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingCoilsDX
       \\reference DesuperHeatingCoilSources
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Rated High Speed Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note gross capacity excluding supply air fan heat
       \\note rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N2 , \\field Rated High Speed Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note sensible and total capacities do not include supply fan heat
  N3 , \\field Rated High Speed COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N4 , \\field Rated High Speed Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, Rated SHR
       \\note and Rated COP. Should be between 0.00004027 m3/s and .00006041 m3/s per watt
       \\note of rated total cooling capacity.
  N5 , \\field Unit Internal Static Air Pressure
       \\note Enter pressure drop for the unit containing the coil.
       \\note This value is only used to calculate Energy Efficiency Ratio
       \\note (EER), Integrated Energy Efficiency Ratio (IEER), and the
       \\note Standard Rating (Net) Cooling Capacity. This value is not
       \\note used for modeling the evaporator fan during simulations.
       \\type real
       \\units Pa
       \\minimum> 0.0
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A6 , \\field Total Cooling Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A7 , \\field Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A8 , \\field Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A9 , \\field Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (cooling load/steady state capacity)
  N6 , \\field Rated Low Speed Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note gross capacity excluding supply air fan heat
       \\note rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N7 , \\field Rated Low Speed Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note sensible and total capacities do not include supply fan heat
  N8 , \\field Rated Low Speed COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N9 , \\field Rated Low Speed Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, Rated SHR
       \\note and Rated COP. Should be between 0.00004027 m3/s and .00006041 m3/s per watt
       \\note of rated total cooling capacity.
  A10, \\field Low Speed Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A11, \\field Low Speed Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A12, \\field Condenser Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
  A13, \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N10, \\field High Speed Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N11, \\field High Speed Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N12, \\field High Speed Evaporative Condenser Pump Rated Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at high speed
  N13, \\field Low Speed Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N14, \\field Low Speed Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N15, \\field Low Speed Evaporative Condenser Pump Rated Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at low speed
  A14, \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A15, \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  N16, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the DX coil is off.
  N17, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A16, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A17, \\field Sensible Heat Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*db + e*db**2 + f*wb*db
       \\note wb = entering wet-bulb temperature seen by the DX cooling coil (C)
       \\note db = entering dry-bulb temperature seen by the DX cooling coil (C)
       \\note entering temperature can be outside air or pretreated air.
  A18, \\field Sensible Heat Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A19, \\field Low Speed Sensible Heat Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*db + e*db**2 + f*wb*db
       \\note wb = entering wet-bulb temperature seen by the DX cooling coil (C)
       \\note db = entering dry-bulb temperature seen by the DX cooling coil (C)
       \\note entering temperature can be outside air or pretreated air.
  A20; \\field Low Speed Sensible Heat Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow

Coil:Cooling:DX:MultiSpeed,
       \\memo Direct expansion (DX) cooling coil and condensing unit (includes electric or
       \\memo engine-driven compressor and condenser fan), multi-speed (or variable-speed).
       \\memo Optional moisture evaporation from wet coil when compressor cycles off with continuous
       \\memo fan operation. Requires two to four sets of performance data and will interpolate
       \\memo between speeds. Modeled as a single coil (multi-speed compressor or multiple
       \\memo compressors with row split or intertwined coil).
  \\min-fields 55
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingCoilsDXMultiSpeed
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
  A6 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  A7 , \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A8 , \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A9 , \\field Apply Part Load Fraction to Speeds Greater than 1
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A10, \\field Apply Latent Degradation to Speeds Greater than 1
       \\type choice
       \\key Yes
       \\key No
       \\default No
  N1 , \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\units W
       \\ip-units W
  N2 , \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0.0
       \\default 10.0
       \\units C
  N3,  \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the DX coil is off.
  N4,  \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A11, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A12, \\field Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default NaturalGas
  N5 , \\field Number of Speeds
       \\required-field
       \\type integer
       \\minimum 2
       \\maximum 4
       \\note Enter the number of the following sets of data for coil capacity, SHR, COP,
       \\note flow rate, and associated curves.
  N6 , \\field Speed 1 Rated Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Gross capacity excluding supply air fan heat
       \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
       \\note Speed 1 is defined as low speed
  N7 , \\field Speed 1 Rated Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note Sensible and total capacities do not include supply fan heat
  N8 , \\field Speed 1 Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N9 , \\field Speed 1 Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, rated SHR and rated
       \\note COP should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note cooling capacity
  N10, \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total cooling capacity.
       \\note This value is only used to calculate Seasonal Energy Efficiency Ratio (SEER), and the
       \\note Standard Rating (Net) Cooling Capacity which will be outputs in the EnergyPlus eio file.
       \\note This value is not used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A13, \\field Speed 1 Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A14, \\field Speed 1 Total Cooling Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = Fraction of the full load Flow
  A15, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A16, \\field Speed 1 Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A17, \\field Speed 1 Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (cooling load/steady state capacity)
  N11, \\field Speed 1 Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
  N12, \\field Speed 1 Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation Rate from the Cooling Coil (when
       \\note the compressor first turns off) and the Coil's steady state latent capacity
       \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
  N13, \\field Speed 1 Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation
       \\note model is disabled.
  N14, \\field Speed 1 Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation
       \\note model is disabled.
  N15, \\field Speed 1 Rated Waste Heat Fraction of Power Input
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note Recoverable waste heat at full load and rated conditions
  A18, \\field Speed 1 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N16, \\field Speed 1 Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N17, \\field Speed 1 Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N18, \\field Speed 1 Rated Evaporative Condenser Pump Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at high speed
  N19, \\field Speed 2 Rated Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Gross capacity excluding supply air fan heat
       \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N20, \\field Speed 2 Rated Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note Sensible and total capacities do not include supply fan heat
  N21, \\field Speed 2 Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N22, \\field Speed 2 Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, rated SHR and rated
       \\note COP should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note cooling capacity for Speed 2.
  N23, \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total cooling capacity.
       \\note This value is only used to calculate Seasonal Energy Efficiency Ratio (SEER), and the
       \\note Standard Rating (Net) Cooling Capacity which will be outputs in the EnergyPlus eio file.
       \\note This value is not used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A19, \\field Speed 2 Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A20, \\field Speed 2 Total Cooling Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A21, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A22, \\field Speed 2 Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = Fraction of the full load Flow
  A23, \\field Speed 2 Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (Cooling load/steady state capacity)
  N24, \\field Speed 2 Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
  N25, \\field Speed 2 Ratio of Initial Moisture Evaporation Rate and steady state Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
       \\note the compressor first turns off) and the coil's steady state latent capacity
       \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
  N26, \\field Speed 2 Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation
       \\note model is disabled.
  N27, \\field Speed 2 Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation
       \\note model is disabled.
  N28, \\field Speed 2 Rated Waste Heat Fraction of Power Input
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note Recoverable waste heat at full load and rated conditions
  A24, \\field Speed 2 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N29, \\field Speed 2 Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N30, \\field Speed 2 Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N31, \\field Speed 2 Rated Evaporative Condenser Pump Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at low speed
  N32, \\field Speed 3 Rated Total Cooling Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Gross capacity excluding supply air fan heat
       \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N33, \\field Speed 3 Rated Sensible Heat Ratio
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note Sensible and total capacities do not include supply fan heat
  N34, \\field Speed 3 Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N35, \\field Speed 3 Rated Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, rated SHR and rated
       \\note COP should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note cooling capacity for Speed 3.
  N36, \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total cooling capacity.
       \\note This value is only used to calculate Seasonal Energy Efficiency Ratio (SEER), and the
       \\note Standard Rating (Net) Cooling Capacity which will be outputs in the EnergyPlus eio file.
       \\note This value is not used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A25, \\field Speed 3 Total Cooling Capacity Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A26, \\field Speed 3 Total Cooling Capacity Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A27, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A28, \\field Speed 3 Energy Input Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A29, \\field Speed 3 Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (Cooling load/steady state capacity)
  N37, \\field Speed 3 Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
  N38, \\field Speed 3 Ratio of Initial Moisture Evaporation Rate and steady state Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
       \\note the compressor first turns off) and the coil's steady state latent capacity
       \\note at rated air flow and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
  N39, \\field Speed 3 Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation
       \\note model is disabled.
  N40, \\field Speed 3 Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation
       \\note model is disabled.
  N41, \\field Speed 3 Rated Waste Heat Fraction of Power Input
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note Recoverable waste heat at full load and rated conditions
  A30, \\field Speed 3 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N42, \\field Speed 3 Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N43, \\field Speed 3 Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N44, \\field Speed 3 Rated Evaporative Condenser Pump Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at Low speed
  N45, \\field Speed 4 Rated Total Cooling Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Gross capacity excluding supply air fan heat
       \\note Rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N46, \\field Speed 4 Rated Sensible Heat Ratio
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note Sensible and total capacities do not include supply fan heat
  N47, \\field Speed 4 Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N48, \\field Speed 4 Rated Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total cooling capacity, rated SHR and rated
       \\note COP should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note cooling capacity for Speed 4
  N49, \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total cooling capacity.
       \\note This value is only used to calculate Seasonal Energy Efficiency Ratio (SEER), and the
       \\note Standard Rating (Net) Cooling Capacity which will be outputs in the EnergyPlus eio file.
       \\note This value is not used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A31, \\field Speed 4 Total Cooling Capacity Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A32, \\field Speed 4 Total Cooling Capacity Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A33, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A34, \\field Speed 4 Energy Input Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A35, \\field Speed 4 Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (cooling load/steady state capacity)
  N50, \\field Speed 4 Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
  N51, \\field Speed 4 Ratio of Initial Moisture Evaporation Rate and steady state Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
       \\note the compressor first turns off) and the coil's steady state latent capacity
       \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
  N52, \\field Speed 4 Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation
       \\note model is disabled.
  N53, \\field Speed 4 Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation
       \\note model is disabled.
  N54, \\field Speed 4 Rated Waste Heat Fraction of Power Input
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note Recoverable waste heat at full load and rated conditions
  A36, \\field Speed 4 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N55, \\field Speed 4 Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N56, \\field Speed 4 Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N57; \\field Speed 4 Rated Evaporative Condenser Pump Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump at Speed 4

Coil:Cooling:DX:VariableSpeed,
        \\memo Direct expansion (DX) cooling coil and condensing unit (includes electric compressor
        \\memo and condenser fan), variable-speed. Optional inputs for moisture evaporation from
        \\memo wet coil when compressor cycles off with continuous fan operation. Requires two to
        \\memo ten sets of performance data and will interpolate between speeds. Modeled as a
        \\memo single coil with variable-speed compressor.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference CoolingCoilsDXVariableSpeed
   A2,  \\field Indoor Air Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Indoor Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Number of Speeds
        \\units dimensionless
        \\type integer
        \\required-field
        \\minimum 1
        \\maximum 10
        \\default 2
   N2,  \\field Nominal Speed Level
        \\units dimensionless
        \\type integer
        \\default 2
        \\required-field
        \\note must be lower than or equal to the highest speed number
   N3,  \\field Rated Total Cooling Capacity At Selected Nominal Speed Level
        \\units w
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N4,  \\field Rated Volumetric Air Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N5,  \\field Nominal Time for Condensate to Begin Leaving the Coil
        \\units s
        \\type real
        \\minimum 0
        \\required-field
        \\default 0
   N6,  \\field Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity
        \\units dimensionless
        \\type real
        \\minimum 0
        \\default 0
        \\required-field
   A4,  \\field Energy Part Load Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*PLR + c*PLR**2
        \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
        \\note PLR = part load ratio (cooling load/steady state capacity)
   A5,  \\field Condenser Air Inlet Node Name
        \\type node
        \\note Enter the name of an outdoor air node. This node name is also specified in
        \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
   A6,  \\field Condenser Type
        \\type choice
        \\key AirCooled
        \\key EvaporativelyCooled
        \\default AirCooled
   N7,  \\field Evaporative Condenser Pump Rated Power Consumption
        \\type real
        \\units W
        \\minimum 0.0
        \\default 0.0
        \\autosizable
        \\note Rated power consumed by the evaporative condenser's water pump
   N8,  \\field Crankcase Heater Capacity
        \\type real
        \\minimum 0.0
        \\default 0.0
        \\units W
        \\ip-units W
   N9,  \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
        \\type real
        \\minimum 0.0
        \\default 10.0
        \\units C
   A7,  \\field Supply Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
   A8,  \\field Condensate Collection Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
   N10, \\field Basin Heater Capacity
        \\type real
        \\units W/K
        \\minimum 0.0
        \\default 0.0
        \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
        \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
        \\note For this situation, the heater maintains the basin water temperature at the basin heater
        \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
        \\note The basin heater only operates when the DX coil is off.
   N11, \\field Basin Heater Setpoint Temperature
        \\type real
        \\units C
        \\minimum 2.0
        \\default 2.0
        \\note This field is only used for Condenser Type = EvaporativelyCooled.
        \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
   A9,  \\field Basin Heater Operating Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note This field is only used for Condenser Type = EvaporativelyCooled.
        \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
        \\note air dry-bulb temperature is below the basin heater setpoint temperature.
        \\note If a schedule name is not entered, the basin heater is allowed to operate
        \\note throughout the entire simulation.
   N12, \\field Speed 1 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
        \\required-field
   N13, \\field Speed 1 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
        \\required-field
   N14, \\field Speed 1 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   N15, \\field Speed 1 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   N16, \\field Speed 1 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\note This field is only used for Condenser Type = EvaporativelyCooled
   N17, \\field Speed 1 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\note This field is only used for Condenser Type = EvaporativelyCooled
   A10, \\field Speed 1 Total Cooling Capacity Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A11, \\field Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A12, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A13, \\field Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N18, \\field Speed 2 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N19, \\field Speed 2 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N20, \\field Speed 2 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N21, \\field Speed 2 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N22, \\field Speed 2 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N23, \\field Speed 2 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A14, \\field Speed 2 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A15, \\field Speed 2 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A16, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A17, \\field Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N24, \\field Speed 3 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N25, \\field Speed 3 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N26, \\field Speed 3 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N27, \\field Speed 3 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N28, \\field Speed 3 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N29, \\field Speed 3 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A18, \\field Speed 3 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A19, \\field Speed 3 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A20, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A21, \\field Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N30, \\field Speed 4 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N31, \\field Speed 4 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N32, \\field Speed 4 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N33, \\field Speed 4 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N34, \\field Speed 4 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N35, \\field Speed 4 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A22, \\field Speed 4 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A23, \\field Speed 4 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A24, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A25, \\field Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N36, \\field Speed 5 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N37, \\field Speed 5 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N38, \\field Speed 5 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N39, \\field Speed 5 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N40, \\field Speed 5 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N41, \\field Speed 5 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A26, \\field Speed 5 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A27, \\field Speed 5 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A28, \\field Speed 5 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A29, \\field Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N42, \\field Speed 6 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N43, \\field Speed 6 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N44, \\field Speed 6 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N45, \\field Speed 6 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N46, \\field Speed 6 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N47, \\field Speed 6 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A30, \\field Speed 6 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A31, \\field Speed 6 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A32, \\field Speed 6 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A33, \\field Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N48, \\field Speed 7 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N49, \\field Speed 7 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N50, \\field Speed 7 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N51, \\field Speed 7 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N52, \\field Speed 7 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N53, \\field Speed 7 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A34, \\field Speed 7 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A35, \\field Speed 7 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A36, \\field Speed 7 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A37, \\field Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N54, \\field Speed 8 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N55, \\field Speed 8 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N56, \\field Speed 8 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N57, \\field Speed 8 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N58, \\field Speed 8 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N59, \\field Speed 8 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
   A38, \\field Speed 8 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A39, \\field Speed 8 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A40, \\field Speed 8 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A41, \\field Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N60, \\field Speed 9 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N61, \\field Speed 9 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N62, \\field Speed 9 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N63, \\field Speed 9 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N64, \\field Speed 9 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\note optional
   N65, \\field Speed 9 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\note optional
   A42, \\field Speed 9 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A43, \\field Speed 9 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A44, \\field Speed 9 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A45, \\field Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N66, \\field Speed 10 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N67, \\field Speed 10 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N68, \\field Speed 10 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N69, \\field Speed 10 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N70, \\field Speed 10 Reference Unit Condenser Flow Rate at Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\note optional
   N71, \\field Speed 10 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\note optional
   A46, \\field Speed 10 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A47, \\field Speed 10 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A48, \\field Speed 10 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*odb + e*odb**2 + f*wb*odb
        \\note wb = entering wet-bulb temperature (C)
        \\note odb = air entering temperature seen by the condenser (C)
   A49; \\field Speed 10 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow

Coil:Cooling:DX:TwoStageWithHumidityControlMode,
        \\memo Direct expansion (DX) cooling coil and condensing unit (includes electric compressor
        \\memo and condenser fan), two-stage with humidity control mode (e.g. sub-cool or hot gas
        \\memo reheat). Optional inputs for moisture evaporation from wet coil when compressor
        \\memo cycles off with continuous fan operation. Requires two to four sets of performance
        \\memo data, see CoilPerformance:DX:Cooling. Stages are modeled as a face-split coil.
  \\min-fields 10
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingCoilsDX
       \\reference CoolingCoilsDXMultiMode
       \\reference CoolingCoilsDXMultiModeOrSingleSpeed
       \\reference DesuperHeatingCoilSources
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  N1 , \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\units W
       \\ip-units W
  N2 , \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0.0
       \\default 10.0
       \\units C
  N3 , \\field Number of Capacity Stages
       \\type integer
       \\minimum 1
       \\maximum 2
       \\default 1
  N4 , \\field Number of Enhanced Dehumidification Modes
       \\type integer
       \\minimum 0
       \\maximum 1
       \\default 0
  A5 , \\field Normal Mode Stage 1 Coil Performance Object Type
       \\required-field
       \\type choice
       \\key CoilPerformance:DX:Cooling
  A6 , \\field Normal Mode Stage 1 Coil Performance Name
       \\required-field
       \\type object-list
       \\object-list CoilPerformanceDX
  A7 , \\field Normal Mode Stage 1+2 Coil Performance Object Type
       \\type choice
       \\key CoilPerformance:DX:Cooling
  A8 , \\field Normal Mode Stage 1+2 Coil Performance Name
       \\type object-list
       \\object-list CoilPerformanceDX
  A9 , \\field Dehumidification Mode 1 Stage 1 Coil Performance Object Type
       \\type choice
       \\key CoilPerformance:DX:Cooling
  A10, \\field Dehumidification Mode 1 Stage 1 Coil Performance Name
       \\type object-list
       \\object-list CoilPerformanceDX
  A11, \\field Dehumidification Mode 1 Stage 1+2 Coil Performance Object Type
       \\type choice
       \\key CoilPerformance:DX:Cooling
  A12, \\field Dehumidification Mode 1 Stage 1+2 Coil Performance Name
       \\type object-list
       \\object-list CoilPerformanceDX
  A13, \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A14, \\field Condensate Collection Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  N5,  \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the DX coil is off.
  N6,  \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A15; \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.

CoilPerformance:DX:Cooling,
       \\memo Used to specify DX cooling coil performance for one mode of operation for a
       \\memo Coil:Cooling:DX:TwoStageWithHumidityControlMode object which may reference one to four
       \\memo CoilPerformance:DX:Cooling objects depending on the specified number of stages and
       \\memo dehumidification modes. In nearly all cases, the Rated Air Flow Rate will be the same
       \\memo for all performance objects associated with a given coil. If bypass is specified,
       \\memo the Rated Air Flow Rate includes both the bypassed flow and the flow through the
       \\memo active part of the coil.
  \\min-fields 11
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoilPerformanceDX
  N1 , \\field Rated Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note gross capacity excluding supply air fan heat
       \\note rating point: air entering the cooling coil at 26.7 C dry-bulb/19.4 C wet-bulb, and
       \\note air entering the outdoor condenser coil at 35 C dry-bulb/23.9 C wet-bulb
  N2 , \\field Rated Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum 0.5
       \\maximum 1.0
       \\autosizable
       \\note Rated sensible heat ratio (gross sensible capacity/gross total capacity)
       \\note sensible and total capacities do not include supply fan heat
  N3 , \\field Rated COP
       \\note Gross cooling capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply fan heat or supply fan electrical energy input
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.0
  N4 , \\field Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to Rated total Cooling capacity, Rated SHR and Rated COP
  N5 , \\field Fraction of Air Flow Bypassed Around Coil
       \\type real
       \\minimum 0.0
       \\maximum< 1.0
       \\default 0.0
       \\note Fraction of Rated Air Flow Rate which bypasses the cooling coil
       \\note in this performance mode.  The remaining portion of the flow
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total cooling capacity.
       \\note This is used to model face-split coils on multi-stage units or bypass dampers.
       \\note If total flow rate varies during simulation, the same fraction is bypassed.
  A2 , \\field Total Cooling Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A3 , \\field Total Cooling Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A4 , \\field Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*edb + e*edb**2 + f*wb*edb
       \\note wb = entering wet-bulb temperature (C)
       \\note edb = dry-bulb temperature seen by the condenser (C)
  A5 , \\field Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A6 , \\field Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (cooling load/steady state capacity)
  N6 , \\field Nominal Time for Condensate Removal to Begin
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 3000.0
       \\default 0.0
       \\note The nominal time for condensate to begin leaving the coil's condensate
       \\note drain line at the coil's rated air flow rate and temperature conditions.
       \\note Nominal time is equal to the ratio of the energy of the coil's maximum
       \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
       \\note Suggested value is 1000; zero value means latent degradation model is disabled.
  N7 , \\field Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
       \\note the compressor first turns off) and the coil's steady state latent capacity
       \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
       \\note means latent degradation model is disabled.
  N8 , \\field Maximum Cycling Rate
       \\type real
       \\units cycles/hr
       \\minimum 0.0
       \\maximum 5.0
       \\default 0.0
       \\note The maximum on-off cycling rate for the compressor, which occurs at 50% run time
       \\note fraction. Suggested value is 3; zero value means latent degradation model is disabled.
  N9 , \\field Latent Capacity Time Constant
       \\type real
       \\units s
       \\minimum 0.0
       \\maximum 500.0
       \\default 0.0
       \\note Time constant for the cooling coil's latent capacity to reach steady state after
       \\note startup. Suggested value is 45; zero value means latent degradation model is disabled.
  A7 , \\field Condenser Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.
  A8 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N10 , \\field Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
  N11, \\field Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use
  N12, \\field Evaporative Condenser Pump Rated Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump
  A9,  \\field Sensible Heat Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*wb + c*wb**2 + d*db + e*db**2 + f*wb*db
       \\note wb = entering wet-bulb temperature seen by the DX cooling coil (C)
       \\note db = entering dry-bulb temperature seen by the DX cooling coil (C)
       \\note entering temperature can be outside air or pretreated air.
  A10; \\field Sensible Heat Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow

Coil:Cooling:DX:VariableRefrigerantFlow,
       \\memo Variable refrigerant flow (VRF) direct expansion (DX) cooling coil. Used with
       \\memo ZoneHVAC:TerminalUnit:VariableRefrigerantFlow. Condensing unit is modeled separately,
       \\memo see AirConditioner:VariableRefrigerantFlow.
  \\min-fields 9
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingCoilsDXVarRefrigFlow
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Rated Total Cooling Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Cooling capacity excluding supply air fan heat
  N2 , \\field Rated Sensible Heat Ratio
       \\required-field
       \\type real
       \\minimum> 0.0
       \\autosizable
       \\note Sensible heat ratio excluding supply air fan heat
  N3 , \\field Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Volume flow rate corresponding to rated total cooling capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total cooling capacity
  A3 , \\field Cooling Capacity Ratio Modifier Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\object-list BiquadraticCurves
  A4 , \\field Cooling Capacity Modifier Curve Function of Flow Fraction Name
       \\required-field
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A5 , \\field Coil Air Inlet Node
       \\required-field
       \\type node
  A6 , \\field Coil Air Outlet Node
       \\required-field
       \\type node
  A7 ; \\field Name of Water Storage Tank for Condensate Collection
       \\type object-list
       \\object-list WaterStorageTankNames

Coil:Heating:DX:VariableRefrigerantFlow,
       \\memo Variable refrigerant flow (VRF) direct expansion (DX) heating coil (air-to-air heat
       \\memo pump). Used with ZoneHVAC:TerminalUnit:VariableRefrigerantFlow. Condensing unit is
       \\memo modeled separately, see AirConditioner:VariableRefrigerantFlow.
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilsDX
       \\reference HeatingCoilsDXSingleSpeed
       \\reference HeatingCoilsDXVarRefrigFlow
  A2 , \\field Availability Schedule
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Rated Total Heating Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outside dry-bulb temp 8.33 C, outside wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering wetbulb 15.55 C
  N2 , \\field Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note volume flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total heating capacity
  A3 , \\field Coil Air Inlet Node
       \\required-field
       \\type node
  A4 , \\field Coil Air Outlet Node
       \\required-field
       \\type node
  A5 , \\field Heating Capacity Ratio Modifier Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\object-list BiquadraticCurves
  A6 ; \\field Heating Capacity Modifier Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow

Coil:Heating:Water,
       \\memo Hot water heating coil, NTU-effectiveness model, assumes a cross-flow heat exchanger.
       \\memo Two options for capacity inputs: UA and water flow rate or capacity and design
       \\memo temperatures.
       \\min-fields 15
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilName
       \\reference HeatingCoilsWater
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field U-Factor Times Area Value
       \\note UA value under rating conditions
       \\units W/K
       \\autosizable
       \\default autosize
  N2 , \\field Maximum Water Flow Rate
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\default autosize
  A3 , \\field Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A6 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A7 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key NominalCapacity
       \\default UFactorTimesAreaAndDesignWaterFlowRate
  N3 , \\field Rated Capacity
       \\type real
       \\units W
       \\autosizable
       \\minimum 0
       \\default autosize
  N4 , \\field Rated Inlet Water Temperature
       \\units C
       \\type real
       \\default 82.2
  N5, \\field Rated Inlet Air Temperature
      \\units C
      \\type real
      \\default 16.6
  N6, \\field Rated Outlet Water Temperature
      \\units C
      \\type real
      \\default 71.1
  N7, \\field Rated Outlet Air Temperature
      \\units C
      \\type real
      \\default 32.2
  N8; \\field Rated Ratio for Air and Water Convection
      \\type real
      \\default 0.5
      \\minimum> 0

Coil:Heating:Steam,
       \\memo Steam heating coil. Condenses and sub-cools steam at loop pressure and discharges
       \\memo condensate through steam traps to low pressure condensate line.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilName
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Maximum Steam Flow Rate
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Degree of SubCooling
       \\units C
       \\minimum 1.0
       \\maximum 5.0
  N3 , \\field Degree of Loop SubCooling
       \\units C
       \\minimum 10.0
       \\default 20.0
  A3 , \\field Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A6 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A7 , \\field Coil Control Type
       \\note Use ZoneLoadControl if the coil is contained within another component such as an air
       \\note terminal unit, zone HVAC equipment, or unitary system. Use TemperatureSetpointControl
       \\note if the coil is located directly in an air loop branch or outdoor air equipment list.
       \\type choice
       \\key TemperatureSetpointControl
       \\key ZoneLoadControl
  A8 ; \\field Temperature Setpoint Node Name
       \\type node
       \\note Required if Coil Control Type is TemperatureSetpointControl

Coil:Heating:Electric,
       \\memo Electric heating coil. If the coil is located directly in an air loop branch or
       \\memo outdoor air equipment list, then it is controlled on leaving air temperature and
       \\memo the Temperature Setpoint Node Name must be specified. If the coil is contained within
       \\memo another component such as an air terminal unit, zone HVAC equipment, or unitary
       \\memo system, then the coil is controlled by the parent component and the setpoint node name
       \\memo is not entered.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference HeatingCoilName
       \\reference HeatingCoilsGasElec
       \\reference HeatingCoilsGasElecDXSS
       \\reference HeatingCoilsGasElecDesup
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Efficiency
       \\maximum 1.0
       \\minimum> 0.0
       \\default 1.0
  N2 , \\field Nominal Capacity
       \\units W
       \\autosizable
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 ; \\field Temperature Setpoint Node Name
       \\type node
       \\note Required if coil is temperature controlled.

Coil:Heating:Electric:MultiStage,
       \\memo Electric heating coil, multi-stage. If the coil is located directly in an air loop
       \\memo branch or outdoor air equipment list, then it is controlled on leaving air temperature
       \\memo and the Temperature Setpoint Node Name must be specified. If the coil is contained
       \\memo within another component such as an air terminal unit, zone HVAC equipment, or unitary
       \\memo system, then the coil is controlled by the parent component and the setpoint node name
       \\memo is not entered.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference HeatingCoilsElectricMultiStage
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Temperature Setpoint Node Name
       \\type node
       \\note Required if coil is temperature controlled.
       \\note controlled
  N1 , \\field Number of Stages
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 4
       \\note Enter the number of the following sets of data for coil
       \\note capacity and Efficiency.
  N2, \\field Stage 1 Efficiency
       \\required-field
       \\type real
       \\units W/W
       \\minimum> 0.0
  N3, \\field Stage 1 Nominal Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N4, \\field Stage 2 Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N5, \\field Stage 2 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N6, \\field Stage 3 Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N7, \\field Stage 3 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N8, \\field Stage 4 Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N9; \\field Stage 4 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable

Coil:Heating:Gas,
       \\memo Gas heating coil. If the coil is located directly in an air loop branch or
       \\memo outdoor air equipment list, then it is controlled on leaving air temperature and
       \\memo the Temperature Setpoint Node Name must be specified. If the coil is contained within
       \\memo another component such as an air terminal unit, zone HVAC equipment, or unitary
       \\memo system, then the coil is controlled by the parent component and the setpoint node name
       \\memo is not entered.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilName
       \\reference HeatingCoilsGasElec
       \\reference HeatingCoilsGasElecDXSS
       \\reference HeatingCoilsGasElecDesup
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Gas Burner Efficiency
       \\maximum 1.0
       \\minimum 0.0
       \\default 0.8
  N2 , \\field Nominal Capacity
       \\units W
       \\autosizable
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Temperature Setpoint Node Name
       \\type node
       \\note optional, used if coil is temperature control and not load-base controlled
  N3 , \\field Parasitic Electric Load
       \\units W
       \\note parasitic electric load associated with the gas coil operation
       \\note such as an inducer fan, etc..  This will be modified by the part load ratio to reflect
       \\note the time of operation in a timestep.
       \\ip-units W
  A6 , \\field Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve, PLF = a + b*PLR + c*PLR**2
       \\note cubic curve, PLF = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLF = part load fraction
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
       \\note Coil runtime fraction = Part Load Ratio / PLF
       \\note This part load degradation is for coil performance & will
       \\note increase the gas consumption of the coil due to transient coil operation.
  N4 ; \\field Parasitic Gas Load
       \\units W
       \\note parasitic gas load associated with the gas coil operation (i.e., standing pilot)

Coil:Heating:Gas:MultiStage,
       \\memo Gas heating coil, multi-stage. If the coil is located directly in an air loop
       \\memo branch or outdoor air equipment list, then it is controlled on leaving air temperature
       \\memo and the Temperature Setpoint Node Name must be specified. If the coil is contained
       \\memo within another component such as an air terminal unit, zone HVAC equipment, or unitary
       \\memo system, then the coil is controlled by the parent component and the setpoint node name
       \\memo is not entered.
       \\min-fields 8
  A1 , \\field Name
       \\required-field
       \\reference HeatingCoilsGasMultiStage
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Temperature Setpoint Node Name
       \\type node
       \\note optional, used if coil is temperature control and not load-base
       \\note controlled.
  A6 , \\field Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve, PLF = a + b*PLR + c*PLR**2
       \\note cubic curve, PLF = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLF = part load fraction
       \\note PLR = part load ratio (sensible heating load/steady state
       \\note heating capacity)
       \\note Coil runtime fraction = Part Load Ratio / PLF
       \\note This part load degradation is for coil performance & will
       \\note increase the gas consumption of the coil due to transient coil
       \\note operation.
  N1,  \\field Parasitic Gas Load
       \\units W
       \\note parasitic gas load associated with the gas coil operation (i.e.,
       \\note standing pilot)
  N2 , \\field Number of Stages
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 4
       \\note Enter the number of the following sets of data for coil
       \\note capacity and Gas Burner Efficiency.
  N3, \\field Stage 1 Gas Burner Efficiency
       \\required-field
       \\type real
       \\units W/W
       \\minimum> 0.0
  N4, \\field Stage 1 Nominal Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N5, \\field Stage 1 Parasitic Electric Load
       \\units W
       \\note Stage 1 parasitic electric load associated with the gas coil operation
       \\note such as an inducer fan, etc.  This will be modified by the part
       \\note load ratio to reflect the time of operation in a timestep.
       \\ip-units W
  N6, \\field Stage 2 Gas Burner Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N7, \\field Stage 2 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N8, \\field Stage 2 Parasitic Electric Load
       \\units W
       \\note Stage 2 parasitic electric load associated with the gas coil operation
       \\note such as an inducer fan, etc.  This will be modified by the part
       \\note load ratio to reflect the time of operation in a timestep.
       \\ip-units W
  N9, \\field Stage 3 Gas Burner Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N10, \\field Stage 3 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N11, \\field Stage 3 Parasitic Electric Load
       \\units W
       \\note Stage 3 parasitic electric load associated with the gas coil operation
       \\note such as an inducer fan, etc.  This will be modified by the part
       \\note load ratio to reflect the time of operation in a timestep.
       \\ip-units W
  N12, \\field Stage 4 Gas Burner Efficiency
       \\type real
       \\units W/W
       \\minimum> 0.0
  N13, \\field Stage 4 Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
  N14; \\field Stage 4 Parasitic Electric Load
       \\units W
       \\note Stage 4 parasitic electric load associated with the gas coil operation
       \\note such as an inducer fan, etc.  This will be modified by the part
       \\note load ratio to reflect the time of operation in a timestep.
       \\ip-units W

Coil:Heating:Desuperheater,
       \\memo Desuperheater air heating coil. The heating energy provided by this coil is reclaimed
       \\memo from the superheated refrigerant gas leaving a compressor and does not impact the
       \\memo performance of the compressor. If the coil is located directly in an air loop
       \\memo branch or outdoor air equipment list, then it is controlled on leaving air temperature
       \\memo and the Temperature Setpoint Node Name must be specified. If the coil is contained
       \\memo within another component such as a unitary system, then the coil is controlled by the
       \\memo parent component and the setpoint node name is not entered.
       \\min-fields 7
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilsGasElecDesup
       \\reference HeatingCoilsDesuperheater
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Heat Reclaim Recovery Efficiency
       \\type real
       \\minimum 0.0
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Heating Source Object Type
       \\required-field
       \\type choice
       \\key Coil:Cooling:DX:SingleSpeed
       \\key Coil:Cooling:DX:TwoSpeed
       \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
       \\key Refrigeration:CompressorRack
       \\key Refrigeration:Condenser:AirCooled
       \\key Refrigeration:Condenser:EvaporativeCooled
       \\key Refrigeration:Condenser:WaterCooled
  A6 , \\field Heating Source Name
       \\required-field
       \\type object-list
       \\object-list DesuperHeatingCoilSources
  A7 , \\field Temperature Setpoint Node Name
       \\type node
       \\note Required if coil is temperature controlled.
       \\note Temperature-based control requires the use of a SetpointManager object
  N2 ; \\field Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0
       \\note parasitic electric load associated with the desuperheater coil operation
       \\note such as solenoid valves, etc.
       \\ip-units W

Coil:Heating:DX:SingleSpeed,
       \\memo Direct expansion (DX) heating coil (air-to-air heat pump) and compressor unit
       \\memo (includes electric compressor and outdoor fan), single-speed, with defrost controls.
       \\min-fields 21
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatingCoilsDX
       \\reference HeatingCoilsGasElecDXSS
       \\reference HeatingCoilsDXSingleSpeed
       \\reference AFNCoilNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Rated Total Heating Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering wet-bulb 15.55 C
  N2 , \\field Rated COP
       \\note Rated heating capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply air fan heat or supply air fan electrical energy
       \\required-field
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\note does not include supply air fan heat or supply air fan electrical energy
  N3 , \\field Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total heating capacity
  N4 , \\field Rated Evaporator Fan Power Per Volume Flow Rate
       \\note Enter the evaporator fan power per air volume flow rate at the rated test conditions.
       \\note The test conditions vary external static pressure based on total heating capacity.
       \\note This value is only used to calculate Heating Seasonal Performance Factor(HSPF). This value is not
       \\note used for modeling the evaporator fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Total Heating Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note Biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the heating capacity to be sensitive to both iat and oat.
  A6 , \\field Total Heating Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
 A7 ,  \\field Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the energy input ratio to be sensitive to both iat and oat.
  A8 , \\field Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A9 , \\field Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
  A10, \\field Defrost Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note biquadratic curve = a + b*wb + c*wb**2 + d*oat + e*oat**2 + f*wb*oat
       \\note wb = wet-bulb temperature (C) of air entering the indoor coil
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note only required if ReverseCycle defrost strategy is specified
  N5 , \\field Minimum Outdoor Dry-Bulb Temperature for Compressor Operation
       \\type real
       \\minimum -20.0
       \\default -8.0
       \\units C
  N6 , \\field Outdoor Dry-Bulb Temperature to Turn On Compressor
       \\type real
       \\units C
       \\note The outdoor tempearture when the compressor is automatically turned back on following an
       \\note automatic shut off because of low outdoor dry-bulb temperature. This field is only used
       \\note for the calculation of HSPF. If this field is not provided, then outdoor bin temperature
       \\note used in the HSPF calculation is always considered to be greater than this temperature and
       \\note 'Minimum Outdoor Dry-Bulb Temperature for Compressor Operation' field described above.
       \\note This assumption is based on AHRI standard 210/240 (2008) and can introduce significant error
       \\note in the final value of HSPF.
  N7 , \\field Maximum Outdoor Dry-Bulb Temperature for Defrost Operation
       \\type real
       \\minimum 0.0
       \\maximum 7.22
       \\default 5.0
       \\units C
  N8,  \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\units W
       \\ip-units W
  N9,  \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0.0
       \\default 10.0
       \\units C
  A11, \\field Defrost Strategy
       \\type choice
       \\key ReverseCycle
       \\key Resistive
       \\default ReverseCycle
  A12, \\field Defrost Control
       \\type choice
       \\key Timed
       \\key OnDemand
       \\default Timed
  N10,  \\field Defrost Time Period Fraction
       \\type real
       \\minimum 0.0
       \\default 0.058333
       \\note Fraction of time in defrost mode
       \\note only applicable if timed defrost control is specified
  N11,  \\field Resistive Defrost Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\units W
       \\note only applicable if resistive defrost strategy is specified
       \\ip-units W
  N12, \\field Region number for calculating HSPF
       \\type integer
       \\minimum 1
       \\maximum 6
       \\default 4
       \\note Standard Region number for which HSPF and other standard ratings are calculated
  A13; \\field Evaporator Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node. This node name is also specified in
       \\note an OutdoorAir:Node or OutdoorAir:NodeList object.

Coil:Heating:DX:MultiSpeed,
       \\memo Direct expansion (DX) heating coil (air-to-air heat pump) and compressor unit
       \\memo (includes electric or engine-driven compressor and outdoor fan), multi-speed
       \\memo (or variable-speed), with defrost controls. Requires two to four sets
       \\memo of performance data and will interpolate between speeds.
       \\min-fields 40
  A1 , \\field Name
       \\required-field
       \\reference HeatingCoilsDXMultiSpeed
       \\reference AFNCoilNames
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Air Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Air Outlet Node Name
       \\required-field
       \\type node
  N1 , \\field Minimum Outdoor Dry-Bulb Temperature for Compressor Operation
       \\type real
       \\minimum -20.0
       \\default -8.0
       \\units C
  N2 , \\field Outdoor Dry-Bulb Temperature to Turn On Compressor
       \\type real
       \\units C
       \\note The outdoor temperature when the compressor is automatically turned back on following an
       \\note automatic shut off because of low outdoor dry-bulb temperature. This field is only used
       \\note for the calculation of HSPF. If this field is not provided, then outdoor bin temperature
       \\note used in the HSPF calculation is always considered to be greater than this temperature and
       \\note 'Minimum Outdoor Dry-Bulb Temperature for Compressor Operation' field described above.
       \\note This assumption is based on AHRI standard 210/240 (2008) and can introduce significant error
       \\note in the final value of HSPF.
  N3 , \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\units W
       \\ip-units W
  N4 , \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0.0
       \\default 10.0
       \\units C
  A5 , \\field Defrost Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note biquadratic curve = a + b*wb + c*wb**2 + d*oat + e*oat**2 + f*wb*oat
       \\note wb = wet-bulb temperature (C) of air entering the indoor coil
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note only required if ReverseCycle defrost strategy is specified
  N5 , \\field Maximum Outdoor Dry-Bulb Temperature for Defrost Operation
       \\type real
       \\minimum 0.0
       \\maximum 7.22
       \\default 5.0
       \\units C
  A6 , \\field Defrost Strategy
       \\type choice
       \\key ReverseCycle
       \\key Resistive
       \\default ReverseCycle
  A7 , \\field Defrost Control
       \\type choice
       \\key Timed
       \\key OnDemand
       \\default Timed
  N6 , \\field Defrost Time Period Fraction
       \\type real
       \\minimum 0.0
       \\default 0.058333
       \\note Fraction of time in defrost mode
       \\note only applicable if timed defrost control is specified
  N7 , \\field Resistive Defrost Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\units W
       \\note only applicable if resistive defrost strategy is specified
       \\ip-units W
  A8 , \\field Apply Part Load Fraction to Speeds Greater than 1
       \\type choice
       \\key Yes
       \\key No
       \\default No
  A9 , \\field Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default NaturalGas
  N8,  \\field Region number for Calculating HSPF
       \\type integer
       \\minimum 1
       \\maximum 6
       \\default 4
       \\note Standard Region number for which HSPF and other standard ratings are calculated
  N9 , \\field Number of Speeds
       \\required-field
       \\type integer
       \\minimum 2
       \\maximum 4
       \\note Enter the number of the following sets of data for coil capacity, COP,
       \\note flow rate, and associated curves.
  N10, \\field Speed 1 Rated Total Heating Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering
       \\note wet-bulb 15.55 C
  N11, \\field Speed 1 Rated COP
       \\note Rated heating capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply air fan heat or supply air fan electrical energy
       \\required-field
       \\type real
       \\units W/W
       \\minimum> 0.0
  N12, \\field Speed 1 Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note heating capacity
  N13, \\field Speed 1 Rated Supply Air Fan Power Per Volume Flow Rate
       \\note Enter the supply air fan power per air volume flow rate at the rated speed 1 test conditions.
       \\note The test conditions vary external static pressure based on total heating capacity. This value
       \\note is only used to calculate Heating Seasonal Performance Factor(HSPF). This value is not
       \\note used for modeling the supply air fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A10, \\field Speed 1 Total Heating Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the heating capacity to be sensitive to both iat and oat.
  A11, \\field Speed 1 Total Heating Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A12, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the energy input ratio to be sensitive to both iat and oat.
  A13, \\field Speed 1 Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A14, \\field Speed 1 Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
  N14, \\field Speed 1 Rated Waste Heat Fraction of Power Input
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note recoverable waste heat at full load and rated conditions
  A15, \\field Speed 1 Waste Heat Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N15, \\field Speed 2 Rated Total Heating Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering
       \\note wet-bulb 15.55 C
  N16, \\field Speed 2 Rated COP
       \\note Rated heating capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply air fan heat or supply air fan electrical energy
       \\required-field
       \\type real
       \\units W/W
       \\minimum> 0.0
  N17, \\field Speed 2 Rated Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note heating capacity
  N18, \\field Speed 2 Rated Supply Air Fan Power Per Volume Flow Rate
       \\note Enter the supply air fan power per air volume flow rate at the rated speed 2 test conditions.
       \\note The test conditions vary external static pressure based on total heating capacity. This value
       \\note is only used to calculate Heating Seasonal Performance Factor(HSPF). This value is not
       \\note used for modeling the supply air fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A16, \\field Speed 2 Total Heating Capacity Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the heating capacity to be sensitive to both iat and oat.
  A17, \\field Speed 2 Total Heating Capacity Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A18, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the energy input ratio to be sensitive to both iat and oat.
  A19, \\field Speed 2 Energy Input Ratio Function of Flow Fraction Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A20, \\field Speed 2 Part Load Fraction Correlation Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
  N19, \\field Speed 2 Rated Waste Heat Fraction of Power Input
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note recoverable waste heat at full load and rated conditions
  A21, \\field Speed 2 Waste Heat Function of Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N20, \\field Speed 3 Rated Total Heating Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering
       \\note wet-bulb 15.55 C
  N21, \\field Speed 3 Rated COP
       \\note Rated heating capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply air fan heat or supply air fan electrical energy
       \\type real
       \\units W/W
       \\minimum> 0.0
  N22, \\field Speed 3 Rated Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note heating capacity
  N23, \\field Speed 3 Rated Supply Air Fan Power Per Volume Flow Rate
       \\note Enter the supply air fan power per air volume flow rate at the rated speed 3 test conditions.
       \\note The test conditions vary external static pressure based on total heating capacity. This value
       \\note is only used to calculate Heating Seasonal Performance Factor(HSPF). This value is not
       \\note used for modeling the supply air fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A22, \\field Speed 3 Total Heating Capacity Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the heating capacity to be sensitive to both iat and oat.
  A23, \\field Speed 3 Total Heating Capacity Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A24, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the energy input ratio to be sensitive to both iat and oat.
  A25, \\field Speed 3 Energy Input Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A26, \\field Speed 3 Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
  N24, \\field Speed 3 Rated Waste Heat Fraction of Power Input
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note recoverable waste heat at full load and rated conditions
  A27, \\field Speed 3 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)
  N25, \\field Speed 4 Rated Total Heating Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note capacity excluding supply air fan heat
       \\note rating point outdoor dry-bulb temp 8.33 C, outdoor wet-bulb temp 6.11 C
       \\note rating point heating coil entering air dry-bulb 21.11 C, coil entering
       \\note wet-bulb 15.55 C
  N26, \\field Speed 4 Rated COP
       \\note Rated heating capacity divided by power input to the compressor and outdoor fan,
       \\note does not include supply air fan heat or supply air fan electrical energy
       \\type real
       \\units W/W
       \\minimum> 0.0
  N27, \\field Speed 4 Rated Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Flow rate corresponding to rated total capacity
       \\note should be between 0.00004027 m3/s and .00006041 m3/s per watt of rated total
       \\note heating capacity
  N28, \\field Speed 4 Rated Supply Air Fan Power Per Volume Flow Rate
       \\note Enter the supply air fan power per air volume flow rate at the rated speed 4 test conditions.
       \\note The test conditions vary external static pressure based on total heating capacity. This value
       \\note is only used to calculate Heating Seasonal Performance Factor(HSPF). This value is not
       \\note used for modeling the supply air fan during simulations.
       \\type real
       \\units W/(m3/s)
       \\minimum 0.0
       \\maximum 1250.0
       \\default 773.3
  A28, \\field Speed 4 Total Heating Capacity Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the heating capacity to be sensitive to both iat and oat.
  A29, \\field Speed 4 Total Heating Capacity Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A30, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticQuadraticCubicCurves
       \\note quadratic curve = a + b*oat + c*oat**2
       \\note cubic curve = a + b*oat + c*oat**2 + d*oat**3
       \\note biquadratic curve = a + b*iat + c*iat**2 + d*oat + e*oat**2 + f*iat*oat
       \\note oat = outdoor air dry-bulb temperature (C)
       \\note iat = indoor air dry-bulb temperature (C)
       \\note biquadratic curve is recommended if sufficient manufacturer data is
       \\note available for the energy input ratio to be sensitive to both iat and oat.
  A31, \\field Speed 4 Energy Input Ratio Function of Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*ff + c*ff**2
       \\note cubic curve = a + b*ff + c*ff**2 + d*ff**3
       \\note ff = fraction of the full load flow
  A32, \\field Speed 4 Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
       \\note PLR = part load ratio (sensible heating load/steady state heating capacity)
  N29, \\field Speed 4 Rated Waste Heat Fraction of Power Input
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
       \\note recoverable waste heat at full load and rated conditions
  A33; \\field Speed 4 Waste Heat Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note curve = a + b*odb + c*odb**2 + d*db + e*db**2 + f*odb*db
       \\note odb = Outdoor air dry-bulb temperature (C)
       \\note db = entering coil dry-bulb temperature (C)

Coil:Heating:DX:VariableSpeed,
        \\memo Direct expansion (DX) heating coil (air-to-air heat pump) and compressor unit
        \\memo (includes electric compressor and outdoor fan), variable-speed, with defrost
        \\memo controls. Requires two to ten sets of performance data and will interpolate between
        \\memo speeds.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference HeatingCoilsDXVariableSpeed
   A2,  \\field Indoor Air Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Indoor Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Number of Speeds
        \\units dimensionless
        \\type integer
        \\required-field
        \\minimum 1
        \\maximum 10
        \\default 2
   N2,  \\field Nominal Speed Level
        \\units dimensionless
        \\type integer
        \\default 2
        \\required-field
        \\note must be lower than or equal to the highest speed number
   N3,  \\field Rated Heating Capacity At Selected Nominal Speed Level
        \\units w
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N4,  \\field Rated Volumetric Air Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   A4,  \\field Energy Part Load Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*PLR + c*PLR**2
        \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
        \\note PLR = part load ratio (heating load/steady state capacity)
   A5,  \\field Defrost Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note biquadratic curve = a + b*wb + c*wb**2 + d*oat + e*oat**2 + f*wb*oat
        \\note wb = wet-bulb temperature (C) of air entering the indoor coil
        \\note oat = outdoor air dry-bulb temperature (C)
        \\note only required if ReverseCycle defrost strategy is specified
   N5,  \\field Minimum Outdoor Dry-Bulb Temperature for Compressor Operation
        \\type real
        \\minimum -50.0
        \\default -8.0
        \\units C
   N6,  \\field Outdoor Dry-Bulb Temperature to Turn On Compressor
        \\type real
        \\units C
        \\note The outdoor tempearture when the compressor is automatically turned back on following an
        \\note automatic shut off because of low outdoor dry-bulb temperature. This field is only used
        \\note for the calculation of HSPF. If this field is not provided, then outdoor bin temperature
        \\note used in the HSPF calculation is always considered to be greater than this temperature and
        \\note 'Minimum Outdoor Dry-Bulb Temperature for Compressor Operation' field described above.
        \\note This assumption is based on AHRI standard 210/240 (2008) and can introduce significant error
        \\note in the final value of HSPF.
   N7,  \\field Maximum Outdoor Dry-Bulb Temperature for Defrost Operation
        \\type real
        \\minimum 0.0
        \\maximum 7.22
        \\default 5.0
        \\units C
   N8,  \\field Crankcase Heater Capacity
        \\type real
        \\minimum 0.0
        \\default 0.0
        \\units W
        \\ip-units W
   N9,  \\field Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation
        \\type real
        \\minimum 0.0
        \\default 10.0
        \\units C
   A6,  \\field Defrost Strategy
        \\type choice
        \\key ReverseCycle
        \\key Resistive
        \\default ReverseCycle
   A7,  \\field Defrost Control
        \\type choice
        \\key Timed
        \\key OnDemand
        \\default Timed
   N10, \\field Defrost Time Period Fraction
        \\type real
        \\minimum 0.0
        \\default 0.058333
        \\note Fraction of time in defrost mode
        \\note only applicable if timed defrost control is specified
   N11, \\field Resistive Defrost Heater Capacity
        \\type real
        \\minimum 0.0
        \\default 0.0
        \\autosizable
        \\units W
        \\note only applicable if resistive defrost strategy is specified
        \\ip-units W
   N12, \\field Speed 1 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
        \\required-field
   N13, \\field Speed 1 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   N14,  \\field Speed 1 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   A8,  \\field Speed 1 Total Heating Capacity Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A9,  \\field Speed 1 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A10, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A11, \\field Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N15, \\field Speed 2 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N16, \\field Speed 2 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N17, \\field Speed 2 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A12, \\field Speed 2 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A13, \\field Speed 2 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A14, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A15, \\field Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N18, \\field Speed 3 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N19, \\field Speed 3 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N20, \\field Speed 3 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A16, \\field Speed 3 Total Heating Capacity Function of Temperature Curve Name
         \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A17, \\field Speed 3 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A18, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A19, \\field Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N21, \\field Speed 4 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N22, \\field Speed 4 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N23, \\field Speed 4 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A20, \\field Speed 4 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A21, \\field Speed 4 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A22, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A23, \\field Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N24, \\field Speed 5 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N25, \\field Speed 5 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N26, \\field Speed 5 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A24, \\field Speed 5 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A25, \\field Speed 5 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A26, \\field Speed 5 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A27, \\field Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N27, \\field Speed 6 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N28, \\field Speed 6 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N29, \\field Speed 6 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A28, \\field Speed 6 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A29, \\field Speed 6 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A30, \\field Speed 6 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A31, \\field Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N30, \\field Speed 7 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N31, \\field Speed 7 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N32, \\field Speed 7 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A32, \\field Speed 7 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A33, \\field Speed 7 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A34, \\field Speed 7 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A35, \\field Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name
         \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N33, \\field Speed 8 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N34, \\field Speed 8 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N35, \\field Speed 8 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A36, \\field Speed 8 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A37, \\field Speed 8 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A38, \\field Speed 8 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A39, \\field Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N36, \\field Speed 9 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N37, \\field Speed 9 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N38, \\field Speed 9 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A40, \\field Speed 9 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A41, \\field Speed 9 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A42, \\field Speed 9 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A43, \\field Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   N39, \\field Speed 10 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N40, \\field Speed 10 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N41, \\field Speed 10 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A44, \\field Speed 10 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A45, \\field Speed 10 Total  Heating Capacity Function of Air Flow Fraction Curve Name
         \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A46, \\field Speed 10 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*oat + e*oat**2 + f*db*oat
        \\note db = entering air dry bulb temperature (C)
        \\note oat = air entering temperature seen by the evaporator (C)
   A47; \\field Speed 10 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow

Coil:Cooling:WaterToAirHeatPump:ParameterEstimation,
        \\memo Direct expansion (DX) cooling coil for water-to-air heat pump (includes electric
        \\memo compressor), single-speed, parameter estimation model. Optional inputs for moisture
        \\memo evaporation from wet coil when compressor cycles off with continuous fan operation.
        \\memo Parameter estimation model is a deterministic model that requires a consistent set of
        \\memo parameters to describe the operating conditions of the heat pump components.
   A1 , \\field Name
        \\required-field
        \\type alpha
        \\reference CoolingCoilsWaterToAirHP
   A2 , \\field Compressor Type
        \\required-field
        \\type choice
        \\key Reciprocating
        \\key Rotary
        \\key Scroll
        \\note Parameters 1-5 are as named below.
        \\note Parameters 6-10 depend on the type of compressor and fluid.
        \\note Refer to the InputOutputReference on the parameters required
   A3,  \\field Refrigerant Type
        \\type object-list
        \\object-list FluidNames
        \\required-field
        \\default R22
   N1,  \\field Design Source Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N2,  \\field Nominal Cooling Coil Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N3,  \\field Nominal Time for Condensate Removal to Begin
        \\type real
        \\units s
        \\minimum 0.0
        \\maximum 3000.0
        \\default 0.0
        \\note The nominal time for condensate to begin leaving the coil's condensate
        \\note drain line at the coil's rated air flow rate and temperature conditions.
        \\note Nominal time is equal to the ratio of the energy of the coil's maximum
        \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
        \\note Suggested value is 1000; zero value means latent degradation model is disabled.
   N4,  \\field Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\maximum 5.0
        \\default 0.0
        \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
        \\note the compressor first turns off) and the coil's steady state latent capacity
        \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
        \\note means latent degradation model is disabled.
   N5,  \\field High Pressure Cutoff
        \\required-field
        \\minimum> 0.0
        \\units Pa
   N6,  \\field Low Pressure Cutoff
        \\required-field
        \\type real
        \\minimum 0.0
        \\units Pa
   A4,  \\field Water Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Water Outlet Node Name
        \\required-field
        \\type node
   A6,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A7,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N7,  \\field Load Side Total Heat Transfer Coefficient
        \\note Previously called Parameter 1
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units W/K
   N8,  \\field Load Side Outside Surface Heat Transfer Coefficient
        \\note Previously called Parameter 2
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units W/K
   N9,  \\field Superheat Temperature at the Evaporator Outlet
        \\note Previously called Parameter 3
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units C
   N10, \\field Compressor Power Losses
        \\note Accounts for the loss of work due to mechanical and electrical losses in the compressor.
        \\note Previously called Parameter 4
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units W
   N11, \\field Compressor Efficiency
        \\note Previously called Parameter 5
        \\required-field
        \\minimum> 0.0
        \\type real
   N12, \\field Compressor Piston Displacement
        \\note Use when Compressor Type is Reciprocating or Rotary
        \\note Leave this field blank for Compressor Type is Scroll.
        \\note Previously part of Parameter 6
        \\units m3/s
        \\minimum> 0.0
        \\type real
   N13, \\field Compressor Suction/Discharge Pressure Drop
        \\note Used when Compressor Type is Rotary or Reciprocating
        \\note Leave this field blank for Compressor Type is Scroll.
        \\note Previously part of Parameter 7
        \\units Pa
        \\minimum> 0.0
        \\type real
   N14, \\field Compressor Clearance Factor
        \\note Used when Compressor Type is Reciprocating.
        \\note Leave this field blank for Compressor Type is Rotary or Scroll.
        \\note Previously part of Parameter 8
        \\units dimensionless
        \\minimum> 0.0
        \\type real
   N15, \\field Refrigerant Volume Flow Rate
        \\note Use when Compressor Type is Scroll
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 6
        \\units m3/s
        \\minimum> 0.0
        \\type real
   N16, \\field Volume Ratio
        \\note Use when Compressor Type is Scroll.
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 7
        \\units dimensionless
        \\minimum> 0.0
        \\type real
   N17, \\field Leak Rate Coefficient
        \\note Use when Compressor Type is Scroll.
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 8
        \\minimum 0.0
        \\type real
   N18, \\field Source Side Heat Transfer Coefficient
        \\note Use when Source Side Fluid Name is Water
        \\note Leave this field blank when Source Side Fluid Name is an antifreeze
        \\note Previously part of Parameter 9
        \\units W/K
        \\minimum 0.0
        \\type real
   N19, \\field Source Side Heat Transfer Resistance1
        \\note Use when Source Side Fluid Name is an antifreeze
        \\note Leave this field blank for Source Side Fluid is Water
        \\note Previously part of Parameter 9
        \\minimum 0.0
        \\units dimensionless
        \\type real
   N20; \\field Source Side Heat Transfer Resistance2
        \\note Use when Source Side Fluid Name is an antifreeze
        \\note Leave this field blank for Source Side Fluid is Water
        \\note Previously part of Parameter 10
        \\units W/K
        \\minimum 0.0
        \\type real

Coil:Heating:WaterToAirHeatPump:ParameterEstimation,
        \\memo Direct expansion (DX) heating coil for water-to-air heat pump (includes electric
        \\memo compressor), single-speed, parameter estimation model. Parameter estimation model is
        \\memo a deterministic model that requires a consistent set of parameters to describe
        \\memo the operating conditions of the heat pump components.
   A1 , \\field Name
        \\required-field
        \\type alpha
        \\reference HeatingCoilsWaterToAirHP
   A2 , \\field Compressor Type
        \\required-field
        \\type choice
        \\key Reciprocating
        \\key Rotary
        \\key Scroll
        \\note Parameters 1-4 are as named below.
        \\note Parameters 5-9 depend on the type of compressor.
        \\note Refer to the InputOutputReference on the parameters required
   A3,  \\field Refrigerant Type
        \\type object-list
        \\object-list FluidNames
        \\required-field
        \\default R22
   N1,  \\field Design Source Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N2,  \\field Nominal Heating Coil Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N3, \\field High Pressure Cutoff
        \\required-field
        \\type real
        \\minimum> 0.0
   N4, \\field Low Pressure Cutoff
        \\required-field
        \\type real
        \\minimum 0.0
        \\units Pa
   A4,  \\field Water Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Water Outlet Node Name
        \\required-field
        \\type node
   A6,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A7,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N5,  \\field Load Side Total Heat Transfer Coefficient
        \\note Previously called Parameter 1
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units W/K
   N6,  \\field Superheat Temperature at the Evaporator Outlet
        \\note Previously called Parameter 2
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units C
   N7,  \\field Compressor Power Losses
        \\note Accounts for the loss of work due to mechanical and electrical losses in the compressor.
        \\note Previously called Parameter 3
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units W
   N8,  \\field Compressor Efficiency
        \\note Previously called Parameter 4
        \\required-field
        \\minimum> 0.0
        \\type real
   N9,  \\field Compressor Piston Displacement
        \\note Use when Compressor Type is Reciprocating or Rotary
        \\note Leave this field blank for Compressor Type is Scroll.
        \\note Previously part of Parameter 5
        \\units m3/s
        \\minimum> 0.0
        \\type real
   N10, \\field Compressor Suction/Discharge Pressure Drop
        \\note Used when Compressor Type is Rotary or Reciprocating
        \\note Leave this field blank for Compressor Type is Scroll.
        \\note Previously part of Parameter 6
        \\units Pa
        \\minimum> 0.0
        \\type real
   N11, \\field Compressor Clearance Factor
        \\note Used when Compressor Type is Reciprocating.
        \\note Leave this field blank for Compressor Type is Rotary or Scroll.
        \\note Previously part of Parameter 7
        \\units dimensionless
        \\minimum> 0.0
        \\type real
   N12, \\field Refrigerant Volume Flow Rate
        \\note Use when Compressor Type is Scroll
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 5
        \\units m3/s
        \\minimum> 0.0
        \\type real
   N13, \\field Volume Ratio
        \\note Use when Compressor Type is Scroll.
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 6
        \\units dimensionless
        \\minimum> 0.0
        \\type real
   N14, \\field Leak Rate Coefficient
        \\note Use when Compressor Type is Scroll.
        \\note Leave this field blank for Compressor Type is Rotary or Reciprocating.
        \\note Previously part of Parameter 7
        \\units dimensionless
        \\minimum 0.0
        \\type real
   N15, \\field Source Side Heat Transfer Coefficient
        \\note Use when Source Side Fluid Name is Water
        \\note Leave this field blank when Source Side Fluid is an antifreeze
        \\note Previously part of Parameter 8
        \\units W/K
        \\minimum 0.0
        \\type real
   N16, \\field Source Side Heat Transfer Resistance1
        \\note Use when Source Side Fluid Name is an antifreeze
        \\note Leave this field blank for Source Side Fluid is Water
        \\note Previously part of Parameter 8
        \\minimum 0.0
        \\units dimensionless
        \\type real
   N17; \\field Source Side Heat Transfer Resistance2
        \\note Use when Source Side Fluid Name is an antifreeze
        \\note Leave this field blank for Source Side Fluid is Water
        \\note Previously part of Parameter 9
        \\units W/K
        \\minimum 0.0
        \\type real

Coil:Cooling:WaterToAirHeatPump:EquationFit,
        \\memo Direct expansion (DX) cooling coil for water-to-air heat pump (includes electric
        \\memo compressor), single-speed, equation-fit model. Optional inputs for moisture
        \\memo evaporation from wet coil when compressor cycles off with continuous fan operation.
        \\memo Equation-fit model uses normalized curves to describe the heat pump performance.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference CoolingCoilsWaterToAirHP
   A2,  \\field Water Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Water Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Rated Air Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\autosizable
   N2,  \\field Rated Water Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\autosizable
   N3,  \\field Rated Total Cooling Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
        \\autosizable
   N4,  \\field Rated Sensible Cooling Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
        \\autosizable
   N5,  \\field Rated Cooling Coefficient of Performance
        \\required-field
        \\type real
        \\minimum> 0.0
   N6,  \\field Total Cooling Capacity Coefficient 1
        \\required-field
        \\type real
   N7, \\field Total Cooling Capacity Coefficient 2
        \\required-field
        \\type real
   N8, \\field Total Cooling Capacity Coefficient 3
        \\required-field
        \\type real
   N9, \\field Total Cooling Capacity Coefficient 4
        \\required-field
        \\type real
   N10, \\field Total Cooling Capacity Coefficient 5
        \\required-field
        \\type real
   N11, \\field Sensible Cooling Capacity Coefficient 1
        \\required-field
        \\type real
   N12, \\field Sensible Cooling Capacity Coefficient 2
        \\required-field
        \\type real
   N13, \\field Sensible Cooling Capacity Coefficient 3
        \\required-field
        \\type real
   N14, \\field Sensible Cooling Capacity Coefficient 4
        \\required-field
        \\type real
   N15, \\field Sensible Cooling Capacity Coefficient 5
        \\required-field
        \\type real
   N16, \\field Sensible Cooling Capacity Coefficient 6
        \\required-field
        \\type real
   N17, \\field Cooling Power Consumption Coefficient 1
        \\required-field
        \\type real
   N18, \\field Cooling Power Consumption Coefficient 2
        \\required-field
        \\type real
   N19, \\field Cooling Power Consumption Coefficient 3
        \\required-field
        \\type real
   N20, \\field Cooling Power Consumption Coefficient 4
        \\required-field
        \\type real
   N21, \\field Cooling Power Consumption Coefficient 5
        \\required-field
        \\type real
   N22, \\field Nominal Time for Condensate Removal to Begin
        \\type real
        \\units s
        \\minimum 0.0
        \\maximum 3000.0
        \\default 0.0
        \\note The nominal time for condensate to begin leaving the coil's condensate
        \\note drain line at the coil's rated air flow and temperature conditions.
        \\note Nominal time is equal to the ratio of the energy of the coil's maximum
        \\note condensate holding capacity (J) to the coil's steady state latent capacity (W).
        \\note Suggested value is 1000; zero value means latent degradation model is disabled.
   N23; \\field Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\maximum 5.0
        \\default 0.0
        \\note Ratio of the initial moisture evaporation rate from the cooling coil (when
        \\note the compressor first turns off) and the coil's steady state latent capacity
        \\note at rated air flow rate and temperature conditions. Suggested value is 1.5; zero value
        \\note means latent degradation model is disabled.

Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit,
        \\memo Direct expansion (DX) cooling coil for water-to-air heat pump (includes electric
        \\memo compressor), variable-speed, equation-fit model. Optional inputs for moisture
        \\memo evaporation from wet coil when compressor cycles off with continuous fan operation.
        \\memo Equation-fit model uses normalized curves to describe the heat pump performance.
        \\memo Requires two to ten sets of performance data and will interpolate between speeds.
        \\memo Modeled as a single coil with variable-speed compressor.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference CoolingCoilsWaterToAirVSHP
   A2,  \\field Water-to-Refrigerant HX Water Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Water-to-Refrigerant HX Water Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Indoor Air Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Indoor Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Number of Speeds
        \\units dimensionless
        \\type integer
        \\required-field
        \\minimum 1
        \\maximum 10
        \\default 2
   N2,  \\field Nominal Speed Level
        \\units dimensionless
        \\type integer
        \\default 2
        \\required-field
        \\note must be lower than or equal to the highest speed number
   N3,  \\field Rated Total Cooling Capacity At Selected Nominal Speed Level
        \\units w
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N4,  \\field Rated Volumetric Air Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N5,  \\field Rated Volumetric Water Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N6,  \\field Nominal Time for Condensate to Begin Leaving the Coil
        \\units s
        \\type real
        \\minimum 0
        \\required-field
        \\default 0
   N7,  \\field Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity
        \\units dimensionless
        \\type real
        \\minimum 0
        \\default 0
        \\required-field
   N8,  \\field Flag for Using Hot Gas Reheat, 0 or 1
        \\note Flag for using hot gas reheat, 0 - not used, 1 - used
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
        \\default 0
   A6,  \\field Energy Part Load Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*PLR + c*PLR**2
        \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
        \\note PLR = part load ratio (cooling load/steady state capacity)
   N9,  \\field Speed 1 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
        \\required-field
   N10, \\field Speed 1 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
        \\required-field
   N11, \\field Speed 1 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   N12, \\field Speed 1 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   N13, \\field Speed 1 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   A7,  \\field Speed 1 Total Cooling Capacity Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A8,   \\field Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A9,  \\field Speed 1 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A10, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A11, \\field Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A12, \\field Speed 1 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N14, \\field Speed 1 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   A13, \\field Speed 1 Waste Heat Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N15, \\field Speed 2 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N16, \\field Speed 2 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N17, \\field Speed 2 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N18, \\field Speed 2 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N19, \\field Speed 2 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A14, \\field Speed 2 Total Cooling Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A15, \\field Speed 2 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A16, \\field Speed 2 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A17, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewet + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A18, \\field Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A19, \\field Speed 2 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N20, \\field Speed 2 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A20, \\field Speed 2 Waste Heat Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N21, \\field Speed 3 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N22, \\field Speed 3 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N23, \\field Speed 3 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N24, \\field Speed 3 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N25, \\field Speed 3 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A21, \\field Speed 3 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A22, \\field Speed 3 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A23, \\field Speed 3 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A24, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A25, \\field Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A26, \\field Speed 3 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N26, \\field Speed 3 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A27, \\field Speed 3 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N27, \\field Speed 4 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N28, \\field Speed 4 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N29, \\field Speed 4 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N30, \\field Speed 4 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N31, \\field Speed 4 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A28, \\field Speed 4 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A29, \\field Speed 4 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A30, \\field Speed 4 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A31, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A32, \\field Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A33, \\field Speed 4 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N32, \\field Speed 4 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A34, \\field Speed 4 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N33, \\field Speed 5 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N34, \\field Speed 5 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N35, \\field Speed 5 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N36, \\field Speed 5 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N37, \\field Speed 5 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A35, \\field Speed 5 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A36, \\field Speed 5 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A37, \\field Speed 5 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A38, \\field Speed 5 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A39, \\field Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A40, \\field Speed 5 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N38, \\field Speed 5 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A41, \\field Speed 5 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N39, \\field Speed 6 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N40, \\field Speed 6 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N41, \\field Speed 6 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N42, \\field Speed 6 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N43, \\field Speed 6 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A42, \\field Speed 6 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A43, \\field Speed 6 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A44, \\field Speed 6 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A45, \\field Speed 6 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A46, \\field Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A47, \\field Speed 6 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N44, \\field Speed 6 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A48, \\field Speed 6 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N45, \\field Speed 7 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N46, \\field Speed 7 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N47, \\field Speed 7 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N48, \\field Speed 7 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N49, \\field Speed 7 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A49, \\field Speed 7 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A50, \\field Speed 7 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A51, \\field Speed 7 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A52, \\field Speed 7 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A53, \\field Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A54, \\field Speed 7 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N50, \\field Speed 7 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A55, \\field Speed 7 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N51, \\field Speed 8 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N52, \\field Speed 8 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N53, \\field Speed 8 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N54, \\field Speed 8 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N55, \\field Speed 8 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A56, \\field Speed 8 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A57, \\field Speed 8 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A58, \\field Speed 8 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A59, \\field Speed 8 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A60, \\field Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A61, \\field Speed 8 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N56, \\field Speed 8 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A62, \\field Speed 8 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N57, \\field Speed 9 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N58, \\field Speed 9 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N59, \\field Speed 9 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N60, \\field Speed 9 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N61, \\field Speed 9 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A63, \\field Speed 9 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A64, \\field Speed 9 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A65, \\field Speed 9 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A66, \\field Speed 9 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A67, \\field Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A68, \\field Speed 9 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N62, \\field Speed 9 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A69, \\field Speed 9 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   N63, \\field Speed 10 Reference Unit Total Cooling Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N64, \\field Speed 10 Reference Unit Sensible Heat Ratio At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\Maximum 1.0
   N65, \\field Speed 10 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N66, \\field Speed 10 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N67, \\field Speed 10 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A70, \\field Speed 10 Total Cooling Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A71, \\field Speed 10 Total Cooling Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A72, \\field Speed 10 Total Cooling Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A73, \\field Speed 10 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)
   A74, \\field Speed 10 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A75, \\field Speed 10 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N68, \\field Speed 10 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A76; \\field Speed 10 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*wb + c*wb**2 + d*ewt + e*ewt**2 + f*wb*ewt
        \\note wb = entering wet-bulb temperature (C)
        \\note ewt = water entering temperature seen by the condenser (C)

Coil:Heating:WaterToAirHeatPump:EquationFit,
        \\memo Direct expansion (DX) heating coil for water-to-air heat pump (includes electric
        \\memo compressor), single-speed, equation-fit model. Equation-fit model uses normalized
        \\memo curves to describe the heat pump performance.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference HeatingCoilsWaterToAirHP
   A2,  \\field Water Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Water Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Rated Air Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\autosizable
   N2,  \\field Rated Water Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
        \\autosizable
   N3,  \\field Rated Heating Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
        \\autosizable
   N4,  \\field Rated Heating Coefficient of Performance
        \\required-field
        \\type real
        \\minimum> 0.0
   N5,  \\field Heating Capacity Coefficient 1
        \\required-field
        \\type real
   N6,  \\field Heating Capacity Coefficient 2
        \\required-field
        \\type real
   N7,  \\field Heating Capacity Coefficient 3
        \\required-field
        \\type real
   N8,  \\field Heating Capacity Coefficient 4
        \\required-field
        \\type real
   N9,  \\field Heating Capacity Coefficient 5
        \\required-field
        \\type real
   N10, \\field Heating Power Consumption Coefficient 1
        \\required-field
        \\type real
   N11, \\field Heating Power Consumption Coefficient 2
        \\required-field
        \\type real
   N12, \\field Heating Power Consumption Coefficient 3
        \\required-field
        \\type real
   N13, \\field Heating Power Consumption Coefficient 4
        \\required-field
        \\type real
   N14; \\field Heating Power Consumption Coefficient 5
        \\required-field
        \\type real

Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit,
        \\memo Direct expansion (DX) heating coil for water-to-air heat pump (includes electric
        \\memo compressor), variable-speed, equation-fit model. Equation-fit model uses normalized
        \\memo curves to describe the heat pump performance. Requires two to ten sets of performance
        \\memo data and will interpolate between speeds.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference HeatingCoilsWaterToAirVSHP
   A2,  \\field Water-to-Refrigerant HX Water Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Water-to-Refrigerant HX Water Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Indoor Air Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Indoor Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Number of Speeds
        \\units dimensionless
       \\type integer
        \\required-field
        \\minimum 1
        \\maximum 10
        \\default 2
   N2,  \\field Nominal Speed Level
        \\units dimensionless
        \\type integer
        \\default 2
        \\required-field
        \\note must be lower than or equal to the highest speed number
   N3,  \\field Rated Heating Capacity At Selected Nominal Speed Level
        \\units w
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N4,  \\field Rated Volumetric Air Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   N5,  \\field Rated Volumetric Water Flow Rate At Selected Nominal Speed Level
        \\units m3/s
        \\type real
        \\autosizable
        \\default autosize
        \\required-field
   A6,  \\field Energy Part Load Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*PLR + c*PLR**2
        \\note cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
        \\note PLR = part load ratio (heating load/steady state capacity)
   N6,  \\field Speed 1 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
        \\required-field
   N7,  \\field Speed 1 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   N8,  \\field Speed 1 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   N9,  \\field Speed 1 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
        \\required-field
   A7,  \\field Speed 1 Total Heating Capacity Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A8,  \\field Speed 1 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A9,  \\field Speed 1 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A10, \\field Speed 1 Energy Input Ratio Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A11, \\field Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A12, \\field Speed 1 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N10, \\field Speed 1 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
        \\required-field
   A13, \\field Speed 1 Waste Heat Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N11, \\field Speed 2 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N12, \\field Speed 2 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N13, \\field Speed 2 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N14, \\field Speed 2 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A14, \\field Speed 2 Total Heating Capacity Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A15, \\field Speed 2 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A16, \\field Speed 2 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A17, \\field Speed 2 Energy Input Ratio Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A18, \\field Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A19, \\field Speed 2 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N15, \\field Speed 2 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A20, \\field Speed 2 Waste Heat Function of Temperature Curve Name
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N16, \\field Speed 3 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N17, \\field Speed 3 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N18, \\field Speed 3 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N19, \\field Speed 3 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A21, \\field Speed 3 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A22, \\field Speed 3 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A23, \\field Speed 3 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A24, \\field Speed 3 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A25, \\field Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A26, \\field Speed 3 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N20, \\field Speed 3 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A27, \\field Speed 3 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N21, \\field Speed 4 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N22, \\field Speed 4 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N23, \\field Speed 4 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N24, \\field Speed 4 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A28, \\field Speed 4 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A29, \\field Speed 4 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A30, \\field Speed 4 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A31, \\field Speed 4 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A32, \\field Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A33, \\field Speed 4 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N25, \\field Speed 4 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A34, \\field Speed 4 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N26, \\field Speed 5 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N27, \\field Speed 5 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N28, \\field Speed 5 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N29, \\field Speed 5 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A35, \\field Speed 5 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A36, \\field Speed 5 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A37, \\field Speed 5 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A38, \\field Speed 5 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A39, \\field Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A40, \\field Speed 5 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N30, \\field Speed 5 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A41, \\field Speed 5 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N31, \\field Speed 6 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N32, \\field Speed 6 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N33, \\field Speed 6 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N34, \\field Speed 6 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A42, \\field Speed 6 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A43, \\field Speed 6 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A44, \\field Speed 6 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A45, \\field Speed 6 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A46, \\field Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A47, \\field Speed 6 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N35, \\field Speed 6 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A48, \\field Speed 6 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N36, \\field Speed 7 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N37, \\field Speed 7 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N38, \\field Speed 7 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N39, \\field Speed 7 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A49, \\field Speed 7 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A50, \\field Speed 7 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A51, \\field Speed 7 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A52, \\field Speed 7 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A53, \\field Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A54, \\field Speed 7 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N40, \\field Speed 7 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A55, \\field Speed 7 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N41, \\field Speed 8 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N42, \\field Speed 8 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N43, \\field Speed 8 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N44, \\field Speed 8 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A56, \\field Speed 8 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A57, \\field Speed 8 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A58, \\field Speed 8 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A59, \\field Speed 8 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A60, \\field Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A61, \\field Speed 8 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N45, \\field Speed 8 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A62, \\field Speed 8 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N46, \\field Speed 9 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N47, \\field Speed 9 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N48, \\field Speed 9 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N49, \\field Speed 9 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A63, \\field Speed 9 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A64, \\field Speed 9 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A65, \\field Speed 9 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A66, \\field Speed 9 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A67, \\field Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A68, \\field Speed 9 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N50, \\field Speed 9 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A69, \\field Speed 9 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   N51, \\field Speed 10 Reference Unit Total Heating Capacity At Rated Conditions
        \\units w
        \\type real
        \\minimum 0
   N52, \\field Speed 10 Reference Unit COP At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   N53, \\field Speed 10 Reference Unit Air Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   N54, \\field Speed 10 Reference Unit Water Flow Rate At Rated Conditions
        \\units m3/s
        \\type real
        \\minimum 0
   A70, \\field Speed 10 Total Heating Capacity Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A71, \\field Speed 10 Total  Heating Capacity Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A72, \\field Speed 10 Total Heating Capacity Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   A73, \\field Speed 10 Energy Input Ratio Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)
   A74, \\field Speed 10 Energy Input Ratio Function of Air Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffa + c*ffa**2
        \\note cubic curve = a + b*ffa + c*ffa**2 + d*ffa**3
        \\note ffa = Fraction of the full load Air Flow
   A75, \\field Speed 10 Energy Input Ratio Function of Water Flow Fraction Curve Name
        \\note optional
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note quadratic curve = a + b*ffw + c*ffw**2
        \\note cubic curve = a + b*ffw + c*ffw**2 + d*ffw**3
        \\note ffw = Fraction of the full load Water Flow
   N55, \\field Speed 10 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions
        \\units dimensionless
        \\type real
        \\minimum 0
   A76; \\field Speed 10 Waste Heat Function of Temperature Curve Name
        \\note optional
        \\type object-list
        \\object-list BiquadraticCurves
        \\note curve = a + b*db + c*db**2 + d*ewt + e*ewt**2 + f*db*ewt
        \\note db = entering air dry bulb temperature (C)
        \\note ewt = water entering temperature seen by the evaporator (C)

Coil:WaterHeating:AirToWaterHeatPump,
       \\memo Heat pump water heater (HPWH) heating coil, air-to-water direct-expansion (DX)
       \\memo system which includes a water heating coil, evaporator air coil, evaporator
       \\memo fan, electric compressor, and water pump. Part of a WaterHeater:HeatPump system.
       \\min-fields 21
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference HeatPumpWaterHeaterDXCoils
       \\note Unique name for this instance of a heat pump water heater DX coil.
  N1 , \\field Rated Heating Capacity
       \\required-field
       \\type real
       \\units W
       \\minimum> 0
       \\note Heating capacity at the rated inlet air temperatures, rated condenser inlet
       \\note water temperature, rated air flow rate, and rated water flow rate.
       \\note Can optionally include condenser pump heat.
  N2 , \\field Rated COP
       \\type real
       \\units W/W
       \\minimum> 0
       \\default 3.2
       \\note Heating coefficient of performance at the rated inlet air temperatures,
       \\note rated condenser inlet water temperature, rated air flow rate, and rated water flow rate.
       \\note Can optionally include condenser pump power and evaporator fan power (see fields below).
  N3 , \\field Rated Sensible Heat Ratio
       \\type real
       \\minimum 0.5
       \\maximum 1
       \\default 0.85
       \\note Gross air-side sensible heat ratio at the rated inlet air temperatures,
       \\note rated condenser inlet water temperature, rated air flow rate, and rated water flow rate.
       \\note Sensible heat ratio equals gross sensible cooling capacity divided by gross total cooling
       \\note capacity. Rated SHR (gross) should not include evaporator fan heat, only sensible cooling
       \\note and dehumidification by the coil alone.
  N4 , \\field Rated Evaporator Inlet Air Dry-Bulb Temperature
       \\type real
       \\units C
       \\minimum> 5
       \\default 19.7
       \\note Evaporator inlet air dry-bulb temperature corresponding to rated coil performance
       \\note (heating capacity, COP and SHR).
  N5 , \\field Rated Evaporator Inlet Air Wet-Bulb Temperature
       \\type real
       \\units C
       \\minimum> 5
       \\default 13.5
       \\note Evaporator inlet air wet-bulb temperature corresponding to rated coil performance
       \\note (heating capacity, COP and SHR).
  N6 , \\field Rated Condenser Inlet Water Temperature
       \\type real
       \\units C
       \\minimum> 25
       \\default 57.5
       \\note Condenser inlet water temperature corresponding to rated coil performance
       \\note (heating capacity, COP and SHR).
  N7 , \\field Rated Evaporator Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autocalculatable
       \\note Evaporator air flow rate corresponding to rated coil performance
       \\note (heating capacity, COP and SHR).
       \\note Default is 5.035E-5 m3/s/W (31.25 cfm/MBH) of rated heating capacity when autocalculated.
  N8 , \\field Rated Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autocalculatable
       \\note Condenser water flow rate corresponding to rated coil performance
       \\note (heating capacity, COP and SHR).
       \\note Default is 4.487E-8 m3/s/W (0.208 gpm/MBH) of rated heating capacity when autocalculated.
       \\note A warning message will be issued if the ratio of Rated Condenser Water Flow Rate
       \\note to Heating Capacity is less than 1.79405E-8 m3/s/W (0.083 gpm/MBH)
       \\note or greater than 8.97024E-8 m3/s/W (0.417 gpm/MBH), but the simulation will continue.
  A2 , \\field Evaporator Fan Power Included in Rated COP
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\note Select Yes if the evaporator fan power is included in the rated COP. This choice field
       \\note impacts the calculation of compressor electric power.
  A3 , \\field Condenser Pump Power Included in Rated COP
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Select Yes if the condenser pump power is included in the rated COP. This choice field
       \\note impacts the calculation of compressor electric power.
  A4 , \\field Condenser Pump Heat Included in Rated Heating Capacity and Rated COP
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Select Yes if the condenser pump heat is included in the rated heating capacity and
       \\note rated COP. This choice field impacts the calculation of water heating capacity.
  N9 , \\field Condenser Water Pump Power
       \\type real
       \\units W
       \\minimum 0
       \\default 0
       \\note A warning message will be issued if the ratio of Condenser Water Pump Power to Rated
       \\note Heating Capacity exceeds 0.1422 W/W (41.67 Watts/MBH), but the simulation will continue.
  N10, \\field Fraction of Condenser Pump Heat to Water
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0.2
       \\note Fraction of pump heat transferred to the condenser water. The pump is assumed
       \\note to be located downstream of the condenser.
  A5 , \\field Evaporator Air Inlet Node Name
       \\required-field
       \\type node
       \\note The node from which the DX coil draws its inlet air.
  A6 , \\field Evaporator Air Outlet Node Name
       \\required-field
       \\type node
       \\note The node to which the DX coil sends its outlet air.
  A7 , \\field Condenser Water Inlet Node Name
       \\required-field
       \\type node
       \\note The node from which the DX coil condenser draws its inlet water.
       \\note This name should match the source side outlet node name in the associated
       \\note water heater tank object.
  A8 , \\field Condenser Water Outlet Node Name
       \\required-field
       \\type node
       \\note The node to which the DX coil condenser sends its outlet water.
       \\note This name should match the source side inlet node name in the associated
       \\note water heater tank object.
  N11, \\field Crankcase Heater Capacity
       \\type real
       \\minimum 0
       \\default 0
       \\units W
       \\note The compressor crankcase heater only operates when the dry-bulb temperature of air
       \\note surrounding the compressor is below the Maximum Ambient Temperature for Crankcase
       \\note Heater Operation and the DX coil is off.  The ambient temperature surrounding the
       \\note compressor is set by the WaterHeater:HeatPump parent object (field Compressor Location).
  N12, \\field Maximum Ambient Temperature for Crankcase Heater Operation
       \\type real
       \\minimum 0
       \\default 10
       \\units C
       \\note The compressor crankcase heater only operates when the dry-bulb temperature of air
       \\note surrounding the compressor is below the Maximum Outdoor Temperature for Crankcase
       \\note Heater Operation and the unit is off. The ambient temperature surrounding the
       \\note compressor is set by the WaterHeater:HeatPump parent object (field Compressor Location).
  A9 , \\field Evaporator Air Temperature Type for Curve Objects
       \\type choice
       \\key DryBulbTemperature
       \\key WetBulbTemperature
       \\default WetBulbTemperature
       \\note Determines temperature type for heating capacity curves and
       \\note heating COP curves. This input determines whether
       \\note the inlet air dry-bulb or wet-bulb temperature is used to evaluate these curves.
  A10, \\field Heating Capacity Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCubicCurves
       \\note Heating capacity modifier curve (function of temperature) should be biquadratic or cubic.
       \\note Biquadratic curve = a + b(ta) + c(ta)^2 + d(tw) + e(tw)^2 + f(ta)(tw).
       \\note Cubic curve = a + b(ta) + c(ta)^2 + d(ta)^3.
       \\note ta = evaporator inlet air [dry-bulb or wet-bulb] temperature (C).
       \\note tw = condenser inlet water temperature (C).
       \\note The field Evaporator Air Temperature Type for Curve Objects determines if dry-bulb or wet-bulb
       \\note is used as the evaporator inlet air temperature (ta).
  A11, \\field Heating Capacity Function of Air Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Heating capacity modifier curve (function of air flow fraction) should be quadratic or cubic.
       \\note Quadratic curve = a + b(ff) + c(ff)^2.
       \\note Cubic curve = a + b(ff) + c(ff)^2 + d(ff)^3.
       \\note ff = fraction of the rated evaporator air flow rate.
       \\note Use curve coefficients of 1,0,0 or leave this field blank when neglecting performance impacts
       \\note due to variations in air flow rate fraction.
  A12, \\field Heating Capacity Function of Water Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Heating capacity modifier curve (function of water flow fraction) should be quadratic or cubic.
       \\note Quadratic curve = a + b(ff) + c(ff)^2.
       \\note Cubic curve = a + b(ff) + c(ff)^2 + d(ff)^3.
       \\note ff = fraction of the rated condenser water flow rate.
       \\note Use curve coefficients of 1,0,0 or leave this field blank when neglecting performance impacts
       \\note due to variations in water flow rate fraction.
  A13, \\field Heating COP Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCubicCurves
       \\note Heating COP modifier curve (function of temperature) should be biquadratic or cubic.
       \\note Biquadratic curve = a + b(ta) + c(ta)^2 + d(tw) + e(tw)^2 + f(ta)(tw).
       \\note Cubic curve = a + b(ta) + c(ta)^2 + d(ta)^3.
       \\note ta = evaporator inlet air [dry-bulb or wet-bulb] temperature (C).
       \\note tw = condenser inlet water temperature (C).
       \\note The field Evaporator Air Temperature Type for Curve Objects determines if dry-bulb or wet-bulb
       \\note is used as the evaporator inlet air temperature (ta).
  A14, \\field Heating COP Function of Air Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Heating COP modifier curve (function of air flow fraction) should be quadratic or cubic.
       \\note Quadratic curve = a + b(ff) + c(ff)^2.
       \\note Cubic curve = a + b(ff) + c(ff)^2 + d(ff)^3.
       \\note ff = fraction of the rated evaporator air flow rate.
       \\note Use curve coefficients of 1,0,0 or leave this field blank when neglecting performance impacts
       \\note due to variations in air flow rate fraction.
  A15, \\field Heating COP Function of Water Flow Fraction Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Heating COP modifier curve (function of water flow fraction) should be quadratic or cubic.
       \\note Quadratic curve = a + b(ff) + c(ff)^2.
       \\note Cubic curve = a + b(ff) + c(ff)^2 + d(ff)^3.
       \\note ff = fraction of the rated condenser water flow rate.
       \\note Use curve coefficients of 1,0,0 or leave this field blank when neglecting performance impacts
       \\note due to variations in water flow rate fraction.
  A16; \\field Part Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Part Load Fraction Correlation (function of part load ratio) should be quadratic or cubic.
       \\note Quadratic curve = a + b(PLR) + c(PLR)^2.
       \\note Cubic curve = a + b(PLR) + c(PLR)^2 + d(PLR)^3.
       \\note PLR = part load ratio (heating delivered/steady state heating capacity).
       \\note Use curve coefficients of 1,0,0 or leave this field blank when neglecting performance impacts
       \\note due to variations in part load ratio.

Coil:WaterHeating:Desuperheater,
       \\memo Desuperheater air heating coil. The heating energy provided by this coil is reclaimed
       \\memo from the superheated refrigerant gas leaving a compressor and does not impact the
       \\memo performance of the compressor. This coil must be used with a water heater tank, see
       \\memo Water Heater:Mixed.
       \\min-fields 18
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WaterHeatingCoilsDesuperheater
       \\note Unique name for this instance of a desuperheater water heating coil.
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values of 0 denote the desuperheater heating coil is off and the parasitic electric
       \\note energy is also off.
  A3 , \\field Setpoint Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Defines the cut-out temperature where the desuperheater water heating coil turns off.
       \\note The desuperheater heating coil setpoint temperature should always be greater
       \\note than the water tank's heater (element or burner) setpoint temperature. Temperature
       \\note schedule values should be in degrees C.
  N1 , \\field Dead Band Temperature Difference
       \\type real
       \\units deltaC
       \\minimum> 0
       \\maximum 20
       \\default 5
       \\note Setpoint temperature minus the dead band temperature difference defines
       \\note the cut-in temperature where the desuperheater water heating coil turns on.
       \\note The water tank's heater (element or burner) setpoint temperature
       \\note should always be less than the desuperheater heating coil cut-in temperature.
  N2 , \\field Rated Heat Reclaim Recovery Efficiency
       \\type real
       \\minimum> 0.0
       \\note Enter the fraction of waste heat reclaimed by the desuperheater water heating coil.
  N3 , \\field Rated Inlet Water Temperature
       \\required-field
       \\type real
       \\units C
       \\note The inlet water temperature corresponding to the rated heat reclaim recovery efficiency.
  N4 , \\field Rated Outdoor Air Temperature
       \\required-field
       \\type real
       \\units C
       \\note The outdoor air dry-bulb temperature corresponding to the
       \\note rated heat reclaim recovery efficiency.
  N5 , \\field Maximum Inlet Water Temperature for Heat Reclaim
       \\required-field
       \\type real
       \\units C
       \\note The desuperheater water heating coil is off when the inlet water temperature is above
       \\note the maximum inlet water temperature for heat reclaim.
  A4 , \\field Heat Reclaim Efficiency Function of Temperature Curve Name
       \\type object-list
       \\object-list BiquadraticCurves
       \\note A biquadratic curve defining the performance of the desuperheater heating coil.
       \\note Performance can be specified as a function of inlet water temperature, outdoor air
       \\note dry-bulb temperature, or both.
       \\note Curve = a + b*Tw + c*Tw**2 + d*odb + e*odb**2 + f*Tw*odb.
       \\note Tw = desuperheater heating coil entering water temperature (C).
       \\note Odb = outdoor dry-bulb temperature at DX system condenser (C).
  A5 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note The node from which the desuperheater heating coil draws its inlet water.
       \\note This name should match the source side outlet node name in the associated
       \\note water heater tank object.
  A6 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note The node to which the desuperheater heating coil sends its outlet water.
       \\note This name should match the source side inlet node name in the associated
       \\note water heater tank object.
  A7 , \\field Tank Object Type
       \\required-field
       \\type choice
       \\key WaterHeater:Mixed
       \\default WaterHeater:Mixed
       \\note Specify the type of water heater tank used by this desuperheater water heating coil.
  A8 , \\field Tank Name
       \\required-field
       \\type alpha
       \\note The name of the water heater tank used by this desuperheater water heating coil.
       \\note Needs to match the name used in the corresponding water heater object.
       \\object-list WaterHeaterMixedNames
  A9 , \\field Heating Source Object Type
       \\required-field
       \\type choice
       \\key Coil:Cooling:DX:SingleSpeed
       \\key Coil:Cooling:DX:TwoSpeed
       \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
       \\key Refrigeration:CompressorRack
       \\key Refrigeration:Condenser:AirCooled
       \\key Refrigeration:Condenser:EvaporativeCooled
       \\key Refrigeration:Condenser:WaterCooled
       \\note The type of DX system that is providing waste heat for reclaim.
  A10, \\field Heating Source Name
       \\required-field
       \\type object-list
       \\object-list DesuperHeatingCoilSources
       \\note The name of the DX system used for heat reclaim.
  N6 , \\field Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0
       \\note The operating water flow rate.
  N7 , \\field Water Pump Power
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\note The water circulation pump electric power.
  N8 , \\field Fraction of Pump Heat to Water
       \\type real
       \\minimum 0
       \\maximum 1
       \\default 0.2
       \\note The fraction of pump heat transferred to the water. The pump is assumed to be downstream of
       \\note the desuperheater water heating coil.
  N9 , \\field On-Cycle Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0
       \\default 0
       \\note Parasitic electric power consumed when the desuperheater water heating coil operates.
       \\note Parasitic electric load does not contribute to water heating or the zone air heat balance.
       \\ip-units W
  N10; \\field Off-Cycle Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0
       \\default 0
       \\note Parasitic electric load consumed when the desuperheater water heating coil is off.
       \\note Parasitic electric load does not contribute to water heating or the zone air heat balance.
       \\note Off-cycle parasitic power is 0 when the availability schedule is 0.
       \\ip-units W

CoilSystem:Cooling:DX,
        \\memo Virtual container component that consists of a DX cooling coil and its associated
        \\memo controls. This control object supports several different types of DX cooling coils
        \\memo and may be placed directly in an air loop branch or outdoor air equipment list.
        \\min-fields 7
   A1 , \\field Name
        \\required-field
        \\type alpha
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field DX Cooling Coil System Inlet Node Name
        \\required-field
        \\type node
   A4 , \\field DX Cooling Coil System Outlet Node Name
        \\required-field
        \\type node
   A5 , \\field DX Cooling Coil System Sensor Node Name
        \\required-field
        \\type node
   A6,  \\field Cooling Coil Object Type
        \\type choice
        \\required-field
        \\key Coil:Cooling:DX:SingleSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\key Coil:Cooling:DX:TwoSpeed
        \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
        \\key Coil:Cooling:DX:VariableSpeed
   A7,  \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDX
        \\object-list CoolingCoilsDXVariableSpeed
   A8,  \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key Multimode
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only
        \\note Multimode = activate enhanced dehumidification mode
        \\note as needed and meet sensible load. If no sensible load
        \\note exists, and Run on Latent Load = Yes, and a latent
        \\note load exists, the unit will operate to meet the latent load.
        \\note Valid only with Coil:Cooling:DX:TwoStageWithHumidityControlMode
        \\note or CoilSystem:Cooling:DX:HeatExchangerAssisted.
        \\note CoolReheat = cool beyond the dry bulb setpoint.
        \\note as required to meet the humidity setpoint.
        \\note Valid for all coil types.
        \\note For all dehumidification controls, the max
        \\note humidity setpoint on the Sensor Node is used.
        \\note SetpointManager:SingleZone:Humidity:Maximum,
        \\note SetpointManager:MultiZone:Humidity:Maximum, or
        \\note SetpointManager:MultiZone:MaximumHumidity:Average, and
        \\note SetpointManager:OutdoorAirPretreat (optional) objects.
   A9,  \\field Run on Sensible Load
        \\type choice
        \\key Yes
        \\key No
        \\default Yes
        \\note If Yes, unit will run if there is a sensible load.
        \\note If No, unit will not run if there is only a sensible load.
        \\note Dehumidification controls will be active if specified.
  A10,  \\field Run on Latent Load
        \\type choice
        \\key Yes
        \\key No
        \\default No
        \\note If Yes, unit will run if there is a latent load.
        \\note even if there is no sensible load.
        \\note If No, unit will not run only if there is a latent load.
        \\note Dehumidification controls will be active if specified.
  A11,  \\field Use DOAS DX Cooling Coil
        \\type choice
        \\key Yes
        \\key No
        \\default No
        \\note If Yes, the DX cooling coil runs as 100% DOAS DX coil.
        \\note If No, the DX cooling coil runs as a regular DX coil.
        \\note If left blank the default is regular dx coil.
  N1 ;  \\field DOAS DX Cooling Coil Leaving Minimum Air Temperature
        \\type real
        \\units C
        \\minimum 0.0
        \\maximum 7.2
        \\default 2.0
        \\note DX cooling coil leaving minimum air temperature defines, minimum DOAS DX cooling coil
        \\note leaving air temperature that should be maintained to avoid frost formation. This input
        \\note field is optional and only used along with the input field above.

CoilSystem:Heating:DX,
        \\memo Virtual container component that consists of a DX heating coil (heat pump) and its
        \\memo associated controls. This control object supports two different types of DX heating
        \\memo coils and may be placed directly in an air loop branch or outdoor air equipment list.
        \\min-fields 4
   A1 , \\field Name
        \\required-field
        \\type alpha
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Heating Coil Object Type
        \\type choice
        \\required-field
        \\key Coil:Heating:DX:SingleSpeed
        \\key Coil:Heating:DX:VariableSpeed
   A4;  \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsDXSingleSpeed
        \\object-list HeatingCoilsDXVariableSpeed

CoilSystem:Cooling:Water:HeatExchangerAssisted,
       \\memo Virtual component consisting of a chilled-water cooling coil and an air-to-air heat
       \\memo exchanger. The air-to-air heat exchanger precools the air entering the cooling coil
       \\memo and reuses this energy to reheat the supply air leaving the cooling coil. This heat
       \\memo exchange process improves the latent removal performance of the cooling coil (lower
       \\memo sensible heat ratio).
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\reference CoolingCoilsWater
  A2 , \\field Heat Exchanger Object Type
       \\required-field
       \\type choice
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
  A3 , \\field Heat Exchanger Name
       \\required-field
  A4 , \\field Cooling Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Cooling:Water
       \\key Coil:Cooling:Water:DetailedGeometry
  A5 ; \\field Cooling Coil Name
       \\required-field
       \\type object-list
       \\object-list CoolingCoilsWaterNoHX

CoilSystem:Cooling:DX:HeatExchangerAssisted,
       \\memo Virtual component consisting of a direct expansion (DX) cooling coil and an
       \\memo air-to-air heat exchanger. The air-to-air heat exchanger precools the air entering the
       \\memo cooling coil and reuses this energy to reheat the supply air leaving the cooling
       \\memo coil. This heat exchange process improves the latent removal performance of the
       \\memo cooling coil (lower sensible heat ratio).
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\reference CoolingCoilsDX
       \\reference CoolingCoilsDXSingleSpeed
       \\reference CoolingCoilsDXMultiModeOrSingleSpeed
  A2 , \\field Heat Exchanger Object Type
       \\required-field
       \\type choice
       \\key HeatExchanger:AirToAir:FlatPlate
       \\key HeatExchanger:AirToAir:SensibleAndLatent
       \\key HeatExchanger:Desiccant:BalancedFlow
  A3 , \\field Heat Exchanger Name
       \\required-field
       \\type object-list
       \\object-list HXAirToAirNames
  A4 , \\field Cooling Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:Cooling:DX:SingleSpeed
  A5 ; \\field Cooling Coil Name
       \\required-field
       \\type object-list
       \\object-list CoolingCoilsDXBypassFactorEmpirical


\\Group Evaporative Coolers

EvaporativeCooler:Direct:CelDekPad,
       \\memo Direct evaporative cooler with rigid media evaporative pad and recirculating water
       \\memo pump. This model has no controls other than its availability schedule.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Direct Pad Area
       \\units m2
       \\minimum 0.0
  N2 , \\field Direct Pad Depth
       \\units m
       \\minimum 0.0
  N3 , \\field Recirculating Water Pump Power Consumption
       \\units W
       \\ip-units W
  A3 , \\field Air Inlet Node Name
       \\type node
  A4 , \\field Air Outlet Node Name
       \\type node
  A5 , \\Field Control Type
       \\note This field is not currently used and can be left blank
  A6 ; \\Field Water Supply Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames

EvaporativeCooler:Indirect:CelDekPad,
       \\memo Indirect evaporative cooler with rigid media evaporative pad, recirculating water
       \\memo pump, and secondary air fan. This model has no controls other than its availability
       \\memo schedule.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Direct Pad Area
       \\units m2
       \\minimum 0.0
  N2 , \\field Direct Pad Depth
       \\units m
       \\minimum 0.0
  N3 , \\field Recirculating Water Pump Power Consumption
       \\units W
       \\ip-units W
  N4 , \\field Secondary Fan Flow Rate
       \\units m3/s
       \\minimum 0.0
  N5 , \\field Secondary Fan Efficiency
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  N6 , \\field Secondary Fan Delta Pressure
       \\units Pa
       \\minimum 0.0
       \\ip-units inH2O
  N7 , \\field Indirect Heat Exchanger Effectiveness
       \\type real
       \\minimum 0.0
  A3 , \\field Primary Air Inlet Node Name
       \\type node
  A4 , \\field Primary Air Outlet Node Name
       \\type node
  A5 , \\Field Control Type
       \\note This field is not currently used and can be left blank
  A6 , \\Field Water Supply Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A7 ; \\field Secondary Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node

EvaporativeCooler:Indirect:WetCoil,
       \\memo Indirect evaporative cooler with wetted coil, recirculating water pump, and secondary
       \\memo air fan. This model has no controls other than its availability schedule.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Coil Maximum Efficiency
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N2 , \\field Coil Flow Ratio
       \\type real
  N3 , \\field Recirculating Water Pump Power Consumption
       \\units W
       \\ip-units W
  N4 , \\field Secondary Fan Flow Rate
       \\units m3/s
       \\minimum 0.0
  N5 , \\field Secondary Fan Efficiency
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  N6 , \\field Secondary Fan Delta Pressure
       \\units Pa
       \\minimum 0.0
       \\ip-units inH2O
  A3 , \\field Primary Air Inlet Node Name
       \\type node
  A4 , \\field Primary Air Outlet Node Name
       \\type node
  A5 , \\Field Control Type
       \\note This field is not currently used and can be left blank
  A6 , \\Field Water Supply Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A7 ; \\field Secondary Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node

EvaporativeCooler:Indirect:ResearchSpecial,
       \\memo Indirect evaporative cooler with user-specified effectiveness (can represent rigid pad
       \\memo or wetted coil), recirculating water pump, and secondary air fan. This model is
       \\memo controlled to meet the primary air outlet temperature setpoint.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Cooler Maximum Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 2.0
  N2 , \\field Cooler Flow Ratio
       \\type real
  N3 , \\field Recirculating Water Pump Power Consumption
       \\units W
       \\ip-units W
  N4 , \\field Secondary Fan Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\autosizable
  N5 , \\field Secondary Fan Efficiency
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  N6 , \\field Secondary Fan Delta Pressure
       \\units Pa
       \\minimum 0.0
       \\ip-units inH2O
  A3 , \\field Primary Air Inlet Node Name
       \\type node
  A4 , \\field Primary Air Outlet Node Name
       \\type node
  A5 , \\Field Control Type
  N7 , \\field Dewpoint Effectiveness Factor
  A6 , \\Field Secondary Air Inlet Node Name
       \\type node
  A7 , \\Field Sensor Node Name
       \\type node
  A8 , \\Field Relief Air Inlet Node Name
       \\type node
  A9 , \\Field Water Supply Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  N8 , \\field Drift Loss Fraction
       \\type real
       \\minimum 0.0
       \\note Rate of drift loss as a fraction of evaporated water flow rate
  N9 ; \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\note Characterizes the rate of blowdown in the evaporative cooler.
       \\note Blowdown is water intentionally drained from the cooler in order to offset the build up
       \\note of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note A typical value is 3.  If left blank then there is no blowdown.

EvaporativeCooler:Direct:ResearchSpecial,
       \\memo Direct evaporative cooler with user-specified effectiveness (can represent rigid pad
       \\memo or similar media), and recirculating water pump, and secondary air fan. This model is
       \\memo controlled to meet the primary air outlet temperature setpoint.
  A1 , \\field Name
       \\type alpha
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Cooler Effectiveness
       \\note effectiveness with respect to wetbulb depression
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N2 , \\field Recirculating Water Pump Power Consumption
       \\units W
       \\ip-units W
       \\minimum 0.0
  A3 , \\field Air Inlet Node Name
       \\type node
  A4 , \\field Air Outlet Node Name
       \\type node
  A5 , \\Field Sensor Node Name
       \\type node
  A6 , \\Field Water Supply Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  N3 , \\field Drift Loss Fraction
       \\type real
       \\minimum 0.0
       \\note Rate of drift loss as a fraction of evaporated water flow rate
  N4 ; \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\note Characterizes the rate of blowdown in the evaporative cooler.
       \\note Blowdown is water intentionally drained from the cooler in order to offset the build up
       \\note of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note A typical value is 3. If left blank then there is no blowdown.

\\Group Humidifiers and Dehumidifiers

Humidifier:Steam:Electric,
      \\memo Electrically heated steam humidifier with fan.
  A1, \\field Name
      \\required-field
  A2, \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Rated Capacity
      \\note Capacity is m3/s of water at 5.05 C
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\ip-units gal/min
  N2, \\field Rated Power
      \\type real
      \\units W
      \\minimum 0.0
      \\ip-units W
      \\autosizable
      \\note if autosized the rated power is calculated from the rated capacity
      \\note and enthalpy rise of water from 20.0C to 100.0C steam and assumes
      \\note electric to thermal energy conversion efficiency of 100.0%
  N3, \\field Rated Fan Power
      \\type real
      \\units W
      \\minimum 0.0
      \\ip-units W
  N4, \\field Standby Power
      \\type real
      \\units W
      \\minimum 0.0
      \\ip-units W
  A3, \\field Air Inlet Node Name
       \\type node
  A4, \\field Air Outlet Node Name
       \\type node
  A5; \\field Water Storage Tank Name
      \\type object-list
      \\object-list WaterStorageTankNames

Dehumidifier:Desiccant:NoFans,
       \\memo This object models a solid desiccant dehumidifier. The process
       \\memo air stream is the air which is dehumidified. The regeneration air
       \\memo stream is the air which is heated to regenerate the desiccant.
       \\memo This object determines the process air outlet conditions, the
       \\memo load on the regeneration heating coil, the electric power consumption
       \\memo for the wheel rotor motor, and the regeneration air fan mass flow rate.
       \\memo All other heat exchangers are modeled as separate objects connected
       \\memo to the inlet and outlet nodes of the dehumidifier. The solid
       \\memo desiccant dehumidifier is typically used in an AirLoopHVAC:OutdoorAirSystem,
       \\memo but can also be specified in any AirLoopHVAC.
   A1, \\field Name
       \\required-field
   A2, \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
   A3, \\field Process Air Inlet Node Name
       \\note This is the node entering the process side of the desiccant wheel.
       \\type node
   A4, \\field Process Air Outlet Node Name
       \\note This is the node leaving the process side of the desiccant wheel.
       \\type node
   A5, \\field Regeneration Air Inlet Node Name
       \\note This is the node entering the regeneration side of the desiccant wheel
       \\note after the regeneration coil.
       \\type node
   A6, \\field Regeneration Fan Inlet Node Name
       \\note Node for air entering the regeneration fan, mass flow is set
       \\note by the desiccant dehumidifier module.
       \\type node
   A7, \\field Control Type
       \\note Type of setpoint control:
       \\note LeavingMaximumHumidityRatioSetpoint means that the unit is controlled
       \\note to deliver air at the Leaving Max Humidity Ratio Setpoint (see below),
       \\note SystemNodeMaximumHumidityRatioSetpoint means that the leaving humidity
       \\note ratio setpoint is the System Node Humidity Ratio Max property
       \\note of the Process Air Outlet Node.  A Setpoint
       \\note object must be used to control this setpoint.
       \\note Both control types use bypass dampers to prevent overdrying.
       \\type choice
       \\key LeavingMaximumHumidityRatioSetpoint
       \\key SystemNodeMaximumHumidityRatioSetpoint
   N1, \\field Leaving Maximum Humidity Ratio Setpoint
       \\note Fixed setpoint for maximum process air leaving humidity ratio
       \\note Applicable only when Control Type = LeavingMaximumHumidityRatioSetpoint.
       \\type real
       \\units kgWater/kgDryAir
   N2, \\field Nominal Process Air Flow Rate
       \\note Process air flow rate at nominal conditions
       \\type real
       \\units m3/s
       \\minimum> 0.0
   N3, \\field Nominal Process Air Velocity
       \\note Process air velocity at nominal flow
       \\note When using Performance Model Type of Default, must be 2.032 to 4.064 m/s (400 to 800 fpm)
       \\type real
       \\units m/s
       \\minimum> 0.0
       \\maximum 6
   N4, \\field Rotor Power
       \\note Power input to wheel rotor motor
       \\type real
       \\units W
       \\minimum 0.0
       \\ip-units W
   A8, \\field Regeneration Coil Object Type
       \\note heating coil type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Water
       \\key Coil:Heating:Steam
       \\note works with gas, electric, hot water and steam heating coils
   A9, \\field Regeneration Coil Name
       \\note Name of heating coil object for regeneration air
       \\type object-list
       \\object-list HeatingCoilName
  A10, \\field Regeneration Fan Object Type
       \\note Type of fan object for regeneration air.  When using the Default
       \\note Performance Model Type (see below), only Fan:VariableVolume is valid.
       \\type choice
       \\key Fan:VariableVolume
       \\key Fan:ConstantVolume
  A11, \\field Regeneration Fan Name
       \\note Name of fan object for regeneration air
       \\type object-list
       \\object-list FansCVandVAV
  A12, \\field Performance Model Type
       \\note Specifies whether the default performance model or user-specified
       \\note curves should be used to model the performance.  The default model
       \\note is a generic solid desiccant wheel using performance curves of the form:
       \\note curve = C1 + C2*edb + C3*edb**2 + C4*ew + C5*ew**2 + C6*vel + C7*vel**2
       \\note + C8*edb*ew + C9*edb**2*ew**2 + C10*edb*vel + C11*edb**2*vel**2
       \\note + C12*ew*vel + C13*ew**2*vel**2 + C14*ALOG(edb) + C15*ALOG(ew) + C16*ALOG(vel)
       \\note   edb = process entering dry-bulb temperature [C]
       \\note   ew  = process entering humidity ratio [kgWater/kgDryAir]
       \\note   vel = process air velocity [m/s]
       \\note If UserCurves are specified, then performance is calculated as follows:
       \\note   Leaving Dry-Bulb = (Leaving Dry-Bulb fTW Curve) * (Leaving Dry-Bulb fV Curve)
       \\note   Leaving Humidity Ratio = (Leaving Humidity Ratio fTW Curve) * (Leaving Humidity Ratio fV Curve)
       \\note   Regen Energy = (Regen Energy fTW Curve) * (Regen Energy fV Curve)
       \\note   Regen Velocity = (Regen Velocity fTW Curve) * (Regen Velocity fV Curve)
       \\type choice
       \\key Default
       \\key UserCurves
  A13, \\field Leaving Dry-Bulb Function of Entering Dry-Bulb and Humidity Ratio Curve Name
       \\note Leaving dry-bulb of process air as a function of entering dry-bulb
       \\note and entering humidity ratio, biquadratic curve
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = C1 + C2*edb + C3*edb**2 + C4*ew + C5*ew**2 + C6*edb*ew
       \\note edb = process entering dry-bulb temperature [C]
       \\note ew  = process entering humidity ratio [kgWater/kgDryAir]
  A14, \\field Leaving Dry-Bulb Function of Air Velocity Curve Name
       \\note Leaving dry-bulb of process air as a function of air velocity,
       \\note quadratic curve.
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = C1 + C2*v + C3*v**2
       \\note v = process air velocity [m/s]
  A15, \\field Leaving Humidity Ratio Function of Entering Dry-Bulb and Humidity Ratio Curve Name
       \\note Leaving humidity ratio of process air as a function of entering dry-bulb
       \\note and entering humidity ratio, biquadratic curve
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = C1 + C2*edb + C3*edb**2 + C4*ew + C5*ew**2 + C6*edb*ew
       \\note edb = process entering dry-bulb temperature [C]
       \\note ew  = process entering humidity ratio [kgWater/kgDryAir]
  A16, \\field Leaving Humidity Ratio Function of Air Velocity Curve Name
       \\note Leaving humidity ratio of process air as a function of
       \\note process air velocity, quadratic curve.
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = C1 + C2*v + C3*v**2
       \\note v = process air velocity [m/s]
  A17, \\field Regeneration Energy Function of Entering Dry-Bulb and Humidity Ratio Curve Name
       \\note Regeneration energy [J/kg of water removed] as a function of
       \\note entering dry-bulb and entering humidity ratio, biquadratic curve
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = C1 + C2*edb + C3*edb**2 + C4*ew + C5*ew**2 + C6*edb*ew
       \\note edb = process entering dry-bulb temperature [C]
       \\note ew  = process entering humidity ratio [kgWater/kgDryAir]
  A18, \\field Regeneration Energy Function of Air Velocity Curve Name
       \\note Regeneration energy [J/kg of water removed] as a function of
       \\note process air velocity, quadratic curve.
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = C1 + C2*v + C3*v**2
       \\note v = process air velocity [m/s]
  A19, \\field Regeneration Velocity Function of Entering Dry-Bulb and Humidity Ratio Curve Name
       \\note Regeneration velocity [m/s] as a function of
       \\note entering dry-bulb and entering humidity ratio, biquadratic curve
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = C1 + C2*edb + C3*edb**2 + C4*ew + C5*ew**2 + C6*edb*ew
       \\note edb = process entering dry-bulb temperature [C]
       \\note ew  = process entering humidity ratio [kgWater/kgDryAir]
  A20, \\field Regeneration Velocity Function of Air Velocity Curve Name
       \\note Regeneration velocity [m/s] as a function of
       \\note process air velocity, quadratic curve.
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = C1 + C2*v + C3*v**2
       \\note v = process air velocity [m/s]
   N5; \\field Nominal Regeneration Temperature
       \\note Nominal regen temperature upon which the regen energy modifier
       \\note  curve is based.  Not used if Default if chosen for the field Performance Mode Type.
       \\note 121 C is a commonly used value.
       \\type real
       \\units C
       \\minimum 40
       \\maximum 250

Dehumidifier:Desiccant:System,
      \\memo This compound object models a desiccant heat exchanger, an optional
      \\memo heater, and associated fans. The process air stream is the air which
      \\memo is dehumidified. The regeneration air stream is the air which is
      \\memo heated to regenerate the desiccant. The desiccant heat exchanger
      \\memo transfers both sensible and latent energy between the process and
      \\memo regeneration air streams. The desiccant dehumidifier is typically used
      \\memo in an AirLoopHVAC:OutdoorAirSystem, but can also be specified in any AirLoopHVAC.
      \\min-fields 8
  A1,  \\field Name
       \\required-field
       \\type alpha
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Desiccant Heat Exchanger Object Type
       \\required-field
       \\type choice
       \\key HeatExchanger:Desiccant:BalancedFlow
  A4,  \\field Desiccant Heat Exchanger Name
       \\required-field
       \\type object-list
       \\object-list HXDesiccantBalanced
  A5,  \\field Sensor Node Name
       \\required-field
       \\type node
  A6,  \\field Regeneration Air Fan Object Type
       \\required-field
       \\type choice
       \\key Fan:OnOff
       \\key Fan:ConstantVolume
  A7,  \\field Regeneration Air Fan Name
       \\required-field
       \\type object-list
       \\object-list FansOnOffandVAV
  A8,  \\field Regeneration Air Fan Placement
       \\type choice
       \\key BlowThrough
       \\key DrawThrough
       \\default DrawThrough
  A9,  \\field Regeneration Air Heater Object Type
       \\type choice
       \\key Coil:Heating:Electric
       \\key Coil:Heating:Gas
       \\key Coil:Heating:Water
       \\key Coil:Heating:Steam
       \\note works with gas, electric, hot water and steam heating coils
  A10, \\field Regeneration Air Heater Name
       \\type object-list
       \\object-list HeatingCoilName
  N1,  \\field Regeneration Inlet Air Setpoint Temperature
       \\type real
       \\units C
  A11, \\field Companion Cooling Coil Object Type
       \\type choice
       \\key Coil:Cooling:DX:SingleSpeed
       \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
  A12, \\field Companion Cooling Coil Name
       \\type object-list
       \\object-list CoolingCoilsDXMultiModeOrSingleSpeed
  A13, \\field Companion Cooling Coil Upstream of Dehumidifier Process Inlet
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Select Yes if the companion cooling coil is located directly upstream
       \\note of the desiccant heat exchanger's process air inlet node.
  A14, \\field Companion Coil Regeneration Air Heating
       \\type choice
       \\key Yes
       \\key No
       \\default No
  N2,  \\field Exhaust Fan Maximum Flow Rate
       \\type real
       \\units m3/s
  N3,  \\field Exhaust Fan Maximum Power
       \\type real
       \\units W
  A15; \\field Exhaust Fan Power Curve Name
       \\type alpha
       \\note Curve object type must be Curve:Quadratic or Curve:Cubic
       \\object-list QuadraticCurves
       \\object-list CubicCurves

\\Group Heat Recovery

HeatExchanger:AirToAir:FlatPlate,
      \\memo Flat plate air-to-air heat exchanger, typically used for exhaust or relief air heat
      \\memo recovery.
      \\min-fields 15
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference HXAirToAirNames
      \\reference HXAirToAirFlatPlateNames
      \\reference AFNHeatExchangerNames
  A2, \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Flow Arrangement Type
      \\type choice
      \\key CounterFlow
      \\key ParallelFlow
      \\key CrossFlowBothUnmixed
  A4, \\field Economizer Lockout
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
      \\note Yes means that the heat exchanger will be locked out (off)
      \\note when the economizer is operating or high humidity control is active
  N1, \\field Ratio of Supply to Secondary hA Values
      \\type real
      \\note Ratio of h*A for supply side to h*A for exhaust side
      \\minimum 0.0
  N2, \\field Nominal Supply Air Flow Rate
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m3/s
  N3, \\field Nominal Supply Air Inlet Temperature
      \\required-field
      \\type real
      \\units C
  N4, \\field Nominal Supply Air Outlet Temperature
      \\required-field
      \\type real
      \\units C
  N5, \\field Nominal Secondary Air Flow Rate
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m3/s
  N6, \\field Nominal Secondary Air Inlet Temperature
      \\required-field
      \\type real
      \\units C
  N7, \\field Nominal Electric Power
      \\type real
      \\units W
      \\ip-units W
  A5, \\field Supply Air Inlet Node Name
      \\required-field
      \\type node
  A6, \\field Supply Air Outlet Node Name
      \\required-field
      \\type node
  A7, \\field Secondary Air Inlet Node Name
      \\required-field
      \\type node
  A8; \\field Secondary Air Outlet Node Name
      \\required-field
      \\type node

HeatExchanger:AirToAir:SensibleAndLatent,
      \\memo This object models an air-to-air heat exchanger using effectiveness relationships.
      \\memo The heat exchanger can transfer sensible energy, latent energy, or both between the
      \\memo supply (primary) and exhaust (secondary) air streams.
      \\min-fields 19
  A1,  \\field Name
      \\required-field
      \\type alpha
      \\reference HXAirToAirNames
      \\reference HXAirToAirSensibleAndLatentNames
      \\reference AFNHeatExchangerNames
  A2, \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames
  N1, \\field Nominal Supply Air Flow Rate
      \\required-field
      \\type real
      \\autosizable
      \\minimum> 0.0
      \\units m3/s
  N2, \\field Sensible Effectiveness at 100% Heating Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N3, \\field Latent Effectiveness at 100% Heating Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N4, \\field Sensible Effectiveness at 75% Heating Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N5, \\field Latent Effectiveness at 75% Heating Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N6, \\field Sensible Effectiveness at 100% Cooling Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N7, \\field Latent Effectiveness at 100% Cooling Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N8, \\field Sensible Effectiveness at 75% Cooling Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  N9, \\field Latent Effectiveness at 75% Cooling Air Flow
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
  A3, \\field Supply Air Inlet Node Name
      \\required-field
      \\type node
  A4, \\field Supply Air Outlet Node Name
      \\required-field
      \\type node
  A5, \\field Exhaust Air Inlet Node Name
      \\required-field
      \\type node
  A6, \\field Exhaust Air Outlet Node Name
      \\required-field
      \\type node
  N10,\\field Nominal Electric Power
      \\type real
      \\units W
      \\ip-units W
      \\minimum 0.0
      \\default 0.0
  A7, \\field Supply Air Outlet Temperature Control
      \\type choice
      \\key No
      \\key Yes
      \\default No
  A8, \\field Heat Exchanger Type
      \\type choice
      \\key Plate
      \\key Rotary
      \\default Plate
  A9, \\field Frost Control Type
      \\type choice
      \\key None
      \\key ExhaustAirRecirculation
      \\key ExhaustOnly
      \\key MinimumExhaustTemperature
      \\default None
  N11,\\field Threshold Temperature
      \\type real
      \\units C
      \\default 1.7
      \\note Supply (outdoor) air inlet temp threshold for exhaust air recirculation and
      \\note exhaust only frost control types. Exhaust air outlet threshold Temperature for
      \\note minimum exhaust temperature frost control type.
  N12,\\field Initial Defrost Time Fraction
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.083
      \\note Fraction of the time when frost control will be invoked at the threshold temperature.
      \\note This field only used for exhaust air recirc and exhaust-only frost control types.
  N13,\\field Rate of Defrost Time Fraction Increase
      \\type real
      \\units 1/K
      \\minimum 0.0
      \\default 0.012
      \\note Rate of increase in defrost time fraction as actual temp falls below threshold temperature.
      \\note This field only used for exhaust air recirc and exhaust-only frost control types.
  A10;\\field Economizer Lockout
      \\type choice
      \\key Yes
      \\key No
      \\default Yes
      \\note Yes means that the heat exchanger will be locked out (off)
      \\note when the economizer is operating or high humidity control is active

HeatExchanger:Desiccant:BalancedFlow,
      \\memo This object models a balanced desiccant heat exchanger.
      \\memo The heat exchanger transfers both sensible and latent energy between the
      \\memo process and regeneration air streams. The air flow rate and face velocity
      \\memo are assumed to be the same on both the process and regeneration sides of the
      \\memo heat exchanger.
      \\min-fields 8
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference HXDesiccantBalanced
      \\reference HXAirToAirNames
      \\reference AFNHeatExchangerNames
  A2, \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Regeneration Air Inlet Node Name
      \\required-field
      \\type node
  A4, \\field Regeneration Air Outlet Node Name
      \\required-field
      \\type node
  A5, \\field Process Air Inlet Node Name
      \\required-field
      \\type node
  A6, \\field Process Air Outlet Node Name
      \\required-field
      \\type node
  A7, \\field Heat Exchanger Performance Object Type
      \\type choice
      \\key HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1
      \\default HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1
  A8, \\field Heat Exchanger Performance Name
      \\required-field
      \\type object-list
      \\object-list DesiccantHXPerfData
  A9; \\field Economizer Lockout
      \\type choice
      \\key Yes
      \\key No
      \\default No
      \\note Yes means that the heat exchanger will be locked out (off)
      \\note when the economizer is operating or high humidity control is active

HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,
       \\memo RTO = B1 + B2*RWI + B3*RTI + B4*(RWI/RTI) + B5*PWI + B6*PTI + B7*(PWI/PTI)
       \\memo       + B8*RFV
       \\memo RWO = C1 + C2*RWI + C3*RTI + C4*(RWI/RTI) + C5*PWI + C6*PTI + C7*(PWI/PTI)
       \\memo       + C8*RFV
       \\memo where,
       \\memo RTO = Dry-bulb temperature of the regeneration outlet air (C)
       \\memo RWO = Humidity ratio of the regeneration outlet air (kgWater/kgDryAir)
       \\memo RWI = Humidity ratio of the regeneration inlet air (kgWater/kgDryAir)
       \\memo RTI = Dry-bulb temperature of the regeneration inlet air (C)
       \\memo PWI = Humidity ratio of the process inlet air (kgWater/kgDryAir)
       \\memo PTI = Dry-bulb temperature of the process inlet air (C)
       \\memo RFV = Regeneration Face Velocity (m/s)
       \\min-fields 52
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DesiccantHXPerfData
  N1,  \\field Nominal Air Flow Rate
       \\note Air flow rate at nominal conditions (assumed to be the same for both sides
       \\note of the heat exchanger).
       \\type real
       \\units m3/s
       \\minimum> 0.0
  N2 , \\field Nominal Air Face Velocity
       \\required-field
       \\type real
       \\units m/s
       \\minimum> 0.0
       \\maximum 6.0
  N3 , \\field Nominal Electric Power
       \\note Parasitic electric power (e.g., desiccant wheel motor)
       \\type real
       \\units W
       \\ip-units W
       \\minimum 0.0
       \\default 0.0
  N4 , \\field Temperature Equation Coefficient 1
       \\required-field
       \\type real
  N5 , \\field Temperature Equation Coefficient 2
       \\required-field
       \\type real
  N6 , \\field Temperature Equation Coefficient 3
       \\required-field
       \\type real
  N7 , \\field Temperature Equation Coefficient 4
       \\required-field
       \\type real
  N8 , \\field Temperature Equation Coefficient 5
       \\required-field
       \\type real
  N9 , \\field Temperature Equation Coefficient 6
       \\required-field
       \\type real
  N10, \\field Temperature Equation Coefficient 7
       \\required-field
       \\type real
  N11, \\field Temperature Equation Coefficient 8
       \\required-field
       \\type real
  N12, \\field Minimum Regeneration Inlet Air Humidity Ratio for Temperature Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N13, \\field Maximum Regeneration Inlet Air Humidity Ratio for Temperature Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N14, \\field Minimum Regeneration Inlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N15, \\field Maximum Regeneration Inlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N16, \\field Minimum Process Inlet Air Humidity Ratio for Temperature Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N17, \\field Maximum Process Inlet Air Humidity Ratio for Temperature Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N18, \\field Minimum Process Inlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N19, \\field Maximum Process Inlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N20, \\field Minimum Regeneration Air Velocity for Temperature Equation
       \\required-field
       \\type real
       \\units m/s
       \\minimum> 0.0
  N21, \\field Maximum Regeneration Air Velocity for Temperature Equation
       \\required-field
       \\type real
       \\units m/s
       \\minimum> 0.0
  N22, \\field Minimum Regeneration Outlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N23, \\field Maximum Regeneration Outlet Air Temperature for Temperature Equation
       \\required-field
       \\type real
       \\units C
  N24, \\field Minimum Regeneration Inlet Air Relative Humidity for Temperature Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N25, \\field Maximum Regeneration Inlet Air Relative Humidity for Temperature Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N26, \\field Minimum Process Inlet Air Relative Humidity for Temperature Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N27, \\field Maximum Process Inlet Air Relative Humidity for Temperature Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N28, \\field Humidity Ratio Equation Coefficient 1
       \\required-field
       \\type real
  N29, \\field Humidity Ratio Equation Coefficient 2
       \\required-field
       \\type real
  N30, \\field Humidity Ratio Equation Coefficient 3
       \\required-field
       \\type real
  N31, \\field Humidity Ratio Equation Coefficient 4
       \\required-field
       \\type real
  N32, \\field Humidity Ratio Equation Coefficient 5
       \\required-field
       \\type real
  N33, \\field Humidity Ratio Equation Coefficient 6
       \\required-field
       \\type real
  N34, \\field Humidity Ratio Equation Coefficient 7
       \\required-field
       \\type real
  N35, \\field Humidity Ratio Equation Coefficient 8
       \\required-field
       \\type real
  N36, \\field Minimum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N37, \\field Maximum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N38, \\field Minimum Regeneration Inlet Air Temperature for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units C
  N39, \\field Maximum Regeneration Inlet Air Temperature for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units C
  N40, \\field Minimum Process Inlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N41, \\field Maximum Process Inlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N42, \\field Minimum Process Inlet Air Temperature for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units C
  N43, \\field Maximum Process Inlet Air Temperature for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units C
  N44, \\field Minimum Regeneration Air Velocity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units m/s
       \\minimum> 0.0
  N45, \\field Maximum Regeneration Air Velocity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units m/s
       \\minimum> 0.0
  N46, \\field Minimum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N47, \\field Maximum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units kgWater/kgDryAir
       \\minimum 0.0
       \\maximum 1.0
  N48, \\field Minimum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N49, \\field Maximum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N50, \\field Minimum Process Inlet Air Relative Humidity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
  N51; \\field Maximum Process Inlet Air Relative Humidity for Humidity Ratio Equation
       \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0

\\group Unitary Equipment

AirLoopHVAC:Unitary:Furnace:HeatOnly,
        \\memo Unitary system, heating-only with constant volume supply fan (continuous or cycling)
        \\memo and heating coil (gas, electric, hot water, or steam). Identical to
        \\memo AirLoopHVAC:UnitaryHeatOnly.
        \\min-fields 13
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Furnace Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Furnace Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A fan operating mode schedule value of 0 indicates cycling fan mode (supply air
        \\note fan cycles on and off in tandem with the heating coil).
        \\note Any other schedule value indicates continuous fan mode (supply air fan operates
        \\note continuously regardless of heating coil operation).
        \\note Leaving this schedule name blank will default to cycling fan mode for the
        \\note entire simulation period.
   N1,  \\field Maximum Supply Air Temperature
        \\type real
        \\units C
        \\autosizable
        \\default 80.0
   N2,  \\field Supply Air Flow Rate
        \\required-field
        \\type real
        \\note  This value should be > 0 and <= than the fan air flow rate.
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
   A6,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A7,  \\field Supply Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with continuous fan operating mode (i.e. fan
        \\note operating mode schedule values are greater than 0).
   A8,  \\field Supply Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
   A9 , \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A10, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A11; \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName

AirLoopHVAC:Unitary:Furnace:HeatCool,
        \\memo Unitary system, heating and cooling with constant volume supply fan (continuous or
        \\memo cycling), direct expansion (DX) cooling coil, heating coil (gas, electric,
        \\memo hot water, or steam), and optional reheat coil for dehumidification control.
        \\memo Identical to AirLoopHVAC:UnitaryHeatCool.
        \\min-fields 17
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
        \\note A schedule value greater than zero (usually 1 is used) indicates that the unit is
        \\note available to operate as needed. A value less than or equal to zero (usually zero
        \\note is used) denotes that the unit must be off.
   A3,  \\field Furnace Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Furnace Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A fan operating mode schedule value of 0 indicates cycling fan mode (supply air
        \\note fan cycles on and off in tandem with the cooling or heating coil).
        \\note Any other schedule value indicates continuous fan mode (supply air fan operates
        \\note continuously regardless of cooling or heating coil operation). Provide a schedule
        \\note with non-zero values when high humidity control is specified.
        \\note Leaving this schedule name blank will default to cycling fan mode for the
        \\note entire simulation period.
   N1,  \\field Maximum Supply Air Temperature
        \\type real
        \\units C
        \\autosizable
        \\default 80.0
   N2,  \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
   N3,  \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow fate.
   N4,  \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
        \\note Only used when fan operating mode is continuous (disregarded for cycling fan mode).
        \\note This air flow rate is used when no heating or cooling is required (i.e., the DX coil
        \\note compressor and heating coil are off). If this field is left blank or zero, the supply
        \\note air flow rate from the previous on cycle (either cooling or heating) is used.
   A6,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A7,  \\field Supply Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with continuous fan operating mode (i.e. supply
        \\note air fan operating mode schedule values not equal to 0).
   A8,  \\field Supply Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
   A9 , \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A10, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A11, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
   A12, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\note  Only works with DX cooling coil types
   A13, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
   A14, \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key Multimode
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only
        \\note Multimode = activate enhanced dehumidification mode
        \\note as needed and meet sensible load.  Valid only with
        \\note cooling coil type CoilSystem:Cooling:DX:HeatExchangerAssisted.
        \\note This control mode allows the heat exchanger to be turned
        \\note on and off based on the zone dehumidification requirements.
        \\note A ZoneControl:Humidistat object is also required.
        \\note CoolReheat = cool beyond the dry bulb setpoint.
        \\note as required to meet the humidity setpoint.  Valid with all
        \\note cooling coil types. When a heat exchanger assisted cooling
        \\note coil is used, the heat exchanger is locked on at all times.
        \\note A ZoneControl:Humidistat object is also required.
   A15, \\field Reheat Coil Object Type
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Desuperheater
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note Only required if dehumidification control type is "CoolReheat"
        \\note works with gas, electric, hot water and steam heating coils
   A16; \\field Reheat Coil Name
        \\note Only required if dehumidification control type is "CoolReheat"
        \\type object-list
        \\object-list HeatingCoilName

AirLoopHVAC:UnitaryHeatOnly,
        \\memo Unitary system, heating-only with constant volume supply fan (continuous or cycling)
        \\memo and heating coil (gas, electric, hot water, or steam). Identical to
        \\memo AirLoopHVAC:Unitary:Furnace:HeatOnly.
        \\min-fields 13
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Unitary System Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Unitary System Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A fan operating mode schedule value of 0 indicates cycling fan mode (supply air
        \\note fan cycles on and off in tandem with the heating coil).
        \\note Any other schedule value indicates continuous fan mode (supply air fan operates
        \\note continuously regardless of heating coil operation).
        \\note Leaving this schedule name blank will default to cycling fan mode for the
        \\note entire simulation period.
   N1,  \\field Maximum Supply Air Temperature
        \\type real
        \\units C
        \\autosizable
        \\default 80.0
   N2,  \\field Supply Air Flow Rate
        \\required-field
        \\type real
        \\note  This value should be > 0 and <= than the fan air flow rate.
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
   A6,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A7,  \\field Supply Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with continuous fan operating mode (i.e. fan
        \\note operating mode schedule values are greater than 0).
   A8,  \\field Supply Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
   A9 , \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A10, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A11; \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName

AirLoopHVAC:UnitaryHeatCool,
        \\memo Unitary system, heating and cooling with constant volume supply fan (continuous or
        \\memo cycling), direct expansion (DX) cooling coil, heating coil (gas, electric,
        \\memo hot water, or steam), and optional reheat coil for dehumidification control.
        \\memo Identical to AirLoopHVAC:Unitary:Furnace:HeatCool.
        \\min-fields 17
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Unitary System Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Unitary System Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A fan operating mode schedule value of 0 indicates cycling fan mode (supply air
        \\note fan cycles on and off in tandem with the cooling or heating coil).
        \\note Any other schedule value indicates continuous fan mode (supply air fan operates
        \\note continuously regardless of cooling or heating coil operation). Provide a schedule
        \\note with non-zero values when high humidity control is specified.
        \\note Leaving this schedule name blank will default to cycling fan mode for the
        \\note entire simulation period.
   N1,  \\field Maximum Supply Air Temperature
        \\type real
        \\units C
        \\autosizable
        \\default 80.0
   N2,  \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
   N3,  \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
   N4,  \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
        \\note Only used when fan operating mode is continuous (disregarded for cycling fan mode).
        \\note This air flow rate is used when no heating or cooling is required (i.e., the DX coil
        \\note compressor and heating coil are off). If this field is left blank or zero, the supply
        \\note air flow rate from the previous on cycle (either cooling or heating) is used.
   A6,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A7,  \\field Supply Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with continuous fan operating mode (i.e. supply
        \\note air fan operating mode schedule values not equal to 0).
   A8,  \\field Supply Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
   A9 , \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A10, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A11, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
   A12, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key Coil:Cooling:DX:VariableSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\note  Only works with DX cooling coil types or
        \\note Coil:Cooling:DX:VariableSpeed.
   A13, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
        \\object-list CoolingCoilsDXVariableSpeed
   A14, \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key Multimode
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only
        \\note Multimode = activate enhanced dehumidification mode
        \\note as needed and meet sensible load.  Valid only with
        \\note cooling coil type CoilSystem:Cooling:DX:HeatExchangerAssisted.
        \\note This control mode allows the heat exchanger to be turned
        \\note on and off based on the zone dehumidification requirements.
        \\note A ZoneControl:Humidistat object is also required.
        \\note CoolReheat = cool beyond the dry bulb setpoint.
        \\note as required to meet the humidity setpoint.  Valid with all
        \\note cooling coil types. When a heat exchanger assisted Cooling
        \\note coil is used, the heat exchanger is locked on at all times.
        \\note A ZoneControl:Humidistat object is also required.
   A15, \\field Reheat Coil Object Type
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Desuperheater
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note Only required if dehumidification control type is "CoolReheat"
        \\note works with gas, electric, desuperheating, hot water and steam heating coils
   A16; \\field Reheat Coil Name
        \\note Only required if dehumidification control type is "CoolReheat"
        \\type object-list
        \\object-list HeatingCoilName

AirLoopHVAC:UnitaryHeatPump:AirToAir,
        \\memo Unitary heat pump system, heating and cooling, single-speed with supply fan, direct
        \\memo expansion (DX) cooling coil, DX heating coil (air-to-air heat pump), and supplemental
        \\memo heating coil (gas, electric, hot water, or steam).
        \\min-fields 19
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
        \\note A schedule value greater than zero (usually 1 is used) indicates that the unit is
        \\note available to operate as needed. A value less than or equal to zero (usually zero
        \\note is used) denotes that the unit must be off.
   A3,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
   N2,  \\field Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
   N3,  \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
        \\note Must be less than or equal to the fan's maximum flow rate.
        \\note Only used when fan operating mode is continuous (disregarded for cycling fan mode).
        \\note This air flow rate is used when no heating or cooling is required (i.e., the DX coil
        \\note compressor and supplemental heating coil are off). If this field is left blank or zero,
        \\note the supply air flow rate from the previous on cycle (either cooling or heating) is used.
   A5,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A6,  \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Fan:ConstantVolume only works with continuous fan operating mode (i.e. fan
        \\note operating mode schedule values are greater than 0 or the fan operating mode
        \\note schedule name field is left blank).
   A7,  \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
        \\note  Needs to match in the fan object
   A8,  \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:DX:SingleSpeed
        \\key Coil:Heating:DX:VariableSpeed
        \\note  Only works with Coil:Heating:DX:SingleSpeed or
        \\note Coil:Heating:DX:VariableSpeed
   A9,  \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsDXSingleSpeed
        \\object-list HeatingCoilsDXVariableSpeed
        \\note  Needs to match in the DX heating coil object
   A10, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key Coil:Cooling:DX:VariableSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\note Only works with Coil:Cooling:DX:SingleSpeed or
        \\note CoilSystem:Cooling:DX:HeatExchangerAssisted or
        \\note Coil:Cooling:DX:VariableSpeed
   A11, \\field Cooling Coil Name
        \\required-field
        \\note Needs to match in the DX cooling coil object
        \\type object-list
        \\object-list CoolingCoilsDXSingleSpeed
        \\object-list CoolingCoilsDXVariableSpeed
   A12, \\field Supplemental Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A13, \\field Supplemental Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note  Needs to match in the supplemental heating coil object
   N4 , \\field Maximum Supply Air Temperature from Supplemental Heater
        \\required-field
        \\type real
        \\units C
        \\autosizable
   N5 , \\field Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation
        \\type real
        \\maximum 21.0
        \\default 21.0
        \\units C
   A14, \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A15, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A fan operating mode schedule value of 0 indicates cycling fan mode (supply air
        \\note fan cycles on and off in tandem with the cooling or heating coil).
        \\note Any other schedule value indicates continuous fan mode (supply air fan operates
        \\note continuously regardless of cooling or heating coil operation).
        \\note Leaving this schedule name blank will default to cycling fan mode for the
        \\note entire simulation period.
   A16; \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key Multimode
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only
        \\note Multimode = activate enhanced dehumidification mode
        \\note as needed and meet sensible load.  Valid only with
        \\note cooling coil type CoilSystem:Cooling:DX:HeatExchangerAssisted.
        \\note This control mode allows the heat exchanger to be turned
        \\note on and off based on the zone dehumidification requirements.
        \\note A ZoneControl:Humidistat object is also required.
        \\note CoolReheat = cool beyond the dry bulb setpoint.
        \\note as required to meet the humidity setpoint.  Valid with all
        \\note cooling coil types. When a heat exchanger assisted Cooling
        \\note coil is used, the heat exchanger is locked on at all times.
        \\note A ZoneControl:Humidistat object is also required.

AirLoopHVAC:UnitaryHeatPump:WaterToAir,
        \\memo Unitary heat pump system, heating and cooling, single-speed with constant volume
        \\memo supply fan (continuous or cycling), direct expansion (DX) cooling coil, DX heating
        \\memo coil (water-to-air heat pump), and supplemental heating coil (gas, electric,
        \\memo hot water, or steam).
        \\min-fields 25
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Supply Air Flow Rate
        \\required-field
        \\type real
        \\note  This value should be > 0 and <= than the fan air flow rate.
        \\units m3/s
        \\minimum> 0.0
        \\autosizable
   A5,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A6,  \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\note  Only works with On/Off Fan
   A7,  \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansOnOff
        \\note Needs to match Fan:OnOff object
   A8,  \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:WaterToAirHeatPump:ParameterEstimation
        \\key Coil:Heating:WaterToAirHeatPump:EquationFit
        \\key Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit
   A9,  \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsWaterToAirHP
        \\object-list HeatingCoilsWaterToAirVSHP
        \\note  Needs to match in the water-to-air heatpump heating coil object
   N2,  \\field Heating Convergence
        \\required-field
        \\type real
        \\minimum> 0.0
        \\default 0.001
   A10, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:WaterToAirHeatPump:ParameterEstimation
        \\key Coil:Cooling:WaterToAirHeatPump:EquationFit
        \\key Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit
   A11, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsWaterToAirHP
        \\object-list CoolingCoilsWaterToAirVSHP
        \\note Needs to match in the water-to-air heatpump cooling coil object
   N3,  \\field Cooling Convergence
        \\required-field
        \\type real
        \\minimum> 0.0
        \\default 0.001
   N4,  \\field Maximum Cycling Rate
        \\type real
        \\units cycles/hr
        \\minimum 0.0
        \\maximum 5.0
        \\default 2.5
        \\note The maximum on-off cycling rate for the compressor
        \\note Suggested value is 2.5 for a typical heat pump
   N5,  \\field Heat Pump Time Constant
        \\type real
        \\units s
        \\minimum 0.0
        \\maximum 500.0
        \\default 60.0
        \\note Time constant for the cooling coil's capacity to reach steady state after startup
        \\note Suggested value is 60 for a typical heat pump
   N6,  \\field Fraction of On-Cycle Power Use
        \\minimum 0.0
        \\maximum 0.05
        \\default 0.01
        \\note The fraction of on-cycle power use to adjust the part load fraction based on
        \\note the off-cycle power consumption due to crankcase heaters, controls, fans, and etc.
        \\note Suggested value is 0.01 for a typical heat pump
   N7,  \\field Heat Pump Fan Delay Time
        \\units s
        \\minimum 0.0
        \\default 60
        \\note Programmed time delay for heat pump fan to shut off after compressor cycle off.
        \\note Only required when fan operating mode is cycling
        \\note Enter 0 when fan operating mode is continuous
   A12, \\field Supplemental Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A13, \\field Supplemental Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note  Needs to match in the supplemental heating coil object
   N8,  \\field Maximum Supply Air Temperature from Supplemental Heater
        \\required-field
        \\type real
        \\units C
        \\autosizable
   N9, \\field Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation
        \\type real
        \\maximum 21.0
        \\default 21.0
        \\units C
   A14, \\field Outdoor Dry-Bulb Temperature Sensor Node Name
        \\type node
   A15, \\field Fan Placement
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\default BlowThrough
   A16, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that controls fan operation. Schedule values of 0 denote
        \\note cycling fan operation (fan cycles with cooling or heating coil). Schedule values greater
        \\note than 0 denote constant fan operation (fan runs continually regardless of coil operation).
        \\note The fan operating mode defaults to cycling fan operation if this field is left blank.
   A17, \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only
        \\note CoolReheat = cool beyond the dry bulb setpoint.
        \\note as required to meet the humidity setpoint.  Valid only with
        \\note Coil:Cooling:WaterToAirHeatPump:EquationFit or
        \\note Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit
   A18; \\field Heat Pump Coil Water Flow Mode
        \\type choice
        \\key Constant
        \\key Cycling
        \\key ConstantOnDemand
        \\default Cycling
        \\note used only when the heat pump coils are of the type WaterToAirHeatPump:EquationFit
        \\note Constant results in 100% water flow regardless of compressor PLR
        \\note Cycling results in water flow that matches compressor PLR
        \\note ConstantOnDemand results in 100% water flow whenever the coil is on, but is 0% whenever the coil has no load

AirLoopHVAC:UnitaryHeatCool:VAVChangeoverBypass,
        \\memo Unitary system, heating and cooling with constant volume supply fan (continuous or
        \\memo cycling), direct expansion (DX) cooling coil, heating coil (gas, electric,
        \\memo hot water, steam, or DX air-to-air heat pump) and bypass damper for variable volume
        \\memo flow to terminal units. Used with AirTerminal:SingleDuct:VAV:HeatAndCool:Reheat
        \\memo or AirTerminal:SingleDuct:VAV:HeatAndCool:NoReheat.
        \\min-fields 23
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note Enter a unique name for this unitary system.
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the availability schedule name. Schedule values of zero denote system
        \\note is Off. Non-zero schedule values denote system is available to operate.
   N1 , \\field System Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0.0
        \\note Enter the system air flow rate during cooling
        \\note operation or specify autosize.
   N2 , \\field System Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0.0
        \\note Enter the system air flow rate during heating
        \\note operation or specify autosize.
   N3 , \\field System Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
        \\note Only used when the supply air fan operating mode is continuous (see field
        \\note Supply air fan operating mode schedule name). This system air flow rate
        \\note is used when no heating or cooling is required and the coils are off.
        \\note If this field is left blank or zero, the system air flow rate from the
        \\note previous on cycle (either cooling or heating) is used.
   N4 , \\field Outdoor Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum 0.0
        \\note Enter the outdoor air flow rate during
        \\note cooling operation or specify autosize.
   N5 , \\field Outdoor Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum 0.0
        \\note Enter the outdoor air flow rate during
        \\note heating operation or specify autosize.
   N6 , \\field Outdoor Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0.0
        \\autosizable
        \\note Only used when the supply air fan operating mode is continuous (see field
        \\note Supply air fan operating mode schedule name). This outdoor air flow rate
        \\note is used when no heating or cooling is required and the coils are off.
        \\note If this field is left blank or zero, the outdoor air flow rate from the previous on cycle
        \\note (either cooling or heating) is used.
   A3 , \\field Outdoor Air Flow Rate Multiplier Schedule Name
        \\type alpha
        \\object-list ScheduleNames
        \\note Enter the name of a schedule that contains multipliers for the outdoor air
        \\note flow rates. Schedule values must be from 0 to 1.
        \\note If field is left blank, model assumes multiplier is 1 for the entire simulation period.
   A4 , \\field Air Inlet Node Name
        \\required-field
        \\type node
        \\note Enter the name of the unitary system's air inlet node.
   A5 , \\field Bypass Duct Mixer Node Name
        \\required-field
        \\type node
        \\note Enter the name of the bypass duct mixer node. This name should be the name
        \\note of the return air node for the outdoor air mixer associated with this system.
        \\note This node name must be different from the air inlet node name.
   A6 , \\field Bypass Duct Splitter Node Name
        \\required-field
        \\type node
        \\note Enter the name of the bypass duct splitter node.
        \\note This splitter air node is the outlet node of the last component in this unitary
        \\note system. For blow through fan placement, the splitter air node is the outlet
        \\note node of the heating coil. For draw through fan placement, the splitter node
        \\note is the outlet node of the supply air fan.
   A7 , \\field Air Outlet Node Name
        \\required-field
        \\type node
        \\note Enter the name of the unitary system's air outlet node.
   A8,  \\field Outdoor Air Mixer Object Type
        \\required-field
        \\type choice
        \\key OutdoorAir:Mixer
        \\note currently only one type OutdoorAir:Mixer object is available.
   A9 , \\field Outdoor Air Mixer Name
        \\required-field
        \\type object-list
        \\object-list OutdoorAirMixers
        \\note Enter the name of the outdoor air mixer used with this unitary system.
   A10, \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Specify the type of supply air fan used in this unitary system.
   A11, \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
        \\note Enter the name of the supply air fan used in this unitary system.
   A12, \\field Supply Air Fan Placement
        \\required-field
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\note Specify supply air fan placement as either blow through or draw through.
        \\note BlowThrough means the supply air fan is located before the cooling
        \\note coil. DrawThrough means the supply air fan is located after the heating coil.
   A13, \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule to control the supply air fan. Schedule Name values of zero
        \\note mean that the supply air fan will cycle off if there is no cooling or heating load
        \\note in any of the zones being served by this system. Non-zero schedule values mean
        \\note that the supply air fan will operate continuously even if there is no cooling or
        \\note heating load in any of the zones being served. If this field is left blank,
        \\note the supply air fan will operate continuously for the entire simulation period.
   A14, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:SingleSpeed
        \\key CoilSystem:Cooling:DX:HeatExchangerAssisted
        \\key Coil:Cooling:DX:TwoStageWithHumidityControlMode
        \\note Specify the type of cooling coil used in this unitary system.
   A15, \\field Cooling Coil Name
        \\required-field
        \\type object-list
        \\object-list CoolingCoilsDXMultiModeOrSingleSpeed
        \\note Enter the name of the cooling coil used in this unitary system.
   A16, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:DX:SingleSpeed
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with DX, gas, electric, hot water and steam heating coils
        \\note Specify the type of heating coil used in this unitary system.
   A17, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilName
        \\note Enter the name of the heating coil used in this unitary system.
   A18, \\field Priority Control Mode
        \\type choice
        \\key CoolingPriority
        \\key HeatingPriority
        \\key ZonePriority
        \\default ZonePriority
        \\note CoolingPriority = system provides cooling if any zone requires cooling.
        \\note HeatingPriority = system provides heating if any zone requires heating.
        \\note ZonePriority = system controlled based on the total number of zones
        \\note requiring cooling or heating (highest number of zones
        \\note in cooling or heating determines the system's operating mode).
   N7 , \\field Minimum Outlet Air Temperature During Cooling Operation
        \\type real
        \\units C
        \\minimum> 0.0
        \\default 8.0
        \\note Specify the minimum outlet air temperature allowed for this unitary system
        \\note during cooling operation. This value should be less than the maximum outlet
        \\note air temperature during heating operation.
   N8 , \\field Maximum Outlet Air Temperature During Heating Operation
        \\type real
        \\units C
        \\minimum> 0.0
        \\default 50.0
        \\note Specify the maximum outlet air temperature allowed for this unitary system
        \\note during heating operation. This value should be greater than the minimum outlet
        \\note air temperature during cooling operation.
   A19; \\field Dehumidification Control Type
        \\type choice
        \\key None
        \\key Multimode
        \\key CoolReheat
        \\default None
        \\note None = meet sensible load only.
        \\note Multimode = activate enhanced dehumidification mode
        \\note as needed and meet sensible load.  Valid only with
        \\note Coil:Cooling:DX:TwoStageWithHumidityControlMode.
        \\note CoolReheat = cool beyond the Dry-Bulb temperature setpoint
        \\note as required to meet the humidity setpoint.  Valid only with
        \\note Coil:Cooling:DX:TwoStageWithHumidityControlMode.
        \\note For all dehumidification controls, the max humidity setpoint
        \\note on this unitary system's air outlet node is used.
        \\note This must be set using ZoneControl:Humidistat and
        \\note SetpointManager:SingleZone:Humidity:Maximum,
        \\note SetpointManager:MultiZone:Humidity:Maximum or
        \\note SetpointManager:MultiZone:MaximumHumidity:Average objects.

AirLoopHVAC:UnitaryHeatPump:AirToAir:MultiSpeed,
        \\memo Unitary system, heating and cooling, multi-speed with constant volume supply fan
        \\memo (continuous or cycling), direct expansion (DX) cooling coil, heating coil
        \\memo (DX air-to-air heat pump, gas, electric, hot water, or steam), and supplemental
        \\memo heating coil (gas, electric, hot water, or steam).
        \\min-fields 31
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Air Inlet Node Name
        \\required-field
        \\type node
   A4,  \\field Air Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Controlling Zone or Thermostat Location
        \\required-field
        \\type object-list
        \\object-list ZoneNames
   A6,  \\field Supply Air Fan Object Type
        \\required-field
        \\type choice
        \\key Fan:OnOff
        \\key Fan:ConstantVolume
        \\note Select the type of supply air fan used in this unitary system.
   A7,  \\field Supply Air Fan Name
        \\required-field
        \\type object-list
        \\object-list FansCVandOnOff
        \\note Enter the name of the supply air fan used in this unitary system.
   A8,  \\field Supply Air Fan Placement
        \\required-field
        \\type choice
        \\key BlowThrough
        \\key DrawThrough
        \\note Select supply air fan placement as either BlowThrough or DrawThrough.
        \\note BlowThrough means the supply air fan is located before the cooling
        \\note coil. DrawThrough means the supply air fan is located after the heating coil
        \\note but before the optional supplemental heating coil.
   A9 , \\field Supply Air Fan Operating Mode Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Enter the name of a schedule to control the supply air fan. Schedule values of zero
        \\note mean that the supply air fan will cycle off if there is no cooling or heating load
        \\note in the control zone. Non-zero schedule values mean that the supply air fan
        \\note will operate continuously even if there is no cooling or heating load
        \\note in the control zone. If this field is left blank, the supply air fan will
        \\note operate continuously for the entire simulation period.
   A10, \\field Heating Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Heating:DX:MultiSpeed
        \\key Coil:Heating:Electric:MultiStage
        \\key Coil:Heating:Gas:MultiStage
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note Multi Speed DX, Electric, Gas, and Single speed Water and Steam coils
   A11, \\field Heating Coil Name
        \\required-field
        \\type object-list
        \\object-list HeatingCoilsElectricMultiStage
        \\object-list HeatingCoilsGasMultiStage
   N1 , \\field Minimum Outdoor Dry-Bulb Temperature for Compressor Operation
        \\type real
        \\minimum -20.0
        \\default -8.0
        \\units C
        \\note Needs to match the corresponding minimum outdoor temperature defined
        \\note in the DX heating coil object.
   A12, \\field Cooling Coil Object Type
        \\required-field
        \\type choice
        \\key Coil:Cooling:DX:MultiSpeed
        \\note Only works with Coil:Cooling:DX:MultiSpeed
   A13, \\field Cooling Coil Name
        \\required-field
        \\note Needs to match in the DX Cooling Coil object
        \\type object-list
        \\object-list CoolingCoilsDXMultiSpeed
   A14, \\field Supplemental Heating Coil Object Type
        \\type choice
        \\key Coil:Heating:Gas
        \\key Coil:Heating:Electric
        \\key Coil:Heating:Water
        \\key Coil:Heating:Steam
        \\note works with gas, electric, hot water and steam heating coils
   A15, \\field Supplemental Heating Coil Name
        \\type object-list
        \\object-list HeatingCoilName
        \\note Needs to match in the supplemental heating coil object
   N2 , \\field Maximum Supply Air Temperature from Supplemental Heater
        \\type real
        \\units C
        \\autosizable
   N3 , \\field Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation
        \\type real
        \\maximum 21.0
        \\default 21.0
        \\units C
   N4 , \\field Auxiliary On-Cycle Electric Power
        \\type real
        \\units W
        \\minimum 0
        \\default 0
   N5 , \\field Auxiliary Off-Cycle Electric Power
        \\type real
        \\units W
        \\minimum 0
        \\default 0
   N6 , \\field Design Heat Recovery Water Flow Rate
        \\type real
        \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
        \\note Used for heat recovery to an EnergyPlus plant loop.
        \\units m3/s
        \\minimum 0.0
        \\default 0.0
   N7,  \\field Maximum Temperature for Heat Recovery
        \\units C
        \\maximum 100.0
        \\minimum 0.0
        \\default 80.0
   A16, \\field Heat Recovery Water Inlet Node Name
        \\type node
   A17, \\field Heat Recovery Water Outlet Node Name
        \\type node
   N8 , \\field Supply Air Flow Rate When No Cooling or Heating is Needed
        \\type real
        \\units m3/s
        \\minimum 0
        \\autosizable
        \\note Only used when the supply air fan operating mode is continuous (see field
        \\note Supply Air Fan Operating Mode Schedule Name). This air flow rate
        \\note is used when no heating or cooling is required and the coils are off.
        \\note If this field is left blank or zero, the supply air flow rate from the
        \\note previous on cycle (either cooling or heating) is used.
   N9, \\field Number of Speeds for Heating
        \\required-field
        \\type integer
        \\minimum 1
        \\maximum 4
        \\note Enter the number of the following sets of data for air flow rates.
        \\note If Heating Coil Object Type is Coil:Heating:Water or Coil:Heating:Steam,
        \\note this field should be 1.
   N10, \\field Number of Speeds for Cooling
        \\required-field
        \\type integer
        \\minimum 2
        \\maximum 4
        \\note Enter the number of the following sets of data for air flow rates.
   N11, \\field Speed 1 Supply Air Flow Rate During Heating Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during heating
        \\note operation or specify autosize.
   N12, \\field Speed 2 Supply Air Flow Rate During Heating Operation
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during heating
        \\note operation or specify autosize.
   N13, \\field Speed 3 Supply Air Flow Rate During Heating Operation
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during heating
        \\note operation or specify autosize.
   N14, \\field Speed 4 Supply Air Flow Rate During Heating Operation
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during heating
        \\note operation or specify autosize.
   N15, \\field Speed 1 Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during cooling
        \\note operation or specify autosize.
   N16, \\field Speed 2 Supply Air Flow Rate During Cooling Operation
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during cooling
        \\note operation or specify autosize.
   N17, \\field Speed 3 Supply Air Flow Rate During Cooling Operation
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during cooling
        \\note operation or specify autosize.
   N18; \\field Speed 4 Supply Air Flow Rate During Cooling Operation
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum> 0
        \\note Enter the operating supply air flow rate during cooling
        \\note operation or specify autosize.



\\group Variable Refrigerant Flow Equipment

AirConditioner:VariableRefrigerantFlow,
       \\memo Variable refrigerant flow (VRF) air-to-air heat pump condensing unit (includes one
       \\memo or more electric compressors and outdoor fan). Serves one or more VRF zone terminal
       \\memo units. See ZoneHVAC:TerminalUnit:VariableRefrigerantFlow and ZoneTerminalUnitList.
       \\min-fields 37
  A1 , \\field Heat Pump Name
       \\required-field
       \\type alpha
       \\note Enter a unique name for this variable refrigerant flow heat pump.
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
       \\note Enter the name of a schedule that defines the availability of the unit.
       \\note Schedule values of 0 denote the unit is off. All other values denote the unit is available.
       \\note If this field is left blank, the unit is available the entire simulation.
  N1 , \\field Rated Total Cooling Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\note Enter the total cooling capacity in watts at rated conditions or set to autosize.
  N2 , \\field Rated Cooling COP
       \\type real
       \\units W/W
       \\minimum> 0.0
       \\default 3.3
       \\note Enter the coefficient of performance at rated conditions or leave blank to use default.
       \\note COP includes compressor and condenser fan electrical energy input
       \\note COP does not include supply fan heat or supply fan electrical energy input
  N3 , \\field Minimum Outdoor Temperature in Cooling Mode
       \\type real
       \\units C
       \\default -6.0
       \\note Enter the minimum outdoor temperature allowed for cooling operation.
       \\note Cooling is disabled below this temperature.
  N4 , \\field Maximum Outdoor Temperature in Cooling Mode
       \\type real
       \\units C
       \\default 43.0
       \\note Enter the maximum outdoor temperature allowed for cooling operation.
       \\note Cooling is disabled above this temperature.
  A3 , \\field Cooling Capacity Ratio Modifier Function of Low Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter a curve name that represents full load cooling capacity ratio as a
       \\note function of outdoor dry-bulb temperature and indoor wet-bulb temperature.
       \\note Up to two curves are allowed if the performance cannot be represented by a single curve.
       \\note The following two fields are used if two curves are required.
  A4 , \\field Cooling Capacity Ratio Boundary Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve object is used to allow separate low and high cooling capacity ratio
       \\note performance curves. This curve represents a line passing through the points where
       \\note performance changes. The curve calculates outdoor dry-bulb temperature given weighted
       \\note average indoor wet-bulb temperature. If a single performance curve is used,
       \\note leave this field blank.
  A5 , \\field Cooling Capacity Ratio Modifier Function of High Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note This curve object is used to describe the high outdoor temperature
       \\note performance curve used to describe cooling capacity ratio.
       \\note This curve is used when a single performance curve does not accurately describe
       \\note cooling capacity ratio as a function of temperature.
       \\note If a single performance curve is used, leave this field blank.
  A6 , \\field Cooling Energy Input Ratio Modifier Function of Low Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter a curve name that represents cooling energy ratio as a function of
       \\note outdoor dry-bulb temperature and indoor wet-bulb temperature
  A7 , \\field Cooling Energy Input Ratio Boundary Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve object is used to allow separate low and high cooling energy input ratio
       \\note performance curves. This curve represents a line passing through the points where
       \\note performance changes. The curve calculates outdoor dry-bulb temperature given weighted
       \\note average indoor wet-bulb temperature. If a single performance curve is used,
       \\note leave this field blank.
  A8 , \\field Cooling Energy Input Ratio Modifier Function of High Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note This curve object is used to describe the high outdoor temperature
       \\note performance curve used to describe cooling energy ratio.
       \\note This curve is used when a single performance curve does not accurately describe
       \\note cooling energy ratio as a function of temperature
  A9 , \\field Cooling Energy Input Ratio Modifier Function of Low Part-Load Ratio Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note Enter a curve name that represents cooling energy ratio as a function of
       \\note part-load ratio for part-load ratios less than or equal to 1.
       \\note If this field is left blank, the model assumes energy is proportional to
       \\note part-load ratio.
  A10, \\field Cooling Energy Input Ratio Modifier Function of High Part-Load Ratio Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note Enter a curve name that represents cooling energy ratio as a function of
       \\note part-load ratio for part-load ratios greater than 1. Part-load ratios
       \\note can exceed 1 in variable speed compresson systems.
       \\note If this field is left blank, the model assumes energy is proportional to
       \\note part-load ratio.
  A11, \\field Cooling Combination Ratio Correction Factor Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve defines how rated capacity changes when the total indoor terminal unit cooling
       \\note capacity is greater than the Rated Total Cooling Capacity defined in this object.
       \\note If this field is left blank, the model assumes total indoor terminal unit cooling
       \\note capacity is equal to the Rated Total Cooling Capacity defined above.
  A12, \\field Cooling Part-Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve defines the cycling losses when the heat pump compressor cycles on and off
       \\note below the Minimum Heat Pump Part-Load Ratio specified in the field below.
  N5 , \\field Rated Total Heating Capacity
       \\type real
       \\units W
       \\autosizable
       \\note Enter the total heating capacity in watts at rated conditions or set to autosize.
  N6 , \\field Rated Total Heating Capacity Sizing Ratio
       \\type real
       \\units W/W
       \\minimum 1.0
       \\default 1.0
       \\note If the Rated Total Heating Capacity is autosized, the heating capacity is sized
       \\note to be equal to the cooling capacity multiplied by this sizing ratio. The zone
       \\note terminal unit heating coils are also sized using this ratio unless the sizing
       \\note ratio input in the ZoneHVAC:TerminalUnit:VariableRefrigerantFlow object is entered.
  N7 , \\field Rated Heating COP
       \\type real
       \\units W/W
       \\note COP includes compressor and condenser fan electrical energy input
       \\note COP does not include supply fan heat or supply fan electrical energy input
       \\default 3.4
  N8 , \\field Minimum Outdoor Temperature in Heating Mode
       \\type real
       \\units C
       \\default -20.0
       \\note Enter the minimum outdoor temperature allowed for cooling operation.
  N9 , \\field Maximum Outdoor Temperature in Heating Mode
       \\type real
       \\units C
       \\default 16.0
       \\note Enter the maximum outdoor temperature allowed for heating operation.
  A13, \\field Heating Capacity Ratio Modifier Function of Low Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter a curve name that represents full load heating capacity ratio as a
       \\note function of outdoor wet-bulb temperature and indoor dry-bulb temperature.
       \\note Outdoor dry-bulb temperature may be used if wet-bulb temperature data is unavailable.
       \\note See Heating Performance Curve Outdoor Temperature Type input below to determine which
       \\note outdoor temperature type to use.
       \\note Up to two curves are allowed if the performance cannot be represented by a single curve.
       \\note The following two fields are used if two curves are required.
  A14, \\field Heating Capacity Ratio Boundary Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve object is used to allow separate low and high heating capacity ratio
       \\note performance curves. This curve represents a line passing through the points where
       \\note performance changes. The curve calculates outdoor dry-bulb or wet-bulb temperature
       \\note given weighted average indoor dry-bulb temperature. See Heating Performance Curve
       \\note Outdoor Temperature Type input below to determine which outdoor temperature type to use.
       \\note If a single performance curve is used, leave this field blank.
  A15, \\field Heating Capacity Ratio Modifier Function of High Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note This curve object is used to describe the high outdoor temperature
       \\note performance curve used to describe heating capacity ratio.
       \\note This curve is used when a single performance curve does not accurately describe
       \\note heating capacity ratio as a function of temperature.
       \\note If a single performance curve is used, leave this field blank.
  A16, \\field Heating Energy Input Ratio Modifier Function of Low Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter a curve name that represents heating energy ratio as a function of
       \\note outdoor wet-bulb temperature and indoor dry-bulb temperature
       \\note Outdoor dry-bulb temperature may be used if wet-bulb temperature data is unavailable.
       \\note See Heating Performance Curve Outdoor Temperature Type input below to determine which
       \\note outdoor temperature type to use.
  A17, \\field Heating Energy Input Ratio Boundary Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve object is used to allow separate low and high heating energy input ratio
       \\note performance curves. This curve represents a line passing through the points where
       \\note performance changes. The curve calculates outdoor dry-bulb or wet-bulb temperature
       \\note given weighted average indoor dry-bulb temperature. See Heating Performance Curve
       \\note Outdoor Temperature Type input below to determine which outdoor temperature type to use.
       \\note If a single performance curve is used, leave this field blank.
  A18, \\field Heating Energy Input Ratio Modifier Function of High Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note This curve object is used to allow separate performance curves for heating energy.
       \\note If a single performance curve is used, leave this field blank.
  A19, \\field Heating Performance Curve Outdoor Temperature Type
       \\type choice
       \\key DryBulbTemperature
       \\key WetBulbTemperature
       \\default WetBulbTemperature
       \\note Determines temperature type for heating capacity curves and heating energy curves.
       \\note This input determines whether the outdoor air dry-bulb or wet-bulb temperature
       \\note is used to evaluate these curves.
  A20, \\field Heating Energy Input Ratio Modifier Function of Low Part-Load Ratio Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note This curve represents the heating energy input ratio for part-load ratios less than 1.
  A21, \\field Heating Energy Input Ratio Modifier Function of High Part-Load Ratio Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve represents the heating energy input ratio for part-load ratios greater than 1.
  A22, \\field Heating Combination Ratio Correction Factor Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve defines how rated capacity changes when the total indoor terminal unit heating
       \\note capacity is greater than the Rated Total Heating Capacity defined in this object.
       \\note If this field is left blank, the model assumes total indoor terminal unit heating
       \\note capacity is equal to the Rated Total Heating Capacity defined above.
  A23, \\field Heating Part-Load Fraction Correlation Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\note This curve defines the cycling losses when the heat pump compressor cycles on and off
       \\note below the Minimum Heat Pump Part-Load Ratio specified in the following field.
  N10, \\field Minimum Heat Pump Part-Load Ratio
       \\type real
       \\note Enter the minimum heat pump part-load ratio (PLR). When the cooling or heating PLR is
       \\note below this value, the heat pump compressor will cycle to meet the cooling or heating
       \\note demand.
       \\default 0.15
  A24, \\field Zone Name for Master Thermostat Location
       \\type object-list
       \\object-list ZoneNames
       \\note Enter the name of the zone where the master thermostat is located.
  A25, \\field Master Thermostat Priority Control Type
       \\type choice
       \\key LoadPriority
       \\key ZonePriority
       \\key ThermostatOffsetPriority
       \\key MasterThermostatPriority
       \\key Scheduled
       \\default MasterThermostatPriority
       \\note Choose a thermostat control logic scheme. If these control types fail to control zone
       \\note temperature within a reasonable limit, consider using multiple VRF systems
  A26, \\field Thermostat Priority Schedule Name
       \\type alpha
       \\object-list ScheduleNames
       \\note this field is required if Master Thermostat Priority Control Type is Scheduled.
       \\note Schedule values of 0 denote cooling, 1 for heating, and all other values disable the system.
  A27, \\field Zone Terminal Unit List Name
       \\required-field
       \\type object-list
       \\object-list ZoneTerminalUnitListNames
       \\note Enter the name of a ZoneTerminalUnitList. This list connects zone terminal units to this
       \\note heat pump.
  A28, \\field Heat Pump Waste Heat Recovery
       \\type choice
       \\key No
       \\key Yes
       \\default No
       \\note This field is reserved for future use. The only valid choice is No.
  N11, \\field Equivalent Piping Length used for Piping Correction Factor in Cooling Mode
       \\type real
       \\units m
       \\note Enter the equivalent length of the farthest terminal unit from the condenser
  N12, \\field Vertical Height used for Piping Correction Factor
       \\type real
       \\units m
       \\note Enter the height difference between the highest and lowest terminal unit
  A29, \\field Piping Correction Factor for Length in Cooling Mode Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\object-list BiQuadraticCurves
       \\note PCF = a0 + a1*L + a2*L^2 + a3*L^3 + a4*H
       \\note PCF = a0 + a1*L + a2*L^2 + a3*CR + a4*CR^2 + a5*(L)(CR)
       \\note where L = length and CR = combination ratio
       \\note specifies coefficients for a0, a1, a2, and a3 in the PCF equation
  N13, \\field Piping Correction Factor for Height in Cooling Mode Coefficient
       \\type real
       \\default 0
       \\note PCF = a0 + a1*L + a2*L^2 + a3*L^3 + a4*H
       \\note PCF = a0 + a1*L + a2*L^2 + a3*CR + a4*CR^2 + a5*(L)(CR) + a6*H
       \\note where L = length, H = height, and CR = combination ratio
       \\note specifies coefficient a4 (or a6 for biquadratic) in the PCF equation
  N14, \\field Equivalent Piping Length used for Piping Correction Factor in Heating Mode
       \\type real
       \\units m
       \\note Enter the equivalent length of the farthest terminal unit from the condenser
  A30, \\field Piping Correction Factor for Length in Heating Mode Curve Name
       \\type object-list
       \\object-list LinearCurves
       \\object-list QuadraticCubicCurves
       \\object-list BiQuadraticCurves
       \\note PCF = a0 + a1*L + a2*L^2 + a3*L^3 + a4*H
       \\note PCF = a0 + a1*L + a2*L^2 + a3*CR + a4*CR^2 + a5*(L)(CR) + a6*H
       \\note where L = length and CR = combination ratio
       \\note specifies coefficients for a0, a1, a2, and a3 (or a0-a5 for biquadratic) in the PCF equation
  N15, \\field Piping Correction Factor for Height in Heating Mode Coefficient
       \\type real
       \\default 0
       \\note PCF = a0 + a1*L + a2*L^2 + a3*L^3 + a4*H
       \\note PCF = a0 + a1*L + a2*L^2 + a3*CR + a4*CR^2 + a5*(L)(CR) + a6*H
       \\note where L = length, H = height, and CR = combination ratio
       \\note specifies coefficient a4 (or a6 for biquadratic) in the PCF equation
  N16, \\field Crankcase Heater Power per Compressor
       \\type real
       \\units W
       \\default 33.0
       \\note Enter the value of the resistive heater located in the compressor(s). This heater
       \\note is used to warm the refrigerant and oil when the compressor is off.
  N17, \\field Number of Compressors
       \\type integer
       \\default 2
       \\note Enter the total number of compressor. This input is used only for crankcase
       \\note heater calculations.
  N18, \\field Ratio of Compressor Size to Total Compressor Capacity
       \\type real
       \\default 0.5
       \\note Enter the ratio of the first stage compressor to total compressor capacity.
       \\note All other compressors are assumed to be equally sized. This inputs is used
       \\note only for crankcase heater calculations.
  N19, \\field Maximum Outdoor Dry-bulb Temperature for Crankcase Heater
       \\type real
       \\units C
       \\default 5.0
       \\note Enter the maximum outdoor temperature above which the crankcase heaters are disabled.
  A31, \\field Defrost Strategy
       \\type choice
       \\key ReverseCycle
       \\key Resistive
       \\default Resistive
       \\note Select a defrost strategy. Reverse cycle reverses the operating mode from heating to cooling
       \\note to melt frost formation on the condenser coil. The resistive strategy uses a resitive heater
       \\note to melt the frost.
  A32, \\field Defrost Control
       \\type choice
       \\key Timed
       \\key OnDemand
       \\default Timed
       \\note Choose a defrost control type. Either use a fixed Timed defrost period or select
       \\note OnDemand to defrost only when necessary.
  A33, \\field Defrost Energy Input Ratio Modifier Function of Temperature Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note A valid performance curve must be used if reversecycle defrost strategy is selected.
  N20, \\field Defrost Time Period Fraction
       \\type real
       \\minimum 0.0
       \\default 0.058333
       \\note Fraction of time in defrost mode.
       \\note Only applicable if timed defrost control is specified.
  N21, \\field Resistive Defrost Heater Capacity
       \\type real
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\units W
       \\note Enter the size of the resistive defrost heating element.
       \\note Only applicable if resistive defrost strategy is specified
       \\ip-units W
  N22, \\field Maximum Outdoor Dry-bulb Temperature for Defrost Operation
       \\type real
       \\units C
       \\default 5.0
       \\note Enter the maximum outdoor temperature above which the crankcase heaters are disabled.
  A34, \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key EvaporativelyCooled
       \\key WaterCooled
       \\default AirCooled
       \\note Select either an air cooled or evaporatively cooled condenser.
  A35, \\field Condenser Inlet Node Name
       \\type node
       \\note Choose an outside air node name or leave this field blank to use weather data.
       \\note If this field is blank, the Condenser Type is assumed to be AirCooled.
       \\note This input must be specified if Condenser Type = WaterCooled.
  A36, \\field Condenser Outlet Node Name
       \\type node
       \\note Enter a water outlet node name if Condenser Type = WaterCooled.
       \\note Leave this field blank if Condenser Type = Air or EvaporativelyCooled.
  N23, \\field Water Condenser Volume Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\note Only used when Condenser Type = WaterCooled.
  N24, \\field Evaporative Condenser Effectiveness
       \\type real
       \\units dimensionless
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.9
       \\note Enter the effectiveness of the evaporatively cooled condenser.
       \\note This field is only used when the Condenser Type = EvaporativelyCooled.
  N25, \\field Evaporative Condenser Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Used to calculate evaporative condenser water use.
       \\note This field is only used when the Condenser Type = EvaporativelyCooled.
  N26, \\field Evaporative Condenser Pump Rated Power Consumption
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\note Rated power consumed by the evaporative condenser's water pump.
       \\note This field is only used when the Condenser Type = EvaporativelyCooled.
  A37, \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
       \\note A separate storage tank may be used to supply an evaporatively cooled condenser.
  N27, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the DX coil is off.
  N28, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A38, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A39, \\field Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default Electricity
  N29, \\field Minimum Outdoor Temperature in Heat Recovery Mode
       \\type real
       \\units C
       \\note The minimum outdoor temperature below which heat
       \\note recovery mode will not operate.
  N30, \\field Maximum Outdoor Temperature in Heat Recovery Mode
       \\type real
       \\units C
       \\note The maximum outdoor temperature above which heat
       \\note recovery mode will not operate.
  A40, \\field Heat Recovery Cooling Capacity Modifier Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter the name of a performance curve which represents
       \\note the change in cooling capacity when heat recovery is active
       \\note A default constant of 0.9 is used if this input is blank
  N31, \\field Initial Heat Recovery Cooling Capacity Fraction
       \\type real
       \\default 0.5
       \\note Enter the fractional capacity available at the start
       \\note of heat recovery mode. The capacity exponentially approaches
       \\note the steady-state value according to the inputs for
       \\note Heat Recovery Cooling Capacity Modifier and Heat Recovery
       \\note Cooling Capacity Time Constant
  N32, \\field Heat Recovery Cooling Capacity Time Constant
       \\type real
       \\units hr
       \\default 0.15
       \\note Enter the time constant used to model the transition
       \\note from cooling only mode to heat recovery mode
  A41, \\field Heat Recovery Cooling Energy Modifier Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter the name of a performance curve which represents
       \\note the change in cooling energy when heat recovery is active
       \\note A default constant of 1.1 is used if this input is blank
  N33, \\field Initial Heat Recovery Cooling Energy Fraction
       \\type real
       \\default 1.0
       \\note Enter the fractional electric consumption rate at the start
       \\note of heat recovery mode. The electric consumption rate exponentially
       \\note approaches the steady-state value according to the inputs for
       \\note Heat Recovery Cooling Energy Modifier and Heat Recovery
       \\note Cooling Energy Time Constant
  N34, \\field Heat Recovery Cooling Energy Time Constant
       \\type real
       \\units hr
       \\default 0
       \\note Enter the time constant used to model the transition
       \\note from cooling only mode to heat recovery mode
  A42, \\field Heat Recovery Heating Capacity Modifier Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter the name of a performance curve which represents
       \\note the change in heating capacity when heat recovery is active
       \\note A default constant of 1.1 is used if this input is blank
  N35, \\field Initial Heat Recovery Heating Capacity Fraction
       \\type real
       \\default 1
       \\note Enter the fractional capacity available at the start
       \\note of heat recovery mode. The capacity exponentially approaches
       \\note the steady-state value according to the inputs for
       \\note Heat Recovery Heating Capacity Modifier and Heat Recovery
       \\note Heating Capacity Time Constant
  N36, \\field Heat Recovery Heating Capacity Time Constant
       \\type real
       \\units hr
       \\default 0.15
       \\note Enter the time constant used to model the transition
       \\note from cooling only mode to heat recovery mode
  A43, \\field Heat Recovery Heating Energy Modifier Curve Name
       \\type object-list
       \\object-list BiQuadraticCurves
       \\note Enter the name of a performance curve which represents
       \\note the change in heating electric consumption rate when heat recovery is active
       \\note A default constant of 1.1 is used if this input is blank
  N37, \\field Initial Heat Recovery Heating Energy Fraction
       \\type real
       \\default 1
       \\note Enter the fractional electric consumption rate at the start
       \\note of heat recovery mode. The electric consumption rate exponentially
       \\note approaches the steady-state value according to the inputs for
       \\note Heat Recovery Cooling Energy Modifier and Heat Recovery
       \\note Cooling Energy Time Constant
  N38; \\field Heat Recovery Heating Energy Time Constant
       \\type real
       \\units hr
       \\default 0
       \\note Enter the time constant used to model the transition
       \\note from cooling only mode to heat recovery mode

ZoneTerminalUnitList,
          \\memo List of variable refrigerant flow (VRF) terminal units served by a given VRF condensing
          \\memo unit. See ZoneHVAC:TerminalUnit:VariableRefrigerantFlow and
          \\memo AirConditioner:VariableRefrigerantFlow.
          \\min-fields 2
          \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
  A1 ,    \\field Zone Terminal Unit List Name
          \\begin-extensible
          \\required-field
          \\type alpha
          \\reference ZoneTerminalUnitListNames
  A2 ,    \\field Zone Terminal Unit Name 1
          \\required-field
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A3 ,    \\field Zone Terminal Unit Name 2
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A4 ,    \\field Zone Terminal Unit Name 3
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A5 ,    \\field Zone Terminal Unit Name 4
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A6 ,    \\field Zone Terminal Unit Name 5
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A7 ,    \\field Zone Terminal Unit Name 6
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A8 ,    \\field Zone Terminal Unit Name 7
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A9 ,    \\field Zone Terminal Unit Name 8
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A10,    \\field Zone Terminal Unit Name 9
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A11,    \\field Zone Terminal Unit Name 10
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A12,    \\field Zone Terminal Unit Name 11
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A13,    \\field Zone Terminal Unit Name 12
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A14,    \\field Zone Terminal Unit Name 13
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A15,    \\field Zone Terminal Unit Name 14
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A16,    \\field Zone Terminal Unit Name 15
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A17,    \\field Zone Terminal Unit Name 16
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A18,    \\field Zone Terminal Unit Name 17
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A19,    \\field Zone Terminal Unit Name 18
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A20,    \\field Zone Terminal Unit Name 19
          \\type object-list
          \\object-list ZoneTerminalUnitNames
  A21;    \\field Zone Terminal Unit Name 20
          \\type object-list
          \\object-list ZoneTerminalUnitNames

\\group Controllers
!*****************Controller****************************
! A Controller mimics the function of an actual physical controller, though it
! is somewhat more limited.  It can sense one node variable, compare it with
! its setpoint, and determine the desired value for another node variable.
!
! Rule: A Controller can not span a loop manager boundary (but a Setpoint
! Manager can). Thus in an air loop the sensed node and the controlled
! device must be in the air loop.  This means that a single zone system can
! not be modeled with a simple controller sensing zone temperature and
! controlling coil water flow.  Instead this must be modeled as a Setpoint
! Manager sensing a zone temperature and resetting the supply air temperature
! setpoint each timestep.  This is artificial but should simplify the control
! modelling task.
!
!
Controller:WaterCoil,
       \\memo Controller for a water coil which is located directly in an air loop branch or
       \\memo outdoor air equipment list. Controls the coil water flow to meet the specified
       \\memo leaving air setpoint(s). Used with Coil:Heating:Water, Coil:Cooling:Water,
       \\memo Coil:Cooling:Water:DetailedGeometry, and
       \\memo CoilSystem:Cooling:Water:HeatexchangerAssisted.
  \\min-fields 9
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference AirLoopControllers
  A2 , \\field Control Variable
       \\required-field
       \\type choice
       \\key Temperature
       \\key HumidityRatio
       \\key TemperatureAndHumidityRatio
       \\note keys HumidityRatio or TemperatureAndHumidityRatio
       \\note requires a ZoneControl:Humidistat object along
       \\note with SetpointManager:SingleZone:Humidity:Maximum,
       \\note SetpointManager:MultiZone:MaximumHumidity:Average, or
       \\note SetpointManager:Multizone:Humidity:Maximum object
  A3 , \\field Action
       \\type choice
       \\key Normal
       \\key Reverse
       \\note Leave blank to have this automatically selected from coil type.
       \\note Chilled water coils should be reverse action
       \\note Hot water coils should be normal action
  A4 , \\field Actuator Variable
       \\required-field
       \\type choice
       \\key Flow
  A5 , \\field Sensor Node Name
       \\required-field
       \\type node
  A6 , \\field Actuator Node Name
       \\required-field
       \\type node
  N1 , \\field Controller Convergence Tolerance
       \\units deltaC
       \\type real
       \\default autosize
       \\autosizable
  N2 , \\field Maximum Actuated Flow
       \\type real
       \\units m3/s
       \\autosizable
  N3 ; \\field Minimum Actuated Flow
       \\type real
       \\default 0.0000001
       \\units m3/s

Controller:OutdoorAir,
       \\memo Controller to set the outdoor air flow rate for an air loop. Control options include
       \\memo fixed, proportional, scheduled, economizer, and demand-controlled ventilation.
  \\min-fields 16
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference AirLoopControllers
  A2 , \\field Relief Air Outlet Node Name
       \\type node
       \\required-field
  A3 , \\field Return Air Node Name
       \\type node
       \\required-field
  A4 , \\field Mixed Air Node Name
       \\type node
       \\required-field
  A5 , \\field Actuator Node Name
       \\type node
       \\required-field
       \\note Outdoor air inlet node entering the first pre-treat component if any
  N1 , \\field Minimum Outdoor Air Flow Rate
       \\type real
       \\required-field
       \\units m3/s
       \\autosizable
  N2 , \\field Maximum Outdoor Air Flow Rate
       \\type real
       \\required-field
       \\units m3/s
       \\autosizable
  A6 , \\field Economizer Control Type
       \\type choice
       \\key FixedDryBulb
       \\key FixedEnthalpy
       \\key DifferentialDryBulb
       \\key DifferentialEnthalpy
       \\key FixedDewPointAndDryBulb
       \\key ElectronicEnthalpy
       \\key DifferentialDryBulbAndEnthalpy
       \\key NoEconomizer
       \\default NoEconomizer
  A7 , \\field Economizer Control Action Type
       \\type choice
       \\key ModulateFlow
       \\key MinimumFlowWithBypass
       \\default ModulateFlow
  N3 , \\field Economizer Maximum Limit Dry-Bulb Temperature
       \\type real
       \\units C
       \\note Enter the maximum outdoor dry-bulb temperature limit for FixedDryBulb
       \\note economizer control type. No input or blank input means this limit is
       \\note not operative. Limit is applied regardless of economizer control type.
  N4 , \\field Economizer Maximum Limit Enthalpy
       \\type real
       \\units J/kg
       \\note Enter the maximum outdoor enthalpy limit for FixedEnthalpy economizer control type.
       \\note No input or blank input means this limit is not operative
       \\note Limit is applied regardless of economizer control type.
  N5 , \\field Economizer Maximum Limit Dewpoint Temperature
       \\type real
       \\units C
       \\note Enter the maximum outdoor dewpoint temperature limit for FixedDewPointAndDryBulb
       \\note economizer control type. No input or blank input means this limit is not operative.
       \\note Limit is applied regardless of economizer control type.
  A8 , \\field Electronic Enthalpy Limit Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Enter the name of a quadratic or cubic curve which defines the maximum outdoor
       \\note humidity ratio (function of outdoor dry-bulb temperature) for ElectronicEnthalpy
       \\note economizer control type. No input or blank input means this limit is not operative
       \\note Limit is applied regardless of economizer control type.
  N6 , \\field Economizer Minimum Limit Dry-Bulb Temperature
       \\type real
       \\units C
       \\note Enter the minimum outdoor dry-bulb temperature limit for economizer control.
       \\note No input or blank input means this limit is not operative
       \\note Limit is applied regardless of economizer control type.
  A9 , \\field Lockout Type
       \\type choice
       \\key NoLockout
       \\key LockoutWithHeating
       \\key LockoutWithCompressor
       \\default NoLockout
  A10, \\field Minimum Limit Type
       \\type choice
       \\key FixedMinimum
       \\key ProportionalMinimum
       \\default ProportionalMinimum
  A11, \\field Minimum Outdoor Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values multiply the minimum outdoor air flow rate
  A12, \\field Minimum Fraction of Outdoor Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note schedule values multiply the design/mixed air flow rate
  A13, \\field Maximum Fraction of Outdoor Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note schedule values multiply the design/mixed air flow rate
  A14, \\field Mechanical Ventilation Controller Name
       \\note Enter the name of a Controller:MechanicalVentilation object.
       \\type object-list
       \\object-list ControllerMechanicalVentNames
       \\note Optional field for defining outdoor ventilation air based on flow rate per unit floor
       \\note area and flow rate per person. Simplified method of demand-controlled ventilation.
  A15, \\field Time of Day Economizer Control Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Optional schedule to simulate "push-button" type economizer control.
       \\note Schedule values greater than 0 indicate time-of-day economizer control is enabled.
       \\note Economizer control may be used with or without the high humidity control option.
       \\note When used together, high humidity control has priority over economizer control.
       \\note If the field Economizer Control Type = NoEconomizer, then this option is disabled.
  A16, \\field High Humidity Control
       \\type choice
       \\key Yes
       \\key No
       \\default No
       \\note Optional field to enable modified outdoor air flow rates based on zone relative humidity.
       \\note Select Yes to modify outdoor air flow rate based on a zone humidistat.
       \\note Select No to disable this feature.
       \\note If the field Economizer Control Type = NoEconomizer, then this option is disabled.
  A17, \\field Humidistat Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\note Enter the name of the zone where the humidistat is located.
       \\note This field is only used when the field High Humidity Control = Yes.
  N7 , \\field High Humidity Outdoor Air Flow Ratio
       \\type real
       \\minimum> 0.0
       \\default 1.0
       \\note Enter the ratio of outdoor air to the maximum outdoor air flow rate when modified air
       \\note flow rates are active based on high indoor humidity.
       \\note The minimum value must be greater than 0.
       \\note This field is only used when the field High Humidity Control = Yes.
  A18, \\field Control High Indoor Humidity Based on Outdoor Humidity Ratio
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\note If No is selected, the outdoor air flow rate is modified any time indoor relative
       \\note humidity is above the humidistat setpoint. If Yes is selected, the outdoor air
       \\note flow rate is modified any time the indoor relative humidity is above the humidistat
       \\note setpoint and the outdoor humidity ratio is less than the indoor humidity ratio.
       \\note This field is only used when the field High Humidity Control = Yes.
  A19; \\field Heat Recovery Bypass Control Type
       \\type Choice
       \\key BypassWhenWithinEconomizerLimits
       \\key BypassWhenOAFlowGreaterThanMinimum
       \\default BypassWhenWithinEconomizerLimits
       \\note BypassWhenWithinEconomizerLimits specifies that heat recovery
       \\note is active only when the economizer is off because conditions
       \\note are outside the economizer control limits
       \\note BypassWhenOAFlowGreaterThanMinimum specifies enhanced economizer
       \\note controls to allow heat recovery when economizer is active
       \\note (within limits) but the outdoor air flow rate is at the minimum.

Controller:MechanicalVentilation,
        \\memo This object is used in conjuction with Controller:OutdoorAir to specify outdoor
        \\memo ventilation air based on outdoor air specified in the DesignSpecification:OutdoorAir object
        \\memo The Controller:OutdoorAir object is associated with a specific air loop, so the
        \\memo outdoor air flow rates specified in Controller:MechanicalVentilation correspond to the zones
        \\memo attached to that specific air loop.
        \\min-fields 8
        \\extensible:3 - repeat last three fields, remembering to remove ; from "inner" fields.
        \\memo Duplicate groups of Zone name, Design Specification Outdoor Air Object Name,
        \\memo and Design Specification Zone Air Distribution Object Name to increase allowable number of entries
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference ControllerMechanicalVentNames
   A2,  \\field Availability Schedule Name
        \\note If this field is blank, the controller uses the values from the associated Controller:OutdoorAir.
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values greater than 0 indicate mechanical ventilation is enabled
   A3,  \\field Demand Controlled Ventilation
        \\type choice
        \\key Yes
        \\key No
        \\default No
   A4,  \\field System Outdoor Air Method
        \\type choice
        \\key ZoneSum
        \\key VentilationRateProcedure
        \\key IndoorAirQualityProcedure
        \\key ProportionalControl
        \\key IndoorAirQualityProcedureGenericContaminant
        \\default VentilationRateProcedure
   N1,  \\field Zone Maximum Outdoor Air Fraction
        \\type real
        \\default 1.0
        \\minimum> 0.0
        \\units dimensionless
   A5,  \\field Zone 1 Name
        \\begin-extensible
        \\required-field
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A6,  \\field Design Specification Outdoor Air Object Name 1
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A7,  \\field Design Specification Zone Air Distribution Object Name 1
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A8,  \\field Zone 2 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A9,  \\field Design Specification Outdoor Air Object Name 2
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A10, \\field Design Specification Zone Air Distribution Object Name 2
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A11, \\field Zone 3 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A12, \\field Design Specification Outdoor Air Object Name 3
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A13, \\field Design Specification Zone Air Distribution Object Name 3
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A14, \\field Zone 4 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A15, \\field Design Specification Outdoor Air Object Name 4
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A16, \\field Design Specification Zone Air Distribution Object Name 4
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A17, \\field Zone 5 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A18, \\field Design Specification Outdoor Air Object Name 5
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A19, \\field Design Specification Zone Air Distribution Object Name 5
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A20, \\field Zone 6 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A21, \\field Design Specification Outdoor Air Object Name 6
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A22, \\field Design Specification Zone Air Distribution Object Name 6
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A23, \\field Zone 7 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A24, \\field Design Specification Outdoor Air Object Name 7
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A25, \\field Design Specification Zone Air Distribution Object Name 7
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A26, \\field Zone 8 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A27, \\field Design Specification Outdoor Air Object Name 8
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A28, \\field Design Specification Zone Air Distribution Object Name 8
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A29, \\field Zone 9 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A30, \\field Design Specification Outdoor Air Object Name 9
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A31, \\field Design Specification Zone Air Distribution Object Name 9
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A32, \\field Zone 10 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A33, \\field Design Specification Outdoor Air Object Name 10
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A34, \\field Design Specification Zone Air Distribution Object Name 10
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A35, \\field Zone 11 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A36, \\field Design Specification Outdoor Air Object Name 11
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A37, \\field Design Specification Zone Air Distribution Object Name 11
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A38, \\field Zone 12 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A39, \\field Design Specification Outdoor Air Object Name 12
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A40, \\field Design Specification Zone Air Distribution Object Name 12
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A41, \\field Zone 13 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A42, \\field Design Specification Outdoor Air Object Name 13
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A43, \\field Design Specification Zone Air Distribution Object Name 13
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A44, \\field Zone 14 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A45, \\field Design Specification Outdoor Air Object Name 14
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A46, \\field Design Specification Zone Air Distribution Object Name 14
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A47, \\field Zone 15 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A48, \\field Design Specification Outdoor Air Object Name 15
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A49, \\field Design Specification Zone Air Distribution Object Name 15
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A50, \\field Zone 16 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A51, \\field Design Specification Outdoor Air Object Name 16
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A52, \\field Design Specification Zone Air Distribution Object Name 16
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A53, \\field Zone 17 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A54, \\field Design Specification Outdoor Air Object Name 17
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A55, \\field Design Specification Zone Air Distribution Object Name 17
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A56, \\field Zone 18 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A57, \\field Design Specification Outdoor Air Object Name 18
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A58, \\field Design Specification Zone Air Distribution Object Name 18
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A59, \\field Zone 19 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A60, \\field Design Specification Outdoor Air Object Name 19
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A61, \\field Design Specification Zone Air Distribution Object Name 19
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A62, \\field Zone 20 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A63, \\field Design Specification Outdoor Air Object Name 20
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A64, \\field Design Specification Zone Air Distribution Object Name 20
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A65, \\field Zone 21 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A66, \\field Design Specification Outdoor Air Object Name 21
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A67, \\field Design Specification Zone Air Distribution Object Name 21
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A68, \\field Zone 22 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A69, \\field Design Specification Outdoor Air Object Name 22
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A70, \\field Design Specification Zone Air Distribution Object Name 22
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A71, \\field Zone 23 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A72, \\field Design Specification Outdoor Air Object Name 23
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A73, \\field Design Specification Zone Air Distribution Object Name 23
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A74, \\field Zone 24 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A75, \\field Design Specification Outdoor Air Object Name 24
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A76, \\field Design Specification Zone Air Distribution Object Name 24
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A77, \\field Zone 25 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A78, \\field Design Specification Outdoor Air Object Name 25
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A79, \\field Design Specification Zone Air Distribution Object Name 25
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A80, \\field Zone 26 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A81, \\field Design Specification Outdoor Air Object Name 26
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A82, \\field Design Specification Zone Air Distribution Object Name 26
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A83, \\field Zone 27 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A84, \\field Design Specification Outdoor Air Object Name 27
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A85, \\field Design Specification Zone Air Distribution Object Name 27
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A86, \\field Zone 28 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A87, \\field Design Specification Outdoor Air Object Name 28
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A88, \\field Design Specification Zone Air Distribution Object Name 28
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A89, \\field Zone 29 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A90, \\field Design Specification Outdoor Air Object Name 29
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A91, \\field Design Specification Zone Air Distribution Object Name 29
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A92, \\field Zone 30 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A93, \\field Design Specification Outdoor Air Object Name 30
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A94, \\field Design Specification Zone Air Distribution Object Name 30
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A95, \\field Zone 31 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A96, \\field Design Specification Outdoor Air Object Name 31
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A97, \\field Design Specification Zone Air Distribution Object Name 31
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A98, \\field Zone 32 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A99, \\field Design Specification Outdoor Air Object Name 32
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A100,\\field Design Specification Zone Air Distribution Object Name 32
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A101,\\field Zone 33 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A102,\\field Design Specification Outdoor Air Object Name 33
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A103,\\field Design Specification Zone Air Distribution Object Name 33
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A104,\\field Zone 34 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A105,\\field Design Specification Outdoor Air Object Name 34
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A106,\\field Design Specification Zone Air Distribution Object Name 34
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A107,\\field Zone 35 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A108,\\field Design Specification Outdoor Air Object Name 35
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A109,\\field Design Specification Zone Air Distribution Object Name 35
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A110,\\field Zone 36 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A111,\\field Design Specification Outdoor Air Object Name 36
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A112,\\field Design Specification Zone Air Distribution Object Name 36
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A113,\\field Zone 37 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A114,\\field Design Specification Outdoor Air Object Name 37
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A115,\\field Design Specification Zone Air Distribution Object Name 37
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A116,\\field Zone 38 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A117,\\field Design Specification Outdoor Air Object Name 38
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A118,\\field Design Specification Zone Air Distribution Object Name 38
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A119,\\field Zone 39 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A120,\\field Design Specification Outdoor Air Object Name 39
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A121,\\field Design Specification Zone Air Distribution Object Name 39
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A122,\\field Zone 40 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A123,\\field Design Specification Outdoor Air Object Name 40
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A124,\\field Design Specification Zone Air Distribution Object Name 40
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A125,\\field Zone 41 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A126,\\field Design Specification Outdoor Air Object Name 41
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A127,\\field Design Specification Zone Air Distribution Object Name 41
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A128,\\field Zone 42 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A129,\\field Design Specification Outdoor Air Object Name 42
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A130,\\field Design Specification Zone Air Distribution Object Name 42
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A131,\\field Zone 43 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A132,\\field Design Specification Outdoor Air Object Name 43
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A133,\\field Design Specification Zone Air Distribution Object Name 43
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A134,\\field Zone 44 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A135,\\field Design Specification Outdoor Air Object Name 44
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A136,\\field Design Specification Zone Air Distribution Object Name 44
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A137,\\field Zone 45 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A138,\\field Design Specification Outdoor Air Object Name 45
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A139,\\field Design Specification Zone Air Distribution Object Name 45
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A140,\\field Zone 46 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A141,\\field Design Specification Outdoor Air Object Name 46
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A142,\\field Design Specification Zone Air Distribution Object Name 46
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A143,\\field Zone 47 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A144,\\field Design Specification Outdoor Air Object Name 47
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A145,\\field Design Specification Zone Air Distribution Object Name 47
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A146,\\field Zone 48 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A147,\\field Design Specification Outdoor Air Object Name 48
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A148,\\field Design Specification Zone Air Distribution Object Name 48
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A149,\\field Zone 49 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A150,\\field Design Specification Outdoor Air Object Name 49
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A151,\\field Design Specification Zone Air Distribution Object Name 49
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames
   A152,\\field Zone 50 Name
        \\type object-list
        \\object-list ZoneAndZoneListNames
        \\note A zone name or a zone list name may be used here
   A153,\\field Design Specification Outdoor Air Object Name 50
        \\type object-list
        \\object-list DesignSpecificationOutdoorAirNames
   A154;\\field Design Specification Zone Air Distribution Object Name 50
        \\type object-list
        \\object-list DesignSpecificationZoneAirDistributionNames

AirLoopHVAC:ControllerList,
       \\memo List controllers in order of control sequence
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ControllerLists
  A2 , \\field Controller 1 Object Type
       \\required-field
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A3 , \\field Controller 1 Name
       \\required-field
       \\type object-list
       \\object-list AirLoopControllers
  A4 , \\field  Controller 2 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A5 , \\field Controller 2 Name
       \\type object-list
       \\object-list AirLoopControllers
  A6 , \\field  Controller 3 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A7 , \\field Controller 3 Name
       \\type object-list
       \\object-list AirLoopControllers
  A8 , \\field  Controller 4 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A9 , \\field Controller 4 Name
       \\type object-list
       \\object-list AirLoopControllers
  A10, \\field  Controller 5 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A11, \\field Controller 5 Name
       \\type object-list
       \\object-list AirLoopControllers
  A12, \\field  Controller 6 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A13, \\field Controller 6 Name
       \\type object-list
       \\object-list AirLoopControllers
  A14, \\field  Controller 7 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A15, \\field Controller 7 Name
       \\type object-list
       \\object-list AirLoopControllers
  A16, \\field  Controller 8 Object Type
       \\type choice
       \\key Controller:WaterCoil
       \\key Controller:OutdoorAir
  A17; \\field Controller 8 Name
       \\type object-list
       \\object-list AirLoopControllers

 !*****************AIR LOOP SPECIFICATION*********************
 ! Air system heating and cooling loops are specified in the following configuration statements.
 ! The AirLoopHVAC input objects connect return and supply air paths from
 ! the zone equipment to supply components on an air loop.
 ! There can only be one return air path per air loop
 ! and only up to 3 supply air streams (assuming a 3-deck multizone is the
 ! most complex supply air configuration) per air system.

\\group Air Distribution

AirLoopHVAC,
       \\min-fields 10
       \\memo Defines a central forced air system.
   A1, \\field Name
       \\required-field
       \\type alpha
       \\reference AirPrimaryLoops
   A2, \\field Controller List Name
       \\note Enter the name of an AirLoopHVAC:ControllerList object.
       \\type object-list
       \\object-list ControllerLists
   A3, \\field Availability Manager List Name
       \\note Enter the name of an AvailabilityManagerAssignmentList object.
       \\type object-list
       \\object-list SystemAvailabilityManagerLists
   N1, \\field Design Supply Air Flow Rate
       \\default 0
       \\units m3/s
       \\autosizable
   A4, \\field Branch List Name
       \\note Name of a BranchList containing all the branches in this air loop
       \\required-field
       \\type object-list
       \\object-list BranchLists
   A5, \\field Connector List Name
       \\note Name of a ConnectorList containing all the splitters and mixers in the loop
       \\type object-list
       \\object-list ConnectorLists
   A6, \\field Supply Side Inlet Node Name
       \\note Name of inlet node where return air enters the supply side of the air loop
       \\required-field
       \\type node
   A7, \\field Demand Side Outlet Node Name
       \\note Name of outlet node where return air leaves the demand side and enters the supply side.
       \\required-field
       \\type node
   A8, \\field Demand Side Inlet Node Names
       \\note Name of a Node or NodeList containing the inlet node(s) supplying air to zone equipment.
       \\required-field
       \\type node
   A9; \\field Supply Side Outlet Node Names
       \\note Name of a Node or NodeList containing the outlet node(s) supplying air to the demand side.
       \\required-field
       \\type node

AirLoopHVAC:OutdoorAirSystem:EquipmentList,
       \\memo List equipment in simulation order
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AirLoopOAEquipmentLists
  A2 , \\field Component 1 Object Type
       \\required-field
  A3 , \\field Component 1 Name
       \\required-field
  A4 , \\field Component 2 Object Type
  A5 , \\field Component 2 Name
  A6 , \\field Component 3 Object Type
  A7 , \\field Component 3 Name
  A8 , \\field Component 4 Object Type
  A9 , \\field Component 4 Name
  A10, \\field Component 5 Object Type
  A11, \\field Component 5 Name
  A12, \\field Component 6 Object Type
  A13, \\field Component 6 Name
  A14, \\field Component 7 Object Type
  A15, \\field Component 7 Name
  A16, \\field Component 8 Object Type
  A17, \\field Component 8 Name
  A18, \\field Component 9 Object Type
  A19; \\field Component 9 Name

AirLoopHVAC:OutdoorAirSystem,
       \\memo Outdoor air subsystem for an AirLoopHVAC. Includes an outdoor air mixing box and
       \\memo optional outdoor air conditioning equipment such as heat recovery, preheat, and precool
       \\memo coils. From the perspective of the primary air loop the outdoor air system is treated
       \\memo as a single component.
       \\min-fields 3
   A1, \\field Name
       \\required-field
       \\type alpha
   A2, \\field Controller List Name
       \\note Enter the name of an AirLoopHVAC:ControllerList object.
       \\required-field
       \\type object-list
       \\object-list ControllerLists
   A3, \\field Outdoor Air Equipment List Name
       \\note Enter the name of an AirLoopHVAC:OutdoorAirSystem:EquipmentList object.
       \\required-field
       \\type object-list
       \\object-list AirLoopOAEquipmentLists
   A4; \\field Availability Manager List Name
       \\note Enter the name of an AvailabilityManagerAssignmentList object.
       \\type object-list
       \\object-list SystemAvailabilityManagerLists

OutdoorAir:Mixer,
      \\memo Outdoor air mixer. Node names cannot be duplicated within a single OutdoorAir:Mixer
      \\memo object or across all outdoor air mixers.
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference OutdoorAirMixers
  A2, \\field Mixed Air Node Name
      \\note Name of Mixed Air Node
      \\required-field
      \\type node
  A3, \\field Outdoor Air Stream Node Name
      \\note Name of Outdoor Air Stream Node
      \\required-field
      \\type node
  A4, \\field Relief Air Stream Node Name
      \\note Name of Relief Air Stream Node
      \\required-field
      \\type node
  A5; \\field Return Air Stream Node Name
      \\note Name of Return Air Stream Node
      \\required-field
      \\type node

AirLoopHVAC:ZoneSplitter,
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Split one air stream into N outlet streams (currently 500 per air loop, but extensible).  Node names
       \\memo cannot be duplicated within a single zone splitter (AirLoopHVAC:ZoneSplitter) list.
   A1, \\field Name
       \\required-field
       \\reference ZoneSplitters
       \\reference SupplyPathComponentNames
   A2, \\field Inlet Node Name
       \\required-field
       \\type node
   A3, \\field Outlet 1 Node Name
       \\begin-extensible
       \\required-field
       \\type node
   A4, \\field Outlet 2 Node Name
       \\type node
   A5, \\field Outlet 3 Node Name
       \\type node
   A6, \\field Outlet 4 Node Name
       \\type node
   A7, \\field Outlet 5 Node Name
       \\type node
   A8, \\field Outlet 6 Node Name
       \\type node
   A9, \\field Outlet 7 Node Name
       \\type node
   A10, \\field Outlet 8 Node Name
        \\type node
   A11, \\field Outlet 9 Node Name
        \\type node
   A12, \\field Outlet 10 Node Name
        \\type node
   A13, \\field Outlet 11 Node Name
        \\type node
   A14, \\field Outlet 12 Node Name
        \\type node
   A15, \\field Outlet 13 Node Name
        \\type node
   A16, \\field Outlet 14 Node Name
        \\type node
   A17, \\field Outlet 15 Node Name
        \\type node
   A18, \\field Outlet 16 Node Name
        \\type node
   A19, \\field Outlet 17 Node Name
        \\type node
   A20, \\field Outlet 18 Node Name
        \\type node
   A21, \\field Outlet 19 Node Name
        \\type node
   A22, \\field Outlet 20 Node Name
        \\type node
   A23, \\field Outlet 21 Node Name
        \\type node
   A24, \\field Outlet 22 Node Name
        \\type node
   A25, \\field Outlet 23 Node Name
        \\type node
   A26, \\field Outlet 24 Node Name
        \\type node
   A27, \\field Outlet 25 Node Name
        \\type node
   A28, \\field Outlet 26 Node Name
        \\type node
   A29, \\field Outlet 27 Node Name
        \\type node
   A30, \\field Outlet 28 Node Name
        \\type node
   A31, \\field Outlet 29 Node Name
        \\type node
   A32, \\field Outlet 30 Node Name
        \\type node
   A33, \\field Outlet 31 Node Name
        \\type node
   A34, \\field Outlet 32 Node Name
        \\type node
   A35, \\field Outlet 33 Node Name
        \\type node
   A36, \\field Outlet 34 Node Name
        \\type node
   A37, \\field Outlet 35 Node Name
        \\type node
   A38, \\field Outlet 36 Node Name
        \\type node
   A39, \\field Outlet 37 Node Name
        \\type node
   A40, \\field Outlet 38 Node Name
        \\type node
   A41, \\field Outlet 39 Node Name
        \\type node
   A42, \\field Outlet 40 Node Name
        \\type node
   A43, \\field Outlet 41 Node Name
        \\type node
   A44, \\field Outlet 42 Node Name
        \\type node
   A45, \\field Outlet 43 Node Name
        \\type node
   A46, \\field Outlet 44 Node Name
        \\type node
   A47, \\field Outlet 45 Node Name
        \\type node
   A48, \\field Outlet 46 Node Name
        \\type node
   A49, \\field Outlet 47 Node Name
        \\type node
   A50, \\field Outlet 48 Node Name
        \\type node
   A51, \\field Outlet 49 Node Name
        \\type node
   A52, \\field Outlet 50 Node Name
        \\type node
   A53, \\field Outlet 51 Node Name
        \\type node
   A54, \\field Outlet 52 Node Name
        \\type node
   A55, \\field Outlet 53 Node Name
        \\type node
   A56, \\field Outlet 54 Node Name
        \\type node
   A57, \\field Outlet 55 Node Name
        \\type node
   A58, \\field Outlet 56 Node Name
        \\type node
   A59, \\field Outlet 57 Node Name
        \\type node
   A60, \\field Outlet 58 Node Name
        \\type node
   A61, \\field Outlet 59 Node Name
        \\type node
   A62, \\field Outlet 60 Node Name
        \\type node
   A63, \\field Outlet 61 Node Name
        \\type node
   A64, \\field Outlet 62 Node Name
        \\type node
   A65, \\field Outlet 63 Node Name
        \\type node
   A66, \\field Outlet 64 Node Name
        \\type node
   A67, \\field Outlet 65 Node Name
        \\type node
   A68, \\field Outlet 66 Node Name
        \\type node
   A69, \\field Outlet 67 Node Name
        \\type node
   A70, \\field Outlet 68 Node Name
        \\type node
   A71, \\field Outlet 69 Node Name
        \\type node
   A72, \\field Outlet 70 Node Name
        \\type node
   A73, \\field Outlet 71 Node Name
        \\type node
   A74, \\field Outlet 72 Node Name
        \\type node
   A75, \\field Outlet 73 Node Name
        \\type node
   A76, \\field Outlet 74 Node Name
        \\type node
   A77, \\field Outlet 75 Node Name
        \\type node
   A78, \\field Outlet 76 Node Name
        \\type node
   A79, \\field Outlet 77 Node Name
        \\type node
   A80, \\field Outlet 78 Node Name
        \\type node
   A81, \\field Outlet 79 Node Name
        \\type node
   A82, \\field Outlet 80 Node Name
        \\type node
   A83, \\field Outlet 81 Node Name
        \\type node
   A84, \\field Outlet 82 Node Name
        \\type node
   A85, \\field Outlet 83 Node Name
        \\type node
   A86, \\field Outlet 84 Node Name
        \\type node
   A87, \\field Outlet 85 Node Name
        \\type node
   A88, \\field Outlet 86 Node Name
        \\type node
   A89, \\field Outlet 87 Node Name
        \\type node
   A90, \\field Outlet 88 Node Name
        \\type node
   A91, \\field Outlet 89 Node Name
        \\type node
   A92, \\field Outlet 90 Node Name
        \\type node
   A93, \\field Outlet 91 Node Name
        \\type node
   A94, \\field Outlet 92 Node Name
        \\type node
   A95, \\field Outlet 93 Node Name
        \\type node
   A96, \\field Outlet 94 Node Name
        \\type node
   A97, \\field Outlet 95 Node Name
        \\type node
   A98, \\field Outlet 96 Node Name
        \\type node
   A99, \\field Outlet 97 Node Name
        \\type node
   A100, \\field Outlet 98 Node Name
         \\type node
   A101, \\field Outlet 99 Node Name
         \\type node
   A102, \\field Outlet 100 Node Name
         \\type node
   A103,A104,A105,A106,A107,A108,A109,A110,A111,A112, \\note fields as indicated
   A113,A114,A115,A116,A117,A118,A119,A120,A121,A122, \\note fields as indicated
   A123,A124,A125,A126,A127,A128,A129,A130,A131,A132, \\note fields as indicated
   A133,A134,A135,A136,A137,A138,A139,A140,A141,A142, \\note fields as indicated
   A143,A144,A145,A146,A147,A148,A149,A150,A151,A152, \\note fields as indicated
   A153,A154,A155,A156,A157,A158,A159,A160,A161,A162, \\note fields as indicated
   A163,A164,A165,A166,A167,A168,A169,A170,A171,A172, \\note fields as indicated
   A173,A174,A175,A176,A177,A178,A179,A180,A181,A182, \\note fields as indicated
   A183,A184,A185,A186,A187,A188,A189,A190,A191,A192, \\note fields as indicated
   A193,A194,A195,A196,A197,A198,A199,A200,A201,A202, \\note fields as indicated
   A203,A204,A205,A206,A207,A208,A209,A210,A211,A212, \\note fields as indicated
   A213,A214,A215,A216,A217,A218,A219,A220,A221,A222, \\note fields as indicated
   A223,A224,A225,A226,A227,A228,A229,A230,A231,A232, \\note fields as indicated
   A233,A234,A235,A236,A237,A238,A239,A240,A241,A242, \\note fields as indicated
   A243,A244,A245,A246,A247,A248,A249,A250,A251,A252, \\note fields as indicated
   A253,A254,A255,A256,A257,A258,A259,A260,A261,A262, \\note fields as indicated
   A263,A264,A265,A266,A267,A268,A269,A270,A271,A272, \\note fields as indicated
   A273,A274,A275,A276,A277,A278,A279,A280,A281,A282, \\note fields as indicated
   A283,A284,A285,A286,A287,A288,A289,A290,A291,A292, \\note fields as indicated
   A293,A294,A295,A296,A297,A298,A299,A300,A301,A302, \\note fields as indicated
   A303,A304,A305,A306,A307,A308,A309,A310,A311,A312, \\note fields as indicated
   A313,A314,A315,A316,A317,A318,A319,A320,A321,A322, \\note fields as indicated
   A323,A324,A325,A326,A327,A328,A329,A330,A331,A332, \\note fields as indicated
   A333,A334,A335,A336,A337,A338,A339,A340,A341,A342, \\note fields as indicated
   A343,A344,A345,A346,A347,A348,A349,A350,A351,A352, \\note fields as indicated
   A353,A354,A355,A356,A357,A358,A359,A360,A361,A362, \\note fields as indicated
   A363,A364,A365,A366,A367,A368,A369,A370,A371,A372, \\note fields as indicated
   A373,A374,A375,A376,A377,A378,A379,A380,A381,A382, \\note fields as indicated
   A383,A384,A385,A386,A387,A388,A389,A390,A391,A392, \\note fields as indicated
   A393,A394,A395,A396,A397,A398,A399,A400,A401,A402, \\note fields as indicated
   A403,A404,A405,A406,A407,A408,A409,A410,A411,A412, \\note fields as indicated
   A413,A414,A415,A416,A417,A418,A419,A420,A421,A422, \\note fields as indicated
   A423,A424,A425,A426,A427,A428,A429,A430,A431,A432, \\note fields as indicated
   A433,A434,A435,A436,A437,A438,A439,A440,A441,A442, \\note fields as indicated
   A443,A444,A445,A446,A447,A448,A449,A450,A451,A452, \\note fields as indicated
   A453,A454,A455,A456,A457,A458,A459,A460,A461,A462, \\note fields as indicated
   A463,A464,A465,A466,A467,A468,A469,A470,A471,A472, \\note fields as indicated
   A473,A474,A475,A476,A477,A478,A479,A480,A481,A482, \\note fields as indicated
   A483,A484,A485,A486,A487,A488,A489,A490,A491,A492, \\note fields as indicated
   A493,A494,A495,A496,A497,A498,A499,A500,A501,A502; \\note fields as indicated

AirLoopHVAC:SupplyPlenum,
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Connects 1 zone inlet air stream, through zone supply plenum, to one or more outlets.
       \\memo Node names cannot be duplicated within a single supply plenum list.
   A1, \\field Name
       \\required-field
       \\reference ZonePlenums
       \\reference SupplyPathComponentNames
   A2, \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
   A3, \\field Zone Node Name
       \\required-field
       \\type node
   A4, \\field Inlet Node Name
       \\required-field
       \\type node
   A5, \\field Outlet 1 Node Name
       \\begin-extensible
       \\required-field
       \\type node
   A6, \\field Outlet 2 Node Name
       \\type node
   A7, \\field Outlet 3 Node Name
       \\type node
   A8, \\field Outlet 4 Node Name
       \\type node
   A9, \\field Outlet 5 Node Name
       \\type node
   A10, \\field Outlet 6 Node Name
        \\type node
   A11, \\field Outlet 7 Node Name
        \\type node
   A12, \\field Outlet 8 Node Name
        \\type node
   A13, \\field Outlet 9 Node Name
        \\type node
   A14, \\field Outlet 10 Node Name
        \\type node
   A15, \\field Outlet 11 Node Name
        \\type node
   A16, \\field Outlet 12 Node Name
        \\type node
   A17, \\field Outlet 13 Node Name
        \\type node
   A18, \\field Outlet 14 Node Name
        \\type node
   A19, \\field Outlet 15 Node Name
        \\type node
   A20, \\field Outlet 16 Node Name
        \\type node
   A21, \\field Outlet 17 Node Name
        \\type node
   A22, \\field Outlet 18 Node Name
        \\type node
   A23, \\field Outlet 19 Node Name
        \\type node
   A24, \\field Outlet 20 Node Name
        \\type node
   A25, \\field Outlet 21 Node Name
        \\type node
   A26, \\field Outlet 22 Node Name
        \\type node
   A27, \\field Outlet 23 Node Name
        \\type node
   A28, \\field Outlet 24 Node Name
        \\type node
   A29, \\field Outlet 25 Node Name
        \\type node
   A30, \\field Outlet 26 Node Name
        \\type node
   A31, \\field Outlet 27 Node Name
        \\type node
   A32, \\field Outlet 28 Node Name
        \\type node
   A33, \\field Outlet 29 Node Name
        \\type node
   A34, \\field Outlet 30 Node Name
        \\type node
   A35, \\field Outlet 31 Node Name
        \\type node
   A36, \\field Outlet 32 Node Name
        \\type node
   A37, \\field Outlet 33 Node Name
        \\type node
   A38, \\field Outlet 34 Node Name
        \\type node
   A39, \\field Outlet 35 Node Name
        \\type node
   A40, \\field Outlet 36 Node Name
        \\type node
   A41, \\field Outlet 37 Node Name
        \\type node
   A42, \\field Outlet 38 Node Name
        \\type node
   A43, \\field Outlet 39 Node Name
        \\type node
   A44, \\field Outlet 40 Node Name
        \\type node
   A45, \\field Outlet 41 Node Name
        \\type node
   A46, \\field Outlet 42 Node Name
        \\type node
   A47, \\field Outlet 43 Node Name
        \\type node
   A48, \\field Outlet 44 Node Name
        \\type node
   A49, \\field Outlet 45 Node Name
        \\type node
   A50, \\field Outlet 46 Node Name
        \\type node
   A51, \\field Outlet 47 Node Name
        \\type node
   A52, \\field Outlet 48 Node Name
        \\type node
   A53, \\field Outlet 49 Node Name
        \\type node
   A54, \\field Outlet 50 Node Name
        \\type node
   A55, \\field Outlet 51 Node Name
        \\type node
   A56, \\field Outlet 52 Node Name
        \\type node
   A57, \\field Outlet 53 Node Name
        \\type node
   A58, \\field Outlet 54 Node Name
        \\type node
   A59, \\field Outlet 55 Node Name
        \\type node
   A60, \\field Outlet 56 Node Name
        \\type node
   A61, \\field Outlet 57 Node Name
        \\type node
   A62, \\field Outlet 58 Node Name
        \\type node
   A63, \\field Outlet 59 Node Name
        \\type node
   A64, \\field Outlet 60 Node Name
        \\type node
   A65, \\field Outlet 61 Node Name
        \\type node
   A66, \\field Outlet 62 Node Name
        \\type node
   A67, \\field Outlet 63 Node Name
        \\type node
   A68, \\field Outlet 64 Node Name
        \\type node
   A69, \\field Outlet 65 Node Name
        \\type node
   A70, \\field Outlet 66 Node Name
        \\type node
   A71, \\field Outlet 67 Node Name
        \\type node
   A72, \\field Outlet 68 Node Name
        \\type node
   A73, \\field Outlet 69 Node Name
        \\type node
   A74, \\field Outlet 70 Node Name
        \\type node
   A75, \\field Outlet 71 Node Name
        \\type node
   A76, \\field Outlet 72 Node Name
        \\type node
   A77, \\field Outlet 73 Node Name
        \\type node
   A78, \\field Outlet 74 Node Name
        \\type node
   A79, \\field Outlet 75 Node Name
        \\type node
   A80, \\field Outlet 76 Node Name
        \\type node
   A81, \\field Outlet 77 Node Name
        \\type node
   A82, \\field Outlet 78 Node Name
        \\type node
   A83, \\field Outlet 79 Node Name
        \\type node
   A84, \\field Outlet 80 Node Name
        \\type node
   A85, \\field Outlet 81 Node Name
        \\type node
   A86, \\field Outlet 82 Node Name
        \\type node
   A87, \\field Outlet 83 Node Name
        \\type node
   A88, \\field Outlet 84 Node Name
        \\type node
   A89, \\field Outlet 85 Node Name
        \\type node
   A90, \\field Outlet 86 Node Name
        \\type node
   A91, \\field Outlet 87 Node Name
        \\type node
   A92, \\field Outlet 88 Node Name
        \\type node
   A93, \\field Outlet 89 Node Name
        \\type node
   A94, \\field Outlet 90 Node Name
        \\type node
   A95, \\field Outlet 91 Node Name
        \\type node
   A96, \\field Outlet 92 Node Name
        \\type node
   A97, \\field Outlet 93 Node Name
        \\type node
   A98, \\field Outlet 94 Node Name
        \\type node
   A99, \\field Outlet 95 Node Name
         \\type node
   A100, \\field Outlet 96 Node Name
         \\type node
   A101, \\field Outlet 97 Node Name
         \\type node
   A102, \\field Outlet 98 Node Name
         \\type node
   A103, \\field Outlet 99 Node Name
         \\type node
   A104, \\field Outlet 100 Node Name
         \\type node
   A105,A106,A107,A108,A109,A110,A111,A112,A113,A114, \\note fields as indicated
   A115,A116,A117,A118,A119,A120,A121,A122,A123,A124, \\note fields as indicated
   A125,A126,A127,A128,A129,A130,A131,A132,A133,A134, \\note fields as indicated
   A135,A136,A137,A138,A139,A140,A141,A142,A143,A144, \\note fields as indicated
   A145,A146,A147,A148,A149,A150,A151,A152,A153,A154, \\note fields as indicated
   A155,A156,A157,A158,A159,A160,A161,A162,A163,A164, \\note fields as indicated
   A165,A166,A167,A168,A169,A170,A171,A172,A173,A174, \\note fields as indicated
   A175,A176,A177,A178,A179,A180,A181,A182,A183,A184, \\note fields as indicated
   A185,A186,A187,A188,A189,A190,A191,A192,A193,A194, \\note fields as indicated
   A195,A196,A197,A198,A199,A200,A201,A202,A203,A204, \\note fields as indicated
   A205,A206,A207,A208,A209,A210,A211,A212,A213,A214, \\note fields as indicated
   A215,A216,A217,A218,A219,A220,A221,A222,A223,A224, \\note fields as indicated
   A225,A226,A227,A228,A229,A230,A231,A232,A233,A234, \\note fields as indicated
   A235,A236,A237,A238,A239,A240,A241,A242,A243,A244, \\note fields as indicated
   A245,A246,A247,A248,A249,A250,A251,A252,A253,A254, \\note fields as indicated
   A255,A256,A257,A258,A259,A260,A261,A262,A263,A264, \\note fields as indicated
   A265,A266,A267,A268,A269,A270,A271,A272,A273,A274, \\note fields as indicated
   A275,A276,A277,A278,A279,A280,A281,A282,A283,A284, \\note fields as indicated
   A285,A286,A287,A288,A289,A290,A291,A292,A293,A294, \\note fields as indicated
   A295,A296,A297,A298,A299,A300,A301,A302,A303,A304, \\note fields as indicated
   A305,A306,A307,A308,A309,A310,A311,A312,A313,A314, \\note fields as indicated
   A315,A316,A317,A318,A319,A320,A321,A322,A323,A324, \\note fields as indicated
   A325,A326,A327,A328,A329,A330,A331,A332,A333,A334, \\note fields as indicated
   A335,A336,A337,A338,A339,A340,A341,A342,A343,A344, \\note fields as indicated
   A345,A346,A347,A348,A349,A350,A351,A352,A353,A354, \\note fields as indicated
   A355,A356,A357,A358,A359,A360,A361,A362,A363,A364, \\note fields as indicated
   A365,A366,A367,A368,A369,A370,A371,A372,A373,A374, \\note fields as indicated
   A375,A376,A377,A378,A379,A380,A381,A382,A383,A384, \\note fields as indicated
   A385,A386,A387,A388,A389,A390,A391,A392,A393,A394, \\note fields as indicated
   A395,A396,A397,A398,A399,A400,A401,A402,A403,A404, \\note fields as indicated
   A405,A406,A407,A408,A409,A410,A411,A412,A413,A414, \\note fields as indicated
   A415,A416,A417,A418,A419,A420,A421,A422,A423,A424, \\note fields as indicated
   A425,A426,A427,A428,A429,A430,A431,A432,A433,A434, \\note fields as indicated
   A435,A436,A437,A438,A439,A440,A441,A442,A443,A444, \\note fields as indicated
   A445,A446,A447,A448,A449,A450,A451,A452,A453,A454, \\note fields as indicated
   A455,A456,A457,A458,A459,A460,A461,A462,A463,A464, \\note fields as indicated
   A465,A466,A467,A468,A469,A470,A471,A472,A473,A474, \\note fields as indicated
   A475,A476,A477,A478,A479,A480,A481,A482,A483,A484, \\note fields as indicated
   A485,A486,A487,A488,A489,A490,A491,A492,A493,A494, \\note fields as indicated
   A495,A496,A497,A498,A499,A500,A501,A502,A503,A504; \\note fields as indicated

AirLoopHVAC:SupplyPath,
       \\extensible:2 Just duplicate last two fields and comments (changing numbering, please)
       \\memo A supply path can only contain AirLoopHVAC:ZoneSplitter and AirLoopHVAC:SupplyPlenum objects
       \\memo which may be in series or parallel.
   A1, \\field Name
       \\required-field
       \\reference ZoneSupplyAirPaths
   A2, \\field Supply Air Path Inlet Node Name
       \\required-field
       \\type node
   A3, \\field Component 1 Object Type
       \\begin-extensible
       \\note Supply path components must be listed in flow order.
       \\required-field
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
   A4, \\field Component 1 Name
       \\required-field
       \\type object-list
       \\object-list SupplyPathComponentNames
   A5, \\field Component 2 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
   A6, \\field Component 2 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
   A7, \\field Component 3 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
   A8, \\field Component 3 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
   A9, \\field Component 4 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A10, \\field Component 4 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A11, \\field Component 5 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A12, \\field Component 5 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A13, \\field Component 6 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A14, \\field Component 6 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A15, \\field Component 7 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A16, \\field Component 7 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A17, \\field Component 8 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A18, \\field Component 8 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A19, \\field Component 9 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A20, \\field Component 9 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A21, \\field Component 10 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A22, \\field Component 10 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A23, \\field Component 11 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A24, \\field Component 11 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A25, \\field Component 12 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A26, \\field Component 12 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A27, \\field Component 13 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A28, \\field Component 13 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A29, \\field Component 14 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A30, \\field Component 14 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A31, \\field Component 15 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A32, \\field Component 15 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A33, \\field Component 16 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A34, \\field Component 16 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A35, \\field Component 17 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A36, \\field Component 17 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A37, \\field Component 18 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A38, \\field Component 18 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A39, \\field Component 19 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A40, \\field Component 19 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A41, \\field Component 20 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A42, \\field Component 20 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A43, \\field Component 21 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A44, \\field Component 21 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A45, \\field Component 22 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A46, \\field Component 22 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A47, \\field Component 23 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A48, \\field Component 23 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A49, \\field Component 24 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A50, \\field Component 24 Name
       \\type object-list
       \\object-list SupplyPathComponentNames
  A51, \\field Component 25 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneSplitter
       \\key AirLoopHVAC:SupplyPlenum
  A52; \\field Component 25 Name
       \\type object-list
       \\object-list SupplyPathComponentNames

AirLoopHVAC:ZoneMixer,
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Mix N inlet air streams into one (currently 500 per air loop, but extensible).  Node names cannot
       \\memo be duplicated within a single zone mixer (AirLoopHVAC:ZoneMixer) list.
   A1, \\field Name
       \\required-field
       \\reference ZoneMixers
       \\reference ReturnPathComponentNames
   A2, \\field Outlet Node Name
       \\required-field
       \\type node
   A3, \\field Inlet 1 Node Name
       \\begin-extensible
       \\required-field
       \\type node
   A4, \\field Inlet 2 Node Name
       \\type node
   A5, \\field Inlet 3 Node Name
       \\type node
   A6, \\field Inlet 4 Node Name
       \\type node
   A7, \\field Inlet 5 Node Name
       \\type node
   A8, \\field Inlet 6 Node Name
       \\type node
   A9, \\field Inlet 7 Node Name
       \\type node
   A10, \\field Inlet 8 Node Name
        \\type node
   A11, \\field Inlet 9 Node Name
        \\type node
   A12, \\field Inlet 10 Node Name
        \\type node
   A13, \\field Inlet 11 Node Name
        \\type node
   A14, \\field Inlet 12 Node Name
        \\type node
   A15, \\field Inlet 13 Node Name
        \\type node
   A16, \\field Inlet 14 Node Name
        \\type node
   A17, \\field Inlet 15 Node Name
        \\type node
   A18, \\field Inlet 16 Node Name
        \\type node
   A19, \\field Inlet 17 Node Name
        \\type node
   A20, \\field Inlet 18 Node Name
        \\type node
   A21, \\field Inlet 19 Node Name
        \\type node
   A22, \\field Inlet 20 Node Name
        \\type node
   A23, \\field Inlet 21 Node Name
        \\type node
   A24, \\field Inlet 22 Node Name
        \\type node
   A25, \\field Inlet 23 Node Name
        \\type node
   A26, \\field Inlet 24 Node Name
        \\type node
   A27, \\field Inlet 25 Node Name
        \\type node
   A28, \\field Inlet 26 Node Name
        \\type node
   A29, \\field Inlet 27 Node Name
        \\type node
   A30, \\field Inlet 28 Node Name
        \\type node
   A31, \\field Inlet 29 Node Name
        \\type node
   A32, \\field Inlet 30 Node Name
        \\type node
   A33, \\field Inlet 31 Node Name
        \\type node
   A34, \\field Inlet 32 Node Name
        \\type node
   A35, \\field Inlet 33 Node Name
        \\type node
   A36, \\field Inlet 34 Node Name
        \\type node
   A37, \\field Inlet 35 Node Name
        \\type node
   A38, \\field Inlet 36 Node Name
        \\type node
   A39, \\field Inlet 37 Node Name
        \\type node
   A40, \\field Inlet 38 Node Name
        \\type node
   A41, \\field Inlet 39 Node Name
        \\type node
   A42, \\field Inlet 40 Node Name
        \\type node
   A43, \\field Inlet 41 Node Name
        \\type node
   A44, \\field Inlet 42 Node Name
        \\type node
   A45, \\field Inlet 43 Node Name
        \\type node
   A46, \\field Inlet 44 Node Name
        \\type node
   A47, \\field Inlet 45 Node Name
        \\type node
   A48, \\field Inlet 46 Node Name
        \\type node
   A49, \\field Inlet 47 Node Name
        \\type node
   A50, \\field Inlet 48 Node Name
        \\type node
   A51, \\field Inlet 49 Node Name
        \\type node
   A52, \\field Inlet 50 Node Name
        \\type node
   A53, \\field Inlet 51 Node Name
        \\type node
   A54, \\field Inlet 52 Node Name
        \\type node
   A55, \\field Inlet 53 Node Name
        \\type node
   A56, \\field Inlet 54 Node Name
        \\type node
   A57, \\field Inlet 55 Node Name
        \\type node
   A58, \\field Inlet 56 Node Name
        \\type node
   A59, \\field Inlet 57 Node Name
        \\type node
   A60, \\field Inlet 58 Node Name
        \\type node
   A61, \\field Inlet 59 Node Name
        \\type node
   A62, \\field Inlet 60 Node Name
        \\type node
   A63, \\field Inlet 61 Node Name
        \\type node
   A64, \\field Inlet 62 Node Name
        \\type node
   A65, \\field Inlet 63 Node Name
        \\type node
   A66, \\field Inlet 64 Node Name
        \\type node
   A67, \\field Inlet 65 Node Name
        \\type node
   A68, \\field Inlet 66 Node Name
        \\type node
   A69, \\field Inlet 67 Node Name
        \\type node
   A70, \\field Inlet 68 Node Name
        \\type node
   A71, \\field Inlet 69 Node Name
        \\type node
   A72, \\field Inlet 70 Node Name
        \\type node
   A73, \\field Inlet 71 Node Name
        \\type node
   A74, \\field Inlet 72 Node Name
        \\type node
   A75, \\field Inlet 73 Node Name
        \\type node
   A76, \\field Inlet 74 Node Name
        \\type node
   A77, \\field Inlet 75 Node Name
        \\type node
   A78, \\field Inlet 76 Node Name
        \\type node
   A79, \\field Inlet 77 Node Name
        \\type node
   A80, \\field Inlet 78 Node Name
        \\type node
   A81, \\field Inlet 79 Node Name
        \\type node
   A82, \\field Inlet 80 Node Name
        \\type node
   A83, \\field Inlet 81 Node Name
        \\type node
   A84, \\field Inlet 82 Node Name
        \\type node
   A85, \\field Inlet 83 Node Name
        \\type node
   A86, \\field Inlet 84 Node Name
        \\type node
   A87, \\field Inlet 85 Node Name
        \\type node
   A88, \\field Inlet 86 Node Name
        \\type node
   A89, \\field Inlet 87 Node Name
        \\type node
   A90, \\field Inlet 88 Node Name
        \\type node
   A91, \\field Inlet 89 Node Name
        \\type node
   A92, \\field Inlet 90 Node Name
        \\type node
   A93, \\field Inlet 91 Node Name
        \\type node
   A94, \\field Inlet 92 Node Name
        \\type node
   A95, \\field Inlet 93 Node Name
        \\type node
   A96, \\field Inlet 94 Node Name
        \\type node
   A97, \\field Inlet 95 Node Name
        \\type node
   A98, \\field Inlet 96 Node Name
        \\type node
   A99, \\field Inlet 97 Node Name
         \\type node
   A100, \\field Inlet 98 Node Name
         \\type node
   A101, \\field Inlet 99 Node Name
         \\type node
   A102, \\field Inlet 100 Node Name
         \\type node
   A103,A104,A105,A106,A107,A108,A109,A110,A111,A112, \\note fields as indicated
   A113,A114,A115,A116,A117,A118,A119,A120,A121,A122, \\note fields as indicated
   A123,A124,A125,A126,A127,A128,A129,A130,A131,A132, \\note fields as indicated
   A133,A134,A135,A136,A137,A138,A139,A140,A141,A142, \\note fields as indicated
   A143,A144,A145,A146,A147,A148,A149,A150,A151,A152, \\note fields as indicated
   A153,A154,A155,A156,A157,A158,A159,A160,A161,A162, \\note fields as indicated
   A163,A164,A165,A166,A167,A168,A169,A170,A171,A172, \\note fields as indicated
   A173,A174,A175,A176,A177,A178,A179,A180,A181,A182, \\note fields as indicated
   A183,A184,A185,A186,A187,A188,A189,A190,A191,A192, \\note fields as indicated
   A193,A194,A195,A196,A197,A198,A199,A200,A201,A202, \\note fields as indicated
   A203,A204,A205,A206,A207,A208,A209,A210,A211,A212, \\note fields as indicated
   A213,A214,A215,A216,A217,A218,A219,A220,A221,A222, \\note fields as indicated
   A223,A224,A225,A226,A227,A228,A229,A230,A231,A232, \\note fields as indicated
   A233,A234,A235,A236,A237,A238,A239,A240,A241,A242, \\note fields as indicated
   A243,A244,A245,A246,A247,A248,A249,A250,A251,A252, \\note fields as indicated
   A253,A254,A255,A256,A257,A258,A259,A260,A261,A262, \\note fields as indicated
   A263,A264,A265,A266,A267,A268,A269,A270,A271,A272, \\note fields as indicated
   A273,A274,A275,A276,A277,A278,A279,A280,A281,A282, \\note fields as indicated
   A283,A284,A285,A286,A287,A288,A289,A290,A291,A292, \\note fields as indicated
   A293,A294,A295,A296,A297,A298,A299,A300,A301,A302, \\note fields as indicated
   A303,A304,A305,A306,A307,A308,A309,A310,A311,A312, \\note fields as indicated
   A313,A314,A315,A316,A317,A318,A319,A320,A321,A322, \\note fields as indicated
   A323,A324,A325,A326,A327,A328,A329,A330,A331,A332, \\note fields as indicated
   A333,A334,A335,A336,A337,A338,A339,A340,A341,A342, \\note fields as indicated
   A343,A344,A345,A346,A347,A348,A349,A350,A351,A352, \\note fields as indicated
   A353,A354,A355,A356,A357,A358,A359,A360,A361,A362, \\note fields as indicated
   A363,A364,A365,A366,A367,A368,A369,A370,A371,A372, \\note fields as indicated
   A373,A374,A375,A376,A377,A378,A379,A380,A381,A382, \\note fields as indicated
   A383,A384,A385,A386,A387,A388,A389,A390,A391,A392, \\note fields as indicated
   A393,A394,A395,A396,A397,A398,A399,A400,A401,A402, \\note fields as indicated
   A403,A404,A405,A406,A407,A408,A409,A410,A411,A412, \\note fields as indicated
   A413,A414,A415,A416,A417,A418,A419,A420,A421,A422, \\note fields as indicated
   A423,A424,A425,A426,A427,A428,A429,A430,A431,A432, \\note fields as indicated
   A433,A434,A435,A436,A437,A438,A439,A440,A441,A442, \\note fields as indicated
   A443,A444,A445,A446,A447,A448,A449,A450,A451,A452, \\note fields as indicated
   A453,A454,A455,A456,A457,A458,A459,A460,A461,A462, \\note fields as indicated
   A463,A464,A465,A466,A467,A468,A469,A470,A471,A472, \\note fields as indicated
   A473,A474,A475,A476,A477,A478,A479,A480,A481,A482, \\note fields as indicated
   A483,A484,A485,A486,A487,A488,A489,A490,A491,A492, \\note fields as indicated
   A493,A494,A495,A496,A497,A498,A499,A500,A501,A502; \\note fields as indicated

AirLoopHVAC:ReturnPlenum,
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Connects N zone inlet air streams, through zone return plenum, to outlet
       \\memo (currently 500 per air loop)
       \\memo Node names cannot be duplicated within a single plenum list.
   A1, \\field Name
       \\required-field
       \\reference ReturnPathComponentNames
   A2, \\field Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
   A3, \\field Zone Node Name
       \\required-field
       \\type node
   A4, \\field Outlet Node Name
       \\required-field
       \\type node
   A5, \\field Induced Air Outlet Node or NodeList Name
       \\type node
   A6, \\field Inlet 1 Node Name
       \\begin-extensible
       \\required-field
       \\type node
   A7, \\field Inlet 2 Node Name
       \\type node
   A8, \\field Inlet 3 Node Name
       \\type node
   A9, \\field Inlet 4 Node Name
       \\type node
   A10,\\field Inlet 5 Node Name
       \\type node
   A11, \\field Inlet 6 Node Name
        \\type node
   A12, \\field Inlet 7 Node Name
        \\type node
   A13, \\field Inlet 8 Node Name
        \\type node
   A14, \\field Inlet 9 Node Name
        \\type node
   A15, \\field Inlet 10 Node Name
        \\type node
   A16, \\field Inlet 11 Node Name
        \\type node
   A17, \\field Inlet 12 Node Name
        \\type node
   A18, \\field Inlet 13 Node Name
        \\type node
   A19, \\field Inlet 14 Node Name
        \\type node
   A20, \\field Inlet 15 Node Name
        \\type node
   A21, \\field Inlet 16 Node Name
        \\type node
   A22, \\field Inlet 17 Node Name
        \\type node
   A23, \\field Inlet 18 Node Name
        \\type node
   A24, \\field Inlet 19 Node Name
        \\type node
   A25, \\field Inlet 20 Node Name
        \\type node
   A26, \\field Inlet 21 Node Name
        \\type node
   A27, \\field Inlet 22 Node Name
        \\type node
   A28, \\field Inlet 23 Node Name
        \\type node
   A29, \\field Inlet 24 Node Name
        \\type node
   A30, \\field Inlet 25 Node Name
        \\type node
   A31, \\field Inlet 26 Node Name
        \\type node
   A32, \\field Inlet 27 Node Name
        \\type node
   A33, \\field Inlet 28 Node Name
        \\type node
   A34, \\field Inlet 29 Node Name
        \\type node
   A35, \\field Inlet 30 Node Name
        \\type node
   A36, \\field Inlet 31 Node Name
        \\type node
   A37, \\field Inlet 32 Node Name
        \\type node
   A38, \\field Inlet 33 Node Name
        \\type node
   A39, \\field Inlet 34 Node Name
        \\type node
   A40, \\field Inlet 35 Node Name
        \\type node
   A41, \\field Inlet 36 Node Name
        \\type node
   A42, \\field Inlet 37 Node Name
        \\type node
   A43, \\field Inlet 38 Node Name
        \\type node
   A44, \\field Inlet 39 Node Name
        \\type node
   A45, \\field Inlet 40 Node Name
        \\type node
   A46, \\field Inlet 41 Node Name
        \\type node
   A47, \\field Inlet 42 Node Name
        \\type node
   A48, \\field Inlet 43 Node Name
        \\type node
   A49, \\field Inlet 44 Node Name
        \\type node
   A50, \\field Inlet 45 Node Name
        \\type node
   A51, \\field Inlet 46 Node Name
        \\type node
   A52, \\field Inlet 47 Node Name
        \\type node
   A53, \\field Inlet 48 Node Name
        \\type node
   A54, \\field Inlet 49 Node Name
        \\type node
   A55, \\field Inlet 50 Node Name
        \\type node
   A56, \\field Inlet 51 Node Name
        \\type node
   A57, \\field Inlet 52 Node Name
        \\type node
   A58, \\field Inlet 53 Node Name
        \\type node
   A59, \\field Inlet 54 Node Name
        \\type node
   A60, \\field Inlet 55 Node Name
        \\type node
   A61, \\field Inlet 56 Node Name
        \\type node
   A62, \\field Inlet 57 Node Name
        \\type node
   A63, \\field Inlet 58 Node Name
        \\type node
   A64, \\field Inlet 59 Node Name
        \\type node
   A65, \\field Inlet 60 Node Name
        \\type node
   A66, \\field Inlet 61 Node Name
        \\type node
   A67, \\field Inlet 62 Node Name
        \\type node
   A68, \\field Inlet 63 Node Name
        \\type node
   A69, \\field Inlet 64 Node Name
        \\type node
   A70, \\field Inlet 65 Node Name
        \\type node
   A71, \\field Inlet 66 Node Name
        \\type node
   A72, \\field Inlet 67 Node Name
        \\type node
   A73, \\field Inlet 68 Node Name
        \\type node
   A74, \\field Inlet 69 Node Name
        \\type node
   A75, \\field Inlet 70 Node Name
        \\type node
   A76, \\field Inlet 71 Node Name
        \\type node
   A77, \\field Inlet 72 Node Name
        \\type node
   A78, \\field Inlet 73 Node Name
        \\type node
   A79, \\field Inlet 74 Node Name
        \\type node
   A80, \\field Inlet 75 Node Name
        \\type node
   A81, \\field Inlet 76 Node Name
        \\type node
   A82, \\field Inlet 77 Node Name
        \\type node
   A83, \\field Inlet 78 Node Name
        \\type node
   A84, \\field Inlet 79 Node Name
        \\type node
   A85, \\field Inlet 80 Node Name
        \\type node
   A86, \\field Inlet 81 Node Name
        \\type node
   A87, \\field Inlet 82 Node Name
        \\type node
   A88, \\field Inlet 83 Node Name
        \\type node
   A89, \\field Inlet 84 Node Name
        \\type node
   A90, \\field Inlet 85 Node Name
        \\type node
   A91, \\field Inlet 86 Node Name
        \\type node
   A92, \\field Inlet 87 Node Name
        \\type node
   A93, \\field Inlet 88 Node Name
        \\type node
   A94, \\field Inlet 89 Node Name
        \\type node
   A95, \\field Inlet 90 Node Name
        \\type node
   A96, \\field Inlet 91 Node Name
        \\type node
   A97, \\field Inlet 92 Node Name
        \\type node
   A98, \\field Inlet 93 Node Name
        \\type node
   A99, \\field Inlet 94 Node Name
        \\type node
   A100, \\field Inlet 95 Node Name
         \\type node
   A101, \\field Inlet 96 Node Name
         \\type node
   A102, \\field Inlet 97 Node Name
         \\type node
   A103, \\field Inlet 98 Node Name
         \\type node
   A104, \\field Inlet 99 Node Name
         \\type node
   A105, \\field Inlet 100 Node Name
         \\type node
   A106,A107,A108,A109,A110,A111,A112,A113,A114,A115, \\note fields as indicated
   A116,A117,A118,A119,A120,A121,A122,A123,A124,A125, \\note fields as indicated
   A126,A127,A128,A129,A130,A131,A132,A133,A134,A135, \\note fields as indicated
   A136,A137,A138,A139,A140,A141,A142,A143,A144,A145, \\note fields as indicated
   A146,A147,A148,A149,A150,A151,A152,A153,A154,A155, \\note fields as indicated
   A156,A157,A158,A159,A160,A161,A162,A163,A164,A165, \\note fields as indicated
   A166,A167,A168,A169,A170,A171,A172,A173,A174,A175, \\note fields as indicated
   A176,A177,A178,A179,A180,A181,A182,A183,A184,A185, \\note fields as indicated
   A186,A187,A188,A189,A190,A191,A192,A193,A194,A195, \\note fields as indicated
   A196,A197,A198,A199,A200,A201,A202,A203,A204,A205, \\note fields as indicated
   A206,A207,A208,A209,A210,A211,A212,A213,A214,A215, \\note fields as indicated
   A216,A217,A218,A219,A220,A221,A222,A223,A224,A225, \\note fields as indicated
   A226,A227,A228,A229,A230,A231,A232,A233,A234,A235, \\note fields as indicated
   A236,A237,A238,A239,A240,A241,A242,A243,A244,A245, \\note fields as indicated
   A246,A247,A248,A249,A250,A251,A252,A253,A254,A255, \\note fields as indicated
   A256,A257,A258,A259,A260,A261,A262,A263,A264,A265, \\note fields as indicated
   A266,A267,A268,A269,A270,A271,A272,A273,A274,A275, \\note fields as indicated
   A276,A277,A278,A279,A280,A281,A282,A283,A284,A285, \\note fields as indicated
   A286,A287,A288,A289,A290,A291,A292,A293,A294,A295, \\note fields as indicated
   A296,A297,A298,A299,A300,A301,A302,A303,A304,A305, \\note fields as indicated
   A306,A307,A308,A309,A310,A311,A312,A313,A314,A315, \\note fields as indicated
   A316,A317,A318,A319,A320,A321,A322,A323,A324,A325, \\note fields as indicated
   A326,A327,A328,A329,A330,A331,A332,A333,A334,A335, \\note fields as indicated
   A336,A337,A338,A339,A340,A341,A342,A343,A344,A345, \\note fields as indicated
   A346,A347,A348,A349,A350,A351,A352,A353,A354,A355, \\note fields as indicated
   A356,A357,A358,A359,A360,A361,A362,A363,A364,A365, \\note fields as indicated
   A366,A367,A368,A369,A370,A371,A372,A373,A374,A375, \\note fields as indicated
   A376,A377,A378,A379,A380,A381,A382,A383,A384,A385, \\note fields as indicated
   A386,A387,A388,A389,A390,A391,A392,A393,A394,A395, \\note fields as indicated
   A396,A397,A398,A399,A400,A401,A402,A403,A404,A405, \\note fields as indicated
   A406,A407,A408,A409,A410,A411,A412,A413,A414,A415, \\note fields as indicated
   A416,A417,A418,A419,A420,A421,A422,A423,A424,A425, \\note fields as indicated
   A426,A427,A428,A429,A430,A431,A432,A433,A434,A435, \\note fields as indicated
   A436,A437,A438,A439,A440,A441,A442,A443,A444,A445, \\note fields as indicated
   A446,A447,A448,A449,A450,A451,A452,A453,A454,A455, \\note fields as indicated
   A456,A457,A458,A459,A460,A461,A462,A463,A464,A465, \\note fields as indicated
   A466,A467,A468,A469,A470,A471,A472,A473,A474,A475, \\note fields as indicated
   A476,A477,A478,A479,A480,A481,A482,A483,A484,A485, \\note fields as indicated
   A486,A487,A488,A489,A490,A491,A492,A493,A494,A495, \\note fields as indicated
   A496,A497,A498,A499,A500,A501,A502,A503,A504,A505; \\note fields as indicated

AirLoopHVAC:ReturnPath,
       \\extensible:2 Just duplicate last two fields and comments (changing numbering, please)
       \\memo A return air path can only contain one AirLoopHVAC:ZoneMixer
       \\memo and one or more AirLoopHVAC:ReturnPlenum objects.
   A1, \\field Name
       \\required-field
       \\reference ZoneReturnAirPaths
   A2, \\field Return Air Path Outlet Node Name
       \\required-field
       \\type node
   A3, \\field Component 1 Object Type
       \\begin-extensible
       \\required-field
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
   A4, \\field Component 1 Name
       \\required-field
       \\type object-list
       \\object-list ReturnPathComponentNames
   A5, \\field Component 2 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
   A6, \\field Component 2 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
   A7, \\field Component 3 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
   A8, \\field Component 3 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
   A9, \\field Component 4 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A10, \\field Component 4 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A11, \\field Component 5 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A12, \\field Component 5 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A13, \\field Component 6 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A14, \\field Component 6 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A15, \\field Component 7 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A16, \\field Component 7 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A17, \\field Component 8 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A18, \\field Component 8 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A19, \\field Component 9 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A20, \\field Component 9 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A21, \\field Component 10 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A22, \\field Component 10 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A23, \\field Component 11 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A24, \\field Component 11 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A25, \\field Component 12 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A26, \\field Component 12 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A27, \\field Component 13 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A28, \\field Component 13 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A29, \\field Component 14 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A30, \\field Component 14 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A31, \\field Component 15 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A32, \\field Component 15 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A33, \\field Component 16 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A34, \\field Component 16 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A35, \\field Component 17 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A36, \\field Component 17 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A37, \\field Component 18 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A38, \\field Component 18 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A39, \\field Component 19 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A40, \\field Component 19 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A41, \\field Component 20 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A42, \\field Component 20 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A43, \\field Component 21 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A44, \\field Component 21 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A45, \\field Component 22 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A46, \\field Component 22 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A47, \\field Component 23 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A48, \\field Component 23 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A49, \\field Component 24 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A50, \\field Component 24 Name
       \\type object-list
       \\object-list ReturnPathComponentNames
  A51, \\field Component 25 Object Type
       \\type choice
       \\key AirLoopHVAC:ZoneMixer
       \\key AirLoopHVAC:ReturnPlenum
  A52; \\field Component 25 Name
       \\type object-list
       \\object-list ReturnPathComponentNames

\\group Node-Branch Management

Branch,
        \\extensible:5 Just duplicate last 5 fields and comments (changing numbering, please)
        \\memo List components on the branch in simulation and connection order
        \\memo Note: this should NOT include splitters or mixers which define
        \\memo endpoints of branches
   A1,  \\field Name
        \\required-field
        \\reference Branches
   N1, \\field Maximum Flow Rate
        \\default 0
        \\units m3/s
        \\minimum 0
        \\autosizable
   A2, \\field Pressure Drop Curve Name
        \\type object-list
        \\object-list PressureDropCurves
        \\object-list ExponentCurves
        \\object-list LinearOrQuadraticCurves
        \\object-list CubicCurves
        \\note Optional field to include this branch in plant pressure drop calculations
        \\note This field is only relevant for branches in PlantLoops and CondenserLoops
        \\note Air loops do not account for pressure drop using this field
        \\note Valid curve types are: Curve:Functional:PressureDrop or
        \\note one of Curve:{Linear,Quadratic,Cubic,Exponent}')
   A3, \\field Component 1 Object Type
        \\begin-extensible
        \\required-field
   A4, \\field Component 1 Name
        \\required-field
   A5, \\field Component 1 Inlet Node Name
        \\required-field
        \\type node
   A6, \\field Component 1 Outlet Node Name
        \\required-field
        \\type node
   A7, \\field Component 1 Branch Control Type
       \\note field is no longer used
   A8, \\field Component 2 Object Type
   A9, \\field Component 2 Name
   A10,\\field Component 2 Inlet Node Name
        \\type node
   A11, \\field Component 2 Outlet Node Name
        \\type node
   A12, \\field Component 2 Branch Control Type
       \\note field is no longer used
   A13, \\field Component 3 Object Type
   A14, \\field Component 3 Name
   A15, \\field Component 3 Inlet Node Name
        \\type node
   A16, \\field Component 3 Outlet Node Name
        \\type node
   A17, \\field Component 3 Branch Control Type
       \\note field is no longer useds
   A18, \\field Component 4 Object Type
   A19, \\field Component 4 Name
   A20, \\field Component 4 Inlet Node Name
        \\type node
   A21, \\field Component 4 Outlet Node Name
        \\type node
   A22, \\field Component 4 Branch Control Type
       \\note field is no longer used
   A23, \\field Component 5 Object Type
   A24, \\field Component 5 Name
   A25, \\field Component 5 Inlet Node Name
        \\type node
   A26, \\field Component 5 Outlet Node Name
        \\type node
   A27, \\field Component 5 Branch Control Type
       \\note field is no longer used
   A28, \\field Component 6 Object Type
   A29, \\field Component 6 Name
   A30, \\field Component 6 Inlet Node Name
        \\type node
   A31, \\field Component 6 Outlet Node Name
        \\type node
   A32, \\field Component 6 Branch Control Type
       \\note field is no longer used
   A33, \\field Component 7 Object Type
   A34, \\field Component 7 Name
   A35, \\field Component 7 Inlet Node Name
        \\type node
   A36, \\field Component 7 Outlet Node Name
        \\type node
   A37, \\field Component 7 Branch Control Type
       \\note field is no longer used
   A38, \\field Component 8 Object Type
   A39, \\field Component 8 Name
   A40, \\field Component 8 Inlet Node Name
        \\type node
   A41, \\field Component 8 Outlet Node Name
        \\type node
   A42, \\field Component 8 Branch Control Type
       \\note field is no longer used
   A43, \\field Component 9 Object Type
   A44, \\field Component 9 Name
   A45, \\field Component 9 Inlet Node Name
        \\type node
   A46, \\field Component 9 Outlet Node Name
        \\type node
   A47, \\field Component 9 Branch Control Type
       \\note field is no longer used
   A48, \\field Component 10 Object Type
   A49, \\field Component 10 Name
   A50, \\field Component 10 Inlet Node Name
        \\type node
   A51, \\field Component 10 Outlet Node Name
        \\type node
   A52, \\field Component 10 Branch Control Type
       \\note field is no longer used
   A53, \\field Component 11 Object Type
   A54, \\field Component 11 Name
   A55, \\field Component 11 Inlet Node Name
        \\type node
   A56, \\field Component 11 Outlet Node Name
        \\type node
   A57; \\field Component 11 Branch Control Type
       \\note field is no longer used

BranchList,
 \\extensible:1 Just duplicate last field and comments (changing numbering, please)
 \\memo Branches MUST be listed in Flow order: Inlet branch, then parallel branches, then Outlet branch.
 \\memo Branches are simulated in the order listed.  Branch names cannot be duplicated within a single branch list.
   A1,  \\field Name
       \\required-field
       \\reference BranchLists
   A2, \\field Branch 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list Branches
   A3, \\field Branch 2 Name
       \\type object-list
       \\object-list Branches
   A4, \\field Branch 3 Name
       \\type object-list
       \\object-list Branches
   A5, \\field Branch 4 Name
       \\type object-list
       \\object-list Branches
   A6, \\field Branch 5 Name
       \\type object-list
       \\object-list Branches
   A7, \\field Branch 6 Name
       \\type object-list
       \\object-list Branches
   A8, \\field Branch 7 Name
       \\type object-list
       \\object-list Branches
   A9, \\field Branch 8 Name
       \\type object-list
       \\object-list Branches
   A10, \\field Branch 9 Name
        \\type object-list
        \\object-list Branches
   A11, \\field Branch 10 Name
        \\type object-list
        \\object-list Branches
   A12, \\field Branch 11 Name
        \\type object-list
        \\object-list Branches
   A13, \\field Branch 12 Name
       \\type object-list
        \\object-list Branches
   A14, \\field Branch 13 Name
        \\type object-list
        \\object-list Branches
   A15, \\field Branch 14 Name
        \\type object-list
        \\object-list Branches
   A16, \\field Branch 15 Name
        \\type object-list
        \\object-list Branches
   A17, \\field Branch 16 Name
        \\type object-list
        \\object-list Branches
   A18, \\field Branch 17 Name
        \\type object-list
        \\object-list Branches
   A19, \\field Branch 18 Name
        \\type object-list
        \\object-list Branches
   A20, \\field Branch 19 Name
        \\type object-list
        \\object-list Branches
   A21, \\field Branch 20 Name
        \\type object-list
        \\object-list Branches
   A22, \\field Branch 21 Name
        \\type object-list
        \\object-list Branches
   A23, \\field Branch 22 Name
        \\type object-list
        \\object-list Branches
   A24, \\field Branch 23 Name
        \\type object-list
        \\object-list Branches
   A25, \\field Branch 24 Name
        \\type object-list
        \\object-list Branches
   A26, \\field Branch 25 Name
        \\type object-list
        \\object-list Branches
   A27, \\field Branch 26 Name
        \\type object-list
        \\object-list Branches
   A28, \\field Branch 27 Name
        \\type object-list
        \\object-list Branches
   A29, \\field Branch 28 Name
        \\type object-list
        \\object-list Branches
   A30, \\field Branch 29 Name
        \\type object-list
        \\object-list Branches
   A31, \\field Branch 30 Name
        \\type object-list
        \\object-list Branches
   A32, \\field Branch 31 Name
        \\type object-list
        \\object-list Branches
   A33, \\field Branch 32 Name
        \\type object-list
        \\object-list Branches
   A34, \\field Branch 33 Name
        \\type object-list
        \\object-list Branches
   A35, \\field Branch 34 Name
        \\type object-list
        \\object-list Branches
   A36, \\field Branch 35 Name
        \\type object-list
        \\object-list Branches
   A37, \\field Branch 36 Name
        \\type object-list
        \\object-list Branches
   A38, \\field Branch 37 Name
        \\type object-list
        \\object-list Branches
   A39, \\field Branch 38 Name
        \\type object-list
        \\object-list Branches
   A40, \\field Branch 39 Name
        \\type object-list
        \\object-list Branches
   A41, \\field Branch 40 Name
        \\type object-list
        \\object-list Branches
   A42, \\field Branch 41 Name
        \\type object-list
        \\object-list Branches
   A43, \\field Branch 42 Name
        \\type object-list
        \\object-list Branches
   A44, \\field Branch 43 Name
        \\type object-list
        \\object-list Branches
   A45, \\field Branch 44 Name
        \\type object-list
        \\object-list Branches
   A46, \\field Branch 45 Name
        \\type object-list
        \\object-list Branches
   A47, \\field Branch 46 Name
        \\type object-list
        \\object-list Branches
   A48, \\field Branch 47 Name
        \\type object-list
        \\object-list Branches
   A49, \\field Branch 48 Name
        \\type object-list
        \\object-list Branches
   A50, \\field Branch 49 Name
        \\type object-list
        \\object-list Branches
   A51, \\field Branch 50 Name
        \\type object-list
        \\object-list Branches
   A52, \\field Branch 51 Name
        \\type object-list
        \\object-list Branches
   A53, \\field Branch 52 Name
        \\type object-list
        \\object-list Branches
   A54, \\field Branch 53 Name
        \\type object-list
        \\object-list Branches
   A55, \\field Branch 54 Name
        \\type object-list
        \\object-list Branches
   A56, \\field Branch 55 Name
        \\type object-list
        \\object-list Branches
   A57, \\field Branch 56 Name
        \\type object-list
        \\object-list Branches
   A58, \\field Branch 57 Name
        \\type object-list
        \\object-list Branches
   A59, \\field Branch 58 Name
        \\type object-list
        \\object-list Branches
   A60, \\field Branch 59 Name
        \\type object-list
        \\object-list Branches
   A61, \\field Branch 60 Name
        \\type object-list
        \\object-list Branches
   A62, \\field Branch 61 Name
        \\type object-list
        \\object-list Branches
   A63, \\field Branch 62 Name
        \\type object-list
        \\object-list Branches
   A64, \\field Branch 63 Name
        \\type object-list
        \\object-list Branches
   A65, \\field Branch 64 Name
        \\type object-list
        \\object-list Branches
   A66, \\field Branch 65 Name
        \\type object-list
        \\object-list Branches
   A67, \\field Branch 66 Name
        \\type object-list
        \\object-list Branches
   A68, \\field Branch 67 Name
        \\type object-list
        \\object-list Branches
   A69, \\field Branch 68 Name
        \\type object-list
        \\object-list Branches
   A70, \\field Branch 69 Name
        \\type object-list
        \\object-list Branches
   A71, \\field Branch 70 Name
        \\type object-list
        \\object-list Branches
   A72, \\field Branch 71 Name
        \\type object-list
        \\object-list Branches
   A73, \\field Branch 72 Name
        \\type object-list
        \\object-list Branches
   A74, \\field Branch 73 Name
        \\type object-list
        \\object-list Branches
   A75, \\field Branch 74 Name
        \\type object-list
        \\object-list Branches
   A76, \\field Branch 75 Name
        \\type object-list
        \\object-list Branches
   A77, \\field Branch 76 Name
        \\type object-list
        \\object-list Branches
   A78, \\field Branch 77 Name
        \\type object-list
        \\object-list Branches
   A79, \\field Branch 78 Name
        \\type object-list
        \\object-list Branches
   A80, \\field Branch 79 Name
        \\type object-list
        \\object-list Branches
   A81, \\field Branch 80 Name
        \\type object-list
        \\object-list Branches
   A82, \\field Branch 81 Name
        \\type object-list
        \\object-list Branches
   A83, \\field Branch 82 Name
        \\type object-list
        \\object-list Branches
   A84, \\field Branch 83 Name
        \\type object-list
        \\object-list Branches
   A85, \\field Branch 84 Name
        \\type object-list
        \\object-list Branches
   A86, \\field Branch 85 Name
        \\type object-list
        \\object-list Branches
   A87, \\field Branch 86 Name
        \\type object-list
        \\object-list Branches
   A88, \\field Branch 87 Name
        \\type object-list
        \\object-list Branches
   A89, \\field Branch 88 Name
        \\type object-list
        \\object-list Branches
   A90, \\field Branch 89 Name
        \\type object-list
        \\object-list Branches
   A91, \\field Branch 90 Name
        \\type object-list
        \\object-list Branches
   A92, \\field Branch 91 Name
        \\type object-list
        \\object-list Branches
   A93, \\field Branch 92 Name
        \\type object-list
        \\object-list Branches
   A94, \\field Branch 93 Name
        \\type object-list
        \\object-list Branches
   A95, \\field Branch 94 Name
        \\type object-list
        \\object-list Branches
   A96, \\field Branch 95 Name
        \\type object-list
        \\object-list Branches
   A97, \\field Branch 96 Name
        \\type object-list
        \\object-list Branches
   A98, \\field Branch 97 Name
        \\type object-list
        \\object-list Branches
   A99, \\field Branch 98 Name
        \\type object-list
        \\object-list Branches
  A100, \\field Branch 99 Name
        \\type object-list
        \\object-list Branches
  A101, \\field Branch 100 Name
        \\type object-list
        \\object-list Branches
  A102,A103,A104,A105,A106,A107,A108,A109,A110,A111, \\note fields as indicated
  A112,A113,A114,A115,A116,A117,A118,A119,A120,A121, \\note fields as indicated
  A122,A123,A124,A125,A126,A127,A128,A129,A130,A131, \\note fields as indicated
  A132,A133,A134,A135,A136,A137,A138,A139,A140,A141, \\note fields as indicated
  A142,A143,A144,A145,A146,A147,A148,A149,A150,A151, \\note fields as indicated
  A152,A153,A154,A155,A156,A157,A158,A159,A160,A161, \\note fields as indicated
  A162,A163,A164,A165,A166,A167,A168,A169,A170,A171, \\note fields as indicated
  A172,A173,A174,A175,A176,A177,A178,A179,A180,A181, \\note fields as indicated
  A182,A183,A184,A185,A186,A187,A188,A189,A190,A191, \\note fields as indicated
  A192,A193,A194,A195,A196,A197,A198,A199,A200,A201, \\note fields as indicated
  A202,A203,A204,A205,A206,A207,A208,A209,A210,A211, \\note fields as indicated
  A212,A213,A214,A215,A216,A217,A218,A219,A220,A221, \\note fields as indicated
  A222,A223,A224,A225,A226,A227,A228,A229,A230,A231, \\note fields as indicated
  A232,A233,A234,A235,A236,A237,A238,A239,A240,A241, \\note fields as indicated
  A242,A243,A244,A245,A246,A247,A248,A249,A250,A251, \\note fields as indicated
  A252,A253,A254,A255,A256,A257,A258,A259,A260,A261, \\note fields as indicated
  A262,A263,A264,A265,A266,A267,A268,A269,A270,A271, \\note fields as indicated
  A272,A273,A274,A275,A276,A277,A278,A279,A280,A281, \\note fields as indicated
  A282,A283,A284,A285,A286,A287,A288,A289,A290,A291, \\note fields as indicated
  A292,A293,A294,A295,A296,A297,A298,A299,A300,A301, \\note fields as indicated
  A302,A303,A304,A305,A306,A307,A308,A309,A310,A311, \\note fields as indicated
  A312,A313,A314,A315,A316,A317,A318,A319,A320,A321, \\note fields as indicated
  A322,A323,A324,A325,A326,A327,A328,A329,A330,A331, \\note fields as indicated
  A332,A333,A334,A335,A336,A337,A338,A339,A340,A341, \\note fields as indicated
  A342,A343,A344,A345,A346,A347,A348,A349,A350,A351, \\note fields as indicated
  A352,A353,A354,A355,A356,A357,A358,A359,A360,A361, \\note fields as indicated
  A362,A363,A364,A365,A366,A367,A368,A369,A370,A371, \\note fields as indicated
  A372,A373,A374,A375,A376,A377,A378,A379,A380,A381, \\note fields as indicated
  A382,A383,A384,A385,A386,A387,A388,A389,A390,A391, \\note fields as indicated
  A392,A393,A394,A395,A396,A397,A398,A399,A400,A401, \\note fields as indicated
  A402,A403,A404,A405,A406,A407,A408,A409,A410,A411, \\note fields as indicated
  A412,A413,A414,A415,A416,A417,A418,A419,A420,A421, \\note fields as indicated
  A422,A423,A424,A425,A426,A427,A428,A429,A430,A431, \\note fields as indicated
  A432,A433,A434,A435,A436,A437,A438,A439,A440,A441, \\note fields as indicated
  A442,A443,A444,A445,A446,A447,A448,A449,A450,A451, \\note fields as indicated
  A452,A453,A454,A455,A456,A457,A458,A459,A460,A461, \\note fields as indicated
  A462,A463,A464,A465,A466,A467,A468,A469,A470,A471, \\note fields as indicated
  A472,A473,A474,A475,A476,A477,A478,A479,A480,A481, \\note fields as indicated
  A482,A483,A484,A485,A486,A487,A488,A489,A490,A491, \\note fields as indicated
  A492,A493,A494,A495,A496,A497,A498,A499,A500,A501; \\note fields as indicated

Connector:Splitter,
  \\min-fields 3
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Split one air/water stream into N outlet streams.  Branch names cannot be duplicated
       \\memo within a single Splitter list.
   A1, \\field Name
        \\required-field
   A2, \\field Inlet Branch Name
        \\required-field
        \\type object-list
        \\object-list Branches
   A3, \\field Outlet Branch 1 Name
        \\begin-extensible
        \\required-field
        \\type object-list
        \\object-list Branches
   A4, \\field Outlet Branch 2 Name
        \\type object-list
        \\object-list Branches
   A5, \\field Outlet Branch 3 Name
        \\type object-list
        \\object-list Branches
   A6, \\field Outlet Branch 4 Name
        \\type object-list
        \\object-list Branches
   A7, \\field Outlet Branch 5 Name
        \\type object-list
        \\object-list Branches
   A8, \\field Outlet Branch 6 Name
        \\type object-list
        \\object-list Branches
   A9, \\field Outlet Branch 7 Name
        \\type object-list
        \\object-list Branches
   A10, \\field Outlet Branch 8 Name
        \\type object-list
        \\object-list Branches
   A11, \\field Outlet Branch 9 Name
        \\type object-list
        \\object-list Branches
   A12, \\field Outlet Branch 10 Name
        \\type object-list
        \\object-list Branches
   A13, \\field Outlet Branch 11 Name
        \\type object-list
        \\object-list Branches
   A14, \\field Outlet Branch 12 Name
        \\type object-list
        \\object-list Branches
   A15, \\field Outlet Branch 13 Name
        \\type object-list
        \\object-list Branches
   A16, \\field Outlet Branch 14 Name
        \\type object-list
        \\object-list Branches
   A17, \\field Outlet Branch 15 Name
        \\type object-list
        \\object-list Branches
   A18, \\field Outlet Branch 16 Name
        \\type object-list
        \\object-list Branches
   A19, \\field Outlet Branch 17 Name
        \\type object-list
        \\object-list Branches
   A20, \\field Outlet Branch 18 Name
        \\type object-list
        \\object-list Branches
   A21, \\field Outlet Branch 19 Name
        \\type object-list
        \\object-list Branches
   A22, \\field Outlet Branch 20 Name
        \\type object-list
        \\object-list Branches
   A23, \\field Outlet Branch 21 Name
        \\type object-list
        \\object-list Branches
   A24, \\field Outlet Branch 22 Name
        \\type object-list
        \\object-list Branches
   A25, \\field Outlet Branch 23 Name
        \\type object-list
        \\object-list Branches
   A26, \\field Outlet Branch 24 Name
        \\type object-list
        \\object-list Branches
   A27, \\field Outlet Branch 25 Name
        \\type object-list
        \\object-list Branches
   A28, \\field Outlet Branch 26 Name
        \\type object-list
        \\object-list Branches
   A29, \\field Outlet Branch 27 Name
        \\type object-list
        \\object-list Branches
   A30, \\field Outlet Branch 28 Name
        \\type object-list
        \\object-list Branches
   A31, \\field Outlet Branch 29 Name
        \\type object-list
        \\object-list Branches
   A32, \\field Outlet Branch 30 Name
        \\type object-list
        \\object-list Branches
   A33, \\field Outlet Branch 31 Name
        \\type object-list
        \\object-list Branches
   A34, \\field Outlet Branch 32 Name
        \\type object-list
        \\object-list Branches
   A35, \\field Outlet Branch 33 Name
        \\type object-list
        \\object-list Branches
   A36, \\field Outlet Branch 34 Name
        \\type object-list
        \\object-list Branches
   A37, \\field Outlet Branch 35 Name
        \\type object-list
        \\object-list Branches
   A38, \\field Outlet Branch 36 Name
        \\type object-list
        \\object-list Branches
   A39, \\field Outlet Branch 37 Name
        \\type object-list
        \\object-list Branches
   A40, \\field Outlet Branch 38 Name
        \\type object-list
        \\object-list Branches
   A41, \\field Outlet Branch 39 Name
        \\type object-list
        \\object-list Branches
   A42, \\field Outlet Branch 40 Name
        \\type object-list
        \\object-list Branches
   A43, \\field Outlet Branch 41 Name
        \\type object-list
        \\object-list Branches
   A44, \\field Outlet Branch 42 Name
        \\type object-list
        \\object-list Branches
   A45, \\field Outlet Branch 43 Name
        \\type object-list
        \\object-list Branches
   A46, \\field Outlet Branch 44 Name
        \\type object-list
        \\object-list Branches
   A47, \\field Outlet Branch 45 Name
        \\type object-list
        \\object-list Branches
   A48, \\field Outlet Branch 46 Name
        \\type object-list
        \\object-list Branches
   A49, \\field Outlet Branch 47 Name
        \\type object-list
        \\object-list Branches
   A50, \\field Outlet Branch 48 Name
        \\type object-list
        \\object-list Branches
   A51, \\field Outlet Branch 49 Name
        \\type object-list
        \\object-list Branches
   A52, \\field Outlet Branch 50 Name
        \\type object-list
        \\object-list Branches
   A53, \\field Outlet Branch 51 Name
        \\type object-list
        \\object-list Branches
   A54, \\field Outlet Branch 52 Name
        \\type object-list
        \\object-list Branches
   A55, \\field Outlet Branch 53 Name
        \\type object-list
        \\object-list Branches
   A56, \\field Outlet Branch 54 Name
        \\type object-list
        \\object-list Branches
   A57, \\field Outlet Branch 55 Name
        \\type object-list
        \\object-list Branches
   A58, \\field Outlet Branch 56 Name
        \\type object-list
        \\object-list Branches
   A59, \\field Outlet Branch 57 Name
        \\type object-list
        \\object-list Branches
   A60, \\field Outlet Branch 58 Name
        \\type object-list
        \\object-list Branches
   A61, \\field Outlet Branch 59 Name
        \\type object-list
        \\object-list Branches
   A62, \\field Outlet Branch 60 Name
        \\type object-list
        \\object-list Branches
   A63, \\field Outlet Branch 61 Name
        \\type object-list
        \\object-list Branches
   A64, \\field Outlet Branch 62 Name
        \\type object-list
        \\object-list Branches
   A65, \\field Outlet Branch 63 Name
        \\type object-list
        \\object-list Branches
   A66, \\field Outlet Branch 64 Name
        \\type object-list
        \\object-list Branches
   A67, \\field Outlet Branch 65 Name
        \\type object-list
        \\object-list Branches
   A68, \\field Outlet Branch 66 Name
        \\type object-list
        \\object-list Branches
   A69, \\field Outlet Branch 67 Name
        \\type object-list
        \\object-list Branches
   A70, \\field Outlet Branch 68 Name
        \\type object-list
        \\object-list Branches
   A71, \\field Outlet Branch 69 Name
        \\type object-list
        \\object-list Branches
   A72, \\field Outlet Branch 70 Name
        \\type object-list
        \\object-list Branches
   A73, \\field Outlet Branch 71 Name
        \\type object-list
        \\object-list Branches
   A74, \\field Outlet Branch 72 Name
        \\type object-list
        \\object-list Branches
   A75, \\field Outlet Branch 73 Name
        \\type object-list
        \\object-list Branches
   A76, \\field Outlet Branch 74 Name
        \\type object-list
        \\object-list Branches
   A77, \\field Outlet Branch 75 Name
        \\type object-list
        \\object-list Branches
   A78, \\field Outlet Branch 76 Name
        \\type object-list
        \\object-list Branches
   A79, \\field Outlet Branch 77 Name
        \\type object-list
        \\object-list Branches
   A80, \\field Outlet Branch 78 Name
        \\type object-list
        \\object-list Branches
   A81, \\field Outlet Branch 79 Name
        \\type object-list
        \\object-list Branches
   A82, \\field Outlet Branch 80 Name
        \\type object-list
        \\object-list Branches
   A83, \\field Outlet Branch 81 Name
        \\type object-list
        \\object-list Branches
   A84, \\field Outlet Branch 82 Name
        \\type object-list
        \\object-list Branches
   A85, \\field Outlet Branch 83 Name
        \\type object-list
        \\object-list Branches
   A86, \\field Outlet Branch 84 Name
        \\type object-list
        \\object-list Branches
   A87, \\field Outlet Branch 85 Name
        \\type object-list
        \\object-list Branches
   A88, \\field Outlet Branch 86 Name
        \\type object-list
        \\object-list Branches
   A89, \\field Outlet Branch 87 Name
        \\type object-list
        \\object-list Branches
   A90, \\field Outlet Branch 88 Name
        \\type object-list
        \\object-list Branches
   A91, \\field Outlet Branch 89 Name
        \\type object-list
        \\object-list Branches
   A92, \\field Outlet Branch 90 Name
        \\type object-list
        \\object-list Branches
   A93, \\field Outlet Branch 91 Name
        \\type object-list
        \\object-list Branches
   A94, \\field Outlet Branch 92 Name
        \\type object-list
        \\object-list Branches
   A95, \\field Outlet Branch 93 Name
        \\type object-list
        \\object-list Branches
   A96, \\field Outlet Branch 94 Name
        \\type object-list
        \\object-list Branches
   A97, \\field Outlet Branch 95 Name
        \\type object-list
        \\object-list Branches
   A98, \\field Outlet Branch 96 Name
        \\type object-list
        \\object-list Branches
   A99, \\field Outlet Branch 97 Name
        \\type object-list
        \\object-list Branches
  A100, \\field Outlet Branch 98 Name
        \\type object-list
        \\object-list Branches
  A101, \\field Outlet Branch 99 Name
        \\type object-list
        \\object-list Branches
  A102, \\field Outlet Branch 100 Name
        \\type object-list
        \\object-list Branches
  A103,A104,A105,A106,A107,A108,A109,A110,A111,A112, \\note fields as indicated
  A113,A114,A115,A116,A117,A118,A119,A120,A121,A122, \\note fields as indicated
  A123,A124,A125,A126,A127,A128,A129,A130,A131,A132, \\note fields as indicated
  A133,A134,A135,A136,A137,A138,A139,A140,A141,A142, \\note fields as indicated
  A143,A144,A145,A146,A147,A148,A149,A150,A151,A152, \\note fields as indicated
  A153,A154,A155,A156,A157,A158,A159,A160,A161,A162, \\note fields as indicated
  A163,A164,A165,A166,A167,A168,A169,A170,A171,A172, \\note fields as indicated
  A173,A174,A175,A176,A177,A178,A179,A180,A181,A182, \\note fields as indicated
  A183,A184,A185,A186,A187,A188,A189,A190,A191,A192, \\note fields as indicated
  A193,A194,A195,A196,A197,A198,A199,A200,A201,A202, \\note fields as indicated
  A203,A204,A205,A206,A207,A208,A209,A210,A211,A212, \\note fields as indicated
  A213,A214,A215,A216,A217,A218,A219,A220,A221,A222, \\note fields as indicated
  A223,A224,A225,A226,A227,A228,A229,A230,A231,A232, \\note fields as indicated
  A233,A234,A235,A236,A237,A238,A239,A240,A241,A242, \\note fields as indicated
  A243,A244,A245,A246,A247,A248,A249,A250,A251,A252, \\note fields as indicated
  A253,A254,A255,A256,A257,A258,A259,A260,A261,A262, \\note fields as indicated
  A263,A264,A265,A266,A267,A268,A269,A270,A271,A272, \\note fields as indicated
  A273,A274,A275,A276,A277,A278,A279,A280,A281,A282, \\note fields as indicated
  A283,A284,A285,A286,A287,A288,A289,A290,A291,A292, \\note fields as indicated
  A293,A294,A295,A296,A297,A298,A299,A300,A301,A302, \\note fields as indicated
  A303,A304,A305,A306,A307,A308,A309,A310,A311,A312, \\note fields as indicated
  A313,A314,A315,A316,A317,A318,A319,A320,A321,A322, \\note fields as indicated
  A323,A324,A325,A326,A327,A328,A329,A330,A331,A332, \\note fields as indicated
  A333,A334,A335,A336,A337,A338,A339,A340,A341,A342, \\note fields as indicated
  A343,A344,A345,A346,A347,A348,A349,A350,A351,A352, \\note fields as indicated
  A353,A354,A355,A356,A357,A358,A359,A360,A361,A362, \\note fields as indicated
  A363,A364,A365,A366,A367,A368,A369,A370,A371,A372, \\note fields as indicated
  A373,A374,A375,A376,A377,A378,A379,A380,A381,A382, \\note fields as indicated
  A383,A384,A385,A386,A387,A388,A389,A390,A391,A392, \\note fields as indicated
  A393,A394,A395,A396,A397,A398,A399,A400,A401,A402, \\note fields as indicated
  A403,A404,A405,A406,A407,A408,A409,A410,A411,A412, \\note fields as indicated
  A413,A414,A415,A416,A417,A418,A419,A420,A421,A422, \\note fields as indicated
  A423,A424,A425,A426,A427,A428,A429,A430,A431,A432, \\note fields as indicated
  A433,A434,A435,A436,A437,A438,A439,A440,A441,A442, \\note fields as indicated
  A443,A444,A445,A446,A447,A448,A449,A450,A451,A452, \\note fields as indicated
  A453,A454,A455,A456,A457,A458,A459,A460,A461,A462, \\note fields as indicated
  A463,A464,A465,A466,A467,A468,A469,A470,A471,A472, \\note fields as indicated
  A473,A474,A475,A476,A477,A478,A479,A480,A481,A482, \\note fields as indicated
  A483,A484,A485,A486,A487,A488,A489,A490,A491,A492, \\note fields as indicated
  A493,A494,A495,A496,A497,A498,A499,A500,A501,A502; \\note fields as indicated

Connector:Mixer,
  \\min-fields 3
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo Mix N inlet air/water streams into one.  Branch names cannot be duplicated within
       \\memo a single mixer list.
   A1 , \\field Name
        \\required-field
   A2 , \\field Outlet Branch Name
        \\required-field
        \\type object-list
        \\object-list Branches
   A3 , \\field Inlet Branch 1 Name
        \\begin-extensible
        \\required-field
        \\type object-list
        \\object-list Branches
   A4 , \\field Inlet Branch 2 Name
        \\type object-list
        \\object-list Branches
   A5 , \\field Inlet Branch 3 Name
        \\type object-list
        \\object-list Branches
   A6 , \\field Inlet Branch 4 Name
        \\type object-list
        \\object-list Branches
   A7 , \\field Inlet Branch 5 Name
        \\type object-list
        \\object-list Branches
   A8 , \\field Inlet Branch 6 Name
        \\type object-list
        \\object-list Branches
   A9 , \\field Inlet Branch 7 Name
        \\type object-list
        \\object-list Branches
   A10, \\field Inlet Branch 8 Name
        \\type object-list
        \\object-list Branches
   A11, \\field Inlet Branch 9 Name
        \\type object-list
        \\object-list Branches
   A12, \\field Inlet Branch 10 Name
        \\type object-list
        \\object-list Branches
   A13, \\field Inlet Branch 11 Name
        \\type object-list
        \\object-list Branches
   A14, \\field Inlet Branch 12 Name
        \\type object-list
        \\object-list Branches
   A15, \\field Inlet Branch 13 Name
        \\type object-list
        \\object-list Branches
   A16, \\field Inlet Branch 14 Name
        \\type object-list
        \\object-list Branches
   A17, \\field Inlet Branch 15 Name
        \\type object-list
        \\object-list Branches
   A18, \\field Inlet Branch 16 Name
        \\type object-list
        \\object-list Branches
   A19, \\field Inlet Branch 17 Name
        \\type object-list
        \\object-list Branches
   A20, \\field Inlet Branch 18 Name
        \\type object-list
        \\object-list Branches
   A21, \\field Inlet Branch 19 Name
        \\type object-list
        \\object-list Branches
   A22, \\field Inlet Branch 20 Name
        \\type object-list
        \\object-list Branches
   A23, \\field Inlet Branch 21 Name
        \\type object-list
        \\object-list Branches
   A24, \\field Inlet Branch 22 Name
        \\type object-list
        \\object-list Branches
   A25, \\field Inlet Branch 23 Name
        \\type object-list
        \\object-list Branches
   A26, \\field Inlet Branch 24 Name
        \\type object-list
        \\object-list Branches
   A27, \\field Inlet Branch 25 Name
        \\type object-list
        \\object-list Branches
   A28, \\field Inlet Branch 26 Name
        \\type object-list
        \\object-list Branches
   A29, \\field Inlet Branch 27 Name
        \\type object-list
        \\object-list Branches
   A30, \\field Inlet Branch 28 Name
        \\type object-list
        \\object-list Branches
   A31, \\field Inlet Branch 29 Name
        \\type object-list
        \\object-list Branches
   A32, \\field Inlet Branch 30 Name
        \\type object-list
        \\object-list Branches
   A33, \\field Inlet Branch 31 Name
        \\type object-list
        \\object-list Branches
   A34, \\field Inlet Branch 32 Name
        \\type object-list
        \\object-list Branches
   A35, \\field Inlet Branch 33 Name
        \\type object-list
        \\object-list Branches
   A36, \\field Inlet Branch 34 Name
        \\type object-list
        \\object-list Branches
   A37, \\field Inlet Branch 35 Name
        \\type object-list
        \\object-list Branches
   A38, \\field Inlet Branch 36 Name
        \\type object-list
        \\object-list Branches
   A39, \\field Inlet Branch 37 Name
        \\type object-list
        \\object-list Branches
   A40, \\field Inlet Branch 38 Name
        \\type object-list
        \\object-list Branches
   A41, \\field Inlet Branch 39 Name
        \\type object-list
        \\object-list Branches
   A42, \\field Inlet Branch 40 Name
        \\type object-list
        \\object-list Branches
   A43, \\field Inlet Branch 41 Name
        \\type object-list
        \\object-list Branches
   A44, \\field Inlet Branch 42 Name
        \\type object-list
        \\object-list Branches
   A45, \\field Inlet Branch 43 Name
        \\type object-list
        \\object-list Branches
   A46, \\field Inlet Branch 44 Name
        \\type object-list
        \\object-list Branches
   A47, \\field Inlet Branch 45 Name
        \\type object-list
        \\object-list Branches
   A48, \\field Inlet Branch 46 Name
        \\type object-list
        \\object-list Branches
   A49, \\field Inlet Branch 47 Name
        \\type object-list
        \\object-list Branches
   A50, \\field Inlet Branch 48 Name
        \\type object-list
        \\object-list Branches
   A51, \\field Inlet Branch 49 Name
        \\type object-list
        \\object-list Branches
   A52, \\field Inlet Branch 50 Name
        \\type object-list
        \\object-list Branches
   A53, \\field Inlet Branch 51 Name
        \\type object-list
        \\object-list Branches
   A54, \\field Inlet Branch 52 Name
        \\type object-list
        \\object-list Branches
   A55, \\field Inlet Branch 53 Name
        \\type object-list
        \\object-list Branches
   A56, \\field Inlet Branch 54 Name
        \\type object-list
        \\object-list Branches
   A57, \\field Inlet Branch 55 Name
        \\type object-list
        \\object-list Branches
   A58, \\field Inlet Branch 56 Name
        \\type object-list
        \\object-list Branches
   A59, \\field Inlet Branch 57 Name
        \\type object-list
        \\object-list Branches
   A60, \\field Inlet Branch 58 Name
        \\type object-list
        \\object-list Branches
   A61, \\field Inlet Branch 59 Name
        \\type object-list
        \\object-list Branches
   A62, \\field Inlet Branch 60 Name
        \\type object-list
        \\object-list Branches
   A63, \\field Inlet Branch 61 Name
        \\type object-list
        \\object-list Branches
   A64, \\field Inlet Branch 62 Name
        \\type object-list
        \\object-list Branches
   A65, \\field Inlet Branch 63 Name
        \\type object-list
        \\object-list Branches
   A66, \\field Inlet Branch 64 Name
        \\type object-list
        \\object-list Branches
   A67, \\field Inlet Branch 65 Name
        \\type object-list
        \\object-list Branches
   A68, \\field Inlet Branch 66 Name
        \\type object-list
        \\object-list Branches
   A69, \\field Inlet Branch 67 Name
        \\type object-list
        \\object-list Branches
   A70, \\field Inlet Branch 68 Name
        \\type object-list
        \\object-list Branches
   A71, \\field Inlet Branch 69 Name
        \\type object-list
        \\object-list Branches
   A72, \\field Inlet Branch 70 Name
        \\type object-list
        \\object-list Branches
   A73, \\field Inlet Branch 71 Name
        \\type object-list
        \\object-list Branches
   A74, \\field Inlet Branch 72 Name
        \\type object-list
        \\object-list Branches
   A75, \\field Inlet Branch 73 Name
        \\type object-list
        \\object-list Branches
   A76, \\field Inlet Branch 74 Name
        \\type object-list
        \\object-list Branches
   A77, \\field Inlet Branch 75 Name
        \\type object-list
        \\object-list Branches
   A78, \\field Inlet Branch 76 Name
        \\type object-list
        \\object-list Branches
   A79, \\field Inlet Branch 77 Name
        \\type object-list
        \\object-list Branches
   A80, \\field Inlet Branch 78 Name
        \\type object-list
        \\object-list Branches
   A81, \\field Inlet Branch 79 Name
        \\type object-list
        \\object-list Branches
   A82, \\field Inlet Branch 80 Name
        \\type object-list
        \\object-list Branches
   A83, \\field Inlet Branch 81 Name
        \\type object-list
        \\object-list Branches
   A84, \\field Inlet Branch 82 Name
        \\type object-list
        \\object-list Branches
   A85, \\field Inlet Branch 83 Name
        \\type object-list
        \\object-list Branches
   A86, \\field Inlet Branch 84 Name
        \\type object-list
        \\object-list Branches
   A87, \\field Inlet Branch 85 Name
        \\type object-list
        \\object-list Branches
   A88, \\field Inlet Branch 86 Name
        \\type object-list
        \\object-list Branches
   A89, \\field Inlet Branch 87 Name
        \\type object-list
        \\object-list Branches
   A90, \\field Inlet Branch 88 Name
        \\type object-list
        \\object-list Branches
   A91, \\field Inlet Branch 89 Name
        \\type object-list
        \\object-list Branches
   A92, \\field Inlet Branch 90 Name
        \\type object-list
        \\object-list Branches
   A93, \\field Inlet Branch 91 Name
        \\type object-list
        \\object-list Branches
   A94, \\field Inlet Branch 92 Name
        \\type object-list
        \\object-list Branches
   A95, \\field Inlet Branch 93 Name
        \\type object-list
        \\object-list Branches
   A96, \\field Inlet Branch 94 Name
        \\type object-list
        \\object-list Branches
   A97, \\field Inlet Branch 95 Name
        \\type object-list
        \\object-list Branches
   A98, \\field Inlet Branch 96 Name
        \\type object-list
        \\object-list Branches
   A99, \\field Inlet Branch 97 Name
        \\type object-list
        \\object-list Branches
  A100, \\field Inlet Branch 98 Name
        \\type object-list
        \\object-list Branches
  A101, \\field Inlet Branch 99 Name
        \\type object-list
        \\object-list Branches
  A102, \\field Inlet Branch 100 Name
        \\type object-list
        \\object-list Branches
  A103,A104,A105,A106,A107,A108,A109,A110,A111,A112, \\note fields as indicated
  A113,A114,A115,A116,A117,A118,A119,A120,A121,A122, \\note fields as indicated
  A123,A124,A125,A126,A127,A128,A129,A130,A131,A132, \\note fields as indicated
  A133,A134,A135,A136,A137,A138,A139,A140,A141,A142, \\note fields as indicated
  A143,A144,A145,A146,A147,A148,A149,A150,A151,A152, \\note fields as indicated
  A153,A154,A155,A156,A157,A158,A159,A160,A161,A162, \\note fields as indicated
  A163,A164,A165,A166,A167,A168,A169,A170,A171,A172, \\note fields as indicated
  A173,A174,A175,A176,A177,A178,A179,A180,A181,A182, \\note fields as indicated
  A183,A184,A185,A186,A187,A188,A189,A190,A191,A192, \\note fields as indicated
  A193,A194,A195,A196,A197,A198,A199,A200,A201,A202, \\note fields as indicated
  A203,A204,A205,A206,A207,A208,A209,A210,A211,A212, \\note fields as indicated
  A213,A214,A215,A216,A217,A218,A219,A220,A221,A222, \\note fields as indicated
  A223,A224,A225,A226,A227,A228,A229,A230,A231,A232, \\note fields as indicated
  A233,A234,A235,A236,A237,A238,A239,A240,A241,A242, \\note fields as indicated
  A243,A244,A245,A246,A247,A248,A249,A250,A251,A252, \\note fields as indicated
  A253,A254,A255,A256,A257,A258,A259,A260,A261,A262, \\note fields as indicated
  A263,A264,A265,A266,A267,A268,A269,A270,A271,A272, \\note fields as indicated
  A273,A274,A275,A276,A277,A278,A279,A280,A281,A282, \\note fields as indicated
  A283,A284,A285,A286,A287,A288,A289,A290,A291,A292, \\note fields as indicated
  A293,A294,A295,A296,A297,A298,A299,A300,A301,A302, \\note fields as indicated
  A303,A304,A305,A306,A307,A308,A309,A310,A311,A312, \\note fields as indicated
  A313,A314,A315,A316,A317,A318,A319,A320,A321,A322, \\note fields as indicated
  A323,A324,A325,A326,A327,A328,A329,A330,A331,A332, \\note fields as indicated
  A333,A334,A335,A336,A337,A338,A339,A340,A341,A342, \\note fields as indicated
  A343,A344,A345,A346,A347,A348,A349,A350,A351,A352, \\note fields as indicated
  A353,A354,A355,A356,A357,A358,A359,A360,A361,A362, \\note fields as indicated
  A363,A364,A365,A366,A367,A368,A369,A370,A371,A372, \\note fields as indicated
  A373,A374,A375,A376,A377,A378,A379,A380,A381,A382, \\note fields as indicated
  A383,A384,A385,A386,A387,A388,A389,A390,A391,A392, \\note fields as indicated
  A393,A394,A395,A396,A397,A398,A399,A400,A401,A402, \\note fields as indicated
  A403,A404,A405,A406,A407,A408,A409,A410,A411,A412, \\note fields as indicated
  A413,A414,A415,A416,A417,A418,A419,A420,A421,A422, \\note fields as indicated
  A423,A424,A425,A426,A427,A428,A429,A430,A431,A432, \\note fields as indicated
  A433,A434,A435,A436,A437,A438,A439,A440,A441,A442, \\note fields as indicated
  A443,A444,A445,A446,A447,A448,A449,A450,A451,A452, \\note fields as indicated
  A453,A454,A455,A456,A457,A458,A459,A460,A461,A462, \\note fields as indicated
  A463,A464,A465,A466,A467,A468,A469,A470,A471,A472, \\note fields as indicated
  A473,A474,A475,A476,A477,A478,A479,A480,A481,A482, \\note fields as indicated
  A483,A484,A485,A486,A487,A488,A489,A490,A491,A492, \\note fields as indicated
  A493,A494,A495,A496,A497,A498,A499,A500,A501,A502; \\note fields as indicated

ConnectorList,
        \\memo only two connectors allowed per loop
        \\memo if two entered, one must be Connector:Splitter and one must be Connector:Mixer
    A1, \\field Name
        \\required-field
        \\reference ConnectorLists
    A2, \\field Connector 1 Object Type
        \\required-field
        \\type choice
        \\key Connector:Splitter
        \\key Connector:Mixer
    A3, \\field Connector 1 Name
        \\required-field
    A4, \\field Connector 2 Object Type
        \\type choice
        \\key Connector:Splitter
        \\key Connector:Mixer
    A5; \\field Connector 2 Name

NodeList,
  \\min-fields 2
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo This object is used in places where lists of nodes may be
       \\memo needed, e.g. ZoneHVAC:EquipmentConnections field Zone Air Inlet Node or NodeList Name
  A1 , \\field Name
       \\required-field
       \\type node
       \\reference NodeLists
  A2 , \\field Node 1 Name
       \\required-field
       \\begin-extensible
       \\type node
  A3 , \\field Node 2 Name
       \\type node
  A4 , \\field Node 3 Name
       \\type node
  A5 , \\field Node 4 Name
       \\type node
  A6 , \\field Node 5 Name
       \\type node
  A7 , \\field Node 6 Name
       \\type node
  A8 , \\field Node 7 Name
       \\type node
  A9 , \\field Node 8 Name
       \\type node
  A10, \\field Node 9 Name
       \\type node
  A11, \\field Node 10 Name
       \\type node
  A12, \\field Node 11 Name
       \\type node
  A13, \\field Node 12 Name
       \\type node
  A14, \\field Node 13 Name
       \\type node
  A15, \\field Node 14 Name
       \\type node
  A16, \\field Node 15 Name
       \\type node
  A17, \\field Node 16 Name
       \\type node
  A18, \\field Node 17 Name
       \\type node
  A19, \\field Node 18 Name
       \\type node
  A20, \\field Node 19 Name
       \\type node
  A21, \\field Node 20 Name
       \\type node
  A22, \\field Node 21 Name
       \\type node
  A23, \\field Node 22 Name
       \\type node
  A24, \\field Node 23 Name
       \\type node
  A25, \\field Node 24 Name
       \\type node
  A26, \\field Node 25 Name
       \\type node
  A27, \\field Node 26 Name
       \\type node
  A28, \\field Node 27 Name
       \\type node
  A29, \\field Node 28 Name
       \\type node
  A30, \\field Node 29 Name
       \\type node
  A31, \\field Node 30 Name
       \\type node
  A32, \\field Node 31 Name
       \\type node
  A33, \\field Node 32 Name
       \\type node
  A34, \\field Node 33 Name
       \\type node
  A35, \\field Node 34 Name
       \\type node
  A36, \\field Node 35 Name
       \\type node
  A37, \\field Node 36 Name
       \\type node
  A38, \\field Node 37 Name
       \\type node
  A39, \\field Node 38 Name
       \\type node
  A40, \\field Node 39 Name
       \\type node
  A41, \\field Node 40 Name
       \\type node
  A42, \\field Node 41 Name
       \\type node
  A43, \\field Node 42 Name
       \\type node
  A44, \\field Node 43 Name
       \\type node
  A45, \\field Node 44 Name
       \\type node
  A46, \\field Node 45 Name
       \\type node
  A47, \\field Node 46 Name
       \\type node
  A48, \\field Node 47 Name
       \\type node
  A49, \\field Node 48 Name
       \\type node
  A50, \\field Node 49 Name
       \\type node
  A51, \\field Node 50 Name
       \\type node
  A52, \\field Node 51 Name
       \\type node
  A53, \\field Node 52 Name
       \\type node
  A54, \\field Node 53 Name
       \\type node
  A55, \\field Node 54 Name
       \\type node
  A56, \\field Node 55 Name
       \\type node
  A57, \\field Node 56 Name
       \\type node
  A58, \\field Node 57 Name
       \\type node
  A59, \\field Node 58 Name
       \\type node
  A60, \\field Node 59 Name
       \\type node
  A61, \\field Node 60 Name
       \\type node
  A62, \\field Node 61 Name
       \\type node
  A63, \\field Node 62 Name
       \\type node
  A64, \\field Node 63 Name
       \\type node
  A65, \\field Node 64 Name
       \\type node
  A66, \\field Node 65 Name
       \\type node
  A67, \\field Node 66 Name
       \\type node
  A68, \\field Node 67 Name
       \\type node
  A69, \\field Node 68 Name
       \\type node
  A70, \\field Node 69 Name
       \\type node
  A71, \\field Node 70 Name
       \\type node
  A72, \\field Node 71 Name
       \\type node
  A73, \\field Node 72 Name
       \\type node
  A74, \\field Node 73 Name
       \\type node
  A75, \\field Node 74 Name
       \\type node
  A76, \\field Node 75 Name
       \\type node
  A77, \\field Node 76 Name
       \\type node
  A78, \\field Node 77 Name
       \\type node
  A79, \\field Node 78 Name
       \\type node
  A80, \\field Node 79 Name
       \\type node
  A81, \\field Node 80 Name
       \\type node
  A82, \\field Node 81 Name
       \\type node
  A83, \\field Node 82 Name
       \\type node
  A84, \\field Node 83 Name
       \\type node
  A85, \\field Node 84 Name
       \\type node
  A86, \\field Node 85 Name
       \\type node
  A87, \\field Node 86 Name
       \\type node
  A88, \\field Node 87 Name
       \\type node
  A89, \\field Node 88 Name
       \\type node
  A90, \\field Node 89 Name
       \\type node
  A91, \\field Node 90 Name
       \\type node
  A92, \\field Node 91 Name
       \\type node
  A93, \\field Node 92 Name
       \\type node
  A94, \\field Node 93 Name
       \\type node
  A95, \\field Node 94 Name
       \\type node
  A96, \\field Node 95 Name
       \\type node
  A97, \\field Node 96 Name
       \\type node
  A98, \\field Node 97 Name
       \\type node
  A99, \\field Node 98 Name
       \\type node
  A100, \\field Node 99 Name
        \\type node
  A101, \\field Node 100 Name
        \\type node
  A102,A103,A104,A105,A106,A107,A108,A109,A110,A111, \\note fields as indicated
  A112,A113,A114,A115,A116,A117,A118,A119,A120,A121, \\note fields as indicated
  A122,A123,A124,A125,A126,A127,A128,A129,A130,A131, \\note fields as indicated
  A132,A133,A134,A135,A136,A137,A138,A139,A140,A141, \\note fields as indicated
  A142,A143,A144,A145,A146,A147,A148,A149,A150,A151, \\note fields as indicated
  A152,A153,A154,A155,A156,A157,A158,A159,A160,A161, \\note fields as indicated
  A162,A163,A164,A165,A166,A167,A168,A169,A170,A171, \\note fields as indicated
  A172,A173,A174,A175,A176,A177,A178,A179,A180,A181, \\note fields as indicated
  A182,A183,A184,A185,A186,A187,A188,A189,A190,A191, \\note fields as indicated
  A192,A193,A194,A195,A196,A197,A198,A199,A200,A201, \\note fields as indicated
  A202,A203,A204,A205,A206,A207,A208,A209,A210,A211, \\note fields as indicated
  A212,A213,A214,A215,A216,A217,A218,A219,A220,A221, \\note fields as indicated
  A222,A223,A224,A225,A226,A227,A228,A229,A230,A231, \\note fields as indicated
  A232,A233,A234,A235,A236,A237,A238,A239,A240,A241, \\note fields as indicated
  A242,A243,A244,A245,A246,A247,A248,A249,A250,A251, \\note fields as indicated
  A252,A253,A254,A255,A256,A257,A258,A259,A260,A261, \\note fields as indicated
  A262,A263,A264,A265,A266,A267,A268,A269,A270,A271, \\note fields as indicated
  A272,A273,A274,A275,A276,A277,A278,A279,A280,A281, \\note fields as indicated
  A282,A283,A284,A285,A286,A287,A288,A289,A290,A291, \\note fields as indicated
  A292,A293,A294,A295,A296,A297,A298,A299,A300,A301, \\note fields as indicated
  A302,A303,A304,A305,A306,A307,A308,A309,A310,A311, \\note fields as indicated
  A312,A313,A314,A315,A316,A317,A318,A319,A320,A321, \\note fields as indicated
  A322,A323,A324,A325,A326,A327,A328,A329,A330,A331, \\note fields as indicated
  A332,A333,A334,A335,A336,A337,A338,A339,A340,A341, \\note fields as indicated
  A342,A343,A344,A345,A346,A347,A348,A349,A350,A351, \\note fields as indicated
  A352,A353,A354,A355,A356,A357,A358,A359,A360,A361, \\note fields as indicated
  A362,A363,A364,A365,A366,A367,A368,A369,A370,A371, \\note fields as indicated
  A372,A373,A374,A375,A376,A377,A378,A379,A380,A381, \\note fields as indicated
  A382,A383,A384,A385,A386,A387,A388,A389,A390,A391, \\note fields as indicated
  A392,A393,A394,A395,A396,A397,A398,A399,A400,A401, \\note fields as indicated
  A402,A403,A404,A405,A406,A407,A408,A409,A410,A411, \\note fields as indicated
  A412,A413,A414,A415,A416,A417,A418,A419,A420,A421, \\note fields as indicated
  A422,A423,A424,A425,A426,A427,A428,A429,A430,A431, \\note fields as indicated
  A432,A433,A434,A435,A436,A437,A438,A439,A440,A441, \\note fields as indicated
  A442,A443,A444,A445,A446,A447,A448,A449,A450,A451, \\note fields as indicated
  A452,A453,A454,A455,A456,A457,A458,A459,A460,A461, \\note fields as indicated
  A462,A463,A464,A465,A466,A467,A468,A469,A470,A471, \\note fields as indicated
  A472,A473,A474,A475,A476,A477,A478,A479,A480,A481, \\note fields as indicated
  A482,A483,A484,A485,A486,A487,A488,A489,A490,A491, \\note fields as indicated
  A492,A493,A494,A495,A496,A497,A498,A499,A500,A501; \\note fields as indicated

OutdoorAir:Node,
       \\memo This object sets the temperature and humidity conditions
       \\memo for an outdoor air node.  It allows the height above ground to be
       \\memo specified.  This object may be used more than once.
       \\memo The same node name may not appear in both an OutdoorAir:Node object and
       \\memo an OutdoorAir:NodeList object.
  A1,  \\field Name
       \\required-field
       \\type node
  N1 ; \\field Height Above Ground
       \\note A value less than zero indicates that the height will be ignored and the weather file conditions will be used.
       \\type real
       \\units m
       \\default -1.0

OutdoorAir:NodeList,
       \\extensible:1 Just duplicate last field and comments (changing numbering, please)
       \\memo This object sets the temperature and humidity conditions
       \\memo for an outdoor air node using the weather data values.
       \\memo to vary outdoor air node conditions with height above ground
       \\memo use OutdoorAir:Node instead of this object.
       \\memo This object may be used more than once.
       \\memo The same node name may not appear in both an OutdoorAir:Node object and
       \\memo an OutdoorAir:NodeList object.
  A1 , \\field Node or NodeList Name 1
       \\required-field
       \\type node
       \\begin-extensible
  A2 , \\field Node or NodeList Name 2
       \\type node
  A3 , \\field Node or NodeList Name 3
       \\type node
  A4 , \\field Node or NodeList Name 4
       \\type node
  A5 , \\field Node or NodeList Name 5
       \\type node
  A6 , \\field Node or NodeList Name 6
       \\type node
  A7 , \\field Node or NodeList Name 7
       \\type node
  A8 , \\field Node or NodeList Name 8
       \\type node
  A9 , \\field Node or NodeList Name 9
       \\type node
  A10, \\field Node or NodeList Name 10
       \\type node
  A11, \\field Node or NodeList Name 11
       \\type node
  A12, \\field Node or NodeList Name 12
       \\type node
  A13, \\field Node or NodeList Name 13
       \\type node
  A14, \\field Node or NodeList Name 14
       \\type node
  A15, \\field Node or NodeList Name 15
       \\type node
  A16, \\field Node or NodeList Name 16
       \\type node
  A17, \\field Node or NodeList Name 17
       \\type node
  A18, \\field Node or NodeList Name 18
       \\type node
  A19, \\field Node or NodeList Name 19
       \\type node
  A20, \\field Node or NodeList Name 20
       \\type node
  A21, \\field Node or NodeList Name 21
       \\type node
  A22, \\field Node or NodeList Name 22
       \\type node
  A23, \\field Node or NodeList Name 23
       \\type node
  A24, \\field Node or NodeList Name 24
       \\type node
  A25, \\field Node or NodeList Name 25
       \\type node
  A26, \\field Node or NodeList Name 26
       \\type node
  A27, \\field Node or NodeList Name 27
       \\type node
  A28, \\field Node or NodeList Name 28
       \\type node
  A29, \\field Node or NodeList Name 29
       \\type node
  A30, \\field Node or NodeList Name 30
       \\type node
  A31, \\field Node or NodeList Name 31
       \\type node
  A32, \\field Node or NodeList Name 32
       \\type node
  A33, \\field Node or NodeList Name 33
       \\type node
  A34, \\field Node or NodeList Name 34
       \\type node
  A35, \\field Node or NodeList Name 35
       \\type node
  A36, \\field Node or NodeList Name 36
       \\type node
  A37, \\field Node or NodeList Name 37
       \\type node
  A38, \\field Node or NodeList Name 38
       \\type node
  A39, \\field Node or NodeList Name 39
       \\type node
  A40, \\field Node or NodeList Name 40
       \\type node
  A41, \\field Node or NodeList Name 41
       \\type node
  A42, \\field Node or NodeList Name 42
       \\type node
  A43, \\field Node or NodeList Name 43
       \\type node
  A44, \\field Node or NodeList Name 44
       \\type node
  A45, \\field Node or NodeList Name 45
       \\type node
  A46, \\field Node or NodeList Name 46
       \\type node
  A47, \\field Node or NodeList Name 47
       \\type node
  A48, \\field Node or NodeList Name 48
       \\type node
  A49, \\field Node or NodeList Name 49
       \\type node
  A50, \\field Node or NodeList Name 50
       \\type node
  A51, \\field Node or NodeList Name 51
       \\type node
  A52, \\field Node or NodeList Name 52
       \\type node
  A53, \\field Node or NodeList Name 53
       \\type node
  A54, \\field Node or NodeList Name 54
       \\type node
  A55, \\field Node or NodeList Name 55
       \\type node
  A56, \\field Node or NodeList Name 56
       \\type node
  A57, \\field Node or NodeList Name 57
       \\type node
  A58, \\field Node or NodeList Name 58
       \\type node
  A59, \\field Node or NodeList Name 59
       \\type node
  A60, \\field Node or NodeList Name 60
       \\type node
  A61, \\field Node or NodeList Name 61
       \\type node
  A62, \\field Node or NodeList Name 62
       \\type node
  A63, \\field Node or NodeList Name 63
       \\type node
  A64, \\field Node or NodeList Name 64
       \\type node
  A65, \\field Node or NodeList Name 65
       \\type node
  A66, \\field Node or NodeList Name 66
       \\type node
  A67, \\field Node or NodeList Name 67
       \\type node
  A68, \\field Node or NodeList Name 68
       \\type node
  A69, \\field Node or NodeList Name 69
       \\type node
  A70, \\field Node or NodeList Name 70
       \\type node
  A71, \\field Node or NodeList Name 71
       \\type node
  A72, \\field Node or NodeList Name 72
       \\type node
  A73, \\field Node or NodeList Name 73
       \\type node
  A74, \\field Node or NodeList Name 74
       \\type node
  A75, \\field Node or NodeList Name 75
       \\type node
  A76, \\field Node or NodeList Name 76
       \\type node
  A77, \\field Node or NodeList Name 77
       \\type node
  A78, \\field Node or NodeList Name 78
       \\type node
  A79, \\field Node or NodeList Name 79
       \\type node
  A80, \\field Node or NodeList Name 80
       \\type node
  A81, \\field Node or NodeList Name 81
       \\type node
  A82, \\field Node or NodeList Name 82
       \\type node
  A83, \\field Node or NodeList Name 83
       \\type node
  A84, \\field Node or NodeList Name 84
       \\type node
  A85, \\field Node or NodeList Name 85
       \\type node
  A86, \\field Node or NodeList Name 86
       \\type node
  A87, \\field Node or NodeList Name 87
       \\type node
  A88, \\field Node or NodeList Name 88
       \\type node
  A89, \\field Node or NodeList Name 89
       \\type node
  A90, \\field Node or NodeList Name 90
       \\type node
  A91, \\field Node or NodeList Name 91
       \\type node
  A92, \\field Node or NodeList Name 92
       \\type node
  A93, \\field Node or NodeList Name 93
       \\type node
  A94, \\field Node or NodeList Name 94
       \\type node
  A95, \\field Node or NodeList Name 95
       \\type node
  A96, \\field Node or NodeList Name 96
       \\type node
  A97, \\field Node or NodeList Name 97
       \\type node
  A98, \\field Node or NodeList Name 98
       \\type node
  A99, \\field Node or NodeList Name 99
       \\type node
  A100, \\field Node or NodeList Name 100
        \\type node
  A101,A102,A103,A104,A105,A106,A107,A108,A109,A110, \\note fields as indicated
  A111,A112,A113,A114,A115,A116,A117,A118,A119,A120, \\note fields as indicated
  A121,A122,A123,A124,A125,A126,A127,A128,A129,A130, \\note fields as indicated
  A131,A132,A133,A134,A135,A136,A137,A138,A139,A140, \\note fields as indicated
  A141,A142,A143,A144,A145,A146,A147,A148,A149,A150, \\note fields as indicated
  A151,A152,A153,A154,A155,A156,A157,A158,A159,A160, \\note fields as indicated
  A161,A162,A163,A164,A165,A166,A167,A168,A169,A170, \\note fields as indicated
  A171,A172,A173,A174,A175,A176,A177,A178,A179,A180, \\note fields as indicated
  A181,A182,A183,A184,A185,A186,A187,A188,A189,A190, \\note fields as indicated
  A191,A192,A193,A194,A195,A196,A197,A198,A199,A200, \\note fields as indicated
  A201,A202,A203,A204,A205,A206,A207,A208,A209,A210, \\note fields as indicated
  A211,A212,A213,A214,A215,A216,A217,A218,A219,A220, \\note fields as indicated
  A221,A222,A223,A224,A225,A226,A227,A228,A229,A230, \\note fields as indicated
  A231,A232,A233,A234,A235,A236,A237,A238,A239,A240, \\note fields as indicated
  A241,A242,A243,A244,A245,A246,A247,A248,A249,A250, \\note fields as indicated
  A251,A252,A253,A254,A255,A256,A257,A258,A259,A260, \\note fields as indicated
  A261,A262,A263,A264,A265,A266,A267,A268,A269,A270, \\note fields as indicated
  A271,A272,A273,A274,A275,A276,A277,A278,A279,A280, \\note fields as indicated
  A281,A282,A283,A284,A285,A286,A287,A288,A289,A290, \\note fields as indicated
  A291,A292,A293,A294,A295,A296,A297,A298,A299,A300, \\note fields as indicated
  A301,A302,A303,A304,A305,A306,A307,A308,A309,A310, \\note fields as indicated
  A311,A312,A313,A314,A315,A316,A317,A318,A319,A320, \\note fields as indicated
  A321,A322,A323,A324,A325,A326,A327,A328,A329,A330, \\note fields as indicated
  A331,A332,A333,A334,A335,A336,A337,A338,A339,A340, \\note fields as indicated
  A341,A342,A343,A344,A345,A346,A347,A348,A349,A350, \\note fields as indicated
  A351,A352,A353,A354,A355,A356,A357,A358,A359,A360, \\note fields as indicated
  A361,A362,A363,A364,A365,A366,A367,A368,A369,A370, \\note fields as indicated
  A371,A372,A373,A374,A375,A376,A377,A378,A379,A380, \\note fields as indicated
  A381,A382,A383,A384,A385,A386,A387,A388,A389,A390, \\note fields as indicated
  A391,A392,A393,A394,A395,A396,A397,A398,A399,A400, \\note fields as indicated
  A401,A402,A403,A404,A405,A406,A407,A408,A409,A410, \\note fields as indicated
  A411,A412,A413,A414,A415,A416,A417,A418,A419,A420, \\note fields as indicated
  A421,A422,A423,A424,A425,A426,A427,A428,A429,A430, \\note fields as indicated
  A431,A432,A433,A434,A435,A436,A437,A438,A439,A440, \\note fields as indicated
  A441,A442,A443,A444,A445,A446,A447,A448,A449,A450, \\note fields as indicated
  A451,A452,A453,A454,A455,A456,A457,A458,A459,A460, \\note fields as indicated
  A461,A462,A463,A464,A465,A466,A467,A468,A469,A470, \\note fields as indicated
  A471,A472,A473,A474,A475,A476,A477,A478,A479,A480, \\note fields as indicated
  A481,A482,A483,A484,A485,A486,A487,A488,A489,A490, \\note fields as indicated
  A491,A492,A493,A494,A495,A496,A497,A498,A499,A500; \\note fields as indicated

Pipe:Adiabatic,
       \\memo Passes Inlet Node state variables to Outlet Node state variables
   A1,  \\field Name
        \\required-field
   A2, \\field Inlet Node Name
        \\required-field
        \\type node
  A3; \\field Outlet Node Name
        \\required-field
        \\type node

Pipe:Adiabatic:Steam,
       \\memo Passes Inlet Node state variables to Outlet Node state variables
   A1,  \\field Name
        \\required-field
   A2, \\field Inlet Node Name
        \\required-field
        \\type node
  A3; \\field Outlet Node Name
        \\required-field
        \\type node

Pipe:Indoor,
        \\memo Pipe model with transport delay and heat transfer to the environment.
   A1,  \\field Name
        \\required-field
   A2,  \\field Construction Name
        \\required-field
        \\type object-list
        \\object-list ConstructionNames
   A3,  \\field Fluid Inlet Node Name
        \\type node
        \\required-field
   A4,  \\field Fluid Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Environment Type
        \\type choice
        \\key Zone
        \\key Schedule
        \\default Zone
   A6,  \\field Ambient Temperature Zone Name
        \\type object-list
        \\object-list ZoneNames
   A7,  \\field Ambient Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   A8,  \\field Ambient Air Velocity Schedule Name
        \\type object-list
        \\object-list ScheduleNames
   N1,  \\field Pipe Inside Diameter
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N2;  \\field Pipe Length
        \\type real
        \\units m
        \\minimum> 0.0

Pipe:Outdoor,
        \\memo Pipe model with transport delay and heat transfer to the environment.
   A1,  \\field Name
        \\required-field
   A2,  \\field Construction Name
        \\required-field
        \\type object-list
        \\object-list ConstructionNames
   A3,  \\field Fluid Inlet Node Name
        \\type node
        \\required-field
   A4,  \\field Fluid Outlet Node Name
        \\required-field
        \\type node
   A5,  \\field Ambient Temperature Outdoor Air Node Name
        \\type node
   N1,  \\field Pipe Inside Diameter
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N2;  \\field Pipe Length
        \\type real
        \\units m
        \\minimum> 0.0

Pipe:Underground,
        \\memo Buried Pipe model: For pipes buried at a depth less
        \\memo than one meter, this is an alternative object to:
        \\memo HeatExchanger:Surface
   A1,  \\field Name
        \\required-field
   A2,  \\field Construction Name
        \\required-field
        \\type object-list
        \\object-list ConstructionNames
   A3,  \\field Fluid Inlet Node Name
        \\type node
        \\required-field
   A4,  \\field Fluid Outlet Node Name
        \\type node
        \\required-field
   A5,  \\field Sun Exposure
        \\required-field
        \\type choice
        \\key SunExposed
        \\key NoSun
   N1,  \\field Pipe Inside Diameter
        \\note pipe thickness is defined in the Construction object
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N2,  \\field Pipe Length
        \\type real
        \\units m
        \\minimum> 0.0
   A6,  \\field Soil Material Name
        \\type alpha
        \\required-field
   N3,  \\field Average Soil Surface Temperature
        \\note optional
        \\units C
        \\type real
        \\default 0
   N4,  \\field Amplitude of Soil Surface Temperature
        \\note optional
        \\units C
        \\type real
        \\minimum 0
        \\default 0
   N5;  \\field Phase Constant of Soil Surface Temperature
        \\note optional
        \\units days
        \\type real
        \\minimum 0
        \\default 0

PipingSystem:Underground:Domain,
        \\memo The ground domain object for underground piping system simulation.
        \\min-fields 31
        \\extensible:1
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference PipingSystemUndergroundDomainNames
   N1,  \\field Xmax
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\note Domain extent in the local 'X' direction
   N2,  \\field Ymax
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\note Domain extent in the local 'Y' direction
   N3,  \\field Zmax
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\note Domain extent in the local 'Y' direction
   N4,  \\field X-Direction Mesh Density Parameter
        \\type integer
        \\minimum> 0
        \\default 4
        \\note If mesh type is symmetric geometric, this should be an even number.
   A2,  \\field X-Direction Mesh Type
        \\required-field
        \\type choice
        \\key Uniform
        \\key SymmetricGeometric
   N5,  \\field X-Direction Geometric Coefficient
        \\note optional
        \\type real
        \\minimum 1
        \\default 1.3
        \\maximum 2
        \\note Only used if mesh type is symmetric geometric
   N6,  \\field Y-Direction Mesh Density Parameter
        \\type integer
        \\minimum> 0
        \\default 4
        \\note If mesh type is symmetric geometric, this should be an even number.
   A3,  \\field Y-Direction Mesh Type
        \\required-field
        \\type choice
        \\key Uniform
        \\key SymmetricGeometric
   N7,  \\field Y-Direction Geometric Coefficient
        \\note optional
        \\type real
        \\minimum 1
        \\default 1.3
        \\maximum 2
        \\note Only used if mesh type is symmetric geometric
   N8,  \\field Z-Direction Mesh Density Parameter
        \\type integer
        \\minimum> 0
        \\default 4
        \\note If mesh type is symmetric geometric, this should be an even number.
   A4,  \\field Z-Direction Mesh Type
        \\required-field
        \\type choice
        \\key Uniform
        \\key SymmetricGeometric
   N9,  \\field Z-Direction Geometric Coefficient
        \\note optional
        \\type real
        \\minimum 1
        \\default 1.3
        \\maximum 2
        \\note Only used if mesh type is symmetric geometric
   N10, \\field Soil Thermal Conductivity
        \\required-field
        \\type real
        \\units W/m-K
        \\minimum> 0
   N11, \\field Soil Density
        \\required-field
        \\type real
        \\units kg/m3
        \\minimum> 0
   N12, \\field Soil Specific Heat
        \\required-field
        \\type real
        \\units J/kg-K
        \\minimum> 0
        \\note This is a dry soil property, which is adjusted for freezing effects
        \\note   by the simulation algorithm.
   N13, \\field Soil Moisture Content Volume Fraction
        \\type real
        \\units percent
        \\minimum 0
        \\maximum 100
        \\default 30
   N14, \\field Soil Moisture Content Volume Fraction at Saturation
        \\type real
        \\units percent
        \\minimum 0
        \\maximum 100
        \\default 50
   N15, \\field Kusuda-Achenbach Average Surface Temperature
        \\required-field
        \\type real
        \\units C
   N16, \\field Kusuda-Achenbach Average Amplitude of Surface Temperature
        \\required-field
        \\type real
        \\units C
   N17, \\field Kusuda-Achenbach Phase Shift of Minimum Surface Temperature
        \\required-field
        \\type real
        \\units days
   A5,  \\field This Domain Includes Basement Surface Interaction
        \\type choice
        \\key Yes
        \\key No
        \\default No
        \\note if Yes, then the following basement inputs are used
        \\note if No, then the following basement inputs are *ignored*
   N18, \\field Width of Basement Floor in Ground Domain
        \\type real
        \\units m
        \\note Required only if Domain Has Basement Interaction
   N19, \\field Depth of Basement Wall In Ground Domain
        \\type real
        \\units m
        \\note Required only if Domain Has Basement Interaction
   A6,  \\field Shift Pipe X Coordinates By Basement Width
        \\type choice
        \\key Yes
        \\key No
        \\note Required only if Domain Has Basement Interaction
   A7,  \\field Name of Basement Wall Boundary Condition Model
        \\type object-list
        \\object-list OSCMNames
        \\note Required only if Domain Has Basement Interaction
   A8,  \\field Name of Basement Floor Boundary Condition Model
        \\type object-list
        \\object-list OSCMNames
        \\note Required only if Domain Has Basement Interaction
   N20, \\field Convergence Criterion for the Outer Cartesian Domain Iteration Loop
        \\type real
        \\units deltaC
        \\minimum 0.000001
        \\maximum 0.5
        \\default 0.001
   N21, \\field Maximum Iterations in the Outer Cartesian Domain Iteration Loop
        \\type integer
        \\minimum 3
        \\maximum 10000
        \\default 500
   N22, \\field Evapotranspiration Ground Cover Parameter
        \\type real
        \\minimum 0
        \\maximum 1.5
        \\default 0.4
        \\note This specifies the ground cover effects during evapotranspiration
        \\note  calculations.  The value roughly represents the following cases:
        \\note  = 0   : concrete or other solid, non-permeable ground surface material
        \\note  = 0.5 : short grass, much like a manicured lawn
        \\note  = 1   : standard reference state (12 cm grass)
        \\note  = 1.5 : wild growth
   N23, \\field Number of Pipe Circuits Entered for this Domain
        \\required-field
        \\type integer
        \\minimum 1
   A9,  \\field Pipe Circuit 1
        \\required-field
        \\begin-extensible
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain
   A10, \\field Pipe Circuit 2
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain
   A11, \\field Pipe Circuit 3
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain
   A12, \\field Pipe Circuit 4
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain
   A13, \\field Pipe Circuit 5
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain
   A14; \\field Pipe Circuit 6
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundCircuitNames
        \\note Name of a pipe circuit to be simulated in this domain

PipingSystem:Underground:PipeCircuit,
        \\memo The pipe circuit object in an underground piping system.
        \\memo This object is simulated within an underground piping domain object
        \\memo  and connected on a branch on a plant loop.
        \\min-fields 15
        \\extensible:1
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference PipingSystemUndergroundCircuitNames
   N1,  \\field Pipe Thermal Conductivity
        \\required-field
        \\type real
        \\units W/m-K
        \\minimum> 0
   N2,  \\field Pipe Density
        \\required-field
        \\type real
        \\units kg/m3
        \\minimum> 0
   N3,  \\field Pipe Specific Heat
        \\required-field
        \\type real
        \\units J/kg-K
        \\minimum> 0
   N4,  \\field Pipe Inner Diameter
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
   N5,  \\field Pipe Outer Diameter
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
   N6,  \\field Design Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0
   A2,  \\field Circuit Inlet Node
        \\required-field
        \\type node
   A3,  \\field Circuit Outlet Node
        \\required-field
        \\type node
   N7,  \\field Convergence Criterion for the Inner Radial Iteration Loop
        \\type real
        \\units deltaC
        \\minimum 0.000001
        \\maximum 0.5
        \\default 0.001
   N8,  \\field Maximum Iterations in the Inner Radial Iteration Loop
        \\type integer
        \\minimum 3
        \\maximum 10000
        \\default 500
   N9,  \\field Number of Soil Nodes in the Inner Radial Near Pipe Mesh Region
        \\type integer
        \\minimum 1
        \\maximum 15
        \\default 3
   N10, \\field Radial Thickness of Inner Radial Near Pipe Mesh Region
        \\required-field
        \\type real
        \\minimum> 0
        \\note Required because it must be selected by user instead of being
        \\note  inferred from circuit/domain object inputs.
   N11, \\field Number of Pipe Segments Entered for this Pipe Circuit
        \\required-field
        \\type integer
        \\minimum 1
   A4,  \\field Pipe Segment 1
        \\required-field
        \\begin-extensible
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit
   A5,  \\field Pipe Segment 2
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit
   A6,  \\field Pipe Segment 3
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit
   A7,  \\field Pipe Segment 4
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit
   A8,  \\field Pipe Segment 5
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit
   A9;  \\field Pipe Segment 6
        \\note optional
        \\type object-list
        \\object-list PipingSystemUndergroundSegmentNames
        \\note Name of a pipe segment to be included in this pipe circuit

PipingSystem:Underground:PipeSegment,
        \\memo The pipe segment to be used in an underground piping system
        \\memo This object represents a single pipe leg positioned axially
        \\memo  in the local z-direction, at a given x, y location in the domain
        \\min-fields 4
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference PipingSystemUndergroundSegmentNames
   N1,  \\field X Position
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\note This segment will be centered at this distance from the x=0
        \\note  domain surface or the basement wall surface, based on whether
        \\note  a basement exists in this domain and the selection of the
        \\note  shift input field found in the domain object.
   N2,  \\field Y Position
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\note This segment will be centered at this distance away from the
        \\note  ground surface; thus this value represents the burial depth
        \\note  of this pipe segment.
   A3;  \\field Flow Direction
        \\required-field
        \\type choice
        \\key IncreasingZ
        \\key DecreasingZ
        \\note This segment will be simulated such that the flow is in the
        \\note  selected direction.  This can allow for detailed analysis
        \\note  of circuiting effects in a single domain.

Duct,
       \\memo Passes inlet node state variables to outlet node state variables
   A1,  \\field Name
        \\required-field
   A2, \\field Inlet Node Name
        \\required-field
        \\type node
   A3; \\field Outlet Node Name
        \\required-field
        \\type node

\\Group Pumps

Pump:VariableSpeed,
       \\memo This pump model is described in the ASHRAE secondary HVAC toolkit.
       \\min-fields 14
  A1 , \\field Name
        \\required-field
  A2 , \\field Inlet Node Name
        \\required-field
        \\type node
  A3 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field Rated Flow Rate
        \\units m3/s
        \\autosizable
        \\ip-units gal/min
        \\minimum> 0
  N2 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
        \\ip-units ftH2O
  N3 , \\field Rated Power Consumption
        \\units W
        \\autosizable
        \\ip-units W
  N4 , \\field Motor Efficiency
        \\type real
        \\minimum> 0.0
        \\maximum 1.0
        \\default 0.9
  N5 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  N6 , \\field Coefficient 1 of the Part Load Performance Curve
       \\default 0.0
  N7 , \\field Coefficient 2 of the Part Load Performance Curve
       \\default 1.0
  N8 , \\field Coefficient 3 of the Part Load Performance Curve
       \\default 0.0
  N9 , \\field Coefficient 4 of the Part Load Performance Curve
       \\default 0.0
  N10, \\field Minimum Flow Rate
        \\note This value can be zero and will be defaulted to that if not specified.
        \\units m3/s
        \\default 0.0
        \\ip-units gal/min
  A4 , \\field Pump Control Type
        \\type choice
        \\key Continuous
        \\key Intermittent
        \\default Continuous
  A5 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the rated flow rate of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A6 , \\field Pump Curve Name
        \\type object-list
        \\object-list AllCurves
        \\note This references any single independent variable polynomial curve in order to
        \\note  do pressure vs. flow calculations for this pump.  The available types are then:
        \\note  Linear, Quadratic, Cubic, and Quartic
        \\note The non-dimensional pump pressure relationship is of the following form:
        \\note  (psi = C4*phi^4 + C3*phi^3 + C2*phi^2 + C1*phi + C0)
        \\note Where the nondimensional variables are defined as:
        \\note  delP = rho * ((N/60)^2) * (D^2) * psi
        \\note  mdot = rho * (N/60) * (D^3) * phi
  N11 , \\field Impeller Diameter
        \\units m
        \\type real
        \\note "D" in above expression in field A6
  A7 , \\field VFD Control Type
         \\type choice
         \\key ManualControl
         \\key PressureSetpointControl
  A8 , \\field Pump rpm Schedule Name
        \\note Modifies the rpm of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A9 ,  \\field Minimum Pressure Schedule
        \\units Pa
        \\type object-list
        \\object-list ScheduleNames
  A10,  \\field Maximum Pressure Schedule
        \\units Pa
        \\type object-list
        \\object-list ScheduleNames
  A11,  \\field Minimum RPM Schedule
        \\units Rotations Per Minute
        \\type object-list
        \\object-list ScheduleNames
  A12,  \\field Maximum RPM Schedule
        \\units Rotations Per Minute
        \\type object-list
        \\object-list ScheduleNames
  A13,  \\field Zone Name
        \\note optional, if used pump losses transfered to zone as internal gains
        \\type Object-list
        \\object-list ZoneNames
  N12;  \\field Skin Loss Radiative Fraction
        \\note optional. If zone identified in previous field then this determines
        \\note the split between convection and radiation for the skin losses
        \\type real
        \\minimum 0.0
        \\maximum 1.0

Pump:ConstantSpeed,
       \\memo This pump model is described in the ASHRAE secondary HVAC toolkit.
       \\min-fields 9
  A1,  \\field Name
        \\required-field
  A2 , \\field Inlet Node Name
        \\required-field
        \\type node
  A3 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field Rated Flow Rate
        \\units m3/s
        \\autosizable
        \\ip-units gal/min
        \\minimum> 0
  N2 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
        \\ip-units ftH2O
  N3 , \\field Rated Power Consumption
        \\units W
        \\autosizable
        \\ip-units W
  N4 , \\field Motor Efficiency
        \\type real
        \\minimum> 0.0
        \\maximum 1.0
        \\default 0.9
  N5 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  A4 , \\field Pump Control Type
        \\type choice
        \\key Continuous
        \\key Intermittent
        \\default Continuous
  A5 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the rated flow rate of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A6 , \\field Pump Curve Name
        \\type object-list
        \\object-list AllCurves
        \\note This references any single independent variable polynomial curve in order to
        \\note  do pressure vs. flow calculations for this pump.  The available types are then:
        \\note  Linear, Quadratic, Cubic, and Quartic
        \\note The non-dimensional pump pressure relationship is of the following form:
        \\note  (psi = C4*phi^4 + C3*phi^3 + C2*phi^2 + C1*phi + C0)
        \\note Where the nondimensional variables are defined as:
        \\note  delP = rho * ((N/60)^2) * (D^2) * psi
        \\note  mdot = rho * (N/60) * (D^3) * phi
  N6 , \\field Impeller Diameter
        \\units m
        \\type real
        \\note "D" in above expression in field A6
  N7 , \\field Rotational Speed
        \\units rev/min
        \\type real
        \\note "N" in above expression in field A6
  A7,   \\field Zone Name
        \\note optional, if used pump losses transfered to zone as internal gains
        \\type Object-list
        \\object-list ZoneNames
  N8;   \\field Skin Loss Radiative Fraction
        \\note optional. If zone identified in previous field then this determines
        \\note the split between convection and radiation for the skin losses
        \\type real
        \\minimum 0.0
        \\maximum 1.0


Pump:VariableSpeed:Condensate,
       \\memo This pump model is described in the ASHRAE secondary HVAC toolkit.
       \\memo Variable Speed Condensate pump for Steam Systems
       \\min-fields 13
  A1 , \\field Name
        \\required-field
  A2 , \\field Inlet Node Name
        \\required-field
        \\type node
  A3 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field Rated Flow Rate
        \\units m3/s
        \\autosizable
        \\ip-units gal/min
        \\minimum> 0
  N2 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
        \\ip-units ftH2O
  N3 , \\field Rated Power Consumption
        \\units W
        \\autosizable
        \\ip-units W
  N4 , \\field Motor Efficiency
        \\type real
        \\minimum> 0.0
        \\maximum 1.0
        \\default 0.9
  N5 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  N6 , \\field Coefficient 1 of the Part Load Performance Curve
       \\default 0.0
  N7 , \\field Coefficient 2 of the Part Load Performance Curve
       \\default 1.0
  N8 , \\field Coefficient 3 of the Part Load Performance Curve
       \\default 0.0
  N9 , \\field Coefficient 4 of the Part Load Performance Curve
       \\default 0.0
  A4 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the rated flow rate of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A5 ,  \\field Zone Name
        \\note optional, if used pump losses transfered to zone as internal gains
        \\type Object-list
        \\object-list ZoneNames
  N10;  \\field Skin Loss Radiative Fraction
        \\note optional. If zone identified in previous field then this determines
        \\note the split between convection and radiation for the skin losses
        \\type real
        \\minimum 0.0
        \\maximum 1.0

HeaderedPumps:ConstantSpeed,
       \\memo This Headered pump object describes a pump bank with more than 1 pump in parallel
       \\min-fields 9
  A1 , \\field Name
        \\required-field
  A2 , \\field Inlet Node Name
        \\required-field
        \\type node
  A3 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field Total Rated Flow Rate
        \\note If the field is not autosized set to the flow rate to
        \\note the total flow when all pumps are running at full load
        \\units m3/s
        \\autosizable
        \\ip-units gal/min
        \\minimum> 0
  N2 , \\field Number of Pumps in Bank
       \\type integer
  A4 , \\field Flow Sequencing Control Scheme
        \\type choice
        \\key Sequential
        \\default Sequential
  N3 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
        \\ip-units ftH2O
  N4 , \\field Rated Power Consumption
        \\note If the field is not autosized set to the power consumed by the pump bank
        \\note when all the pumps are running at nominal flow
        \\units W
        \\autosizable
        \\ip-units W
  N5 , \\field Motor Efficiency
        \\type real
        \\minimum> 0.0
        \\maximum 1.0
        \\default 0.9
  N6 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  A5 , \\field Pump Control Type
        \\type choice
        \\key Continuous
        \\key Intermittent
        \\default Continuous
  A6 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the rated flow rate of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A7 ,  \\field Zone Name
        \\note optional, if used pump losses transfered to zone as internal gains
        \\type Object-list
        \\object-list ZoneNames
  N7 ;  \\field Skin Loss Radiative Fraction
        \\note optional. If zone identified in previous field then this determines
        \\note the split between convection and radiation for the skin losses
        \\type real
        \\minimum 0.0
        \\maximum 1.0

HeaderedPumps:VariableSpeed,
       \\memo This Headered pump object describes a pump bank with more than 1 pump in parallel
       \\min-fields 14
  A1 , \\field Name
        \\required-field
  A2 , \\field Inlet Node Name
        \\required-field
        \\type node
  A3 , \\field Outlet Node Name
        \\required-field
        \\type node
  N1 , \\field Total Rated Flow Rate
        \\note If the field is not autosized set to the flow rate to
        \\note the total flow when all pumps are running at full load
        \\units m3/s
        \\autosizable
        \\ip-units gal/min
        \\minimum> 0
  N2 , \\Field Number of Pumps in Bank
       \\type integer
  A4 , \\Field Flow Sequencing Control Scheme
        \\type choice
        \\key Sequential
        \\default Sequential
  N3 , \\field Rated Pump Head
        \\units Pa
        \\default 179352
        \\note default head is 60 feet
        \\ip-units ftH2O
  N4 , \\field Rated Power Consumption
        \\note If the field is not autosized set to the power consumed by the pump bank
        \\note when all the pumps are running at nominal flow
        \\units W
        \\autosizable
        \\ip-units W
  N5 , \\field Motor Efficiency
        \\type real
        \\minimum> 0.0
        \\maximum 1.0
        \\default 0.9
  N6 , \\field Fraction of Motor Inefficiencies to Fluid Stream
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  N7 , \\field Coefficient 1 of the Part Load Performance Curve
       \\default 0.0
  N8 , \\field Coefficient 2 of the Part Load Performance Curve
       \\default 1.0
  N9 , \\field Coefficient 3 of the Part Load Performance Curve
       \\default 0.0
  N10, \\field Coefficient 4 of the Part Load Performance Curve
       \\default 0.0
  N11, \\field Minimum Flow Rate Fraction
        \\note This value can be zero and will be defaulted to that if not specified.
        \\default 0.0
        \\minimum 0.0
        \\maximum 1.0
  A5 , \\field Pump Control Type
        \\type choice
        \\key Continuous
        \\key Intermittent
        \\default Continuous
  A6 , \\field Pump Flow Rate Schedule Name
        \\note Modifies the rated flow rate of the pump on a time basis. Default is
        \\note that the pump is on and runs according to its other operational requirements
        \\note specified above.  The schedule is for special pump operations.
        \\type object-list
        \\object-list ScheduleNames
  A7 ,  \\field Zone Name
        \\note optional, if used pump losses transfered to zone as internal gains
        \\type Object-list
        \\object-list ZoneNames
  N12;  \\field Skin Loss Radiative Fraction
        \\note optional. If zone identified in previous field then this determines
        \\note the split between convection and radiation for the skin losses
        \\type real
        \\minimum 0.0
        \\maximum 1.0

\\group Plant-Condenser Flow Control

TemperingValve,
      \\memo Temperature-controlled diversion valve used to divert flow around one or more plant
      \\memo components such as a hot water heater. It can only be used on one of two branches
      \\memo between a Splitter and a Mixer.
  A1, \\field Name
      \\required-field
      \\type alpha
  A2, \\field Inlet Node Name
      \\required-field
      \\type node
      \\note Name of a Node
  A3, \\field Outlet Node Name
      \\required-field
      \\type node
      \\note Name of a Node
  A4, \\field Stream 2 Source Node Name
      \\required-field
      \\type node
      \\note Name of a Node
  A5, \\field Temperature Setpoint Node Name
      \\required-field
      \\type node
      \\note Name of a Node
  A6; \\field Pump Outlet Node Name
      \\required-field
      \\type node


\\Group Non-Zone Equipment

LoadProfile:Plant,
       \\memo Used to simulate a scheduled plant loop demand profile.  Load and flow rate are
       \\memo specified using schedules. Positive values are heating loads, and negative values are
       \\memo cooling loads. The actual load met is dependent on the performance of the supply
       \\memo loop components.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Outlet Node Name
       \\required-field
       \\type node
  A4 , \\field Load Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values are load in [W]
  N1 , \\field Peak Flow Rate
       \\required-field
       \\type real
       \\units m3/s
  A5 ; \\field Flow Rate Fraction Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames


\\Group Solar Collectors

SolarCollectorPerformance:FlatPlate,
       \\memo Thermal and optical performance parameters for a single flat plate solar collector
       \\memo module. These parameters are based on the testing methodologies described in ASHRAE
       \\memo Standards 93 and 96 which are used Solar Rating and Certification Corporation (SRCC)
       \\memo Directory of SRCC Certified Solar Collector Ratings. See EnergyPlus DataSets file
       \\memo SolarCollectors.idf.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference FlatPlateSolarCollectorParameters
  N1 , \\field Gross Area
       \\required-field
       \\type real
       \\minimum> 0
       \\units m2
  A2,  \\field Test Fluid
       \\required-field
       \\type choice
       \\key Water
       \\default Water
  N2,  \\field Test Flow Rate
       \\required-field
       \\type real
       \\minimum> 0
       \\units m3/s
  A3 , \\field Test Correlation Type
       \\required-field
       \\type choice
       \\key Inlet
       \\key Average
       \\key Outlet
  N3 , \\field Coefficient 1 of Efficiency Equation
       \\required-field
       \\type real
       \\units dimensionless
       \\note Y-intercept term
  N4 , \\field Coefficient 2 of Efficiency Equation
       \\required-field
       \\type real
       \\units W/m2-K
       \\note 1st Order term
  N5 , \\field Coefficient 3 of Efficiency Equation
       \\type real
       \\units W/m2-K2
       \\note 2nd order term
  N6 , \\field Coefficient 2 of Incident Angle Modifier
       \\type real
       \\note 1st order term
  N7 ; \\field Coefficient 3 of Incident Angle Modifier
       \\type real
       \\note 2nd order term

SolarCollector:FlatPlate:Water,
       \\memo Flat plate water solar collector (single glazed, unglazed, or evacuated tube).
       \\memo Thermal and optical properties are taken from the referenced
       \\memo SolarCollectorPerformance:FlatPlate object. Collector tilt, azimuth, and gross area
       \\memo are taken from the referenced building surface or shading surface. The collector
       \\memo surface participates normally in all shading calculations.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field SolarCollectorPerformance Name
       \\required-field
       \\type object-list
       \\object-list FlatPlateSolarCollectorParameters
  A3 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A4 , \\field Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Outlet Node Name
       \\required-field
       \\type node
  N1 ; \\field Maximum Flow Rate
       \\type real
       \\minimum> 0
       \\units m3/s

SolarCollector:FlatPlate:PhotovoltaicThermal,
       \\memo Models hybrid photovoltaic-thermal (PVT) solar collectors that convert incident solar
       \\memo energy into both electricity and useful thermal energy by heating air or water.
  A1 , \\field Name
  A2 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A3 , \\field Photovoltaic-Thermal Model Performance Name
       \\type object-list
       \\object-list FlatPlatePVTParameters
  A4 , \\field Photovoltaic Name
       \\note Enter the name of a Generator:Photovoltaic object.
       \\type object-list
       \\object-list PVGeneratorNames
  A5 , \\field Thermal Working Fluid Type
       \\type choice
       \\key Water
       \\key Air
  A6 , \\field Water Inlet Node Name
       \\type node
  A7 , \\field Water Outlet Node Name
       \\type node
  A8 , \\field Air Inlet Node Name
       \\type node
  A9 , \\field Air Outlet Node Name
       \\type node
  N1 ; \\Field Design Flow Rate
       \\units m3/s
       \\autosizable

SolarCollectorPerformance:PhotovoltaicThermal:Simple,
       \\memo Thermal performance parameters for a hybrid photovoltaic-thermal (PVT) solar collector.
  A1 , \\field Name
       \\reference FlatPlatePVTParameters
  N1 , \\field Fraction of Surface Area with Active Thermal Collector
       \\required-field
       \\type real
       \\units dimensionless
       \\minimum> 0.0
       \\maximum 1.0
  A2 , \\field Thermal Conversion Efficiency Input Mode Type
       \\type choice
       \\key Fixed
       \\key Scheduled
  N2 , \\field Value for Thermal Conversion Efficiency if Fixed
       \\note Efficiency = (thermal power generated [W])/(incident solar[W])
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A3 , \\field Thermal Conversion Efficiency Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N3 ; \\field Front Surface Emittance
       \\type real
       \\default 0.84
       \\minimum> 0.00
       \\maximum< 1.00

SolarCollector:IntegralCollectorStorage,
       \\memo Glazed solar collector with integral storage unit. Thermal and optical properties are
       \\memo taken from the referenced SolarCollectorPerformance:IntegralCollectorStorage object.
       \\memo Collector tilt, azimuth, and gross area are taken from the referenced building surface
       \\memo or shading surface. The collector surface participates normally in all shading
       \\memo calculations.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field IntegralCollectorStorageParameters Name
       \\required-field
       \\type object-list
       \\object-list CollectorStoragePerformance
  A3 , \\field Surface Name
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A4 , \\field Bottom Surface Boundary Conditions Type
       \\required-field
       \\type choice
       \\Key OtherSideConditionsModel
       \\Key AmbientAir
       \\Default AmbientAir
  A5 , \\field Boundary Condition Model Name
       \\type alpha
       \\note Enter the name of a SurfaceProperty:OtherSideConditionsModel
       \\note object. Specified only if the boundary condition type is
       \\note OtherSideConditionsModel, otherwise leave it blank
  A6 , \\field Inlet Node Name
       \\required-field
       \\type node
  A7 , \\field Outlet Node Name
       \\required-field
       \\type node
  N1 ; \\field Maximum Flow Rate
       \\type real
       \\minimum> 0
       \\units m3/s

SolarCollectorPerformance:IntegralCollectorStorage,
       \\memo Thermal and optical performance parameters for a single glazed solar collector with
       \\memo integral storage unit.
  A1,  \\field Name
       \\required-field
       \\type alpha
       \\reference CollectorStoragePerformance
  A2,  \\field ICS Collector Type
       \\required-field
       \\type choice
       \\Key RectangularTank
       \\note Currently only RectangularTank ICS collector type is available.
  N1,  \\field Gross Area
       \\type real
       \\minimum> 0
       \\units m2
  N2,  \\field Collector Water Volume
       \\type real
       \\minimum> 0
       \\units m3
  N3,  \\field Bottom Heat Loss Conductance
       \\required-field
       \\type real
       \\minimum> 0
       \\default 0.40
       \\units W/m2-K
       \\note Heat loss conductance of the collector bottom insulation
  N4,  \\field Side Heat Loss Conductance
       \\required-field
       \\type real
       \\minimum> 0
       \\default 0.60
       \\units W/m2-K
       \\note heat loss conductance of the collector side insulation
  N5,  \\field Aspect Ratio
       \\required-field
       \\type real
       \\minimum> 0.5
       \\maximum< 1.0
       \\default 0.8
       \\units m
       \\note This value is ratio of the width (short side) to length
       \\note (long side of) of the collector.  Used to calculate the
       \\note perimeter of the collector
  N6,  \\field Collector Side Height
       \\required-field
       \\type real
       \\minimum> 0
       \\maximum< 0.30
       \\default 0.20
       \\units m
       \\note This value is used to estimate collector side area for the heat
       \\note loss calculation through the collector side
  N7,  \\field Thermal Mass of Absorber Plate
       \\required-field
       \\type real
       \\minimum 0
       \\default 0
       \\units J/m2-K
       \\note Calculated from the specific heat, density and thickness
       \\note of the absorber plate.
  N8,  \\field Number of Covers
       \\required-field
       \\type integer
       \\minimum 1
       \\maximum 2
       \\default 2
       \\note Number of transparent covers. Common practice is to use low-iron
       \\note glass as the outer cover and very thin transparent sheet such as
       \\note Teflon as the inner cover.
  N9,  \\field Cover Spacing
       \\required-field
       \\type real
       \\minimum> 0
       \\maximum 0.20
       \\default 0.05
       \\units m
       \\note The gap between the transparent covers and between the inner cover
       \\note and the absorber plate
  N10,  \\field Refractive Index of Outer Cover
       \\required-field
       \\type real
       \\minimum 1.0
       \\maximum 2.0
       \\default 1.526
       \\units dimensionless
       \\note Refractive index of outer cover. Typically low-iron glass is used
       \\note as the outer cover material, and used as the default outer cover
       \\note with a vallue of 1.526.
  N11,  \\field Extinction Coefficient Times Thickness of Outer Cover
       \\required-field
       \\type real
       \\minimum 0.
       \\default 0.045
       \\units dimensionless
       \\note Clear glass has extinction coefficient of about 15 [1/m]
       \\note and with thickness of 3.0mm, the product of the extinction
       \\note coefficient and thickness becomes 0.045 (=15 * 0.003)
  N12,  \\field Emissivity of Outer Cover
       \\required-field
       \\type real
       \\minimum> 0.
       \\maximum< 1.0
       \\default 0.88
       \\units dimensionless
       \\note Thermal emissivity of the outer cover, commonly glass is used as
       \\note the out collector cover material.
  N13,  \\field Refractive Index of Inner Cover
       \\required-field
       \\type real
       \\minimum 1.0
       \\maximum 2.0
       \\default 1.37
       \\units dimensionless
       \\note Typical material is very thin sheet of Teflon (PTFE). The default
       \\note value is refractive index of Teflon.
  N14, \\field Extinction Coefficient Times Thickness of the inner Cover
       \\required-field
       \\type real
       \\minimum 0.
       \\default 0.008
       \\units dimensionless
       \\note Default inner cover is very thin sheet of Teflon with
       \\note extinction coefficient of approximately 40.0 and a thickness
       \\note 0.2mm yields a default value of 0.008.
  N15, \\field Emmissivity of Inner Cover
       \\required-field
       \\type real
       \\minimum> 0.
       \\maximum< 1.0
       \\default 0.88
       \\units dimensionless
       \\note Thermal emissivity of the inner cover matrial
  N16, \\field Absorptance of Absorber Plate
       \\required-field
       \\type real
       \\minimum> 0.
       \\maximum< 1.0
       \\default 0.96
       \\units dimensionless
       \\note The absober plate solar absorptance.  Copper is assumed as
       \\note the default absorber plate.
  N17; \\field Emissivity of Absorber Plate
       \\required-field
       \\type real
       \\minimum> 0.
       \\maximum< 1.0
       \\default 0.30
       \\units dimensionless
       \\note Thermal emissivity of the absorber plate

SolarCollector:UnglazedTranspired,
       \\memo Unglazed transpired solar collector (UTSC) used to condition outdoor air. This type of
       \\memo collector is generally used to heat air drawn through perforated absorbers and also
       \\memo recover heat conducted out through the underlying surfae. This object represents a
       \\memo single collector attached to one or more building or shading surfaces and to one or
       \\memo more outdoor air systems.
       \\min-fields 23
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference UTSCNames
  A2,  \\field Boundary Conditions Model Name
       \\required-field
       \\type object-list
       \\object-list OSCMNames
       \\note Enter the name of a SurfaceProperty:OtherSideConditionsModel object
  A3 , \\field Availability Schedule Name
       \\note Availability schedule name for this collector. Schedule value > 0 means it is available.
       \\note If this field is blank, the collector is always available.
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Inlet Node Name
       \\type node
       \\note required field if no SolarCollector:UnglazedTranspired:Multisystem
  A5 , \\field Outlet Node Name
       \\type node
       \\note required field if no SolarCollector:UnglazedTranspired:Multisystem
  A6,  \\field Setpoint Node Name
       \\type node
       \\note This node is where the mixed air setpoint is determined.
       \\note required field if no SolarCollector:UnglazedTranspired:Multisystem
  A7,  \\field Zone Node Name
       \\type node
       \\note This node is used to indentify the affected zone
       \\note required field if no SolarCollector:UnglazedTranspired:Multisystem
  A8,  \\field Free Heating Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N1,  \\field Diameter of Perforations in Collector
       \\required-field
       \\type real
       \\minimum> 0
       \\units m
  N2,  \\field Distance Between Perforations in Collector
       \\required-field
       \\type real
       \\minimum> 0
       \\units m
  N3,  \\field Thermal Emissivity of Collector Surface
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\units dimensionless
  N4,  \\field Solar Absorbtivity of Collector Surface
       \\required-field
       \\type real
       \\minimum 0
       \\maximum 1
       \\units dimensionless
  N5, \\field Effective Overall Height of Collector
       \\required-field
       \\type real
       \\minimum> 0.0
  N6,  \\field Effective Gap Thickness of Plenum Behind Collector
       \\required-field
       \\type real
       \\minimum> 0.
       \\units m
       \\note if corrugated, use average depth
  N7,  \\field Effective Cross Section Area of Plenum Behind Collector
       \\required-field
       \\type real
       \\minimum> 0
       \\units m2
       \\note if corrugated, use average depth
  A9,  \\field Hole Layout Pattern for Pitch
       \\type choice
       \\key Triangle
       \\key Square
       \\default Square
  A10,  \\field Heat Exchange Effectiveness Correlation
       \\type choice
       \\key Kutscher1994
       \\key VanDeckerHollandsBrunger2001
       \\default Kutscher1994
  N8,  \\field Ratio of Actual Collector Surface Area to Projected Surface Area
       \\type real
       \\note This parameter is used to help account for corrugations in the collector
       \\default 1.0
       \\minimum 1.0
       \\maximum 2.0
       \\units dimensionless
  A11, \\field Roughness of Collector
       \\required-field
       \\type choice
       \\key VeryRough
       \\key Rough
       \\key MediumRough
       \\key MediumSmooth
       \\key Smooth
       \\key VerySmooth
  N9, \\field Collector Thickness
       \\type real
       \\note Collector thickness is not required for Kutscher correlation
       \\note Collector thickness is required for Van Decker et al. correlation
       \\minimum 0.0005
       \\maximum 0.007
       \\units m
  N10, \\field Effectiveness for Perforations with Respect to Wind
       \\note Cv
       \\type real
       \\units dimensionless
       \\default 0.25
       \\minimum> 0
       \\maximum 1.5
  N11, \\field Discharge Coefficient for Openings with Respect to Buoyancy Driven Flow
       \\note Cd
       \\type real
       \\units dimensionless
       \\default 0.65
       \\minimum> 0.0
       \\maximum 1.5
  A12, \\field Surface 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A13, \\field Surface 2 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A14, \\field Surface 3 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A15, \\field Surface 4 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A16, \\field Surface 5 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A17, \\field Surface 6 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A18, \\field Surface 7 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A19, \\field Surface 8 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A20, \\field Surface 9 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A21; \\field Surface 10 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames

SolarCollector:UnglazedTranspired:Multisystem,
       \\extensible:4 - repeat last four fields, remembering to remove ; from "inner" fields.
       \\memo quad-tuples of inlet, outlet, control, and zone nodes
       \\memo for multiple different outdoor air systems attached to same collector
  A1 , \\field Solar Collector Name
       \\required-field
       \\note Enter the name of a SolarCollector:UnglazedTranspired object.
       \\type object-list
       \\object-list UTSCNames
  A2 , \\field Outdoor Air System 1 Collector Inlet Node
       \\begin-extensible
       \\type node
  A3 , \\field Outdoor Air System 1 Collector Outlet Node
       \\type node
  A4 , \\field Outdoor Air System 1 Mixed Air Node
       \\type node
  A5 , \\field Outdoor Air System 1 Zone Node
       \\type node
  A6 , \\field Outdoor Air System 2 Collector Inlet Node
       \\type node
  A7 , \\field Outdoor Air System 2 Collector Outlet Node
       \\type node
  A8 , \\field Outdoor Air System 2 Mixed Air Node
       \\type node
  A9 , \\field Outdoor Air System 2 Zone Node
       \\type node
  A10, \\field Outdoor Air System 3 Collector Inlet Node
       \\type node
  A11, \\field Outdoor Air System 3 Collector Outlet Node
       \\type node
  A12, \\field Outdoor Air System 3 Mixed Air Node
       \\type node
  A13, \\field Outdoor Air System 3 Zone Node
       \\type node
  A14, \\field Outdoor Air System 4 Collector Inlet Node
       \\type node
  A15, \\field Outdoor Air System 4 Collector Outlet Node
       \\type node
  A16, \\field Outdoor Air System 4 Mixed Air Node
       \\type node
  A17, \\field Outdoor Air System 4 Zone Node
       \\type node
  A18, \\field Outdoor Air System 5 Collector Inlet Node
       \\type node
  A19, \\field Outdoor Air System 5 Collector Outlet Node
       \\type node
  A20, \\field Outdoor Air System 5 Mixed Air Node
       \\type node
  A21; \\field Outdoor Air System 5 Zone Node
       \\type node

!*****************PLANT EQUIPMENT*********************
!KEY--Plant Equipment:  The following types are used to specify
!                  equipment connected to the supply side of
!                  the supply side of the plant loop.  Note
!                  that at least one inlet and outlet node
!                  must be specified for each node
!  CLASS of the following comes first (naming convention) separated further by :

\\Group Plant Heating and Cooling Equipment

Boiler:HotWater,
    \\min-fields 13
       \\memo This boiler model is an adaptation of the empirical model from the Building
       \\memo Loads and System Thermodynamics (BLAST) program.  Boiler performance
       \\memo curves are generated by fitting catalog data to polynomial equations.
       \\memo A constant efficiency boiler may be modeled by leaving the normalized
       \\memo boiler efficiency curve name input blank.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference Boilers
  A2 , \\field Fuel Type
       \\required-field
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
  N1 , \\field Nominal Capacity
       \\type real
       \\units W
       \\autosizable
       \\minimum 0.0
  N2 , \\field Nominal Thermal Efficiency
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\note Based on the higher heating value of fuel.
  A3 , \\field Efficiency Curve Temperature Evaluation Variable
       \\type choice
       \\key EnteringBoiler
       \\key LeavingBoiler
  A4 , \\field Normalized Boiler Efficiency Curve Name
       \\type object-list
       \\object-list AllCurves
       \\note Linear, Quadratic and Cubic efficiency curves are solely a function of PLR.
       \\note All other efficiency curve types are a function of PLR and boiler water temperature.
       \\note Linear = C1 + C2*PLR
       \\note Quadratic = C1 + C2*PLR + C3*PLR^2
       \\note Cubic = C1 + C2*PLR + C3*PLR^2 + C4*PLR^3
       \\note Biquadratic = C1 + C2*PLR + C3*PLR^2 + C4*T + C5*T^2 + C6*PLR*T
       \\note QuadraticLinear = C1 + C2*PLR + C3*PLR^2 + (C4 + C5*PLR + C6*PLR^2)*T
       \\note BiCubic = C1+C2*PLR+C3*PLR^2+C4*T+C5*T^2+C6*PLR*T+C7*PLR^3+C8*T^3+C9*PLR^2*T+C10*PLR*T^2
       \\note TriQuadratic curves are not allowed.
       \\note PLR = part-load ratio
       \\note T = boiler water temperature (either entering or leaving).
  N3 , \\field Design Water Outlet Temperature
       \\type real
       \\units C
  N4 , \\field Design Water Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
  N5 , \\field Minimum Part Load Ratio
       \\type real
       \\minimum 0.0
       \\default 0.0
  N6 , \\field Maximum Part Load Ratio
       \\type real
       \\minimum 0.0
       \\default 1.0
  N7 , \\field Optimum Part Load Ratio
       \\type real
       \\minimum 0.0
       \\default 1.0
  A5 , \\field Boiler Water Inlet Node Name
       \\required-field
       \\type node
  A6 , \\field Boiler Water Outlet Node Name
       \\required-field
       \\type node
  N8 , \\field Water Outlet Upper Temperature Limit
       \\type real
       \\units C
       \\default 99.9
  A7 , \\field Boiler Flow Mode
       \\note Select operating mode for fluid flow through the boiler. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by boiler to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by boiler to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N9 , \\field Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0.0
  N10; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

Boiler:Steam,
       \\memo This boiler model is an adaptation of the empirical model from the Building
       \\memo Loads and System Thermodynamics (BLAST) program.  Boiler performance
       \\memo curves are generated by fitting catalog data to third order
       \\memo polynomial equations.  A constant efficiency boiler is modeled by setting
       \\memo the fuel use coefficients as follows: N9=1, N10=0, N11=0
  A1 , \\field Name
       \\required-field
  A2 , \\field Fuel Type
       \\required-field
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
  N1 , \\field Maximum Operating Pressure
       \\units Kpa
  N2 , \\field Theoretical Efficiency
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N3 , \\field Design Outlet Steam Temperature
       \\units C
  N4 , \\field Nominal Capacity
       \\units W
       \\autosizable
  N5 , \\field Minimum Part Load Ratio
       \\minimum 0.0
  N6 , \\field Maximum Part Load Ratio
       \\minimum 0.0
  N7 , \\field Optimum Part Load Ratio
       \\minimum 0.0
  N8 , \\field Coefficient 1 of Fuel Use Function of Part Load Ratio Curve
  N9 , \\field Coefficient 2 of Fuel Use Function of Part Load Ratio Curve
  N10, \\field Coefficient 3 of Fuel Use Function of Part Load Ratio Curve
  A3 , \\field Water Inlet Node Name
       \\type node
  A4 , \\field Steam Outlet Node Name
       \\type node
  N11; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

Chiller:Electric:EIR,
\\min-fields 23
   \\memo This chiller model is the empirical model from the DOE-2 building Energy
   \\memo simulation program. Chiller performance at off-reference conditions is modeled
   \\memo using three polynomial equations. Three curves objects are required.
  A1 , \\field Name
       \\type alpha
       \\reference Chillers
       \\required-field
  N1 , \\field Reference Capacity
       \\type real
       \\units W
       \\required-field
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Reference COP
       \\note Efficiency of the chiller compressor (cooling output/compressor energy input).
       \\note Condenser fan power should not be included here.
       \\type real
       \\units W/W
       \\required-field
       \\minimum> 0.0
  N3 , \\field Reference Leaving Chilled Water Temperature
       \\type real
       \\default 6.67
       \\units C
  N4 , \\field Reference Entering Condenser Fluid Temperature
       \\type real
       \\default 29.4
       \\units C
  N5 , \\field Reference Chilled Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autosizable
       \\ip-units gal/min
  N6 , \\field Reference Condenser Fluid Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum 0.0
       \\ip-units gal/min
       \\note This field is only used for Condenser Type = AirCooled or EvaporativelyCooled
       \\note when Heat Recovery is specified
  A2 , \\field Cooling Capacity Function of Temperature Curve Name
       \\note Cooling capacity as a function of CW supply temp and entering condenser temp
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*ECT + e*ECT**2 + f*CWS*ECT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A3 , \\field Electric Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note Electric Input Ratio (EIR) as a function of temperature
       \\note EIR = 1/COP
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*ECT + e*ECT**2 + f*CWS*ECT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A4 , \\field Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note Electric Input Ratio (EIR) as a function of Part Load Ratio (PLR)
       \\note EIR = 1/COP
       \\required-field
       \\type object-list
       \\object-list QuadraticCurves
       \\note quadratic curve = a + b*PLR + c*PLR**2
       \\note PLR = part load ratio (cooling load/steady state capacity)
  N7 , \\field Minimum Part Load Ratio
       \\type real
       \\note Part load ratio below which the chiller starts cycling on/off to meet the load.
       \\note Must be less than or equal to Maximum Part Load Ratio.
       \\minimum 0.0
       \\default 0.1
  N8 , \\field Maximum Part Load Ratio
       \\type real
       \\note Maximum allowable part load ratio. Must be greater than or equal to Minimum Part Load Ratio.
       \\minimum> 0.0
       \\default 1.0
  N9 , \\field Optimum Part Load Ratio
       \\type real
       \\note Optimum part load ratio where the chiller is most efficient.
       \\note Must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\minimum> 0.0
       \\default 1.0
  N10, \\field Minimum Unloading Ratio
       \\type real
       \\note Part load ratio where the chiller can no longer unload and false loading begins.
       \\note Minimum unloading ratio must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\minimum 0.0
       \\default 0.2
  A5 , \\field Chilled Water Inlet Node Name
       \\type node
       \\required-field
  A6 , \\field Chilled Water Outlet Node Name
       \\type node
       \\required-field
  A7 , \\field Condenser Inlet Node Name
       \\type node
       \\note Not required if air-cooled or evaporativly-cooled
  A8 , \\field Condenser Outlet Node Name
       \\type node
       \\note Not required if air-cooled or evaporatively-cooled
  A9 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\key EvaporativelyCooled
       \\default WaterCooled
  N11, \\field Condenser Fan Power Ratio
       \\type real
       \\units W/W
       \\minimum 0.0
       \\default 0.0
       \\note Use for air-cooled or evaporatively-cooled condensers.
       \\note Ratio of condenser fan power to reference chiller capacity
  N12, \\field Fraction of Compressor Electric Consumption Rejected by Condenser
       \\type real
       \\note Fraction of compressor electrical energy that must be rejected by the condenser.
       \\note Enter a value of 1.0 when modeling hermetic chillers.
       \\note For open chillers, enter the compressor motor efficiency.
       \\note This value should be greater than 0.6 for praticle applications.
       \\minimum> 0.0
       \\maximum 1.0
       \\default 1.0
  N13, \\field Leaving Chilled Water Lower Temperature Limit
       \\type real
       \\default 2.0
       \\units C
  A10, \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N14, \\field Design Heat Recovery Water Flow Rate
       \\type real
       \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\autosizable
       \\ip-units gal/min
  A11, \\field Heat Recovery Inlet Node Name
       \\type node
  A12, \\field Heat Recovery Outlet Node Name
       \\type node
  N15, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N16, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the chiller is not operating.
  N17, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A13, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  N18, \\field Condenser Heat Recovery Relative Capacity Fraction
       \\note This optional field is the fraction of total rejected heat that can be recovered at full load
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A14, \\field Heat Recovery Inlet High Temperature Limit Schedule Name
       \\note This optional schedule of temperatures will turn off heat recovery if inlet exceeds the value
       \\type object-list
       \\object-list ScheduleNames
  A15; \\field Heat Recovery Leaving Temperature Setpoint Node Name
       \\note This optional field provides control over the heat recovery
       \\note Using this triggers a model more suited to series bundle and chillers with higher temperature heat recovery
       \\note If this field is not used, the bundles are modeled as being in parallel
       \\type node


Chiller:Electric:ReformulatedEIR,
\\min-fields 21
   \\memo This chiller model is an empirical model, a reformulated version of Chiller:Electric:EIR
   \\memo where the performance is a function of condenser leaving fluid Temperature instead of
   \\memo condenser entering fluid Temperature. Chiller performance at off-reference conditions is
   \\memo modeled using three polynomial equations. Three curve objects are required.
  A1 , \\field Name
       \\type alpha
       \\reference Chillers
       \\required-field
  N1 , \\field Reference Capacity
       \\type real
       \\units W
       \\required-field
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Reference COP
       \\note Efficiency of the chiller compressor (cooling output/compressor energy input).
       \\note Condenser fan power should not be included here.
       \\type real
       \\units W/W
       \\required-field
       \\minimum> 0.0
  N3 , \\field Reference Leaving Chilled Water Temperature
       \\type real
       \\default 6.67
       \\units C
  N4 , \\field Reference Leaving Condenser Water Temperature
       \\type real
       \\default 35.0
       \\units C
  N5 , \\field Reference Chilled Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autosizable
       \\ip-units gal/min
  N6 , \\field Reference Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum> 0.0
       \\ip-units gal/min
  A2 , \\field Cooling Capacity Function of Temperature Curve Name
       \\note Cooling capacity as a function of supply (leaving) chilled water temperature
       \\note and leaving condenser fluid temperature
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*LCT + e*LCT**2 + f*CWS*LCT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
  A3 , \\field Electric Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note Electric Input Ratio (EIR) as a function of supply (leaving) chilled water temperature
       \\note and leaving condenser fluid temperature.   EIR = 1/COP.
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*LCT + e*LCT**2 + f*CWS*LCT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
  A4 , \\field Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note Electric Input Ratio (EIR) as a function of Part Load Ratio (PLR)
       \\note EIR = 1/COP
       \\required-field
       \\type object-list
       \\object-list BicubicCurves
       \\note curve = a + b*LCT + c*LCT**2 + d*PLR + e*PLR**2 + f*LCT*PLR + g*0 + h*PLR**3
       \\note + i*0 + j*0
       \\note PLR = part load ratio (cooling load/steady state capacity)
       \\note LCT = leaving condenser fluid temperature(C)
  N7 , \\field Minimum Part Load Ratio
       \\type real
       \\note Part load ratio below which the chiller starts cycling on/off to meet the load.
       \\note Must be less than or equal to Maximum Part Load Ratio.
       \\minimum 0.0
       \\default 0.1
  N8 , \\field Maximum Part Load Ratio
       \\type real
       \\note Maximum allowable part load ratio. Must be greater than or equal to Minimum Part Load Ratio.
       \\minimum> 0.0
       \\default 1.0
  N9 , \\field Optimum Part Load Ratio
       \\type real
       \\note Optimum part load ratio where the chiller is most efficient.
       \\note Must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\minimum> 0.0
       \\default 1.0
  N10, \\field Minimum Unloading Ratio
       \\type real
       \\note Part load ratio where the chiller can no longer unload and false loading begins.
       \\note Minimum unloading ratio must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\minimum 0.0
       \\default 0.2
  A5 , \\field Chilled Water Inlet Node Name
       \\type node
       \\required-field
  A6 , \\field Chilled Water Outlet Node Name
       \\type node
       \\required-field
  A7 , \\field Condenser Inlet Node Name
       \\type node
       \\required-field
  A8 , \\field Condenser Outlet Node Name
       \\type node
       \\required-field
  N11, \\field Fraction of Compressor Electric Consumption Rejected by Condenser
       \\type real
       \\note Fraction of compressor electrical energy that must be rejected by the condenser.
       \\note Enter a value of 1.0 when modeling hermetic chillers.
       \\note For open chillers, enter the compressor motor efficiency.
       \\note This value should be greater than 0.6 for praticle applications.
       \\minimum> 0.0
       \\maximum 1.0
       \\default 1.0
  N12, \\field Leaving Chilled Water Lower Temperature Limit
       \\type real
       \\default 2.0
       \\units C
  A9, \\field Chiller Flow Mode Type
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N13, \\field Design Heat Recovery Water Flow Rate
       \\type real
       \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
       \\autosizable
  A10, \\field Heat Recovery Inlet Node Name
       \\type node
  A11, \\field Heat Recovery Outlet Node Name
       \\type node
  N14, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N15, \\field Condenser Heat Recovery Relative Capacity Fraction
       \\note This optional field is the fraction of total rejected heat that can be recovered at full load
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A12, \\field Heat Recovery Inlet High Temperature Limit Schedule Name
       \\note This optional schedule of temperatures will turn off heat recovery if inlet exceeds the value
       \\type object-list
       \\object-list ScheduleNames
  A13; \\field Heat Recovery Leaving Temperature Setpoint Node Name
       \\note This optional field provides control over the heat recovery
       \\note Using this triggers a model more suited to series bundle and chillers with higher temperature heat recovery
       \\note If this field is not used, the bundles are modeled as being in parallel
       \\type node

Chiller:Electric,
    \\min-fields 27
       \\memo This chiller model is the empirical model from the Building Loads
       \\memo and System Thermodynamics (BLAST) program.  Chiller performance
       \\memo curves are generated by fitting catalog data to third order
       \\memo polynomial equations.  Three sets of coefficients are required.
  A1 , \\field Name
       \\required-field
  A2 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N1 , \\field Nominal Capacity
       \\units W
       \\required-field
       \\autosizable
       \\minimum 0.0
  N2 , \\field Nominal COP
       \\required-field
       \\units W/W
       \\minimum> 0.0
  A3 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Inlet Node Name
       \\type node
  A6 , \\field Condenser Outlet Node Name
       \\type node
  N3 , \\field Minimum Part Load Ratio
       \\minimum 0.0
  N4 , \\field Maximum Part Load Ratio
       \\minimum 0.0
  N5 , \\field Optimum Part Load Ratio
       \\minimum 0.0
  N6 , \\field Design Condenser Inlet Temperature
       \\units C
  N7 , \\field Temperature Rise Coefficient
       \\required-field
       \\minimum> 0.0
  N8 , \\field Design Chilled Water Outlet Temperature
       \\units C
  N9 , \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the maximum flow & for constant flow this is the flow.
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
  N10, \\field Design Condenser Fluid Flow Rate
       \\units m3/s
       \\autosizable
       \\minimum 0.0
       \\ip-units gal/min
       \\note This field is only used for Condenser Type = AirCooled or EvaporativelyCooled
       \\note when Heat Recovery is specified
  N11, \\field Coefficient 1 of Capacity Ratio Curve
  N12, \\field Coefficient 2 of Capacity Ratio Curve
  N13, \\field Coefficient 3 of Capacity Ratio Curve
  N14, \\field Coefficient 1 of Power Ratio Curve
  N15, \\field Coefficient 2 of Power Ratio Curve
  N16, \\field Coefficient 3 of Power Ratio Curve
  N17, \\field Coefficient 1 of Full Load Ratio Curve
  N18, \\field Coefficient 2 of Full Load Ratio Curve
  N19, \\field Coefficient 3 of Full Load Ratio Curve
  N20, \\field Chilled Water Outlet Temperature Lower Limit
       \\units C
  A7,  \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N21, \\field Design Heat Recovery Water Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
       \\ip-units gal/min
       \\autosizable
  A8 , \\field Heat Recovery Inlet Node Name
       \\type node
  A9 , \\field Heat Recovery Outlet Node Name
       \\type node
  N22, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N23, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the chiller is not operating.
  N24, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A10, \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  N25, \\field Condenser Heat Recovery Relative Capacity Fraction
       \\note This optional field is the fraction of total rejected heat that can be recovered at full load
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  A11, \\field Heat Recovery Inlet High Temperature Limit Schedule Name
       \\note This optional schedule of temperatures will turn off heat recovery if inlet exceeds the value
       \\type object-list
       \\object-list ScheduleNames
  A12; \\field Heat Recovery Leaving Temperature Setpoint Node Name
       \\note This optional field provides control over the heat recovery
       \\note Using this triggers a model more suited to series bundle and chillers with higher temperature heat recovery
       \\note If this field is not used, the bundles are modeled as being in parallel
       \\type node

Chiller:Absorption:Indirect,
   \\min-fields 17
       \\memo This indirect absorption chiller model is an enhanced model from the
       \\memo Building Loads and System Thermodynamics (BLAST) program.  Chiller
       \\memo performance curves are generated by fitting catalog data to third order
       \\memo polynomial equations. The chiller capacity is a function of condenser,
       \\memo chilled water, and generator temperatures. The heat input is a function
       \\memo of part-load ratio, condenser temperature, and chilled water temperature.
  A1 , \\field Name
       \\type alpha
       \\required-field
  N1 , \\field Nominal Capacity
       \\type real
       \\required-field
       \\units W
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Nominal Pumping Power
       \\type real
       \\required-field
       \\units W
       \\ip-units W
       \\minimum 0.0
       \\autosizable
  A2 , \\field Chilled Water Inlet Node Name
       \\type node
       \\required-field
  A3 , \\field Chilled Water Outlet Node Name
       \\type node
       \\required-field
  A4 , \\field Condenser Inlet Node Name
       \\type node
       \\required-field
  A5 , \\field Condenser Outlet Node Name
       \\type node
       \\required-field
  N3 , \\field Minimum Part Load Ratio
       \\type real
       \\minimum 0.0
  N4 , \\field Maximum Part Load Ratio
       \\type real
       \\minimum 0.0
  N5 , \\field Optimum Part Load Ratio
       \\type real
       \\minimum 0.0
  N6 , \\field Design Condenser Inlet Temperature
       \\type real
       \\units C
       \\default 30.0
       \\note Used only when condenser flow rate is autosized.
  N7 , \\field Condenser Inlet Temperature Lower Limit
       \\type real
       \\units C
       \\default 15.0
       \\note Provides warnings when entering condenser temperature is below minimum.
       \\note Capacity is not adjusted when entering condenser temperature is below minimum.
  N8 , \\field Chilled Water Outlet Temperature Lower Limit
       \\type real
       \\units C
       \\default 5.0
       \\note Capacity is adjusted when leaving chilled water temperature is below minimum.
  N9 , \\field Design Chilled Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autosizable
       \\ip-units gal/min
       \\note For variable flow this is the max flow & for constant flow this is the flow.
  N10, \\field Design Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum> 0.0
       \\ip-units gal/min
  A6 , \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  A7,  \\field Generator Heat Input Function of Part Load Ratio Curve Name
       \\type object-list
       \\required-field
       \\object-list QuadraticCubicCurves
  A8,  \\field Pump Electric Input Function of Part Load Ratio Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
  A9 , \\field Generator Inlet Node Name
       \\type node
       \\note Enter the generator inlet node name which connects this chiller to a
       \\note steam or hot water plant, otherwise leave this field blank.
       \\note Generator nodes are used to model heat input to the chiller.
  A10, \\field Generator Outlet Node Name
       \\type node
       \\note Enter the generator outlet node name which connects this chiller to a
       \\note steam or hot water plant, otherwise leave this field blank.
       \\note Generator nodes are used to model heat input to the chiller.
  A11, \\field Capacity Correction Function of Condenser Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Curve which shows the change in normailized capacity to changes in condenser temperature.
  A12, \\field Capacity Correction Function of Chilled Water Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Curve which shows the change in normailized capacity to changes in leaving chilled water temperature.
  A13, \\field Capacity Correction Function of Generator Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Used when generator fluid type is hot water
       \\note Curve which shows the change in normailized capacity to changes in generator temperature.
  A14, \\field Generator Heat Input Correction Function of Condenser Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Curve which shows the change in normailized heat input to changes in condenser temperature.
  A15, \\field Generator Heat Input Correction Function of Chilled Water Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
       \\note Curve which shows the change in normailized heat input to changes in leaving chilled water temperature.
  A16, \\field Generator Heat Source Type
       \\type choice
       \\key HotWater
       \\key Steam
       \\default Steam
       \\note The Generator side of the chiller can be connected to a hot water or steam plant where the
       \\note generator inlet and outlet nodes are connected to a plant loop. If the generator is not
       \\note connected to a plant loop, and the generator inlet/outlet nodes are not used, this field should be
       \\note specified as steam or left blank. When a plant is not used, the model assumes steam as the heat source.
  N11, \\field Design Generator Fluid Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\note  For variable flow this is the max flow and for constant flow this is the flow.
  N12, \\field Temperature Lower Limit Generator Inlet
       \\type real
       \\units C
       \\default 0.0
       \\note Provides warnings when entering generator temperature is below minimum.
       \\note Capacity is not adjusted when entering generator temperature is below minimum.
  N13, \\field Degree of Subcooling in Steam Generator
       \\type real
       \\units C
       \\minimum 0.0
       \\maximum 20.0
       \\default 1.0
       \\note This field is not used when the generator inlet/outlet nodes are not specified or
       \\note the generator is connected to a hot water loop.
  N14, \\field Degree of Subcooling in Steam Condensate Loop
       \\type real
       \\units C
       \\minimum 0.0
       \\default 0.0
       \\note This field is not used when the generator inlet/outlet nodes are not specified or
       \\note the generator is connected to a hot water loop.
  N15; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

Chiller:Absorption,
   \\min-fields 23
       \\memo This indirect absorption chiller model is the empirical model from the
       \\memo Building Loads and System Thermodynamics (BLAST) program.  Chiller
       \\memo performance curves are generated by fitting catalog data to third order
       \\memo polynomial equations.  Two sets of coefficients are required.
  A1 , \\field Name
       \\required-field
  N1 , \\field Nominal Capacity
       \\type real
       \\units W
       \\required-field
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Nominal Pumping Power
       \\units W
       \\required-field
       \\ip-units W
       \\minimum 0.0
       \\autosizable
  A2 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A4 , \\field Condenser Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Outlet Node Name
       \\required-field
       \\type node
  N3 , \\field Minimum Part Load Ratio
       \\minimum 0.0
  N4 , \\field Maximum Part Load Ratio
       \\minimum 0.0
  N5 , \\field Optimum Part Load Ratio
       \\minimum 0.0
  N6 , \\field Design Condenser Inlet Temperature
       \\units C
  N7 , \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the max flow & for constant flow this is the flow.
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autosizable
       \\ip-units gal/min
  N8 , \\field Design Condenser Water Flow Rate
       \\note  The steam use coefficients below specify the
       \\note  steam use as a fraction of chiller operating capacity
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum> 0.0
       \\ip-units gal/min
  N9 , \\field Coefficient 1 of the Hot Water or Steam Use Part Load Ratio Curve
  N10, \\field Coefficient 2 of the Hot Water or Steam Use Part Load Ratio Curve
  N11, \\field Coefficient 3 of the Hot Water or Steam Use Part Load Ratio Curve
  N12, \\field Coefficient 1 of the Pump Electric Use Part Load Ratio Curve
       \\note  The pump electric use coefficients specify the
       \\note  pumping power as a Fraction of Nominal pumping power
  N13, \\field Coefficient 2 of the Pump Electric Use Part Load Ratio Curve
  N14, \\field Coefficient 3 of the Pump Electric Use Part Load Ratio Curve
  N15, \\field Chilled Water Outlet Temperature Lower Limit
       \\units C
  A6 , \\field Generator Inlet Node Name
       \\type node
  A7 , \\field Generator Outlet Node Name
       \\type node
  A8 , \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  A9 , \\field Generator Heat Source Type
       \\type choice
       \\key Steam
       \\key HotWater
       \\default Steam
       \\note The Generator side of the chiller can be connected to a hot water or steam plant where the
       \\note generator inlet and outlet nodes are connected to a plant loop. If the generator is not
       \\note connected to a plant loop, and the generator inlet/outlet nodes are not used, this field should be
       \\note specified as steam or left blank. When a plant is not used, the model assumes steam as the heat source.
  N16, \\field Design Generator Fluid Flow Rate
       \\type real
       \\autosizable
       \\units m3/s
       \\minimum> 0.0
  N17, \\field Degree of Subcooling in Steam Generator
       \\type real
       \\units C
       \\default 1.0
       \\note This field is not used when the generator inlet/outlet nodes are not specified or
       \\note the generator is connected to a hot water loop.
  N18; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

Chiller:ConstantCOP,
  \\min-fields 12
       \\memo This constant COP chiller model provides a means of quickly specifying a
       \\memo Chiller where performance data is not available.
  A1 , \\field Name
       \\required-field
  N1 , \\field Nominal Capacity
       \\type real
       \\autosizable
       \\units W
       \\required-field
       \\minimum 0.0
  N2 , \\field Nominal COP
       \\type real
       \\units W/W
       \\required-field
       \\minimum> 0.0
  N3 , \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the maximum flow and for constant flow this is the flow.
       \\units m3/s
       \\ip-units gal/min
       \\autosizable
       \\minimum 0.0
  N4 , \\field Design Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\ip-units gal/min
       \\autosizable
       \\minimum 0.0
       \\note This field is not used for Condenser Type = AirCooled or EvaporativelyCooled
  A2 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A4 , \\field Condenser Inlet Node Name
       \\type node
  A5 , \\field Condenser Outlet Node Name
       \\type node
  A6 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  A7 , \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N5 , \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N6 , \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the chiller is not operating.
  N7 , \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A8 ; \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.

Chiller:EngineDriven,
   \\min-fields 43
       \\memo This chiller model is the empirical model from the Building Loads
       \\memo and System Thermodynamics (BLAST) program.  Chiller performance
       \\memo curves are generated by fitting catalog data to third order
       \\memo polynomial equations.  Three sets of coefficients are required.
  A1 , \\field Name
       \\required-field
  A2 , \\field Condenser Type
       \\required-field
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N1 , \\field Nominal Capacity
       \\units W
       \\required-field
       \\autosizable
       \\minimum 0.0
  N2 , \\field Nominal COP
       \\required-field
       \\units W/W
       \\note Nominal Refrigeration Cycle COP
       \\minimum> 0.0
  A3 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Inlet Node Name
       \\type node
  A6 , \\field Condenser Outlet Node Name
       \\type node
  N3 , \\field Minimum Part Load Ratio
       \\minimum 0.0
  N4 , \\field Maximum Part Load Ratio
       \\minimum 0.0
  N5 , \\field Optimum Part Load Ratio
       \\minimum 0.0
  N6 , \\field Design Condenser Inlet Temperature
       \\units C
  N7 , \\field Temperature Rise Coefficient
       \\required-field
       \\minimum> 0.0
  N8 , \\field Design Chilled Water Outlet Temperature
       \\units C
  N9 , \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the maximum flow and for constant flow this is the flow.
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
  N10, \\field Design Condenser Water Flow Rate
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
       \\note This field is not used for Condenser Type = AirCooled or EvaporativelyCooled
  N11, \\field Coefficient 1 of Capacity Ratio Curve
  N12, \\field Coefficient 2 of Capacity Ratio Curve
  N13, \\field Coefficient 3 of Capacity Ratio Curve
  N14, \\field Coefficient 1 of Power Ratio Curve
  N15, \\field Coefficient 2 of Power Ratio Curve
  N16, \\field Coefficient 3 of Power Ratio Curve
  N17, \\field Coefficient 1 of Full Load Ratio Curve
  N18, \\field Coefficient 2 of Full Load Ratio Curve
  N19, \\field Coefficient 3 of Full Load Ratio Curve
  N20, \\field Chilled Water Outlet Temperature Lower Limit
       \\units C
       \\note  Special EngineDriven Chiller Parameters Below
  A7, \\field Fuel Use Curve Name
       \\note Curve is a function of Part Load Ratio (PLR)
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of evaporator heat transfer rate to nominal capacity
  A8, \\field Jacket Heat Recovery Curve Name
       \\note Curve is a function of Part Load Ratio (PLR)
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of evaporator heat transfer rate to nominal capacity
  A9, \\field Lube Heat Recovery Curve Name
       \\note Curve is a function of Part Load Ratio (PLR)
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of evaporator heat transfer rate to nominal capacity
  A10, \\field Total Exhaust Energy Curve Name
       \\note Curve is a function of Part Load Ratio (PLR)
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of evaporator heat transfer rate to nominal capacity
  A11, \\field Exhaust Temperature Curve Name
       \\note Curve is a function of Part Load Ratio (PLR)
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of evaporator heat transfer rate to nominal capacity
  N21, \\field Coefficient 1 of U-Factor Times Area Curve
       \\note curve = C1 * (nominal capacity)^C2
  N22, \\field Coefficient 2 of U-Factor Times Area Curve
       \\note curve = C1 * (nominal capacity)^C2
  N23, \\field Maximum Exhaust Flow per Unit of Power Output
       \\units (kg/s)/W
       \\minimum 0.0
  N24, \\field Design Minimum Exhaust Temperature
       \\units C
  A12, \\field Fuel Type
       \\required-field
       \\type choice
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
  N25, \\field Fuel Higher Heating Value
       \\units kJ/kg
  N26, \\field Design Heat Recovery Water Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
       \\ip-units gal/min
  A13, \\field Heat Recovery Inlet Node Name
       \\type node
  A14, \\field Heat Recovery Outlet Node Name
       \\type node
  A15, \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  N27, \\field Maximum Temperature for Heat Recovery at Heat Recovery Outlet Node
       \\units C
       \\maximum 100.0
       \\minimum 0.0
       \\default 60.0
  N28, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N29, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, the heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the chiller is not operating.
  N30, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A16; \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.

Chiller:CombustionTurbine,
   \\min-fields 56
       \\memo This chiller model is the empirical model from the Building Loads
       \\memo and System Thermodynamics (BLAST) program.  Chiller performance
       \\memo curves are generated by fitting catalog data to third order
       \\memo polynomial equations.  Three sets of coefficients are required.
  A1 , \\field Name
       \\required-field
  A2 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\key EvaporativelyCooled
       \\default AirCooled
  N1 , \\field Nominal Capacity
       \\units W
       \\required-field
       \\autosizable
       \\minimum 0.0
  N2 , \\field Nominal COP
       \\required-field
       \\units W/W
       \\minimum> 0.0
  A3 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Inlet Node Name
       \\type node
  A6 , \\field Condenser Outlet Node Name
       \\type node
  N3 , \\field Minimum Part Load Ratio
       \\minimum 0.0
  N4 , \\field Maximum Part Load Ratio
       \\minimum 0.0
  N5 , \\field Optimum Part Load Ratio
       \\minimum 0.0
  N6 , \\field Design Condenser Inlet Temperature
       \\units C
  N7 , \\field Temperature Rise Coefficient
       \\required-field
       \\minimum> 0.0
  N8 , \\field Design Chilled Water Outlet Temperature
       \\units C
  N9 , \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the max flow & for constant flow this is the flow.
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
  N10, \\field Design Condenser Water Flow Rate
       \\units m3/s
       \\autosizable
       \\ip-units gal/min
       \\minimum 0.0
       \\note This field is not used for Condenser Type = AirCooled or EvaporativelyCooled
  N11, \\field Coefficient 1 of Capacity Ratio Curve
  N12, \\field Coefficient 2 of Capacity Ratio Curve
  N13, \\field Coefficient 3 of Capacity Ratio Curve
  N14, \\field Coefficient 1 of Power Ratio Curve
  N15, \\field Coefficient 2 of Power Ratio Curve
  N16, \\field Coefficient 3 of Power Ratio Curve
  N17, \\field Coefficient 1 of Full Load Ratio Curve
  N18, \\field Coefficient 2 of Full Load Ratio Curve
  N19, \\field Coefficient 3 of Full Load Ratio Curve
  N20, \\field Chilled Water Outlet Temperature Lower Limit
       \\units C
       \\note  Special Gas Turbine Chiller Parameters Below
  N21, \\field Coefficient 1 of Fuel Input Curve
  N22, \\field Coefficient 2 of Fuel Input Curve
  N23, \\field Coefficient 3 of Fuel Input Curve
  N24, \\field Coefficient 1 of Temperature Based Fuel Input Curve
  N25, \\field Coefficient 2 of Temperature Based Fuel Input Curve
  N26, \\field Coefficient 3 of Temperature Based Fuel Input Curve
  N27, \\field Coefficient 1 of Exhaust Flow Curve
  N28, \\field Coefficient 2 of Exhaust Flow Curve
  N29, \\field Coefficient 3 of Exhaust Flow Curve
  N30, \\field Coefficient 1 of Exhaust Gas Temperature Curve
  N31, \\field Coefficient 2 of Exhaust Gas Temperature Curve
  N32, \\field Coefficient 3 of Exhaust Gas Temperature Curve
  N33, \\field Coefficient 1 of Temperature Based Exhaust Gas Temperature Curve
  N34, \\field Coefficient 2 of Temperature Based Exhaust Gas Temperature Curve
  N35, \\field Coefficient 3 of Temperature Based Exhaust Gas Temperature Curve
  N36, \\field Coefficient 1 of Recovery Lube Heat Curve
  N37, \\field Coefficient 2 of Recovery Lube Heat Curve
  N38, \\field Coefficient 3 of Recovery Lube Heat Curve
  N39, \\field Coefficient 1 of U-Factor Times Area Curve
  N40, \\field Coefficient 2 of U-Factor Times Area Curve
  N41, \\field Gas Turbine Engine Capacity
       \\autosizable
       \\units W
       \\minimum> 0.0
  N42, \\field Maximum Exhaust Flow per Unit of Power Output
       \\units (kg/s)/W
  N43, \\field Design Steam Saturation Temperature
       \\units C
  N44, \\field Fuel Higher Heating Value
       \\units kJ/kg
  N45, \\field Design Heat Recovery Water Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\note If non-zero, then the heat recovery inlet and outlet node names must be entered.
       \\ip-units gal/min
  A7 , \\field Heat Recovery Inlet Node Name
       \\type node
  A8 , \\field Heat Recovery Outlet Node Name
       \\type node
  A9 , \\field Chiller Flow Mode
       \\note Select operating mode for fluid flow through the chiller. "NotModulated" is for
       \\note either variable or constant pumping with flow controlled by the external plant system.
       \\note "ConstantFlow" is for constant pumping with flow controlled by chiller to operate at
       \\note full design flow rate.  "LeavingSetpointModulated" is for variable pumping with flow
       \\note controlled by chiller to vary flow to target a leaving temperature setpoint.
       \\type choice
       \\key ConstantFlow
       \\key LeavingSetpointModulated
       \\key NotModulated
       \\default NotModulated
  A10, \\field Fuel Type
       \\required-field
       \\type choice
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default NaturalGas
  N46, \\field Heat Recovery Maximum Temperature
       \\units C
       \\maximum 100.0
       \\minimum 0.0
       \\default 80.0
  N47, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  N48, \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
       \\note when the basin heater is available (field Basin Heater Operating Schedule Name).
       \\note For this situation, The heater maintains the basin water temperature at the basin heater
       \\note setpoint temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when the chiller is not operating.
  N49, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on.
  A11; \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field is only used for Condenser Type = EvaporativelyCooled.
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.


ChillerHeater:Absorption:DirectFired,
  \\min-fields 35
  \\memo Direct fired gas absorption chiller-heater using performance curves similar to DOE-2
  A1,  \\field Name
       \\required-field
  N1 , \\field Nominal Cooling Capacity
       \\required-field
       \\autosizable
       \\default autosize
       \\units W
  N2 , \\field Heating to Cooling Capacity Ratio
       \\note A positive fraction that represents the ratio of the
       \\note heating capacity divided by the cooling capacity at rated conditions.
       \\minimum  0
       \\default  0.8
  N3 , \\field Fuel Input to Cooling Output Ratio
            \\note The positive fraction that represents the ratio of the
            \\note instantaneous fuel used divided by the cooling capacity at rated conditions.
            \\minimum> 0
            \\default  0.97
  N4   ,  \\field Fuel Input to Heating Output Ratio
            \\note The positive fraction that represents the ratio of the
            \\note instantaneous fuel used divided by the nominal heating capacity.
            \\minimum  0
            \\default  1.25
  N5   ,  \\field Electric Input to Cooling Output Ratio
            \\note The positive fraction that represents the ratio of the
            \\note instantaneous electricity used divided by the cooling capacity at rated conditions.
            \\note If the chiller is both heating and cooling only the cooling electricity is used.
            \\minimum  0
            \\default  0.01
  N6   ,  \\field Electric Input to Heating Output Ratio
            \\note The positive fraction that represents the ratio of the
            \\note instantaneous electricity used divided by the nominal heating capacity.
            \\note If the chiller is both heating and cooling only the cooling electricity is used.
            \\minimum  0
            \\default  0
  A2   ,  \\field Chilled Water Inlet Node Name
          \\required-field
          \\type node
  A3   ,  \\field Chilled Water Outlet Node Name
          \\required-field
          \\type node
  A4   ,  \\field Condenser Inlet Node Name
          \\required-field
          \\type node
  A5   ,  \\field Condenser Outlet Node Name
          \\required-field
          \\type node
  A6   ,  \\field Hot Water Inlet Node Name
          \\required-field
          \\type node
  A7   ,  \\field Hot Water Outlet Node Name
          \\required-field
          \\type node
  N7   ,  \\field Minimum Part Load Ratio
            \\note The positive fraction that represents the minimum cooling output possible when
            \\note operated continually at rated temperature conditions divided by the nominal
            \\note cooling capacity at those same conditions.  If the load on the chiller is below
            \\note this fraction the chiller will cycle.
            \\minimum> 0
            \\default  0.1
  N8   ,  \\field Maximum Part Load Ratio
            \\note The positive fraction that represents the maximum cooling output possible at
            \\note rated temperature conditions divided by the nominal cooling capacity at those
            \\note same conditions.  If greater than 1.0, the chiller is typically thought of as
            \\note capable of being overloaded.
            \\minimum  0.5
            \\default  1.0
  N9   ,  \\field Optimum Part Load Ratio
            \\note The positive fraction that represents the optimal cooling output at rated
            \\note temperature conditions divided by the nominal cooling capacity at those same
            \\note conditions.  It represents the most desirable operating point for the chiller.
            \\minimum> 0
            \\default  1.0
  N10  ,  \\field Design Entering Condenser Water Temperature
            \\note The temperature of the water entering the condenser of the chiller when
            \\note operating at design conditions.  This is usually based on the temperature
            \\note delivered by the cooling tower in a water cooled application.
            \\default 29
            \\units C
  N11  ,  \\field Design Leaving Chilled Water Temperature
            \\note The temperature of the water leaving the evaporator of the chiller when
            \\note operating at design conditions also called the chilled water supply temperature
            \\note or leaving chilled water temperature.
            \\default 7
            \\units C
  N12 ,   \\field Design Chilled Water Flow Rate
            \\note  For variable volume this is the max flow & for constant flow this is the flow.
            \\autosizable
            \\default autosize
            \\units m3/s
            \\ip-units gal/min
  N13  ,  \\field Design Condenser Water Flow Rate
            \\note The water flow rate at design conditions through the condenser.
            \\autosizable
            \\default autosize
            \\required-field
            \\units m3/s
            \\ip-units gal/min
            \\note This field is not used for Condenser Type = AirCooled
  N14  ,  \\field Design Hot Water Flow Rate
            \\note The water flow rate at design conditions through the heater side.
            \\autosizable
            \\default autosize
            \\units m3/s
            \\ip-units gal/min
  A8   ,  \\field Cooling Capacity Function of Temperature Curve Name
            \\note The CoolCapFT curve represents the fraction of the cooling capacity of the chiller as it
            \\note varies by temperature.  The curve is normalized so that at design conditions the
            \\note value of the curve should be 1.0.  This is a biquadratic curve with the
            \\note input variables being the leaving chilled water temperature and either
            \\note the entering or leaving condenser water temperature.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A9   ,  \\field Fuel Input to Cooling Output Ratio Function of Temperature Curve Name
            \\note The curve represents the fraction of the fuel input to the chiller at full load as
            \\note it varies by temperature.  The curve is normalized so that at design conditions the
            \\note value of the curve should be 1.0.  This is a biquadratic curve with the
            \\note input variables being the leaving chilled water temperature and either
            \\note the entering or leaving condenser water temperature.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A10  ,  \\field Fuel Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
            \\note The curve represents the fraction of the fuel input to the chiller as the load on
            \\note the chiller varies but the operating temperatures remain at the design values.
            \\note The curve is normalized so that at full load the value of the curve should be 1.0.
            \\note The curve is usually linear or quadratic.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A11  ,  \\field Electric Input to Cooling Output Ratio Function of Temperature Curve Name
            \\note The curve represents the fraction of the electricity to the chiller at full load as
            \\note it varies by temperature.  The curve is normalized so that at design conditions the
            \\note value of the curve should be 1.0.  This is a biquadratic curve with the
            \\note input variables being the leaving chilled water temperature and either
            \\note the entering or leaving condenser water temperature.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A12  ,  \\field Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
            \\note The curve represents the fraction of the electricity to the chiller as the load on
            \\note the chiller varies but the operating temperatures remain at the design values.
            \\note The curve is normalized so that at full load the value of the curve should be 1.0.
            \\note The curve is usually linear or quadratic.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A13  ,  \\field Heating Capacity Function of Cooling Capacity Curve Name
            \\note The curve represents how the heating capacity of the chiller varies with cooling
            \\note capacity when the chiller is simultaeous heating and cooling.  The curve is normalized
            \\note so an input of 1.0 represents the nominal cooling capacity and an output of 1.0
            \\note represents the full heating capacity (see the Heating to cooling capacity ratio input)
            \\note The curve is usually linear or quadratic.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A14  ,  \\field Fuel Input to Heat Output Ratio During Heating Only Operation Curve Name
            \\note When the chiller is operating as only a heater, this curve is used to represent the
            \\note fraction of fuel used as the heating load varies.  It is normalized so that a value
            \\note of 1.0 is the full heating capacity.  The curve is usually linear or quadratic and
            \\note will probably be similar to a boiler curve for most chillers.
            \\object-list LinearCurves
            \\object-list QuadraticCurves
            \\object-list CubicCurves
            \\object-list QuarticCurves
            \\object-list ExponentCurves
  A15  ,  \\field Temperature Curve Input Variable
            \\type choice
            \\key LeavingCondenser
            \\key EnteringCondenser
            \\default EnteringCondenser
            \\note Sets the second independent variable in the three temperature dependent performance
            \\note curves to either the leaving or entering condenser water temperature.  Manufacturers
            \\note express the performance of their chillers using either the leaving condenser water
            \\note temperature (to the tower) or the entering condenser water temperature (from the tower).
  A16  ,  \\field Condenser Type
            \\note The condenser can either be air cooled or connected to a cooling tower.
            \\type choice
            \\key AirCooled
            \\key WaterCooled
            \\default WaterCooled
  N15  ,  \\field Chilled Water Temperature Lower Limit
            \\note The chilled water supply temperature below which the chiller
            \\note will shut off.
            \\units C
            \\default 2.0
  N16  ,  \\field Fuel Higher Heating Value
            \\note Not currently used.
            \\units kJ/kg
            \\default 0.0
  A17  ,  \\field Chiller Flow Mode
       \\deprecated
  A18  ,  \\field Fuel Type
            \\required-field
            \\type choice
            \\key NaturalGas
            \\key PropaneGas
            \\key Diesel
            \\key Gasoline
            \\key FuelOil#1
            \\key FuelOil#2
            \\key OtherFuel1
            \\key OtherFuel2
            \\default NaturalGas
  N17  ;  \\field Sizing Factor
            \\note Multiplies the autosized capacity and flow rates
            \\type real
            \\minimum> 0.0
            \\default 1.0

ChillerHeater:Absorption:DoubleEffect,
  \\min-fields 34
  \\memo Exhaust fired absorption chiller-heater using performance curves similar to DOE-2
  A1,  \\field Name
       \\type alpha
       \\required-field
  N1 , \\field Nominal Cooling Capacity
       \\required-field
       \\autosizable
       \\default autosize
       \\units W
  N2 , \\field Heating to Cooling Capacity Ratio
       \\note A positive fraction that represents the ratio of the
       \\note heating capacity divided by the cooling capacity at rated conditions.
       \\minimum 0
       \\default 0.8
  N3 , \\field Thermal Energy Input to Cooling Output Ratio
       \\note The positive fraction that represents the ratio of the
       \\note instantaneous fuel used divided by the cooling capacity at rated conditions.
       \\minimum> 0
       \\default 0.97
  N4 , \\field Thermal Energy Input to Heating Output Ratio
       \\note The positive fraction that represents the ratio of the
       \\note instantaneous fuel used divided by the nominal heating capacity.
       \\minimum 0
       \\default 1.25
  N5 , \\field Electric Input to Cooling Output Ratio
       \\note The positive fraction that represents the ratio of the
       \\note instantaneous electricity used divided by the cooling capacity at rated conditions.
       \\note If the chiller is both heating and cooling only the cooling electricity is used.
       \\minimum 0
       \\default 0.01
  N6 , \\field Electric Input to Heating Output Ratio
       \\note The positive fraction that represents the ratio of the
       \\note instantaneous electricity used divided by the nominal heating capacity.
       \\note If the chiller is both heating and cooling only the cooling electricity is used.
       \\minimum 0
       \\default 0
  A2 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  A4 , \\field Condenser Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Condenser Outlet Node Name
       \\required-field
       \\type node
  A6 , \\field Hot Water Inlet Node Name
       \\required-field
       \\type node
  A7 , \\field Hot Water Outlet Node Name
       \\required-field
       \\type node
  N7 , \\field Minimum Part Load Ratio
       \\note The positive fraction that represents the minimum cooling output possible when
       \\note operated continually at rated temperature conditions divided by the nominal
       \\note cooling capacity at those same conditions.  If the load on the chiller is below
       \\note this fraction the chiller will cycle.
       \\minimum> 0
       \\default 0.1
  N8 , \\field Maximum Part Load Ratio
       \\note The positive fraction that represents the maximum cooling output possible at
       \\note rated temperature conditions divided by the nominal cooling capacity at those
       \\note same conditions.  If greater than 1.0, the chiller is typically thought of as
       \\note capable of being overloaded.
       \\minimum 0.5
       \\default 1.0
  N9 , \\field Optimum Part Load Ratio
       \\note The positive fraction that represents the optimal cooling output at rated
       \\note temperature conditions divided by the nominal cooling capacity at those same
       \\note conditions.  It represents the most desirable operating point for the chiller.
       \\minimum> 0
       \\default 1.0
  N10, \\field Design Entering Condenser Water Temperature
       \\note The temperature of the water entering the condenser of the chiller when
       \\note operating at design conditions.  This is usually based on the temperature
       \\note delivered by the cooling tower in a water cooled application.
       \\default 29
       \\units C
  N11, \\field Design Leaving Chilled Water Temperature
       \\note The temperature of the water leaving the evaporator of the chiller when
       \\note operating at design conditions also called the chilled water supply temperature
       \\note or leaving chilled water temperature.
       \\default 7
       \\units C
  N12, \\field Design Chilled Water Flow Rate
       \\note  For variable volume this is the max flow & for constant flow this is the flow.
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
  N13, \\field Design Condenser Water Flow Rate
       \\note The water flow rate at design conditions through the condenser.
       \\note This field is not used for Condenser Type = AirCooled
       \\autosizable
       \\default autosize
       \\required-field
       \\units m3/s
       \\ip-units gal/min
  N14, \\field Design Hot Water Flow Rate
       \\note The water flow rate at design conditions through the heater side.
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
  A8 , \\field Cooling Capacity Function of Temperature Curve Name
       \\note The CoolCapFT curve represents the fraction of the cooling capacity of the chiller as it
       \\note varies by temperature.  The curve is normalized so that at design conditions the
       \\note value of the curve should be 1.0.  This is a biquadratic curve with the
       \\note input variables being the leaving chilled water temperature and either
       \\note the entering or leaving condenser water temperature.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A9 , \\field Fuel Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note The curve represents the fraction of the fuel input to the chiller at full load as
       \\note it varies by temperature.  The curve is normalized so that at design conditions the
       \\note value of the curve should be 1.0.  This is a biquadratic curve with the
       \\note input variables being the leaving chilled water temperature and either
       \\note the entering or leaving condenser water temperature.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A10, \\field Fuel Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note The curve represents the fraction of the fuel input to the chiller as the load on
       \\note the chiller varies but the operating temperatures remain at the design values.
       \\note The curve is normalized so that at full load the value of the curve should be 1.0.
       \\note The curve is usually linear or quadratic.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A11, \\field Electric Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note The curve represents the fraction of the electricity to the chiller at full load as
       \\note it varies by temperature.  The curve is normalized so that at design conditions the
       \\note value of the curve should be 1.0.  This is a biquadratic curve with the
       \\note input variables being the leaving chilled water temperature and either
       \\note the entering or leaving condenser water temperature.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A12, \\field Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note The curve represents the fraction of the electricity to the chiller as the load on
       \\note the chiller varies but the operating temperatures remain at the design values.
       \\note The curve is normalized so that at full load the value of the curve should be 1.0.
       \\note The curve is usually linear or quadratic.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A13, \\field Heating Capacity Function of Cooling Capacity Curve Name
       \\note The curve represents how the heating capacity of the chiller varies with cooling
       \\note capacity when the chiller is simultaeous heating and cooling.  The curve is normalized
       \\note so an input of 1.0 represents the nominal cooling capacity and an output of 1.0
       \\note represents the full heating capacity (see the Heating to cooling capacity ratio input)
       \\note The curve is usually linear or quadratic.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A14, \\field Fuel Input to Heat Output Ratio During Heating Only Operation Curve Name
       \\note When the chiller is operating as only a heater, this curve is used to represent the
       \\note fraction of fuel used as the heating load varies.  It is normalized so that a value
       \\note of 1.0 is the full heating capacity.  The curve is usually linear or quadratic and
       \\note will probably be similar to a boiler curve for most chillers.
       \\object-list LinearCurves
       \\object-list QuadraticCurves
       \\object-list CubicCurves
       \\object-list QuarticCurves
       \\object-list ExponentCurves
  A15, \\field Temperature Curve Input Variable
       \\type choice
       \\key LeavingCondenser
       \\key EnteringCondenser
       \\default EnteringCondenser
       \\note Sets the second independent variable in the three temperature dependent performance
       \\note curves to either the leaving or entering condenser water temperature.  Manufacturers
       \\note express the performance of their chillers using either the leaving condenser water
       \\note temperature (to the tower) or the entering condenser water temperature (from the tower).
  A16, \\field Condenser Type
       \\note The condenser can either be air cooled or connected to a cooling tower.
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\default WaterCooled
  N15, \\field Chilled Water Temperature Lower Limit
       \\note The chilled water supply temperature below which the chiller
       \\note will shut off.
       \\units C
       \\default 2.0
  A17, \\field Exhaust Source Object Type
       \\type choice
       \\required-field
       \\key Generator:MicroTurbine
  A18, \\field Exhaust Source Object Name
       \\type object-list
       \\object-list MicroTurbineGeneratorNames
  N16; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

HeatPump:WaterToWater:EquationFit:Heating,
        \\memo simple water-water hp curve-fit model
        \\min-fields 20
   A1,  \\field Name
        \\required-field
   A2,  \\field Source Side Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Source Side Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Load Side Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Load Side Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Rated Load Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N2,  \\field Rated Source Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N3,  \\field Rated Heating Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N4,  \\field Rated Heating Power Consumption
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N5,  \\field Heating Capacity Coefficient 1
        \\required-field
        \\type real
   N6,  \\field Heating Capacity Coefficient 2
        \\required-field
        \\type real
   N7,  \\field Heating Capacity Coefficient 3
        \\required-field
        \\type real
   N8,  \\field Heating Capacity Coefficient 4
        \\required-field
        \\type real
   N9,  \\field Heating Capacity Coefficient 5
        \\required-field
        \\type real
   N10, \\field Heating Compressor Power Coefficient 1
        \\required-field
        \\type real
   N11, \\field Heating Compressor Power Coefficient 2
        \\required-field
        \\type real
   N12, \\field Heating Compressor Power Coefficient 3
        \\required-field
        \\type real
   N13, \\field Heating Compressor Power Coefficient 4
        \\required-field
        \\type real
   N14, \\field Heating Compressor Power Coefficient 5
        \\required-field
        \\type real
   N15; \\field Cycle Time
        \\required-field
        \\type real
        \\units hr
        \\minimum> 0.016
        \\default .1

HeatPump:WaterToWater:EquationFit:Cooling,
        \\memo simple water-water heatpump curve-fit model
        \\min-fields 20
   A1,  \\field Name
        \\required-field
   A2,  \\field Source Side Inlet Node Name
        \\required-field
        \\type node
   A3,  \\field Source Side Outlet Node Name
        \\required-field
        \\type node
   A4,  \\field Load Side Inlet Node Name
        \\required-field
        \\type node
   A5,  \\field Load Side Outlet Node Name
        \\required-field
        \\type node
   N1,  \\field Rated Load Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N2,  \\field Rated Source Side Flow Rate
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units m3/s
   N3,  \\field Rated Cooling Capacity
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N4,  \\field Rated Cooling Power Consumption
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units W
   N5,  \\field Cooling Capacity Coefficient 1
        \\required-field
        \\type real
   N6,  \\field Cooling Capacity Coefficient 2
        \\required-field
        \\type real
   N7,  \\field Cooling Capacity Coefficient 3
        \\required-field
        \\type real
   N8,  \\field Cooling Capacity Coefficient 4
        \\required-field
        \\type real
   N9,  \\field Cooling Capacity Coefficient 5
        \\required-field
        \\type real
   N10, \\field Cooling Compressor Power Coefficient 1
        \\required-field
        \\type real
   N11, \\field Cooling Compressor Power Coefficient 2
        \\required-field
        \\type real
   N12, \\field Cooling Compressor Power Coefficient 3
        \\required-field
        \\type real
   N13, \\field Cooling Compressor Power Coefficient 4
        \\required-field
        \\type real
   N14, \\field Cooling Compressor Power Coefficient 5
        \\required-field
        \\type real
   N15; \\field Cycle Time
        \\required-field
        \\type real
        \\units hr
        \\minimum> 0.016
        \\default .1

HeatPump:WaterToWater:ParameterEstimation:Cooling,
  \\memo OSU parameter estimation model
  \\min-fields 20
  A1,  \\field Name
       \\required-field
  A2,\\field Source Side Inlet Node Name
       \\required-field
       \\type node
  A3,\\field Source Side Outlet Node Name
       \\required-field
       \\type node
  A4,\\field Load Side Inlet Node Name
       \\required-field
       \\type node
  A5,\\field Load Side Outlet Node Name
       \\required-field
       \\type node
  N1,\\field Nominal COP
       \\units W/W
     \\minimum> 0
  N2,\\field Nominal Capacity
       \\units W
     \\minimum> 0
  N3,\\field Minimum Part Load Ratio
       \\minimum 0.0
  N4,\\field Maximum Part Load Ratio
       \\minimum 0.0
  N5,\\field Optimum Part Load Ratio
       \\minimum 0.0
  N6,\\field Load Side Flow Rate
       \\units m3/s
       \\minimum> 0.0
       \\ip-units gal/min
  N7,\\field Source Side Flow Rate
       \\units m3/s
       \\minimum> 0.0
       \\ip-units gal/min
  N8,\\field Load Side Heat Transfer Coefficient
     \\units W/K
     \\minimum> 0
  N9,\\field Source Side Heat Transfer Coefficient
     \\units W/K
     \\minimum> 0
  N10,\\field Piston Displacement
      \\units m3/s
     \\minimum> 0
  N11,\\field Compressor Clearance Factor
     \\minimum> 0
  N12,\\field Compressor Suction and Discharge Pressure Drop
      \\units Pa
     \\minimum> 0
  N13,\\field Superheating
      \\units C
     \\minimum> 0
  N14,\\field Constant Part of Electromechanical Power Losses
      \\units W
     \\minimum> 0
  N15,\\field Loss Factor
      \\note Used to define electromechanical loss that is proportional
      \\note to the theoretical power %
     \\minimum> 0
  N16,\\field High Pressure Cut Off
      \\units Pa
      \\minimum 0.0
      \\default 500000000
  N17,\\field Low Pressure Cut Off
      \\units Pa
      \\minimum 0.0
      \\default 0.0
  N18;\\field Cycle Time
      \\units hr
      \\minimum> 0.016
      \\default .1

HeatPump:WaterToWater:ParameterEstimation:Heating,
   \\memo OSU parameter estimation model
   \\min-fields 20
  A1,  \\field Name
       \\required-field
  A2,\\field Source Side Inlet Node Name
       \\required-field
       \\type node
  A3,\\field Source Side Outlet Node Name
       \\required-field
       \\type node
  A4,\\field Load Side Inlet Node Name
       \\required-field
       \\type node
  A5,\\field Load Side Outlet Node Name
       \\required-field
       \\type node
  N1,\\field Nominal COP
       \\units W/W
     \\minimum> 0
  N2,\\field Nominal Capacity
       \\units W
     \\minimum> 0
  N3,\\field Minimum Part Load Ratio
       \\minimum 0.0
  N4,\\field Maximum Part Load Ratio
       \\minimum 0.0
  N5,\\field Optimum Part Load Ratio
       \\minimum 0.0
  N6,\\field Load Side Flow Rate
       \\units m3/s
     \\minimum> 0
     \\ip-units gal/min
  N7,\\field Source Side Flow Rate
       \\units m3/s
     \\minimum> 0
     \\ip-units gal/min
  N8,\\field Load Side Heat Transfer Coefficient
     \\units W/K
     \\minimum> 0
  N9,\\field Source Side Heat Transfer Coefficient
      \\units W/K
     \\minimum> 0
  N10,\\field Piston Displacement
      \\units m3/s
     \\minimum> 0
  N11,\\field Compressor Clearance Factor
     \\minimum> 0
  N12,\\field Compressor Suction and Discharge Pressure Drop
      \\units Pa
     \\minimum> 0
  N13,\\field Superheating
      \\units C
     \\minimum> 0
  N14,\\field Constant Part of Electromechanical Power Losses
      \\units W
     \\minimum> 0
  N15,\\field Loss Factor
      \\note Used to define electromechanical loss that is proportional
      \\note to the theoretical power %
     \\minimum> 0
  N16,\\field High Pressure Cut Off
      \\units Pa
      \\minimum 0.0
      \\default 500000000
  N17,\\field Low Pressure Cut Off
      \\units Pa
      \\minimum 0.0
      \\default 0.0
  N18;\\field Cycle Time
      \\units hr
      \\minimum> 0.016
      \\default .1

DistrictCooling,
       \\memo Centralized source of chilled water, such as a district cooling system.
  A1 , \\field Name
       \\required-field
  A2 , \\field Chilled Water Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Chilled Water Outlet Node Name
       \\required-field
       \\type node
  N1 ; \\field Nominal Capacity
       \\required-field
       \\units W
       \\minimum 0.0

DistrictHeating,
       \\memo Centralized source of hot water, such as a district heating system.
  A1 , \\field Name
       \\required-field
  A2 , \\field Hot Water Inlet Node Name
       \\required-field
       \\type node
  A3 , \\field Hot Water Outlet Node Name
       \\required-field
       \\type node
  N1 ; \\field Nominal Capacity
       \\required-field
       \\units W
       \\minimum 0.0

PlantComponent:TemperatureSource,
       \\memo Simulates an object of pre-determined (constant or scheduled) source temperature
       \\memo The object introduces fluid into the plant loop at the specified temperature and
       \\memo  at the same flow rate as the fluid enters the component
       \\memo Fluid entering the component vanishes equivalent to the relief air in an air system
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference RiverWellOrSeawaterSourceNames
       \\note Component Name
  A2 , \\field Inlet Node
       \\required-field
       \\type node
       \\note Name of the source inlet node
  A3 , \\field Outlet Node
       \\required-field
       \\type node
       \\note Name of the source outlet node
  N1 , \\field Design Volume Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note The design volumetric flow rate for this source
       \\ip-units gal/min
  A4 , \\field Temperature Specification Type
       \\type choice
       \\key Constant
       \\key Scheduled
  N2 , \\field Source Temperature
       \\type real
       \\units C
       \\note Used if Temperature Specification Type = Constant
  A5 ; \\field Source Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\units C
       \\note Used if Temperature Specification Type = Scheduled

CentralHeatPumpSystem,
       \\min-fields 14
       \\memo This chiller bank can contain multiple chiller heaters and heat pump performance objects.
       \\memo Its function is to encapsulate the extra controls needed to turn individual modules on/off
       \\memo and whether they are to operate in cooling-only, heating-only or simultaneous cooling/heating
       \\memo mode and whether to connect the source water to the evaporator or condenser side.
  A1,  \\field Name
       \\type alpha
       \\required-field
  A2,  \\field Control Method
       \\type choice
       \\key SmartMixing
       \\default SmartMixing
  A3,  \\field Cooling Loop Inlet Node Name
       \\type alpha
       \\required-field
  A4,  \\field Cooling Loop Outlet Node Name
       \\type alpha
       \\required-field
  A5,  \\field Source Loop Inlet Node Name
       \\type alpha
       \\required-field
  A6,  \\field Source Loop Outlet Node Name
       \\type alpha
       \\required-field
  A7,  \\field Heating Loop Inlet Node Name
       \\type alpha
       \\required-field
  A8,  \\field Heating Loop Outlet Node Name
       \\type alpha
       \\required-field
  N1,  \\field Ancillary Power
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\note Power as demanded from any auxiliary controls
  A9,  \\field Ancillary Operation Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This value from this schedule is multiplied times the Ancilliary Power
  A10, \\field Chiller Heater Modules Performance Component Object Type 1
       \\required-field
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A11, \\field Chiller Heater Modules Performance Component Name 1
       \\required-field
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A12, \\field Chiller Heater Modules Control Schedule Name 1
       \\type object-list
       \\object-list ScheduleNames
       \\required-field
  N2,  \\field Number of Chiller Heater Modules 1
       \\type integer
       \\minimum 1
       \\default 1
       \\required-field
  A13, \\field Chiller Heater Modules Performance Component Object Type 2
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A14, \\field Chiller Heater Modules Performance Component Name 2
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A15, \\field Chiller Heater Modules Control Schedule Name 2
       \\type object-list
       \\object-list ScheduleNames
  N3,  \\field Number of Chiller Heater Modules 2
       \\type integer
       \\minimum 1
       \\default 1
  A16, \\field Chiller Heater Performance Component Object Type 3
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A17, \\field Chiller Heater Performance Component Name 3
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A18, \\field Chiller Heater Modules Control Schedule Name 3
       \\type object-list
       \\object-list ScheduleNames
  N4,  \\field Number of Chiller Heater Modules 3
       \\type integer
       \\minimum 1
       \\default 1
  A19, \\field Chiller Heater Modules Performance Component Object Type 4
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A20, \\field Chiller Heater Modules Performance Component Name 4
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A21, \\field Chiller Heater Modules Control Schedule Name 4
       \\type object-list
       \\object-list ScheduleNames
  N5,  \\field Number of Chiller Heater Modules 4
       \\type integer
       \\minimum 1
       \\default 1
  A22, \\field Chiller Heater Modules Performance Component Object Type 5
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A23, \\field Chiller Heater Models Performance Component Name 5
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A24, \\field Chiller Heater Modules Control Schedule Name 5
       \\type object-list
       \\object-list ScheduleNames
  N6,  \\field Number of Chiller Heater Modules 5
       \\type integer
       \\minimum 1
       \\default 1
  A25, \\field Chiller Heater Modules Performance Component Object Type 6
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A26, \\field Chiller Heater Modules Performance Component Name 6
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A27, \\field Chiller Heater Modules Control Schedule Name 6
       \\type object-list
       \\object-list ScheduleNames
  N7,  \\field Number of Chiller Heater Modules 6
       \\type integer
       \\minimum 1
       \\default 1
  A28, \\field Chiller Heater Modules Performance Component Object Type 7
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A29, \\field Chiller Heater Modules Performance Component Name 7
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A30, \\field Chiller Heater Modules Control Schedule Name 7
       \\type object-list
       \\object-list ScheduleNames
  N8,  \\field Number of Chiller Heater Modules 7
       \\type integer
       \\minimum 1
       \\default 1
  A31, \\field Chiller Heater Modules Performance Component Object Type 8
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A32, \\field Chiller Heater Modules Performance Component Name 8
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A33, \\field Chiller Heater Modules Control Schedule Name 8
       \\type object-list
       \\object-list ScheduleNames
  N9,  \\field Number of Chiller Heater Modules 8
       \\type integer
       \\minimum 1
       \\default 1
  A34, \\field Chiller Heater Modules Performance Component Object Type 9
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A35, \\field Chiller Heater Modules Performance Component Name 9
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A36, \\field Chiller Heater Modules Control Schedule Name 9
       \\type object-list
       \\object-list ScheduleNames
  N10, \\field Number of Chiller Heater Modules 9
       \\type integer
       \\minimum 1
       \\default 1
  A37, \\field Chiller Heater Modules Performance Component Object Type 10
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A38, \\field Chiller Heater Modules Performance Component Name 10
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A39, \\field Chiller Heater Modules Control Schedule Name 10
       \\type object-list
       \\object-list ScheduleNames
  N11, \\field Number of Chiller Heater Modules 10
       \\type integer
       \\minimum 1
       \\default 1
  A40, \\field Chiller Heater Modules Performance Component Object Type 11
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A41, \\field Chiller Heater Modules Performance Component Name 11
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A42, \\field Chiller Heater Module Control Schedule Name 11
       \\type object-list
       \\object-list ScheduleNames
  N12, \\field Number of Chiller Heater Modules 11
       \\type integer
       \\minimum 1
       \\default 1
  A43, \\field Chiller Heater Modules Performance Component Object Type 12
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A44, \\field Chiller Heater Modules Performance Component Name 12
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A45, \\field Chiller Heater Modules Control Schedule Name 12
       \\type object-list
       \\object-list ScheduleNames
  N13, \\field Number of Chiller Heater Modules 12
       \\type integer
       \\minimum 1
       \\default 1
  A46, \\field Chiller Heater Modules Performance Component Object Type 13
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A47, \\field Chiller Heater Modules Performance Component Name 13
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A48, \\field Chiller Heater Modules Control Schedule Name 13
       \\type object-list
       \\object-list ScheduleNames
  N14, \\field Number of Chiller Heater Modules 13
       \\type integer
       \\minimum 1
       \\default 1
  A49, \\field Chiller Heater Modules Performance Component Object Type 14
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A50, \\field Chiller Heater Modules Performance Component Name 14
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A51, \\field Chiller Heater Modules Control Schedule Name 14
       \\type object-list
       \\object-list ScheduleNames
  N15, \\field Number of Chiller Heater Modules 14
       \\type integer
       \\minimum 1
       \\default 1
  A52, \\field Chiller Heater Modules Performance Component Object Type 15
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A53, \\field Chiller Heater Modules Performance Component Name 15
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A54, \\field Chiller Heater Modules Control Schedule Name 15
       \\type object-list
       \\object-list ScheduleNames
  N16, \\field Number of Chiller Heater Modules 15
       \\type integer
       \\minimum 1
       \\default 1
  A55, \\field Chiller Heater Modules Performance Component Object Type 16
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A56, \\field Chiller Heater Modules Performance Component Name 16
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A57, \\field Chiller Heater Modules Control Schedule Name 16
       \\type object-list
       \\object-list ScheduleNames
  N17, \\field Number of Chiller Heater Modules 16
       \\type integer
       \\minimum 1
       \\default 1
  A58, \\field Chiller Heater Modules Performance Component Object Type 17
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A59, \\field Chiller Heater Modules Performance Component Name 17
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A60, \\field Chiller Heater Modules Control Schedule Name 17
       \\type object-list
       \\object-list ScheduleNames
  N18, \\field Number of Chiller Heater Modules 17
       \\type integer
       \\minimum 1
       \\default 1
  A61, \\field Chiller Heater Modules Performance Component Object Type 18
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A62, \\field Chiller Heater Modules Performance Component Name 18
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A63, \\field Chiller Heater Modules Control Control Schedule Name 18
       \\type object-list
       \\object-list ScheduleNames
  N19, \\field Number of Chiller Heater Modules 18
       \\type integer
       \\minimum 1
       \\default 1
  A64, \\field Chiller Heater Modules Performance Component Object Type 19
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A65, \\field Chiller Heater Modules Performance Component Name 19
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A66, \\field Chiller Heater Modules Control Schedule Name 19
       \\type object-list
       \\object-list ScheduleNames
  N20, \\field Number of Chiller Heater Modules 19
       \\type integer
       \\minimum 1
       \\default 1
  A67, \\field Chiller Heater Modules Performance Component Object Type 20
       \\type choice
       \\key ChillerHeaterPerformance:Electric:EIR
  A68, \\field Chiller Heater Modules Performance Component Name 20
       \\type object-list
       \\object-list ChillerHeaterEIRNames
  A69, \\field Chiller Heater Modules Control Schedule Name 20
       \\type object-list
       \\object-list ScheduleNames
  N21; \\field Number of Chiller Heater Modules 20
       \\type integer
       \\minimum 1
       \\default 1

ChillerHeaterPerformance:Electric:EIR,
       \\min-fields 29
       \\memo This chiller model is a generic chiller-heater where the cooling mode performance is a
       \\memo function of condenser entering or leaving fluid temperature and the heating mode performance
       \\memo is typically a function of condenser leaving fluid temperature. Performance at off-reference
       \\memo conditions is modeled using three polynomial equations per mode. Six curve objects are required.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference ChillerHeaterEIRNames
  N1 , \\field Reference Cooling Mode Evaporator Capacity
       \\type real
       \\units W
       \\required-field
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Reference Cooling Mode COP
       \\note Efficiency of the chiller compressor (cooling output/compressor energy input).
       \\type real
       \\units W/W
       \\required-field
       \\minimum> 0.0
  N3 , \\field Reference Cooling Mode Leaving Chilled Water Temperature
       \\type real
       \\default 6.67
       \\units C
  N4 , \\field Reference Cooling Mode Entering Condenser Fluid Temperature
       \\type real
       \\default 29.44
       \\units C
  N5 , \\field Reference Cooling Mode Leaving Condenser Water Temperature
       \\type real
       \\default 35.0
       \\units C
  N6 , \\field Reference Heating Mode Cooling Capacity Ratio
       \\note During simultaneous cooling-heating mode, this ratio is relative to the Reference Cooling Mode Cooling Capacity
       \\note (Evaporator capacity at simul clg-htg mode ref condition)/ (Evaporator capacity at cooling mode ref condition)
       \\type real
       \\default 0.75
  N7 , \\field Reference Heating Mode Cooling Power Input Ratio
       \\note During simultaneous cooling-heating mode, this ratio is relative to the Reference Cooling Mode COP
       \\note (Power at simultaneous clg-htg mode reference condition)/ (Power at cooling mode reference condition)
       \\type real
       \\default 1.38
       \\minimum> 0.0
  N8 , \\field Reference Heating Mode Leaving Chilled Water Temperature
       \\note During simultaneous cooling-heating mode
       \\type real
       \\default 6.67
       \\units C
  N9 , \\field Reference Heating Mode Leaving Condenser Water Temperature
       \\note During simultaneous cooling-heating mode
       \\type real
       \\default 49
       \\units C
  N10, \\field Reference Heating Mode Entering Condenser Fluid Temperature
       \\type real
       \\default 29.44
       \\units C
  N11, \\field Heating Mode Entering Chilled Water Temperature Low Limit
       \\note During simultaneous cooling-heating mode
       \\type real
       \\default 12.22
       \\units C
  A2 , \\field Chilled Water Flow Mode Type
       \\note Sets chilled water flow rate to either constant or variable.
       \\type choice
       \\key ConstantFlow
       \\key VariableFlow
       \\default ConstantFlow
  N12, \\field Design Chilled Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autosizable
       \\ip-units gal/min
  N13, \\field Design Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
  N14, \\field Design Hot Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal/min
  N15, \\field Compressor Motor Efficiency
       \\type real
       \\note Fraction of compressor electrical energy that must be rejected by the condenser.
       \\note Enter 1.0 or leave this field blank for a hermetic compressor.
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A3 , \\field Condenser Type
       \\type choice
       \\key AirCooled
       \\key WaterCooled
       \\default WaterCooled
  A4 , \\field Cooling Mode Temperature Curve Condenser Water Independent Variable
       \\type choice
       \\key EnteringCondenser
       \\key LeavingCondenser
       \\default EnteringCondenser
       \\note Sets the second independent variable in the three temperature dependent performance
       \\note curves to either the leaving or entering condenser water temperature. Manufacturers
       \\note express the performance of their chillers using either the leaving condenser water
       \\note temperature (to the tower) or the entering condenser water temperature (from the tower).
       \\note Cooling mode is generally a stronger function of Entering Condenser Fluid Temperature
  A5 , \\field Cooling Mode Cooling Capacity Function of Temperature Curve Name
       \\note Cooling capacity as a function of leaving chilled water temperature
       \\note and either entering or leaving condenser fluid temperature
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*TCond + e*TCond**2 + f*CWS*TCond
       \\note    If ClgModeCondWaterCurveInputVariable = EnteringCondenser, TCond = ECT
       \\note    If ClgModeCondWaterCurveInputVariable = LeavingCondenser, TCond = LCT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A6 , \\field Cooling Mode Electric Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note Electric Input Ratio (EIR) as a function of supply (leaving) chilled water temperature
       \\note and leaving condenser fluid temperature.   EIR = 1/COP.
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*TCond + e*TCond**2 + f*CWS*TCond
       \\note If ClgModeCondWaterCurveInputVariable = EnteringCondenser, TCond = ECT
       \\note If ClgModeCondWaterCurveInputVariable = LeavingCondenser, TCond = LCT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A7 , \\field Cooling Mode Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note Electric Input Ratio (EIR) as a function of Part Load Ratio (PLR)
       \\note EIR = 1/COP
       \\required-field
       \\type object-list
       \\object-list BicubicCurves
       \\object-list QuadraticCurves
       \\note  Bicubic = a + b*TCond + c*TCond**2 + d*PLR + e*PLR**2 + f*TCond*PLR +g*0 + h*PLR**3+i*0+j*0
       \\note    If ClgModeCondWaterCurveInputVariable = EnteringCondenser, TCond = ECT
       \\note    If ClgModeCondWaterCurveInputVariable = LeavingCondenser, TCond = LCT
       \\note    Normally, a bicubic curve here should be in terms of LCT rather than ECT
       \\note    Also, a bicubic curve is more applicable for variable-speed compressor motor drives
       \\note or
       \\note  Quadratic = a + b*PLR + c*PLR**2
       \\note PLR = part load ratio (cooling load/steady-state capacity)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  N16, \\field Cooling Mode Cooling Capacity Optimum Part Load Ratio
       \\type real
       \\note Optimum part load ratio where the chiller is most efficient.
       \\note Must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\note The Min/Max PLR are taken from their assoicated EIR-FPLR curve references.
       \\minimum> 0.0
       \\default 1.0
  A8 , \\field Heating Mode Temperature Curve Condenser Water Independent Variable
       \\type choice
       \\key EnteringCondenser
       \\key LeavingCondenser
       \\default LeavingCondenser
       \\note Sets the second independent variable in the three temperature dependent performance
       \\note curves to either the leaving or entering condenser water temperature. Manufacturers
       \\note express the performance of their chillers using either the leaving condenser water
       \\note temperature (to the tower) or the entering condenser water temperature (from the tower).
       \\note Heating mode (or Simul Clg/Htg Load) should be a function of Leaving Condenser Fluid Temperature
       \\note Only use EnteringCondenser as a last resort in case no performance data exists for LeavingCondenser
  A9 , \\field Heating Mode Cooling Capacity Function of Temperature Curve Name
       \\note Evaporator (cooling) capacity as a function of leaving chilled water temperature
       \\note and leaving condenser fluid temperature when in heating or simultaneous cool/heat mode
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*TCond + e*TCond**2 + f*CWS*TCond
       \\note    If independent variable = EnteringCondenser, TCond = ECT
       \\note    If independent variable = LeavingCondenser, TCond = LCT
       \\note CWS = supply (leaving) chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A10, \\field Heating Mode Electric Input to Cooling Output Ratio Function of Temperature Curve Name
       \\note Electric Input Ratio (EIR) as a function of leaving chilled water temperature when in heating or simultaneous cool/heat mode
       \\note and leaving condenser fluid temperature.   EIR = 1/COP.
       \\required-field
       \\type object-list
       \\object-list BiquadraticCurves
       \\note curve = a + b*CWS + c*CWS**2 + d*TCond + e*TCond**2 + f*CWS*TCond
       \\note    If independent variable = EnteringCondenser, TCond = ECT
       \\note    If independent variable = LeavingCondenser, TCond = LCT
       \\note CWS = leaving chilled water temperature(C)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  A11, \\field Heating Mode Electric Input to Cooling Output Ratio Function of Part Load Ratio Curve Name
       \\note Electric Input Ratio (EIR) as a function of Part Load Ratio (PLR) when in heating or simultaneous cool/heat mode
       \\note EIR = 1/COP
       \\required-field
       \\type object-list
       \\object-list BicubicCurves
       \\object-list QuadraticCurves
       \\note  Bicubic = a + b*LCT + c*LCT**2 + d*PLR + e*PLR**2 + f*LCT*PLR + g*0 + h*PLR**3 + i*0 + j*0
       \\note    Normally, a bicubic curve here should be in terms of LCT rather than ECT
       \\note    Also, a bicubic curve is more applicable for variable-speed compressor motor drives
       \\note or
       \\note  Quadratic = a + b*PLR + c*PLR**2
       \\note PLR = part load ratio (cooling load/steady-state capacity)
       \\note LCT = leaving condenser fluid temperature(C)
       \\note ECT = entering condenser fluid temperature(C)
  N17, \\field Heating Mode Cooling Capacity Optimum Part Load Ratio
       \\type real
       \\note Optimum part load ratio where the chiller is most efficient when in heating or simultaneous cool/heat mode.
       \\note Must be greater than or equal to the Minimum Part Load Ratio
       \\note and less than or equal to the Maximum Part Load Ratio.
       \\note The Min/Max PLR are taken from their assoicated EIR-FPLR curve references.
       \\minimum> 0.0
       \\default 1.0
  N18; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

\\group Condenser Equipment and Heat Exchangers

!*****************CONDENSING EQUIPMENT*********************
CoolingTower:SingleSpeed,
       \\min-fields 9
       \\memo This tower model is based on Merkel's theory, which is also the basis
       \\memo for the tower model in ASHRAE's HVAC1 Toolkit. The closed-circuit cooling tower
       \\memo is modeled as a counter flow heat exchanger with a single-speed fan drawing air
       \\memo through the tower (induced-draft configuration).
       \\memo Added fluid bypass as an additional capacity control. 8/2008.
       \\memo For a multi-cell tower, the capacity and air/water flow rate inputs are for the entire tower.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingTowers
       \\note Tower Name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of tower water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of tower water outlet node
  N1 , \\field Design Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Leave field blank if tower performance input method is NominalCapacity
       \\ip-units gal/min
  N2 , \\field Design Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N3 , \\field Fan Power at Design Air Flow Rate
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power
  N4 , \\field U-Factor Times Area Value at Design Air Flow Rate
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Leave field blank if tower performance input method is NominalCapacity
  N5 , \\field Air Flow Rate in Free Convection Regime
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\autosizable
  N6 , \\field U-Factor Times Area Value at Free Convection Air Flow Rate
       \\type real
       \\units W/K
       \\minimum 0.0
       \\maximum 300000.0
       \\default 0.0
       \\autosizable
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key NominalCapacity
       \\default NominalCapacity
       \\note User can define tower thermal performance by specifying the tower UA,
       \\note the Design Air Flow Rate and the Design Water Flow Rate,
       \\note or by specifying the tower nominal capacity
  N7, \\field Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal tower capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature. Design water flow rate assumed to be 5.382E-8 m3/s per watt
       \\note (3 gpm/ton). Nominal tower capacity times (1.25) gives the actual tower
       \\note heat rejection at these operating conditions.
  N8 , \\field Free Convection Capacity
       \\type real
       \\units W
       \\minimum 0.0
       \\note Tower capacity in free convection regime with entering water at 35C (95F),
       \\note leaving water at 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature
       \\note and 35C (95F) dry-bulb temperature. Design water flow rate assumed to be
       \\note 5.382E-8 m3/s per watt of nominal tower capacity (3 gpm/ton). Tower free
       \\note convection capacity times (1.25) gives the actual tower heat rejection at these
       \\note operating conditions.
  N9 , \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This heater maintains the basin water temperature at the basin heater setpoint
       \\note temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when water is not flowing through the tower.
  N10, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on
  A5 , \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A6 , \\field Evaporation Loss Mode
       \\type choice
       \\key LossFactor
       \\key SaturatedExit
  N11 , \\field Evaporation Loss Factor
       \\type real
       \\units percent/K
       \\note Rate of water evaporation from the cooling tower and lost to the outdoor air [%/K]
       \\note Evaporation loss is calculated as percentage of the circulating condenser water rate
       \\note Value entered here is percent-per-degree K of temperature drop in the condenser water
       \\note Typical values are from 0.15 to 0.27 [%/K].
       \\default 0.2
  N12, \\field Drift Loss Percent
       \\type real
       \\default 0.008
       \\units percent
       \\note Rate of drift loss as a percentage of circulating condenser water flow rate
       \\note Typical values are between 0.002 and 0.2% The default value is 0.008%
  A7 , \\field Blowdown Calculation Mode
       \\type choice
       \\key ConcentrationRatio
       \\key ScheduledRate
  N13, \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\default 3.0
       \\note Characterizes the rate of blowdown in the cooling tower.
       \\note Blowdown is water intentionally drained from the tower in order to offset the build up
       \\note of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note Typical values for tower operation are 3 to 5.  The default value is 3.
  A8 , \\field Blowdown Makeup Water Usage Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Makeup water usage due to blowdown results from occasionally draining a small amount
       \\note of water in the tower basin to purge scale or other contaminants to reduce their
       \\note concentration in order to maintain an acceptable level of water quality.
       \\note Schedule values should reflect water usage in m3/s.
  A9 , \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A10 , \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node
  A11, \\field Capacity Control
       \\type choice
       \\key FanCycling
       \\key FluidBypass
       \\default FanCycling
  N14, \\field Number of Cells
       \\type integer
       \\minimum 1
       \\default 1
  A12, \\field Cell Control
       \\type Choice
       \\key MinimalCell
       \\key MaximalCell
       \\default MinimalCell
  N15, \\field Cell Minimum  Water Flow Rate Fraction
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\default 0.33
       \\note The allowable mininal fraction of the nominal flow rate per cell
  N16, \\field Cell Maximum Water Flow Rate Fraction
       \\type real
       \\minimum 1
       \\default 2.5
       \\note The allowable maximal fraction of the nominal flow rate per cell
  N17; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

CoolingTower:TwoSpeed,
       \\min-fields 12
       \\memo This tower model is based on Merkel's theory, which is also the basis
       \\memo for the tower model in ASHRAE's HVAC1 Toolkit. The closed-circuit cooling tower
       \\memo is modeled as a counter flow heat exchanger with a two-speed fan drawing air
       \\memo through the tower (induced-draft configuration).
       \\memo For a multi-cell tower, the capacity and air/water flow rate inputs are for the entire tower.
  A1,  \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingTowers
       \\note Tower Name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of tower Water Inlet Node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of tower Water Outlet Node
  N1 , \\field Design Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Leave field blank if Tower Performance Input Method is NominalCapacity
       \\ip-units gal/min
  N2 , \\field Air Flow Rate at High Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N3 , \\field Fan Power at High Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at high speed
  N4 , \\field U-Factor Times Area Value at High Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Leave field blank if Tower Performance Input Method is NominalCapacity
  N5 , \\field Air Flow Rate at Low Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Low speed air flow rate must be less than high speed air flow rate
       \\note Low speed air flow rate must be greater than free convection air flow rate
  N6 , \\field Fan Power at Low Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at low speed
  N7 , \\field U-Factor Times Area Value at Low Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 300000.0
       \\autosizable
       \\note Leave field blank if tower Performance Input Method is NominalCapacity
       \\note Low speed tower UA must be less than high speed tower UA
       \\note Low speed tower UA must be greater than free convection tower UA
  N8 , \\field Air Flow Rate in Free Convection Regime
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\autosizable
  N9 , \\field U-Factor Times Area Value at Free Convection Air Flow Rate
       \\type real
       \\units W/K
       \\minimum 0.0
       \\maximum 300000.0
       \\default 0.0
       \\autosizable
       \\note Leave field blank if Tower Performance Input Method is NominalCapacity
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key NominalCapacity
       \\default NominalCapacity
       \\note User can define tower thermal performance by specifying the tower UA,
       \\note the Design Air Flow Rate and the Design Water Flow Rate,
       \\note or by specifying the tower nominal capacity
  N10, \\field High Speed Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal tower capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature, with the tower fan operating at high speed. Design water
       \\note flow rate assumed to be 5.382E-8 m3/s per watt(3 gpm/ton). Nominal tower capacity
       \\note times (1.25) gives the actual tower heat rejection at these operating conditions.
  N11, \\field Low Speed Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal tower capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature, with the tower fan operating at low speed. Design water flow
       \\note rate assumed to be 5.382E-8 m3/s per watt of tower high-speed nominal capacity
       \\note (3 gpm/ton). Nominal tower capacity times (1.25) gives the actual tower heat
       \\note rejection at these operating conditions.
  N12, \\field Free Convection Capacity
       \\type real
       \\units W
       \\minimum 0.0
       \\note Tower capacity in free convection regime with entering water at 35C (95F),
       \\note leaving water at 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature
       \\note and 35C (95F) dry-bulb temperature. Design water flow rate assumed to be
       \\note 5.382E-8 m3/s per watt of tower high-speed nominal capacity (3 gpm/ton). Tower
       \\note free convection capacity times (1.25) gives the actual tower heat rejection at
       \\note these operating conditions.
  N13 , \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This heater maintains the basin water temperature at the basin heater setpoint
       \\note temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when water is not flowing through the tower.
  N14, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on
  A5 , \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A6 , \\field Evaporation Loss Mode
       \\type choice
       \\key LossFactor
       \\key SaturatedExit
  N15, \\field Evaporation Loss Factor
       \\type real
       \\units percent/K
       \\note Rate of water evaporated from the cooling tower and lost to the outdoor air [%/K]
       \\note Evaporation loss is calculated as percentage of the circulating condenser water rate
       \\note Value entered here is percent-per-degree K of temperature drop in the condenser water
       \\note Typical values are from 0.15 to 0.27 [%/K].
       \\default 0.2
  N16, \\field Drift Loss Percent
       \\type real
       \\units percent
       \\default 0.008
       \\note Rate of drift loss as a percentage of circulating condenser water flow rate
       \\note Typical values are between 0.002 and 0.2% The default value is 0.008%
  A7 , \\field Blowdown Calculation Mode
       \\type choice
       \\key ConcentrationRatio
       \\key ScheduledRate
  N17, \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\default 3.0
       \\note Characterizes the rate of blowdown in the cooling tower.
       \\note Blowdown is water intentionally drained from the tower in order to offset the build up
       \\note of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note Typical values for tower operation are 3 to 5.  The default value is 3.
  A8 , \\field Blowdown Makeup Water Usage Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Makeup water usage due to blowdown results from occasionally draining some amount
       \\note of water in the tower basin to purge scale or other contaminants to reduce their
       \\note concentration in order to maintain an acceptable level of water quality.
       \\note Schedule values should reflect water usage in m3/s.
  A9 , \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A10 , \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node
  N18, \\field Number of Cells
       \\type integer
       \\minimum 1
       \\default 1
  A11, \\field Cell Control
       \\type Choice
       \\key MinimalCell
       \\key MaximalCell
       \\default MinimalCell
  N19, \\field Cell Minimum  Water Flow Rate Fraction
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\default 0.33
       \\note The allowable mininal fraction of the nominal flow rate per cell
  N20, \\field Cell Maximum Water Flow Rate Fraction
       \\type real
       \\minimum 1
       \\default 2.5
       \\note The allowable maximal fraction of the nominal flow rate per cell
  N21; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

CoolingTower:VariableSpeed,
       \\min-fields 15
       \\memo This tower model is based on purely empirical algorithms derived from manufacturer's
       \\memo performance data or field measurements. The user can select from two existing
       \\memo algorithms (CoolTools or YorkCalc), or they can enter their own correlation for
       \\memo approach temperature by using a variable speed tower model coefficient object.
       \\memo For a multi-cell tower, the capacity and air/water flow rate inputs are for the entire tower.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CoolingTowers
       \\note Tower Name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of tower water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of tower water outlet node
  A4 , \\field Model Type
       \\type choice
       \\key CoolToolsCrossFlow
       \\key CoolToolsUserDefined
       \\key YorkCalc
       \\key YorkCalcUserDefined
       \\default YorkCalc
       \\note Determines the coefficients and form of the equation for calculating
       \\note approach temperature
  A5 , \\field Model Coefficient Name
       \\type object-list
       \\object-list VariableSpeedTowerCoefficient
       \\note Name of the tower model coefficient object.
       \\note Used only when tower Model Type is either CoolToolsUserDefined or YorkCalcUserDefined.
  N1 , \\field Design Inlet Air Wet-Bulb Temperature
       \\type real
       \\units C
       \\default 25.6
       \\minimum 20.0
       \\note Enter the tower's design inlet air wet-bulb temperature
  N2 , \\field Design Approach Temperature
       \\type real
       \\units deltaC
       \\default 3.9
       \\minimum> 0
       \\note Enter the approach temperature corresponding to the design inlet air
       \\note wet-bulb temperature and design range temperature.
       \\note Design approach temp = outlet water temperature minus inlet air wet-bulb temperature
       \\note at design conditions.
  N3 , \\field Design Range Temperature
       \\type real
       \\units deltaC
       \\default 5.6
       \\minimum> 0
       \\note Enter the range temperature corresponding to the design inlet air
       \\note wet-bulb temperature and design approach temperature.
       \\note Design range = inlet water temperature minus outlet water temperature
       \\note at design conditions.
  N4 , \\field Design Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
       \\note Water flow rate through the tower at design conditions
  N5 , \\field Design Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Design (maximum) air flow rate through the tower
  N6 , \\field Design Fan Power
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note Enter the fan motor electric input power at design (max) air flow through the tower
       \\note Standard conversion for horsepower is 1 HP = 745.7 W
  A6 , \\field Fan Power Ratio Function of Air Flow Rate Ratio Curve Name
       \\type object-list
       \\object-list CubicCurves
       \\note FPR = a + b*AFR + c*AFR**2 + d*AFR**3
       \\note FPR = fraction of the design fan power
       \\note AFR = fraction of the design air flow rate
       \\note If left blank, then fan power is assumed to be proportional to
       \\note (air flow rate ratio)^3
  N7 , \\field Minimum Air Flow Rate Ratio
       \\type real
       \\minimum 0.2
       \\maximum 0.5
       \\default 0.2
       \\note Enter the minimum air flow rate ratio. This is typically determined by the variable
       \\note speed drive that controls the fan motor speed. Valid entries are from 0.2 to 0.5.
  N8 , \\field Fraction of Tower Capacity in Free Convection Regime
       \\type real
       \\minimum 0.0
       \\maximum 0.2
       \\default 0.125
       \\note Enter the fraction of tower capacity in the free convection regime. This is the
       \\note fraction of the tower capacity, at the current inlet air wet-bulb temperature,
       \\note that is available when the tower fan is off. Manufacturers typically estimate the
       \\note free convection capacity at approximately 10-15%. Values are entered as a fraction
       \\note and can range from 0 to 0.2.
  N9 , \\field Basin Heater Capacity
       \\type real
       \\units W/K
       \\minimum 0.0
       \\default 0.0
       \\note This heater maintains the basin water temperature at the basin heater setpoint
       \\note temperature when the outdoor air temperature falls below the setpoint temperature.
       \\note The basin heater only operates when water is not flowing through the tower.
  N10, \\field Basin Heater Setpoint Temperature
       \\type real
       \\units C
       \\minimum 2.0
       \\default 2.0
       \\note Enter the outdoor dry-bulb temperature when the basin heater turns on
  A7 , \\field Basin Heater Operating Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values greater than 0 allow the basin heater to operate whenever the outdoor
       \\note air dry-bulb temperature is below the basin heater setpoint temperature.
       \\note If a schedule name is not entered, the basin heater is allowed to operate
       \\note throughout the entire simulation.
  A8 , \\field Evaporation Loss Mode
       \\type choice
       \\key LossFactor
       \\key SaturatedExit
  N11, \\field Evaporation Loss Factor
       \\type real
       \\units percent/K
       \\note Rate of water evaporated from the cooling tower and lost to the outdoor air [%/K]
       \\note Evaporation loss is calculated as percentage of the circulating condenser water rate
       \\note Value entered here is percent-per-degree K of temperature drop in the condenser water
       \\note Typical values are from 0.15 to 0.27 [percent/K].
       \\default 0.2
  N12, \\field Drift Loss Percent
       \\type real
       \\units percent
       \\note Rate of drift loss as a percentage of circulating condenser water flow rate
       \\note Typical values are between 0.002 and 0.2% The default value is 0.008%
  A9 , \\field Blowdown Calculation Mode
       \\type choice
       \\key ConcentrationRatio
       \\key ScheduledRate
  N13, \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\default 3.0
       \\note Characterizes the rate of blowdown in the cooling tower.
       \\note Blowdown is water intentionally drained from the tower in order to offset the build up
       \\note of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note Typical values for tower operation are 3 to 5.  The default value is 3.
  A10, \\field Blowdown Makeup Water Usage Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Makeup water usage due to blowdown results from occasionally draining a small amount
       \\note of water in the tower basin to purge scale or other contaminants to reduce their
       \\note concentration in order to maintain an acceptable level of water quality.
       \\note Schedule values should reflect water usage in m3/s.
  A11, \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A12, \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node
  N14, \\field Number of Cells
       \\type integer
       \\minimum 1
       \\default 1
  A13, \\field Cell Control
       \\type Choice
       \\key MinimalCell
       \\key MaximalCell
       \\default MinimalCell
  N15, \\field Cell Minimum  Water Flow Rate Fraction
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
       \\default 0.33
       \\note The allowable mininal fraction of the nominal flow rate per cell
  N16, \\field Cell Maximum Water Flow Rate Fraction
       \\type real
       \\minimum 1
       \\default 2.5
       \\note The allowable maximal fraction of the nominal flow rate per cell
  N17; \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0

CoolingTowerPerformance:CoolTools,
       \\min-fields 44
       \\memo This object is used to define coefficients for the approach temperature
       \\memo correlation for a variable speed cooling tower when tower Model Type is
       \\memo specified as CoolToolsUserDefined in the object CoolingTower:VariableSpeed.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference VariableSpeedTowerCoefficient
  N1 , \\field Minimum Inlet Air Wet-Bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\note Minimum valid inlet air wet-bulb temperature for this approach
       \\note temperature correlation.
  N2 , \\field Maximum Inlet Air Wet-Bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\note Maximum valid inlet air wet-bulb temperature for this approach
       \\note temperature correlation.
  N3 , \\field Minimum Range Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Minimum valid range temperature for this approach temperature
       \\note correlation.
  N4 , \\field Maximum Range Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Maximum valid range temperature for this approach temperature
       \\note correlation.
  N5 , \\field Minimum Approach Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Minimum valid approach temperature for this correlation.
  N6 , \\field Maximum Approach Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Maximum valid approach temperature for this correlation.
  N7 , \\field Minimum Water Flow Rate Ratio
       \\required-field
       \\type real
       \\note Minimum valid water flow rate ratio for this approach
       \\note temperature correlation.
  N8 , \\field Maximum Water Flow Rate Ratio
       \\required-field
       \\type real
       \\note Maximum valid water flow rate ratio for this approach
       \\note temperature correlation.
  N9 , \\field Coefficient 1
       \\required-field
       \\type real
  N10, \\field Coefficient 2
       \\required-field
       \\type real
  N11, \\field Coefficient 3
       \\required-field
       \\type real
  N12, \\field Coefficient 4
       \\required-field
       \\type real
  N13, \\field Coefficient 5
       \\required-field
       \\type real
  N14, \\field Coefficient 6
       \\required-field
       \\type real
  N15, \\field Coefficient 7
       \\required-field
       \\type real
  N16, \\field Coefficient 8
       \\required-field
       \\type real
  N17, \\field Coefficient 9
       \\required-field
       \\type real
  N18, \\field Coefficient 10
       \\required-field
       \\type real
  N19, \\field Coefficient 11
       \\required-field
       \\type real
  N20, \\field Coefficient 12
       \\required-field
       \\type real
  N21, \\field Coefficient 13
       \\required-field
       \\type real
  N22, \\field Coefficient 14
       \\required-field
       \\type real
  N23, \\field Coefficient 15
       \\required-field
       \\type real
  N24, \\field Coefficient 16
       \\required-field
       \\type real
  N25, \\field Coefficient 17
       \\required-field
       \\type real
  N26, \\field Coefficient 18
       \\required-field
       \\type real
  N27, \\field Coefficient 19
       \\required-field
       \\type real
  N28, \\field Coefficient 20
       \\required-field
       \\type real
  N29, \\field Coefficient 21
       \\required-field
       \\type real
  N30, \\field Coefficient 22
       \\required-field
       \\type real
  N31, \\field Coefficient 23
       \\required-field
       \\type real
  N32, \\field Coefficient 24
       \\required-field
       \\type real
  N33, \\field Coefficient 25
       \\required-field
       \\type real
  N34, \\field Coefficient 26
       \\required-field
       \\type real
  N35, \\field Coefficient 27
       \\required-field
       \\type real
  N36, \\field Coefficient 28
       \\required-field
       \\type real
  N37, \\field Coefficient 29
       \\required-field
       \\type real
  N38, \\field Coefficient 30
       \\required-field
       \\type real
  N39, \\field Coefficient 31
       \\required-field
       \\type real
  N40, \\field Coefficient 32
       \\required-field
       \\type real
  N41, \\field Coefficient 33
       \\required-field
       \\type real
  N42, \\field Coefficient 34
       \\required-field
       \\type real
  N43; \\field Coefficient 35
       \\required-field
       \\type real

CoolingTowerPerformance:YorkCalc,
       \\min-fields 37
       \\memo This object is used to define coefficients for the approach temperature
       \\memo correlation for a variable speed cooling tower when tower Model Type is
       \\memo specified as YorkCalcUserDefined in the object CoolingTower:VariableSpeed.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference VariableSpeedTowerCoefficient
  N1 , \\field Minimum Inlet Air Wet-Bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\note Minimum valid inlet air wet-bulb temperature for this approach
       \\note temperature correlation.
  N2 , \\field Maximum Inlet Air Wet-Bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\note Maximum valid inlet air wet-bulb temperature for this approach
       \\note temperature correlation.
  N3 , \\field Minimum Range Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Minimum valid range temperature for this approach temperature
       \\note correlation.
  N4 , \\field Maximum Range Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Maximum valid range temperature for this approach temperature
       \\note correlation.
  N5 , \\field Minimum Approach Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Minimum valid approach temperature for this correlation.
  N6 , \\field Maximum Approach Temperature
       \\required-field
       \\type real
       \\units deltaC
       \\note Maximum valid approach temperature for this correlation.
  N7 , \\field Minimum Water Flow Rate Ratio
       \\required-field
       \\type real
       \\note Minimum valid water flow rate ratio for this approach
       \\note temperature correlation.
  N8 , \\field Maximum Water Flow Rate Ratio
       \\required-field
       \\type real
       \\note Maximum valid water flow rate ratio for this approach
       \\note temperature correlation.
  N9 , \\field Maximum Liquid to Gas Ratio
       \\required-field
       \\type real
       \\note Maximum liquid (water) to gas (air) ratio for this approach
       \\note temperature correlation.
  N10, \\field Coefficient 1
       \\required-field
       \\type real
  N11, \\field Coefficient 2
       \\required-field
       \\type real
  N12, \\field Coefficient 3
       \\required-field
       \\type real
  N13, \\field Coefficient 4
       \\required-field
       \\type real
  N14, \\field Coefficient 5
       \\required-field
       \\type real
  N15, \\field Coefficient 6
       \\required-field
       \\type real
  N16, \\field Coefficient 7
       \\required-field
       \\type real
  N17, \\field Coefficient 8
       \\required-field
       \\type real
  N18, \\field Coefficient 9
       \\required-field
       \\type real
  N19, \\field Coefficient 10
       \\required-field
       \\type real
  N20, \\field Coefficient 11
       \\required-field
       \\type real
  N21, \\field Coefficient 12
       \\required-field
       \\type real
  N22, \\field Coefficient 13
       \\required-field
       \\type real
  N23, \\field Coefficient 14
       \\required-field
       \\type real
  N24, \\field Coefficient 15
       \\required-field
       \\type real
  N25, \\field Coefficient 16
       \\required-field
       \\type real
  N26, \\field Coefficient 17
       \\required-field
       \\type real
  N27, \\field Coefficient 18
       \\required-field
       \\type real
  N28, \\field Coefficient 19
       \\required-field
       \\type real
  N29, \\field Coefficient 20
       \\required-field
       \\type real
  N30, \\field Coefficient 21
       \\required-field
       \\type real
  N31, \\field Coefficient 22
       \\required-field
       \\type real
  N32, \\field Coefficient 23
       \\required-field
       \\type real
  N33, \\field Coefficient 24
       \\required-field
       \\type real
  N34, \\field Coefficient 25
       \\required-field
       \\type real
  N35, \\field Coefficient 26
       \\required-field
       \\type real
  N36; \\field Coefficient 27
       \\required-field
       \\type real

EvaporativeFluidCooler:SingleSpeed,
       \\min-fields 9
       \\memo This model is based on Merkel's theory, which is also the basis
       \\memo for the cooling tower model in EnergyPlus. The Evaporative fluid cooler
       \\memo is modeled as a counter flow heat exchanger.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note Fluid Cooler Name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of Fluid Cooler water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of Fluid Cooler water outlet node
  N1 , \\field Design Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Fan Power at Design Air Flow Rate
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power
  N3 , \\field Design Spray Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\ip-units gal/min
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key StandardDesignCapacity
       \\key UserSpecifiedDesignCapacity
       \\note User can define fluid cooler thermal performance by specifying the fluid cooler UA
       \\note and the Design Water Flow Rate, or by specifying the fluid cooler Standard Design
       \\note Capacity or by specifying Design Capacity for Non standard conditions.
  A5 , \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node
  N4 , \\field Standard Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Standard design capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature. Design water flow rate assumed to be 5.382E-8 m3/s per watt
       \\note (3 gpm/ton). Standard design capacity times (1.25) gives the actual fluid cooler
       \\note heat rejection at these operating conditions.
       \\note Only used for Performance Input Method = StandardDesignCapacity;
       \\note for other input methods, this field is ignored.
       \\note The standard conditions mentioned above for Standard design capacity are already
       \\note specified in the EnergyPlus. So the input fields such as design entering water
       \\note temp., design entering air wet-bulb and dry-bulb temp. and design water flow rate, if
       \\note provided in the input, will be ignored for the StandardDesignCapacity performance input
       \\note method. Also, the standard conditions are for water as a fluid type so this performance input
       \\note method can only be used with water as a fluid type (as specified in CondenserLoop object).
  N5 , \\field U-factor Times Area Value at Design Air Flow Rate
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Only used for Performance Input Method = UFactorTimesAreaAndDesignWaterFlowRate;
       \\note for other Performance Input Methods, this field is ignored.
  N6 , \\field Design Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
       \\note Input value is ignored if fluid cooler Performance Input Method= StandardDesignCapacity.
  N7 , \\field User Specified Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
  N8 , \\field Design Entering Water Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Water Temperature must be greater than Design Entering Air Temperature.
  N9 , \\field Design Entering Air Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Air Temperature must be greater than Design Entering Air Wet-bulb
       \\note Temperature.
  N10, \\field Design Entering Air Wet-bulb Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Air Wet-bulb Temperature must be less than Design Entering Air
       \\note Temperature.
  A6 , \\field Capacity Control
       \\type choice
       \\key FanCycling
       \\key FluidBypass
       \\default FanCycling
  N11, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  A7 , \\field Evaporation Loss Mode
       \\type choice
       \\key LossFactor
       \\key SaturatedExit
       \\default SaturatedExit
  N12, \\field Evaporation Loss Factor
       \\type real
       \\units percent/K
       \\note Rate of water evaporation from the Fluid Cooler and lost to the outdoor air [%/K]
       \\note Empirical correlation is used to calculate default loss factor if it not explicitly provided.
  N13, \\field Drift Loss Percent
       \\type real
       \\default 0.008
       \\units percent
       \\note Rate of drift loss as a percentage of circulating spray water flow rate
       \\note Default value for this field in under investigation. For now Cooling tower's drift loss
       \\note percent default value is taken here.
  A8 , \\field Blowdown Calculation Mode
       \\type choice
       \\key ConcentrationRatio
       \\key ScheduledRate
       \\default ConcentrationRatio
  N14, \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\default 3.0
       \\note Characterizes the rate of blowdown in the Evaporative Fluid Cooler.
       \\note Blowdown is water intentionally drained from the basin in order to offset the build
       \\note up of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note Default value for this field in under investigation. For now Cooling tower's
       \\note Blowdown Concentration Ratio percent default value is taken here.
  A9 , \\field Blowdown Makeup Water Usage Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Makeup water usage due to blowdown results from occasionally draining a small
       \\note amount of water in the Fluid Cooler basin to purge scale or other contaminants to
       \\note reduce their concentration in order to maintain an acceptable level of water quality.
       \\note Schedule values should reflect water usage in m3/s.
  A10; \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames

EvaporativeFluidCooler:TwoSpeed,
       \\min-fields 12
       \\memo This model is based on Merkel's theory, which is also the basis
       \\memo for the cooling tower model in EnergyPlus. The Evaporative fluid cooler
       \\memo is modeled as a counter flow heat exchanger.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note fluid cooler name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water outlet node
  N1 , \\field Air Flow Rate at High Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N2 , \\field Fan Power at High Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at high speed
  N3 , \\field Air Flow Rate at Low Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Low speed air flow rate must be less than high speed air flow rate
  N4 , \\field Fan Power at Low Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at low speed
  N5 , \\field Design Spray Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\ip-units gal/min
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key StandardDesignCapacity
       \\key UserSpecifiedDesignCapacity
       \\note User can define fluid cooler thermal performance by specifying the fluid cooler UA
       \\note and the Design Water Flow Rate, or by specifying the fluid cooler Standard Design
       \\note Capacity or by specifying Design Capacity for Non standard conditions.
  A5 , \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node
  N6 , \\field High-Speed Standard Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Standard design capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature. Design water flow rate assumed to be 5.382E-8 m3/s per watt
       \\note (3 gpm/ton). Standard design capacity times (1.25) gives the actual fluid cooler
       \\note heat rejection at these operating conditions.
       \\note Only used for Performance Input Method = StandardDesignCapacity;
       \\note for other input methods, this field is ignored.
       \\note The standard conditions mentioned above for Standard design capacity are already
       \\note specified in the EnergyPlus. So the input fields such as design entering water
       \\note temp., design entering air wet-bulb and dry-bulb temp. and design water flow rate, if
       \\note provided in the input, will be ignored for the StandardDesignCapacity performance input
       \\note method. Also, the standard conditions are for water as a fluid type so this performance input
       \\note method can only be used with water as a fluid type (as specified in CondenserLoop object).
  N7 , \\field Low-Speed Standard Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Standard design capacity with entering water at 35C (95F), leaving water at
       \\note 29.44C (85F), entering air at 25.56C (78F) wet-bulb temperature and 35C (95F)
       \\note dry-bulb temperature. Design water flow rate assumed to be 5.382E-8 m3/s per watt
       \\note (3 gpm/ton). Standard design capacity times (1.25) gives the actual fluid cooler
       \\note heat rejection at these operating conditions.
       \\note Only used for Performance Input Method = StandardDesignCapacity;
       \\note for other input methods, this field is ignored.
       \\note The standard conditions mentioned above for Standard design capacity are already
       \\note specified in the EnergyPlus. So the input fields such as design entering water
       \\note temp., design entering air wet-bulb and dry-bulb temp. and design water flow rate, if
       \\note provided in the input, will be ignored for the StandardDesignCapacity performance input
       \\note method. Also, the standard conditions are for water as a fluid type so this performance input
       \\note method can only be used with water as a fluid type (as specified in CondenserLoop object).
  N8 , \\field U-factor Times Area Value at High Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Only used for Performance Input Method = UFactorTimesAreaAndDesignWaterFlowRate;
       \\note for other Performance Input Methods, this field is ignored.
  N9 , \\field U-factor Times Area Value at Low Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 300000.0
       \\autosizable
       \\note Only used for Performance Input Method = UFactorTimesAreaAndDesignWaterFlowRate;
       \\note for other input methods, this field is ignored.
       \\note Low speed fluid cooler UA must be less than high speed fluid cooler UA
  N10, \\field Design Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
       \\note Input value is ignored if fluid cooler Performance Input Method= StandardDesignCapacity
  N11, \\field High-Speed User Specified Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
  N12, \\field Low-Speed User Specified Design Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
  N13, \\field Design Entering Water Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Water Temperature must be greater than Design Entering Air Temperature.
  N14, \\field Design Entering Air Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Air Temperature must be greater than Design Entering Air Wet-bulb
       \\note Temperature.
  N15, \\field Design Entering Air Wet-bulb Temperature
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Only used for Performance Input Method = UserSpecifiedDesignCapacity;
       \\note for other Performance Input Methods, this field is ignored.
       \\note Design Entering Air Wet-bulb Temperature must be less than Design Entering Air
       \\note Temperature.
  N16, \\field Sizing Factor
       \\note Multiplies the autosized capacity and flow rates
       \\type real
       \\minimum> 0.0
       \\default 1.0
  A6 , \\field Evaporation Loss Mode
       \\type choice
       \\key LossFactor
       \\key SaturatedExit
       \\default SaturatedExit
  N17, \\field Evaporation Loss Factor
       \\type real
       \\units percent/K
       \\note Rate of water evaporation from the Fluid Cooler and lost to the outdoor air [%/K]
       \\note Empirical correlation is used to calculate default loss factor if it not explicitly provided.
  N18, \\field Drift Loss Percent
       \\type real
       \\units percent
       \\default 0.008
       \\note Default value is under investigation. For now cooling tower's default value is taken.
  A7 , \\field Blowdown Calculation Mode
       \\type choice
       \\key ConcentrationRatio
       \\key ScheduledRate
       \\default ConcentrationRatio
  N19, \\field Blowdown Concentration Ratio
       \\type real
       \\minimum 2.0
       \\default 3.0
       \\note Characterizes the rate of blowdown in the Evaporative Fluid Cooler.
       \\note Blowdown is water intentionally drained from the Evaporative Fluid Cooler in order to offset the
       \\note build up of solids in the water that would otherwise occur because of evaporation.
       \\note Ratio of solids in the blowdown water to solids in the make up water.
       \\note Default value is under investigation. For now cooling tower's default value is taken.
  A8 , \\field Blowdown Makeup Water Usage Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Makeup water usage due to blowdown results from occasionally draining some amount
       \\note of water in the Evaporative Fluid Cooler basin to purge scale or other contaminants to reduce
       \\note their concentration in order to maintain an acceptable level of water quality.
       \\note Schedule values should reflect water usage in m3/s.
  A9 ; \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames

FluidCooler:SingleSpeed,
       \\min-fields 12
       \\memo The fluid cooler is modeled as a cross flow heat exchanger (both streams unmixed) with
       \\memo single-speed fans (induced draft configuration).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note fluid cooler name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water outlet node
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key NominalCapacity
       \\default NominalCapacity
       \\note User can define fluid cooler thermal performance by specifying the fluid cooler UA
       \\note and the Design Water Flow Rate, or by specifying the fluid cooler nominal capacity
  N1 , \\field U-factor Times Area Value at Design Air Flow Rate
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Leave field blank if fluid cooler Performance Input Method is NominalCapacity
  N2 , \\field Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal fluid cooler capacity
  N3 , \\field Design Entering Water Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Water Temperature must be specified for both the performance input methods and
       \\note its value must be greater than Design Entering Air Temperature.
  N4 , \\field Design Entering Air Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Air Temperature must be specified for both the performance input methods and
       \\note its value must be greater than Design Entering Air Wet-bulb Temperature.
  N5 , \\field Design Entering Air Wet-bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Air Wet-bulb Temperature must be specified for both the performance input methods and
       \\note its value must be less than Design Entering Air Temperature.
  N6 , \\field Design Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
  N7 , \\field Design Air Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
  N8 , \\field Fan Power at Design Air Flow Rate
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power
  A5 ; \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node

FluidCooler:TwoSpeed,
       \\min-fields 16
       \\memo The fluid cooler is modeled as a cross flow heat exchanger (both streams unmixed) with
       \\memo two-speed fans (induced draft configuration).
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note fluid cooler name
  A2 , \\field Water Inlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water inlet node
  A3 , \\field Water Outlet Node Name
       \\required-field
       \\type node
       \\note Name of fluid cooler water outlet node
  A4 , \\field Performance Input Method
       \\type Choice
       \\key UFactorTimesAreaAndDesignWaterFlowRate
       \\key NominalCapacity
       \\default NominalCapacity
       \\note User can define fluid cooler thermal performance by specifying the fluid cooler UA
       \\note and the Design Water Flow Rate, or by specifying the fluid cooler nominal capacity
  N1 , \\field U-factor Times Area Value at High Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 2100000.0
       \\autosizable
       \\note Leave field blank if fluid cooler Performance Input Method is NominalCapacity
  N2 , \\field U-factor Times Area Value at Low Fan Speed
       \\type real
       \\units W/K
       \\minimum> 0.0
       \\maximum 300000.0
       \\autosizable
       \\note Leave field blank if fluid cooler Performance Input Method is NominalCapacity
       \\note Low speed fluid cooler UA must be less than high speed fluid cooler UA
       \\note Low speed fluid cooler UA must be greater than free convection fluid cooler UA
  N3, \\field High-Speed Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal fluid cooler capacity at high fan speed
  N4, \\field Low-Speed Nominal Capacity
       \\type real
       \\units W
       \\minimum> 0.0
       \\note Nominal fluid cooler capacity at low fan speed
  N5 , \\field Design Entering Water Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Water Temperature must be specified for both the performance input methods and
       \\note its value must be greater than Design Entering Air Temperature.
  N6 , \\field Design Entering Air Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Air Temperature must be specified for both the performance input methods and
       \\note its value must be greater than Design Entering Air Wet-bulb Temperature.
  N7 , \\field Design Entering Air Wet-bulb Temperature
       \\required-field
       \\type real
       \\units C
       \\minimum> 0.0
       \\ip-units F
       \\note Design Entering Air Wet-bulb Temperature must be specified for both the performance input methods and
       \\note its value must be less than Design Entering Air Temperature.
  N8 , \\field Design Water Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\ip-units gal/min
  N9 , \\field Air Flow Rate at High Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Air Flow Rate at High Fan Speed must be greater than Air Flow Rate at Low Fan Speed
  N10, \\field Fan Power at High Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at high speed
  N11, \\field Air Flow Rate at Low Fan Speed
       \\required-field
       \\type real
       \\units m3/s
       \\minimum> 0.0
       \\autosizable
       \\note Air Flow Rate at Low Fan Speed must be less than Air Flow Rate at High Fan Speed
  N12, \\field Fan Power at Low Fan Speed
       \\required-field
       \\type real
       \\units W
       \\minimum> 0.0
       \\autosizable
       \\ip-units W
       \\note This is the fan motor electric input power at low speed
  A5 ; \\field Outdoor Air Inlet Node Name
       \\type node

GroundHeatExchanger:Vertical,
 \\min-fields 21
 \\memo Variable short time step vertical ground heat exchanger model based on
 \\memo Yavuztruk, C., J.D.Spitler. 1999. A Short Time Step response Factor Model for
 \\memo Vertical Ground Loop Heat Exchangers
 \\memo The Fluid Type in the associated condenser loop must be same for which the
 \\memo g-functions below are calculated.
  A1,  \\field Name
        \\required-field
  A2,   \\field Inlet Node Name
        \\required-field
        \\type node
  A3,   \\field Outlet Node Name
        \\required-field
        \\type node
  N1,   \\field Maximum Flow Rate
        \\units m3/s
        \\minimum> 0.0
        \\ip-units gal/min
  N2,   \\field Number of Bore Holes
        \\minimum> 0.0
  N3,   \\field Bore Hole Length
        \\units m
        \\minimum> 0.0
  N4,   \\field Bore Hole Radius
        \\units m
        \\minimum> 0.0
  N5,   \\field Ground Thermal Conductivity
        \\units W/m-K
        \\minimum> 0.0
  N6,   \\field Ground Thermal Heat Capacity
        \\units J/m3-K
        \\minimum> 0.0
  N7,   \\field Ground Temperature
        \\units C
        \\minimum> 0.0
  N8,   \\field Design Flow Rate
        \\units m3/s
        \\minimum> 0.0
        \\ip-units gal/min
  N9,   \\field Grout Thermal Conductivity
        \\units W/m-K
        \\minimum> 0.0
  N10,  \\field Pipe Thermal Conductivity
        \\units W/m-K
        \\minimum> 0.0
  N11,  \\field Pipe Out Diameter
        \\units m
        \\minimum> 0.0
        \\ip-units in
  N12,  \\field U-Tube Distance
        \\units m
        \\minimum> 0.0
  N13,  \\field Pipe Thickness
        \\units m
        \\minimum> 0.0
        \\ip-units in
  N14,  \\field Maximum Length of Simulation
        \\minimum> 0.0
  N15,  \\field G-Function Reference Ratio
        \\type real
        \\units dimensionless
        \\minimum> 0.0
        \\default 0.0005
  N16,  \\field Number of Data Pairs of the G Function
        \\minimum> 0.0
        \\maximum 100
  N17, \\field G-Function Ln(T/Ts) Value 1
       \\type real
  N18, \\field G-Function G Value 1
       \\type real
  N19, \\field G-Function Ln(T/Ts) Value 2
       \\type real
  N20, \\field G-Function G Value 2
       \\type real
  N21, \\field G-Function Ln(T/Ts) Value 3
       \\type real
  N22, \\field G-Function G Value 3
       \\type real
  N23, \\field G-Function Ln(T/Ts) Value 4
       \\type real
  N24, \\field G-Function G Value 4
       \\type real
  N25, \\field G-Function Ln(T/Ts) Value 5
       \\type real
  N26, \\field G-Function G Value 5
       \\type real
  N27, \\field G-Function Ln(T/Ts) Value 6
       \\type real
  N28, \\field G-Function G Value 6
       \\type real
  N29, \\field G-Function Ln(T/Ts) Value 7
       \\type real
  N30, \\field G-Function G Value 7
       \\type real
  N31, \\field G-Function Ln(T/Ts) Value 8
       \\type real
  N32, \\field G-Function G Value 8
       \\type real
  N33, \\field G-Function Ln(T/Ts) Value 9
       \\type real
  N34, \\field G-Function G Value 9
       \\type real
  N35, \\field G-Function Ln(T/Ts) Value 10
       \\type real
  N36, \\field G-Function G Value 10
       \\type real
  N37, \\field G-Function Ln(T/Ts) Value 11
       \\type real
  N38, \\field G-Function G Value 11
       \\type real
  N39, \\field G-Function Ln(T/Ts) Value 12
       \\type real
  N40, \\field G-Function G Value 12
       \\type real
  N41, \\field G-Function Ln(T/Ts) Value 13
       \\type real
  N42, \\field G-Function G Value 13
       \\type real
  N43, \\field G-Function Ln(T/Ts) Value 14
       \\type real
  N44, \\field G-Function G Value 14
       \\type real
  N45, \\field G-Function Ln(T/Ts) Value 15
       \\type real
  N46, \\field G-Function G Value 15
       \\type real
  N47, \\field G-Function Ln(T/Ts) Value 16
       \\type real
  N48, \\field G-Function G Value 16
       \\type real
  N49, \\field G-Function Ln(T/Ts) Value 17
       \\type real
  N50, \\field G-Function G Value 17
       \\type real
  N51, \\field G-Function Ln(T/Ts) Value 18
       \\type real
  N52, \\field G-Function G Value 18
       \\type real
  N53, \\field G-Function Ln(T/Ts) Value 19
       \\type real
  N54, \\field G-Function G Value 19
       \\type real
  N55, \\field G-Function Ln(T/Ts) Value 20
       \\type real
  N56, \\field G-Function G Value 20
       \\type real
  N57, \\field G-Function Ln(T/Ts) Value 21
       \\type real
  N58, \\field G-Function G Value 21
       \\type real
  N59, \\field G-Function Ln(T/Ts) Value 22
       \\type real
  N60, \\field G-Function G Value 22
       \\type real
  N61, \\field G-Function Ln(T/Ts) Value 23
       \\type real
  N62, \\field G-Function G Value 23
       \\type real
  N63, \\field G-Function Ln(T/Ts) Value 24
       \\type real
  N64, \\field G-Function G Value 24
       \\type real
  N65, \\field G-Function Ln(T/Ts) Value 25
       \\type real
  N66, \\field G-Function G Value 25
       \\type real
  N67, \\field G-Function Ln(T/Ts) Value 26
       \\type real
  N68, \\field G-Function G Value 26
       \\type real
  N69, \\field G-Function Ln(T/Ts) Value 27
       \\type real
  N70, \\field G-Function G Value 27
       \\type real
  N71, \\field G-Function Ln(T/Ts) Value 28
       \\type real
  N72, \\field G-Function G Value 28
       \\type real
  N73, \\field G-Function Ln(T/Ts) Value 29
       \\type real
  N74, \\field G-Function G Value 29
       \\type real
  N75, \\field G-Function Ln(T/Ts) Value 30
       \\type real
  N76, \\field G-Function G Value 30
       \\type real
  N77, \\field G-Function Ln(T/Ts) Value 31
       \\type real
  N78, \\field G-Function G Value 31
       \\type real
  N79, \\field G-Function Ln(T/Ts) Value 32
       \\type real
  N80, \\field G-Function G Value 32
       \\type real
  N81, \\field G-Function Ln(T/Ts) Value 33
       \\type real
  N82, \\field G-Function G Value 33
       \\type real
  N83, \\field G-Function Ln(T/Ts) Value 34
       \\type real
  N84, \\field G-Function G Value 34
       \\type real
  N85, \\field G-Function Ln(T/Ts) Value 35
       \\type real
  N86, \\field G-Function G Value 35
       \\type real
  N87, \\field G-Function Ln(T/Ts) Value 36
       \\type real
  N88, \\field G-Function G Value 36
       \\type real
  N89, \\field G-Function Ln(T/Ts) Value 37
       \\type real
  N90, \\field G-Function G Value 37
       \\type real
  N91, \\field G-Function Ln(T/Ts) Value 38
       \\type real
  N92, \\field G-Function G Value 38
       \\type real
  N93, \\field G-Function Ln(T/Ts) Value 39
       \\type real
  N94, \\field G-Function G Value 39
       \\type real
  N95, \\field G-Function Ln(T/Ts) Value 40
       \\type real
  N96, \\field G-Function G Value 40
       \\type real
  N97, \\field G-Function Ln(T/Ts) Value 41
        \\type real
  N98, \\field G-Function G Value 41
        \\type real
  N99, \\field G-Function Ln(T/Ts) Value 42
        \\type real
  N100, \\field G-Function G Value 42
        \\type real
  N101, \\field G-Function Ln(T/Ts) Value 43
        \\type real
  N102, \\field G-Function G Value 43
        \\type real
  N103, \\field G-Function Ln(T/Ts) Value 44
        \\type real
  N104, \\field G-Function G Value 44
        \\type real
  N105, \\field G-Function Ln(T/Ts) Value 45
        \\type real
  N106, \\field G-Function G Value 45
        \\type real
  N107, \\field G-Function Ln(T/Ts) Value 46
        \\type real
  N108, \\field G-Function G Value 46
        \\type real
  N109, \\field G-Function Ln(T/Ts) Value 47
        \\type real
  N110, \\field G-Function G Value 47
        \\type real
  N111, \\field G-Function Ln(T/Ts) Value 48
        \\type real
  N112, \\field G-Function G Value 48
        \\type real
  N113, \\field G-Function Ln(T/Ts) Value 49
        \\type real
  N114, \\field G-Function G Value 49
        \\type real
  N115, \\field G-Function Ln(T/Ts) Value 50
        \\type real
  N116, \\field G-Function G Value 50
        \\type real
  N117, \\field G-Function Ln(T/Ts) Value 51
        \\type real
  N118, \\field G-Function G Value 51
        \\type real
  N119, \\field G-Function Ln(T/Ts) Value 52
        \\type real
  N120, \\field G-Function G Value 52
        \\type real
  N121, \\field G-Function Ln(T/Ts) Value 53
        \\type real
  N122, \\field G-Function G Value 53
        \\type real
  N123, \\field G-Function Ln(T/Ts) Value 54
        \\type real
  N124, \\field G-Function G Value 54
        \\type real
  N125, \\field G-Function Ln(T/Ts) Value 55
        \\type real
  N126, \\field G-Function G Value 55
        \\type real
  N127, \\field G-Function Ln(T/Ts) Value 56
        \\type real
  N128, \\field G-Function G Value 56
        \\type real
  N129, \\field G-Function Ln(T/Ts) Value 57
        \\type real
  N130, \\field G-Function G Value 57
        \\type real
  N131, \\field G-Function Ln(T/Ts) Value 58
        \\type real
  N132, \\field G-Function G Value 58
        \\type real
  N133, \\field G-Function Ln(T/Ts) Value 59
        \\type real
  N134, \\field G-Function G Value 59
        \\type real
  N135, \\field G-Function Ln(T/Ts) Value 60
        \\type real
  N136, \\field G-Function G Value 60
        \\type real
  N137, \\field G-Function Ln(T/Ts) Value 61
        \\type real
  N138, \\field G-Function G Value 61
        \\type real
  N139, \\field G-Function Ln(T/Ts) Value 62
        \\type real
  N140, \\field G-Function G Value 62
        \\type real
  N141, \\field G-Function Ln(T/Ts) Value 63
        \\type real
  N142, \\field G-Function G Value 63
        \\type real
  N143, \\field G-Function Ln(T/Ts) Value 64
        \\type real
  N144, \\field G-Function G Value 64
        \\type real
  N145, \\field G-Function Ln(T/Ts) Value 65
        \\type real
  N146, \\field G-Function G Value 65
        \\type real
  N147, \\field G-Function Ln(T/Ts) Value 66
        \\type real
  N148, \\field G-Function G Value 66
        \\type real
  N149, \\field G-Function Ln(T/Ts) Value 67
        \\type real
  N150, \\field G-Function G Value 67
        \\type real
  N151, \\field G-Function Ln(T/Ts) Value 68
        \\type real
  N152, \\field G-Function G Value 68
        \\type real
  N153, \\field G-Function Ln(T/Ts) Value 69
        \\type real
  N154, \\field G-Function G Value 69
        \\type real
  N155, \\field G-Function Ln(T/Ts) Value 70
        \\type real
  N156, \\field G-Function G Value 70
        \\type real
  N157, \\field G-Function Ln(T/Ts) Value 71
        \\type real
  N158, \\field G-Function G Value 71
        \\type real
  N159, \\field G-Function Ln(T/Ts) Value 72
        \\type real
  N160, \\field G-Function G Value 72
        \\type real
  N161, \\field G-Function Ln(T/Ts) Value 73
        \\type real
  N162, \\field G-Function G Value 73
        \\type real
  N163, \\field G-Function Ln(T/Ts) Value 74
        \\type real
  N164, \\field G-Function G Value 74
        \\type real
  N165, \\field G-Function Ln(T/Ts) Value 75
        \\type real
  N166, \\field G-Function G Value 75
        \\type real
  N167, \\field G-Function Ln(T/Ts) Value 76
        \\type real
  N168, \\field G-Function G Value 76
        \\type real
  N169, \\field G-Function Ln(T/Ts) Value 77
        \\type real
  N170, \\field G-Function G Value 77
        \\type real
  N171, \\field G-Function Ln(T/Ts) Value 78
        \\type real
  N172, \\field G-Function G Value 78
        \\type real
  N173, \\field G-Function Ln(T/Ts) Value 79
        \\type real
  N174, \\field G-Function G Value 79
        \\type real
  N175, \\field G-Function Ln(T/Ts) Value 80
        \\type real
  N176, \\field G-Function G Value 80
        \\type real
  N177, \\field G-Function Ln(T/Ts) Value 81
        \\type real
  N178, \\field G-Function G Value 81
        \\type real
  N179, \\field G-Function Ln(T/Ts) Value 82
        \\type real
  N180, \\field G-Function G Value 82
        \\type real
  N181, \\field G-Function Ln(T/Ts) Value 83
        \\type real
  N182, \\field G-Function G Value 83
        \\type real
  N183, \\field G-Function Ln(T/Ts) Value 84
        \\type real
  N184, \\field G-Function G Value 84
        \\type real
  N185, \\field G-Function Ln(T/Ts) Value 85
        \\type real
  N186, \\field G-Function G Value 85
        \\type real
  N187, \\field G-Function Ln(T/Ts) Value 86
        \\type real
  N188, \\field G-Function G Value 86
        \\type real
  N189, \\field G-Function Ln(T/Ts) Value 87
        \\type real
  N190, \\field G-Function G Value 87
        \\type real
  N191, \\field G-Function Ln(T/Ts) Value 88
        \\type real
  N192, \\field G-Function G Value 88
        \\type real
  N193, \\field G-Function Ln(T/Ts) Value 89
        \\type real
  N194, \\field G-Function G Value 89
        \\type real
  N195, \\field G-Function Ln(T/Ts) Value 90
        \\type real
  N196, \\field G-Function G Value 90
        \\type real
  N197, \\field G-Function Ln(T/Ts) Value 91
        \\type real
  N198, \\field G-Function G Value 91
        \\type real
  N199, \\field G-Function Ln(T/Ts) Value 92
        \\type real
  N200, \\field G-Function G Value 92
        \\type real
  N201, \\field G-Function Ln(T/Ts) Value 93
        \\type real
  N202, \\field G-Function G Value 93
        \\type real
  N203, \\field G-Function Ln(T/Ts) Value 94
        \\type real
  N204, \\field G-Function G Value 94
        \\type real
  N205, \\field G-Function Ln(T/Ts) Value 95
        \\type real
  N206, \\field G-Function G Value 95
        \\type real
  N207, \\field G-Function Ln(T/Ts) Value 96
        \\type real
  N208, \\field G-Function G Value 96
        \\type real
  N209, \\field G-Function Ln(T/Ts) Value 97
        \\type real
  N210, \\field G-Function G Value 97
        \\type real
  N211, \\field G-Function Ln(T/Ts) Value 98
        \\type real
  N212, \\field G-Function G Value 98
        \\type real
  N213, \\field G-Function Ln(T/Ts) Value 99
        \\type real
  N214, \\field G-Function G Value 99
        \\type real
  N215, \\field G-Function Ln(T/Ts) Value 100
        \\type real
  N216; \\field G-Function G Value 100
        \\type real

GroundHeatExchanger:Pond,
        \\memo A model of a shallow pond with immersed pipe loops.
        \\memo Typically used in hybrid geothermal systems and included in the condenser loop.
        \\memo This component may also be used as a simple solar collector.
   A1 , \\field Name
        \\required-field
   A2 , \\field Fluid Inlet Node Name
        \\type node
        \\required-field
   A3 , \\field Fluid Outlet Node Name
        \\required-field
        \\type node
   N1 , \\field Pond Depth
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
   N2 , \\field Pond Area
        \\required-field
        \\type real
        \\units m2
        \\minimum> 0
   N3 , \\field Hydronic Tubing Inside Diameter
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N4 , \\field Hydronic Tubing Outside Diameter
        \\required-field
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N5 , \\field Hydronic Tubing Thermal Conductivity
        \\required-field
        \\type real
        \\units W/m-K
        \\minimum> 0
   N6 , \\field Ground Thermal Conductivity
        \\required-field
        \\type real
        \\units W/m2-K
        \\minimum> 0
   N7 , \\field Number of Tubing Circuits
        \\required-field
        \\type integer
        \\minimum 1
   N8 ; \\field Length of Each Tubing Circuit
        \\required-field
        \\type real
        \\units m
        \\minimum 0

GroundHeatExchanger:Surface,
        \\memo A hydronic surface/panel consisting of a multi-layer construction with embedded rows of tubes.
        \\memo Typically used in hybrid geothermal systems and included in the condenser loop.
        \\memo This component may also be used as a simple solar collector.
        \\memo The bottom surface may be defined as ground-coupled or exposed to wind (eg. bridge deck).
   A1 , \\field Name
        \\required-field
   A2 , \\field Construction Name
        \\required-field
        \\type object-list
        \\object-list ConstructionNames
   A3 , \\field Fluid Inlet Node Name
        \\type node
        \\required-field
   A4 , \\field Fluid Outlet Node Name
        \\required-field
        \\type node
   N1 , \\field Hydronic Tubing Inside Diameter
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
   N2 , \\field Number of Tubing Circuits
        \\type integer
        \\minimum 1
   N3 , \\field Hydronic Tube Spacing
        \\type real
        \\units m
        \\minimum> 0.0
   N4 , \\field Surface Length
        \\type real
        \\units m
        \\minimum> 0.0
   N5 , \\field Surface Width
        \\type real
        \\units m
        \\minimum> 0.0
   A5 ; \\field Lower Surface Environment
        \\type choice
        \\key Ground
        \\key Exposed
        \\default Ground

GroundHeatExchanger:HorizontalTrench,
        \\memo This models a horizontal heat exchanger placed in a series of trenches
        \\memo The model uses the PipingSystem:Underground underlying algorithms,
        \\memo but provides a more usable input interface.
   A1 , \\field Name
        \\required-field
   A2 , \\field Inlet Node Name
        \\type node
        \\required-field
   A3 , \\field Outlet Node Name
        \\type node
        \\required-field
   N1 , \\field Design Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\minimum> 0
   N2 , \\field Trench Length in Pipe Axial Direction
        \\note This is the total pipe axial length of the heat exchanger
        \\note If all pipe trenches are parallel, this is the length of a
        \\note single trench.  If a single, long run of pipe is used with one
        \\note trench, this is the full length of the pipe run.
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units ft
        \\default 50
   N3 , \\field Number of Trenches
        \\note This is the number of horizontal legs that will be used
        \\note in the entire heat exchanger, one pipe per trench
        \\type integer
        \\minimum 1
        \\default 1
   N4 , \\field Horizontal Spacing Between Pipes
        \\note This represents the average horizontal spacing between any two
        \\note trenches for heat exchangers that have multiple trenches
        \\type real
        \\units m
        \\minimum> 0
        \\default 1.0
   N5 , \\field Pipe Inner Diameter
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
        \\default 0.016
   N6 , \\field Pipe Outer Diameter
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units in
        \\default 0.026
   N7 , \\field Burial Depth
        \\note This is the burial depth of the pipes, or the trenches
        \\note containing the pipes
        \\type real
        \\units m
        \\minimum> 0
        \\ip-units ft
        \\default 1.5
   N8 , \\field Soil Thermal Conductivity
        \\type real
        \\units W/m-K
        \\minimum> 0
        \\default 1.08
   N9 , \\field Soil Density
        \\type real
        \\units kg/m3
        \\minimum> 0
        \\default 962
   N10, \\field Soil Specific Heat
        \\required-field
        \\type real
        \\units J/kg-K
        \\minimum> 0
        \\default 2576
   N11, \\field Pipe Thermal Conductivity
        \\type real
        \\units W/m-K
        \\minimum> 0
        \\default 0.3895
   N12, \\field Pipe Density
        \\type real
        \\units kg/m3
        \\minimum> 0
        \\default 641
   N13, \\field Pipe Specific Heat
        \\type real
        \\units J/kg-K
        \\minimum> 0
        \\default 2405
   N14, \\field Soil Moisture Content Percent
        \\type real
        \\units percent
        \\minimum 0
        \\maximum 100
        \\default 30
   N15, \\field Soil Moisture Content Percent at Saturation
        \\type real
        \\units percent
        \\minimum 0
        \\maximum 100
        \\default 50
   N16, \\field Kusuda-Achenbach Average Surface Temperature
        \\note This is the parameter for average annual surface temperature
        \\note This is an optional input in that if it is missing, a
        \\note Site:GroundTemperature:Shallow object must be found in the input
        \\note The undisturbed ground temperature will be approximated from this object
        \\type real
        \\units C
   N17, \\field Kusuda-Achenbach Average Amplitude of Surface Temperature
        \\note This is the parameter for annual average amplitude from average surface temperature
        \\note This is an optional input in that if it is missing, a
        \\note Site:GroundTemperature:Shallow object must be found in the input
        \\note The undisturbed ground temperature will be approximated from this object
        \\type real
        \\units C
   N18, \\field Kusuda-Achenbach Phase Shift of Minimum Surface Temperature
        \\note This is the parameter for phase shift from minimum surface temperature
        \\note This is an optional input in that if it is missing, a
        \\note Site:GroundTemperature:Shallow object must be found in the input
        \\note The undisturbed ground temperature will be approximated from this object
        \\type real
        \\units days
   N19; \\field Evapotranspiration Ground Cover Parameter
        \\type real
        \\minimum 0
        \\maximum 1.5
        \\default 0.4
        \\note This specifies the ground cover effects during evapotranspiration
        \\note  calculations.  The value roughly represents the following cases:
        \\note  = 0   : concrete or other solid, non-permeable ground surface material
        \\note  = 0.5 : short grass, much like a manicured lawn
        \\note  = 1   : standard reference state (12 cm grass)
        \\note  = 1.5 : wild growth

HeatExchanger:FluidToFluid,
        \\memo A fluid/fluid heat exchanger designed to couple the supply side of one loop to the demand side of another loop
        \\memo Loops can be either plant or condenser loops but no air side connections are allowed
        \\min-fields 14
   A1 , \\field Name
        \\required-field
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\note default is that heat exchanger is on
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Loop Demand Side Inlet Node Name
        \\type node
        \\required-field
        \\note This connection is to the demand side of a loop and is the inlet to the heat exchanger
   A4 , \\field Loop Demand Side Outlet Node Name
        \\type node
        \\required-field
        \\note This connection is to the demand side of a loop
   N1 , \\field Loop Demand Side Design Flow Rate
        \\type real
        \\required-field
        \\minimum> 0.0
        \\units m3/s
        \\ip-units gal/min
        \\autosizable
   A5 , \\field Loop Supply Side Inlet Node Name
        \\type node
        \\required-field
   A6 , \\field Loop Supply Side Outlet Node Name
        \\type node
        \\required-field
   N2 , \\field Loop Supply Side Design Flow Rate
        \\type real
        \\required-field
        \\minimum> 0.0
        \\units m3/s
        \\ip-units gal/min
        \\autosizable
   A7 , \\field Heat Exchange Model Type
        \\type choice
        \\key CrossFlowBothUnMixed
        \\key CrossFlowBothMixed
        \\key CrossFlowSupplyMixedDemandUnMixed
        \\key CrossFlowSupplyUnMixedDemandMixed
        \\key ParallelFlow
        \\key CounterFlow
        \\key Ideal
        \\default Ideal
        \\required-field
   N3 , \\field Heat Exchanger U-Factor Times Area Value
        \\type real
        \\units W/k
        \\minimum> 0.0
        \\autosizable
        \\required-field
   A8 , \\field Control Type
        \\type choice
        \\key UncontrolledOn
        \\key OperationSchemeModulated
        \\key OperationSchemeOnOff
        \\key HeatingSetpointModulated
        \\key HeatingSetpointOnOff
        \\key CoolingSetpointModulated
        \\key CoolingSetpointOnOff
        \\key DualDeadbandSetpointModulated
        \\key DualDeadbandSetpointOnOff
        \\key CoolingDifferentialOnOff
        \\key CoolingSetpointOnOffWithComponentOverride
        \\required-field
        \\default UncontrolledOn
   A9 , \\field Heat Exchanger Setpoint Node Name
        \\note Setpoint node is needed with any Control Type that is "*Setpoint*"
        \\type node
   N4 , \\field Minimum Temperature Difference to Activate Heat Exchanger
        \\note Tolerance between control temperatures used to determine if heat exchanger should run.
        \\type real
        \\minimum 0.0
        \\maximum 50
        \\default 0.01
        \\units deltaC
   A10, \\field Heat Transfer Metering End Use Type
        \\note This feild controls end use reporting for heat transfer meters
        \\type choice
        \\key FreeCooling
        \\key HeatRecovery
        \\key HeatRejection
        \\key HeatRecoveryForCooling
        \\key HeatRecoveryForHeating
        \\key LoopToLoop
        \\default LoopToLoop
        \\required-field
   A11, \\field Component Override Loop Supply Side Inlet Node Name
        \\type node
        \\note This field is only used if Control Type is set to CoolingSetpointOnOffWithComponentOverride
   A12, \\field Component Override Loop Demand Side Inlet Node Name
        \\type node
        \\note This field is only used if Control Type is set to CoolingSetpointOnOffWithComponentOverride
   A13, \\field Component Override Cooling Control Temperature Mode
        \\type choice
        \\key WetBulbTemperature
        \\key DryBulbTemperature
        \\key Loop
        \\default Loop
        \\note This field is only used if Control Type is set to CoolingSetpointOnOffWithComponentOverride
   N5 , \\field Sizing Factor
        \\note Multiplies the autosized flow rates for this device
        \\type real
        \\minimum> 0.0
        \\default 1.0
   N6 , \\field Operation Minimum Temperature Limit
        \\note Lower limit on inlet temperatures, heat exchanger will not operate if either inlet is below this limit
        \\type real
        \\units C
   N7 ; \\field Operation Maximum Temperature Limit
        \\note Upper limit on inlet temperatures, heat exchanger will not operate if either inlet is above this limit
        \\type real
        \\units C

\\group Water Heaters and Thermal Storage

WaterHeater:Mixed,
       \\memo Water heater with well-mixed, single-node water tank. May be used to model a tankless
       \\memo water heater (small tank volume), a hot water storage tank (zero heater capacity), or
       \\memo a heat pump water heater (see WaterHeater:HeatPump.)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WaterHeaterNames
       \\reference WaterHeaterMixedNames
  N1 , \\field Tank Volume
       \\autosizable
       \\type real
       \\units m3
       \\minimum 0.0
       \\default 0.0
       \\ip-units gal
  A2 , \\field Setpoint Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N2 , \\field Deadband Temperature Difference
       \\type real
       \\units deltaC
       \\minimum 0.0
       \\default 0.0
  N3 , \\field Maximum Temperature Limit
       \\type real
       \\units C
  A3 , \\field Heater Control Type
       \\type choice
       \\key Cycle
       \\key Modulate
       \\default Cycle
  N4 , \\field Heater Maximum Capacity
       \\autosizable
       \\type real
       \\units W
       \\minimum 0.0
  N5 , \\field Heater Minimum Capacity
       \\note Only used when Heater Control Type is set to Modulate
       \\type real
       \\units W
       \\minimum 0.0
  N6 , \\field Heater Ignition Minimum Flow Rate
       \\note Not yet implemented
       \\type real
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
  N7 , \\field Heater Ignition Delay
       \\note Not yet implemented
       \\type real
       \\units s
       \\minimum 0.0
       \\default 0.0
  A4 , \\field Heater Fuel Type
       \\required-field
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N8 , \\field Heater Thermal Efficiency
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  A5 , \\field Part Load Factor Curve Name
       \\type object-list
       \\object-list QuadraticCubicCurves
  N9 , \\field Off Cycle Parasitic Fuel Consumption Rate
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
  A6 , \\field Off Cycle Parasitic Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N10, \\field Off Cycle Parasitic Heat Fraction to Tank
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N11, \\field On Cycle Parasitic Fuel Consumption Rate
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
  A7 , \\field On Cycle Parasitic Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N12, \\field On Cycle Parasitic Heat Fraction to Tank
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  A8 , \\field Ambient Temperature Indicator
       \\required-field
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
  A9 , \\field Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A10, \\field Ambient Temperature Zone Name
       \\type object-list
       \\object-list ZoneNames
  A11, \\field Ambient Temperature Outdoor Air Node Name
        \\type node
       \\note required for Ambient Temperature Indicator=Outdoors
  N13, \\field Off Cycle Loss Coefficient to Ambient Temperature
       \\type real
       \\units W/K
       \\minimum 0.0
  N14, \\field Off Cycle Loss Fraction to Zone
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N15, \\field On Cycle Loss Coefficient to Ambient Temperature
       \\type real
       \\units W/K
       \\minimum 0.0
  N16, \\field On Cycle Loss Fraction to Zone
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N17, \\field Peak Use Flow Rate
       \\type real
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
       \\note Only used if Use Side Node connections are blank
  A12, \\field Use Flow Rate Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Only used if Use Side Node connections are blank
  A13, \\field Cold Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Only used if Use Side Node connections are blank
       \\note Defaults to water temperatures calculated by Site:WaterMainsTemperature object
  A14, \\field Use Side Inlet Node Name
       \\type node
  A15, \\field Use Side Outlet Node Name
       \\type node
  N18, \\field Use Side Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A16, \\field Source Side Inlet Node Name
       \\type node
  A17, \\field Source Side Outlet Node Name
       \\type node
  N19, \\field Source Side Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N20, \\field Use Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N21, \\field Source Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N22; \\field Indirect Water Heating Recovery Time
       \\type real
       \\default 1.5
       \\note Parameter for autosizing design flow rates for indirectly heated water tanks
       \\note Time required to raise temperature of entire tank from 14.4C to 57.2C
       \\units hr
       \\minimum> 0.0

WaterHeater:Stratified,
       \\memo Water heater with stratified, multi-node water tank. May be used to model a tankless
       \\memo water heater (small tank volume), a hot water storage tank (zero heater capacity), or
       \\memo a heat pump water heater (see WaterHeater:HeatPump.)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WaterHeaterNames
       \\reference WaterHeaterStratifiedNames
  A2 , \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General
  N1 , \\field Tank Volume
       \\autosizable
       \\required-field
       \\type real
       \\units m3
       \\minimum> 0.0
       \\ip-units gal
  N2 , \\field Tank Height
       \\autosizable
       \\required-field
       \\type real
       \\units m
       \\minimum> 0.0
       \\note Height is measured in the axial direction for horizontal cylinders
  A3 , \\field Tank Shape
       \\type choice
       \\key VerticalCylinder
       \\key HorizontalCylinder
       \\key Other
       \\default VerticalCylinder
  N3 , \\field Tank Perimeter
       \\type real
       \\units m
       \\minimum 0.0
       \\note Only used if Tank Shape is Other
  N4 , \\field Maximum Temperature Limit
       \\type real
       \\units C
  A4 , \\field Heater Priority Control
       \\type choice
       \\key MasterSlave
       \\key Simultaneous
       \\default MasterSlave
  A5 , \\field Heater 1 Setpoint Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N5 , \\field Heater 1 Deadband Temperature Difference
       \\type real
       \\units deltaC
       \\minimum 0.0
       \\default 0.0
  N6 , \\field Heater 1 Capacity
       \\type real
       \\units W
       \\minimum 0.0
       \\autosizable
  N7 , \\field Heater 1 Height
       \\type real
       \\units m
       \\minimum 0.0
  A6 , \\field Heater 2 Setpoint Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  N8 , \\field Heater 2 Deadband Temperature Difference
       \\type real
       \\units deltaC
       \\minimum 0.0
       \\default 0.0
  N9 , \\field Heater 2 Capacity
       \\type real
       \\units W
       \\minimum 0.0
  N10, \\field Heater 2 Height
       \\type real
       \\units m
       \\minimum 0.0
  A7 , \\field Heater Fuel Type
       \\required-field
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N11, \\field Heater Thermal Efficiency
       \\required-field
       \\type real
       \\minimum> 0.0
       \\maximum 1.0
  N12, \\field Off Cycle Parasitic Fuel Consumption Rate
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
  A8 , \\field Off Cycle Parasitic Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N13, \\field Off Cycle Parasitic Heat Fraction to Tank
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N14, \\field Off Cycle Parasitic Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
  N15, \\field On Cycle Parasitic Fuel Consumption Rate
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
  A9 , \\field On Cycle Parasitic Fuel Type
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key PropaneGas
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Coal
       \\key Diesel
       \\key Gasoline
       \\key OtherFuel1
       \\key OtherFuel2
       \\key Steam
       \\key DistrictHeating
  N16, \\field On Cycle Parasitic Heat Fraction to Tank
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 0.0
  N17, \\field On Cycle Parasitic Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
  A10, \\field Ambient Temperature Indicator
       \\required-field
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
  A11, \\field Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A12, \\field Ambient Temperature Zone Name
       \\type object-list
       \\object-list ZoneNames
  A13, \\field Ambient Temperature Outdoor Air Node Name
       \\type node
       \\note required for Ambient Temperature Indicater=Outdoors
  N18, \\field Uniform Skin Loss Coefficient per Unit Area to Ambient Temperature
       \\type real
       \\units W/m2-K
       \\minimum 0.0
  N19, \\field Skin Loss Fraction to Zone
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N20, \\field Off Cycle Flue Loss Coefficient to Ambient Temperature
       \\type real
       \\units W/K
       \\minimum 0.0
  N21, \\field Off Cycle Flue Loss Fraction to Zone
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  N22, \\field Peak Use Flow Rate
       \\type real
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
       \\note Only used if Use Side Node connections are blank
  A14, \\field Use Flow Rate Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note If blank, defaults to 1.0 at all times
       \\note Only used if use side node connections are blank
  A15, \\field Cold Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Only used if use side node connections are blank
       \\note Defaults to water temperatures calculated by Site:WaterMainsTemperature object
  A16, \\field Use Side Inlet Node Name
       \\type node
  A17, \\field Use Side Outlet Node Name
       \\type node
  N23, \\field Use Side Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
       \\note The use side effectiveness in the stratified tank model is a simplified analogy of
       \\note a heat exchanger's effectiveness. This effectiveness is equal to the fraction of
       \\note use mass flow rate that directly mixes with the tank fluid. And one minus the
       \\note effectiveness is the fraction that bypasses the tank. The use side mass flow rate
       \\note that bypasses the tank is mixed with the fluid or water leaving the stratified tank.
  N24, \\field Use Side Inlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
       \\note Defaults to bottom of tank
  N25, \\field Use Side Outlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\autocalculatable
       \\default Autocalculate
       \\note Defaults to top of tank
  A18, \\field Source Side Inlet Node Name
       \\type node
  A19, \\field Source Side Outlet Node Name
       \\type node
  N26, \\field Source Side Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
       \\note The source side effectiveness in the stratified tank model is a simplified analogy of
       \\note a heat exchanger's effectiveness. This effectiveness is equal to the fraction of
       \\note source mass flow rate that directly mixes with the tank fluid. And one minus the
       \\note effectiveness is the fraction that bypasses the tank. The source side mass flow rate
       \\note that bypasses the tank is mixed with the fluid or water leaving the stratified tank.
  N27, \\field Source Side Inlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\autocalculatable
       \\default Autocalculate
       \\note Defaults to top of tank
  N28, \\field Source Side Outlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
       \\note Defaults to bottom of tank
  A20, \\field Inlet Mode
       \\type choice
       \\key Fixed
       \\key Seeking
       \\default Fixed
  N29, \\field Use Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N30, \\field Source Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N31, \\field Indirect Water Heating Recovery Time
       \\type real
       \\default 1.5
       \\note Parameter for autosizing design flow rates for indirectly heated water tanks
       \\note time required to raise temperature of entire tank from 14.4C to 57.2C
       \\units hr
       \\minimum> 0.0
  N32, \\field Number of Nodes
       \\type integer
       \\minimum 1
       \\maximum 10
       \\default 1
  N33, \\field Additional Destratification Conductivity
       \\type real
       \\units W/m-K
       \\minimum 0.0
       \\default 0.0
  N34, \\field Node 1 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N35, \\field Node 2 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N36, \\field Node 3 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N37, \\field Node 4 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N38, \\field Node 5 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N39, \\field Node 6 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N40, \\field Node 7 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N41, \\field Node 8 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N42, \\field Node 9 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N43; \\field Node 10 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0

WaterHeater:Sizing,
       \\min-fields 4
       \\memo This input object is used with WaterHeater:Mixed or
       \\memo with WaterHeater:Stratified to autosize tank volume and heater capacity
       \\memo This object is not needed if water heaters are not autosized.
  A1 , \\field WaterHeater Name
       \\required-field
       \\type object-list
       \\object-list WaterHeaterNames
  A2 , \\field Design Mode
       \\type choice
       \\key PeakDraw
       \\key ResidentialHUD-FHAMinimum
       \\key PerPerson
       \\key PerFloorArea
       \\key PerUnit
       \\key PerSolarCollectorArea
  N1 , \\field Time Storage Can Meet Peak Draw
       \\type real
       \\units hr
       \\note Only used for Design Mode = PeakDraw
       \\minimum 0.0
  N2 , \\field Time for Tank Recovery
       \\type real
       \\units hr
       \\note Only used for Design Mode = PeakDraw
       \\minimum 0.0
  N3 , \\field Nominal Tank Volume for Autosizing Plant Connections
       \\type real
       \\units m3
       \\note Only used if Design Mode = PeakDraw and the water heater also
       \\note has autosized flow rates for connections on the demand side of a plant loop
  N4 , \\field Number of Bedrooms
       \\type integer
       \\note Only used for Design Mode = ResidentialHUD-FHAMinimum
       \\minimum 1
  N5 , \\field Number of Bathrooms
       \\type integer
       \\note Only used for Design Mode = ResidentialHUD-FHAMinimum
       \\minimum 1
  N6 , \\field Storage Capacity per Person
       \\type real
       \\units m3/Person
       \\note Only used for Design Mode = PerPerson
       \\minimum 0.0
  N7 , \\field Recovery Capacity per Person
       \\type real
       \\units m3/hr-person
       \\note Only used for Design Mode = PerPerson
       \\minimum 0.0
  N8 , \\field Storage Capacity per Floor Area
       \\type real
       \\units m3/m2
       \\note Only used for Design Mode = PerFloorArea
       \\minimum 0.0
  N9 , \\field Recovery Capacity per Floor Area
       \\type real
       \\units m3/hr-m2
       \\note Only used for Design Mode = PerFloorArea
       \\minimum 0.0
  N10 , \\field Number of Units
       \\type real
       \\note Only used for Design Mode = PerUnit
  N11 , \\field Storage Capacity per Unit
       \\units m3
       \\type real
       \\note Only used for Design Mode = PerUnit
       \\minimum 0.0
  N12 , \\field Recovery Capacity PerUnit
       \\units m3/hr
       \\type real
       \\note Only used for Design Mode = PerUnit
       \\minimum 0.0
  N13 ,\\field Storage Capacity per Collector Area
       \\units m3/m2
       \\type real
       \\note Only used for Design Mode = PerSolarCollectorArea
       \\minimum 0.0
  N14 ; \\field Height Aspect Ratio
       \\type real
       \\note only used if for WaterHeater:Stratified
       \\minimum 0.0

WaterHeater:HeatPump,
       \\min-fields 31
       \\memo This object models an air-source heat pump for water heating.
       \\memo WaterHeater:HeatPump is a compound object that references other component objects -
       \\memo Coil:WaterHeating:AirToWaterHeatPump, Fan:OnOff, WaterHeater:Mixed
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note Unique name for this instance of a heat pump water heater.
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
       \\note Schedule values of 0 denote the heat pump compressor is off and the parasitic electric
       \\note energy is also off.
  A3,  \\field Compressor Setpoint Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note Defines the cut-out temperature where the heat pump compressor turns off.
       \\note The heat pump compressor setpoint temperature should always be greater
       \\note than the water tank's heater (element or burner) setpoint temperature.
  N1 , \\field Dead Band Temperature Difference
       \\type real
       \\units deltaC
       \\minimum> 0.0
       \\maximum 20
       \\default 5.0
       \\note Setpoint temperature minus the dead band temperature difference defines
       \\note the cut-in temperature where the heat pump compressor turns on.
       \\note The water tank's heater (element or burner) setpoint temperature
       \\note should always be less than the heat pump compressor cut-in temperature.
  A4 , \\field Condenser Water Inlet Node Name
       \\required-field
       \\type node
       \\note Should match the field Source Outlet Node Name in the water heater tank object.
       \\note Should also match the Condenser Water Inlet Node Name in the associated
       \\note DX coil object.
  A5 , \\field Condenser Water Outlet Node Name
       \\required-field
       \\type node
       \\note Should match the field Source Inlet Node Name in water heater tank object.
       \\note Should also match the Condenser Water Outlet Node Name in the associated
       \\note DX Coil object.
  N2 , \\field Condenser Water Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autocalculatable
       \\note Actual water flow rate through the heat pump's water coil (condenser).
       \\note If autocalculated, the water flow rate is set equal to 4.487E-8 m3/s/W times
       \\note the rated heating capacity of the heat pump's DX coil.
  N3 , \\field Evaporator Air Flow Rate
       \\type real
       \\units m3/s
       \\minimum> 0
       \\autocalculatable
       \\note Actual air flow rate across the heat pump's air coil (evaporator).
       \\note If autocalculated, the air flow rate is set equal to 5.035E-5 m3/s/W times
       \\note the rated heating capacity of the heat pump's DX coil.
  A6 , \\field Inlet Air Configuration
       \\required-field
       \\type choice
       \\key Schedule
       \\key ZoneAirOnly
       \\key OutdoorAirOnly
       \\key ZoneAndOutdoorAir
       \\note Defines the configuration of the airflow path through the air coil and fan section.
  A7 , \\field Air Inlet Node Name
       \\type node
       \\note Zone air exhaust node name if Inlet Air Configuration is ZoneAirOnly or
       \\note ZoneAndOutdoorAir.
       \\note Simply a unique Node Name if Inlet Air Configuration is Schedule.
       \\note Otherwise, leave field blank.
  A8 , \\field Air Outlet Node Name
       \\type node
       \\note Zone Air Inlet Node Name if Inlet Air Configuration is ZoneAirOnly or
       \\note ZoneAndOutdoorAir.
       \\note Simply a unique Node Name if Inlet Air Configuration is Schedule.
       \\note Otherwise, leave field blank.
  A9 , \\field Outdoor Air Node Name
       \\type node
       \\note Outdoor air node name if inlet air configuration is ZoneAndOutdoorAir
       \\note or OutdoorAirOnly, otherwise leave field blank.
  A10, \\field Exhaust Air Node Name
       \\type node
       \\note Simply a unique Node Name if Inlet Air Configuration is ZoneAndOutdoorAir
       \\note or OutdoorAirOnly, otherwise leave field blank.
  A11, \\field Inlet Air Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Used only if Inlet Air Configuration is Schedule, otherwise leave blank.
       \\note Schedule values should be degrees C.
  A12, \\field Inlet Air Humidity Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Used only if Inlet Air Configuration is Schedule, otherwise leave blank.
       \\note Schedule values are entered as a fraction (e.g. 0.5 is equal to 50%RH)
  A13, \\field Inlet Air Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\note Used only if Inlet Air Configuration is ZoneAirOnly or ZoneAndOutdoorAir.
       \\note Otherwise, leave field blank.
  A14, \\field Tank Object Type
       \\required-field
       \\type choice
       \\key WaterHeater:Mixed
       \\key WaterHeater:Stratified
       \\default WaterHeater:Mixed
       \\note Specify the type of water heater tank used by this heat pump water heater.
  A15, \\field Tank Name
       \\required-field
       \\type object-list
       \\object-list WaterHeaterNames
       \\note Needs to match the name used in the corresponding Water Heater object.
  A16, \\field Tank Use Side Inlet Node Name
       \\type node
       \\note Used only when the heat pump water heater is connected to a plant loop,
       \\note otherwise leave blank. Needs to match the name used in the corresponding
       \\note Water Heater object when connected to a plant loop.
  A17, \\field Tank Use Side Outlet Node Name
       \\type node
       \\note Used only when the heat pump water heater is connected to a plant loop,
       \\note otherwise leave blank. Needs to match the name used in the corresponding
       \\note Water Heater object when connected to a plant loop.
  A18, \\field DX Coil Object Type
       \\required-field
       \\type choice
       \\key Coil:WaterHeating:AirToWaterHeatPump
       \\default Coil:WaterHeating:AirToWaterHeatPump
       \\note Specify the type of DX coil used by this heat pump water heater. The only
       \\note valid choice is Coil:WaterHeating:AirToWaterHeatPump.
  A19, \\field DX Coil Name
       \\required-field
       \\type object-list
       \\object-list HeatPumpWaterHeaterDXCoils
       \\note Must match the name used in the corresponding Coil:WaterHeating:AirToWaterHeatPump object.
  N4 , \\field Minimum Inlet Air Temperature for Compressor Operation
       \\type real
       \\units C
       \\minimum 5
       \\default 10
       \\note Heat pump compressor will not operate when the inlet air dry-bulb temperature
       \\note is below this value.
  A20, \\field Compressor Location
       \\required-field
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
       \\note If Zone is selected, Inlet Air Configuration must be ZoneAirOnly or
       \\note ZoneAndOutdoorAir. If Schedule is selected, then you must provide a
       \\note Compressor Ambient Temperature Schedule Name below.
  A21, \\field Compressor Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Used only if Compressor Location is Schedule, otherwise leave field blank.
  A22, \\field Fan Object Type
       \\required-field
       \\type choice
       \\key Fan:OnOff
       \\default Fan:OnOff
       \\note Specify the type of fan used by this heat pump water heater. The only
       \\note valid choice is Fan:OnOff.
  A23, \\field Fan Name
       \\required-field
       \\type object-list
       \\object-list FansOnOff
       \\note Needs to match the name used in the corresponding Fan:OnOff object.
  A24, \\field Fan Placement
       \\type choice
       \\key BlowThrough
       \\key DrawThrough
       \\default DrawThrough
       \\note BlowThrough means the fan is located before the air coil (upstream).
       \\note DrawThrough means the fan is located after the air coil (downstream).
  N5 , \\field On Cycle Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\note Parasitic electric power consumed when the heat pump compressor operates.
       \\note Does not contribute to water heating but can impact the zone air heat balance.
  N6 , \\field Off Cycle Parasitic Electric Load
       \\type real
       \\units W
       \\minimum 0.0
       \\default 0.0
       \\note Parasitic electric power consumed when the heat pump compressor is off.
       \\note Does not contribute to water heating but can impact the zone air heat balance.
       \\note Off-cycle parasitic power is 0 when the availability schedule is 0.
  A25, \\field Parasitic Heat Rejection Location
       \\type choice
       \\key Zone
       \\key Outdoors
       \\default Outdoors
       \\note This field determines if the parasitic electric load impacts the zone air
       \\note heat balance. If Zone is selected, Inlet Air Configuration must be
       \\note ZoneAirOnly or ZoneAndOutdoorAir.
  A26, \\field Inlet Air Mixer Node Name
       \\type node
       \\note Required only if Inlet Air Configuration is ZoneAndOutdoorAir, otherwise
       \\note leave field blank.
  A27, \\field Outlet Air Splitter Node Name
       \\type node
       \\note Required only if Inlet Air Configuration is ZoneAndOutdoorAir, otherwise
       \\note leave field blank.
  A28, \\field Inlet Air Mixer Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Required only if Inlet Air Configuration is ZoneAndOutdoorAir, otherwise
       \\note leave field blank. Schedule values define whether the heat pump draws its
       \\note inlet air from the zone, outdoors or a combination of zone and outdoor air.
       \\note A schedule value of 0 denotes inlet air is drawn only from the zone.
       \\note A schedule value of 1 denotes inlet air is drawn only from outdoors.
       \\note Schedule values between 0 and 1 denote a mixture of zone and outdoor air
       \\note proportional to the schedule value.
  A29; \\field Control Sensor Location In Stratified Tank
       \\note Used to indicate height of control sensor if Tank Object Type is WaterHeater:Stratified
       \\type choice
       \\key Heater1
       \\key Heater2
       \\key SourceInlet
       \\key SourceOutlet
       \\key UseInlet
       \\key UseOutlet
       \\default Heater1

ThermalStorage:Ice:Simple,
      \\min-fields 5
      \\memo This ice storage model is a simplified model
      \\memo It requires a setpoint placed on the Chilled Water Side Outlet Node
      \\memo It should be placed in the chilled water supply side outlet branch
      \\memo followed by a pipe.
      \\memo Use the PlantEquipmentOperation:ComponentSetpoint plant operation scheme.
  A1, \\field Name
      \\type alpha
      \\required-field
  A2, \\field Ice Storage Type
      \\note IceOnCoilInternal = Ice-on-Coil, internal melt
      \\note IceOnCoilExternal = Ice-on-Coil, external melt
      \\type choice
      \\required-field
      \\key IceOnCoilInternal
      \\key IceOnCoilExternal
  N1, \\field Capacity
      \\required-field
      \\type real
      \\units GJ
      \\ip-units ton-hrs
  A3, \\field Inlet Node Name
      \\type node
      \\required-field
  A4; \\field Outlet Node Name
      \\type node
      \\required-field

ThermalStorage:Ice:Detailed,
    \\min-fields 14
       \\memo This input syntax is intended to describe a thermal storage system that
       \\memo includes smaller containers filled with water that are placed in a larger
       \\memo tank or series of tanks.
       \\memo The model uses polynomial equations to describe the system performance.
  A1 , \\field Name
       \\required-field
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Capacity
       \\note This includes only the latent storage capacity
       \\units GJ
       \\required-field
  A3 , \\field Inlet Node Name
       \\required-field
       \\type node
  A4 , \\field Outlet Node Name
       \\required-field
       \\type node
  A5 , \\field Discharging Curve Object Type
       \\type choice
       \\key Curve:QuadraticLinear
  A6 , \\field Discharging Curve Name
       \\type alpha
       \\object-list QuadLinearCurves
  A7 , \\field Charging Curve Object Type
       \\type choice
       \\key Curve:QuadraticLinear
  A8 , \\field Charging Curve Name
       \\type alpha
       \\object-list QuadLinearCurves
  N2 , \\field Timestep of the Curve Data
       \\units hr
  N3 , \\field Parasitic Electric Load During Discharging
       \\units dimensionless
  N4 , \\field Parasitic Electric Load During Charging
       \\units dimensionless
  N5 , \\field Tank Loss Coefficient
       \\note This is the fraction the total storage capacity that is lost or melts
       \\note each hour
       \\units dimensionless
  N6 , \\field Freezing Temperature of Storage Medium
       \\note This temperature is typically 0C for water.
       \\note Simply changing this temperature without adjusting the performance
       \\note parameters input above is inappropriate and not advised.
       \\units C
       \\default 0.0
  A9 ; \\field Thaw Process Indicator
       \\note This field determines whether the system uses internal or external melt
       \\note during discharging.  This will then have an impact on charging performance.
       \\type choice
       \\key InsideMelt
       \\key OutsideMelt
       \\default OutsideMelt

ThermalStorage:ChilledWater:Mixed,
       \\memo Chilled water storage with a well-mixed, single-node tank. The chilled water is
       \\memo "used" by drawing from the "Use Side" of the water tank.  The tank is indirectly
       \\memo charged by circulating cold water through the "Source Side" of the water tank.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ChilledWaterStorageNames
  N1 , \\field Tank Volume
       \\type real
       \\units m3
       \\minimum> 0.0
       \\default 0.1
       \\ip-units gal
  A2 , \\field Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N2 , \\field Deadband Temperature Difference
       \\type real
       \\units deltaC
       \\minimum> 0.0
       \\default 0.5
  N3 , \\field Minimum Temperature Limit
       \\type real
       \\units C
  N4 , \\field Nominal Cooling Capacity
       \\type real
       \\units W
  A3 , \\field Ambient Temperature Indicator
       \\required-field
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
  A4 , \\field Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A5 , \\field Ambient Temperature Zone Name
       \\type object-list
       \\object-list ZoneNames
  A6 , \\field Ambient Temperature Outdoor Air Node Name
       \\type node
       \\note required when field Ambient Temperature Indicator=Outdoors
  N5 , \\field Heat Gain Coefficient from Ambient Temperature
       \\type real
       \\units W/K
       \\minimum 0.0
  A7 , \\field Use Side Inlet Node Name
       \\type node
  A8 , \\field Use Side Outlet Node Name
       \\type node
  N6 , \\field Use Side Heat Transfer Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A9 , \\field Use Side Availability Schedule Name
       \\note Availability schedule name for use side. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N7 , \\field Use Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  A10, \\field Source Side Inlet Node Name
       \\type node
  A11, \\field Source Side Outlet Node Name
       \\type node
  N8, \\field Source Side Heat Transfer Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
  A12, \\field Source Side Availability Schedule Name
       \\note Availability schedule name for source side. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N9 , \\field Source Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N10; \\field Tank Recovery Time
       \\type real
       \\default 4.0
       \\note Parameter for autosizing design flow rates for indirectly cooled water tanks
       \\note time required to lower temperature of entire tank from 14.4C to 9.0C
       \\units hr
       \\minimum> 0.0

ThermalStorage:ChilledWater:Stratified,
       \\memo Chilled water storage with astratified, multi-node tank. The chilled water is
       \\memo "used" by drawing from the "Use Side" of the water tank.  The tank is indirectly
       \\memo charged by circulating cold water through the "Source Side" of the water tank.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ChilledWaterStorageNames
  N1 , \\field Tank Volume
       \\required-field
       \\type real
       \\units m3
       \\minimum> 0.0
       \\ip-units gal
  N2 , \\field Tank Height
       \\required-field
       \\type real
       \\units m
       \\minimum> 0.0
       \\note Height is measured in the axial direction for horizontal cylinders
  A2 , \\field Tank Shape
       \\type choice
       \\key VerticalCylinder
       \\key HorizontalCylinder
       \\key Other
       \\default VerticalCylinder
  N3 , \\field Tank Perimeter
       \\type real
       \\units m
       \\minimum 0.0
       \\note Only used if Tank Shape is Other
  A3 , \\field Setpoint Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N4 , \\field Deadband Temperature Difference
       \\type real
       \\units deltaC
       \\minimum 0.0
       \\default 0.0
  N5,  \\field Temperature Sensor Height
       \\units m
       \\minimum 0.0
  N6 , \\field Minimum Temperature Limit
       \\type real
       \\units C
  N7 , \\field Nominal Cooling Capacity
       \\type real
       \\units W
  A4 , \\field Ambient Temperature Indicator
       \\required-field
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
  A5 , \\field Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A6 , \\field Ambient Temperature Zone Name
       \\type object-list
       \\object-list ZoneNames
  A7 , \\field Ambient Temperature Outdoor Air Node Name
       \\type node
       \\note required for Ambient Temperature Indicator=Outdoors
  N8, \\field Uniform Skin Loss Coefficient per Unit Area to Ambient Temperature
       \\type real
       \\units W/m2-K
       \\minimum 0.0
  A8 , \\field Use Side Inlet Node Name
       \\type node
  A9, \\field Use Side Outlet Node Name
       \\type node
  N9 , \\field Use Side Heat Transfer Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
       \\note The use side effectiveness in the stratified tank model is a simplified analogy of
       \\note a heat exchanger's effectiveness. This effectiveness is equal to the fraction of
       \\note use mass flow rate that directly mixes with the tank fluid. And one minus the
       \\note effectiveness is the fraction that bypasses the tank. The use side mass flow rate
       \\note that bypasses the tank is mixed with the fluid or water leaving the stratified tank.
  A10, \\field Use Side Availability Schedule Name
       \\note Availability schedule name for use side. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N10 , \\field Use Side Inlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\autocalculatable
       \\default autocalculate
       \\note Defaults to top of tank
  N11, \\field Use Side Outlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
       \\note Defaults to bottom of tank
  N12, \\field Use Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  A11, \\field Source Side Inlet Node Name
       \\type node
  A12, \\field Source Side Outlet Node Name
       \\type node
  N13, \\field Source Side Heat Transfer Effectiveness
       \\type real
       \\minimum 0.0
       \\maximum 1.0
       \\default 1.0
       \\note The source side effectiveness in the stratified tank model is a simplified analogy of
       \\note a heat exchanger's effectiveness. This effectiveness is equal to the fraction of
       \\note source mass flow rate that directly mixes with the tank fluid. And one minus the
       \\note effectiveness is the fraction that bypasses the tank. The source side mass flow rate
       \\note that bypasses the tank is mixed with the fluid or water leaving the stratified tank.
  A13, \\field Source Side Availability Schedule Name
       \\note Availability schedule name for use side. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  N14, \\field Source Side Inlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\default 0.0
       \\note Defaults to bottom of tank
  N15, \\field Source Side Outlet Height
       \\type real
       \\units m
       \\minimum 0.0
       \\autocalculatable
       \\default autocalculate
       \\note Defaults to top of tank
  N16, \\field Source Side Design Flow Rate
       \\type real
       \\autosizable
       \\default autosize
       \\units m3/s
       \\ip-units gal/min
       \\minimum 0.0
  N17, \\field Tank Recovery Time
       \\type real
       \\default 4.0
       \\note Parameter for autosizing design flow rates for indirectly cooled water tanks
       \\note time required to lower temperature of entire tank from 14.4C to 9.0C
       \\units hr
       \\minimum> 0.0
  A14, \\field Inlet Mode
       \\type choice
       \\key Fixed
       \\key Seeking
       \\default Fixed
  N18, \\field Number of Nodes
       \\type integer
       \\minimum 1
       \\maximum 10
       \\default 1
  N19, \\field Additional Destratification Conductivity
       \\type real
       \\units W/m-K
       \\minimum 0.0
       \\default 0.0
  N20, \\field Node 1 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N21, \\field Node 2 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N22, \\field Node 3 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N23, \\field Node 4 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N24, \\field Node 5 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N25, \\field Node 6 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N26, \\field Node 7 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N27, \\field Node 8 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N28, \\field Node 9 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0
  N29; \\field Node 10 Additional Loss Coefficient
       \\type real
       \\units W/m2-K
       \\default 0.0

\\group Plant-Condenser Loops
!*****************PLANT AND CONDENSER LOOP SPECIFICATION*********************
!Plant and condenser loops are specified as follows:

PlantLoop,
       \\memo Defines a central plant loop.
  A1 , \\field Name
        \\required-field
       \\reference PlantLoops
  A2 , \\field Fluid Type
        \\required-field
        \\type choice
        \\key Water
        \\key Steam
        \\key UserDefinedFluidType
        \\default Water
  A3 , \\field User Defined Fluid Type
       \\type object-list
       \\object-list FluidAndGlycolNames
       \\note This field is only required when Fluid Type is UserDefinedFluidType
  A4 , \\field Plant Equipment Operation Scheme Name
        \\required-field
        \\type object-list
        \\object-list PlantOperationSchemes
  A5 , \\field Loop Temperature Setpoint Node Name
        \\required-field
       \\type node
  N1 , \\field Maximum Loop Temperature
        \\required-field
        \\units C
  N2 , \\field Minimum Loop Temperature
        \\required-field
        \\units C
  N3 , \\field Maximum Loop Flow Rate
        \\required-field
        \\type real
        \\units m3/s
        \\autosizable
        \\minimum 0
        \\ip-units gal/min
  N4 , \\field Minimum Loop Flow Rate
        \\type real
        \\units m3/s
        \\default 0.0
        \\ip-units gal/min
  N5 , \\field Plant Loop Volume
        \\type real
        \\units m3
        \\autocalculatable
        \\minimum 0.0
        \\default Autocalculate
        \\ip-units gal
  A6,  \\field Plant Side Inlet Node Name
        \\required-field
        \\type node
  A7,  \\field Plant Side Outlet Node Name
        \\required-field
        \\type node
  A8,  \\field Plant Side Branch List Name
        \\required-field
        \\type object-list
        \\object-list BranchLists
  A9,  \\field Plant Side Connector List Name
        \\type object-list
        \\object-list ConnectorLists
  A10, \\field Demand Side Inlet Node Name
        \\required-field
        \\type node
  A11, \\field Demand Side Outlet Node Name
        \\required-field
        \\type node
  A12, \\field Demand Side Branch List Name
        \\required-field
        \\type object-list
        \\object-list BranchLists
  A13, \\field Demand Side Connector List Name
        \\type object-list
        \\object-list ConnectorLists
  A14, \\field Load Distribution Scheme
        \\type choice
        \\key Optimal
        \\key Sequential
        \\key Uniform
        \\default Sequential
  A15, \\field Availability Manager List Name
        \\type object-list
        \\object-list SystemAvailabilityManagerLists
  A16, \\field Plant Loop Demand Calculation Scheme
        \\type choice
        \\key SingleSetpoint
        \\key DualSetpointDeadband
        \\default SingleSetpoint
  A17,  \\field Common Pipe Simulation
        \\note Specifies a primary-secondary loop configuration. The plant side is the
        \\note primary loop, and the demand side is the secondary loop.
        \\note A secondary supply pump is required on the demand side.
        \\note None = Primary-only, no secondary simulation
        \\note CommonPipe = Primary-secondary with no temperature control at primary-secondary interface
        \\note TwoWayCommonPipe = Primary-secondary with control of secondary supply temperature or
        \\note primary return temperature (requires a setpoint be placed on the
        \\note plant side or demand side inlet node).
        \\type choice
        \\key CommonPipe
        \\key TwoWayCommonPipe
        \\key None
        \\default None
  A18;  \\field Pressure Simulation Type
        \\type choice
        \\key PumpPowerCorrection
        \\key LoopFlowCorrection
        \\key None
        \\default None

CondenserLoop,
       \\memo Defines a central plant condenser loop. CondenserLoop and PlantLoop are nearly
       \\memo identical except some components and operation schemes are applicable to only one
       \\memo loop type or the other.
  A1 , \\field Name
        \\required-field
        \\reference PlantLoops
  A2 , \\field Fluid Type
        \\required-field
        \\type choice
        \\key Water
        \\key UserDefinedFluidType
        \\default Water
  A3 , \\field User Defined Fluid Type
       \\type object-list
       \\object-list FluidAndGlycolNames
       \\note This field is only required when Fluid Type is UserDefinedFluidType
  A4 , \\field Condenser Equipment Operation Scheme Name
        \\required-field
        \\type object-list
        \\object-list CondenserOperationSchemes
  A5 , \\field Condenser Loop Temperature Setpoint Node Name
        \\required-field
        \\type node
  N1 , \\field Maximum Loop Temperature
        \\required-field
        \\units C
  N2 , \\field Minimum Loop Temperature
        \\required-field
        \\units C
  N3 , \\field Maximum Loop Flow Rate
        \\type real
        \\required-field
        \\units m3/s
        \\autosizable
        \\minimum> 0.0
        \\ip-units gal/min
  N4 , \\field Minimum Loop Flow Rate
        \\type real
        \\units m3/s
        \\default 0.0
        \\ip-units gal/min
  N5,  \\field Condenser Loop Volume
        \\type real
        \\units m3
        \\autocalculatable
        \\default Autocalculate
        \\minimum 0.0
        \\ip-units gal
  A6,  \\field Condenser Side Inlet Node Name
        \\required-field
        \\type node
  A7,  \\field Condenser Side Outlet Node Name
        \\required-field
        \\type node
  A8,  \\field Condenser Side Branch List Name
        \\required-field
        \\type object-list
        \\object-list BranchLists
  A9,  \\field Condenser Side Connector List Name
        \\required-field
        \\type object-list
        \\object-list ConnectorLists
  A10, \\field Demand Side Inlet Node Name
        \\required-field
        \\type node
  A11, \\field Demand Side Outlet Node Name
        \\required-field
        \\type node
  A12, \\field Condenser Demand Side Branch List Name
        \\required-field
        \\type object-list
        \\object-list BranchLists
  A13, \\field Condenser Demand Side Connector List Name
        \\required-field
        \\type object-list
        \\object-list ConnectorLists
  A14, \\field Load Distribution Scheme
        \\type choice
        \\key Optimal
        \\key Sequential
        \\default Sequential
  A15; \\field Pressure Simulation Type
        \\type choice
        \\key PumpPowerCorrection
        \\key LoopFlowCorrection
        \\key None
        \\default None

\\group Plant-Condenser Control

PlantEquipmentList,
    \\memo List plant equipment in order of operating priority, 1st in list will be used 1st, etc
    \\memo Use only plant equipment in this list.
    \\memo If no equipment object types and equipment names are specified, then the corresponding
    \\memo PlantEquipmentOperation:* object will assume all available plant equipment for the loop
    \\memo should be OFF (not operate) within the specified lower/upper limit.
    \\min-fields 1
  A1 , \\field Name
        \\required-field
        \\reference PlantAndCondenserEquipmentLists
        \\reference PlantEquipmentLists
  A2 , \\field Equipment 1 Object Type
  A3 , \\field Equipment 1 Name
  A4 , \\field Equipment 2 Object Type
  A5 , \\field Equipment 2 Name
  A6 , \\field Equipment 3 Object Type
  A7 , \\field Equipment 3 Name
  A8 , \\field Equipment 4 Object Type
  A9 , \\field Equipment 4 Name
  A10, \\field Equipment 5 Object Type
  A11, \\field Equipment 5 Name
  A12, \\field Equipment 6 Object Type
  A13, \\field Equipment 6 Name
  A14, \\field Equipment 7 Object Type
  A15, \\field Equipment 7 Name
  A16, \\field Equipment 8 Object Type
  A17, \\field Equipment 8 Name
  A18, \\field Equipment 9 Object Type
  A19, \\field Equipment 9 Name
  A20, \\field Equipment 10 Object Type
  A21; \\field Equipment 10 Name

CondenserEquipmentList,
    \\memo List condenser equipment in order of operating priority, 1st in list will be used 1st, etc
    \\memo Use only condenser equipment in this list.
    \\memo If no equipment object types and equipment names are specified, then the corresponding
    \\memo PlantEquipmentOperation:* object will assume all available condenser equipment for the loop
    \\memo should be OFF (not operate) within the specified lower/upper limit.
    \\min-fields 1
  A1 , \\field Name
        \\required-field
        \\reference PlantAndCondenserEquipmentLists
        \\reference CondenserEquipmentLists
  A2 , \\field Equipment 1 Object Type
  A3 , \\field Equipment 1 Name
  A4 , \\field Equipment 2 Object Type
  A5 , \\field Equipment 2 Name
  A6 , \\field Equipment 3 Object Type
  A7 , \\field Equipment 3 Name
  A8 , \\field Equipment 4 Object Type
  A9 , \\field Equipment 4 Name
  A10, \\field Equipment 5 Object Type
  A11, \\field Equipment 5 Name
  A12, \\field Equipment 6 Object Type
  A13, \\field Equipment 6 Name
  A14, \\field Equipment 7 Object Type
  A15, \\field Equipment 7 Name
  A16, \\field Equipment 8 Object Type
  A17, \\field Equipment 8 Name
  A18, \\field Equipment 9 Object Type
  A19, \\field Equipment 9 Name
  A20, \\field Equipment 10 Object Type
  A21; \\field Equipment 10 Name

!Plant Control Schemes:

PlantEquipmentOperation:Uncontrolled,
       \\memo Plant equipment operation scheme for uncontrolled operation. Specifies a group of
       \\memo equipment that runs if the loop is active, unless turned off by the loop flow resolver
       \\memo to maintain continuity in the fluid loop.
    \\min-fields 2
   A1 ,\\field Name
       \\required-field
       \\type alpha
       \\reference ControlSchemeList
   A2 ;\\field Equipment List Name
       \\required-field
       \\type object-list
       \\object-list PlantAndCondenserEquipmentLists

PlantEquipmentOperation:CoolingLoad,
       \\memo Plant equipment operation scheme for cooling load range operation. Specifies one or
       \\memo more groups of equipment which are available to operate for successive cooling load
       \\memo ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Load Range 1 Lower Limit
        \\required-field
       \\type real
       \\units W
       \\minimum 0.0
   N2 ,\\field Load Range 1 Upper Limit
        \\required-field
       \\type real
       \\units W
       \\minimum 0.0
   A2 ,\\field Range 1 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N3 ,\\field Load Range 2 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N4 ,\\field Load Range 2 Upper Limit
       \\type real
       \\units W
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N5 ,\\field Load Range 3 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N6 ,\\field Load Range 3 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N7 ,\\field Load Range 4 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N8 ,\\field Load Range 4 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N9 ,\\field Load Range 5 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N10,\\field Load Range 5 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N11,\\field Load Range 6 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N12,\\field Load Range 6 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N13,\\field Load Range 7 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N14,\\field Load Range 7 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N15,\\field Load Range 8 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N16,\\field Load Range 8 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N17,\\field Load Range 9 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N18,\\field Load Range 9 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N19,\\field Load Range 10 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N20,\\field Load Range 10 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists

PlantEquipmentOperation:HeatingLoad,
       \\memo Plant equipment operation scheme for heating load range operation. Specifies one or
       \\memo more groups of equipment which are available to operate for successive heating load
       \\memo ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Load Range 1 Lower Limit
        \\required-field
       \\type real
       \\units W
       \\minimum 0.0
   N2 ,\\field Load Range 1 Upper Limit
        \\required-field
       \\type real
       \\units W
       \\minimum 0.0
   A2 ,\\field Range 1 Equipment List Name
        \\required-field
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N3 ,\\field Load Range 2 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N4 ,\\field Load Range 2 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N5 ,\\field Load Range 3 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N6 ,\\field Load Range 3 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N7 ,\\field Load Range 4 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N8 ,\\field Load Range 4 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N9 ,\\field Load Range 5 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N10,\\field Load Range 5 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N11,\\field Load Range 6 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N12,\\field Load Range 6 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N13,\\field Load Range 7 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N14,\\field Load Range 7 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N15,\\field Load Range 8 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N16,\\field Load Range 8 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N17,\\field Load Range 9 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N18,\\field Load Range 9 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists
   N19,\\field Load Range 10 Lower Limit
       \\type real
       \\units W
       \\minimum 0.0
   N20,\\field Load Range 10 Upper Limit
       \\type real
       \\units W
       \\minimum 0.0
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list PlantAndCondenserEquipmentLists

PlantEquipmentOperation:OutdoorDryBulb,
       \\memo Plant equipment operation scheme for outdoor dry-bulb temperature range operation.
       \\memo Specifies one or more groups of equipment which are available to operate for
       \\memo successive outdoor dry-bulb temperature ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Dry-Bulb Temperature Range 1 Lower Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N2 ,\\field Dry-Bulb Temperature Range 1 Upper Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A2 ,\\field Range 1 Equipment List Name
        \\required-field
        \\type object-list
        \\object-list CondenserEquipmentLists
   N3 ,\\field Dry-Bulb Temperature Range 2 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N4 ,\\field Dry-Bulb Temperature Range 2 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N5 ,\\field Dry-Bulb Temperature Range 3 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N6 ,\\field Dry-Bulb Temperature Range 3 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N7 ,\\field Dry-Bulb Temperature Range 4 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N8 ,\\field Dry-Bulb Temperature Range 4 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N9 ,\\field Dry-Bulb Temperature Range 5 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N10,\\field Dry-Bulb Temperature Range 5 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N11,\\field Dry-Bulb Temperature Range 6 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N12,\\field Dry-Bulb Temperature Range 6 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N13,\\field Dry-Bulb Temperature Range 7 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N14,\\field Dry-Bulb Temperature Range 7 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N15,\\field Dry-Bulb Temperature Range 8 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N16,\\field Dry-Bulb Temperature Range 8 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N17,\\field Dry-Bulb Temperature Range 9 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N18,\\field Dry-Bulb Temperature Range 9 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N19,\\field Dry-Bulb Temperature Range 10 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N20,\\field Dry-Bulb Temperature Range 10 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists

PlantEquipmentOperation:OutdoorWetBulb,
       \\memo Plant equipment operation scheme for outdoor wet-bulb temperature range operation.
       \\memo Specifies one or more groups of equipment which are available to operate for
       \\memo successive outdoor wet-bulb temperature ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Wet-Bulb Temperature Range 1 Lower Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N2 ,\\field Wet-Bulb Temperature Range 1 Upper Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A2 ,\\field Range 1 Equipment List Name
        \\required-field
        \\type object-list
        \\object-list CondenserEquipmentLists
   N3 ,\\field Wet-Bulb Temperature Range 2 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N4 ,\\field Wet-Bulb Temperature Range 2 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N5 ,\\field Wet-Bulb Temperature Range 3 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N6 ,\\field Wet-Bulb Temperature Range 3 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N7 ,\\field Wet-Bulb Temperature Range 4 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N8 ,\\field Wet-Bulb Temperature Range 4 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N9 ,\\field Wet-Bulb Temperature Range 5 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N10,\\field Wet-Bulb Temperature Range 5 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N11,\\field Wet-Bulb Temperature Range 6 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N12,\\field Wet-Bulb Temperature Range 6 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N13,\\field Wet-Bulb Temperature Range 7 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N14,\\field Wet-Bulb Temperature Range 7 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N15,\\field Wet-Bulb Temperature Range 8 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N16,\\field Wet-Bulb Temperature Range 8 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N17,\\field Wet-Bulb Temperature Range 9 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N18,\\field Wet-Bulb Temperature Range 9 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N19,\\field Wet-Bulb Temperature Range 10 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N20,\\field Wet-Bulb Temperature Range 10 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists

PlantEquipmentOperation:OutdoorRelativeHumidity,
       \\memo Plant equipment operation scheme for outdoor relative humidity range operation.
       \\memo Specifies one or more groups of equipment which are available to operate for
       \\memo successive outdoor relative humidity ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Relative Humidity Range 1 Lower Limit
        \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N2 ,\\field Relative Humidity Range 1 Upper Limit
        \\required-field
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A2 ,\\field Range 1 Equipment List Name
        \\required-field
        \\type object-list
        \\object-list CondenserEquipmentLists
   N3 ,\\field Relative Humidity Range 2 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N4 ,\\field Relative Humidity Range 2 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N5 ,\\field Relative Humidity Range 3 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N6 ,\\field Relative Humidity Range 3 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N7 ,\\field Relative Humidity Range 4 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N8 ,\\field Relative Humidity Range 4 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N9 ,\\field Relative Humidity Range 5 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N10,\\field Relative Humidity Range 5 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N11,\\field Relative Humidity Range 6 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N12,\\field Relative Humidity Range 6 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N13,\\field Relative Humidity Range 7 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N14,\\field Relative Humidity Range 7 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N15,\\field Relative Humidity Range 8 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N16,\\field Relative Humidity Range 8 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N17,\\field Relative Humidity Range 9 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N18,\\field Relative Humidity Range 9 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N19,\\field Relative Humidity Range 10 Lower Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   N20,\\field Relative Humidity Range 10 Upper Limit
       \\type real
       \\units percent
       \\minimum 0.0
       \\maximum 100.0
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists

PlantEquipmentOperation:OutdoorDewpoint,
       \\memo Plant equipment operation scheme for outdoor dewpoint temperature range operation.
       \\memo Specifies one or more groups of equipment which are available to operate for
       \\memo successive outdoor dewpoint temperature ranges.
    \\min-fields 4
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   N1 ,\\field Dewpoint Temperature Range 1 Lower Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N2 ,\\field Dewpoint Temperature Range 1 Upper Limit
        \\required-field
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A2 ,\\field Range 1 Equipment List Name
        \\required-field
        \\type object-list
        \\object-list CondenserEquipmentLists
   N3 ,\\field Dewpoint Temperature Range 2 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N4 ,\\field Dewpoint Temperature Range 2 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A3 ,\\field Range 2 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N5 ,\\field Dewpoint Temperature Range 3 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N6 ,\\field Dewpoint Temperature Range 3 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A4 ,\\field Range 3 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N7 ,\\field Dewpoint Temperature Range 4 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N8 ,\\field Dewpoint Temperature Range 4 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A5 ,\\field Range 4 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N9 ,\\field Dewpoint Temperature Range 5 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N10,\\field Dewpoint Temperature Range 5 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A6 ,\\field Range 5 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N11,\\field Dewpoint Temperature Range 6 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N12,\\field Dewpoint Temperature Range 6 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A7 ,\\field Range 6 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N13,\\field Dewpoint Temperature Range 7 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N14,\\field Dewpoint Temperature Range 7 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A8 ,\\field Range 7 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N15,\\field Dewpoint Temperature Range 8 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N16,\\field Dewpoint Temperature Range 8 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A9 ,\\field Range 8 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N17,\\field Dewpoint Temperature Range 9 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N18,\\field Dewpoint Temperature Range 9 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A10,\\field Range 9 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists
   N19,\\field Dewpoint Temperature Range 10 Lower Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   N20,\\field Dewpoint Temperature Range 10 Upper Limit
       \\type real
       \\units C
       \\minimum -70
       \\maximum 70
   A11;\\field Range 10 Equipment List Name
        \\type object-list
        \\object-list CondenserEquipmentLists

PlantEquipmentOperation:ComponentSetpoint,
       \\memo Plant equipment operation scheme for component setpoint operation. Specifies one or
       \\memo pieces of equipment which are controlled to meet the temperature setpoint at the
       \\memo component outlet node.
    \\min-fields 7
   A1, \\field Name
       \\required-field
       \\reference ControlSchemeList
   A2, \\field Equipment 1 Object Type
       \\required-field
       \\type alpha
   A3, \\field Equipment 1 Name
       \\required-field
       \\type alpha
   A4, \\field Demand Calculation 1 Node Name
       \\required-field
       \\type node
   A5, \\field Setpoint 1 Node Name
       \\required-field
       \\type node
   N1, \\field Component 1 Flow Rate
       \\required-field
       \\type real
       \\units m3/s
       \\autosizable
   A6, \\field Operation 1 Type
       \\required-field
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A7, \\field Equipment 2 Object Type
       \\type alpha
   A8, \\field Equipment 2 Name
       \\type alpha
   A9, \\field Demand Calculation 2 Node Name
       \\type node
   A10, \\field Setpoint 2 Node Name
        \\type node
   N2, \\field Component 2 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A11,\\field Operation 2 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A12, \\field Equipment 3 Object Type
       \\type alpha
   A13, \\field Equipment 3 Name
       \\type alpha
   A14, \\field Demand Calculation 3 Node Name
        \\type node
   A15, \\field Setpoint 3 Node Name
        \\type node
   N3, \\field Component 3 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A16, \\field Operation 3 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A17, \\field Equipment 4 Object Type
       \\type alpha
   A18, \\field Equipment 4 Name
       \\type alpha
   A19,\\field Demand Calculation 4 Node Name
       \\type node
   A20,\\field Setpoint 4 Node Name
       \\type node
   N4, \\field Component 4 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A21, \\field Operation 4 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A22, \\field Equipment 5 Object Type
       \\type alpha
   A23, \\field Equipment 5 Name
       \\type alpha
   A24,\\field Demand Calculation 5 Node Name
       \\type node
   A25,\\field Setpoint 5 Node Name
       \\type node
   N5, \\field Component 5 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A26, \\field Operation 5 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A27, \\field Equipment 6 Object Type
       \\type alpha
   A28, \\field Equipment 6 Name
       \\type alpha
   A29,\\field Demand Calculation 6 Node Name
       \\type node
   A30,\\field Setpoint 6 Node Name
       \\type node
   N6, \\field Component 6 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A31, \\field Operation 6 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A32, \\field Equipment 7 Object Type
       \\type alpha
   A33, \\field Equipment 7 Name
       \\type alpha
   A34,\\field Demand Calculation 7 Node Name
       \\type node
   A35,\\field Setpoint 7 Node Name
       \\type node
   N7, \\field Component 7 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A36, \\field Operation 7 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A37, \\field Equipment 8 Object Type
       \\type alpha
   A38, \\field Equipment 8 Name
       \\type alpha
   A39,\\field Demand Calculation 8 Node Name
       \\type node
   A40,\\field Setpoint 8 Node Name
       \\type node
   N8, \\field Component 8 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A41, \\field Operation 8 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A42, \\field Equipment 9 Object Type
       \\type alpha
   A43, \\field Equipment 9 Name
       \\type alpha
   A44,\\field Demand Calculation 9 Node Name
       \\type node
   A45,\\field Setpoint 9 Node Name
       \\type node
   N9, \\field Component 9 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A46, \\field Operation 9 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual
   A47, \\field Equipment 10 Object Type
       \\type alpha
   A48, \\field Equipment 10 Name
       \\type alpha
   A49,\\field Demand Calculation 10 Node Name
       \\type node
   A50,\\field Setpoint 10 Node Name
       \\type node
   N10, \\field Component 10 Flow Rate
       \\type real
       \\units m3/s
       \\autosizable
   A51; \\field Operation 10 Type
       \\type choice
       \\key Heating
       \\key Cooling
       \\key Dual

PlantEquipmentOperation:OutdoorDryBulbDifference,
       \\memo Plant equipment operation scheme for outdoor dry-bulb temperature difference
       \\memo operation. Specifies one or more groups of equipment which are available to operate
       \\memo for successive ranges based the difference between a reference node temperature and
       \\memo the outdoor dry-bulb temperature.
    \\min-fields 5
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   A2 ,\\field Reference Temperature Node Name
       \\required-field
       \\type node
   N1,\\field Dry-Bulb Temperature Difference Range 1 Lower Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N2,\\field Dry-Bulb Temperature Difference Range 1 Upper Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A3 ,\\field Range 1 Equipment List Name
       \\required-field
       \\type object-list
       \\object-list CondenserEquipmentLists
   N3,\\field Dry-Bulb Temperature Difference Range 2 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N4,\\field Dry-Bulb Temperature Difference Range 2 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A4 ,\\field Range 2 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N5,\\field Dry-Bulb Temperature Difference Range 3 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N6,\\field Dry-Bulb Temperature Difference Range 3 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A5 ,\\field Range 3 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N7,\\field Dry-Bulb Temperature Difference Range 4 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N8,\\field Dry-Bulb Temperature Difference Range 4 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A6,\\field Range 4 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N9,\\field Dry-Bulb Temperature Difference Range 5 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N10,\\field Dry-Bulb Temperature Difference Range 5 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A7,\\field Range 5 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N11,\\field Dry-Bulb Temperature Difference Range 6 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N12,\\field Dry-Bulb Temperature Difference Range 6 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A8,\\field Range 6 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N13,\\field Dry-Bulb Temperature Difference Range 7 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N14,\\field Dry-Bulb Temperature Difference Range 7 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A9,\\field Range 7 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N15,\\field Dry-Bulb Temperature Difference Range 8 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N16,\\field Dry-Bulb Temperature Difference Range 8 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A10,\\field Range 8 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N17,\\field Dry-Bulb Temperature Difference Range 9 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N18,\\field Dry-Bulb Temperature Difference Range 9 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A11,\\field Range 9 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N19,\\field Dry-Bulb Temperature Difference Range 10 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N20,\\field Dry-Bulb Temperature Difference Range 10 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A12;\\field Range 10 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists

PlantEquipmentOperation:OutdoorWetBulbDifference,
       \\memo Plant equipment operation scheme for outdoor wet-bulb temperature difference
       \\memo operation. Specifies one or more groups of equipment which are available to operate
       \\memo for successive ranges based the difference between a reference node temperature and
       \\memo the outdoor wet-bulb temperature.
    \\min-fields 5
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   A2 ,\\field Reference Temperature Node Name
       \\required-field
       \\type node
   N1,\\field Wet-Bulb Temperature Difference Range 1 Lower Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N2,\\field Wet-Bulb Temperature Difference Range 1 Upper Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A3 ,\\field Range 1 Equipment List Name
       \\required-field
       \\type object-list
       \\object-list CondenserEquipmentLists
   N3,\\field Wet-Bulb Temperature Difference Range 2 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N4,\\field Wet-Bulb Temperature Difference Range 2 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A4 ,\\field Range 2 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N5,\\field Wet-Bulb Temperature Difference Range 3 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N6,\\field Wet-Bulb Temperature Difference Range 3 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A5 ,\\field Range 3 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N7,\\field Wet-Bulb Temperature Difference Range 4 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N8,\\field Wet-Bulb Temperature Difference Range 4 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A6, \\field Range 4 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N9,\\field Wet-Bulb Temperature Difference Range 5 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N10,\\field Wet-Bulb Temperature Difference Range 5 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A7, \\field Range 5 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N11,\\field Wet-Bulb Temperature Difference Range 6 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N12,\\field Wet-Bulb Temperature Difference Range 6 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A8, \\field Range 6 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N13,\\field Wet-Bulb Temperature Difference Range 7 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N14,\\field Wet-Bulb Temperature Difference Range 7 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A9, \\field Range 7 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N15,\\field Wet-Bulb Temperature Difference Range 8 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N16,\\field Wet-Bulb Temperature Difference Range 8 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A10,\\field Range 8 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N17,\\field Wet-Bulb Temperature Difference Range 9 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N18,\\field Wet-Bulb Temperature Difference Range 9 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A11,\\field Range 9 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N19,\\field Wet-Bulb Temperature Difference Range 10 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N20,\\field Wet-Bulb Temperature Difference Range 10 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A12;\\field Range 10 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists

PlantEquipmentOperation:OutdoorDewpointDifference,
       \\memo Plant equipment operation scheme for outdoor dewpoint temperature difference
       \\memo operation. Specifies one or more groups of equipment which are available to operate
       \\memo for successive ranges based the difference between a reference node temperature and
       \\memo the outdoor dewpoint temperature.
    \\min-fields 5
   A1 ,\\field Name
        \\required-field
        \\reference ControlSchemeList
   A2 ,\\field Reference Temperature Node Name
       \\required-field
       \\type node
   N1,\\field Dewpoint Temperature Difference Range 1 Lower Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N2,\\field Dewpoint Temperature Difference Range 1 Upper Limit
       \\required-field
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A3 ,\\field Range 1 Equipment List Name
       \\required-field
       \\type object-list
       \\object-list CondenserEquipmentLists
   N3,\\field Dewpoint Temperature Difference Range 2 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N4,\\field Dewpoint Temperature Difference Range 2 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A4 ,\\field Range 2 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N5,\\field Dewpoint Temperature Difference Range 3 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N6,\\field Dewpoint Temperature Difference Range 3 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A5 ,\\field Range 3 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N7,\\field Dewpoint Temperature Difference Range 4 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N8,\\field Dewpoint Temperature Difference Range 4 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A6, \\field Range 4 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N9,\\field Dewpoint Temperature Difference Range 5 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N10,\\field Dewpoint Temperature Difference Range 5 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A7, \\field Range 5 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N11,\\field Dewpoint Temperature Difference Range 6 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N12,\\field Dewpoint Temperature Difference Range 6 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A8, \\field Range 6 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N13,\\field Dewpoint Temperature Difference Range 7 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N14,\\field Dewpoint Temperature Difference Range 7 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A9, \\field Range 7 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N15,\\field Dewpoint Temperature Difference Range 8 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N16,\\field Dewpoint Temperature Difference Range 8 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A10,\\field Range 8 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N17,\\field Dewpoint Temperature Difference Range 9 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N18,\\field Dewpoint Temperature Difference Range 9 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A11,\\field Range 9 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists
   N19,\\field Dewpoint Temperature Difference Range 10 Lower Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   N20,\\field Dewpoint Temperature Difference Range 10 Upper Limit
       \\type real
       \\units deltaC
       \\minimum -50.0
       \\maximum 100.0
   A12;\\field Range 10 Equipment List Name
       \\type object-list
       \\object-list CondenserEquipmentLists

PlantEquipmentOperationSchemes,
       \\memo Operation schemes are listed in "priority" order.  Note that each scheme
       \\memo must address the entire load and/or condition ranges for the simulation.
       \\memo The actual one selected for use will be the first that is "Scheduled"
       \\memo on.  That is, if control scheme 1 is not "on" and control scheme 2
       \\memo is -- then control scheme 2 is selected.
       \\memo Only plant equipment should be listed on a Control Scheme for this item.
       \\min-fields 4
  A1 , \\field Name
        \\required-field
       \\reference PlantOperationSchemes
  A2 , \\field Control Scheme 1 Object Type
        \\required-field
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A3 , \\field Control Scheme 1 Name
        \\required-field
        \\type object-list
        \\object-list ControlSchemeList
  A4 , \\field Control Scheme 1 Schedule Name
        \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A5 , \\field Control Scheme 2 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A6 , \\field Control Scheme 2 Name
        \\type object-list
        \\object-list ControlSchemeList
  A7 , \\field Control Scheme 2 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A8 , \\field Control Scheme 3 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A9 , \\field Control Scheme 3 Name
        \\type object-list
        \\object-list ControlSchemeList
  A10, \\field Control Scheme 3 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A11, \\field Control Scheme 4 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A12, \\field Control Scheme 4 Name
        \\type object-list
        \\object-list ControlSchemeList
  A13, \\field Control Scheme 4 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A14, \\field Control Scheme 5 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A15, \\field Control Scheme 5 Name
        \\type object-list
        \\object-list ControlSchemeList
  A16, \\field Control Scheme 5 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A17, \\field Control Scheme 6 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A18, \\field Control Scheme 6 Name
        \\type object-list
        \\object-list ControlSchemeList
  A19, \\field Control Scheme 6 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A20, \\field Control Scheme 7 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A21, \\field Control Scheme 7 Name
        \\type object-list
        \\object-list ControlSchemeList
  A22, \\field Control Scheme 7 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A23, \\field Control Scheme 8 Object Type
       \\type choice
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:ComponentSetpoint
  A24, \\field Control Scheme 8 Name
        \\type object-list
        \\object-list ControlSchemeList
  A25; \\field Control Scheme 8 Schedule Name
       \\type object-list
       \\object-list ScheduleNames

CondenserEquipmentOperationSchemes,
       \\memo Operation schemes are listed in "priority" order.  Note that each scheme
       \\memo must address the entire load and/or condition ranges for the simulation.
       \\memo The actual one selected for use will be the first that is "Scheduled"
       \\memo on.  That is, if control scheme 1 is not "on" and control scheme 2
       \\memo is -- then control scheme 2 is selected.
       \\memo Only condenser equipment should be listed on a Control Scheme for this item.
       \\min-fields 4
  A1 , \\field Name
        \\required-field
       \\reference CondenserOperationSchemes
  A2 , \\field Control Scheme 1 Object Type
        \\required-field
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A3 , \\field Control Scheme 1 Name
        \\required-field
        \\type object-list
        \\object-list ControlSchemeList
  A4 , \\field Control Scheme 1 Schedule Name
        \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A5 , \\field Control Scheme 2 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A6 , \\field Control Scheme 2 Name
        \\type object-list
        \\object-list ControlSchemeList
  A7 , \\field Control Scheme 2 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A8 , \\field Control Scheme 3 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A9 , \\field Control Scheme 3 Name
        \\type object-list
        \\object-list ControlSchemeList
  A10, \\field Control Scheme 3 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A11, \\field Control Scheme 4 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A12, \\field Control Scheme 4 Name
        \\type object-list
        \\object-list ControlSchemeList
  A13, \\field Control Scheme 4 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A14, \\field Control Scheme 5 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A15, \\field Control Scheme 5 Name
        \\type object-list
        \\object-list ControlSchemeList
  A16, \\field Control Scheme 5 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A17, \\field Control Scheme 6 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A18, \\field Control Scheme 6 Name
        \\type object-list
        \\object-list ControlSchemeList
  A19, \\field Control Scheme 6 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A20, \\field Control Scheme 7 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A21, \\field Control Scheme 7 Name
        \\type object-list
        \\object-list ControlSchemeList
  A22, \\field Control Scheme 7 Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A23, \\field Control Scheme 8 Object Type
       \\type choice
       \\key PlantEquipmentOperation:Uncontrolled
       \\key PlantEquipmentOperation:CoolingLoad
       \\key PlantEquipmentOperation:HeatingLoad
       \\key PlantEquipmentOperation:OutdoorDryBulb
       \\key PlantEquipmentOperation:OutdoorWetBulb
       \\key PlantEquipmentOperation:OutdoorRelativeHumidity
       \\key PlantEquipmentOperation:OutdoorDewpoint
       \\key PlantEquipmentOperation:OutdoorDryBulbDifference
       \\key PlantEquipmentOperation:OutdoorWetBulbDifference
       \\key PlantEquipmentOperation:OutdoorDewpointDifference
  A24, \\field Control Scheme 8 Name
        \\type object-list
        \\object-list ControlSchemeList
  A25; \\field Control Scheme 8 Schedule Name
       \\type object-list
       \\object-list ScheduleNames

\\group Energy Management System (EMS)

EnergyManagementSystem:Sensor,
       \\memo Declares EMS variable as a sensor
       \\memo a list of output variables and meters that can be reported are available after a run on
       \\memo the report (.rdd) or meter dictionary file (.mdd) if the Output:VariableDictionary
       \\memo has been requested.
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Output:Variable or Output:Meter Index Key Name
       \\type alpha
  A3 ; \\field Output:Variable or Output:Meter Name
       \\required-field
       \\type external-list
       \\external-list autoRDDvariable
       \\external-list autoRDDmeter

EnergyManagementSystem:Actuator,
       \\memo Hardware portion of EMS used to set up actuators in the model
       \\min-fields 4
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Actuated Component Unique Name
       \\required-field
       \\type alpha
  A3 , \\field Actuated Component Type
       \\required-field
       \\type alpha
  A4 ; \\field Actuated Component Control Type
       \\required-field
       \\type alpha

EnergyManagementSystem:ProgramCallingManager,
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
       \\memo Input EMS program. a program needs a name
       \\memo a description of when it should be called
       \\memo and then lines of program code for EMS Runtime language
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ProgramNames
       \\note no spaces allowed in name
  A2 , \\field EnergyPlus Model Calling Point
       \\type choice
       \\key BeginNewEnvironment
       \\key AfterNewEnvironmentWarmUpIsComplete
       \\key BeginTimestepBeforePredictor
       \\key AfterPredictorBeforeHVACManagers
       \\key AfterPredictorAfterHVACManagers
       \\key InsideHVACSystemIterationLoop
       \\key EndOfZoneTimestepBeforeZoneReporting
       \\key EndOfZoneTimestepAfterZoneReporting
       \\key EndOfSystemTimestepBeforeHVACReporting
       \\key EndOfSystemTimestepAfterHVACReporting
       \\key EndOfZoneSizing
       \\key EndOfSystemSizing
       \\key AfterComponentInputReadIn
       \\key UserDefinedComponentModel
  A3 , \\field Program Name 1
       \\begin-extensible
       \\required-field
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A4 , \\field Program Name 2
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A5 , \\field Program Name 3
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A6 , \\field Program Name 4
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A7 , \\field Program Name 5
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A8 , \\field Program Name 6
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A9 , \\field Program Name 7
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A10, \\field Program Name 8
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A11, \\field Program Name 9
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A12, \\field Program Name 10
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A13, \\field Program Name 11
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A14, \\field Program Name 12
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A15, \\field Program Name 13
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A16, \\field Program Name 14
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A17, \\field Program Name 15
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A18, \\field Program Name 16
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A19, \\field Program Name 17
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A20, \\field Program Name 18
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A21, \\field Program Name 19
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A22, \\field Program Name 20
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A23, \\field Program Name 21
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A24, \\field Program Name 22
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A25, \\field Program Name 23
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A26, \\field Program Name 24
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames
  A27; \\field Program Name 25
       \\type object-list
       \\note no spaces allowed in name
       \\object-list ErlProgramNames

EnergyManagementSystem:Program,
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
       \\memo This input defines an Erl program
       \\memo Each field after the name is a line of EMS Runtime Language
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ErlProgramNames
       \\note no spaces allowed in name
  A2 , \\field Program Line 1
       \\begin-extensible
       \\type alpha
       \\required-field
  A3 , \\field Program Line 2
       \\type alpha
  A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15, A16, A17, A18, A19, \\note fields as indicated
  A20, A21, A22, A23, A24, A25, A26, A27, A28, A29, A30, A31, A32, A33, A34, \\note fields as indicated
  A35, A36, A37, A38, A39, A40, A41, A42, A43, A44, A45, A46, A47, A48, A49, \\note fields as indicated
  A50, A51, A52, A53, A54, A55, A56, A57, A58, A59, A60, A61, A62, A63, A64, \\note fields as indicated
  A65, A66, A67, A68, A69, A70, A71, A72, A73, A74, A75, A76, A77, A78, A79, \\note fields as indicated
  A80, A81, A82, A83, A84, A85, A86, A87, A88, A89, A90, A91, A92, A93, A94, \\note fields as indicated
  A95, A96, A97, A98, A99, A100, A101, A102, A103, A104, A105, A106, A107,   \\note fields as indicated
  A108, A109, A110, A111, A112, A113, A114, A115, A116, A117, A118, A119, A120; \\note fields as indicated

EnergyManagementSystem:Subroutine,
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
       \\memo This input defines an Erl program subroutine
       \\memo Each field after the name is a line of EMS Runtime Language
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ErlSubProgramNames
       \\note no spaces allowed in name
  A2 , \\field Program Line 1
       \\begin-extensible
       \\type alpha
  A3 , \\field Program Line 2
       \\note repeats
       \\type alpha
  A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15, A16, A17, A18, A19,  \\note fields as indicated
  A20, A21, A22, A23, A24, A25, A26, A27, A28, A29, A30, A31, A32, A33, A34, \\note fields as indicated
  A35, A36, A37, A38, A39, A40, A41, A42, A43, A44, A45, A46, A47, A48, A49, \\note fields as indicated
  A50, A51, A52, A53, A54, A55, A56, A57, A58, A59, A60, A61, A62, A63, A64, \\note fields as indicated
  A65, A66, A67, A68, A69, A70, A71, A72, A73, A74, A75, A76, A77, A78, A79, \\note fields as indicated
  A80, A81, A82, A83, A84, A85, A86, A87, A88, A89, A90, A91, A92, A93, A94, \\note fields as indicated
  A95, A96, A97, A98, A99, A100, A101, A102, A103, A104, A105, A106, A107,  \\note fields as indicated
  A108, A109, A110, A111, A112, A113, A114, A115, A116, A117, A118, A119, A120;  \\note fields as indicated

EnergyManagementSystem:GlobalVariable,
       \\extensible:1 - repeat last field, remembering to remove ; from "inner" fields.
       \\memo Declares Erl variable as having global scope
       \\memo No spaces allowed in names used for Erl variables
       \\min-fields 1
  A1 , \\field Erl Variable 1 Name
       \\begin-extensible
       \\required-field
       \\type alpha
       \\note no spaces allowed in name
  A2 , \\field Erl Variable 2 Name
       \\type alpha
  A3 , \\field Erl Variable 3 Name
       \\note repeats
       \\type alpha
  A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15, A16, A17, A18, A19;  \\note fields as indicated

EnergyManagementSystem:OutputVariable,
       \\memo This object sets up an EnergyPlus output variable from an Erl variable
       \\min-fields 4
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\retaincase
  A2 , \\field EMS Variable Name
       \\required-field
       \\type alpha
       \\note must be an acceptable EMS variable
  A3 , \\field Type of Data in Variable
       \\required-field
       \\type choice
       \\key Averaged
       \\key Summed
  A4 , \\field Update Frequency
       \\required-field
       \\type choice
       \\key ZoneTimestep
       \\key SystemTimestep
  A5 , \\field EMS Program or Subroutine Name
       \\type alpha
       \\note optional for global scope variables, required for local scope variables
  A6 ; \\field Units
       \\note optional but will result in dimensionless units for blank
       \\note EnergyPlus units are standard SI units
       \\type alpha

EnergyManagementSystem:MeteredOutputVariable,
       \\memo This object sets up an EnergyPlus output variable from an Erl variable
       \\min-fields 7
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\retaincase
  A2 , \\field EMS Variable Name
       \\required-field
       \\type alpha
       \\note must be an acceptable EMS variable, no spaces
  A3 , \\field Update Frequency
       \\required-field
       \\type choice
       \\key ZoneTimestep
       \\key SystemTimestep
  A4 , \\field EMS Program or Subroutine Name
       \\type alpha
       \\note optional for global scope variables, required for local scope variables
  A5 , \\field Resource Type
       \\required-field
       \\note choose the type of fuel, water, electricity, pollution or heat rate that should be metered.
       \\type choice
       \\key Electricity
       \\key NaturalGas
       \\key Gasoline
       \\key Diesel
       \\key Coal
       \\key FuelOil#1
       \\key FuelOil#2
       \\key Propane
       \\key OtherFuel1
       \\key OtherFuel2
       \\key WaterUse
       \\key OnSiteWaterProduced
       \\key MainsWaterSupply
       \\key RainWaterCollected
       \\key WellWaterDrawn
       \\key CondensateWaterCollected
       \\key EnergyTransfer
       \\key Steam
       \\key DistrictCooling
       \\key DistrictHeating
       \\key ElectricityProducedOnSite
       \\key SolarWaterHeating
       \\key SolarAirHeating
  A6 , \\field Group Type
       \\note choose a general classification, building (internal services), HVAC (air sytems), or plant (hydronic systems)
       \\type choice
       \\required-field
       \\key Building
       \\key HVAC
       \\key Plant
  A7 , \\field End-Use Category
       \\note choose how the metered output should be classified for end-use category
       \\required-field
       \\type choice
       \\key Heating
       \\key Cooling
       \\key InteriorLights
       \\key ExteriorLights
       \\key InteriorEquipment
       \\key ExteriorEquipment
       \\key Fans
       \\key Pumps
       \\key HeatRejection
       \\key Humidifier
       \\key HeatRecovery
       \\key WaterSystems
       \\key Refrigeration
       \\key OnSiteGeneration
  A8 , \\field End-Use Subcategory
       \\type alpha
       \\note enter a user-defined subcategory for this metered output
  A9 ; \\field Units
       \\note optional but will result in dimensionless units for blank
       \\note EnergyPlus units are standard SI units
       \\type alpha


EnergyManagementSystem:TrendVariable,
       \\memo This object sets up an EMS trend variable from an Erl variable
       \\memo A trend variable logs values across timesteps
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note no spaces allowed in name
  A2 , \\field EMS Variable Name
       \\required-field
       \\type alpha
       \\note must be a global scope EMS variable
  N1 ; \\field Number of Timesteps to be Logged
       \\required-field
       \\type integer
       \\minimum 1

EnergyManagementSystem:InternalVariable,
       \\memo Declares EMS variable as an internal data variable
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Internal Data Index Key Name
       \\type alpha
  A3 ; \\field Internal Data Type
       \\required-field
       \\type alpha

EnergyManagementSystem:CurveOrTableIndexVariable,
       \\memo Declares EMS variable that identifies a curve or table
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 ; \\field Curve or Table Object Name
       \\type object-list
       \\object-list AllCurves
       \\required-field

EnergyManagementSystem:ConstructionIndexVariable,
       \\memo Declares EMS variable that identifies a construction
       \\min-fields 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 ; \\field Construction Object Name
       \\type object-list
       \\object-list ConstructionNames
       \\required-field

\\group External Interface

ExternalInterface,
        \\memo This object activates the external interface of EnergyPlus. If the object
        \\memo ExternalInterface is present, then all ExtnernalInterface:* objects will receive
        \\memo their values from the BCVTB interface or from FMUs at each zone time step.
        \\memo If this object is not present, then the values of these objects will be fixed at the
        \\memo value declared in the "initial value" field of the corresponding object, and a
        \\memo warning will be written to the EnergyPlus error file.
   A1;  \\field Name of External Interface
        \\note Name of External Interface
        \\required-field
        \\type choice
        \\key PtolemyServer
        \\key FunctionalMockupUnitImport
        \\key FunctionalMockupUnitExport
        \\note Currently, the only valid entries are PtolemyServer, FunctionalMockupUnitImport, and FunctionalMockupUnitExport.

ExternalInterface:Schedule,
       \\min-fields 3
       \\memo A ExternalInterface:Schedule contains only one value,
       \\memo which is used during the warm-up period and the system sizing.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
       \\reference ExternalInterfaceScheduleNames
  A2 , \\field Schedule Type Limits Name
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during warm-up and system sizing.

ExternalInterface:Variable,
       \\memo This input object is similar to EnergyManagementSystem:GlobalVariable. However, at
       \\memo the beginning of each zone time step, its value is set to the value received from the
       \\memo external interface. During the warm-up period and the system sizing, its value
       \\memo is set to the value specified by the field "initial value." This object can be used
       \\memo to move data into Erl subroutines.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during warm-up and system sizing.

ExternalInterface:Actuator,
       \\memo Hardware portion of EMS used to set up actuators in the model
       \\min-fields 4
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Actuated Component Unique Name
       \\required-field
       \\type alpha
  A3 , \\field Actuated Component Type
       \\required-field
       \\type alpha
  A4 , \\field Actuated Component Control Type
       \\required-field
       \\type alpha
  N1 ; \\field Optional Initial Value
       \\type real
       \\note If specified, it is used during warm-up and system sizing.
       \\note If not specified, then the actuator only overwrites the
       \\note actuated component after the warm-up and system sizing.

ExternalInterface:FunctionalMockupUnitImport,
       \\memo This object declares an FMU
  A1 , \\field FMU File Name
       \\required-field
       \\type alpha
       \\retaincase
       \\reference FMUFilename
  N1 , \\field FMU Timeout
       \\note in milli-seconds
       \\type real
       \\required-field
       \\units ms
       \\default 0.0
  N2 ; \\field FMU LoggingOn
       \\type integer
       \\required-field
       \\default 0

ExternalInterface:FunctionalMockupUnitImport:From:Variable,
       \\memo This object declares an FMU input variable
       \\min-fields 5
  A1 , \\field Output:Variable Index Key Name
       \\required-field
       \\type alpha
  A2 , \\field Output:Variable Name
       \\required-field
       \\type external-list
       \\external-list autoRDDvariable
  A3 , \\field FMU File Name
       \\required-field
       \\retaincase
       \\type object-list
       \\object-list FMUFileName
  A4 , \\field FMU Instance Name
       \\required-field
       \\type alpha
       \\retaincase
  A5 ; \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase

ExternalInterface:FunctionalMockupUnitImport:To:Schedule,
       \\memo This objects contains only one value, which is used during the first
       \\memo call of EnergyPlus
       \\min-fields 6
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
       \\reference ExternalInterfaceScheduleNames
  A2 , \\field Schedule Type Limits Names
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field FMU File Name
       \\required-field
       \\retaincase
       \\type object-list
       \\object-list FMUFileName
  A4 , \\field FMU Instance Name
       \\required-field
       \\type alpha
       \\retaincase
  A5 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.

ExternalInterface:FunctionalMockupUnitImport:To:Actuator,
       \\memo Hardware portion of EMS used to set up actuators in the model
       \\memo that are dynamically updated from the FMU.
       \\min-fields 8
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a read-only variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Actuated Component Unique Name
       \\required-field
       \\type alpha
  A3 , \\field Actuated Component Type
       \\required-field
       \\type alpha
  A4 , \\field Actuated Component Control Type
       \\required-field
       \\type alpha
  A5 , \\field FMU File Name
       \\required-field
       \\retaincase
       \\type object-list
       \\object-list FMUFileName
  A6 , \\field FMU Instance Name
       \\required-field
       \\type alpha
       \\retaincase
  A7 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.


ExternalInterface:FunctionalMockupUnitImport:To:Variable,
       \\memo Declares Erl variable as having global scope
       \\memo No spaces allowed in names used for Erl variables
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field FMU File Name
       \\required-field
       \\retaincase
       \\type object-list
       \\object-list FMUFileName
  A3 , \\field FMU Instance Name
       \\required-field
       \\type alpha
       \\retaincase
  A4 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.

ExternalInterface:FunctionalMockupUnitExport:From:Variable,
       \\memo This object declares an FMU input variable
       \\min-fields 3
  A1 , \\field Output:Variable Index Key Name
       \\required-field
       \\type alpha
  A2 , \\field Output:Variable Name
       \\required-field
       \\type external-list
       \\external-list autoRDDvariable
  A3 ; \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase

ExternalInterface:FunctionalMockupUnitExport:To:Schedule,
       \\memo This objects contains only one value, which is used during the first
       \\memo call of EnergyPlus
       \\min-fields 4
  A1 , \\field Schedule Name
       \\required-field
       \\type alpha
       \\reference ScheduleNames
       \\reference ExternalInterfaceScheduleNames
  A2 , \\field Schedule Type Limits Names
       \\type object-list
       \\object-list ScheduleTypeLimitsNames
  A3 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.

ExternalInterface:FunctionalMockupUnitExport:To:Actuator,
       \\memo Hardware portion of EMS used to set up actuators in the model
       \\memo that are dynamically updated from the FMU.
       \\min-fields 6
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a read-only variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field Actuated Component Unique Name
       \\required-field
       \\type alpha
  A3 , \\field Actuated Component Type
       \\required-field
       \\type alpha
  A4 , \\field Actuated Component Control Type
       \\required-field
       \\type alpha
  A5 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.

ExternalInterface:FunctionalMockupUnitExport:To:Variable,
       \\memo Declares Erl variable as having global scope
       \\memo No spaces allowed in names used for Erl variables
       \\min-fields 3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\note This name becomes a variable for use in Erl programs
       \\note no spaces allowed in name
  A2 , \\field FMU Variable Name
       \\required-field
       \\type alpha
       \\retaincase
  N1 ; \\field Initial Value
       \\type real
       \\required-field
       \\note Used during the first call of EnergyPlus.

\\group User Defined HVAC and Plant Component Models

ZoneHVAC:ForcedAir:UserDefined,
        \\memo Defines a generic zone air unit for custom modeling
        \\memo using Energy Management System or External Interface
        \\min-fields 8
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note This is the name of the zone unit
   A2 , \\field Overall Model Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A3 , \\field Model Setup and Sizing Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A4 , \\field Primary Air Inlet Node Name
        \\required-field
        \\type node
        \\note Air inlet node for the unit must be a zone air exhaust Node.
   A5 , \\field Primary Air Outlet Node Name
        \\required-field
        \\type node
        \\note Air outlet node for the unit must be a zone air inlet node.
   A6,  \\field Secondary Air Inlet Node Name
        \\type node
        \\note Inlet air used for heat rejection or air source
   A7,  \\field Secondary Air Outlet Node Name
        \\type node
        \\note Outlet air used for heat rejection or air source
   N1 , \\field Number of Plant Loop Connections
        \\type integer
        \\minimum 0
        \\maximum 3
        \\required-field
   A8 , \\field Plant Connection 1 Inlet Node Name
        \\type node
   A9, \\field Plant Connection 1 Outlet Node Name
        \\type node
   A10, \\field Plant Connection 2 Inlet Node Name
        \\type node
   A11, \\field Plant Connection 2 Outlet Node Name
        \\type node
   A12, \\field Plant Connection 3 Inlet Node Name
        \\type node
   A13, \\field Plant Connection 3 Outlet Node Name
        \\type node
   A14, \\field Supply Inlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for alternate source of water consumed by device
   A15, \\field Collection Outlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for collection of condensate by device
   A16; \\field Ambient Zone Name
        \\type object-list
        \\object-list ZoneNames
        \\note Used for modeling device losses to surrounding zone

 AirTerminal:SingleDuct:UserDefined,
        \\memo Defines a generic single duct air terminal unit for custom modeling
        \\memo using Energy Management System or External Interface
        \\min-fields 8
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note This is the name of the air terminal
   A2 , \\field Overall Model Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A3 , \\field Model Setup and Sizing Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A4 , \\field Primary Air Inlet Node Name
        \\required-field
        \\type node
        \\note Air inlet node for the unit must be a zone splitter outlet.
   A5 , \\field Primary Air Outlet Node Name
        \\required-field
        \\type node
        \\note Air outlet node for the unit must be a zone air inlet node.
   A6,  \\field Secondary Air Inlet Node Name
        \\type node
        \\note Inlet air used for heat rejection or air source
   A7,  \\field Secondary Air Outlet Node Name
        \\type node
        \\note Outlet air used for heat rejection or air source
   N1,  \\field Number of Plant Loop Connections
        \\type integer
        \\minimum 0
        \\maximum 2
        \\required-field
   A8 , \\field Plant Connection 1 Inlet Node Name
        \\type node
        \\required-field
   A9 , \\field Plant Connection 1 Outlet Node Name
        \\type node
        \\required-field
   A10, \\field Plant Connection 2 Inlet Node Name
        \\type node
   A11, \\field Plant Connection 2 Outlet Node Name
        \\type node
   A12, \\field Supply Inlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for alternate source of water consumed by device
   A13, \\field Collection Outlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for collection of condensate by device
   A14; \\field Ambient Zone Name
        \\type object-list
        \\object-list ZoneNames
        \\note Used for modeling device losses to surrounding zone

 Coil:UserDefined,
        \\memo Defines a generic air system component for custom modeling
        \\memo using Energy Management System or External Interface
        \\min-fields 9
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note This is the name of the coil
   A2 , \\field Overall Model Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A3 , \\field Model Setup and Sizing Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
        \\required-field
   N1 , \\field Number of Air Connections
        \\type integer
        \\minimum 1
        \\maximum 2
        \\required-field
   A4 , \\field Air Connection 1 Inlet Node Name
        \\type node
        \\note Inlet air for primary air stream
        \\required-field
   A5 , \\field Air Connection 1 Outlet Node Name
        \\type node
        \\note Outlet air for primary air stream
        \\required-field
   A6 , \\field Air Connection 2 Inlet Node Name
        \\type node
        \\note Inlet air for secondary air stream
   A7 , \\field Air Connection 2 Outlet Node Name
        \\type node
        \\note Outlet air for secondary air stream
   A8 , \\field Plant Connection is Used
        \\type choice
        \\key Yes
        \\key No
   A9 , \\field Plant Connection Inlet Node Name
        \\type node
   A10 ,\\field Plant Connection Outlet Node Name
        \\type node
   A11, \\field Supply Inlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for alternate source of water consumed by device
   A12, \\field Collection Outlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for collection of condensate by device
   A13; \\field Ambient Zone Name
        \\type object-list
        \\object-list ZoneNames
        \\note Used for modeling device losses to surrounding zone

 PlantComponent:UserDefined,
        \\memo Defines a generic plant component for custom modeling
        \\memo using Energy Management System or External Interface
        \\min-fields 9
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note This is the name of the plant component
   A2 , \\field Main Model Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   N1 , \\field Number of Plant Loop Connections
        \\type integer
        \\minimum 1
        \\maximum 4
        \\required-field
   A3 , \\field Plant Connection 1 Inlet Node Name
        \\type node
        \\required-field
   A4 , \\field Plant Connection 1 Outlet Node Name
        \\type node
        \\required-field
   A5 , \\field Plant Connection 1 Loading Mode
        \\type choice
        \\key DemandsLoad
        \\key MeetsLoadWithPassiveCapacity
        \\key MeetsLoadWithNominalCapacity
        \\key MeetsLoadWithNominalCapacityLowOutLimit
        \\key MeetsLoadWithNominalCapacityHiOutLimit
        \\required-field
   A6 , \\field Plant Connection 1 Loop Flow Request Mode
        \\type Choice
        \\key NeedsFlowIfLoopOn
        \\key NeedsFlowAndTurnsLoopOn
        \\key ReceivesWhateverFlowAvailable
        \\required-field
   A7 , \\field Plant Connection 1 Initialization Program Calling Manager Name
        \\object-list ProgramNames
   A8  ,\\field Plant Connection 1 Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A9 , \\field Plant Connection 2 Inlet Node Name
        \\type node
   A10, \\field Plant Connection 2 Outlet Node Name
        \\type node
   A11, \\field Plant Connection 2 Loading Mode
        \\type choice
        \\key DemandsLoad
        \\key MeetLoadWithPassiveCapacity
        \\key MeetLoadWithNominalCapacity
        \\key MeetLoadWithNominalCapacityLowOutLimit
        \\key MeetLoadWithNominalCapacityHiOutLimit
   A12, \\field Plant Connection 2 Loop Flow Request Mode
        \\type Choice
        \\key NeedsFlowIfLoopOn
        \\key NeedsFlowAndTurnsLoopOn
        \\key ReceivesWhateverFlowAvailable
   A13, \\field Plant Connection 2 Initialization Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A14, \\field Plant Connection 2 Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A15, \\field Plant Connection 3 Inlet Node Name
        \\type node
   A16, \\field Plant Connection 3 Outlet Node Name
        \\type node
   A17, \\field Plant Connection 3 Loading Mode
        \\type choice
        \\key DemandsLoad
        \\key MeetLoadWithPassiveCapacity
        \\key MeetLoadWithNominalCapacity
        \\key MeetLoadWithNominalCapacityLowOutLimit
        \\key MeetLoadWithNominalCapacityHiOutLimit
   A18, \\field Plant Connection 3 Loop Flow Request Mode
        \\type Choice
        \\key NeedsFlowIfLoopOn
        \\key NeedsFlowAndTurnsLoopOn
        \\key ReceivesWhateverFlowAvailable
   A19, \\field Plant Connection 3 Initialization Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A20, \\field Plant Connection 3 Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A21, \\field Plant Connection 4 Inlet Node Name
        \\type node
   A22, \\field Plant Connection 4 Outlet Node Name
        \\type node
   A23, \\field Plant Connection 4 Loading Mode
        \\type choice
        \\key DemandsLoad
        \\key MeetLoadWithPassiveCapacity
        \\key MeetLoadWithNominalCapacity
        \\key MeetLoadWithNominalCapacityLowOutLimit
        \\key MeetLoadWithNominalCapacityHiOutLimit
   A24, \\field Plant Connection 4 Loop Flow Request Mode
        \\type Choice
        \\key NeedsFlowIfLoopOn
        \\key NeedsFlowAndTurnsLoopOn
        \\key ReceivesWhateverFlowAvailable
   A25, \\field Plant Connection 4 Initialization Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A26, \\field Plant Connection 4 Simulation Program Calling Manager Name
        \\type object-list
        \\object-list ProgramNames
   A27, \\field Air Connection Inlet Node Name
        \\type node
        \\note Inlet air used for heat rejection or air source
   A28, \\field Air Connection Outlet Node Name
        \\type node
        \\note Outlet air used for heat rejection or air source
   A29, \\field Supply Inlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for alternate source of water consumed by device
   A30, \\field Collection Outlet Water Storage Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note Water use storage tank for collection of condensate by device
   A31; \\field Ambient Zone Name
        \\type object-list
        \\object-list ZoneNames
        \\note Used for modeling device losses to surrounding zone


\\group System Availability Managers

AvailabilityManager:Scheduled,
       \\min-fields 2
       \\memo Determines the availability of a loop or system: whether it is on or off.
       \\memo Schedule overrides fan/pump schedule.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

AvailabilityManager:ScheduledOn,
       \\min-fields 2
       \\memo Determines the availability of a loop or system: only controls the turn on action.
       \\memo Schedule overrides fan/pump schedule.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

AvailabilityManager:ScheduledOff,
       \\min-fields 2
       \\memo Determines the availability of a loop or system: only controls the turn off action.
       \\memo Schedule overrides fan/pump schedule.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 ; \\field Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames

AvailabilityManager:NightCycle,
       \\min-fields 6
       \\memo Determines the availability of a loop or system: whether it is on or off.
       \\memo Depending on zone temperatures, overrides Schedules and forces system Fans on.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Applicability Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Fan Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Control Type
       \\type choice
       \\key StayOff
       \\key CycleOnAny
       \\key CycleOnControlZone
       \\key CycleOnAnyZoneFansOnly
       \\default StayOff
       \\note When AvailabilityManager:NightCycle is used in the zone component availability
       \\note manager assignment list, the key choices for Control Type would only be
       \\note StayOff and CycleOnControlZone
  N1 , \\field Thermostat Tolerance
       \\default 1.0
       \\units deltaC
  N2 , \\field Cycling Run Time
       \\default 3600.
       \\units s
  A5 ; \\field Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\note When AvailabilityManager:NightCycle is used in the zone component availability
       \\note manager assignment list, the Control Zone Name should be the name of the zone in which the
       \\note zone component is.

AvailabilityManager:DifferentialThermostat,
       \\memo Overrides fan/pump schedules depending on temperature difference between two nodes.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Hot Node Name
       \\required-field
       \\type node
  A3 , \\field Cold Node Name
       \\required-field
       \\type node
  N1 , \\field Temperature Difference On Limit
       \\required-field
       \\type real
       \\units deltaC
  N2 ; \\field Temperature Difference Off Limit
       \\type real
       \\units deltaC
       \\note Defaults to Temperature Difference On Limit.

AvailabilityManager:HighTemperatureTurnOff,
       \\memo Overrides fan/pump schedules depending on temperature at sensor node.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Sensor Node Name
       \\required-field
       \\type node
  N1 ; \\field Temperature
       \\required-field
       \\type real
       \\units C

AvailabilityManager:HighTemperatureTurnOn,
       \\memo Overrides fan/pump schedules depending on temperature at sensor node.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Sensor Node Name
       \\required-field
       \\type node
  N1 ; \\field Temperature
       \\required-field
       \\type real
       \\units C

AvailabilityManager:LowTemperatureTurnOff,
       \\memo Overrides fan/pump schedules depending on temperature at sensor node.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Sensor Node Name
       \\required-field
       \\type node
  N1 , \\field Temperature
       \\required-field
       \\type real
       \\units C
  A3 ; \\field Applicability Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note If blank, defaults to always active

AvailabilityManager:LowTemperatureTurnOn,
       \\memo Overrides fan/pump schedules depending on temperature at sensor node.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Sensor Node Name
       \\required-field
       \\type node
  N1 ; \\field Temperature
       \\required-field
       \\type real
       \\units C

AvailabilityManager:NightVentilation,
       \\min-fields 7
       \\memo depending on zone and outdoor conditions overides fan schedule to do
       \\memo precooling with outdoor air
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field Applicability Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Fan Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A4 , \\field Ventilation Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note One zone temperature must be above this scheduled temperature
       \\note for night ventilation to be enabled
  N1 , \\field Ventilation Temperature Difference
       \\units deltaC
       \\default 2.0
       \\note The outdoor air temperature minus the control zone temperature
       \\note must be greater than the ventilation delta T
  N2 , \\field Ventilation Temperature Low Limit
       \\units C
       \\default 15.
       \\note Night ventilation is disabled if any conditioned zone served by
       \\note the system falls below this temperature
  N3 , \\field Night Venting Flow Fraction
       \\minimum 0.0
       \\default 1.
       \\note the fraction (could be > 1) of the design system Flow Rate at which
       \\note night ventilation will be done
  A5 ; \\field Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\required-field
       \\note When AvailabilityManager:NightVentilation is used in the zone component availability
       \\note manager assignment list, the Control Zone Name should be the name of the zone in which the
       \\note zone component is.

AvailabilityManager:HybridVentilation,
       \\min-fields 13
       \\memo Depending on zone and outdoor conditions overrides window/door opening controls
       \\memo to maximize natural ventilation and turn off an HVAC system when ventilation control
       \\memo conditions are met.
       \\memo This object (zone ventilation object name) has not been instrumented to work with
       \\memo global Zone or Zone List names option for Ventilation:DesignFlowRate.  In order to
       \\memo use, you must enter the single <Ventilation:DesignFlowRate> name in that
       \\memo field. If it is a part of a global ventilation assignement the name will be
       \\memo <Zone Name> <global Ventilation:DesignFlowRate> name.
       \\memo Currently, hybrid ventilation manager is restricted to one per zone. It can either be applied
       \\memo through the air loop or directly to the zone. If hybrid ventilation manager is applied to an
       \\memo air loop and one of the zones served by that air loop also has hybrid ventilation manager,
       \\memo then zone hybrid ventilation manager is disabled.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SystemAvailabilityManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC or HVACTemplate:System:* object.
       \\type object-list
       \\object-list AirPrimaryLoops
       \\object-list HVACTemplateSystems
       \\note If this field is left blank, hybrid ventilation managers will be
       \\note simulated for zone equipment control
  A3 , \\field Controlled Zone Name
       \\required-field
       \\type object-list
       \\object-list ZoneNames
       \\note the controlled zone name should be a zone where a thermostat or humidistat is located
       \\note served by an air primary loop.
  A4 , \\field Ventilation Control Mode Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
       \\note The Ventilation control mode contains appropriate integer control types.
       \\note 0 - uncontrolled (Natural ventilation and HVAC system are controlled by themselves)
       \\note 1 = Temperature control
       \\note 2 = Enthalpy control
       \\note 3 = Dewpoint control
       \\note 4 = Outdoor ventilation air control
  A5 , \\field Use Weather File Rain Indicators
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\note If Yes, ventilation is shutoff when there is rain
       \\note If No, there is no rain control
  N1 , \\field Maximum Wind Speed
       \\type real
       \\units m/s
       \\minimum 0.0
       \\maximum 40.0
       \\default 40.0
       \\note this is the wind speed above which ventilation is shutoff
  N2 , \\field Minimum Outdoor Temperature
       \\note this is the outdoor temperature below which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
  N3 , \\field Maximum Outdoor Temperature
       \\note this is the outdoor temperature above which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
  N4 , \\field Minimum Outdoor Enthalpy
       \\note this is the outdoor Enthalpy below which ventilation is shutoff
       \\units J/kg
       \\type real
       \\minimum> 0.0
       \\maximum< 300000.0
  N5 , \\field Maximum Outdoor Enthalpy
       \\note this is the outdoor Enthalpy above which ventilation is shutoff
       \\units J/kg
       \\type real
       \\minimum> 0
       \\maximum< 300000.0
  N6 , \\field Minimum Outdoor Dewpoint
       \\note this is the outdoor temperature below which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default -100
       \\note Applicable only if Ventilation Control Mode = 3
  N7 , \\field Maximum Outdoor Dewpoint
       \\note this is the outdoor dewpoint above which ventilation is shutoff
       \\units C
       \\type real
       \\minimum -100
       \\maximum 100
       \\default 100
       \\note Applicable only if Ventilation Control Mode = 3
  A6 , \\field Minimum Outdoor Ventilation Air Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Used only if Ventilation Control Mode = 4
  A7 , \\field Opening Factor Function of Wind Speed Curve Name
       \\type object-list
       \\object-list LinearOrQuadraticCurves
       \\note linear curve = a + b*WS
       \\note quadratic curve = a + b*WS + c*WS**2
       \\note WS = wind speed (m/s)
  A8 , \\field AirflowNetwork Control Type Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note The schedule is used to incorporate operation of AirflowNetwork large opening
       \\note objects and HVAC system operation.
  A9 , \\field Simple Airflow Control Type Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note The schedule is used to incorporate operation of simple airflow objects and HVAC
       \\note system operation.
       \\note The simple airflow objects are Ventilation and Mixing only
  A10; \\field ZoneVentilation Object Name
       \\note This fieldhas not been instrumented to work with
       \\note global Zone or Zone List names option for Ventilation:DesignFlowRate.  In order to
       \\note use, you must enter the single <Ventilation:DesignFlowRate> name in this field.
       \\note If it is a part of a global ventilation assignement the name will be
       \\note <Zone Name> <global Ventilation:DesignFlowRate> name.
       \\type object-list
       \\object-list VentilationNames
       \\note The other ZoneVentilation:* and ZoneMixing objects controlled in the same AirLoopHVAC
       \\note will work in the same way as this ventilation object.

AvailabilityManagerAssignmentList,
       \\memo Defines the applicable managers used for an AirLoopHVAC or PlantLoop. The priority of
       \\memo availability managers is based on a set of rules and are specific to the type of loop.
       \\memo The output from each availability manager is an availability status flag:
       \\memo NoAction, ForceOff, CycleOn, or CycleOnZoneFansOnly (used only for air loops).
       \\min-fields 3
       \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference SystemAvailabilityManagerLists
  A2 , \\field Availability Manager 1 Object Type
       \\begin-extensible
       \\required-field
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A3 , \\field Availability Manager 1 Name
       \\required-field
       \\type object-list
       \\object-list SystemAvailabilityManagers
  A4 , \\field Availability Manager 2 Object Type
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A5 , \\field Availability Manager 2 Name
       \\type object-list
       \\object-list SystemAvailabilityManagers
  A6 , \\field Availability Manager 3 Object Type
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A7 , \\field Availability Manager 3 Name
       \\type object-list
       \\object-list SystemAvailabilityManagers
  A8 , \\field Availability Manager 4 Object Type
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A9 , \\field Availability Manager 4 Name
       \\type object-list
       \\object-list SystemAvailabilityManagers
  A10, \\field Availability Manager 5 Object Type
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A11, \\field Availability Manager 5 Name
       \\type object-list
       \\object-list SystemAvailabilityManagers
  A12, \\field Availability Manager 6 Object Type
       \\type choice
       \\key AvailabilityManager:Scheduled
       \\key AvailabilityManager:ScheduledOn
       \\key AvailabilityManager:ScheduledOff
       \\key AvailabilityManager:NightCycle
       \\key AvailabilityManager:DifferentialThermostat
       \\key AvailabilityManager:HighTemperatureTurnOff
       \\key AvailabilityManager:HighTemperatureTurnOn
       \\key AvailabilityManager:LowTemperatureTurnOff
       \\key AvailabilityManager:LowTemperatureTurnOn
       \\key AvailabilityManager:NightVentilation
  A13; \\field Availability Manager 6 Name
       \\type object-list
       \\object-list SystemAvailabilityManagers


\\group Setpoint Managers
! We have a two level hierarchy: Setpoint Managers and Controllers.  Set
! Point Managers are simulated at a high level - they resemble the function
! of an Energy Management System.  A Setpoint Manager can sense any node
! variables and use them to determine the setpoint of any other node variables.

SetpointManager:Scheduled,
     \\memo The simplest Setpoint Manager simply uses a schedule to determine one
     \\memo or more setpoints. Values of the nodes are not used as input.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\key MaximumTemperature
       \\key MinimumTemperature
       \\key HumidityRatio
       \\key MaximumHumidityRatio
       \\key MinimumHumidityRatio
       \\key MassFlowRate
       \\key MaximumMassFlowRate
       \\key MinimumMassFlowRate
       \\required-field
  A3 , \\field Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\required-field
  A4 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node


SetpointManager:Scheduled:DualSetpoint,
     \\memo This setpoint manager places a high and low schedule value
     \\memo on one or more nodes.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  A3 , \\field High Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\required-field
  A4 , \\field Low Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\required-field
  A5 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which temperature will be set
       \\required-field
       \\type node

SetpointManager:OutdoorAirReset,
     \\memo The Outdoor Air Reset Setpoint Manager sets the supply air
     \\memo temperature according to the outdoor air temperature using a reset rule.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  N1 , \\field Setpoint at Outdoor Low Temperature
       \\units C
       \\required-field
  N2 , \\field Outdoor Low Temperature
       \\units C
       \\required-field
  N3 , \\field Setpoint at Outdoor High Temperature
       \\units C
       \\required-field
  N4 , \\field Outdoor High Temperature
       \\units C
       \\required-field
  A3 , \\field Setpoint Node or NodeList Name
       \\note Node(s) at which temperature will be set
       \\required-field
       \\type node
  A4 , \\field Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Optional input.
       \\note Schedule allows scheduling of the outdoor air reset rule - a schedule value
       \\note of 1 means use the first rule; a value of 2 means use the second rule.
  N5 , \\field Setpoint at Outdoor Low Temperature 2
       \\note 2nd outdoor air temperature reset rule
       \\units C
  N6 , \\field Outdoor Low Temperature 2
       \\note 2nd outdoor air temperature reset rule
       \\units C
  N7 , \\field Setpoint at Outdoor High Temperature 2
       \\note 2nd outdoor air temperature reset rule
       \\units C
  N8 ; \\field Outdoor High Temperature 2
       \\note 2nd outdoor air temperature reset rule
       \\units C

SetpointManager:SingleZone:Reheat,
       \\memo This setpoint manager detects the control zone load, zone inlet node flow rate, and
       \\memo zone node temperature and calculates a setpoint temperature for the supply air that
       \\memo will satisfy the zone load (heating or cooling) for the control zone. This setpoint
       \\memo manager is not limited to reheat applications.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  N1 , \\field Minimum Supply Air Temperature
       \\units C
       \\default -99
  N2 , \\field Maximum Supply Air Temperature
       \\units C
       \\default 99
  A3 , \\field Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\required-field
  A4 , \\field Zone Node Name
       \\required-field
       \\type node
  A5 , \\field Zone Inlet Node Name
       \\required-field
       \\type node
  A6 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which the temperature will be set
       \\required-field
       \\type node

SetpointManager:SingleZone:Heating,
       \\memo This setpoint manager detects the control zone load to meet the current heating
       \\memo setpoint, zone inlet node flow rate, and zone node temperature, and calculates a
       \\memo setpoint temperature for the supply air that will satisfy the zone heating load for
       \\memo the control zone.
       \\min-fields 8
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  N1 , \\field Minimum Supply Air Temperature
       \\units C
       \\default -99
  N2 , \\field Maximum Supply Air Temperature
       \\units C
       \\default 99
  A3 , \\field Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\required-field
  A4 , \\field Zone Node Name
       \\required-field
       \\type node
  A5 , \\field Zone Inlet Node Name
       \\required-field
       \\type node
  A6 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which the temperature will be set
       \\required-field
       \\type node

SetpointManager:SingleZone:Cooling,
       \\memo This setpoint manager detects the control zone load to meet the current cooling
       \\memo setpoint, zone inlet node flow rate, and zone node temperature, and calculates a
       \\memo setpoint temperature for the supply air that will satisfy the zone cooling load for
       \\memo the control zone.
       \\min-fields 8
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  N1 , \\field Minimum Supply Air Temperature
       \\units C
       \\default -99
  N2 , \\field Maximum Supply Air Temperature
       \\units C
       \\default 99
  A3 , \\field Control Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\required-field
  A4 , \\field Zone Node Name
       \\required-field
       \\type node
  A5 , \\field Zone Inlet Node Name
       \\required-field
       \\type node
  A6 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which the temperature will be set
       \\required-field
       \\type node

SetpointManager:SingleZone:Humidity:Minimum,
     \\memo The Single Zone Minimum Humidity Setpoint Manager allows the
     \\memo control of a single zone minimum humidity level.
     \\memo This setpoint manager can be used in conjunction with
     \\memo object ZoneControl:Humidistat to detect humidity levels.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\deprecated
  A3 , \\field Schedule Name
       \\deprecated
  A4 , \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which humidity ratio setpoint will be set
  A5 ; \\field Control Zone Air Node Name
       \\required-field
       \\type node
       \\note Name of the zone air node for the humidity control zone

SetpointManager:SingleZone:Humidity:Maximum,
     \\memo The Single Zone Maximum Humidity Setpoint Manager allows the
     \\memo control of a single zone maximum humidity level.
     \\memo This setpoint manager can be used in conjunction with
     \\memo object ZoneControl:Humidistat to detect humidity levels.
       \\min-fields 5
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\deprecated
  A3 , \\field Schedule Name
       \\deprecated
  A4 , \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which humidity ratio setpoint will be set
  A5 ; \\field Control Zone Air Node Name
       \\required-field
       \\type node
       \\note Name of the zone air node for the humidity control zone

SetpointManager:MixedAir,
     \\memo The Mixed Air Setpoint Manager is meant to be used in conjunction
     \\memo with a Controller:OutdoorAir object. This setpoint manager is used
     \\memo to establish a temperature setpoint at the mixed air node.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  A3 , \\field Reference Setpoint Node Name
       \\required-field
       \\type node
  A4 , \\field Fan Inlet Node Name
       \\required-field
       \\type node
  A5 , \\field Fan Outlet Node Name
       \\required-field
       \\type node
  A6 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set

SetpointManager:OutdoorAirPretreat,
       \\min-fields 11
       \\memo This setpoint manager determines the required
       \\memo conditions at the outdoor air stream node which will
       \\memo produce the reference setpoint condition at the
       \\memo mixed air node when mixed with the return air stream
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\key HumidityRatio
       \\key MaximumHumidityRatio
       \\key MinimumHumidityRatio
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\default -99
       \\note Applicable only if Control variable is Temperature
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\default 99
       \\note Applicable only if Control variable is Temperature
  N3 , \\field Minimum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\maximum 1.0
       \\default 0.00001
       \\note Applicable only if Control variable is
       \\note MaximumHumidityRatio, MinimumHumidityRatio, or HumidityRatio - then minimum is 0.00001
  N4 , \\field Maximum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\maximum 1.0
       \\default 1.0
       \\note Applicable only if Control variable is
       \\note MaximumHumidityRatio, MinimumHumidityRatio, or HumidityRatio - then minimum is 0.00001
  A3 , \\field Reference Setpoint Node Name
       \\type node
       \\note The current setpoint at this node is the
       \\note desired condition for the Mixed Air Node
       \\note This node must have a valid setpoint
       \\note which has been set by another setpoint manager
  A4 , \\field Mixed Air Stream Node Name
       \\type node
       \\note Name of Mixed Air Node
       \\required-field
  A5 , \\field Outdoor Air Stream Node Name
       \\type node
       \\note Name of Outdoor Air Stream Node
       \\required-field
  A6 , \\field Return Air Stream Node Name
       \\type node
       \\note Name of Return Air Stream Node
       \\required-field
  A7 ; \\field Setpoint Node or NodeList Name
       \\type node
       \\required-field
       \\note Node(s) at which the temperature or humidity
       \\note ratio will be set

SetpointManager:Warmest,
     \\memo This SetpointManager resets the cooling supply air temperature
     \\memo of a central forced air HVAC system according to the
     \\memo cooling demand of the warmest zone.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  A3 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 12.
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 18.
  A4 , \\field Strategy
       \\type choice
       \\key MaximumTemperature
       \\default MaximumTemperature
  A5 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set

SetpointManager:Coldest,
     \\memo This SetpointManager is used in dual duct systems to reset
     \\memo the setpoint temperature of the air in the heating supply duct.
     \\memo Usually it is used in conjunction with a SetpointManager:Warmest
     \\memo resetting the temperature of the air in the cooling supply duct.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\default Temperature
  A3 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object.
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 20.
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 50.
  A4 , \\field Strategy
       \\type choice
       \\key MinimumTemperature
       \\default MinimumTemperature
  A5 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set

SetpointManager:ReturnAirBypassFlow,
       \\min-fields 4
       \\memo This setpoint manager determines the required
       \\memo mass flow rate through a return air bypass duct
       \\memo to meet the specified temperature setpoint
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Flow
       \\default Flow
  A3 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object.
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  A4 ; \\field Temperature Setpoint Schedule Name
       \\type object-list
       \\object-list ScheduleNames

SetpointManager:WarmestTemperatureFlow,
       \\memo This setpoint manager sets both the supply air temperature
       \\memo and the supply air flow rate.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
  A3 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object.
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 12.
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 18.
  A4 , \\field Strategy
       \\note For TemperatureFirst the manager tries to find the highest setpoint temperature
       \\note that will satisfy all the zone cooling loads at minimum supply air flow rate.
       \\note If this setpoint temperature is less than the minimum, the setpoint temperature is set
       \\note to the minimum, and the supply air flow rate is increased to meet the loads.
       \\note For FlowFirst the manager tries to find the lowest supply air flow rate
       \\note that will satisfy all the zone cooling loads at the maximum setpoint temperature.
       \\note If this flow is greater than the maximum, the flow is set to the maximum and the
       \\note setpoint temperature is reduced to satisfy the cooling loads.
       \\type choice
       \\key TemperatureFirst
       \\key FlowFirst
       \\default TemperatureFirst
  A5 , \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set
  N3 ; \\field Minimum Turndown Ratio
       \\note Fraction of the maximum supply air flow rate.
       \\note Used to define the minimum supply flow for the TemperatureFirst strategy.
       \\units dimensionless
       \\type real
       \\minimum> 0.0
       \\default 0.2

SetpointManager:MultiZone:Heating:Average,
       \\min-fields 5
       \\memo This setpoint manager sets the average supply air temperature based on the heating load
       \\memo requirements of all controlled zones in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 20.
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 50.
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set

SetpointManager:MultiZone:Cooling:Average,
       \\min-fields 5
       \\memo This setpoint manager sets the average supply air temperature based on the cooling load
       \\memo requirements of all controlled zones in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 12.
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
       \\minimum> 0.0
       \\default 18.
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the temperature will be set

SetpointManager:MultiZone:MinimumHumidity:Average,
       \\min-fields 5
       \\memo This setpoint manager sets the average supply air minimum humidity ratio based on moisture
       \\memo load requirements of all controlled zones in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.005
  N2 , \\field Maximum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.012
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the humidity ratio will be set

SetpointManager:MultiZone:MaximumHumidity:Average,
       \\min-fields 5
       \\memo This setpoint manager sets the average supply air maximum humidity ratio based on moisture
       \\memo load requirements of all controlled zones in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.008
  N2 , \\field Maximum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.015
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the humidity ratio will be set

SetpointManager:MultiZone:Humidity:Minimum,
       \\min-fields 5
       \\memo This setpoint manager sets the minimum supply air humidity ratio based on humidification
       \\memo requirements of a controlled zone with critical humidity ratio setpoint (i.e., a zone with
       \\memo the highest humidity ratio setpoint) in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.005
  N2 , \\field Maximum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.012
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the humidity ratio will be set

SetpointManager:MultiZone:Humidity:Maximum,
       \\min-fields 5
       \\memo This setpoint manager sets the maximum supply air humidity ratio based on dehumidification
       \\memo requirements of a controlled zone with critical humidity ratio setpoint (i.e., a zone with
       \\memo the lowest humidity ratio setpoint) in an air loop served by a central air-conditioner.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field HVAC Air Loop Name
       \\note Enter the name of an AirLoopHVAC object
       \\required-field
       \\type object-list
       \\object-list AirPrimaryLoops
  N1 , \\field Minimum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.008
  N2 , \\field Maximum Setpoint Humidity Ratio
       \\units kgWater/kgDryAir
       \\type real
       \\minimum> 0.0
       \\default 0.015
  A3 ; \\field Setpoint Node or NodeList Name
       \\required-field
       \\type node
       \\note Node(s) at which the humidity ratio will be set

SetpointManager:FollowOutdoorAirTemperature,
     \\memo This setpoint manager is used to place a temperature setpoint on a system node
     \\memo that is derived from the current outdoor air environmental conditions.
     \\memo The outdoor air conditions are obtained from the weather information during the simulation.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\key MinimumTemperature
       \\key MaximumTemperature
       \\default Temperature
  A3 , \\field Reference Temperature Type
       \\type choice
       \\key OutdoorAirWetBulb
       \\key OutdoorAirDryBulb
       \\default OutdoorAirWetBulb
  N1 , \\field Offset Temperature Difference
       \\units deltaC
       \\type real
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
  N3 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
  A4 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node

SetpointManager:FollowSystemNodeTemperature,
     \\memo This setpoint manager is used to place a temperature setpoint on a
     \\memo system node that is derived from the current temperatures at a separate
     \\memo system node.  The current value of the temperature at a reference node
     \\memo is obtained and used to generate setpoint on a second system node.
     \\memo If the reference node is also designated to be an outdoor air (intake) node,
     \\memo then this setpoint manager can be used to follow outdoor air conditions
     \\memo that are adjusted for altitude.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\key MinimumTemperature
       \\key MaximumTemperature
       \\default Temperature
  A3,  \\field Reference Node Name
       \\type node
  A4 , \\field Reference Temperature Type
       \\type choice
       \\key NodeWetBulb
       \\key NodeDryBulb
       \\default NodeDryBulb
  N1 , \\field Offset Temperature Difference
       \\units deltaC
       \\type real
  N2 , \\field Maximum Limit Setpoint Temperature
       \\units C
       \\type real
  N3 , \\field Minimum Limit Setpoint Temperature
       \\units C
       \\type real
  A5 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node

SetpointManager:FollowGroundTemperature,
     \\memo This setpoint manager is used to place a temperature setpoint on a
     \\memo system node that is derived from a current ground temperature.
     \\memo The ground temperatures are specified in different
     \\memo Site:GroundTemperature:* objects and used during the simulation.
     \\memo This setpoint manager is primarily intended for condenser or plant loops
     \\memo using some type of ground heat exchanger.
  A1 , \\field Name
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\key Temperature
       \\key MinimumTemperature
       \\key MaximumTemperature
       \\default Temperature
  A3 , \\field Reference Ground Temperature Object Type
       \\type choice
       \\key Site:GroundTemperature:BuildingSurface
       \\key Site:GroundTemperature:Shallow
       \\key Site:GroundTemperature:Deep
       \\key Site:GroundTemperature:FCfactorMethod
  N1 , \\field Offset Temperature Difference
       \\units deltaC
       \\type real
  N2 , \\field Maximum Setpoint Temperature
       \\units C
       \\type real
  N3 , \\field Minimum Setpoint Temperature
       \\units C
       \\type real
  A4 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node

SetpointManager:CondenserEnteringReset,
       \\min-fields 10
       \\memo This setpoint manager uses one curve to determine the optimum condenser entering water temperature
       \\memo for a given timestep and two other curves to place boundary conditions on the setpoint value.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\required-field
       \\key Temperature
       \\default Temperature
  A3 , \\field Default Condenser Entering Water Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\required-field
       \\note This scheduled setpoint value is only used in a given timestep if the
       \\note "Optimized" Condenser Entering Temperature does not fall within the prescribed
       \\note boundary conditions.
  A4 , \\field Minimum Design Wetbulb Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadLinearCurves
  A5 , \\field Minimum Outside Air Wetbulb Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadLinearCurves
  A6 , \\field Optimized Cond Entering Water Temperature Curve Name
       \\required-field
       \\type object-list
       \\object-list QuadLinearCurves
  N1 , \\field Minimum Lift
       \\type real
       \\units deltaC
       \\default 11.1
  N2 , \\field Maximum Condenser Entering Water Temperature
       \\type real
       \\units C
       \\default 32
  N3 , \\field Cooling Tower Design Inlet Air Wet-Bulb Temperature
       \\type real
       \\units C
       \\default 25.56
  A7 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node

SetpointManager:CondenserEnteringReset:Ideal,
       \\min-fields 5
       \\memo This setpoint manager determine the ideal optimum condenser entering water temperature
       \\memo setpoint for a given timestep.
  A1 , \\field Name
       \\type alpha
       \\required-field
       \\reference SetpointManagers
  A2 , \\field Control Variable
       \\type choice
       \\required-field
       \\key Temperature
       \\default Temperature
  N1 , \\field Minimum Lift
       \\type real
       \\units deltaC
       \\default 11.1
  N2 , \\field Maximum Condenser Entering Water Temperature
       \\type real
       \\units C
       \\default 32
  A3 ; \\field Setpoint Node or NodeList Name
       \\note Node(s) at which control variable will be set
       \\required-field
       \\type node


\\group Refrigeration

Refrigeration:Case,
        \\memo The Refrigeration Case object works in conjunction with a compressor rack, a
        \\memo refrigeration system, or a secondary loop to simulate the performance of a
        \\memo refrigerated case system. The object calculates the energy use for lights, fans and
        \\memo anti-sweat heaters and accounts for the sensible and latent heat exchange with the
        \\memo surrounding environment (termed "case credits") which impacts the temperature
        \\memo and humidity in the zone where the case is located.
        \\min-fields 28
  A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCaseAndWalkInAndListNames
        \\reference RefrigerationCaseAndWalkInNames
  A2 ,  \\field Availability Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
  A3 ,  \\field Zone Name
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\required-field
        \\type object-list
        \\object-list ZoneNames
  N1 ,  \\field Rated Ambient Temperature
        \\type real
        \\units C
        \\default 23.9
        \\minimum> 0.0
  N2 ,  \\field Rated Ambient Relative Humidity
        \\type real
        \\units percent
        \\default 55.0
        \\minimum> 0.0
        \\maximum< 100.0
  N3 ,  \\field Rated Total Cooling Capacity per Unit Length
        \\type real
        \\units W/m
        \\default 1900
        \\minimum> 0.0
  N4 ,  \\field Rated Latent Heat Ratio
        \\type real
        \\default 0.3
        \\minimum 0.0
        \\maximum 1.0
  N5 ,  \\field Rated Runtime Fraction
        \\type real
        \\default 0.85
        \\minimum> 0.0
        \\maximum 1.0
  N6 ,  \\field Case Length
        \\type real
        \\units m
        \\default 3.0
        \\minimum> 0.0
  N7,   \\field Case Operating Temperature
        \\type real
        \\units C
        \\default 1.1
        \\maximum< 20.0
  A4 ,  \\field Latent Case Credit Curve Type
        \\type choice
        \\key CaseTemperatureMethod
        \\key RelativeHumidityMethod
        \\key DewpointMethod
        \\default CaseTemperatureMethod
  A5 ,  \\field Latent Case Credit Curve Name
        \\required-field
        \\type object-list
        \\object-list CubicCurves
  N8 ,  \\field Standard Case Fan Power per Unit Length
        \\type real
        \\units W/m
        \\default 75.0
        \\minimum 0.0
  N9 ,  \\field Operating Case Fan Power per Unit Length
        \\type real
        \\units W/m
        \\default 75.0
        \\minimum 0.0
  N10,  \\field Standard Case Lighting Power per Unit Length
        \\type real
        \\units W/m
        \\default 90.0
  N11,  \\field Installed Case Lighting Power per Unit Length
        \\type real
        \\units W/m
        \\note default set equal to Standard Case Lighting Power per Unit Length
  A6 ,  \\field Case Lighting Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N12,  \\field Fraction of Lighting Energy to Case
        \\type real
        \\default 1.0
        \\minimum 0.0
        \\maximum 1.0
  N13,  \\field Case Anti-Sweat Heater Power per Unit Length
        \\type real
        \\units W/m
        \\minimum 0
        \\default 0
  N14,  \\field Minimum Anti-Sweat Heater Power per Unit Length
        \\type real
        \\units W/m
        \\minimum 0
        \\default 0
        \\note This field is only applicable to the Linear, Dewpoint Method, and
        \\note Heat Balance Method anti-sweat heater control types
  A7 ,  \\field Anti-Sweat Heater Control Type
        \\type choice
        \\key None
        \\key Constant
        \\key Linear
        \\key DewpointMethod
        \\key HeatBalanceMethod
        \\default None
  N15,  \\field Humidity at Zero Anti-Sweat Heater Energy
        \\type real
        \\units percent
        \\default -10.0
        \\note This field is only applicable to Linear AS heater control type
        \\note Zone relative humidity (%) where anti-sweat heater energy is zero
  N16,  \\field Case Height
        \\type real
        \\units m
        \\minimum 0
        \\default 1.5
        \\note This field only applicable to Heat Balance Method AS heater control type
        \\note Height must be greater than zero if Heat Balance Method AS heater control is selected
  N17,  \\field Fraction of Anti-Sweat Heater Energy to Case
        \\type real
        \\default 1.0
        \\minimum 0.0
        \\maximum 1.0
  N18,  \\field Case Defrost Power per Unit Length
        \\type real
        \\units W/m
        \\note Used to evaluate load on case as well as power or heat consumption
        \\default 0.0
        \\minimum 0.0
  A8,   \\field Case Defrost Type
        \\type choice
        \\key None
        \\key OffCycle
        \\key HotGas
        \\key Electric
        \\key HotFluid
        \\key HotGasWithTemperatureTermination
        \\key ElectricWithTemperatureTermination
        \\key HotFluidWithTemperatureTermination
        \\default OffCycle
  A9,   \\field Case Defrost Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note A case defrost schedule name is required unless case defrost type = None
  A10,  \\field Case Defrost Drip-Down Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note If left blank, the defrost schedule will be used
        \\note The start time for each defrost period in this drip-down schedule should coincide with
        \\note the start time for each defrost period in the case defrost schedule (previous input
        \\note field).The length of each defrost drip-down period must be greater than or equal to the
        \\note corresponding defrost period specified in the case defrost schedule. This extra time
        \\note allows the melted frost to drip from the coil before refrigeration is restarted.
  A11,  \\field Defrost Energy Correction Curve Type
        \\type choice
        \\key None
        \\key CaseTemperatureMethod
        \\key RelativeHumidityMethod
        \\key DewpointMethod
        \\default None
        \\note Case Temperature, Relative Humidity, and Dewpoint Method are applicable to case defrost
        \\note types with temperature termination only.
  A12,  \\field Defrost Energy Correction Curve Name
        \\type object-list
        \\object-list CubicCurves
        \\note Defrost Energy Correction Curve Name is applicable to case defrost types
        \\note with temperature termination only.
  N19,  \\field Under Case HVAC Return Air Fraction
        \\type real
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.0
  A13,  \\field Refrigerated Case Restocking Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should be in units of Watts per unit case length (W/m)
        \\note Leave this field blank if no restocking is to be modeled
  A14,  \\field Case Credit Fraction Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should be from 0 to 1
        \\note Leave this field blank if no case credit fraction is to be applied
  N20,  \\field Design Evaporator Temperature or Brine Inlet Temperature
        \\note Required for detailed refrigeration system, not for compressor rack
        \\note For a DX system, enter the saturated temperature for refrigerant pressure leaving case
        \\note For a brine-cooled cooled (secondary system) case, enter the brine inlet temperature
        \\type real
        \\units C
        \\minimum -70.0
        \\maximum 40.0
        \\note Default is 5 C less than case operating temperature
  N21;  \\field Average Refrigerant Charge Inventory
        \\units kg/m
        \\type real
        \\default 0.0

Refrigeration:CompressorRack,
        \\memo Works in conjunction with the refrigeration case and walkin objects to simulate the
        \\memo performance of a refrigerated case system. This object models the electric
        \\memo consumption of the rack compressors and the condenser fans. Heat can be rejected
        \\memo eitheroutdoors or to a zone. Compressor rack waste heat can also be reclaimed for
        \\memo use by an optional air- or water-heating coil (Coil:Heating:Desuperheater and
        \\memo Coil:WaterHeating:Desuperheater).
        \\min-fields 25
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigeratedCaseCompressorRack
        \\reference DesuperHeatingCoilSources
  A2 ,  \\field Heat Rejection Location
        \\type choice
        \\key Outdoors
        \\key Zone
        \\default Outdoors
  N1 ,  \\field Design Compressor Rack COP
        \\type real
        \\units W/W
        \\minimum> 0.0
        \\default 2.0
        \\note It is important that this COP correspond to the lowest saturated suction
        \\note      temperature needed to serve all refrigeration loads
  A3 ,  \\field Compressor Rack COP Function of Temperature Curve Name
        \\required-field
        \\type object-list
        \\object-list QuadraticCubicCurves
        \\note It is important that this COP curve correspond to the lowest saturated suction
        \\note      temperature needed to serve all refrigeration loads
  N2 ,  \\field Design Condenser Fan Power
        \\note Design power for condenser fan(s).
        \\type real
        \\units W
        \\minimum 0.0
        \\default 250.0
  A4 ,  \\field Condenser Fan Power Function of Temperature Curve Name
        \\type object-list
        \\object-list QuadraticCubicCurves
  A5 ,  \\field Condenser Type
        \\type choice
        \\key AirCooled
        \\key EvaporativelyCooled
        \\key WaterCooled
        \\default AirCooled
        \\note Applicable only when Heat Rejection Location is Outdoors.
  A6 ,  \\field Water-Cooled Condenser Inlet Node Name
        \\type node
  A7 ,  \\field Water-Cooled Condenser Outlet Node Name
        \\type node
  A8 ,  \\field Water-Cooled Loop Flow Type
        \\type choice
        \\key VariableFlow
        \\key ConstantFlow
        \\default VariableFlow
        \\note Applicable only when Condenser Type is WaterCooled.
  A9 ,  \\field Water-Cooled Condenser Outlet Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Applicable only when loop Flow type is VariableFlow.
  N3,   \\field Water-Cooled Condenser Design Flow Rate
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\note Applicable only when loop flow type is ConstantFlow.
  N4,   \\field Water-Cooled Condenser Maximum Flow Rate
        \\type real
        \\units m3/s
        \\minimum> 0.0
  N5,   \\field Water-Cooled Condenser Maximum Water Outlet Temperature
        \\type real
        \\units C
        \\minimum 10.0
        \\maximum 60.0
        \\default 55.0
  N6,   \\field Water-Cooled Condenser Minimum Water Inlet Temperature
        \\type real
        \\units C
        \\minimum 10.0
        \\maximum 30.0
        \\default 10.0
  A10 , \\field Evaporative Condenser Availability Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note This field is only used for Condenser Type = EvaporativelyCooled.
        \\note Schedule values greater than 0 indicate that evaporative cooling of the
        \\note condenser is available. This schedule allows the user to define seasonal
        \\note shutdown/draining of the water cooling system in cold climate applications.
        \\note For periods with schedule values of 0, the condenser operates as AirCooled.
  N7 ,  \\field Evaporative Condenser Effectiveness
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\maximum 1.0
        \\default 0.9
        \\note Applicable only for Condenser Type = EvaporativlyCooled.
  N8 ,  \\field Evaporative Condenser Air Flow Rate
        \\type real
        \\units m3/s
        \\minimum> 0.0
        \\autocalculatable
        \\default Autocalculate
        \\note Applicable only for Condenser Type = EvaporativelyCooled.
        \\note Used to calculate evaporative condenser water use.
  N9 ,  \\field Basin Heater Capacity
        \\type real
        \\units W/K
        \\minimum 0.0
        \\default 200.0
        \\note This field is only used for Condenser Type = EvaporativelyCooled and for periods
        \\note when the evaporatively cooled condenser is available (field Evaporative Condenser Availability
        \\note Schedule Name). For this situation, the heater heats the basin water when the
        \\note outdoor air dry-bulb temperature falls below the setpoint temperature, but
        \\note only when the condenser fans are off (i.e., no refrigerated case load).
  N10 ,  \\field Basin Heater Setpoint Temperature
        \\type real
        \\units C
        \\minimum 2.0
        \\default 2.0
        \\note Enter the outdoor dry-bulb temperature at which the basin heater turns on.
  N11 ,  \\field Design Evaporative Condenser Water Pump Power
        \\type real
        \\units W
        \\minimum 0.0
        \\default 1000.0
        \\autocalculatable
        \\note Design recirc water pump power for Condenser Type = EvaporativelyCooled.
        \\note Applicable only for Condenser Type = EvaporativelyCooled.
  A11 , \\field Evaporative Water Supply Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note If blank, water supply is from Mains.
        \\note Applicable only for Condenser Type = EvaporativelyCooled.
  A12 , \\field Condenser Air Inlet Node Name
        \\type node
        \\note Applicable only when Heat Rejection Location is Outdoors and Condenser Type is
        \\note not WaterCooled; otherwise, leave field blank. If field is left blank with
        \\note Heat Rejection Location = Outdoors, then the model assumes that the Inlet Air
        \\note conditions are the outdoor air conditions for the current timestep
        \\note (e.g., no adjustment for height above ground).
  A13 , \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  A14 , \\field Refrigeration Case Name or WalkIn Name or CaseAndWalkInList Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:Walkin or
        \\note Refrigeration:CaseAndWalkinList object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInAndListNames
  A15  ; \\field Heat Rejection Zone Name
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\note Required only if walk-in[s] are connected to this rack
        \\note AND the heat rejection location is "Zone"
        \\type object-list
        \\object-list ZoneNames



Refrigeration:CaseAndWalkInList,
        \\memo Provides a list of all the refrigerated cases, walk in coolers, or air chillers
        \\memo cooled by a single refrigeration system.  Note that the names of all cases,
        \\memo walk-ins ,air chillers, and CaseAndWalkInLists must be unique.  That is, you cannot
        \\memo give a list the same name as one of list items. This list may contain a combination
        \\memo of case and walk-in names OR a list of air chiller names.  Air chillers
        \\memo may not be included in any list that also includes cases or walk-ins.
        \\extensible:1  repeat last field, remembering to remove ; from "inner" fields.
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCaseAndWalkInAndListNames
  A2 ,  \\field Case or WalkIn 1 Name
        \\begin-extensible
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A3 ,  \\field Case or WalkIn 2 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A4 ,  \\field Case or WalkIn 3 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A5 ,  \\field Case or WalkIn 4 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A6 ,  \\field Case or WalkIn 5 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A7 ,  \\field Case or WalkIn 6 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A8 ,  \\field Case or WalkIn 7 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A9 ,  \\field Case or WalkIn 8 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A10 , \\field Case or WalkIn 9 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A11 , \\field Case or WalkIn 10 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A12 , \\field Case or WalkIn 11 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A13 , \\field Case or WalkIn 12 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A14 , \\field Case or WalkIn 13 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A15 , \\field Case or WalkIn 14 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A16 , \\field Case or WalkIn 15 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A17 , \\field Case or WalkIn 16 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A18 , \\field Case or WalkIn 17 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A19 , \\field Case or WalkIn 18 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A20 , \\field Case or WalkIn 19 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames
  A21 ; \\field Case or WalkIn 20 Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInNames

Refrigeration:Condenser:AirCooled,
        \\memo Air cooled condenser for a refrigeration system (Refrigeration:System).
        \\min-fields 5
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationAllTypesCondenserNames
        \\reference DesuperHeatingCoilSources
  A2 ,  \\field Rated Effective Total Heat Rejection Rate Curve Name
        \\note Rating as per ARI 460
        \\note Be sure the rating corresponds to the correct refrigerant
        \\type object-list
        \\object-list LinearCurves
        \\note HeatRejection(W)=C1 +C2(Condensing Temp - Entering Air Temp, deg C)
        \\note Will be adjusted for elevation automatically
  N1 ,  \\field Rated Subcooling Temperature Difference
        \\type real
        \\units DeltaC
        \\default 0.0
        \\minimum 0.0
        \\note must correspond to rating given for total heat rejection effect
  A3 ,  \\field Condenser Fan Speed Control Type
        \\type choice
        \\key Fixed
        \\key FixedLinear
        \\key VariableSpeed
        \\key TwoSpeed
        \\default Fixed
  N2 ,  \\field Rated Fan Power
        \\note Power for condenser fan(s) corresponding to rated total heat rejection effect.
        \\type real
        \\units W
        \\minimum 0.0
        \\default 250.0
        \\required-field
  N3 ,  \\field Minimum Fan Air Flow Ratio
        \\note Minimum air flow fraction through condenser fan
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\default 0.2
  A4 ,  \\field Air Inlet Node Name or Zone Name
        \\type node
        \\note If field is left blank,
        \\note then the model assumes that the inlet air
        \\note conditions are the outdoor air conditions for the current timestep
        \\note (e.g., no adjustment for height above ground).
        \\note If the condenser rejects heat to a conditioned zone, enter the zone name here.
  A5 ,  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  N4 ,  \\field Condenser Refrigerant Operating Charge Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N5 ,  \\field Condensate Receiver Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N6 ;  \\field Condensate Piping Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input

Refrigeration:Condenser:EvaporativeCooled,
        \\memo Evaporative-cooled condenser for a refrigeration system (Refrigeration:System).
        \\min-fields 10
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationAllTypesCondenserNames
        \\reference DesuperHeatingCoilSources
  N1 ,  \\field Rated Effective Total Heat Rejection Rate
        \\note Rating as per ARI 490
        \\required-field
        \\minimum 0.0
        \\note Be sure the rating corresponds to the correct refrigerant
        \\type real
        \\units W
  N2 ,  \\field Rated Subcooling Temperature Difference
        \\type real
        \\units DeltaC
        \\default 0.0
        \\minimum 0.0
        \\note must correspond to rating given for total heat rejection effect
  A2 ,  \\field Fan Speed Control Type
        \\type choice
        \\key Fixed
        \\key FixedLinear
        \\key VariableSpeed
        \\key TwoSpeed
        \\default Fixed
  N3 ,  \\field Rated Fan Power
        \\note Power for condenser fan(s) corresponding to rated total heat rejection effect.
        \\type real
        \\units W
        \\minimum 0.0
        \\required-field
  N4 ,  \\field Minimum Fan Air Flow Ratio
        \\note Minimum air flow fraction through condenser fan
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\default 0.2
  N5 ,  \\field Approach Temperature Constant Term
        \\type real
        \\units C
        \\minimum 0.0
        \\maximum 20.0
        \\default 6.63
        \\note A1 in delta T = A1 + A2(hrcf) + A3/(hrcf) + A4(Twb)
  N6 ,  \\field Approach Temperature Coefficient 2
        \\type real
        \\units C
        \\minimum 0.0
        \\maximum 20.0
        \\default 0.468
        \\note A2 in delta T = A1 + A2(hrcf) +A3/(hrcf) +A4(Twb)
  N7 ,  \\field Approach Temperature Coefficient 3
        \\type real
        \\units C
        \\minimum 0.0
        \\maximum 30.0
        \\default 17.93
        \\note A3 in delta T = A1 + A2(hrcf) +A3/(hrcf) +A4(Twb)
  N8 ,  \\field Approach Temperature Coefficient 4
        \\type real
        \\units dimensionless
        \\minimum -20.0
        \\maximum 20.0
        \\default -0.322
        \\note A4 in deltaT=A1 + A2(hrcf) +A3/(hrcf) +A4(Twb)
  N9 ,  \\field Minimum Capacity Factor
        \\type real
        \\units dimensionless
        \\default 0.50
        \\note taken from manufacturer's Heat Rejection Capacity Factor Table
  N10,  \\field Maximum Capacity Factor
        \\type real
        \\units dimensionless
        \\default 5.0
        \\note taken from manufacturer's Heat Rejection Capacity Factor Table
  A3 ,  \\field Air Inlet Node Name
        \\type node
        \\note If field is left blank,
        \\note then the model assumes that the inlet air
        \\note conditions are the outdoor air conditions for the current timestep
        \\note (e.g., no adjustment for height above ground).
  N11 , \\field Rated Air Flow Rate
        \\type real
        \\units m3/s
        \\autocalculatable
        \\default autocalculate
        \\note Used to calculate evaporative condenser water use and fan energy use.
  N12 , \\field Basin Heater Capacity
        \\type real
        \\units W/K
        \\minimum 0.0
        \\default 200.0
        \\note This field is only used for periods
        \\note when the evap condenser is available (field Evaporative Condenser Availability
        \\note Schedule). For this situation, the heater heats the basin water when the
        \\note outdoor air dry-bulb temperature falls below the set point temperature, but
        \\note only when the condenser fans are off (i.e., no refrigerated case load).
  N13 , \\field Basin Heater Setpoint Temperature
        \\type real
        \\units C
        \\minimum 2.0
        \\default 2.0
        \\note Enter the outdoor dry-bulb temperature at which the basin heater turns on.
  N14 , \\field Rated Water Pump Power
        \\type real
        \\units W
        \\default 1000.0
        \\autocalculatable
        \\note Design recirculating water pump power.
  A4 ,  \\field Evaporative Water Supply Tank Name
        \\type object-list
        \\object-list WaterStorageTankNames
        \\note If blank, water supply is from Mains.
  A5 ,  \\field Evaporative Condenser Availability Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values greater than 0 indicate that evaporative cooling of the
        \\note condenser is available. This schedule allows the user to define seasonal
        \\note shutdown/draining of the water cooling system in cold climate applications.
        \\note For periods with schedule values of 0, the condenser operates as Air Cooled.
  A6,   \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  N15 , \\field Condenser Refrigerant Operating Charge Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N16 , \\field Condensate Receiver Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N17 ; \\field Condensate Piping Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input

Refrigeration:Condenser:WaterCooled,
        \\memo Water cooled condenser for a refrigeration system (Refrigeration:System).
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationAllTypesCondenserNames
        \\reference DesuperHeatingCoilSources
  N1 ,  \\field Rated Effective Total Heat Rejection Rate
        \\note Rating as per ARI 450
        \\note Be sure the rating corresponds to the correct refrigerant
        \\type real
        \\units W
        \\minimum> 0.0
        \\note not used in calculations, only for identification and output
  N2 ,  \\field Rated Condensing Temperature
        \\required-field
        \\type real
        \\minimum> 0.0
        \\units C
        \\note must correspond to rating given for total heat rejection effect
  N3 ,  \\field Rated Subcooling Temperature Difference
        \\default 0.0
        \\minimum 0.0
        \\type real
        \\units DeltaC
        \\note must correspond to rating given for total heat rejection effect
  N4,   \\field Rated Water Inlet Temperature
        \\required-field
        \\minimum> 0.0
        \\type real
        \\units C
        \\note must correspond to rating given for total heat rejection effect
  A2 ,  \\field Water Inlet Node Name
        \\type node
  A3 ,  \\field Water Outlet Node Name
        \\type node
  A4 ,  \\field Water-Cooled Loop Flow Type
        \\type choice
        \\key VariableFlow
        \\key ConstantFlow
        \\default VariableFlow
  A5 ,  \\field Water Outlet Temperature Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Applicable only when loop flow type is Variable Flow.
  N5 ,  \\field Water Design Flow Rate
        \\type real
        \\units m3/s
        \\note note required units must be converted from L/s as specified in ARI 450-2007
        \\minimum> 0.0
        \\note Applicable only when loop flow type is Constant Flow.
  N6 ,  \\field Water Maximum Flow Rate
        \\type real
        \\units m3/s
        \\minimum> 0.0
  N7 ,  \\field Water Maximum Water Outlet Temperature
        \\type real
        \\units C
        \\minimum 10.0
        \\maximum 60.0
        \\default 55.0
  N8 ,  \\field Water Minimum Water Inlet Temperature
        \\type real
        \\units C
        \\minimum 10.0
        \\maximum 30.0
        \\default 10.0
        \\note related to the minimum allowed refrigeration system condensing temperature
  A6 ,  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  N9 ,  \\field Condenser Refrigerant Operating Charge Inventory
        \\type real
        \\units kg
        \\note optional input
  N10 , \\field Condensate Receiver Refrigerant Inventory
        \\type real
        \\units kg
        \\note optional input
  N11 ; \\field Condensate Piping Refrigerant Inventory
        \\type real
        \\units kg
        \\note optional input

Refrigeration:Condenser:Cascade,
        \\memo Cascade condenser for a refrigeration system (Refrigeration:System). The cascade
        \\memo condenser is unlike the other condenser options because it rejects heat to another,
        \\memo higher-temperature, refrigeration system. That is, the cascade condenser acts as a
        \\memo heat rejection object for one system, but acts as a refrigeration load for another
        \\memo system.
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationAllTypesCondenserNames
        \\reference RefrigerationCascadeCondenserAndSecondarySystemNames
        \\reference RefrigerationSecondarySystemAndCascadeCondenserAndTransferLoadListNames
  N1 ,  \\field Rated Condensing Temperature
        \\note This is the condensing temperature for the lower temperature secondary loop
        \\required-field
        \\type real
        \\units C
  N2 ,  \\field Rated Approach Temperature Difference
        \\note This is the difference between the condensing and evaporating temperatures
        \\default 3.0
        \\type real
        \\minimum> 0.0
        \\units DeltaC
  N3 ,  \\field Rated Effective Total Heat Rejection Rate
        \\required-field
        \\note used for identification and rough system size error checking
        \\type real
        \\minimum> 0.0
        \\units W
  A2 ,  \\field Condensing Temperature Control Type
        \\type choice
        \\key Fixed
        \\key Float
        \\default Fixed
        \\note Fixed keeps condensing temperature constant
        \\note Float sets the condensing temperature according to
        \\note      the other loads on the higher temperature system
  N4 ,  \\field Condenser Refrigerant Operating Charge Inventory
        \\type real
        \\units kg
        \\note optional input
  N5 , \\field Condensate Receiver Refrigerant Inventory
        \\type real
        \\units kg
        \\note optional input
  N6 ; \\field Condensate Piping Refrigerant Inventory
        \\type real
        \\units kg
        \\note optional input

Refrigeration:GasCooler:AirCooled,
        \\memo The transcritical refrigeration system requires a single gas cooler to reject the
        \\memo system heat.
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationAllTypesGasCoolerNames
  A2 ,  \\field Rated Total Heat Rejection Rate Curve Name
        \\required-field
        \\type object-list
        \\object-list LinearCurves
        \\note Be sure the rating corresponds to the correct refrigerant (R744)
        \\note HeatRejection(W)=C1 +C2(Gas Cooler Outlet Temp - Entering Air Temp, deg C)
        \\note Will be adjusted for elevation automatically
  A3 ,  \\field Gas Cooler Fan Speed Control Type
        \\type choice
        \\key Fixed
        \\key FixedLinear
        \\key VariableSpeed
        \\key TwoSpeed
        \\default Fixed
  N1 ,  \\field Rated Fan Power
        \\note Power for gas cooler fan(s) corresponding to rated total heat rejection effect.
        \\type real
        \\units W
        \\minimum 0.0
        \\default 5000.0
        \\required-field
  N2 ,  \\field Minimum Fan Air Flow Ratio
        \\note Minimum air flow fraction through gas cooler fan
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\default 0.2
  N3 ,  \\field Transition Temperature
        \\note Temperature at which system transitions between subcritical and transcritical operation.
        \\type real
        \\units C
        \\default 27.0
  N4 ,  \\field Transcritical Approach Temperature
        \\note Temperature difference between the CO2 exiting the gas cooler and the air entering the
        \\note gas cooler during transcritical operation.
        \\type real
        \\units DeltaC
        \\default 3.0
  N5 ,  \\field Subcritical Temperature Difference
        \\note Temperature difference between the saturated condensing temperature and the air
        \\note temperature during subcritical operation.
        \\type real
        \\units DeltaC
        \\default 10.0
  N6 ,  \\field Minimum Condensing Temperature
        \\note Minimum saturated condensing temperature during subcritical operation.
        \\type real
        \\units C
        \\default 10.0
  A4 ,  \\field Air Inlet Node Name
        \\type node
        \\note If field is left blank,
        \\note then the model assumes that the inlet air
        \\note conditions are the outdoor air conditions for the current timestep
        \\note (e.g., no adjustment for height above ground).
  A5 ,  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  N7 ,  \\field Gas Cooler Refrigerant Operating Charge Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N8 ,  \\field Gas Cooler Receiver Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input
  N9 ;  \\field Gas Cooler Outlet Piping Refrigerant Inventory
        \\type real
        \\units kg
        \\default 0.0
        \\note optional input

Refrigeration:TransferLoadList,
        \\memo A refrigeration system may provide cooling to other, secondary, systems through
        \\memo either a secondary loop or a cascade condenser. If multiple transfer loads are served
        \\memo by a single primary system, use this list to group them together for reference by the
        \\memo primary system (see the field "Refrigeration Transfer Load or TransferLoad List Name"
        \\memo in the Refrigeration:System object).
        \\extensible:1  repeat last field, remembering to remove ; from "inner" fields.
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationSecondarySystemAndCascadeCondenserAndTransferLoadListNames
  A2 ,  \\field Cascade Condenser Name or Secondary System 1 Name
        \\begin-extensible
        \\required-field
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A3 ,  \\field Cascade Condenser Name or Secondary System 2 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A4 ,  \\field Cascade Condenser Name or Secondary System 3 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A5 ,  \\field Cascade Condenser Name or Secondary System 4 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A6 ,  \\field Cascade Condenser Name or Secondary System 5 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A7 ,  \\field Cascade Condenser Name or Secondary System 6 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A8 ,  \\field Cascade Condenser Name or Secondary System 7 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A9 ,  \\field Cascade Condenser Name or Secondary System 8 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames
  A10;  \\field Cascade Condenser Name or Secondary System 9 Name
        \\note Enter the name of a Refrigeration:Condenser:Cascade object OR
        \\note   the name of a Refrigeration:SecondarySystem object.
        \\type object-list
        \\object-list RefrigerationCascadeCondenserAndSecondarySystemNames

Refrigeration:Subcooler,
        \\memo Two types of subcoolers are modeled by the detailed refrigeration system.  The
        \\memo liquid suction heat exchanger uses cool suction gas to subcool the hot condensate
        \\memo after it leaves the condenser and before it reaches the thermal expansion valve.
        \\memo A mechanical subcooler is used to transfer cooling capacity from one refrigeration
        \\memo system to another.
        \\min-fields 5
   A1 , \\field Name
        \\type alpha
        \\required-field
        \\reference RefrigerationSubcoolerNames
   A2 , \\field Subcooler Type
        \\type choice
        \\key Mechanical
        \\key LiquidSuction
        \\default LiquidSuction
        \\note plan to add ambient subcoolers at future time
   N1 , \\field Liquid Suction Design Subcooling Temperature Difference
        \\type real
        \\units DeltaC
        \\note Applicable only and required for liquid suction heat exchangers
        \\note design liquid suction subcooling
   N2 , \\field Design Liquid Inlet Temperature
        \\type real
        \\units C
        \\note design inlet temperature on liquid side
        \\note Applicable only and required for liquid suction heat exchangers (LSHX)
   N3 , \\field Design Vapor Inlet Temperature
        \\type real
        \\units C
        \\note design inlet temperature on vapor side
        \\note Applicable only and required for liquid suction heat exchangers (LSHX)
        \\note Design vapor inlet temperature must be less than or equal to
        \\note      the Liquid inlet design temp
   A3 , \\field Capacity-Providing System
        \\type object-list
        \\object-list RefrigerationSystemNames
        \\note Name of the Detailed Refrigeration System providing cooling capacity
        \\note Applicable only and required for mechanical subcoolers
   N4 ; \\field Outlet Control Temperature
        \\type real
        \\units C
        \\note Control Temperature Out for subcooled liquid
        \\note Applicable only and required for mechanical subcoolers

Refrigeration:Compressor,
        \\memo Refrigeration system compressor. Data is available for many compressors
        \\memo in the RefrigerationCompressor.idf dataset
        \\min-fields 6
   A1 , \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCompressorAndListNames
        \\reference RefrigerationCompressorNames
   A2 , \\field Refrigeration Compressor Power Curve Name
        \\required-field
        \\type object-list
        \\object-list BicubicCurves
        \\note the input order for the Curve:Bicubic does not
        \\note match the ARI 540-2004 Eq. 1 coefficient order
        \\note N1 is ARI_C1, N2 is ARI_C2, N3 is ARI_C4, N4 is ARI_C3,
        \\note N5 is ARI_C6, N6 is ARI_C5, N7 is ARI_C7, N8 is ARI_C10,
        \\note N9 is ARI_C8, N10 is ARI_C9,
        \\note N11 is Minimum Suction dewpoint temperature,
        \\note N12 is Maximum Suction dewpoint temperature,
        \\note N13 is Minimum Discharge dewpoint temperature,
        \\note N14 is Maximum Discharge dewpoint temperature
   A3 , \\field Refrigeration Compressor Capacity Curve Name
        \\required-field
        \\type object-list
        \\object-list BicubicCurves
        \\note the input order for the Curve:Bicubic does not
        \\note match the ARI 540-2004 Eq. 1 coefficient order
        \\note N1 is ARI_C1, N2 is ARI_C2, N3 is ARI_C4, N4 is ARI_C3,
        \\note N5 is ARI_C6, N6 is ARI_C5, N7 is ARI_C7, N8 is ARI_C10,
        \\note N9 is ARI_C8, N10 is ARI_C9,
        \\note N11 is Minimum Suction dewpoint temperature,
        \\note N12 is Maximum Suction dewpoint temperature,
        \\note N13 is Minimum Discharge dewpoint temperature,
        \\note N14 is Maximum Discharge dewpoint temperature
   N1 , \\field Rated Superheat
        \\units deltaC
        \\type real
        \\note Use this input field OR the next, not both
        \\note This is used if the compressor rating is based upon
        \\note degrees of superheat
   N2 , \\field Rated Return Gas Temperature
        \\units C
        \\type real
        \\note Use this input field OR the previous, not both
        \\note This is used if the compressor rating is based upon
        \\note rated return gas temperature (Rated Suction Temperature)
   N3 , \\field Rated Liquid Temperature
        \\units C
        \\type real
        \\note Use this input field OR the next, not both
        \\note This is used if the compressor rating is based upon
        \\note rated liquid temperature at the expansion valve
   N4 , \\field Rated Subcooling
        \\units deltaC
        \\type real
        \\note Use this input field OR the previous, not both
        \\note This is used if the compressor rating is based upon
        \\note degrees of subcooling
   A4 , \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
   A5 , \\field Mode of Operation
        \\retaincase
        \\type choice
        \\key Subcritical
        \\key Transcritical
        \\default Subcritical
   A6 , \\field Transcritical Compressor Power Curve Name
        \\type object-list
        \\object-list BicubicCurves
   A7 ; \\field Transcritical Compressor Capacity Curve Name
        \\type object-list
        \\object-list BicubicCurves

Refrigeration:CompressorList,
        \\memo List of all the compressors included within a single refrigeration system
        \\memo (Refrigeration:System). Each list must contain at least one compressor.
        \\memo The order in which the individual compressors are listed here will be the
        \\memo order in which the compressors are dispatched to meet the system load.
        \\min-fields 2
        \\extensible:1  repeat last field, remembering to remove ; from "inner" fields.
        \\memo IMPORTANT: List compressor names in the order in which the compressors will be loaded
        \\memo Data is available for many compressors in the RefrigerationCompressor.idf dataset
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCompressorAndListNames
  A2 ,  \\field Refrigeration Compressor 1 Name
        \\begin-extensible
        \\note Enter the name of a Refrigeration:Compressor object.
        \\required-field
        \\type object-list
        \\object-list RefrigerationCompressorNames
  A3 ,  \\field Refrigeration Compressor 2 Name
        \\note Enter the name of a Refrigeration:Compressor object.
        \\type object-list
        \\object-list RefrigerationCompressorNames
  A4 ,  \\field Refrigeration Compressor 3 Name
        \\note Enter the name of a Refrigeration:Compressor object.
        \\type object-list
        \\object-list RefrigerationCompressorNames
  A5 ;  \\field Refrigeration Compressor 4 Name
        \\note Enter the name of a Refrigeration:Compressor object.
        \\type object-list
        \\object-list RefrigerationCompressorNames

Refrigeration:System,
        \\memo Simulates the performance of a supermarket refrigeration system when used along with
        \\memo other objects to define the refrigeration load(s), the compressor(s), and the
        \\memo condenser.
        \\min-fields 7
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationSystemNames
  A2 , \\field Refrigerated Case or Walkin or CaseAndWalkInList Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\note If there is more than one refrigerated case or walkin served by this system,
        \\note  enter the name of a Refrigeration:CaseAndWalkInList object.
        \\note Only cases and walkins served directly by the system should be included in this list.
        \\note Any cases served indirectly via a secondary chiller should NOT be included in this list
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInAndListNames
  A3  , \\field Refrigeration Transfer Load or TransferLoad List Name
        \\note Enter the name of a Refrigeration:SecondarySystem object OR
        \\note   a Refrigeration:Condenser:Cascade object OR,
        \\note   a Refrigeration:TransferLoadList object.
        \\note A transfer load is identified as one which moves the load from one system to another.
        \\note   So if you have more than one such load (including cascade condensers and secondary
        \\note   loops) served by the same system, use a TransferLoadList object.
        \\type object-list
        \\object-list RefrigerationSecondarySystemAndCascadeCondenserAndTransferLoadListNames
  A4 ,  \\field Refrigeration Condenser Name
        \\required-field
        \\type object-list
        \\object-list RefrigerationAllTypesCondenserNames
  A5,  \\field Compressor or CompressorList Name
        \\required-field
        \\type object-list
        \\object-list RefrigerationCompressorAndListNames
  N1 ,  \\field Minimum Condensing Temperature
        \\type real
        \\units C
        \\required-field
        \\note related to the proper operation of the thermal expansion
        \\note valves and compressors
  A6 ,  \\field Refrigeration System Working Fluid Type
        \\note Fluid property data for the refrigerant must be entered.
        \\note The fluid property data, including the objects:
        \\note FluidProperties:Name, FluidProperties:Temperatures,
        \\note FluidProperties:Saturated and FluidProperties:Superheated
        \\note can be copied from the FluidPropertiesRefData.idf dataset
        \\required-field
        \\type object-list
        \\object-list FluidNames
  A7 ,  \\field Suction Temperature Control Type
        \\type choice
        \\key FloatSuctionTemperature
        \\key ConstantSuctionTemperature
        \\default ConstantSuctionTemperature
  A8 ,  \\field Mechanical Subcooler Name
        \\type object-list
        \\object-list RefrigerationSubcoolerNames
        \\note Optional Field
        \\note Recipient of refrigeration capacity, that is receives cool liquid
        \\note    from another refrigeraiton system to help meet aggregate case loads
  A9 ,  \\field Liquid Suction Heat Exchanger Subcooler Name
        \\type object-list
        \\object-list RefrigerationSubcoolerNames
        \\note Optional Field
        \\note Liquid Suction Heat Exchanger Name, or leave blank
  N2 ,  \\field Sum UA Suction Piping
        \\type real
        \\default 0.0
        \\units W/K
        \\note Use only if you want to include suction piping heat gain in refrigeration load
  A10,  \\field Suction Piping Zone Name
        \\note This will be used to determine the temperature used for distribution piping heat gain
        \\note  and the pipe heat gains  as cooling credit for the zone.
        \\note  Required only if Sum UA Distribution Piping >0.0
        \\type object-list
        \\object-list ZoneNames
  A11,  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General
  N3 ,  \\field Number of Compressor Stages
        \\type choice
        \\key 1
        \\key 2
        \\default 1
  A12,  \\field Intercooler Type
        \\type choice
        \\key None
        \\key Flash Intercooler
        \\key Shell-and-Coil Intercooler
        \\default None
  N4 ,  \\field Shell-and-Coil Intercooler Effectiveness
        \\type real
        \\default 0.8
  A13;  \\field High-Stage Compressor or CompressorList Name
        \\type object-list
        \\object-list RefrigerationCompressorAndListNames

Refrigeration:TranscriticalSystem,
        \\memo Detailed transcritical carbon dioxide (CO2) booster refrigeration systems used in
        \\memo supermarkets.  The object allows for modeling either a single stage system with
        \\memo medium-temperature loads or a two stage system with both medium- and low-temperature
        \\memo loads.
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationSystemNames
  A2 ,  \\field System Type
        \\required-field
        \\type choice
        \\key SingleStage
        \\key TwoStage
  A3 ,  \\field Medium Temperature Refrigerated Case or Walkin or CaseAndWalkInList Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\note If there is more than one refrigerated case or walkin served by this system,
        \\note enter the name of a Refrigeration:CaseAndWalkInList object.
        \\note Only medium temperature cases and walkins served directly by the system should
        \\note be included in this list.
        \\required-field
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInAndListNames
  A4 ,  \\field Low Temperature Refrigerated Case or Walkin or CaseAndWalkInList Name
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\note If there is more than one refrigerated case or walkin served by this system,
        \\note enter the name of a Refrigeration:CaseAndWalkInList object.
        \\note Only low temperature cases and walkins served directly by the system should be
        \\note included in this list.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInAndListNames
  A5 ,  \\field Refrigeration Gas Cooler Name
        \\required-field
        \\type object-list
        \\object-list RefrigerationAllTypesGasCoolerNames
  A6 ,  \\field High Pressure Compressor or CompressorList Name
        \\required-field
        \\type object-list
        \\object-list RefrigerationCompressorAndListNames
  A7 ,  \\field Low Pressure Compressor or CompressorList Name
        \\type object-list
        \\object-list RefrigerationCompressorAndListNames
  N1 ,  \\field Receiver Pressure
        \\type real
        \\default 4000000
        \\units Pa
  N2 ,  \\field Subcooler Effectiveness
        \\type real
        \\default 0.4
  A8 ,  \\field Refrigeration System Working Fluid Type
        \\note Fluid property data for the refrigerant must be entered.
        \\note The fluid property data, including the objects:
        \\note FluidProperties:Name, FluidProperties:Temperatures,
        \\note FluidProperties:Saturated and FluidProperties:Superheated
        \\note can be copied from the FluidPropertiesRefData.idf dataset
        \\required-field
        \\type object-list
        \\object-list FluidNames
  N3 ,  \\field Sum UA Suction Piping for Medium Temperature Loads
        \\type real
        \\default 0.0
        \\units W/K
        \\note Use only if you want to include suction piping heat gain in refrigeration load
  A9 ,  \\field Medium Temperature Suction Piping Zone Name
        \\note This will be used to determine the temperature used for distribution piping heat
        \\note gain and the pipe heat gains as cooling credit for the zone.
        \\note Required only if Sum UA Distribution Piping for Medium Temperature Loads > 0.0
        \\type object-list
        \\object-list ZoneNames
  N4 ,  \\field Sum UA Suction Piping for Low Temperature Loads
        \\type real
        \\default 0.0
        \\units W/K
        \\note Use only if you want to include suction piping heat gain in refrigeration load
  A10,  \\field Low Temperature Suction Piping Zone Name
        \\note This will be used to determine the temperature used for distribution piping heat
        \\note gain and the pipe heat gains as cooling credit for the zone.
        \\note Required only if Sum UA Distribution Piping for Low Temperature Loads > 0.0
        \\type object-list
        \\object-list ZoneNames
  A11;  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General

Refrigeration:SecondarySystem,
        \\memo Works in conjunction with refrigerated cases and walkins to simulate the performance
        \\memo of a secondary loop supermarket refrigeration system. Heat from the refrigeration
        \\memo loads served by the secondary loop is absorbed by a primary refrigeration system
        \\memo (Refrigeration:System). The SecondarySystem object simulates a heat exchanger that
        \\memo is an evaporator, or refrigeration load, on the primary refrigeration system.
        \\min-fields 14
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCascadeCondenserAndSecondarySystemNames
        \\reference RefrigerationSecondarySystemAndCascadeCondenserAndTransferLoadListNames
  A2 ,  \\field Refrigerated Case or Walkin or CaseAndWalkInList Name
        \\required-field
        \\note Enter the name of a Refrigeration:Case or Refrigeration:WalkIn object.
        \\note If there is more than one refrigerated case or walkin served by this secondary system,
        \\note  enter the name of a Refrigeration:CaseAndWalkInList object.
        \\type object-list
        \\object-list RefrigerationCaseAndWalkInAndListNames
  A3 ,  \\field Circulating Fluid Type
        \\type choice
        \\key FluidAlwaysLiquid
        \\key FluidPhaseChange
        \\required-field
        \\note If "FluidAlwaysLiquid" is selected, the fluid properties
        \\note    must be input using the objects: FluidProperties:Name,
        \\note    FluidProperties:GlycolConcentration, and, if user defined fluid type,
        \\note    FluidProperties:Temperatures and FluidProperties:Concentration.
        \\note    Many sets of fluid properties can be found in GlycolPropertiesRefData.idf.
        \\note If "FluidPhaseChange" is selected, the refrigerant properties
        \\note    must be input using the objects: (if user defined fluid type): FluidProperties:Name,
        \\note    FluidProperties:Temperatures, FluidProperties:Saturated, and
        \\note    FluidProperties:Superheated.
        \\note    Many sets of refrigerant data can be found in FluidPropertiesRefData.idf.
  A4 ,  \\field Circulating Fluid Name
        \\note This must correspond to a name in the FluidProperties:Name object.
        \\required-field
        \\type object-list
        \\object-list FluidAndGlycolNames
  N1 ,  \\field Evaporator Capacity
        \\type real
        \\minimum 0.0
        \\units W
        \\note For "FluidAlwaysLiquid", at least one of the two, Evaporator Capacity OR
        \\note    Evaporator Flow Rate for Secondary Fluid, is required.
        \\note For "FluidPhaseChange", the default capacity is the sum of the rated capacities of the
        \\note    Cases and Walk-ins served by the secondary loop.
  N2 ,  \\field Evaporator Flow Rate for Secondary Fluid
        \\type real
        \\minimum 0.0
        \\units M3/s
        \\note For "FluidAlwaysLiquid", at least one of the two, Evaporator Capacity OR
        \\note    Evaporator Flow Rate for Secondary Fluid, is required.
        \\note For "FluidPhaseChange" loops, this input is not used. (see PhaseChange Circulating
        \\note Rate)
  N3 ,  \\field Evaporator Evaporating Temperature
        \\type real
        \\required-field
        \\units C
        \\note  This is the evaporating temperature in the heat exchanger
        \\note      used to chill or condense the secondary loop circulating fluid.
        \\note      It is NOT the temperature in any cases or walk-ins served by the
        \\note      secondary loop.
  N4 ,  \\field Evaporator Approach Temperature Difference
        \\type real
        \\required-field
        \\units DeltaC
        \\note  For "FluidAlwaysLiquid", this is the rated difference between the temperature of the
        \\note      circulating fluid leaving the heat exchanger
        \\note      and the heat exchanger's rated evaporating temperature.
        \\note  For "FluidPhaseChange", this is the difference between the temperature of the
        \\note      evaporating and condensing temperatures in the heat exchanger.
  N5 ,  \\field Evaporator Range Temperature Difference
        \\type real
        \\units DeltaC
        \\note  For "FluidAlwaysLiquid", this is the rated difference between the temperature of the
        \\note      circulating fluid entering the heat exchanger and the temperature of the
        \\note      circulating fluid leaving the heat exchanger, and is Required.
        \\note  For "FluidPhaseChange", this input is not used.
  N6 ,  \\field Number of Pumps in Loop
        \\type integer
        \\default 1
  N7 ,  \\field Total Pump Flow Rate
        \\type real
        \\minimum 0.0
        \\units M3/s
        \\note For "FluidAlwaysLiquid",if not input, Evaporator Flow Rate for Secondary Fluid
        \\note       will be used.
        \\note For "FluidPhaseChange", if not input, this will be calculated using the
        \\note       PhaseChange Circulating Rate.
  N8 ,  \\field Total Pump Power
        \\type real
        \\minimum 0.0
        \\units W
        \\note Either the Total Pump Power or the Total Pump Head is required.
  N9 ,  \\field Total Pump Head
        \\type real
        \\minimum 0.0
        \\units Pa
        \\note Either the Total Pump Power or the Total Pump Head is required.
  N10,  \\field PhaseChange Circulating Rate
        \\type real
        \\minimum 1.0
        \\units dimensionless
        \\default 2.5
        \\note This is the total mass flow at the pump divided by the gaseous mass flow
        \\note      leaving the refrigeration load.
  A5 ,  \\field Pump Drive Type
        \\type choice
        \\key Constant
        \\key Variable
        \\default Constant
  A6 ,  \\field Variable Speed Pump Cubic Curve Name
        \\type object-list
        \\object-list CubicCurves
        \\note Variable Speed Pump Curve Name is applicable to variable speed pumps
        \\note only.
  N11 , \\field Pump Motor Heat to Fluid
        \\type real
        \\units dimensionless
        \\minimum 0.5
        \\maximum 1.0
        \\default 0.85
        \\note This is the portion of the pump motor heat added to secondary circulating fluid
        \\note    and is equal to the motor efficiency for non-hermetic motor.
        \\note Enter 1.0 for a semi-hermetic motor.
  N12,  \\field Sum UA Distribution Piping
        \\type real
        \\default 0.0
        \\units W/K
        \\note Use only if you want to include distribution piping heat gain in refrigeration load.
  A7 ,  \\field Distribution Piping Zone Name
        \\note This will be used to determine the temperature used for distribution piping heat gain.
        \\note  The pipe heat gains are also counted as cooling credit for the zone.
        \\note  Required only if Sum UA Distribution Piping >0.0
        \\type object-list
        \\object-list ZoneNames
  N13,  \\field Sum UA Receiver/Separator Shell
        \\type real
        \\default 0.0
        \\units W/K
        \\note Use only if you want to include Receiver/Separator Shell heat gain in refrigeration load.
  A8 ,  \\field Receiver/Separator Zone Name
        \\note This will be used to determine the temperature used for Receiver/Separator Shell heat gain.
        \\note  The shell heat gains are also counted as cooling credit for the zone.
        \\note  Required only if Sum UA Receiver/Separator Shell >0.0
        \\type object-list
        \\object-list ZoneNames
  N14,  \\field Evaporator Refrigerant Inventory
        \\Units kg
        \\type real
        \\default 0.0
        \\note This value refers to the refrigerant circulating within the primary system providing
        \\note    cooling to the chiller for the secondary loop, not to the fluid circulating
        \\note    within the secondary loop itself.
  A9;  \\field End-Use Subcategory
        \\type alpha
        \\retaincase
        \\default General


 Refrigeration:WalkIn,
        \\memo Works in conjunction with a compressor rack, a refrigeration system, or a
        \\memo refrigeration secondary system to simulate the performance of a walk-in cooler.
        \\memo The walk-in cooler model uses information at rated conditions along with input
        \\memo descriptions for heat transfer surfaces facing multiple zones to determine
        \\memo performance.
        \\extensible:12 -- duplicate last 12 fields, remembering to remove ; from "inner" fields.
        \\min-fields 28
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCaseAndWalkInNames
        \\reference RefrigerationCaseAndWalkInAndListNames
  A2 ,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
  N1 ,  \\field Rated Coil Cooling Capacity
        \\type real
        \\units W
        \\required-field
  N2 ,  \\field Operating Temperature
        \\type real
        \\units C
        \\required-field
        \\maximum< 20.0
  N3 ,  \\field Rated Cooling Source Temperature
        \\note If DXEvaporator, use evaporating temperature (saturated suction temperature)
        \\note If BrineCoil, use Brine entering temperature
        \\required-field
        \\type real
        \\units C
        \\minimum -70.0
        \\maximum 40.
        \\note used to set minimum suction pressure for DX systems and
        \\note    minimum brine temp for secondary systems
  N4 ,  \\field Rated Total Heating Power
        \\type real
        \\units W
        \\required-field
        \\note Include total for all anti-sweat, door, drip-pan, and floor heater power
        \\note Do not include defrost heater power
  A3 ,  \\field Heating Power Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Values will be used to multiply the total heating power
        \\note Values in the schedule should be between 0.0 and 1.0
        \\note For example, this could be used if display door antisweat heaters
        \\note are turned off at night
        \\note Defaults to always on if schedule name left blank.
  N5 ,  \\field Rated Cooling Coil Fan Power
        \\type real
        \\units W
        \\default 375.0
        \\minimum 0.
  N6 ,  \\field Rated Circulation Fan Power
        \\type real
        \\units W
        \\default 0.0
        \\minimum 0.0
  N7 ,  \\field Rated Total Lighting Power
        \\type real
        \\units W
        \\required-field
        \\note Enter the total (display + task) installed lighting power.
  A4 ,  \\field Lighting Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The schedule should contain values between 0 and 1
        \\note Defaults to always on if schedule name left blank.
  A5 ,  \\field Defrost Type
        \\type choice
        \\key HotFluid
        \\key Electric
        \\key None
        \\key OffCycle
        \\default Electric
        \\note HotFluid includes either hot gas defrost for a DX system or
        \\note   Hot Brine defrost if this walk in is cooled by brine from a secondary chiller
  A6 ,  \\field Defrost Control Type
        \\type choice
        \\key TimeSchedule
        \\key TemperatureTermination
        \\default TimeSchedule
  A7 ,  \\field Defrost Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\required-field
        \\note The schedule values should be 0 (off) or 1 (on)
  A8 ,  \\field Defrost Drip-Down Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The schedule values should be 0 (off) or 1 (on)
        \\note The start time for each defrost period in this drip-down schedule should coincide with
        \\note the start time for each defrost period in the defrost schedule (previous input
        \\note field).The length of each defrost drip-down period must be greater than or equal to the
        \\note corresponding defrost period specified in the defrost schedule. This extra time
        \\note allows the melted frost to drip from the coil before refrigeration is restarted.
  N8 ,  \\field Defrost Power
        \\type real
        \\units W
        \\minimum 0.0
        \\note needed for all defrost types except none and offcycle
  N9 ,  \\field Temperature Termination Defrost Fraction to Ice
        \\type real
        \\units dimensionless
        \\minimum> 0.0
        \\maximum 1.0
        \\note This is the portion of the defrost energy that is available to melt frost
        \\note     Needed only for defrost control type TemperatureTermination
        \\note     defaults to 0.7 for electric defrost and to 0.3 for hot fluid defrost
  A9 ,  \\field Restocking Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should be in units of Watts
        \\note Leave this field blank if no restocking is to be modeled
  N10,  \\field Average Refrigerant Charge Inventory
        \\Units kg
        \\type real
        \\default 0.0
        \\note This value is only used if the Cooling Source Type is DXEvaporator
  N11 , \\field Insulated Floor Surface Area
        \\type real
        \\required-field
        \\units m2
        \\minimum> 0.0
        \\note floor area of walk-in cooler
  N12 , \\field Insulated Floor U-Value
        \\type real
        \\units W/m2-K
        \\default 0.3154
        \\minimum> 0.0
        \\note The default value corresponds to R18
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R15 is U 0.3785 W/m2-K
        \\note R5 is U 1.136 W/m2-K
  A10 , \\field Zone 1 Name
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\begin-extensible
        \\required-field
        \\type object-list
        \\object-list ZoneNames
        \\note The walkin cooler can face multiple zones.  The heat exchange with each zone
        \\note    must be input separately
  N13 , \\field Total Insulated Surface Area Facing Zone 1
        \\type real
        \\required-field
        \\units m2
        \\minimum> 0.0
        \\note Area should include walls and ceilings, but not doors
  N14 , \\field Insulated Surface U-Value Facing Zone 1
        \\type real
        \\units W/m2-K
        \\default 0.3154
        \\minimum> 0.0
        \\note The default value corresponds to R18
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R15 is U 0.3785 W/m2-K
        \\note R5 is U 1.136 W/m2-K
  N15 , \\field Area of Glass Reach In Doors Facing Zone 1
        \\type real
        \\units m2
        \\default 0.0
  N16 , \\field Height of Glass Reach In Doors Facing Zone 1
        \\type real
        \\units m
        \\default 1.5
  N17 , \\field Glass Reach In Door U Value Facing Zone 1
        \\type real
        \\units W/m2-K
        \\default 1.136
        \\minimum> 0.0
        \\note The default value corresponds to R5
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
  A11 , \\field Glass Reach In Door Opening Schedule Name Facing Zone 1
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  N18 , \\field Area of Stocking Doors Facing Zone 1
        \\type real
        \\units m2
        \\default 0.0
  N19 , \\field Height of Stocking Doors Facing Zone 1
        \\type real
        \\units m
        \\default 3.0
  N20 , \\field Stocking Door U Value Facing Zone 1
        \\type real
        \\units W/m2-K
        \\default 0.3785
        \\minimum> 0.0
        \\note The default value corresponds to R15
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R5 is U 1.136 W/m2-K
        \\note R18 is U 0.3154 W/m2-K
  A12 , \\field Stocking Door Opening Schedule Name Facing Zone 1
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  A13 , \\field Stocking Door Opening Protection Type Facing Zone 1
        \\type choice
        \\key None
        \\key AirCurtain
        \\key StripCurtain
        \\note Use StripCurtain for hanging strips or airlock vestibules
        \\default AirCurtain
  A14 , \\field Zone 2 Name
        \\note required if more than one zone
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\type object-list
        \\object-list ZoneNames
  N21 , \\field Total Insulated Surface Area Facing Zone 2
        \\type real
        \\units m2
        \\minimum> 0.0
        \\note Area should include walls and ceilings, but not doors
        \\note required if more than one zone
  N22 , \\field Insulated Surface U-Value Facing Zone 2
        \\type real
        \\units W/m2-K
        \\default 0.3154
        \\minimum> 0.0
        \\note The default value corresponds to R18
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R15 is U 0.3785 W/m2-K
        \\note R5 is U 1.136 W/m2-K
  N23 , \\field Area of Glass Reach In Doors Facing Zone 2
        \\type real
        \\units m2
        \\default 0.0
  N24 , \\field Height of Glass Reach In Doors Facing Zone 2
        \\type real
        \\units m
        \\default 1.5
  N25 , \\field Glass Reach In Door U Value Facing Zone 2
        \\type real
        \\units W/m2-K
        \\default 1.136
        \\minimum> 0.0
        \\note The default value corresponds to R5
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
  A15 , \\field Glass Reach In Door Opening Schedule Name Facing Zone 2
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  N26 , \\field Area of Stocking Doors Facing Zone 2
        \\type real
        \\units m2
        \\default 0.0
  N27 , \\field Height of Stocking Doors Facing Zone 2
        \\type real
        \\units m
        \\default 3.0
  N28 , \\field Stocking Door U Value Facing Zone 2
        \\type real
        \\units W/m2-K
        \\default 0.3785
        \\minimum> 0.0
        \\note The default value corresponds to R15
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R5 is U 1.136 W/m2-K
        \\note R18 is U 0.3154 W/m2-K
  A16 , \\field Stocking Door Opening Schedule Name Facing Zone 2
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  A17 , \\field Stocking Door Opening Protection Type Facing Zone 2
        \\type choice
        \\key None
        \\key AirCurtain
        \\key StripCurtain
        \\note Use StripCurtain for hanging strips or airlock vestibules
        \\default AirCurtain
        \\note last extensible set begins here
        \\note dup remaining 12 lines as many times as there are zones exchanging heat with walkin
        \\note  Move the ending semicolon and increment the field numbers.
  A18 , \\field Zone 3 Name
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\note required if more than two zones
        \\type object-list
        \\object-list ZoneNames
  N29 , \\field Total Insulated Surface Area Facing Zone 3
        \\type real
        \\note required if more than two zones
        \\units m2
        \\minimum> 0.0
        \\note Area should include walls and ceilings, but not doors
  N30 , \\field Insulated Surface U-Value Facing Zone 3
        \\type real
        \\units W/m2-K
        \\default 0.3154
        \\minimum> 0.0
        \\note The default value corresponds to R18
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R15 is U 0.3785 W/m2-K
        \\note R5 is U 1.136 W/m2-K
  N31 , \\field Area of Glass Reach In Doors Facing Zone 3
        \\type real
        \\units m2
        \\default 0.0
  N32 , \\field Height of Glass Reach In Doors Facing Zone 3
        \\type real
        \\units m
        \\default 1.5
  N33 , \\field Glass Reach In Door U Value Facing Zone 3
        \\type real
        \\units W/m2-K
        \\default 1.136
        \\minimum> 0.0
        \\note The default value corresponds to R5
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
  A19,  \\field Glass Reach In Door Opening Schedule Name Facing Zone 3
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  N34 , \\field Area of Stocking Doors Facing Zone 3
        \\type real
        \\units m2
        \\default 0.0
  N35 , \\field Height of Stocking Doors Facing Zone 3
        \\type real
        \\units m
        \\default 3.0
  N36 , \\field Stocking Door U Value Facing Zone 3
        \\type real
        \\units W/m2-K
        \\default 0.3785
        \\minimum> 0.0
        \\note The default value corresponds to R15
        \\note To convert other Archaic American R-values to U, divide 5.678 by the R-value
        \\note Some examples:
        \\note R5 is U 1.136 W/m2-K
        \\note R18 is U 0.3154 W/m2-K
  A20 , \\field Stocking Door Opening Schedule Name Facing Zone 3
        \\type object-list
        \\object-list ScheduleNames
        \\note Schedule values should all be between 0.0 and 1.0.
        \\note For example, if the door is open 30% of the time during working hours,
        \\note  then the schedule would hold the value 0.3 during working hours
        \\note  and 0 during other hours
        \\note  If no schedule name is entered, default is 5% open time during all hours
  A21 ; \\field Stocking Door Opening Protection Type Facing Zone 3
        \\type choice
        \\key None
        \\key AirCurtain
        \\key StripCurtain
        \\note Use StripCurtain for hanging strips or airlock vestibules
        \\default AirCurtain

Refrigeration:AirChiller,
        \\memo Works in conjunction with a refrigeration chiller set, compressor rack, a
        \\memo refrigeration system, or a refrigeration secondary system to simulate the performance
        \\memo of an air chiller, similar to one found in a refrigerated warehouse. Energy use for
        \\memo fans and heaters is modeled based on inputs for nominal power, schedules, and control
        \\memo type. The air chiller model accounts for the sensible and latent heat exchange
        \\memo with the surrounding environment.
        \\min-fields 23
  A1 ,  \\field Name
        \\required-field
        \\type alpha
        \\reference RefrigerationCaseAndWalkInNames
        \\reference RefrigerationCaseAndWalkInAndListNames
        \\reference RefrigerationAirChillerNames
  A2 ,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
  A3 ,  \\field Capacity Rating Type
        \\required-field
        \\note In each case, select the rating option that corresponds to the expected service conditions.
        \\note For example, U.S. manufacturers quote a separate Unit Load Factor for wet or frosted coils.
        \\note If the evaporating temperature is less than 0C, input the frosted coil value.
        \\note Within the European convention, select SC1, 2, 3, 4, or 5 depending upon the expected evaporating temperature.        \\type choice
        \\key UnitLoadFactorSensibleOnly
        \\key CapacityTotalSpecificConditions
        \\key EuropeanSC1Standard
        \\key EuropeanSC1NominalWet
        \\key EuropeanSC2Standard
        \\key EuropeanSC2NominalWet
        \\key EuropeanSC3Standard
        \\key FixedLinear
        \\key EuropeanSC3NominalWet
        \\key EuropeanSC4Standard
        \\key EuropeanSC4NominalWet
        \\key EuropeanSC5Standard
        \\key EuropeanSC5NominalWet
  N1 ,  \\field Rated Unit Load Factor
        \\type real
        \\units W/K
        \\note The sensible cooling capacity in watts (W/C) at rated conditions.
        \\note  The value entered for this field must be greater than zero, with no default value.
        \\note This value is only used if the Capacity Rating Type is UnitLoadFactorSensibleOnly.
        \\note The value given must be based upon the difference between the chiller inlet and
        \\note outlet air temperatures, not on the difference between the zone mean temperature
        \\note and the outlet air temperature
  N2 ,  \\field Rated Capacity
        \\type real
        \\units W
        \\note This value is only used if the Capacity Rating Type is NOT UnitLoadFactorSensibleOnly.
        \\note For CapacityTotalSpecificConditions, this capacity includes both sensible and latent
        \\note at the conditions given in the next two fields.
        \\note Note that the European Standard ratings are sensible only and
        \\note the European Nominal ratings include latent capacity as well.
        \\note The value given here must correspond to the capacity rating type given previously
  N3 ,  \\field Rated Relative Humidity
        \\type real
        \\units percent
        \\maximum 100
        \\default 85
        \\note This field is ONLY used if the Capacity Rating Type is CapacityTotalSpecificConditions and
        \\note represents the relative humidity at rated conditions. The default is 85.
  N4 ,  \\field Rated Cooling Source Temperature
        \\note If DXEvaporator, use evaporating temperature (saturated suction temperature)
        \\note If BrineCoil, use Brine entering temperature
        \\required-field
        \\type real
        \\units C
        \\minimum -70.0
        \\maximum 40.
        \\note used to set minimum suction pressure for DX systems and
        \\note    minimum brine temp for secondary systems
  N5 ,  \\field Rated Temperature Difference DT1
        \\required-field
        \\type real
        \\units deltaC
        \\minimum 0.0
        \\maximum 20.
        \\note The rated difference between the air entering the refrigeration chiller and the
        \\note cooling source temperature in degrees C.
  N6 ,  \\field Maximum Temperature Difference Between Inlet Air and Evaporating Temperature
        \\type real
        \\units deltaC
        \\minimum 0.0
        \\maximum 25.
        \\note The maximum difference between the air entering the refrigeration chiller and the
        \\note cooling source temperature in degrees C used to limit capacity during pull-down.
        \\note defaults to 1.3 times the Rated Temperature Difference DT1
  N7 ,  \\field Coil Material Correction Factor
        \\type real
        \\units dimensionless
        \\default 1.0
        \\note This is the manufacturer's correction factor for coil material corresponding to rating
  N8 ,  \\field Refrigerant Correction Factor
        \\type real
        \\units dimensionless
        \\default 1.0
        \\note This is the manufacturer's correction factor for refrigerant corresponding to rating
  A4 ,  \\field Capacity Correction Curve Type
        \\note In each case, select the correction curve type that corresponds to the rating type.
        \\type choice
        \\key LinearSHR60
        \\key QuadraticSHR
        \\key European
        \\key TabularRHxDT1xTRoom
        \\note default LinearSHR60 unless Capcity Rating Type = CapacityTotalSpecificConditions
  A5 ,  \\field Capacity Correction Curve Name
        \\type object-list
        \\object-list LinearCurves
        \\object-list QuadraticCurves
        \\note Can also be the name of a "Table:OneIndependentVariable" or a "Table:MultiVariableLookup"
        \\note Should be blank for LinearSHR60 correction curve type
  N9 ,  \\field SHR60 Correction Factor
        \\type real
        \\units dimensionless
        \\default 1.48
        \\maximum 1.67
        \\note only used when the capacity correction curve type is LinearSHR60
  N10,  \\field Rated Total Heating Power
        \\type real
        \\units W
        \\required-field
        \\note Include total for all heater power
        \\note Do not include defrost heater power
  A6 ,  \\field Heating Power Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note Values will be used to multiply the total heating power
        \\note Values in the schedule should be between 0.0 and 1.0
        \\note Defaults to always on if schedule name left blank.
  A7 ,  \\field Fan Speed Control Type
        \\type choice
        \\key Fixed
        \\key FixedLinear
        \\key VariableSpeed
        \\key TwoSpeed
        \\default Fixed
  N11,  \\field Rated Fan Power
        \\type real
        \\units W
        \\default 375.0
        \\minimum 0.
  N12,  \\field Rated Air Flow
        \\required-field
        \\type real
        \\units m3/s
  N13,  \\field Minimum Fan Air Flow Ratio
        \\note Minimum air flow fraction through fan
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\default 0.2
  A8 ,  \\field Defrost Type
        \\type choice
        \\key HotFluid
        \\key Electric
        \\key None
        \\key OffCycle
        \\default Electric
        \\note HotFluid includes either hot gas defrost for a DX system or
        \\note   Hot Brine defrost if this walk in is cooled by brine from a secondary chiller
  A9 ,  \\field Defrost Control Type
        \\type choice
        \\key TimeSchedule
        \\key TemperatureTermination
        \\default TimeSchedule
  A10,  \\field Defrost Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\required-field
        \\note The schedule values should be 0 (off) or 1 (on)
  A11,  \\field Defrost Drip-Down Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The schedule values should be 0 (off) or 1 (on)
        \\note The start time for each defrost period in this drip-down schedule should coincide with
        \\note the start time for each defrost period in the defrost schedule (previous input
        \\note field).The length of each defrost drip-down period must be greater than or equal to the
        \\note corresponding defrost period specified in the defrost schedule. This extra time
        \\note allows the melted frost to drip from the coil before refrigeration is restarted.
  N14,  \\field Defrost Power
        \\type real
        \\units W
        \\minimum 0.0
        \\note needed for all defrost types except none and offcycle
  N15,  \\field Temperature Termination Defrost Fraction to Ice
        \\type real
        \\units dimensionless
        \\minimum> 0.0
        \\maximum 1.0
        \\note This is the portion of the defrost energy that is available to melt frost
        \\note     Needed only for defrost control type TemperatureTermination
        \\note     defaults to 0.7 for electric defrost and to 0.3 for hot fluid defrost
  A12,  \\field Vertical Location
        \\type choice
        \\key Ceiling
        \\key Middle
        \\key Floor
        \\default Middle
  N16;  \\field Average Refrigerant Charge Inventory
        \\Units kg
        \\type real
        \\default 0.0
        \\note This value is only used if the Cooling Source Type is DXEvaporator

ZoneHVAC:RefrigerationChillerSet,
        \\memo Works in conjunction with one or multiple air chillers, compressor racks,
        \\memo refrigeration systems, or refrigeration secondary system objects to simulate the
        \\memo performance of a group of air chillers cooling a single zone. The chiller set
        \\memo model passes information about the zone conditions to determine the performance of
        \\memo individual chiller coils within the set, thus providing the sensible and latent heat
        \\memo exchange with the zone environment.
        \\extensible:1 -- duplicate last field, remembering to remove ; from "inner" fields.
        \\min-fields 6
  A1 ,  \\field Name
        \\required-field
        \\type alpha
  A2 ,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
  A3  , \\field Zone Name
        \\note This must be a controlled zone and appear in a ZoneHVAC:EquipmentConnections object.
        \\type object-list
        \\object-list ZoneNames
  A4 ,  \\field Air Inlet Node Name
        \\type node
        \\note Not used - reserved for future use
        \\note Name of the zone exhaust node (see Node) from which the refrigeration chiller
        \\note draws its indoor air.
        \\note This should be one of the zone exhaust nodes for the zone cooled by the chiller set.
  A5 ,  \\field Air Outlet Node Name
        \\type node
        \\note Not used - reserved for future use
        \\note The name of the node where the chiller coil sends its outlet air,
        \\note which must be one of the inlet air nodes for the zone which is being cooled.
  A6 ,  \\field Air Chiller 1 Name
        \\begin-extensible
        \\type alpha
        \\note This is the first chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A7 ,  \\field Air Chiller 2 Name
        \\type alpha
        \\note This is the second chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A8 ,  \\field Air Chiller 3 Name
        \\type alpha
        \\note This is the third chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A9 ,  \\field Air Chiller 4 Name
        \\type alpha
        \\note This is the fourth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A10,  \\field Air Chiller 5 Name
        \\type alpha
        \\note This is the fifth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A11,  \\field Air Chiller 6 Name
        \\type alpha
        \\note This is the sixth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A12,  \\field Air Chiller 7 Name
        \\type alpha
        \\note This is the seventh chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A13,  \\field Air Chiller 8 Name
        \\type alpha
        \\note This is the eighth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A14,  \\field Air Chiller 9 Name
        \\type alpha
        \\note This is the ninth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A15,  \\field Air Chiller 10 Name
        \\type alpha
        \\note This is the tenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A16,  \\field Air Chiller 11 Name
        \\type alpha
        \\note This is the eleventh chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A17,  \\field Air Chiller 12 Name
        \\type alpha
        \\note This is the twelth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A18,  \\field Air Chiller 13 Name
        \\type alpha
        \\note This is the thirteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A19,  \\field Air Chiller 14 Name
        \\type alpha
        \\note This is the fourteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A20,  \\field Air Chiller 15 Name
        \\type alpha
        \\note This is the fifteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A21,  \\field Air Chiller 16 Name
        \\type alpha
        \\note This is the sixteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A22,  \\field Air Chiller 17 Name
        \\type alpha
        \\note This is the seventeenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A23,  \\field Air Chiller 18 Name
        \\type alpha
        \\note This is the eighteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A24,  \\field Air Chiller 19 Name
        \\type alpha
        \\note This is the nineteenth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
  A25,  \\field Air Chiller 20 Name
        \\type alpha
        \\note This is the twentieth chiller turned on to meet the zone load
        \\object-list RefrigerationAirChillerNames
     A26,A27,A28,A29,A30,A31,A32,A33,A34,A35,A36,A37,A38,A39,A40,A41,A42,A43,A44,A45, \\note fields as indicated
     A46,A47,A48,A49,A50,A51,A52,A53,A54,A55,A56,A57,A58,A59,A60,A61,A62,A63,A64,A65, \\note fields as indicated
     A66,A67,A68,A69,A70,A71,A72,A73,A74,A75,A76,A77,A78,A79,A80,A81,A82,A83,A84,A85, \\note fields as indicated
     A86,A87,A88,A89,A90,A91,A92,A93,A94,A95,A96,A97,A98,A99,A100;                    \\note fields as indicated

\\Group Demand Limiting Controls

DemandManagerAssignmentList,
       \\extensible:2 Duplicate the last two fields DemandManager Object Type and DemandManager Name
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Meter Name
       \\required-field
       \\type external-list
       \\external-list autoRDDmeter
  A3 , \\field Demand Limit Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N1 , \\field Demand Limit Safety Fraction
       \\required-field
       \\type real
       \\minimum 0.0
  A4 , \\field Billing Period Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field should reference the same schedule as the month schedule name field of the
       \\note UtilityCost:Tariff object, if used.
       \\note If blank, defaults to regular divisions between months.
  A5 , \\field Peak Period Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note This field should reference the same schedule as the period schedule name field of the
       \\note UtilityCost:Tariff object, if used.
       \\note If blank, defaults to always on peak.
  N2 , \\field Demand Window Length
       \\required-field
       \\type integer
       \\minimum> 0
       \\units minutes
  A6 , \\field Demand Manager Priority
       \\required-field
       \\type choice
       \\key Sequential
       \\key All
  A7 , \\field DemandManager 1 Object Type
       \\begin-extensible
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A8 , \\field DemandManager 1 Name
       \\type object-list
       \\object-list DemandManagerNames
  A9 , \\field DemandManager 2 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A10, \\field DemandManager 2 Name
       \\type object-list
       \\object-list DemandManagerNames
  A11, \\field DemandManager 3 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A12, \\field DemandManager 3 Name
       \\type object-list
       \\object-list DemandManagerNames
  A13, \\field DemandManager 4 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A14, \\field DemandManager 4 Name
       \\type object-list
       \\object-list DemandManagerNames
  A15, \\field DemandManager 5 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A16, \\field DemandManager 5 Name
       \\type object-list
       \\object-list DemandManagerNames
  A17, \\field DemandManager 6 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A18, \\field DemandManager 6 Name
       \\type object-list
       \\object-list DemandManagerNames
  A19, \\field DemandManager 7 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A20, \\field DemandManager 7 Name
       \\type object-list
       \\object-list DemandManagerNames
  A21, \\field DemandManager 8 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A22, \\field DemandManager 8 Name
       \\type object-list
       \\object-list DemandManagerNames
  A23, \\field DemandManager 9 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A24, \\field DemandManager 9 Name
       \\type object-list
       \\object-list DemandManagerNames
  A25, \\field DemandManager 10 Object Type
       \\type choice
       \\key DemandManager:ExteriorLights
       \\key DemandManager:Lights
       \\key DemandManager:ElectricEquipment
       \\key DemandManager:Thermostats
  A26; \\field DemandManager 10 Name
       \\type object-list
       \\object-list DemandManagerNames

DemandManager:ExteriorLights,
       \\memo used for demand limiting Exterior:Lights objects.
       \\extensible:1 Duplicate the last field Exterior:Lights Name
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DemandManagerNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Limit Control
       \\required-field
       \\type choice
       \\key Off
       \\key Fixed
  N1 , \\field Minimum Limit Duration
       \\type integer
       \\minimum> 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  N2 , \\field Maximum Limit Fraction
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N3 , \\field Limit Step Change
       \\note Not yet implemented
       \\type real
  A4 , \\field Selection Control
       \\required-field
       \\type choice
       \\key All
       \\key RotateMany
       \\key RotateOne
  N4 , \\field Rotation Duration
       \\type integer
       \\minimum 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  A5 , \\field Exterior Lights 1 Name
       \\begin-extensible
       \\note Enter the name of an Exterior:Lights object.
       \\required-field
       \\type object-list
       \\object-list ExteriorLightsNames
  A6 , \\field Exterior Lights 2 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A7 , \\field Exterior Lights 3 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A8 , \\field Exterior Lights 4 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A9 , \\field Exterior Lights 5 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A10, \\field Exterior Lights 6 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A11, \\field Exterior Lights 7 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A12, \\field Exterior Lights 8 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A13, \\field Exterior Lights 9 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames
  A14; \\field Exterior Lights 10 Name
       \\note Enter the name of an Exterior:Lights object.
       \\type object-list
       \\object-list ExteriorLightsNames

DemandManager:Lights,
       \\memo used for demand limiting Lights objects.
       \\extensible:1 Duplicate the last field Lights Name
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DemandManagerNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Limit Control
       \\required-field
       \\type choice
       \\key Off
       \\key Fixed
  N1 , \\field Minimum Limit Duration
       \\type integer
       \\minimum> 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  N2 , \\field Maximum Limit Fraction
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N3 , \\field Limit Step Change
       \\note Not yet implemented
       \\type real
  A4 , \\field Selection Control
       \\required-field
       \\type choice
       \\key All
       \\key RotateMany
       \\key RotateOne
  N4 , \\field Rotation Duration
       \\type integer
       \\minimum 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  A5 , \\field Lights 1 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list LightsNames
  A6 , \\field Lights 2 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A7 , \\field Lights 3 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A8 , \\field Lights 4 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A9 , \\field Lights 5 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A10, \\field Lights 6 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A11, \\field Lights 7 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A12, \\field Lights 8 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A13, \\field Lights 9 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames
  A14; \\field Lights 10 Name
       \\note Enter the name of an Lights object.
       \\note if ZoneList option is used on the Lights object,
       \\note a single lights object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Lights Object Name>.
       \\type object-list
       \\object-list LightsNames

DemandManager:ElectricEquipment,
       \\memo used for demand limiting ElectricEquipment objects.
       \\extensible:1 Duplicate the last field ElectricEquipment Name
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DemandManagerNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Limit Control
       \\required-field
       \\type choice
       \\key Off
       \\key Fixed
  N1 , \\field Minimum Limit Duration
       \\type integer
       \\minimum> 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  N2 , \\field Maximum Limit Fraction
       \\type real
       \\minimum 0.0
       \\maximum 1.0
  N3 , \\field Limit Step Change
       \\note Not yet implemented
       \\type real
  A4 , \\field Selection Control
       \\required-field
       \\type choice
       \\key All
       \\key RotateMany
       \\key RotateOne
  N4 , \\field Rotation Duration
       \\type integer
       \\minimum 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  A5 , \\field Electric Equipment 1 Name
       \\begin-extensible
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\required-field
       \\type object-list
       \\object-list ElectricEquipmentNames
  A6 , \\field Electric Equipment 2 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A7 , \\field Electric Equipment 3 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A8 , \\field Electric Equipment 4 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A9 , \\field Electric Equipment 5 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A10, \\field Electric Equipment 6 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A11, \\field Electric Equipment 7 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A12, \\field Electric Equipment 8 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A13, \\field Electric Equipment 9 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames
  A14; \\field Electric Equipment 10 Name
       \\note Enter the name of an ElectricEquipment object.
       \\note if ZoneList option is used on the ElectricEquipment object,
       \\note a single equipment object from that assignment
       \\note can be selected by entering <Zone Name><space><Global ElectricEquipment Object Name>.
       \\type object-list
       \\object-list ElectricEquipmentNames

DemandManager:Thermostats,
       \\memo used for demand limiting ZoneControl:Thermostat objects.
       \\extensible:1 Duplicate the last field ZoneControl:Thermostat Name
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DemandManagerNames
  A2 , \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3 , \\field Reset Control
       \\required-field
       \\type choice
       \\key Off
       \\key Fixed
  N1 , \\field Minimum Reset Duration
       \\type integer
       \\minimum> 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  N2 , \\field Maximum Heating Setpoint Reset
       \\required-field
       \\type real
       \\units C
  N3 , \\field Maximum Cooling Setpoint Reset
       \\required-field
       \\type real
       \\units C
  N4 , \\field Reset Step Change
       \\note Not yet implemented
       \\type real
  A4 , \\field Selection Control
       \\required-field
       \\type choice
       \\key All
       \\key RotateMany
       \\key RotateOne
  N5 , \\field Rotation Duration
       \\type integer
       \\minimum 0
       \\units minutes
       \\note If blank, duration defaults to the timestep
  A5 , \\field Thermostat 1 Name
       \\begin-extensible
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\required-field
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A6 , \\field Thermostat 2 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A7 , \\field Thermostat 3 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A8 , \\field Thermostat 4 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A9 , \\field Thermostat 5 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A10, \\field Thermostat 6 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A11, \\field Thermostat 7 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A12, \\field Thermostat 8 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A13, \\field Thermostat 9 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames
  A14; \\field Thermostat 10 Name
       \\note Enter the name of a ZoneControl:Thermostat object.
       \\note if ZoneList option is used on the ZoneControl:Thermostat object,
       \\note a single thermostat object from that assignment
       \\note can be selected by entering <Zone Name><space><Global Thermostat Object Name>.
       \\type object-list
       \\object-list ZoneControlThermostaticNames


\\Group Electric Load Center-Generator Specifications

Generator:InternalCombustionEngine,
       \\memo This generator model is the empirical model from the Building Loads
       \\memo and System Thermodynamics (BLAST) program.  Engine performance
       \\memo curves are generated by fitting catalog data to second order
       \\memo polynomial equations.  Three sets of coefficients are required.
  A1, \\field Name
      \\required-field
      \\reference GeneratorNames
  N1, \\field Rated Power Output
       \\units W
  A2, \\field Electric Circuit Node Name
  N2, \\field Minimum Part Load Ratio
      \\minimum 0.0
      \\maximum 1.0
  N3, \\field Maximum Part Load Ratio
      \\minimum> 0.0
      \\maximum 1.0
  N4, \\field Optimum Part Load Ratio
  A3, \\field Shaft Power Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  A4, \\field Jacket Heat Recovery Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  A5, \\field Lube Heat Recovery Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  A6, \\field Total Exhaust Energy Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  A7, \\field Exhaust Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  N5, \\field Coefficient 1 of U-Factor Times Area Curve
       \\note curve = C1 * Generator Rated Power Output**C2
  N6, \\field Coefficient 2 of U-Factor Times Area Curve
       \\note curve = C1 * Generator Rated Power Output**C2
  N7, \\field Maximum Exhaust Flow per Unit of Power Output
       \\units (kg/s)/W
  N8, \\field Design Minimum Exhaust Temperature
       \\units C
  N9, \\field Fuel Higher Heating Value
       \\units kJ/kg
  N10, \\field Design Heat Recovery Water Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\note if non-zero, then inlet, outlet nodes must be entered.
       \\ip-units gal/min
  A8, \\field Heat Recovery Inlet Node Name
      \\type node
  A9, \\field Heat Recovery Outlet Node Name
      \\type node
  A10,\\field Fuel Type
       \\required-field
       \\type choice
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default Diesel
  N11; \\field Heat Recovery Maximum Temperature
       \\units C
       \\maximum 100.0
       \\minimum 0.0
       \\default 80.0

Generator:CombustionTurbine,
       \\memo This generator model is the empirical model from the Building Loads
       \\memo and System Thermodynamics (BLAST) program.  Generator performance
       \\memo curves are generated by fitting catalog data to second order
       \\memo polynomial equations.  Three sets of coefficients are required.
  A1, \\field Name
      \\required-field
      \\reference GeneratorNames
  N1, \\field Rated Power Output
      \\units W
  A2, \\field Electric Circuit Node Name
  N2, \\field Minimum Part Load Ratio
      \\minimum 0.0
      \\maximum 1.0
  N3, \\field Maximum Part Load Ratio
      \\minimum> 0.0
      \\maximum 1.0
  N4, \\field Optimum Part Load Ratio
  A3, \\field Part Load Based Fuel Input Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
       \\note this curve is multiplied to the Temperature Based Fuel Input Curve
       \\note to determine Fuel Energy In
  A4, \\field Temperature Based Fuel Input Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*AT + c*AT**2
       \\note AT = Ambient Delta T
       \\note this curve is multiplied to the Part Load Based Fuel Input Curve
       \\note to determine Fuel Energy In
  A5, \\field Exhaust Flow Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*AT + c*AT**2
       \\note AT = Ambient Delta T
  A6, \\field Part Load Based Exhaust Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
       \\note this curve is multiplied to the Temperature Based Exhaust Temperature Curve
       \\note to determine Exhaust Temperature
  A7, \\field Temperature Based Exhaust Temperature Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*AT + c*AT**2
       \\note AT = Ambient Delta T
       \\note this curve is multiplied to the Part Load Based Exhaust Temperatuer Curve
       \\note to determine Exhaust Temperature
  A8, \\field Heat Recovery Lube Energy Curve Name
       \\type object-list
       \\object-list QuadraticCurves
       \\note curve = a + b*PLR + c*PLR**2
       \\note PLR = Ratio of Generator Load to Rated Power Output
  N5, \\field Coefficient 1 of U-Factor Times Area Curve
      \\note curve = C1 * Rated Power Output**C2
  N6, \\field Coefficient 2 of U-Factor Times Area Curve
      \\note curve = C1 * Rated Power Output**C2
  N7, \\field Maximum Exhaust Flow per Unit of Power Output
      \\units (Kg/s)/W
  N8, \\field Design Minimum Exhaust Temperature
       \\units C
  N9, \\field Design Air Inlet Temperature
       \\units C
  N10, \\field Fuel Higher Heating Value
       \\units kJ/kg
  N11, \\field Design Heat Recovery Water Flow Rate
       \\units m3/s
       \\minimum 0.0
       \\default 0.0
       \\note if non-zero, then inlet, outlet nodes must be entered.
       \\ip-units gal/min
  A9 , \\field Heat Recovery Inlet Node Name
       \\type node
  A10, \\field Heat Recovery Outlet Node Name
       \\type node
  A11, \\field Fuel Type
       \\required-field
       \\type choice
       \\key NaturalGas
       \\key PropaneGas
       \\key Diesel
       \\key Gasoline
       \\key FuelOil#1
       \\key FuelOil#2
       \\key OtherFuel1
       \\key OtherFuel2
       \\default NaturalGas
  N12, \\field Heat Recovery Maximum Temperature
       \\units C
       \\maximum 100.0
       \\minimum 0.0
       \\default 80.0
  A12; \\field Outdoor Air Inlet Node Name
       \\type node
       \\note Enter the name of an outdoor air node

Generator:MicroTurbine,
      \\memo MicroTurbine generators are small combustion turbines (e.g., 25kW to 500kW). The model
      \\memo calculates electrical power output, fuel use, standby and ancillary power.
      \\memo Energy recovery from exhaust air can be used to heat water.
      \\min-fields 11
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference GeneratorNames
      \\reference MicroTurbineGeneratorNames
  N1, \\field Reference Electrical Power Output
      \\required-field
      \\type real
      \\units W
      \\minimum> 0.0
  N2, \\field Minimum Full Load Electrical Power Output
      \\type real
      \\units W
      \\minimum 0.0
      \\default 0.0
  N3, \\field Maximum Full Load Electrical Power Output
      \\type real
      \\units W
      \\minimum> 0.0
      \\note If left blank, Maximum Full Load Electrical Power Output will be set
      \\note equal to the Reference Electrical Power Output.
  N4, \\field Reference Electrical Efficiency Using Lower Heating Value
      \\required-field
      \\type real
      \\minimum> 0.0
      \\maximum 1.0
      \\note Electric power output divided by fuel energy input (LHV basis)
      \\note at reference conditions.
  N5, \\field Reference Combustion Air Inlet Temperature
      \\type real
      \\units C
      \\default 15.0
  N6, \\field Reference Combustion Air Inlet Humidity Ratio
      \\type real
      \\units kgWater/kgDryAir
      \\minimum> 0.0
      \\default 0.00638
  N7, \\field Reference Elevation
      \\type real
      \\units m
      \\minimum -300.0
      \\default 0.0
  A2, \\field Electrical Power Function of Temperature and Elevation Curve Name
      \\required-field
      \\type object-list
      \\object-list BiQuadraticCurves
      \\note curve = a + b*T + c*T**2 + d*Elev + e*Elev**2 + f*T*Elev
      \\note T = combustion air inlet temperature (C)
      \\note Elev = elevation (m)
  A3, \\field Electrical Efficiency Function of Temperature Curve Name
      \\required-field
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*T + c*T**2
      \\note Cubic curve = a + b*T + c*T**2 + d*T**3
      \\note T = combustion air inlet temperature (C)
  A4, \\field Electrical Efficiency Function of Part Load Ratio Curve Name
      \\required-field
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*PLR + c*PLR**2
      \\note Cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
      \\note PLR = ratio of Generator Load to steady state Electrical Power Output at
      \\note current operating conditions
  A5, \\field Fuel Type
      \\type choice
      \\key NaturalGas
      \\key PropaneGas
      \\default NaturalGas
  N8, \\field Fuel Higher Heating Value
      \\type real
      \\units kJ/kg
      \\default 50000
      \\minimum> 0.0
  N9, \\field Fuel Lower Heating Value
      \\type real
      \\units kJ/kg
      \\default 45450
      \\minimum> 0.0
 N10, \\field Standby Power
      \\type real
      \\units W
      \\default 0.0
      \\minimum 0.0
      \\note Electric power consumed when the generator is available but not being called
      \\note by the Electric Load Center.
 N11, \\field Ancillary Power
      \\type real
      \\units W
      \\default 0.0
      \\minimum 0.0
      \\note Electric power consumed by ancillary equipment (e.g., external fuel pressurization pump).
      \\note Set to zero if Reference Electrical Power Output is the 'net' value (ancillary power
      \\note already deducted). Input value is positive, but indicates negative electric generation.
  A6, \\field Ancillary Power Function of Fuel Input Curve Name
      \\type object-list
      \\object-list QuadraticCurves
      \\note Quadratic curve = a + b*mdot + c*mdot**2
      \\note mdot = fuel mass flow rate (kg/s)
      \\note If left blank, model assumes ancillary power defined in previous field is constant
      \\note whenever the generator is operating.
  A7, \\field Heat Recovery Water Inlet Node Name
      \\type node
  A8, \\field Heat Recovery Water Outlet Node Name
      \\type node
 N12, \\field Reference Thermal Efficiency Using Lower Heat Value
      \\type real
      \\minimum 0.0
      \\maximum 1.0
      \\default 0.0
      \\note Reference thermal efficiency (heat recovery to water) based on the
      \\note Lower Heating Value (LHV) of the fuel.
 N13, \\field Reference Inlet Water Temperature
      \\type real
      \\units C
  A9, \\field Heat Recovery Water Flow Operating Mode
      \\type choice
      \\key PlantControl
      \\key InternalControl
      \\default PlantControl
      \\note PlantControl means the heat recovery water flow rate is determined by the plant,
      \\note but the user needs to supply a heat recovery water flow rate.
      \\note InternalControl means the heat recovery water flow rate is controlled by this generator.
      \\note If 'InternalControl' is selected, then the user needs to supply a reference heat
      \\note recovery water flow rate and optionally the name of a heat recovery flow rate modifier curve.
 N14, \\field Reference Heat Recovery Water Flow Rate
      \\type real
      \\units m3/s
      \\minimum> 0.0
 A10, \\field Heat Recovery Water Flow Rate Function of Temperature and Power Curve Name
      \\type object-list
      \\object-list BiQuadraticCurves
      \\note curve = a + b*T + c*T**2 + d*Pnet + e*Pnet + f*T*Pnet
      \\note T = heat recovery inlet water temperature
      \\note Pnet = net power output = electric power output - ancillary power
      \\note If left blank, model assumes the heat recovery water flow rate is constant whenever the
      \\note generator is operating, at the Reference HR Water Flow Rate defined in the previous field.
 A11, \\field Thermal Efficiency Function of Temperature and Elevation Curve Name
      \\type object-list
      \\object-list BicubicBiquadraticCurves
      \\note Bicubic curve = a + b*T + c*T**2 + d*Elev + e*Elev**2 + f*T*Elev + g*T**3 + h*Elev**3 + i*T**2*Elev + j*T*Elev**2
      \\note Biquadratic curve = a + b*T + c*T**2 + d*Elev + e*Elev**2 + f*T*Elev
      \\note T = combustion air inlet temperature (C)
      \\note Elev = elevation (m)
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 A12, \\field Heat Recovery Rate Function of Part Load Ratio Curve Name
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*PLR + c*PLR**2
      \\note Cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
      \\note PLR = ratio of Generator Load to steady state Electrical Power Output at
      \\note current operating conditions
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 A13, \\field Heat Recovery Rate Function of Inlet Water Temperature Curve Name
      \\type object-list
      \\object-list QuadraticCurves
      \\note Quadratic curve = a + b*T + c*T**2
      \\note T = inlet water temperature (C)
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 A14, \\field Heat Recovery Rate Function of Water Flow Rate Curve Name
      \\type object-list
      \\object-list QuadraticCurves
      \\note Quadratic curve = a + b*Flow + c*Flow**2
      \\note Flow = flow rate of water through the heat exchanger (m3/s)
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 N15, \\field Minimum Heat Recovery Water Flow Rate
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.0
 N16, \\field Maximum Heat Recovery Water Flow Rate
      \\type real
      \\units m3/s
      \\minimum 0.0
      \\default 0.0
 N17, \\field Maximum Heat Recovery Water Temperature
      \\type real
      \\units C
 A15, \\field Combustion Air Inlet Node Name
      \\type node
      \\note Must be an outdoor air node.
 A16, \\field Combustion Air Outlet Node Name
      \\type node
 N18, \\field Reference Exhaust Air Mass Flow Rate
      \\type real
      \\units kg/s
      \\minimum> 0.0
 A17, \\field Exhaust Air Flow Rate Function of Temperature Curve Name
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*T + c*T**2
      \\note Cubic curve = a + b*T + c*T**2 + d*T**3
      \\note T = combustion air inlet temperature (C)
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 A18, \\field Exhaust Air Flow Rate Function of Part Load Ratio Curve Name
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*PLR + c*PLR**2
      \\note Cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
      \\note PLR = ratio of Generator Load to steady state Electrical Power Output at
      \\note current operating conditions.
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 N19, \\field Nominal Exhaust Air Outlet Temperature
      \\type real
      \\note Exhaust air outlet temperature at reference conditions.
 A19, \\field Exhaust Air Temperature Function of Temperature Curve Name
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*T + c*T**2
      \\note Cubic curve = a + b*T + c*T**2 + d*T**3
      \\note T = combustion air inlet temperature (C)
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.
 A20; \\field Exhaust Air Temperature Function of Part Load Ratio Curve Name
      \\type object-list
      \\object-list QuadraticCubicCurves
      \\note Quadratic curve = a + b*PLR + c*PLR**2
      \\note Cubic curve = a + b*PLR + c*PLR**2 + d*PLR**3
      \\note PLR = ratio of Generator Load to steady state Electrical Power Output at
      \\note current operating conditions.
      \\note If field is left blank, model assumes this modifier equals 1 for entire simulation.

Generator:Photovoltaic,
        \\memo Describes an array of photovoltaic (PV) modules.  A series of different PV arrays
        \\memo can be connected to a single electric load center (and inverter) by listing them all
        \\memo in an ElectricLoadCenter:Generator object. PV performance is taken from the
        \\memo referenced PhotovoltaicPerformance:* object. Array tilt, azimuth, and gross area
        \\memo are taken from the referenced building surface or shading surface. The array
        \\memo surface participates normally in all shading calculations.
   A1 , \\field Name
        \\required-field
        \\reference GeneratorNames
        \\reference PVGeneratorNames
   A2 , \\field Surface Name
        \\required-field
        \\type object-list
        \\object-list AllShadingAndHTSurfNames
   A3 , \\field Photovoltaic Performance Object Type
        \\type choice
        \\key PhotovoltaicPerformance:Simple
        \\key PhotovoltaicPerformance:EquivalentOne-Diode
        \\key PhotovoltaicPerformance:Sandia
   A4 , \\field Module Performance Name
        \\note PV array modeling details
        \\type object-list
        \\object-list PVModules
   A5 , \\field Heat Transfer Integration Mode
        \\type choice
        \\key Decoupled
        \\key DecoupledUllebergDynamic
        \\key IntegratedSurfaceOutsideFace
        \\key IntegratedTranspiredCollector
        \\key IntegratedExteriorVentedCavity
        \\key PhotovoltaicThermalSolarCollector
        \\default Decoupled
   N1 , \\field Number of Series Strings in Parallel
        \\note number of series-wired strings of PV modules that are in parallel
        \\default 1
        \\units dimensionless
        \\minimum 1
   N2 ; \\field Number of Modules in Series
        \\note Number of PV modules wired in series for each string.
        \\default 1
        \\units dimensionless
        \\minimum 1

PhotovoltaicPerformance:Simple,
        \\memo Describes a simple model of photovoltaics that may be useful for early phase
        \\memo design analysis. In this model the user has direct access to the efficiency with
        \\memo which surfaces convert incident solar radiation to electricity and need not specify
        \\memo arrays of specific modules.
   A1 , \\field Name
        \\reference PVModules
   N1 , \\field Fraction of Surface Area with Active Solar Cells
        \\required-field
        \\type real
        \\units dimensionless
        \\minimum 0.0
        \\maximum 1.0
   A2 , \\field Conversion Efficiency Input Mode
        \\type choice
        \\key Fixed
        \\key Scheduled
   N2 , \\field Value for Cell Efficiency if Fixed
        \\note Efficiency = (power generated [W])/(incident solar[W])
        \\type real
        \\minimum 0.0
        \\maximum 1.0
   A3 ; \\field Efficiency Schedule Name
        \\type object-list
        \\object-list ScheduleNames

PhotovoltaicPerformance:EquivalentOne-Diode,
        \\memo Describes the performance characteristics of Photovoltaic (PV) modules to be modeled
        \\memo using an equivalent one-diode circuit.  This model is also known as
        \\memo the 4- or 5-parameter TRNSYS model for photovoltaics.
   A1 , \\field Name
        \\reference PVModules
   A2 , \\field Cell type
        \\type choice
        \\key CrystallineSilicon
        \\key AmorphousSilicon
   N1 , \\field Number of Cells in Series
         \\default 36
         \\type integer
         \\units dimensionless
         \\minimum 0
   N2 , \\field Active Area
         \\note The total power output of the array is determined by the
         \\note number of modules (see above).  The Active Area is only
         \\note used to calculate the PV Array Efficiency output variable.
         \\default 0.89
         \\type real
         \\units m2
         \\minimum 0.1
   N3 , \\field Transmittance Absorptance Product
         \\default 0.95
         \\type real
         \\units dimensionless
         \\minimum 0.0
         \\maximum 1.0
   N4 , \\field Semiconductor Bandgap
         \\default 1.12
         \\type real
         \\units eV
         \\minimum 0.0
   N5 , \\field Shunt Resistance
         \\default 1000000.0
         \\type real
         \\units ohms
         \\minimum 0.0
   N6 , \\field Short Circuit Current
         \\default 6.5
         \\type real
         \\units A
         \\minimum 0.0
   N7, \\field Open Circuit Voltage
         \\default 21.6
         \\type real
         \\units V
         \\minimum 0.0
   N8, \\field Reference Temperature
         \\default 25
         \\type real
         \\units C
         \\minimum 0.0
   N9, \\field Reference Insolation
         \\default 1000
         \\type real
         \\units W/m2
         \\minimum 0.0
   N10, \\field Module Current at Maximum Power
         \\note Single module current at the maximum power point
         \\note and reference conditions.  Module Current, Module Voltage,
         \\note Number of Modules in Parallel and Number of Modules in Series
         \\note determine the maximum power output of the array.
         \\default 5.9
         \\type real
         \\units A
         \\minimum 0.0
   N11, \\field Module Voltage at Maximum Power
         \\note Single module voltage at the maximum power point
         \\note and reference conditions.  Module Current, Module Voltage,
         \\note Number of Modules in Parallel and Number of Modules in Series
         \\note determine the maximum power output of the array.
         \\default 17
         \\type real
         \\units V
         \\minimum 0.0
   N12, \\field Temperature Coefficient of Short Circuit Current
         \\default -0.079
         \\type real
         \\units A/K
   N13, \\field Temperature Coefficient of Open Circuit Voltage
         \\default 0.02
         \\type real
         \\units V/K
   N14, \\field Nominal Operating Cell Temperature Test Ambient Temperature
         \\default 20
         \\type real
         \\units C
         \\minimum 0.0
   N15, \\field Nominal Operating Cell Temperature Test Cell Temperature
         \\default 40
         \\type real
         \\units C
         \\minimum 0.0
   N16, \\field Nominal Operating Cell Temperature Test Insolation
         \\default 800
         \\type real
         \\units W/m2
         \\minimum 0.0
   N17, \\field Module Heat Loss Coefficient
         \\default 30
         \\type real
         \\units W/m2-K
         \\minimum 0.0
   N18; \\field Total Heat Capacity
         \\default 50000
         \\type real
         \\units J/m2-K
         \\minimum 0.0

PhotovoltaicPerformance:Sandia,
        \\memo Describes performance input data needed for specific makes and models of production
        \\memo PV panels using the empirical coefficients assembled by Sandia National Laboratory.
   A1 , \\field Name
        \\reference PVModules
   N1 , \\field Active Area
        \\note (m2, single module)
        \\default 1.0
        \\type real
        \\units m2
        \\minimum 0.0
   N2 , \\field Number of Cells in Series
        \\default 1
        \\type integer
        \\units dimensionless
        \\minimum 1
   N3 , \\field Number of Cells in Parallel
        \\default 1
        \\type integer
        \\units dimensionless
        \\minimum 1
   N4 , \\field Short Circuit Current
        \\note (Amps)
        \\type real
        \\units A
   N5 , \\field Open Circuit Voltage
        \\note (Volts)
        \\type real
        \\units V
   N6 , \\field Current at Maximum Power Point
        \\note (Amps)
        \\type real
        \\units A
   N7 , \\field Voltage at Maximum Power Point
        \\note (Volts)
        \\type real
        \\units V
   N8 , \\field Sandia Database Parameter aIsc
        \\note (1/degC)
        \\type real
        \\units 1/K
   N9 , \\field Sandia Database Parameter aImp
        \\note (1/degC)
        \\type real
        \\units 1/K
   N10 , \\field Sandia Database Parameter c0
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N11 , \\field Sandia Database Parameter c1
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N12 , \\field Sandia Database Parameter BVoc0
         \\note (Volts/degC)
         \\type real
         \\units V/K
   N13 , \\field Sandia Database Parameter mBVoc
         \\note (Volts/degC)
         \\type real
         \\units V/K
   N14 , \\field Sandia Database Parameter BVmp0
         \\note (Volts/degC)
         \\type real
         \\units V/K
   N15 , \\field Sandia Database Parameter mBVmp
         \\note (Volts/degC)
         \\type real
         \\units V/K
   N16 , \\field Diode Factor
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N17 , \\field Sandia Database Parameter c2
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N18 , \\field Sandia Database Parameter c3
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N19 , \\field Sandia Database Parameter a0
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N20 , \\field Sandia Database Parameter a1
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N21 , \\field Sandia Database Parameter a2
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N22 , \\field Sandia Database Parameter a3
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N23 , \\field Sandia Database Parameter a4
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N24 , \\field Sandia Database Parameter b0
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N25 , \\field Sandia Database Parameter b1
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N26 , \\field Sandia Database Parameter b2
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N27 , \\field Sandia Database Parameter b3
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N28 , \\field Sandia Database Parameter b4
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N29 , \\field Sandia Database Parameter b5
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N30 , \\field Sandia Database Parameter Delta(Tc)
         \\note (deg C)
         \\type real
         \\units deltaC
   N31 , \\field Sandia Database Parameter fd
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N32 , \\field Sandia Database Parameter a
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N33 , \\field Sandia Database Parameter b
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N34 , \\field Sandia Database Parameter c4
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N35 , \\field Sandia Database Parameter c5
         \\note (nondimensional)
         \\type real
         \\units dimensionless
   N36 , \\field Sandia Database Parameter Ix0
         \\note (Amps)
         \\type real
   N37 , \\field Sandia Database Parameter Ixx0
         \\note (Amps)
         \\type real
   N38 , \\field Sandia Database Parameter c6
         \\note (nondimensional)
         \\type real
   N39 ; \\field Sandia Database Parameter c7
         \\note (nondimensional)
         \\type real

Generator:FuelCell,
       \\memo This generator model is the FC model from IEA Annex 42
  A1, \\field Name
      \\required-field
      \\reference GeneratorNames
  A2, \\field Power Module Name
      \\note Enter then name of a Generator:FuelCell:PowerModule object.
      \\required-field
      \\type object-list
      \\object-list FCPMNames
  A3, \\field Air Supply Name
      \\note Enter then name of a Generator:FuelCell:AirSupply object.
      \\required-field
      \\type object-list
      \\object-list FCAirSupNames
  A4, \\field Fuel Supply Name
      \\note Enter then name of a Generator:FuelSupply object.
      \\required-field
      \\type object-list
      \\object-list GenFuelSupNames
  A5, \\field Water Supply Name
      \\note Enter then name of a Generator:FuelCell:WaterSupply object.
      \\required-field
      \\type object-list
      \\object-list FCWaterSupNames
  A6, \\field Auxiliary Heater Name
      \\note Enter the name of a Generator:FuelCell:AuxiliaryHeater object.
      \\required-field
      \\type object-list
      \\object-list FCAuxHeatNames
  A7, \\field Heat Exchanger Name
      \\note Enter the name of a Generator:FuelCell:ExhaustGasToWaterHeatExchanger object.
      \\required-field
      \\type object-list
      \\object-list FCExhaustHXNames
  A8, \\field Electrical Storage Name
      \\note Enter the name of a Generator:FuelCell:ElectricalStorage object.
      \\required-field
      \\type object-list
      \\object-list FCStorageNames
  A9, \\field Inverter Name
      \\note Enter the name of a Generator:FuelCell:Inverter object.
      \\required-field
      \\type object-list
      \\object-list FCInverterNames
  A10; \\field Stack Cooler Name
      \\note Enter the name of a Generator:FuelCell:StackCooler object.
      \\note optional, used for PEMFC
      \\type object-list
      \\object-list FCStackCoolerNames

Generator:FuelCell:PowerModule,
      \\memo Describe the core power module subsystem of a fuel cell power generator. This includes
      \\memo the fuel cell stack, fuel reformer, and whatever ancillary devices are included inside.
      \\memo If the model has multiple FC generators that are of the exact same type, then only one
      \\memo of these objects is needed and all the Generator:FuelCell objects can reference it.
  A1, \\field Name
      \\required-field
      \\reference FCPMNames
  A2, \\field Efficiency Curve Mode
      \\type choice
      \\key Annex42
      \\key Normalized
  A3, \\field Efficiency Curve Name
      \\type object-list
      \\object-list QuadraticCurves
      \\required-field
  N1, \\field Nominal Efficiency
      \\note This field is not used.
  N2, \\field Nominal Electrical Power
      \\note This field is not used
      \\units W
  N3, \\field Number of Stops at Start of Simulation
      \\note this is Nstops in SOFC model specification
  N4, \\field Cycling Performance Degradation Coefficient
      \\note this is D in SOFC model specification
  N5, \\field Number of Run Hours at Beginning of Simulation
      \\units hr
  N6, \\field Accumulated Run Time Degradation Coefficient
      \\note this is L in SOFC model specification
  N7, \\field Run Time Degradation Initiation Time Threshold
      \\units hr
  N8, \\field Power Up Transient Limit
      \\units W/s
      \\note Maximum rate of change in elecrical output [power increasing]
  N9, \\field Power Down Transient Limit
      \\units W/s
      \\note Maximum rate of change in elecrical output [power decreasing]
      \\note Enter positive value for rate of change
  N10,\\field Start Up Time
      \\units s
      \\note Time from start up to normal operation
  N11,\\field Start Up Fuel
      \\units kmol
  N12,\\field Start Up Electricity Consumption
      \\units J
  N13,\\field Start Up Electricity Produced
      \\units J
  N14,\\field Shut Down Time
      \\units s
  N15,\\field Shut Down Fuel
      \\units kmol
  N16,\\field Shut Down Electricity Consumption
      \\units J
  N17,\\field Ancilliary Electricity Constant Term
  N18,\\field Ancilliary Electricity Linear Term
  A4, \\field Skin Loss Calculation Mode
      \\type choice
      \\key ConstantRate
      \\key UAForProcessGasTemperature
      \\key QuadraticFunctionOfFuelRate
  A5, \\field Zone Name
      \\type object-list
      \\object-list ZoneNames
  N19,\\field Skin Loss Radiative Fraction
      \\minimum 0.0
      \\maximum 1.0
  N20,\\field Constant Skin Loss Rate
      \\units W
  N21,\\field Skin Loss U-Factor Times Area Term
      \\units W/K
  A6, \\field Skin Loss Quadratic Curve Name
      \\type object-list
      \\object-list QuadraticCurves
      \\note curve is function of fuel use rate
  N22,\\field Dilution Air Flow Rate
      \\units kmol/s
  N23,\\field Stack Heat loss to Dilution Air
      \\units W
  A7, \\field Dilution Inlet Air Node Name
      \\type node
  A8 ,\\field Dilution Outlet Air Node Name
      \\type node
  N24, \\field Minimum Operating Point
       \\units W
  N25; \\field Maximum Operating Point
       \\units W

Generator:FuelCell:AirSupply,
      \\memo Used to define details of the air supply subsystem for a fuel cell power generator.
      \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
  A1, \\field Name
      \\required-field
      \\reference FCAirSupNames
  A2, \\field Air Inlet Node Name
      \\type node
  A3, \\field Blower Power Curve Name
      \\type object-list
      \\object-list CubicCurves
  N1, \\field Blower Heat Loss Factor
      \\minimum 0.0
      \\maximum 1.0
  A4, \\field Air Supply Rate Calculation Mode
      \\required-field
      \\type choice
      \\key AirRatiobyStoics
      \\key QuadraticFunctionofElectricPower
      \\key QuadraticFunctionofFuelRate
  N2, \\field Stoichiometric Ratio
      \\note This is the excess air "stoics"
      \\note the value entered is incremented by 1 in the model.
  A5, \\field Air Rate Function of Electric Power Curve Name
      \\type object-list
      \\object-list QuadraticCurves
  N3, \\field Air Rate Air Temperature Coefficient
  A6, \\field Air Rate Function of Fuel Rate Curve Name
      \\type object-list
      \\object-list QuadraticCurves
  A7, \\field Air Intake Heat Recovery Mode
      \\required-field
      \\type choice
      \\key NoRecovery
      \\key RecoverBurnerInverterStorage
      \\key RecoverAuxiliaryBurner
      \\key RecoverInverterandStorage
      \\key RecoverInverter
      \\key RecoverElectricalStorage
  A8, \\field Air Supply Constituent Mode
      \\required-field
      \\type choice
      \\key AmbientAir
      \\key UserDefinedConstituents
  N4, \\field Number of UserDefined Constituents
      \\maximum 5
  A9, \\field Constituent 1 Name
      \\begin-extensible
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
  N5, \\field Molar Fraction 1
      \\minimum 0.0
      \\maximum 1.0
  A10,\\field Constituent 2 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
  N6, \\field Molar Fraction 2
      \\minimum 0.0
      \\maximum 1.0
  A11,\\field Constituent 3 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
  N7, \\field Molar Fraction 3
      \\minimum 0.0
      \\maximum 1.0
  A12,\\field Constituent 4 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
  N8, \\field Molar Fraction 4
      \\minimum 0.0
      \\maximum 1.0
  A13,\\field Constituent 5 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
  N9; \\field Molar Fraction 5
      \\minimum 0.0
      \\maximum 1.0

Generator:FuelCell:WaterSupply,
      \\memo Used to provide details of the water supply subsystem for a fuel cell power generator.
      \\memo This water is used for steam reforming of the fuel and is not the same
      \\memo as the water used for thermal heat recovery.
  A1, \\field Name
      \\required-field
      \\reference FCWaterSupNames
  A2, \\field Reformer Water Flow Rate Function of Fuel Rate Curve Name
      \\type object-list
      \\object-list QuadraticCurves
  A3, \\field Reformer Water Pump Power Function of Fuel Rate Curve Name
      \\type object-list
      \\object-list CubicCurves
  N1, \\field Pump Heat Loss Factor
  A4, \\field Water Temperature Modeling Mode
      \\type choice
      \\key TemperatureFromAirNode
      \\key TemperatureFromWaterNode
      \\key TemperatureFromSchedule
      \\key MainsWaterTemperature
  A5, \\field Water Temperature Reference Node Name
      \\type node
  A6; \\field Water Temperature Schedule Name
      \\type object-list
      \\object-list ScheduleNames

Generator:FuelCell:AuxiliaryHeater,
      \\memo Intended for modeling an auxiliary heater for a fuel cell power generator, however this
      \\memo portion of the model is not yet available. The program still requires one of these
      \\memo objects be included even though the data are not yet used (so that internal data
      \\memo structures can be allocated).
  A1, \\field Name
      \\required-field
      \\reference FCAuxHeatNames
  N1, \\field Excess Air Ratio
  N2, \\field Ancilliary Power Constant Term
  N3, \\field Ancilliary Power Linear Term
  N4, \\field Skin Loss U-Factor Times Area Value
      \\units W/K
  A2, \\field Skin Loss Destination
      \\type choice
      \\key SurroundingZone
      \\key AirInletForFuelCell
  A3, \\field Zone Name to Receive Skin Losses
      \\type object-list
      \\object-list ZoneNames
  A4, \\field Heating Capacity Units
      \\type choice
      \\key Watts
      \\key kmol/s
  N5, \\field Maximum Heating Capacity in Watts
      \\units W
  N6, \\field Minimum Heating Capacity in Watts
      \\units W
  N7, \\field Maximum Heating Capacity in Kmol per Second
      \\units kmol/s
  N8; \\field Minimum Heating Capacity in Kmol per Second
      \\units kmol/s

Generator:FuelCell:ExhaustGasToWaterHeatExchanger,
      \\memo Describes the exhaust gas heat exchanger subsystem of a fuel cell power generator
      \\memo used to recovery thermal energy
  A1, \\field Name
      \\required-field
      \\reference FCExhaustHXNames
  A2, \\field Heat Recovery Water Inlet Node Name
      \\type node
  A3, \\field Heat Recovery Water Outlet Node Name
      \\type node
  N1, \\field Heat Recovery Water Maximum Flow Rate
      \\units m3/s
  A4, \\field Exhaust Outlet Air Node Name
      \\type node
  A5, \\field Heat Exchanger Calculation Method
      \\type choice
      \\key FixedEffectiveness
      \\key EmpiricalUAeff
      \\key FundementalUAeff
      \\key Condensing
  N2, \\field Method 1 Heat Exchanger Effectiveness
  N3, \\field Method 2 Parameter hxs0
  N4, \\field Method 2 Parameter hxs1
  N5, \\field Method 2 Parameter hxs2
  N6, \\field Method 2 Parameter hxs3
  N7, \\field Method 2 Parameter hxs4
  N8, \\field Method 3 h0Gas Coefficient
  N9, \\field Method 3 NdotGasRef Coefficient
  N10, \\field Method 3 n Coefficient
  N11, \\field Method 3 Gas Area
       \\units m2
  N12, \\field Method 3 h0 Water Coefficient
  N13, \\field Method 3 N dot Water ref Coefficient
  N14, \\field Method 3 m Coefficient
  N15, \\field Method 3 Water Area
       \\units m2
  N16, \\field Method 3 F Adjustment Factor
  N17, \\field Method 4 hxl1 Coefficient
  N18, \\field Method 4 hxl2 Coefficient
  N19; \\field Method 4 Condensation Threshold
       \\units C

Generator:FuelCell:ElectricalStorage,
      \\memo Used to describe the electrical storage subsystem for a fuel cell power generator.
      \\memo The electrical storage model is a very simple "constrained bucket" model.
      \\memo Note that this electrical storage is embedded within the FC device.
  A1, \\field Name
      \\required-field
      \\reference FCStorageNames
  A2, \\field Choice of Model
      \\type choice
      \\key SimpleEfficiencyWithConstraints
  N1, \\field Nominal Charging Energetic Efficiency
      \\maximum 1.0
      \\minimum 0.0
  N2, \\field Nominal Discharging Energetic Efficiency
      \\maximum 1.0
      \\minimum 0.0
  N3, \\field Simple Maximum Capacity
      \\units J
  N4, \\field Simple Maximum Power Draw
      \\units W
  N5, \\field Simple Maximum Power Store
      \\units W
  N6; \\field Initial Charge State
      \\units J

Generator:FuelCell:Inverter,
      \\memo Used to describe the power condition unit subsystem of a fuel cell power generator.
      \\memo This object models an inverter system contained within a fuel cell system that
      \\memo converts from direct current (DC) to alternating current (AC).
  A1, \\field Name
      \\required-field
      \\reference FCInverterNames
  A2, \\field Inverter Efficiency Calculation Mode
      \\type choice
      \\key Quadratic
      \\key Constant
  N1, \\field Inverter Efficiency
      \\minimum 0.0
      \\maximum 1.0
  A3; \\field Efficiency Function of DC Power Curve Name
      \\type object-list
      \\object-list QuadraticCurves

 Generator:FuelCell:StackCooler,
      \\memo This object is optional and is used to define details needed to model the stack cooler
      \\memo on PEMFC.
  A1, \\field Name
      \\required-field
      \\type alpha
      \\reference FCStackCoolerNames
  A2, \\field Heat Recovery Water Inlet Node Name
      \\type node
  A3, \\field Heat Recovery Water Outlet Node Name
      \\type node
  N1,  \\field Nominal Stack Temperature
       \\units C
  N2,  \\field Actual Stack Temperature
       \\units C
  N3,  \\field Coefficient r0
  N4,  \\field Coefficient r1
  N5,  \\field Coefficient r2
  N6,  \\field Coefficient r3
  N7,  \\field Stack Coolant Flow Rate
       \\units kg/s
  N8,  \\field Stack Cooler U-Factor Times Area Value
       \\units W/K
  N9,  \\field Fs-cogen Adjustment Factor
  N10, \\field Stack Cogeneration Exchanger Area
       \\units m2
  N11, \\field Stack Cogeneration Exchanger Nominal Flow Rate
       \\units kg/s
  N12, \\field Stack Cogeneration Exchanger Nominal Heat Transfer Coefficient
       \\units W/m2-K
  N13, \\field Stack Cogeneration Exchanger Nominal Heat Transfer Coefficient Exponent
  N14, \\field Stack Cooler Pump Power
       \\units W
  N15, \\field Stack Cooler Pump Heat Loss Fraction
       \\minimum 0.0
       \\maximum 1.0
  N16, \\field Stack Air Cooler Fan Coefficient f0
  N17, \\field Stack Air Cooler Fan Coefficient f1
  N18; \\field Stack Air Cooler Fan Coefficient f2

Generator:MicroCHP,
      \\memo Small-scale combined heat and power (micro CHP) electric generator using the model
      \\memo developed by IEA/ECBCS Annex 42 - see www.cogen-sim.net. The model was developed
      \\memo for both internal combustion and Stirling cycle
      \\memo engines, but might be used for other types of residential CHP devices.
  A1, \\field Name
      \\reference MicroCHPNames
      \\reference GeneratorNames
  A2, \\field Performance Parameters Name
      \\note Enter the name of a Generator:MicroCHP:NonNormalizedParameters object.
      \\type object-list
      \\object-list MicroCHPParametersNames
  A3, \\field Zone Name
      \\type object-list
      \\object-list ZoneNames
  A4, \\field Cooling Water Inlet Node Name
      \\type node
  A5, \\field Cooling Water Outlet Node Name
      \\type node
  A6, \\field Air Inlet Node Name
      \\type node
  A7, \\field Air Outlet Node Name
      \\type node
  A8, \\field Generator Fuel Supply Name
      \\type object-list
      \\object-list GenFuelSupNames
      \\note Enter the name of a Generator:FuelSupply object.
  A9; \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames

Generator:MicroCHP:NonNormalizedParameters,
      \\memo This object is referenced by a Generator:MicroCHP object and provides the
      \\memo non-normalized parameters for the MicroCHP generator model.
  A1, \\field Name
      \\reference MicroCHPParametersNames
  N1, \\field Maximum Electric Power
      \\units W
  N2, \\field Minimum Electric Power
      \\units W
  N3, \\field Minimum Cooling Water Flow Rate
      \\units kg/s
  N4, \\field Maximum Cooling Water Temperature
      \\units C
  A2, \\field Electrical Efficiency Curve Name
      \\note TriQuadratic
      \\type object-list
      \\object-list TriQuadraticCurves
  A3, \\field Thermal Efficiency Curve Name
      \\note TriQuadratic
      \\type object-list
      \\object-list TriQuadraticCurves
  A4, \\field Cooling Water Flow Rate Mode
      \\type choice
      \\key PlantControl
      \\key InternalControl
  A5, \\field Cooling Water Flow Rate Curve Name
      \\type object-list
      \\object-list BiquadraticCurves
  A6, \\field Air Flow Rate Curve Name
      \\type object-list
       \\object-list QuadraticCurves
  N5,  \\field Maximum Net Electrical Power Rate of Change
       \\units W/s
  N6,  \\field Maximum Fuel Flow Rate of Change
       \\units kg/s2
  N7,  \\field Heat Exchanger U-Factor Times Area Value
       \\units W/K
  N8,  \\field Skin Loss U-Factor Times Area Value
       \\units W/K
  N9,  \\field Skin Loss Radiative Fraction
       \\type real
  N10,  \\field Aggregated Thermal Mass of Energy Conversion Portion of Generator
       \\units W/K
       \\minimum> 0.0
  N11,  \\field Aggregated Thermal Mass of Heat Recovery Portion of Generator
       \\units W/K
       \\minimum> 0.0
  N12, \\field Standby Power
       \\units W
  A7, \\field Warm Up Mode
       \\type choice
       \\key NominalEngineTemperature
       \\key TimeDelay
       \\note Stirling engines use Nominal Engine Temperature
       \\note Internal combustion engines use Time Delay
  N13, \\field Warm Up Fuel Flow Rate Coefficient
  N14, \\field Nominal Engine Operating Temperature
       \\units C
  N15, \\field Warm Up Power Coefficient
  N16, \\field Warm Up Fuel Flow Rate Limit Ratio
  N17, \\field Warm Up Delay Time
       \\units s
  N18, \\field Cool Down Power
       \\units W
  N19, \\field Cool Down Delay Time
       \\units s
  A8; \\field Restart Mode
       \\type choice
       \\key MandatoryCoolDown
       \\key OptionalCoolDown

Generator:FuelSupply,
      \\memo Used only with Generator:FuelCell and Generator:MicroCHP
  A1, \\field Name
      \\required-field
      \\reference GenFuelSupNames
  A2, \\field Fuel Temperature Modeling Mode
      \\type choice
      \\key TemperatureFromAirNode
      \\key Scheduled
  A3, \\field Fuel Temperature Reference Node Name
      \\type node
  A4, \\field Fuel Temperature Schedule Name
      \\type alpha
      \\object-list ScheduleNames
  A5, \\field Compressor Power Function of Fuel Rate Curve Name
      \\type object-list
      \\object-list CubicCurves
  N1, \\field Compressor Heat Loss Factor
      \\minimum 0.0
      \\maximum 1.0
  A6, \\field Fuel Type
      \\type choice
      \\key GaseousConstituents
      \\key LiquidGeneric
  N2, \\field Liquid Generic Fuel Lower Heating Value
      \\units kJ/kg
  N3, \\field Liquid Generic Fuel Higher Heating Value
      \\units kJ/kg
  N4, \\field Liquid Generic Fuel Molecular Weight
      \\units g/mol
  N5, \\field Liquid Generic Fuel CO2 Emission Factor
  N6, \\field Number of Constituents in Gaseous Constituent Fuel Supply
      \\minimum 0.0
      \\maximum 12.0
  ! sets of input data on each constituent
  A7, \\field Constituent 1 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N7, \\field Constituent 1 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A8, \\field Constituent 2 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N8, \\field Constituent 2 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A9, \\field Constituent 3 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N9, \\field Constituent 3 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A10, \\field Constituent 4 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N10, \\field Constituent 4 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A11, \\field Constituent 5 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N11, \\field Constituent 5 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A12, \\field Constituent 6 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N12, \\field Constituent 6 Molar Fraction
        \\minimum 0.0
      \\maximum 1.0
  A13, \\field Constituent 7 Name
      \\type choice
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N13, \\field Constituent 7 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A14, \\field Constituent 8 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N14, \\field Constituent 8 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A15, \\field Constituent 9 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N15, \\field Constituent 9 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A16, \\field Constituent 10 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N16, \\field Constituent 10 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A17, \\field Constituent 11 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N17, \\field Constituent 11 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0
  A18, \\field Constituent 12 Name
      \\type choice
      \\key CarbonDioxide
      \\key Nitrogen
      \\key Oxygen
      \\key Water
      \\key Argon
      \\key Hydrogen
      \\key Methane
      \\key Ethane
      \\key Propane
      \\key Butane
      \\key Pentane
      \\key Hexane
      \\key Methanol
      \\key Ethanol
  N18; \\field Constituent 12 Molar Fraction
      \\minimum 0.0
      \\maximum 1.0

Generator:WindTurbine,
      \\memo Wind turbine generator.
      \\min-fields 26
  A1, \\field Name
      \\required-field
      \\reference GeneratorNames
  A2, \\field Availability Schedule Name
      \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
      \\note If this field is blank, the system is always available.
      \\type object-list
      \\object-list ScheduleNames
  A3, \\field Rotor Type
      \\required-field
      \\note allowed values are: Horizontal Axis Wind Turbine or Vertical Axis Wind Turbine
      \\type choice
      \\key HorizontalAxisWindTurbine
      \\key VerticalAxisWindTurbine
      \\default HorizontalAxisWindTurbine
  A4, \\field Power Control
      \\required-field
      \\note Constant power output is obtained in the last three control types
      \\note when the wind speed exceeds the rated wind speed.
      \\note allowd values are: Fixed Speed Fixed Pitch, Fixed Speed Variable Pitch,
      \\note Variable Speed Fixed Pitch or Variable Speed Variable Pitch
      \\type choice
      \\key FixedSpeedFixedPitch
      \\key FixedSpeedVariablePitch
      \\key VariableSpeedFixedPitch
      \\key VariableSpeedVariablePitch
      \\default VariableSpeedVariablePitch
  N1, \\field Rated Rotor Speed
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units rev/min
  N2, \\field Rotor Diameter
      \\note This field is the diameter of the perpendicular circle of the Vertical Axis Wind Turbine system
      \\note from the upright pole on the ground.
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m
  N3, \\field Overall Height
      \\note This field is the height of the hub for the Horizontal Axis Wind Turbines and
      \\note of the pole for the Vertical Axis Wind Turbines.
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m
  N4, \\field Number of Blades
      \\required-field
      \\minimum 2
      \\default 3
  N5, \\field Rated Power
      \\required-field
      \\note This field is the nominal power at the rated wind speed.
      \\note Users should input maximum power in case of Fixed Speed Fixed Pitch control type.
      \\type real
      \\minimum> 0.0
      \\units W
  N6, \\field Rated Wind Speed
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m/s
  N7, \\field Cut In Wind Speed
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m/s
  N8, \\field Cut Out Wind Speed
      \\required-field
      \\type real
      \\minimum> 0.0
      \\units m/s
  N9, \\field Fraction system Efficiency
      \\type real
      \\minimum> 0.0
      \\maximum 1.0
      \\default 0.835
  N10,\\field Maximum Tip Speed Ratio
      \\required-field
      \\type real
      \\minimum> 0.0
      \\maximum 12.0
      \\default 5.0
  N11,\\field Maximum Power Coefficient
      \\note This field should be input if the rotor type is Horizontal Axis Wind Turbine
      \\type real
      \\minimum> 0.0
      \\maximum 0.59
      \\default 0.25
  N12,\\field Annual Local Average Wind Speed
      \\type real
      \\units m/s
      \\minimum> 0.0
  N13,\\field Height for Local Average Wind Speed
      \\type real
      \\units m
      \\minimum> 0.0
      \\default 50.0
  N14,\\field Blade Chord Area
      \\type real
      \\units m2
  N15,\\field Blade Drag Coefficient
      \\note This field is only for Vertical Axis Wind Turbine..
      \\note The user must input this field if the rotor type is Vertical Axis Wind Turbine.
      \\type real
      \\default 0.9
  N16,\\field Blade Lift Coefficient
      \\note This field is only for Vertical Axis Wind Turbine..
      \\note The user must input this field if the rotor type is Vertical Axis Wind Turbine.
      \\type real
      \\default 0.05
  N17,\\field Power Coefficient C1
      \\note This field is only available for Horizontal Axis Wind Turbine.
      \\note The user should input all six parameters
      \\note so that the analytic approximation is assumed.
      \\note The simple approximation will be assumed,
      \\note if any field C1 through C6 is not input.
      \\note Leave this field blank, if the manufacturers's data is unavailable
      \\note so that the simple approximation will be assumed.
      \\minimum> 0.0
      \\type real
      \\default 0.5176
  N18,\\field Power Coefficient C2
      \\minimum> 0.0
      \\type real
      \\default 116.0
  N19,\\field Power Coefficient C3
      \\minimum> 0.0
      \\type real
      \\default 0.4
  N20,\\field Power Coefficient C4
      \\minimum 0.0
      \\type real
      \\default 0.0
  N21,\\field Power Coefficient C5
      \\minimum> 0.0
      \\type real
      \\default 5.0
  N22;\\field Power Coefficient C6
      \\minimum> 0.0
      \\type real
      \\default 21.0

ElectricLoadCenter:Generators,
       \\memo List of electric power generators to include in the simulation including the name and
       \\memo type of each generators along with availability schedule, rated power output,
       \\memo and thermal-to-electrical power ratio.
       \\extensible:5 - repeat set of five fields, remembering to remove ; from "inner" fields.
       \\min-fields 6
  A1 , \\field Name
       \\required-field
       \\reference GeneratorLists
  A2 , \\field Generator 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list GeneratorNames
  A3 , \\field Generator 1 Object Type
       \\required-field
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N1 , \\field Generator 1 Rated Electric Power Output
       \\type real
       \\units W
  A4 , \\field Generator 1 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N2 , \\field Generator 1 Rated Thermal to Electrical Power Ratio
  A5 , \\field Generator 2 Name
       \\type object-list
       \\object-list GeneratorNames
  A6 , \\field Generator 2 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N3 , \\field Generator 2 Rated Electric Power Output
       \\type real
       \\units W
  A7,  \\field Generator 2 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N4,  \\field Generator 2 Rated Thermal to Electrical Power Ratio
  A8,  \\field Generator 3 Name
       \\type object-list
       \\object-list GeneratorNames
  A9,  \\field Generator 3 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N5,  \\field Generator 3 Rated Electric Power Output
       \\type real
       \\units W
  A10, \\field Generator 3 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N6,  \\field Generator 3 Rated Thermal to Electrical Power Ratio
  A11, \\field Generator 4 Name
       \\type object-list
       \\object-list GeneratorNames
  A12, \\field Generator 4 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N7,  \\field Generator 4 Rated Electric Power Output
       \\type real
       \\units W
  A13, \\field Generator 4 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N8,  \\field Generator 4 Rated Thermal to Electrical Power Ratio
  A14, \\field Generator 5 Name
       \\type object-list
       \\object-list GeneratorNames
  A15, \\field Generator 5 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
   N9, \\field Generator 5 Rated Electric Power Output
       \\type real
       \\units W
  A16, \\field Generator 5 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N10, \\field Generator 5 Rated Thermal to Electrical Power Ratio
  A17, \\field Generator 6 Name
       \\type object-list
       \\object-list GeneratorNames
  A18, \\field Generator 6 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N11, \\field Generator 6 Rated Electric Power Output
       \\type real
       \\units W
  A19, \\field Generator 6 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N12,  \\field Generator 6 Rated Thermal to Electrical Power Ratio
  A20, \\field Generator 7 Name
       \\type object-list
       \\object-list GeneratorNames
  A21, \\field Generator 7 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N13, \\field Generator 7 Rated Electric Power Output
       \\type real
       \\units W
  A22, \\field Generator 7 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N14, \\field Generator 7 Rated Thermal to Electrical Power Ratio
  A23, \\field Generator 8 Name
       \\type object-list
       \\object-list GeneratorNames
  A24, \\field Generator 8 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N15, \\field Generator 8 Rated Electric Power Output
       \\type real
       \\units W
  A25, \\field Generator 8 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N16, \\field Generator 8 Rated Thermal to Electrical Power Ratio
  A26, \\field Generator 9 Name
       \\type object-list
       \\object-list GeneratorNames
  A27, \\field Generator 9 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N17, \\field Generator 9 Rated Electric Power Output
       \\type real
       \\units W
  A28, \\field Generator 9 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N18, \\field Generator 9 Rated Thermal to Electrical Power Ratio
  A29, \\field Generator 10 Name
       \\type object-list
       \\object-list GeneratorNames
  A30, \\field Generator 10 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N19, \\field Generator 10 Rated Electric Power Output
       \\type real
       \\units W
  A31, \\field Generator 10 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N20, \\field Generator 10 Rated Thermal to Electrical Power Ratio
  A32, \\field Generator 11 Name
       \\type object-list
       \\object-list GeneratorNames
  A33, \\field Generator 11 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N21, \\field Generator 11 Rated Electric Power Output
       \\type real
       \\units W
  A34, \\field Generator 11 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N22, \\field Generator 11 Rated Thermal to Electrical Power Ratio
  A35, \\field Generator 12 Name
       \\type object-list
       \\object-list GeneratorNames
  A36, \\field Generator 12 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N23, \\field Generator 12 Rated Electric Power Output
       \\type real
       \\units W
  A37, \\field Generator 12 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N24, \\field Generator 12 Rated Thermal to Electrical Power Ratio
  A38, \\field Generator 13 Name
       \\type object-list
       \\object-list GeneratorNames
  A39, \\field Generator 13 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N25, \\field Generator 13 Rated Electric Power Output
       \\type real
       \\units W
  A40, \\field Generator 13 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N26, \\field Generator 13 Rated Thermal to Electrical Power Ratio
  A41, \\field Generator 14 Name
       \\type object-list
       \\object-list GeneratorNames
  A42, \\field Generator 14 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N27, \\field Generator 14 Rated Electric Power Output
       \\type real
       \\units W
  A43, \\field Generator 14 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N28, \\field Generator 14 Rated Thermal to Electrical Power Ratio
  A44, \\field Generator 15 Name
       \\type object-list
       \\object-list GeneratorNames
  A45, \\field Generator 15 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N29, \\field Generator 15 Rated Electric Power Output
       \\type real
       \\units W
  A46, \\field Generator 15 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N30, \\field Generator 15 Rated Thermal to Electrical Power Ratio
  A47, \\field Generator 16 Name
       \\type object-list
       \\object-list GeneratorNames
  A48, \\field Generator 16 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N31, \\field Generator 16 Rated Electric Power Output
       \\type real
       \\units W
  A49, \\field Generator 16 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N32, \\field Generator 16 Rated Thermal to Electrical Power Ratio
  A50, \\field Generator 17 Name
       \\type object-list
       \\object-list GeneratorNames
  A51, \\field Generator 17 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N33, \\field Generator 17 Rated Electric Power Output
       \\type real
       \\units W
  A52, \\field Generator 17 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N34, \\field Generator 17 Rated Thermal to Electrical Power Ratio
  A53, \\field Generator 18 Name
       \\type object-list
       \\object-list GeneratorNames
  A54, \\field Generator 18 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N35, \\field Generator 18 Rated Electric Power Output
       \\type real
       \\units W
  A55, \\field Generator 18 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N36, \\field Generator 18 Rated Thermal to Electrical Power Ratio
  A56, \\field Generator 19 Name
       \\type object-list
       \\object-list GeneratorNames
  A57, \\field Generator 19 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N37, \\field Generator 19 Rated Electric Power Output
       \\type real
       \\units W
  A58, \\field Generator 19 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N38, \\field Generator 19 Rated Thermal to Electrical Power Ratio
  A59, \\field Generator 20 Name
       \\type object-list
       \\object-list GeneratorNames
  A60, \\field Generator 20 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N39, \\field Generator 20 Rated Electric Power Output
       \\type real
       \\units W
  A61, \\field Generator 20 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N40, \\field Generator 20 Rated Thermal to Electrical Power Ratio
  A62, \\field Generator 21 Name
       \\type object-list
       \\object-list GeneratorNames
  A63, \\field Generator 21 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N41, \\field Generator 21 Rated Electric Power Output
       \\type real
       \\units W
  A64, \\field Generator 21 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N42, \\field Generator 21 Rated Thermal to Electrical Power Ratio
  A65, \\field Generator 22 Name
       \\type object-list
       \\object-list GeneratorNames
  A66, \\field Generator 22 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N43, \\field Generator 22 Rated Electric Power Output
       \\type real
       \\units W
  A67, \\field Generator 22 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N44, \\field Generator 22 Rated Thermal to Electrical Power Ratio
  A68, \\field Generator 23 Name
       \\type object-list
       \\object-list GeneratorNames
  A69, \\field Generator 23 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N45, \\field Generator 23 Rated Electric Power Output
       \\type real
       \\units W
  A70, \\field Generator 23 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N46, \\field Generator 23 Rated Thermal to Electrical Power Ratio
  A71, \\field Generator 24 Name
       \\type object-list
       \\object-list GeneratorNames
  A72, \\field Generator 24 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N47, \\field Generator 24 Rated Electric Power Output
       \\type real
       \\units W
  A73, \\field Generator 24 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N48, \\field Generator 24 Rated Thermal to Electrical Power Ratio
  A74, \\field Generator 25 Name
       \\type object-list
       \\object-list GeneratorNames
  A75, \\field Generator 25 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N49, \\field Generator 25 Rated Electric Power Output
       \\type real
       \\units W
  A76, \\field Generator 25 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N50, \\field Generator 25 Rated Thermal to Electrical Power Ratio
  A77, \\field Generator 26 Name
       \\type object-list
       \\object-list GeneratorNames
  A78, \\field Generator 26 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N51, \\field Generator 26 Rated Electric Power Output
       \\type real
       \\units W
  A79, \\field Generator 26 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N52, \\field Generator 26 Rated Thermal to Electrical Power Ratio
  A80, \\field Generator 27 Name
       \\type object-list
       \\object-list GeneratorNames
  A81, \\field Generator 27 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N53, \\field Generator 27 Rated Electric Power Output
       \\type real
       \\units W
  A82, \\field Generator 27 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N54, \\field Generator 27 Rated Thermal to Electrical Power Ratio
  A83, \\field Generator 28 Name
       \\type object-list
       \\object-list GeneratorNames
  A84, \\field Generator 28 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N55, \\field Generator 28 Rated Electric Power Output
       \\type real
       \\units W
  A85, \\field Generator 28 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N56, \\field Generator 28 Rated Thermal to Electrical Power Ratio
  A86, \\field Generator 29 Name
       \\type object-list
       \\object-list GeneratorNames
  A87, \\field Generator 29 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N57, \\field Generator 29 Rated Electric Power Output
       \\type real
       \\units W
  A88, \\field Generator 29 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N58, \\field Generator 29 Rated Thermal to Electrical Power Ratio
  A89, \\field Generator 30 Name
       \\type object-list
       \\object-list GeneratorNames
  A90, \\field Generator 30 Object Type
       \\type choice
       \\key Generator:InternalCombustionEngine
       \\key Generator:CombustionTurbine
       \\key Generator:Photovoltaic
       \\key Generator:FuelCell
       \\key Generator:MicroCHP
       \\key Generator:MicroTurbine
       \\key Generator:WindTurbine
  N59, \\field Generator 30 Rated Electric Power Output
       \\type real
       \\units W
  A91, \\field Generator 30 Availability Schedule Name
       \\note Availability schedule name for this generator. Schedule value > 0 means the generator is available.
       \\note If this field is blank, the generator is always available.
       \\type object-list
       \\object-list ScheduleNames
  N60; \\field Generator 30 Rated Thermal to Electrical Power Ratio

ElectricLoadCenter:Inverter:Simple,
        \\memo Electric power inverter to convert from direct current (DC) to alternating current
        \\memo (AC) in an electric load center that contains photovoltaic modules. This input
        \\memo object is for the simplest inverter model and uses a fixed efficiency.
   A1 , \\field Name
        \\reference InverterList
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note enter name of zone to receive inverter losses as heat
        \\note if blank then inverter is assumed to be outdoors
        \\type object-list
        \\object-list ZoneNames
   N1 , \\field Radiative Fraction
        \\minimum 0
        \\maximum 1.0
   N2 ; \\field Inverter Efficiency
        \\minimum 0
        \\maximum 1.0

ElectricLoadCenter:Inverter:FunctionOfPower,
        \\memo Electric power inverter to convert from direct current (DC) to alternating current
        \\memo (AC) in an electric load center that contains photovoltaic modules. This input
        \\memo object is for an inverter model where efficiency is a function of normalized
        \\memo power.
   A1 , \\field Name
        \\reference InverterList
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Enter name of zone to receive inverter losses as heat
        \\note if blank then inverter is assumed to be outdoors
        \\type object-list
        \\object-list ZoneNames
   N1 , \\field Radiative Fraction
   A4 , \\field Efficiency Function of Power Curve Name
        \\note curve describes efficiency as a function of power
        \\note curve is normalized relative to rated power in next field
        \\object-list LinearCurves
        \\object-list QuadraticCurves
        \\object-list CubicCurves
   N2,  \\field Rated Maximum Continuous Input Power
        \\units W
   N3 , \\field Minimum Efficiency
        \\minimum 0
        \\maximum 1.0
   N4 , \\field Maximum Efficiency
        \\minimum 0
        \\maximum 1.0
   N5 , \\field Minimum Power Output
        \\units W
   N6 , \\field Maximum Power Output
        \\units W
   N7 ; \\field Ancillary Power Consumed In Standby
        \\units W

ElectricLoadCenter:Inverter:LookUpTable,
        \\memo   California Energy Commission tests and publishs data on inverters
        \\memo   This inverter model interpolates using CEC test data
        \\memo   Input data are at http://www.gosolarcalifornia.org/equipment/inverter_tests/summaries
   A1 , \\field Name
        \\reference InverterList
   A2 , \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3 , \\field Zone Name
        \\note Enter name of zone to receive inverter losses as heat
        \\note if blank then inverter is assumed to be outdoors
        \\type object-list
        \\object-list ZoneNames
   N1 , \\field Radiative Fraction
        \\minimum 0
        \\maximum 1.0
   N2 , \\field Rated Maximum Continuous Output Power
        \\units W
   N3 , \\field Night Tare Loss Power
        \\units W
   N4 , \\field Nominal Voltage Input
        \\units V
   N5 , \\field Efficiency at 10% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0
   N6 , \\field Efficiency at 20% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0
   N7 , \\field Efficiency at 30% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0
   N8 , \\field Efficiency at 50% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0
   N9 , \\field Efficiency at 75% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0
   N10; \\field Efficiency at 100% Power and Nominal Voltage
        \\minimum 0
        \\maximum 1.0

ElectricLoadCenter:Storage:Simple,
       \\memo Used to model storage of electricity in an electric load center.  This is a simple
       \\memo model that does not attempt to represent any of the characteristics of a real
       \\memo storage device such as a battery.  The type of power, AC or DC, depends on
       \\memo the configuration chosen as the Electrical Buss Type in the
       \\memo ElectricLoadCenter:Distribution object.
  A1,  \\field Name
       \\reference ElecStorageList
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Zone Name
       \\note Enter name of zone to receive storage losses as heat
       \\note if blank then storage is assumed to be outdoors
       \\type object-list
       \\object-list ZoneNames
  N1 , \\field Radiative Fraction for Zone Heat Gains
       \\maximum 1.0
       \\minimum 0.0
  N2,  \\field Nominal Energetic Efficiency for Charging
       \\maximum 1.0
       \\minimum 0.0
  N3,  \\field Nominal Discharging Energetic Efficiency
       \\maximum 1.0
       \\minimum 0.0
  N4,  \\field Maximum Storage Capacity
       \\units J
  N5,  \\field Maximum Power for Discharging
        \\units W
  N6,  \\field Maximum Power for Charging
        \\units W
  N7;  \\field Initial State of Charge
        \\units J

ElectricLoadCenter:Storage:Battery,
        \\memo Uses the kinetic battery model (KiBaM) to simulate rechargeable battery banks in an
        \\memo electrical load center. The battery bank is a collection of one or more individual
        \\memo battery modules. Given the surplus or deficit power from the electrical system and
        \\memo the state of charge from the previous time step, this object can model the voltage,
        \\memo current, and energy losses with charging and discharging during each time step.
        \\memo The cumulative battery damage can be also modeled and reported at the end of
        \\memo each simulation run.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference ElecStorageList
   A2,  \\field Availability Schedule Name
        \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
        \\note If this field is blank, the system is always available.
        \\type object-list
        \\object-list ScheduleNames
   A3,  \\field Zone Name
        \\type object-list
        \\object-list ZoneNames
        \\note Enter name of zone to receive electrical storage losses as heat
        \\note if blank then electrical storage losses are dissipated to outdoors
   N1,  \\field Radiative Fraction
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\default 0
   N2,  \\field Number of Battery Modules in Parallel
        \\type integer
        \\minimum 1
        \\default 1
        \\note A module usually consists of several cells.
        \\note The total number of modules in the battery bank
        \\note is equal to number of modules in parallel times
        \\note number of modules in series.
   N3,  \\field Number of Battery Modules in Series
        \\type integer
        \\minimum 1
        \\default 1
        \\note A module usually consists of several cells.
        \\note The total number of modules in the battery bank
        \\note is equal to number of modules in parallel times
        \\note number of modules in series.
   N4,  \\field Maximum Module Capacity
        \\type real
        \\minimum 0
        \\units Ah
        \\note The capacity is for each module.
        \\note A model parameter from manufacture's data or test data.
   N5,  \\field Initial Fractional State of Charge
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\default 1.0
        \\note The state of charge is evaluated based on the
        \\note maximum capacity defined in the next field.
   N6,  \\field Fraction of Available Charge Capacity
        \\type real
        \\minimum 0
        \\maximum 1.0
        \\note A model parameter usually derived from test data by curve fitting.
   N7,  \\field Change Rate from Bound Charge to Available Charge
        \\minimum 0
        \\units 1/hr
        \\note A model parameter usually derived from test data by curve fitting.
   N8,  \\field Fully Charged Module Open Circuit Voltage
        \\type real
        \\units V
        \\minimum 0
        \\note The voltage is for each battery module.
   N9, \\field Fully Discharged Module Open Circuit Voltage
        \\type real
        \\units V
        \\minimum 0
        \\note The voltage is for each battery module.
   A4, \\field Voltage Change Curve Name for Charging
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Determines how the open circuit voltage change with state of charge relative to the fully discharged state.
   A5, \\field Voltage Change Curve Name for Discharging
        \\type object-list
        \\object-list RectangularHyperbola2Curves
        \\note Determines how the open circuit voltage change with state of charge relative to the fully charged state.
   N10, \\field Module Internal Electrical Resistance
        \\type real
        \\units ohms
        \\minimum 0
        \\note A model parameter from manufacture or derived from test data.
        \\note Internal resistance is assumed to be constant.
        \\note The internal resistance is for each battery module.
   N11, \\field Maximum Module Discharging Current
        \\type real
        \\units A
        \\minimum 0
        \\note The constraint on discharging current is for each battery module.
   N12, \\field Module Cut-off Voltage
        \\type real
        \\units V
        \\minimum 0
        \\note The voltage constraint is for each battery module.
   N13, \\field Module Charge Rate Limit
        \\type real
        \\note units 1/hr
        \\minimum 0
        \\default 1.0
        \\note Charge rate limit is the division between charging current the remaining capacity.
        \\note The constraint on charging current is for each module.
   A6,  \\field Battery Life Calculation
        \\type choice
        \\key Yes
        \\key No
        \\default No
   N14, \\field Number of Cycle Bins
        \\type integer
        \\minimum 5
        \\default 10
        \\note Only required when battery life calculation is activated
   A7; \\field Battery Life Curve Name
        \\type object-list
        \\object-list DoubleExponentialDecayCurves
        \\note Determines the number of cycles to failure in relation to cycle range.
        \\note Only required when battery life calculation is activated.

ElectricLoadCenter:Transformer,
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
       \\extensible:1
  A1,  \\field Name
       \\required-field
       \\reference TransformerNames
  A2,  \\field Availability Schedule Name
       \\note Availability schedule name for this system. Schedule value > 0 means the system is available.
       \\note If this field is blank, the system is always available.
       \\type object-list
       \\object-list ScheduleNames
  A3,  \\field Transformer Usage
       \\type choice
       \\key PowerInFromGrid
       \\key PowerOutFromOnsiteGeneration
       \\default PowerInFromGrid
       \\note A transformer can be used to transfer electric energy from utility grid to
       \\note building (PowerInFromGrid)or from building onsite generation to
       \\note the grid (PowerOutFromOnsiteGeneration)
  A4,  \\field Zone Name
       \\type object-list
       \\object-list ZoneNames
       \\note Enter name of zone to receive transformer losses as heat
       \\note if blank then transformer losses are dissipated to outdoors
  N1,  \\field Radiative Fraction
       \\type real
       \\minimum 0
       \\maximum 1.0
       \\default 0
  N2,  \\field Rated Capacity
       \\type real
       \\minimum 0
       \\units VA
       \\note the unit is VA, instead of kVA as usually shown on transformer nameplates.
  N3,  \\field Phase
       \\type choice
       \\key 1
       \\key 3
       \\default 3
       \\note Must be single or three phase transformer.
       \\note NOT used in the current model.
  A5,  \\field Conductor Material
       \\type choice
       \\key Copper
       \\key Aluminum
       \\default Aluminum
       \\note Winding material used by the transformer.
  N4,  \\field Full Load Temperature Rise
       \\type real
       \\units C
       \\minimum 50
       \\maximum 180
       \\default 150
  N5,  \\field Fraction of Eddy Current Losses
       \\type real
       \\minimum 0
       \\maximum 1.0
       \\default 0.1
  A6,  \\field Performance Input Method
       \\type choice
       \\key RatedLosses
       \\key NominalEfficiency
       \\default RatedLosses
       \\note User can define transformer performance by specifying
       \\note load and no load losses at rated conditions or
       \\note nameplate efficiency and maximum efficiency
  N6,  \\field Rated No Load Loss
       \\type real
       \\units W
       \\minimum> 0
       \\note Only required when RatedLosses is the performance input method
  N7,  \\field Rated Load Loss
       \\type real
       \\units W
       \\minimum 0
       \\note Only required when RatedLosses is the performance input method
  N8,  \\field Nameplate Efficiency
       \\type real
       \\minimum> 0
       \\maximum 1.0
       \\default 0.98
       \\note Only required when NominalEfficiency is the performance input method
  N9,  \\field Per Unit Load for Nameplate Efficiency
       \\type real
       \\minimum> 0
       \\maximum 1.0
       \\default 0.35
       \\note Percentage of the rated capacity at which the nameplate efficiency is defined
       \\note Only required when NominalEfficiency is the performance input method
  N10, \\field Reference Temperature for Nameplate Efficiency
       \\type real
       \\units C
       \\minimum 20
       \\maximum 150
       \\default 75
       \\note Conductor operating temperature at which the nameplate efficiency is defined
       \\note Only required when NominalEfficiency is the performance input method
  N11, \\field Per Unit Load for Maximum Efficiency
       \\type real
       \\minimum> 0
       \\maximum 1.0
       \\note Percentage of the rate capacity at which the maximum efficiency is obtained
       \\note Only required when NominalEfficiency is the performance input method
  A7,  \\field Consider Transformer Loss for Utility Cost
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\note Only required when the transformer is used for power in from the utility grid
  A8,  \\field Meter 1 Name
       \\begin-extensible
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A9,  \\field Meter 2 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A10, \\field Meter 3 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A11, \\field Meter 4 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A12, \\field Meter 5 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A13, \\field Meter 6 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A14, \\field Meter 7 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A15, \\field Meter 8 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A16, \\field Meter 9 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid
  A17; \\field Meter 10 Name
       \\type external-list
       \\external-list autoRDDmeter
       \\note Must be an electric meter (with electricity as the resource type)
       \\note Only required when transformer is used for power in from the utility grid

ElectricLoadCenter:Distribution,
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
  A1 , \\field Name
       \\required-field
  A2 , \\field Generator List Name
       \\type object-list
       \\object-list GeneratorLists
  A3,  \\field Generator Operation Scheme Type
       \\note required if Generator List is entered.
       \\type choice
       \\key Baseload
       \\key DemandLimit
       \\key TrackElectrical
       \\key TrackSchedule
       \\key TrackMeter
       \\key FollowThermal
       \\key FollowThermalLimitElectrical
  N1,  \\field Demand Limit Scheme Purchased Electric Demand Limit
       \\type real
       \\units W
  A4,  \\field Track Schedule Name Scheme Schedule Name
       \\note required when Generator Operation Scheme Type=TrackSchedule
       \\type object-list
       \\object-list ScheduleNames
  A5,  \\field Track Meter Scheme Meter Name
       \\note required when Generator Operation Scheme Type=TrackMeter
       \\type external-list
       \\external-list autoRDDmeter
  A6,  \\field Electrical Buss Type
       \\type choice
       \\key AlternatingCurrent
       \\key AlternatingCurrentWithStorage
       \\key DirectCurrentWithInverter
       \\key DirectCurrentWithInverterDCStorage
       \\key DirectCurrentWithInverterACStorage
       \\default AlternatingCurrent
  A7,  \\field Inverter Object Name
       \\note required when Electrical Buss Type=DirectCurrentWithInverter, DirectCurrentWithInverterDCStorage,
       \\note or DirectCurrentWithInverterACStorage
       \\type object-list
       \\object-list InverterList
  A8,  \\field Electrical Storage Object Name
       \\note required when Electrical Buss Type=AlternatingCurrentWithStorage, DirectCurrentWithInverterDCStorage,
       \\note or DirectCurrentWithInverterACStorage
       \\type object-list
       \\object-list ElecStorageList
  A9;  \\field Transformer Object Name
       \\note required when power needs to be output from onsite generation to the grid via transformer
       \\type object-list
       \\object-list TransformerNames

\\Group Water Systems

WaterUse:Equipment,
       \\memo A generalized object for simulating all water end uses. Hot and cold water uses are
       \\memo included, as well as controlled mixing of hot and cold water at the tap. The
       \\memo WaterUse:Equipment object can be used stand-alone, or coupled into a plant loop using
       \\memo the WaterUse:Connections object (see below). The WaterUse:Connections object allows
       \\memo water uses to be linked to WaterUse:Storage objects to store and draw reclaimed water.
       \\memo The object can also simulate drainwater heat recovery.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WaterUseEquipmentNames
  A2 , \\field End-Use Subcategory
       \\type alpha
       \\retaincase
       \\default General
  N1 , \\field Peak Flow Rate
       \\required-field
       \\units m3/s
       \\type real
       \\minimum 0.0
  A3 , \\field Flow Rate Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to 1.0 at all times
  A4 , \\field Target Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to hot water supply temperature
  A5 , \\field Hot Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to cold water supply temperature
  A6 , \\field Cold Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to water temperatures calculated by Site:WaterMainsTemperature object
  A7 , \\field Zone Name
       \\type object-list
       \\object-list ZoneNames
  A8 , \\field Sensible Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to 0.0 at all times
  A9 ; \\field Latent Fraction Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to 0.0 at all times

WaterUse:Connections,
       \\memo A subsystem that groups together multiple WaterUse:Equipment components.
       \\memo As its name suggests, the object provides connections that are shared by these
       \\memo components, including: 1. Inlet node and outlet node connections to a plant loop
       \\memo 2. Connections to WaterUse:Storage objects to store and draw reclaimed water
       \\memo 3. Internal connections to simulate drainwater heat recovery.
       \\extensible:1 - repeat last field
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Inlet Node Name
       \\type node
  A3 , \\field Outlet Node Name
       \\type node
  A4 , \\field Supply Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
       \\note If blank, or tank is empty, defaults to fresh water from the mains
  A5 , \\field Reclamation Water Storage Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A6 , \\field Hot Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to cold water supply temperature
  A7 , \\field Cold Water Supply Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Defaults to water temperatures calculated by Site:WaterMainsTemperature object
  A8 , \\field Drain Water Heat Exchanger Type
       \\type choice
       \\key None
       \\key Ideal
       \\key CounterFlow
       \\key CrossFlow
       \\default None
  A9 , \\field Drain Water Heat Exchanger Destination
       \\type choice
       \\key Plant
       \\key Equipment
       \\key PlantAndEquipment
       \\default Plant
  N1 , \\field Drain Water Heat Exchanger U-Factor Times Area
       \\type real
       \\units W/K
       \\minimum 0.0
  A10, \\field Water Use Equipment 1 Name
       \\begin-extensible
       \\note Enter the name of a WaterUse:Equipment object.
       \\required-field
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A11, \\field Water Use Equipment 2 Name
       \\note Enter the name of a WaterUse:Equipment object.
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A12, \\field Water Use Equipment 3 Name
       \\note Enter the name of a WaterUse:Equipment object.
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A13, \\field Water Use Equipment 4 Name
       \\note Enter the name of a WaterUse:Equipment object.
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A14, \\field Water Use Equipment 5 Name
       \\note Enter the name of a WaterUse:Equipment object.
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A15, \\field Water Use Equipment 6 Name
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A16, \\field Water Use Equipment 7 Name
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A17, \\field Water Use Equipment 8 Name
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A18, \\field Water Use Equipment 9 Name
       \\type object-list
       \\object-list WaterUseEquipmentNames
  A19; \\field Water Use Equipment 10 Name
       \\type object-list
       \\object-list WaterUseEquipmentNames

WaterUse:Storage,
       \\memo A water storage tank. If the building model is to include any on-site
       \\memo water collection, wells, or storing and reuse of graywater, then a WaterUse:Storage
       \\memo object is needed. Each WaterUse:Storage can serve as a central node and make
       \\memo connections to numerous sources of supply or numerous components with demand. If a
       \\memo maximum capacity is not specified, the tank is assumed to have unlimited capacity.
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference WaterStorageTankNames
  A2 , \\field Water Quality Subcategory
       \\type alpha
  N1 , \\field Maximum Capacity
       \\type real
       \\units m3
       \\note Defaults to unlimited capacity.
  N2 , \\field Initial Volume
       \\type real
       \\units m3
  N3 , \\field Design In Flow Rate
       \\type real
       \\units m3/s
       \\note Defaults to unlimited flow.
  N4 , \\field Design Out Flow Rate
       \\type real
       \\units m3/s
       \\note Defaults to unlimited flow.
  A3 , \\field Overflow Destination
       \\type object-list
       \\object-list WaterStorageTankNames
       \\note If blank, overflow is discarded
  A4 , \\field Type of Supply Controlled by Float Valve
       \\type choice
       \\key None
       \\key Mains
       \\key GroundwaterWell
       \\key OtherTank
  N5 , \\field Float Valve On Capacity
       \\type real
       \\units m3
       \\note Lower range of target storage level e.g. float valve kicks on
  N6 , \\field Float Valve Off Capacity
       \\type real
       \\units m3
       \\note Upper range of target storage level e.g. float valve kicks off
  N7 , \\field Backup Mains Capacity
       \\type real
       \\units m3
       \\note Lower range of secondary target storage level
       \\note used to keep tanks at a minimum level using
       \\note mains water if well can't keep up
  A5,  \\field Other Tank Name
       \\type object-list
       \\object-list WaterStorageTankNames
  A6 , \\field Water Thermal Mode
       \\type choice
       \\key ScheduledTemperature
       \\key ThermalModel
  A7 , \\field Water Temperature Schedule Name
       \\required-field
       \\type object-list
       \\object-list ScheduleNames
  A8 , \\field Ambient Temperature Indicator
       \\type choice
       \\key Schedule
       \\key Zone
       \\key Outdoors
  A9 , \\field Ambient Temperature Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  A10, \\field Zone Name
       \\type object-list
       \\object-list ZoneNames
  N8 , \\field Tank Surface Area
       \\type real
       \\units m2
  N9 , \\field Tank U Value
       \\type real
       \\units W/m2-K
  A11; \\field Tank Outside Surface Material Name
       \\type object-list
       \\object-list MaterialName

WaterUse:Well,
       \\memo Simulates on-site water supply from a well. Well water is pumped out of the ground
       \\memo into a WaterUse:Storage. The operation of the ground water well is controlled by the
       \\memo associated WaterUse:Storage which is assumed to be operated as a vented cistern with
       \\memo no pressure tank.
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Storage Tank Name
       \\required-field
       \\type object-list
       \\object-list WaterStorageTankNames
  N1 , \\field Pump Depth
       \\type real
       \\units m
  N2 , \\field Pump Rated Flow Rate
       \\type real
       \\units m3/s
  N3 , \\field Pump Rated Head
       \\type real
       \\units Pa
  N4 , \\field Pump Rated Power Consumption
       \\type real
       \\units W
  N5 , \\field Pump Efficiency
       \\type real
  N6 , \\field Well Recovery Rate
       \\type real
       \\units m3/s
  N7 , \\field Nominal Well Storage Volume
       \\type real
       \\units m3
  A3 , \\field Water Table Depth Mode
       \\type choice
       \\key Constant
       \\key Scheduled
  N8 , \\field Water Table Depth
       \\type real
       \\units m
  A4 ; \\field Water Table Depth Schedule Name
       \\type object-list
       \\object-list ScheduleNames

WaterUse:RainCollector,
       \\memo Used for harvesting rainwater falling on building surfaces. The rainwater is sent to a
       \\memo WaterUse:Storage object. In order to use this object it is necessary to also include
       \\memo a Site:Precipitation object to describe the rates of rainfall.
       \\extensible:1 - repeat last field
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Storage Tank Name
       \\required-field
       \\type object-list
       \\object-list WaterStorageTankNames
  A3,  \\field Loss Factor Mode
       \\type choice
       \\key Constant
       \\key Scheduled
  N1 , \\field Collection Loss Factor
       \\type real
       \\note this is the portion of rain
       \\note that is lost in the process of collecting it
       \\note the rain collected is one minus this factor
  A4 , \\field Collection Loss Factor Schedule Name
       \\type object-list
       \\object-list ScheduleNames
  N2 , \\field Maximum Collection Rate
       \\type real
       \\units m3/s
       \\note Defaults to unlimited flow.
  A5 , \\field Collection Surface 1 Name
       \\begin-extensible
       \\required-field
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A6 , \\field Collection Surface 2 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A7 , \\field Collection Surface 3 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A8 , \\field Collection Surface 4 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A9 , \\field Collection Surface 5 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A10, \\field Collection Surface 6 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A11, \\field Collection Surface 7 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A12, \\field Collection Surface 8 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A13, \\field Collection Surface 9 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames
  A14; \\field Collection Surface 10 Name
       \\type object-list
       \\object-list AllShadingAndHTSurfNames

\\group General Data Entry


Matrix:TwoDimension,
       \\extensible:1
       \\memo matrix data in row-major order
       \\memo list each row keeping the columns in order
       \\memo number of values must equal N1 x N2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DataMatrices
  N1 , \\field Number of Rows
       \\required-field
       \\type integer
  N2 , \\field Number of Columns
       \\required-field
       \\type integer
  N3 , \\field Value 1
       \\begin-extensible
       \\type real
  N4 , \\field Value 2
       \\type real
  N5 , \\field Value 3
       \\type real
  N6 , \\field Value 4
       \\type real
  N7,N8,N9,N10,N11,N12,N13,N14,N15,N16,N17,N18,N19,N20,  \\note fields as indicated
  N21,N22,N23,N24,N25,N26,N27,N28,N29,N30,N31,N32,N33,N34,N35,N36,N37,N38,N39,N40,  \\note fields as indicated
  N41,N42,N43,N44,N45,N46,N47,N48,N49,N50,N51,N52,N53,N54,N55,N56,N57,N58,N59,N60,  \\note fields as indicated
  N61,N62,N63,N64,N65,N66,N67,N68,N69,N70,N71,N72,N73,N74,N75,N76,N77,N78,N79,N80,  \\note fields as indicated
  N81,N82,N83,N84,N85,N86,N87,N88,N89,N90,N91,N92,N93,N94,N95,N96,N97,N98,N99,N100,  \\note fields as indicated
  N101,N102,N103,N104,N105,N106,N107,N108,N109,N110,N111,N112,N113,N114,N115,N116,N117,N118,N119,N120,  \\note fields as indicated
  N121,N122,N123,N124,N125,N126,N127,N128,N129,N130,N131,N132,N133,N134,N135,N136,N137,N138,N139,N140,  \\note fields as indicated
  N141,N142,N143,N144,N145,N146,N147,N148,N149,N150,N151,N152,N153,N154,N155,N156,N157,N158,N159,N160,  \\note fields as indicated
  N161,N162,N163,N164,N165,N166,N167,N168,N169,N170,N171,N172,N173,N174,N175,N176,N177,N178,N179,N180,  \\note fields as indicated
  N181,N182,N183,N184,N185,N186,N187,N188,N189,N190,N191,N192,N193,N194,N195,N196,N197,N198,N199,N200,  \\note fields as indicated
  N201,N202,N203,N204,N205,N206,N207,N208,N209,N210,N211,N212,N213,N214,N215,N216,N217,N218,N219,N220,  \\note fields as indicated
  N221,N222,N223,N224,N225,N226,N227,N228,N229,N230,N231,N232,N233,N234,N235,N236,N237,N238,N239,N240,  \\note fields as indicated
  N241,N242,N243,N244,N245,N246,N247,N248,N249,N250,N251,N252,N253,N254,N255,N256,N257,N258,N259,N260,  \\note fields as indicated
  N261,N262,N263,N264,N265,N266,N267,N268,N269,N270,N271,N272,N273,N274,N275,N276,N277,N278,N279,N280,  \\note fields as indicated
  N281,N282,N283,N284,N285,N286,N287,N288,N289,N290,N291,N292,N293,N294,N295,N296,N297,N298,N299,N300,  \\note fields as indicated
  N301,N302,N303,N304,N305,N306,N307,N308,N309,N310,N311,N312,N313,N314,N315,N316,N317,N318,N319,N320,  \\note fields as indicated
  N321,N322,N323,N324,N325,N326,N327,N328,N329,N330,N331,N332,N333,N334,N335,N336,N337,N338,N339,N340,  \\note fields as indicated
  N341,N342,N343,N344,N345,N346,N347,N348,N349,N350,N351,N352,N353,N354,N355,N356,N357,N358,N359,N360,  \\note fields as indicated
  N361,N362,N363,N364,N365,N366,N367,N368,N369,N370,N371,N372,N373,N374,N375,N376,N377,N378,N379,N380,  \\note fields as indicated
  N381,N382,N383,N384,N385,N386,N387,N388,N389,N390,N391,N392,N393,N394,N395,N396,N397,N398,N399,N400,  \\note fields as indicated
  N401,N402,N403,N404,N405,N406,N407,N408,N409,N410,N411,N412,N413,N414,N415,N416,N417,N418,N419,N420,  \\note fields as indicated
  N421,N422,N423,N424,N425,N426,N427,N428,N429,N430,N431,N432,N433,N434,N435,N436,N437,N438,N439,N440,  \\note fields as indicated
  N441,N442,N443,N444,N445,N446,N447,N448,N449,N450,N451,N452,N453,N454,N455,N456,N457,N458,N459,N460,  \\note fields as indicated
  N461,N462,N463,N464,N465,N466,N467,N468,N469,N470,N471,N472,N473,N474,N475,N476,N477,N478,N479,N480,  \\note fields as indicated
  N481,N482,N483,N484,N485,N486,N487,N488,N489,N490,N491,N492,N493,N494,N495,N496,N497,N498,N499,N500,  \\note fields as indicated
  N501,N502,N503,N504,N505,N506,N507,N508,N509,N510,N511,N512,N513,N514,N515,N516,N517,N518,N519,N520,  \\note fields as indicated
  N521,N522,N523,N524,N525,N526,N527,N528,N529,N530,N531,N532,N533,N534,N535,N536,N537,N538,N539,N540,  \\note fields as indicated
  N541,N542,N543,N544,N545,N546,N547,N548,N549,N550,N551,N552,N553,N554,N555,N556,N557,N558,N559,N560,  \\note fields as indicated
  N561,N562,N563,N564,N565,N566,N567,N568,N569,N570,N571,N572,N573,N574,N575,N576,N577,N578,N579,N580,  \\note fields as indicated
  N581,N582,N583,N584,N585,N586,N587,N588,N589,N590,N591,N592,N593,N594,N595,N596,N597,N598,N599,N600,  \\note fields as indicated
  N601,N602,N603,N604,N605,N606,N607,N608,N609,N610,N611,N612,N613,N614,N615,N616,N617,N618,N619,N620,  \\note fields as indicated
  N621,N622,N623,N624,N625,N626,N627,N628,N629,N630,N631,N632,N633,N634,N635,N636,N637,N638,N639,N640,  \\note fields as indicated
  N641,N642,N643,N644,N645,N646,N647,N648,N649,N650,N651,N652,N653,N654,N655,N656,N657,N658,N659,N660,  \\note fields as indicated
  N661,N662,N663,N664,N665,N666,N667,N668,N669,N670,N671,N672,N673,N674,N675,N676,N677,N678,N679,N680,  \\note fields as indicated
  N681,N682,N683,N684,N685,N686,N687,N688,N689,N690,N691,N692,N693,N694,N695,N696,N697,N698,N699,N700,  \\note fields as indicated
  N701,N702,N703,N704,N705,N706,N707,N708,N709,N710,N711,N712,N713,N714,N715,N716,N717,N718,N719,N720,  \\note fields as indicated
  N721,N722,N723,N724,N725,N726,N727,N728,N729,N730,N731,N732,N733,N734,N735,N736,N737,N738,N739,N740,  \\note fields as indicated
  N741,N742,N743,N744,N745,N746,N747,N748,N749,N750,N751,N752,N753,N754,N755,N756,N757,N758,N759,N760,  \\note fields as indicated
  N761,N762,N763,N764,N765,N766,N767,N768,N769,N770,N771,N772,N773,N774,N775,N776,N777,N778,N779,N780,  \\note fields as indicated
  N781,N782,N783,N784,N785,N786,N787,N788,N789,N790,N791,N792,N793,N794,N795,N796,N797,N798,N799,N800,  \\note fields as indicated
  N801,N802,N803,N804,N805,N806,N807,N808,N809,N810,N811,N812,N813,N814,N815,N816,N817,N818,N819,N820,  \\note fields as indicated
  N821,N822,N823,N824,N825,N826,N827,N828,N829,N830,N831,N832,N833,N834,N835,N836,N837,N838,N839,N840,  \\note fields as indicated
  N841,N842,N843,N844,N845,N846,N847,N848,N849,N850,N851,N852,N853,N854,N855,N856,N857,N858,N859,N860,  \\note fields as indicated
  N861,N862,N863,N864,N865,N866,N867,N868,N869,N870,N871,N872,N873,N874,N875,N876,N877,N878,N879,N880,  \\note fields as indicated
  N881,N882,N883,N884,N885,N886,N887,N888,N889,N890,N891,N892,N893,N894,N895,N896,N897,N898,N899,N900,  \\note fields as indicated
  N901,N902,N903,N904,N905,N906,N907,N908,N909,N910,N911,N912,N913,N914,N915,N916,N917,N918,N919,N920,  \\note fields as indicated
  N921,N922,N923,N924,N925,N926,N927,N928,N929,N930,N931,N932,N933,N934,N935,N936,N937,N938,N939,N940,  \\note fields as indicated
  N941,N942,N943,N944,N945,N946,N947,N948,N949,N950,N951,N952,N953,N954,N955,N956,N957,N958,N959,N960,  \\note fields as indicated
  N961,N962,N963,N964,N965,N966,N967,N968,N969,N970,N971,N972,N973,N974,N975,N976,N977,N978,N979,N980,  \\note fields as indicated
  N981,N982,N983,N984,N985,N986,N987,N988,N989,N990,N991,N992,N993,N994,N995,N996,N997,N998,N999,N1000,  \\note fields as indicated
  N1001,N1002,N1003,N1004,N1005,N1006,N1007,N1008,N1009,N1010,N1011,N1012,N1013,N1014,N1015,N1016,N1017,N1018,N1019,N1020,  \\note fields as indicated
  N1021,N1022,N1023,N1024,N1025,N1026,N1027,N1028,N1029,N1030,N1031,N1032,N1033,N1034,N1035,N1036,N1037,N1038,N1039,N1040,  \\note fields as indicated
  N1041,N1042,N1043,N1044,N1045,N1046,N1047,N1048,N1049,N1050,N1051,N1052,N1053,N1054,N1055,N1056,N1057,N1058,N1059,N1060,  \\note fields as indicated
  N1061,N1062,N1063,N1064,N1065,N1066,N1067,N1068,N1069,N1070,N1071,N1072,N1073,N1074,N1075,N1076,N1077,N1078,N1079,N1080,  \\note fields as indicated
  N1081,N1082,N1083,N1084,N1085,N1086,N1087,N1088,N1089,N1090,N1091,N1092,N1093,N1094,N1095,N1096,N1097,N1098,N1099,N1100,  \\note fields as indicated
  N1101,N1102,N1103,N1104,N1105,N1106,N1107,N1108,N1109,N1110,N1111,N1112,N1113,N1114,N1115,N1116,N1117,N1118,N1119,N1120,  \\note fields as indicated
  N1121,N1122,N1123,N1124,N1125,N1126,N1127,N1128,N1129,N1130,N1131,N1132,N1133,N1134,N1135,N1136,N1137,N1138,N1139,N1140,  \\note fields as indicated
  N1141,N1142,N1143,N1144,N1145,N1146,N1147,N1148,N1149,N1150,N1151,N1152,N1153,N1154,N1155,N1156,N1157,N1158,N1159,N1160,  \\note fields as indicated
  N1161,N1162,N1163,N1164,N1165,N1166,N1167,N1168,N1169,N1170,N1171,N1172,N1173,N1174,N1175,N1176,N1177,N1178,N1179,N1180,  \\note fields as indicated
  N1181,N1182,N1183,N1184,N1185,N1186,N1187,N1188,N1189,N1190,N1191,N1192,N1193,N1194,N1195,N1196,N1197,N1198,N1199,N1200,  \\note fields as indicated
  N1201,N1202,N1203,N1204,N1205,N1206,N1207,N1208,N1209,N1210,N1211,N1212,N1213,N1214,N1215,N1216,N1217,N1218,N1219,N1220,  \\note fields as indicated
  N1221,N1222,N1223,N1224,N1225,N1226,N1227,N1228,N1229,N1230,N1231,N1232,N1233,N1234,N1235,N1236,N1237,N1238,N1239,N1240,  \\note fields as indicated
  N1241,N1242,N1243,N1244,N1245,N1246,N1247,N1248,N1249,N1250,N1251,N1252,N1253,N1254,N1255,N1256,N1257,N1258,N1259,N1260,  \\note fields as indicated
  N1261,N1262,N1263,N1264,N1265,N1266,N1267,N1268,N1269,N1270,N1271,N1272,N1273,N1274,N1275,N1276,N1277,N1278,N1279,N1280,  \\note fields as indicated
  N1281,N1282,N1283,N1284,N1285,N1286,N1287,N1288,N1289,N1290,N1291,N1292,N1293,N1294,N1295,N1296,N1297,N1298,N1299,N1300,  \\note fields as indicated
  N1301,N1302,N1303,N1304,N1305,N1306,N1307,N1308,N1309,N1310,N1311,N1312,N1313,N1314,N1315,N1316,N1317,N1318,N1319,N1320,  \\note fields as indicated
  N1321,N1322,N1323,N1324,N1325,N1326,N1327,N1328,N1329,N1330,N1331,N1332,N1333,N1334,N1335,N1336,N1337,N1338,N1339,N1340,  \\note fields as indicated
  N1341,N1342,N1343,N1344,N1345,N1346,N1347,N1348,N1349,N1350,N1351,N1352,N1353,N1354,N1355,N1356,N1357,N1358,N1359,N1360,  \\note fields as indicated
  N1361,N1362,N1363,N1364,N1365,N1366,N1367,N1368,N1369,N1370,N1371,N1372,N1373,N1374,N1375,N1376,N1377,N1378,N1379,N1380,  \\note fields as indicated
  N1381,N1382,N1383,N1384,N1385,N1386,N1387,N1388,N1389,N1390,N1391,N1392,N1393,N1394,N1395,N1396,N1397,N1398,N1399,N1400,  \\note fields as indicated
  N1401,N1402,N1403,N1404,N1405,N1406,N1407,N1408,N1409,N1410,N1411,N1412,N1413,N1414,N1415,N1416,N1417,N1418,N1419,N1420,  \\note fields as indicated
  N1421,N1422,N1423,N1424,N1425,N1426,N1427,N1428,N1429,N1430,N1431,N1432,N1433,N1434,N1435,N1436,N1437,N1438,N1439,N1440,  \\note fields as indicated
  N1441,N1442,N1443,N1444,N1445,N1446,N1447,N1448,N1449,N1450,N1451,N1452,N1453,N1454,N1455,N1456,N1457,N1458,N1459,N1460,  \\note fields as indicated
  N1461,N1462,N1463,N1464,N1465,N1466,N1467,N1468,N1469,N1470,N1471,N1472,N1473,N1474,N1475,N1476,N1477,N1478,N1479,N1480,  \\note fields as indicated
  N1481,N1482,N1483,N1484,N1485,N1486,N1487,N1488,N1489,N1490,N1491,N1492,N1493,N1494,N1495,N1496,N1497,N1498,N1499,N1500,  \\note fields as indicated
  N1501,N1502,N1503,N1504,N1505,N1506,N1507,N1508,N1509,N1510,N1511,N1512,N1513,N1514,N1515,N1516,N1517,N1518,N1519,N1520,  \\note fields as indicated
  N1521,N1522,N1523,N1524,N1525,N1526,N1527,N1528,N1529,N1530,N1531,N1532,N1533,N1534,N1535,N1536,N1537,N1538,N1539,N1540,  \\note fields as indicated
  N1541,N1542,N1543,N1544,N1545,N1546,N1547,N1548,N1549,N1550,N1551,N1552,N1553,N1554,N1555,N1556,N1557,N1558,N1559,N1560,  \\note fields as indicated
  N1561,N1562,N1563,N1564,N1565,N1566,N1567,N1568,N1569,N1570,N1571,N1572,N1573,N1574,N1575,N1576,N1577,N1578,N1579,N1580,  \\note fields as indicated
  N1581,N1582,N1583,N1584,N1585,N1586,N1587,N1588,N1589,N1590,N1591,N1592,N1593,N1594,N1595,N1596,N1597,N1598,N1599,N1600,  \\note fields as indicated
  N1601,N1602,N1603,N1604,N1605,N1606,N1607,N1608,N1609,N1610,N1611,N1612,N1613,N1614,N1615,N1616,N1617,N1618,N1619,N1620,  \\note fields as indicated
  N1621,N1622,N1623,N1624,N1625,N1626,N1627,N1628,N1629,N1630,N1631,N1632,N1633,N1634,N1635,N1636,N1637,N1638,N1639,N1640,  \\note fields as indicated
  N1641,N1642,N1643,N1644,N1645,N1646,N1647,N1648,N1649,N1650,N1651,N1652,N1653,N1654,N1655,N1656,N1657,N1658,N1659,N1660,  \\note fields as indicated
  N1661,N1662,N1663,N1664,N1665,N1666,N1667,N1668,N1669,N1670,N1671,N1672,N1673,N1674,N1675,N1676,N1677,N1678,N1679,N1680,  \\note fields as indicated
  N1681,N1682,N1683,N1684,N1685,N1686,N1687,N1688,N1689,N1690,N1691,N1692,N1693,N1694,N1695,N1696,N1697,N1698,N1699,N1700,  \\note fields as indicated
  N1701,N1702,N1703,N1704,N1705,N1706,N1707,N1708,N1709,N1710,N1711,N1712,N1713,N1714,N1715,N1716,N1717,N1718,N1719,N1720,  \\note fields as indicated
  N1721,N1722,N1723,N1724,N1725,N1726,N1727,N1728,N1729,N1730,N1731,N1732,N1733,N1734,N1735,N1736,N1737,N1738,N1739,N1740,  \\note fields as indicated
  N1741,N1742,N1743,N1744,N1745,N1746,N1747,N1748,N1749,N1750,N1751,N1752,N1753,N1754,N1755,N1756,N1757,N1758,N1759,N1760,  \\note fields as indicated
  N1761,N1762,N1763,N1764,N1765,N1766,N1767,N1768,N1769,N1770,N1771,N1772,N1773,N1774,N1775,N1776,N1777,N1778,N1779,N1780,  \\note fields as indicated
  N1781,N1782,N1783,N1784,N1785,N1786,N1787,N1788,N1789,N1790,N1791,N1792,N1793,N1794,N1795,N1796,N1797,N1798,N1799,N1800,  \\note fields as indicated
  N1801,N1802,N1803,N1804,N1805,N1806,N1807,N1808,N1809,N1810,N1811,N1812,N1813,N1814,N1815,N1816,N1817,N1818,N1819,N1820,  \\note fields as indicated
  N1821,N1822,N1823,N1824,N1825,N1826,N1827,N1828,N1829,N1830,N1831,N1832,N1833,N1834,N1835,N1836,N1837,N1838,N1839,N1840,  \\note fields as indicated
  N1841,N1842,N1843,N1844,N1845,N1846,N1847,N1848,N1849,N1850,N1851,N1852,N1853,N1854,N1855,N1856,N1857,N1858,N1859,N1860,  \\note fields as indicated
  N1861,N1862,N1863,N1864,N1865,N1866,N1867,N1868,N1869,N1870,N1871,N1872,N1873,N1874,N1875,N1876,N1877,N1878,N1879,N1880,  \\note fields as indicated
  N1881,N1882,N1883,N1884,N1885,N1886,N1887,N1888,N1889,N1890,N1891,N1892,N1893,N1894,N1895,N1896,N1897,N1898,N1899,N1900,  \\note fields as indicated
  N1901,N1902,N1903,N1904,N1905,N1906,N1907,N1908,N1909,N1910,N1911,N1912,N1913,N1914,N1915,N1916,N1917,N1918,N1919,N1920,  \\note fields as indicated
  N1921,N1922,N1923,N1924,N1925,N1926,N1927,N1928,N1929,N1930,N1931,N1932,N1933,N1934,N1935,N1936,N1937,N1938,N1939,N1940,  \\note fields as indicated
  N1941,N1942,N1943,N1944,N1945,N1946,N1947,N1948,N1949,N1950,N1951,N1952,N1953,N1954,N1955,N1956,N1957,N1958,N1959,N1960,  \\note fields as indicated
  N1961,N1962,N1963,N1964,N1965,N1966,N1967,N1968,N1969,N1970,N1971,N1972,N1973,N1974,N1975,N1976,N1977,N1978,N1979,N1980,  \\note fields as indicated
  N1981,N1982,N1983,N1984,N1985,N1986,N1987,N1988,N1989,N1990,N1991,N1992,N1993,N1994,N1995,N1996,N1997,N1998,N1999,N2000,  \\note fields as indicated
  N2001,N2002,N2003,N2004,N2005,N2006,N2007,N2008,N2009,N2010,N2011,N2012,N2013,N2014,N2015,N2016,N2017,N2018,N2019,N2020,  \\note fields as indicated
  N2021,N2022,N2023,N2024,N2025,N2026,N2027,N2028,N2029,N2030,N2031,N2032,N2033,N2034,N2035,N2036,N2037,N2038,N2039,N2040,  \\note fields as indicated
  N2041,N2042,N2043,N2044,N2045,N2046,N2047,N2048,N2049,N2050,N2051,N2052,N2053,N2054,N2055,N2056,N2057,N2058,N2059,N2060,  \\note fields as indicated
  N2061,N2062,N2063,N2064,N2065,N2066,N2067,N2068,N2069,N2070,N2071,N2072,N2073,N2074,N2075,N2076,N2077,N2078,N2079,N2080,  \\note fields as indicated
  N2081,N2082,N2083,N2084,N2085,N2086,N2087,N2088,N2089,N2090,N2091,N2092,N2093,N2094,N2095,N2096,N2097,N2098,N2099,N2100,  \\note fields as indicated
  N2101,N2102,N2103,N2104,N2105,N2106,N2107,N2108,N2109,N2110,N2111,N2112,N2113,N2114,N2115,N2116,N2117,N2118,N2119,N2120,  \\note fields as indicated
  N2121,N2122,N2123,N2124,N2125,N2126,N2127,N2128,N2129,N2130,N2131,N2132,N2133,N2134,N2135,N2136,N2137,N2138,N2139,N2140,  \\note fields as indicated
  N2141,N2142,N2143,N2144,N2145,N2146,N2147,N2148,N2149,N2150,N2151,N2152,N2153,N2154,N2155,N2156,N2157,N2158,N2159,N2160,  \\note fields as indicated
  N2161,N2162,N2163,N2164,N2165,N2166,N2167,N2168,N2169,N2170,N2171,N2172,N2173,N2174,N2175,N2176,N2177,N2178,N2179,N2180,  \\note fields as indicated
  N2181,N2182,N2183,N2184,N2185,N2186,N2187,N2188,N2189,N2190,N2191,N2192,N2193,N2194,N2195,N2196,N2197,N2198,N2199,N2200,  \\note fields as indicated
  N2201,N2202,N2203,N2204,N2205,N2206,N2207,N2208,N2209,N2210,N2211,N2212,N2213,N2214,N2215,N2216,N2217,N2218,N2219,N2220,  \\note fields as indicated
  N2221,N2222,N2223,N2224,N2225,N2226,N2227,N2228,N2229,N2230,N2231,N2232,N2233,N2234,N2235,N2236,N2237,N2238,N2239,N2240,  \\note fields as indicated
  N2241,N2242,N2243,N2244,N2245,N2246,N2247,N2248,N2249,N2250,N2251,N2252,N2253,N2254,N2255,N2256,N2257,N2258,N2259,N2260,  \\note fields as indicated
  N2261,N2262,N2263,N2264,N2265,N2266,N2267,N2268,N2269,N2270,N2271,N2272,N2273,N2274,N2275,N2276,N2277,N2278,N2279,N2280,  \\note fields as indicated
  N2281,N2282,N2283,N2284,N2285,N2286,N2287,N2288,N2289,N2290,N2291,N2292,N2293,N2294,N2295,N2296,N2297,N2298,N2299,N2300,  \\note fields as indicated
  N2301,N2302,N2303,N2304,N2305,N2306,N2307,N2308,N2309,N2310,N2311,N2312,N2313,N2314,N2315,N2316,N2317,N2318,N2319,N2320,  \\note fields as indicated
  N2321,N2322,N2323,N2324,N2325,N2326,N2327,N2328,N2329,N2330,N2331,N2332,N2333,N2334,N2335,N2336,N2337,N2338,N2339,N2340,  \\note fields as indicated
  N2341,N2342,N2343,N2344,N2345,N2346,N2347,N2348,N2349,N2350,N2351,N2352,N2353,N2354,N2355,N2356,N2357,N2358,N2359,N2360,  \\note fields as indicated
  N2361,N2362,N2363,N2364,N2365,N2366,N2367,N2368,N2369,N2370,N2371,N2372,N2373,N2374,N2375,N2376,N2377,N2378,N2379,N2380,  \\note fields as indicated
  N2381,N2382,N2383,N2384,N2385,N2386,N2387,N2388,N2389,N2390,N2391,N2392,N2393,N2394,N2395,N2396,N2397,N2398,N2399,N2400,  \\note fields as indicated
  N2401,N2402,N2403,N2404,N2405,N2406,N2407,N2408,N2409,N2410,N2411,N2412,N2413,N2414,N2415,N2416,N2417,N2418,N2419,N2420,  \\note fields as indicated
  N2421,N2422,N2423,N2424,N2425,N2426,N2427,N2428,N2429,N2430,N2431,N2432,N2433,N2434,N2435,N2436,N2437,N2438,N2439,N2440,  \\note fields as indicated
  N2441,N2442,N2443,N2444,N2445,N2446,N2447,N2448,N2449,N2450,N2451,N2452,N2453,N2454,N2455,N2456,N2457,N2458,N2459,N2460,  \\note fields as indicated
  N2461,N2462,N2463,N2464,N2465,N2466,N2467,N2468,N2469,N2470,N2471,N2472,N2473,N2474,N2475,N2476,N2477,N2478,N2479,N2480,  \\note fields as indicated
  N2481,N2482,N2483,N2484,N2485,N2486,N2487,N2488,N2489,N2490,N2491,N2492,N2493,N2494,N2495,N2496,N2497,N2498,N2499,N2500,  \\note fields as indicated
  N2501,N2502,N2503,N2504,N2505,N2506,N2507,N2508,N2509,N2510,N2511,N2512,N2513,N2514,N2515,N2516,N2517,N2518,N2519,N2520,  \\note fields as indicated
  N2521,N2522,N2523,N2524,N2525,N2526,N2527,N2528,N2529,N2530,N2531,N2532,N2533,N2534,N2535,N2536,N2537,N2538,N2539,N2540,  \\note fields as indicated
  N2541,N2542,N2543,N2544,N2545,N2546,N2547,N2548,N2549,N2550,N2551,N2552,N2553,N2554,N2555,N2556,N2557,N2558,N2559,N2560,  \\note fields as indicated
  N2561,N2562,N2563,N2564,N2565,N2566,N2567,N2568,N2569,N2570,N2571,N2572,N2573,N2574,N2575,N2576,N2577,N2578,N2579,N2580,  \\note fields as indicated
  N2581,N2582,N2583,N2584,N2585,N2586,N2587,N2588,N2589,N2590,N2591,N2592,N2593,N2594,N2595,N2596,N2597,N2598,N2599,N2600,  \\note fields as indicated
  N2601,N2602,N2603,N2604,N2605,N2606,N2607,N2608,N2609,N2610,N2611,N2612,N2613,N2614,N2615,N2616,N2617,N2618,N2619,N2620,  \\note fields as indicated
  N2621,N2622,N2623,N2624,N2625,N2626,N2627,N2628,N2629,N2630,N2631,N2632,N2633,N2634,N2635,N2636,N2637,N2638,N2639,N2640,  \\note fields as indicated
  N2641,N2642,N2643,N2644,N2645,N2646,N2647,N2648,N2649,N2650,N2651,N2652,N2653,N2654,N2655,N2656,N2657,N2658,N2659,N2660,  \\note fields as indicated
  N2661,N2662,N2663,N2664,N2665,N2666,N2667,N2668,N2669,N2670,N2671,N2672,N2673,N2674,N2675,N2676,N2677,N2678,N2679,N2680,  \\note fields as indicated
  N2681,N2682,N2683,N2684,N2685,N2686,N2687,N2688,N2689,N2690,N2691,N2692,N2693,N2694,N2695,N2696,N2697,N2698,N2699,N2700,  \\note fields as indicated
  N2701,N2702,N2703,N2704,N2705,N2706,N2707,N2708,N2709,N2710,N2711,N2712,N2713,N2714,N2715,N2716,N2717,N2718,N2719,N2720,  \\note fields as indicated
  N2721,N2722,N2723,N2724,N2725,N2726,N2727,N2728,N2729,N2730,N2731,N2732,N2733,N2734,N2735,N2736,N2737,N2738,N2739,N2740,  \\note fields as indicated
  N2741,N2742,N2743,N2744,N2745,N2746,N2747,N2748,N2749,N2750,N2751,N2752,N2753,N2754,N2755,N2756,N2757,N2758,N2759,N2760,  \\note fields as indicated
  N2761,N2762,N2763,N2764,N2765,N2766,N2767,N2768,N2769,N2770,N2771,N2772,N2773,N2774,N2775,N2776,N2777,N2778,N2779,N2780,  \\note fields as indicated
  N2781,N2782,N2783,N2784,N2785,N2786,N2787,N2788,N2789,N2790,N2791,N2792,N2793,N2794,N2795,N2796,N2797,N2798,N2799,N2800,  \\note fields as indicated
  N2801,N2802,N2803,N2804,N2805,N2806,N2807,N2808,N2809,N2810,N2811,N2812,N2813,N2814,N2815,N2816,N2817,N2818,N2819,N2820,  \\note fields as indicated
  N2821,N2822,N2823,N2824,N2825,N2826,N2827,N2828,N2829,N2830,N2831,N2832,N2833,N2834,N2835,N2836,N2837,N2838,N2839,N2840,  \\note fields as indicated
  N2841,N2842,N2843,N2844,N2845,N2846,N2847,N2848,N2849,N2850,N2851,N2852,N2853,N2854,N2855,N2856,N2857,N2858,N2859,N2860,  \\note fields as indicated
  N2861,N2862,N2863,N2864,N2865,N2866,N2867,N2868,N2869,N2870,N2871,N2872,N2873,N2874,N2875,N2876,N2877,N2878,N2879,N2880,  \\note fields as indicated
  N2881,N2882,N2883,N2884,N2885,N2886,N2887,N2888,N2889,N2890,N2891,N2892,N2893,N2894,N2895,N2896,N2897,N2898,N2899,N2900,  \\note fields as indicated
  N2901,N2902,N2903,N2904,N2905,N2906,N2907,N2908,N2909,N2910,N2911,N2912,N2913,N2914,N2915,N2916,N2917,N2918,N2919,N2920,  \\note fields as indicated
  N2921,N2922,N2923,N2924,N2925,N2926,N2927,N2928,N2929,N2930,N2931,N2932,N2933,N2934,N2935,N2936,N2937,N2938,N2939,N2940,  \\note fields as indicated
  N2941,N2942,N2943,N2944,N2945,N2946,N2947,N2948,N2949,N2950,N2951,N2952,N2953,N2954,N2955,N2956,N2957,N2958,N2959,N2960,  \\note fields as indicated
  N2961,N2962,N2963,N2964,N2965,N2966,N2967,N2968,N2969,N2970,N2971,N2972,N2973,N2974,N2975,N2976,N2977,N2978,N2979,N2980,  \\note fields as indicated
  N2981,N2982,N2983,N2984,N2985,N2986,N2987,N2988,N2989,N2990,N2991,N2992,N2993,N2994,N2995,N2996,N2997,N2998,N2999,N3000,  \\note fields as indicated
  N3001,N3002,N3003,N3004,N3005,N3006,N3007,N3008,N3009,N3010,N3011,N3012,N3013,N3014,N3015,N3016,N3017,N3018,N3019,N3020,  \\note fields as indicated
  N3021,N3022,N3023,N3024,N3025,N3026,N3027,N3028,N3029,N3030,N3031,N3032,N3033,N3034,N3035,N3036,N3037,N3038,N3039,N3040,  \\note fields as indicated
  N3041,N3042,N3043,N3044,N3045,N3046,N3047,N3048,N3049,N3050,N3051,N3052,N3053,N3054,N3055,N3056,N3057,N3058,N3059,N3060,  \\note fields as indicated
  N3061,N3062,N3063,N3064,N3065,N3066,N3067,N3068,N3069,N3070,N3071,N3072,N3073,N3074,N3075,N3076,N3077,N3078,N3079,N3080,  \\note fields as indicated
  N3081,N3082,N3083,N3084,N3085,N3086,N3087,N3088,N3089,N3090,N3091,N3092,N3093,N3094,N3095,N3096,N3097,N3098,N3099,N3100,  \\note fields as indicated
  N3101,N3102,N3103,N3104,N3105,N3106,N3107,N3108,N3109,N3110,N3111,N3112,N3113,N3114,N3115,N3116,N3117,N3118,N3119,N3120,  \\note fields as indicated
  N3121,N3122,N3123,N3124,N3125,N3126,N3127,N3128,N3129,N3130,N3131,N3132,N3133,N3134,N3135,N3136,N3137,N3138,N3139,N3140,  \\note fields as indicated
  N3141,N3142,N3143,N3144,N3145,N3146,N3147,N3148,N3149,N3150,N3151,N3152,N3153,N3154,N3155,N3156,N3157,N3158,N3159,N3160,  \\note fields as indicated
  N3161,N3162,N3163,N3164,N3165,N3166,N3167,N3168,N3169,N3170,N3171,N3172,N3173,N3174,N3175,N3176,N3177,N3178,N3179,N3180,  \\note fields as indicated
  N3181,N3182,N3183,N3184,N3185,N3186,N3187,N3188,N3189,N3190,N3191,N3192,N3193,N3194,N3195,N3196,N3197,N3198,N3199,N3200,  \\note fields as indicated
  N3201,N3202,N3203,N3204,N3205,N3206,N3207,N3208,N3209,N3210,N3211,N3212,N3213,N3214,N3215,N3216,N3217,N3218,N3219,N3220,  \\note fields as indicated
  N3221,N3222,N3223,N3224,N3225,N3226,N3227,N3228,N3229,N3230,N3231,N3232,N3233,N3234,N3235,N3236,N3237,N3238,N3239,N3240,  \\note fields as indicated
  N3241,N3242,N3243,N3244,N3245,N3246,N3247,N3248,N3249,N3250,N3251,N3252,N3253,N3254,N3255,N3256,N3257,N3258,N3259,N3260,  \\note fields as indicated
  N3261,N3262,N3263,N3264,N3265,N3266,N3267,N3268,N3269,N3270,N3271,N3272,N3273,N3274,N3275,N3276,N3277,N3278,N3279,N3280,  \\note fields as indicated
  N3281,N3282,N3283,N3284,N3285,N3286,N3287,N3288,N3289,N3290,N3291,N3292,N3293,N3294,N3295,N3296,N3297,N3298,N3299,N3300,  \\note fields as indicated
  N3301,N3302,N3303,N3304,N3305,N3306,N3307,N3308,N3309,N3310,N3311,N3312,N3313,N3314,N3315,N3316,N3317,N3318,N3319,N3320,  \\note fields as indicated
  N3321,N3322,N3323,N3324,N3325,N3326,N3327,N3328,N3329,N3330,N3331,N3332,N3333,N3334,N3335,N3336,N3337,N3338,N3339,N3340,  \\note fields as indicated
  N3341,N3342,N3343,N3344,N3345,N3346,N3347,N3348,N3349,N3350,N3351,N3352,N3353,N3354,N3355,N3356,N3357,N3358,N3359,N3360,  \\note fields as indicated
  N3361,N3362,N3363,N3364,N3365,N3366,N3367,N3368,N3369,N3370,N3371,N3372,N3373,N3374,N3375,N3376,N3377,N3378,N3379,N3380,  \\note fields as indicated
  N3381,N3382,N3383,N3384,N3385,N3386,N3387,N3388,N3389,N3390,N3391,N3392,N3393,N3394,N3395,N3396,N3397,N3398,N3399,N3400,  \\note fields as indicated
  N3401,N3402,N3403,N3404,N3405,N3406,N3407,N3408,N3409,N3410,N3411,N3412,N3413,N3414,N3415,N3416,N3417,N3418,N3419,N3420,  \\note fields as indicated
  N3421,N3422,N3423,N3424,N3425,N3426,N3427,N3428,N3429,N3430,N3431,N3432,N3433,N3434,N3435,N3436,N3437,N3438,N3439,N3440,  \\note fields as indicated
  N3441,N3442,N3443,N3444,N3445,N3446,N3447,N3448,N3449,N3450,N3451,N3452,N3453,N3454,N3455,N3456,N3457,N3458,N3459,N3460,  \\note fields as indicated
  N3461,N3462,N3463,N3464,N3465,N3466,N3467,N3468,N3469,N3470,N3471,N3472,N3473,N3474,N3475,N3476,N3477,N3478,N3479,N3480,  \\note fields as indicated
  N3481,N3482,N3483,N3484,N3485,N3486,N3487,N3488,N3489,N3490,N3491,N3492,N3493,N3494,N3495,N3496,N3497,N3498,N3499,N3500,  \\note fields as indicated
  N3501,N3502,N3503,N3504,N3505,N3506,N3507,N3508,N3509,N3510,N3511,N3512,N3513,N3514,N3515,N3516,N3517,N3518,N3519,N3520,  \\note fields as indicated
  N3521,N3522,N3523,N3524,N3525,N3526,N3527,N3528,N3529,N3530,N3531,N3532,N3533,N3534,N3535,N3536,N3537,N3538,N3539,N3540,  \\note fields as indicated
  N3541,N3542,N3543,N3544,N3545,N3546,N3547,N3548,N3549,N3550,N3551,N3552,N3553,N3554,N3555,N3556,N3557,N3558,N3559,N3560,  \\note fields as indicated
  N3561,N3562,N3563,N3564,N3565,N3566,N3567,N3568,N3569,N3570,N3571,N3572,N3573,N3574,N3575,N3576,N3577,N3578,N3579,N3580,  \\note fields as indicated
  N3581,N3582,N3583,N3584,N3585,N3586,N3587,N3588,N3589,N3590,N3591,N3592,N3593,N3594,N3595,N3596,N3597,N3598,N3599,N3600,  \\note fields as indicated
  N3601,N3602,N3603,N3604,N3605,N3606,N3607,N3608,N3609,N3610,N3611,N3612,N3613,N3614,N3615,N3616,N3617,N3618,N3619,N3620,  \\note fields as indicated
  N3621,N3622,N3623,N3624,N3625,N3626,N3627,N3628,N3629,N3630,N3631,N3632,N3633,N3634,N3635,N3636,N3637,N3638,N3639,N3640,  \\note fields as indicated
  N3641,N3642,N3643,N3644,N3645,N3646,N3647,N3648,N3649,N3650,N3651,N3652,N3653,N3654,N3655,N3656,N3657,N3658,N3659,N3660,  \\note fields as indicated
  N3661,N3662,N3663,N3664,N3665,N3666,N3667,N3668,N3669,N3670,N3671,N3672,N3673,N3674,N3675,N3676,N3677,N3678,N3679,N3680,  \\note fields as indicated
  N3681,N3682,N3683,N3684,N3685,N3686,N3687,N3688,N3689,N3690,N3691,N3692,N3693,N3694,N3695,N3696,N3697,N3698,N3699,N3700,  \\note fields as indicated
  N3701,N3702,N3703,N3704,N3705,N3706,N3707,N3708,N3709,N3710,N3711,N3712,N3713,N3714,N3715,N3716,N3717,N3718,N3719,N3720,  \\note fields as indicated
  N3721,N3722,N3723,N3724,N3725,N3726,N3727,N3728,N3729,N3730,N3731,N3732,N3733,N3734,N3735,N3736,N3737,N3738,N3739,N3740,  \\note fields as indicated
  N3741,N3742,N3743,N3744,N3745,N3746,N3747,N3748,N3749,N3750,N3751,N3752,N3753,N3754,N3755,N3756,N3757,N3758,N3759,N3760,  \\note fields as indicated
  N3761,N3762,N3763,N3764,N3765,N3766,N3767,N3768,N3769,N3770,N3771,N3772,N3773,N3774,N3775,N3776,N3777,N3778,N3779,N3780,  \\note fields as indicated
  N3781,N3782,N3783,N3784,N3785,N3786,N3787,N3788,N3789,N3790,N3791,N3792,N3793,N3794,N3795,N3796,N3797,N3798,N3799,N3800,  \\note fields as indicated
  N3801,N3802,N3803,N3804,N3805,N3806,N3807,N3808,N3809,N3810,N3811,N3812,N3813,N3814,N3815,N3816,N3817,N3818,N3819,N3820,  \\note fields as indicated
  N3821,N3822,N3823,N3824,N3825,N3826,N3827,N3828,N3829,N3830,N3831,N3832,N3833,N3834,N3835,N3836,N3837,N3838,N3839,N3840,  \\note fields as indicated
  N3841,N3842,N3843,N3844,N3845,N3846,N3847,N3848,N3849,N3850,N3851,N3852,N3853,N3854,N3855,N3856,N3857,N3858,N3859,N3860,  \\note fields as indicated
  N3861,N3862,N3863,N3864,N3865,N3866,N3867,N3868,N3869,N3870,N3871,N3872,N3873,N3874,N3875,N3876,N3877,N3878,N3879,N3880,  \\note fields as indicated
  N3881,N3882,N3883,N3884,N3885,N3886,N3887,N3888,N3889,N3890,N3891,N3892,N3893,N3894,N3895,N3896,N3897,N3898,N3899,N3900,  \\note fields as indicated
  N3901,N3902,N3903,N3904,N3905,N3906,N3907,N3908,N3909,N3910,N3911,N3912,N3913,N3914,N3915,N3916,N3917,N3918,N3919,N3920,  \\note fields as indicated
  N3921,N3922,N3923,N3924,N3925,N3926,N3927,N3928,N3929,N3930,N3931,N3932,N3933,N3934,N3935,N3936,N3937,N3938,N3939,N3940,  \\note fields as indicated
  N3941,N3942,N3943,N3944,N3945,N3946,N3947,N3948,N3949,N3950,N3951,N3952,N3953,N3954,N3955,N3956,N3957,N3958,N3959,N3960,  \\note fields as indicated
  N3961,N3962,N3963,N3964,N3965,N3966,N3967,N3968,N3969,N3970,N3971,N3972,N3973,N3974,N3975,N3976,N3977,N3978,N3979,N3980,  \\note fields as indicated
  N3981,N3982,N3983,N3984,N3985,N3986,N3987,N3988,N3989,N3990,N3991,N3992,N3993,N3994,N3995,N3996,N3997,N3998,N3999,N4000,  \\note fields as indicated
  N4001,N4002,N4003,N4004,N4005,N4006,N4007,N4008,N4009,N4010,N4011,N4012,N4013,N4014,N4015,N4016,N4017,N4018,N4019,N4020,  \\note fields as indicated
  N4021,N4022,N4023,N4024,N4025,N4026,N4027,N4028,N4029,N4030,N4031,N4032,N4033,N4034,N4035,N4036,N4037,N4038,N4039,N4040,  \\note fields as indicated
  N4041,N4042,N4043,N4044,N4045,N4046,N4047,N4048,N4049,N4050,N4051,N4052,N4053,N4054,N4055,N4056,N4057,N4058,N4059,N4060,  \\note fields as indicated
  N4061,N4062,N4063,N4064,N4065,N4066,N4067,N4068,N4069,N4070,N4071,N4072,N4073,N4074,N4075,N4076,N4077,N4078,N4079,N4080,  \\note fields as indicated
  N4081,N4082,N4083,N4084,N4085,N4086,N4087,N4088,N4089,N4090,N4091,N4092,N4093,N4094,N4095,N4096,N4097,N4098,N4099,N4100,  \\note fields as indicated
  N4101,N4102,N4103,N4104,N4105,N4106,N4107,N4108,N4109,N4110,N4111,N4112,N4113,N4114,N4115,N4116,N4117,N4118,N4119,N4120,  \\note fields as indicated
  N4121,N4122,N4123,N4124,N4125,N4126,N4127,N4128,N4129,N4130,N4131,N4132,N4133,N4134,N4135,N4136,N4137,N4138,N4139,N4140,  \\note fields as indicated
  N4141,N4142,N4143,N4144,N4145,N4146,N4147,N4148,N4149,N4150,N4151,N4152,N4153,N4154,N4155,N4156,N4157,N4158,N4159,N4160,  \\note fields as indicated
  N4161,N4162,N4163,N4164,N4165,N4166,N4167,N4168,N4169,N4170,N4171,N4172,N4173,N4174,N4175,N4176,N4177,N4178,N4179,N4180,  \\note fields as indicated
  N4181,N4182,N4183,N4184,N4185,N4186,N4187,N4188,N4189,N4190,N4191,N4192,N4193,N4194,N4195,N4196,N4197,N4198,N4199,N4200,  \\note fields as indicated
  N4201,N4202,N4203,N4204,N4205,N4206,N4207,N4208,N4209,N4210,N4211,N4212,N4213,N4214,N4215,N4216,N4217,N4218,N4219,N4220,  \\note fields as indicated
  N4221,N4222,N4223,N4224,N4225,N4226,N4227,N4228,N4229,N4230,N4231,N4232,N4233,N4234,N4235,N4236,N4237,N4238,N4239,N4240,  \\note fields as indicated
  N4241,N4242,N4243,N4244,N4245,N4246,N4247,N4248,N4249,N4250,N4251,N4252,N4253,N4254,N4255,N4256,N4257,N4258,N4259,N4260,  \\note fields as indicated
  N4261,N4262,N4263,N4264,N4265,N4266,N4267,N4268,N4269,N4270,N4271,N4272,N4273,N4274,N4275,N4276,N4277,N4278,N4279,N4280,  \\note fields as indicated
  N4281,N4282,N4283,N4284,N4285,N4286,N4287,N4288,N4289,N4290,N4291,N4292,N4293,N4294,N4295,N4296,N4297,N4298,N4299,N4300,  \\note fields as indicated
  N4301,N4302,N4303,N4304,N4305,N4306,N4307,N4308,N4309,N4310,N4311,N4312,N4313,N4314,N4315,N4316,N4317,N4318,N4319,N4320,  \\note fields as indicated
  N4321,N4322,N4323,N4324,N4325,N4326,N4327,N4328,N4329,N4330,N4331,N4332,N4333,N4334,N4335,N4336,N4337,N4338,N4339,N4340,  \\note fields as indicated
  N4341,N4342,N4343,N4344,N4345,N4346,N4347,N4348,N4349,N4350,N4351,N4352,N4353,N4354,N4355,N4356,N4357,N4358,N4359,N4360,  \\note fields as indicated
  N4361,N4362,N4363,N4364,N4365,N4366,N4367,N4368,N4369,N4370,N4371,N4372,N4373,N4374,N4375,N4376,N4377,N4378,N4379,N4380,  \\note fields as indicated
  N4381,N4382,N4383,N4384,N4385,N4386,N4387,N4388,N4389,N4390,N4391,N4392,N4393,N4394,N4395,N4396,N4397,N4398,N4399,N4400,  \\note fields as indicated
  N4401,N4402,N4403,N4404,N4405,N4406,N4407,N4408,N4409,N4410,N4411,N4412,N4413,N4414,N4415,N4416,N4417,N4418,N4419,N4420,  \\note fields as indicated
  N4421,N4422,N4423,N4424,N4425,N4426,N4427,N4428,N4429,N4430,N4431,N4432,N4433,N4434,N4435,N4436,N4437,N4438,N4439,N4440,  \\note fields as indicated
  N4441,N4442,N4443,N4444,N4445,N4446,N4447,N4448,N4449,N4450,N4451,N4452,N4453,N4454,N4455,N4456,N4457,N4458,N4459,N4460,  \\note fields as indicated
  N4461,N4462,N4463,N4464,N4465,N4466,N4467,N4468,N4469,N4470,N4471,N4472,N4473,N4474,N4475,N4476,N4477,N4478,N4479,N4480,  \\note fields as indicated
  N4481,N4482,N4483,N4484,N4485,N4486,N4487,N4488,N4489,N4490,N4491,N4492,N4493,N4494,N4495,N4496,N4497,N4498,N4499,N4500,  \\note fields as indicated
  N4501,N4502,N4503,N4504,N4505,N4506,N4507,N4508,N4509,N4510,N4511,N4512,N4513,N4514,N4515,N4516,N4517,N4518,N4519,N4520,  \\note fields as indicated
  N4521,N4522,N4523,N4524,N4525,N4526,N4527,N4528,N4529,N4530,N4531,N4532,N4533,N4534,N4535,N4536,N4537,N4538,N4539,N4540,  \\note fields as indicated
  N4541,N4542,N4543,N4544,N4545,N4546,N4547,N4548,N4549,N4550,N4551,N4552,N4553,N4554,N4555,N4556,N4557,N4558,N4559,N4560,  \\note fields as indicated
  N4561,N4562,N4563,N4564,N4565,N4566,N4567,N4568,N4569,N4570,N4571,N4572,N4573,N4574,N4575,N4576,N4577,N4578,N4579,N4580,  \\note fields as indicated
  N4581,N4582,N4583,N4584,N4585,N4586,N4587,N4588,N4589,N4590,N4591,N4592,N4593,N4594,N4595,N4596,N4597,N4598,N4599,N4600,  \\note fields as indicated
  N4601,N4602,N4603,N4604,N4605,N4606,N4607,N4608,N4609,N4610,N4611,N4612,N4613,N4614,N4615,N4616,N4617,N4618,N4619,N4620,  \\note fields as indicated
  N4621,N4622,N4623,N4624,N4625,N4626,N4627,N4628,N4629,N4630,N4631,N4632,N4633,N4634,N4635,N4636,N4637,N4638,N4639,N4640,  \\note fields as indicated
  N4641,N4642,N4643,N4644,N4645,N4646,N4647,N4648,N4649,N4650,N4651,N4652,N4653,N4654,N4655,N4656,N4657,N4658,N4659,N4660,  \\note fields as indicated
  N4661,N4662,N4663,N4664,N4665,N4666,N4667,N4668,N4669,N4670,N4671,N4672,N4673,N4674,N4675,N4676,N4677,N4678,N4679,N4680,  \\note fields as indicated
  N4681,N4682,N4683,N4684,N4685,N4686,N4687,N4688,N4689,N4690,N4691,N4692,N4693,N4694,N4695,N4696,N4697,N4698,N4699,N4700,  \\note fields as indicated
  N4701,N4702,N4703,N4704,N4705,N4706,N4707,N4708,N4709,N4710,N4711,N4712,N4713,N4714,N4715,N4716,N4717,N4718,N4719,N4720,  \\note fields as indicated
  N4721,N4722,N4723,N4724,N4725,N4726,N4727,N4728,N4729,N4730,N4731,N4732,N4733,N4734,N4735,N4736,N4737,N4738,N4739,N4740,  \\note fields as indicated
  N4741,N4742,N4743,N4744,N4745,N4746,N4747,N4748,N4749,N4750,N4751,N4752,N4753,N4754,N4755,N4756,N4757,N4758,N4759,N4760,  \\note fields as indicated
  N4761,N4762,N4763,N4764,N4765,N4766,N4767,N4768,N4769,N4770,N4771,N4772,N4773,N4774,N4775,N4776,N4777,N4778,N4779,N4780,  \\note fields as indicated
  N4781,N4782,N4783,N4784,N4785,N4786,N4787,N4788,N4789,N4790,N4791,N4792,N4793,N4794,N4795,N4796,N4797,N4798,N4799,N4800,  \\note fields as indicated
  N4801,N4802,N4803,N4804,N4805,N4806,N4807,N4808,N4809,N4810,N4811,N4812,N4813,N4814,N4815,N4816,N4817,N4818,N4819,N4820,  \\note fields as indicated
  N4821,N4822,N4823,N4824,N4825,N4826,N4827,N4828,N4829,N4830,N4831,N4832,N4833,N4834,N4835,N4836,N4837,N4838,N4839,N4840,  \\note fields as indicated
  N4841,N4842,N4843,N4844,N4845,N4846,N4847,N4848,N4849,N4850,N4851,N4852,N4853,N4854,N4855,N4856,N4857,N4858,N4859,N4860,  \\note fields as indicated
  N4861,N4862,N4863,N4864,N4865,N4866,N4867,N4868,N4869,N4870,N4871,N4872,N4873,N4874,N4875,N4876,N4877,N4878,N4879,N4880,  \\note fields as indicated
  N4881,N4882,N4883,N4884,N4885,N4886,N4887,N4888,N4889,N4890,N4891,N4892,N4893,N4894,N4895,N4896,N4897,N4898,N4899,N4900,  \\note fields as indicated
  N4901,N4902,N4903,N4904,N4905,N4906,N4907,N4908,N4909,N4910,N4911,N4912,N4913,N4914,N4915,N4916,N4917,N4918,N4919,N4920,  \\note fields as indicated
  N4921,N4922,N4923,N4924,N4925,N4926,N4927,N4928,N4929,N4930,N4931,N4932,N4933,N4934,N4935,N4936,N4937,N4938,N4939,N4940,  \\note fields as indicated
  N4941,N4942,N4943,N4944,N4945,N4946,N4947,N4948,N4949,N4950,N4951,N4952,N4953,N4954,N4955,N4956,N4957,N4958,N4959,N4960,  \\note fields as indicated
  N4961,N4962,N4963,N4964,N4965,N4966,N4967,N4968,N4969,N4970,N4971,N4972,N4973,N4974,N4975,N4976,N4977,N4978,N4979,N4980,  \\note fields as indicated
  N4981,N4982,N4983,N4984,N4985,N4986,N4987,N4988,N4989,N4990,N4991,N4992,N4993,N4994,N4995,N4996,N4997,N4998,N4999,N5000,  \\note fields as indicated
  N5001,N5002,N5003,N5004,N5005,N5006,N5007,N5008,N5009,N5010,N5011,N5012,N5013,N5014,N5015,N5016,N5017,N5018,N5019,N5020,  \\note fields as indicated
  N5021,N5022,N5023,N5024,N5025,N5026,N5027,N5028,N5029,N5030,N5031,N5032,N5033,N5034,N5035,N5036,N5037,N5038,N5039,N5040,  \\note fields as indicated
  N5041,N5042,N5043,N5044,N5045,N5046,N5047,N5048,N5049,N5050,N5051,N5052,N5053,N5054,N5055,N5056,N5057,N5058,N5059,N5060,  \\note fields as indicated
  N5061,N5062,N5063,N5064,N5065,N5066,N5067,N5068,N5069,N5070,N5071,N5072,N5073,N5074,N5075,N5076,N5077,N5078,N5079,N5080,  \\note fields as indicated
  N5081,N5082,N5083,N5084,N5085,N5086,N5087,N5088,N5089,N5090,N5091,N5092,N5093,N5094,N5095,N5096,N5097,N5098,N5099,N5100,  \\note fields as indicated
  N5101,N5102,N5103,N5104,N5105,N5106,N5107,N5108,N5109,N5110,N5111,N5112,N5113,N5114,N5115,N5116,N5117,N5118,N5119,N5120,  \\note fields as indicated
  N5121,N5122,N5123,N5124,N5125,N5126,N5127,N5128,N5129,N5130,N5131,N5132,N5133,N5134,N5135,N5136,N5137,N5138,N5139,N5140,  \\note fields as indicated
  N5141,N5142,N5143,N5144,N5145,N5146,N5147,N5148,N5149,N5150,N5151,N5152,N5153,N5154,N5155,N5156,N5157,N5158,N5159,N5160,  \\note fields as indicated
  N5161,N5162,N5163,N5164,N5165,N5166,N5167,N5168,N5169,N5170,N5171,N5172,N5173,N5174,N5175,N5176,N5177,N5178,N5179,N5180,  \\note fields as indicated
  N5181,N5182,N5183,N5184,N5185,N5186,N5187,N5188,N5189,N5190,N5191,N5192,N5193,N5194,N5195,N5196,N5197,N5198,N5199,N5200,  \\note fields as indicated
  N5201,N5202,N5203,N5204,N5205,N5206,N5207,N5208,N5209,N5210,N5211,N5212,N5213,N5214,N5215,N5216,N5217,N5218,N5219,N5220,  \\note fields as indicated
  N5221,N5222,N5223,N5224,N5225,N5226,N5227,N5228,N5229,N5230,N5231,N5232,N5233,N5234,N5235,N5236,N5237,N5238,N5239,N5240,  \\note fields as indicated
  N5241,N5242,N5243,N5244,N5245,N5246,N5247,N5248,N5249,N5250,N5251,N5252,N5253,N5254,N5255,N5256,N5257,N5258,N5259,N5260,  \\note fields as indicated
  N5261,N5262,N5263,N5264,N5265,N5266,N5267,N5268,N5269,N5270,N5271,N5272,N5273,N5274,N5275,N5276,N5277,N5278,N5279,N5280,  \\note fields as indicated
  N5281,N5282,N5283,N5284,N5285,N5286,N5287,N5288,N5289,N5290,N5291,N5292,N5293,N5294,N5295,N5296,N5297,N5298,N5299,N5300,  \\note fields as indicated
  N5301,N5302,N5303,N5304,N5305,N5306,N5307,N5308,N5309,N5310,N5311,N5312,N5313,N5314,N5315,N5316,N5317,N5318,N5319,N5320,  \\note fields as indicated
  N5321,N5322,N5323,N5324,N5325,N5326,N5327,N5328,N5329,N5330,N5331,N5332,N5333,N5334,N5335,N5336,N5337,N5338,N5339,N5340,  \\note fields as indicated
  N5341,N5342,N5343,N5344,N5345,N5346,N5347,N5348,N5349,N5350,N5351,N5352,N5353,N5354,N5355,N5356,N5357,N5358,N5359,N5360,  \\note fields as indicated
  N5361,N5362,N5363,N5364,N5365,N5366,N5367,N5368,N5369,N5370,N5371,N5372,N5373,N5374,N5375,N5376,N5377,N5378,N5379,N5380,  \\note fields as indicated
  N5381,N5382,N5383,N5384,N5385,N5386,N5387,N5388,N5389,N5390,N5391,N5392,N5393,N5394,N5395,N5396,N5397,N5398,N5399,N5400,  \\note fields as indicated
  N5401,N5402,N5403,N5404,N5405,N5406,N5407,N5408,N5409,N5410,N5411,N5412,N5413,N5414,N5415,N5416,N5417,N5418,N5419,N5420,  \\note fields as indicated
  N5421,N5422,N5423,N5424,N5425,N5426,N5427,N5428,N5429,N5430,N5431,N5432,N5433,N5434,N5435,N5436,N5437,N5438,N5439,N5440,  \\note fields as indicated
  N5441,N5442,N5443,N5444,N5445,N5446,N5447,N5448,N5449,N5450,N5451,N5452,N5453,N5454,N5455,N5456,N5457,N5458,N5459,N5460,  \\note fields as indicated
  N5461,N5462,N5463,N5464,N5465,N5466,N5467,N5468,N5469,N5470,N5471,N5472,N5473,N5474,N5475,N5476,N5477,N5478,N5479,N5480,  \\note fields as indicated
  N5481,N5482,N5483,N5484,N5485,N5486,N5487,N5488,N5489,N5490,N5491,N5492,N5493,N5494,N5495,N5496,N5497,N5498,N5499,N5500,  \\note fields as indicated
  N5501,N5502,N5503,N5504,N5505,N5506,N5507,N5508,N5509,N5510,N5511,N5512,N5513,N5514,N5515,N5516,N5517,N5518,N5519,N5520,  \\note fields as indicated
  N5521,N5522,N5523,N5524,N5525,N5526,N5527,N5528,N5529,N5530,N5531,N5532,N5533,N5534,N5535,N5536,N5537,N5538,N5539,N5540,  \\note fields as indicated
  N5541,N5542,N5543,N5544,N5545,N5546,N5547,N5548,N5549,N5550,N5551,N5552,N5553,N5554,N5555,N5556,N5557,N5558,N5559,N5560,  \\note fields as indicated
  N5561,N5562,N5563,N5564,N5565,N5566,N5567,N5568,N5569,N5570,N5571,N5572,N5573,N5574,N5575,N5576,N5577,N5578,N5579,N5580,  \\note fields as indicated
  N5581,N5582,N5583,N5584,N5585,N5586,N5587,N5588,N5589,N5590,N5591,N5592,N5593,N5594,N5595,N5596,N5597,N5598,N5599,N5600,  \\note fields as indicated
  N5601,N5602,N5603,N5604,N5605,N5606,N5607,N5608,N5609,N5610,N5611,N5612,N5613,N5614,N5615,N5616,N5617,N5618,N5619,N5620,  \\note fields as indicated
  N5621,N5622,N5623,N5624,N5625,N5626,N5627,N5628,N5629,N5630,N5631,N5632,N5633,N5634,N5635,N5636,N5637,N5638,N5639,N5640,  \\note fields as indicated
  N5641,N5642,N5643,N5644,N5645,N5646,N5647,N5648,N5649,N5650,N5651,N5652,N5653,N5654,N5655,N5656,N5657,N5658,N5659,N5660,  \\note fields as indicated
  N5661,N5662,N5663,N5664,N5665,N5666,N5667,N5668,N5669,N5670,N5671,N5672,N5673,N5674,N5675,N5676,N5677,N5678,N5679,N5680,  \\note fields as indicated
  N5681,N5682,N5683,N5684,N5685,N5686,N5687,N5688,N5689,N5690,N5691,N5692,N5693,N5694,N5695,N5696,N5697,N5698,N5699,N5700,  \\note fields as indicated
  N5701,N5702,N5703,N5704,N5705,N5706,N5707,N5708,N5709,N5710,N5711,N5712,N5713,N5714,N5715,N5716,N5717,N5718,N5719,N5720,  \\note fields as indicated
  N5721,N5722,N5723,N5724,N5725,N5726,N5727,N5728,N5729,N5730,N5731,N5732,N5733,N5734,N5735,N5736,N5737,N5738,N5739,N5740,  \\note fields as indicated
  N5741,N5742,N5743,N5744,N5745,N5746,N5747,N5748,N5749,N5750,N5751,N5752,N5753,N5754,N5755,N5756,N5757,N5758,N5759,N5760,  \\note fields as indicated
  N5761,N5762,N5763,N5764,N5765,N5766,N5767,N5768,N5769,N5770,N5771,N5772,N5773,N5774,N5775,N5776,N5777,N5778,N5779,N5780,  \\note fields as indicated
  N5781,N5782,N5783,N5784,N5785,N5786,N5787,N5788,N5789,N5790,N5791,N5792,N5793,N5794,N5795,N5796,N5797,N5798,N5799,N5800,  \\note fields as indicated
  N5801,N5802,N5803,N5804,N5805,N5806,N5807,N5808,N5809,N5810,N5811,N5812,N5813,N5814,N5815,N5816,N5817,N5818,N5819,N5820,  \\note fields as indicated
  N5821,N5822,N5823,N5824,N5825,N5826,N5827,N5828,N5829,N5830,N5831,N5832,N5833,N5834,N5835,N5836,N5837,N5838,N5839,N5840,  \\note fields as indicated
  N5841,N5842,N5843,N5844,N5845,N5846,N5847,N5848,N5849,N5850,N5851,N5852,N5853,N5854,N5855,N5856,N5857,N5858,N5859,N5860,  \\note fields as indicated
  N5861,N5862,N5863,N5864,N5865,N5866,N5867,N5868,N5869,N5870,N5871,N5872,N5873,N5874,N5875,N5876,N5877,N5878,N5879,N5880,  \\note fields as indicated
  N5881,N5882,N5883,N5884,N5885,N5886,N5887,N5888,N5889,N5890,N5891,N5892,N5893,N5894,N5895,N5896,N5897,N5898,N5899,N5900,  \\note fields as indicated
  N5901,N5902,N5903,N5904,N5905,N5906,N5907,N5908,N5909,N5910,N5911,N5912,N5913,N5914,N5915,N5916,N5917,N5918,N5919,N5920,  \\note fields as indicated
  N5921,N5922,N5923,N5924,N5925,N5926,N5927,N5928,N5929,N5930,N5931,N5932,N5933,N5934,N5935,N5936,N5937,N5938,N5939,N5940,  \\note fields as indicated
  N5941,N5942,N5943,N5944,N5945,N5946,N5947,N5948,N5949,N5950,N5951,N5952,N5953,N5954,N5955,N5956,N5957,N5958,N5959,N5960,  \\note fields as indicated
  N5961,N5962,N5963,N5964,N5965,N5966,N5967,N5968,N5969,N5970,N5971,N5972,N5973,N5974,N5975,N5976,N5977,N5978,N5979,N5980,  \\note fields as indicated
  N5981,N5982,N5983,N5984,N5985,N5986,N5987,N5988,N5989,N5990,N5991,N5992,N5993,N5994,N5995,N5996,N5997,N5998,N5999,N6000,  \\note fields as indicated
  N6001,N6002,N6003,N6004,N6005,N6006,N6007,N6008,N6009,N6010,N6011,N6012,N6013,N6014,N6015,N6016,N6017,N6018,N6019,N6020,  \\note fields as indicated
  N6021,N6022,N6023,N6024,N6025,N6026,N6027,N6028,N6029,N6030,N6031,N6032,N6033,N6034,N6035,N6036,N6037,N6038,N6039,N6040,  \\note fields as indicated
  N6041,N6042,N6043,N6044,N6045,N6046,N6047,N6048,N6049,N6050,N6051,N6052,N6053,N6054,N6055,N6056,N6057,N6058,N6059,N6060,  \\note fields as indicated
  N6061,N6062,N6063,N6064,N6065,N6066,N6067,N6068,N6069,N6070,N6071,N6072,N6073,N6074,N6075,N6076,N6077,N6078,N6079,N6080,  \\note fields as indicated
  N6081,N6082,N6083,N6084,N6085,N6086,N6087,N6088,N6089,N6090,N6091,N6092,N6093,N6094,N6095,N6096,N6097,N6098,N6099,N6100,  \\note fields as indicated
  N6101,N6102,N6103,N6104,N6105,N6106,N6107,N6108,N6109,N6110,N6111,N6112,N6113,N6114,N6115,N6116,N6117,N6118,N6119,N6120,  \\note fields as indicated
  N6121,N6122,N6123,N6124,N6125,N6126,N6127,N6128,N6129,N6130,N6131,N6132,N6133,N6134,N6135,N6136,N6137,N6138,N6139,N6140,  \\note fields as indicated
  N6141,N6142,N6143,N6144,N6145,N6146,N6147,N6148,N6149,N6150,N6151,N6152,N6153,N6154,N6155,N6156,N6157,N6158,N6159,N6160,  \\note fields as indicated
  N6161,N6162,N6163,N6164,N6165,N6166,N6167,N6168,N6169,N6170,N6171,N6172,N6173,N6174,N6175,N6176,N6177,N6178,N6179,N6180,  \\note fields as indicated
  N6181,N6182,N6183,N6184,N6185,N6186,N6187,N6188,N6189,N6190,N6191,N6192,N6193,N6194,N6195,N6196,N6197,N6198,N6199,N6200,  \\note fields as indicated
  N6201,N6202,N6203,N6204,N6205,N6206,N6207,N6208,N6209,N6210,N6211,N6212,N6213,N6214,N6215,N6216,N6217,N6218,N6219,N6220,  \\note fields as indicated
  N6221,N6222,N6223,N6224,N6225,N6226,N6227,N6228,N6229,N6230,N6231,N6232,N6233,N6234,N6235,N6236,N6237,N6238,N6239,N6240,  \\note fields as indicated
  N6241,N6242,N6243,N6244,N6245,N6246,N6247,N6248,N6249,N6250,N6251,N6252,N6253,N6254,N6255,N6256,N6257,N6258,N6259,N6260,  \\note fields as indicated
  N6261,N6262,N6263,N6264,N6265,N6266,N6267,N6268,N6269,N6270,N6271,N6272,N6273,N6274,N6275,N6276,N6277,N6278,N6279,N6280,  \\note fields as indicated
  N6281,N6282,N6283,N6284,N6285,N6286,N6287,N6288,N6289,N6290,N6291,N6292,N6293,N6294,N6295,N6296,N6297,N6298,N6299,N6300,  \\note fields as indicated
  N6301,N6302,N6303,N6304,N6305,N6306,N6307,N6308,N6309,N6310,N6311,N6312,N6313,N6314,N6315,N6316,N6317,N6318,N6319,N6320,  \\note fields as indicated
  N6321,N6322,N6323,N6324,N6325,N6326,N6327,N6328,N6329,N6330,N6331,N6332,N6333,N6334,N6335,N6336,N6337,N6338,N6339,N6340,  \\note fields as indicated
  N6341,N6342,N6343,N6344,N6345,N6346,N6347,N6348,N6349,N6350,N6351,N6352,N6353,N6354,N6355,N6356,N6357,N6358,N6359,N6360,  \\note fields as indicated
  N6361,N6362,N6363,N6364,N6365,N6366,N6367,N6368,N6369,N6370,N6371,N6372,N6373,N6374,N6375,N6376,N6377,N6378,N6379,N6380,  \\note fields as indicated
  N6381,N6382,N6383,N6384,N6385,N6386,N6387,N6388,N6389,N6390,N6391,N6392,N6393,N6394,N6395,N6396,N6397,N6398,N6399,N6400,  \\note fields as indicated
  N6401,N6402,N6403,N6404,N6405,N6406,N6407,N6408,N6409,N6410,N6411,N6412,N6413,N6414,N6415,N6416,N6417,N6418,N6419,N6420,  \\note fields as indicated
  N6421,N6422,N6423,N6424,N6425,N6426,N6427,N6428,N6429,N6430,N6431,N6432,N6433,N6434,N6435,N6436,N6437,N6438,N6439,N6440,  \\note fields as indicated
  N6441,N6442,N6443,N6444,N6445,N6446,N6447,N6448,N6449,N6450,N6451,N6452,N6453,N6454,N6455,N6456,N6457,N6458,N6459,N6460,  \\note fields as indicated
  N6461,N6462,N6463,N6464,N6465,N6466,N6467,N6468,N6469,N6470,N6471,N6472,N6473,N6474,N6475,N6476,N6477,N6478,N6479,N6480,  \\note fields as indicated
  N6481,N6482,N6483,N6484,N6485,N6486,N6487,N6488,N6489,N6490,N6491,N6492,N6493,N6494,N6495,N6496,N6497,N6498,N6499,N6500,  \\note fields as indicated
  N6501,N6502,N6503,N6504,N6505,N6506,N6507,N6508,N6509,N6510,N6511,N6512,N6513,N6514,N6515,N6516,N6517,N6518,N6519,N6520,  \\note fields as indicated
  N6521,N6522,N6523,N6524,N6525,N6526,N6527,N6528,N6529,N6530,N6531,N6532,N6533,N6534,N6535,N6536,N6537,N6538,N6539,N6540,  \\note fields as indicated
  N6541,N6542,N6543,N6544,N6545,N6546,N6547,N6548,N6549,N6550,N6551,N6552,N6553,N6554,N6555,N6556,N6557,N6558,N6559,N6560,  \\note fields as indicated
  N6561,N6562,N6563,N6564,N6565,N6566,N6567,N6568,N6569,N6570,N6571,N6572,N6573,N6574,N6575,N6576,N6577,N6578,N6579,N6580,  \\note fields as indicated
  N6581,N6582,N6583,N6584,N6585,N6586,N6587,N6588,N6589,N6590,N6591,N6592,N6593,N6594,N6595,N6596,N6597,N6598,N6599,N6600,  \\note fields as indicated
  N6601,N6602,N6603,N6604,N6605,N6606,N6607,N6608,N6609,N6610,N6611,N6612,N6613,N6614,N6615,N6616,N6617,N6618,N6619,N6620,  \\note fields as indicated
  N6621,N6622,N6623,N6624,N6625,N6626,N6627,N6628,N6629,N6630,N6631,N6632,N6633,N6634,N6635,N6636,N6637,N6638,N6639,N6640,  \\note fields as indicated
  N6641,N6642,N6643,N6644,N6645,N6646,N6647,N6648,N6649,N6650,N6651,N6652,N6653,N6654,N6655,N6656,N6657,N6658,N6659,N6660,  \\note fields as indicated
  N6661,N6662,N6663,N6664,N6665,N6666,N6667,N6668,N6669,N6670,N6671,N6672,N6673,N6674,N6675,N6676,N6677,N6678,N6679,N6680,  \\note fields as indicated
  N6681,N6682,N6683,N6684,N6685,N6686,N6687,N6688,N6689,N6690,N6691,N6692,N6693,N6694,N6695,N6696,N6697,N6698,N6699,N6700,  \\note fields as indicated
  N6701,N6702,N6703,N6704,N6705,N6706,N6707,N6708,N6709,N6710,N6711,N6712,N6713,N6714,N6715,N6716,N6717,N6718,N6719,N6720,  \\note fields as indicated
  N6721,N6722,N6723,N6724,N6725,N6726,N6727,N6728,N6729,N6730,N6731,N6732,N6733,N6734,N6735,N6736,N6737,N6738,N6739,N6740,  \\note fields as indicated
  N6741,N6742,N6743,N6744,N6745,N6746,N6747,N6748,N6749,N6750,N6751,N6752,N6753,N6754,N6755,N6756,N6757,N6758,N6759,N6760,  \\note fields as indicated
  N6761,N6762,N6763,N6764,N6765,N6766,N6767,N6768,N6769,N6770,N6771,N6772,N6773,N6774,N6775,N6776,N6777,N6778,N6779,N6780,  \\note fields as indicated
  N6781,N6782,N6783,N6784,N6785,N6786,N6787,N6788,N6789,N6790,N6791,N6792,N6793,N6794,N6795,N6796,N6797,N6798,N6799,N6800,  \\note fields as indicated
  N6801,N6802,N6803,N6804,N6805,N6806,N6807,N6808,N6809,N6810,N6811,N6812,N6813,N6814,N6815,N6816,N6817,N6818,N6819,N6820,  \\note fields as indicated
  N6821,N6822,N6823,N6824,N6825,N6826,N6827,N6828,N6829,N6830,N6831,N6832,N6833,N6834,N6835,N6836,N6837,N6838,N6839,N6840,  \\note fields as indicated
  N6841,N6842,N6843,N6844,N6845,N6846,N6847,N6848,N6849,N6850,N6851,N6852,N6853,N6854,N6855,N6856,N6857,N6858,N6859,N6860,  \\note fields as indicated
  N6861,N6862,N6863,N6864,N6865,N6866,N6867,N6868,N6869,N6870,N6871,N6872,N6873,N6874,N6875,N6876,N6877,N6878,N6879,N6880,  \\note fields as indicated
  N6881,N6882,N6883,N6884,N6885,N6886,N6887,N6888,N6889,N6890,N6891,N6892,N6893,N6894,N6895,N6896,N6897,N6898,N6899,N6900,  \\note fields as indicated
  N6901,N6902,N6903,N6904,N6905,N6906,N6907,N6908,N6909,N6910,N6911,N6912,N6913,N6914,N6915,N6916,N6917,N6918,N6919,N6920,  \\note fields as indicated
  N6921,N6922,N6923,N6924,N6925,N6926,N6927,N6928,N6929,N6930,N6931,N6932,N6933,N6934,N6935,N6936,N6937,N6938,N6939,N6940,  \\note fields as indicated
  N6941,N6942,N6943,N6944,N6945,N6946,N6947,N6948,N6949,N6950,N6951,N6952,N6953,N6954,N6955,N6956,N6957,N6958,N6959,N6960,  \\note fields as indicated
  N6961,N6962,N6963,N6964,N6965,N6966,N6967,N6968,N6969,N6970,N6971,N6972,N6973,N6974,N6975,N6976,N6977,N6978,N6979,N6980,  \\note fields as indicated
  N6981,N6982,N6983,N6984,N6985,N6986,N6987,N6988,N6989,N6990,N6991,N6992,N6993,N6994,N6995,N6996,N6997,N6998,N6999,N7000,  \\note fields as indicated
  N7001,N7002,N7003,N7004,N7005,N7006,N7007,N7008,N7009,N7010,N7011,N7012,N7013,N7014,N7015,N7016,N7017,N7018,N7019,N7020,  \\note fields as indicated
  N7021,N7022,N7023,N7024,N7025,N7026,N7027,N7028,N7029,N7030,N7031,N7032,N7033,N7034,N7035,N7036,N7037,N7038,N7039,N7040,  \\note fields as indicated
  N7041,N7042,N7043,N7044,N7045,N7046,N7047,N7048,N7049,N7050,N7051,N7052,N7053,N7054,N7055,N7056,N7057,N7058,N7059,N7060,  \\note fields as indicated
  N7061,N7062,N7063,N7064,N7065,N7066,N7067,N7068,N7069,N7070,N7071,N7072,N7073,N7074,N7075,N7076,N7077,N7078,N7079,N7080,  \\note fields as indicated
  N7081,N7082,N7083,N7084,N7085,N7086,N7087,N7088,N7089,N7090,N7091,N7092,N7093,N7094,N7095,N7096,N7097,N7098,N7099,N7100,  \\note fields as indicated
  N7101,N7102,N7103,N7104,N7105,N7106,N7107,N7108,N7109,N7110,N7111,N7112,N7113,N7114,N7115,N7116,N7117,N7118,N7119,N7120,  \\note fields as indicated
  N7121,N7122,N7123,N7124,N7125,N7126,N7127,N7128,N7129,N7130,N7131,N7132,N7133,N7134,N7135,N7136,N7137,N7138,N7139,N7140,  \\note fields as indicated
  N7141,N7142,N7143,N7144,N7145,N7146,N7147,N7148,N7149,N7150,N7151,N7152,N7153,N7154,N7155,N7156,N7157,N7158,N7159,N7160,  \\note fields as indicated
  N7161,N7162,N7163,N7164,N7165,N7166,N7167,N7168,N7169,N7170,N7171,N7172,N7173,N7174,N7175,N7176,N7177,N7178,N7179,N7180,  \\note fields as indicated
  N7181,N7182,N7183,N7184,N7185,N7186,N7187,N7188,N7189,N7190,N7191,N7192,N7193,N7194,N7195,N7196,N7197,N7198,N7199,N7200,  \\note fields as indicated
  N7201,N7202,N7203,N7204,N7205,N7206,N7207,N7208,N7209,N7210,N7211,N7212,N7213,N7214,N7215,N7216,N7217,N7218,N7219,N7220,  \\note fields as indicated
  N7221,N7222,N7223,N7224,N7225,N7226,N7227,N7228,N7229,N7230,N7231,N7232,N7233,N7234,N7235,N7236,N7237,N7238,N7239,N7240,  \\note fields as indicated
  N7241,N7242,N7243,N7244,N7245,N7246,N7247,N7248,N7249,N7250,N7251,N7252,N7253,N7254,N7255,N7256,N7257,N7258,N7259,N7260,  \\note fields as indicated
  N7261,N7262,N7263,N7264,N7265,N7266,N7267,N7268,N7269,N7270,N7271,N7272,N7273,N7274,N7275,N7276,N7277,N7278,N7279,N7280,  \\note fields as indicated
  N7281,N7282,N7283,N7284,N7285,N7286,N7287,N7288,N7289,N7290,N7291,N7292,N7293,N7294,N7295,N7296,N7297,N7298,N7299,N7300,  \\note fields as indicated
  N7301,N7302,N7303,N7304,N7305,N7306,N7307,N7308,N7309,N7310,N7311,N7312,N7313,N7314,N7315,N7316,N7317,N7318,N7319,N7320,  \\note fields as indicated
  N7321,N7322,N7323,N7324,N7325,N7326,N7327,N7328,N7329,N7330,N7331,N7332,N7333,N7334,N7335,N7336,N7337,N7338,N7339,N7340,  \\note fields as indicated
  N7341,N7342,N7343,N7344,N7345,N7346,N7347,N7348,N7349,N7350,N7351,N7352,N7353,N7354,N7355,N7356,N7357,N7358,N7359,N7360,  \\note fields as indicated
  N7361,N7362,N7363,N7364,N7365,N7366,N7367,N7368,N7369,N7370,N7371,N7372,N7373,N7374,N7375,N7376,N7377,N7378,N7379,N7380,  \\note fields as indicated
  N7381,N7382,N7383,N7384,N7385,N7386,N7387,N7388,N7389,N7390,N7391,N7392,N7393,N7394,N7395,N7396,N7397,N7398,N7399,N7400,  \\note fields as indicated
  N7401,N7402,N7403,N7404,N7405,N7406,N7407,N7408,N7409,N7410,N7411,N7412,N7413,N7414,N7415,N7416,N7417,N7418,N7419,N7420,  \\note fields as indicated
  N7421,N7422,N7423,N7424,N7425,N7426,N7427,N7428,N7429,N7430,N7431,N7432,N7433,N7434,N7435,N7436,N7437,N7438,N7439,N7440,  \\note fields as indicated
  N7441,N7442,N7443,N7444,N7445,N7446,N7447,N7448,N7449,N7450,N7451,N7452,N7453,N7454,N7455,N7456,N7457,N7458,N7459,N7460,  \\note fields as indicated
  N7461,N7462,N7463,N7464,N7465,N7466,N7467,N7468,N7469,N7470,N7471,N7472,N7473,N7474,N7475,N7476,N7477,N7478,N7479,N7480,  \\note fields as indicated
  N7481,N7482,N7483,N7484,N7485,N7486,N7487,N7488,N7489,N7490,N7491,N7492,N7493,N7494,N7495,N7496,N7497,N7498,N7499,N7500,  \\note fields as indicated
  N7501,N7502,N7503,N7504,N7505,N7506,N7507,N7508,N7509,N7510,N7511,N7512,N7513,N7514,N7515,N7516,N7517,N7518,N7519,N7520,  \\note fields as indicated
  N7521,N7522,N7523,N7524,N7525,N7526,N7527,N7528,N7529,N7530,N7531,N7532,N7533,N7534,N7535,N7536,N7537,N7538,N7539,N7540,  \\note fields as indicated
  N7541,N7542,N7543,N7544,N7545,N7546,N7547,N7548,N7549,N7550,N7551,N7552,N7553,N7554,N7555,N7556,N7557,N7558,N7559,N7560,  \\note fields as indicated
  N7561,N7562,N7563,N7564,N7565,N7566,N7567,N7568,N7569,N7570,N7571,N7572,N7573,N7574,N7575,N7576,N7577,N7578,N7579,N7580,  \\note fields as indicated
  N7581,N7582,N7583,N7584,N7585,N7586,N7587,N7588,N7589,N7590,N7591,N7592,N7593,N7594,N7595,N7596,N7597,N7598,N7599,N7600,  \\note fields as indicated
  N7601,N7602,N7603,N7604,N7605,N7606,N7607,N7608,N7609,N7610,N7611,N7612,N7613,N7614,N7615,N7616,N7617,N7618,N7619,N7620,  \\note fields as indicated
  N7621,N7622,N7623,N7624,N7625,N7626,N7627,N7628,N7629,N7630,N7631,N7632,N7633,N7634,N7635,N7636,N7637,N7638,N7639,N7640,  \\note fields as indicated
  N7641,N7642,N7643,N7644,N7645,N7646,N7647,N7648,N7649,N7650,N7651,N7652,N7653,N7654,N7655,N7656,N7657,N7658,N7659,N7660,  \\note fields as indicated
  N7661,N7662,N7663,N7664,N7665,N7666,N7667,N7668,N7669,N7670,N7671,N7672,N7673,N7674,N7675,N7676,N7677,N7678,N7679,N7680,  \\note fields as indicated
  N7681,N7682,N7683,N7684,N7685,N7686,N7687,N7688,N7689,N7690,N7691,N7692,N7693,N7694,N7695,N7696,N7697,N7698,N7699,N7700,  \\note fields as indicated
  N7701,N7702,N7703,N7704,N7705,N7706,N7707,N7708,N7709,N7710,N7711,N7712,N7713,N7714,N7715,N7716,N7717,N7718,N7719,N7720,  \\note fields as indicated
  N7721,N7722,N7723,N7724,N7725,N7726,N7727,N7728,N7729,N7730,N7731,N7732,N7733,N7734,N7735,N7736,N7737,N7738,N7739,N7740,  \\note fields as indicated
  N7741,N7742,N7743,N7744,N7745,N7746,N7747,N7748,N7749,N7750,N7751,N7752,N7753,N7754,N7755,N7756,N7757,N7758,N7759,N7760,  \\note fields as indicated
  N7761,N7762,N7763,N7764,N7765,N7766,N7767,N7768,N7769,N7770,N7771,N7772,N7773,N7774,N7775,N7776,N7777,N7778,N7779,N7780,  \\note fields as indicated
  N7781,N7782,N7783,N7784,N7785,N7786,N7787,N7788,N7789,N7790,N7791,N7792,N7793,N7794,N7795,N7796,N7797,N7798,N7799,N7800,  \\note fields as indicated
  N7801,N7802,N7803,N7804,N7805,N7806,N7807,N7808,N7809,N7810,N7811,N7812,N7813,N7814,N7815,N7816,N7817,N7818,N7819,N7820,  \\note fields as indicated
  N7821,N7822,N7823,N7824,N7825,N7826,N7827,N7828,N7829,N7830,N7831,N7832,N7833,N7834,N7835,N7836,N7837,N7838,N7839,N7840,  \\note fields as indicated
  N7841,N7842,N7843,N7844,N7845,N7846,N7847,N7848,N7849,N7850,N7851,N7852,N7853,N7854,N7855,N7856,N7857,N7858,N7859,N7860,  \\note fields as indicated
  N7861,N7862,N7863,N7864,N7865,N7866,N7867,N7868,N7869,N7870,N7871,N7872,N7873,N7874,N7875,N7876,N7877,N7878,N7879,N7880,  \\note fields as indicated
  N7881,N7882,N7883,N7884,N7885,N7886,N7887,N7888,N7889,N7890,N7891,N7892,N7893,N7894,N7895,N7896,N7897,N7898,N7899,N7900,  \\note fields as indicated
  N7901,N7902,N7903,N7904,N7905,N7906,N7907,N7908,N7909,N7910,N7911,N7912,N7913,N7914,N7915,N7916,N7917,N7918,N7919,N7920,  \\note fields as indicated
  N7921,N7922,N7923,N7924,N7925,N7926,N7927,N7928,N7929,N7930,N7931,N7932,N7933,N7934,N7935,N7936,N7937,N7938,N7939,N7940,  \\note fields as indicated
  N7941,N7942,N7943,N7944,N7945,N7946,N7947,N7948,N7949,N7950,N7951,N7952,N7953,N7954,N7955,N7956,N7957,N7958,N7959,N7960,  \\note fields as indicated
  N7961,N7962,N7963,N7964,N7965,N7966,N7967,N7968,N7969,N7970,N7971,N7972,N7973,N7974,N7975,N7976,N7977,N7978,N7979,N7980,  \\note fields as indicated
  N7981,N7982,N7983,N7984,N7985,N7986,N7987,N7988,N7989,N7990,N7991,N7992,N7993,N7994,N7995,N7996,N7997,N7998,N7999,N8000,  \\note fields as indicated
  N8001,N8002,N8003,N8004,N8005,N8006,N8007,N8008,N8009,N8010,N8011,N8012,N8013,N8014,N8015,N8016,N8017,N8018,N8019,N8020,  \\note fields as indicated
  N8021,N8022,N8023,N8024,N8025,N8026,N8027,N8028,N8029,N8030,N8031,N8032,N8033,N8034,N8035,N8036,N8037,N8038,N8039,N8040,  \\note fields as indicated
  N8041,N8042,N8043,N8044,N8045,N8046,N8047,N8048,N8049,N8050,N8051,N8052,N8053,N8054,N8055,N8056,N8057,N8058,N8059,N8060,  \\note fields as indicated
  N8061,N8062,N8063,N8064,N8065,N8066,N8067,N8068,N8069,N8070,N8071,N8072,N8073,N8074,N8075,N8076,N8077,N8078,N8079,N8080,  \\note fields as indicated
  N8081,N8082,N8083,N8084,N8085,N8086,N8087,N8088,N8089,N8090,N8091,N8092,N8093,N8094,N8095,N8096,N8097,N8098,N8099,N8100,  \\note fields as indicated
  N8101,N8102,N8103,N8104,N8105,N8106,N8107,N8108,N8109,N8110,N8111,N8112,N8113,N8114,N8115,N8116,N8117,N8118,N8119,N8120,  \\note fields as indicated
  N8121,N8122,N8123,N8124,N8125,N8126,N8127,N8128,N8129,N8130,N8131,N8132,N8133,N8134,N8135,N8136,N8137,N8138,N8139,N8140,  \\note fields as indicated
  N8141,N8142,N8143,N8144,N8145,N8146,N8147,N8148,N8149,N8150,N8151,N8152,N8153,N8154,N8155,N8156,N8157,N8158,N8159,N8160,  \\note fields as indicated
  N8161,N8162,N8163,N8164,N8165,N8166,N8167,N8168,N8169,N8170,N8171,N8172,N8173,N8174,N8175,N8176,N8177,N8178,N8179,N8180,  \\note fields as indicated
  N8181,N8182,N8183,N8184,N8185,N8186,N8187,N8188,N8189,N8190,N8191,N8192,N8193,N8194,N8195,N8196,N8197,N8198,N8199,N8200,  \\note fields as indicated
  N8201,N8202,N8203,N8204,N8205,N8206,N8207,N8208,N8209,N8210,N8211,N8212,N8213,N8214,N8215,N8216,N8217,N8218,N8219,N8220,  \\note fields as indicated
  N8221,N8222,N8223,N8224,N8225,N8226,N8227,N8228,N8229,N8230,N8231,N8232,N8233,N8234,N8235,N8236,N8237,N8238,N8239,N8240,  \\note fields as indicated
  N8241,N8242,N8243,N8244,N8245,N8246,N8247,N8248,N8249,N8250,N8251,N8252,N8253,N8254,N8255,N8256,N8257,N8258,N8259,N8260,  \\note fields as indicated
  N8261,N8262,N8263,N8264,N8265,N8266,N8267,N8268,N8269,N8270,N8271,N8272,N8273,N8274,N8275,N8276,N8277,N8278,N8279,N8280,  \\note fields as indicated
  N8281,N8282,N8283,N8284,N8285,N8286,N8287,N8288,N8289,N8290,N8291,N8292,N8293,N8294,N8295,N8296,N8297,N8298,N8299,N8300,  \\note fields as indicated
  N8301,N8302,N8303,N8304,N8305,N8306,N8307,N8308,N8309,N8310,N8311,N8312,N8313,N8314,N8315,N8316,N8317,N8318,N8319,N8320,  \\note fields as indicated
  N8321,N8322,N8323,N8324,N8325,N8326,N8327,N8328,N8329,N8330,N8331,N8332,N8333,N8334,N8335,N8336,N8337,N8338,N8339,N8340,  \\note fields as indicated
  N8341,N8342,N8343,N8344,N8345,N8346,N8347,N8348,N8349,N8350,N8351,N8352,N8353,N8354,N8355,N8356,N8357,N8358,N8359,N8360,  \\note fields as indicated
  N8361,N8362,N8363,N8364,N8365,N8366,N8367,N8368,N8369,N8370,N8371,N8372,N8373,N8374,N8375,N8376,N8377,N8378,N8379,N8380,  \\note fields as indicated
  N8381,N8382,N8383,N8384,N8385,N8386,N8387,N8388,N8389,N8390,N8391,N8392,N8393,N8394,N8395,N8396,N8397,N8398,N8399,N8400,  \\note fields as indicated
  N8401,N8402,N8403,N8404,N8405,N8406,N8407,N8408,N8409,N8410,N8411,N8412,N8413,N8414,N8415,N8416,N8417,N8418,N8419,N8420,  \\note fields as indicated
  N8421,N8422,N8423,N8424,N8425,N8426,N8427,N8428,N8429,N8430,N8431,N8432,N8433,N8434,N8435,N8436,N8437,N8438,N8439,N8440,  \\note fields as indicated
  N8441,N8442,N8443,N8444,N8445,N8446,N8447,N8448,N8449,N8450,N8451,N8452,N8453,N8454,N8455,N8456,N8457,N8458,N8459,N8460,  \\note fields as indicated
  N8461,N8462,N8463,N8464,N8465,N8466,N8467,N8468,N8469,N8470,N8471,N8472,N8473,N8474,N8475,N8476,N8477,N8478,N8479,N8480,  \\note fields as indicated
  N8481,N8482,N8483,N8484,N8485,N8486,N8487,N8488,N8489,N8490,N8491,N8492,N8493,N8494,N8495,N8496,N8497,N8498,N8499,N8500,  \\note fields as indicated
  N8501,N8502,N8503,N8504,N8505,N8506,N8507,N8508,N8509,N8510,N8511,N8512,N8513,N8514,N8515,N8516,N8517,N8518,N8519,N8520,  \\note fields as indicated
  N8521,N8522,N8523,N8524,N8525,N8526,N8527,N8528,N8529,N8530,N8531,N8532,N8533,N8534,N8535,N8536,N8537,N8538,N8539,N8540,  \\note fields as indicated
  N8541,N8542,N8543,N8544,N8545,N8546,N8547,N8548,N8549,N8550,N8551,N8552,N8553,N8554,N8555,N8556,N8557,N8558,N8559,N8560,  \\note fields as indicated
  N8561,N8562,N8563,N8564,N8565,N8566,N8567,N8568,N8569,N8570,N8571,N8572,N8573,N8574,N8575,N8576,N8577,N8578,N8579,N8580,  \\note fields as indicated
  N8581,N8582,N8583,N8584,N8585,N8586,N8587,N8588,N8589,N8590,N8591,N8592,N8593,N8594,N8595,N8596,N8597,N8598,N8599,N8600,  \\note fields as indicated
  N8601,N8602,N8603,N8604,N8605,N8606,N8607,N8608,N8609,N8610,N8611,N8612,N8613,N8614,N8615,N8616,N8617,N8618,N8619,N8620,  \\note fields as indicated
  N8621,N8622,N8623,N8624,N8625,N8626,N8627,N8628,N8629,N8630,N8631,N8632,N8633,N8634,N8635,N8636,N8637,N8638,N8639,N8640,  \\note fields as indicated
  N8641,N8642,N8643,N8644,N8645,N8646,N8647,N8648,N8649,N8650,N8651,N8652,N8653,N8654,N8655,N8656,N8657,N8658,N8659,N8660,  \\note fields as indicated
  N8661,N8662,N8663,N8664,N8665,N8666,N8667,N8668,N8669,N8670,N8671,N8672,N8673,N8674,N8675,N8676,N8677,N8678,N8679,N8680,  \\note fields as indicated
  N8681,N8682,N8683,N8684,N8685,N8686,N8687,N8688,N8689,N8690,N8691,N8692,N8693,N8694,N8695,N8696,N8697,N8698,N8699,N8700,  \\note fields as indicated
  N8701,N8702,N8703,N8704,N8705,N8706,N8707,N8708,N8709,N8710,N8711,N8712,N8713,N8714,N8715,N8716,N8717,N8718,N8719,N8720,  \\note fields as indicated
  N8721,N8722,N8723,N8724,N8725,N8726,N8727,N8728,N8729,N8730,N8731,N8732,N8733,N8734,N8735,N8736,N8737,N8738,N8739,N8740,  \\note fields as indicated
  N8741,N8742,N8743,N8744,N8745,N8746,N8747,N8748,N8749,N8750,N8751,N8752,N8753,N8754,N8755,N8756,N8757,N8758,N8759,N8760,  \\note fields as indicated
  N8761,N8762,N8763,N8764,N8765,N8766,N8767,N8768,N8769,N8770,N8771,N8772,N8773,N8774,N8775,N8776,N8777,N8778,N8779,N8780,  \\note fields as indicated
  N8781,N8782,N8783,N8784,N8785,N8786,N8787,N8788,N8789,N8790,N8791,N8792,N8793,N8794,N8795,N8796,N8797,N8798,N8799,N8800,  \\note fields as indicated
  N8801,N8802,N8803,N8804,N8805,N8806,N8807,N8808,N8809,N8810,N8811,N8812,N8813,N8814,N8815,N8816,N8817,N8818,N8819,N8820,  \\note fields as indicated
  N8821,N8822,N8823,N8824,N8825,N8826,N8827,N8828,N8829,N8830,N8831,N8832,N8833,N8834,N8835,N8836,N8837,N8838,N8839,N8840,  \\note fields as indicated
  N8841,N8842,N8843,N8844,N8845,N8846,N8847,N8848,N8849,N8850,N8851,N8852,N8853,N8854,N8855,N8856,N8857,N8858,N8859,N8860,  \\note fields as indicated
  N8861,N8862,N8863,N8864,N8865,N8866,N8867,N8868,N8869,N8870,N8871,N8872,N8873,N8874,N8875,N8876,N8877,N8878,N8879,N8880,  \\note fields as indicated
  N8881,N8882,N8883,N8884,N8885,N8886,N8887,N8888,N8889,N8890,N8891,N8892,N8893,N8894,N8895,N8896,N8897,N8898,N8899,N8900,  \\note fields as indicated
  N8901,N8902,N8903,N8904,N8905,N8906,N8907,N8908,N8909,N8910,N8911,N8912,N8913,N8914,N8915,N8916,N8917,N8918,N8919,N8920,  \\note fields as indicated
  N8921,N8922,N8923,N8924,N8925,N8926,N8927,N8928,N8929,N8930,N8931,N8932,N8933,N8934,N8935,N8936,N8937,N8938,N8939,N8940,  \\note fields as indicated
  N8941,N8942,N8943,N8944,N8945,N8946,N8947,N8948,N8949,N8950,N8951,N8952,N8953,N8954,N8955,N8956,N8957,N8958,N8959,N8960,  \\note fields as indicated
  N8961,N8962,N8963,N8964,N8965,N8966,N8967,N8968,N8969,N8970,N8971,N8972,N8973,N8974,N8975,N8976,N8977,N8978,N8979,N8980,  \\note fields as indicated
  N8981,N8982,N8983,N8984,N8985,N8986,N8987,N8988,N8989,N8990,N8991,N8992,N8993,N8994,N8995,N8996,N8997,N8998,N8999,N9000,  \\note fields as indicated
  N9001,N9002,N9003,N9004,N9005,N9006,N9007,N9008,N9009,N9010,N9011,N9012,N9013,N9014,N9015,N9016,N9017,N9018,N9019,N9020,  \\note fields as indicated
  N9021,N9022,N9023,N9024,N9025,N9026,N9027,N9028,N9029,N9030,N9031,N9032,N9033,N9034,N9035,N9036,N9037,N9038,N9039,N9040,  \\note fields as indicated
  N9041,N9042,N9043,N9044,N9045,N9046,N9047,N9048,N9049,N9050,N9051,N9052,N9053,N9054,N9055,N9056,N9057,N9058,N9059,N9060,  \\note fields as indicated
  N9061,N9062,N9063,N9064,N9065,N9066,N9067,N9068,N9069,N9070,N9071,N9072,N9073,N9074,N9075,N9076,N9077,N9078,N9079,N9080,  \\note fields as indicated
  N9081,N9082,N9083,N9084,N9085,N9086,N9087,N9088,N9089,N9090,N9091,N9092,N9093,N9094,N9095,N9096,N9097,N9098,N9099,N9100,  \\note fields as indicated
  N9101,N9102,N9103,N9104,N9105,N9106,N9107,N9108,N9109,N9110,N9111,N9112,N9113,N9114,N9115,N9116,N9117,N9118,N9119,N9120,  \\note fields as indicated
  N9121,N9122,N9123,N9124,N9125,N9126,N9127,N9128,N9129,N9130,N9131,N9132,N9133,N9134,N9135,N9136,N9137,N9138,N9139,N9140,  \\note fields as indicated
  N9141,N9142,N9143,N9144,N9145,N9146,N9147,N9148,N9149,N9150,N9151,N9152,N9153,N9154,N9155,N9156,N9157,N9158,N9159,N9160,  \\note fields as indicated
  N9161,N9162,N9163,N9164,N9165,N9166,N9167,N9168,N9169,N9170,N9171,N9172,N9173,N9174,N9175,N9176,N9177,N9178,N9179,N9180,  \\note fields as indicated
  N9181,N9182,N9183,N9184,N9185,N9186,N9187,N9188,N9189,N9190,N9191,N9192,N9193,N9194,N9195,N9196,N9197,N9198,N9199,N9200,  \\note fields as indicated
  N9201,N9202,N9203,N9204,N9205,N9206,N9207,N9208,N9209,N9210,N9211,N9212,N9213,N9214,N9215,N9216,N9217,N9218,N9219,N9220,  \\note fields as indicated
  N9221,N9222,N9223,N9224,N9225,N9226,N9227,N9228,N9229,N9230,N9231,N9232,N9233,N9234,N9235,N9236,N9237,N9238,N9239,N9240,  \\note fields as indicated
  N9241,N9242,N9243,N9244,N9245,N9246,N9247,N9248,N9249,N9250,N9251,N9252,N9253,N9254,N9255,N9256,N9257,N9258,N9259,N9260,  \\note fields as indicated
  N9261,N9262,N9263,N9264,N9265,N9266,N9267,N9268,N9269,N9270,N9271,N9272,N9273,N9274,N9275,N9276,N9277,N9278,N9279,N9280,  \\note fields as indicated
  N9281,N9282,N9283,N9284,N9285,N9286,N9287,N9288,N9289,N9290,N9291,N9292,N9293,N9294,N9295,N9296,N9297,N9298,N9299,N9300,  \\note fields as indicated
  N9301,N9302,N9303,N9304,N9305,N9306,N9307,N9308,N9309,N9310,N9311,N9312,N9313,N9314,N9315,N9316,N9317,N9318,N9319,N9320,  \\note fields as indicated
  N9321,N9322,N9323,N9324,N9325,N9326,N9327,N9328,N9329,N9330,N9331,N9332,N9333,N9334,N9335,N9336,N9337,N9338,N9339,N9340,  \\note fields as indicated
  N9341,N9342,N9343,N9344,N9345,N9346,N9347,N9348,N9349,N9350,N9351,N9352,N9353,N9354,N9355,N9356,N9357,N9358,N9359,N9360,  \\note fields as indicated
  N9361,N9362,N9363,N9364,N9365,N9366,N9367,N9368,N9369,N9370,N9371,N9372,N9373,N9374,N9375,N9376,N9377,N9378,N9379,N9380,  \\note fields as indicated
  N9381,N9382,N9383,N9384,N9385,N9386,N9387,N9388,N9389,N9390,N9391,N9392,N9393,N9394,N9395,N9396,N9397,N9398,N9399,N9400,  \\note fields as indicated
  N9401,N9402,N9403,N9404,N9405,N9406,N9407,N9408,N9409,N9410,N9411,N9412,N9413,N9414,N9415,N9416,N9417,N9418,N9419,N9420,  \\note fields as indicated
  N9421,N9422,N9423,N9424,N9425,N9426,N9427,N9428,N9429,N9430,N9431,N9432,N9433,N9434,N9435,N9436,N9437,N9438,N9439,N9440,  \\note fields as indicated
  N9441,N9442,N9443,N9444,N9445,N9446,N9447,N9448,N9449,N9450,N9451,N9452,N9453,N9454,N9455,N9456,N9457,N9458,N9459,N9460,  \\note fields as indicated
  N9461,N9462,N9463,N9464,N9465,N9466,N9467,N9468,N9469,N9470,N9471,N9472,N9473,N9474,N9475,N9476,N9477,N9478,N9479,N9480,  \\note fields as indicated
  N9481,N9482,N9483,N9484,N9485,N9486,N9487,N9488,N9489,N9490,N9491,N9492,N9493,N9494,N9495,N9496,N9497,N9498,N9499,N9500,  \\note fields as indicated
  N9501,N9502,N9503,N9504,N9505,N9506,N9507,N9508,N9509,N9510,N9511,N9512,N9513,N9514,N9515,N9516,N9517,N9518,N9519,N9520,  \\note fields as indicated
  N9521,N9522,N9523,N9524,N9525,N9526,N9527,N9528,N9529,N9530,N9531,N9532,N9533,N9534,N9535,N9536,N9537,N9538,N9539,N9540,  \\note fields as indicated
  N9541,N9542,N9543,N9544,N9545,N9546,N9547,N9548,N9549,N9550,N9551,N9552,N9553,N9554,N9555,N9556,N9557,N9558,N9559,N9560,  \\note fields as indicated
  N9561,N9562,N9563,N9564,N9565,N9566,N9567,N9568,N9569,N9570,N9571,N9572,N9573,N9574,N9575,N9576,N9577,N9578,N9579,N9580,  \\note fields as indicated
  N9581,N9582,N9583,N9584,N9585,N9586,N9587,N9588,N9589,N9590,N9591,N9592,N9593,N9594,N9595,N9596,N9597,N9598,N9599,N9600,  \\note fields as indicated
  N9601,N9602,N9603,N9604,N9605,N9606,N9607,N9608,N9609,N9610,N9611,N9612,N9613,N9614,N9615,N9616,N9617,N9618,N9619,N9620,  \\note fields as indicated
  N9621,N9622,N9623,N9624,N9625,N9626,N9627,N9628,N9629,N9630,N9631,N9632,N9633,N9634,N9635,N9636,N9637,N9638,N9639,N9640,  \\note fields as indicated
  N9641,N9642,N9643,N9644,N9645,N9646,N9647,N9648,N9649,N9650,N9651,N9652,N9653,N9654,N9655,N9656,N9657,N9658,N9659,N9660,  \\note fields as indicated
  N9661,N9662,N9663,N9664,N9665,N9666,N9667,N9668,N9669,N9670,N9671,N9672,N9673,N9674,N9675,N9676,N9677,N9678,N9679,N9680,  \\note fields as indicated
  N9681,N9682,N9683,N9684,N9685,N9686,N9687,N9688,N9689,N9690,N9691,N9692,N9693,N9694,N9695,N9696,N9697,N9698,N9699,N9700,  \\note fields as indicated
  N9701,N9702,N9703,N9704,N9705,N9706,N9707,N9708,N9709,N9710,N9711,N9712,N9713,N9714,N9715,N9716,N9717,N9718,N9719,N9720,  \\note fields as indicated
  N9721,N9722,N9723,N9724,N9725,N9726,N9727,N9728,N9729,N9730,N9731,N9732,N9733,N9734,N9735,N9736,N9737,N9738,N9739,N9740,  \\note fields as indicated
  N9741,N9742,N9743,N9744,N9745,N9746,N9747,N9748,N9749,N9750,N9751,N9752,N9753,N9754,N9755,N9756,N9757,N9758,N9759,N9760,  \\note fields as indicated
  N9761,N9762,N9763,N9764,N9765,N9766,N9767,N9768,N9769,N9770,N9771,N9772,N9773,N9774,N9775,N9776,N9777,N9778,N9779,N9780,  \\note fields as indicated
  N9781,N9782,N9783,N9784,N9785,N9786,N9787,N9788,N9789,N9790,N9791,N9792,N9793,N9794,N9795,N9796,N9797,N9798,N9799,N9800,  \\note fields as indicated
  N9801,N9802,N9803,N9804,N9805,N9806,N9807,N9808,N9809,N9810,N9811,N9812,N9813,N9814,N9815,N9816,N9817,N9818,N9819,N9820,  \\note fields as indicated
  N9821,N9822,N9823,N9824,N9825,N9826,N9827,N9828,N9829,N9830,N9831,N9832,N9833,N9834,N9835,N9836,N9837,N9838,N9839,N9840,  \\note fields as indicated
  N9841,N9842,N9843,N9844,N9845,N9846,N9847,N9848,N9849,N9850,N9851,N9852,N9853,N9854,N9855,N9856,N9857,N9858,N9859,N9860,  \\note fields as indicated
  N9861,N9862,N9863,N9864,N9865,N9866,N9867,N9868,N9869,N9870,N9871,N9872,N9873,N9874,N9875,N9876,N9877,N9878,N9879,N9880,  \\note fields as indicated
  N9881,N9882,N9883,N9884,N9885,N9886,N9887,N9888,N9889,N9890,N9891,N9892,N9893,N9894,N9895,N9896,N9897,N9898,N9899,N9900,  \\note fields as indicated
  N9901,N9902,N9903,N9904,N9905,N9906,N9907,N9908,N9909,N9910,N9911,N9912,N9913,N9914,N9915,N9916,N9917,N9918,N9919,N9920,  \\note fields as indicated
  N9921,N9922,N9923,N9924,N9925,N9926,N9927,N9928,N9929,N9930,N9931,N9932,N9933,N9934,N9935,N9936,N9937,N9938,N9939,N9940,  \\note fields as indicated
  N9941,N9942,N9943,N9944,N9945,N9946,N9947,N9948,N9949,N9950,N9951,N9952,N9953,N9954,N9955,N9956,N9957,N9958,N9959,N9960,  \\note fields as indicated
  N9961,N9962,N9963,N9964,N9965,N9966,N9967,N9968,N9969,N9970,N9971,N9972,N9973,N9974,N9975,N9976,N9977,N9978,N9979,N9980,  \\note fields as indicated
  N9981,N9982,N9983,N9984,N9985,N9986,N9987,N9988,N9989,N9990,N9991,N9992,N9993,N9994,N9995,N9996,N9997,N9998,N9999,N10000,  \\note fields as indicated
  N10001,N10002,N10003,N10004,N10005,N10006,N10007,N10008,N10009,N10010,N10011,N10012,N10013,N10014,N10015,N10016,N10017,N10018,N10019,N10020,  \\note fields as indicated
  N10021,N10022,N10023,N10024,N10025,N10026,N10027,N10028,N10029,N10030,N10031,N10032,N10033,N10034,N10035,N10036,N10037,N10038,N10039,N10040,  \\note fields as indicated
  N10041,N10042,N10043,N10044,N10045,N10046,N10047,N10048,N10049,N10050,N10051,N10052,N10053,N10054,N10055,N10056,N10057,N10058,N10059,N10060,  \\note fields as indicated
  N10061,N10062,N10063,N10064,N10065,N10066,N10067,N10068,N10069,N10070,N10071,N10072,N10073,N10074,N10075,N10076,N10077,N10078,N10079,N10080,  \\note fields as indicated
  N10081,N10082,N10083,N10084,N10085,N10086,N10087,N10088,N10089,N10090,N10091,N10092,N10093,N10094,N10095,N10096,N10097,N10098,N10099,N10100,  \\note fields as indicated
  N10101,N10102,N10103,N10104,N10105,N10106,N10107,N10108,N10109,N10110,N10111,N10112,N10113,N10114,N10115,N10116,N10117,N10118,N10119,N10120,  \\note fields as indicated
  N10121,N10122,N10123,N10124,N10125,N10126,N10127,N10128,N10129,N10130,N10131,N10132,N10133,N10134,N10135,N10136,N10137,N10138,N10139,N10140,  \\note fields as indicated
  N10141,N10142,N10143,N10144,N10145,N10146,N10147,N10148,N10149,N10150,N10151,N10152,N10153,N10154,N10155,N10156,N10157,N10158,N10159,N10160,  \\note fields as indicated
  N10161,N10162,N10163,N10164,N10165,N10166,N10167,N10168,N10169,N10170,N10171,N10172,N10173,N10174,N10175,N10176,N10177,N10178,N10179,N10180,  \\note fields as indicated
  N10181,N10182,N10183,N10184,N10185,N10186,N10187,N10188,N10189,N10190,N10191,N10192,N10193,N10194,N10195,N10196,N10197,N10198,N10199,N10200,  \\note fields as indicated
  N10201,N10202,N10203,N10204,N10205,N10206,N10207,N10208,N10209,N10210,N10211,N10212,N10213,N10214,N10215,N10216,N10217,N10218,N10219,N10220,  \\note fields as indicated
  N10221,N10222,N10223,N10224,N10225,N10226,N10227,N10228,N10229,N10230,N10231,N10232,N10233,N10234,N10235,N10236,N10237,N10238,N10239,N10240,  \\note fields as indicated
  N10241,N10242,N10243,N10244,N10245,N10246,N10247,N10248,N10249,N10250,N10251,N10252,N10253,N10254,N10255,N10256,N10257,N10258,N10259,N10260,  \\note fields as indicated
  N10261,N10262,N10263,N10264,N10265,N10266,N10267,N10268,N10269,N10270,N10271,N10272,N10273,N10274,N10275,N10276,N10277,N10278,N10279,N10280,  \\note fields as indicated
  N10281,N10282,N10283,N10284,N10285,N10286,N10287,N10288,N10289,N10290,N10291,N10292,N10293,N10294,N10295,N10296,N10297,N10298,N10299,N10300,  \\note fields as indicated
  N10301,N10302,N10303,N10304,N10305,N10306,N10307,N10308,N10309,N10310,N10311,N10312,N10313,N10314,N10315,N10316,N10317,N10318,N10319,N10320,  \\note fields as indicated
  N10321,N10322,N10323,N10324,N10325,N10326,N10327,N10328,N10329,N10330,N10331,N10332,N10333,N10334,N10335,N10336,N10337,N10338,N10339,N10340,  \\note fields as indicated
  N10341,N10342,N10343,N10344,N10345,N10346,N10347,N10348,N10349,N10350,N10351,N10352,N10353,N10354,N10355,N10356,N10357,N10358,N10359,N10360,  \\note fields as indicated
  N10361,N10362,N10363,N10364,N10365,N10366,N10367,N10368,N10369,N10370,N10371,N10372,N10373,N10374,N10375,N10376,N10377,N10378,N10379,N10380,  \\note fields as indicated
  N10381,N10382,N10383,N10384,N10385,N10386,N10387,N10388,N10389,N10390,N10391,N10392,N10393,N10394,N10395,N10396,N10397,N10398,N10399,N10400,  \\note fields as indicated
  N10401,N10402,N10403,N10404,N10405,N10406,N10407,N10408,N10409,N10410,N10411,N10412,N10413,N10414,N10415,N10416,N10417,N10418,N10419,N10420,  \\note fields as indicated
  N10421,N10422,N10423,N10424,N10425,N10426,N10427,N10428,N10429,N10430,N10431,N10432,N10433,N10434,N10435,N10436,N10437,N10438,N10439,N10440,  \\note fields as indicated
  N10441,N10442,N10443,N10444,N10445,N10446,N10447,N10448,N10449,N10450,N10451,N10452,N10453,N10454,N10455,N10456,N10457,N10458,N10459,N10460,  \\note fields as indicated
  N10461,N10462,N10463,N10464,N10465,N10466,N10467,N10468,N10469,N10470,N10471,N10472,N10473,N10474,N10475,N10476,N10477,N10478,N10479,N10480,  \\note fields as indicated
  N10481,N10482,N10483,N10484,N10485,N10486,N10487,N10488,N10489,N10490,N10491,N10492,N10493,N10494,N10495,N10496,N10497,N10498,N10499,N10500,  \\note fields as indicated
  N10501,N10502,N10503,N10504,N10505,N10506,N10507,N10508,N10509,N10510,N10511,N10512,N10513,N10514,N10515,N10516,N10517,N10518,N10519,N10520,  \\note fields as indicated
  N10521,N10522,N10523,N10524,N10525,N10526,N10527,N10528,N10529,N10530,N10531,N10532,N10533,N10534,N10535,N10536,N10537,N10538,N10539,N10540,  \\note fields as indicated
  N10541,N10542,N10543,N10544,N10545,N10546,N10547,N10548,N10549,N10550,N10551,N10552,N10553,N10554,N10555,N10556,N10557,N10558,N10559,N10560,  \\note fields as indicated
  N10561,N10562,N10563,N10564,N10565,N10566,N10567,N10568,N10569,N10570,N10571,N10572,N10573,N10574,N10575,N10576,N10577,N10578,N10579,N10580,  \\note fields as indicated
  N10581,N10582,N10583,N10584,N10585,N10586,N10587,N10588,N10589,N10590,N10591,N10592,N10593,N10594,N10595,N10596,N10597,N10598,N10599,N10600,  \\note fields as indicated
  N10601,N10602,N10603,N10604,N10605,N10606,N10607,N10608,N10609,N10610,N10611,N10612,N10613,N10614,N10615,N10616,N10617,N10618,N10619,N10620,  \\note fields as indicated
  N10621,N10622,N10623,N10624,N10625,N10626,N10627,N10628,N10629,N10630,N10631,N10632,N10633,N10634,N10635,N10636,N10637,N10638,N10639,N10640,  \\note fields as indicated
  N10641,N10642,N10643,N10644,N10645,N10646,N10647,N10648,N10649,N10650,N10651,N10652,N10653,N10654,N10655,N10656,N10657,N10658,N10659,N10660,  \\note fields as indicated
  N10661,N10662,N10663,N10664,N10665,N10666,N10667,N10668,N10669,N10670,N10671,N10672,N10673,N10674,N10675,N10676,N10677,N10678,N10679,N10680,  \\note fields as indicated
  N10681,N10682,N10683,N10684,N10685,N10686,N10687,N10688,N10689,N10690,N10691,N10692,N10693,N10694,N10695,N10696,N10697,N10698,N10699,N10700,  \\note fields as indicated
  N10701,N10702,N10703,N10704,N10705,N10706,N10707,N10708,N10709,N10710,N10711,N10712,N10713,N10714,N10715,N10716,N10717,N10718,N10719,N10720,  \\note fields as indicated
  N10721,N10722,N10723,N10724,N10725,N10726,N10727,N10728,N10729,N10730,N10731,N10732,N10733,N10734,N10735,N10736,N10737,N10738,N10739,N10740,  \\note fields as indicated
  N10741,N10742,N10743,N10744,N10745,N10746,N10747,N10748,N10749,N10750,N10751,N10752,N10753,N10754,N10755,N10756,N10757,N10758,N10759,N10760,  \\note fields as indicated
  N10761,N10762,N10763,N10764,N10765,N10766,N10767,N10768,N10769,N10770,N10771,N10772,N10773,N10774,N10775,N10776,N10777,N10778,N10779,N10780,  \\note fields as indicated
  N10781,N10782,N10783,N10784,N10785,N10786,N10787,N10788,N10789,N10790,N10791,N10792,N10793,N10794,N10795,N10796,N10797,N10798,N10799,N10800,  \\note fields as indicated
  N10801,N10802,N10803,N10804,N10805,N10806,N10807,N10808,N10809,N10810,N10811,N10812,N10813,N10814,N10815,N10816,N10817,N10818,N10819,N10820,  \\note fields as indicated
  N10821,N10822,N10823,N10824,N10825,N10826,N10827,N10828,N10829,N10830,N10831,N10832,N10833,N10834,N10835,N10836,N10837,N10838,N10839,N10840,  \\note fields as indicated
  N10841,N10842,N10843,N10844,N10845,N10846,N10847,N10848,N10849,N10850,N10851,N10852,N10853,N10854,N10855,N10856,N10857,N10858,N10859,N10860,  \\note fields as indicated
  N10861,N10862,N10863,N10864,N10865,N10866,N10867,N10868,N10869,N10870,N10871,N10872,N10873,N10874,N10875,N10876,N10877,N10878,N10879,N10880,  \\note fields as indicated
  N10881,N10882,N10883,N10884,N10885,N10886,N10887,N10888,N10889,N10890,N10891,N10892,N10893,N10894,N10895,N10896,N10897,N10898,N10899,N10900,  \\note fields as indicated
  N10901,N10902,N10903,N10904,N10905,N10906,N10907,N10908,N10909,N10910,N10911,N10912,N10913,N10914,N10915,N10916,N10917,N10918,N10919,N10920,  \\note fields as indicated
  N10921,N10922,N10923,N10924,N10925,N10926,N10927,N10928,N10929,N10930,N10931,N10932,N10933,N10934,N10935,N10936,N10937,N10938,N10939,N10940,  \\note fields as indicated
  N10941,N10942,N10943,N10944,N10945,N10946,N10947,N10948,N10949,N10950,N10951,N10952,N10953,N10954,N10955,N10956,N10957,N10958,N10959,N10960,  \\note fields as indicated
  N10961,N10962,N10963,N10964,N10965,N10966,N10967,N10968,N10969,N10970,N10971,N10972,N10973,N10974,N10975,N10976,N10977,N10978,N10979,N10980,  \\note fields as indicated
  N10981,N10982,N10983,N10984,N10985,N10986,N10987,N10988,N10989,N10990,N10991,N10992,N10993,N10994,N10995,N10996,N10997,N10998,N10999,N11000,  \\note fields as indicated
  N11001,N11002,N11003,N11004,N11005,N11006,N11007,N11008,N11009,N11010,N11011,N11012,N11013,N11014,N11015,N11016,N11017,N11018,N11019,N11020,  \\note fields as indicated
  N11021,N11022,N11023,N11024,N11025,N11026,N11027,N11028,N11029,N11030,N11031,N11032,N11033,N11034,N11035,N11036,N11037,N11038,N11039,N11040,  \\note fields as indicated
  N11041,N11042,N11043,N11044,N11045,N11046,N11047,N11048,N11049,N11050,N11051,N11052,N11053,N11054,N11055,N11056,N11057,N11058,N11059,N11060,  \\note fields as indicated
  N11061,N11062,N11063,N11064,N11065,N11066,N11067,N11068,N11069,N11070,N11071,N11072,N11073,N11074,N11075,N11076,N11077,N11078,N11079,N11080,  \\note fields as indicated
  N11081,N11082,N11083,N11084,N11085,N11086,N11087,N11088,N11089,N11090,N11091,N11092,N11093,N11094,N11095,N11096,N11097,N11098,N11099,N11100,  \\note fields as indicated
  N11101,N11102,N11103,N11104,N11105,N11106,N11107,N11108,N11109,N11110,N11111,N11112,N11113,N11114,N11115,N11116,N11117,N11118,N11119,N11120,  \\note fields as indicated
  N11121,N11122,N11123,N11124,N11125,N11126,N11127,N11128,N11129,N11130,N11131,N11132,N11133,N11134,N11135,N11136,N11137,N11138,N11139,N11140,  \\note fields as indicated
  N11141,N11142,N11143,N11144,N11145,N11146,N11147,N11148,N11149,N11150,N11151,N11152,N11153,N11154,N11155,N11156,N11157,N11158,N11159,N11160,  \\note fields as indicated
  N11161,N11162,N11163,N11164,N11165,N11166,N11167,N11168,N11169,N11170,N11171,N11172,N11173,N11174,N11175,N11176,N11177,N11178,N11179,N11180,  \\note fields as indicated
  N11181,N11182,N11183,N11184,N11185,N11186,N11187,N11188,N11189,N11190,N11191,N11192,N11193,N11194,N11195,N11196,N11197,N11198,N11199,N11200,  \\note fields as indicated
  N11201,N11202,N11203,N11204,N11205,N11206,N11207,N11208,N11209,N11210,N11211,N11212,N11213,N11214,N11215,N11216,N11217,N11218,N11219,N11220,  \\note fields as indicated
  N11221,N11222,N11223,N11224,N11225,N11226,N11227,N11228,N11229,N11230,N11231,N11232,N11233,N11234,N11235,N11236,N11237,N11238,N11239,N11240,  \\note fields as indicated
  N11241,N11242,N11243,N11244,N11245,N11246,N11247,N11248,N11249,N11250,N11251,N11252,N11253,N11254,N11255,N11256,N11257,N11258,N11259,N11260,  \\note fields as indicated
  N11261,N11262,N11263,N11264,N11265,N11266,N11267,N11268,N11269,N11270,N11271,N11272,N11273,N11274,N11275,N11276,N11277,N11278,N11279,N11280,  \\note fields as indicated
  N11281,N11282,N11283,N11284,N11285,N11286,N11287,N11288,N11289,N11290,N11291,N11292,N11293,N11294,N11295,N11296,N11297,N11298,N11299,N11300,  \\note fields as indicated
  N11301,N11302,N11303,N11304,N11305,N11306,N11307,N11308,N11309,N11310,N11311,N11312,N11313,N11314,N11315,N11316,N11317,N11318,N11319,N11320,  \\note fields as indicated
  N11321,N11322,N11323,N11324,N11325,N11326,N11327,N11328,N11329,N11330,N11331,N11332,N11333,N11334,N11335,N11336,N11337,N11338,N11339,N11340,  \\note fields as indicated
  N11341,N11342,N11343,N11344,N11345,N11346,N11347,N11348,N11349,N11350,N11351,N11352,N11353,N11354,N11355,N11356,N11357,N11358,N11359,N11360,  \\note fields as indicated
  N11361,N11362,N11363,N11364,N11365,N11366,N11367,N11368,N11369,N11370,N11371,N11372,N11373,N11374,N11375,N11376,N11377,N11378,N11379,N11380,  \\note fields as indicated
  N11381,N11382,N11383,N11384,N11385,N11386,N11387,N11388,N11389,N11390,N11391,N11392,N11393,N11394,N11395,N11396,N11397,N11398,N11399,N11400,  \\note fields as indicated
  N11401,N11402,N11403,N11404,N11405,N11406,N11407,N11408,N11409,N11410,N11411,N11412,N11413,N11414,N11415,N11416,N11417,N11418,N11419,N11420,  \\note fields as indicated
  N11421,N11422,N11423,N11424,N11425,N11426,N11427,N11428,N11429,N11430,N11431,N11432,N11433,N11434,N11435,N11436,N11437,N11438,N11439,N11440,  \\note fields as indicated
  N11441,N11442,N11443,N11444,N11445,N11446,N11447,N11448,N11449,N11450,N11451,N11452,N11453,N11454,N11455,N11456,N11457,N11458,N11459,N11460,  \\note fields as indicated
  N11461,N11462,N11463,N11464,N11465,N11466,N11467,N11468,N11469,N11470,N11471,N11472,N11473,N11474,N11475,N11476,N11477,N11478,N11479,N11480,  \\note fields as indicated
  N11481,N11482,N11483,N11484,N11485,N11486,N11487,N11488,N11489,N11490,N11491,N11492,N11493,N11494,N11495,N11496,N11497,N11498,N11499,N11500,  \\note fields as indicated
  N11501,N11502,N11503,N11504,N11505,N11506,N11507,N11508,N11509,N11510,N11511,N11512,N11513,N11514,N11515,N11516,N11517,N11518,N11519,N11520,  \\note fields as indicated
  N11521,N11522,N11523,N11524,N11525,N11526,N11527,N11528,N11529,N11530,N11531,N11532,N11533,N11534,N11535,N11536,N11537,N11538,N11539,N11540,  \\note fields as indicated
  N11541,N11542,N11543,N11544,N11545,N11546,N11547,N11548,N11549,N11550,N11551,N11552,N11553,N11554,N11555,N11556,N11557,N11558,N11559,N11560,  \\note fields as indicated
  N11561,N11562,N11563,N11564,N11565,N11566,N11567,N11568,N11569,N11570,N11571,N11572,N11573,N11574,N11575,N11576,N11577,N11578,N11579,N11580,  \\note fields as indicated
  N11581,N11582,N11583,N11584,N11585,N11586,N11587,N11588,N11589,N11590,N11591,N11592,N11593,N11594,N11595,N11596,N11597,N11598,N11599,N11600,  \\note fields as indicated
  N11601,N11602,N11603,N11604,N11605,N11606,N11607,N11608,N11609,N11610,N11611,N11612,N11613,N11614,N11615,N11616,N11617,N11618,N11619,N11620,  \\note fields as indicated
  N11621,N11622,N11623,N11624,N11625,N11626,N11627,N11628,N11629,N11630,N11631,N11632,N11633,N11634,N11635,N11636,N11637,N11638,N11639,N11640,  \\note fields as indicated
  N11641,N11642,N11643,N11644,N11645,N11646,N11647,N11648,N11649,N11650,N11651,N11652,N11653,N11654,N11655,N11656,N11657,N11658,N11659,N11660,  \\note fields as indicated
  N11661,N11662,N11663,N11664,N11665,N11666,N11667,N11668,N11669,N11670,N11671,N11672,N11673,N11674,N11675,N11676,N11677,N11678,N11679,N11680,  \\note fields as indicated
  N11681,N11682,N11683,N11684,N11685,N11686,N11687,N11688,N11689,N11690,N11691,N11692,N11693,N11694,N11695,N11696,N11697,N11698,N11699,N11700,  \\note fields as indicated
  N11701,N11702,N11703,N11704,N11705,N11706,N11707,N11708,N11709,N11710,N11711,N11712,N11713,N11714,N11715,N11716,N11717,N11718,N11719,N11720,  \\note fields as indicated
  N11721,N11722,N11723,N11724,N11725,N11726,N11727,N11728,N11729,N11730,N11731,N11732,N11733,N11734,N11735,N11736,N11737,N11738,N11739,N11740,  \\note fields as indicated
  N11741,N11742,N11743,N11744,N11745,N11746,N11747,N11748,N11749,N11750,N11751,N11752,N11753,N11754,N11755,N11756,N11757,N11758,N11759,N11760,  \\note fields as indicated
  N11761,N11762,N11763,N11764,N11765,N11766,N11767,N11768,N11769,N11770,N11771,N11772,N11773,N11774,N11775,N11776,N11777,N11778,N11779,N11780,  \\note fields as indicated
  N11781,N11782,N11783,N11784,N11785,N11786,N11787,N11788,N11789,N11790,N11791,N11792,N11793,N11794,N11795,N11796,N11797,N11798,N11799,N11800,  \\note fields as indicated
  N11801,N11802,N11803,N11804,N11805,N11806,N11807,N11808,N11809,N11810,N11811,N11812,N11813,N11814,N11815,N11816,N11817,N11818,N11819,N11820,  \\note fields as indicated
  N11821,N11822,N11823,N11824,N11825,N11826,N11827,N11828,N11829,N11830,N11831,N11832,N11833,N11834,N11835,N11836,N11837,N11838,N11839,N11840,  \\note fields as indicated
  N11841,N11842,N11843,N11844,N11845,N11846,N11847,N11848,N11849,N11850,N11851,N11852,N11853,N11854,N11855,N11856,N11857,N11858,N11859,N11860,  \\note fields as indicated
  N11861,N11862,N11863,N11864,N11865,N11866,N11867,N11868,N11869,N11870,N11871,N11872,N11873,N11874,N11875,N11876,N11877,N11878,N11879,N11880,  \\note fields as indicated
  N11881,N11882,N11883,N11884,N11885,N11886,N11887,N11888,N11889,N11890,N11891,N11892,N11893,N11894,N11895,N11896,N11897,N11898,N11899,N11900,  \\note fields as indicated
  N11901,N11902,N11903,N11904,N11905,N11906,N11907,N11908,N11909,N11910,N11911,N11912,N11913,N11914,N11915,N11916,N11917,N11918,N11919,N11920,  \\note fields as indicated
  N11921,N11922,N11923,N11924,N11925,N11926,N11927,N11928,N11929,N11930,N11931,N11932,N11933,N11934,N11935,N11936,N11937,N11938,N11939,N11940,  \\note fields as indicated
  N11941,N11942,N11943,N11944,N11945,N11946,N11947,N11948,N11949,N11950,N11951,N11952,N11953,N11954,N11955,N11956,N11957,N11958,N11959,N11960,  \\note fields as indicated
  N11961,N11962,N11963,N11964,N11965,N11966,N11967,N11968,N11969,N11970,N11971,N11972,N11973,N11974,N11975,N11976,N11977,N11978,N11979,N11980,  \\note fields as indicated
  N11981,N11982,N11983,N11984,N11985,N11986,N11987,N11988,N11989,N11990,N11991,N11992,N11993,N11994,N11995,N11996,N11997,N11998,N11999,N12000,  \\note fields as indicated
  N12001,N12002,N12003,N12004,N12005,N12006,N12007,N12008,N12009,N12010,N12011,N12012,N12013,N12014,N12015,N12016,N12017,N12018,N12019,N12020,  \\note fields as indicated
  N12021,N12022,N12023,N12024,N12025,N12026,N12027,N12028,N12029,N12030,N12031,N12032,N12033,N12034,N12035,N12036,N12037,N12038,N12039,N12040,  \\note fields as indicated
  N12041,N12042,N12043,N12044,N12045,N12046,N12047,N12048,N12049,N12050,N12051,N12052,N12053,N12054,N12055,N12056,N12057,N12058,N12059,N12060,  \\note fields as indicated
  N12061,N12062,N12063,N12064,N12065,N12066,N12067,N12068,N12069,N12070,N12071,N12072,N12073,N12074,N12075,N12076,N12077,N12078,N12079,N12080,  \\note fields as indicated
  N12081,N12082,N12083,N12084,N12085,N12086,N12087,N12088,N12089,N12090,N12091,N12092,N12093,N12094,N12095,N12096,N12097,N12098,N12099,N12100,  \\note fields as indicated
  N12101,N12102,N12103,N12104,N12105,N12106,N12107,N12108,N12109,N12110,N12111,N12112,N12113,N12114,N12115,N12116,N12117,N12118,N12119,N12120,  \\note fields as indicated
  N12121,N12122,N12123,N12124,N12125,N12126,N12127,N12128,N12129,N12130,N12131,N12132,N12133,N12134,N12135,N12136,N12137,N12138,N12139,N12140,  \\note fields as indicated
  N12141,N12142,N12143,N12144,N12145,N12146,N12147,N12148,N12149,N12150,N12151,N12152,N12153,N12154,N12155,N12156,N12157,N12158,N12159,N12160,  \\note fields as indicated
  N12161,N12162,N12163,N12164,N12165,N12166,N12167,N12168,N12169,N12170,N12171,N12172,N12173,N12174,N12175,N12176,N12177,N12178,N12179,N12180,  \\note fields as indicated
  N12181,N12182,N12183,N12184,N12185,N12186,N12187,N12188,N12189,N12190,N12191,N12192,N12193,N12194,N12195,N12196,N12197,N12198,N12199,N12200,  \\note fields as indicated
  N12201,N12202,N12203,N12204,N12205,N12206,N12207,N12208,N12209,N12210,N12211,N12212,N12213,N12214,N12215,N12216,N12217,N12218,N12219,N12220,  \\note fields as indicated
  N12221,N12222,N12223,N12224,N12225,N12226,N12227,N12228,N12229,N12230,N12231,N12232,N12233,N12234,N12235,N12236,N12237,N12238,N12239,N12240,  \\note fields as indicated
  N12241,N12242,N12243,N12244,N12245,N12246,N12247,N12248,N12249,N12250,N12251,N12252,N12253,N12254,N12255,N12256,N12257,N12258,N12259,N12260,  \\note fields as indicated
  N12261,N12262,N12263,N12264,N12265,N12266,N12267,N12268,N12269,N12270,N12271,N12272,N12273,N12274,N12275,N12276,N12277,N12278,N12279,N12280,  \\note fields as indicated
  N12281,N12282,N12283,N12284,N12285,N12286,N12287,N12288,N12289,N12290,N12291,N12292,N12293,N12294,N12295,N12296,N12297,N12298,N12299,N12300,  \\note fields as indicated
  N12301,N12302,N12303,N12304,N12305,N12306,N12307,N12308,N12309,N12310,N12311,N12312,N12313,N12314,N12315,N12316,N12317,N12318,N12319,N12320,  \\note fields as indicated
  N12321,N12322,N12323,N12324,N12325,N12326,N12327,N12328,N12329,N12330,N12331,N12332,N12333,N12334,N12335,N12336,N12337,N12338,N12339,N12340,  \\note fields as indicated
  N12341,N12342,N12343,N12344,N12345,N12346,N12347,N12348,N12349,N12350,N12351,N12352,N12353,N12354,N12355,N12356,N12357,N12358,N12359,N12360,  \\note fields as indicated
  N12361,N12362,N12363,N12364,N12365,N12366,N12367,N12368,N12369,N12370,N12371,N12372,N12373,N12374,N12375,N12376,N12377,N12378,N12379,N12380,  \\note fields as indicated
  N12381,N12382,N12383,N12384,N12385,N12386,N12387,N12388,N12389,N12390,N12391,N12392,N12393,N12394,N12395,N12396,N12397,N12398,N12399,N12400,  \\note fields as indicated
  N12401,N12402,N12403,N12404,N12405,N12406,N12407,N12408,N12409,N12410,N12411,N12412,N12413,N12414,N12415,N12416,N12417,N12418,N12419,N12420,  \\note fields as indicated
  N12421,N12422,N12423,N12424,N12425,N12426,N12427,N12428,N12429,N12430,N12431,N12432,N12433,N12434,N12435,N12436,N12437,N12438,N12439,N12440,  \\note fields as indicated
  N12441,N12442,N12443,N12444,N12445,N12446,N12447,N12448,N12449,N12450,N12451,N12452,N12453,N12454,N12455,N12456,N12457,N12458,N12459,N12460,  \\note fields as indicated
  N12461,N12462,N12463,N12464,N12465,N12466,N12467,N12468,N12469,N12470,N12471,N12472,N12473,N12474,N12475,N12476,N12477,N12478,N12479,N12480,  \\note fields as indicated
  N12481,N12482,N12483,N12484,N12485,N12486,N12487,N12488,N12489,N12490,N12491,N12492,N12493,N12494,N12495,N12496,N12497,N12498,N12499,N12500,  \\note fields as indicated
  N12501,N12502,N12503,N12504,N12505,N12506,N12507,N12508,N12509,N12510,N12511,N12512,N12513,N12514,N12515,N12516,N12517,N12518,N12519,N12520,  \\note fields as indicated
  N12521,N12522,N12523,N12524,N12525,N12526,N12527,N12528,N12529,N12530,N12531,N12532,N12533,N12534,N12535,N12536,N12537,N12538,N12539,N12540,  \\note fields as indicated
  N12541,N12542,N12543,N12544,N12545,N12546,N12547,N12548,N12549,N12550,N12551,N12552,N12553,N12554,N12555,N12556,N12557,N12558,N12559,N12560,  \\note fields as indicated
  N12561,N12562,N12563,N12564,N12565,N12566,N12567,N12568,N12569,N12570,N12571,N12572,N12573,N12574,N12575,N12576,N12577,N12578,N12579,N12580,  \\note fields as indicated
  N12581,N12582,N12583,N12584,N12585,N12586,N12587,N12588,N12589,N12590,N12591,N12592,N12593,N12594,N12595,N12596,N12597,N12598,N12599,N12600,  \\note fields as indicated
  N12601,N12602,N12603,N12604,N12605,N12606,N12607,N12608,N12609,N12610,N12611,N12612,N12613,N12614,N12615,N12616,N12617,N12618,N12619,N12620,  \\note fields as indicated
  N12621,N12622,N12623,N12624,N12625,N12626,N12627,N12628,N12629,N12630,N12631,N12632,N12633,N12634,N12635,N12636,N12637,N12638,N12639,N12640,  \\note fields as indicated
  N12641,N12642,N12643,N12644,N12645,N12646,N12647,N12648,N12649,N12650,N12651,N12652,N12653,N12654,N12655,N12656,N12657,N12658,N12659,N12660,  \\note fields as indicated
  N12661,N12662,N12663,N12664,N12665,N12666,N12667,N12668,N12669,N12670,N12671,N12672,N12673,N12674,N12675,N12676,N12677,N12678,N12679,N12680,  \\note fields as indicated
  N12681,N12682,N12683,N12684,N12685,N12686,N12687,N12688,N12689,N12690,N12691,N12692,N12693,N12694,N12695,N12696,N12697,N12698,N12699,N12700,  \\note fields as indicated
  N12701,N12702,N12703,N12704,N12705,N12706,N12707,N12708,N12709,N12710,N12711,N12712,N12713,N12714,N12715,N12716,N12717,N12718,N12719,N12720,  \\note fields as indicated
  N12721,N12722,N12723,N12724,N12725,N12726,N12727,N12728,N12729,N12730,N12731,N12732,N12733,N12734,N12735,N12736,N12737,N12738,N12739,N12740,  \\note fields as indicated
  N12741,N12742,N12743,N12744,N12745,N12746,N12747,N12748,N12749,N12750,N12751,N12752,N12753,N12754,N12755,N12756,N12757,N12758,N12759,N12760,  \\note fields as indicated
  N12761,N12762,N12763,N12764,N12765,N12766,N12767,N12768,N12769,N12770,N12771,N12772,N12773,N12774,N12775,N12776,N12777,N12778,N12779,N12780,  \\note fields as indicated
  N12781,N12782,N12783,N12784,N12785,N12786,N12787,N12788,N12789,N12790,N12791,N12792,N12793,N12794,N12795,N12796,N12797,N12798,N12799,N12800,  \\note fields as indicated
  N12801,N12802,N12803,N12804,N12805,N12806,N12807,N12808,N12809,N12810,N12811,N12812,N12813,N12814,N12815,N12816,N12817,N12818,N12819,N12820,  \\note fields as indicated
  N12821,N12822,N12823,N12824,N12825,N12826,N12827,N12828,N12829,N12830,N12831,N12832,N12833,N12834,N12835,N12836,N12837,N12838,N12839,N12840,  \\note fields as indicated
  N12841,N12842,N12843,N12844,N12845,N12846,N12847,N12848,N12849,N12850,N12851,N12852,N12853,N12854,N12855,N12856,N12857,N12858,N12859,N12860,  \\note fields as indicated
  N12861,N12862,N12863,N12864,N12865,N12866,N12867,N12868,N12869,N12870,N12871,N12872,N12873,N12874,N12875,N12876,N12877,N12878,N12879,N12880,  \\note fields as indicated
  N12881,N12882,N12883,N12884,N12885,N12886,N12887,N12888,N12889,N12890,N12891,N12892,N12893,N12894,N12895,N12896,N12897,N12898,N12899,N12900,  \\note fields as indicated
  N12901,N12902,N12903,N12904,N12905,N12906,N12907,N12908,N12909,N12910,N12911,N12912,N12913,N12914,N12915,N12916,N12917,N12918,N12919,N12920,  \\note fields as indicated
  N12921,N12922,N12923,N12924,N12925,N12926,N12927,N12928,N12929,N12930,N12931,N12932,N12933,N12934,N12935,N12936,N12937,N12938,N12939,N12940,  \\note fields as indicated
  N12941,N12942,N12943,N12944,N12945,N12946,N12947,N12948,N12949,N12950,N12951,N12952,N12953,N12954,N12955,N12956,N12957,N12958,N12959,N12960,  \\note fields as indicated
  N12961,N12962,N12963,N12964,N12965,N12966,N12967,N12968,N12969,N12970,N12971,N12972,N12973,N12974,N12975,N12976,N12977,N12978,N12979,N12980,  \\note fields as indicated
  N12981,N12982,N12983,N12984,N12985,N12986,N12987,N12988,N12989,N12990,N12991,N12992,N12993,N12994,N12995,N12996,N12997,N12998,N12999,N13000,  \\note fields as indicated
  N13001,N13002,N13003,N13004,N13005,N13006,N13007,N13008,N13009,N13010,N13011,N13012,N13013,N13014,N13015,N13016,N13017,N13018,N13019,N13020,  \\note fields as indicated
  N13021,N13022,N13023,N13024,N13025,N13026,N13027,N13028,N13029,N13030,N13031,N13032,N13033,N13034,N13035,N13036,N13037,N13038,N13039,N13040,  \\note fields as indicated
  N13041,N13042,N13043,N13044,N13045,N13046,N13047,N13048,N13049,N13050,N13051,N13052,N13053,N13054,N13055,N13056,N13057,N13058,N13059,N13060,  \\note fields as indicated
  N13061,N13062,N13063,N13064,N13065,N13066,N13067,N13068,N13069,N13070,N13071,N13072,N13073,N13074,N13075,N13076,N13077,N13078,N13079,N13080,  \\note fields as indicated
  N13081,N13082,N13083,N13084,N13085,N13086,N13087,N13088,N13089,N13090,N13091,N13092,N13093,N13094,N13095,N13096,N13097,N13098,N13099,N13100,  \\note fields as indicated
  N13101,N13102,N13103,N13104,N13105,N13106,N13107,N13108,N13109,N13110,N13111,N13112,N13113,N13114,N13115,N13116,N13117,N13118,N13119,N13120,  \\note fields as indicated
  N13121,N13122,N13123,N13124,N13125,N13126,N13127,N13128,N13129,N13130,N13131,N13132,N13133,N13134,N13135,N13136,N13137,N13138,N13139,N13140,  \\note fields as indicated
  N13141,N13142,N13143,N13144,N13145,N13146,N13147,N13148,N13149,N13150,N13151,N13152,N13153,N13154,N13155,N13156,N13157,N13158,N13159,N13160,  \\note fields as indicated
  N13161,N13162,N13163,N13164,N13165,N13166,N13167,N13168,N13169,N13170,N13171,N13172,N13173,N13174,N13175,N13176,N13177,N13178,N13179,N13180,  \\note fields as indicated
  N13181,N13182,N13183,N13184,N13185,N13186,N13187,N13188,N13189,N13190,N13191,N13192,N13193,N13194,N13195,N13196,N13197,N13198,N13199,N13200,  \\note fields as indicated
  N13201,N13202,N13203,N13204,N13205,N13206,N13207,N13208,N13209,N13210,N13211,N13212,N13213,N13214,N13215,N13216,N13217,N13218,N13219,N13220,  \\note fields as indicated
  N13221,N13222,N13223,N13224,N13225,N13226,N13227,N13228,N13229,N13230,N13231,N13232,N13233,N13234,N13235,N13236,N13237,N13238,N13239,N13240,  \\note fields as indicated
  N13241,N13242,N13243,N13244,N13245,N13246,N13247,N13248,N13249,N13250,N13251,N13252,N13253,N13254,N13255,N13256,N13257,N13258,N13259,N13260,  \\note fields as indicated
  N13261,N13262,N13263,N13264,N13265,N13266,N13267,N13268,N13269,N13270,N13271,N13272,N13273,N13274,N13275,N13276,N13277,N13278,N13279,N13280,  \\note fields as indicated
  N13281,N13282,N13283,N13284,N13285,N13286,N13287,N13288,N13289,N13290,N13291,N13292,N13293,N13294,N13295,N13296,N13297,N13298,N13299,N13300,  \\note fields as indicated
  N13301,N13302,N13303,N13304,N13305,N13306,N13307,N13308,N13309,N13310,N13311,N13312,N13313,N13314,N13315,N13316,N13317,N13318,N13319,N13320,  \\note fields as indicated
  N13321,N13322,N13323,N13324,N13325,N13326,N13327,N13328,N13329,N13330,N13331,N13332,N13333,N13334,N13335,N13336,N13337,N13338,N13339,N13340,  \\note fields as indicated
  N13341,N13342,N13343,N13344,N13345,N13346,N13347,N13348,N13349,N13350,N13351,N13352,N13353,N13354,N13355,N13356,N13357,N13358,N13359,N13360,  \\note fields as indicated
  N13361,N13362,N13363,N13364,N13365,N13366,N13367,N13368,N13369,N13370,N13371,N13372,N13373,N13374,N13375,N13376,N13377,N13378,N13379,N13380,  \\note fields as indicated
  N13381,N13382,N13383,N13384,N13385,N13386,N13387,N13388,N13389,N13390,N13391,N13392,N13393,N13394,N13395,N13396,N13397,N13398,N13399,N13400,  \\note fields as indicated
  N13401,N13402,N13403,N13404,N13405,N13406,N13407,N13408,N13409,N13410,N13411,N13412,N13413,N13414,N13415,N13416,N13417,N13418,N13419,N13420,  \\note fields as indicated
  N13421,N13422,N13423,N13424,N13425,N13426,N13427,N13428,N13429,N13430,N13431,N13432,N13433,N13434,N13435,N13436,N13437,N13438,N13439,N13440,  \\note fields as indicated
  N13441,N13442,N13443,N13444,N13445,N13446,N13447,N13448,N13449,N13450,N13451,N13452,N13453,N13454,N13455,N13456,N13457,N13458,N13459,N13460,  \\note fields as indicated
  N13461,N13462,N13463,N13464,N13465,N13466,N13467,N13468,N13469,N13470,N13471,N13472,N13473,N13474,N13475,N13476,N13477,N13478,N13479,N13480,  \\note fields as indicated
  N13481,N13482,N13483,N13484,N13485,N13486,N13487,N13488,N13489,N13490,N13491,N13492,N13493,N13494,N13495,N13496,N13497,N13498,N13499,N13500,  \\note fields as indicated
  N13501,N13502,N13503,N13504,N13505,N13506,N13507,N13508,N13509,N13510,N13511,N13512,N13513,N13514,N13515,N13516,N13517,N13518,N13519,N13520,  \\note fields as indicated
  N13521,N13522,N13523,N13524,N13525,N13526,N13527,N13528,N13529,N13530,N13531,N13532,N13533,N13534,N13535,N13536,N13537,N13538,N13539,N13540,  \\note fields as indicated
  N13541,N13542,N13543,N13544,N13545,N13546,N13547,N13548,N13549,N13550,N13551,N13552,N13553,N13554,N13555,N13556,N13557,N13558,N13559,N13560,  \\note fields as indicated
  N13561,N13562,N13563,N13564,N13565,N13566,N13567,N13568,N13569,N13570,N13571,N13572,N13573,N13574,N13575,N13576,N13577,N13578,N13579,N13580,  \\note fields as indicated
  N13581,N13582,N13583,N13584,N13585,N13586,N13587,N13588,N13589,N13590,N13591,N13592,N13593,N13594,N13595,N13596,N13597,N13598,N13599,N13600,  \\note fields as indicated
  N13601,N13602,N13603,N13604,N13605,N13606,N13607,N13608,N13609,N13610,N13611,N13612,N13613,N13614,N13615,N13616,N13617,N13618,N13619,N13620,  \\note fields as indicated
  N13621,N13622,N13623,N13624,N13625,N13626,N13627,N13628,N13629,N13630,N13631,N13632,N13633,N13634,N13635,N13636,N13637,N13638,N13639,N13640,  \\note fields as indicated
  N13641,N13642,N13643,N13644,N13645,N13646,N13647,N13648,N13649,N13650,N13651,N13652,N13653,N13654,N13655,N13656,N13657,N13658,N13659,N13660,  \\note fields as indicated
  N13661,N13662,N13663,N13664,N13665,N13666,N13667,N13668,N13669,N13670,N13671,N13672,N13673,N13674,N13675,N13676,N13677,N13678,N13679,N13680,  \\note fields as indicated
  N13681,N13682,N13683,N13684,N13685,N13686,N13687,N13688,N13689,N13690,N13691,N13692,N13693,N13694,N13695,N13696,N13697,N13698,N13699,N13700,  \\note fields as indicated
  N13701,N13702,N13703,N13704,N13705,N13706,N13707,N13708,N13709,N13710,N13711,N13712,N13713,N13714,N13715,N13716,N13717,N13718,N13719,N13720,  \\note fields as indicated
  N13721,N13722,N13723,N13724,N13725,N13726,N13727,N13728,N13729,N13730,N13731,N13732,N13733,N13734,N13735,N13736,N13737,N13738,N13739,N13740,  \\note fields as indicated
  N13741,N13742,N13743,N13744,N13745,N13746,N13747,N13748,N13749,N13750,N13751,N13752,N13753,N13754,N13755,N13756,N13757,N13758,N13759,N13760,  \\note fields as indicated
  N13761,N13762,N13763,N13764,N13765,N13766,N13767,N13768,N13769,N13770,N13771,N13772,N13773,N13774,N13775,N13776,N13777,N13778,N13779,N13780,  \\note fields as indicated
  N13781,N13782,N13783,N13784,N13785,N13786,N13787,N13788,N13789,N13790,N13791,N13792,N13793,N13794,N13795,N13796,N13797,N13798,N13799,N13800,  \\note fields as indicated
  N13801,N13802,N13803,N13804,N13805,N13806,N13807,N13808,N13809,N13810,N13811,N13812,N13813,N13814,N13815,N13816,N13817,N13818,N13819,N13820,  \\note fields as indicated
  N13821,N13822,N13823,N13824,N13825,N13826,N13827,N13828,N13829,N13830,N13831,N13832,N13833,N13834,N13835,N13836,N13837,N13838,N13839,N13840,  \\note fields as indicated
  N13841,N13842,N13843,N13844,N13845,N13846,N13847,N13848,N13849,N13850,N13851,N13852,N13853,N13854,N13855,N13856,N13857,N13858,N13859,N13860,  \\note fields as indicated
  N13861,N13862,N13863,N13864,N13865,N13866,N13867,N13868,N13869,N13870,N13871,N13872,N13873,N13874,N13875,N13876,N13877,N13878,N13879,N13880,  \\note fields as indicated
  N13881,N13882,N13883,N13884,N13885,N13886,N13887,N13888,N13889,N13890,N13891,N13892,N13893,N13894,N13895,N13896,N13897,N13898,N13899,N13900,  \\note fields as indicated
  N13901,N13902,N13903,N13904,N13905,N13906,N13907,N13908,N13909,N13910,N13911,N13912,N13913,N13914,N13915,N13916,N13917,N13918,N13919,N13920,  \\note fields as indicated
  N13921,N13922,N13923,N13924,N13925,N13926,N13927,N13928,N13929,N13930,N13931,N13932,N13933,N13934,N13935,N13936,N13937,N13938,N13939,N13940,  \\note fields as indicated
  N13941,N13942,N13943,N13944,N13945,N13946,N13947,N13948,N13949,N13950,N13951,N13952,N13953,N13954,N13955,N13956,N13957,N13958,N13959,N13960,  \\note fields as indicated
  N13961,N13962,N13963,N13964,N13965,N13966,N13967,N13968,N13969,N13970,N13971,N13972,N13973,N13974,N13975,N13976,N13977,N13978,N13979,N13980,  \\note fields as indicated
  N13981,N13982,N13983,N13984,N13985,N13986,N13987,N13988,N13989,N13990,N13991,N13992,N13993,N13994,N13995,N13996,N13997,N13998,N13999,N14000,  \\note fields as indicated
  N14001,N14002,N14003,N14004,N14005,N14006,N14007,N14008,N14009,N14010,N14011,N14012,N14013,N14014,N14015,N14016,N14017,N14018,N14019,N14020,  \\note fields as indicated
  N14021,N14022,N14023,N14024,N14025,N14026,N14027,N14028,N14029,N14030,N14031,N14032,N14033,N14034,N14035,N14036,N14037,N14038,N14039,N14040,  \\note fields as indicated
  N14041,N14042,N14043,N14044,N14045,N14046,N14047,N14048,N14049,N14050,N14051,N14052,N14053,N14054,N14055,N14056,N14057,N14058,N14059,N14060,  \\note fields as indicated
  N14061,N14062,N14063,N14064,N14065,N14066,N14067,N14068,N14069,N14070,N14071,N14072,N14073,N14074,N14075,N14076,N14077,N14078,N14079,N14080,  \\note fields as indicated
  N14081,N14082,N14083,N14084,N14085,N14086,N14087,N14088,N14089,N14090,N14091,N14092,N14093,N14094,N14095,N14096,N14097,N14098,N14099,N14100,  \\note fields as indicated
  N14101,N14102,N14103,N14104,N14105,N14106,N14107,N14108,N14109,N14110,N14111,N14112,N14113,N14114,N14115,N14116,N14117,N14118,N14119,N14120,  \\note fields as indicated
  N14121,N14122,N14123,N14124,N14125,N14126,N14127,N14128,N14129,N14130,N14131,N14132,N14133,N14134,N14135,N14136,N14137,N14138,N14139,N14140,  \\note fields as indicated
  N14141,N14142,N14143,N14144,N14145,N14146,N14147,N14148,N14149,N14150,N14151,N14152,N14153,N14154,N14155,N14156,N14157,N14158,N14159,N14160,  \\note fields as indicated
  N14161,N14162,N14163,N14164,N14165,N14166,N14167,N14168,N14169,N14170,N14171,N14172,N14173,N14174,N14175,N14176,N14177,N14178,N14179,N14180,  \\note fields as indicated
  N14181,N14182,N14183,N14184,N14185,N14186,N14187,N14188,N14189,N14190,N14191,N14192,N14193,N14194,N14195,N14196,N14197,N14198,N14199,N14200,  \\note fields as indicated
  N14201,N14202,N14203,N14204,N14205,N14206,N14207,N14208,N14209,N14210,N14211,N14212,N14213,N14214,N14215,N14216,N14217,N14218,N14219,N14220,  \\note fields as indicated
  N14221,N14222,N14223,N14224,N14225,N14226,N14227,N14228,N14229,N14230,N14231,N14232,N14233,N14234,N14235,N14236,N14237,N14238,N14239,N14240,  \\note fields as indicated
  N14241,N14242,N14243,N14244,N14245,N14246,N14247,N14248,N14249,N14250,N14251,N14252,N14253,N14254,N14255,N14256,N14257,N14258,N14259,N14260,  \\note fields as indicated
  N14261,N14262,N14263,N14264,N14265,N14266,N14267,N14268,N14269,N14270,N14271,N14272,N14273,N14274,N14275,N14276,N14277,N14278,N14279,N14280,  \\note fields as indicated
  N14281,N14282,N14283,N14284,N14285,N14286,N14287,N14288,N14289,N14290,N14291,N14292,N14293,N14294,N14295,N14296,N14297,N14298,N14299,N14300,  \\note fields as indicated
  N14301,N14302,N14303,N14304,N14305,N14306,N14307,N14308,N14309,N14310,N14311,N14312,N14313,N14314,N14315,N14316,N14317,N14318,N14319,N14320,  \\note fields as indicated
  N14321,N14322,N14323,N14324,N14325,N14326,N14327,N14328,N14329,N14330,N14331,N14332,N14333,N14334,N14335,N14336,N14337,N14338,N14339,N14340,  \\note fields as indicated
  N14341,N14342,N14343,N14344,N14345,N14346,N14347,N14348,N14349,N14350,N14351,N14352,N14353,N14354,N14355,N14356,N14357,N14358,N14359,N14360,  \\note fields as indicated
  N14361,N14362,N14363,N14364,N14365,N14366,N14367,N14368,N14369,N14370,N14371,N14372,N14373,N14374,N14375,N14376,N14377,N14378,N14379,N14380,  \\note fields as indicated
  N14381,N14382,N14383,N14384,N14385,N14386,N14387,N14388,N14389,N14390,N14391,N14392,N14393,N14394,N14395,N14396,N14397,N14398,N14399,N14400,  \\note fields as indicated
  N14401,N14402,N14403,N14404,N14405,N14406,N14407,N14408,N14409,N14410,N14411,N14412,N14413,N14414,N14415,N14416,N14417,N14418,N14419,N14420,  \\note fields as indicated
  N14421,N14422,N14423,N14424,N14425,N14426,N14427,N14428,N14429,N14430,N14431,N14432,N14433,N14434,N14435,N14436,N14437,N14438,N14439,N14440,  \\note fields as indicated
  N14441,N14442,N14443,N14444,N14445,N14446,N14447,N14448,N14449,N14450,N14451,N14452,N14453,N14454,N14455,N14456,N14457,N14458,N14459,N14460,  \\note fields as indicated
  N14461,N14462,N14463,N14464,N14465,N14466,N14467,N14468,N14469,N14470,N14471,N14472,N14473,N14474,N14475,N14476,N14477,N14478,N14479,N14480,  \\note fields as indicated
  N14481,N14482,N14483,N14484,N14485,N14486,N14487,N14488,N14489,N14490,N14491,N14492,N14493,N14494,N14495,N14496,N14497,N14498,N14499,N14500,  \\note fields as indicated
  N14501,N14502,N14503,N14504,N14505,N14506,N14507,N14508,N14509,N14510,N14511,N14512,N14513,N14514,N14515,N14516,N14517,N14518,N14519,N14520,  \\note fields as indicated
  N14521,N14522,N14523,N14524,N14525,N14526,N14527,N14528,N14529,N14530,N14531,N14532,N14533,N14534,N14535,N14536,N14537,N14538,N14539,N14540,  \\note fields as indicated
  N14541,N14542,N14543,N14544,N14545,N14546,N14547,N14548,N14549,N14550,N14551,N14552,N14553,N14554,N14555,N14556,N14557,N14558,N14559,N14560,  \\note fields as indicated
  N14561,N14562,N14563,N14564,N14565,N14566,N14567,N14568,N14569,N14570,N14571,N14572,N14573,N14574,N14575,N14576,N14577,N14578,N14579,N14580,  \\note fields as indicated
  N14581,N14582,N14583,N14584,N14585,N14586,N14587,N14588,N14589,N14590,N14591,N14592,N14593,N14594,N14595,N14596,N14597,N14598,N14599,N14600,  \\note fields as indicated
  N14601,N14602,N14603,N14604,N14605,N14606,N14607,N14608,N14609,N14610,N14611,N14612,N14613,N14614,N14615,N14616,N14617,N14618,N14619,N14620,  \\note fields as indicated
  N14621,N14622,N14623,N14624,N14625,N14626,N14627,N14628,N14629,N14630,N14631,N14632,N14633,N14634,N14635,N14636,N14637,N14638,N14639,N14640,  \\note fields as indicated
  N14641,N14642,N14643,N14644,N14645,N14646,N14647,N14648,N14649,N14650,N14651,N14652,N14653,N14654,N14655,N14656,N14657,N14658,N14659,N14660,  \\note fields as indicated
  N14661,N14662,N14663,N14664,N14665,N14666,N14667,N14668,N14669,N14670,N14671,N14672,N14673,N14674,N14675,N14676,N14677,N14678,N14679,N14680,  \\note fields as indicated
  N14681,N14682,N14683,N14684,N14685,N14686,N14687,N14688,N14689,N14690,N14691,N14692,N14693,N14694,N14695,N14696,N14697,N14698,N14699,N14700,  \\note fields as indicated
  N14701,N14702,N14703,N14704,N14705,N14706,N14707,N14708,N14709,N14710,N14711,N14712,N14713,N14714,N14715,N14716,N14717,N14718,N14719,N14720,  \\note fields as indicated
  N14721,N14722,N14723,N14724,N14725,N14726,N14727,N14728,N14729,N14730,N14731,N14732,N14733,N14734,N14735,N14736,N14737,N14738,N14739,N14740,  \\note fields as indicated
  N14741,N14742,N14743,N14744,N14745,N14746,N14747,N14748,N14749,N14750,N14751,N14752,N14753,N14754,N14755,N14756,N14757,N14758,N14759,N14760,  \\note fields as indicated
  N14761,N14762,N14763,N14764,N14765,N14766,N14767,N14768,N14769,N14770,N14771,N14772,N14773,N14774,N14775,N14776,N14777,N14778,N14779,N14780,  \\note fields as indicated
  N14781,N14782,N14783,N14784,N14785,N14786,N14787,N14788,N14789,N14790,N14791,N14792,N14793,N14794,N14795,N14796,N14797,N14798,N14799,N14800,  \\note fields as indicated
  N14801,N14802,N14803,N14804,N14805,N14806,N14807,N14808,N14809,N14810,N14811,N14812,N14813,N14814,N14815,N14816,N14817,N14818,N14819,N14820,  \\note fields as indicated
  N14821,N14822,N14823,N14824,N14825,N14826,N14827,N14828,N14829,N14830,N14831,N14832,N14833,N14834,N14835,N14836,N14837,N14838,N14839,N14840,  \\note fields as indicated
  N14841,N14842,N14843,N14844,N14845,N14846,N14847,N14848,N14849,N14850,N14851,N14852,N14853,N14854,N14855,N14856,N14857,N14858,N14859,N14860,  \\note fields as indicated
  N14861,N14862,N14863,N14864,N14865,N14866,N14867,N14868,N14869,N14870,N14871,N14872,N14873,N14874,N14875,N14876,N14877,N14878,N14879,N14880,  \\note fields as indicated
  N14881,N14882,N14883,N14884,N14885,N14886,N14887,N14888,N14889,N14890,N14891,N14892,N14893,N14894,N14895,N14896,N14897,N14898,N14899,N14900,  \\note fields as indicated
  N14901,N14902,N14903,N14904,N14905,N14906,N14907,N14908,N14909,N14910,N14911,N14912,N14913,N14914,N14915,N14916,N14917,N14918,N14919,N14920,  \\note fields as indicated
  N14921,N14922,N14923,N14924,N14925,N14926,N14927,N14928,N14929,N14930,N14931,N14932,N14933,N14934,N14935,N14936,N14937,N14938,N14939,N14940,  \\note fields as indicated
  N14941,N14942,N14943,N14944,N14945,N14946,N14947,N14948,N14949,N14950,N14951,N14952,N14953,N14954,N14955,N14956,N14957,N14958,N14959,N14960,  \\note fields as indicated
  N14961,N14962,N14963,N14964,N14965,N14966,N14967,N14968,N14969,N14970,N14971,N14972,N14973,N14974,N14975,N14976,N14977,N14978,N14979,N14980,  \\note fields as indicated
  N14981,N14982,N14983,N14984,N14985,N14986,N14987,N14988,N14989,N14990,N14991,N14992,N14993,N14994,N14995,N14996,N14997,N14998,N14999,N15000,  \\note fields as indicated
  N15001,N15002,N15003,N15004,N15005,N15006,N15007,N15008,N15009,N15010,N15011,N15012,N15013,N15014,N15015,N15016,N15017,N15018,N15019,N15020,  \\note fields as indicated
  N15021,N15022,N15023,N15024,N15025,N15026,N15027,N15028,N15029,N15030,N15031,N15032,N15033,N15034,N15035,N15036,N15037,N15038,N15039,N15040,  \\note fields as indicated
  N15041,N15042,N15043,N15044,N15045,N15046,N15047,N15048,N15049,N15050,N15051,N15052,N15053,N15054,N15055,N15056,N15057,N15058,N15059,N15060,  \\note fields as indicated
  N15061,N15062,N15063,N15064,N15065,N15066,N15067,N15068,N15069,N15070,N15071,N15072,N15073,N15074,N15075,N15076,N15077,N15078,N15079,N15080,  \\note fields as indicated
  N15081,N15082,N15083,N15084,N15085,N15086,N15087,N15088,N15089,N15090,N15091,N15092,N15093,N15094,N15095,N15096,N15097,N15098,N15099,N15100,  \\note fields as indicated
  N15101,N15102,N15103,N15104,N15105,N15106,N15107,N15108,N15109,N15110,N15111,N15112,N15113,N15114,N15115,N15116,N15117,N15118,N15119,N15120,  \\note fields as indicated
  N15121,N15122,N15123,N15124,N15125,N15126,N15127,N15128,N15129,N15130,N15131,N15132,N15133,N15134,N15135,N15136,N15137,N15138,N15139,N15140,  \\note fields as indicated
  N15141,N15142,N15143,N15144,N15145,N15146,N15147,N15148,N15149,N15150,N15151,N15152,N15153,N15154,N15155,N15156,N15157,N15158,N15159,N15160,  \\note fields as indicated
  N15161,N15162,N15163,N15164,N15165,N15166,N15167,N15168,N15169,N15170,N15171,N15172,N15173,N15174,N15175,N15176,N15177,N15178,N15179,N15180,  \\note fields as indicated
  N15181,N15182,N15183,N15184,N15185,N15186,N15187,N15188,N15189,N15190,N15191,N15192,N15193,N15194,N15195,N15196,N15197,N15198,N15199,N15200,  \\note fields as indicated
  N15201,N15202,N15203,N15204,N15205,N15206,N15207,N15208,N15209,N15210,N15211,N15212,N15213,N15214,N15215,N15216,N15217,N15218,N15219,N15220,  \\note fields as indicated
  N15221,N15222,N15223,N15224,N15225,N15226,N15227,N15228,N15229,N15230,N15231,N15232,N15233,N15234,N15235,N15236,N15237,N15238,N15239,N15240,  \\note fields as indicated
  N15241,N15242,N15243,N15244,N15245,N15246,N15247,N15248,N15249,N15250,N15251,N15252,N15253,N15254,N15255,N15256,N15257,N15258,N15259,N15260,  \\note fields as indicated
  N15261,N15262,N15263,N15264,N15265,N15266,N15267,N15268,N15269,N15270,N15271,N15272,N15273,N15274,N15275,N15276,N15277,N15278,N15279,N15280,  \\note fields as indicated
  N15281,N15282,N15283,N15284,N15285,N15286,N15287,N15288,N15289,N15290,N15291,N15292,N15293,N15294,N15295,N15296,N15297,N15298,N15299,N15300,  \\note fields as indicated
  N15301,N15302,N15303,N15304,N15305,N15306,N15307,N15308,N15309,N15310,N15311,N15312,N15313,N15314,N15315,N15316,N15317,N15318,N15319,N15320,  \\note fields as indicated
  N15321,N15322,N15323,N15324,N15325,N15326,N15327,N15328,N15329,N15330,N15331,N15332,N15333,N15334,N15335,N15336,N15337,N15338,N15339,N15340,  \\note fields as indicated
  N15341,N15342,N15343,N15344,N15345,N15346,N15347,N15348,N15349,N15350,N15351,N15352,N15353,N15354,N15355,N15356,N15357,N15358,N15359,N15360,  \\note fields as indicated
  N15361,N15362,N15363,N15364,N15365,N15366,N15367,N15368,N15369,N15370,N15371,N15372,N15373,N15374,N15375,N15376,N15377,N15378,N15379,N15380,  \\note fields as indicated
  N15381,N15382,N15383,N15384,N15385,N15386,N15387,N15388,N15389,N15390,N15391,N15392,N15393,N15394,N15395,N15396,N15397,N15398,N15399,N15400,  \\note fields as indicated
  N15401,N15402,N15403,N15404,N15405,N15406,N15407,N15408,N15409,N15410,N15411,N15412,N15413,N15414,N15415,N15416,N15417,N15418,N15419,N15420,  \\note fields as indicated
  N15421,N15422,N15423,N15424,N15425,N15426,N15427,N15428,N15429,N15430,N15431,N15432,N15433,N15434,N15435,N15436,N15437,N15438,N15439,N15440,  \\note fields as indicated
  N15441,N15442,N15443,N15444,N15445,N15446,N15447,N15448,N15449,N15450,N15451,N15452,N15453,N15454,N15455,N15456,N15457,N15458,N15459,N15460,  \\note fields as indicated
  N15461,N15462,N15463,N15464,N15465,N15466,N15467,N15468,N15469,N15470,N15471,N15472,N15473,N15474,N15475,N15476,N15477,N15478,N15479,N15480,  \\note fields as indicated
  N15481,N15482,N15483,N15484,N15485,N15486,N15487,N15488,N15489,N15490,N15491,N15492,N15493,N15494,N15495,N15496,N15497,N15498,N15499,N15500,  \\note fields as indicated
  N15501,N15502,N15503,N15504,N15505,N15506,N15507,N15508,N15509,N15510,N15511,N15512,N15513,N15514,N15515,N15516,N15517,N15518,N15519,N15520,  \\note fields as indicated
  N15521,N15522,N15523,N15524,N15525,N15526,N15527,N15528,N15529,N15530,N15531,N15532,N15533,N15534,N15535,N15536,N15537,N15538,N15539,N15540,  \\note fields as indicated
  N15541,N15542,N15543,N15544,N15545,N15546,N15547,N15548,N15549,N15550,N15551,N15552,N15553,N15554,N15555,N15556,N15557,N15558,N15559,N15560,  \\note fields as indicated
  N15561,N15562,N15563,N15564,N15565,N15566,N15567,N15568,N15569,N15570,N15571,N15572,N15573,N15574,N15575,N15576,N15577,N15578,N15579,N15580,  \\note fields as indicated
  N15581,N15582,N15583,N15584,N15585,N15586,N15587,N15588,N15589,N15590,N15591,N15592,N15593,N15594,N15595,N15596,N15597,N15598,N15599,N15600,  \\note fields as indicated
  N15601,N15602,N15603,N15604,N15605,N15606,N15607,N15608,N15609,N15610,N15611,N15612,N15613,N15614,N15615,N15616,N15617,N15618,N15619,N15620,  \\note fields as indicated
  N15621,N15622,N15623,N15624,N15625,N15626,N15627,N15628,N15629,N15630,N15631,N15632,N15633,N15634,N15635,N15636,N15637,N15638,N15639,N15640,  \\note fields as indicated
  N15641,N15642,N15643,N15644,N15645,N15646,N15647,N15648,N15649,N15650,N15651,N15652,N15653,N15654,N15655,N15656,N15657,N15658,N15659,N15660,  \\note fields as indicated
  N15661,N15662,N15663,N15664,N15665,N15666,N15667,N15668,N15669,N15670,N15671,N15672,N15673,N15674,N15675,N15676,N15677,N15678,N15679,N15680,  \\note fields as indicated
  N15681,N15682,N15683,N15684,N15685,N15686,N15687,N15688,N15689,N15690,N15691,N15692,N15693,N15694,N15695,N15696,N15697,N15698,N15699,N15700,  \\note fields as indicated
  N15701,N15702,N15703,N15704,N15705,N15706,N15707,N15708,N15709,N15710,N15711,N15712,N15713,N15714,N15715,N15716,N15717,N15718,N15719,N15720,  \\note fields as indicated
  N15721,N15722,N15723,N15724,N15725,N15726,N15727,N15728,N15729,N15730,N15731,N15732,N15733,N15734,N15735,N15736,N15737,N15738,N15739,N15740,  \\note fields as indicated
  N15741,N15742,N15743,N15744,N15745,N15746,N15747,N15748,N15749,N15750,N15751,N15752,N15753,N15754,N15755,N15756,N15757,N15758,N15759,N15760,  \\note fields as indicated
  N15761,N15762,N15763,N15764,N15765,N15766,N15767,N15768,N15769,N15770,N15771,N15772,N15773,N15774,N15775,N15776,N15777,N15778,N15779,N15780,  \\note fields as indicated
  N15781,N15782,N15783,N15784,N15785,N15786,N15787,N15788,N15789,N15790,N15791,N15792,N15793,N15794,N15795,N15796,N15797,N15798,N15799,N15800,  \\note fields as indicated
  N15801,N15802,N15803,N15804,N15805,N15806,N15807,N15808,N15809,N15810,N15811,N15812,N15813,N15814,N15815,N15816,N15817,N15818,N15819,N15820,  \\note fields as indicated
  N15821,N15822,N15823,N15824,N15825,N15826,N15827,N15828,N15829,N15830,N15831,N15832,N15833,N15834,N15835,N15836,N15837,N15838,N15839,N15840,  \\note fields as indicated
  N15841,N15842,N15843,N15844,N15845,N15846,N15847,N15848,N15849,N15850,N15851,N15852,N15853,N15854,N15855,N15856,N15857,N15858,N15859,N15860,  \\note fields as indicated
  N15861,N15862,N15863,N15864,N15865,N15866,N15867,N15868,N15869,N15870,N15871,N15872,N15873,N15874,N15875,N15876,N15877,N15878,N15879,N15880,  \\note fields as indicated
  N15881,N15882,N15883,N15884,N15885,N15886,N15887,N15888,N15889,N15890,N15891,N15892,N15893,N15894,N15895,N15896,N15897,N15898,N15899,N15900,  \\note fields as indicated
  N15901,N15902,N15903,N15904,N15905,N15906,N15907,N15908,N15909,N15910,N15911,N15912,N15913,N15914,N15915,N15916,N15917,N15918,N15919,N15920,  \\note fields as indicated
  N15921,N15922,N15923,N15924,N15925,N15926,N15927,N15928,N15929,N15930,N15931,N15932,N15933,N15934,N15935,N15936,N15937,N15938,N15939,N15940,  \\note fields as indicated
  N15941,N15942,N15943,N15944,N15945,N15946,N15947,N15948,N15949,N15950,N15951,N15952,N15953,N15954,N15955,N15956,N15957,N15958,N15959,N15960,  \\note fields as indicated
  N15961,N15962,N15963,N15964,N15965,N15966,N15967,N15968,N15969,N15970,N15971,N15972,N15973,N15974,N15975,N15976,N15977,N15978,N15979,N15980,  \\note fields as indicated
  N15981,N15982,N15983,N15984,N15985,N15986,N15987,N15988,N15989,N15990,N15991,N15992,N15993,N15994,N15995,N15996,N15997,N15998,N15999,N16000,  \\note fields as indicated
  N16001,N16002,N16003,N16004,N16005,N16006,N16007,N16008,N16009,N16010,N16011,N16012,N16013,N16014,N16015,N16016,N16017,N16018,N16019,N16020,  \\note fields as indicated
  N16021,N16022,N16023,N16024,N16025,N16026,N16027,N16028,N16029,N16030,N16031,N16032,N16033,N16034,N16035,N16036,N16037,N16038,N16039,N16040,  \\note fields as indicated
  N16041,N16042,N16043,N16044,N16045,N16046,N16047,N16048,N16049,N16050,N16051,N16052,N16053,N16054,N16055,N16056,N16057,N16058,N16059,N16060,  \\note fields as indicated
  N16061,N16062,N16063,N16064,N16065,N16066,N16067,N16068,N16069,N16070,N16071,N16072,N16073,N16074,N16075,N16076,N16077,N16078,N16079,N16080,  \\note fields as indicated
  N16081,N16082,N16083,N16084,N16085,N16086,N16087,N16088,N16089,N16090,N16091,N16092,N16093,N16094,N16095,N16096,N16097,N16098,N16099,N16100,  \\note fields as indicated
  N16101,N16102,N16103,N16104,N16105,N16106,N16107,N16108,N16109,N16110,N16111,N16112,N16113,N16114,N16115,N16116,N16117,N16118,N16119,N16120,  \\note fields as indicated
  N16121,N16122,N16123,N16124,N16125,N16126,N16127,N16128,N16129,N16130,N16131,N16132,N16133,N16134,N16135,N16136,N16137,N16138,N16139,N16140,  \\note fields as indicated
  N16141,N16142,N16143,N16144,N16145,N16146,N16147,N16148,N16149,N16150,N16151,N16152,N16153,N16154,N16155,N16156,N16157,N16158,N16159,N16160,  \\note fields as indicated
  N16161,N16162,N16163,N16164,N16165,N16166,N16167,N16168,N16169,N16170,N16171,N16172,N16173,N16174,N16175,N16176,N16177,N16178,N16179,N16180,  \\note fields as indicated
  N16181,N16182,N16183,N16184,N16185,N16186,N16187,N16188,N16189,N16190,N16191,N16192,N16193,N16194,N16195,N16196,N16197,N16198,N16199,N16200,  \\note fields as indicated
  N16201,N16202,N16203,N16204,N16205,N16206,N16207,N16208,N16209,N16210,N16211,N16212,N16213,N16214,N16215,N16216,N16217,N16218,N16219,N16220,  \\note fields as indicated
  N16221,N16222,N16223,N16224,N16225,N16226,N16227,N16228,N16229,N16230,N16231,N16232,N16233,N16234,N16235,N16236,N16237,N16238,N16239,N16240,  \\note fields as indicated
  N16241,N16242,N16243,N16244,N16245,N16246,N16247,N16248,N16249,N16250,N16251,N16252,N16253,N16254,N16255,N16256,N16257,N16258,N16259,N16260,  \\note fields as indicated
  N16261,N16262,N16263,N16264,N16265,N16266,N16267,N16268,N16269,N16270,N16271,N16272,N16273,N16274,N16275,N16276,N16277,N16278,N16279,N16280,  \\note fields as indicated
  N16281,N16282,N16283,N16284,N16285,N16286,N16287,N16288,N16289,N16290,N16291,N16292,N16293,N16294,N16295,N16296,N16297,N16298,N16299,N16300,  \\note fields as indicated
  N16301,N16302,N16303,N16304,N16305,N16306,N16307,N16308,N16309,N16310,N16311,N16312,N16313,N16314,N16315,N16316,N16317,N16318,N16319,N16320,  \\note fields as indicated
  N16321,N16322,N16323,N16324,N16325,N16326,N16327,N16328,N16329,N16330,N16331,N16332,N16333,N16334,N16335,N16336,N16337,N16338,N16339,N16340,  \\note fields as indicated
  N16341,N16342,N16343,N16344,N16345,N16346,N16347,N16348,N16349,N16350,N16351,N16352,N16353,N16354,N16355,N16356,N16357,N16358,N16359,N16360,  \\note fields as indicated
  N16361,N16362,N16363,N16364,N16365,N16366,N16367,N16368,N16369,N16370,N16371,N16372,N16373,N16374,N16375,N16376,N16377,N16378,N16379,N16380,  \\note fields as indicated
  N16381,N16382,N16383,N16384,N16385,N16386,N16387,N16388,N16389,N16390,N16391,N16392,N16393,N16394,N16395,N16396,N16397,N16398,N16399,N16400,  \\note fields as indicated
  N16401,N16402,N16403,N16404,N16405,N16406,N16407,N16408,N16409,N16410,N16411,N16412,N16413,N16414,N16415,N16416,N16417,N16418,N16419,N16420,  \\note fields as indicated
  N16421,N16422,N16423,N16424,N16425,N16426,N16427,N16428,N16429,N16430,N16431,N16432,N16433,N16434,N16435,N16436,N16437,N16438,N16439,N16440,  \\note fields as indicated
  N16441,N16442,N16443,N16444,N16445,N16446,N16447,N16448,N16449,N16450,N16451,N16452,N16453,N16454,N16455,N16456,N16457,N16458,N16459,N16460,  \\note fields as indicated
  N16461,N16462,N16463,N16464,N16465,N16466,N16467,N16468,N16469,N16470,N16471,N16472,N16473,N16474,N16475,N16476,N16477,N16478,N16479,N16480,  \\note fields as indicated
  N16481,N16482,N16483,N16484,N16485,N16486,N16487,N16488,N16489,N16490,N16491,N16492,N16493,N16494,N16495,N16496,N16497,N16498,N16499,N16500,  \\note fields as indicated
  N16501,N16502,N16503,N16504,N16505,N16506,N16507,N16508,N16509,N16510,N16511,N16512,N16513,N16514,N16515,N16516,N16517,N16518,N16519,N16520,  \\note fields as indicated
  N16521,N16522,N16523,N16524,N16525,N16526,N16527,N16528,N16529,N16530,N16531,N16532,N16533,N16534,N16535,N16536,N16537,N16538,N16539,N16540,  \\note fields as indicated
  N16541,N16542,N16543,N16544,N16545,N16546,N16547,N16548,N16549,N16550,N16551,N16552,N16553,N16554,N16555,N16556,N16557,N16558,N16559,N16560,  \\note fields as indicated
  N16561,N16562,N16563,N16564,N16565,N16566,N16567,N16568,N16569,N16570,N16571,N16572,N16573,N16574,N16575,N16576,N16577,N16578,N16579,N16580,  \\note fields as indicated
  N16581,N16582,N16583,N16584,N16585,N16586,N16587,N16588,N16589,N16590,N16591,N16592,N16593,N16594,N16595,N16596,N16597,N16598,N16599,N16600,  \\note fields as indicated
  N16601,N16602,N16603,N16604,N16605,N16606,N16607,N16608,N16609,N16610,N16611,N16612,N16613,N16614,N16615,N16616,N16617,N16618,N16619,N16620,  \\note fields as indicated
  N16621,N16622,N16623,N16624,N16625,N16626,N16627,N16628,N16629,N16630,N16631,N16632,N16633,N16634,N16635,N16636,N16637,N16638,N16639,N16640,  \\note fields as indicated
  N16641,N16642,N16643,N16644,N16645,N16646,N16647,N16648,N16649,N16650,N16651,N16652,N16653,N16654,N16655,N16656,N16657,N16658,N16659,N16660,  \\note fields as indicated
  N16661,N16662,N16663,N16664,N16665,N16666,N16667,N16668,N16669,N16670,N16671,N16672,N16673,N16674,N16675,N16676,N16677,N16678,N16679,N16680,  \\note fields as indicated
  N16681,N16682,N16683,N16684,N16685,N16686,N16687,N16688,N16689,N16690,N16691,N16692,N16693,N16694,N16695,N16696,N16697,N16698,N16699,N16700,  \\note fields as indicated
  N16701,N16702,N16703,N16704,N16705,N16706,N16707,N16708,N16709,N16710,N16711,N16712,N16713,N16714,N16715,N16716,N16717,N16718,N16719,N16720,  \\note fields as indicated
  N16721,N16722,N16723,N16724,N16725,N16726,N16727,N16728,N16729,N16730,N16731,N16732,N16733,N16734,N16735,N16736,N16737,N16738,N16739,N16740,  \\note fields as indicated
  N16741,N16742,N16743,N16744,N16745,N16746,N16747,N16748,N16749,N16750,N16751,N16752,N16753,N16754,N16755,N16756,N16757,N16758,N16759,N16760,  \\note fields as indicated
  N16761,N16762,N16763,N16764,N16765,N16766,N16767,N16768,N16769,N16770,N16771,N16772,N16773,N16774,N16775,N16776,N16777,N16778,N16779,N16780,  \\note fields as indicated
  N16781,N16782,N16783,N16784,N16785,N16786,N16787,N16788,N16789,N16790,N16791,N16792,N16793,N16794,N16795,N16796,N16797,N16798,N16799,N16800,  \\note fields as indicated
  N16801,N16802,N16803,N16804,N16805,N16806,N16807,N16808,N16809,N16810,N16811,N16812,N16813,N16814,N16815,N16816,N16817,N16818,N16819,N16820,  \\note fields as indicated
  N16821,N16822,N16823,N16824,N16825,N16826,N16827,N16828,N16829,N16830,N16831,N16832,N16833,N16834,N16835,N16836,N16837,N16838,N16839,N16840,  \\note fields as indicated
  N16841,N16842,N16843,N16844,N16845,N16846,N16847,N16848,N16849,N16850,N16851,N16852,N16853,N16854,N16855,N16856,N16857,N16858,N16859,N16860,  \\note fields as indicated
  N16861,N16862,N16863,N16864,N16865,N16866,N16867,N16868,N16869,N16870,N16871,N16872,N16873,N16874,N16875,N16876,N16877,N16878,N16879,N16880,  \\note fields as indicated
  N16881,N16882,N16883,N16884,N16885,N16886,N16887,N16888,N16889,N16890,N16891,N16892,N16893,N16894,N16895,N16896,N16897,N16898,N16899,N16900,  \\note fields as indicated
  N16901,N16902,N16903,N16904,N16905,N16906,N16907,N16908,N16909,N16910,N16911,N16912,N16913,N16914,N16915,N16916,N16917,N16918,N16919,N16920,  \\note fields as indicated
  N16921,N16922,N16923,N16924,N16925,N16926,N16927,N16928,N16929,N16930,N16931,N16932,N16933,N16934,N16935,N16936,N16937,N16938,N16939,N16940,  \\note fields as indicated
  N16941,N16942,N16943,N16944,N16945,N16946,N16947,N16948,N16949,N16950,N16951,N16952,N16953,N16954,N16955,N16956,N16957,N16958,N16959,N16960,  \\note fields as indicated
  N16961,N16962,N16963,N16964,N16965,N16966,N16967,N16968,N16969,N16970,N16971,N16972,N16973,N16974,N16975,N16976,N16977,N16978,N16979,N16980,  \\note fields as indicated
  N16981,N16982,N16983,N16984,N16985,N16986,N16987,N16988,N16989,N16990,N16991,N16992,N16993,N16994,N16995,N16996,N16997,N16998,N16999,N17000,  \\note fields as indicated
  N17001,N17002,N17003,N17004,N17005,N17006,N17007,N17008,N17009,N17010,N17011,N17012,N17013,N17014,N17015,N17016,N17017,N17018,N17019,N17020,  \\note fields as indicated
  N17021,N17022,N17023,N17024,N17025,N17026,N17027,N17028,N17029,N17030,N17031,N17032,N17033,N17034,N17035,N17036,N17037,N17038,N17039,N17040,  \\note fields as indicated
  N17041,N17042,N17043,N17044,N17045,N17046,N17047,N17048,N17049,N17050,N17051,N17052,N17053,N17054,N17055,N17056,N17057,N17058,N17059,N17060,  \\note fields as indicated
  N17061,N17062,N17063,N17064,N17065,N17066,N17067,N17068,N17069,N17070,N17071,N17072,N17073,N17074,N17075,N17076,N17077,N17078,N17079,N17080,  \\note fields as indicated
  N17081,N17082,N17083,N17084,N17085,N17086,N17087,N17088,N17089,N17090,N17091,N17092,N17093,N17094,N17095,N17096,N17097,N17098,N17099,N17100,  \\note fields as indicated
  N17101,N17102,N17103,N17104,N17105,N17106,N17107,N17108,N17109,N17110,N17111,N17112,N17113,N17114,N17115,N17116,N17117,N17118,N17119,N17120,  \\note fields as indicated
  N17121,N17122,N17123,N17124,N17125,N17126,N17127,N17128,N17129,N17130,N17131,N17132,N17133,N17134,N17135,N17136,N17137,N17138,N17139,N17140,  \\note fields as indicated
  N17141,N17142,N17143,N17144,N17145,N17146,N17147,N17148,N17149,N17150,N17151,N17152,N17153,N17154,N17155,N17156,N17157,N17158,N17159,N17160,  \\note fields as indicated
  N17161,N17162,N17163,N17164,N17165,N17166,N17167,N17168,N17169,N17170,N17171,N17172,N17173,N17174,N17175,N17176,N17177,N17178,N17179,N17180,  \\note fields as indicated
  N17181,N17182,N17183,N17184,N17185,N17186,N17187,N17188,N17189,N17190,N17191,N17192,N17193,N17194,N17195,N17196,N17197,N17198,N17199,N17200,  \\note fields as indicated
  N17201,N17202,N17203,N17204,N17205,N17206,N17207,N17208,N17209,N17210,N17211,N17212,N17213,N17214,N17215,N17216,N17217,N17218,N17219,N17220,  \\note fields as indicated
  N17221,N17222,N17223,N17224,N17225,N17226,N17227,N17228,N17229,N17230,N17231,N17232,N17233,N17234,N17235,N17236,N17237,N17238,N17239,N17240,  \\note fields as indicated
  N17241,N17242,N17243,N17244,N17245,N17246,N17247,N17248,N17249,N17250,N17251,N17252,N17253,N17254,N17255,N17256,N17257,N17258,N17259,N17260,  \\note fields as indicated
  N17261,N17262,N17263,N17264,N17265,N17266,N17267,N17268,N17269,N17270,N17271,N17272,N17273,N17274,N17275,N17276,N17277,N17278,N17279,N17280,  \\note fields as indicated
  N17281,N17282,N17283,N17284,N17285,N17286,N17287,N17288,N17289,N17290,N17291,N17292,N17293,N17294,N17295,N17296,N17297,N17298,N17299,N17300,  \\note fields as indicated
  N17301,N17302,N17303,N17304,N17305,N17306,N17307,N17308,N17309,N17310,N17311,N17312,N17313,N17314,N17315,N17316,N17317,N17318,N17319,N17320,  \\note fields as indicated
  N17321,N17322,N17323,N17324,N17325,N17326,N17327,N17328,N17329,N17330,N17331,N17332,N17333,N17334,N17335,N17336,N17337,N17338,N17339,N17340,  \\note fields as indicated
  N17341,N17342,N17343,N17344,N17345,N17346,N17347,N17348,N17349,N17350,N17351,N17352,N17353,N17354,N17355,N17356,N17357,N17358,N17359,N17360,  \\note fields as indicated
  N17361,N17362,N17363,N17364,N17365,N17366,N17367,N17368,N17369,N17370,N17371,N17372,N17373,N17374,N17375,N17376,N17377,N17378,N17379,N17380,  \\note fields as indicated
  N17381,N17382,N17383,N17384,N17385,N17386,N17387,N17388,N17389,N17390,N17391,N17392,N17393,N17394,N17395,N17396,N17397,N17398,N17399,N17400,  \\note fields as indicated
  N17401,N17402,N17403,N17404,N17405,N17406,N17407,N17408,N17409,N17410,N17411,N17412,N17413,N17414,N17415,N17416,N17417,N17418,N17419,N17420,  \\note fields as indicated
  N17421,N17422,N17423,N17424,N17425,N17426,N17427,N17428,N17429,N17430,N17431,N17432,N17433,N17434,N17435,N17436,N17437,N17438,N17439,N17440,  \\note fields as indicated
  N17441,N17442,N17443,N17444,N17445,N17446,N17447,N17448,N17449,N17450,N17451,N17452,N17453,N17454,N17455,N17456,N17457,N17458,N17459,N17460,  \\note fields as indicated
  N17461,N17462,N17463,N17464,N17465,N17466,N17467,N17468,N17469,N17470,N17471,N17472,N17473,N17474,N17475,N17476,N17477,N17478,N17479,N17480,  \\note fields as indicated
  N17481,N17482,N17483,N17484,N17485,N17486,N17487,N17488,N17489,N17490,N17491,N17492,N17493,N17494,N17495,N17496,N17497,N17498,N17499,N17500,  \\note fields as indicated
  N17501,N17502,N17503,N17504,N17505,N17506,N17507,N17508,N17509,N17510,N17511,N17512,N17513,N17514,N17515,N17516,N17517,N17518,N17519,N17520,  \\note fields as indicated
  N17521,N17522,N17523,N17524,N17525,N17526,N17527,N17528,N17529,N17530,N17531,N17532,N17533,N17534,N17535,N17536,N17537,N17538,N17539,N17540,  \\note fields as indicated
  N17541,N17542,N17543,N17544,N17545,N17546,N17547,N17548,N17549,N17550,N17551,N17552,N17553,N17554,N17555,N17556,N17557,N17558,N17559,N17560,  \\note fields as indicated
  N17561,N17562,N17563,N17564,N17565,N17566,N17567,N17568,N17569,N17570,N17571,N17572,N17573,N17574,N17575,N17576,N17577,N17578,N17579,N17580,  \\note fields as indicated
  N17581,N17582,N17583,N17584,N17585,N17586,N17587,N17588,N17589,N17590,N17591,N17592,N17593,N17594,N17595,N17596,N17597,N17598,N17599,N17600,  \\note fields as indicated
  N17601,N17602,N17603,N17604,N17605,N17606,N17607,N17608,N17609,N17610,N17611,N17612,N17613,N17614,N17615,N17616,N17617,N17618,N17619,N17620,  \\note fields as indicated
  N17621,N17622,N17623,N17624,N17625,N17626,N17627,N17628,N17629,N17630,N17631,N17632,N17633,N17634,N17635,N17636,N17637,N17638,N17639,N17640,  \\note fields as indicated
  N17641,N17642,N17643,N17644,N17645,N17646,N17647,N17648,N17649,N17650,N17651,N17652,N17653,N17654,N17655,N17656,N17657,N17658,N17659,N17660,  \\note fields as indicated
  N17661,N17662,N17663,N17664,N17665,N17666,N17667,N17668,N17669,N17670,N17671,N17672,N17673,N17674,N17675,N17676,N17677,N17678,N17679,N17680,  \\note fields as indicated
  N17681,N17682,N17683,N17684,N17685,N17686,N17687,N17688,N17689,N17690,N17691,N17692,N17693,N17694,N17695,N17696,N17697,N17698,N17699,N17700,  \\note fields as indicated
  N17701,N17702,N17703,N17704,N17705,N17706,N17707,N17708,N17709,N17710,N17711,N17712,N17713,N17714,N17715,N17716,N17717,N17718,N17719,N17720,  \\note fields as indicated
  N17721,N17722,N17723,N17724,N17725,N17726,N17727,N17728,N17729,N17730,N17731,N17732,N17733,N17734,N17735,N17736,N17737,N17738,N17739,N17740,  \\note fields as indicated
  N17741,N17742,N17743,N17744,N17745,N17746,N17747,N17748,N17749,N17750,N17751,N17752,N17753,N17754,N17755,N17756,N17757,N17758,N17759,N17760,  \\note fields as indicated
  N17761,N17762,N17763,N17764,N17765,N17766,N17767,N17768,N17769,N17770,N17771,N17772,N17773,N17774,N17775,N17776,N17777,N17778,N17779,N17780,  \\note fields as indicated
  N17781,N17782,N17783,N17784,N17785,N17786,N17787,N17788,N17789,N17790,N17791,N17792,N17793,N17794,N17795,N17796,N17797,N17798,N17799,N17800,  \\note fields as indicated
  N17801,N17802,N17803,N17804,N17805,N17806,N17807,N17808,N17809,N17810,N17811,N17812,N17813,N17814,N17815,N17816,N17817,N17818,N17819,N17820,  \\note fields as indicated
  N17821,N17822,N17823,N17824,N17825,N17826,N17827,N17828,N17829,N17830,N17831,N17832,N17833,N17834,N17835,N17836,N17837,N17838,N17839,N17840,  \\note fields as indicated
  N17841,N17842,N17843,N17844,N17845,N17846,N17847,N17848,N17849,N17850,N17851,N17852,N17853,N17854,N17855,N17856,N17857,N17858,N17859,N17860,  \\note fields as indicated
  N17861,N17862,N17863,N17864,N17865,N17866,N17867,N17868,N17869,N17870,N17871,N17872,N17873,N17874,N17875,N17876,N17877,N17878,N17879,N17880,  \\note fields as indicated
  N17881,N17882,N17883,N17884,N17885,N17886,N17887,N17888,N17889,N17890,N17891,N17892,N17893,N17894,N17895,N17896,N17897,N17898,N17899,N17900,  \\note fields as indicated
  N17901,N17902,N17903,N17904,N17905,N17906,N17907,N17908,N17909,N17910,N17911,N17912,N17913,N17914,N17915,N17916,N17917,N17918,N17919,N17920,  \\note fields as indicated
  N17921,N17922,N17923,N17924,N17925,N17926,N17927,N17928,N17929,N17930,N17931,N17932,N17933,N17934,N17935,N17936,N17937,N17938,N17939,N17940,  \\note fields as indicated
  N17941,N17942,N17943,N17944,N17945,N17946,N17947,N17948,N17949,N17950,N17951,N17952,N17953,N17954,N17955,N17956,N17957,N17958,N17959,N17960,  \\note fields as indicated
  N17961,N17962,N17963,N17964,N17965,N17966,N17967,N17968,N17969,N17970,N17971,N17972,N17973,N17974,N17975,N17976,N17977,N17978,N17979,N17980,  \\note fields as indicated
  N17981,N17982,N17983,N17984,N17985,N17986,N17987,N17988,N17989,N17990,N17991,N17992,N17993,N17994,N17995,N17996,N17997,N17998,N17999,N18000,  \\note fields as indicated
  N18001,N18002,N18003,N18004,N18005,N18006,N18007,N18008,N18009,N18010,N18011,N18012,N18013,N18014,N18015,N18016,N18017,N18018,N18019,N18020,  \\note fields as indicated
  N18021,N18022,N18023,N18024,N18025,N18026,N18027,N18028,N18029,N18030,N18031,N18032,N18033,N18034,N18035,N18036,N18037,N18038,N18039,N18040,  \\note fields as indicated
  N18041,N18042,N18043,N18044,N18045,N18046,N18047,N18048,N18049,N18050,N18051,N18052,N18053,N18054,N18055,N18056,N18057,N18058,N18059,N18060,  \\note fields as indicated
  N18061,N18062,N18063,N18064,N18065,N18066,N18067,N18068,N18069,N18070,N18071,N18072,N18073,N18074,N18075,N18076,N18077,N18078,N18079,N18080,  \\note fields as indicated
  N18081,N18082,N18083,N18084,N18085,N18086,N18087,N18088,N18089,N18090,N18091,N18092,N18093,N18094,N18095,N18096,N18097,N18098,N18099,N18100,  \\note fields as indicated
  N18101,N18102,N18103,N18104,N18105,N18106,N18107,N18108,N18109,N18110,N18111,N18112,N18113,N18114,N18115,N18116,N18117,N18118,N18119,N18120,  \\note fields as indicated
  N18121,N18122,N18123,N18124,N18125,N18126,N18127,N18128,N18129,N18130,N18131,N18132,N18133,N18134,N18135,N18136,N18137,N18138,N18139,N18140,  \\note fields as indicated
  N18141,N18142,N18143,N18144,N18145,N18146,N18147,N18148,N18149,N18150,N18151,N18152,N18153,N18154,N18155,N18156,N18157,N18158,N18159,N18160,  \\note fields as indicated
  N18161,N18162,N18163,N18164,N18165,N18166,N18167,N18168,N18169,N18170,N18171,N18172,N18173,N18174,N18175,N18176,N18177,N18178,N18179,N18180,  \\note fields as indicated
  N18181,N18182,N18183,N18184,N18185,N18186,N18187,N18188,N18189,N18190,N18191,N18192,N18193,N18194,N18195,N18196,N18197,N18198,N18199,N18200,  \\note fields as indicated
  N18201,N18202,N18203,N18204,N18205,N18206,N18207,N18208,N18209,N18210,N18211,N18212,N18213,N18214,N18215,N18216,N18217,N18218,N18219,N18220,  \\note fields as indicated
  N18221,N18222,N18223,N18224,N18225,N18226,N18227,N18228,N18229,N18230,N18231,N18232,N18233,N18234,N18235,N18236,N18237,N18238,N18239,N18240,  \\note fields as indicated
  N18241,N18242,N18243,N18244,N18245,N18246,N18247,N18248,N18249,N18250,N18251,N18252,N18253,N18254,N18255,N18256,N18257,N18258,N18259,N18260,  \\note fields as indicated
  N18261,N18262,N18263,N18264,N18265,N18266,N18267,N18268,N18269,N18270,N18271,N18272,N18273,N18274,N18275,N18276,N18277,N18278,N18279,N18280,  \\note fields as indicated
  N18281,N18282,N18283,N18284,N18285,N18286,N18287,N18288,N18289,N18290,N18291,N18292,N18293,N18294,N18295,N18296,N18297,N18298,N18299,N18300,  \\note fields as indicated
  N18301,N18302,N18303,N18304,N18305,N18306,N18307,N18308,N18309,N18310,N18311,N18312,N18313,N18314,N18315,N18316,N18317,N18318,N18319,N18320,  \\note fields as indicated
  N18321,N18322,N18323,N18324,N18325,N18326,N18327,N18328,N18329,N18330,N18331,N18332,N18333,N18334,N18335,N18336,N18337,N18338,N18339,N18340,  \\note fields as indicated
  N18341,N18342,N18343,N18344,N18345,N18346,N18347,N18348,N18349,N18350,N18351,N18352,N18353,N18354,N18355,N18356,N18357,N18358,N18359,N18360,  \\note fields as indicated
  N18361,N18362,N18363,N18364,N18365,N18366,N18367,N18368,N18369,N18370,N18371,N18372,N18373,N18374,N18375,N18376,N18377,N18378,N18379,N18380,  \\note fields as indicated
  N18381,N18382,N18383,N18384,N18385,N18386,N18387,N18388,N18389,N18390,N18391,N18392,N18393,N18394,N18395,N18396,N18397,N18398,N18399,N18400,  \\note fields as indicated
  N18401,N18402,N18403,N18404,N18405,N18406,N18407,N18408,N18409,N18410,N18411,N18412,N18413,N18414,N18415,N18416,N18417,N18418,N18419,N18420,  \\note fields as indicated
  N18421,N18422,N18423,N18424,N18425,N18426,N18427,N18428,N18429,N18430,N18431,N18432,N18433,N18434,N18435,N18436,N18437,N18438,N18439,N18440,  \\note fields as indicated
  N18441,N18442,N18443,N18444,N18445,N18446,N18447,N18448,N18449,N18450,N18451,N18452,N18453,N18454,N18455,N18456,N18457,N18458,N18459,N18460,  \\note fields as indicated
  N18461,N18462,N18463,N18464,N18465,N18466,N18467,N18468,N18469,N18470,N18471,N18472,N18473,N18474,N18475,N18476,N18477,N18478,N18479,N18480,  \\note fields as indicated
  N18481,N18482,N18483,N18484,N18485,N18486,N18487,N18488,N18489,N18490,N18491,N18492,N18493,N18494,N18495,N18496,N18497,N18498,N18499,N18500,  \\note fields as indicated
  N18501,N18502,N18503,N18504,N18505,N18506,N18507,N18508,N18509,N18510,N18511,N18512,N18513,N18514,N18515,N18516,N18517,N18518,N18519,N18520,  \\note fields as indicated
  N18521,N18522,N18523,N18524,N18525,N18526,N18527,N18528,N18529,N18530,N18531,N18532,N18533,N18534,N18535,N18536,N18537,N18538,N18539,N18540,  \\note fields as indicated
  N18541,N18542,N18543,N18544,N18545,N18546,N18547,N18548,N18549,N18550,N18551,N18552,N18553,N18554,N18555,N18556,N18557,N18558,N18559,N18560,  \\note fields as indicated
  N18561,N18562,N18563,N18564,N18565,N18566,N18567,N18568,N18569,N18570,N18571,N18572,N18573,N18574,N18575,N18576,N18577,N18578,N18579,N18580,  \\note fields as indicated
  N18581,N18582,N18583,N18584,N18585,N18586,N18587,N18588,N18589,N18590,N18591,N18592,N18593,N18594,N18595,N18596,N18597,N18598,N18599,N18600,  \\note fields as indicated
  N18601,N18602,N18603,N18604,N18605,N18606,N18607,N18608,N18609,N18610,N18611,N18612,N18613,N18614,N18615,N18616,N18617,N18618,N18619,N18620,  \\note fields as indicated
  N18621,N18622,N18623,N18624,N18625,N18626,N18627,N18628,N18629,N18630,N18631,N18632,N18633,N18634,N18635,N18636,N18637,N18638,N18639,N18640,  \\note fields as indicated
  N18641,N18642,N18643,N18644,N18645,N18646,N18647,N18648,N18649,N18650,N18651,N18652,N18653,N18654,N18655,N18656,N18657,N18658,N18659,N18660,  \\note fields as indicated
  N18661,N18662,N18663,N18664,N18665,N18666,N18667,N18668,N18669,N18670,N18671,N18672,N18673,N18674,N18675,N18676,N18677,N18678,N18679,N18680,  \\note fields as indicated
  N18681,N18682,N18683,N18684,N18685,N18686,N18687,N18688,N18689,N18690,N18691,N18692,N18693,N18694,N18695,N18696,N18697,N18698,N18699,N18700,  \\note fields as indicated
  N18701,N18702,N18703,N18704,N18705,N18706,N18707,N18708,N18709,N18710,N18711,N18712,N18713,N18714,N18715,N18716,N18717,N18718,N18719,N18720,  \\note fields as indicated
  N18721,N18722,N18723,N18724,N18725,N18726,N18727,N18728,N18729,N18730,N18731,N18732,N18733,N18734,N18735,N18736,N18737,N18738,N18739,N18740,  \\note fields as indicated
  N18741,N18742,N18743,N18744,N18745,N18746,N18747,N18748,N18749,N18750,N18751,N18752,N18753,N18754,N18755,N18756,N18757,N18758,N18759,N18760,  \\note fields as indicated
  N18761,N18762,N18763,N18764,N18765,N18766,N18767,N18768,N18769,N18770,N18771,N18772,N18773,N18774,N18775,N18776,N18777,N18778,N18779,N18780,  \\note fields as indicated
  N18781,N18782,N18783,N18784,N18785,N18786,N18787,N18788,N18789,N18790,N18791,N18792,N18793,N18794,N18795,N18796,N18797,N18798,N18799,N18800,  \\note fields as indicated
  N18801,N18802,N18803,N18804,N18805,N18806,N18807,N18808,N18809,N18810,N18811,N18812,N18813,N18814,N18815,N18816,N18817,N18818,N18819,N18820,  \\note fields as indicated
  N18821,N18822,N18823,N18824,N18825,N18826,N18827,N18828,N18829,N18830,N18831,N18832,N18833,N18834,N18835,N18836,N18837,N18838,N18839,N18840,  \\note fields as indicated
  N18841,N18842,N18843,N18844,N18845,N18846,N18847,N18848,N18849,N18850,N18851,N18852,N18853,N18854,N18855,N18856,N18857,N18858,N18859,N18860,  \\note fields as indicated
  N18861,N18862,N18863,N18864,N18865,N18866,N18867,N18868,N18869,N18870,N18871,N18872,N18873,N18874,N18875,N18876,N18877,N18878,N18879,N18880,  \\note fields as indicated
  N18881,N18882,N18883,N18884,N18885,N18886,N18887,N18888,N18889,N18890,N18891,N18892,N18893,N18894,N18895,N18896,N18897,N18898,N18899,N18900,  \\note fields as indicated
  N18901,N18902,N18903,N18904,N18905,N18906,N18907,N18908,N18909,N18910,N18911,N18912,N18913,N18914,N18915,N18916,N18917,N18918,N18919,N18920,  \\note fields as indicated
  N18921,N18922,N18923,N18924,N18925,N18926,N18927,N18928,N18929,N18930,N18931,N18932,N18933,N18934,N18935,N18936,N18937,N18938,N18939,N18940,  \\note fields as indicated
  N18941,N18942,N18943,N18944,N18945,N18946,N18947,N18948,N18949,N18950,N18951,N18952,N18953,N18954,N18955,N18956,N18957,N18958,N18959,N18960,  \\note fields as indicated
  N18961,N18962,N18963,N18964,N18965,N18966,N18967,N18968,N18969,N18970,N18971,N18972,N18973,N18974,N18975,N18976,N18977,N18978,N18979,N18980,  \\note fields as indicated
  N18981,N18982,N18983,N18984,N18985,N18986,N18987,N18988,N18989,N18990,N18991,N18992,N18993,N18994,N18995,N18996,N18997,N18998,N18999,N19000,  \\note fields as indicated
  N19001,N19002,N19003,N19004,N19005,N19006,N19007,N19008,N19009,N19010,N19011,N19012,N19013,N19014,N19015,N19016,N19017,N19018,N19019,N19020,  \\note fields as indicated
  N19021,N19022,N19023,N19024,N19025,N19026,N19027,N19028,N19029,N19030,N19031,N19032,N19033,N19034,N19035,N19036,N19037,N19038,N19039,N19040,  \\note fields as indicated
  N19041,N19042,N19043,N19044,N19045,N19046,N19047,N19048,N19049,N19050,N19051,N19052,N19053,N19054,N19055,N19056,N19057,N19058,N19059,N19060,  \\note fields as indicated
  N19061,N19062,N19063,N19064,N19065,N19066,N19067,N19068,N19069,N19070,N19071,N19072,N19073,N19074,N19075,N19076,N19077,N19078,N19079,N19080,  \\note fields as indicated
  N19081,N19082,N19083,N19084,N19085,N19086,N19087,N19088,N19089,N19090,N19091,N19092,N19093,N19094,N19095,N19096,N19097,N19098,N19099,N19100,  \\note fields as indicated
  N19101,N19102,N19103,N19104,N19105,N19106,N19107,N19108,N19109,N19110,N19111,N19112,N19113,N19114,N19115,N19116,N19117,N19118,N19119,N19120,  \\note fields as indicated
  N19121,N19122,N19123,N19124,N19125,N19126,N19127,N19128,N19129,N19130,N19131,N19132,N19133,N19134,N19135,N19136,N19137,N19138,N19139,N19140,  \\note fields as indicated
  N19141,N19142,N19143,N19144,N19145,N19146,N19147,N19148,N19149,N19150,N19151,N19152,N19153,N19154,N19155,N19156,N19157,N19158,N19159,N19160,  \\note fields as indicated
  N19161,N19162,N19163,N19164,N19165,N19166,N19167,N19168,N19169,N19170,N19171,N19172,N19173,N19174,N19175,N19176,N19177,N19178,N19179,N19180,  \\note fields as indicated
  N19181,N19182,N19183,N19184,N19185,N19186,N19187,N19188,N19189,N19190,N19191,N19192,N19193,N19194,N19195,N19196,N19197,N19198,N19199,N19200,  \\note fields as indicated
  N19201,N19202,N19203,N19204,N19205,N19206,N19207,N19208,N19209,N19210,N19211,N19212,N19213,N19214,N19215,N19216,N19217,N19218,N19219,N19220,  \\note fields as indicated
  N19221,N19222,N19223,N19224,N19225,N19226,N19227,N19228,N19229,N19230,N19231,N19232,N19233,N19234,N19235,N19236,N19237,N19238,N19239,N19240,  \\note fields as indicated
  N19241,N19242,N19243,N19244,N19245,N19246,N19247,N19248,N19249,N19250,N19251,N19252,N19253,N19254,N19255,N19256,N19257,N19258,N19259,N19260,  \\note fields as indicated
  N19261,N19262,N19263,N19264,N19265,N19266,N19267,N19268,N19269,N19270,N19271,N19272,N19273,N19274,N19275,N19276,N19277,N19278,N19279,N19280,  \\note fields as indicated
  N19281,N19282,N19283,N19284,N19285,N19286,N19287,N19288,N19289,N19290,N19291,N19292,N19293,N19294,N19295,N19296,N19297,N19298,N19299,N19300,  \\note fields as indicated
  N19301,N19302,N19303,N19304,N19305,N19306,N19307,N19308,N19309,N19310,N19311,N19312,N19313,N19314,N19315,N19316,N19317,N19318,N19319,N19320,  \\note fields as indicated
  N19321,N19322,N19323,N19324,N19325,N19326,N19327,N19328,N19329,N19330,N19331,N19332,N19333,N19334,N19335,N19336,N19337,N19338,N19339,N19340,  \\note fields as indicated
  N19341,N19342,N19343,N19344,N19345,N19346,N19347,N19348,N19349,N19350,N19351,N19352,N19353,N19354,N19355,N19356,N19357,N19358,N19359,N19360,  \\note fields as indicated
  N19361,N19362,N19363,N19364,N19365,N19366,N19367,N19368,N19369,N19370,N19371,N19372,N19373,N19374,N19375,N19376,N19377,N19378,N19379,N19380,  \\note fields as indicated
  N19381,N19382,N19383,N19384,N19385,N19386,N19387,N19388,N19389,N19390,N19391,N19392,N19393,N19394,N19395,N19396,N19397,N19398,N19399,N19400,  \\note fields as indicated
  N19401,N19402,N19403,N19404,N19405,N19406,N19407,N19408,N19409,N19410,N19411,N19412,N19413,N19414,N19415,N19416,N19417,N19418,N19419,N19420,  \\note fields as indicated
  N19421,N19422,N19423,N19424,N19425,N19426,N19427,N19428,N19429,N19430,N19431,N19432,N19433,N19434,N19435,N19436,N19437,N19438,N19439,N19440,  \\note fields as indicated
  N19441,N19442,N19443,N19444,N19445,N19446,N19447,N19448,N19449,N19450,N19451,N19452,N19453,N19454,N19455,N19456,N19457,N19458,N19459,N19460,  \\note fields as indicated
  N19461,N19462,N19463,N19464,N19465,N19466,N19467,N19468,N19469,N19470,N19471,N19472,N19473,N19474,N19475,N19476,N19477,N19478,N19479,N19480,  \\note fields as indicated
  N19481,N19482,N19483,N19484,N19485,N19486,N19487,N19488,N19489,N19490,N19491,N19492,N19493,N19494,N19495,N19496,N19497,N19498,N19499,N19500,  \\note fields as indicated
  N19501,N19502,N19503,N19504,N19505,N19506,N19507,N19508,N19509,N19510,N19511,N19512,N19513,N19514,N19515,N19516,N19517,N19518,N19519,N19520,  \\note fields as indicated
  N19521,N19522,N19523,N19524,N19525,N19526,N19527,N19528,N19529,N19530,N19531,N19532,N19533,N19534,N19535,N19536,N19537,N19538,N19539,N19540,  \\note fields as indicated
  N19541,N19542,N19543,N19544,N19545,N19546,N19547,N19548,N19549,N19550,N19551,N19552,N19553,N19554,N19555,N19556,N19557,N19558,N19559,N19560,  \\note fields as indicated
  N19561,N19562,N19563,N19564,N19565,N19566,N19567,N19568,N19569,N19570,N19571,N19572,N19573,N19574,N19575,N19576,N19577,N19578,N19579,N19580,  \\note fields as indicated
  N19581,N19582,N19583,N19584,N19585,N19586,N19587,N19588,N19589,N19590,N19591,N19592,N19593,N19594,N19595,N19596,N19597,N19598,N19599,N19600,  \\note fields as indicated
  N19601,N19602,N19603,N19604,N19605,N19606,N19607,N19608,N19609,N19610,N19611,N19612,N19613,N19614,N19615,N19616,N19617,N19618,N19619,N19620,  \\note fields as indicated
  N19621,N19622,N19623,N19624,N19625,N19626,N19627,N19628,N19629,N19630,N19631,N19632,N19633,N19634,N19635,N19636,N19637,N19638,N19639,N19640,  \\note fields as indicated
  N19641,N19642,N19643,N19644,N19645,N19646,N19647,N19648,N19649,N19650,N19651,N19652,N19653,N19654,N19655,N19656,N19657,N19658,N19659,N19660,  \\note fields as indicated
  N19661,N19662,N19663,N19664,N19665,N19666,N19667,N19668,N19669,N19670,N19671,N19672,N19673,N19674,N19675,N19676,N19677,N19678,N19679,N19680,  \\note fields as indicated
  N19681,N19682,N19683,N19684,N19685,N19686,N19687,N19688,N19689,N19690,N19691,N19692,N19693,N19694,N19695,N19696,N19697,N19698,N19699,N19700,  \\note fields as indicated
  N19701,N19702,N19703,N19704,N19705,N19706,N19707,N19708,N19709,N19710,N19711,N19712,N19713,N19714,N19715,N19716,N19717,N19718,N19719,N19720,  \\note fields as indicated
  N19721,N19722,N19723,N19724,N19725,N19726,N19727,N19728,N19729,N19730,N19731,N19732,N19733,N19734,N19735,N19736,N19737,N19738,N19739,N19740,  \\note fields as indicated
  N19741,N19742,N19743,N19744,N19745,N19746,N19747,N19748,N19749,N19750,N19751,N19752,N19753,N19754,N19755,N19756,N19757,N19758,N19759,N19760,  \\note fields as indicated
  N19761,N19762,N19763,N19764,N19765,N19766,N19767,N19768,N19769,N19770,N19771,N19772,N19773,N19774,N19775,N19776,N19777,N19778,N19779,N19780,  \\note fields as indicated
  N19781,N19782,N19783,N19784,N19785,N19786,N19787,N19788,N19789,N19790,N19791,N19792,N19793,N19794,N19795,N19796,N19797,N19798,N19799,N19800,  \\note fields as indicated
  N19801,N19802,N19803,N19804,N19805,N19806,N19807,N19808,N19809,N19810,N19811,N19812,N19813,N19814,N19815,N19816,N19817,N19818,N19819,N19820,  \\note fields as indicated
  N19821,N19822,N19823,N19824,N19825,N19826,N19827,N19828,N19829,N19830,N19831,N19832,N19833,N19834,N19835,N19836,N19837,N19838,N19839,N19840,  \\note fields as indicated
  N19841,N19842,N19843,N19844,N19845,N19846,N19847,N19848,N19849,N19850,N19851,N19852,N19853,N19854,N19855,N19856,N19857,N19858,N19859,N19860,  \\note fields as indicated
  N19861,N19862,N19863,N19864,N19865,N19866,N19867,N19868,N19869,N19870,N19871,N19872,N19873,N19874,N19875,N19876,N19877,N19878,N19879,N19880,  \\note fields as indicated
  N19881,N19882,N19883,N19884,N19885,N19886,N19887,N19888,N19889,N19890,N19891,N19892,N19893,N19894,N19895,N19896,N19897,N19898,N19899,N19900,  \\note fields as indicated
  N19901,N19902,N19903,N19904,N19905,N19906,N19907,N19908,N19909,N19910,N19911,N19912,N19913,N19914,N19915,N19916,N19917,N19918,N19919,N19920,  \\note fields as indicated
  N19921,N19922,N19923,N19924,N19925,N19926,N19927,N19928,N19929,N19930,N19931,N19932,N19933,N19934,N19935,N19936,N19937,N19938,N19939,N19940,  \\note fields as indicated
  N19941,N19942,N19943,N19944,N19945,N19946,N19947,N19948,N19949,N19950,N19951,N19952,N19953,N19954,N19955,N19956,N19957,N19958,N19959,N19960,  \\note fields as indicated
  N19961,N19962,N19963,N19964,N19965,N19966,N19967,N19968,N19969,N19970,N19971,N19972,N19973,N19974,N19975,N19976,N19977,N19978,N19979,N19980,  \\note fields as indicated
  N19981,N19982,N19983,N19984,N19985,N19986,N19987,N19988,N19989,N19990,N19991,N19992,N19993,N19994,N19995,N19996,N19997,N19998,N19999,N20000,  \\note fields as indicated
  N20001,N20002,N20003,N20004,N20005,N20006,N20007,N20008,N20009,N20010,N20011,N20012,N20013,N20014,N20015,N20016,N20017,N20018,N20019,N20020,  \\note fields as indicated
  N20021,N20022,N20023,N20024,N20025,N20026,N20027,N20028,N20029,N20030,N20031,N20032,N20033,N20034,N20035,N20036,N20037,N20038,N20039,N20040,  \\note fields as indicated
  N20041,N20042,N20043,N20044,N20045,N20046,N20047,N20048,N20049,N20050,N20051,N20052,N20053,N20054,N20055,N20056,N20057,N20058,N20059,N20060,  \\note fields as indicated
  N20061,N20062,N20063,N20064,N20065,N20066,N20067,N20068,N20069,N20070,N20071,N20072,N20073,N20074,N20075,N20076,N20077,N20078,N20079,N20080,  \\note fields as indicated
  N20081,N20082,N20083,N20084,N20085,N20086,N20087,N20088,N20089,N20090,N20091,N20092,N20093,N20094,N20095,N20096,N20097,N20098,N20099,N20100,  \\note fields as indicated
  N20101,N20102,N20103,N20104,N20105,N20106,N20107,N20108,N20109,N20110,N20111,N20112,N20113,N20114,N20115,N20116,N20117,N20118,N20119,N20120,  \\note fields as indicated
  N20121,N20122,N20123,N20124,N20125,N20126,N20127,N20128,N20129,N20130,N20131,N20132,N20133,N20134,N20135,N20136,N20137,N20138,N20139,N20140,  \\note fields as indicated
  N20141,N20142,N20143,N20144,N20145,N20146,N20147,N20148,N20149,N20150,N20151,N20152,N20153,N20154,N20155,N20156,N20157,N20158,N20159,N20160,  \\note fields as indicated
  N20161,N20162,N20163,N20164,N20165,N20166,N20167,N20168,N20169,N20170,N20171,N20172,N20173,N20174,N20175,N20176,N20177,N20178,N20179,N20180,  \\note fields as indicated
  N20181,N20182,N20183,N20184,N20185,N20186,N20187,N20188,N20189,N20190,N20191,N20192,N20193,N20194,N20195,N20196,N20197,N20198,N20199,N20200,  \\note fields as indicated
  N20201,N20202,N20203,N20204,N20205,N20206,N20207,N20208,N20209,N20210,N20211,N20212,N20213,N20214,N20215,N20216,N20217,N20218,N20219,N20220,  \\note fields as indicated
  N20221,N20222,N20223,N20224,N20225,N20226,N20227,N20228,N20229,N20230,N20231,N20232,N20233,N20234,N20235,N20236,N20237,N20238,N20239,N20240,  \\note fields as indicated
  N20241,N20242,N20243,N20244,N20245,N20246,N20247,N20248,N20249,N20250,N20251,N20252,N20253,N20254,N20255,N20256,N20257,N20258,N20259,N20260,  \\note fields as indicated
  N20261,N20262,N20263,N20264,N20265,N20266,N20267,N20268,N20269,N20270,N20271,N20272,N20273,N20274,N20275,N20276,N20277,N20278,N20279,N20280,  \\note fields as indicated
  N20281,N20282,N20283,N20284,N20285,N20286,N20287,N20288,N20289,N20290,N20291,N20292,N20293,N20294,N20295,N20296,N20297,N20298,N20299,N20300,  \\note fields as indicated
  N20301,N20302,N20303,N20304,N20305,N20306,N20307,N20308,N20309,N20310,N20311,N20312,N20313,N20314,N20315,N20316,N20317,N20318,N20319,N20320,  \\note fields as indicated
  N20321,N20322,N20323,N20324,N20325,N20326,N20327,N20328,N20329,N20330,N20331,N20332,N20333,N20334,N20335,N20336,N20337,N20338,N20339,N20340,  \\note fields as indicated
  N20341,N20342,N20343,N20344,N20345,N20346,N20347,N20348,N20349,N20350,N20351,N20352,N20353,N20354,N20355,N20356,N20357,N20358,N20359,N20360,  \\note fields as indicated
  N20361,N20362,N20363,N20364,N20365,N20366,N20367,N20368,N20369,N20370,N20371,N20372,N20373,N20374,N20375,N20376,N20377,N20378,N20379,N20380,  \\note fields as indicated
  N20381,N20382,N20383,N20384,N20385,N20386,N20387,N20388,N20389,N20390,N20391,N20392,N20393,N20394,N20395,N20396,N20397,N20398,N20399,N20400,  \\note fields as indicated
  N20401,N20402,N20403,N20404,N20405,N20406,N20407,N20408,N20409,N20410,N20411,N20412,N20413,N20414,N20415,N20416,N20417,N20418,N20419,N20420,  \\note fields as indicated
  N20421,N20422,N20423,N20424,N20425,N20426,N20427,N20428,N20429,N20430,N20431,N20432,N20433,N20434,N20435,N20436,N20437,N20438,N20439,N20440,  \\note fields as indicated
  N20441,N20442,N20443,N20444,N20445,N20446,N20447,N20448,N20449,N20450,N20451,N20452,N20453,N20454,N20455,N20456,N20457,N20458,N20459,N20460,  \\note fields as indicated
  N20461,N20462,N20463,N20464,N20465,N20466,N20467,N20468,N20469,N20470,N20471,N20472,N20473,N20474,N20475,N20476,N20477,N20478,N20479,N20480,  \\note fields as indicated
  N20481,N20482,N20483,N20484,N20485,N20486,N20487,N20488,N20489,N20490,N20491,N20492,N20493,N20494,N20495,N20496,N20497,N20498,N20499,N20500,  \\note fields as indicated
  N20501,N20502,N20503,N20504,N20505,N20506,N20507,N20508,N20509,N20510,N20511,N20512,N20513,N20514,N20515,N20516,N20517,N20518,N20519,N20520,  \\note fields as indicated
  N20521,N20522,N20523,N20524,N20525,N20526,N20527,N20528,N20529,N20530,N20531,N20532,N20533,N20534,N20535,N20536,N20537,N20538,N20539,N20540,  \\note fields as indicated
  N20541,N20542,N20543,N20544,N20545,N20546,N20547,N20548,N20549,N20550,N20551,N20552,N20553,N20554,N20555,N20556,N20557,N20558,N20559,N20560,  \\note fields as indicated
  N20561,N20562,N20563,N20564,N20565,N20566,N20567,N20568,N20569,N20570,N20571,N20572,N20573,N20574,N20575,N20576,N20577,N20578,N20579,N20580,  \\note fields as indicated
  N20581,N20582,N20583,N20584,N20585,N20586,N20587,N20588,N20589,N20590,N20591,N20592,N20593,N20594,N20595,N20596,N20597,N20598,N20599,N20600,  \\note fields as indicated
  N20601,N20602,N20603,N20604,N20605,N20606,N20607,N20608,N20609,N20610,N20611,N20612,N20613,N20614,N20615,N20616,N20617,N20618,N20619,N20620,  \\note fields as indicated
  N20621,N20622,N20623,N20624,N20625,N20626,N20627,N20628,N20629,N20630,N20631,N20632,N20633,N20634,N20635,N20636,N20637,N20638,N20639,N20640,  \\note fields as indicated
  N20641,N20642,N20643,N20644,N20645,N20646,N20647,N20648,N20649,N20650,N20651,N20652,N20653,N20654,N20655,N20656,N20657,N20658,N20659,N20660,  \\note fields as indicated
  N20661,N20662,N20663,N20664,N20665,N20666,N20667,N20668,N20669,N20670,N20671,N20672,N20673,N20674,N20675,N20676,N20677,N20678,N20679,N20680,  \\note fields as indicated
  N20681,N20682,N20683,N20684,N20685,N20686,N20687,N20688,N20689,N20690,N20691,N20692,N20693,N20694,N20695,N20696,N20697,N20698,N20699,N20700,  \\note fields as indicated
  N20701,N20702,N20703,N20704,N20705,N20706,N20707,N20708,N20709,N20710,N20711,N20712,N20713,N20714,N20715,N20716,N20717,N20718,N20719,N20720,  \\note fields as indicated
  N20721,N20722,N20723,N20724,N20725,N20726,N20727,N20728,N20729,N20730,N20731,N20732,N20733,N20734,N20735,N20736,N20737,N20738,N20739,N20740,  \\note fields as indicated
  N20741,N20742,N20743,N20744,N20745,N20746,N20747,N20748,N20749,N20750,N20751,N20752,N20753,N20754,N20755,N20756,N20757,N20758,N20759,N20760,  \\note fields as indicated
  N20761,N20762,N20763,N20764,N20765,N20766,N20767,N20768,N20769,N20770,N20771,N20772,N20773,N20774,N20775,N20776,N20777,N20778,N20779,N20780,  \\note fields as indicated
  N20781,N20782,N20783,N20784,N20785,N20786,N20787,N20788,N20789,N20790,N20791,N20792,N20793,N20794,N20795,N20796,N20797,N20798,N20799,N20800,  \\note fields as indicated
  N20801,N20802,N20803,N20804,N20805,N20806,N20807,N20808,N20809,N20810,N20811,N20812,N20813,N20814,N20815,N20816,N20817,N20818,N20819,N20820,  \\note fields as indicated
  N20821,N20822,N20823,N20824,N20825,N20826,N20827,N20828,N20829,N20830,N20831,N20832,N20833,N20834,N20835,N20836,N20837,N20838,N20839,N20840,  \\note fields as indicated
  N20841,N20842,N20843,N20844,N20845,N20846,N20847,N20848,N20849,N20850,N20851,N20852,N20853,N20854,N20855,N20856,N20857,N20858,N20859,N20860,  \\note fields as indicated
  N20861,N20862,N20863,N20864,N20865,N20866,N20867,N20868,N20869,N20870,N20871,N20872,N20873,N20874,N20875,N20876,N20877,N20878,N20879,N20880,  \\note fields as indicated
  N20881,N20882,N20883,N20884,N20885,N20886,N20887,N20888,N20889,N20890,N20891,N20892,N20893,N20894,N20895,N20896,N20897,N20898,N20899,N20900,  \\note fields as indicated
  N20901,N20902,N20903,N20904,N20905,N20906,N20907,N20908,N20909,N20910,N20911,N20912,N20913,N20914,N20915,N20916,N20917,N20918,N20919,N20920,  \\note fields as indicated
  N20921,N20922,N20923,N20924,N20925,N20926,N20927,N20928,N20929,N20930,N20931,N20932,N20933,N20934,N20935,N20936,N20937,N20938,N20939,N20940,  \\note fields as indicated
  N20941,N20942,N20943,N20944,N20945,N20946,N20947,N20948,N20949,N20950,N20951,N20952,N20953,N20954,N20955,N20956,N20957,N20958,N20959,N20960,  \\note fields as indicated
  N20961,N20962,N20963,N20964,N20965,N20966,N20967,N20968,N20969,N20970,N20971,N20972,N20973,N20974,N20975,N20976,N20977,N20978,N20979,N20980,  \\note fields as indicated
  N20981,N20982,N20983,N20984,N20985,N20986,N20987,N20988,N20989,N20990,N20991,N20992,N20993,N20994,N20995,N20996,N20997,N20998,N20999,N21000,  \\note fields as indicated
  N21001,N21002,N21003,N21004,N21005,N21006,N21007,N21008,N21009,N21010,N21011,N21012,N21013,N21014,N21015,N21016,N21017,N21018,N21019,N21020,  \\note fields as indicated
  N21021,N21022,N21023,N21024,N21025,N21026,N21027,N21028,N21029,N21030,N21031,N21032,N21033,N21034,N21035,N21036,N21037,N21038,N21039,N21040;  \\note fields as indicated

\\group Performance Curves

Curve:Linear,
       \\memo Linear curve with one independent variable.
       \\memo Input for the linear curve consists of a curve name, the two coefficients, and the
       \\memo maximum and minimum valid independent variable values. Optional inputs for
       \\memo curve minimum and maximum may be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*x
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference LinearCurves
       \\reference LinearOrQuadraticCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N4 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N5 , \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A3
  N6 , \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A3
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key Pressure
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:QuadLinear,
       \\memo Linear curve with four independent variables.
       \\memo Input for the linear curve consists of a curve name, the two coefficients, and the
       \\memo maximum and minimum valid independent variable values. Optional inputs for curve
       \\memo minimum and maximum may be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*w + C3*x + C4*y + C5*z
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference QuadLinearCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 w
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x
       \\required-field
       \\type real
  N4 , \\field Coefficient4 y
       \\required-field
       \\type real
  N5 , \\field Coefficient5 z
       \\required-field
       \\type real
  N7 , \\field Minimum Value of w
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Maximum Value of w
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N9 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N10 ,\\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N11 , \\field Minimum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A4
  N12, \\field Maximum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A4
  N13, \\field Minimum Value of z
       \\required-field
       \\type real
       \\unitsBasedOnField A5
  N14, \\field Maximum Value of z
       \\required-field
       \\type real
       \\unitsBasedOnField A5
  N15, \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A4
  N16, \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A4
  A2 , \\field Input Unit Type for w
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\key VolumetricFlowPerPower
       \\default Dimensionless
  A3 , \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\key VolumetricFlowPerPower
       \\default Dimensionless
  A4 , \\field Input Unit Type for y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\key VolumetricFlowPerPower
       \\default Dimensionless
  A5;  \\field Input Unit Type for z
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\key VolumetricFlowPerPower
       \\default Dimensionless

Curve:Quadratic,
       \\memo Quadratic curve with one independent variable.
       \\memo Input for a quadratic curve consists of the curve name, the three coefficients, and
       \\memo the maximum and minimum valid independent variable values. Optional inputs for curve
       \\memo minimum and maximum may be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*x + C3*x**2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference QuadraticCurves
       \\reference LinearOrQuadraticCurves
       \\reference AllCurves
       \\reference QuadraticCubicCurves
       \\reference BiquadraticQuadraticCubicCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N5 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A3
  N7 , \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A3
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Cubic,
       \\memo Cubic curve with one independent variable.
       \\memo Input for a cubic curve consists of the curve name, the 4 coefficients, and the
       \\memo maximum and minimum valid independent variable values. Optional inputs for curve
       \\memo minimum and maximum may be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*x + C3*x**2 + C4*x**3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference CubicCurves
       \\reference AllCurves
       \\reference QuadraticCubicCurves
       \\reference BiquadraticCubicCurves
       \\reference BiquadraticQuadraticCubicCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Coefficient4 x**3
       \\required-field
       \\type real
  N5 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N7 , \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A3
  N8 , \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A3
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Quartic,
       \\memo Quartic (fourth order polynomial) curve with one independent variable.
       \\memo Input for a Quartic curve consists of the curve name, the
       \\memo five coefficients, and the maximum and minimum valid independent variable values.
       \\memo Optional inputs for curve minimum and maximum may be used to limit the
       \\memo output of the performance curve.
       \\memo curve = C1 + C2*x + C3*x**2 + C4*x**3 + C5*x**4
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference QuarticCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Coefficient4 x**3
       \\required-field
       \\type real
  N5 , \\field Coefficient5 x**4
       \\required-field
       \\type real
  N6 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N7 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N9 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2 , \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3 ; \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Exponent,
       \\memo Exponent curve with one independent variable.
       \\memo Input for a exponent curve consists of the curve name, the 3 coefficients, and the
       \\memo maximum and minimum valid independent variable values. Optional inputs for curve
       \\memo minimum and maximum may be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*x**C3
       \\memo The independent variable x is raised to the C3 power, multiplied by C2, and C1 is added to the result.
    \\min-fields 6
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExponentCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 Constant
       \\required-field
       \\type real
  N3 , \\field Coefficient3 Constant
       \\required-field
       \\type real
  N4 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\note Specify the minimum value of the independent variable x allowed
       \\unitsBasedOnField A2
  N5 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\note Specify the maximum value of the independent variable x allowed
       \\unitsBasedOnField A2
  N6 , \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A3
  N7 , \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A3
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Bicubic,
       \\memo Cubic curve with two independent variables. Input consists of the
       \\memo curve name, the ten coefficients, and the minimum and maximum values for each of
       \\memo the independent variables. Optional inputs for curve minimum and maximum may
       \\memo be used to limit the output of the performance curve.
       \\memo curve = C1 + C2*x + C3*x**2 + C4*y + C5*y**2 + C6*x*y + C7*x**3 + C8*y**3 + C9*x**2*y
       \\memo + C10*x*y**2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllCurves
       \\reference BicubicCurves
       \\reference BicubicBiquadraticCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Coefficient4 y
       \\required-field
       \\type real
  N5 , \\field Coefficient5 y**2
       \\required-field
       \\type real
  N6 , \\field Coefficient6 x*y
       \\required-field
       \\type real
  N7 , \\field Coefficient7 x**3
       \\required-field
       \\type real
  N8 , \\field Coefficient8 y**3
       \\required-field
       \\type real
  N9 , \\field Coefficient9 x**2*y
       \\required-field
       \\type real
  N10, \\field Coefficient10 x*y**2
       \\required-field
       \\type real
  N11, \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N12, \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N13, \\field Minimum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N14, \\field Maximum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N15, \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A4
  N16, \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A4
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3,  \\field Input Unit Type for Y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A4;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Biquadratic,
       \\memo Quadratic curve with two independent variables. Input consists of the curve name, the
       \\memo six coefficients, and min and max values for each of the independent variables.
       \\memo Optional inputs for curve minimum and maximum may be used to limit the
       \\memo output of the performance curve.
       \\memo curve = C1 + C2*x + C3*x**2 + C4*y + C5*y**2 + C6*x*y
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference BiquadraticCurves
       \\reference AllCurves
       \\reference BiquadraticCubicCurves
       \\reference BiquadraticQuadraticCubicCurves
       \\reference BicubicBiquadraticCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Coefficient4 y
       \\required-field
       \\type real
  N5 , \\field Coefficient5 y**2
       \\required-field
       \\type real
  N6 , \\field Coefficient6 x*y
       \\required-field
       \\type real
  N7 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N9 , \\field Minimum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N10, \\field Maximum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N11, \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A4
  N12, \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A4
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3,  \\field Input Unit Type for Y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A4;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:QuadraticLinear,
       \\memo Quadratic-linear curve with two independent variables. Input consists of the curve
       \\memo name, the six coefficients, and min and max values for each of the independent
       \\memo variables. Optional inputs for curve minimum and maximum may be used to limit the
       \\memo output of the performance curve.
       \\memo curve = (C1 + C2*x + C3*x**2) + (C4 + C5*x + C6*x**2)*y
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference QuadraticLinearCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\required-field
       \\type real
  N2 , \\field Coefficient2 x
       \\required-field
       \\type real
  N3 , \\field Coefficient3 x**2
       \\required-field
       \\type real
  N4 , \\field Coefficient4 y
       \\required-field
       \\type real
  N5 , \\field Coefficient5 x*y
       \\required-field
       \\type real
  N6 , \\field Coefficient6 x**2*y
       \\required-field
       \\type real
  N7 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N9 , \\field Minimum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N10, \\field Maximum Value of y
       \\required-field
       \\type real
       \\unitsBasedOnField A3
  N11, \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A4
  N12, \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A4
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3,  \\field Input Unit Type for Y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A4;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Triquadratic,
       \\memo Quadratic curve with three independent variables. Input consists of the curve name,
       \\memo the twenty seven coefficients, and min and max values for each of the independent
       \\memo variables. Optional inputs for curve minimum and maximum may be used to
       \\memo limit the output of the performance curve.
       \\memo curve = a0 + a1*x**2 + a2*x + a3*y**2 + a4*y
       \\memo         + a5*z**2 + a6*z + a7*x**2*y**2 + a8*x*y
       \\memo         + a9*x*y**2 + a10*x**2*y + a11*x**2*z**2
       \\memo         + a12*x*z + a13*x*z**2 + a14*x**2*z + a15*y**2*z**2
       \\memo         + a16*y*z + a17*y*z**2 + a18*y**2*z + a19*x**2*y**2*z**2
       \\memo         + a20*x**2*y**2*z + a21*x**2*y*z**2 + a22*x*y**2*z**2
       \\memo         + a23*x**2*y*z + a24*x*y**2*z + a25*x*y*z**2 +a26*x*y*z
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference TriQuadraticCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 Constant
       \\type real
  N2 , \\field Coefficient2 x**2
       \\type real
  N3 , \\field Coefficient3 x
       \\type real
  N4 , \\field Coefficient4 y**2
       \\type real
  N5 , \\field Coefficient5 y
       \\type real
  N6 , \\field Coefficient6 z**2
       \\type real
  N7 , \\field Coefficient7 z
       \\type real
  N8 , \\field Coefficient8 x**2*y**2
       \\type real
  N9 , \\field Coefficient9 x*y
       \\type real
  N10, \\field Coefficient10 x*y**2
       \\type real
  N11, \\field Coefficient11 x**2*y
       \\type real
  N12, \\field Coefficient12 x**2*z**2
       \\type real
  N13, \\field Coefficient13 x*z
       \\type real
  N14, \\field Coefficient14 x*z**2
       \\type real
  N15, \\field Coefficient15 x**2*z
       \\type real
  N16, \\field Coefficient16 y**2*z**2
       \\type real
  N17, \\field Coefficient17 y*z
       \\type real
  N18, \\field Coefficient18 y*z**2
       \\type real
  N19, \\field Coefficient19 y**2*z
       \\type real
  N20, \\field Coefficient20 x**2*y**2*z**2
       \\type real
  N21, \\field Coefficient21 x**2*y**2*z
       \\type real
  N22, \\field Coefficient22 x**2*y*z**2
       \\type real
  N23, \\field Coefficient23 x*y**2*z**2
       \\type real
  N24, \\field Coefficient24 x**2*y*z
       \\type real
  N25, \\field Coefficient25 x*y**2*z
       \\type real
  N26, \\field Coefficient26 x*y*z**2
       \\type real
  N27, \\field Coefficient27 x*y*z
       \\type real
  N28, \\field Minimum Value of x
       \\type real
       \\unitsBasedOnField A2
  N29, \\field Maximum Value of x
       \\type real
       \\unitsBasedOnField A2
  N30, \\field Minimum Value of y
       \\type real
       \\unitsBasedOnField A3
  N31, \\field Maximum Value of y
       \\type real
       \\unitsBasedOnField A3
  N32, \\field Minimum Value of z
       \\type real
       \\unitsBasedOnField A4
  N33, \\field Maximum Value of z
       \\type real
       \\unitsBasedOnField A4
  N34, \\field Minimum Curve Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A5
  N35, \\field Maximum Curve Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A5
  A2,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A3,  \\field Input Unit Type for Y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A4,  \\field Input Unit Type for Z
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A5;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless

Curve:Functional:PressureDrop,
       \\min-fields 5
       \\memo Sets up curve information for minor loss and/or friction
       \\memo calculations in plant pressure simulations
       \\memo Expression: DeltaP = {K + f*(L/D)} * (rho * V^2) / 2
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllCurves
       \\reference PressureDropCurves
  N1 , \\field Diameter
       \\required-field
       \\units m
       \\type real
       \\note "D" in above expression, used to also calculate local velocity
       \\minimum> 0
  N2 , \\field Minor Loss Coefficient
       \\units dimensionless
       \\type real
       \\note "K" in above expression
       \\minimum> 0
  N3 , \\field Length
       \\units m
       \\type real
       \\note "L" in above expression
       \\minimum> 0
  N4 , \\field Roughness
       \\units m
       \\type real
       \\note This will be used to calculate "f" from Moody-chart approximations
       \\minimum> 0
  N5 ; \\field Fixed Friction Factor
       \\note Optional way to set a constant value for "f", instead of using
       \\note  internal Moody-chart approximations
       \\minimum> 0

Curve:FanPressureRise,
       \\memo Special curve type with two independent variables.
       \\memo Input for the fan total pressure rise curve consists of the curve name, the four
       \\memo coefficients, and the maximum and minimum valid independent variable values. Optional
       \\memo inputs for the curve minimum and maximum may be used to limit the output of the
       \\memo performance curve.
       \\memo curve = C1*Qfan**2+C2*Qfan+C3*Qfan*(Psm-Po)**0.5+C4*(Psm-Po)
       \\memo Po assumed to be zero
       \\memo See InputOut Reference for curve details
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference FanPressureRiseCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Coefficient4 C4
       \\required-field
       \\type real
  N5 , \\field Minimum Value of Qfan
       \\required-field
       \\type real
       \\units m3/s
  N6 , \\field Maximum Value of Qfan
       \\required-field
       \\type real
       \\units m3/s
  N7 , \\field Minimum Value of Psm
       \\required-field
       \\type real
       \\units Pa
       \\ip-units Pa
  N8 , \\field Maximum Value of Psm
       \\required-field
       \\type real
       \\units Pa
       \\ip-units Pa
  N9 , \\field Minimum Curve Output
       \\type real
       \\units Pa
       \\ip-units Pa
       \\note Specify the minimum value calculated by this curve object
  N10 ; \\field Maximum Curve Output
        \\type real
        \\units Pa
        \\ip-units Pa
        \\note Specify the maximum value calculated by this curve object

Curve:ExponentialSkewNormal,
       \\memo Exponential-modified skew normal curve with one independent variable.
       \\memo Input consists of the curve name, the four coefficients, and the maximum
       \\memo and minimum valid independent variable values. Optional inputs for the curve minimum
       \\memo and maximum may be used to limit the output of the performance curve.
       \\memo curve = see Input Output Reference
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExponentialSkewNormalCurves
       \\reference AllCurves
       \\note See InputOut Reference for curve description
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Coefficient4 C4
       \\required-field
       \\type real
  N5 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N7 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N8 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

Curve:Sigmoid,
       \\memo Sigmoid curve with one independent variable.
       \\memo Input consists of the curve name, the five coefficients, and the maximum and minimum
       \\memo valid independent variable values. Optional inputs for the curve minimum and maximum
       \\memo may be used to limit the output of the performance curve.
       \\memo curve = C1+C2/[1+exp((C3-x)/C4)]**C5
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference SigmoidCurves
       \\reference AllCurves
       \\note See InputOut Reference for curve description
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Coefficient4 C4
       \\required-field
       \\type real
  N5 , \\field Coefficient5 C5
       \\required-field
       \\type real
  N6 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N7 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N9 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

Curve:RectangularHyperbola1,
       \\memo Rectangular hyperbola type 1 curve with one independent variable.
       \\memo Input consists of the curve name, the three coefficients, and the maximum and
       \\memo minimum valid independent variable values. Optional inputs for the curve minimum and
       \\memo maximum may be used to limit the output of the performance curve.
       \\memo curve = ((C1*x)/(C2+x))+C3
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference RectangularHyperbola1Curves
       \\reference AllCurves
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N5 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N7 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

Curve:RectangularHyperbola2,
       \\memo Rectangular hyperbola type 2 curve with one independent variable.
       \\memo Input consists of the curve name, the three coefficients, and the maximum and
       \\memo minimum valid independent variable values. Optional inputs for the curve minimum and
       \\memo maximum may be used to limit the output of the performance curve.
       \\memo curve = ((C1*x)/(C2+x))+(C3*x)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference RectangularHyperbola2Curves
       \\reference AllCurves
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N5 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N7 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

Curve:ExponentialDecay,
       \\memo Exponential decay curve with one independent variable.
       \\memo Input consists of the curve name, the three coefficients, and the maximum and minimum
       \\memo valid independent variable values. Optional inputs for the curve minimum and
       \\memo maximum may be used to limit the output of the performance curve.
       \\memo curve = C1+C2*exp(C3*x)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference ExponentialDecayCurves
       \\reference AllCurves
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N5 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N6 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N7 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

Curve:DoubleExponentialDecay,
       \\memo Double exponential decay curve with one independent variable.
       \\memo Input consists of the curve name, the five coefficients, and the maximum and minimum
       \\memo valid independent variable values. Optional inputs for the curve minimum and
       \\memo maximum may be used to limit the output of the performance curve.
       \\memo curve = C1+C2*exp(C3*x)+C4*exp(C5*x)
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference DoubleExponentialDecayCurves
  N1 , \\field Coefficient1 C1
       \\required-field
       \\type real
  N2 , \\field Coefficient2 C2
       \\required-field
       \\type real
  N3 , \\field Coefficient3 C3
       \\required-field
       \\type real
  N4 , \\field Coefficient3 C4
       \\required-field
       \\type real
  N5 , \\field Coefficient3 C5
       \\required-field
       \\type real
  N6 , \\field Minimum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N7 , \\field Maximum Value of x
       \\required-field
       \\type real
       \\unitsBasedOnField A2
  N8 , \\field Minimum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the minimum value calculated by this curve object
  N9 , \\field Maximum Curve Output
       \\type real
       \\unitsBasedOnField A3
       \\note Specify the maximum value calculated by this curve object
  A2,  \\field Input Unit Type for x
       \\type choice
       \\key Dimensionless
       \\default Dimensionless
  A3;  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\default Dimensionless

\\group Performance Tables

Table:OneIndependentVariable,
       \\extensible:2 repeat last two fields
       \\min-fields 14
       \\memo Allows entry of tabular data pairs as alternate input
       \\memo for performance curve objects.
       \\memo Performance curve objects can be created using these inputs.
       \\memo Linear Table Equation: Output = a + bX
       \\memo Linear solution requires a minimum of 2 data pairs
       \\memo Quadratic Table Equation: Output = a + b*X + c*X**2
       \\memo Quadratic solution requires a minimum of 3 data pairs
       \\memo Cubic Table Equation: Output = a + b*X + c* X**2 + d*X**3
       \\memo Cubic solution requires a minimum of 4 data pairs
       \\memo Quartic Table Equation: Output = a + b*X + c* X**2 + d*X**3 + e*X**4
       \\memo Quartic solution requires a minimum of 5 data pairs
       \\memo Exponent Table Equation: Output = a + b*X**c
       \\memo Exponent solution requires a minimum of 4 data pairs
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllCurves
  A2 , \\field Curve Type
       \\required-field
       \\type choice
       \\key Linear
       \\key Quadratic
       \\key Cubic
       \\key Quartic
       \\key Exponent
  A3 , \\field Interpolation Method
       \\type choice
       \\key LinearInterpolationOfTable
       \\key EvaluateCurveToLimits
       \\key LagrangeInterpolationLinearExtrapolation
  N1 , \\field Minimum Value of X
       \\type real
       \\unitsBasedOnField A4
       \\note used only when Interpolation Type is Evaluate Curve
       \\note to Limits
  N2 , \\field Maximum Value of X
       \\type real
       \\unitsBasedOnField A4
       \\note used only when Interpolation Type is Evaluate Curve
       \\note to Limits
  N3 , \\field Minimum Table Output
       \\type real
       \\note Specify the minimum value calculated by this table
       \\note lookup object
       \\unitsBasedOnField A5
       \\note used only when Interpolation Type is Evaluate Curve
       \\note to Limits
  N4 , \\field Maximum Table Output
       \\type real
       \\note Specify the maximum value calculated by this table
       \\note lookup object
       \\unitsBasedOnField A5
       \\note used only when Interpolation Type is Evaluate Curve
       \\note to Limits
  A4,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A5,  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless
  N5 , \\field Normalization Reference
       \\type real
       \\note This field is used to normalize the following ouput data.
       \\note The minimum and maximum table output fields are also normalized.
       \\note If this field is blank or 1, the table data presented
       \\note in the following fields will be used with normalization
       \\note reference set to 1.
  N6 , \\field X Value #1
       \\required-field
       \\type real
       \\begin-extensible
  N7 , \\field Output Value #1
       \\required-field
       \\type real
  N8 , \\field X Value #2
       \\required-field
       \\type real
  N9 , \\field Output Value #2
       \\required-field
       \\type real
  N10, \\field X Value #3
       \\type real
  N11, \\field Output Value #3
       \\type real
  N12, \\field X Value #4
       \\type real
  N13, \\field Output Value #4
       \\type real
  N14, \\field X Value #5
       \\type real
  N15, \\field Output Value #5
       \\type real
  N16, \\field X Value #6
       \\type real
  N17, \\field Output Value #6
       \\type real
  N18, \\field X Value #7
       \\type real
  N19, \\field Output Value #7
       \\type real
  N20,N21,N22,N23,N24,N25,N26,N27,N28,N29,N30,N31,N32,N33,N34,N35,N36,N37,N38,N39, \\note fields as indicated
  N40,N41,N42,N43,N44,N45,N46,N47,N48,N49,N50,N51,N52,N53,N54,N55,N56,N57,N58,N59, \\note fields as indicated
  N60,N61,N62,N63,N64,N65,N66,N67,N68,N69,N70,N71,N72,N73,N74,N75,N76,N77,N78,N79, \\note fields as indicated
  N80,N81,N82,N83,N84,N85,N86,N87,N88,N89,N90,N91,N92,N93,N94,N95,N96,N97,N98,N99, \\note fields as indicated
  N100,N101,N102,N103,N104,N105,N106,N107,N108,N109,N110,N111,N112,N113,N114,N115,N116,N117,N118,N119; \\note fields as indicated


Table:TwoIndependentVariables,
       \\extensible:3 repeat last three fields
       \\min-fields 22
       \\memo Allows entry of tabular data pairs as alternate input
       \\memo for performance curve objects.
       \\memo Performance curve objects can be created using these inputs.
       \\memo BiQuadratic Table Equation: Output = a + bX + cX**2 + dY + eY**2 + fXY
       \\memo BiQuadratic solution requires a minimum of 6 data pairs
       \\memo QuadraticLinear Table Equation: Output = a + bX + cX**2 + dY + eXY + fX**2Y
       \\memo QuadraticLinear solution requires a minimum of 6 data pairs
  A1 , \\field Name
       \\required-field
       \\type alpha
       \\reference AllCurves
  A2 , \\field Curve Type
       \\required-field
       \\type choice
       \\key BiQuadratic
       \\key QuadraticLinear
  A3 , \\field Interpolation Method
       \\type choice
       \\key LinearInterpolationOfTable
       \\key EvaluateCurveToLimits
       \\key LagrangeInterpolationLinearExtrapolation
       \\default LagrangeInterpolationLinearExtrapolation
  N1 , \\field Minimum Value of X
       \\type real
       \\unitsBasedOnField A4
  N2 , \\field Maximum Value of X
       \\type real
       \\unitsBasedOnField A4
  N3 , \\field Minimum Value of Y
       \\type real
       \\unitsBasedOnField A5
  N4 , \\field Maximum Value of Y
       \\type real
       \\unitsBasedOnField A5
  N5 , \\field Minimum Table Output
       \\type real
       \\note Specify the minimum value calculated by this table lookup object
       \\unitsBasedOnField A6
  N6 , \\field Maximum Table Output
       \\type real
       \\note Specify the maximum value calculated by this table lookup object
       \\unitsBasedOnField A6
  A4,  \\field Input Unit Type for X
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A5,  \\field Input Unit Type for Y
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A6,  \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless
  N7 , \\field Normalization Reference
       \\type real
       \\note This field is used to normalize the following output data.
       \\note The minimum and maximum table output fields are also normalized.
       \\note If this field is blank or 1, the table data presented below will be used.
  N8 , \\field X Value #1
       \\required-field
       \\type real
       \\begin-extensible
  N9 , \\field Y Value #1
       \\required-field
       \\type real
  N10, \\field Output Value #1
       \\required-field
       \\type real
  N11, \\field X Value #2
       \\required-field
       \\type real
  N12, \\field Y Value #2
       \\required-field
       \\type real
  N13, \\field Output Value #2
       \\required-field
       \\type real
  N14, \\field X Value #3
       \\required-field
       \\type real
  N15, \\field Y Value #3
       \\required-field
       \\type real
  N16, \\field Output Value #3
       \\required-field
       \\type real
  N17, \\field X Value #4
       \\required-field
       \\type real
  N18, \\field Y Value #4
       \\required-field
       \\type real
  N19, \\field Output Value #4
       \\required-field
       \\type real
  N20,N21,N22,N23,N24,N25,N26,N27,N28,N29,N30,N31,N32,N33,N34,N35,N36,N37,N38,N39, \\note fields as indicated
  N40,N41,N42,N43,N44,N45,N46,N47,N48,N49,N50,N51,N52,N53,N54,N55,N56,N57,N58,N59, \\note fields as indicated
  N60,N61,N62,N63,N64,N65,N66,N67,N68,N69,N70,N71,N72,N73,N74,N75,N76,N77,N78,N79, \\note fields as indicated
  N80,N81,N82,N83,N84,N85,N86,N87,N88,N89,N90,N91,N92,N93,N94,N95,N96,N97,N98,N99, \\note fields as indicated
  N100,N101,N102,N103,N104,N105,N106,N107,N108,N109,N110,N111,N112,N113,N114,N115,N116,N117,N118,N119, \\note fields as indicated
  N120,N121,N122,N123,N124,N125,N126,N127,N128,N129,N130,N131,N132,N133,N134,N135,N136,N137,N138,N139, \\note fields as indicated
  N140,N141,N142,N143,N144,N145,N146,N147,N148,N149,N150,N151,N152,N153,N154,N155,N156,N157,N158,N159, \\note fields as indicated
  N160,N161,N162,N163,N164,N165,N166,N167,N168,N169,N170,N171,N172,N173,N174,N175,N176,N177,N178,N179, \\note fields as indicated
  N180,N181,N182,N183,N184,N185,N186,N187,N188,N189,N190,N191,N192,N193,N194,N195,N196,N197,N198,N199, \\note fields as indicated
  N200,N201,N202,N203,N204,N205,N206,N207,N208,N209,N210,N211,N212,N213,N214,N215,N216,N217,N218,N219, \\note fields as indicated
  N220,N221,N222,N223,N224,N225,N226,N227,N228,N229,N230,N231,N232,N233,N234,N235,N236,N237,N238,N239, \\note fields as indicated
  N240,N241,N242,N243,N244,N245,N246,N247,N248,N249,N250,N251,N252,N253,N254,N255,N256,N257,N258,N259, \\note fields as indicated
  N260,N261,N262,N263,N264,N265,N266,N267,N268,N269,N270,N271,N272,N273,N274,N275,N276,N277,N278,N279, \\note fields as indicated
  N280,N281,N282,N283,N284,N285,N286,N287,N288,N289,N290,N291,N292,N293,N294,N295,N296,N297,N298,N299, \\note fields as indicated
  N300,N301,N302,N303,N304,N305,N306,N307,N308,N309,N310,N311,N312,N313,N314,N315,N316,N317,N318,N319, \\note fields as indicated
  N320,N321,N322,N323,N324,N325,N326,N327,N328,N329,N330,N331,N332,N333,N334,N335,N336,N337,N338,N339, \\note fields as indicated
  N340,N341,N342,N343,N344,N345,N346,N347,N348,N349,N350,N351,N352,N353,N354,N355,N356,N357,N358,N359, \\note fields as indicated
  N360,N361,N362,N363,N364,N365,N366,N367,N368,N369,N370,N371,N372,N373,N374,N375,N376,N377,N378,N379, \\note fields as indicated
  N380,N381,N382,N383,N384,N385,N386,N387,N388,N389,N390,N391,N392,N393,N394,N395,N396,N397,N398,N399, \\note fields as indicated
  N400,N401,N402,N403,N404,N405,N406,N407,N408,N409,N410,N411,N412,N413,N414,N415,N416,N417,N418,N419, \\note fields as indicated
  N420,N421,N422,N423,N424,N425,N426,N427,N428,N429,N430,N431,N432,N433,N434,N435,N436,N437,N438,N439, \\note fields as indicated
  N440,N441,N442,N443,N444,N445,N446,N447,N448,N449,N450,N451,N452,N453,N454,N455,N456,N457,N458,N459, \\note fields as indicated
  N460,N461,N462,N463,N464,N465,N466,N467,N468,N469,N470,N471,N472,N473,N474,N475,N476,N477,N478,N479, \\note fields as indicated
  N480,N481,N482,N483,N484,N485,N486,N487,N488,N489,N490,N491,N492,N493,N494,N495,N496,N497,N498,N499; \\note fields as indicated

Table:MultiVariableLookup,
       \\memo The multi-variable lookup table can represent from 1 to 5 independent variables and
       \\memo can interpolate these independent variables up to a 4th order polynomial. The
       \\memo polynomial order is assumed to be the number of interpolation points (n) minus 1.
       \\memo When any independent variable value is outside the table limits, linear extrapolation
       \\memo is used to predict the table result and is based on the two nearest data points in the
       \\memo table for that particularindependent variable.
       \\extensible:20 repeat last twenty fields remembering to move the semi-colon to the last value
       \\min-fields 27
  A1 , \\field Name
       \\required-field
       \\type alpha
  A2 , \\field Interpolation Method
       \\type choice
       \\key LinearInterpolationOfTable
       \\key EvaluateCurveToLimits
       \\key LagrangeInterpolationLinearExtrapolation
       \\default LagrangeInterpolationLinearExtrapolation
  N1 , \\field Number of Interpolation Points
       \\type integer
       \\minimum> 1
       \\maximum 4
       \\default 3
  A3 , \\field Curve Type
       \\type choice
       \\key Linear
       \\key Quadratic
       \\key Cubic
       \\key Quartic
       \\key Exponent
       \\key BiQuadratic
       \\key QuadraticLinear
       \\key BiCubic
       \\key TriQuadratic
       \\key Other
       \\note The curve types BiCubic and TriQuadratic may not
       \\note be used with Interpolation Method = EvaluateCurveToLimits
  A4 , \\field Table Data Format
       \\type choice
       \\key SingleLineIndependentVariableWithMatrix
       \\default SingleLineIndependentVariableWithMatrix
  A5 , \\field External File Name
       \\type alpha
       \\retaincase
  A6 , \\field X1 Sort Order
       \\type choice
       \\key Ascending
       \\key Descending
       \\default Ascending
  A7 , \\field X2 Sort Order
       \\type choice
       \\key Ascending
       \\key Descending
       \\default Ascending
  N2 , \\field Normalization Reference
       \\type real
       \\note This field is used to normalize the table output data.
       \\note The minimum and maximum table output fields are also normalized.
       \\note If this field is blank or 1, the table data will be directly used.
       \\note This field is not allowed to be set equal to 0.
  N3 , \\field Minimum Value of X1
       \\type real
       \\unitsBasedOnField A8
  N4 , \\field Maximum Value of X1
       \\type real
       \\unitsBasedOnField A8
  N5 , \\field Minimum Value of X2
       \\type real
       \\unitsBasedOnField A9
  N6 , \\field Maximum Value of X2
       \\type real
       \\unitsBasedOnField A9
  N7 , \\field Minimum Value of X3
       \\type real
       \\unitsBasedOnField A10
  N8 , \\field Maximum Value of X3
       \\type real
       \\unitsBasedOnField A10
  N9 , \\field Minimum Value of X4
       \\type real
       \\unitsBasedOnField A11
  N10, \\field Maximum Value of X4
       \\type real
       \\unitsBasedOnField A11
  N11, \\field Minimum Value of X5
       \\type real
       \\unitsBasedOnField A12
  N12, \\field Maximum Value of X5
       \\type real
       \\unitsBasedOnField A12
  N13, \\field Minimum Table Output
       \\type real
       \\note Specify the minimum value calculated by this curve object
       \\unitsBasedOnField A13
  N14, \\field Maximum Table Output
       \\type real
       \\note Specify the maximum value calculated by this curve object
       \\unitsBasedOnField A13
  A8 , \\field Input Unit Type for X1
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A9 , \\field Input Unit Type for X2
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A10, \\field Input Unit Type for X3
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A11, \\field Input Unit Type for X4
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A12, \\field Input Unit Type for X5
       \\type choice
       \\key Dimensionless
       \\key Temperature
       \\key VolumetricFlow
       \\key MassFlow
       \\key Power
       \\key Distance
       \\default Dimensionless
  A13, \\field Output Unit Type
       \\type choice
       \\key Dimensionless
       \\key Capacity
       \\key Power
       \\default Dimensionless
  N15, \\field Number of Independent Variables
       \\type integer
       \\minimum> 0
       \\maximum 5
  N16, \\field Number of Values for Independent Variable X1
       \\type integer
  N17, \\field Field 1 Determined by the Number of Independent Variables
       \\type real
  N18, \\field Field 2 Determined by the Number of Independent Variables
  N19, \\field Field 3 Determined by the Number of Independent Variables
       \\begin-extensible
  N20,N21,N22,N23,N24,N25,N26,N27,N28,N29,N30,N31,N32,N33,N34,N35,N36,N37,N38,N39, \\note fields as indicated
  N40,N41,N42,N43,N44,N45,N46,N47,N48,N49,N50,N51,N52,N53,N54,N55,N56,N57,N58,N59, \\note fields as indicated
  N60,N61,N62,N63,N64,N65,N66,N67,N68,N69,N70,N71,N72,N73,N74,N75,N76,N77,N78,N79, \\note fields as indicated
  N80,N81,N82,N83,N84,N85,N86,N87,N88,N89,N90,N91,N92,N93,N94,N95,N96,N97,N98,N99, \\note fields as indicated
  N100,N101,N102,N103,N104,N105,N106,N107,N108,N109,N110,N111,N112,N113,N114,N115,N116,N117,N118,N119, \\note fields as indicated
  N120,N121,N122,N123,N124,N125,N126,N127,N128,N129,N130,N131,N132,N133,N134,N135,N136,N137,N138,N139, \\note fields as indicated
  N140,N141,N142,N143,N144,N145,N146,N147,N148,N149,N150,N151,N152,N153,N154,N155,N156,N157,N158,N159, \\note fields as indicated
  N160,N161,N162,N163,N164,N165,N166,N167,N168,N169,N170,N171,N172,N173,N174,N175,N176,N177,N178,N179, \\note fields as indicated
  N180,N181,N182,N183,N184,N185,N186,N187,N188,N189,N190,N191,N192,N193,N194,N195,N196,N197,N198,N199, \\note fields as indicated
  N200,N201,N202,N203,N204,N205,N206,N207,N208,N209,N210,N211,N212,N213,N214,N215,N216,N217,N218,N219, \\note fields as indicated
  N220,N221,N222,N223,N224,N225,N226,N227,N228,N229,N230,N231,N232,N233,N234,N235,N236,N237,N238,N239, \\note fields as indicated
  N240,N241,N242,N243,N244,N245,N246,N247,N248,N249,N250,N251,N252,N253,N254,N255,N256,N257,N258,N259, \\note fields as indicated
  N260,N261,N262,N263,N264,N265,N266,N267,N268,N269,N270,N271,N272,N273,N274,N275,N276,N277,N278,N279, \\note fields as indicated
  N280,N281,N282,N283,N284,N285,N286,N287,N288,N289,N290,N291,N292,N293,N294,N295,N296,N297,N298,N299, \\note fields as indicated
  N300,N301,N302,N303,N304,N305,N306,N307,N308,N309,N310,N311,N312,N313,N314,N315,N316,N317,N318,N319, \\note fields as indicated
  N320,N321,N322,N323,N324,N325,N326,N327,N328,N329,N330,N331,N332,N333,N334,N335,N336,N337,N338,N339, \\note fields as indicated
  N340,N341,N342,N343,N344,N345,N346,N347,N348,N349,N350,N351,N352,N353,N354,N355,N356,N357,N358,N359, \\note fields as indicated
  N360,N361,N362,N363,N364,N365,N366,N367,N368,N369,N370,N371,N372,N373,N374,N375,N376,N377,N378,N379, \\note fields as indicated
  N380,N381,N382,N383,N384,N385,N386,N387,N388,N389,N390,N391,N392,N393,N394,N395,N396,N397,N398,N399, \\note fields as indicated
  N400,N401,N402,N403,N404,N405,N406,N407,N408,N409,N410,N411,N412,N413,N414,N415,N416,N417,N418,N419, \\note fields as indicated
  N420,N421,N422,N423,N424,N425,N426,N427,N428,N429,N430,N431,N432,N433,N434,N435,N436,N437,N438,N439, \\note fields as indicated
  N440,N441,N442,N443,N444,N445,N446,N447,N448,N449,N450,N451,N452,N453,N454,N455,N456,N457,N458,N459, \\note fields as indicated
  N460,N461,N462,N463,N464,N465,N466,N467,N468,N469,N470,N471,N472,N473,N474,N475,N476,N477,N478,N479, \\note fields as indicated
  N480,N481,N482,N483,N484,N485,N486,N487,N488,N489,N490,N491,N492,N493,N494,N495,N496,N497,N498,N499, \\note fields as indicated
  N500,N501,N502,N503,N504,N505,N506,N507,N508,N509,N510,N511,N512,N513,N514,N515,N516,N517,N518,N519, \\note fields as indicated
  N520,N521,N522,N523,N524,N525,N526,N527,N528,N529,N530,N531,N532,N533,N534,N535,N536,N537,N538,N539, \\note fields as indicated
  N540,N541,N542,N543,N544,N545,N546,N547,N548,N549,N550,N551,N552,N553,N554,N555,N556,N557,N558,N559, \\note fields as indicated
  N560,N561,N562,N563,N564,N565,N566,N567,N568,N569,N570,N571,N572,N573,N574,N575,N576,N577,N578,N579, \\note fields as indicated
  N580,N581,N582,N583,N584,N585,N586,N587,N588,N589,N590,N591,N592,N593,N594,N595,N596,N597,N598,N599, \\note fields as indicated
  N600,N601,N602,N603,N604,N605,N606,N607,N608,N609,N610,N611,N612,N613,N614,N615,N616,N617,N618,N619, \\note fields as indicated
  N620,N621,N622,N623,N624,N625,N626,N627,N628,N629,N630,N631,N632,N633,N634,N635,N636,N637,N638,N639, \\note fields as indicated
  N640,N641,N642,N643,N644,N645,N646,N647,N648,N649,N650,N651,N652,N653,N654,N655,N656,N657,N658,N659, \\note fields as indicated
  N660,N661,N662,N663,N664,N665,N666,N667,N668,N669,N670,N671,N672,N673,N674,N675,N676,N677,N678,N679, \\note fields as indicated
  N680,N681,N682,N683,N684,N685,N686,N687,N688,N689,N690,N691,N692,N693,N694,N695,N696,N697,N698,N699, \\note fields as indicated
  N700,N701,N702,N703,N704,N705,N706,N707,N708,N709,N710,N711,N712,N713,N714,N715,N716,N717,N718,N719, \\note fields as indicated
  N720,N721,N722,N723,N724,N725,N726,N727,N728,N729,N730,N731,N732,N733,N734,N735,N736,N737,N738,N739, \\note fields as indicated
  N740,N741,N742,N743,N744,N745,N746,N747,N748,N749,N750,N751,N752,N753,N754,N755,N756,N757,N758,N759, \\note fields as indicated
  N760,N761,N762,N763,N764,N765,N766,N767,N768,N769,N770,N771,N772,N773,N774,N775,N776,N777,N778,N779, \\note fields as indicated
  N780,N781,N782,N783,N784,N785,N786,N787,N788,N789,N790,N791,N792,N793,N794,N795,N796,N797,N798,N799, \\note fields as indicated
  N800,N801,N802,N803,N804,N805,N806,N807,N808,N809,N810,N811,N812,N813,N814,N815,N816,N817,N818,N819, \\note fields as indicated
  N820,N821,N822,N823,N824,N825,N826,N827,N828,N829,N830,N831,N832,N833,N834,N835,N836,N837,N838,N839, \\note fields as indicated
  N840,N841,N842,N843,N844,N845,N846,N847,N848,N849,N850,N851,N852,N853,N854,N855,N856,N857,N858,N859, \\note fields as indicated
  N860,N861,N862,N863,N864,N865,N866,N867,N868,N869,N870,N871,N872,N873,N874,N875,N876,N877,N878,N879, \\note fields as indicated
  N880,N881,N882,N883,N884,N885,N886,N887,N888,N889,N890,N891,N892,N893,N894,N895,N896,N897,N898,N899, \\note fields as indicated
  N900,N901,N902,N903,N904,N905,N906,N907,N908,N909,N910,N911,N912,N913,N914,N915,N916,N917,N918,N919, \\note fields as indicated
  N920,N921,N922,N923,N924,N925,N926,N927,N928,N929,N930,N931,N932,N933,N934,N935,N936,N937,N938,N939, \\note fields as indicated
  N940,N941,N942,N943,N944,N945,N946,N947,N948,N949,N950,N951,N952,N953,N954,N955,N956,N957,N958,N959, \\note fields as indicated
  N960,N961,N962,N963,N964,N965,N966,N967,N968,N969,N970,N971,N972,N973,N974,N975,N976,N977,N978,N979, \\note fields as indicated
  N980,N981,N982,N983,N984,N985,N986,N987,N988,N989,N990,N991,N992,N993,N994,N995,N996,N997,N998,N999, \\note fields as indicated
  N1000,N1001,N1002,N1003,N1004,N1005,N1006,N1007,N1008,N1009,N1010,N1011,N1012,N1013,N1014,N1015,N1016,N1017,N1018,N1019, \\note fields as indicated
  N1020,N1021,N1022,N1023,N1024,N1025,N1026,N1027,N1028,N1029,N1030,N1031,N1032,N1033,N1034,N1035,N1036,N1037,N1038,N1039, \\note fields as indicated
  N1040,N1041,N1042,N1043,N1044,N1045,N1046,N1047,N1048,N1049,N1050,N1051,N1052,N1053,N1054,N1055,N1056,N1057,N1058,N1059, \\note fields as indicated
  N1060,N1061,N1062,N1063,N1064,N1065,N1066,N1067,N1068,N1069,N1070,N1071,N1072,N1073,N1074,N1075,N1076,N1077,N1078,N1079, \\note fields as indicated
  N1080,N1081,N1082,N1083,N1084,N1085,N1086,N1087,N1088,N1089,N1090,N1091,N1092,N1093,N1094,N1095,N1096,N1097,N1098,N1099, \\note fields as indicated
  N1100,N1101,N1102,N1103,N1104,N1105,N1106,N1107,N1108,N1109,N1110,N1111,N1112,N1113,N1114,N1115,N1116,N1117,N1118,N1119, \\note fields as indicated
  N1120,N1121,N1122,N1123,N1124,N1125,N1126,N1127,N1128,N1129,N1130,N1131,N1132,N1133,N1134,N1135,N1136,N1137,N1138,N1139, \\note fields as indicated
  N1140,N1141,N1142,N1143,N1144,N1145,N1146,N1147,N1148,N1149,N1150,N1151,N1152,N1153,N1154,N1155,N1156,N1157,N1158,N1159, \\note fields as indicated
  N1160,N1161,N1162,N1163,N1164,N1165,N1166,N1167,N1168,N1169,N1170,N1171,N1172,N1173,N1174,N1175,N1176,N1177,N1178,N1179, \\note fields as indicated
  N1180,N1181,N1182,N1183,N1184,N1185,N1186,N1187,N1188,N1189,N1190,N1191,N1192,N1193,N1194,N1195,N1196,N1197,N1198,N1199, \\note fields as indicated
  N1200,N1201,N1202,N1203,N1204,N1205,N1206,N1207,N1208,N1209,N1210,N1211,N1212,N1213,N1214,N1215,N1216,N1217,N1218,N1219, \\note fields as indicated
  N1220,N1221,N1222,N1223,N1224,N1225,N1226,N1227,N1228,N1229,N1230,N1231,N1232,N1233,N1234,N1235,N1236,N1237,N1238,N1239, \\note fields as indicated
  N1240,N1241,N1242,N1243,N1244,N1245,N1246,N1247,N1248,N1249,N1250,N1251,N1252,N1253,N1254,N1255,N1256,N1257,N1258,N1259, \\note fields as indicated
  N1260,N1261,N1262,N1263,N1264,N1265,N1266,N1267,N1268,N1269,N1270,N1271,N1272,N1273,N1274,N1275,N1276,N1277,N1278,N1279, \\note fields as indicated
  N1280,N1281,N1282,N1283,N1284,N1285,N1286,N1287,N1288,N1289,N1290,N1291,N1292,N1293,N1294,N1295,N1296,N1297,N1298,N1299, \\note fields as indicated
  N1300,N1301,N1302,N1303,N1304,N1305,N1306,N1307,N1308,N1309,N1310,N1311,N1312,N1313,N1314,N1315,N1316,N1317,N1318,N1319, \\note fields as indicated
  N1320,N1321,N1322,N1323,N1324,N1325,N1326,N1327,N1328,N1329,N1330,N1331,N1332,N1333,N1334,N1335,N1336,N1337,N1338,N1339, \\note fields as indicated
  N1340,N1341,N1342,N1343,N1344,N1345,N1346,N1347,N1348,N1349,N1350,N1351,N1352,N1353,N1354,N1355,N1356,N1357,N1358,N1359, \\note fields as indicated
  N1360,N1361,N1362,N1363,N1364,N1365,N1366,N1367,N1368,N1369,N1370,N1371,N1372,N1373,N1374,N1375,N1376,N1377,N1378,N1379, \\note fields as indicated
  N1380,N1381,N1382,N1383,N1384,N1385,N1386,N1387,N1388,N1389,N1390,N1391,N1392,N1393,N1394,N1395,N1396,N1397,N1398,N1399, \\note fields as indicated
  N1400,N1401,N1402,N1403,N1404,N1405,N1406,N1407,N1408,N1409,N1410,N1411,N1412,N1413,N1414,N1415,N1416,N1417,N1418,N1419, \\note fields as indicated
  N1420,N1421,N1422,N1423,N1424,N1425,N1426,N1427,N1428,N1429,N1430,N1431,N1432,N1433,N1434,N1435,N1436,N1437,N1438,N1439, \\note fields as indicated
  N1440,N1441,N1442,N1443,N1444,N1445,N1446,N1447,N1448,N1449,N1450,N1451,N1452,N1453,N1454,N1455,N1456,N1457,N1458,N1459, \\note fields as indicated
  N1460,N1461,N1462,N1463,N1464,N1465,N1466,N1467,N1468,N1469,N1470,N1471,N1472,N1473,N1474,N1475,N1476,N1477,N1478,N1479, \\note fields as indicated
  N1480,N1481,N1482,N1483,N1484,N1485,N1486,N1487,N1488,N1489,N1490,N1491,N1492,N1493,N1494,N1495,N1496,N1497,N1498,N1499, \\note fields as indicated
  N1500,N1501,N1502,N1503,N1504,N1505,N1506,N1507,N1508,N1509,N1510,N1511,N1512,N1513,N1514,N1515,N1516,N1517,N1518,N1519, \\note fields as indicated
  N1520,N1521,N1522,N1523,N1524,N1525,N1526,N1527,N1528,N1529,N1530,N1531,N1532,N1533,N1534,N1535,N1536,N1537,N1538,N1539, \\note fields as indicated
  N1540,N1541,N1542,N1543,N1544,N1545,N1546,N1547,N1548,N1549,N1550,N1551,N1552,N1553,N1554,N1555,N1556,N1557,N1558,N1559, \\note fields as indicated
  N1560,N1561,N1562,N1563,N1564,N1565,N1566,N1567,N1568,N1569,N1570,N1571,N1572,N1573,N1574,N1575,N1576,N1577,N1578,N1579, \\note fields as indicated
  N1580,N1581,N1582,N1583,N1584,N1585,N1586,N1587,N1588,N1589,N1590,N1591,N1592,N1593,N1594,N1595,N1596,N1597,N1598,N1599, \\note fields as indicated
  N1600,N1601,N1602,N1603,N1604,N1605,N1606,N1607,N1608,N1609,N1610,N1611,N1612,N1613,N1614,N1615,N1616,N1617,N1618,N1619, \\note fields as indicated
  N1620,N1621,N1622,N1623,N1624,N1625,N1626,N1627,N1628,N1629,N1630,N1631,N1632,N1633,N1634,N1635,N1636,N1637,N1638,N1639, \\note fields as indicated
  N1640,N1641,N1642,N1643,N1644,N1645,N1646,N1647,N1648,N1649,N1650,N1651,N1652,N1653,N1654,N1655,N1656,N1657,N1658,N1659, \\note fields as indicated
  N1660,N1661,N1662,N1663,N1664,N1665,N1666,N1667,N1668,N1669,N1670,N1671,N1672,N1673,N1674,N1675,N1676,N1677,N1678,N1679, \\note fields as indicated
  N1680,N1681,N1682,N1683,N1684,N1685,N1686,N1687,N1688,N1689,N1690,N1691,N1692,N1693,N1694,N1695,N1696,N1697,N1698,N1699, \\note fields as indicated
  N1700,N1701,N1702,N1703,N1704,N1705,N1706,N1707,N1708,N1709,N1710,N1711,N1712,N1713,N1714,N1715,N1716,N1717,N1718,N1719, \\note fields as indicated
  N1720,N1721,N1722,N1723,N1724,N1725,N1726,N1727,N1728,N1729,N1730,N1731,N1732,N1733,N1734,N1735,N1736,N1737,N1738,N1739, \\note fields as indicated
  N1740,N1741,N1742,N1743,N1744,N1745,N1746,N1747,N1748,N1749,N1750,N1751,N1752,N1753,N1754,N1755,N1756,N1757,N1758,N1759, \\note fields as indicated
  N1760,N1761,N1762,N1763,N1764,N1765,N1766,N1767,N1768,N1769,N1770,N1771,N1772,N1773,N1774,N1775,N1776,N1777,N1778,N1779, \\note fields as indicated
  N1780,N1781,N1782,N1783,N1784,N1785,N1786,N1787,N1788,N1789,N1790,N1791,N1792,N1793,N1794,N1795,N1796,N1797,N1798,N1799, \\note fields as indicated
  N1800,N1801,N1802,N1803,N1804,N1805,N1806,N1807,N1808,N1809,N1810,N1811,N1812,N1813,N1814,N1815,N1816,N1817,N1818,N1819, \\note fields as indicated
  N1820,N1821,N1822,N1823,N1824,N1825,N1826,N1827,N1828,N1829,N1830,N1831,N1832,N1833,N1834,N1835,N1836,N1837,N1838,N1839, \\note fields as indicated
  N1840,N1841,N1842,N1843,N1844,N1845,N1846,N1847,N1848,N1849,N1850,N1851,N1852,N1853,N1854,N1855,N1856,N1857,N1858,N1859, \\note fields as indicated
  N1860,N1861,N1862,N1863,N1864,N1865,N1866,N1867,N1868,N1869,N1870,N1871,N1872,N1873,N1874,N1875,N1876,N1877,N1878,N1879, \\note fields as indicated
  N1880,N1881,N1882,N1883,N1884,N1885,N1886,N1887,N1888,N1889,N1890,N1891,N1892,N1893,N1894,N1895,N1896,N1897,N1898,N1899, \\note fields as indicated
  N1900,N1901,N1902,N1903,N1904,N1905,N1906,N1907,N1908,N1909,N1910,N1911,N1912,N1913,N1914,N1915,N1916,N1917,N1918,N1919, \\note fields as indicated
  N1920,N1921,N1922,N1923,N1924,N1925,N1926,N1927,N1928,N1929,N1930,N1931,N1932,N1933,N1934,N1935,N1936,N1937,N1938,N1939, \\note fields as indicated
  N1940,N1941,N1942,N1943,N1944,N1945,N1946,N1947,N1948,N1949,N1950,N1951,N1952,N1953,N1954,N1955,N1956,N1957,N1958,N1959, \\note fields as indicated
  N1960,N1961,N1962,N1963,N1964,N1965,N1966,N1967,N1968,N1969,N1970,N1971,N1972,N1973,N1974,N1975,N1976,N1977,N1978,N1979, \\note fields as indicated
  N1980,N1981,N1982,N1983,N1984,N1985,N1986,N1987,N1988,N1989,N1990,N1991,N1992,N1993,N1994,N1995,N1996,N1997,N1998,N1999, \\note fields as indicated
  N2000,N2001,N2002,N2003,N2004,N2005,N2006,N2007,N2008,N2009,N2010,N2011,N2012,N2013,N2014,N2015,N2016,N2017,N2018,N2019, \\note fields as indicated
  N2020,N2021,N2022,N2023,N2024,N2025,N2026,N2027,N2028,N2029,N2030,N2031,N2032,N2033,N2034,N2035,N2036,N2037,N2038,N2039, \\note fields as indicated
  N2040,N2041,N2042,N2043,N2044,N2045,N2046,N2047,N2048,N2049,N2050,N2051,N2052,N2053,N2054,N2055,N2056,N2057,N2058,N2059, \\note fields as indicated
  N2060,N2061,N2062,N2063,N2064,N2065,N2066,N2067,N2068,N2069,N2070,N2071,N2072,N2073,N2074,N2075,N2076,N2077,N2078,N2079, \\note fields as indicated
  N2080,N2081,N2082,N2083,N2084,N2085,N2086,N2087,N2088,N2089,N2090,N2091,N2092,N2093,N2094,N2095,N2096,N2097,N2098,N2099, \\note fields as indicated
  N2100,N2101,N2102,N2103,N2104,N2105,N2106,N2107,N2108,N2109,N2110,N2111,N2112,N2113,N2114,N2115,N2116,N2117,N2118,N2119, \\note fields as indicated
  N2120,N2121,N2122,N2123,N2124,N2125,N2126,N2127,N2128,N2129,N2130,N2131,N2132,N2133,N2134,N2135,N2136,N2137,N2138,N2139, \\note fields as indicated
  N2140,N2141,N2142,N2143,N2144,N2145,N2146,N2147,N2148,N2149,N2150,N2151,N2152,N2153,N2154,N2155,N2156,N2157,N2158,N2159, \\note fields as indicated
  N2160,N2161,N2162,N2163,N2164,N2165,N2166,N2167,N2168,N2169,N2170,N2171,N2172,N2173,N2174,N2175,N2176,N2177,N2178,N2179, \\note fields as indicated
  N2180,N2181,N2182,N2183,N2184,N2185,N2186,N2187,N2188,N2189,N2190,N2191,N2192,N2193,N2194,N2195,N2196,N2197,N2198,N2199, \\note fields as indicated
  N2200,N2201,N2202,N2203,N2204,N2205,N2206,N2207,N2208,N2209,N2210,N2211,N2212,N2213,N2214,N2215,N2216,N2217,N2218,N2219, \\note fields as indicated
  N2220,N2221,N2222,N2223,N2224,N2225,N2226,N2227,N2228,N2229,N2230,N2231,N2232,N2233,N2234,N2235,N2236,N2237,N2238,N2239, \\note fields as indicated
  N2240,N2241,N2242,N2243,N2244,N2245,N2246,N2247,N2248,N2249,N2250,N2251,N2252,N2253,N2254,N2255,N2256,N2257,N2258,N2259, \\note fields as indicated
  N2260,N2261,N2262,N2263,N2264,N2265,N2266,N2267,N2268,N2269,N2270,N2271,N2272,N2273,N2274,N2275,N2276,N2277,N2278,N2279, \\note fields as indicated
  N2280,N2281,N2282,N2283,N2284,N2285,N2286,N2287,N2288,N2289,N2290,N2291,N2292,N2293,N2294,N2295,N2296,N2297,N2298,N2299, \\note fields as indicated
  N2300,N2301,N2302,N2303,N2304,N2305,N2306,N2307,N2308,N2309,N2310,N2311,N2312,N2313,N2314,N2315,N2316,N2317,N2318,N2319, \\note fields as indicated
  N2320,N2321,N2322,N2323,N2324,N2325,N2326,N2327,N2328,N2329,N2330,N2331,N2332,N2333,N2334,N2335,N2336,N2337,N2338,N2339, \\note fields as indicated
  N2340,N2341,N2342,N2343,N2344,N2345,N2346,N2347,N2348,N2349,N2350,N2351,N2352,N2353,N2354,N2355,N2356,N2357,N2358,N2359, \\note fields as indicated
  N2360,N2361,N2362,N2363,N2364,N2365,N2366,N2367,N2368,N2369,N2370,N2371,N2372,N2373,N2374,N2375,N2376,N2377,N2378,N2379, \\note fields as indicated
  N2380,N2381,N2382,N2383,N2384,N2385,N2386,N2387,N2388,N2389,N2390,N2391,N2392,N2393,N2394,N2395,N2396,N2397,N2398,N2399, \\note fields as indicated
  N2400,N2401,N2402,N2403,N2404,N2405,N2406,N2407,N2408,N2409,N2410,N2411,N2412,N2413,N2414,N2415,N2416,N2417,N2418,N2419, \\note fields as indicated
  N2420,N2421,N2422,N2423,N2424,N2425,N2426,N2427,N2428,N2429,N2430,N2431,N2432,N2433,N2434,N2435,N2436,N2437,N2438,N2439, \\note fields as indicated
  N2440,N2441,N2442,N2443,N2444,N2445,N2446,N2447,N2448,N2449,N2450,N2451,N2452,N2453,N2454,N2455,N2456,N2457,N2458,N2459, \\note fields as indicated
  N2460,N2461,N2462,N2463,N2464,N2465,N2466,N2467,N2468,N2469,N2470,N2471,N2472,N2473,N2474,N2475,N2476,N2477,N2478,N2479, \\note fields as indicated
  N2480,N2481,N2482,N2483,N2484,N2485,N2486,N2487,N2488,N2489,N2490,N2491,N2492,N2493,N2494,N2495,N2496,N2497,N2498,N2499, \\note fields as indicated
  N2500,N2501,N2502,N2503,N2504,N2505,N2506,N2507,N2508,N2509,N2510,N2511,N2512,N2513,N2514,N2515,N2516,N2517,N2518,N2519, \\note fields as indicated
  N2520,N2521,N2522,N2523,N2524,N2525,N2526,N2527,N2528,N2529,N2530,N2531,N2532,N2533,N2534,N2535,N2536,N2537,N2538,N2539, \\note fields as indicated
  N2540,N2541,N2542,N2543,N2544,N2545,N2546,N2547,N2548,N2549,N2550,N2551,N2552,N2553,N2554,N2555,N2556,N2557,N2558,N2559, \\note fields as indicated
  N2560,N2561,N2562,N2563,N2564,N2565,N2566,N2567,N2568,N2569,N2570,N2571,N2572,N2573,N2574,N2575,N2576,N2577,N2578,N2579, \\note fields as indicated
  N2580,N2581,N2582,N2583,N2584,N2585,N2586,N2587,N2588,N2589,N2590,N2591,N2592,N2593,N2594,N2595,N2596,N2597,N2598,N2599, \\note fields as indicated
  N2600,N2601,N2602,N2603,N2604,N2605,N2606,N2607,N2608,N2609,N2610,N2611,N2612,N2613,N2614,N2615,N2616,N2617,N2618,N2619, \\note fields as indicated
  N2620,N2621,N2622,N2623,N2624,N2625,N2626,N2627,N2628,N2629,N2630,N2631,N2632,N2633,N2634,N2635,N2636,N2637,N2638,N2639, \\note fields as indicated
  N2640,N2641,N2642,N2643,N2644,N2645,N2646,N2647,N2648,N2649,N2650,N2651,N2652,N2653,N2654,N2655,N2656,N2657,N2658,N2659, \\note fields as indicated
  N2660,N2661,N2662,N2663,N2664,N2665,N2666,N2667,N2668,N2669,N2670,N2671,N2672,N2673,N2674,N2675,N2676,N2677,N2678,N2679, \\note fields as indicated
  N2680,N2681,N2682,N2683,N2684,N2685,N2686,N2687,N2688,N2689,N2690,N2691,N2692,N2693,N2694,N2695,N2696,N2697,N2698,N2699, \\note fields as indicated
  N2700,N2701,N2702,N2703,N2704,N2705,N2706,N2707,N2708,N2709,N2710,N2711,N2712,N2713,N2714,N2715,N2716,N2717,N2718,N2719, \\note fields as indicated
  N2720,N2721,N2722,N2723,N2724,N2725,N2726,N2727,N2728,N2729,N2730,N2731,N2732,N2733,N2734,N2735,N2736,N2737,N2738,N2739, \\note fields as indicated
  N2740,N2741,N2742,N2743,N2744,N2745,N2746,N2747,N2748,N2749,N2750,N2751,N2752,N2753,N2754,N2755,N2756,N2757,N2758,N2759, \\note fields as indicated
  N2760,N2761,N2762,N2763,N2764,N2765,N2766,N2767,N2768,N2769,N2770,N2771,N2772,N2773,N2774,N2775,N2776,N2777,N2778,N2779, \\note fields as indicated
  N2780,N2781,N2782,N2783,N2784,N2785,N2786,N2787,N2788,N2789,N2790,N2791,N2792,N2793,N2794,N2795,N2796,N2797,N2798,N2799, \\note fields as indicated
  N2800,N2801,N2802,N2803,N2804,N2805,N2806,N2807,N2808,N2809,N2810,N2811,N2812,N2813,N2814,N2815,N2816,N2817,N2818,N2819, \\note fields as indicated
  N2820,N2821,N2822,N2823,N2824,N2825,N2826,N2827,N2828,N2829,N2830,N2831,N2832,N2833,N2834,N2835,N2836,N2837,N2838,N2839, \\note fields as indicated
  N2840,N2841,N2842,N2843,N2844,N2845,N2846,N2847,N2848,N2849,N2850,N2851,N2852,N2853,N2854,N2855,N2856,N2857,N2858,N2859, \\note fields as indicated
  N2860,N2861,N2862,N2863,N2864,N2865,N2866,N2867,N2868,N2869,N2870,N2871,N2872,N2873,N2874,N2875,N2876,N2877,N2878,N2879, \\note fields as indicated
  N2880,N2881,N2882,N2883,N2884,N2885,N2886,N2887,N2888,N2889,N2890,N2891,N2892,N2893,N2894,N2895,N2896,N2897,N2898,N2899, \\note fields as indicated
  N2900,N2901,N2902,N2903,N2904,N2905,N2906,N2907,N2908,N2909,N2910,N2911,N2912,N2913,N2914,N2915,N2916,N2917,N2918,N2919, \\note fields as indicated
  N2920,N2921,N2922,N2923,N2924,N2925,N2926,N2927,N2928,N2929,N2930,N2931,N2932,N2933,N2934,N2935,N2936,N2937,N2938,N2939, \\note fields as indicated
  N2940,N2941,N2942,N2943,N2944,N2945,N2946,N2947,N2948,N2949,N2950,N2951,N2952,N2953,N2954,N2955,N2956,N2957,N2958,N2959, \\note fields as indicated
  N2960,N2961,N2962,N2963,N2964,N2965,N2966,N2967,N2968,N2969,N2970,N2971,N2972,N2973,N2974,N2975,N2976,N2977,N2978,N2979, \\note fields as indicated
  N2980,N2981,N2982,N2983,N2984,N2985,N2986,N2987,N2988,N2989,N2990,N2991,N2992,N2993,N2994,N2995,N2996,N2997,N2998,N2999, \\note fields as indicated
  N3000,N3001,N3002,N3003,N3004,N3005,N3006,N3007,N3008,N3009,N3010,N3011,N3012,N3013,N3014,N3015,N3016,N3017,N3018,N3019, \\note fields as indicated
  N3020,N3021,N3022,N3023,N3024,N3025,N3026,N3027,N3028,N3029,N3030,N3031,N3032,N3033,N3034,N3035,N3036,N3037,N3038,N3039, \\note fields as indicated
  N3040,N3041,N3042,N3043,N3044,N3045,N3046,N3047,N3048,N3049,N3050,N3051,N3052,N3053,N3054,N3055,N3056,N3057,N3058,N3059, \\note fields as indicated
  N3060,N3061,N3062,N3063,N3064,N3065,N3066,N3067,N3068,N3069,N3070,N3071,N3072,N3073,N3074,N3075,N3076,N3077,N3078,N3079, \\note fields as indicated
  N3080,N3081,N3082,N3083,N3084,N3085,N3086,N3087,N3088,N3089,N3090,N3091,N3092,N3093,N3094,N3095,N3096,N3097,N3098,N3099; \\note fields as indicated

\\Group Fluid Properties

FluidProperties:Name,
       \\memo potential fluid name/type in the input file
       \\memo repeat this object for each fluid
  A1,  \\field Fluid Name
       \\type alpha
       \\required-field
       \\reference FluidNames
       \\reference FluidAndGlycolNames
  A2;  \\field Fluid Type
       \\required-field
       \\type choice
       \\key Refrigerant
       \\key Glycol

FluidProperties:GlycolConcentration,
       \\memo glycol and what concentration it is
  A1,  \\field Name
       \\type alpha
       \\required-field
       \\reference GlycolConcentrations
  A2,  \\field Glycol Type
       \\required-field
       \\type choice
       \\key EthyleneGlycol
       \\key PropyleneGlycol
       \\key UserDefinedGlycolType
       \\note or UserDefined Fluid (must show up as a glycol in FluidProperties:Name object)
  A3,  \\field User Defined Glycol Name
       \\type object-list
       \\object-list FluidAndGlycolNames
  N1;  \\field Glycol Concentration
       \\type real
       \\minimum 0.0
       \\maximum 1.0

FluidProperties:Temperatures,
      \\memo property values for fluid properties
      \\memo list of up to 250 temperatures, note that number of property values must match the number of properties
      \\memo in other words, there must be a one-to-one correspondence between the property values in this list and
      \\memo the actual properties list in other syntax
      \\memo degrees C (for all temperature inputs)
      \\format FluidProperty
  A1, \\field Name
      \\type alpha
      \\reference FluidPropertyTemperatures
  N1, \\field Temperature 1
      \\type real
      \\units C
  N2, \\field Temperature 2
      \\type real
      \\units C
  N3, \\field Temperature 3
      \\type real
      \\units C
  N4, \\field Temperature 4
      \\type real
      \\units C
  N5, \\field Temperature 5
      \\type real
      \\units C
  N6, \\field Temperature 6
      \\type real
      \\units C
  N7, \\field Temperature 7
      \\type real
      \\units C
  N8, \\field Temperature 8
      \\type real
      \\units C
  N9, \\field Temperature 9
      \\type real
      \\units C
  N10, \\field Temperature 10
       \\type real
       \\units C
  N11, \\field Temperature 11
       \\type real
       \\units C
  N12, \\field Temperature 12
       \\type real
       \\units C
  N13, \\field Temperature 13
       \\type real
       \\units C
  N14, \\field Temperature 14
       \\type real
       \\units C
  N15, \\field Temperature 15
       \\type real
       \\units C
  N16, \\field Temperature 16
       \\type real
       \\units C
  N17, \\field Temperature 17
       \\type real
       \\units C
  N18, \\field Temperature 18
       \\type real
       \\units C
  N19, \\field Temperature 19
       \\type real
       \\units C
  N20, \\field Temperature 20
       \\type real
       \\units C
  N21, \\field Temperature 21
       \\type real
       \\units C
  N22, \\field Temperature 22
       \\type real
       \\units C
  N23, \\field Temperature 23
       \\type real
       \\units C
  N24, \\field Temperature 24
       \\type real
       \\units C
  N25, \\field Temperature 25
       \\type real
       \\units C
  N26, \\field Temperature 26
       \\type real
       \\units C
  N27, \\field Temperature 27
       \\type real
       \\units C
  N28, \\field Temperature 28
       \\type real
       \\units C
  N29, \\field Temperature 29
       \\type real
       \\units C
  N30, \\field Temperature 30
       \\type real
       \\units C
  N31, \\field Temperature 31
       \\type real
       \\units C
  N32, \\field Temperature 32
       \\type real
       \\units C
  N33, \\field Temperature 33
       \\type real
       \\units C
  N34, \\field Temperature 34
       \\type real
       \\units C
  N35, \\field Temperature 35
       \\type real
       \\units C
  N36, \\field Temperature 36
       \\type real
       \\units C
  N37, \\field Temperature 37
       \\type real
       \\units C
  N38, \\field Temperature 38
       \\type real
       \\units C
  N39, \\field Temperature 39
       \\type real
       \\units C
  N40, \\field Temperature 40
       \\type real
       \\units C
  N41, \\field Temperature 41
       \\type real
       \\units C
  N42, \\field Temperature 42
       \\type real
       \\units C
  N43, \\field Temperature 43
       \\type real
       \\units C
  N44, \\field Temperature 44
       \\type real
       \\units C
  N45, \\field Temperature 45
       \\type real
       \\units C
  N46, \\field Temperature 46
       \\type real
       \\units C
  N47, \\field Temperature 47
       \\type real
       \\units C
  N48, \\field Temperature 48
       \\type real
       \\units C
  N49, \\field Temperature 49
       \\type real
       \\units C
  N50, \\field Temperature 50
       \\type real
       \\units C
  N51, \\field Temperature 51
       \\type real
       \\units C
  N52, \\field Temperature 52
       \\type real
       \\units C
  N53, \\field Temperature 53
       \\type real
       \\units C
  N54, \\field Temperature 54
       \\type real
       \\units C
  N55, \\field Temperature 55
       \\type real
       \\units C
  N56, \\field Temperature 56
       \\type real
       \\units C
  N57, \\field Temperature 57
       \\type real
       \\units C
  N58, \\field Temperature 58
       \\type real
       \\units C
  N59, \\field Temperature 59
       \\type real
       \\units C
  N60, \\field Temperature 60
       \\type real
       \\units C
  N61, \\field Temperature 61
       \\type real
       \\units C
  N62, \\field Temperature 62
       \\type real
       \\units C
  N63, \\field Temperature 63
       \\type real
       \\units C
  N64, \\field Temperature 64
       \\type real
       \\units C
  N65, \\field Temperature 65
       \\type real
       \\units C
  N66, \\field Temperature 66
       \\type real
       \\units C
  N67, \\field Temperature 67
       \\type real
       \\units C
  N68, \\field Temperature 68
       \\type real
       \\units C
  N69, \\field Temperature 69
       \\type real
       \\units C
  N70, \\field Temperature 70
       \\type real
       \\units C
  N71, \\field Temperature 71
       \\type real
       \\units C
  N72, \\field Temperature 72
       \\type real
       \\units C
  N73, \\field Temperature 73
       \\type real
       \\units C
  N74, \\field Temperature 74
       \\type real
       \\units C
  N75, \\field Temperature 75
       \\type real
       \\units C
  N76, \\field Temperature 76
       \\type real
       \\units C
  N77, \\field Temperature 77
       \\type real
       \\units C
  N78, \\field Temperature 78
       \\type real
       \\units C
  N79, \\field Temperature 79
       \\type real
       \\units C
  N80, \\field Temperature 80
       \\type real
       \\units C
  N81, \\field Temperature 81
       \\type real
       \\units C
  N82, \\field Temperature 82
       \\type real
       \\units C
  N83, \\field Temperature 83
       \\type real
       \\units C
  N84, \\field Temperature 84
       \\type real
       \\units C
  N85, \\field Temperature 85
       \\type real
       \\units C
  N86, \\field Temperature 86
       \\type real
       \\units C
  N87, \\field Temperature 87
       \\type real
       \\units C
  N88, \\field Temperature 88
       \\type real
       \\units C
  N89, \\field Temperature 89
       \\type real
       \\units C
  N90, \\field Temperature 90
       \\type real
       \\units C
  N91, \\field Temperature 91
       \\type real
       \\units C
  N92, \\field Temperature 92
       \\type real
       \\units C
  N93, \\field Temperature 93
       \\type real
       \\units C
  N94, \\field Temperature 94
       \\type real
       \\units C
  N95, \\field Temperature 95
       \\type real
       \\units C
  N96, \\field Temperature 96
       \\type real
       \\units C
  N97, \\field Temperature 97
       \\type real
       \\units C
  N98, \\field Temperature 98
       \\type real
       \\units C
  N99, \\field Temperature 99
       \\type real
       \\units C
  N100, \\field Temperature 100
        \\type real
        \\units C
  N101, \\field Temperature 101
        \\type real
        \\units C
  N102, \\field Temperature 102
        \\type real
        \\units C
  N103, \\field Temperature 103
        \\type real
        \\units C
  N104, \\field Temperature 104
        \\type real
        \\units C
  N105, \\field Temperature 105
        \\type real
        \\units C
  N106, \\field Temperature 106
        \\type real
        \\units C
  N107, \\field Temperature 107
        \\type real
        \\units C
  N108, \\field Temperature 108
        \\type real
        \\units C
  N109, \\field Temperature 109
        \\type real
        \\units C
  N110, \\field Temperature 110
        \\type real
        \\units C
  N111, \\field Temperature 111
        \\type real
        \\units C
  N112, \\field Temperature 112
        \\type real
        \\units C
  N113, \\field Temperature 113
        \\type real
        \\units C
  N114, \\field Temperature 114
        \\type real
        \\units C
  N115, \\field Temperature 115
        \\type real
        \\units C
  N116, \\field Temperature 116
        \\type real
        \\units C
  N117, \\field Temperature 117
        \\type real
        \\units C
  N118, \\field Temperature 118
        \\type real
        \\units C
  N119, \\field Temperature 119
        \\type real
        \\units C
  N120, \\field Temperature 120
        \\type real
        \\units C
  N121, \\field Temperature 121
        \\type real
        \\units C
  N122, \\field Temperature 122
        \\type real
        \\units C
  N123, \\field Temperature 123
        \\type real
        \\units C
  N124, \\field Temperature 124
        \\type real
        \\units C
  N125, \\field Temperature 125
        \\type real
        \\units C
  N126, \\field Temperature 126
        \\type real
        \\units C
  N127, \\field Temperature 127
        \\type real
        \\units C
  N128, \\field Temperature 128
        \\type real
        \\units C
  N129, \\field Temperature 129
        \\type real
        \\units C
  N130, \\field Temperature 130
        \\type real
        \\units C
  N131, \\field Temperature 131
        \\type real
        \\units C
  N132, \\field Temperature 132
        \\type real
        \\units C
  N133, \\field Temperature 133
        \\type real
        \\units C
  N134, \\field Temperature 134
        \\type real
        \\units C
  N135, \\field Temperature 135
        \\type real
        \\units C
  N136, \\field Temperature 136
        \\type real
        \\units C
  N137, \\field Temperature 137
        \\type real
        \\units C
  N138, \\field Temperature 138
        \\type real
        \\units C
  N139, \\field Temperature 139
        \\type real
        \\units C
  N140, \\field Temperature 140
        \\type real
        \\units C
  N141, \\field Temperature 141
        \\type real
        \\units C
  N142, \\field Temperature 142
        \\type real
        \\units C
  N143, \\field Temperature 143
        \\type real
        \\units C
  N144, \\field Temperature 144
        \\type real
        \\units C
  N145, \\field Temperature 145
        \\type real
        \\units C
  N146, \\field Temperature 146
        \\type real
        \\units C
  N147, \\field Temperature 147
        \\type real
        \\units C
  N148, \\field Temperature 148
        \\type real
        \\units C
  N149, \\field Temperature 149
        \\type real
        \\units C
  N150, \\field Temperature 150
        \\type real
        \\units C
  N151, \\field Temperature 151
        \\type real
        \\units C
  N152, \\field Temperature 152
        \\type real
        \\units C
  N153, \\field Temperature 153
        \\type real
        \\units C
  N154, \\field Temperature 154
        \\type real
        \\units C
  N155, \\field Temperature 155
        \\type real
        \\units C
  N156, \\field Temperature 156
        \\type real
        \\units C
  N157, \\field Temperature 157
        \\type real
        \\units C
  N158, \\field Temperature 158
        \\type real
        \\units C
  N159, \\field Temperature 159
        \\type real
        \\units C
  N160, \\field Temperature 160
        \\type real
        \\units C
  N161, \\field Temperature 161
        \\type real
        \\units C
  N162, \\field Temperature 162
        \\type real
        \\units C
  N163, \\field Temperature 163
        \\type real
        \\units C
  N164, \\field Temperature 164
        \\type real
        \\units C
  N165, \\field Temperature 165
        \\type real
        \\units C
  N166, \\field Temperature 166
        \\type real
        \\units C
  N167, \\field Temperature 167
        \\type real
        \\units C
  N168, \\field Temperature 168
        \\type real
        \\units C
  N169, \\field Temperature 169
        \\type real
        \\units C
  N170, \\field Temperature 170
        \\type real
        \\units C
  N171, \\field Temperature 171
        \\type real
        \\units C
  N172, \\field Temperature 172
        \\type real
        \\units C
  N173, \\field Temperature 173
        \\type real
        \\units C
  N174, \\field Temperature 174
        \\type real
        \\units C
  N175, \\field Temperature 175
        \\type real
        \\units C
  N176, \\field Temperature 176
        \\type real
        \\units C
  N177, \\field Temperature 177
        \\type real
        \\units C
  N178, \\field Temperature 178
        \\type real
        \\units C
  N179, \\field Temperature 179
        \\type real
        \\units C
  N180, \\field Temperature 180
        \\type real
        \\units C
  N181, \\field Temperature 181
        \\type real
        \\units C
  N182, \\field Temperature 182
        \\type real
        \\units C
  N183, \\field Temperature 183
        \\type real
        \\units C
  N184, \\field Temperature 184
        \\type real
        \\units C
  N185, \\field Temperature 185
        \\type real
        \\units C
  N186, \\field Temperature 186
        \\type real
        \\units C
  N187, \\field Temperature 187
        \\type real
        \\units C
  N188, \\field Temperature 188
        \\type real
        \\units C
  N189, \\field Temperature 189
        \\type real
        \\units C
  N190, \\field Temperature 190
        \\type real
        \\units C
  N191, \\field Temperature 191
        \\type real
        \\units C
  N192, \\field Temperature 192
        \\type real
        \\units C
  N193, \\field Temperature 193
        \\type real
        \\units C
  N194, \\field Temperature 194
        \\type real
        \\units C
  N195, \\field Temperature 195
        \\type real
        \\units C
  N196, \\field Temperature 196
        \\type real
        \\units C
  N197, \\field Temperature 197
        \\type real
        \\units C
  N198, \\field Temperature 198
        \\type real
        \\units C
  N199, \\field Temperature 199
        \\type real
        \\units C
  N200, \\field Temperature 200
        \\type real
        \\units C
  N201, \\field Temperature 201
        \\type real
        \\units C
  N202, \\field Temperature 202
        \\type real
        \\units C
  N203, \\field Temperature 203
        \\type real
        \\units C
  N204, \\field Temperature 204
        \\type real
        \\units C
  N205, \\field Temperature 205
        \\type real
        \\units C
  N206, \\field Temperature 206
        \\type real
        \\units C
  N207, \\field Temperature 207
        \\type real
        \\units C
  N208, \\field Temperature 208
        \\type real
        \\units C
  N209, \\field Temperature 209
        \\type real
        \\units C
  N210, \\field Temperature 210
        \\type real
        \\units C
  N211, \\field Temperature 211
        \\type real
        \\units C
  N212, \\field Temperature 212
        \\type real
        \\units C
  N213, \\field Temperature 213
        \\type real
        \\units C
  N214, \\field Temperature 214
        \\type real
        \\units C
  N215, \\field Temperature 215
        \\type real
        \\units C
  N216, \\field Temperature 216
        \\type real
        \\units C
  N217, \\field Temperature 217
        \\type real
        \\units C
  N218, \\field Temperature 218
        \\type real
        \\units C
  N219, \\field Temperature 219
        \\type real
        \\units C
  N220, \\field Temperature 220
        \\type real
        \\units C
  N221, \\field Temperature 221
        \\type real
        \\units C
  N222, \\field Temperature 222
        \\type real
        \\units C
  N223, \\field Temperature 223
        \\type real
        \\units C
  N224, \\field Temperature 224
        \\type real
        \\units C
  N225, \\field Temperature 225
        \\type real
        \\units C
  N226, \\field Temperature 226
        \\type real
        \\units C
  N227, \\field Temperature 227
        \\type real
        \\units C
  N228, \\field Temperature 228
        \\type real
        \\units C
  N229, \\field Temperature 229
        \\type real
        \\units C
  N230, \\field Temperature 230
        \\type real
        \\units C
  N231, \\field Temperature 231
        \\type real
        \\units C
  N232, \\field Temperature 232
        \\type real
        \\units C
  N233, \\field Temperature 233
        \\type real
        \\units C
  N234, \\field Temperature 234
        \\type real
        \\units C
  N235, \\field Temperature 235
        \\type real
        \\units C
  N236, \\field Temperature 236
        \\type real
        \\units C
  N237, \\field Temperature 237
        \\type real
        \\units C
  N238, \\field Temperature 238
        \\type real
        \\units C
  N239, \\field Temperature 239
        \\type real
        \\units C
  N240, \\field Temperature 240
        \\type real
        \\units C
  N241, \\field Temperature 241
        \\type real
        \\units C
  N242, \\field Temperature 242
        \\type real
        \\units C
  N243, \\field Temperature 243
        \\type real
        \\units C
  N244, \\field Temperature 244
        \\type real
        \\units C
  N245, \\field Temperature 245
        \\type real
        \\units C
  N246, \\field Temperature 246
        \\type real
        \\units C
  N247, \\field Temperature 247
        \\type real
        \\units C
  N248, \\field Temperature 248
        \\type real
        \\units C
  N249, \\field Temperature 249
        \\type real
        \\units C
  N250; \\field Temperature 250
        \\type real
        \\units C

FluidProperties:Saturated,
      \\memo fluid properties for the saturated region
      \\format FluidProperty
  A1, \\field Name
      \\type object-list
      \\object-list FluidNames
  A2, \\field Fluid Property Type
      \\note Enthalpy Units are J/kg
      \\note Density Units are kg/m3
      \\note SpecificHeat Units are J/kg-K
      \\note Pressure Units are Pa
      \\type choice
      \\key Enthalpy     ! Units are J/kg
      \\key Density      ! Units are kg/m3
      \\key SpecificHeat ! Units are J/kg-K
      \\key Pressure     ! Units are Pa
  A3, \\field Fluid Phase
      \\note Fluid=saturated fluid
      \\note FluidGas=saturated vapor
      \\type choice
      \\key Fluid        ! saturated fluid
      \\key FluidGas     ! saturated vapor
  A4, \\field Temperature Values Name
      \\note Enter the name of a FluidProperties:Temperatures object.
      \\type object-list
      \\object-list FluidPropertyTemperatures
  N1, \\field Property Value 1
      \\type real
      \\unitsBasedOnField A2
  N2, \\field Property Value 2
      \\type real
      \\unitsBasedOnField A2
  N3, \\field Property Value 3
      \\type real
      \\unitsBasedOnField A2
  N4, \\field Property Value 4
      \\type real
      \\unitsBasedOnField A2
  N5, \\field Property Value 5
      \\type real
      \\unitsBasedOnField A2
  N6, \\field Property Value 6
      \\type real
      \\unitsBasedOnField A2
  N7, \\field Property Value 7
      \\type real
      \\unitsBasedOnField A2
  N8, \\field Property Value 8
      \\type real
      \\unitsBasedOnField A2
  N9, \\field Property Value 9
      \\type real
      \\unitsBasedOnField A2
  N10, \\field Property Value 10
       \\type real
       \\unitsBasedOnField A2
  N11, \\field Property Value 11
       \\type real
       \\unitsBasedOnField A2
  N12, \\field Property Value 12
       \\type real
       \\unitsBasedOnField A2
  N13, \\field Property Value 13
       \\type real
       \\unitsBasedOnField A2
  N14, \\field Property Value 14
       \\type real
       \\unitsBasedOnField A2
  N15, \\field Property Value 15
       \\type real
       \\unitsBasedOnField A2
  N16, \\field Property Value 16
       \\type real
       \\unitsBasedOnField A2
  N17, \\field Property Value 17
       \\type real
       \\unitsBasedOnField A2
  N18, \\field Property Value 18
       \\type real
       \\unitsBasedOnField A2
  N19, \\field Property Value 19
       \\type real
       \\unitsBasedOnField A2
  N20, \\field Property Value 20
       \\type real
       \\unitsBasedOnField A2
  N21, \\field Property Value 21
       \\type real
       \\unitsBasedOnField A2
  N22, \\field Property Value 22
       \\type real
       \\unitsBasedOnField A2
  N23, \\field Property Value 23
       \\type real
       \\unitsBasedOnField A2
  N24, \\field Property Value 24
       \\type real
       \\unitsBasedOnField A2
  N25, \\field Property Value 25
       \\type real
       \\unitsBasedOnField A2
  N26, \\field Property Value 26
       \\type real
       \\unitsBasedOnField A2
  N27, \\field Property Value 27
       \\type real
       \\unitsBasedOnField A2
  N28, \\field Property Value 28
       \\type real
       \\unitsBasedOnField A2
  N29, \\field Property Value 29
       \\type real
       \\unitsBasedOnField A2
  N30, \\field Property Value 30
       \\type real
       \\unitsBasedOnField A2
  N31, \\field Property Value 31
       \\type real
       \\unitsBasedOnField A2
  N32, \\field Property Value 32
       \\type real
       \\unitsBasedOnField A2
  N33, \\field Property Value 33
       \\type real
       \\unitsBasedOnField A2
  N34, \\field Property Value 34
       \\type real
       \\unitsBasedOnField A2
  N35, \\field Property Value 35
       \\type real
       \\unitsBasedOnField A2
  N36, \\field Property Value 36
       \\type real
       \\unitsBasedOnField A2
  N37, \\field Property Value 37
       \\type real
       \\unitsBasedOnField A2
  N38, \\field Property Value 38
       \\type real
       \\unitsBasedOnField A2
  N39, \\field Property Value 39
       \\type real
       \\unitsBasedOnField A2
  N40, \\field Property Value 40
       \\type real
       \\unitsBasedOnField A2
  N41, \\field Property Value 41
       \\type real
       \\unitsBasedOnField A2
  N42, \\field Property Value 42
       \\type real
       \\unitsBasedOnField A2
  N43, \\field Property Value 43
       \\type real
       \\unitsBasedOnField A2
  N44, \\field Property Value 44
       \\type real
       \\unitsBasedOnField A2
  N45, \\field Property Value 45
       \\type real
       \\unitsBasedOnField A2
  N46, \\field Property Value 46
       \\type real
       \\unitsBasedOnField A2
  N47, \\field Property Value 47
       \\type real
       \\unitsBasedOnField A2
  N48, \\field Property Value 48
       \\type real
       \\unitsBasedOnField A2
  N49, \\field Property Value 49
       \\type real
       \\unitsBasedOnField A2
  N50, \\field Property Value 50
       \\type real
       \\unitsBasedOnField A2
  N51, \\field Property Value 51
       \\type real
       \\unitsBasedOnField A2
  N52, \\field Property Value 52
       \\type real
       \\unitsBasedOnField A2
  N53, \\field Property Value 53
       \\type real
       \\unitsBasedOnField A2
  N54, \\field Property Value 54
       \\type real
       \\unitsBasedOnField A2
  N55, \\field Property Value 55
       \\type real
       \\unitsBasedOnField A2
  N56, \\field Property Value 56
       \\type real
       \\unitsBasedOnField A2
  N57, \\field Property Value 57
       \\type real
       \\unitsBasedOnField A2
  N58, \\field Property Value 58
       \\type real
       \\unitsBasedOnField A2
  N59, \\field Property Value 59
       \\type real
       \\unitsBasedOnField A2
  N60, \\field Property Value 60
       \\type real
       \\unitsBasedOnField A2
  N61, \\field Property Value 61
       \\type real
       \\unitsBasedOnField A2
  N62, \\field Property Value 62
       \\type real
       \\unitsBasedOnField A2
  N63, \\field Property Value 63
       \\type real
       \\unitsBasedOnField A2
  N64, \\field Property Value 64
       \\type real
       \\unitsBasedOnField A2
  N65, \\field Property Value 65
       \\type real
       \\unitsBasedOnField A2
  N66, \\field Property Value 66
       \\type real
       \\unitsBasedOnField A2
  N67, \\field Property Value 67
       \\type real
       \\unitsBasedOnField A2
  N68, \\field Property Value 68
       \\type real
       \\unitsBasedOnField A2
  N69, \\field Property Value 69
       \\type real
       \\unitsBasedOnField A2
  N70, \\field Property Value 70
       \\type real
       \\unitsBasedOnField A2
  N71, \\field Property Value 71
       \\type real
       \\unitsBasedOnField A2
  N72, \\field Property Value 72
       \\type real
       \\unitsBasedOnField A2
  N73, \\field Property Value 73
       \\type real
       \\unitsBasedOnField A2
  N74, \\field Property Value 74
       \\type real
       \\unitsBasedOnField A2
  N75, \\field Property Value 75
       \\type real
       \\unitsBasedOnField A2
  N76, \\field Property Value 76
       \\type real
       \\unitsBasedOnField A2
  N77, \\field Property Value 77
       \\type real
       \\unitsBasedOnField A2
  N78, \\field Property Value 78
       \\type real
       \\unitsBasedOnField A2
  N79, \\field Property Value 79
       \\type real
       \\unitsBasedOnField A2
  N80, \\field Property Value 80
       \\type real
       \\unitsBasedOnField A2
  N81, \\field Property Value 81
       \\type real
       \\unitsBasedOnField A2
  N82, \\field Property Value 82
       \\type real
       \\unitsBasedOnField A2
  N83, \\field Property Value 83
       \\type real
       \\unitsBasedOnField A2
  N84, \\field Property Value 84
       \\type real
       \\unitsBasedOnField A2
  N85, \\field Property Value 85
       \\type real
       \\unitsBasedOnField A2
  N86, \\field Property Value 86
       \\type real
       \\unitsBasedOnField A2
  N87, \\field Property Value 87
       \\type real
       \\unitsBasedOnField A2
  N88, \\field Property Value 88
       \\type real
       \\unitsBasedOnField A2
  N89, \\field Property Value 89
       \\type real
       \\unitsBasedOnField A2
  N90, \\field Property Value 90
       \\type real
       \\unitsBasedOnField A2
  N91, \\field Property Value 91
       \\type real
       \\unitsBasedOnField A2
  N92, \\field Property Value 92
       \\type real
       \\unitsBasedOnField A2
  N93, \\field Property Value 93
       \\type real
       \\unitsBasedOnField A2
  N94, \\field Property Value 94
       \\type real
       \\unitsBasedOnField A2
  N95, \\field Property Value 95
       \\type real
       \\unitsBasedOnField A2
  N96, \\field Property Value 96
       \\type real
       \\unitsBasedOnField A2
  N97, \\field Property Value 97
       \\type real
       \\unitsBasedOnField A2
  N98, \\field Property Value 98
       \\type real
       \\unitsBasedOnField A2
  N99, \\field Property Value 99
       \\type real
       \\unitsBasedOnField A2
  N100, \\field Property Value 100
        \\type real
        \\unitsBasedOnField A2
  N101, \\field Property Value 101
        \\type real
        \\unitsBasedOnField A2
  N102, \\field Property Value 102
        \\type real
        \\unitsBasedOnField A2
  N103, \\field Property Value 103
        \\type real
        \\unitsBasedOnField A2
  N104, \\field Property Value 104
        \\type real
        \\unitsBasedOnField A2
  N105, \\field Property Value 105
        \\type real
        \\unitsBasedOnField A2
  N106, \\field Property Value 106
        \\type real
        \\unitsBasedOnField A2
  N107, \\field Property Value 107
        \\type real
        \\unitsBasedOnField A2
  N108, \\field Property Value 108
        \\type real
        \\unitsBasedOnField A2
  N109, \\field Property Value 109
        \\type real
        \\unitsBasedOnField A2
  N110, \\field Property Value 110
        \\type real
        \\unitsBasedOnField A2
  N111, \\field Property Value 111
        \\type real
        \\unitsBasedOnField A2
  N112, \\field Property Value 112
        \\type real
        \\unitsBasedOnField A2
  N113, \\field Property Value 113
        \\type real
        \\unitsBasedOnField A2
  N114, \\field Property Value 114
        \\type real
        \\unitsBasedOnField A2
  N115, \\field Property Value 115
        \\type real
        \\unitsBasedOnField A2
  N116, \\field Property Value 116
        \\type real
        \\unitsBasedOnField A2
  N117, \\field Property Value 117
        \\type real
        \\unitsBasedOnField A2
  N118, \\field Property Value 118
        \\type real
        \\unitsBasedOnField A2
  N119, \\field Property Value 119
        \\type real
        \\unitsBasedOnField A2
  N120, \\field Property Value 120
        \\type real
        \\unitsBasedOnField A2
  N121, \\field Property Value 121
        \\type real
        \\unitsBasedOnField A2
  N122, \\field Property Value 122
        \\type real
        \\unitsBasedOnField A2
  N123, \\field Property Value 123
        \\type real
        \\unitsBasedOnField A2
  N124, \\field Property Value 124
        \\type real
        \\unitsBasedOnField A2
  N125, \\field Property Value 125
        \\type real
        \\unitsBasedOnField A2
  N126, \\field Property Value 126
        \\type real
        \\unitsBasedOnField A2
  N127, \\field Property Value 127
        \\type real
        \\unitsBasedOnField A2
  N128, \\field Property Value 128
        \\type real
        \\unitsBasedOnField A2
  N129, \\field Property Value 129
        \\type real
        \\unitsBasedOnField A2
  N130, \\field Property Value 130
        \\type real
        \\unitsBasedOnField A2
  N131, \\field Property Value 131
        \\type real
        \\unitsBasedOnField A2
  N132, \\field Property Value 132
        \\type real
        \\unitsBasedOnField A2
  N133, \\field Property Value 133
        \\type real
        \\unitsBasedOnField A2
  N134, \\field Property Value 134
        \\type real
        \\unitsBasedOnField A2
  N135, \\field Property Value 135
        \\type real
        \\unitsBasedOnField A2
  N136, \\field Property Value 136
        \\type real
        \\unitsBasedOnField A2
  N137, \\field Property Value 137
        \\type real
        \\unitsBasedOnField A2
  N138, \\field Property Value 138
        \\type real
        \\unitsBasedOnField A2
  N139, \\field Property Value 139
        \\type real
        \\unitsBasedOnField A2
  N140, \\field Property Value 140
        \\type real
        \\unitsBasedOnField A2
  N141, \\field Property Value 141
        \\type real
        \\unitsBasedOnField A2
  N142, \\field Property Value 142
        \\type real
        \\unitsBasedOnField A2
  N143, \\field Property Value 143
        \\type real
        \\unitsBasedOnField A2
  N144, \\field Property Value 144
        \\type real
        \\unitsBasedOnField A2
  N145, \\field Property Value 145
        \\type real
        \\unitsBasedOnField A2
  N146, \\field Property Value 146
        \\type real
        \\unitsBasedOnField A2
  N147, \\field Property Value 147
        \\type real
        \\unitsBasedOnField A2
  N148, \\field Property Value 148
        \\type real
        \\unitsBasedOnField A2
  N149, \\field Property Value 149
        \\type real
        \\unitsBasedOnField A2
  N150, \\field Property Value 150
        \\type real
        \\unitsBasedOnField A2
  N151, \\field Property Value 151
        \\type real
        \\unitsBasedOnField A2
  N152, \\field Property Value 152
        \\type real
        \\unitsBasedOnField A2
  N153, \\field Property Value 153
        \\type real
        \\unitsBasedOnField A2
  N154, \\field Property Value 154
        \\type real
        \\unitsBasedOnField A2
  N155, \\field Property Value 155
        \\type real
        \\unitsBasedOnField A2
  N156, \\field Property Value 156
        \\type real
        \\unitsBasedOnField A2
  N157, \\field Property Value 157
        \\type real
        \\unitsBasedOnField A2
  N158, \\field Property Value 158
        \\type real
        \\unitsBasedOnField A2
  N159, \\field Property Value 159
        \\type real
        \\unitsBasedOnField A2
  N160, \\field Property Value 160
        \\type real
        \\unitsBasedOnField A2
  N161, \\field Property Value 161
        \\type real
        \\unitsBasedOnField A2
  N162, \\field Property Value 162
        \\type real
        \\unitsBasedOnField A2
  N163, \\field Property Value 163
        \\type real
        \\unitsBasedOnField A2
  N164, \\field Property Value 164
        \\type real
        \\unitsBasedOnField A2
  N165, \\field Property Value 165
        \\type real
        \\unitsBasedOnField A2
  N166, \\field Property Value 166
        \\type real
        \\unitsBasedOnField A2
  N167, \\field Property Value 167
        \\type real
        \\unitsBasedOnField A2
  N168, \\field Property Value 168
        \\type real
        \\unitsBasedOnField A2
  N169, \\field Property Value 169
        \\type real
        \\unitsBasedOnField A2
  N170, \\field Property Value 170
        \\type real
        \\unitsBasedOnField A2
  N171, \\field Property Value 171
        \\type real
        \\unitsBasedOnField A2
  N172, \\field Property Value 172
        \\type real
        \\unitsBasedOnField A2
  N173, \\field Property Value 173
        \\type real
        \\unitsBasedOnField A2
  N174, \\field Property Value 174
        \\type real
        \\unitsBasedOnField A2
  N175, \\field Property Value 175
        \\type real
        \\unitsBasedOnField A2
  N176, \\field Property Value 176
        \\type real
        \\unitsBasedOnField A2
  N177, \\field Property Value 177
        \\type real
        \\unitsBasedOnField A2
  N178, \\field Property Value 178
        \\type real
        \\unitsBasedOnField A2
  N179, \\field Property Value 179
        \\type real
        \\unitsBasedOnField A2
  N180, \\field Property Value 180
        \\type real
        \\unitsBasedOnField A2
  N181, \\field Property Value 181
        \\type real
        \\unitsBasedOnField A2
  N182, \\field Property Value 182
        \\type real
        \\unitsBasedOnField A2
  N183, \\field Property Value 183
        \\type real
        \\unitsBasedOnField A2
  N184, \\field Property Value 184
        \\type real
        \\unitsBasedOnField A2
  N185, \\field Property Value 185
        \\type real
        \\unitsBasedOnField A2
  N186, \\field Property Value 186
        \\type real
        \\unitsBasedOnField A2
  N187, \\field Property Value 187
        \\type real
        \\unitsBasedOnField A2
  N188, \\field Property Value 188
        \\type real
        \\unitsBasedOnField A2
  N189, \\field Property Value 189
        \\type real
        \\unitsBasedOnField A2
  N190, \\field Property Value 190
        \\type real
        \\unitsBasedOnField A2
  N191, \\field Property Value 191
        \\type real
        \\unitsBasedOnField A2
  N192, \\field Property Value 192
        \\type real
        \\unitsBasedOnField A2
  N193, \\field Property Value 193
        \\type real
        \\unitsBasedOnField A2
  N194, \\field Property Value 194
        \\type real
        \\unitsBasedOnField A2
  N195, \\field Property Value 195
        \\type real
        \\unitsBasedOnField A2
  N196, \\field Property Value 196
        \\type real
        \\unitsBasedOnField A2
  N197, \\field Property Value 197
        \\type real
        \\unitsBasedOnField A2
  N198, \\field Property Value 198
        \\type real
        \\unitsBasedOnField A2
  N199, \\field Property Value 199
        \\type real
        \\unitsBasedOnField A2
  N200, \\field Property Value 200
        \\type real
        \\unitsBasedOnField A2
  N201, \\field Property Value 201
        \\type real
        \\unitsBasedOnField A2
  N202, \\field Property Value 202
        \\type real
        \\unitsBasedOnField A2
  N203, \\field Property Value 203
        \\type real
        \\unitsBasedOnField A2
  N204, \\field Property Value 204
        \\type real
        \\unitsBasedOnField A2
  N205, \\field Property Value 205
        \\type real
        \\unitsBasedOnField A2
  N206, \\field Property Value 206
        \\type real
        \\unitsBasedOnField A2
  N207, \\field Property Value 207
        \\type real
        \\unitsBasedOnField A2
  N208, \\field Property Value 208
        \\type real
        \\unitsBasedOnField A2
  N209, \\field Property Value 209
        \\type real
        \\unitsBasedOnField A2
  N210, \\field Property Value 210
        \\type real
        \\unitsBasedOnField A2
  N211, \\field Property Value 211
        \\type real
        \\unitsBasedOnField A2
  N212, \\field Property Value 212
        \\type real
        \\unitsBasedOnField A2
  N213, \\field Property Value 213
        \\type real
        \\unitsBasedOnField A2
  N214, \\field Property Value 214
        \\type real
        \\unitsBasedOnField A2
  N215, \\field Property Value 215
        \\type real
        \\unitsBasedOnField A2
  N216, \\field Property Value 216
        \\type real
        \\unitsBasedOnField A2
  N217, \\field Property Value 217
        \\type real
        \\unitsBasedOnField A2
  N218, \\field Property Value 218
        \\type real
        \\unitsBasedOnField A2
  N219, \\field Property Value 219
        \\type real
        \\unitsBasedOnField A2
  N220, \\field Property Value 220
        \\type real
        \\unitsBasedOnField A2
  N221, \\field Property Value 221
        \\type real
        \\unitsBasedOnField A2
  N222, \\field Property Value 222
        \\type real
        \\unitsBasedOnField A2
  N223, \\field Property Value 223
        \\type real
        \\unitsBasedOnField A2
  N224, \\field Property Value 224
        \\type real
        \\unitsBasedOnField A2
  N225, \\field Property Value 225
        \\type real
        \\unitsBasedOnField A2
  N226, \\field Property Value 226
        \\type real
        \\unitsBasedOnField A2
  N227, \\field Property Value 227
        \\type real
        \\unitsBasedOnField A2
  N228, \\field Property Value 228
        \\type real
        \\unitsBasedOnField A2
  N229, \\field Property Value 229
        \\type real
        \\unitsBasedOnField A2
  N230, \\field Property Value 230
        \\type real
        \\unitsBasedOnField A2
  N231, \\field Property Value 231
        \\type real
        \\unitsBasedOnField A2
  N232, \\field Property Value 232
        \\type real
        \\unitsBasedOnField A2
  N233, \\field Property Value 233
        \\type real
        \\unitsBasedOnField A2
  N234, \\field Property Value 234
        \\type real
        \\unitsBasedOnField A2
  N235, \\field Property Value 235
        \\type real
        \\unitsBasedOnField A2
  N236, \\field Property Value 236
        \\type real
        \\unitsBasedOnField A2
  N237, \\field Property Value 237
        \\type real
        \\unitsBasedOnField A2
  N238, \\field Property Value 238
        \\type real
        \\unitsBasedOnField A2
  N239, \\field Property Value 239
        \\type real
        \\unitsBasedOnField A2
  N240, \\field Property Value 240
        \\type real
        \\unitsBasedOnField A2
  N241, \\field Property Value 241
        \\type real
        \\unitsBasedOnField A2
  N242, \\field Property Value 242
        \\type real
        \\unitsBasedOnField A2
  N243, \\field Property Value 243
        \\type real
        \\unitsBasedOnField A2
  N244, \\field Property Value 244
        \\type real
        \\unitsBasedOnField A2
  N245, \\field Property Value 245
        \\type real
        \\unitsBasedOnField A2
  N246, \\field Property Value 246
        \\type real
        \\unitsBasedOnField A2
  N247, \\field Property Value 247
        \\type real
        \\unitsBasedOnField A2
  N248, \\field Property Value 248
        \\type real
        \\unitsBasedOnField A2
  N249, \\field Property Value 249
        \\type real
        \\unitsBasedOnField A2
  N250; \\field Property Value 250
        \\type real
        \\unitsBasedOnField A2


FluidProperties:Superheated,
      \\memo fluid properties for the superheated region
      \\format FluidProperty
  A1, \\field Fluid Name
      \\type object-list
      \\object-list FluidNames
  A2, \\field Fluid Property Type
      \\note Enthalpy Units are J/kg
      \\note Density Units are kg/m3
      \\type choice
      \\key Enthalpy     ! Units are J/kg
      \\key Density      ! Units are kg/m3
  A3, \\field Temperature Values Name
      \\note Enter the name of a FluidProperties:Temperatures object.
      \\type object-list
      \\object-list FluidPropertyTemperatures
  N1, \\field Pressure
      \\note pressure for this list of properties
      \\type real
      \\units Pa
      \\minimum> 0.0
  N2, \\field Property Value 1
      \\type real
      \\unitsBasedOnField A2
  N3, \\field Property Value 2
      \\type real
      \\unitsBasedOnField A2
  N4, \\field Property Value 3
      \\type real
      \\unitsBasedOnField A2
  N5, \\field Property Value 4
      \\type real
      \\unitsBasedOnField A2
  N6, \\field Property Value 5
      \\type real
      \\unitsBasedOnField A2
  N7, \\field Property Value 6
      \\type real
      \\unitsBasedOnField A2
  N8, \\field Property Value 7
      \\type real
      \\unitsBasedOnField A2
  N9, \\field Property Value 8
      \\type real
      \\unitsBasedOnField A2
  N10, \\field Property Value 9
       \\type real
       \\unitsBasedOnField A2
  N11, \\field Property Value 10
       \\type real
       \\unitsBasedOnField A2
  N12, \\field Property Value 11
       \\type real
       \\unitsBasedOnField A2
  N13, \\field Property Value 12
       \\type real
       \\unitsBasedOnField A2
  N14, \\field Property Value 13
       \\type real
       \\unitsBasedOnField A2
  N15, \\field Property Value 14
       \\type real
       \\unitsBasedOnField A2
  N16, \\field Property Value 15
       \\type real
       \\unitsBasedOnField A2
  N17, \\field Property Value 16
       \\type real
       \\unitsBasedOnField A2
  N18, \\field Property Value 17
       \\type real
       \\unitsBasedOnField A2
  N19, \\field Property Value 18
       \\type real
       \\unitsBasedOnField A2
  N20, \\field Property Value 19
       \\type real
       \\unitsBasedOnField A2
  N21, \\field Property Value 20
       \\type real
       \\unitsBasedOnField A2
  N22, \\field Property Value 21
       \\type real
       \\unitsBasedOnField A2
  N23, \\field Property Value 22
       \\type real
       \\unitsBasedOnField A2
  N24, \\field Property Value 23
       \\type real
       \\unitsBasedOnField A2
  N25, \\field Property Value 24
       \\type real
       \\unitsBasedOnField A2
  N26, \\field Property Value 25
       \\type real
       \\unitsBasedOnField A2
  N27, \\field Property Value 26
       \\type real
       \\unitsBasedOnField A2
  N28, \\field Property Value 27
       \\type real
       \\unitsBasedOnField A2
  N29, \\field Property Value 28
       \\type real
       \\unitsBasedOnField A2
  N30, \\field Property Value 29
       \\type real
       \\unitsBasedOnField A2
  N31, \\field Property Value 30
       \\type real
       \\unitsBasedOnField A2
  N32, \\field Property Value 31
       \\type real
       \\unitsBasedOnField A2
  N33, \\field Property Value 32
       \\type real
       \\unitsBasedOnField A2
  N34, \\field Property Value 33
       \\type real
       \\unitsBasedOnField A2
  N35, \\field Property Value 34
       \\type real
       \\unitsBasedOnField A2
  N36, \\field Property Value 35
       \\type real
       \\unitsBasedOnField A2
  N37, \\field Property Value 36
       \\type real
       \\unitsBasedOnField A2
  N38, \\field Property Value 37
       \\type real
       \\unitsBasedOnField A2
  N39, \\field Property Value 38
       \\type real
       \\unitsBasedOnField A2
  N40, \\field Property Value 39
       \\type real
       \\unitsBasedOnField A2
  N41, \\field Property Value 40
       \\type real
       \\unitsBasedOnField A2
  N42, \\field Property Value 41
       \\type real
       \\unitsBasedOnField A2
  N43, \\field Property Value 42
       \\type real
       \\unitsBasedOnField A2
  N44, \\field Property Value 43
       \\type real
       \\unitsBasedOnField A2
  N45, \\field Property Value 44
       \\type real
       \\unitsBasedOnField A2
  N46, \\field Property Value 45
       \\type real
       \\unitsBasedOnField A2
  N47, \\field Property Value 46
       \\type real
       \\unitsBasedOnField A2
  N48, \\field Property Value 47
       \\type real
       \\unitsBasedOnField A2
  N49, \\field Property Value 48
       \\type real
       \\unitsBasedOnField A2
  N50, \\field Property Value 49
       \\type real
       \\unitsBasedOnField A2
  N51, \\field Property Value 50
       \\type real
       \\unitsBasedOnField A2
  N52, \\field Property Value 51
       \\type real
       \\unitsBasedOnField A2
  N53, \\field Property Value 52
       \\type real
       \\unitsBasedOnField A2
  N54, \\field Property Value 53
       \\type real
       \\unitsBasedOnField A2
  N55, \\field Property Value 54
       \\type real
       \\unitsBasedOnField A2
  N56, \\field Property Value 55
       \\type real
       \\unitsBasedOnField A2
  N57, \\field Property Value 56
       \\type real
       \\unitsBasedOnField A2
  N58, \\field Property Value 57
       \\type real
       \\unitsBasedOnField A2
  N59, \\field Property Value 58
       \\type real
       \\unitsBasedOnField A2
  N60, \\field Property Value 59
       \\type real
       \\unitsBasedOnField A2
  N61, \\field Property Value 60
       \\type real
       \\unitsBasedOnField A2
  N62, \\field Property Value 61
       \\type real
       \\unitsBasedOnField A2
  N63, \\field Property Value 62
       \\type real
       \\unitsBasedOnField A2
  N64, \\field Property Value 63
       \\type real
       \\unitsBasedOnField A2
  N65, \\field Property Value 64
       \\type real
       \\unitsBasedOnField A2
  N66, \\field Property Value 65
       \\type real
       \\unitsBasedOnField A2
  N67, \\field Property Value 66
       \\type real
       \\unitsBasedOnField A2
  N68, \\field Property Value 67
       \\type real
       \\unitsBasedOnField A2
  N69, \\field Property Value 68
       \\type real
       \\unitsBasedOnField A2
  N70, \\field Property Value 69
       \\type real
       \\unitsBasedOnField A2
  N71, \\field Property Value 70
       \\type real
       \\unitsBasedOnField A2
  N72, \\field Property Value 71
       \\type real
       \\unitsBasedOnField A2
  N73, \\field Property Value 72
       \\type real
       \\unitsBasedOnField A2
  N74, \\field Property Value 73
       \\type real
       \\unitsBasedOnField A2
  N75, \\field Property Value 74
       \\type real
       \\unitsBasedOnField A2
  N76, \\field Property Value 75
       \\type real
       \\unitsBasedOnField A2
  N77, \\field Property Value 76
       \\type real
       \\unitsBasedOnField A2
  N78, \\field Property Value 77
       \\type real
       \\unitsBasedOnField A2
  N79, \\field Property Value 78
       \\type real
       \\unitsBasedOnField A2
  N80, \\field Property Value 79
       \\type real
       \\unitsBasedOnField A2
  N81, \\field Property Value 80
       \\type real
       \\unitsBasedOnField A2
  N82, \\field Property Value 81
       \\type real
       \\unitsBasedOnField A2
  N83, \\field Property Value 82
       \\type real
       \\unitsBasedOnField A2
  N84, \\field Property Value 83
       \\type real
       \\unitsBasedOnField A2
  N85, \\field Property Value 84
       \\type real
       \\unitsBasedOnField A2
  N86, \\field Property Value 85
       \\type real
       \\unitsBasedOnField A2
  N87, \\field Property Value 86
       \\type real
       \\unitsBasedOnField A2
  N88, \\field Property Value 87
       \\type real
       \\unitsBasedOnField A2
  N89, \\field Property Value 88
       \\type real
       \\unitsBasedOnField A2
  N90, \\field Property Value 89
       \\type real
       \\unitsBasedOnField A2
  N91, \\field Property Value 90
       \\type real
       \\unitsBasedOnField A2
  N92, \\field Property Value 91
       \\type real
       \\unitsBasedOnField A2
  N93, \\field Property Value 92
       \\type real
       \\unitsBasedOnField A2
  N94, \\field Property Value 93
       \\type real
       \\unitsBasedOnField A2
  N95, \\field Property Value 94
       \\type real
       \\unitsBasedOnField A2
  N96, \\field Property Value 95
       \\type real
       \\unitsBasedOnField A2
  N97, \\field Property Value 96
       \\type real
       \\unitsBasedOnField A2
  N98, \\field Property Value 97
       \\type real
       \\unitsBasedOnField A2
  N99, \\field Property Value 98
       \\type real
       \\unitsBasedOnField A2
  N100, \\field Property Value 99
        \\type real
        \\unitsBasedOnField A2
  N101, \\field Property Value 100
        \\type real
        \\unitsBasedOnField A2
  N102, \\field Property Value 101
        \\type real
        \\unitsBasedOnField A2
  N103, \\field Property Value 102
        \\type real
        \\unitsBasedOnField A2
  N104, \\field Property Value 103
        \\type real
        \\unitsBasedOnField A2
  N105, \\field Property Value 104
        \\type real
        \\unitsBasedOnField A2
  N106, \\field Property Value 105
        \\type real
        \\unitsBasedOnField A2
  N107, \\field Property Value 106
        \\type real
        \\unitsBasedOnField A2
  N108, \\field Property Value 107
        \\type real
        \\unitsBasedOnField A2
  N109, \\field Property Value 108
        \\type real
        \\unitsBasedOnField A2
  N110, \\field Property Value 109
        \\type real
        \\unitsBasedOnField A2
  N111, \\field Property Value 110
        \\type real
        \\unitsBasedOnField A2
  N112, \\field Property Value 111
        \\type real
        \\unitsBasedOnField A2
  N113, \\field Property Value 112
        \\type real
        \\unitsBasedOnField A2
  N114, \\field Property Value 113
        \\type real
        \\unitsBasedOnField A2
  N115, \\field Property Value 114
        \\type real
        \\unitsBasedOnField A2
  N116, \\field Property Value 115
        \\type real
        \\unitsBasedOnField A2
  N117, \\field Property Value 116
        \\type real
        \\unitsBasedOnField A2
  N118, \\field Property Value 117
        \\type real
        \\unitsBasedOnField A2
  N119, \\field Property Value 118
        \\type real
        \\unitsBasedOnField A2
  N120, \\field Property Value 119
        \\type real
        \\unitsBasedOnField A2
  N121, \\field Property Value 120
        \\type real
        \\unitsBasedOnField A2
  N122, \\field Property Value 121
        \\type real
        \\unitsBasedOnField A2
  N123, \\field Property Value 122
        \\type real
        \\unitsBasedOnField A2
  N124, \\field Property Value 123
        \\type real
        \\unitsBasedOnField A2
  N125, \\field Property Value 124
        \\type real
        \\unitsBasedOnField A2
  N126, \\field Property Value 125
        \\type real
        \\unitsBasedOnField A2
  N127, \\field Property Value 126
        \\type real
        \\unitsBasedOnField A2
  N128, \\field Property Value 127
        \\type real
        \\unitsBasedOnField A2
  N129, \\field Property Value 128
        \\type real
        \\unitsBasedOnField A2
  N130, \\field Property Value 129
        \\type real
        \\unitsBasedOnField A2
  N131, \\field Property Value 130
        \\type real
        \\unitsBasedOnField A2
  N132, \\field Property Value 131
        \\type real
        \\unitsBasedOnField A2
  N133, \\field Property Value 132
        \\type real
        \\unitsBasedOnField A2
  N134, \\field Property Value 133
        \\type real
        \\unitsBasedOnField A2
  N135, \\field Property Value 134
        \\type real
        \\unitsBasedOnField A2
  N136, \\field Property Value 135
        \\type real
        \\unitsBasedOnField A2
  N137, \\field Property Value 136
        \\type real
        \\unitsBasedOnField A2
  N138, \\field Property Value 137
        \\type real
        \\unitsBasedOnField A2
  N139, \\field Property Value 138
        \\type real
        \\unitsBasedOnField A2
  N140, \\field Property Value 139
        \\type real
        \\unitsBasedOnField A2
  N141, \\field Property Value 140
        \\type real
        \\unitsBasedOnField A2
  N142, \\field Property Value 141
        \\type real
        \\unitsBasedOnField A2
  N143, \\field Property Value 142
        \\type real
        \\unitsBasedOnField A2
  N144, \\field Property Value 143
        \\type real
        \\unitsBasedOnField A2
  N145, \\field Property Value 144
        \\type real
        \\unitsBasedOnField A2
  N146, \\field Property Value 145
        \\type real
        \\unitsBasedOnField A2
  N147, \\field Property Value 146
        \\type real
        \\unitsBasedOnField A2
  N148, \\field Property Value 147
        \\type real
        \\unitsBasedOnField A2
  N149, \\field Property Value 148
        \\type real
        \\unitsBasedOnField A2
  N150, \\field Property Value 149
        \\type real
        \\unitsBasedOnField A2
  N151, \\field Property Value 150
        \\type real
        \\unitsBasedOnField A2
  N152, \\field Property Value 151
        \\type real
        \\unitsBasedOnField A2
  N153, \\field Property Value 152
        \\type real
        \\unitsBasedOnField A2
  N154, \\field Property Value 153
        \\type real
        \\unitsBasedOnField A2
  N155, \\field Property Value 154
        \\type real
        \\unitsBasedOnField A2
  N156, \\field Property Value 155
        \\type real
        \\unitsBasedOnField A2
  N157, \\field Property Value 156
        \\type real
        \\unitsBasedOnField A2
  N158, \\field Property Value 157
        \\type real
        \\unitsBasedOnField A2
  N159, \\field Property Value 158
        \\type real
        \\unitsBasedOnField A2
  N160, \\field Property Value 159
        \\type real
        \\unitsBasedOnField A2
  N161, \\field Property Value 160
        \\type real
        \\unitsBasedOnField A2
  N162, \\field Property Value 161
        \\type real
        \\unitsBasedOnField A2
  N163, \\field Property Value 162
        \\type real
        \\unitsBasedOnField A2
  N164, \\field Property Value 163
        \\type real
        \\unitsBasedOnField A2
  N165, \\field Property Value 164
        \\type real
        \\unitsBasedOnField A2
  N166, \\field Property Value 165
        \\type real
        \\unitsBasedOnField A2
  N167, \\field Property Value 166
        \\type real
        \\unitsBasedOnField A2
  N168, \\field Property Value 167
        \\type real
        \\unitsBasedOnField A2
  N169, \\field Property Value 168
        \\type real
        \\unitsBasedOnField A2
  N170, \\field Property Value 169
        \\type real
        \\unitsBasedOnField A2
  N171, \\field Property Value 170
        \\type real
        \\unitsBasedOnField A2
  N172, \\field Property Value 171
        \\type real
        \\unitsBasedOnField A2
  N173, \\field Property Value 172
        \\type real
        \\unitsBasedOnField A2
  N174, \\field Property Value 173
        \\type real
        \\unitsBasedOnField A2
  N175, \\field Property Value 174
        \\type real
        \\unitsBasedOnField A2
  N176, \\field Property Value 175
        \\type real
        \\unitsBasedOnField A2
  N177, \\field Property Value 176
        \\type real
        \\unitsBasedOnField A2
  N178, \\field Property Value 177
        \\type real
        \\unitsBasedOnField A2
  N179, \\field Property Value 178
        \\type real
        \\unitsBasedOnField A2
  N180, \\field Property Value 179
        \\type real
        \\unitsBasedOnField A2
  N181, \\field Property Value 180
        \\type real
        \\unitsBasedOnField A2
  N182, \\field Property Value 181
        \\type real
        \\unitsBasedOnField A2
  N183, \\field Property Value 182
        \\type real
        \\unitsBasedOnField A2
  N184, \\field Property Value 183
        \\type real
        \\unitsBasedOnField A2
  N185, \\field Property Value 184
        \\type real
        \\unitsBasedOnField A2
  N186, \\field Property Value 185
        \\type real
        \\unitsBasedOnField A2
  N187, \\field Property Value 186
        \\type real
        \\unitsBasedOnField A2
  N188, \\field Property Value 187
        \\type real
        \\unitsBasedOnField A2
  N189, \\field Property Value 188
        \\type real
        \\unitsBasedOnField A2
  N190, \\field Property Value 189
        \\type real
        \\unitsBasedOnField A2
  N191, \\field Property Value 190
        \\type real
        \\unitsBasedOnField A2
  N192, \\field Property Value 191
        \\type real
        \\unitsBasedOnField A2
  N193, \\field Property Value 192
        \\type real
        \\unitsBasedOnField A2
  N194, \\field Property Value 193
        \\type real
        \\unitsBasedOnField A2
  N195, \\field Property Value 194
        \\type real
        \\unitsBasedOnField A2
  N196, \\field Property Value 195
        \\type real
        \\unitsBasedOnField A2
  N197, \\field Property Value 196
        \\type real
        \\unitsBasedOnField A2
  N198, \\field Property Value 197
        \\type real
        \\unitsBasedOnField A2
  N199, \\field Property Value 198
        \\type real
        \\unitsBasedOnField A2
  N200, \\field Property Value 199
        \\type real
        \\unitsBasedOnField A2
  N201, \\field Property Value 200
        \\type real
        \\unitsBasedOnField A2
  N202, \\field Property Value 201
        \\type real
        \\unitsBasedOnField A2
  N203, \\field Property Value 202
        \\type real
        \\unitsBasedOnField A2
  N204, \\field Property Value 203
        \\type real
        \\unitsBasedOnField A2
  N205, \\field Property Value 204
        \\type real
        \\unitsBasedOnField A2
  N206, \\field Property Value 205
        \\type real
        \\unitsBasedOnField A2
  N207, \\field Property Value 206
        \\type real
        \\unitsBasedOnField A2
  N208, \\field Property Value 207
        \\type real
        \\unitsBasedOnField A2
  N209, \\field Property Value 208
        \\type real
        \\unitsBasedOnField A2
  N210, \\field Property Value 209
        \\type real
        \\unitsBasedOnField A2
  N211, \\field Property Value 210
        \\type real
        \\unitsBasedOnField A2
  N212, \\field Property Value 211
        \\type real
        \\unitsBasedOnField A2
  N213, \\field Property Value 212
        \\type real
        \\unitsBasedOnField A2
  N214, \\field Property Value 213
        \\type real
        \\unitsBasedOnField A2
  N215, \\field Property Value 214
        \\type real
        \\unitsBasedOnField A2
  N216, \\field Property Value 215
        \\type real
        \\unitsBasedOnField A2
  N217, \\field Property Value 216
        \\type real
        \\unitsBasedOnField A2
  N218, \\field Property Value 217
        \\type real
        \\unitsBasedOnField A2
  N219, \\field Property Value 218
        \\type real
        \\unitsBasedOnField A2
  N220, \\field Property Value 219
        \\type real
        \\unitsBasedOnField A2
  N221, \\field Property Value 220
        \\type real
        \\unitsBasedOnField A2
  N222, \\field Property Value 221
        \\type real
        \\unitsBasedOnField A2
  N223, \\field Property Value 222
        \\type real
        \\unitsBasedOnField A2
  N224, \\field Property Value 223
        \\type real
        \\unitsBasedOnField A2
  N225, \\field Property Value 224
        \\type real
        \\unitsBasedOnField A2
  N226, \\field Property Value 225
        \\type real
        \\unitsBasedOnField A2
  N227, \\field Property Value 226
        \\type real
        \\unitsBasedOnField A2
  N228, \\field Property Value 227
        \\type real
        \\unitsBasedOnField A2
  N229, \\field Property Value 228
        \\type real
        \\unitsBasedOnField A2
  N230, \\field Property Value 229
        \\type real
        \\unitsBasedOnField A2
  N231, \\field Property Value 230
        \\type real
        \\unitsBasedOnField A2
  N232, \\field Property Value 231
        \\type real
        \\unitsBasedOnField A2
  N233, \\field Property Value 232
        \\type real
        \\unitsBasedOnField A2
  N234, \\field Property Value 233
        \\type real
        \\unitsBasedOnField A2
  N235, \\field Property Value 234
        \\type real
        \\unitsBasedOnField A2
  N236, \\field Property Value 235
        \\type real
        \\unitsBasedOnField A2
  N237, \\field Property Value 236
        \\type real
        \\unitsBasedOnField A2
  N238, \\field Property Value 237
        \\type real
        \\unitsBasedOnField A2
  N239, \\field Property Value 238
        \\type real
        \\unitsBasedOnField A2
  N240, \\field Property Value 239
        \\type real
        \\unitsBasedOnField A2
  N241, \\field Property Value 240
        \\type real
        \\unitsBasedOnField A2
  N242, \\field Property Value 241
        \\type real
        \\unitsBasedOnField A2
  N243, \\field Property Value 242
        \\type real
        \\unitsBasedOnField A2
  N244, \\field Property Value 243
        \\type real
        \\unitsBasedOnField A2
  N245, \\field Property Value 244
        \\type real
        \\unitsBasedOnField A2
  N246, \\field Property Value 245
        \\type real
        \\unitsBasedOnField A2
  N247, \\field Property Value 246
        \\type real
        \\unitsBasedOnField A2
  N248, \\field Property Value 247
        \\type real
        \\unitsBasedOnField A2
  N249, \\field Property Value 248
        \\type real
        \\unitsBasedOnField A2
  N250, \\field Property Value 249
        \\type real
        \\unitsBasedOnField A2
  N251; \\field Property Value 250
        \\type real
        \\unitsBasedOnField A2

FluidProperties:Concentration,
      \\memo fluid properties for water/other fluid mixtures
      \\format FluidProperty
  A1, \\field Fluid Name
      \\type object-list
      \\object-list FluidNames
      \\note should not be any of the defaults (Water, EthyleneGlycol, or PropyleneGlycol)
  A2, \\field Fluid Property Type
      \\note Density Units are kg/m3
      \\note SpecificHeat Units are J/kg-K
      \\note Conductivity Units are W/m-K
      \\note Viscosity Units are N-s/m2
      \\type choice
      \\key Density      ! Units are kg/m3
      \\key SpecificHeat ! Units are J/kg-K
      \\key Conductivity ! Units are W/m-K
      \\key Viscosity    ! Units are N-s/m2
  A3, \\field Temperature Values Name
      \\note Enter the name of a FluidProperties:Temperatures object.
      \\type object-list
      \\object-list FluidPropertyTemperatures
  N1, \\field Concentration
      \\note Glycol concentration for this list of properties entered as a fraction
      \\type real
      \\units dimensionless
      \\minimum 0.0
      \\maximum 1.0
  N2, \\field Property Value 1
      \\type real
      \\unitsBasedOnField A2
  N3, \\field Property Value 2
      \\type real
      \\unitsBasedOnField A2
  N4, \\field Property Value 3
      \\type real
      \\unitsBasedOnField A2
  N5, \\field Property Value 4
      \\type real
      \\unitsBasedOnField A2
  N6, \\field Property Value 5
      \\type real
      \\unitsBasedOnField A2
  N7, \\field Property Value 6
      \\type real
      \\unitsBasedOnField A2
  N8, \\field Property Value 7
      \\type real
      \\unitsBasedOnField A2
  N9, \\field Property Value 8
      \\type real
      \\unitsBasedOnField A2
  N10, \\field Property Value 9
       \\type real
       \\unitsBasedOnField A2
  N11, \\field Property Value 10
       \\type real
       \\unitsBasedOnField A2
  N12, \\field Property Value 11
       \\type real
       \\unitsBasedOnField A2
  N13, \\field Property Value 12
       \\type real
       \\unitsBasedOnField A2
  N14, \\field Property Value 13
       \\type real
       \\unitsBasedOnField A2
  N15, \\field Property Value 14
       \\type real
       \\unitsBasedOnField A2
  N16, \\field Property Value 15
       \\type real
       \\unitsBasedOnField A2
  N17, \\field Property Value 16
       \\type real
       \\unitsBasedOnField A2
  N18, \\field Property Value 17
       \\type real
       \\unitsBasedOnField A2
  N19, \\field Property Value 18
       \\type real
       \\unitsBasedOnField A2
  N20, \\field Property Value 19
       \\type real
       \\unitsBasedOnField A2
  N21, \\field Property Value 20
       \\type real
       \\unitsBasedOnField A2
  N22, \\field Property Value 21
       \\type real
       \\unitsBasedOnField A2
  N23, \\field Property Value 22
       \\type real
       \\unitsBasedOnField A2
  N24, \\field Property Value 23
       \\type real
       \\unitsBasedOnField A2
  N25, \\field Property Value 24
       \\type real
       \\unitsBasedOnField A2
  N26, \\field Property Value 25
       \\type real
       \\unitsBasedOnField A2
  N27, \\field Property Value 26
       \\type real
       \\unitsBasedOnField A2
  N28, \\field Property Value 27
       \\type real
       \\unitsBasedOnField A2
  N29, \\field Property Value 28
       \\type real
       \\unitsBasedOnField A2
  N30, \\field Property Value 29
       \\type real
       \\unitsBasedOnField A2
  N31, \\field Property Value 30
       \\type real
       \\unitsBasedOnField A2
  N32, \\field Property Value 31
       \\type real
       \\unitsBasedOnField A2
  N33, \\field Property Value 32
       \\type real
       \\unitsBasedOnField A2
  N34, \\field Property Value 33
       \\type real
       \\unitsBasedOnField A2
  N35, \\field Property Value 34
       \\type real
       \\unitsBasedOnField A2
  N36, \\field Property Value 35
       \\type real
       \\unitsBasedOnField A2
  N37, \\field Property Value 36
       \\type real
       \\unitsBasedOnField A2
  N38, \\field Property Value 37
       \\type real
       \\unitsBasedOnField A2
  N39, \\field Property Value 38
       \\type real
       \\unitsBasedOnField A2
  N40, \\field Property Value 39
       \\type real
       \\unitsBasedOnField A2
  N41, \\field Property Value 40
       \\type real
       \\unitsBasedOnField A2
  N42, \\field Property Value 41
       \\type real
       \\unitsBasedOnField A2
  N43, \\field Property Value 42
       \\type real
       \\unitsBasedOnField A2
  N44, \\field Property Value 43
       \\type real
       \\unitsBasedOnField A2
  N45, \\field Property Value 44
       \\type real
       \\unitsBasedOnField A2
  N46, \\field Property Value 45
       \\type real
       \\unitsBasedOnField A2
  N47, \\field Property Value 46
       \\type real
       \\unitsBasedOnField A2
  N48, \\field Property Value 47
       \\type real
       \\unitsBasedOnField A2
  N49, \\field Property Value 48
       \\type real
       \\unitsBasedOnField A2
  N50, \\field Property Value 49
       \\type real
       \\unitsBasedOnField A2
  N51, \\field Property Value 50
       \\type real
       \\unitsBasedOnField A2
  N52, \\field Property Value 51
       \\type real
       \\unitsBasedOnField A2
  N53, \\field Property Value 52
       \\type real
       \\unitsBasedOnField A2
  N54, \\field Property Value 53
       \\type real
       \\unitsBasedOnField A2
  N55, \\field Property Value 54
       \\type real
       \\unitsBasedOnField A2
  N56, \\field Property Value 55
       \\type real
       \\unitsBasedOnField A2
  N57, \\field Property Value 56
       \\type real
       \\unitsBasedOnField A2
  N58, \\field Property Value 57
       \\type real
       \\unitsBasedOnField A2
  N59, \\field Property Value 58
       \\type real
       \\unitsBasedOnField A2
  N60, \\field Property Value 59
       \\type real
       \\unitsBasedOnField A2
  N61, \\field Property Value 60
       \\type real
       \\unitsBasedOnField A2
  N62, \\field Property Value 61
       \\type real
       \\unitsBasedOnField A2
  N63, \\field Property Value 62
       \\type real
       \\unitsBasedOnField A2
  N64, \\field Property Value 63
       \\type real
       \\unitsBasedOnField A2
  N65, \\field Property Value 64
       \\type real
       \\unitsBasedOnField A2
  N66, \\field Property Value 65
       \\type real
       \\unitsBasedOnField A2
  N67, \\field Property Value 66
       \\type real
       \\unitsBasedOnField A2
  N68, \\field Property Value 67
       \\type real
       \\unitsBasedOnField A2
  N69, \\field Property Value 68
       \\type real
       \\unitsBasedOnField A2
  N70, \\field Property Value 69
       \\type real
       \\unitsBasedOnField A2
  N71, \\field Property Value 70
       \\type real
       \\unitsBasedOnField A2
  N72, \\field Property Value 71
       \\type real
       \\unitsBasedOnField A2
  N73, \\field Property Value 72
       \\type real
       \\unitsBasedOnField A2
  N74, \\field Property Value 73
       \\type real
       \\unitsBasedOnField A2
  N75, \\field Property Value 74
       \\type real
       \\unitsBasedOnField A2
  N76, \\field Property Value 75
       \\type real
       \\unitsBasedOnField A2
  N77, \\field Property Value 76
       \\type real
       \\unitsBasedOnField A2
  N78, \\field Property Value 77
       \\type real
       \\unitsBasedOnField A2
  N79, \\field Property Value 78
       \\type real
       \\unitsBasedOnField A2
  N80, \\field Property Value 79
       \\type real
       \\unitsBasedOnField A2
  N81, \\field Property Value 80
       \\type real
       \\unitsBasedOnField A2
  N82, \\field Property Value 81
       \\type real
       \\unitsBasedOnField A2
  N83, \\field Property Value 82
       \\type real
       \\unitsBasedOnField A2
  N84, \\field Property Value 83
       \\type real
       \\unitsBasedOnField A2
  N85, \\field Property Value 84
       \\type real
       \\unitsBasedOnField A2
  N86, \\field Property Value 85
       \\type real
       \\unitsBasedOnField A2
  N87, \\field Property Value 86
       \\type real
       \\unitsBasedOnField A2
  N88, \\field Property Value 87
       \\type real
       \\unitsBasedOnField A2
  N89, \\field Property Value 88
       \\type real
       \\unitsBasedOnField A2
  N90, \\field Property Value 89
       \\type real
       \\unitsBasedOnField A2
  N91, \\field Property Value 90
       \\type real
       \\unitsBasedOnField A2
  N92, \\field Property Value 91
       \\type real
       \\unitsBasedOnField A2
  N93, \\field Property Value 92
       \\type real
       \\unitsBasedOnField A2
  N94, \\field Property Value 93
       \\type real
       \\unitsBasedOnField A2
  N95, \\field Property Value 94
       \\type real
       \\unitsBasedOnField A2
  N96, \\field Property Value 95
       \\type real
       \\unitsBasedOnField A2
  N97, \\field Property Value 96
       \\type real
       \\unitsBasedOnField A2
  N98, \\field Property Value 97
       \\type real
       \\unitsBasedOnField A2
  N99, \\field Property Value 98
       \\type real
       \\unitsBasedOnField A2
  N100, \\field Property Value 99
        \\type real
        \\unitsBasedOnField A2
  N101, \\field Property Value 100
        \\type real
        \\unitsBasedOnField A2
  N102, \\field Property Value 101
        \\type real
        \\unitsBasedOnField A2
  N103, \\field Property Value 102
        \\type real
        \\unitsBasedOnField A2
  N104, \\field Property Value 103
        \\type real
        \\unitsBasedOnField A2
  N105, \\field Property Value 104
        \\type real
        \\unitsBasedOnField A2
  N106, \\field Property Value 105
        \\type real
        \\unitsBasedOnField A2
  N107, \\field Property Value 106
        \\type real
        \\unitsBasedOnField A2
  N108, \\field Property Value 107
        \\type real
        \\unitsBasedOnField A2
  N109, \\field Property Value 108
        \\type real
        \\unitsBasedOnField A2
  N110, \\field Property Value 109
        \\type real
        \\unitsBasedOnField A2
  N111, \\field Property Value 110
        \\type real
        \\unitsBasedOnField A2
  N112, \\field Property Value 111
        \\type real
        \\unitsBasedOnField A2
  N113, \\field Property Value 112
        \\type real
        \\unitsBasedOnField A2
  N114, \\field Property Value 113
        \\type real
        \\unitsBasedOnField A2
  N115, \\field Property Value 114
        \\type real
        \\unitsBasedOnField A2
  N116, \\field Property Value 115
        \\type real
        \\unitsBasedOnField A2
  N117, \\field Property Value 116
        \\type real
        \\unitsBasedOnField A2
  N118, \\field Property Value 117
        \\type real
        \\unitsBasedOnField A2
  N119, \\field Property Value 118
        \\type real
        \\unitsBasedOnField A2
  N120, \\field Property Value 119
        \\type real
        \\unitsBasedOnField A2
  N121, \\field Property Value 120
        \\type real
        \\unitsBasedOnField A2
  N122, \\field Property Value 121
        \\type real
        \\unitsBasedOnField A2
  N123, \\field Property Value 122
        \\type real
        \\unitsBasedOnField A2
  N124, \\field Property Value 123
        \\type real
        \\unitsBasedOnField A2
  N125, \\field Property Value 124
        \\type real
        \\unitsBasedOnField A2
  N126, \\field Property Value 125
        \\type real
        \\unitsBasedOnField A2
  N127, \\field Property Value 126
        \\type real
        \\unitsBasedOnField A2
  N128, \\field Property Value 127
        \\type real
        \\unitsBasedOnField A2
  N129, \\field Property Value 128
        \\type real
        \\unitsBasedOnField A2
  N130, \\field Property Value 129
        \\type real
        \\unitsBasedOnField A2
  N131, \\field Property Value 130
        \\type real
        \\unitsBasedOnField A2
  N132, \\field Property Value 131
        \\type real
        \\unitsBasedOnField A2
  N133, \\field Property Value 132
        \\type real
        \\unitsBasedOnField A2
  N134, \\field Property Value 133
        \\type real
        \\unitsBasedOnField A2
  N135, \\field Property Value 134
        \\type real
        \\unitsBasedOnField A2
  N136, \\field Property Value 135
        \\type real
        \\unitsBasedOnField A2
  N137, \\field Property Value 136
        \\type real
        \\unitsBasedOnField A2
  N138, \\field Property Value 137
        \\type real
        \\unitsBasedOnField A2
  N139, \\field Property Value 138
        \\type real
        \\unitsBasedOnField A2
  N140, \\field Property Value 139
        \\type real
        \\unitsBasedOnField A2
  N141, \\field Property Value 140
        \\type real
        \\unitsBasedOnField A2
  N142, \\field Property Value 141
        \\type real
        \\unitsBasedOnField A2
  N143, \\field Property Value 142
        \\type real
        \\unitsBasedOnField A2
  N144, \\field Property Value 143
        \\type real
        \\unitsBasedOnField A2
  N145, \\field Property Value 144
        \\type real
        \\unitsBasedOnField A2
  N146, \\field Property Value 145
        \\type real
        \\unitsBasedOnField A2
  N147, \\field Property Value 146
        \\type real
        \\unitsBasedOnField A2
  N148, \\field Property Value 147
        \\type real
        \\unitsBasedOnField A2
  N149, \\field Property Value 148
        \\type real
        \\unitsBasedOnField A2
  N150, \\field Property Value 149
        \\type real
        \\unitsBasedOnField A2
  N151, \\field Property Value 150
        \\type real
        \\unitsBasedOnField A2
  N152, \\field Property Value 151
        \\type real
        \\unitsBasedOnField A2
  N153, \\field Property Value 152
        \\type real
        \\unitsBasedOnField A2
  N154, \\field Property Value 153
        \\type real
        \\unitsBasedOnField A2
  N155, \\field Property Value 154
        \\type real
        \\unitsBasedOnField A2
  N156, \\field Property Value 155
        \\type real
        \\unitsBasedOnField A2
  N157, \\field Property Value 156
        \\type real
        \\unitsBasedOnField A2
  N158, \\field Property Value 157
        \\type real
        \\unitsBasedOnField A2
  N159, \\field Property Value 158
        \\type real
        \\unitsBasedOnField A2
  N160, \\field Property Value 159
        \\type real
        \\unitsBasedOnField A2
  N161, \\field Property Value 160
        \\type real
        \\unitsBasedOnField A2
  N162, \\field Property Value 161
        \\type real
        \\unitsBasedOnField A2
  N163, \\field Property Value 162
        \\type real
        \\unitsBasedOnField A2
  N164, \\field Property Value 163
        \\type real
        \\unitsBasedOnField A2
  N165, \\field Property Value 164
        \\type real
        \\unitsBasedOnField A2
  N166, \\field Property Value 165
        \\type real
        \\unitsBasedOnField A2
  N167, \\field Property Value 166
        \\type real
        \\unitsBasedOnField A2
  N168, \\field Property Value 167
        \\type real
        \\unitsBasedOnField A2
  N169, \\field Property Value 168
        \\type real
        \\unitsBasedOnField A2
  N170, \\field Property Value 169
        \\type real
        \\unitsBasedOnField A2
  N171, \\field Property Value 170
        \\type real
        \\unitsBasedOnField A2
  N172, \\field Property Value 171
        \\type real
        \\unitsBasedOnField A2
  N173, \\field Property Value 172
        \\type real
        \\unitsBasedOnField A2
  N174, \\field Property Value 173
        \\type real
        \\unitsBasedOnField A2
  N175, \\field Property Value 174
        \\type real
        \\unitsBasedOnField A2
  N176, \\field Property Value 175
        \\type real
        \\unitsBasedOnField A2
  N177, \\field Property Value 176
        \\type real
        \\unitsBasedOnField A2
  N178, \\field Property Value 177
        \\type real
        \\unitsBasedOnField A2
  N179, \\field Property Value 178
        \\type real
        \\unitsBasedOnField A2
  N180, \\field Property Value 179
        \\type real
        \\unitsBasedOnField A2
  N181, \\field Property Value 180
        \\type real
        \\unitsBasedOnField A2
  N182, \\field Property Value 181
        \\type real
        \\unitsBasedOnField A2
  N183, \\field Property Value 182
        \\type real
        \\unitsBasedOnField A2
  N184, \\field Property Value 183
        \\type real
        \\unitsBasedOnField A2
  N185, \\field Property Value 184
        \\type real
        \\unitsBasedOnField A2
  N186, \\field Property Value 185
        \\type real
        \\unitsBasedOnField A2
  N187, \\field Property Value 186
        \\type real
        \\unitsBasedOnField A2
  N188, \\field Property Value 187
        \\type real
        \\unitsBasedOnField A2
  N189, \\field Property Value 188
        \\type real
        \\unitsBasedOnField A2
  N190, \\field Property Value 189
        \\type real
        \\unitsBasedOnField A2
  N191, \\field Property Value 190
        \\type real
        \\unitsBasedOnField A2
  N192, \\field Property Value 191
        \\type real
        \\unitsBasedOnField A2
  N193, \\field Property Value 192
        \\type real
        \\unitsBasedOnField A2
  N194, \\field Property Value 193
        \\type real
        \\unitsBasedOnField A2
  N195, \\field Property Value 194
        \\type real
        \\unitsBasedOnField A2
  N196, \\field Property Value 195
        \\type real
        \\unitsBasedOnField A2
  N197, \\field Property Value 196
        \\type real
        \\unitsBasedOnField A2
  N198, \\field Property Value 197
        \\type real
        \\unitsBasedOnField A2
  N199, \\field Property Value 198
        \\type real
        \\unitsBasedOnField A2
  N200, \\field Property Value 199
        \\type real
        \\unitsBasedOnField A2
  N201, \\field Property Value 200
        \\type real
        \\unitsBasedOnField A2
  N202, \\field Property Value 201
        \\type real
        \\unitsBasedOnField A2
  N203, \\field Property Value 202
        \\type real
        \\unitsBasedOnField A2
  N204, \\field Property Value 203
        \\type real
        \\unitsBasedOnField A2
  N205, \\field Property Value 204
        \\type real
        \\unitsBasedOnField A2
  N206, \\field Property Value 205
        \\type real
        \\unitsBasedOnField A2
  N207, \\field Property Value 206
        \\type real
        \\unitsBasedOnField A2
  N208, \\field Property Value 207
        \\type real
        \\unitsBasedOnField A2
  N209, \\field Property Value 208
        \\type real
        \\unitsBasedOnField A2
  N210, \\field Property Value 209
        \\type real
        \\unitsBasedOnField A2
  N211, \\field Property Value 210
        \\type real
        \\unitsBasedOnField A2
  N212, \\field Property Value 211
        \\type real
        \\unitsBasedOnField A2
  N213, \\field Property Value 212
        \\type real
        \\unitsBasedOnField A2
  N214, \\field Property Value 213
        \\type real
        \\unitsBasedOnField A2
  N215, \\field Property Value 214
        \\type real
        \\unitsBasedOnField A2
  N216, \\field Property Value 215
        \\type real
        \\unitsBasedOnField A2
  N217, \\field Property Value 216
        \\type real
        \\unitsBasedOnField A2
  N218, \\field Property Value 217
        \\type real
        \\unitsBasedOnField A2
  N219, \\field Property Value 218
        \\type real
        \\unitsBasedOnField A2
  N220, \\field Property Value 219
        \\type real
        \\unitsBasedOnField A2
  N221, \\field Property Value 220
        \\type real
        \\unitsBasedOnField A2
  N222, \\field Property Value 221
        \\type real
        \\unitsBasedOnField A2
  N223, \\field Property Value 222
        \\type real
        \\unitsBasedOnField A2
  N224, \\field Property Value 223
        \\type real
        \\unitsBasedOnField A2
  N225, \\field Property Value 224
        \\type real
        \\unitsBasedOnField A2
  N226, \\field Property Value 225
        \\type real
        \\unitsBasedOnField A2
  N227, \\field Property Value 226
        \\type real
        \\unitsBasedOnField A2
  N228, \\field Property Value 227
        \\type real
        \\unitsBasedOnField A2
  N229, \\field Property Value 228
        \\type real
        \\unitsBasedOnField A2
  N230, \\field Property Value 229
        \\type real
        \\unitsBasedOnField A2
  N231, \\field Property Value 230
        \\type real
        \\unitsBasedOnField A2
  N232, \\field Property Value 231
        \\type real
        \\unitsBasedOnField A2
  N233, \\field Property Value 232
        \\type real
        \\unitsBasedOnField A2
  N234, \\field Property Value 233
        \\type real
        \\unitsBasedOnField A2
  N235, \\field Property Value 234
        \\type real
        \\unitsBasedOnField A2
  N236, \\field Property Value 235
        \\type real
        \\unitsBasedOnField A2
  N237, \\field Property Value 236
        \\type real
        \\unitsBasedOnField A2
  N238, \\field Property Value 237
        \\type real
        \\unitsBasedOnField A2
  N239, \\field Property Value 238
        \\type real
        \\unitsBasedOnField A2
  N240, \\field Property Value 239
        \\type real
        \\unitsBasedOnField A2
  N241, \\field Property Value 240
        \\type real
        \\unitsBasedOnField A2
  N242, \\field Property Value 241
        \\type real
        \\unitsBasedOnField A2
  N243, \\field Property Value 242
        \\type real
        \\unitsBasedOnField A2
  N244, \\field Property Value 243
        \\type real
        \\unitsBasedOnField A2
  N245, \\field Property Value 244
        \\type real
        \\unitsBasedOnField A2
  N246, \\field Property Value 245
        \\type real
        \\unitsBasedOnField A2
  N247, \\field Property Value 246
        \\type real
        \\unitsBasedOnField A2
  N248, \\field Property Value 247
        \\type real
        \\unitsBasedOnField A2
  N249, \\field Property Value 248
        \\type real
        \\unitsBasedOnField A2
  N250, \\field Property Value 249
        \\type real
        \\unitsBasedOnField A2
  N251; \\field Property Value 250
        \\type real
        \\unitsBasedOnField A2

\\Group Economics

CurrencyType,
       \\memo If CurrencyType is not specified, it will default to USD and produce $ in the reports.
       \\unique-object
  A1;   \\field Monetary Unit
        \\required-field
        \\note The commonly used three letter currency code for the units of money for the country or region.
        \\note Based on ISO 4217 currency codes.  Common currency codes are USD for $ and EUR for Euros.
        \\type choice
        \\key USD
        \\key AFN
        \\key ALL
        \\key ANG
        \\key ARS
        \\key AUD
        \\key AWG
        \\key AZN
        \\key BAM
        \\key BBD
        \\key BGN
        \\key BMD
        \\key BND
        \\key BOB
        \\key BRL
        \\key BSD
        \\key BWP
        \\key BYR
        \\key BZD
        \\key CAD
        \\key CHF
        \\key CLP
        \\key CNY
        \\key COP
        \\key CRC
        \\key CUP
        \\key CZK
        \\key DKK
        \\key DOP
        \\key EEK
        \\key EGP
        \\key EUR
        \\key FJD
        \\key GBP
        \\key GHC
        \\key GIP
        \\key GTQ
        \\key GYD
        \\key HKD
        \\key HNL
        \\key HRK
        \\key HUF
        \\key IDR
        \\key ILS
        \\key IMP
        \\key INR
        \\key IRR
        \\key ISK
        \\key JEP
        \\key JMD
        \\key JPY
        \\key KGS
        \\key KHR
        \\key KPW
        \\key KRW
        \\key KYD
        \\key KZT
        \\key LAK
        \\key LBP
        \\key LKR
        \\key LRD
        \\key LTL
        \\key LVL
        \\key MKD
        \\key MNT
        \\key MUR
        \\key MXN
        \\key MYR
        \\key MZN
        \\key NAD
        \\key NGN
        \\key NIO
        \\key NOK
        \\key NPR
        \\key NZD
        \\key OMR
        \\key PAB
        \\key PEN
        \\key PHP
        \\key PKR
        \\key PLN
        \\key PYG
        \\key QAR
        \\key RON
        \\key RSD
        \\key RUB
        \\key SAR
        \\key SBD
        \\key SCR
        \\key SEK
        \\key SGD
        \\key SHP
        \\key SOS
        \\key SRD
        \\key SVC
        \\key SYP
        \\key THB
        \\key TRL
        \\key TRY
        \\key TTD
        \\key TVD
        \\key TWD
        \\key UAH
        \\key UYU
        \\key UZS
        \\key VEF
        \\key VND
        \\key XCD
        \\key YER
        \\key ZAR
        \\key ZWD

ComponentCost:Adjustments,
       \\memo Used to perform various modifications to the construction costs to arrive at an
       \\memo estimate for total project costs. This object allows extending the line item model
       \\memo so that the overall costs of the project will reflect various profit and fees.
   N1 ,\\field Miscellaneous Cost per Conditioned Area
       \\type real
       \\note based on conditioned floor area
       \\note for cost not accounted for in current line item cost model
       \\units $/m2
   N2 ,\\field Design and Engineering Fees
       \\type real
       \\units dimensionless
   N3 ,\\field Contractor Fee
       \\type real
       \\units dimensionless
   N4 ,\\field Contingency
       \\type real
       \\units dimensionless
   N5 ,\\field Permits, Bonding and Insurance
       \\type real
       \\units dimensionless
   N6 ,\\field Commissioning Fee
       \\type real
       \\units dimensionless
   N7 ;\\field Regional Adjustment Factor
       \\type real
       \\units dimensionless
       \\note for use with average data in line item and Misc cost models

ComponentCost:Reference,
       \\memo Used to allow comparing the current cost estimate to the results of a previous
       \\memo estimate for a reference building. This object parallels the ComponentCost:Adjustments
       \\memo object but adds a field for entering the cost line item model result for the reference
       \\memo building. The factors entered in this object are applied to the reference building
       \\memo while the factors listed in the ComponentCost:Adjustments object are applied to the
       \\memo current building model cost estimate.
   N1 ,\\field Reference Building Line Item Costs
       \\type real
       \\units $
       \\note should be comparable to the components in current line item cost model
   N2 ,\\field Reference Building Miscellaneous Cost per Conditioned Area
       \\type real
       \\units $/m2
       \\note based on conditioned floor area
       \\note for cost not accounted for in reference line item costs
   N3 ,\\field Reference Building Design and Engineering Fees
       \\type real
       \\units dimensionless
   N4 ,\\field Reference Building Contractor Fee
       \\type real
       \\units dimensionless
   N5 ,\\field Reference Building Contingency
       \\type real
   N6 ,\\field Reference Building Permits, Bonding and Insurance
       \\type real
       \\units dimensionless
   N7 ,\\field Reference Building Commissioning Fee
       \\type real
       \\units dimensionless
   N8 ;\\field Reference Building Regional Adjustment Factor
       \\type real
       \\units dimensionless
       \\note for use with average data in line item and Misc cost models

ComponentCost:LineItem,
       \\memo Each instance of this object creates a cost line item and will contribute to the total
       \\memo for a cost estimate.
   A1 ,\\field Name
       \\type alpha
   A2 ,\\field Type
       \\type alpha
   A3 ,\\field Line Item Type
       \\required-field
       \\type choice
       \\key General
       \\key Construction
       \\key Coil:DX
       \\key Coil:Cooling:DX:SingleSpeed
       \\key Coil:Heating:Gas
       \\key Chiller:Electric
       \\key Daylighting:Controls
       \\key Shading:Zone:Detailed
       \\key Lights
       \\key Generator:Photovoltaic
       \\note  extend choice-keys as Cases are added to code
   A4 ,\\field Item Name
       \\required-field
       \\note wildcard "*" is acceptable for some components
   A5 ,\\field Object End-Use Key
       \\type alpha
       \\note not yet used
   N1 ,\\field Cost per Each
       \\type real
       \\units $
   N2 ,\\field Cost per Area
       \\type real
       \\units $/m2
   N3 ,\\field Cost per Unit of Output Capacity
       \\type real
       \\units $/kW
   N4 ,\\field Cost per Unit of Output Capacity per COP
       \\type real
       \\units $/kW
       \\note The value is per change in COP.
   N5 ,\\field Cost per Volume
       \\type real
       \\units $/m3
   N6 ,\\field Cost per Volume Rate
       \\type real
       \\units $/(m3/s)
   N7 ,\\field Cost per Energy per Temperature Difference
       \\type real
       \\units $/(W/K)
       \\note as in for use with UA sizing of Coils
   N8 ;\\field Quantity
       \\type real
       \\units dimensionless
       \\note optional for use with Cost per Each and "General" object Type

UtilityCost:Tariff,
        \\memo Defines the name of a utility cost tariff, the type of tariff, and other details
        \\memo about the overall tariff. Each other object that is part of the tariff model
        \\memo references the tariff name.  See UtilityCost:Charge:Simple, UtilityCost:Charge:Block,
        \\memo UtilityCost:Ratchet, UtilityCost:Qualify, UtilityCost:Variable and
        \\memo UtilityCost:Computation objects.
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\reference UtilityCostTariffs
        \\note The name of the tariff. Tariffs are sometimes called rates. The name is used in identifying
        \\note the output results and in associating all of the charges and other objects that make up a tariff.
   A2,  \\field Output Meter Name
        \\note The name of any standard meter or custom meter or but usually set to either Electricity:Facility or Gas:Facility
        \\required-field
        \\type external-list
        \\external-list autoRDDmeter
   A3,  \\field Conversion Factor Choice
        \\type choice
        \\key UserDefined
        \\key kWh
        \\key Therm
        \\key MMBtu
        \\key MJ
        \\key kBtu
        \\key MCF
        \\key CCF
        \\note A choice that allows several different predefined conversion factors to be used; otherwise user
        \\note defined conversion factors are used as defined in the next two fields.
   N1,  \\field Energy Conversion Factor
        \\type real
        \\note Is a multiplier used to convert energy into the units specified by the utility in their tariff. If
        \\note left blank it defaults to 1 (no conversion). This field should will be used only if Conversion Factor
        \\note Choice is set to UserDefined.  Within EnergyPlus energy always has units of J (joules).  For
        \\note conversion from J to kWh use the value of 0.0000002778. This is also used for all objects that
        \\note reference the UtilityCost:Tariff.
   N2,  \\field Demand Conversion Factor
        \\type real
        \\note Is a multiplier used to convert demand into the units specified by the utility in their tariff. If
        \\note left blank it defaults to 1 (no conversion).  This field should will be used only if Conversion
        \\note Factor Choice is set to UserDefined.  Within EnergyPlus demand always has units of J/s (joules/sec)
        \\note which equivalent to W (watts).  For conversion from W to kW use the value of 0.001. This is also used
        \\note for all objects that reference the UtilityCost:Tariff.
   A4,  \\field Time of Use Period Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The name of the schedule that defines the time-of-use periods that occur each day. The values for the
        \\note different variables are: 1 for Peak. 2 for Shoulder. 3 for OffPeak. 4 for MidPeak.
   A5,  \\field Season Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The name of a schedule that defines the seasons.  The schedule values are: 1 for Winter. 2 for Spring.
        \\note 3 for Summer. 4 for Autumn.
   A6,  \\field Month Schedule Name
        \\type object-list
        \\object-list ScheduleNames
        \\note The name of the schedule that defines the billing periods of the year. Normally this entry is allowed
        \\note to default and a schedule will be internally used that has the breaks between billing periods occurring
        \\note at the same time as the breaks between months i.e. at midnight prior to the first day of the month.
        \\note If other billing periods are used such as two month cycles or a single bill for an entire season such
        \\note as some natural gas companies do in the summer then the month schedule may be used to redefine it.
        \\note Make sure that the month schedule and season schedule are consistent otherwise an error will be issued.
   A7,  \\field Demand Window Length
        \\type choice
        \\key QuarterHour
        \\key HalfHour
        \\key FullHour
        \\key Day
        \\key Week
        \\note The determination of demand can vary by utility. Some utilities use the peak instantaneous demand
        \\note measured but most use a fifteen minute average demand or a one hour average demand. Some gas utilities
        \\note measure demand as the use during the peak day or peak week.
   A8,  \\field Monthly Charge or Variable Name
        \\note The fixed monthly service charge that many utilities have.  The entry may be numeric and gets added to
        \\note the ServiceCharges variable or if a variable name is entered here its values for each month are used.
   A9,  \\field Minimum Monthly Charge or Variable Name
        \\note The minimum total charge for the tariff or if a variable name is entered here its
        \\note values for each month are used.
   A10, \\field Real Time Pricing Charge Schedule Name
        \\note Used with real time pricing rates. The name of a schedule that contains the cost of
        \\note energy for that particular time period of the year. Real time rates can be modeled using a charge
        \\note schedule with the actual real time prices entered in the schedule.
        \\object-list ScheduleNames
   A11, \\field Customer Baseline Load Schedule Name
        \\note Used with real time pricing rates. The name of a schedule that contains the baseline
        \\note energy use for the customer. Many real time rates apply the charges as a credit or debit only to the
        \\note difference between the baseline use and the actual use.
        \\object-list ScheduleNames
   A12, \\field Group Name
        \\note The group name of the tariff such as distribution transmission supplier etc. If more than one tariff
        \\note with the same group name is present and qualifies only the lowest cost tariff is used. Usually the group
        \\note name field is left blank which results in all tariffs using the same meter variable being compared and
        \\note the lowest cost one being selected.
   A13; \\field Buy Or Sell
        \\type  choice
        \\key   BuyFromUtility
        \\key   SellToUtility
        \\key   NetMetering
        \\default BuyFromUtility
        \\note Sets whether the tariff is used for buying selling or both to the utility.  This
        \\note should be allowed to default to buyFromUtility unless a power generation system is included in the
        \\note building that may generate more power than the building needs during the year

UtilityCost:Qualify,
        \\memo The qualify object allows only tariffs to be selected based on limits which may apply
        \\memo such as maximum or minimum demand requirements. If the results of the simulation fall
        \\memo outside of the range of qualifications, that tariff is still calculated but the
        \\memo "Qualified" entry will say "No" and the UtilityCost:Qualify that caused its exclusion
        \\memo is shown. Multiple UtilityCost:Qualify objects can appear for the same tarriff and
        \\memo they can be based on any variable.
   A1,  \\field Name
        \\required-field
        \\note Displayed in the report if the tariff does not qualify
   A2,  \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Qualify.
   A3,  \\field Variable Name
        \\required-field
        \\note The name of the variable used. For energy and demand the automatically created variables totalEnergy
        \\note and totalDemand should be used respectively.
   A4,  \\field Qualify Type
        \\type choice
        \\key Minimum
        \\key Maximum
        \\default Maximum
   A5,  \\field Threshold Value or Variable Name
        \\required-field
        \\note The minimum or maximum value for the qualify. If the variable has values that are less than this value
        \\note when the qualify type is minimum then the tariff may be disqualified.  If the variable has values that
        \\note are greater than this value when the qualify type is maximum then the tariff may be disqualified.
   A6,  \\field Season
        \\type choice
        \\key Annual
        \\key Summer
        \\key Winter
        \\key Spring
        \\key Fall
        \\note If the UtilityCost:Qualify only applies to a season enter the season name. If this field is left blank
        \\note it defaults to Annual.
   A7,  \\field Threshold Test
        \\type choice
        \\key Count
        \\key Consecutive
        \\default Consecutive
        \\note Uses the number in Number of Months in one of two different ways depending on the Threshold  Test. If
        \\note the Threshold Test is set to Count then the qualification is based on the count of the total number
        \\note of months per year.  If the Threshold Test is set to consecutive then the qualification is based on
        \\note a consecutive number of months.
   N1;  \\field Number of Months
        \\minimum 1
        \\maximum 12
        \\note A number from 1 to 12.  If no value entered 12 is assumed when the qualify type is minimum and 1 when
        \\note the qualify type is maximum.  This is the number of months that the threshold test applies to determine
        \\note if the rate qualifies or not.  If the season is less than 12 months (if it is not annual) then the
        \\note value is automatically reduced to the number of months of the seaon.

UtilityCost:Charge:Simple,
        \\memo UtilityCost:Charge:Simple is one of the most often used objects for tariff
        \\memo calculation. It is used to compute energy and demand charges that are very simple.
        \\memo It may also be used for taxes, surcharges and any other charges that occur on a
        \\memo utility bill. Multiple UtilityCost:Charge:Simple objects may be defined for a single
        \\memo tariff and they will be added together.
   A1,  \\field Name
        \\required-field
        \\note Charge Variable Name
        \\note This is the name associated with the UtilityCost:Charge:Simple object and will appear in the report.
        \\note In addition the results of the UtilityCost:Charge:Simple calculation are stored in a variable with the
        \\note same name.  That way the results may be used for further calculation.  Spaces are not significant in
        \\note Charge variable names. They are removed during the utility bill calculation process.
   A2,  \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Charge:Simple.
   A3,  \\field Source Variable
        \\required-field
        \\note The name of the source used by the UtilityCost:Charge:Simple.  This is usually the name of the variable
        \\note holding the energy or demand but may also be the name of any variable including the subtotal or basis
        \\note if other charges are based on those. Typical values include totalEnergy totalDemand EnergyCharges DemandCharges
        \\note ServiceCharges Basis Adjustments Surcharges Subtotal Taxes and Total. If it is a time-of-use rate then
        \\note peakEnergy peakDemand shoulderEnergy shoulderDemand offPeakEnergy offPeakDemand midPeakEnergy and midPeakDemand.
        \\note In addition see the Tariff Report to see other native variablles that may be available. Also you can
        \\note create additional user defined variables to model complex tariffs.
   A4,  \\field Season
        \\type choice
        \\key Annual
        \\key Summer
        \\key Winter
        \\key Spring
        \\key Fall
        \\note If this is set to annual the calculations are performed for the UtilityCost:Charge:Simple for the entire
        \\note year (all months) otherwise it is calculated only for those months in the season defined.
   A5,  \\field Category Variable Name
        \\required-field
        \\type choice
        \\key EnergyCharges
        \\key DemandCharges
        \\key ServiceCharges
        \\key Basis
        \\key Adjustment
        \\key Surcharge
        \\key Subtotal
        \\key Taxes
        \\key Total
        \\key NotIncluded
        \\note This field shows where the charge should be added. The reason to enter this field appropriately is so
        \\note that the charge gets reported in a reasonable category.  The charge automatically gets added to the
        \\note variable that is the category.
   A6;  \\field Cost per Unit Value or Variable Name
        \\required-field
        \\note This field contains either a single number or the name of a variable.  The number is multiplied with
        \\note all of the energy or demand or other source that is specified in the source field.  If a variable is
        \\note used then the monthly values of the variable are multiplied against the variable specified in the
        \\note source field.  This field makes it easy to include a simple charge without specifying block sizes.
        \\note This is a good way to include a tax or cost adjustment.

UtilityCost:Charge:Block,
        \\memo Used to compute energy and demand charges (or any other charges) that are structured
        \\memo in blocks of charges. Multiple UtilityCost:Charge:Block objects may be defined for a
        \\memo single tariff and they will be added together.
   A1,  \\field Name
        \\required-field
        \\note Charge Variable Name
        \\note This is the name associated with the UtilityCost:Charge:Block object and will appear in the report.
        \\note In addition the results of the UtilityCost:Charge:Block are stored in a variable with the same name.
        \\note That way the results may be used for further calculation.
   A2,  \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Charge:Block.
   A3,  \\field Source Variable
        \\required-field
        \\note The name of the source used by the UtilityCost:Charge:Block.  This is usually the name of the variable
        \\note holding the energy or demand but may also be the name of any variable including the subtotal or basis if
        \\note other charges are based on those. Typical values include totalEnergy totalDemand EnergyCharges DemandCharges
        \\note ServiceCharges Basis Adjustments Surcharges Subtotal Taxes and Total. If it is a time-of-use rate then
        \\note peakEnergy peakDemand shoulderEnergy shoulderDemand offPeakEnergy offPeakDemand midPeakEnergy and midPeakDemand.
        \\note In addition see the Tariff Report to see other native variablles that may be available. Also you can
        \\note create additional user defined variables to model complex tariffs.
   A4,  \\field Season
        \\type choice
        \\key Annual
        \\key Summer
        \\key Winter
        \\key Spring
        \\key Fall
        \\note If this is set to annual the calculations are performed for the UtilityCost:Charge:Block for the entire
        \\note year (all months) otherwise it is calculated only for those months in the season defined.
   A5,  \\field Category Variable Name
        \\required-field
        \\type choice
        \\key EnergyCharges
        \\key DemandCharges
        \\key ServiceCharges
        \\key Basis
        \\key Adjustment
        \\key Surcharge
        \\key Subtotal
        \\key Taxes
        \\key Total
        \\key NotIncluded
        \\note This field shows where the charge should be added. The reason to enter this field appropriately is so
        \\note that the charge gets reported in a reasonable category.  The charge automatically gets added to the
        \\note variable that is the category.
   A6,  \\field Remaining Into Variable
        \\note If the blocks do not use all of the energy or demand from the source some energy and demand remains
        \\note then the remaining amount should be assigned to a variable. If no variable is assigned and some amount
        \\note of energy or demand is not used in the block structure a warning will be issued.
   A7,  \\field Block Size Multiplier Value or Variable Name
        \\note The sizes of the blocks are usually used directly but if a value or a variable is entered here the block
        \\note sizes entered in the rest of the charge are first multiplied by the entered value prior to being used.
        \\note This is common for rates that are kWh/kW rates and in that case the variable that holds the monthly
        \\note total electric demand would be entered.  If no value is entered a default value of one is assumed so
        \\note that the block sizes remain exactly as entered.  This field is unusual for the EnergyPlus syntax because
        \\note it can be either a number or a name of a variable.
   A8,  \\field Block Size 1 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A9,  \\field Block 1 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A10, \\field Block Size 2 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A11, \\field Block 2 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A12, \\field Block Size 3 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A13, \\field Block 3 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A14, \\field Block Size 4 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A15, \\field Block 4 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A16, \\field Block Size 5 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A17, \\field Block 5 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A18, \\field Block Size 6 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A19, \\field Block 6 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A20, \\field Block Size 7 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A21, \\field Block 7 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A22, \\field Block Size 8 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A23, \\field Block 8 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A24, \\field Block Size 9 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A25, \\field Block 9 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A26, \\field Block Size 10 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A27, \\field Block 10 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A28, \\field Block Size 11 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A29, \\field Block 11 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A30, \\field Block Size 12 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A31, \\field Block 12 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A32, \\field Block Size 13 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A33, \\field Block 13 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A34, \\field Block Size 14 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A35, \\field Block 14 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.
   A36, \\field Block Size 15 Value or Variable Name
        \\note The size of the block of the charges is entered here. For most rates that use multiple blocks this will
        \\note be the value for the block size. Using remaining may be used when the remaining amount should be included
        \\note in that block.  This field is unusual because it can be either a number or a name of a variable.
   A37; \\field Block 15 Cost per Unit Value or Variable Name
        \\note The cost of the block. This field is unusual for the EnergyPlus syntax because it can be either a number
        \\note or a name of a variable.

UtilityCost:Ratchet,
        \\memo Allows the modeling of tariffs that include some type of seasonal ratcheting.
        \\memo Ratchets are most common when used with electric demand charges. A ratchet is when a
        \\memo utility requires that the demand charge for a month with a low demand may be
        \\memo increased to be more consistent with a month that set a higher demand charge.
   A1,  \\field Name
        \\required-field
        \\note Ratchet Variable Name
        \\note The name of the ratchet and the name of the result of this single ratchet.
   A2,  \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Ratchet.
   A3,  \\field Baseline Source Variable
        \\required-field
        \\note When the ratcheted value exceeds the baseline value for a month the ratcheted value is used but when the
        \\note baseline value is greater then the ratcheted value the baseline value is used. Usually the electric
        \\note demand charge is used.  The baseline source variable can be the results of another ratchet object. This
        \\note allows utility tariffs that have multiple ratchets to be modeled.
   A4,  \\field Adjustment Source Variable
        \\required-field
        \\note The variable that the ratchet is calculated from. It is often but not always the same as the baseline
        \\note source variable.  The ratcheting calculations using offset and multiplier are using the values from the
        \\note adjustment source variable. If left blank the adjustment source variable is the same as the baseline
        \\note source variable.
   A5,  \\field Season From
        \\type choice
        \\key Annual
        \\key Summer
        \\key Winter
        \\key Spring
        \\key Fall
        \\key Monthly
        \\note The name of the season that is being examined.  The maximum value for all of the months in the named
        \\note season is what is used with the multiplier and offset.  This is most commonly Summer or Annual.  When
        \\note Monthly is used the adjustment source variable is used directly for all months.
   A6,  \\field Season To
        \\type choice
        \\key Annual
        \\key Summer
        \\key Winter
        \\key Spring
        \\key Fall
        \\note The name of the season when the ratchet would be calculated.  This is most commonly Winter.  The ratchet
        \\note only is applied to the months in the named season. The resulting variable for months not in the Season To
        \\note selection will contain the values as appear in the baseline source variable.
   A7,  \\field Multiplier Value or Variable Name
        \\note Often the ratchet has a clause such as "the current month demand or 90% of the summer month demand".  For
        \\note this case a value of 0.9 would be entered here as the multiplier.  This value may be left blank if no
        \\note multiplier is needed and a value of one will be used as a default.
   A8;  \\field Offset Value or Variable Name
        \\note A less common strategy is to say that the ratchet must be all demand greater than a value in this case
        \\note an offset that is added to the demand may be entered here. If entered it is common for the offset value
        \\note to be negative representing that the demand be reduced.   If no value is entered it is assumed to be
        \\note zero and not affect the ratchet.

UtilityCost:Variable,
        \\memo Allows for the direct entry of monthly values into a utility tariff variable.
  A1,   \\field Name
        \\required-field
  A2,   \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Variable.
  A3,   \\field Variable Type
        \\required-field
        \\type choice
        \\key Energy
        \\key Power
        \\key Dimensionless
        \\key Currency
        \\default Dimensionless
  N1,   \\field January Value
  N2,   \\field February Value
  N3,   \\field March Value
  N4,   \\field April Value
  N5,   \\field May Value
  N6,   \\field June Value
  N7,   \\field July Value
  N8,   \\field August Value
  N9,   \\field September Value
  N10,  \\field October Value
  N11,  \\field November Value
  N12;  \\field December Value

UtilityCost:Computation,
        \\memo The object lists a series of computations that are used to perform the utility bill
        \\memo calculation. The object is only used for complex tariffs that cannot be modeled any
        \\memo other way. For most utility tariffs, UtilityCost:Computation is unnecessary and
        \\memo should be avoided. If UtilityCost:Computation is used, it must contain references
        \\memo to all objects involved in the rate in the order that they should be computed.
  A1,   \\field Name
        \\required-field
  A2,   \\field Tariff Name
        \\required-field
        \\object-list UtilityCostTariffs
        \\note The name of the UtilityCost:Tariff that is associated with this UtilityCost:Variable.
  A3,   \\field Compute Step 1
        \\note Contain a simple language that describes the steps used in the computation process similar to a
        \\note programming language.
  A4,   \\field Compute Step 2
  A5,   \\field Compute Step 3
  A6,   \\field Compute Step 4
  A7,   \\field Compute Step 5
  A8,   \\field Compute Step 6
  A9,   \\field Compute Step 7
  A10,  \\field Compute Step 8
  A11,  \\field Compute Step 9
  A12,  \\field Compute Step 10
  A13,  \\field Compute Step 11
  A14,  \\field Compute Step 12
  A15,  \\field Compute Step 13
  A16,  \\field Compute Step 14
  A17,  \\field Compute Step 15
  A18,  \\field Compute Step 16
  A19,  \\field Compute Step 17
  A20,  \\field Compute Step 18
  A21,  \\field Compute Step 19
  A22,  \\field Compute Step 20
  A23,  \\field Compute Step 21
  A24,  \\field Compute Step 22
  A25,  \\field Compute Step 23
  A26,  \\field Compute Step 24
  A27,  \\field Compute Step 25
  A28,  \\field Compute Step 26
  A29,  \\field Compute Step 27
  A30,  \\field Compute Step 28
  A31,  \\field Compute Step 29
  A32;  \\field Compute Step 30

LifeCycleCost:Parameters,
        \\memo Provides inputs related to the overall life-cycle analysis. It establishes many of
        \\memo the assumptions used in computing the present value. It is important that when
        \\memo comparing the results of multiple simulations that the fields in the
        \\memo LifeCycleCost:Parameters objects are the same for all the simulations. When this
        \\memo object is present the tabular report file will contain the Life-Cycle Cost Report.
   \\unique-object
   \\min-fields 11
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Discounting Convention
        \\type choice
        \\key EndOfYear
        \\key MidYear
        \\key BeginningOfYear
        \\default EndOfYear
        \\note The field specifies if the discounting of future costs should be computed as occurring at the end
        \\note of each year or the middle of each year or the beginning of each year. The most common discounting
        \\note convention uses the end of each year.
   A3,  \\field Inflation Approach
        \\type choice
        \\key ConstantDollar
        \\key CurrentDollar
        \\default ConstantDollar
        \\note This field is used to determine if the analysis should use constant dollars or current dollars
        \\note which is related to how inflation is treated. If ConstantDollar is selected then the Real Discount
        \\note Rate input is used and it excludes the rate of inflation. If CurrentDollar is selected then the
        \\note Nominal Discount Rate input is used and it includes the rate of inflation.
   N1,  \\field Real Discount Rate
        \\type real
        \\note Enter the real discount rate as a decimal. For a 3% rate enter the value 0.03. This input is
        \\note used when the Inflation Approach is ConstantDollar. The real discount rate reflects the interest
        \\note rates needed to make current and future expenditures have comparable equivalent values when
        \\note general inflation is ignored. When Inflation Approach is set to CurrentDollar this input is ignored.
   N2,  \\field Nominal Discount Rate
        \\type real
        \\note Enter the nominal discount rate as a decimal. For a 5% rate enter the value 0.05. This input
        \\note is used when the Inflation Approach is CurrentDollar. The real discount rate reflects the interest
        \\note rates needed to make current and future expenditures have comparable equivalent values when general
        \\note inflation is included. When Inflation Approach is set to ConstantDollar this input is ignored.
   N3,  \\field Inflation
        \\type real
        \\note Enter the rate of inflation for general goods and services as a decimal. For a 2% rate enter
        \\note the value 0.02.
   A4,  \\field Base Date Month
        \\type choice
        \\key January
        \\key February
        \\key March
        \\key April
        \\key May
        \\key June
        \\key July
        \\key August
        \\key September
        \\key October
        \\key November
        \\key December
        \\default January
        \\note Enter the month that is the beginning of study period also known as the beginning of the base period.
   N4,  \\field Base Date Year
        \\type integer
        \\minimum 1900
        \\maximum 2100
        \\note Enter the four digit year that is the beginning of study period such as 2010. The study period is
        \\note also known as the base period.
   A5,  \\field Service Date Month
        \\type choice
        \\key January
        \\key February
        \\key March
        \\key April
        \\key May
        \\key June
        \\key July
        \\key August
        \\key September
        \\key October
        \\key November
        \\key December
        \\default January
        \\note Enter the month that is the beginning of building occupancy. Energy costs computed by EnergyPlus
        \\note are assumed to occur during the year following the service date. The service date must be the
        \\note same or later than the Base Date. This field could also be referred to as part of beneficial
        \\note occupancy date.
   N5,  \\field Service Date Year
        \\type integer
        \\minimum 1900
        \\maximum 2100
        \\note Enter the four digit year that is the beginning of occupancy such as 2010.
   N6,  \\field Length of Study Period in Years
        \\type integer
        \\minimum 1
        \\maximum 100
        \\note Enter the number of years of the study period. It is the number of years that the study continues
        \\note based on the start at the base date. The default value is 25 years. Only integers may be used
        \\note indicating whole years.
   N7,  \\field Tax rate
        \\type real
        \\minimum 0.0
        \\note Enter the overall marginal tax rate for the project costs. This does not include energy or water
        \\note taxes. The tax rate entered should be based on the marginal tax rate for the entity and not the
        \\note average tax rate. Enter the tax rate results in present value calculations after taxes. Most
        \\note analyses do not factor in the impact of taxes and assume that all options under consideration
        \\note have roughly the same tax impact. Due to this many times the tax rate can be left to default
        \\note to zero and the present value results before taxes are used to make decisions. The value
        \\note should be entered as a decimal value. For 15% enter 0.15. For an analysis that does not include
        \\note tax impacts enter 0.0.
   A6;  \\field Depreciation Method
        \\type choice
        \\key ModifiedAcceleratedCostRecoverySystem-3year
        \\key ModifiedAcceleratedCostRecoverySystem-5year
        \\key ModifiedAcceleratedCostRecoverySystem-7year
        \\key ModifiedAcceleratedCostRecoverySystem-10year
        \\key ModifiedAcceleratedCostRecoverySystem-15year
        \\key ModifiedAcceleratedCostRecoverySystem-20year
        \\key StraightLine-27year
        \\key StraightLine-31year
        \\key StraightLine-39year
        \\key StraightLine-40year
        \\key None
        \\default None
        \\note For an analysis that includes income tax impacts this entry describes how capital costs are
        \\note depreciated. Only one depreciation method may be used for an analysis and is applied to all
        \\note capital expenditures.

LifeCycleCost:RecurringCosts,
        \\memo Recurring costs are costs that repeat over time on a regular schedule during the
        \\memo study period. If costs associated with equipment do repeat but not on a regular
        \\memo schedule, use LifeCycleCost:NonrecurringCost objects instead.
        \\min-fields 7
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Category
        \\type choice
        \\key Maintenance
        \\key Repair
        \\key Operation
        \\key Replacement
        \\key MinorOverhaul
        \\key MajorOverhaul
        \\key OtherOperational
        \\default Maintenance
   N1,  \\field Cost
        \\type real
        \\note Enter the cost in dollars (or the appropriate monetary unit) for the recurring costs. Enter
        \\note the cost for each time it occurs. For example if the annual maintenance cost is 500 dolllars
        \\note enter 500 here.
   A3,  \\field Start of Costs
        \\type choice
        \\key ServicePeriod
        \\key BasePeriod
        \\default ServicePeriod
        \\note Enter when the costs start. The First Year of Cost is based on the number of years past the
        \\note Start of Costs. For most maintenance costs the Start of Costs should be Service Period.
   N2,  \\field Years from Start
        \\type integer
        \\minimum 0
        \\maximum 100
        \\note This field and the Months From Start field together represent the time from either the start
        \\note of the Service Period on the service month and year or start of the Base Period on the base
        \\note month and year (depending on the Start of Costs field) that the costs start to occur. Only
        \\note integers should be entered representing whole years.
   N3,  \\field Months from Start
        \\type integer
        \\minimum 0
        \\maximum 1200
        \\note This field and the Years From Start field together represent the time from either the start
        \\note of the Service Period on the service month and year or start of the Base Period on the base
        \\note month and year (depending on the Start of Costs field) that the costs start to occur. Only
        \\note integers should be entered representing whole months. The Years From Start (times 12) and
        \\note Months From Start are added together.
   N4,  \\field Repeat Period Years
        \\type integer
        \\minimum 0
        \\maximum 100
        \\default 1
        \\note This field and the Repeat Period Months field indicate how much time elapses between
        \\note reoccurrences of the cost. For costs that occur every year such the Repeat Period Years
        \\note should be 1 and Repeat Period Months should be 0. Only integers should be entered
        \\note representing whole years.
   N5,  \\field Repeat Period Months
        \\type integer
        \\minimum 0
        \\maximum 1200
        \\default 0
        \\note This field and the Repeat Period Years field indicate how much time elapses between
        \\note reoccurrences of the cost. Only integers should be entered representing whole years.
        \\note The Repeat Period Years (times 12) and Repeat Period Months are added together.
   N6;  \\field Annual escalation rate
        \\type real
        \\minimum -0.3
        \\maximum 0.3
        \\note Enter the annual escalation rate as a decimal. For a 1% rate enter the value 0.01.
        \\note This input is used when the Inflation Approach is CurrentDollar. When Inflation
        \\note Approach is set to ConstantDollar this input is ignored.

LifeCycleCost:NonrecurringCost,
        \\memo A non-recurring cost happens only once during the study period. For costs that occur
        \\memo more than once during the study period on a regular schedule, use the
        \\memo LifeCycleCost:RecurringCost object.
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Category
        \\type choice
        \\key Construction
        \\key Salvage
        \\key OtherCapital
        \\default Construction
   N1,  \\field Cost
        \\type real
        \\note Enter the non-recurring cost value. For construction and other capital costs the value
        \\note entered is typically a positive value. For salvage costs the value entered is typically a
        \\note negative value which represents the money paid to the investor for the equipment at the
        \\note end of the study period.
   A3,  \\field Start of Costs
        \\type choice
        \\key ServicePeriod
        \\key BasePeriod
        \\default ServicePeriod
        \\note Enter when the costs start. The First Year of Cost is based on the number of years past the
        \\note Start of Costs. For most non-recurring costs the Start of Costs should be Base Period which
        \\note begins at the base month and year.
   N2,  \\field Years from Start
        \\type integer
        \\minimum 0
        \\maximum 100
        \\note This field and the Months From Start field together represent the time from either the start
        \\note of the Service Period on the service month and year or start of the Base Period on the base
        \\note month and year (depending on the Start of Cost field) that the costs start to occur. Only
        \\note integers should be entered representing whole years.
   N3;  \\field Months from Start
        \\type integer
        \\minimum 0
        \\maximum 1200
        \\note This field and the Years From Start field together represent the time from either the start
        \\note of the Service Period on the service month and year or start of the Base Period on the base
        \\note month and year (depending on the Start of Cost field) that the costs start to occur. Only
        \\note integers should be entered representing whole months. The Years From Start (times 12) and
        \\note Months From Start are added together.

LifeCycleCost:UsePriceEscalation,
        \\memo Life cycle cost escalation factors. The values for this object may be found in the
        \\memo annual supplement to NIST Handbook 135 in Tables Ca-1 to Ca-5 and are included in an
        \\memo EnergyPlus dataset file.
        \\extensible:1
   A1,  \\field Name
        \\required-field
        \\type alpha
        \\note The identifier used for the object. The name usually identifies the location (such as the
        \\note state or region or country or census area) that the escalations apply to. In addition the
        \\note name should identify the building class such as residential or commercial or industrial
        \\note and the use type such as electricity or natural gas or water.
   A2,  \\field Resource
        \\required-field
        \\type choice
        \\key Electricity
        \\key NaturalGas
        \\key Steam
        \\key Gasoline
        \\key Diesel
        \\key Coal
        \\key FuelOil#1
        \\key FuelOil#2
        \\key Propane
        \\key OtherFuel1
        \\key OtherFuel2
        \\key Water
   N1,  \\field Escalation Start Year
        \\type integer
        \\minimum 1900
        \\maximum 2100
        \\note This field and the Escalation Start Month define the time that corresponds to Year 1 Escalation
        \\note such as 2010 when the escalation rates are applied. This field and the Escalation Start Month
        \\note define the time that escalation begins.
   A3,  \\field Escalation Start Month
        \\type choice
        \\key January
        \\key February
        \\key March
        \\key April
        \\key May
        \\key June
        \\key July
        \\key August
        \\key September
        \\key October
        \\key November
        \\key December
        \\default January
        \\note This field and the Escalation Start Year define the time that corresponds to Year 1 Escalation
        \\note such as 2010 when the escalation rates are applied. This field and the Escalation Start Year
        \\note define the time that escalation begins.
   N2,  \\field Year 1 Escalation
        \\type real
        \\begin-extensible
        \\note The escalation in price of the energy or water use for the first year expressed as a decimal.
   N3,  \\field Year 2 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N4,  \\field Year 3 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N5,  \\field Year 4 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N6,  \\field Year 5 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N7,  \\field Year 6 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N8,  \\field Year 7 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N9,  \\field Year 8 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N10, \\field Year 9 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N11, \\field Year 10 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N12, \\field Year 11 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N13, \\field Year 12 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N14, \\field Year 13 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N15, \\field Year 14 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N16, \\field Year 15 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N17, \\field Year 16 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N18, \\field Year 17 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N19, \\field Year 18 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N20, \\field Year 19 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N21, \\field Year 20 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N22, \\field Year 21 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N23, \\field Year 22 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N24, \\field Year 23 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N25, \\field Year 24 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N26, \\field Year 25 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N27, \\field Year 26 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N28, \\field Year 27 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N29, \\field Year 28 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N30, \\field Year 29 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.
   N31; \\field Year 30 Escalation
        \\type real
        \\note The escalation in price of the energy or water use for the year expressed as a decimal.

LifeCycleCost:UseAdjustment,
        \\memo Used by advanced users to adjust the energy or water use costs for future years. This
        \\memo should not be used for compensating for inflation but should only be used to increase
        \\memo the costs of energy or water based on assumed changes to the actual usage, such as
        \\memo anticipated changes in the future function of the building. The adjustments begin at
        \\memo the start of the service period.
        \\extensible:1
   A1,  \\field Name
        \\required-field
        \\type alpha
   A2,  \\field Resource
        \\required-field
        \\type choice
        \\key Electricity
        \\key NaturalGas
        \\key Steam
        \\key Gasoline
        \\key Diesel
        \\key Coal
        \\key FuelOil#1
        \\key FuelOil#2
        \\key Propane
        \\key OtherFuel1
        \\key OtherFuel2
        \\key Water
   N1,  \\field Year 1 Multiplier
        \\type real
        \\begin-extensible
        \\note The multiplier to be applied to the end-use cost for the first year in the service period.
        \\note The total utility costs for the selected end-use is multiplied by this value. For no change
        \\note enter 1.0.
   N2,  \\field Year 2 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N3,  \\field Year 3 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N4,  \\field Year 4 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N5,  \\field Year 5 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N6,  \\field Year 6 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N7,  \\field Year 7 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N8,  \\field Year 8 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N9,  \\field Year 9 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N10, \\field Year 10 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N11, \\field Year 11 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N12, \\field Year 12 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N13, \\field Year 13 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N14, \\field Year 14 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N15, \\field Year 15 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N16, \\field Year 16 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N17, \\field Year 17 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N18, \\field Year 18 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N19, \\field Year 19 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N20, \\field Year 20 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N21, \\field Year 21 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N22, \\field Year 22 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N23, \\field Year 23 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N24, \\field Year 24 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N25, \\field Year 25 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N26, \\field Year 26 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N27, \\field Year 27 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N28, \\field Year 28 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N29, \\field Year 29 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.
   N30; \\field Year 30 Multiplier
        \\type real
        \\note The multiplier to be applied to the end-use cost for each following year. The total utility
        \\note costs for the selected end-use is multiplied by this value. For no change enter 1.0.

\\Group Parametrics

Parametric:SetValueForRun,
       \\memo Parametric objects allow a set of multiple simulations to be defined in a single idf
       \\memo file. The parametric preprocessor scans the idf for Parametric:* objects then creates
       \\memo and runs multiple idf files, one for each defined simulation.
       \\memo The core parametric object is Parametric:SetValueForRun which defines the name
       \\memo of a parameters and sets the parameter to different values depending on which
       \\memo run is being simulated.
       \\min-fields 2
       \\extensible:1
  A1,  \\field Name
       \\note Parameter Name
       \\note Must begin with the dollar sign character. The second character must be a letter.
       \\note Remaining characters may only be letters or numbers. No spaces allowed.
       \\required-field
       \\type  alpha
  A2,  \\field Value for Run 1
       \\begin-extensible
       \\type  alpha
  A3,  \\field Value for Run 2
       \\type  alpha
  A4,  \\field Value for Run 3
       \\type  alpha
  A5,  \\field Value for Run 4
       \\type  alpha
  A6,  \\field Value for Run 5
       \\type  alpha
  A7,  \\field Value for Run 6
       \\type  alpha
  A8,  \\field Value for Run 7
       \\type  alpha
  A9,  \\field Value for Run 8
       \\type  alpha
  A10, \\field Value for Run 9
       \\type  alpha
  A11, \\field Value for Run 10
       \\type  alpha
  A12, \\field Value for Run 11
       \\type  alpha
  A13, \\field Value for Run 12
       \\type  alpha
  A14, \\field Value for Run 13
       \\type  alpha
  A15, \\field Value for Run 14
       \\type  alpha
  A16, \\field Value for Run 15
       \\type  alpha
  A17, \\field Value for Run 16
       \\type  alpha
  A18, \\field Value for Run 17
       \\type  alpha
  A19, \\field Value for Run 18
       \\type  alpha
  A20, \\field Value for Run 19
       \\type  alpha
  A21, \\field Value for Run 20
       \\type  alpha
  A22, \\field Value for Run 21
       \\type  alpha
  A23, \\field Value for Run 22
       \\type  alpha
  A24, \\field Value for Run 23
       \\type  alpha
  A25, \\field Value for Run 24
       \\type  alpha
  A26, \\field Value for Run 25
       \\type  alpha
  A27, \\field Value for Run 26
       \\type  alpha
  A28, \\field Value for Run 27
       \\type  alpha
  A29, \\field Value for Run 28
       \\type  alpha
  A30, \\field Value for Run 29
       \\type  alpha
  A31, \\field Value for Run 30
       \\type  alpha
  A32, \\field Value for Run 31
       \\type  alpha
  A33, \\field Value for Run 32
       \\type  alpha
  A34, \\field Value for Run 33
       \\type  alpha
  A35, \\field Value for Run 34
       \\type  alpha
  A36, \\field Value for Run 35
       \\type  alpha
  A37, \\field Value for Run 36
       \\type  alpha
  A38, \\field Value for Run 37
       \\type  alpha
  A39, \\field Value for Run 38
       \\type  alpha
  A40, \\field Value for Run 39
       \\type  alpha
  A41, \\field Value for Run 40
       \\type  alpha
  A42, \\field Value for Run 41
       \\type  alpha
  A43, \\field Value for Run 42
       \\type  alpha
  A44, \\field Value for Run 43
       \\type  alpha
  A45, \\field Value for Run 44
       \\type  alpha
  A46, \\field Value for Run 45
       \\type  alpha
  A47, \\field Value for Run 46
       \\type  alpha
  A48, \\field Value for Run 47
       \\type  alpha
  A49, \\field Value for Run 48
       \\type  alpha
  A50, \\field Value for Run 49
       \\type  alpha
  A51, \\field Value for Run 50
       \\type  alpha
  A52, \\field Value for Run 51
       \\type  alpha
  A53, \\field Value for Run 52
       \\type  alpha
  A54, \\field Value for Run 53
       \\type  alpha
  A55, \\field Value for Run 54
       \\type  alpha
  A56, \\field Value for Run 55
       \\type  alpha
  A57, \\field Value for Run 56
       \\type  alpha
  A58, \\field Value for Run 57
       \\type  alpha
  A59, \\field Value for Run 58
       \\type  alpha
  A60, \\field Value for Run 59
       \\type  alpha
  A61, \\field Value for Run 60
       \\type  alpha
  A62, \\field Value for Run 61
       \\type  alpha
  A63, \\field Value for Run 62
       \\type  alpha
  A64, \\field Value for Run 63
       \\type  alpha
  A65, \\field Value for Run 64
       \\type  alpha
  A66, \\field Value for Run 65
       \\type  alpha
  A67, \\field Value for Run 66
       \\type  alpha
  A68, \\field Value for Run 67
       \\type  alpha
  A69, \\field Value for Run 68
       \\type  alpha
  A70, \\field Value for Run 69
       \\type  alpha
  A71, \\field Value for Run 70
       \\type  alpha
  A72, \\field Value for Run 71
       \\type  alpha
  A73, \\field Value for Run 72
       \\type  alpha
  A74, \\field Value for Run 73
       \\type  alpha
  A75, \\field Value for Run 74
       \\type  alpha
  A76, \\field Value for Run 75
       \\type  alpha
  A77, \\field Value for Run 76
       \\type  alpha
  A78, \\field Value for Run 77
       \\type  alpha
  A79, \\field Value for Run 78
       \\type  alpha
  A80, \\field Value for Run 79
       \\type  alpha
  A81, \\field Value for Run 80
       \\type  alpha
  A82, \\field Value for Run 81
       \\type  alpha
  A83, \\field Value for Run 82
       \\type  alpha
  A84, \\field Value for Run 83
       \\type  alpha
  A85, \\field Value for Run 84
       \\type  alpha
  A86, \\field Value for Run 85
       \\type  alpha
  A87, \\field Value for Run 86
       \\type  alpha
  A88, \\field Value for Run 87
       \\type  alpha
  A89, \\field Value for Run 88
       \\type  alpha
  A90, \\field Value for Run 89
       \\type  alpha
  A91, \\field Value for Run 90
       \\type  alpha
  A92, \\field Value for Run 91
       \\type  alpha
  A93, \\field Value for Run 92
       \\type  alpha
  A94, \\field Value for Run 93
       \\type  alpha
  A95, \\field Value for Run 94
       \\type  alpha
  A96, \\field Value for Run 95
       \\type  alpha
  A97, \\field Value for Run 96
       \\type  alpha
  A98, \\field Value for Run 97
       \\type  alpha
  A99, \\field Value for Run 98
       \\type  alpha
  A100, \\field Value for Run 99
        \\type  alpha
  A101; \\field Value for Run 100
        \\type  alpha


Parametric:Logic,
       \\memo This object allows some types of objects to be included for some parametric cases and
       \\memo not for others. For example, you might want an overhang on a window in some
       \\memo parametric runs and not others. A single Parametric:Logic object is allowed per file.
       \\memo Consult the Input Output Reference for available commands and syntax.
       \\unique-object
       \\min-fields 2
       \\extensible:1
  A1,  \\field Name
       \\required-field
       \\type  alpha
  A2,  \\field Parametric Logic Line 1
       \\begin-extensible
       \\type  alpha
  A3,  \\field Parametric Logic Line 2
       \\type  alpha
  A4,  \\field Parametric Logic Line 3
       \\type  alpha
  A5,  \\field Parametric Logic Line 4
       \\type  alpha
  A6,  \\field Parametric Logic Line 5
       \\type  alpha
  A7,  \\field Parametric Logic Line 6
       \\type  alpha
  A8,  \\field Parametric Logic Line 7
       \\type  alpha
  A9,  \\field Parametric Logic Line 8
       \\type  alpha
  A10, \\field Parametric Logic Line 9
       \\type  alpha
  A11, \\field Parametric Logic Line 10
       \\type  alpha
  A12, \\field Parametric Logic Line 11
       \\type  alpha
  A13, \\field Parametric Logic Line 12
       \\type  alpha
  A14, \\field Parametric Logic Line 13
       \\type  alpha
  A15, \\field Parametric Logic Line 14
       \\type  alpha
  A16, \\field Parametric Logic Line 15
       \\type  alpha
  A17, \\field Parametric Logic Line 16
       \\type  alpha
  A18, \\field Parametric Logic Line 17
       \\type  alpha
  A19, \\field Parametric Logic Line 18
       \\type  alpha
  A20, \\field Parametric Logic Line 19
       \\type  alpha
  A21, \\field Parametric Logic Line 20
       \\type  alpha
  A22, \\field Parametric Logic Line 21
       \\type  alpha
  A23, \\field Parametric Logic Line 22
       \\type  alpha
  A24, \\field Parametric Logic Line 23
       \\type  alpha
  A25, \\field Parametric Logic Line 24
       \\type  alpha
  A26, \\field Parametric Logic Line 25
       \\type  alpha
  A27, \\field Parametric Logic Line 26
       \\type  alpha
  A28, \\field Parametric Logic Line 27
       \\type  alpha
  A29, \\field Parametric Logic Line 28
       \\type  alpha
  A30, \\field Parametric Logic Line 29
       \\type  alpha
  A31, \\field Parametric Logic Line 30
       \\type  alpha
  A32, \\field Parametric Logic Line 31
       \\type  alpha
  A33, \\field Parametric Logic Line 32
       \\type  alpha
  A34, \\field Parametric Logic Line 33
       \\type  alpha
  A35, \\field Parametric Logic Line 34
       \\type  alpha
  A36, \\field Parametric Logic Line 35
       \\type  alpha
  A37, \\field Parametric Logic Line 36
       \\type  alpha
  A38, \\field Parametric Logic Line 37
       \\type  alpha
  A39, \\field Parametric Logic Line 38
       \\type  alpha
  A40, \\field Parametric Logic Line 39
       \\type  alpha
  A41, \\field Parametric Logic Line 40
       \\type  alpha
  A42, \\field Parametric Logic Line 41
       \\type  alpha
  A43, \\field Parametric Logic Line 42
       \\type  alpha
  A44, \\field Parametric Logic Line 43
       \\type  alpha
  A45, \\field Parametric Logic Line 44
       \\type  alpha
  A46, \\field Parametric Logic Line 45
       \\type  alpha
  A47, \\field Parametric Logic Line 46
       \\type  alpha
  A48, \\field Parametric Logic Line 47
       \\type  alpha
  A49, \\field Parametric Logic Line 48
       \\type  alpha
  A50, \\field Parametric Logic Line 49
       \\type  alpha
  A51, \\field Parametric Logic Line 50
       \\type  alpha
  A52, \\field Parametric Logic Line 51
       \\type  alpha
  A53, \\field Parametric Logic Line 52
       \\type  alpha
  A54, \\field Parametric Logic Line 53
       \\type  alpha
  A55, \\field Parametric Logic Line 54
       \\type  alpha
  A56, \\field Parametric Logic Line 55
       \\type  alpha
  A57, \\field Parametric Logic Line 56
       \\type  alpha
  A58, \\field Parametric Logic Line 57
       \\type  alpha
  A59, \\field Parametric Logic Line 58
       \\type  alpha
  A60, \\field Parametric Logic Line 59
       \\type  alpha
  A61, \\field Parametric Logic Line 60
       \\type  alpha
  A62, \\field Parametric Logic Line 61
       \\type  alpha
  A63, \\field Parametric Logic Line 62
       \\type  alpha
  A64, \\field Parametric Logic Line 63
       \\type  alpha
  A65, \\field Parametric Logic Line 64
       \\type  alpha
  A66, \\field Parametric Logic Line 65
       \\type  alpha
  A67, \\field Parametric Logic Line 66
       \\type  alpha
  A68, \\field Parametric Logic Line 67
       \\type  alpha
  A69, \\field Parametric Logic Line 68
       \\type  alpha
  A70, \\field Parametric Logic Line 69
       \\type  alpha
  A71, \\field Parametric Logic Line 70
       \\type  alpha
  A72, \\field Parametric Logic Line 71
       \\type  alpha
  A73, \\field Parametric Logic Line 72
       \\type  alpha
  A74, \\field Parametric Logic Line 73
       \\type  alpha
  A75, \\field Parametric Logic Line 74
       \\type  alpha
  A76, \\field Parametric Logic Line 75
       \\type  alpha
  A77, \\field Parametric Logic Line 76
       \\type  alpha
  A78, \\field Parametric Logic Line 77
       \\type  alpha
  A79, \\field Parametric Logic Line 78
       \\type  alpha
  A80, \\field Parametric Logic Line 79
       \\type  alpha
  A81, \\field Parametric Logic Line 80
       \\type  alpha
  A82, \\field Parametric Logic Line 81
       \\type  alpha
  A83, \\field Parametric Logic Line 82
       \\type  alpha
  A84, \\field Parametric Logic Line 83
       \\type  alpha
  A85, \\field Parametric Logic Line 84
       \\type  alpha
  A86, \\field Parametric Logic Line 85
       \\type  alpha
  A87, \\field Parametric Logic Line 86
       \\type  alpha
  A88, \\field Parametric Logic Line 87
       \\type  alpha
  A89, \\field Parametric Logic Line 88
       \\type  alpha
  A90, \\field Parametric Logic Line 89
       \\type  alpha
  A91, \\field Parametric Logic Line 90
       \\type  alpha
  A92, \\field Parametric Logic Line 91
       \\type  alpha
  A93, \\field Parametric Logic Line 92
       \\type  alpha
  A94, \\field Parametric Logic Line 93
       \\type  alpha
  A95, \\field Parametric Logic Line 94
       \\type  alpha
  A96, \\field Parametric Logic Line 95
       \\type  alpha
  A97, \\field Parametric Logic Line 96
       \\type  alpha
  A98, \\field Parametric Logic Line 97
       \\type  alpha
  A99, \\field Parametric Logic Line 98
       \\type  alpha
  A100, \\field Parametric Logic Line 99
        \\type  alpha
  A101; \\field Parametric Logic Line 100
        \\type  alpha

Parametric:RunControl,
       \\memo Controls which parametric runs are simulated. This object is optional. If it is not
       \\memo included, then all parametric runs are performed.
       \\unique-object
       \\min-fields 2
       \\extensible:1
  A1,  \\field Name
       \\type  alpha
  A2,  \\field Perform Run 1
       \\type choice
       \\key Yes
       \\key No
       \\default Yes
       \\begin-extensible
  A3,  \\field Perform Run 2
       \\type choice
       \\key Yes
       \\key No
  A4,  \\field Perform Run 3
       \\type choice
       \\key Yes
       \\key No
  A5,  \\field Perform Run 4
       \\type choice
       \\key Yes
       \\key No
  A6,  \\field Perform Run 5
       \\type choice
       \\key Yes
       \\key No
  A7,  \\field Perform Run 6
       \\type choice
       \\key Yes
       \\key No
  A8,  \\field Perform Run 7
       \\type choice
       \\key Yes
       \\key No
  A9,  \\field Perform Run 8
       \\type choice
       \\key Yes
       \\key No
  A10, \\field Perform Run 9
       \\type choice
       \\key Yes
       \\key No
  A11, \\field Perform Run 10
       \\type choice
       \\key Yes
       \\key No
  A12, \\field Perform Run 11
       \\type choice
       \\key Yes
       \\key No
  A13, \\field Perform Run 12
       \\type choice
       \\key Yes
       \\key No
  A14, \\field Perform Run 13
       \\type choice
       \\key Yes
       \\key No
  A15, \\field Perform Run 14
       \\type choice
       \\key Yes
       \\key No
  A16, \\field Perform Run 15
       \\type choice
       \\key Yes
       \\key No
  A17, \\field Perform Run 16
       \\type choice
       \\key Yes
       \\key No
  A18, \\field Perform Run 17
       \\type choice
       \\key Yes
       \\key No
  A19, \\field Perform Run 18
       \\type choice
       \\key Yes
       \\key No
  A20, \\field Perform Run 19
       \\type choice
       \\key Yes
       \\key No
  A21, \\field Perform Run 20
       \\type choice
       \\key Yes
       \\key No
  A22, \\field Perform Run 21
       \\type choice
       \\key Yes
       \\key No
  A23, \\field Perform Run 22
       \\type choice
       \\key Yes
       \\key No
  A24, \\field Perform Run 23
       \\type choice
       \\key Yes
       \\key No
  A25, \\field Perform Run 24
       \\type choice
       \\key Yes
       \\key No
  A26, \\field Perform Run 25
       \\type choice
       \\key Yes
       \\key No
  A27, \\field Perform Run 26
       \\type choice
       \\key Yes
       \\key No
  A28, \\field Perform Run 27
       \\type choice
       \\key Yes
       \\key No
  A29, \\field Perform Run 28
       \\type choice
       \\key Yes
       \\key No
  A30, \\field Perform Run 29
       \\type choice
       \\key Yes
       \\key No
  A31, \\field Perform Run 30
       \\type choice
       \\key Yes
       \\key No
  A32, \\field Perform Run 31
       \\type choice
       \\key Yes
       \\key No
  A33, \\field Perform Run 32
       \\type choice
       \\key Yes
       \\key No
  A34, \\field Perform Run 33
       \\type choice
       \\key Yes
       \\key No
  A35, \\field Perform Run 34
       \\type choice
       \\key Yes
       \\key No
  A36, \\field Perform Run 35
       \\type choice
       \\key Yes
       \\key No
  A37, \\field Perform Run 36
       \\type choice
       \\key Yes
       \\key No
  A38, \\field Perform Run 37
       \\type choice
       \\key Yes
       \\key No
  A39, \\field Perform Run 38
       \\type choice
       \\key Yes
       \\key No
  A40, \\field Perform Run 39
       \\type choice
       \\key Yes
       \\key No
  A41, \\field Perform Run 40
       \\type choice
       \\key Yes
       \\key No
  A42, \\field Perform Run 41
       \\type choice
       \\key Yes
       \\key No
  A43, \\field Perform Run 42
       \\type choice
       \\key Yes
       \\key No
  A44, \\field Perform Run 43
       \\type choice
       \\key Yes
       \\key No
  A45, \\field Perform Run 44
       \\type choice
       \\key Yes
       \\key No
  A46, \\field Perform Run 45
       \\type choice
       \\key Yes
       \\key No
  A47, \\field Perform Run 46
       \\type choice
       \\key Yes
       \\key No
  A48, \\field Perform Run 47
       \\type choice
       \\key Yes
       \\key No
  A49, \\field Perform Run 48
       \\type choice
       \\key Yes
       \\key No
  A50, \\field Perform Run 49
       \\type choice
       \\key Yes
       \\key No
  A51, \\field Perform Run 50
       \\type choice
       \\key Yes
       \\key No
  A52, \\field Perform Run 51
       \\type choice
       \\key Yes
       \\key No
  A53, \\field Perform Run 52
       \\type choice
       \\key Yes
       \\key No
  A54, \\field Perform Run 53
       \\type choice
       \\key Yes
       \\key No
  A55, \\field Perform Run 54
       \\type choice
       \\key Yes
       \\key No
  A56, \\field Perform Run 55
       \\type choice
       \\key Yes
       \\key No
  A57, \\field Perform Run 56
       \\type choice
       \\key Yes
       \\key No
  A58, \\field Perform Run 57
       \\type choice
       \\key Yes
       \\key No
  A59, \\field Perform Run 58
       \\type choice
       \\key Yes
       \\key No
  A60, \\field Perform Run 59
       \\type choice
       \\key Yes
       \\key No
  A61, \\field Perform Run 60
       \\type choice
       \\key Yes
       \\key No
  A62, \\field Perform Run 61
       \\type choice
       \\key Yes
       \\key No
  A63, \\field Perform Run 62
       \\type choice
       \\key Yes
       \\key No
  A64, \\field Perform Run 63
       \\type choice
       \\key Yes
       \\key No
  A65, \\field Perform Run 64
       \\type choice
       \\key Yes
       \\key No
  A66, \\field Perform Run 65
       \\type choice
       \\key Yes
       \\key No
  A67, \\field Perform Run 66
       \\type choice
       \\key Yes
       \\key No
  A68, \\field Perform Run 67
       \\type choice
       \\key Yes
       \\key No
  A69, \\field Perform Run 68
       \\type choice
       \\key Yes
       \\key No
  A70, \\field Perform Run 69
       \\type choice
       \\key Yes
       \\key No
  A71, \\field Perform Run 70
       \\type choice
       \\key Yes
       \\key No
  A72, \\field Perform Run 71
       \\type choice
       \\key Yes
       \\key No
  A73, \\field Perform Run 72
       \\type choice
       \\key Yes
       \\key No
  A74, \\field Perform Run 73
       \\type choice
       \\key Yes
       \\key No
  A75, \\field Perform Run 74
       \\type choice
       \\key Yes
       \\key No
  A76, \\field Perform Run 75
       \\type choice
       \\key Yes
       \\key No
  A77, \\field Perform Run 76
       \\type choice
       \\key Yes
       \\key No
  A78, \\field Perform Run 77
       \\type choice
       \\key Yes
       \\key No
  A79, \\field Perform Run 78
       \\type choice
       \\key Yes
       \\key No
  A80, \\field Perform Run 79
       \\type choice
       \\key Yes
       \\key No
  A81, \\field Perform Run 80
       \\type choice
       \\key Yes
       \\key No
  A82, \\field Perform Run 81
       \\type choice
       \\key Yes
       \\key No
  A83, \\field Perform Run 82
       \\type choice
       \\key Yes
       \\key No
  A84, \\field Perform Run 83
       \\type choice
       \\key Yes
       \\key No
  A85, \\field Perform Run 84
       \\type choice
       \\key Yes
       \\key No
  A86, \\field Perform Run 85
       \\type choice
       \\key Yes
       \\key No
  A87, \\field Perform Run 86
       \\type choice
       \\key Yes
       \\key No
  A88, \\field Perform Run 87
       \\type choice
       \\key Yes
       \\key No
  A89, \\field Perform Run 88
       \\type choice
       \\key Yes
       \\key No
  A90, \\field Perform Run 89
       \\type choice
       \\key Yes
       \\key No
  A91, \\field Perform Run 90
       \\type choice
       \\key Yes
       \\key No
  A92, \\field Perform Run 91
       \\type choice
       \\key Yes
       \\key No
  A93, \\field Perform Run 92
       \\type choice
       \\key Yes
       \\key No
  A94, \\field Perform Run 93
       \\type choice
       \\key Yes
       \\key No
  A95, \\field Perform Run 94
       \\type choice
       \\key Yes
       \\key No
  A96, \\field Perform Run 95
       \\type choice
       \\key Yes
       \\key No
  A97, \\field Perform Run 96
       \\type choice
       \\key Yes
       \\key No
  A98, \\field Perform Run 97
       \\type choice
       \\key Yes
       \\key No
  A99, \\field Perform Run 98
       \\type choice
       \\key Yes
       \\key No
  A100, \\field Perform Run 99
        \\type choice
        \\key Yes
        \\key No
  A101; \\field Perform Run 100
        \\type choice
        \\key Yes
        \\key No

Parametric:FileNameSuffix,
       \\memo Defines the suffixes to be appended to the idf and output file names for each
       \\memo parametric run. If this object is omitted, the suffix will default to the run number.
       \\unique-object
       \\min-fields 2
       \\extensible:1
  A1,  \\field Name
       \\type  alpha
  A2,  \\field Suffix for File Name in Run 1
       \\type  alpha
       \\begin-extensible
  A3,  \\field Suffix for File Name in Run 2
       \\type  alpha
  A4,  \\field Suffix for File Name in Run 3
       \\type  alpha
  A5,  \\field Suffix for File Name in Run 4
       \\type  alpha
  A6,  \\field Suffix for File Name in Run 5
       \\type  alpha
  A7,  \\field Suffix for File Name in Run 6
       \\type  alpha
  A8,  \\field Suffix for File Name in Run 7
       \\type  alpha
  A9,  \\field Suffix for File Name in Run 8
       \\type  alpha
  A10, \\field Suffix for File Name in Run 9
       \\type  alpha
  A11, \\field Suffix for File Name in Run 10
       \\type  alpha
  A12, \\field Suffix for File Name in Run 11
       \\type  alpha
  A13, \\field Suffix for File Name in Run 12
       \\type  alpha
  A14, \\field Suffix for File Name in Run 13
       \\type  alpha
  A15, \\field Suffix for File Name in Run 14
       \\type  alpha
  A16, \\field Suffix for File Name in Run 15
       \\type  alpha
  A17, \\field Suffix for File Name in Run 16
       \\type  alpha
  A18, \\field Suffix for File Name in Run 17
       \\type  alpha
  A19, \\field Suffix for File Name in Run 18
       \\type  alpha
  A20, \\field Suffix for File Name in Run 19
       \\type  alpha
  A21, \\field Suffix for File Name in Run 20
       \\type  alpha
  A22, \\field Suffix for File Name in Run 21
       \\type  alpha
  A23, \\field Suffix for File Name in Run 22
       \\type  alpha
  A24, \\field Suffix for File Name in Run 23
       \\type  alpha
  A25, \\field Suffix for File Name in Run 24
       \\type  alpha
  A26, \\field Suffix for File Name in Run 25
       \\type  alpha
  A27, \\field Suffix for File Name in Run 26
       \\type  alpha
  A28, \\field Suffix for File Name in Run 27
       \\type  alpha
  A29, \\field Suffix for File Name in Run 28
       \\type  alpha
  A30, \\field Suffix for File Name in Run 29
       \\type  alpha
  A31, \\field Suffix for File Name in Run 30
       \\type  alpha
  A32, \\field Suffix for File Name in Run 31
       \\type  alpha
  A33, \\field Suffix for File Name in Run 32
       \\type  alpha
  A34, \\field Suffix for File Name in Run 33
       \\type  alpha
  A35, \\field Suffix for File Name in Run 34
       \\type  alpha
  A36, \\field Suffix for File Name in Run 35
       \\type  alpha
  A37, \\field Suffix for File Name in Run 36
       \\type  alpha
  A38, \\field Suffix for File Name in Run 37
       \\type  alpha
  A39, \\field Suffix for File Name in Run 38
       \\type  alpha
  A40, \\field Suffix for File Name in Run 39
       \\type  alpha
  A41, \\field Suffix for File Name in Run 40
       \\type  alpha
  A42, \\field Suffix for File Name in Run 41
       \\type  alpha
  A43, \\field Suffix for File Name in Run 42
       \\type  alpha
  A44, \\field Suffix for File Name in Run 43
       \\type  alpha
  A45, \\field Suffix for File Name in Run 44
       \\type  alpha
  A46, \\field Suffix for File Name in Run 45
       \\type  alpha
  A47, \\field Suffix for File Name in Run 46
       \\type  alpha
  A48, \\field Suffix for File Name in Run 47
       \\type  alpha
  A49, \\field Suffix for File Name in Run 48
       \\type  alpha
  A50, \\field Suffix for File Name in Run 49
       \\type  alpha
  A51, \\field Suffix for File Name in Run 50
       \\type  alpha
  A52, \\field Suffix for File Name in Run 51
       \\type  alpha
  A53, \\field Suffix for File Name in Run 52
       \\type  alpha
  A54, \\field Suffix for File Name in Run 53
       \\type  alpha
  A55, \\field Suffix for File Name in Run 54
       \\type  alpha
  A56, \\field Suffix for File Name in Run 55
       \\type  alpha
  A57, \\field Suffix for File Name in Run 56
       \\type  alpha
  A58, \\field Suffix for File Name in Run 57
       \\type  alpha
  A59, \\field Suffix for File Name in Run 58
       \\type  alpha
  A60, \\field Suffix for File Name in Run 59
       \\type  alpha
  A61, \\field Suffix for File Name in Run 60
       \\type  alpha
  A62, \\field Suffix for File Name in Run 61
       \\type  alpha
  A63, \\field Suffix for File Name in Run 62
       \\type  alpha
  A64, \\field Suffix for File Name in Run 63
       \\type  alpha
  A65, \\field Suffix for File Name in Run 64
       \\type  alpha
  A66, \\field Suffix for File Name in Run 65
       \\type  alpha
  A67, \\field Suffix for File Name in Run 66
       \\type  alpha
  A68, \\field Suffix for File Name in Run 67
       \\type  alpha
  A69, \\field Suffix for File Name in Run 68
       \\type  alpha
  A70, \\field Suffix for File Name in Run 69
       \\type  alpha
  A71, \\field Suffix for File Name in Run 70
       \\type  alpha
  A72, \\field Suffix for File Name in Run 71
       \\type  alpha
  A73, \\field Suffix for File Name in Run 72
       \\type  alpha
  A74, \\field Suffix for File Name in Run 73
       \\type  alpha
  A75, \\field Suffix for File Name in Run 74
       \\type  alpha
  A76, \\field Suffix for File Name in Run 75
       \\type  alpha
  A77, \\field Suffix for File Name in Run 76
       \\type  alpha
  A78, \\field Suffix for File Name in Run 77
       \\type  alpha
  A79, \\field Suffix for File Name in Run 78
       \\type  alpha
  A80, \\field Suffix for File Name in Run 79
       \\type  alpha
  A81, \\field Suffix for File Name in Run 80
       \\type  alpha
  A82, \\field Suffix for File Name in Run 81
       \\type  alpha
  A83, \\field Suffix for File Name in Run 82
       \\type  alpha
  A84, \\field Suffix for File Name in Run 83
       \\type  alpha
  A85, \\field Suffix for File Name in Run 84
       \\type  alpha
  A86, \\field Suffix for File Name in Run 85
       \\type  alpha
  A87, \\field Suffix for File Name in Run 86
       \\type  alpha
  A88, \\field Suffix for File Name in Run 87
       \\type  alpha
  A89, \\field Suffix for File Name in Run 88
       \\type  alpha
  A90, \\field Suffix for File Name in Run 89
       \\type  alpha
  A91, \\field Suffix for File Name in Run 90
       \\type  alpha
  A92, \\field Suffix for File Name in Run 91
       \\type  alpha
  A93, \\field Suffix for File Name in Run 92
       \\type  alpha
  A94, \\field Suffix for File Name in Run 93
       \\type  alpha
  A95, \\field Suffix for File Name in Run 94
       \\type  alpha
  A96, \\field Suffix for File Name in Run 95
       \\type  alpha
  A97, \\field Suffix for File Name in Run 96
       \\type  alpha
  A98, \\field Suffix for File Name in Run 97
       \\type  alpha
  A99, \\field Suffix for File Name in Run 98
       \\type  alpha
  A100, \\field Suffix for File Name in Run 99
        \\type  alpha
  A101; \\field Suffix for File Name in Run 100
        \\type  alpha

\\Group Output Reporting

Output:VariableDictionary,
       \\memo Produces a list summarizing the output variables and meters that are available for
       \\memo reporting for the model being simulated (rdd output file). The list varies depending
       \\memo on the types of objects present in the idf file.  For example, variables related to
       \\memo lights will only appear if a Lights object is present. The IDF option generates
       \\memo complete Output:Variable objects to simplify adding the desired output to the idf file.
  \\format singleLine
   A1, \\field Key Field
   \\required-field
   \\type choice
   \\key IDF
   \\key regular
   \\default regular
   A2;  \\field Sort Option
   \\type choice
   \\key Name
   \\key Unsorted

Output:Surfaces:List,
       \\memo Produces a report summarizing the details of surfaces in the eio output file.
  \\format singleLine
   A1, \\field Report Type
   \\required-field
   \\type choice
   \\key Details
   \\key Vertices
   \\key DetailsWithVertices
   \\key ViewFactorInfo
   \\key Lines
   \\key CostInfo
   \\key DecayCurvesfromZoneComponentLoads
   A2; \\field Report Specifications
   \\note (IDF, only for Output:Surfaces:List, Lines report --
   \\note will print transformed coordinates in IDF style)
   \\type choice
   \\key IDF

Output:Surfaces:Drawing,
   \\memo Produces reports/files that are capable of rendering graphically or
   \\memo being imported into other programs. Rendering does not alter the
   \\memo actual inputs/surfaces.
   \\format singleLine
   A1,  \\field Report Type
   \\required-field
   \\type choice
   \\key DXF
   \\key DXF:WireFrame
   \\key VRML
   A2,  \\field Report Specifications 1
   \\type choice
   \\key Triangulate3DFace
   \\key ThickPolyline
   \\key RegularPolyline
   \\default Triangulate3DFace
   \\note Triangulate3DFace (default), ThickPolyline, RegularPolyline apply to DXF
   \\note This field is ignored for DXF:WireFrame and VRML
   A3 ; \\field Report Specifications 2
   \\note Use ColorScheme Name for DXF reports
   \\type object-list
   \\object-list ColorSchemes

Output:Schedules,
        \\memo Produces a condensed reporting that illustrates the full range of schedule values in
        \\memo the eio output file. In the style of input: DaySchedule,  WeekSchedule, and
        \\memo Annual Schedule.
  \\format singleLine
   A1;  \\field Key Field
   \\required-field
   \\type choice
   \\key Hourly
   \\key Timestep

Output:Constructions,
        \\memo Adds a report to the eio output file which shows details for each construction,
        \\memo including overall properties, a list of material layers, and calculated results
        \\memo related to conduction transfer functions.
  \\format singleLine
   A1,  \\field Details Type 1
   \\type choice
   \\key Constructions
   \\key Materials
   A2;  \\field Details Type 2
   \\type choice
   \\key Constructions
   \\key Materials

Output:EnergyManagementSystem,
   \\memo This object is used to control the output produced by the Energy Management System
   \\unique-object
   A1, \\field Actuator Availability Dictionary Reporting
       \\type choice
       \\key None
       \\key NotByUniqueKeyNames
       \\key Verbose
       \\default None
   A2, \\field Internal Variable Availability Dictionary Reporting
       \\type choice
       \\key None
       \\key NotByUniqueKeyNames
       \\key Verbose
       \\default None
   A3; \\field EMS Runtime Language Debug Output Level
       \\type choice
       \\key None
       \\key ErrorsOnly
       \\key Verbose
       \\default None

OutputControl:SurfaceColorScheme,
   \\memo This object is used to set colors for reporting on various building elements particularly for the
   \\memo DXF reports.  We know the user can enter 0 to 255 and the color map is available in DXF output.
   \\memo Therefore, we are limiting the colors in that range.  You can
   \\memo extend by editing the IDD but you do so on your own.  Colors not changed in any scheme will
   \\memo remain as the default scheme uses.
  A1,  \\field Name
       \\required-field
       \\note choose a name or use one of the DataSets
       \\reference ColorSchemes
  A2,  \\field Drawing Element 1 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N1,  \\field Color for Drawing Element 1
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A3,  \\field Drawing Element 2 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N2,  \\field Color for Drawing Element 2
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A4,  \\field Drawing Element 3 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N3,  \\field Color for Drawing Element 3
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A5,  \\field Drawing Element 4 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N4,  \\field Color for Drawing Element 4
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A6,  \\field Drawing Element 5 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N5,  \\field Color for Drawing Element 5
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A7,  \\field Drawing Element 6 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N6,  \\field Color for Drawing Element 6
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A8,  \\field Drawing Element 7 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N7,  \\field Color for Drawing Element 7
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A9,  \\field Drawing Element 8 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N8,  \\field Color for Drawing Element 8
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A10, \\field Drawing Element 9 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N9,  \\field Color for Drawing Element 9
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A11, \\field Drawing Element 10 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N10, \\field Color for Drawing Element 10
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A12, \\field Drawing Element 11 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N11, \\field Color for Drawing Element 11
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A13, \\field Drawing Element 12 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N12, \\field Color for Drawing Element 12
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A14, \\field Drawing Element 13 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N13, \\field Color for Drawing Element 13
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A15, \\field Drawing Element 14 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N14, \\field Color for Drawing Element 14
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255
  A16, \\field Drawing Element 15 Type
       \\type choice
       \\key Text
       \\key Walls
       \\key Windows
       \\key GlassDoors
       \\key Doors
       \\key Roofs
       \\key Floors
       \\key DetachedBuildingShades
       \\key DetachedFixedShades
       \\key AttachedBuildingShades
       \\key Photovoltaics
       \\key TubularDaylightDomes
       \\key TubularDaylightDiffusers
       \\key DaylightReferencePoint1
       \\key DaylightReferencePoint2
  N15; \\field Color for Drawing Element 15
       \\note use color number for output assignment (e.g. DXF)
       \\type integer
       \\minimum 0
       \\maximum 255

Output:Table:SummaryReports,
       \\extensible:1
       \\memo This object allows the user to call report types that are predefined and will appear with the
       \\memo other tabular reports.  These predefined reports are sensitive to the OutputControl:Table:Style object
       \\memo and appear in the same files as the tabular reports.  The entries for this object is a list
       \\memo of the predefined reports that should appear in the tabular report output file.
       \\memo There should be as many fields (A) in this object as there are keys in the following (minus
       \\memo AllSummary+AllMonthly+AllSummaryAndMonthly)
       \\unique-object
   A1, \\field Report 1 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A2, \\field Report 2 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A3, \\field Report 3 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A4, \\field Report 4 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A5, \\field Report 5 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A6, \\field Report 6 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A7, \\field Report 7 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A8, \\field Report 8 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A9, \\field Report 9 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A10,\\field Report 10 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A11,\\field Report 11 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A12, \\field Report 12 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A13, \\field Report 13 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A14, \\field Report 14 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A15, \\field Report 15 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A16, \\field Report 16 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A17, \\field Report 17 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A18, \\field Report 18 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A19, \\field Report 19 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A20, \\field Report 20 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A21, \\field Report 21 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A22, \\field Report 22 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A23, \\field Report 23 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A24, \\field Report 24 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A25, \\field Report 25 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A26, \\field Report 26 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A27, \\field Report 27 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A28, \\field Report 28 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A29, \\field Report 29 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A30, \\field Report 30 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A31, \\field Report 31 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A32, \\field Report 32 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A33, \\field Report 33 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A34, \\field Report 34 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A35, \\field Report 35 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A36, \\field Report 36 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A37, \\field Report 37 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A38, \\field Report 38 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A39, \\field Report 39 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A40, \\field Report 40 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A41, \\field Report 41 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A42, \\field Report 42 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A43, \\field Report 43 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A44, \\field Report 44 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A45, \\field Report 45 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A46, \\field Report 46 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A47, \\field Report 47 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A48, \\field Report 48 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A49, \\field Report 49 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A50, \\field Report 50 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A51, \\field Report 51 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A52, \\field Report 52 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A53, \\field Report 53 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A54, \\field Report 54 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A55, \\field Report 55 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A56, \\field Report 56 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A57, \\field Report 57 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A58, \\field Report 58 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A59, \\field Report 59 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A60, \\field Report 60 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A61, \\field Report 61 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A62, \\field Report 62 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A63, \\field Report 63 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A64, \\field Report 64 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A65, \\field Report 65 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A66, \\field Report 66 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A67, \\field Report 67 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A68, \\field Report 68 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A69, \\field Report 69 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A70, \\field Report 70 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A71, \\field Report 71 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A72, \\field Report 72 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A73, \\field Report 73 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A74, \\field Report 74 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A75, \\field Report 75 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A76, \\field Report 76 Name
       \\begin-extensible
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A77, \\field Report 77 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A78, \\field Report 78 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A79, \\field Report 79 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A80, \\field Report 80 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A81, \\field Report 81 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A82, \\field Report 82 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly
   A83; \\field Report 83 Name
       \\type choice
       \\key AllSummary
       \\key AllSummaryAndSizingPeriod
       \\key AllMonthly
       \\key AllSummaryAndMonthly
       \\key AllSummaryMonthlyAndSizingPeriod
       \\key AnnualBuildingUtilityPerformanceSummary
       \\key InputVerificationandResultsSummary
       \\key DemandEndUseComponentsSummary
       \\key SourceEnergyEndUseComponentsSummary
       \\key ClimaticDataSummary
       \\key EquipmentSummary
       \\key EnvelopeSummary
       \\key SurfaceShadowingSummary
       \\key ShadingSummary
       \\key LightingSummary
       \\key HVACSizingSummary
       \\key SystemSummary
       \\key ComponentSizingSummary
       \\key OutdoorAirSummary
       \\key ObjectCountSummary
       \\key ComponentCostEconomicsSummary
       \\key AdaptiveComfortSummary
       \\key SensibleHeatGainSummary
       \\key ZoneComponentLoadSummary
       \\key Standard62.1Summary
       \\key EnergyMeters
       \\key LEEDSummary
       \\key ZoneCoolingSummaryMonthly
       \\key ZoneHeatingSummaryMonthly
       \\key ZoneElectricSummaryMonthly
       \\key SpaceGainsMonthly
       \\key PeakSpaceGainsMonthly
       \\key SpaceGainComponentsAtCoolingPeakMonthly
       \\key EnergyConsumptionElectricityNaturalGasMonthly
       \\key EnergyConsumptionElectricityGeneratedPropaneMonthly
       \\key EnergyConsumptionDieselFuelOilMonthly
       \\key EnergyConsumptionDistrictHeatingCoolingMonthly
       \\key EnergyConsumptionCoalGasolineMonthly
       \\key EnergyConsumptionOtherFuelsMonthly
       \\key EndUseEnergyConsumptionElectricityMonthly
       \\key EndUseEnergyConsumptionNaturalGasMonthly
       \\key EndUseEnergyConsumptionDieselMonthly
       \\key EndUseEnergyConsumptionFuelOilMonthly
       \\key EndUseEnergyConsumptionCoalMonthly
       \\key EndUseEnergyConsumptionPropaneMonthly
       \\key EndUseEnergyConsumptionGasolineMonthly
       \\key EndUseEnergyConsumptionOtherFuelsMonthly
       \\key PeakEnergyEndUseElectricityPart1Monthly
       \\key PeakEnergyEndUseElectricityPart2Monthly
       \\key ElectricComponentsOfPeakDemandMonthly
       \\key PeakEnergyEndUseNaturalGasMonthly
       \\key PeakEnergyEndUseDieselMonthly
       \\key PeakEnergyEndUseFuelOilMonthly
       \\key PeakEnergyEndUseCoalMonthly
       \\key PeakEnergyEndUsePropaneMonthly
       \\key PeakEnergyEndUseGasolineMonthly
       \\key PeakEnergyEndUseOtherFuelsMonthly
       \\key SetpointsNotMetWithTemperaturesMonthly
       \\key ComfortReportSimple55Monthly
       \\key UnglazedTranspiredSolarCollectorSummaryMonthly
       \\key OccupantComfortDataSummaryMonthly
       \\key ChillerReportMonthly
       \\key TowerReportMonthly
       \\key BoilerReportMonthly
       \\key DXReportMonthly
       \\key WindowReportMonthly
       \\key WindowEnergyReportMonthly
       \\key WindowZoneSummaryMonthly
       \\key WindowEnergyZoneSummaryMonthly
       \\key AverageOutdoorConditionsMonthly
       \\key OutdoorConditionsMaximumDryBulbMonthly
       \\key OutdoorConditionsMinimumDryBulbMonthly
       \\key OutdoorConditionsMaximumWetBulbMonthly
       \\key OutdoorConditionsMaximumDewPointMonthly
       \\key OutdoorGroundConditionsMonthly
       \\key WindowACReportMonthly
       \\key WaterHeaterReportMonthly
       \\key GeneratorReportMonthly
       \\key DaylightingReportMonthly
       \\key CoilReportMonthly
       \\key PlantLoopDemandReportMonthly
       \\key FanReportMonthly
       \\key PumpReportMonthly
       \\key CondLoopDemandReportMonthly
       \\key ZoneTemperatureOscillationReportMonthly
       \\key AirLoopSystemEnergyAndWaterUseMonthly
       \\key AirLoopSystemComponentLoadsMonthly
       \\key AirLoopSystemComponentEnergyUseMonthly
       \\key MechanicalVentilationLoadsMonthly

Output:Table:TimeBins,
       \\memo Produces a bin report in the table output file which shows the amount of time in hours
       \\memo that occurs in different bins for a single specific output variable or meter.
       \\memo Two different types of binning are reported: by month and by hour of the day.
       \\min-fields 5
   A1, \\field Key Value
       \\default *
       \\note use '*' (without quotes) to apply this variable to all keys
   A2, \\field Variable Name
       \\required-field
       \\type external-list
       \\external-list autoRDDvariableMeter
   N1, \\field Interval Start
       \\type real
       \\note The lowest value for the intervals being binned into.
       \\unitsBasedOnField A4
   N2, \\field Interval Size
       \\type real
       \\note The size of the bins starting with Interval start.
       \\unitsBasedOnField A4
   N3, \\field Interval Count
       \\type integer
       \\minimum 1
       \\maximum 20
       \\note The number of bins used. The number of hours below the start of the
       \\note Lowest bin and above the value of the last bin are also shown.
   A3, \\field Schedule Name
       \\type object-list
       \\object-list ScheduleNames
       \\note Optional schedule name. Binning is performed for non-zero hours.
       \\note Binning always performed if left blank.
   A4; \\field Variable Type
       \\type choice
       \\key Energy
       \\key Temperature
       \\key VolumetricFlow
       \\key Power
       \\note Optional input on the type of units for the variable used by other fields in the object.

Output:Table:Monthly,
       \\Memo Provides a generic method of setting up tables of monthly results. The report
       \\memo has multiple columns that are each defined using a repeated group of fields for any
       \\memo number of columns. A single Output:Table:Monthly object often produces multiple
       \\memo tables in the output. A table is produced for every instance of a particular output
       \\memo variable. For example, a table defined with zone variables will be produced once for
       \\memo every zone.
       \\extensible:2
   A1, \\field Name
       \\required-field
       \\type alpha
       \\reference StandardReportNames
   N1, \\field Digits After Decimal
       \\type integer
       \\default 2
       \\minimum 0
       \\maximum 10
! The next two fields are repeated for each column
   A2, \\field Variable or Meter 1 Name
       \\begin-extensible
       \\type external-list
       \\external-list autoRDDvariableMeter
       \\note The name of an output variable or  meter that is available in the RDD file.
   A3, \\field Aggregation Type for Variable or Meter 1
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note The method of aggregation for the selected variable or meter.
       \\note SumOrAverage adds up the values for every timestep in the month if the variable
       \\note   is a sum variable. If the variable is an average variable it reports the average value.
       \\note   This is probably the most common Aggregation Type option to choose.
       \\note Maximum and Minimum find the maximum and minimum value for the month and report it
       \\note   along with the time that it first occurred.  When Maximum or Minimum are selected a
       \\note   column is automatically shown in the output report for the time that it occurred.
       \\note   When Maximum or Minimum are used with an average variable the value is divided
       \\note   by the length of the timestep.
       \\note ValueWhenMaxMin looks at the previous variable in the report that sets a maximum or minimum
       \\note   and displays the value of the current variable at that same timestep.
       \\note   Order of the variables in the report is important when using ValueWhenMaxMin.
       \\note   This can be used when an outdoor temperature should be reported for the time of the
       \\note   maximum cooling load.
       \\note HoursNonZero adds up the elapsed time during the month that this variable is non-zero
       \\note   and would be appropriate to determine the number of hour that a fan operates.
       \\note HoursZero - The HoursZero option adds up the elapsed time during the month that this
       \\note   variable has a zero value and would be appropriate to determine the number of hour
       \\note   that a fan does not operate.
       \\note HoursPositive - The HoursPositive option adds up the elapsed time during the month that
       \\note   this variable has a positive value.  Hours with a zero value are not included.
       \\note HoursNonPositive - The HoursNonPositive option adds up the elaspsed time during the month
       \\note   that this variable has non-positive value. Hours with a negative value and hours with a
       \\note   zero value are all included.
       \\note HoursNegative - The HoursNegative option adds up the elapsed time during the month that
       \\note   this variable has a negative value.  Hours with a zero value are not included.
       \\note HoursNonNegative - The HoursNonNegative option adds up the elaspsed time during the month
       \\note   that this variable has non-negative value. Hours with a positive value and
       \\note   hours with a zero value are all included.
       \\note SumOrAverageDuringHoursShown - Provides the sum or average of the named variable when
       \\note   during the hours that the previous variable displayed with any of the Aggregation Types
       \\note   starting with "Hours". For "sum" type variables adds up the values for each timestep
       \\note   and reports the sum of the value monthly during the hours reported on the previous
       \\note   variable. For "average" type variables, the value shown will be the average for the month
       \\note   during the hours reported on the previous variable.  Order of the variables in
       \\note   the report is important when using this Aggregation Type.
       \\note MaximumDuringHoursShown - Reports the maximum value and the time that the maximum value
       \\note   occurred but only during the hours reported with the previous "hours-" Aggregation Type.
       \\note   When the Maximum option is used with an average variable the value is divided
       \\note   by the length of the timestep.  Order of the variables in the report is important
       \\note   when using this Aggregation Type.
       \\note MinimumDuringHoursShown - Reports the minimum value and the time that the minimum value
       \\note   occurred but only during the hours reported with the previous "hours-" Aggregation Type.
       \\note   When the Minimum option is used with an average variable the value is divided
       \\note   by the length of the timestep.  Order of the variables in the report is important
       \\note   when using this Aggregation Type.
   A4, \\field Variable or Meter 2 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A5, \\field Aggregation Type for Variable or Meter 2
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A6, \\field Variable or Meter 3 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A7, \\field Aggregation Type for Variable or Meter 3
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A8, \\field Variable or Meter 4 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A9, \\field Aggregation Type for Variable or Meter 4
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A10,\\field Variable or Meter 5 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A11,\\field Aggregation Type for Variable or Meter 5
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A12,\\field Variable or Meter 6 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A13,\\field Aggregation Type for Variable or Meter 6
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A14,\\field Variable or Meter 7 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A15,\\field Aggregation Type for Variable or Meter 7
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A16,\\field Variable or Meter 8 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A17,\\field Aggregation Type for Variable or Meter 8
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A18,\\field Variable or Meter 9 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A19,\\field Aggregation Type for Variable or Meter 9
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A20,\\field Variable or Meter 10 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A21,\\field Aggregation Type for Variable or Meter 10
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A22,\\field Variable or Meter 11 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A23,\\field Aggregation Type for Variable or Meter 11
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A24,\\field Variable or Meter 12 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A25,\\field Aggregation Type for Variable or Meter 12
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A26,\\field Variable or Meter 13 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A27,\\field Aggregation Type for Variable or Meter 13
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A28,\\field Variable or Meter 14 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A29,\\field Aggregation Type for Variable or Meter 14
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A30,\\field Variable or Meter 15 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A31,\\field Aggregation Type for Variable or Meter 15
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A32,\\field Variable or Meter 16 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A33,\\field Aggregation Type for Variable or Meter 16
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A34,\\field Variable or Meter 17 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A35,\\field Aggregation Type for Variable or Meter 17
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A36,\\field Variable or Meter 18 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A37,\\field Aggregation Type for Variable or Meter 18
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A38,\\field Variable or Meter 19 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A39,\\field Aggregation Type for Variable or Meter 19
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A40,\\field Variable or Meter 20 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A41,\\field Aggregation Type for Variable or Meter 20
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A42,\\field Variable or Meter 21 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A43,\\field Aggregation Type for Variable or Meter 21
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A44,\\field Variable or Meter 22 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A45,\\field Aggregation Type for Variable or Meter 22
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A46,\\field Variable or Meter 23 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A47,\\field Aggregation Type for Variable or Meter 23
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A48,\\field Variable or Meter 24 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A49,\\field Aggregation Type for Variable or Meter 24
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01
   A50,\\field Variable or Meter 25 Name
       \\type external-list
       \\external-list autoRDDvariableMeter
   A51;\\field Aggregation Type for Variable or Meter 25
       \\type choice
       \\key SumOrAverage
       \\key Maximum
       \\key Minimum
       \\key ValueWhenMaximumOrMinimum
       \\key HoursNonZero
       \\key HoursZero
       \\key HoursPositive
       \\key HoursNonPositive
       \\key HoursNegative
       \\key HoursNonNegative
       \\key SumOrAverageDuringHoursShown
       \\key MaximumDuringHoursShown
       \\key MinimumDuringHoursShown
       \\note See instructions under AggregationType01

OutputControl:Table:Style,
       \\memo default style for the OutputControl:Table:Style is comma -- this works well for
       \\memo importing into spreadsheet programs such as Excel(tm) but not so well for word
       \\memo processing progams -- there tab may be a better choice.  fixed puts spaces between
       \\memo the "columns".  HTML produces tables in HTML. XML produces an XML file.
       \\unique-object
   A1, \\field Column Separator
       \\type choice
       \\key Comma
       \\key Tab
       \\key Fixed
       \\key HTML
       \\key XML
       \\key CommaAndHTML
       \\key CommaAndXML
       \\key TabAndHTML
       \\key XMLandHTML
       \\key All
   A2; \\field Unit Conversion
       \\type choice
       \\key None
       \\key JtoKWH
       \\key JtoMJ
       \\key JtoGJ
       \\key InchPound

OutputControl:ReportingTolerances,
       \\memo Calculations of the time that setpoints are not met use a tolerance of 0.2C.
       \\memo This object allows changing the tolerance used to determine when setpoints are being met.
       \\unique-object
   N1, \\field Tolerance for Time Heating Setpoint Not Met
       \\note If the zone temperature is below the heating setpoint by more than
       \\note this value, the following output variables will increment as appropriate
       \\note Zone Heating Setpoint Not Met Time
       \\note Zone Heating Setpoint Not Met While Occupied Time
       \\note This also impacts table report "Annual Building Utility Performance Summary"
       \\note subtable "Comfort and Setpoint Not Met Summary"
       \\units deltaC
       \\default .2
       \\minimum 0
       \\maximum 10
   N2; \\field Tolerance for Time Cooling Setpoint Not Met
       \\note If the zone temperature is above the cooling setpoint by more than
       \\note this value, the following output variables will increment as appropriate
       \\note Zone Cooling Setpoint Not Met Time
       \\note Zone Cooling Setpoint Not Met While Occupied Time
       \\note This also impacts table report "Annual Building Utility Performance Summary"
       \\note subtable "Comfort and Setpoint Not Met Summary"
       \\units deltaC
       \\default .2
       \\minimum 0
       \\maximum 10

Output:Variable,
       \\memo each Output:Variable command picks variables to be put onto the standard output file (.eso)
       \\memo some variables may not be reported for every simulation.
       \\memo a list of variables that can be reported are available after a run on
       \\memo the report dictionary file (.rdd) if the Output:VariableDictionary has been requested.
       \\format singleLine
  A1 , \\field Key Value
       \\default *
       \\note use '*' (without quotes) to apply this variable to all keys
  A2 , \\field Variable Name
       \\required-field
       \\type external-list
       \\external-list autoRDDvariable
  A3 , \\field Reporting Frequency
       \\type choice
       \\key Detailed
       \\note Detailed lists every instance (i.e. HVAC variable timesteps)
       \\key Timestep
       \\note Timestep refers to the zone Timestep/Number of Timesteps in hour value
       \\note RunPeriod, Environment, and Annual are the same
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod
       \\key Environment
       \\key Annual
       \\default Hourly
       \\note RunPeriod, Environment, and Annual are synonymous
  A4 ; \\field Schedule Name
       \\type object-list
       \\object-list ScheduleNames

Output:Meter,
       \\memo Each Output:Meter command picks meters to be put onto the standard output file (.eso) and
       \\memo meter file (.mtr). Not all meters are reported in every simulation. A list of
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
       \\format singleLine
  A1 , \\field Name
       \\required-field
       \\type external-list
       \\external-list autoRDDmeter
       \\note Form is EnergyUseType:..., e.g. Electricity:* for all Electricity meters
       \\note or EndUse:..., e.g. GeneralLights:* for all General Lights
       \\note Output:Meter puts results on both the eplusout.mtr and eplusout.eso files
  A2 ; \\field Reporting Frequency
       \\type choice
       \\key Timestep
       \\note Timestep refers to the zone Timestep/Number of Timesteps in hour value
       \\note RunPeriod, Environment, and Annual are the same
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod
       \\key Environment
       \\key Annual
       \\default Hourly
       \\note RunPeriod, Environment, and Annual are synonymous

Output:Meter:MeterFileOnly,
       \\memo Each Output:Meter:MeterFileOnly command picks meters to be put only onto meter file (.mtr).
       \\memo Not all meters are reported in every simulation. A list of meters that can be reported
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
       \\format singleLine
  A1,  \\field Name
       \\required-field
       \\type external-list
       \\external-list autoRDDmeter
       \\note Form is EnergyUseType:..., e.g. Electricity:* for all Electricity meters
       \\note or EndUse:..., e.g. GeneralLights:* for all General Lights
       \\note Output:Meter:MeterFileOnly puts results on the eplusout.mtr file only
  A2 ; \\field Reporting Frequency
       \\type choice
       \\key Timestep
       \\note Timestep refers to the zone Timestep/Number of Timesteps in hour value
       \\note RunPeriod, Environment, and Annual are the same
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod
       \\key Environment
       \\key Annual
       \\default Hourly
       \\note RunPeriod, Environment, and Annual are synonymous

Output:Meter:Cumulative,
       \\memo Each Output:Meter:Cumulative command picks meters to be reported cumulatively onto the
       \\memo standard output file (.eso) and meter file (.mtr). Not all meters are reported in every
       \\memo simulation.
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
       \\format singleLine
  A1,  \\field Name
       \\required-field
       \\type external-list
       \\external-list autoRDDmeter
       \\note Form is EnergyUseType:..., e.g. Electricity:* for all Electricity meters
       \\note or EndUse:..., e.g. GeneralLights:* for all General Lights
       \\note Output:Meter:Cumulative puts results on both the eplusout.mtr and eplusout.eso files
  A2 ; \\field Reporting Frequency
       \\type choice
       \\key Timestep
       \\note Timestep refers to the zone Timestep/Number of Timesteps in hour value
       \\note RunPeriod, Environment, and Annual are the same
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod
       \\key Environment
       \\key Annual
       \\default Hourly
       \\note RunPeriod, Environment, and Annual are synonymous

Output:Meter:Cumulative:MeterFileOnly,
       \\memo Each Output:Meter:Cumulative:MeterFileOnly command picks meters to be reported cumulatively
       \\memo onto the standard output file (.eso) and meter file (.mtr). Not all meters are reported in
       \\memo every simulation.
       \\memo a list of meters that can be reported are available after a run on
       \\memo the meter dictionary file (.mdd) if the Output:VariableDictionary has been requested.
       \\format singleLine
  A1,  \\field Name
       \\required-field
       \\type external-list
       \\external-list autoRDDmeter
       \\note Form is EnergyUseType:..., e.g. Electricity:* for all Electricity meters
       \\note or EndUse:..., e.g. GeneralLights:* for all General Lights
       \\note Output:Meter:Cumulative:MeterFileOnly puts results on the eplusout.mtr file only
  A2 ; \\field Reporting Frequency
       \\type choice
       \\key Timestep
       \\note Timestep refers to the zone Timestep/Number of Timesteps in hour value
       \\note RunPeriod, Environment, and Annual are the same
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod
       \\key Environment
       \\key Annual
       \\default Hourly
       \\note RunPeriod, Environment, and Annual are synonymous

Meter:Custom,
   \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
   \\memo Used to allow users to combine specific variables and/or meters into
   \\memo "custom" meter configurations. To access these meters by name, one must
   \\memo first run a simulation to generate the RDD/MDD files and names.
   A1,  \\field Name
        \\required-field
   A2,  \\field Fuel Type
        \\type choice
        \\key Electricity
        \\key NaturalGas
        \\key PropaneGas
        \\key FuelOil#1
        \\key FuelOil#2
        \\key Coal
        \\key Diesel
        \\key Gasoline
        \\key Water
        \\key Generic
        \\key OtherFuel1
        \\key OtherFuel2
   A3,  \\field Key Name 1
        \\begin-extensible
   A4,  \\field Output Variable or Meter Name 1
   A5,  \\field Key Name 2
   A6,  \\field Output Variable or Meter Name 2
   A7,  \\field Key Name 3
   A8,  \\field Output Variable or Meter Name 3
   A9,  \\field Key Name 4
   A10, \\field Output Variable or Meter Name 4
   A11, \\field Key Name 5
   A12, \\field Output Variable or Meter Name 5
   A13, \\field Key Name 6
   A14, \\field Output Variable or Meter Name 6
   A15, \\field Key Name 7
   A16, \\field Output Variable or Meter Name 7
   A17, \\field Key Name 8
   A18, \\field Output Variable or Meter Name 8
   A19, \\field Key Name 9
   A20, \\field Output Variable or Meter Name 9
   A21, \\field Key Name 10
   A22, \\field Output Variable or Meter Name 10
   A23, \\field Key Name 11
   A24, \\field Output Variable or Meter Name 11
   A25, \\field Key Name 12
   A26, \\field Output Variable or Meter Name 12
   A27, \\field Key Name 13
   A28, \\field Output Variable or Meter Name 13
   A29, \\field Key Name 14
   A30, \\field Output Variable or Meter Name 14
   A31, \\field Key Name 15
   A32, \\field Output Variable or Meter Name 15
   A33, \\field Key Name 16
   A34, \\field Output Variable or Meter Name 16
   A35, \\field Key Name 17
   A36, \\field Output Variable or Meter Name 17
   A37, \\field Key Name 18
   A38, \\field Output Variable or Meter Name 18
   A39, \\field Key Name 19
   A40, \\field Output Variable or Meter Name 19
   A41, \\field Key Name 20
   A42, \\field Output Variable or Meter Name 20
   A43, \\field Key Name 21
   A44, \\field Output Variable or Meter Name 21
   A45, \\field Key Name 22
   A46; \\field Output Variable or Meter Name 22

Meter:CustomDecrement,
   \\extensible:2 - repeat last two fields, remembering to remove ; from "inner" fields.
   \\memo Used to allow users to combine specific variables and/or meters into
   \\memo "custom" meter configurations. To access these meters by name, one must
   \\memo first run a simulation to generate the RDD/MDD files and names.
   A1,  \\field Name
        \\required-field
   A2,  \\field Fuel Type
        \\type choice
        \\key Electricity
        \\key NaturalGas
        \\key PropaneGas
        \\key FuelOil#1
        \\key FuelOil#2
        \\key Coal
        \\key Diesel
        \\key Gasoline
        \\key Water
        \\key Generic
        \\key OtherFuel1
        \\key OtherFuel2
   A3,  \\field Source Meter Name
        \\required-field
   A4,  \\field Key Name 1
        \\begin-extensible
   A5,  \\field Output Variable or Meter Name 1
   A6,  \\field Key Name 2
   A7,  \\field Output Variable or Meter Name 2
   A8,  \\field Key Name 3
   A9,  \\field Output Variable or Meter Name 3
   A10, \\field Key Name 4
   A11, \\field Output Variable or Meter Name 4
   A12, \\field Key Name 5
   A13, \\field Output Variable or Meter Name 5
   A14, \\field Key Name 6
   A15, \\field Output Variable or Meter Name 6
   A16, \\field Key Name 7
   A17, \\field Output Variable or Meter Name 7
   A18, \\field Key Name 8
   A19, \\field Output Variable or Meter Name 8
   A20, \\field Key Name 9
   A21, \\field Output Variable or Meter Name 9
   A22, \\field Key Name 10
   A23, \\field Output Variable or Meter Name 10
   A24, \\field Key Name 11
   A25, \\field Output Variable or Meter Name 11
   A26, \\field Key Name 12
   A27, \\field Output Variable or Meter Name 12
   A28, \\field Key Name 13
   A29, \\field Output Variable or Meter Name 13
   A30, \\field Key Name 14
   A31, \\field Output Variable or Meter Name 14
   A32, \\field Key Name 15
   A33, \\field Output Variable or Meter Name 15
   A34, \\field Key Name 16
   A35, \\field Output Variable or Meter Name 16
   A36, \\field Key Name 17
   A37, \\field Output Variable or Meter Name 17
   A38, \\field Key Name 18
   A39, \\field Output Variable or Meter Name 18
   A40, \\field Key Name 19
   A41, \\field Output Variable or Meter Name 19
   A42, \\field Key Name 20
   A43, \\field Output Variable or Meter Name 20
   A44, \\field Key Name 21
   A45, \\field Output Variable or Meter Name 21
   A46, \\field Key Name 22
   A47; \\field Output Variable or Meter Name 22

Output:SQLite,
       \\memo Output from EnergyPlus can be written to an SQLite format file.
       \\unique-object
  A1 ; \\field Option Type
       \\type choice
       \\key Simple
       \\key SimpleAndTabular

Output:EnvironmentalImpactFactors,
   \\memo This is used to Automatically report the facility meters and turn on the Environmental Impact Report calculations
   \\memo for all of the Environmental Factors.
  A1 ; \\field Reporting Frequency
       \\type choice
       \\key Timestep
       \\key Hourly
       \\key Daily
       \\key Monthly
       \\key RunPeriod

EnvironmentalImpactFactors,
   \\memo Used to help convert district and ideal energy use to a fuel type and provide total carbon equivalent with coefficients
   \\memo Also used in Source=>Site conversions.
  N1, \\field District Heating Efficiency
      \\note District heating efficiency used when converted to natural gas
      \\minimum> 0.0
      \\default 0.3
  N2, \\field District Cooling COP
      \\note District cooling COP used when converted to electricity
      \\units W/W
      \\minimum> 0.0
      \\default 3.0
  N3, \\field Steam Conversion Efficiency
      \\note Steam conversion efficiency used to convert steam usage to natural gas
      \\minimum> 0.0
      \\default 0.25
  N4, \\field Total Carbon Equivalent Emission Factor From N2O
      \\units kg/kg
      \\default 80.7272
  N5, \\field Total Carbon Equivalent Emission Factor From CH4
      \\units kg/kg
      \\default 6.2727
  N6; \\field Total Carbon Equivalent Emission Factor From CO2
      \\units kg/kg
      \\default 0.2727

FuelFactors,
     \\memo Provides Fuel Factors for Emissions as well as Source=>Site conversions.
     \\memo OtherFuel1, OtherFuel2 provide options for users who want to create and use
     \\memo fuels that may not be mainstream (biomass, wood, pellets).
  A1,   \\field Existing Fuel Resource Name
        \\type choice
        \\key Electricity
        \\key NaturalGas
        \\key FuelOil#1
        \\key FuelOil#2
        \\key Coal
        \\key Gasoline
        \\key Propane
        \\key Diesel
        \\key OtherFuel1
        \\key OtherFuel2
  A2,   \\field Units of Measure
  N1,   \\field Energy per Unit Factor
  N2,   \\field Source Energy Factor
        \\units J/J
  A3,   \\field Source Energy Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N3,   \\field CO2 Emission Factor
        \\units g/MJ
  A4,   \\field CO2 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N4,   \\field CO Emission Factor
        \\units g/MJ
  A5,   \\field CO Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N5,   \\field CH4 Emission Factor
        \\units g/MJ
  A6,   \\field CH4 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N6,   \\field NOx Emission Factor
        \\units g/MJ
  A7,   \\field NOx Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N7,   \\field N2O Emission Factor
        \\units g/MJ
  A8,   \\field N2O Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N8,   \\field SO2 Emission Factor
        \\units g/MJ
  A9,   \\field SO2 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N9,   \\field PM Emission Factor
        \\units g/MJ
  A10,  \\field PM Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N10,  \\field PM10 Emission Factor
        \\units g/MJ
  A11,  \\field PM10 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N11,  \\field PM2.5 Emission Factor
        \\units g/MJ
  A12,  \\field PM2.5 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N12,  \\field NH3 Emission Factor
        \\units g/MJ
  A13,  \\field NH3 Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N13,  \\field NMVOC Emission Factor
        \\units g/MJ
  A14,  \\field NMVOC Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N14,  \\field Hg Emission Factor
        \\units g/MJ
  A15,  \\field Hg Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N15,  \\field Pb Emission Factor
        \\units g/MJ
  A16,  \\field Pb Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N16,  \\field Water Emission Factor
        \\units L/MJ
  A17,  \\field Water Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N17,  \\field Nuclear High Level Emission Factor
        \\units g/MJ
  A18,  \\field Nuclear High Level Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames
  N18,  \\field Nuclear Low Level Emission Factor
        \\units m3/MJ
  A19;  \\field Nuclear Low Level Emission Factor Schedule Name
        \\type object-list
        \\object-list ScheduleNames

Output:Diagnostics,
    \\memo Special keys to produce certain warning messages or effect certain simulation characteristics.
  A1 , \\field Key 1
       \\type choice
       \\key DisplayAllWarnings
       \\key DisplayExtraWarnings
       \\key DisplayUnusedSchedules
       \\key DisplayUnusedObjects
       \\key DisplayAdvancedReportVariables
       \\key DisplayZoneAirHeatBalanceOffBalance
       \\key DoNotMirrorDetachedShading
       \\key DisplayWeatherMissingDataWarnings
       \\key ReportDuringWarmup
       \\key ReportDetailedWarmupConvergence
  A2 ; \\field Key 2
       \\type choice
       \\key DisplayAllWarnings
       \\key DisplayExtraWarnings
       \\key DisplayUnusedSchedules
       \\key DisplayUnusedObjects
       \\key DisplayAdvancedReportVariables
       \\key DisplayZoneAirHeatBalanceOffBalance
       \\key DoNotMirrorDetachedShading
       \\key DisplayWeatherMissingDataWarnings
       \\key ReportDuringWarmup
       \\key ReportDetailedWarmupConvergence

Output:DebuggingData,
       \\memo switch eplusout.dbg file on or off
       \\unique-object
       \\format singleLine
  N1 , \\field Report Debugging Data
       \\note value=1 then yes all others no
  N2 ; \\field Report During Warmup
       \\note value=1 then always even during warmup  all others no

Output:PreprocessorMessage,
   \\memo This object does not come from a user input.  This is generated by a pre-processor
   \\memo so that various conditions can be gracefully passed on by the InputProcessor.
   A1,  \\field Preprocessor Name
        \\retaincase
   A2,  \\field Error Severity
        \\note Depending on type, InputProcessor may terminate the program.
        \\type choice
        \\key Information
        \\key Warning
        \\key Severe
        \\key Fatal
        \\retaincase
   A3,  \\field Message Line 1
        \\retaincase
   A4,  \\field Message Line 2
        \\retaincase
   A5,  \\field Message Line 3
        \\retaincase
   A6,  \\field Message Line 4
        \\retaincase
   A7,  \\field Message Line 5
        \\retaincase
   A8,  \\field Message Line 6
        \\retaincase
   A9,  \\field Message Line 7
        \\retaincase
   A10, \\field Message Line 8
        \\retaincase
   A11, \\field Message Line 9
        \\retaincase
   A12; \\field Message Line 10
        \\retaincase
"""
